#!/usr/bin/perl

use strict;
use DBAccess;
use Data::Dumper;
use Portal;
use MetadotConfig;


my $server_name = shift(@ARGV) || usage_error();

print "\nUpdating sequences for server '$server_name'\n";

set_server_name($server_name);

my $dbo = DBAccess->get_db_object();
my $dbh = $dbo->get_dbh();
my $tables = $dbo->{tables};

my ($seq, $seq_name, $pk, $pk_max_value, $seq_nextval);

foreach my $table (sort keys(%$tables)) {
    print "\nProcessing table: $table";
    $seq = $tables->{$table}->{seq};
    $pk  = $tables->{$table}->{pk};

    if (existsTable($table)) {
        print " - Table was found\n";
    } else {
        print " - Table does not exist - skipping\n";
        next;
    }

    if (defined($seq)) {
        # get name for reuse
        $seq_name = $seq->name();

        # make sure the sequence actually exists
	unless ($seq->sequenceNameExists()) {
	    print "Creating a new sequence...\n";
	    $seq->create();
	}
	
        print " - Sequence in use; name = $seq_name and primary key = $pk\n";
    } else {
        ## no seq then skip
        print " - This table does not require a sequence\n";
        next;
    }

    ($pk_max_value) = DBAccess->sqlSelect("max($pk)", $table);
    unless ($seq_nextval = $seq->nextval()) {
        print " - Cannot get current value from sequence; skipping\n";
        next;
    }

    print " - Max value of primary key = $pk_max_value; current value of sequence = $seq_nextval\n";

    if ($pk_max_value > $seq_nextval) {
        print " - Current value of sequence is too low; increasing.\n";
        $seq->drop();
        $seq = new DBAccess::OracleSequence($dbh, "$seq_name");
        $seq->create($pk_max_value + 1);
        print " - Sequence $seq_name recreated with :" . $seq->print() . "\n";
    } else {
        print " - Table " . $table . "'s sequence is OK\n";
        next;
    }

}


## we done and we outta here
print "\n\nAll sequences have been updated.\n\n";


sub set_server_name {
    my $server_name = shift || die("Must specify a server name.\n");

    MetadotConfig->set_server_name($server_name);

    if (!MetadotConfig->get_section_name($server_name)) {
        die "\nThere is no server configuration for the server name : $server_name\n\n";
    }
    
    #confirmation
    my $mdot = Portal->new(1);
    use Metadot qw($PARAMS);
    print "Server was set to $server_name, basename = $PARAMS->{basename}\n\n";
}

sub usage_error {
    my $error_string = "\n\nUsage: generate_sequences.pl server_name\n\n";
    my @servers = MetadotConfig->get_all_server_names();
    $error_string .= "       The following servers are available: ".join(',',@servers)."\n\n";

    die($error_string);
}

my @TABLE_LIST;
sub existsTable {
    my $tablename = shift;
    
    unless (defined @TABLE_LIST) {
        my $dbh = DBAccess->get_db_object()->get_dbh();
        @TABLE_LIST = $dbh->tables();
    }
    # get list of all tables in the db
    my $found = 0;
    foreach my $tname (@TABLE_LIST) {
        #print "Checking '$tablename' against '$tname'\n";
        # fist case is for Mysql (table names are returned in the form TABLE_NAME)
        # second case is for Oracle (table names are returned in the form DATABASE_NAME.TABLE_NAME or "DATABASE_NAME"."TABLE_NAME")
        if ($tname =~ /^$tablename$/i || $tname =~ /^(.*)\."?$tablename"?$/i) {
            $found = 1;
            last;
        }
    }
    return $found;
}

