#!/usr/bin/perl -w

use strict;

use Portal;
use Metadot qw($PARAMS);
use MetadotConfig;
use DBAccess;
use Metadot::Permissions;
use Metadot::Serialization::SerializerFactory;

my $server_name = shift @ARGV;
if (defined $server_name)
{
  MetadotConfig->set_server_name($server_name);
  print "Server was set to $server_name, basename = $PARAMS->{basename}\n\n";
}

my $mdot = Portal->new(1);

# get the default serializer
our $DEFAULT_SERIALIZER = Metadot::Serialization::SerializerFactory->get_instance();
print "My default serializer is $DEFAULT_SERIALIZER\n";


&migrate_permissions();


sub migrate_permissions
{
  print "Migrating permissions...\n";

  # get all permissions objects
  my $sth = DBAccess->sqlSelectMany('id,isa,permissions', 'permissions', '1=1');
  while (my ($id, $isa, $data) = $sth->fetchrow())
  {
    my $serializer = Metadot::Serialization::SerializerFactory->get_instance($data);

    # if the serializer for this data is different from the default serializer,
    # then we migrate it

    if (ref($serializer) ne ref($DEFAULT_SERIALIZER)) {
    my $object     = $serializer->deserialize($data);

    # some older permissions objects were blessed as 'Permissions', not
    # 'Metadot::Permissions'
    if (UNIVERSAL::isa($object, 'Permissions')) {
        bless ($object, 'Metadot::Permissions');
    }

    my $migrated   = $DEFAULT_SERIALIZER->serialize($object);

    print "Migrating permissions for: $isa $id\n";
  
    DBAccess->sqlUpdate('permissions', "id=$id AND isa=".DBAccess->quote($isa), permissions => $migrated);
  }
  }

  $sth->finish();
}

