#!/usr/bin/perl
###########################################################################
## <LICENSEINFO>
## Copyright (C) Daniel Guermeur - 1999-2001
## Copyright (C) Metadot Corporation - 2001-2005
##
## This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
##
## Metadot Portal Server software is free software; you can
## redistribute it and/or modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation; either version 2 of
## the License, or (at your option) any later version.
##
## Metadot Portal Server software is distributed in the hope that it
## will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
## of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
## Public License for more details.
##
## You should have received a copy of the GNU General Public License along with
## Metadot Portal Server; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##
## For more information, please contact info@metadot.com.
##</LICENSEINFO>
##
#############################################################################

use strict;
use Text::CSV_XS;
use Data::Dumper;
use Metadot::User::Default;
use Metadot::User::FlexUser;
use Portal;

### Create a portal object and a user (admin) object
my $mdot = Portal->new(1);
my $USER = Metadot::User::Default->new(8);

if( $#ARGV != 1 ){
    print STDERR "Usage: $0 <option> user_import_file.csv\n";
    print STDERR "Options:\n\t-display_fields\tDisplays available fields for input file\n";
    print STDERR "\t-p\t\tProcess the user_import_file\n\n";
    exit;
}

my $option   = $ARGV[0];
my $csv_file = $ARGV[1];

if( $option =~ /^-d/ ){
    print STDERR "The following fields are available to import:\n\n";
    my $available_fields = Metadot::User::FlexUser->get_fields_config();
    foreach my $field (@{ $available_fields->{FIELDS} }) {
        print STDERR "Field name: ".$field->{field_name}."\n";
    }
    print STDERR "\nYou can create your CSV file with the first line containing\n".
                 "any of the headers displayed above separated by comma.\n\n".
                 "Note that the following fields are required:\n".
                 "\t- last_name\n\t- first_name\n\t- email\n\t- password\n";
    print STDERR "\nExample CSV file:\nfirst_name,last_name,password,city,state\n".
                 "Joe,Vega,MyPasSworD,Austin,TX\n".
                 "Marcus,Aurelius,g8lfisGr8,San Franciso, CA\n".
                 "Mike,Johnson,\"123,456\",Detroit,MI\n";
    print STDERR "\nNOTE: If one of the data fields contains a comma, it must be\n".
                 "enclosed in double quotes as in the example above.\n";
    print STDERR "\n";

    exit;
}

if( !-e $csv_file ){
    print STDERR "Error: File[$csv_file] does not exist!\nExiting...\n";
    exit;
}

unless( open(LINES,"<$csv_file") ){
    print STDERR "Cannot open [$csv_file]\nExiting...\n";
    exit;
}


### Get the default register class
my $default_register_class = 'Metadot::Register::Register3000';
my $register_class  = MetadotConfig->get_registration_type();

unless(UNIVERSAL::isa($register_class, $default_register_class)) {
    $register_class = $default_register_class;
}

require(Utility->package_to_filename($register_class));

### I have a valid register object
my $register = $register_class->new();

### Create a new CSV object
my $csv = Text::CSV_XS->new();

my @columns;
my $status;
my $first_row = 1;
my $fields = {};

### Go through the file line by line parsing it and registering users
while( <LINES> ){
   $status = $csv->parse($_);         # parse a CSV string into fields
   @columns = $csv->fields();

   if( $first_row ){
       #parse the column headers
       $first_row = 0;
       for(my $i=0; $i<=$#columns; $i++ ){
          $fields->{$i} = $columns[$i];
       }
       next;
   }

   #print STDERR "\n----------Fields---------\n";
   #print STDERR Dumper($fields);
   #print STDERR "\n++++++++++Fields+++++++++\n";

   my $input = {};
   for(my $i=0; $i<=$#columns; $i++ ){
      $input->{$fields->{$i}} = $columns[$i];
   }

   #Make sure we have an first name field
   unless( defined($input->{first_name}) && $input->{first_name} ne '' ){
      print STDERR "Error: A first_name field is required!\n";
      exit;
   }

   #Make sure we have an last name field
   unless( defined($input->{last_name}) && $input->{last_name} ne '' ){
      print STDERR "Error: A last_name field is required!\n";
      exit;
   }

   #Make sure we have an email field
   unless( defined($input->{email}) && $input->{email} ne '' ){
      print STDERR "Error: A email field is required!\n";
      exit;
   }

   #Make sure we have a password field
   unless( defined($input->{password}) && $input->{password} ne '' ){
      print STDERR "Error: A password field is required!\n";
      exit;
   }

   #Make sure we have a password confirm field
   unless( defined($input->{password_confirm}) ){
      $input->{password_confirm} = $input->{password};
   }

   #Make sure user is not already registered
   if( $register->email_already_registered($input->{email}) ){
       print STDERR "User[".$input->{email}."] is already registered\n";
       next;
   }

   $input->{skip_email_notification} = 1;

   #print Dumper($input);
   my $result = $register->process_registration({input => $input});
   my $user   = $result->{user};
   #print Dumper($user);
}
close(LINES);
