###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package App;

use strict;
use vars '$VERSION';
use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY);

$VERSION=1.0;

use Carp;
use Gizmo::Category;
use Gizmo::Category::Clipboard;

use Data::Dumper;

use vars qw(@ISA $AC);
@ISA=qw(Category);

sub new {
    my $proto = shift;

    # Some gizmos pass only an $id to the constructor, while
    # others send a hashref of arguments. This causes the
    # constructor to always see a hashref of arguments, no
    # matter what the actual parameters were.
    my $args = $class->normalize_constructor_args(shift);
    my $id = $args->{id};


    my $class = ref($proto) || $proto;
    my $self;

    if (defined ($id)) {
        $self = $class->SUPER::new($id);
    } else {
        $self = $class->SUPER::new();
        $self->{display_look} = 'twocol';
    }

    $self->{is_a} = 'Category';
    $self->{app} = undef;

    bless ($self, $class);
    return $self;
}


sub www_show {
    my $self = shift;

    $self->debug_msg(3, "App::www_show()");
    my $content = undef;

    my $iid = $self->defined_or_exit($self->id, "id is missing");

    $content = $self->show();

    $self->print_portal($content);
}

sub display_controller {
    my $self  = shift;
    my $op    = shift;
    my $gizmo = shift;

    if ($gizmo->display_in_category($op)){
        # gizmo_display() must exist in your app.
        my $content = $self->gizmo_display($op, $gizmo);

        $self->print_portal($content);
    } else {
        $gizmo->handle($op);
    }
}

sub show {
    my $self = shift;
    my $output = undef;

    # This method should be overridden in your app.

    $output = $self->SUPER::show();

    return $output;
}

sub print_portal {
    my $self = shift;
    my $content = shift;
    my $output = undef;
    my $iid = $self->id();
    my ($global_look)= $PARAMS->{globallook};
    $self->debug_msg(3, "App::print_portal($self->id())");

    # This method should be overridden in your app if you want to
    # change the http header information or some aspect of the
    # most basic part of the page layout (header, footer, admin panel)
    # and aren't using the templating system.

    $self->print_http_header();

    if($SESSION->get_value('GB_status') eq '1') {
        print $self->process_tempate($iid, $self->show_template($iid));
    } else {
        if ($global_look ne "usetemplate") {
            $output = $self->show_page_top()
                . $self->show_header($self->id())
                . $self->show_portal_panel($self->id(), $self->app())
                . $content
                . $self->show_bottom(0);
            print $output;
        } else {
            if ($self->is_a eq "Category"){
                print $self->expand($iid, $self->show_template($iid));
            } elsif (($self->is_a eq "Discussion") ||
                     ($self->is_a eq "Table") ||
                     ($self->is_a eq "NewsPaperView") ||
                     ($self->is_a eq "Newsitem")) {
                my $class = ref($self);
                my $d = $class->new($iid);
                my $results = $d->show();
                print $DISPLAY->simple_expand($iid, $self, $self->show_template(-1), $results);
            } else {
                print $self->show_page_top(),
                      $self->show_header($iid),
                      $self->show_portal_panel($iid),
                      $self->show_latest_look($content, $iid),
                      $self->show_bottom;
            }
        }
    }

}

1;

__END__

=head1 NAME

App - base class for building Application interfaces. Application interfaces control the
      layout and presentation of data on a page.

=head1 SYNOPSIS

    use App.pm;

    App->new($id);
    $app->www_show();
    $app->display_controller($op, $gizmo);
    $html = $app->show();
    $app->print_portal($content);

=head1 DESCRIPTION

App is the base class that contains all the methods and overridable methods necessary to
implement an Application interface.

=head1 Methods

The new function is the constructor for the object.

    App->new($id);

This method gets the display data from your application's show method and passes it to print_portal. You should not override this method.

    $app->www_show();

This method checks the gizmo to determine if the gizmo's display data should be wrapped in the display of the Application.
$op is the operation being done to the gizmo.
$gizmo = an instance object of the gizmo

    $app->display_controller($op, $gizmo);

=cut
