###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Auditable;
use strict;

# use Data::Dumper;

use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $CGI $ACCESS_BROKER $DISPLAY);
use Metadot::AuthenticatorFactory;
use DBAccess;

#use CGI qw(:all -oldstyle_urls);
# CKIM: :all imports too many subs, and interferes with Gizmo3000's method making
use CGI qw(-oldstyle_urls);

use HTML::Components;
use HTML::TableFrame;
use Carp qw(confess croak);
use Metadot::DBMap;
use Data::Dumper;
use Metadot::Permissions;
use Metadot::RequestCache::AuditableRequestCache;

use constant LEVEL_NAMES => {
                   0 => 'Public Access',
                   2 => 'Logged In',
                   8 => 'Owner',
                   9 => 'Site Mgr',
                   10 => 'Admin',
                   11 => 'No Access',
                  };


# these ops NEVER require login
use constant FREE_OPS => {
  login=>1,
  login_check=>1,
  logout=>1,
  forgot_password=>1,
  auto_login=>1,
  reject_registration=>1,
  require_icq_form=>1,
  require_pager_form=>1,
  accept_registration=>1,
  change_password=>1,
  password_change_complete=>1,
};


# these ops are permitted for restricted users
use constant RESTRICTED_HOME => 'restricted_user_home';
use constant RESTRICTED_FREE_OPS => {
    modify_self         =>1,
    save_self           =>1,
    login               =>1,
    login_check         =>1,
    logout              =>1,
    forgot_password     =>1,
    auto_login          =>1,
    search              =>1,
    edit_on                     =>1,
    edit_off                =>1,
    RESTRICTED_HOME()   =>1,
};



# should always be overridden in derived classes
sub get_AC {
  my $self = shift;
  my $class = ref $self || $self;

  die "&get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

  die "Thou shalt never instantiate Auditables, for they are abstract.";

}


#
# should be overidden in derived classes that can have parents
#
sub get_parent {
  my $self = shift;

  return undef;

}


sub go_back_to_parent_after_editing_permissions {
        return 1;
}


sub not_logged_in_callback {
    my $self = shift;

    my $target = $SESSION->{referer} || 'index.pl';
    $target = $CGI->escape($target);

    my $url1 = Metadot::AuthenticatorFactory->get_authenticator()->get_login_url("target=$target&email=" . $USER->email);

    Utility->md_redirect($url1);
    exit(0);
}


sub delete_permissions {
    my $self = shift;

    my $id = $self->id();
    my $isa = $self->is_a();

    # check if already in the table

    if($isa and defined $id) {
        # XXX move this to Permissions.pm
        my ($count) = DBAccess->sqlSelect("count(*)", "permissions", "id=$id AND isa='$isa'");

        if($count == 1) {
            # XXX move this to Permissions.pm
            DBAccess->sqlDelete("permissions",
                                 "id=$id AND isa='$isa'");
        } elsif($count != 0) {
            croak "should never happen";
        }
    }
}


sub save_permissions {
    my $self = shift;

    $DISPLAY->debug_msg(4, "IN save_permissions");

    Metadot::RequestCache::AuditableRequestCache->drop_cache();

    my $id = $self->get_iid();
    my $isa = $self->is_a();

    my $perms = $self->_get_permissions();
    #my $frozen_perms = $perms->frozen();
    my $frozen_perms = $perms->serialize();

    # check if already in the table
    # XXX move these direct db calls to Permssions.pm

    my ($count) = DBAccess->sqlSelect("count(*)", "permissions", "id=$id AND isa='$isa'");


#     # TEMPORARY: debug all permission saves
#     ##########################################
#     my $logfile = '/tmp/md_permissions_log';
#
#     if (open (PERMISSIONS_LOG, ">>$logfile")) {
#         my $server_name = MetadotConfig->get_server_name();
#         print PERMISSIONS_LOG '='x72, "\n",
#             localtime(time).": writing permissions for $server_name :: $isa :: $id :: count=$count\n",
#             Dumper($perms),"\n";
#
#         close PERMISSIONS_LOG;
#     } else {
#         print STDERR "Could not open log file '$logfile'\n";
#     }


    ##########################################

    if($count == 0) {
        # insert it anew
        # XXX do we need locking?
        DBAccess->sqlInsert('permissions',
                             id => $id,
                             isa => $isa,
                             permissions => $frozen_perms);

    } elsif($count == 1) {
        # update existing row
        DBAccess->sqlUpdate("permissions",
                             "id=$id AND isa='$isa'",
                             permissions => $frozen_perms);

    } else {
        croak "should never happen";
    }
}


sub get_bundle_level {
  my $self = shift;
  my $bundle_id = shift || croak "must always provide an bundle_id: $!";

  my $result = $self->_get_permissions->get_bundle_level($bundle_id);

  # if the level is undefined, this means the gizmo existed before
  # the bundle was defined. therefore, use the default value
  $result = $self->_get_default_level($bundle_id) unless (defined $result);

  return $result;
}

sub set_bundle_level {
  my $self = shift;
  my $bundle_id = shift || croak "must always provide bundle_id: $!";
  my $level = shift;

  croak "must always provide a level" unless defined $level;

  $self->_get_permissions->set_bundle_level($bundle_id, $level);

}


# isn't private because used within upgrade script
sub get_bundle_of_op {
  my $self = shift;

  my $op =  shift || die 'must include op';

  my $ac = $self->get_AC();

  unless(defined $ac->{OPS}) {
    foreach my $bundle_id ($self->_get_bundles()) {
      foreach my $bop ($self->_get_bundle_operations($bundle_id)) {
        $ac->{OPS}{$bop} = $bundle_id;
      }
    }
  }

  return $ac->{OPS}{$op};

}




#The function will be called as is_allowed_to_do($USER, $op).
#It will return a 1 on success, and an undef on failure.
{
  my $cached_parent;
  my $expiration_stamp;

  sub is_allowed_to_do {
    my $self = shift;
    my $op = shift || confess "must provide operation name";
    my $user = shift || $USER;
    my $activate_logged_in_cb = shift || undef;
    my $ignore_logged_in_status = shift || undef;
    my $result;

    my $op_bundle = $self->get_bundle_of_op($op);
    $self->debug_msg(4, "$self\->is_allowed_to_do, id='".($self->{id}||'')."', op='".($op || '')."', user='".($user || '')."', bundle='".($op_bundle || '')."'");

    # check the request cache
    $result = Metadot::RequestCache::AuditableRequestCache->check_cache($self, $op_bundle, $user);

    unless (defined $result) {

        # not found in the request cache, so we trudge on.
        $result = 1;

        # if op is not listed in the AC hash then it is never allowed
        $result &&= ($op_bundle);

        # [vadim] we don't want these operations:
        # cut, delete
        # being accessible for the pre-defined (Core) categories (and we already defended this categories
        # in the sampledata.sql and sampledata_closed.sql)  :-)
        $result &&= (! (($self->can('id') && defined($self->id()) && ($self->id() le 0)) && (defined($op) && $op =~ /^(cut|delete)$/)) );

        my $perms;
        my $userlevel = 0;

        # all Gizmo permissions are first dependent on the navigation parent's "show" op
        my $parent;

        if ($result) {

            $perms = $self->_get_permissions();
            $self->debug_msg(5, "Got permissions object $perms");

            # determining the level of permission the user has on the object

            $userlevel = 2 if( $user->is_logged_in()
                  or $ignore_logged_in_status or not Metadotclass->on_the_web());
            $userlevel = 8 if(($user->uid) && $self->get_owner_uid() && ($self->get_owner_uid() == $user->uid()));
            $userlevel = 9 if($USER->belongs_to_grp('SITE_MGR', $user->{uid}));
            $userlevel = 10 if($USER->belongs_to_grp('ADMIN', $user->{uid}));

            #return undef unless(($userlevel >= $perms->get_bundle_level($op_bundle)) ||
            #                   $perms->is_user_in_acl($op_bundle, $user));
            my $bundle_level = $perms->get_bundle_level($op_bundle);
            $self->debug_msg(5, "Bundle $op_bundle starts with  bundle_level $bundle_level; user_level starts as $userlevel");

            #RESTRICTED-USER: make sure the bundle_level is always higher than the user's level
            if($user->is_restricted()) {
                if ($bundle_level < 8){
                    $bundle_level = 8;
                }
            }


            #### CHECK CONTENT SCHEDULING
            if ($op_bundle eq 'DISP' && ( ($self->{show_from} || $self->{show_end}) && $self->can('is_show_time') && !$self->is_show_time() )) {
                if ($SESSION->admin_mode() && $self->is_allowed_to_do('modify', $USER)){
                    ## nothing to do - modifiers can see in edit mode
                } else {
                    return 0;
                }
            }
            $self->debug_msg(5, "Content scheduling check passed...");
            ### END CHECK CONTENT SCHEDULING


            ### CHECK BROADCAST
            if ($op_bundle eq 'BROADCAST' || $self->_is_broadcast($op_bundle) ) {
                $self->debug_msg(5, "This is a broadcast op");
                unless (Portal->is_broadcast_email_enabled() || $ACCESS_BROKER->is_allowed_to_do('admin_tasks')) {
                    $self->debug_msg(5, "User is not permitted to broadcast");
                    return 0;
                }
            }
            $self->debug_msg(5, "Broadcast check passed...");
            ### END CHECK BROADCAST

            # if the site requires login, bundle_level begins at 2
            # but allow certain ops, such as login, to pass thru
            if (!$PARAMS->{registration_closed}) {
                FREE_OPS()->{register} = 1;
                FREE_OPS()->{register_form}=1;
            }

            # CKIM-LOGIN: determine whether login will be required here
            my $require_login = ( $PARAMS->{require_login} && !FREE_OPS()->{$op} );
            if ( $require_login &&
                 !( $user->is_logged_in()
                 or $ignore_logged_in_status or not Metadotclass->on_the_web() ) ) {

                $SESSION->{referer} = $CGI->url(-path_info=>1, -query=>1);

                # XXX [claudio] had to add this statement to fix
                # the newstyle_url query string returned by CGI.pm
                $SESSION->{referer} =~ tr/;/&/;

                $self->not_logged_in_callback();
            }
            $self->debug_msg(5, "Login requirement check passed...");


            $self->debug_msg(5, "Checking userlevel ".(defined($userlevel) ? $userlevel : '[undef]')." against bundle level ".(defined($userlevel) ? $bundle_level : '[undef]'));
            unless(($userlevel >= $bundle_level) ||
                    $perms->is_user_in_acl($op_bundle, $user)) {

                #$self->not_logged_in_callback() if ($bundle_level == 2 && $activate_logged_in_cb);
                # CKIM: the callback was not setting referer as target
                # RESTRICTED-USER: changed the value below from 2 to 1
                if ($userlevel < 1 && $activate_logged_in_cb) {
                    $SESSION->{referer} = $CGI->url(-path_info=>1, -query=>1);

                    # XXX [claudio] had to add this statement to fix
                    # the newstyle_url query string returned by CGI.pm
                    $SESSION->{referer} =~ tr/;/&/;
                    $self->not_logged_in_callback();
                }

                $result = 0;
            }
        }

        my $parent_bundle;
        if ($result) {
            if($parent_bundle = $self->_get_parent_dependency($op_bundle, $op)) {
                $parent ||= $self->get_parent();
                if(defined $parent) {
                    # any op in the parent will do
                    my ($parent_op) = ($parent->_get_bundle_operations($parent_bundle));

                    $self->debug_msg(5, "Checking in parent ops...");
                    $result &&= ($parent->is_allowed_to_do($parent_op, $user));
                }
                # if parent is not defined it means the parent allows the op
            }
        }

        if ($result) {
            my $perm_bundle_level = $perms->get_bundle_level($op_bundle);
            if(defined($perm_bundle_level) && $perm_bundle_level >= 2) {
                # check if user is_logged in if not then execute the login sequence page
                unless( $user->is_logged_in()
                or $ignore_logged_in_status or not Metadotclass->on_the_web() ) {

                    # redirect and everything
                    # CKIM: the callback was not setting referer as target
                    if ($activate_logged_in_cb) {
                        $SESSION->{referer} = $CGI->url(-path_info=>1, -query=>1);

                        # XXX [claudio] had to add this statement to fix
                        # the newstyle_url query string returned by CGI.pm
                        $SESSION->{referer} =~ tr/;/&/;
                        $self->not_logged_in_callback();
                    }
                    $result = 0;
                }
            }
        }

        # check to see if recursive permissions
        if ( !$result && $perms && $perms->is_recursive() ){
            $parent ||= $self->get_parent();
            if (defined $parent) {
                # get an op from the same bundle in the parent. This is because the parent
                # may not directly support the same op.
                my ($parent_op) = ($parent->_get_bundle_operations($op_bundle));
                if ($parent_op) {
                    $self->debug_msg(5, "Checking inheritance in parent ops...");
                    $result ||= $parent->is_allowed_to_do($parent_op, $user, $activate_logged_in_cb);
                }
            }
        }

        #RESTRICTED-USER
        if (!$result && $user->is_restricted()) {
            if (RESTRICTED_FREE_OPS()->{$op}) {
                # user is restricted, but the op is allowed
                $result = 1;
            }
        }

        Metadot::RequestCache::AuditableRequestCache->update_cache($self, $op_bundle, $user, $result);
    }


    $self->debug_msg(4,"$self\->is_allowed_to_do() returns '".($result||'')."'");
    return ($result);
  }
}

sub get_acl_groups {
    my $self = shift;
    my $perms = $self->_get_permissions();
    my $result = $perms->get_acl_groups();
    return $result;
    }

sub get_acl_users {
    my $self = shift;
    my $perms = $self->_get_permissions();
    my $result = $perms->get_acl_users();
    return $result;
}

sub is_group_acl_pair_there {
    my $self        = shift;
    my $grp_id      = shift || die("Must specify grp_id");
    my $bundle_id   = shift || die("Must specify bundle id");

    my $perms = $self->_get_permissions();
    my $result = $perms->is_group_acl_pair_there($grp_id, $bundle_id);
    return $result;
}

sub is_user_acl_pair_there {
    my $self        = shift;
    my $userid      = shift || die("Must specify userid");
    my $bundle_id   = shift || die("Must specify bundle id");

    my $perms = $self->_get_permissions();
    my $result = $perms->is_user_acl_pair_there($userid, $bundle_id);
    return $result;
}

sub get_groups_who_can {
    my $self = shift;
    my $bundle_id = shift || die("Must specify bundle id");
    my @results = ();

    my $grps = $self->get_acl_groups();
    foreach my $grp (@$grps) {
        if ($self->is_group_acl_pair_there($grp, $bundle_id)) {
            push @results, $grp;
        }
    }
    return \@results;
}

# this method does not look into the groups, only the users
sub get_users_who_can {
    my $self = shift;
    my $bundle_id = shift || die("Must specify bundle id");
    my @results = ();

    my $users = $self->get_acl_users();
    foreach my $user (@$users) {
        if ($self->is_user_acl_pair_there($user, $bundle_id)) {
            push @results, $user;
        }
    }
    my $owner = $self->uid();
    push @results, $owner;

    return \@results;
}

sub get_all_who_can {
    my $self = shift;
    my $bundle_id = shift || die("Must specify bundle id");
    my @results = @{ $self->get_users_who_can($bundle_id) };
    my $groups = $self->get_groups_who_can($bundle_id);
    foreach my $group (@$groups) {
        push @results, Group->get_member_uids($group);
    }

    return \@results;
}

sub edit_permissions_form {
    my $self = shift;
    my $form_params = shift || confess "must include reference to form params hash";   # form params hash
    my $done_redirect = shift || confess "must include done redirect code ref";
    my $user = shift || confess "must provide reference to user object";

    my $owner_uid = $self->uid();
    my $owner_fullname = $USER->get_fullname_from_uid($owner_uid);

    my $perms = $self->_get_permissions();

    my $mode = $form_params->{mode} || 'replace';

    my $recursive;
    unless (defined $recursive) {

        # all modes except replace are recursive
        $recursive = ($mode eq 'replace') ? 0 : 1;
    }


    my $isa = $self->is_a();
    my $id = $self->id();
    my $instance_name = $self->name;
    my $isa_plus_name_title = "<b>$instance_name (" . $self->get_gizmo_name() . ")</b>";

    my $code = sub {
        my $form_params = shift;
        my $submit_buttons = shift;

        my $changed;

        ### process submissions
        # process add group
        if($form_params->{submit_id} eq 'ADD_GROUP') {
            my $group = $form_params->{add_group};
            if($group && ($group ne '')) {
                $form_params->{add_group} = '';
                $perms->add_group_acl($group);
                $changed++;
                foreach my $bundle_id (@{$self->_get_default_on_bundles()}) {
                    $form_params->{"group_acl_$bundle_id\____$group"} = 'on';
                }
            }
        }

        #process add user
        if ($form_params->{submit_id} eq 'ADD_USER') {
            foreach my $param (keys %$form_params) {
                if ($param =~ m/^user_overrides/) { # skip this param type
                    next;
                }
                #otherwise:
                if ($param =~ m/^user(.+)/) {
                    my $uid = $1;
                    $form_params->{$param} = '';
                    $perms->add_user_acl($uid);
                    $changed++;
                    foreach my $bundle_id (@{$self->_get_default_on_bundles()}) {
                        $form_params->{"usr_acl_$bundle_id\____$uid"} = 'on';
                    }
                    $form_params->{"user_overrides_$uid"} = 0;
                }
            }
        }


        # generate confirm page in case of recursive request
        if ($recursive && $form_params->{submit_id} eq 'DONE' && !defined $form_params->{confirmed}) {

            my $confirm = qq(<input type="hidden" name="confirmed" value="1">\n);

            # record general access permissions
            if ( $mode eq 'replace' or $mode eq 'replace_rec' ) {

                my @ordered_bundles = $self->_get_ordered_operation_bundles( $recursive );

                foreach my $bundle_id (@ordered_bundles) {

                    if (defined $form_params->{$bundle_id}) {

                        my $value = $form_params->{$bundle_id};
                        $confirm .= qq(<input type="hidden" name="$bundle_id" value="$value">\n);

                    } else {

                        die "SP: bundle $bundle_id SHOULD BE PRESENT";
                    }
                }
            }

            # record user/group access permissions
            while (my ($param, $value) = each %$form_params) {

                if($param =~ /^group_acl_(.+)____(.+)$/) {

                    $confirm .= qq(<input type="hidden" name="$param" value="$value">\n);

                } elsif ($param =~ /^usr_acl_(.+)____(.+)$/) {

                    $confirm .= qq(<input type="hidden" name="$param" value="$value">\n);
                }
                #aju
                elsif ($param =~ /^user_overrides_(.+)$/) {
                    $confirm .= qq(<input type="hidden" name="$param" value="$value">\n);
                }
            }

            my $gizmo_count = $self->node_count_by_gizmo;
            $confirm .= qq(<br /><h3> This operation will affect the following Gizmos: </h3><p />);
            $confirm .= qq[(Starting point is $isa_plus_name_title)<p />];
            $confirm .= qq(<table>);

            while (my ($gizmo, $count) = each %$gizmo_count) {

                $confirm .=  qq(<tr><td align="right"><b>$gizmo</b>:</td><td align="left">$count</td></tr>);
            }

            $confirm .= "</table><p />";
            $confirm .= $submit_buttons->{DONE};
            $confirm .= $submit_buttons->{CANCEL};

            # notice that we return here, and avoid processing the DONE button
            return $confirm;
        }

        # process done button
        if ($form_params->{submit_id} eq 'DONE') {

        $self->debug_msg(4, "Processing permission changes.");

            my $mod_perms = sub {
                my $gizmo = shift;

                my $perms = $gizmo->_get_permissions();

                my $changed;

                if ( $mode eq 'replace' or $mode eq 'replace_rec' ) { $perms->remove_all_acls() }

                foreach my $param (keys %$form_params) {

                    if( $param =~ /^group_acl_(.+)____(.+)$/ ) {

                        my $bundle_id = $1;
                        my $group = $2;

                        if ( $mode =~ /add_rec|replace|replace_rec/ ) {

                $self->debug_msg(5, "Adding/replacing recursively");
                            if( $gizmo->_has_bundle( $bundle_id ) ) {

                                $perms->add_group_acl($group);
                                $perms->add_group_acl_pair($group, $bundle_id);
                            }

                        } else {

                            if( $gizmo->_has_bundle( $bundle_id ) ) {

                                $perms->remove_group_acl_pair($group, $bundle_id);
                            }
                        }
                        $changed++;

                    } elsif($param =~ /^usr_acl_(.+)____(.+)$/) {

                        my $bundle_id = $1;
                        my $uid = $2;

                        if ($mode =~ /add_rec|replace|replace_rec/) {

                            if( $gizmo->_has_bundle( $bundle_id ) ) {

                                $perms->add_user_acl($uid);
                                $perms->add_user_acl_pair($uid, $bundle_id);
                            }

                        } else {

                            if( $gizmo->_has_bundle( $bundle_id ) ) {

                                $perms->remove_user_acl_pair($uid, $bundle_id);
                            }
                        }

                        $changed++;
                    }
                    elsif ($param =~ m/^user_overrides_(.+)/) {
                        my $uid = $1;
#                       print STDERR "adding user override 'bundle' for $uid\n";
                        $perms->add_user_acl_overrides($uid);
                    }

                }

                $perms->remove_empty_acls();

                # process general access permissions
                if ( $mode eq 'replace' or $mode eq 'replace_rec' ) {

                    my @bundles = $gizmo->_get_bundles();

                    foreach my $bundle_id (@bundles) {

                        if( defined $form_params->{$bundle_id} ) {

                            # this check is redundant, but we'll leave it anyway
                            if( $gizmo->_has_bundle( $bundle_id ) ) {

                                # XXX [claudio] verify that level is above minimum
                                # and that user has edit_permissions access to this gizmo
                                $perms->set_bundle_level($bundle_id, $form_params->{$bundle_id});
                                $changed++;
                            }

                        } else {

                            die "SP: bundle $bundle_id SHOULD BE PRESENT";

                        }
                    }


                    if ($perms->is_recursive() != ($form_params->{inherit_perms} eq 'on')) {
                        $perms->set_recursive( ($form_params->{inherit_perms} eq 'on') );
                        $changed++;
                    }

                }
                # save if modified
                $gizmo->save() if($changed);
            };

            # Will apply permission changes recursively
            $self->apply($recursive, $mod_perms);
            &$done_redirect;
            # finished processing done button
        }


        # process cancel button
        if ($form_params->{submit_id} eq 'CANCEL') {
            $perms->remove_empty_acls();
            $self->save();
            &$done_redirect;
        }

        # process search user
        my $select_user_form = '';
        if($form_params->{submit_id} eq 'SEARCH_USER') {
            $select_user_form = $USER->get_search_user_results({
                                                                EMAIL => "$form_params->{email}",
                                                                FULLNAME => "$form_params->{fullname}",
                                                                NICKNAME => "$form_params->{nickname}",
                                                               },
                                                               $self->{ACL}{FOR_USERS});
            if ($select_user_form =~ m/checkbox/) {
                $select_user_form .= "$submit_buttons->{ADD_USER}\n";
            }
        }


        # save if modified
        $self->save() if($changed);


        #
        #
        # PAGE GENERATION FOLLOWS
        #
        #
        my $show_recursive_perms = $PARAMS->{recursive_perms} &&
          $self->can('allows_recursive_permissions') &&
            $self->allows_recursive_permissions();

        if( $mode eq 'advanced_menu' ) {

            my $table = "<h1>Advanced Permissions Operations</h1>\n";

            $table .= "<p />$isa_plus_name_title<p />";

            $table .= "<table cellspacing=\"15\">";

            if ( $show_recursive_perms ) {

                $table .= "<tr><td valign='top'><a href = '".$DISPLAY->escapeHREF("index.pl?op=edit_permissions&mode=add_rec&isa=$isa&iid=$id")."'>Add Permissions Recursively</a></td>\n";

                $table .= "<td>Use this option to add users or groups to the Access Control Lists of this container and all its descendants.</td></tr>\n";

                $table .= "<tr><td valign='top'><a href = '".$DISPLAY->escapeHREF("index.pl?op=edit_permissions&mode=remove_rec&isa=$isa&iid=$id")."'>Remove Permissions Recursively</a></td>\n";

                $table .= "<td>Use this option to remove users or groups from the Access Control Lists of this container and all its descendants.</td></tr>\n";

                $table .= "<tr><td valign='top'><a href = '".$DISPLAY->escapeHREF("index.pl?op=edit_permissions&mode=replace_rec&isa=$isa&iid=$id")."'>Replace Permissions Recursively</a></td>\n";

                $table .= "<td>Use this option to completely replace Minimum Permissions and Access Control Lists of this container and all its descendants.</td></tr>\n";

            }

            if ( $owner_uid == $USER->{uid} ||
                 $ACCESS_BROKER->is_allowed_to_do('admin_tasks') ) {

                $table .= "<tr><td valign='top'><a href = '".$DISPLAY->escapeHREF("index.pl?op=change_owner&isa=$isa&iid=$id")."'>Change Owner</a></td>";

                $table .= '<td>Use this option to change the owner of this gizmo</td></tr>';

            }

            $table .= '</table>';

            # note that we return here
            return $table;
        }


        my $title;
        my ($help_line, $help_line_row) = ('', '');

        $title = 'Edit Permissions' if $mode eq 'replace';
        $title = 'Replace Permissions Recursively' if $mode eq 'replace_rec';

        if ($mode eq 'add_rec') {
            $title = 'Add Permissions Recursively';
            $help_line = 'Select Groups and Members and check all the operations for which you want to add permissions.';
        }

        if( $mode eq 'remove_rec' ) {
            $title = 'Remove Permissions Recursively';
            $help_line = 'Select Groups and Members and check all the operations for which you want to remove permissions.';
        }

        $help_line && ( $help_line_row = "<tr><td align='left' colspan='3'>$help_line</td></tr>" );

        # make done button row
        my $top_done_button_row = "<tr><td></td><td></td><td align='right'>$submit_buttons->{DONE} &nbsp; $submit_buttons->{CANCEL}</td></tr>\n";

        my $top_control_row = "<tr> <td align = left>Owner: <b>$owner_fullname</b></td><td></td>";

        if ( $mode eq 'replace' ) {

            $top_control_row .= "<td align = right>";

            if ( ( $show_recursive_perms ) or ( $self->can_change_owner() && ( $owner_uid == $USER->{uid} || $ACCESS_BROKER->is_allowed_to_do('admin_tasks') ) ) ) {

                $top_control_row .=
                  "<a href = index.pl?op=edit_permissions&amp;mode=advanced_menu&amp;isa=$isa&amp;iid=$id>Advanced</a> &nbsp;";

            }

        } else {

            $top_control_row .= "<td></td></tr>";

        }

        my $headings .= "<br /><br /><h1>$title</h1>";
        $headings .= "<p />$isa_plus_name_title<p />";
        $headings .= "<table width = 100%>
                $top_done_button_row
                $top_control_row
                $help_line_row
                </table>
        ";

        my @ordered_bundles = $self->_get_ordered_operation_bundles( $recursive );

        my $num_columns = ( @ordered_bundles ) + 1;

        my $min_permissions_row = "<tr><td colspan=$num_columns align='center' bgcolor = #bbbbbb>Minimum Permissions</td></tr>\n";



        # make inherit permissions row
        my $inherit_check;
        if ( defined $form_params->{inherit_perms} ) {
            $inherit_check = $form_params->{inherit_perms};
        } else {
            $inherit_check = $perms->is_recursive();
        }

        ($inherit_check) && ($inherit_check = ' checked ');

        my $inherit_perms_row = "<tr><td colspan=$num_columns align='center' valign='baseline' bgcolor=\"#bbbbbb\">"
          . "<input type=checkbox $inherit_check name=\"inherit_perms\">&nbsp;\n"
            . "<b>Inherit permissions from parent</b>\n"
              . "</td></tr>\n";



        # make first two rows of the table
        my $titles_row = '<tr><td></td>';
        my $base_perms_row = '<tr><td>&nbsp;</td>';

        my $top_titles_row = '<tr><td></td>';

        foreach my $bundle_id (@ordered_bundles) {

            my $bundle_desc = $self->_get_bundle_description( $bundle_id, $recursive );

            $titles_row .= "<td valign='bottom' align='center'><b>$bundle_desc</b></td>";

            # XXX for now default level for bundles only in children is 2
            my $current_level = ( defined $form_params->{$bundle_id} ) ?
              $form_params->{$bundle_id} : $perms->get_bundle_level( $bundle_id );

            unless(defined $current_level) { $current_level = 8 }

            my @levels = $self->_get_bundle_allowed_levels( $bundle_id, $USER, $recursive );

            my $num_levels = @levels;

            if ($num_levels < 2) {

                $base_perms_row .= "<input type=hidden value=$current_level name='$bundle_id'>"

            } else {

                $base_perms_row .= "<td valign = top align = center><select name = '$bundle_id'>\n";
                $top_titles_row .= "<td valign = bottom align = center><b>$bundle_desc</b></td>";
                foreach my $level (@levels) {

                    my $lev_name = $self->_get_level_name($level);
                    if($current_level == $level) {

                        $base_perms_row.= "<option SELECTED value='$level'>$lev_name</option>";
                    } else {

                        $base_perms_row.= "<option value='$level'>$lev_name</option>";
                    }
                }
            }
            $base_perms_row .= "</select></td>\n";
        }

        $top_titles_row .= "</tr>\n";
        $titles_row.="</tr>\n";
        my $explanation_row = "<tr><td colspan = $num_columns align = left>Set the minimum permissions to Public Access if you want everyone to be able to view (or edit) your $isa.

Alternatively, you can restrict viewing (or editing) to only logged in users, or to you, the owner.

If you want to let specific groups or specific people do things with your $isa, but not the general public or logged in users, set the minimum permissions to owner,

add a user or group using the boxes below, and then give that user or group the specific permissions that you wish.</td></tr>";

        $base_perms_row .= "</tr>\n";

        # make group rows
        my $group_rows;
        foreach my $grp_id (@{$perms->get_acl_groups()}) {

            my $group = Group->get_group_name_by_grp_id($grp_id);

            # only show those of groups that have not been deleted
            # when perms are saved ACL items of deleted groups will go away
            if($group) {

                my $in_use;
                my $new_row = "<tr><td align='left'>$group</td>";

                foreach my $bundle_id (@ordered_bundles) {

                    my $checked = '';
                    if(exists $form_params->{submit_id}) {

                        $checked = 'CHECKED' if(exists $form_params->{"group_acl_$bundle_id\____$grp_id"});

                    } else {

                        $checked = 'CHECKED' if( ( $mode eq 'replace' or $mode eq 'replace_rec' ) &&
                                                 $perms->is_group_acl_pair_there($grp_id, $bundle_id) );
                    }

                    $in_use++ if $checked;

                    $new_row .= "<td align='center'><INPUT TYPE=checkbox name=\"group_acl_$bundle_id";
                    $new_row .= "____$grp_id\" $checked></td>";
                }

                if( $in_use ) { $group_rows .= "$new_row</tr>\n" };
            }
        }

        if ($group_rows) {

            $group_rows = "<tr><td bgcolor = #bbbbbb colspan = $num_columns align='center'>Groups</td></tr>" . $titles_row . $group_rows;
        }

        # make individual user rows
        my $user_rows;

        foreach my $uid ( @{$perms->get_acl_users()} ) {
            my $user_name;

            unless ( $user_name = $USER->get_fullname_from_uid( $uid ) ) {

                unless ( $user_name = $USER->get_nickname_from_uid( $uid ) ) {

                    $user_name = $USER->get_email_from_uid( $uid );
                }
            }

            next unless $user_name;

            my $in_use;
            my $new_row = "<tr><td align='left'>$user_name</td>";
            my $ocheck;
            if (exists $form_params->{submit_id}) {
                if ($form_params->{"user_overrides_$uid"}) {
                    $ocheck = "checked";
                }
            }
            elsif (( $mode eq 'replace' or $mode eq 'replace_rec' ) &&
                   $perms->is_user_acl_pair_there($uid, 'user_overrides') ) {
                $ocheck = "checked"; $in_use++;
            }
            foreach my $bundle_id (@ordered_bundles) {
                my $checked = '';

                if (exists $form_params->{submit_id}) {

                    $checked = 'CHECKED' if (exists $form_params->{"usr_acl_$bundle_id\____$uid"});

                } else {

                    $checked = 'CHECKED' if( ( $mode eq 'replace' or $mode eq 'replace_rec' ) &&
                                             $perms->is_user_acl_pair_there($uid, $bundle_id) );

                }

                $in_use++ if $checked;

                $new_row .= "<td align='center'><input type=checkbox name=usr_acl_$bundle_id";
                $new_row .= "____$uid $checked></td>\n";
            }
            $new_row .= "<td>User overrides group permissions: </td>" .
                "<td align='center'><input type=checkbox name=user_overrides_$uid $ocheck></td>";

            if( $in_use ) { $user_rows .= "$new_row</tr>\n" };

        }

        if ($user_rows) {

            $user_rows = "<tr><td bgcolor = #bbbbbb colspan = $num_columns align='center'>Users</td>" . $titles_row . $user_rows;
        }

        # make add-group form
        my $add_group_form = $self->_show_add_group_form( submit_button  => $submit_buttons->{ADD_GROUP},
                                                          selected_group => $form_params->{add_group}
                                                        );

        # make add-individual user form
        my $message = $select_user_form || "Partially or completely fill in one or more pieces of information about a person you would like to include in the permissions list.";

        my $add_user_form = Components->get_search_users({HEADER => "Search and Select Members",
                                                          BUTTON => "$submit_buttons->{SEARCH_USER}",
                                                          MESSAGE => $message,
                                                          ORIENTATION => "horizontal",
                                                          WIDTH => "100%",
                                                         });

        my $top_of_page;
        if( $mode eq 'replace' ) {

            $top_of_page = $min_permissions_row . $top_titles_row . $base_perms_row . $explanation_row . $inherit_perms_row . $group_rows . $user_rows;
        } elsif( $mode eq 'replace_rec' ) {

            $top_of_page = $min_permissions_row . $top_titles_row . $base_perms_row . $explanation_row . $group_rows . $user_rows;

        } else {

            $top_of_page = $group_rows . $user_rows;
        }

        # make done button row
        my $bottom_done_button_row = "<table width=\"100%\"><tr><td colspan=$num_columns align='right'>$submit_buttons->{DONE} &nbsp; $submit_buttons->{CANCEL}</td></tr></table>\n";

        # join everything together
        my $perms_table =
          '<table width="100%">' .
            $top_of_page .
              '</table>'.
                $add_group_form . '<br />' .
                  $add_user_form .
                    $bottom_done_button_row;

        return $headings . $perms_table;
    };


    my $form_config = {
                       CODE => $code,
                       PARAMS_HASH => $form_params,
                       SUBMIT_BUTTONS => {
                                          DONE => { ACTION => "index.pl",
                                                    LABEL => 'Save Changes',
                                                    PARAMS => { id => $id,
                                                                isa =>$isa,
                                                                op => 'edit_permissions',
                                                                mode => $mode,
                                                                submit_id => 'DONE'
                                                              },
                                                  },
                                          ADD_GROUP => { ACTION => "index.pl",
                                                         LABEL => 'Select',
                                                         PARAMS => { id => $id,
                                                                     isa =>$isa,
                                                                     op => 'edit_permissions',
                                                                     mode => $mode,
                                                                     submit_id => 'ADD_GROUP'
                                                                   },
                                                       },
                                          CANCEL => { ACTION => "index.pl",
                                                      LABEL => 'Cancel',
                                                      PARAMS => { id => $id,
                                                                  isa =>$isa,
                                                                  op => 'edit_permissions',
                                                                  mode => $mode,
                                                                  submit_id => 'CANCEL'
                                                                },
                                                    },
                                          SEARCH_USER => { ACTION => "index.pl",
                                                           LABEL => 'Search',
                                                           PARAMS => { id => $id,
                                                                       isa =>$isa,
                                                                       op => 'edit_permissions',
                                                                       mode => $mode,
                                                                       submit_id => 'SEARCH_USER'
                                                                     },
                                                         },
                                          ADD_USER => { ACTION => "index.pl",
                                                        LABEL => 'Select',
                                                        PARAMS => { id => $id,
                                                                    isa => $isa,
                                                                    op => 'edit_permissions',
                                                                    mode => $mode,
                                                                    submit_id => 'ADD_USER'
                                                                  }
                                                      },
                                         },
                      };


    my $page = Components->get_multi_submit_form($form_config);

    return $page;

}


#CKIM-SENDABLE: factored this out of sub edit_permissions_form
sub _show_add_group_form {
    my ($self, %params) = @_;

    my $message_text = $params{message_text} || '<H2>Select a Group</H2>Choose a group you would like to include in the permissions list<br />';
    my $submit_button = $params{submit_button} || '<input type="submit" name="add_new_group" value="Select">';
    my $selected_group = $params{selected_group} || '';

    my $add_group_form ="<table width = 100%><tr colspan = 3><td>$message_text";
    $add_group_form .="Group: <select name='add_group'><option value=''>--</option>";

    # XXX [claudio] group names should ideally come from a GM classwide method

    #CKIM-SENDABLE: Phillipe asked to sort the groups by name
    my $grp_ids = Group->get_grp_ids();
    my %grp_name_lookup;
    foreach my $grp_id (@$grp_ids) {

        unless( Group->is_hidden( $grp_id ) ) {

            $grp_name_lookup{$grp_id} = Group->get_group_name_by_grp_id($grp_id);
        }
    }

    my @sorted_grp_ids = sort {$grp_name_lookup{$a} cmp $grp_name_lookup{$b};} @$grp_ids;
    my $no_admin = $params{no_admin};
    foreach my $grp_id (@sorted_grp_ids) {
        my $selected = ($grp_id eq $selected_group ) ?
          'SELECTED' : '';

        my $group_name = $grp_name_lookup{$grp_id};
        if($group_name) {
            if ((lc $group_name) eq 'admin') {
                if ($no_admin) { #  if 'no_admin' flag is set, don't add admin group to list.
                    next;
                }
            }

            $add_group_form .= "<option $selected value='$grp_id'>";
            $add_group_form .= $group_name;
            $add_group_form .= "</option>\n";
        }
    }
    $add_group_form .= "</select>$submit_button\n</td></tr></table>";

    my $frame_add_group_form = new HTML::TableFrame("#333333", "#efefef", "100%");
    $frame_add_group_form->add($add_group_form);
    $add_group_form = $frame_add_group_form->print;

    return $add_group_form;
}


sub init_permissions {
    my $self = shift;

    ref $self || confess "not to be called as a classwide method";
    my $perms;

    $self->debug_msg(4, "$self\->init_permissions");

    # if a parent was defined, build permissions based on the parent
    my $parent = $self->get_parent();
    $self->debug_msg(5, "$self\->init_permissions has parent ".(defined $parent ? $parent : 'undef'));

    if (defined $parent) {
        my $parent_perms = $parent->_get_permissions();

        $perms = Metadot::Permissions->new();

        foreach my $bundle_id ($self->_get_bundles()) {
            my $level;

            if($self->_inherits_from_parent($bundle_id) and (defined $parent) and
               #(defined $parent_perms-get_bundle_level($bundle_id))) {
               # that method now returns a number even if the bundle doesn't exist
               ($parent_perms->has_bundle($bundle_id))) {

                $level = ($parent_perms->get_bundle_level($bundle_id) >= $self->_get_minimum_level($bundle_id)) ?
                    $parent_perms->get_bundle_level($bundle_id) : $self->_get_default_level($bundle_id);

                # XXX [claudio] we never deny at first any operation to the owner of
                # an object. This is here mainly because the edit_permissions level for
                # the root category is 10, and was being inherited down to the whole
                # tree without remedy
                $level = 8 if($level > 8);

            } else {
                $level = $self->_get_default_level($bundle_id);
            }
            $perms->set_bundle_level($bundle_id, $level);


            # inherit ACL of the parent.
            $perms->copy_bundle_acl($bundle_id, $parent_perms) if (defined $parent_perms);

        }

        # inherit recursion property of parent
        $perms->set_recursive(defined($parent_perms) && $parent_perms->is_recursive());
    } else {
        $perms = $self->get_default_permissions_object();
    }

    $self->_set_permissions($perms);
    return $perms;
}



# XXX [claudio] ineficcient and Gizmo/database dependant.
# this subroutine checks a user's permissions for all the children of a specified item.
sub check_permissions_of_children {
    my $self = shift;
    my $id = shift;
    my $result = "";

    # XXX very inefficient
    my ($recordset) = DBAccess->sqlSelectMany("iid, isa", "instance", "parentiid = $id");

  ITERATE: while (my ($item_id, $item_isa) = $recordset->fetchrow()) {
      require "Gizmo.pm";
      my $item = Gizmo->restore($item_id);

      # aju 02-22-02: this check circumvents a server barf if the user
      # is not logged in AND the perl module corresponding to a
      # gizmo is removed from the code base.  Really, need instead
      # to figure out why this check is only needed when user is not
      # logged in...
      if (!($item->can('is_allowed_to_do'))) {
          print STDERR "Warning: $item doesn't have the method 'is_allowed_to_do'\n";
          return 0;
      }
      $result = $item->is_allowed_to_do('modify', $USER);

      if (defined $result && $result){
          last ITERATE;
      }
  }

    return $result;

}


##############################
#                            #
#   PRIVATE METHODS FOLLOW   #
#                            #
##############################


### returns this object's associated Permissions object
sub _get_permissions {
  my $self = shift;

  my $result = $self->{permissions};
  my $id = ($self->can('id') ? $self->id() : undef);

  $self->debug_msg(4, "In $self\->_get_permissions for id=".(defined $id ? $id : 'undef')." with existing permissions '".(defined $result ? $result : 'undef')."'");


  if ($result) {
      $self->debug_msg(5, "_get_permissions ALREADY HAD permissions for id=".(defined $id ? $id : 'undef'));
  } elsif (defined $id) {
      # try to restore permissions from the database
      $result = $self->_restore_permissions();
      $self->debug_msg(5, "_get_permissions tried to restore permissions for id=".(defined $id ? $id : 'undef').", got ".(defined $result ? $result : 'undef'));
  } else {
      # no id, see if I have already initialized the permissions
      $result = $self->{_initial_permissions};
      $self->debug_msg(5, "_get_permissions checkd for pre-initialized permissions, got ".(defined $result ? $result : 'undef'));
  }

  # finally, just initialize new permissions if neither approach works
  unless ($result) {
    $result = $self->init_permissions();
    $self->debug_msg(5, "_get_permissions initialized permissions, got ".(defined $result ? $result : 'undef'));
  }

  return $result;

}


# loads the permissions from the database
sub _restore_permissions {
    my $self = shift;

    my $result;

    my $id = $self->id();
    my $isa = $self->is_a();

    $self->debug_msg(5, "$self restoring permissions bundle for id=".(defined $id ? $id : 'undef')." and isa=".($isa||''));

   # retrieve a frozen perms object from the perms table
    if($isa and defined $id) {
        my $frozen_perms;
        $DISPLAY->debug_msg(5, "Retrieving permissions from the database");
        ($frozen_perms) = DBAccess->sqlSelect("permissions", "permissions", "id=$id AND isa='$isa'");

        # if it finds a row then deserializes the perms object
        if($frozen_perms) {
            #use Data::Dumper; print STDERR "_get_permissions found frozen permissions:\n",Dumper $frozen_perms;
            $DISPLAY->debug_msg(5, "$self Retrieving permissions using frozen perms $frozen_perms");

            $result = Metadot::Permissions->deserialize($frozen_perms);
            $self->_set_permissions($result);
        }
    }

    return $result;
}



### sets this object's associated Permissions object
sub _set_permissions {
  my $self = shift;
  my $set_perms =  shift;

  confess 'must include $set_perms' unless(defined $set_perms);

  # placing non-id permissions in a different place, so we always know whether
  # the permissions were restored for the instance or if they are the defaults
  if ($self->can('id') && defined $self->id()) {
      $self->{permissions} = $set_perms;
      if ($set_perms) {
          # we don't need these any more
          $self->{_initial_permissions} = undef;
      }
  } else {
      $self->{_initial_permissions} = $set_perms;
  }

  return $self;
}


sub _get_level_name {
  my $proto = shift;
  my $level = shift;

  confess "must provide level name" unless defined $level;

  return LEVEL_NAMES->{$level};
}

sub _get_level_numbers {
  return sort {$a <=> $b} (keys %{ LEVEL_NAMES() });
}



sub _get_parent_dependency {
  my $self = shift;
  my $bundle_id = shift || die 'must provide bundle_id';
  my $op = shift || die 'must provide op';

  my $ac = $self->get_AC();

  return $ac->{BUNDLES}{$bundle_id}{PARENT_DEP}{$op};
}


sub _get_bundles {
  my $proto = shift;

  my $class = ref($proto) || $proto;
  my $ac = $class->get_AC();

  return (keys %{$ac->{BUNDLES}});
}


sub _get_ordered_operation_bundles {
  my $proto = shift;
  my $recursive = shift;

  if($recursive) {
      return $proto->get_all_possible_bundles()
  }

  my $class = ref($proto) || $proto;
  my $ac = $class->get_AC();

  return (@{$ac->{DISPLAY_ORDER}});
}


sub _get_default_on_bundles {
  my $proto = shift;

  my $class = ref($proto) || $proto;
  my $ac = $class->get_AC();

  my @result;
  foreach my $bundle_id (keys %{$ac->{BUNDLES}}) {
    push @result, $bundle_id
      if(lc($ac->{BUNDLES}{$bundle_id}{DEFAULTS_TO}) eq 'on');
  }

  return \@result;

}


sub _get_minimum_level {
  my $proto = shift;
  my $bundle_id = shift || confess "must include \$bundle_id";

  my $class = ref($proto) || $proto;
  my $ac = $class->get_AC();

  return $ac->{BUNDLES}{$bundle_id}{MIN_LEVEL};

}

sub _get_default_level {
  my $proto = shift;
  my $bundle_id = shift || confess "must include \$bundle_id";

  my $class = ref($proto) || $proto;
  my $ac = $class->get_AC();

  return $ac->{BUNDLES}{$bundle_id}{DEFAULT_LEVEL};
}

sub _is_broadcast {
    my $proto = shift;
    my $bundle_id = shift || confess "must include \$bundle_id";

    my $class = ref($proto) || $proto;
    my $ac = $class->get_AC();

    my $result = ($ac->{BUNDLES}{$bundle_id}{BROADCAST});

    return $result;
}


sub _inherits_from_parent {
  my $proto = shift;
  my $bundle_id = shift || confess "must include \$bundle_id ";

  my $class = ref($proto) || $proto;
  my $ac = $class->get_AC();

  return 1 unless defined($ac->{BUNDLES}{$bundle_id}{NO_PARENT_INHERITANCE});
  return undef;
}


### creates and returns a Permissions object
### This method should be overridden in any class which implements getting a permissions object by
### getting a blank permissions object and then adding bundles, ops, and acls.
sub get_default_permissions_object {
  my $proto = shift;

  my $class = ref($proto) || $proto;
  my $ac = $class->get_AC();

  return Metadot::Permissions->generate_permissions_object($ac);
}


sub _get_bundle_operations {
  my $proto = shift;
  my $bundle_id =  shift || die 'must include bundle_id';
  my @result;

  my $class = ref($proto) || $proto;


  # CKIM: shouldn't this work?
  #my $ac = $class->get_AC();
  my $ac = $proto->get_AC();

  # have to watch for null pointers!
  if ($ac) {
      my $bundle = $ac->{BUNDLES}{$bundle_id};
      if ($bundle) {
          if ($bundle->{OPS}) {
              @result = @{ $bundle->{OPS} };
          }
      }
  }

  $proto->debug_msg(5, "$proto\->_get_bundle_operations($bundle_id) ($class) returns @result");
  return @result;
}


sub _get_bundle_maximum_level {
    my $self = shift;
    my $bundle_id =  shift || die 'must include bundle_id';
    my $user = shift || confess "must provide reference to user object";

    my $max = 0;

    if($user->is_owner($self) || $self->is_allowed_to_do('edit_permissions', $user)) {
        $max = 8;
    }

    if($USER->belongs_to_grp('SITE_MGR', $user->{uid})) {
        $max = 9;
    }

    if($USER->belongs_to_grp('ADMIN', $user->{uid})) {
        $max = 10;
    }

    return $max;
}

sub _get_bundle_minimum_level {
  my $self = shift;
  my $bundle_id =  shift || die 'must include bundle_id';
  my $min = 0;

  my $ac = $self->get_AC();

  return $ac->{BUNDLES}{$bundle_id}{MIN_LEVEL};

}


sub _get_bundle_allowed_levels {
  my $self = shift;
  my $bundle_id = shift || die 'must include bundle_id';
  my $user = shift || die "must provide reference to user object";
  my $recursive = shift;

  if($recursive) {

      return $self->get_bundle_allowed_levels_from_all_possible_bundles( $bundle_id, $user )
  }

  my $mini = $self->_get_bundle_minimum_level($bundle_id);
  my $maxi = $self->_get_bundle_maximum_level($bundle_id, $user);

  grep {$_ >= $mini and $_ <= $maxi} $self->_get_level_numbers();

}


sub _get_bundle_description {
  my $self = shift;
  my $bundle_id =  shift || die 'must include bundle_id';
  my $recursive = shift;

  if($recursive) {
      return $self->get_bundle_description_from_all_possible_bundles( $bundle_id )
  }

  my $ac = $self->get_AC();

  $ac->{BUNDLES}{$bundle_id}{DESC};

}


sub apply {
    my $self = shift || croak "no self";
    my $recursive = shift;
    my $function = shift || croak "must provide function";

    my $node = $self->id;

    defined $node || croak "self hasn't got an id";

    # ref to hash mapping iid's to array of children iid's
    # children includes only non-terminal children

    $function->($self);

}


sub can_change_owner {
    # must be overridden in concrete classse that can't change owner
    return 1;
}


# new one
sub _has_bundle {
    my $self = shift;

    my $bundle_id =  shift || die 'must include bundle_id';

    my $ac = $self->get_AC();

    # will auto-vivify non-existing bundles
    if($ac->{BUNDLES}{$bundle_id}{DESC}) {

        return 1;

    } else {

        # must do this to remove auto-vivified bundle
        delete $ac->{BUNDLES}{$bundle_id};
        return 0;
    }
}




1;


__END__

=head1 NAME

Auditable - abstract class.


=head1 SYNOPSIS

    get_AC : interface, subclass needs to implement
    not_logged_in_callback: interface method, subclass needs to implement
    get_parent: interface method, subclass needs to implement
    go_back_to_parent_after_editing_permissions ;
    delete_permissions ;
    save_permissions ;
    get_bundle_level ( $bundle_id);
    set_bundle_level ($bundle_id, #level);
    get_bundle_of_op  ($op);
    is_allowed_to_do ($op, $user, [$activate_logged_in_cb]);
    edit_permissions_form ($form_params, $done_redirect, $user);
    init_permissions;  [object method only]
    check_permissions_of_children ($id);


=head1 DESCRIPTION

=head2 Access Control in Metadot

The AC system comprises infrastructure that allows us to check for
user rights to perform operation on any object in Metadot that
implements the Auditable interface. A class can implement the
Auditable interface by inheriting from the new Auditable class and
overriding some of its methods.

The Auditable class relies in the Permissions class for permissions
storage. Permissions objects implementation is neatly tucked away so
most developers don't ever have to worry about them.

The Auditable class relies on the Operations class to associate
individual methods, or subroutines, with named bundles.  So, the VIEW
bundle may be associated with the methods www_show,
www_show_as_bullet, and so on.  In other words, the permissions object
stores the permissions that a user or group has with respect to a
bundle, and the operations object stores what subroutines are
associated with that bundle.  In that way, we can know what
permissions a user has with respect to a given subroutine.

Together with Permissions.pm and Operations.pm, Auditable.pm supplies
most of the infrastructure needed for making a class be AC
aware. Inheriting Auditable by adding it to the @ISA array of new
classes is thus the best way to make them part of the AC system.


=head2 Instrumenting a Class to do AC Enforcement


1. Inherit from the Auditable class by including Auditable in the @ISA array.

2. Get, and optionally modify, and Operations object. See the operations class for how to do it.

3. Implement the following methods:

     is_a
     name
     id
     save
     uid
     parent_id

See Section III for details on each of them.

4.  Optionally, outside of any methods, get a new default permissions object, modify that object, and then within a class method called
get_default_permissions, return that object.  See the Operations.pm documentation for instructions.

5. Override method &get_parent if class objects can have parents. This method must return a parent object which must also belong to Auditable.

6. Call $self->init_permissions() at the end of its constructor method, after methods in 3 are ready to be called.

7. Call save_permissions() from their save() routine.

8. Call save_permissions() from their add() routine (or from however named routine that commits a new object of the class to the database).

9. Call delete_permissions() from the class destructor.

9. Do permissions enforcement by way of the is_allowed_to_do() call (see index.pl and Category.pm for examples of this).
One must be particularily careful in sheltering all Access Controlled code sections around is_allowed_to_do() conditionals.

10. Pass a User object to the is_allowed_to_do() call (for User object requirements see Section V).

11. Implement www_edit_permissions and call edit_permissions_form() from it (see Gizmo::www_edit_permissions for example).

12. Override &go_back_to_parent_after_editing_permissions to return false for classes that require browsers to return to the object itself whose permissions are being edited, such as Category.

=head2 The Auditable Interface

B<'Must-provide' Methods>

Note:  The methods below do not need to be provided in classes which inherit from other classes which do provide these method.
In particular, Gizmos which inherit from GizmoBuilder don't need to provide any of these methods.

&get_AC

This class method should return one of two things:

An operations object, which describes the relationship between bundles and operations.  The creation of a new operations object,
and its modification, is described in the GizmoBuilder documentation.  Alternatively, this method can return a hash that
describes the relationship between bundles and operations.  That hash is described here.

Here is an example of a get_AC method which returns a hash:  (Note that my $AC is defined outside the method, so that
it does not need to be recreated each time get_AC is called:

my $AC = {
       BUNDLES => {
                   MOD =>    { DESC =>          'Edit',
                               OPS =>            [qw(add create modify save add_category create_category modify_category save_category)],
                               DEFAULT_LEVEL =>  9,
                               MIN_LEVEL =>      9,
                             },
                   DEL =>    { DESC =>          'Delete and Cut',
                               OPS =>            [qw(delete delete_ok delete_category_ok delete_category)],
                               DEFAULT_LEVEL =>  9,
                               MIN_LEVEL =>      9,
                               PARENT_DEP =>     {cut => 'MOD'},
                             },
                   EDITP =>  { DESC =>          'Change<br />Permissions',
                               OPS =>            [qw(edit_permissions set_permissions change_owner)],
                               DEFAULT_LEVEL =>  8,
                               MIN_LEVEL =>      8,
                             },
                  },
          DISPLAY_ORDER => [qw(DISP MOD DEL EDITP)],
        };

  &is_a

Should return the class name, as a string.

  &name

Should return the name of this particular instance of Auditable, which
is used to generate the Edit Permissions page.

  &id

Should return a unique ID associated with this instance. If only one instance is expected to ever exist for this class this method should return 1.

  &save

The only requirement for this method is to call &save_permissions. Normally this method will also save non-Auditable object data to disk (e.g. Gizmo::save).

  &uid

Should return the User ID for the owner of a particular instance of Auditable.

&parent_id

Should return the ID of the parent of a particular instance of Auditable. If class instances can't have parents the it must return 0.

B<OVERRIDEABLE Methods>

  &get_parent

Should return the Auditable object representing the parent of a particular instance of Auditable. If class objects can't have parents then it shouldn't be overidden.

  &go_back_to_parent_after_editing_permissions

Should return false on classes that require browsers to return to the object itself whose permissions are being edited, such as Category. Otherwise it shouldn't be overidden. Should be called from www_edit_permissions (See Gizmo::www_edit_permissions for example of it's use).

B<NON-OVERRIDEABLE Methods>

  &is_allowed_to_do

Should be called from all places in the class code where Access Control enforcement is needed. See Category.pm, index.pl and MyPage.pm for examples of this).

  &edit_permissions_form

Should be called from www_edit_permissions and will return the HTML of the Edit Permissions form, which should be sent to client browsers.

  &init_permissions

Should be called at the end of the class constructor, right after MUST PROVIDE methods are guaranteed to return true data.

  &delete_permissions

Should be called from the class destructor, but before MUST PROVIDE methods cease to work.

  &save_permissions

Should be called from save() and from any method that commits this objects data to disk for later restoring.

  &get_bundle_level
  &set_bundle_level
  &get_bundle_of_op

These are uses to modify/query minimum permissions associated with a
particular instance of Auditable.

=head2 User Object Requirements


User objects are required to:

1. Implement methods:

      belongs_to_grp
      uid
      new

Note: &new should return a User object when passed a UID as parameter.

2. Implement a classwide get_fullname_from_uid method.


=cut

