###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
#!/usr/bin/perl
#
package BasicTool;

use strict;

use DBAccess;
use Carp;
use Utility;
use MyPageElement;

use vars qw(@ISA);
@ISA=qw(MyPageElement);

####################################################

sub new {
  my $proto = shift;
  my $iid  = shift;  #iid of the Tool
  my $uid   = shift; #uid of the user NECESSARY IF you want to retrieve mypage data

  my $class = ref($proto) || $proto;
  my $self  = $class->SUPER::new(undef,$uid); #we don't know the mpid yet

  $iid = undef unless Utility->is_valid_integer($iid); # secu_200401
  $uid = undef unless Utility->is_valid_integer($uid); # secu_200401

  if ((defined $iid) && (defined $uid) ) {
#        print STDERR "checking if gizmo $iid exists\n";
      if (!Gizmo->exists($iid)) {
      print STDERR "Warning: Basic Tool $iid does NOT exist\n";
      return;
      }
      # load instance & mypage information
      ($self->{iid}, $self->{mpid}, $self->{name}, $self->{content}, $self->{col}, $self->{mypageorder})
      = DBAccess->sqlSelect("iid, mpid, name, description, col, mypageorder", "instance, mypage", "mypage.id=$iid and mypage.userid=$uid and mypage.id=instance.iid");
  }
  if ((defined $iid) && !(defined $self->{iid})) {
      # load instance information
      ($self->{iid}, $self->{name}, $self->{content})
      = DBAccess->sqlSelect("iid, name, description", "instance", "iid=$iid");
  }

  $self->{home_url}="";
  $self->{description}="";
  $self->{is_a} = "BasicTool";

  bless ($self, $class);

  return $self;
}

sub _panel {
  my $self = shift ;
  my $iid = $self->iid();

  my $s = "";
  if (defined $iid) {
    $s = $self->SUPER::_panel($iid);
  }
  return $s;
}





1;


__END__

=head1 NAME

BasicTool - a MyPage element that can be used to provide simple html-based functionality such as search or query interfaces

=head1 SYNOPSIS


  use BasicTool;

  $basic_tool = BasicTool->new ($iid, $uid);


=head1 DESCRIPTION

BasicTool instances define an HTML fragment that can be placed in a
user's MyPage to implement, e.g a search or query interface.  The html
fragment is stored persistently in the db using the 'description'
field of the underlying entry in the instance table. BasicTools
inherit most of their functionality from their parent,
L<MyPageElement>.

=cut
