##########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Channel;
use strict;

use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CGI $ACCESS_BROKER );

use Carp;

use Metadotclass;
use Metadot::DBMap;
use Portal;
use Utility;
use DBAccess;
use vars qw(@ISA);

@ISA=qw(Portal);


use constant CHANNEL_TYPE_EXTERNAL   => 'External';
use constant CHANNEL_TYPE_INTERNAL   => 'Internal';
use constant CHANNEL_TYPE_DISCUSSION => 'Discussion';




# aju 02-19-02
my $AC = {
       BUNDLES => {
                   MOD =>    { DESC =>          'Edit',
                               OPS =>            [qw(create modify save)],
                               DEFAULT_LEVEL =>  2,
                               MIN_LEVEL =>      2,
                             },
                   ADD =>    { DESC =>          'Edit',
                               OPS =>            [qw(add)],
                               DEFAULT_LEVEL =>  9,
                               MIN_LEVEL =>      9,
                             },
                   MODC =>    { DESC =>          'Edit Category',
                               OPS =>            [qw( add_category create_category modify_category save_category)],
                               DEFAULT_LEVEL =>  9,
                               MIN_LEVEL =>      9,
                             },
                   DEL =>    { DESC =>          'Delete and Cut',
                               OPS =>            [qw(delete delete_ok)],
                               DEFAULT_LEVEL =>  2,
                               MIN_LEVEL =>      2,
                               PARENT_DEP =>     {cut => 'MOD'},
                             },
                   DELC =>    { DESC =>          'Delete and Cut',
                               OPS =>            [qw( delete_category_ok delete_category)],
                               DEFAULT_LEVEL =>  9,
                               MIN_LEVEL =>      9,
                               PARENT_DEP =>     {cut => 'MOD'},
                             },
                   EDITP =>  { DESC =>          'Change<br />Permissions',
                               OPS =>            [qw(edit_permissions set_permissions change_owner)],
                               DEFAULT_LEVEL =>  8,
                               MIN_LEVEL =>      8,
                             },
                  },
          DISPLAY_ORDER => [qw(DISP MOD DEL EDITP)],
        };

sub get_AC {
    return $AC;
}

sub is_a {
    my $self = shift;

    return ref($self);
}

#sub name {
#   my $self = shift;
#
#   return ref($self);
#}

sub id {
    my $self = shift;

    return 1;
}

# aju: this is defined again below, so this one is not being used.
# sub save {
#   my $self = shift;

#   $self->save_permissions;
# }

sub get_owner_uid {
    my $self = shift;

    return MetadotConfig->get_reserved_uid('ADMIN');
}

sub parent_id {
    my $self = shift;

    return 0;
}

##########################################################
# new Internal or External in input args
sub new {
    my $proto = shift;
    my $cid = shift;
    my $class = ref($proto) || $proto;
    my $self = {};


#      print STDERR "in Channel::new with cid $cid\n";
    $self->{cid}            = undef;
    $self->{url}            = "";
    $self->{id}             = "";
    $self->{is_a}           = "TO_BE_SET";
    $self->{name}           = "";
    $self->{description}    = "";
    $self->{version}        = 0;
    $self->{fetch_every}    = 0;
    $self->{http_status}    = 0;
    $self->{nb_items}       = 0;
    $self->{home_url}       = 0;
    $self->{last_processed} = 0;
    $self->{html}           = 0;
    $self->{version}        = 0;
    $self->{creation_date}  ="";
    $self->{update_date}    ="";
    $self->{last_fetched}   ="";
    $self->{ccid} = "";

    if(defined ($cid)){
#   print STDERR "in channel:new, doing db select\n";
      (
    $self->{cid}            ,
    $self->{url}            ,
    $self->{id}             ,
    $self->{is_a}           ,
    $self->{name}           ,
    $self->{description}    ,
    $self->{version}        ,
    $self->{fetch_every}    ,
    $self->{http_status}    ,
    $self->{nb_items}       ,
    $self->{home_url}       ,
    $self->{last_processed} ,
    $self->{html}           ,
    $self->{version}        ,
    $self->{creation_date}  ,
    $self->{update_date}    ,
    $self->{last_fetched}  ,
    $self->{ccid}
      )
    = DBAccess->sqlSelect("cid, url, iid, isa, name, description, version, fetchevery, httpstatus, nbitems, homeurl, lastprocessed, html, version, creationdate, updatedate, lastfetched, ccid", "channel", "cid=$cid");
    }


    bless ($self, $class);
#      print STDERR "in Channel::new, with isa of " . $self->{is_a} . "\n";

    return $self;
}






####################################################
# WWW FUNCTIONS

sub www_add {
  my $self = shift;
  my $errormsg = shift;
  $self->debug_msg(3, "Channel::www_add()");

  # controls
  if (!defined ($FORM{type})){
    $self->click_back ("Add Form Error:", "type is missing."); exit(0);
  }
  $self->{is_a} = $FORM{type};
  my $type = $FORM{type};
  if (($type ne CHANNEL_TYPE_EXTERNAL()) && ($type ne CHANNEL_TYPE_INTERNAL()) && ($type ne 'Poll')){
    $self->click_back ("Add Form Error:", "$type is not a good type."); exit(0);
  }
  if (!defined ($FORM{parent_id})){
    $self->click_back ("Add Form Error:", "parent id is missing."); exit(0);
  }
  my $parentid = $FORM{parent_id};
  # end of controls

  if ($errormsg) {
    $errormsg = "<p /><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p />\n";
  }

  #$self->print_http_header();
  #print $DISPLAY->show_page_top;
  #print $DISPLAY->simple_header() . "<body>\n";
  #print $DISPLAY->form_top();
  my $form = "<form action='index.pl'  method='post' ENCTYPE=\"multipart/form-data\">
         <input type='hidden' name='op' value='create'>
         <input type='hidden' name='isa' value='Channel'>
         <input type='hidden' name='type' value='$type'>
         <input type='hidden' name='parent_id' value='$parentid'>
         $errormsg\n" .
        $self->show_edit_form("Adding new $type Channel") .
        "<table align='right'><tr><td><input type='submit' value='Create' /><input type='button' onClick=\"javascript:history.go(-1);\" value='Cancel' /></td></tr></table>
     <br />&nbsp;
         </form>\n";
    #print "<tr ><td>&nbsp;</td><td>$form</td><td>&nbsp;</td></tr>";
    #print $DISPLAY->form_bottom;
    #print $DISPLAY->close_html();

  $DISPLAY->print_form($form);
}

sub www_add_category {
  my $self = shift;
  my $errormsg = shift;
  $self->debug_msg(3, "Channel::www_add_category()");
  my $parentid = $FORM{parent_id};

  if ($errormsg) {
    $errormsg = "<p /><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p />\n";
  }

  #$self->print_http_header();
  #print $DISPLAY->show_page_top;
  #print $DISPLAY->simple_header() . "<body>\n";
  #print $DISPLAY->form_top();
  my $external = CHANNEL_TYPE_EXTERNAL();

  my $form =
        "<form action='index.pl'  method='post' ENCTYPE=\"multipart/form-data\">
         <input type='hidden' name='op' value='create_category'>
         <input type='hidden' name='isa' value='Channel'>
         <input type='hidden' name='type' value='$external'>
         <input type='hidden' name='parent_id' value='$parentid'>
         <input type='hidden' name='target' value='$SESSION->{referer}'>
         $errormsg\n" .
        $self->show_category_edit_form("Adding new external news category") .
        "<table align='right'><tr><td><input type='submit' value='Create'></td></tr></table>
     <br />&nbsp;
         </form>\n";
    #print "<tr ><td>&nbsp;</td><td>$form</td><td>&nbsp;</td></tr>";
    #print $DISPLAY->form_bottom;
    #print $DISPLAY->close_html();
    $DISPLAY->print_form($form);
}

sub www_create {
  my $self = shift;
  $self->debug_msg(3, "Channel::www_create()");

  # controls
  if (!defined ($FORM{type})){
    $self->click_back ("Add Form Error:", "type is missing."); exit(0);
  }
  my $type = $FORM{type};
  if (($type eq CHANNEL_TYPE_EXTERNAL()) && ($type eq CHANNEL_TYPE_INTERNAL()) && ($type eq 'Poll')){
    $self->click_back ("Add Form Error:", "$type is not a good type."); exit(0);
  }
  if (!defined ($FORM{parent_id})){
    $self->click_back ("Add Form Error:", "parent id is missing."); exit(0);
  }
  my $parentid = $FORM{parent_id};
  # end of controls

  # fill the object with form values
  if ($type eq CHANNEL_TYPE_EXTERNAL()) {
      if ($FORM{name}) {
      $self->{name} = $FORM{name};
      }
      else {
      $self->{name} = $FORM{url};
      }
  }
#    ($type eq CHANNEL_TYPE_EXTERNAL()) ? $self->name($FORM{url}) : $self->name($FORM{name});
  $self->url($FORM{url});
  $self->description($FORM{description});
  my $ccid = $FORM{ccid} || 1;
  $self->{ccid} =$ccid;
  $self->{is_a} = $type;
  $self->{id} = $parentid;

  # check if everything is ok
  my $errormsg = "";
  $errormsg .= "<li>Too bad! Name field cannot be empty...</li>" if ((!$self->name()) && ($type ne CHANNEL_TYPE_EXTERNAL())) ;
  $errormsg .= "<li>Too bad! URL field cannot be empty...</li>" if ((!$self->url()) && ($type eq CHANNEL_TYPE_EXTERNAL())) ;

  if ($errormsg) {
    $self->www_add($errormsg);
  } else {
    if ($type eq CHANNEL_TYPE_EXTERNAL()) {
      my $url = $self->url;
      if (DBAccess->sqlSelect("cid","channel","isa=".DBAccess->quote(CHANNEL_TYPE_EXTERNAL())." and url='$url'")) {
    $self->click_back ("Cannot add channel with url:<br />$url", "; Channel already exists."); exit(0);
      }
    }
    my $newid = $self->add($type, $self->{id});
    if ($newid != 0){   #### add successful
      if ($type eq CHANNEL_TYPE_EXTERNAL()) {
    $self->two_choices("<Big>New External Channel added!</big><br /><br />What do you want to do?",
                           "confirm",
               "Add another External Channel", "index.pl?isa=Channel&op=add&type=".CHANNEL_TYPE_EXTERNAL()."&parent_id=$parentid",
               "Enough for now!", "index.pl?iid=$parentid&isa=Portal&op=show_channels");
      } else {



      $self->md_redirect("index.pl?iid=$parentid&isa=Portal&op=show_channels")
      }
    } else {
      $self->debug_msg(3, "Item::www_create : error when adding Item to DB");
    }
  }
}

########################################################
sub www_create_category {
  my $self = shift;
  $self->debug_msg(3, "Channel::www_create_category()");
  my $parentid = $FORM{parent_id};

   my $errormsg = "";
  $errormsg .= "<li>Too bad! Name field cannot be empty...</li>" if (! $FORM{cc_name}) ;

  if ($errormsg) {
    $self->www_add_category($errormsg);
    exit(0);
  }
  # end of controls
      my $category = $FORM{cc_name};
      if (DBAccess->sqlSelect("ccid","channel_category","cc_name=".DBAccess->quote($category))) {  #secu_200401
    $self->click_back ("Cannot add category with name:<br />$category", "Category already exists."); exit(0);
      }
    my $ccid = $self->add_category($category);
   #### add successful
    $self->two_choices("<Big>New Channel Category added!</big><br /><br />What do you want to do?",
                           "confirm",
               "Add another Category Channel", "index.pl?isa=Channel&op=add_category&type=".CHANNEL_TYPE_EXTERNAL()."&parent_id=$parentid",
               "Enough for now!", "$FORM{target}");
}
#############################################################################3
sub www_modify {
  my $self = shift;
  my $errormsg = shift;
  $self->debug_msg(3, "Channel::www_modify()");
  my $cid = $self->{cid};
  my $type = lc $self->{is_a};
  my $iid = $self->{id};
#    print STDERR "in Channel::www_modify, with cid $cid, iid $iid and type (isa) $type \n";

  # aju: not clear what the following is intended to do.  the logic is bad. commented
  # it out.
#    # controls
#    if (($type eq CHANNEL_TYPE_EXTERNAL()) && ($type eq CHANNEL_TYPE_INTERNAL()) && ($type eq 'Poll')){
#      $self->click_back ("Add Form Error:", "$type is not a good type."); exit(0);
#    }
  if (!defined $cid){
    $self->click_back ("Error:", "cid is missing.");
    exit(0);
  }
  # end of controls

  # aju: access ctl for internal channels
  $self->_check_internal_channel_ac($type, $iid, 'modify');
  my $target = "";
  if ($FORM{ref}) {
    $target = $FORM{ref};
  } else {
    $target = $CGI->escape($SESSION->{referer});
  }

  $errormsg = (defined $errormsg) ?
              "<p /><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p />\n" :
              "";

  #$self->print_http_header();
  #print $DISPLAY->show_page_top;
  #print $DISPLAY->simple_header() . "<body>\n";
  #print $DISPLAY->form_top();
  my $form =
        "<form action='index.pl' method='post' ENCTYPE=\"multipart/form-data\">
         <input type='hidden' name='op' value='save'>
         <input type='hidden' name='isa' value='Channel'>
         <input type='hidden' name='cid' value='$cid'>
         <input type='hidden' name='ref' value='$target'>
         $errormsg\n".
        $self->show_edit_form("Customizing $type Channel") .
        "<table align='right'><tr><td><input type='submit' value='Save'></td></tr></table>
     <br />&nbsp;
         </form>\n";
  #print "<tr ><td>&nbsp;</td><td>$form</td><td>&nbsp;</td></tr>";
  #print $DISPLAY->form_bottom;
  #print $DISPLAY->close_html();
  $DISPLAY->print_form($form);
}

#############################################################################
sub www_modify_category {
  my $self = shift;
  my $errormsg = shift;
  $self->debug_msg(3, "Channel::www_modify_category()");
  my $ccid = $FORM{ccid};
  $ccid = undef unless Utility->is_valid_integer($ccid); # secu_200401

  my ($catg) = DBAccess->sqlSelect("cc_name","channel_category","ccid=$ccid") if($ccid);
  my $target = "";
  if ($FORM{ref}) {
    $target = $FORM{ref};
  } else {
    $target = $CGI->escape($SESSION->{referer});
  }
  $errormsg = (defined $errormsg) ?
              "<p /><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p />\n" :
              "";

  #$self->print_http_header();
  #print $DISPLAY->show_page_top;
  #print $DISPLAY->simple_header() . "<body>\n";
  #print $DISPLAY->form_top();
  my $form =
        "<form action='index.pl' method='post' ENCTYPE=\"multipart/form-data\">
         <input type='hidden' name='op' value='save_category'>
         <input type='hidden' name='isa' value='Channel'>
         <input type='hidden' name='ccid' value='$ccid'>
         <input type='hidden' name='ref' value='$target'>
         $errormsg\n" .
        $self->show_category_edit_form("Editing existing category") .
        "<table align='right'><tr><td><input type='submit' value='Save'></td></tr></table>
     <br />&nbsp;
         </form>\n";
    #print "<tr ><td>&nbsp;</td><td>$form</td><td>&nbsp;</td></tr>";
    #print $DISPLAY->form_bottom;
    #print $DISPLAY->close_html();

   $DISPLAY->print_form($form);
}

#############################################################################
sub www_save {
  my $self = shift;
  $self->debug_msg(3, "Channel::www_save()");
  my $cid   = $self->{cid};
  my $type = lc $self->{is_a};
  my $iid = $self->{id};
#    print STDERR "in Channel::save with cid $cid and type $type\n";
  # aju: not clear what the following is intended to do.  the logic is bad. commented
  # it out.
#    # controls
#    if (($type eq CHANNEL_TYPE_EXTERNAL()) && ($type eq CHANNEL_TYPE_INTERNAL()) && ($type eq 'Poll')){
#      $self->click_back ("Add Form Error:", "$type is not a good type."); exit(0);
#    }
  if (!defined $cid){
    $self->click_back ("Error:", "cid is missing.");
    exit(0);
  }
  # end of controls

  # aju: access ctl for internal channels
  $self->_check_internal_channel_ac($type, $iid, 'save');

  # fill the object with form values
    if ($type ne lc(CHANNEL_TYPE_EXTERNAL())) {
      $self->name($FORM{name});
      $self->description($FORM{description});
    }
  #aju
  if ($type ne lc(CHANNEL_TYPE_INTERNAL()) && $type ne lc(CHANNEL_TYPE_DISCUSSION())) {
      $self->url($FORM{url});
      $FORM{ccid} = undef unless Utility->is_valid_integer($FORM{ccid}); # secu_200401
      $self->ccid($FORM{ccid});
  }

  #aju
  if ($type ne lc(CHANNEL_TYPE_EXTERNAL()) && $type ne lc(CHANNEL_TYPE_INTERNAL()) && $type ne lc(CHANNEL_TYPE_DISCUSSION())) {
      $self->home_url($FORM{home_url});
      $self->html($FORM{html});
      $self->fetch_every($FORM{fetch_every});
  }

  # check if everything is ok
  my $errormsg = "";
  $errormsg .= "<li>Too bad! Name field cannot be empty...</li>" if ((!$self->name()) && ($type ne lc(CHANNEL_TYPE_EXTERNAL()))) ;
  $errormsg .= "<li>Too bad! URL field cannot be empty...</li>" if ((!$self->url()) && ($type ne lc(CHANNEL_TYPE_INTERNAL())) &&
                                   ($type ne lc(CHANNEL_TYPE_DISCUSSION())) &&
                                   ($type ne 'personaladdrbook')) ;
  $errormsg .= "<li>Too bad! Fetch Every field cannot be empty...</li>" if ((!$self->fetch_every()) &&
                  ($type ne lc(CHANNEL_TYPE_INTERNAL())) && ($type ne lc(CHANNEL_TYPE_DISCUSSION())) &&
                  ($type ne lc(CHANNEL_TYPE_EXTERNAL())) && ($type ne 'personaladdrbook')) ;

  if ($errormsg) {
    $self->www_modify($errormsg);
  } else {
    $self->save();
    my $target = $FORM{ref};
    $target = $CGI->unescape($target);
    $self->md_redirect($target);
  }
}

#############################################################################
sub www_save_category {
  my $self = shift;
  $self->debug_msg(3, "Channel::www_save_category()");

  # check if everything is ok
  my $errormsg = "";
  $errormsg .= "<li>Too bad! Name field cannot be empty...</li>" if (! $FORM{cc_name}) ;

  if ($errormsg) {
    $self->www_modify_category($errormsg);
  } else {
    $self->save_category($FORM{cc_name});
    my $target = $FORM{ref};
    $target = $CGI->unescape($target);
    $self->md_redirect($target);
  }
}
###############################################################################3

sub www_delete {
  my $self = shift;
  my $isa = $self->is_a();
  my $type = lc $self->{is_a};
  my $iid = $self->{id};
  $self->debug_msg(3, "Channel::www_delete()");

  $self->_check_internal_channel_ac($type, $iid, 'delete');

  my $target = $SESSION->{referer};
  $target = $CGI->escape($target);
  $self->two_choices("Do you want to delete this $isa Channel: <br />".$self->name."?",
                     "caution",
             "Yes please", "index.pl?cid=".$self->cid."&isa=Channel&op=delete_ok&ref=$target",
             "No!", $SESSION->{referer});
}

sub www_delete_ok {
  my $self = shift;
  $self->debug_msg(3, "Channel::www_delete_ok()");
  $self->delete();
  my $target = $FORM{ref};
  $target = $CGI->unescape($target);
  $self->md_redirect($target);
}

###############################################################################3

sub www_delete_category {
  my $self = shift;
  $self->debug_msg(3, "Channel::www_delete_category()");
  my $target = $SESSION->{referer};
  $target = $CGI->escape($target);
  my $ccid = $FORM{ccid};
  $ccid = undef unless Utility->is_valid_integer($ccid); # secu_200401

  if (!defined ($ccid) || ($ccid <= 0)){
    $self->click_back ("Delete Category Error:", "ccid is missing or invalid."); exit(0);
  } elsif ($ccid == 1) {
    $self->click_back ("Delete Category Error:", "cannot delete this category - it is the default."); exit(0);
  }

  (my $dcategory) = DBAccess->sqlSelect("cc_name","channel_category","ccid=1");

  my $notice =  ($ccid != 1) ? "<p />All channels will be moved to Category $dcategory if deleted." : "";

  (my $category) = DBAccess->sqlSelect("cc_name","channel_category","ccid=$ccid");
  $self->two_choices("Do you want to delete Category $category? $notice",
                     "caution",
             "Yes please",
                     "index.pl?ccid=".$FORM{ccid}."&isa=Channel&op=delete_category_ok&ref=$target",
             "No!", $SESSION->{referer});
}

sub www_delete_category_ok {
  my $self = shift;
  $self->debug_msg(3, "Channel::www_delete_category_ok()");
  if (!defined($FORM{ccid})) {
      $self->click_back ("Delete Category Error:", "ccid is missing."); exit(0);
  }
  $self->delete_category($FORM{ccid});
  my $target = $FORM{ref};
  $target = $CGI->unescape($target);
  $self->md_redirect($target);
}
####################################################
# BACKEND FUNCTIONS

sub show_edit_form {
    my $self = shift;
    my $title =shift;
    $self->debug_msg(4, "Channel::show_edit_form($title)");
    my $cid = $self->{cid};
    my $isa = $self->{is_a};

    my $name = $self->{name};
    my $url = $self->{url} ;
    my $home_url = $self->{home_url} ;
    my $html = $self->{html} ;
    my $fetch_every = $self->{fetch_every} ;
    my $description = $self->{description};
    my $httpstatus = $self->{http_status};
    my $nbitems = $self->{nb_items};
    my $lastfetched = $self->{last_fetched};
    my $updatedate = $self->{update_date};
    my $version = $self->{version};
    my $ccid = $self->{ccid};

    my $catg_popup = $self->category_popup($ccid);

    my $s = "<h1>$title</h1>\n";

#      print STDERR "in Channel::show_edit_form-- isa is: $isa\n";
    if ($isa eq CHANNEL_TYPE_EXTERNAL()) {
    if ($name) {
        $s.= "<p /><b>Channel Name</b>: $name <br />\n";
    }
    else {
        $s .= "<p />Channel name will be obtained from the URL content after it is initially fetched.<br />\n";
    }
    if ($description) {
        $s.= "<p /><b>Description</b>: $description <br />\n";
    }
    else {
        $s .= "<p />Channel description will be obtained from the URL content after it is initially fetched, as will the 'official' name.<br />\n";
    }

    $s.= "<p /><b>RSS/RDF Channel URL</b> (*)<br />
             <input type='text' name='url' size='45' maxlength='255' value=\"$url\"><br />
         <i>Enter here the location of the RDF/RSS channel, e.g. http://www.reuter.com/news.xml.</i>\n";
    $s.="<p /><b>Channel Category </b> (*)  $catg_popup<br />";
    }
    elsif ($isa eq CHANNEL_TYPE_INTERNAL() || $isa eq CHANNEL_TYPE_DISCUSSION()) {
    $s.= "<p /><b>Channel Name</b> (*)<br />\n<input type='text' name='name' size=45 maxlength=255 value=\"$name\">\n";
    $s.= "<p /><b>Brief Description</b><br />\n<textarea name='description' rows=3 cols=46 WRAP='VIRTUAL'>$description</textarea>\n";
    }
    elsif ($isa eq 'Poll') {
    $s.= "<p /><b>Channel Poll</b> (*)<br />\n<input type='text' name='name' size=45 maxlength=255 value=\"$name\">\n";
    $s.= "<p /><b>Brief Description</b><br />\n<textarea name='description' rows=3 cols=46 WRAP='VIRTUAL'>$description</textarea>\n";
    }
    elsif ($isa eq 'PersonalAddrBook') { #Gizmo Tool
    $s.= "<p /><b>Gizmo Tool Name </b> (*):<br /> <input type = 'text' size = '20' name = 'name' value = \"$name\">
                <p /><b>Description</b>: <br /><textarea name = 'description' wrap=virtual cols=60 rows=5>$description</textarea>";
    }
    else  { #Gizmo Tool
    $s.= "
<p /><b>Gizmo Tool Name </b> (*):<br /> <input type = 'text' size = '20' name = 'name' value = \"$name\">
<p /><b>Description</b>: <br /><textarea name = 'description' wrap=virtual cols=60 rows=5>$description</textarea>
<p /><b>Channel URL</b> (*): <br /><input type='text' name='url' size='50' maxlength='255' value=\"$url\">
<p /><b>Home URL</b>: <br /><input type = 'text' size = '50' name = 'home_url' value = \"$home_url\">
<p /><b>Extra HTML</b>: <br /><textarea name = 'html' wrap=virtual cols=60 rows=5>$html</textarea>
<p /><b>Fetch Every </b> (*): <br /><input type = 'text' size = '10' name = 'fetch_every' value = \"$fetch_every\"> Seconds
";
    }

    $s.= "<br /><br /><br /><i>(*) Mandatory field(s)</i><br /><br /><hr><p />\n";

    if ($cid) {         # for customizing only
    $s.= "Number of items: $nbitems<br />\n" if ($nbitems);
    $s.= "HTTP status: $httpstatus<br />\n" if ($httpstatus);
    $s.= "Last fetched: $lastfetched<br />\n" if ($lastfetched);
    $s.= "Last updated: $updatedate<br />\n" if ($updatedate);
    if ($isa eq CHANNEL_TYPE_EXTERNAL()) {

        my $url_digest = Digest::MD5::md5_hex( $url );

        if (-r "channels/$url_digest.xml"){

        $s .= "Channel file: <a href='channels/$url_digest.xml'>channels/$url_digest.xml</a><br />";

        } else {

        $s .= "<b>No channel file has been fetched yet.<b><br />";
        }
    }
    }

    return $s;
}

sub show_category_edit_form {
  my $self = shift;
  my $title =shift;
  my ($catg) = DBAccess->sqlSelect("cc_name","channel_category","ccid='$FORM{ccid}'") if($FORM{ccid} && Utility->is_valid_integer($FORM{ccid}));  #secu_200401
  $self->debug_msg(4, "Channel::show_category_edit_form($title)");

  my $s = "<h1>$title</h1>\n";
  #my $catg = DBAccess->sqlSelect("cc_name", "channel_category", "ccid=$FORM{ccid}") if($FORM{ccid});

      $s.= "<p /><b>Category Name:</b> (*)
             <input type='text' name='cc_name' size='45' maxlength='50' value=\"$catg\"><br />\n";

  $s.= "<br /><br /><br /><i>(*) Mandatory field(s)</i><br /><br /><hr><p />\n";

  return $s;
}
#####################################################
# sub url must be overloaded else CGI::url will take
# over the call.
#####################################################
sub url {
    my $self = shift;
    $self->debug_msg(5, "Channel::url");
    if (@_) { $self->{url} = shift }
    return $self->{url};
}

sub html {
    my $self = shift;
    $self->debug_msg(5, "Channel::html");
    if (@_) { $self->{html} = shift }
    return $self->{html};
}
####################################################
sub add {
  my $self = shift;
  my ($isa, $iid) = @_;
    $isa ||= $self->{is_a};
#    print STDERR "in Channel::add with isa $isa and iid $iid, name " . $self->{name} . "\n";
  #$self->debug_msg(3, "Channel::add(cid=$cid)");
  my $db_now = Utility->db_now();
#  die "db_now is $db_now\n";
$self->{last_processed} = $self->{last_processed} || $db_now;
  my $cid = DBAccess->sqlInsert("channel",
             url           => $self->{url}            ,
             iid           => $iid            ,
             isa           => $isa           ,
             name          => $self->{name}           ,
             description   => $self->{description}    ,
             version       => $self->{version}        ,
             fetchevery    => $self->{fetch_every}    ,
             httpstatus    => $self->{http_status}    ,
             nbitems       => $self->{nb_items}       ,
             homeurl       => $self->{home_url}       ,
             lastprocessed => $self->{last_processed} ,
             html          => $self->{html}           ,
             version       => $self->{version}        ,
             ccid          => $self->{ccid} ,
             creationdate  => $db_now,
             updatedate    => $db_now

            );
  $self->{cid}=$cid;
  return $cid;
}


####################################################
sub add_category {
  my $self = shift;
  my $category = shift;

  my $ccid = DBAccess->sqlInsert("channel_category",
             cc_name           => $category
            );
  return $ccid;
}

####################################################
sub save {
  my $self = shift;
  my $cid= $self->cid();
  $self->{is_a} = ucfirst($self->{is_a});

  my $db_now = Utility->db_now();
  DBAccess->sqlUpdate("channel", "cid=$cid",
             url           => $self->{url}            ,
             iid           => $self->{id}             ,
             isa           => $self->{is_a}           ,
             name          => $self->{name}           ,
             description   => $self->{description}    ,
             version       => $self->{version}        ,
             fetchevery    => $self->{fetch_every}    ,
             httpstatus    => $self->{http_status}    ,
             nbitems       => $self->{nb_items}       ,
             homeurl       => $self->{home_url}       ,
             lastprocessed => $self->{last_processed} ,
             html          => $self->{html}           ,
             version       => $self->{version}        ,
                     ccid          => $self->{ccid}           ,
             updatedate    => $db_now
            );
   return 1;
}

####################################################
sub save_category {
  my $self = shift;
  my $category = shift;

  if (Utility->is_valid_integer($FORM{ccid})) { # secu_200401
      DBAccess->sqlUpdate("channel_category", "ccid=$FORM{ccid}", cc_name => $category);
  }

   return 1;
}
####################################################
sub create {
  my $self = shift ;
  my ($iid, $isa, $name, $description, $url, $fetchevery)=@_;
  $self->debug_msg(4,"Channel::create(" . ($iid||"") . "," . ($isa||"") . ", " . ($name||"") . ", " . ($description||"") . ", " . ($url||"") . ", " . ($fetchevery||"") . ")");

  if (uc($isa) ne "EXTERNAL"){
    $self->{id} = $iid ;
    $self->{is_a} = $isa ;
    $self->{name} =$name ;
    $self->description($description);
    $self->url($url);
    $self->fetch_every(0);
    $self->version(0);
  }else{
    $self->{id} = $iid;
    $self->{is_a} = $isa;
    $self->url($url);
    $self->fetch_every($fetchevery);
    $self->version(0) ;
  }

  #aju 08-29-01: added params, since 'add' was getting this info from the arg list...
  return $self->add ($isa, $iid);
}
####################################################
sub delete{
  my $self = shift;
  my($cid)= $self->cid();

  if(defined $cid){
    DBAccess->sqlDelete("channel", "cid=$cid");
    DBAccess->sqlDelete("channelitem", "cid=$cid");
    DBAccess->sqlDelete("subscription", "cid=$cid");
    DBAccess->sqlDelete("mypage", "id=$cid and isa != 'BasicTool'");

    return 1;
  }
  return 0;
}

####################################################
sub delete_category{
  my $self = shift;
  my $ccid = shift;
  $ccid = undef unless Utility->is_valid_integer($ccid); # secu_200401
  if(defined $ccid){
    DBAccess->sqlDelete("channel_category", "ccid=$ccid");
    DBAccess->sqlUpdate("channel", "ccid='$ccid'",
                                        ccid => '1');

    return 1;
  }
  return 0;
}

sub get_cid {
    my $self = shift;
    my $result = $self->{cid};
    if (!$result && defined($self->get_iid())) {
        $result = $self->get_cid_for_iid($self->get_iid());
        $self->{cid} = $result;
    }
    return $result;
}

sub get_iid {
    my $self = shift;
    if (ref $self) {
        return $self->{id};
    }
}

sub get_subscription_page_link {
    my $self = shift;
    my $result;
    my $cid = $self->get_cid();
    if ($cid) {
        $result = "userchannel.pl?id=$cid&amp;isa=NewsChannel&amp;op=preview";
    }
    return $result;
}

sub get_channel_for_iid {
    my $self = shift;
    my $iid = shift;
    $iid = $self->get_iid() unless defined($iid);
    my ($cid) = $self->get_cid_for_iid($iid);

    my $channel;
    if ($cid) {
        $channel = $self->new($cid);
    }
    return $channel;
}


sub get_cid_for_iid {
    my $class = shift;
    my $iid = shift;
    my $cid;

    $iid = undef unless Utility->is_valid_integer($iid); # secu_200401
    if (defined($iid) && $iid >= 0) {
        my $extra_where = '';
        if ($iid == 0) {
            $extra_where = " and isa = ".DBAccess->quote(CHANNEL_TYPE_INTERNAL());
        }
        ($cid) = DBAccess->sqlSelect("cid", "channel", "iid=$iid $extra_where");
    }
    return $cid;
}


sub get_channel_ids_where {
    my $class = shift;
    my $where = shift || '1=1';
    my $other = shift;

    my $sth = DBAccess->sqlSelectMany('cid', 'channel', $where, $other);
    my @results = ();
    while ((my ($cid)) = $sth->fetchrow()) {
        push @results, $cid;
    }

    return \@results;
}


sub get_channels_where {
    my $class = shift;
    my $where = shift;
    my $other = shift;

    my @results = ();
    my @cids = @{ $class->get_channel_ids_where($where, $other) };
    foreach my $cid (@cids) {
        push @results, $class->new($cid);
    }

    return \@results;
}


sub get_channels_of_type {
    my $class = shift;
    my $type  = shift;

    if (defined $type) {
        return $class->get_channels_where('isa='.DBAccess->quote($type));
    }
}






#############################################
#### Add the' add to my page' logo for internal channels
sub show_add_to_my_page {
    my $self = shift;
    #aju 08-29-01: the following arg does not seem to be used-- the 'isa' value is pulled from the db
    #via the select stmt.
#      my $iisa = shift;
    my $iid = shift;
    my $s="";
    $self->debug_msg(3,"show_add_to_my_page(iid=$iid)");
    ## select channel id
    $iid = undef unless Utility->is_valid_integer($iid); # secu_200401

    my $link = $self->get_subscription_page_link();

# aju 08-29-01: this code does not appear to be used either-- 'isa' is hardwired to NewsChannel below.
#      if (uc($isa) eq "INTERNAL" || uc($isa) eq "EXTERNAL" ||
#   uc($isa) eq "DISCUSSION" || uc($isa) eq "POLL"){
#   $isa="channel";
#      }

    ## there is a channel available then we show the add logo
    my $cid = $self->get_cid();
    if ($cid){

    my $imagesloc = $PARAMS->{'imageslocation'};

    $s='<A HREF="'.$self->get_subscription_page_link().'">'
        ."<IMG class=\"icon\" SRC='$imagesloc/addchannel.gif' BORDER='0' ALT='Add to MyPage'></a>";
    }
    return $s;
}

####################################################
sub show_panel{
  my $self = shift;
  my $cid = $self->cid();
  my $str = "";
  $self->debug_msg(5, "Channel::show_panel(cid=$cid)");

  if (!$SESSION->admin_mode()) { return "";}

  my $imagesloc = $PARAMS->{'imageslocation'};

  $str = $str . "<a href='index.pl?cid=$cid&amp;isa=Channel&amp;op=modify'><img src=$imagesloc/obj/edit.gif width=\"25\" height=\"11\" border=\"0\" alt='Edit Me!' class='icon'></a>\n";
  $str = $str . "<a href='index.pl?cid=$cid&amp;isa=Channel&amp;op=delete'><img src=$imagesloc/obj/del.gif width=\"11\" height=\"11\" border=\"0\" alt='Delete Me!' class='icon'></a>\n";

  return $str;
}

####################################################
sub show_category_panel{
  my $self = shift;
  my $ccid = shift;
  my $str = "";
  $self->debug_msg(5, "Channel::show_category_panel(ccid=$ccid)");

  if (!$SESSION->admin_mode()) { return "";}

  my $imagesloc = $PARAMS->{'imageslocation'};

  $str = $str . "<a href='index.pl?ccid=$ccid&amp;isa=Channel&amp;op=modify_category'><img src=$imagesloc/obj/edit.gif width=\"25\" height=\"11\" border=\"0\" alt='Edit Me!'></a>\n";
  $str = $str . "<a href='index.pl?ccid=$ccid&amp;isa=Channel&amp;op=delete_category'><img src=$imagesloc/obj/del.gif width=\"11\" height=\"11\" border=\"0\" alt='Delete Me!'></a>\n";

  return $str;
}

###################################################
sub category_popup {
    my $self = shift;
    my $ccid = shift || 1;
    my %CATEGORIES;
    my $c = DBAccess->sqlSelectMany("ccid, cc_name", "channel_category", "ccid > 0", "order by cc_name");
    while(my($k, $v)=$c->fetchrow()){
        $CATEGORIES{$k} = $v;
    }
    my $popup = $CGI->popup_menu(-name => 'ccid', -default => $ccid, -values => [sort { $CATEGORIES{$a} cmp $CATEGORIES{$b} }(keys(%CATEGORIES))], -labels => \%CATEGORIES);

    return $popup;
}



####################################################


####################################################
sub DESTROY{
  # EMPTY destroy method to please mod_perl when
  # it complains about a missing DESTROY function.

}

# aju: access ctl for internal channels
sub _check_internal_channel_ac {
    my ($self, $type, $iid, $op) = @_;
    $type = lc $type;
    # tbd: clean this up so that the list of internal channel types is defined elsewhere
    # and not hardwired here.
    if ($type eq lc(CHANNEL_TYPE_INTERNAL()) || $type eq lc(CHANNEL_TYPE_DISCUSSION()) ) {
    # then see if the user has modification privs
    my $sourceobj;
    if ($type eq lc(CHANNEL_TYPE_DISCUSSION())) {
      $sourceobj = Discussion->new($iid);
      }
      else {
      $sourceobj = Gizmo->new($iid);
      }
      if (!($sourceobj->is_allowed_to_do($op, $USER))) {
      #warning, exit
      $self->print_click_back(
           "Error", "index.pl:<br />Sorry, you are not allowed to do operation:<br /> -$op-<br />$0");
      exit(0);
      }
  }
  else {
      # if not internal channel, only admin can edit
      if (!( $ACCESS_BROKER->is_allowed_to_do('admin_tasks') )) {
      $self->print_click_back(
               "Error", "index.pl:<br />Sorry, you are not allowed to do operation:<br /> -$op-<br />$0");
      exit(0);
      }
  }

}

sub print_click_back {
  my $self = shift;
  my $disp = Metadot::Utils::Display->new();
  $disp->print_click_back(@_);

}





1;


__END__

=head1 NAME

Channel - Functions dealing with all the different types of channels.

=head1 SYNOPSIS

    use Channel;
    $channel = Channel->new ($cid);

    $AC = $channel->get_AC;
    $isa = $channel->is_a ;
    $id = $channel->id ;
    $channel->save ;
    $uid = $channel->get_owner_uid ;
    $id = $channel->parent_id ;
    $channel->www_add ([$errormsg]);
    $channel->www_add_category ([$errormsg]);
    $channel->www_create ;
    $channel->www_create_category ;
    $channel->www_modify ([$errormsg]);
    $channel->www_modify_category ([$errormsg]);
    $channel->www_save ;
    $channel->www_save_category ;
    $channel->www_delete ;
    $channel->www_delete_ok ;
    $channel->www_delete_category ;
    $channel->www_delete_category_ok ;
    $html = $channel->show_edit_form ($title);
    $html = $channel->show_category_edit_form ($title) ;
    $url = $channel->url ([$url]);
    $html = $channel->html ([$html]);
    $cid = $channel->add ($isa, $iid);
    $cid = $channel->get_cid ;
    $ccid = $channel->add_category ($category);
    $boolean = $channel->save ;
    $boolean = $channel->save_category ($category);
    $ccid = $channel->create ($iid, $isa, $name, $description, $url, $fetchevery);
    $boolean = $channel->delete;
    $boolean = $channel->delete_category ($ccid);
    $html = $channel->show_add_to_my_page ($iid);
    $html = $channel->show_panel;
    $html = $channel->show_category_panel ($ccid);
    $popuphtml = $channel->category_popup ([$ccid]);
    $channel->DESTROY;


=head1 DESCRIPTION

Channel contains all the functions for editing, deleting, and adding channels internal end external and making categories into channels.

=head1 Methods


Returns AC for user:

    $AC = $channel->get_AC;

Returns the name of the module currently being used:

    $isa = $channel->is_a ;

Returns id of channel:

    $id = $channel->id ;

Saves permissions:

    $channel->save ;

Returns uid for ADMIN:

    $uid = $channel->get_owner_uid ;

Returns parent_id:

    $id = $channel->parent_id ;

Form to add new channels:

    $channel->www_add ([$errormsg]);

Form for adding external news category:

    $channel->www_add_category ([$errormsg]);

Creates channel, verifies form data:

    $channel->www_create ;

Creates channel category, verifies form data:

    $channel->www_create_category ;

Form for customizing channel:

    $channel->www_modify ([$errormsg]);

Form for modifying channel category:

    $channel->www_modify_category ([$errormsg]);

Saves channel form data depending on its type:

    $channel->www_save ;

Saves channel category form data:

    $channel->www_save_category ;

Displays delete channel confimation screen:

    $channel->www_delete ;

Deletes channel after confirmation:

    $channel->www_delete_ok ;

Displays confirmation screen to delete category channel:

    $channel->www_delete_category ;

Deletes category after confirmation:

    $channel->www_delete_category_ok ;

Edit form for channels, display is depends on type:

    $html = $channel->show_edit_form ($title);

Edit form for channel category:

    $html = $channel->show_category_edit_form ($title) ;

Returns url:

    $url = $channel->url ([$url]);

Returns html:

    $html = $channel->html ([$html]);

Inserts channel form data:

    $cid = $channel->add ($isa, $iid);

Returns next available cid:

    $cid = $channel->get_cid ;

Inserts name inter channel_category table:

    $ccid = $channel->add_category ($category);

Saves permissions:

    $boolean = $channel->save ;

Updates channel_categoty cc_name where ccid = $FORM{ccid}:

    $boolean = $channel->save_category ($category);

Creates channel from shifted in vales:

    $ccid = $channel->create ($iid, $isa, $name, $description, $url, $fetchevery);

Deletes all channel data  where cid or id = $cid from tables channel, channelitem, subscription and mypage:

    $boolean = $channel->delete;

Deletes from channel_category where ccid=$ccid, updates channel ccid to 1 where ccid=$cid:

    $boolean = $channel->delete_category ($ccid);

Displays the add to my page logo for internal channels:

    $html = $channel->show_add_to_my_page ($iid);

Displays Edit and Delete buttons for channels:

    $html = $channel->show_panel;

Displays Edit and Delete buttons for channels:

    $html = $channel->show_category_panel ($ccid);

Calls popup_window to create a JavaScript popup with the information shifted in:

    $popuphtml = $channel->category_popup ([$ccid]);

EMPTY destroy method to please mod_perl when it complains about a missing DESTROY function:

    $channel->DESTROY;

=cut
