###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package CheckPassword;

use strict;

use FileHandle;

sub new { # ($fullname, $password [, $nickname])
    my ($proto, $fullname, $password, $nickname) = @_;
    my $class = ref($proto) || $proto;
    my $self = {};

    bless($self, $class);


    $self->{'FULLNAME'} = $fullname;
    $self->{'PASSWORD'} = $password;
    $self->{'NICKNAME'} = $nickname;
    $self->{'MINLENGTH'} = 6;
    $self->{'ISPELL'} = '/usr/bin/ispell';

    my $names = $nickname ? $fullname . ' ' . $nickname : $fullname;
    $names =~ s/,.*//;
    $names =~ s/^\s+//;
    my @names = split(' ', $names);

    $self->{'NAMES'} = '(' . join('|', grep (length($_) > 2, @names)) . ')';

    return $self;

}

sub check_for_bad_password {
    my $self = shift;

    if (my $error = $self->_bad_password()) {
    return (1, $error);
    }
    return undef;;
}

sub _bad_password {
    my $self = shift;
    my $reason;

    my $minlength = $self->{'MINLENGTH'};
    my $mo;

    my $password = shift || $self->{'PASSWORD'};
    return "password was blank"
    if ($password =~ /^\s*$/);
    return "please use at least $minlength characters password"
    if (length($password) < $minlength);
    return "password contains your username"
    if ($password =~ /\Q$self->{'FULLNAME'}\E/i); # make sure we have this variable
    return "password contains part of your name"
    if ($password =~ /\Q$self->{'NAMES'}\E/i); #this one two
    return "password is an alphabetic sequence"
    if $self->_alphabetic_sequence();
    return "password is a qwerty sequence"
    if $self->_qwerty_sequence();
    return "password is a SS number"
    if $self->_social_security();
    return "password is a date"
    if $self->_date_check();
    return "password is a phone number"
    if $self->_phone_check();
    return "password is a license number"
    if $self->_license_check();
    return "invalid characters (colon or space) in your password"
    if $self->_invalid_chars();
    return "patterns not allowed in password"
    if $self->_pattern();
    return "password contains a dictionary word"
    if $self->_lookup();
   $self->_reverse;

    return "password contains a dictionary word"
    if $self->_lookup($self->{'REV_PASSWORD'});
    return "password is an alphabetic sequence"
    if $self->_alphabetic_sequence($self->{'REV_PASSWORD'});
    return "password is a qwerty sequence"
    if $self->_qwerty_sequence($self->{'REV_PASSWORD'});

    return 0;
}

sub _halve {
    my $self = shift;
    my $password = shift || $self->{'PASSWORD'};

    my ($len) = length($password);
    my ($half) = int($len/2);
    return (substr ($password, 0, $len-$half), substr ($password, $half));
}

sub _pattern {
    my $self = shift;
    my $password = shift || $self->{'PASSWORD'};

    my ($left, $right) = $self->_halve($password);

    if (($left eq reverse ($right)) || ($left eq $right)) {
    return 1;
    }

    return 0;
}


sub _invalid_chars {
    my $self = shift;
    my $password = shift || $self->{'PASSWORD'};

    if (($password =~ /\:/) || ($password =~ /\ /)) {
    return 1;
    }

    return 0;
}

sub _license_check {
    my $self = shift;
    my $password = shift || $self->{'PASSWORD'};

    if ($password =~ /^\d{0,1}[A-Za-z]{3}\d{3}$/) {
    return 1;
    } elsif ($password =~ /^\d{3}[A-Za-z]{3}$/) {
    return 1;
    }
    return 0;
}

sub _phone_check {
    my $self = shift;
    my $password = shift || $self->{'PASSWORD'};

    if ($password =~ /^\d{7}$/) {
    return 1;
    } elsif ($password =~ /^\d{3}\-\d{4}$/) {
    return 1;
    }

    return 0;
}

sub _date_check {
    my $self = shift;
    my $password = shift || $self->{'PASSWORD'};

    if ($password =~ /^\d{1,4}\/\d{1,4}\/\d{1,4}$/) {
    return 1;
    } elsif ($password =~ /^\d{1,4}-\d{1,4}-\d{1,4}$/) {
    return 1;
    }
    return 0;
}

sub _social_security {
    my $self = shift;
    my $password = shift || $self->{'PASSWORD'};

    if ($password =~ /^\d{3}-\d{2}-\d{4}$/) {
    return 1;
    } elsif ($password =~ /^\d{9}$/) {
    return 1;
    }
    return 0;
}

sub _qwerty_sequence {
    my $self = shift;
    my $password = shift || $self->{'PASSWORD'};

    my $qwerty = 'qwertyuiopasdfghjklzxcvbnm';

    if ($qwerty =~ m/\Q$password\E/i) {
    return 1;
    }

    return 0;
}

sub _alphabetic_sequence {
    my $self = shift;
    my $password = shift || $self->{'PASSWORD'};

    my $alphabet = 'abcdefghijklmnopqrstuvwxyz';
    my $numbers = '1234567890';

    if ($alphabet =~ m/\Q$password\E/i) {
    return 1;
    }

    if ($numbers =~ m/\Q$password\E/) {
    return 1;
    }

    $numbers = $password;
    my $letters = $password;

    $numbers =~ s/![0-9]//;
    $letters =~ s/![A-Z][a-z]//;

    return 0;
}

sub _reverse {
    my $self = shift;
    my $password = shift || $self->{'PASSWORD'};

    $self->{'REV_PASSWORD'} = reverse $password;

    return $self->{'REV_PASSWORD'};
}

sub spellcheck {
    my $self = shift;
    my $password = shift;

    my $fh = new FileHandle "../etc/passwd.txt", "r";

    unless (defined $fh) {
    print STDERR "Failed while opening ../etc/passwd.txt.";
    }

    while (<$fh>) {
    if ($_ =~ m/\Q$password\E/) {
        $fh->close();
        return 1;
    }
    }

    $fh->close;
    return 0;
}


sub _lookup {
    my $self = shift;
    my $password = shift || $self->{'PASSWORD'};

    my $password_length = length($password);

    #print "PASSWORD: $password\n";

    ##############################################
    ## Check password for dictionary word
    ##############################################
    my $result = $self->spellcheck($password);
    #print "RESULT: $result\n";
    if ($result) {
    return 1;
    }


    ##############################################
    ## Check that password is not a dictionary
    ## word with first or last letters removed.
    ##############################################
    my $endpassword = substr($password, 1, $password_length);
    my $beginpassword = $password;
    chop($beginpassword);

    $result =  $self->spellcheck($endpassword);
    if ($result) {
    return 1;
    }


    $result = $self->spellcheck($beginpassword);
    if ($result) {
    return 1;
    }



    ##############################################
    ## Split the password and check that both
    ## halfs aren't dictionary words
    ##############################################
    for (my $i = 1; $i < ($password_length - 1); $i++) {
    my $first = substr($password, 0, $i);
    my $second = substr($password, $i, $password_length);

    my $flag = 0;

    my $result = $self->spellcheck($first);
    if ($result) {
        $flag++;
    }

    $result = $self->spellcheck($second);
    if ($result) {
        $flag++;
    }

    if ($flag == 2) {
        return 1;
    }

    }
    return 0;
}

1;


__END__

=head1 NAME

CheckPassword -

=head1 SYNOPSIS

    use CheckPassword;
    $cp = CheckPassword->new  ($fullname, $password [, $nickname]);n

    ($error, $error_msg) = $cp->check_for_bad_password ;


=head1 DESCRIPTION

=cut
