###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package DBAccess;

use strict;
use MetadotConfig;

use ErrorHandling;

# db object cache.
# cache an obj for each virtual server.
my %DB_OBJ_CACHE = ();

sub get_primary_connection_name {
    return '__PRIMARY__';
}

#  return an object corresponding to the db type specified in the db config file
#  (by default <metadot>/etc/DBConfig/metadotdb.conf file).
#  If it is not cached in the class variable, create it.
sub get_db_object {
    my $class = shift;
    my $args  = shift;

    my $connection_name = $args->{connection_name} || $class->get_primary_connection_name();

    my $server_name = MetadotConfig->get_server_name();
    my $dbo = $DB_OBJ_CACHE{$server_name}{$connection_name};
    #if object exists, return it
    if ($dbo) {
       return $dbo;
    }
    else {          # else, need to create db object
    my $type=""; my $db_name = "";
    my $db_user = ""; my $db_pass = ""; my $db_host = ""; my $db_sid = "";
    eval {
            if ($args->{db_type} && $connection_name ne $class->get_primary_connection_name()) {
#print STDERR "\n\nDBAccess::get_db_object() user defined connection:$connection_name\n";
                $type    = $args->{db_type};
                $db_name = 'dbi:mysql:' . $args->{db_name};
                $db_name = $args->{db_host} ? $db_name . ':' . $args->{db_host} : $db_name;
                $db_user = $args->{db_user};
                $db_pass = $args->{db_password};
                $db_host = $args->{db_host};
                $db_sid  = $args->{db_sid};
            } else {
#print STDERR "\n\nDBAccess::get_db_object() Metadot connection\n";
                $type = $class->get_db_type();
                $db_name = MetadotConfig->get_dbname();
                $db_user = MetadotConfig->get_dbusername();
                $db_pass = MetadotConfig->get_dbpassword();
                $db_host = MetadotConfig->get_dbhost();
                $db_sid  = MetadotConfig->get_dbsid();
            }
            require "DBAccess/$type.pm";
            #create object
            $dbo = "DBAccess::$type"->new($server_name, $db_name, $db_user, $db_pass, $db_host, $db_sid, $connection_name);
    };
    ### CATCH
    if ($@) {
        print STDERR "Can not find or load database access module for db type: $type.\n" . $@ . "\n" .
        "Your portal database configuration file is probably in error.\n" .
        "If you are running from the command line, exit and reconfigure.\n";
        ErrorHandling->printError("Can not find database access module for db type: $type.<br />" .
                      $@ . "<br />\n" .
                      "Your portal database configuration file is probably in error.\n");
        die;
    }
    # if didn't catch an error, store the db object
    # and add to cache
    $DB_OBJ_CACHE{$server_name}{$connection_name} = $dbo;
    return $dbo;
    }
}

sub get_db_type {
    my $class = shift;
    my $type = MetadotConfig->get_dbtype();
    #print STDERR "got db type: $type\n";
    return $type;
}

# explicitly close the db_object, in order to help preserve connections
sub close_db_object {
    my $self = shift;
    my $result = $self->get_db_object(@_)->close_dbh();
    return $result;
}

# ================================================
# wrap the DBIObj methods.  These will ONLY work when the cached
# db object is of type DBIObj, and in the case of the lock and unlock methods,
# when the object is of type MySQL.
# The following wrapper methods should be considered 'deprecated',
# since it is now preferrable to access the methods directly from the
# db object itself, via get_db_object() above.
# They are added here to maintain backwards
# compatibility with the current interface.

sub existsTable {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->existsTable(@args);
}

sub sqlLock {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->sqlLock(@args);
}

sub sqlUnlock {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->sqlUnlock(@args);
}

sub sqlSimple {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->sqlSimple(@args);
}

sub sqlSelect {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->sqlSelect(@args);
}

sub sqlSelectMany {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->sqlSelectMany(@args);
}

sub sqlSimpleSelect {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->sqlSimpleSelect(@args);
}

# select the count of the set of rows with multiple distinct fields
# (different databases have different syntax for this)
sub sqlSelectCountDistinct {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->sqlSelectCountDistinct(@args);
}

sub sqlInsert {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->sqlInsert(@args);
}

sub sqlDelete {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->sqlDelete(@args);
}

sub sqlUpdate {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->sqlUpdate(@args);
}

sub sqlInsertReplace {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->sqlInsertReplace(@args);
}

sub sqlInsertUpdate {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->sqlInsertUpdate(@args);
}

sub quote {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->quote(@args);
}

sub xsqlSimple {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->xsqlSimple(@args);
}

sub sqlSaveAppObject {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->sqlSaveAppObject(@args);
}

sub sqlDeleteAppObject {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->sqlDeleteAppObject(@args);
}

sub sqlSelectPrepared {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->sqlSelectPrepared(@args);
}

sub sqlSelectManyPrepared {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->sqlSelectManyPrepared(@args);
}

sub sqlSimplePrepared {
    my $class = shift;
    my @args  = @_;
    return ($class->get_db_object())->sqlSimplePrepared(@args);
}


# return a status code of 1 if there is an error,
# followed by a list of error messages
sub validate_data {
    my $class = shift;
    return $class->get_db_object()->validate_data(@_);
}

# returns the function or keyword that gets the current time
sub db_now_function {
    my $class = shift;
    return ($class->get_db_object()->db_now_function());
}

# returns an expression that will calculate the number of seconds between two datetime
sub db_timediff_function {
    my $class = shift;
    my $time1 = shift;
    my $time2 = shift;

    return ($class->get_db_object()->db_timediff_function($time1,$time2));
}


1;


__END__

=head1 NAME

DBAccess - provides 'wrapper' methods for database access.

=head1 SYNOPSIS

   $dbobj = DBAccess->get_db_object();
   $type = DBAccess->get_db_type();
   DBAccess->sqlLock(@args);
   DBAccess->sqlUnlock(@args);
   DBAccess->sqlSimple(@args);
   DBAccess->sqlSelect(@args);
   DBAccess->sqlSelectMany(@args);
   DBAccess->sqlInsert(@args);
   DBAccess->sqlDelete(@args);
   DBAccess->sqlUpdate(@args);
   DBAccess->quote(@args);
   DBAccess->xsqlSimple(@args);

=head1 DESCRIPTION

Return an object corresponding to the db type specified in the db config file
(by default <metadot>/etc/DBConfig/metadotdb.conf file).
If it is not cached in the class variable, create it.

    $dbobj = DBAccess->get_db_object();

Get the 'type' of the database from the config file.  This will be used to determine
what type of db access object to create.

    $type = DBAccess->get_db_type();

The following methods are all wrappers, which simply call the
corresponding method for the 'db object' returned by calling the
get_db_obj() method.  The arg list is simply passed through to the
object method.  See L<DBAccess::DBIObj> and L<DBAccess::MySQL> for information
on the arguments passed to these methods.

   DBAccess->sqlLock(@args);
   DBAccess->sqlUnlock(@args);
   DBAccess->sqlSimple(@args);
   DBAccess->sqlSelect(@args);
   DBAccess->sqlSelectMany(@args);
   DBAccess->sqlInsert(@args);
   DBAccess->sqlDelete(@args);
   DBAccess->sqlUpdate(@args);
   DBAccess->quote(@args);
   DBAccess->xsqlSimple(@args);

=cut

