###########################################################################
# <LICENSEINFO>
# Metadot SmartSite Server                                                                      
# Copyright (C) 2001 Metadot Corporation                                                        
#                                                                                               
# For more information go to                                                                    
# http://www.metadot.com                                                                        
# or contact us at info@metadot.com                                                             
#                                                                                               
#</LICENSEINFO>
############################################################################


# this class tests the DBSchema package
package DBAccess::DBSchema::Test;
use strict;
use MetadotObject;
use DBAccess::DBSchema;

use vars qw(@ISA);
@ISA = qw(MetadotObject);

use Data::Dumper;

sub execute {
    my $self = shift;
    $self->log('Beginning test....');
    
    $self->log('Defining types');
    my $type_int = DBAccess::DBSchema::Type->new(
        {name => 'int'},
    );
    my $type_text = DBAccess::DBSchema::Type->new(
        {name => 'text'},
    );
    $self->log('Done: ',Dumper($type_int, $type_text));
    

    $self->log('Defining columns');
    my $column1 = DBAccess::DBSchema::Column->new(
        {
            name    => 'iid',
            label   => 'Instance ID',
            type    => $type_int,
        }
    );
    my $column2 = DBAccess::DBSchema::Column->new(
        {
            name    => 'name',
            label   => 'Instance Name',
            type    => $type_text,
        }
    );
    my $column3 = DBAccess::DBSchema::Column->new(
        {
            name    => 'userid',
            label   => 'User ID',
            type    => $type_int,
        }
    );
    my $column4 = DBAccess::DBSchema::Column->new(
        {
            name    => 'name',
            label   => 'User Name',
            type    => $type_text,
        }
    );
    $self->log('Done', Dumper($column1, $column2, $column3, $column4));

    $self->log('Defining tables');
    my $table1 = DBAccess::DBSchema::Table->new(
        {        
            name => 'instance'
        }
    );
    my $table2 = DBAccess::DBSchema::Table->new(
        {
            name => 'users'
        }
    );
    $self->log('Done', Dumper($table1, $table2));

    $self->log('Adding columns to tables');
    $table1->set_column( $column1 );
    $table1->set_column( $column2 );
    $table2->set_column( $column3 );
    $table2->set_column( $column4 );
    $self->log('Done');


    $self->log('Defining schema');
    my $schema = DBAccess::DBSchema->new();
    $self->log('Done');

    $self->log('Adding tables to schema');
    $schema->set_table($table1);
    $schema->set_table($table2);
    $self->log('Done');

    
    $self->log('Dumping schema:', Dumper($schema));

}

# trivial logger
sub log {
    my $self = shift;
    my @messages = @_;

    print STDERR @messages,"\n";
}




