###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package DBAccess::MySQL;

use strict;
use DBAccess::DBIObj;

use vars qw(@ISA);
@ISA=qw(DBAccess::DBIObj);


sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = $class->SUPER::new;
    bless ($self, $class);
    $self->{server_name} =  shift;
    $self->{db_name} = shift;
    $self->{db_user} =  shift;
    $self->{db_pass} =  shift;
    my $ignore       = shift;
    $ignore       = shift;
    $self->{connection_name} =  shift;

#use Data::Dumper;
#print STDERR "\n\nDBAccess::MySQL->new() self:" . Dumper($self) . "\n\n";

    # these are the tables that have auto-generated primary
    # keys, and their associated primary key field
    $self->{sequences} = {
        instance        => { key=>'iid', sequence=>'get_new_iid' },
        users           => { key=>'userid', sequence=>'get_uid' },
        grp             => { key=>'grp_id', sequence=>'get_grp_id' },
        channel         => { key=>'cid', sequence=>'get_cid' },
        channelitem     => { key=>'ciid', sequence=>'get_ciid' },
        mypage          => { key=>'mpid', sequence=>'get_mpid' },
        dynaProperties  => { key=>'propertyId', sequence=>'get_propertyId' },
    };


    return $self;
}


sub get_dbh {
    my $self = shift;
    my $args = shift;

    my $connection_name = $args->{connection_name} || $self->{connection_name} || DBAccess->get_primary_connection_name();
    #my $connection_name = $args->{connection_name} || 'primary';

    my $server_name = $self->{server_name};
    # if the handle for this server is already in the hash, return it
    my $dbh = $self->{DBH_CACHE}->{$server_name}->{$connection_name}->{DBH};
    # if the handle for this server is already in the hash, return it
    if (defined $dbh && $dbh->ping() && !$dbh->state()) {
        return $dbh;
    }
    #if (defined $self->{DBH_CACHE}->{$server_name}->{DBH}) {
    #   return $self->{DBH_CACHE}->{$server_name}->{DBH};
    #}


#print STDERR "\n\nDBAccess::MySQL->get_dbh() connection_name:$connection_name db_name:" . $self->{db_name} . " db_user:" . $self->{db_user} . " db_pass:" . $self->{db_pass} . "\n\n";
    $dbh = DBI->connect($self->{db_name}, $self->{db_user}, $self->{db_pass});

    if(!$dbh){
        my $msg = "<h2>Unable to connect to SQL Server</h2>\n";
        $msg .= 'db_name: ' . $self->{db_name} . '<br />db_user: ' . $self->{db_user} . '<br />db_password: (not displayed)';
        if($ENV{SCRIPT_NAME}){
            ErrorHandling->printError($msg);
        } else {
            print "$msg\n";
        }
    } else {
        $self->_init_dbh($dbh);
    }


    # put the fresh handle in the cache
    $self->{DBH_CACHE}->{$server_name}->{$connection_name}->{DBH} = $dbh;

    return $dbh;
}

sub close_dbh {
    my $self = shift;
    my $dbh = $self->get_dbh(@_);
    my $server_name = $self->{server_name};
    my $result;

    if (defined $dbh) {
        delete $self->{DBH_CACHE}->{$server_name}->{DBH};
        $result = $dbh->disconnect();
    }
    return $result;
}


# select the count of the set of rows with multiple distinct fields
sub sqlSelectCountDistinct {
    my($self, $select, $from, $where, $other)=@_;

    my $dbh = $self->get_dbh();

    my $sql="SELECT COUNT(DISTINCT $select) ";
    $sql.="FROM $from " if $from;
    $sql.="WHERE $where " if $where;
    $sql.="$other" if $other;

    if ($self->{debug_on}) {
        $self->sql_log($sql);
        $self->timer_start();
    }

    my $sth = $dbh->prepare($sql);
    if($sth->execute()) {
        if ($self->{debug_on}) {
            $self->timer_log();
        }
        my ($r) = $sth->fetchrow();
        $sth->finish();

        return $r;
    } else {
        $sth->finish();
        my $err = $dbh->errstr;
        my $errormsg = "sqlSelectCountDistinct: SQL statement:<br /><br />$sql<br /><br />\nError: $err<br /><br />\nat $0";
        ErrorHandling->printError($errormsg);

        return undef;
        die "could not execute: $sql\n";
    }
}


# ========================================

#the 'lock' and 'unlock' methods are currently the only mysql-specific ones used.

sub sqlLock {
    my ($self, $lockname) = @_;

    my $dbh = $self->get_dbh();

    my $sql = "select GET_LOCK('$lockname', 15)";

    return $dbh->do($sql);
}

sub sqlUnlock {
    my ($self, $lockname)=@_;

    my $dbh = $self->get_dbh();

    my $sql="select RELEASE_LOCK('$lockname')";

    return $dbh->do($sql);
}

sub is_sequence_needed {
    my $self = shift;
    my $table = shift;
    my $result = defined($self->get_primary_key_field($table));
    #print STDERR "is_sequence_needed($table) returns '$result'\n";
    return $result;
}

sub get_primary_key_field {
    my $self = shift;
    my $table = shift;
    my $result = $self->{sequences}->{$table}->{key};
    #print STDERR "get_primary_key_field($table) returns '$result'\n";
    return $result;
}

sub get_next_primary_key {
    my $self = shift;
    my $table = shift;
    my $method = $self->{sequences}->{$table}->{sequence};
    my $result = $self->$method();
    #print STDERR "get_next_primary_key($table) returns '$result'\n";
    return $result;
}

sub get_insert_id {
    my $self = shift;
    return $self->get_dbh()->{'mysql_insertid'};
}

################################################
##### get_iid
##### return a brand new iid from DB
################################################
sub get_new_iid {
    my $self = shift;

    my $lock = $self->sqlLock("IID");

    my ($iid)= $self->sqlSelect("value", "params", "name = 'nextiid'");
    my $nextiid = $iid +1;
    $self->sqlUpdate("params", "name = 'nextiid'", (value => $nextiid));

    $self->sqlUnlock("IID");
    #print STDERR "get_new_iid returns '$iid'\n";
    return $iid;
}
################################################
##### get_cid
##### return a brand new cid from DB
################################################
sub get_cid {
  my $self = shift;

  my $lock = $self->sqlLock("CID");

  my ($cid)= $self->sqlSelect("value", "params", "name = 'nextcid'");
  my $nextcid = $cid +1;
  $self->sqlUpdate("params", "name = 'nextcid'", (value => $nextcid));

  $self->sqlUnlock("CID");
  return $cid;
}

################################################
##### get_ciid
##### return a brand new cid from DB
################################################
sub get_ciid {
  my $self = shift;

  my $lock = $self->sqlLock("CIID");

  my ($ciid)= $self->sqlSelect("value", "params", "name = 'nextciid'");
  my $nextciid = $ciid +1;
  $self->sqlUpdate("params", "name = 'nextciid'", (value => $nextciid));

  $self->sqlUnlock("CIID");
  return $ciid;
}

################################################
##### get_uid
##### return a brand new uid from DB
################################################
sub get_uid {
    my $self = shift;

    my $lock = $self->sqlLock("UID");

    my ($uid)= $self->sqlSelect("value", "params", "name = 'nextuid'");
    my $nextuid = $uid +1;
    $self->sqlUpdate("params", "name = 'nextuid'", (value => $nextuid));

    $self->sqlUnlock("UID");

    return $uid;
}

################################################
##### get_grp_id
##### return a brand new grp_id from DB
################################################
sub get_grp_id {
    my $self = shift;
    return $self->_get_sequence_value( keyname=>'nextgrpid');
}

################################################
##### get_mpid
##### return a brand new mpid from DB
################################################
sub get_mpid {
    my $self = shift;

    my $lock = $self->sqlLock("MPID");

    my ($mpid)= $self->sqlSelect("value", "params", "name = 'nextmpid'");
    my $nextmpid = $mpid +1;
    $self->sqlUpdate("params", "name = 'nextmpid'", (value => $nextmpid));

    $self->sqlUnlock("MPID");

    return $mpid;
}

################################################
##### get_propertyId
##### return a brand new propertyId from DB
################################################
sub get_propertyId {
    my $self = shift;
    return $self->_get_sequence_value( keyname=>'propertyId');
}


sub _get_sequence_value {
    my $self = shift;
    my %args = @_;
    my $keyname = $args{keyname} || die("Must define the keyname\n");
    my $lockname = $args{lockname} || $keyname;
    my $increment = $args{increment} || 1;
    my $start_value = $args{start_value} || 1;

    my $lock = $self->sqlLock($lockname);

    my ($keyval)= $self->sqlSelect("value", "params", "name = '$keyname'");
    $keyval ||= $start_value;
    my $nextkeyval = $keyval + $increment;
    $self->sqlInsertUpdate("params", "name = '$keyname'", (name => $keyname, value => $nextkeyval));

    $self->sqlUnlock($lockname);

    return $keyval;
}

sub db_now_function {
    return 'now()';
}

# returns an expression that will calculate the number of seconds between two datetime
sub db_timediff_function {
    my $class = shift;
    my $time1 = shift;
    my $time2 = shift;

    return "unix_timestamp($time1)-unix_timestamp($time2)";
}


1;

__END__

=head1 NAME

DBAccess::MySQL - implements an sql query interface to a MySQL database.

=head1 SYNOPSIS

    use DBAccess::MySQL;

    my$dbobj = MySQL->new($server_name, $db_name, $db_user, $db_passwd);
    $dbobj->get_dbh();
    $dbobj->sqlLock();
    $dbobj->sqlUnlock();

=head1 DESCRIPTION

Implements an sql query interface to a MySQL
database.  Most of this classes' methods are inherited from its
parent, L<DBAccess::DBIObj>.

Constructor.  Takes as parameters the 'web server' name, and the db
name, user acct, and password used with that web server.

     my$dbobj = MySQL->new($server_name, $db_name, $db_user, $db_passwd);

Returns a database handle, using the connect information with which
the object was instantiated.  MySQL-specific code.

    $dbobj->get_dbh();

The following two methods implement MySQL locking and unlocking.

    $dbobj->sqlLock();
    $dbobj->sqlUnlock();
