###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Form::Verify;

use strict;

use CheckPassword;

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $self = {};

    bless($self, $class);


    my @array = ();
    $self->{'ERRORS'} = \@array;
    return $self;
}

## Function checks for "test test"
sub full_name {
    my $self = shift;
    my $fullname = shift;

    if ($fullname =~ m/.?\ .*/) {
    return 1;
    } else {
    return (0, "full name not seperated by space");
    }
}

## Function that checks for valid urls
sub url {
    my $self = shift;
    my $url = shift || return (0, "No url entered");
    my $response = Utility->fetch_url($url);


    if ($response) {
        my $response_code = $response->code();

        if ($response_code < '400') {
            return 1;
        } elsif ($response_code == '401' || $response_code == '402' || $response_code == '403') {
            return 1;
        }
    }
    return (0, "The url entered does not correspond to a valid webpage");

}

## Function that checks for strong password
sub password {
    my $self = shift;
    my ($fullname, $password, $nickname) = @_;

    my $passwd_check = new CheckPassword ($fullname, $password, $nickname);

    my ($error, $error_msg) = $passwd_check->check_for_bad_password();

    if ($error) {
        return (0, $error_msg);
    } else {
        return 1;
    }
}

## Function that checks for long password
sub password_long {
    my $self = shift;
    my ($fullname, $password, $nickname) = @_;

    my $passwd_check = new CheckPassword ($fullname, $password, $nickname);
    my $minlength = $passwd_check->{'MINLENGTH'};

    my $error_msg = undef;
    $error_msg = "please use at least $minlength characters password" if (length($password) < $minlength);

    if ($error_msg) {
        return (0, $error_msg);
    } else {
        return 1;
    }
}

## checks phone numbers only in the case that they are not letters
sub phone {
    my $self = shift;
    my $phone = shift;

    unless ($phone =~ m/[A-Za-z]/) {
    return 1;
    } else {
    return (0, "your phone number contains characters");
    }
}

## checks for text@text.text
sub email {
    my $self = shift;
    my $email = shift;

    if ($email =~ m/.?@.*\./) {
    return 1;
    } else {
    return (0, "This does not appear to be a correct email address.");
    }
}

## checks for dates in many formats
## Add more date formats if neccisary
sub date {
    my $self = shift;
    my ($date, $format) = @_;

#    print STDERR "in Verify::date with date: $date and format: $format\n";
    if ($format eq 'MM/YY') {
    my ($month, $year) = split /\//, $date;

    if ($self->_is_valid_MM($month) &&
        $self->_is_valid_YY) {
        return 1;
    } else {
        return 0;
    }
    } elsif ($format eq 'MM/DD/YY') {
    my ($month, $day, $year) = split /\//, $date;

    if ($self->_is_valid_MM($month) &&
        $self->_is_valid_DD($day) &&
        $self->_is_valid_YY($year)) {
        return 1;
    } else {
        return 0;
    }
    } elsif ($format eq 'MM/DD/YYYY') {
    my ($month, $day, $year) = split /\//, $date;

    if ($self->_is_valid_MM($month) &&
        $self->_is_valid_DD($day) &&
        $self->_is_valid_YYYY($year)) {
        return 1;
    } else {
        return 0;
    }
    }
    elsif ($format eq 'DD/MON/YYYY') {
    my ($day, $month, $year) = split /\//, $date;

    if ($self->_is_valid_DD($day) &&
        $self->_is_valid_MON($month) &&
        $self->_is_valid_YYYY($year)) {
        return 1;
    } else {
        return 0;
    }
    }
    elsif ($format eq 'YYYY/MM/DD') {
    my ($year, $month, $day) = split /\//, $date;

    if ($self->_is_valid_DD($day) &&
        $self->_is_valid_MM($month) &&
        $self->_is_valid_YYYY($year)) {
        return 1;
    } else {
        return 0;
    }
    }
    elsif ($format eq 'YYYY-MM-DD') {
    #print STDERR "in format check for 'yyyy-mm-dd'\n";
    my ($year, $month, $day) = split /-/, $date;

    if ($self->_is_valid_DD($day) &&
        $self->_is_valid_MM($month) &&
        $self->_is_valid_YYYY($year)) {
        return 1;
    } else {
        return 0;
    }
    }
    elsif ($format eq 'DD-MM-YYYY') {
        my ($day, $month, $year) = split /\-/, $date;

        if (($day) && ($month) && ($year)){
        if ($self->_is_valid_DD($day) &&
            $self->_is_valid_MM($month) &&
            $self->_is_valid_YYYY($year)) {
            return 1;
        } else {
            return 0;
        }
        } else {
            return 0;
        }
    }
}

## block of day/month/year functions
sub _is_valid_DD {
    my $self = shift;
    my $day = shift;

    if (($day <= 31) && ($day >= 1)) {
    return 1;
    } else {
    return 0;
    }
}
sub _is_valid_MM {
    my $self = shift;
    my $month = shift;

    if (($month <= 12) && ($month >= 1)) {
    return 1;
    } else {
    return 0;
    }
}
sub _is_valid_MON {
    my $self = shift;
    my $month = lc(shift);
    my @months = ('jan', 'feb', 'mar', 'apr', 'may', 'jun', 'jul', 'aug', 'sep', 'oct', 'nov', 'dec');

    foreach (@months) {
    if ($_ eq $month) {
        return 1;
    }
    }

    return 0;
}
sub _is_valid_YY {
    my $self = shift;
    my $year = shift;

    if (($year <= 99) && ($year >= 1)) {
    return 1;
    } else {
    return 0;
    }
}
sub _is_valid_YYYY {
    my $self = shift;
    my $year = shift;

    if (($year <= 9999) && ($year >= 1)) {
    return 1;
    } else {
    return 0;
    }
}
## END DATE FUNCTIONS

## crude credit card verification only checks for 16 digits
sub cc_num {
    my $self = shift;
    my $string = shift;

    if ($string =~ m/[0-9]{16}/) {
    return 1;
    } else {
    return 0;
    }
}

## check to see if string is more than just whitespace
sub non_empty {
    my $self = shift;
    my $string = shift;

    $string =~ s/\s//g;

    if ($string eq '') {
    return 0;
    } else {
    return 1;
    }
}

## just checks to make sure the form is not empty
sub empty_case {
    my $self = shift;
    my $string = shift;

    if ($string) {
    return 1;
    } else {
    return (0, "is empty");
    }
}

## this function should be called not the functions individually
sub verify {
    my $self = shift;
    my ($type, $name, $value, $format, $fullname, $nickname) = @_;
    my $result = undef;
     my $errormsg = undef;

    if ($type eq 'fullname') {
    ($result, $errormsg) = $self->full_name($value);
                unless($result){
                        $name = $errormsg;
                }
    } elsif ($type eq 'url') {
    ($result, $errormsg) = $self->url($value);
                unless($result){
                        $name = $errormsg;
                }
    } elsif ($type eq 'password') {
    ($result, $errormsg) = $self->password($fullname,$value,$nickname);
              unless ($result) {
                $name = $errormsg;
                }
    } elsif ($type eq 'password_long') {
    ($result, $errormsg) = $self->password_long($fullname,$value,$nickname);
              unless ($result) {
                $name = $errormsg;
                }
    } elsif ($type eq 'phone') {
    ($result, $errormsg) = $self->phone($value);
                unless($result){
                        $name = $errormsg;
                }
    } elsif ($type eq 'date') {
    $result = $self->date($value, $format);
    } elsif ($type eq 'cc_num') {
    $result = $self->data($value);
    } elsif ($type eq 'not_req') {
    return 1;
    } elsif ($type eq 'email') {
    ($result, $errormsg) = $self->email($value);
                unless($result){
                        $name = $errormsg;
                }
    } elsif ($type eq 'icq') {
        return 1;
    } elsif ($type eq 'non_empty') {
    $result = $self->non_empty($value);
    }  else {
    ($result, $errormsg) = $self->empty_case($value);
                unless($result){
                        $name = $name."&nbsp;".$errormsg;
                }
    }

    unless ($result) {
    $self->_add_error($name);
    }
}


sub verify_multiple {
    my $self = shift;
    my $params = shift;

    foreach my $param (@$params) {
    $self->verify($param->{'type'},
              $param->{'msg_name'},
              $param->{'value'},
              $param->{'name'});
    }
}


sub _add_error {
    my $self = shift;
    my $name = shift;

    my $error_ref = $self->{'ERRORS'};

    my @errors = @$error_ref;
    push @errors, $name;
    $self->{'ERRORS'} = \@errors;
}

sub errors {
    my $self = shift;

    my $error_ref = $self->{'ERRORS'};

    my @errors = @$error_ref;
    if (scalar(@errors) > 0) {
    return join("<br /> ", @errors);
    }

    return undef;
}

1;


__END__

=head1 NAME

=head1 SYNOPSIS

=head1 DESCRIPTION

=cut
