###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Gizmo;
use strict;
#use MIME::Types;

use Data::Dumper;
use Time::localtime;

use Metadot::MIME::Types;

use Metadot::PortalFactory;
use Metadot::GizmoBase;
use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $CACHE $ACCESS_BROKER %REQUEST_VARS $CGI $DISPLAY);
use Metadot::MRU;
use Portal;
use Metadot::MetadotController;
use Metadot::BaseGizmoContainer;

use Carp qw(&carp &croak &confess);
use vars qw(@ISA);
use DBAccess;
use Metadot::Permissions;
use Auditable;
use HTML::Components;
use HTML::Verify;
use Metadot::Utils::ModuleMgmt;
use Utility;
use ErrorHandling;
use Metadot::JSForm;
use File::Path;
use Metadot::DBMap;
use Metadot::ObjectAccess::Restorable;
use Metadot::ObjectAccess::Memento;
use Metadot::PopupMgr;
use Metadot::GizmoContainer;
use Metadot::UpdateNotification::UpdateNotifier;

@ISA = qw(Metadot::GizmoBase Portal);

sub get_version {
    return "No Version Information Available";
}

my %GIZMO_LISTS;
my %FOUND_MODULES_HASHES;

my $is_public = "Yes";
my $is_visible_in_add_child_lists = 1;

initialize_gizmo_list();


#####################################################
######               INITIALIZE               #######
#####################################################

sub initialize_gizmo_list {
    #print STDERR "Metadot Portal Server " . MetadotConfig::get_version() . ": Initializing gizmos...\n";

    get_gizmo_list();
}

#aju
sub get_found_modules {
    my $class = shift;

    my $section_name = Metadot::Utils::ModuleMgmt->get_server_sectionname();
    if (exists ($FOUND_MODULES_HASHES{$section_name})) {
        return $FOUND_MODULES_HASHES{$section_name};
    }
    else {
        my $all_modules = Metadot::Utils::ModuleMgmt->read_modules_in_directory("Gizmo");
        while (my ($module, $rel_filename) = each %{$all_modules} ) {
            require "$rel_filename";
        }
        $FOUND_MODULES_HASHES{$section_name} =
           Metadot::Utils::ModuleMgmt->get_supported_modules();
        my $fm = $FOUND_MODULES_HASHES{$section_name};
        return $fm;
    }
}

#aju
sub get_gizmo_list {
    my $class = shift;

    my $section_name = Metadot::Utils::ModuleMgmt->get_server_sectionname();
    if (exists ($GIZMO_LISTS{$section_name})) {
        return @{$GIZMO_LISTS{$section_name}};
    }
    else {
        my %fm = %{get_found_modules()};
    #   print STDERR "in get_found_modules, got found_modules: " , Dumper(\%fm), "\n";
        my @gl = sort { $a->get_gizmo_name() cmp $b->get_gizmo_name() } (keys %fm);
    #   print STDERR "in get_found_modules, got gl: " , Dumper(\@gl), "\n";
        $GIZMO_LISTS{$section_name} = \@gl;
        return @gl;
    }
}

sub lists_gizmos {
    my $class = shift;
    return $class->get_gizmo_list();
}


# Note: this is deprecated. Implement "Navigable" for the same effect
# allows a Gizmo class to report whether it should appear in navigation
sub is_navigation_gizmo {
    my $self = shift;
    return $self->isa('Metadot::Navigable');
}


# this feature used to be closed-source, but now it's open source
sub is_gizmo_shortcut_available {
    my $self = shift;
    return 1;
}

sub filtered_gizmos {
  my $class = shift;
  my @filteredlist;

  #filter out gizmos that have been set to not be public gizmos
  #aju
  my @gl = $class->get_gizmo_list();
#   foreach my $gizmo (@GIZMO_LIST) {
  foreach my $gizmo (@gl) {
    my $public = $gizmo->get_is_public();

    if ((lc $public) ne "no") {
      push(@filteredlist, $gizmo);
    }
  }

  return @filteredlist;
}


# this is now an alias for BaseGizmoContainer->build_child_list
sub build_gizmo_list {
    my ($self, %params) = @_;

    unless (defined($params{parent_iid}) || defined($params{parent_iid})) {
        $params{parent_iid} = $self->id();
    }
    return Metadot::BaseGizmoContainer::build_child_list($self, \%params);
}




###
### This function provides an interface to the class
### variable $is_public. This variable is a boolean
### flag for wether or not to list the gizmo.
###
sub get_is_public {
  my $self = shift;
  return $is_public;
}

###
### This function provides returns true if the gizmo should appear in
### any add_child menus. It is an interface to the class variable
### $is_visible_in_add_child_lists
###
sub is_visible_in_add_child_lists {
    my $self = shift;
    return $is_visible_in_add_child_lists;
}

###
### This function provides a default name for the gizmo
###
sub get_gizmo_name {
  my $self = shift;
  #CKIM-SENDABLE: prevent this method from returning a reference
  return ref($self) || $self;
}








#####################################################
######                                        #######
######               CONSTRUCTOR              #######
######                                        #######
#####################################################

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    # Some gizmos pass only an $id to the constructor, while
    # others send a hashref of arguments. This causes the
    # constructor to always see a hashref of arguments, no
    # matter what the actual parameters were.
    my $self = $class->SUPER::new($class->normalize_constructor_args(@_));

    if (defined $self) {
        # some code uses isa, which is incorrent
        $self->{isa} = $self->is_a();
    }

    return $self;
}


sub initialize {
    my $self = shift;

    $self->debug_msg(5, "In &lt;$self&gt; ".__PACKAGE__."->initialize()");
    $self->SUPER::initialize(@_);

    # for backward compatibility with Gizmos that were using _initialize
    return $self->_initialize(@_);
}

# for backward compatibility with Gizmos that were using _initialize
sub _initialize {
}


#aju 01-25-02
# don't instantiate the gizmo; just check as to whether it exists. return 1 if exists,
# 0 if not.
sub exists {
    my ($class, $id) = @_;

    $id = undef unless Utility->is_valid_integer($id); # secu_200401
    if ((!defined $id) || ($id eq '')) {
        return 0;
    }
    else {
        my $temp = DBAccess->sqlSelect("iid", "instance", "iid=$id");

        if ($temp) {
            return 1;
        }
        else {
            return 0;
        }
    }
}








#####################################################
######                                        #######
######            FRONT-END METHODS           #######
######                                        #######
#####################################################

sub www_show {
  my $self = shift;
  $self->debug_msg(3, "Gizmo::www_show()");

  my $iid = Portal::defined_or_exit($self, $self->id,"id is missing.");

  my $title = $self->name();
  my $content = $self->show();
  $DISPLAY->debug_msg(4, "Gizmo::www_show() finished displaying content.");
  $DISPLAY->print_portal({
      content => $content,
      title   => $self->name(),
      subject => $self,
  });
  $DISPLAY->debug_msg(4, "Gizmo::www_show() finished displaying portal.");
}


##################
sub www_change_owner {
    my $self = shift;
    $self->debug_msg(3, "Gizmo::www_change_owner()");
    my $iid = $self->id();
    my $isa = $self->is_a();
    my $uid = $self->uid();
    my $name = $self->name();
    my $title = "Changing Owner";
    my $headings .= "<h1>$title</h1>";
    $headings .= "<p /><b>$name</b><p />";
    my $message;
    my $header;
    if ($FORM{confirm} && $FORM{uid}) {
    my $new_uid = $FORM{uid};
    my $new_owner_name = $USER->get_fullname_from_uid($new_uid);
    $DISPLAY->two_choices("Are you sure that you would like to give ownership of this $isa to $new_owner_name?",
               "stop",
               "Yes", "index.pl?op=change_owner&isa=$isa&add=1&iid=$iid&new_uid=$new_uid",
               "No", "index.pl?op=change_owner&isa=$isa&iid=$iid");
    } elsif ($FORM{add}) {

    my $new_uid = $FORM{new_uid};

    if( $self->_check_uid_upload_limit( $new_uid, $isa, $iid ) ){

        $self->_change_gizmo_owner_to_uid($iid, $new_uid);
        $DISPLAY->md_redirect("index.pl?iid=".$self->parent_id);

        } else {

        $DISPLAY->two_choices("Operation not allowed!<p />Remaining disk quota of intended new owner is not enough to hold attachment file. Please have intended new owner delete some files to free up space or talk to your site administrator to increase her quota.", 'error',
                   "Back to change owner",
                   "index.pl?op=change_owner&isa=$isa&iid=$iid",
                   "Back to parent category",
                   "index.pl?iid=" . $self->parent_id,
                  );
        }

    } else {
    if ($FORM{search}) {
        $message .=
          "<form method=post action = index.pl>"
        .$DISPLAY->make_hidden()
          ."<input type=hidden name = isa value = '$isa'>
    <input type=hidden name = iid value = '$iid'>
    <input type=hidden name = confirm value = '1'>
    <input type=hidden name = op value = 'change_owner'>";
        my $m = $USER->get_search_user_results({
                            EMAIL => "$FORM{email}",
                            FULLNAME => "$FORM{fullname}",
                            NICKNAME => "$FORM{nickname}",
                            SELECT_ONE => '1'});
        if ($m =~ m/None of the information was filled in.  Please go back and enter some information/i ||
        $m =~ m/No users match the criteria who are not already part of the group/i){
        $message = $m;
        $header = "Search Users Again!";
        }else{
        $message .= $m;
        $message .=    "<input type=submit value = 'Change Owner' class='button'></form>";
        $header = "Select a new owner, or search again";
        }
        } else {
        $message =                                "Search for a person you would like to give ownership of this $isa to.";
        $header = "Search Users";
    }



    my $add_user_form = Components->get_search_users({HEADER => $header,
                              ACTION => "index.pl",
                              MESSAGE => $message,
                              PARAMS => {search => 1, iid => $iid, isa => $isa, op => 'change_owner'},
                              ORIENTATION => "horizontal",
                              WIDTH => "100%",
                             });
    return $DISPLAY->print_form($headings . $add_user_form . "<br /><br />");
    }
}

sub _check_uid_upload_limit{
    my $self = shift;
    my $new_uid = shift;
    my $isa = shift;
    my $iid = shift;

    $iid = undef unless Utility->is_valid_integer($iid); # secu_200401
    my ($file1, $old_size) = DBAccess->sqlSelect("file1, file1_size", "instance", "iid=$iid");

    my $file_size = Metadot::UploadsManager->get_upload_size( { isa => $isa,
                                id => $iid
                                  } );

    $file_size = $file_size || $old_size;

    my ($upload_size, $upload_number) = $USER->get_uploads_space_and_num_files_quota( $new_uid );

    if($upload_size == -1){

    $upload_size = $PARAMS->{'max_total_upload_size'};
    }
    # aju: convert upload_size from M to bytes (sheesh)
    my $upload_quota_bytes = $upload_size * 1048576;

    if($upload_number == -1){

    $upload_number = $PARAMS->{'max_total_number_upload'};
    }

    my ($used_space, $uploaded_number) = $self->space_used( $new_uid );

    #print STDERR "used space for $USER->id is: $used_space FILE SIZE IS $file_size\n";

    my $left = $upload_quota_bytes - $used_space;

    if( ($file_size <= $left) && ($upload_number > $uploaded_number) ) {
    return 1;

    } else {

    return 0;
    }
}

##############
sub _change_gizmo_owner_to_uid {
    my $self = shift;
    my $iid = shift;
    my $new_uid = shift;
    #First we need to see if the person is authorized to do this action.  We'll check to see if the person is in the admin group, or if they are the owner.
    if ( $ACCESS_BROKER->is_allowed_to_do('admin_tasks') || ($USER->{uid} eq $self->uid())) { #either they are in the admin group, or they match the uid of the gizmo (i.e. they are the owner of the gizmo)
    $self->uid($new_uid);
    $self->save();

    Metadot::UploadsManager->change_uploads_userid( { isa => $self->is_a(),
                              id => $self->id(),
                              new_userid => $new_uid } );
    return 1;

    } else {
    return 0;
    }
}

###########################

sub www_edit_permissions {
    my $self = shift;
    $self->debug_msg(3, "Gizmo::www_edit_permissions()");

    my $class = ref($self);
    $self->debug_msg(1, "SELF: $self CLASS: $class OP: edit_permissions");


    my $iid = $self->id();
    my $isa = $self->is_a();

    if (!defined $iid){
    $self->click_back ("Error:", "id is missing.");
    exit(0);
    }
    my $return_location;



    if  ($self->go_back_to_parent_after_editing_permissions) {
    $return_location = sub {
        $DISPLAY->md_redirect($DISPLAY->make_url("index.pl?iid=".$self->parent_id));
    }
    } else {
    $return_location = sub {
        $DISPLAY->md_redirect($DISPLAY->make_url("index.pl?iid=".$iid));
    }
    }


    my $form = $self->edit_permissions_form(\%FORM,
                        $return_location,
                        $USER);
    $DISPLAY->print_form($form);
}

##############
sub go_back_to_parent_after_editing_permissions {
    return 1;
}

##################
# This method implements security infrastructure and should NOT be
# overridden in derived classes.
# XXX [claudio] no longer virtual. update documentation below
##############


##################
#sub www_create {
#  my $self = shift;
#  $self->debug_msg(3, "Gismo::www_create()");
#
#  my $isa = ref $self;
#  my $parentid = Portal::defined_or_exit($self, $self->id,"parent id is missing.");
#  # we re-instanciate the object because we want a brand new one
#  $self = $isa->new({
#            uid => $USER->uid(),
#            parent_id => $parentid,
#           });
#
#  # this method should ALWAYS be overridden in derived classes.
#  $self->process_edit_form();
#}

# this method should ALWAYS be overridden in derived classes.
sub process_edit_form {
  my $self = shift;

  $self->debug_msg(3, "Gizmo::process_edit_form()");
  my $isa = ref $self;

  die "method process_creation_form should always be overidden in default classes: $!";

  # fill the object with form values (SAMPLE IMPLEMENTATION)
  $self->name($FORM{name});
  $self->url($FORM{url});
  $self->description($FORM{description});
  $self->long_description($FORM{longdescription});
  $self->keywords($FORM{keywords});
  # check if everything is ok
  my $errormsg = "";
  $errormsg .= "<li>Too bad! Name field cannot be empty...</li>" if (!$self->name()) ;
  my $parentid = $self->parent_id();
  if ($errormsg) {
    $self->id($parentid);
    $self->www_add($errormsg);
  } else {
    $self->add();
    $DISPLAY->two_choices("<Big>New " . $self->get_gizmo_name() .
               " added!</big><br /><br />What do you want to do?",
                       "confirm",
               "Add another " . $self->get_gizmo_name(), "index.pl?iid=$parentid&isa=$isa&op=add",
               "Enough for now!", "index.pl?iid=$parentid");
  }
}

##############

sub www_modify {
  my $self = shift;
  my $errormsg = shift;
  $self->debug_msg(3, "Gizmo::www_modify()");
  my $iid = $self->id();
  my $isa = $self->is_a();
  my $target = $SESSION->{referer};
  if (!defined $iid){
    $self->click_back ("Error:", "id is missing.");
    exit(0);
  }

  if (!$target){
      $target = "";
  } elsif (!($target =~ m/index\.pl/)) {
      $target .= "index.pl";
  } elsif (!($target =~ m/iid|id/)) {
      if ($target =~ m/\?/) {
          $target .= "&amp;iid=".$self->parent_id();
      } else {
          $target .= "?iid=".$self->parent_id();
      }
  }

    $errormsg = $errormsg ?
                "<p /><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p />\n" :
                "";

  my $form = $self->show_edit_form("Customizing " . $isa->get_gizmo_name());
  my $content = "<form action='index.pl'  method='post' ENCTYPE=\"multipart/form-data\" name=\"fm\">" ;
  $content .= "
<input type='hidden' name='dummy' value='foo'>
<input type='hidden' name='op' value='save'>
<input type='hidden' name='isa' value='$isa'>
<input type='hidden' name='iid' value='$iid'>
<input type='hidden' name='target' value='$target'>
".$DISPLAY->make_hidden()."\n" . $self->get_www_modify_buttons() . "
$errormsg
$form" .
"<p />";
  my $featureset_tag = MetadotConfig->get_featureset_tag() ;
  my $use_tm = MetadotConfig->get_use_template_mgmt() ;
  if ($PARAMS->{'template_mgmt'} && ($featureset_tag eq 'closed')
      && ($use_tm eq 'true')) {
      require Metadot::SystemApp::TemplateApp;
      $content .=  Metadot::SystemApp::TemplateApp->build_url($iid, $isa, 'show',
                   "Manage " . $self->get_gizmo_name() . " Template");
  }
$content .= $self->get_www_modify_buttons() . "
<br />&nbsp;
</form>
";

  $DISPLAY->print_form($content);
}

sub get_www_modify_buttons {
    my $self = shift;
    $self->debug_msg(3, "Gizmo::get_www_modify_buttons()") ;
    return "<table align='right'><tr><td><input type='submit' value='Save' class='button'></td></tr></table>\n";
}

sub www_modify_properties {
  my $self = shift;
  my $errormsg = shift;
  $self->debug_msg(3, "Gizmo::www_modify_properties()");
  my $iid = $self->id();
  my $isa = $self->is_a();
  my $target = $SESSION->{referer};
  if (!defined $iid){
    $DISPLAY->click_back ("Error:", "id is missing.");
    exit(0);
  }

  if (!$target){
      $target = "";
  } elsif (!($target =~ m/index\.pl/)) {
      $target .= "index.pl";
  } elsif (!($target =~ m/iid|id/)) {
      if ($target =~ m/\?/) {
          $target .= "&amp;iid=".$self->parent_id();
      } else {
          $target .= "?iid=".$self->parent_id();
      }
  }

    $errormsg = $errormsg ?
                "<p /><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p />\n" :
                "";

  my $form = $self->show_edit_properties_form("Customizing " . $isa->get_gizmo_name() . " Properties");
  my $content = "<form action='index.pl' method='post' ENCTYPE=\"multipart/form-data\" name=\"properties_form\">" ;
  $content .= "
<input type='hidden' name='dummy' value='foo'>
<input type='hidden' name='op' value='save_properties'>
<input type='hidden' name='isa' value='$isa'>
<input type='hidden' name='iid' value='$iid'>
<input type='hidden' name='target' value='$target'>
".$DISPLAY->make_hidden()."
<table align='right'><tr><td><input type='submit' value='Save' class='button'></td></tr></table>
$errormsg
$form" .
"<p />";

$content .= "<table align='right'><tr><td><input type='submit' value='Save' class='button'></td></tr></table>
<br />&nbsp;
</form>
";

  $DISPLAY->print_form($content);
}

sub show_edit_properties_form {
    my $self = shift;
    $DISPLAY->debug_msg(3, "Gizmo::show_edit_properties_form()") ;

    #my $show_from = $self->process_date_value( date => $self->{show_from} );
    #my $show_end  = $self->process_date_value( date => $self->{show_end} );

    my $show_from = Utility->render_db_date_as_html( $self->{show_from} );
    my $show_end  = Utility->render_db_date_as_html( $self->{show_end} );


    my $s = '';

    $s .= $self->get_short_name_control();

    $s .= $DISPLAY->get_calendar_popup_scripts();
    $s .= "<p>
  <table border='0' cellspacing='0' cellpadding='0'>
    <tr><td>" . $DISPLAY->get_content_live_image() . "</td>
        <td rowspan='3'><h1>Content Scheduling</h1></td></tr>
    <tr><td>" . $DISPLAY->get_content_live_soon_image() . "</td></tr>
    <tr><td>" . $DISPLAY->get_content_expired_image() . "</td></tr>
    </table></p>\n";
    $s .= "<h2>for " . $self->name() . "</h2>\n";
    $s .= "<p>Use the following date fields to specify the date range to show the particular gizmo. These date values are used in addition to the gizmo's permissions to decide if an individual user can view the gizmo and when it is visible.</p>\n";
    $s .= "<p><b>Begin showing on the following date: - [YYYY-MM-DD]</b><br />" . $DISPLAY->get_content_live_image() . $DISPLAY->get_calendar_popup(name => 'show_from', value => $show_from) . "</p>\n";
    $s .= "<p><b>Hide after the following date: - [YYYY-MM-DD]</b><br />" . $DISPLAY->get_content_expired_image() . $DISPLAY->get_calendar_popup(name => 'show_end', value => $show_end) . "</p>\n";
    $s .= "<p>Both fields are optional and you can enter both, none, or just one date. If a field is left blank, no limit will be placed on that date field. All specified dates are inclusive. Once the hide date passes, the gizmo's content is not deleted but hidden from view. Expired content is still visible in Edit mode.</p>\n";
    $s .= "<h2>Content Traffic Lights</h2><p />Use the 'Content Traffic Lights' to help you. While in edit mode, each gizmo with a set scheduling value will display one of the following icons: ";
    $s .= "<p>
  <table border='0' cellspacing='5' cellpadding='0'>
    <tr><td>" . $DISPLAY->get_content_live_image() . "</td><td><b>Content is live.</b></td></tr>
    <tr><td>" . $DISPLAY->get_content_live_soon_image() . "</td><td><b>Content is scheduled to be live soon and is not yet visible in non-editing mode.</b></td></tr>
    <tr><td>" . $DISPLAY->get_content_expired_image() . "</td><td><b>Content has expired and is not visible in non-editing mode.</b></td></tr>
  </table></p>\n";



    # Notifications
    #################################################################################

    $s .= "<h1>Notify users of this ".$self->get_gizmo_name()."</h2>\n";

    # show on-demand notify button
    if ($self->isa('Metadot::Notifiable')) {
       $s .= "<hr/>\n";
       $s .= $self->show_notification_properties();
    }

    # show subscription-based notification properties
    if (Metadot::UpdateNotification::UpdateNotifier->child_has_notifying_parent($self))
    {
        $s .= Metadot::UpdateNotification::UpdateNotifier->show_notification_properties_for_child($self);
    }


    #################################################################################


    # used by cycling containers
    if ($self->isa('Metadot::CyclingContainer')) {
        $s .= "<hr/>\n";
        $s .= $self->show_cycling_properties();
    }

    return $s;
}


sub get_traffic_lights_display {
    my $self = shift;
    return $DISPLAY->get_traffic_lights_display(subject => $self);
}

# now an alias to Utility->process_date_value()
sub process_date_value {
    my $self = shift;
    my %args = @_;
    $self->debug_msg(3, "Gizmo::process_date()") ;

    return Utility->render_db_date_as_html($args{date});
}




# this is now an alias for Display->get_calendar_popup_scripts
sub get_calendar_popup_scripts {
    my $self = shift;
    return $DISPLAY->get_calendar_popup_scripts(@_);
}

# this is now an alias for Display->get_calendar_popup
sub get_calendar_popup {
    my $self = shift;
    return $DISPLAY->get_calendar_popup(@_);
}

sub www_save_properties {
    my $self = shift;
    $self->debug_msg(3, "Gizmo::www_save_properties()") ;

    my $iid = Portal::defined_or_exit($self, $self->id,"id is missing.") ;

    my $error_msg = $self->save_properties();
    if( $error_msg ne '') {
        $self->www_modify_properties( $error_msg );
    } else {
        my $redirect_iid = (!$self->isa('Metadot::Navigable') && $self->parent_id() >= 0) ? $self->parent_id() : $iid;
        my $target = $FORM{delete_image} ? $SESSION->{referer} : "index.pl?iid=$redirect_iid" ;
        $DISPLAY->md_redirect($target);
    }
}

## if you override this method in a subclass (such as Category.pm),
## make sure you call this SUPER method in the end
sub save_properties {
    my $self = shift;
    my %args = @_;
    $self->debug_msg(3, "Gizmo::save_properties()") ;

    my $iid = Portal::defined_or_exit($self, $self->id,"id is missing.") ;

    my $error_msg = '';

    $self->{show_from} = $FORM{show_from} ? $FORM{show_from} . ' 00:00:01' : undef;
    $self->{show_end} = $FORM{show_end} ? $FORM{show_end} . ' 23:59:59' : undef;

    # process cycling
    if ($self->isa('Metadot::CyclingContainer')) {
        $self->process_cycling_properties()
    }

    # process short_name
    $self->process_short_name_control();

    $self->save();

    return $error_msg;
}

##############

sub www_save {
  my $self = shift;
  $self->debug_msg(3, "Gizmo::www_save()");

  my $iid = Portal::defined_or_exit($self, $self->id,"id is missing.");

  # fill the object with form values
  $self->name($FORM{name});
  $self->url($FORM{url});
  $self->description($FORM{description});
  $self->long_description($FORM{longdescription});
  $self->keywords($FORM{keywords});

  # check if everything is ok
  my $errormsg = "";
  $errormsg .= "<li>Too bad! Title field cannot be empty...</li>" if (!$self->name()) ;



  if ($errormsg) {
    $self->www_modify($errormsg);
  } else {
    $self->save();
    $DISPLAY->md_redirect($DISPLAY->make_url("index.pl?iid=".$self->parent_id));
  }
}

##############

sub www_delete {
  my $self = shift;
  my $isa = $self->is_a();
  my $gizmo_name = $isa->get_gizmo_name();

  $DISPLAY->debug_msg(3, "Gizmo::www_delete()");
  $DISPLAY->print_http_header;
  $DISPLAY->two_choices("Do you want to delete this $gizmo_name: <br />".$self->name."?",
                     "caution",
             "Yes please", "index.pl?iid=".$self->id."&isa=$isa&op=delete_ok",
             "No!", $SESSION->{referer});

  exit();
}

##############

sub www_delete_ok {
  my $self = shift;
  $self->debug_msg(3, __PACKAGE__.": $self\::www_delete_ok()");

  unless ($self->delete()) {
      $DISPLAY->click_back('Error deleting.');
      exit(0);
  }

  if ($self->can('_show_edit_success')) {
    $self->_show_edit_success();
  } else {
    $DISPLAY->md_redirect($DISPLAY->make_url("index.pl?iid=" . $self->parent_id));
  }
}

##############

sub www_up {
  my $self = shift;
  $self->debug_msg(3, "Gizmo::www_up()");
  $self->up();
# To do up after inverse sorting in Table
  my $url = $SESSION->{referer};
  $url =~ s/&order.+//;
  $DISPLAY->md_redirect($url);
}

##############

sub www_down {
  my $self = shift;
  $self->debug_msg(3, "Gizmo::www_down()");
  $self->down();
# To do up after inverse sorting in Table
  my $url = $SESSION->{referer};
  $url =~ s/&order.+//;
  $DISPLAY->md_redirect($url);
}

##############

sub www_cut {
  my $self = shift;

  #print STDERR "$self\->www_cut id='$self->{id}'\n";
  $self->debug_msg(3, "Gizmo::www_cut()");
  my $previous_parentid = $self->parent_id();

  if($self->id() != MetadotConfig->get_reserved_iid('ROOT_CATEGORY')) {

      $self->_cut();

  } else {

      $previous_parentid = MetadotConfig->get_reserved_iid('ROOT_CATEGORY');

  }


  my $url = $DISPLAY->make_url("index.pl?iid=" . $previous_parentid);
  #print STDERR "$self\->www_cut REDIRECTING TO $previous_parentid\nURL: $url\n";

  $DISPLAY->md_redirect($url);


}

##############

sub www_clipboard_delete {
  my $self = shift;

  my $new_parent = $FORM{id};
  #unpack list of ids
  my @ids = split(',', $FORM{dl});
  my $count = $FORM{count};
  my $perms_error = 0;
  if ($count eq ($#ids + 1)) {
      foreach my $obj_id (@ids)
      {
          my $gizmo;
          $obj_id = undef unless Utility->is_valid_integer($obj_id); # secu_200401

          # try a newer way of restoring the instance
          #my ($c_isa)=DBAccess->sqlSelect("isa", "instance", "iid=$obj_id");
          #eval {
          #    require "Gizmo/$c_isa.pm";
          #    $gizmo = $c_isa->new($obj_id);
          #};

          $gizmo = Metadot::PortalFactory->get_gizmo_object($obj_id);
          if (defined $gizmo)
          {
              if ($gizmo->is_allowed_to_do('delete', $USER, 1)) {
                  $gizmo->delete("YES");
              }
              else
              {
                  #print STDERR "don't have permissions to delete $obj_id\n";
                  $perms_error = 1;
              }
          }
      }
  }
  else
  {
      #print STDERR "id count did not match up..\n";
  }


  if ($perms_error) {
      ErrorHandling->printError("You did not have permission to delete some of the selected clipboard items.");
  }
  else {
      $DISPLAY->md_redirect($DISPLAY->make_url("index.pl?iid=" . $new_parent));
  }
}



sub www_paste {
  my $self = shift;

  $self->debug_msg(3, "Gizmo::www_paste()");
  # is this necessary? isn't 'self' sufficient?
  #my $new_parent = $FORM{id};
# print STDERR "in www_paste, with form vars: ", Dumper(\%FORM), "\n";

  if (exists($FORM{'del.x'})) { # delete
      my @del_list = ();
      foreach my $key (keys %FORM) {
          $self->debug_msg(3, "delete from clipboard: $key :: " . ($FORM{$key}||"") );
          if($key =~ /^box_\d+/) {
              my $iid = $FORM{$key};
              push @del_list, $iid;
          }
      }
      my $del_string = join(",",  @del_list);
      my $count = $#del_list + 1;
      if ($count > 0) {
      $DISPLAY->two_choices("Do you want to delete the selected objects in the clipboard?<br />If an object is " .
                 "of type 'Page', its children will be deleted as well.",
                 "caution",
                 "Yes", "index.pl?isa=" . $self->{is_a} ."&id=".$self->id()."&op=clipboard_delete&count=$count" .
                 "&dl=$del_string",
                 "No", $SESSION->{referer});
      exit();
      }
  }
  else {            # paste..
      # prevent pasting into the wrong type of object
      if ($self->can('paste')) {
          #print STDERR "pasting items\n";
          my @paste_iids = ();
          foreach my $key (keys %FORM) {
              $self->debug_msg(3, "paste: $key :: " . ($FORM{$key}||"") );

              if($key =~ /^box_\d+/ && defined $FORM{$key} && $FORM{$key} ne '') {
                  push @paste_iids, $FORM{$key}
              }
          }
          $self->paste(\@paste_iids);
      } else {
          $DISPLAY->print_click_back('Error', 'Cannot paste into this page');
      }

  }

  $DISPLAY->md_redirect($DISPLAY->make_url("index.pl?iid=" . $self->id()));
}

sub www_download {
    my $self = shift;
    $self->debug_msg(3, "Gizmo::www_download");

    my $e_g_g_name = "The Metadot Portal Server 4.5.1 Product Team !" ;
    return $self->_get_e_g_g() if $self->name eq $e_g_g_name ;

    my $uid = $self->uid;

    my $iid = $self->id ;
    my $file = $self->file1;


    my $absolutepathfileuploaddir = $PARAMS->{absolutepathfileuploaddir};

    my  $read_file = $absolutepathfileuploaddir . "/$iid/$file";

    $self->transmit_file(
        file        => $file,
        read_file   => $read_file,
    );
}



sub www_force_download {
    my $self = shift;
    my $iid = $self->id ;
    my $file = $self->file1;
    my $absolutepathfileuploaddir = $PARAMS->{absolutepathfileuploaddir};
    my  $read_file = $absolutepathfileuploaddir . "/$iid/$file";

    $self->transmit_file(
        file        => $file,
        read_file   => $read_file,
    );
}
# [vadim] 4.5 last-minute solution for the default NewsPaperView
# to avoid ugly 'bad images'
# return 1 if file is available to download, othewise return undef
sub check_download {
    my $self = shift ;
    $self->debug_msg(3, "Gizmo::check_download") ;
    my $uid = $self->uid ;
    my $iid = $self->id ;
    my $file = $self->file1 ;
    my $absolutepathfileuploaddir = $PARAMS->{absolutepathfileuploaddir} ;
    my $read_file = $absolutepathfileuploaddir . "/$iid/$file" ;
    return undef unless (open OUT, "<$read_file") ;
    close OUT ;
    return 1 ;
}

# [vadim] little easter egg, please don't remove :)
# if you have any question feel free to contact:
# vadim@metadot.com, jelezniakov@hotmail.com
sub _get_e_g_g {
    my $class = shift ;
    my $imageslocation = $PARAMS->{imageslocation} ;
    my $m_file = "e"."g"."g"."."."m"."i"."d" ;
    print "HTTP/1.0 200 OK\nContent-Location: team.html\nContent-disposition: inline; filename=team.html\nContent-Type: text/html\n\n" ;
    { local $/ = undef ; while(<Metadot::JSForm::DATA>) { s/<<E_G_G_._M_I_D>>/$imageslocation\/js\/$m_file/ ; print ; } }
    close Metadot::JSForm::DATA ;
}

##############


#####################################################
######                                        #######
######            BACK-END METHODS            #######
######                                        #######
#####################################################

#####################################################
###          DATABASE BACK-END METHODS            ###
#####################################################


{
    my $search_params = {
             searchable_fields => [ qw(name description keywords) ],
             displayed_fields  => [ 'description' ],
            };

    sub get_search_params {
        my $self = shift;
        return $search_params;
    }
}


# CKIM: The first_position, last_position, and add methods are going to have
#       to leave Gizmo.pm, and go to GizmoContainer.
#

##############################################################################

sub first_position {
  my $self = shift;

  my $p = $self->{parent_id};
  if (defined $p) {
      my $c = $self->{is_a};
      my ($current_lowest_position) = DBAccess->sqlSelect("MIN(instanceorder)", "instance",  "parentiid=$p");
      $current_lowest_position--;
      $self->{instance_order} = $current_lowest_position;
  }
  return $self->{instance_order};

}

##############################################################################

sub last_position {
  my $self = shift;

  my $p = $self->{parent_id};
  if (defined $p) {
      my $c = $self->{is_a};
      my ($current_last_position) = DBAccess->sqlSelect("MAX(instanceorder)", "instance",  "parentiid=$p");
      $self->{instance_order} = ($current_last_position||0) + 1;
  }
  return $self->{instance_order};

}

##############

sub delete {
  my $self = shift;

  $self->debug_msg(3, __PACKAGE__.": $self delete");

  my $result = $self->SUPER::delete(@_);

  $self->reorder_parent();

  return $result;

}

# this was moved from GizmoBase::delete to a separate method to allow
# polymorphism
sub delete_attachments {
    my $self = shift;
    if ($self->file1()) {
      $self->delete_attachment();
    }
}


# allow the parent to determine how reodredering is done, not the children
# aju -- catch case where parent no longer exists.
sub reorder_parent {
    my $self = shift;

    $self->debug_msg(5, __PACKAGE__.": $self\->reorder_parent");
    my $parent = $self->get_parent();
    $self->debug_msg(5, __PACKAGE__.": $self\->reorder_parent has parent $parent");

    if ($parent) {
        return $parent->reorder_children();
    }
    else {
        return undef;
    }
#   return $self->get_parent()->reorder_children();
}

sub reorder_children{
    my $self = shift;
    # CKIM: moved this function to GizmoContainer. Any class that actually
    #       implements this method should be changed to inherit from GizmoContainer instead.
    Metadot::BaseGizmoContainer->reorder_children({parent_obj => $self});
}

##############


# return channel id
sub cid {
  my $s = shift;
  my $id = $s->id();

  my ($cid) = DBAccess->sqlSelect("cid", "channel", "iid=$id and isa='Internal' or isa='Poll'");
  return $cid;
}

##############

# CKIM: this is now an alias for GizmoContainer->move_child_up_down($child, $delta)
sub move_up_down{
    my $self = shift;
    my $delta = shift;

    my $parent = $self->get_parent();
    return $parent->move_child_up_down($self, $delta);
}

##############

sub down{
    my $self = shift;
    return $self->move_up_down(1);
}

##############

sub up {
  my $self = shift;
  return $self->move_up_down(-1);
}

##############

sub print_attributes{
  my $self = shift;
  foreach my $k (sort keys %$self){
    printf "$k: %s\n", $self->{$k};
  }
}

##############

sub add {
    my $self = shift;

    $DISPLAY->debug_msg(4, "adding $self; parent_id=".$self->parent_id());

    if (!defined($self->instance_order()) && $self->new_children_appear_first() ) {
      $self->first_position();
    } elsif( !defined($self->instance_order()) ) {
      $self->last_position();
    }

    return $self->SUPER::add();
}


sub _cut {
  my $self = shift;
  my $id = shift;

  if(!defined($id)){
    $id = $self->{id};
  }

  $self->debug_msg(3, "Gizmo::_cut(id=$id)");
  if(!$id){
    $self->debug_msg(3, "Error Gizmo::cut(id=$id): I don't know what id to cut");
    return 0;
  }

  # clipboard is a Category object
  my $clipboard_iid = $USER->get_clipboard_iid();

  $self->{instance_order} = 100000;
  $self->save();
  $self->reorder_parent();

  $self->parent_id($clipboard_iid);
  $self->save();
  $self->debug_msg(4, "Gizmo is cut (id=$id)");
  # reorders the clipboard
  $self->reorder_parent();
  return 1;
}


#####################################################
###            BASIC BACK-END METHODS             ###
#####################################################


##############

# AUTOLOAD no longer necessary because MetadotObject creates the accessors/mutators

##############
sub list_ascendants {
  my $self = shift;
  my $id = shift ;
  $id = undef unless Utility->is_valid_integer($id); # secu_200401
  $id = $self->id() if(!defined $id);
  my $nbcalls = shift || 0;

  ErrorHandling->printError("Looping error. (Gizmo::list_ascendants)!!") if ($nbcalls > 500);
  # to stop at the right place

  my $root_category = MetadotConfig->get_reserved_iid('ROOT_CATEGORY');
  my $parent_of_root = MetadotConfig->get_reserved_iid('PARENT_OF_ROOT');
  my $portal_parent = MetadotConfig->get_reserved_iid('PORTAL_PARENT');

  return ({ id => $root_category }) if ($id == $parent_of_root || $id == $root_category);

  my ($isa,$parentid) = DBAccess->sqlSelect("isa, parentiid", "instance", "iid=$id");
  $self->debug_msg(3, "listing ascendants: $self, isa=$isa, id=$id, parentid=$parentid");

  # to make user's portal a root node
  return ({ id => $id }) if ($parentid == $portal_parent);
  # in case of broken branch (parent doesn't exist)
  # aju 02-21-02: just return null list rather than barfing. allows orphaned objects
  # to be viewed.
  if (!defined $parentid) {
#        ErrorHandling->printError("Broken branch : parent $id doesn't exist!") if (!defined $parentid);
      #print STDERR "in Gizmo::list_ascendants: Broken branch : parent $id doesn't exist\n";
      return ();
  }
  my @asc = $self->list_ascendants($parentid,$nbcalls+1);
  @asc = (@asc,{ id => $id }) if (($isa eq "Category") || $self->isa('Metadot::Navigable'));
  return @asc;
}

#####################################################
###         WEB DISPLAY BACK-END METHODS          ###
#####################################################

sub show {
  my $self  = shift;
  $self->debug_msg(4, "Gizmo::show()");

  my $name = $self->name;
  my $description = $self->description;
  my $longdescription = $self->long_description;
  my $url = $DISPLAY->make_url($self->url);

  my $s="";
  $s.= "<b>$name</b><br /><br />\n";
  $s.= "$description<br /><br />\n";
  $s.= "$longdescription<br /><br />\n";
  $s.= "<a href='$url'>$url</a><br /><br />\n";

  return $s;
}

sub show_edit_form {
  my $self  = shift;
  my $title = shift;
  $self->debug_msg(4, "Gizmo::show_edit_form($title)");

  die "method show_edit_form should always be overidden in default classes: $!";

  my $name = $self->name;
  escapeHTML($name);
  my $url = $self->url ;
  my $description = $self->description;
  escapeHTML($description);
  my $longdescription = $self->long_description;
  escapeHTML($longdescription);
  my $keywords = $self->keywords;
  escapeHTML($keywords);

  my $s = "<h1>$title</h1>

      <p /><b>Name</b> (*)<br />
      <input type='text' name='name' size='55' maxlength='255' value='$name'>

      <p /><b>URL</b><br />
      <input type='text' name='url' size='55' maxlength='255' value='$url'>

      <p /><b>Description</b><br />
      <textarea name='description' rows=4 cols=46 WRAP='VIRTUAL'>$description</textarea>
      <br />

      <p /><b>Long Description</b><br />
      <textarea name='longdescription' rows=10 cols=46 WRAP='VIRTUAL'>$longdescription</textarea>
      <br />

      <p /><b>Keywords and Synonyms</b><br />
      <textarea name='keywords' rows=2 cols=46 WRAP='VIRTUAL'>$keywords</textarea>
      <br />

      <br /><br /><br /><i>(*) Mandatory field(s)</i><br />";
  return $s;
}

##############

sub show_title {
  my $self = shift;

  my $title = shift;
  $self->debug_msg(3, "Gizmo::show_title()");
  my $s = "" ;

  if (defined $title) {
#    $s = $self->SUPER::show_title($title)
  } else {
    $title = $self->name;
    my $isa = $self->is_a;
    my $panel = $self->show_panel if ($isa eq 'Category' || $isa eq 'Table');
#    $s = $self->SUPER::show_title($title,$panel);
  }

  return $s;
}

##############

sub show_path{
  my $self = shift;
  my $id = $self->id;

  $self->debug_msg(3, "Gizmo::show_path($id)");
  my $userpath = "";
  $userpath = $self->build_path($self->parent_id) if ($id);

  $self->debug_msg(5, "Gizmo::show_path: $userpath");
  return $userpath;
}

##############




sub show_panel{
  my $self = shift;
  my $id   = shift || $self->id();
  my $isa  = shift || $self->is_a();
  $self->debug_msg(5, "Gizmo::show_panel(id=$id,isa=$isa)");

  return '' unless($SESSION->admin_mode());

  my $str = "";
  $str .= $self->show_edit_button();
  $str .= $self->show_properties_button() if $self->is_allowed_to_do('modify_properties', $USER);
  $str .= $self->show_ac_button();

  #CKIM-SHORTCUTS: added the shortcut button
  $str .= $self->show_shortcut_button();
  $str .= $self->show_cut_button();
  $str .= $self->show_del_button();
  return $str;
}

##############

sub show_up_down_buttons{
  my $self   = shift;
  my $id  = shift || $self->id();
  my $isa = shift || $self->is_a();
  my $str;
  $self->debug_msg(5, "Gizmo::show_up_down_buttons(id=$id)");
  return '' unless($SESSION->admin_mode());

  # We only check for one of them since they'll always be bundled together
  return '' unless($self->is_allowed_to_do('up', $USER));
  # check also that we have edit permissions for the parent object -- if not,
  # don't want to show the arrows.
  my $parent = $self->get_parent();
  if ($parent && ($parent->is_allowed_to_do('modify', $USER) || $parent->is_allowed_to_do('add_child', $USER) )) {#
        #print STDERR "in show_up_down_buttons: okay for $id -- user has parent modify perms\n";
  }
  else {
      #print STDERR "parent of $id doesn't allow arrows\n";
      return '';
  }

  my $imageslocation = $PARAMS->{imageslocation};

  $str  = "<a href=\"".$DISPLAY->make_url("index.pl?id=$id&amp;isa=$isa&amp;op=up")."\" class='icon'><img class=\"icon\" src=\"$imageslocation/obj/uparrow.gif\" width=\"11\" height=\"11\" border=\"0\" alt=\"Up\"></a>\n";
  $str .= "<a href=\"".$DISPLAY->make_url("index.pl?id=$id&amp;isa=$isa&amp;op=down")."\" class='icon'><img class=\"icon\" src=\"$imageslocation/obj/downarrow.gif\" width=\"11\" height=\"11\" border=\"0\" alt=\"Down\"></a>\n";

  return $str;
}


##############

sub show_switch_area_panel {
  my $self   = shift;
  my %args   = @_;
  $self->debug_msg(5, "Gizmo::show_switch_area_pannel");

  return '' unless(
      $SESSION->admin_mode() &&
      $PARAMS->{use_controller} &&
      $self->is_allowed_to_do('modify', $USER)
      );

  my $layout_area = $self->get_layout_area();

  my $iid = $self->id();
  if ( $layout_area && $layout_area->is_allowed_to_do('change_area_of_child', $USER) ) {#
      return $layout_area->create_change_area_selection(child => $self);
  } else {
      return '';
  }
}

sub get_layout_area {
  my $self = shift;

  #print STDERR "get_layout_area starts with $self (".$self->id().")\n";
  my $parent = $self->get_parent();

  # find the nearest LayoutArea parent, looking past GizmoContainer objects, but stop if a Navigable object is in the path
  while ($parent && !$parent->isa('LayoutArea') && $parent->isa('Metadot::GizmoContainer') && !$parent->isa('Metadot::Navigable')) {
      $parent = $parent->get_parent();
      #print STDERR "get_layout_area finds parent $parent (".$parent->id().")\n";
  }

  return $parent && $parent->isa('LayoutArea') ? $parent : undef;
}

##############

sub show_edit_button {
  my $self = shift;
  my $id   = shift || $self->id();
  my $isa  = shift || $self->is_a();
  $self->debug_msg(5, "Gizmo::show_edit_button(id=$id,isa=$isa)");

  return '' unless($SESSION->admin_mode());

  return '' unless($self->is_allowed_to_do('modify', $USER));

  my $imageslocation = $PARAMS->{imageslocation};

  my $str = "<a href='".$DISPLAY->make_url("index.pl?id=$id&amp;isa=$isa&amp;op=modify")."' class='icon'><img class=\"icon\" src='$imageslocation/obj/edit.gif' width=\"25\" height=\"11\" border=\"0\" alt='Edit Me!'></a>\n";

  return $str;
}


##############

sub show_del_button {
  my $self = shift;
  my $id   = shift || $self->id();
  my $isa  = shift || $self->is_a();
  $self->debug_msg(5, "Gizmo::show_del_button(id=$id,isa=$isa)");

  return '' unless($SESSION->admin_mode());
  return '' unless($self->is_allowed_to_do('delete', $USER));

  my $imageslocation = $PARAMS->{imageslocation};

  my $str = "<a href=\"".$DISPLAY->make_url("index.pl?id=$id&amp;isa=$isa&amp;op=delete")."\" class='icon'><img class=\"icon\" src=\"$imageslocation/obj/del.gif\" width=\"11\" height=\"11\" border=\"0\" alt=\"Delete Me!\"></a>\n";
  return $str;
}

##############

sub show_cut_button {
    my $self = shift;
    my $id   = shift || $self->id();
    my $isa  = shift || $self->is_a();
    $self->debug_msg(5, "Gizmo::show_cut_button(id=$id,isa=$isa)");

    return '' unless($SESSION->admin_mode());
    return '' unless($self->is_allowed_to_do('cut', $USER));

    my $imageslocation = $PARAMS->{imageslocation};

    my $str = "<a href='".$DISPLAY->make_url("index.pl?id=$id&amp;isa=$isa&amp;op=cut")."' class='icon'><img class=\"icon\" src='$imageslocation/obj/cut.gif' width=\"22\" height=\"11\" border=\"0\" alt='Cut Me!'></a>\n";
    return $str;

}

##############
sub show_properties_button {
  my $self = shift;
  my $id   = shift || $self->id();
  my $isa  = shift || $self->is_a();
  $self->debug_msg(5, "Gizmo::show_edit_button(id=$id,isa=$isa)");

  return '' unless($SESSION->admin_mode());

  return '' unless($self->is_allowed_to_do('modify', $USER));

  my $imageslocation = $PARAMS->{imageslocation};

  my $str = "<a href='".$DISPLAY->make_url("index.pl?id=$id&amp;isa=$isa&amp;op=modify_properties")."' class='icon'><img src='$imageslocation/obj/property.gif' width=11 height=11 border=0 alt='Edit My Properties!'></a>\n";

  return $str;
}

##############

sub show_shortcut_button {
    #CKIM-SHORTCUTS: displays the shortcut button
    my $self = shift;
    my $id   = shift || $self->id();
    my $isa  = shift || $self->is_a();
    $self->debug_msg(5, "Gizmo::show_shortcut_button(id=$id,isa=$isa)");

    return '' unless($SESSION->admin_mode() && $USER->is_logged_in());
    return '' unless($self->is_allowed_to_do('create_shortcut', $USER));

    my $imageslocation = $PARAMS->{imageslocation};

    my $str = "<a href='".$DISPLAY->make_url("index.pl?id=$id&amp;isa=$isa&amp;op=create_shortcut")."' class='icon'><img class=\"icon\" src='$imageslocation/obj/shortcut.gif' width=\"11\" height=\"11\" border=\"0\" alt='Make a Shortcut!'></a>\n";

    return $str;

}

##############

sub show_ac_button {
  my $self = shift;
  my $id   = shift || $self->id();
  my $isa  = shift || $self->is_a();

  $self->debug_msg(5, "Gizmo::show_ac_button(id=$id,isa=$isa)");

  return '' unless($SESSION->admin_mode());

  return '' unless($self->is_allowed_to_do('edit_permissions', $USER));


  my $imageslocation = $PARAMS->{imageslocation};

  my $str = "<a href='".$DISPLAY->make_url("index.pl?id=$id&amp;isa=$isa&amp;op=edit_permissions")."' class='icon'><img class=\"icon\" src='$imageslocation/obj/ac.gif' width=\"24\" height=\"11\" border=\"0\" alt='Change Access Rights!'></a>\n";

  return $str;
}






# will delete attachment in this gizmo if there is one
sub delete_attachment {
  my $self = shift;
  my $iid = $self->id();
  my $file = shift || "file1";
  my $size = shift || 0;
  my $filename = $self->$file() || $self->file1();
  my $uid = $self->uid();

  $self->debug_msg(4, "Gizmo::delete_attachment(iid=$iid, filename=$filename)");  # delete from file system

  my $absolutepathfileuploaddir = $PARAMS->{'absolutepathfileuploaddir'};

  $self->debug_msg(5, "Gizmo::delete_attachment: deleting dir: $absolutepathfileuploaddir/$iid/");
#  rmtree (["$absolutepathfileuploaddir/$iid"]);
  unlink("$absolutepathfileuploaddir/$iid/$filename");
  # delete link from DB
  $self->{file1} = "";
  $self->{file1_size} = $size;
  $self->get_object_accessor()->update(object=>$self);

  #DBAccess->sqlUpdate("instance", "iid=$iid", ($file=>"", file1_size=>"$size"));

  return 1;
}


#################################################################
# upload_file
# script heavily inspired from Muhammad A Muquit upload example.
# $filepath: the filename with the path where the file is located
# $fileiid: the oject unique id that that file refers to.
#################################################################
sub upload_file{
    my $self = shift;
    my $filepath=shift;
    my $fileiid=shift;
    my $uid = shift;
    my $upload_location = shift || '';

    my $iid = $fileiid;
    my $file;
    my $g_overwrite = 0;
    my $bytes_read = 0;
    my $size = 0;
    my $buff = '';
    my $start_time;
    my $time_took;
    my $filename = '';
    my $write_file = '';

    $iid = undef unless Utility->is_valid_integer($iid); # secu_200401
    $self->debug_msg(3, "Portal::upload_file::filepath=$filepath; iid=$fileiid");

    # James Bee" <JamesBee@home.com> reported that from Windows filename
    # such as c:\foo\fille.x saves as c:\foo\file.x, so we've to get the
    # filename out of it
    # look at the last word, hold 1 or more chars before the end of the line
    # that doesn't include / or \, so it will take care of unix path as well
    # if it happens, muquit, Jul-22-1999
    if ($filepath =~ /([^\/\\]+)$/) {
    $filename="$1";
    } else {
    $filename="$filepath";
    }

    # if there's any space in the filename, get rid of them
    #  $filename =~ s/\s+//g;

    if (!$PARAMS->{absolutepathfileuploaddir}){
    my $msg="Sorry the parameter 'upload_directory' is not set therefore the upload is just impossible. Please contact the system administrator";
    $self->click_back($msg);

    exit(0);
    }

    $write_file=$PARAMS->{absolutepathfileuploaddir}. "/" ."$fileiid/" . "$filename";
    #    $write_file=$PARAMS->{absolutepathfileuploaddir}. "/" . "$uid/"."$fileiid/" . "$filename";
    my $DebugFlag=0;
    print "Filename=$filename<br />\n" if $DebugFlag;
    print "Writefile= $write_file<br />\n" if $DebugFlag;

    ### create the upload global directory
    if (! -d $PARAMS->{absolutepathfileuploaddir}) {
        if (!mkdir($PARAMS->{absolutepathfileuploaddir}, 0755)) {
            ErrorHandling->printError("cannot create upload file directory: '"
                          . $PARAMS->{absolutepathfileuploaddir}. "': $?");
            exit(0);
        }
    }

    ### add the file upload directory for a user first.
    $fileiid = $PARAMS->{absolutepathfileuploaddir} ;

    $fileiid .= "/" . "$iid";

    ### create the upload directory with name=iid
    if (! -d "$fileiid" ) {
        unless (mkdir($fileiid, 0755)) {
            $DISPLAY->print_click_back("Upload Error", "Cannot create directory: $fileiid: $?.  If this problem occurs again, please contact the site administrator.");
            exit(0);
        }
    }

    ### moving the file from tmp dir to final location
    if (!open(WFD,">$write_file")) {
    $DISPLAY->print_error("Error opening file for writing: $write_file.");
    return;
    }
    $start_time=time();

#    print STDERR "writing the file $filepath to the file $write_file in the directory $fileiid\n";

    while ($bytes_read=read($filepath,$buff,2096)){
        # while (read($filepath,$buff,2096)){
        $size += $bytes_read;
        binmode WFD;
        print WFD $buff;
    }
    if ($size == 0) {
        $DISPLAY->print_click_back("Error", " Cannot access specified upload file $filepath, or file is empty. Please recheck the file name.");
        exit(0);
    }
    print "size= $size<br />\n" if ($DebugFlag);
    close(WFD);

    my ($upload_size_allowed, $upload_number_allowed) = $USER->get_uploads_space_and_num_files_quota( $uid );

    if($upload_size_allowed == -1){
        $upload_size_allowed = $PARAMS->{max_total_upload_size};
    }
    # aju: convert upload_size from M to bytes (sheesh)
    my $upload_quota_bytes = $upload_size_allowed * 1048576;

    if($upload_number_allowed == -1){
        $upload_number_allowed = $PARAMS->{max_total_number_upload};
    }

    my ($used_space, $uploaded_number) =
      $USER->get_used_uploads_space_and_num_files( $uid );

    my $available_space = $upload_quota_bytes - $used_space;
    #      print STDERR "used space: $used_space; total avail space: $upload_quota_bytes; current avail space: $available_space\n";
    #      print STDERR "file size is: $size\n";

    if (($size > $available_space) || ($uploaded_number >=$upload_number_allowed)){
        unlink($write_file);
        rmdir($fileiid);
        return 'USER_QUOTA_EXCEEDED';
    }

    # check for global quota overrun
    ($used_space, $uploaded_number) = Metadot::UploadsManager->get_global_used_uploads_space_and_num_files();

    $upload_size_allowed = $PARAMS->{global_uploads_quota};

    # multiply by 1M to convert to bytes
    $upload_size_allowed *= 1048576;

    $available_space = $upload_size_allowed - $used_space;

    if ( $size > $available_space ) {
        unlink($write_file);
        rmdir($fileiid);
        return 'GLOBAL_QUOTA_EXCEEDED';
    }

    if ((stat $write_file)[7] <= 0){
        unlink($write_file);
        rmdir($fileiid);
        $DISPLAY->print_error("Could not upload file: $fileiid/$filename. <br />\nFile is empty.");
        return "";
    } else {
      $time_took=time()-$start_time;
      if ($DebugFlag){
        print<<EOF;
        <center>
          <hr noshade size=1 width=\"90%\">
            <table border="0" bgcolor=\"#c0c0c0\" cellpadding="0" cellspacing="0">
              <tr>
                <td>
                  <table border="0" width=\"100%\" cellpadding="10" cellspacing="2">
                    <tr align=\"center\">
                      <td bgcolor=\"#000099\" width=\"100%\">
                        <font color=\"#ffffff\">
                          File
                            <font color=\"#00ffff\"><b>$filename</b></font> of size
                              <font color=\#00ffff\"><b>$size</b></font> bytes is
                                uploaded successfully! (time=$time_took)
                              </font>
                    </td>
                   </tr>
                </table>
              </td>
            </tr>
        </table>
      </center>
EOF
                                                    ;
    }                                #end of if ($DebugFlag)

      $self->{file1_size} = $size;
      DBAccess->sqlUpdate("instance", "iid=$iid",
              file1_size => $size);

      return $filename;
  }
}


sub total_space_used {
    my $self = shift;

    my $from  = 'instance';
    my $where = 'file1_size>=0';

    my ($c) = DBAccess->sqlSelectMany("file1_size", $from, $where);

    my $size;
    while ( my $s = $c->fetchrow() ){
    $size += $s;
    }
    $c->finish();

    #my $number = $c->rows;
    # rows function is not reliable for counting the rows retrieved by SELECT
    my ($number) = DBAccess->sqlSelect("count(*)",$from, $where);

    return ($size, $number);
}


sub space_used{
    my $self = shift;
    my $uid = shift;

    my ($size, $number) = $USER->get_used_uploads_space_and_num_files( $uid );

    return ($size, $number);
}


# CKIM: hopefully this code is not being used, because it's awful
#
# node_count:
# @sql: self join of instance table to build a simple
#   parent-child relationship, grouping on parent
# sub node_count {
#     my $self = shift;
#     my $node = $self->id;
#
#     defined $node || croak "self hasn't got an id";
#
#     # ref to hash mapping iid's to count of children
#     # count includes both non-terminal and terminal children
#     my $count;
#
#     # ref to hash mapping iid's to array of children iid's
#     # children includes only non-terminal children
#     my $children;
#     my $items = DBAccess->sqlSelectMany("count(*), i.iid, i.parentiid", "instance i, instance j", "i.iid = j.parentiid", "group by i.iid");
#     while (my ($cnt, $iid, $parentiid) = $items->fetchrow()) {
#
#         $count->{$iid} = $cnt;
#         push @{$children->{$parentiid}}, $iid;
#     }
#
#     # 1 (node) + count of children
#     return 1 + $self->_child_count($node, $count, $children);
# }
#
# sub _child_count {
#     my $self = shift;
#     my ($node, $count, $children) = @_;
#     my $cnt = $count->{$node};
#     foreach my $child (@{$children->{$node}}) {
#         $cnt += $self->_child_count($child, $count, $children);
#     }
#     return $cnt;
# }



#
# node_count_by_gizmo:
# @sql: self join of instance table to retrieve
#   a list of parentiid to form selection critirea
sub node_count_by_gizmo {
    my $self = shift || croak "no self";
    my $gizmo_count = shift || {}; # keeps a counter for each isa we encounter
    my $visited     = shift || {}; # keeps track of the instances visited, to prevent loops

    my $node = $self->id;
    defined $node || croak "self hasn't got an id";

    $self->debug_msg(5, "In $self\->node_count_by_gizmo($gizmo_count, $visited)");

    # replacement to the previous code
    # the next block replaces some bad code
    $visited->{$node} = 1;

    my $isa = $self->{is_a};
    if (!defined $gizmo_count->{$isa}) {
        $gizmo_count->{$isa} = 1;
    } else {
        $gizmo_count->{$isa}++;
    }

    my $child_sth = DBAccess->sqlSelectMany("iid, isa", "instance", "parentiid=$node");
    while (my ($child_iid, $child_isa) = $child_sth->fetchrow()) {
        unless(defined $visited->{$child_iid}) {
            eval {
                my $child = $child_isa->restore($child_iid);
                $child->node_count_by_gizmo($gizmo_count, $visited);
            };
        }
    }
    $child_sth->finish();

    return $gizmo_count;


    # CKIM: the following is just bad code. It can result in a gigantic query expression
    #       that exceeds the limitations of some databases. Thanks Claudio.

    # ref to hash mapping iid's to array of children iid's
    # this array includes only non-terminal children

#     my $children;                                                                                                                 #
#     my $items = DBAccess->sqlSelectMany("i.iid, i.parentiid", "instance i, instance j", "i.iid = j.parentiid", "order by i.iid"); #
#                                                                                                                                   #
#     while (my ($iid, $parentiid) = $items->fetchrow()) {                                                                          #
#         push @{$children->{$parentiid}}, $iid;                                                                                    #
#     }                                                                                                                             #
#                                                                                                                                   #
#     # retrieve tree rooted in node                                                                                                #
#     my @parents = $self->_sub_parents($node, $children);                                                                          #
#                                                                                                                                   #
#     my $gizmo_count = {};                                                                                                         #
#                                                                                                                                   #
#     my $gizmos = DBAccess->sqlSelectMany("isa, count(*)",                                                                         #
#                          "instance", "parentiid in (".join(',', @parents).")",                                                    #
#                          "group by isa");                                                                                         #
#
#   while (my ($isa, $cnt) = $gizmos->fetchrow()) {
#
#       $gizmo_count->{$isa} = $cnt;
#   }
#
#   # include self since my parent is not in list
#   $gizmo_count->{$self->is_a}++;
#   return $gizmo_count;
}

# CKIM: this method is no longer needed. it was poor claudio code to begin with
######################################################################
# #                                                                  #
# # sub_parents returns all children of $node which are also parents #
# #                                                                  #
# sub _sub_parents {                                                 #
#     my $self = shift || croak "no self";                           #
#     my ($node, $children) = @_;                                    #
#     my @parents = ();                                              #
#     push @parents, $node;                                          #
#     foreach my $child (@{$children->{$node}}) {                    #
#         push @parents, $self->_sub_parents($child, $children);     #
#     }                                                              #
#     return @parents;                                               #
# }                                                                  #
######################################################################

sub my_child_count {
    my $self = shift || croak "no self";

    my $iid = $self->id;

    defined $iid || croak "self hasn't got an id";

    my ($cnt) = DBAccess->sqlSelectMany("count(*)", "instance", "parentiid=$iid");

    return $cnt;
}


sub apply {
    my $self = shift || croak "no self";
    my $recursive = shift;
    my $function = shift || croak "must provide function";
    my $visited  = shift || {}; # used to avoid loops

    my $node = $self->id;
    defined $node || croak "self hasn't got an id";

    $self->debug_msg(5, "$self\->apply($recursive, $function, $visited) with id=$node, isa=".$self->{is_a});
    # ref to hash mapping iid's to array of children iid's
    # children includes only non-terminal children

    $function->($self);

    # the next block replaces some bad code
    if ($recursive) {
        $visited->{$node} = 1;
        my $child_sth = DBAccess->sqlSelectMany("iid", "instance", "parentiid=$node");
        while (my ($child_iid) = $child_sth->fetchrow()) {
            unless(defined $visited->{$child_iid}) {
                eval {
                    my $child = Gizmo->restore($child_iid);
                    $child->apply($recursive, $function, $visited);
                };
            }
        }
        $child_sth->finish();
    }


    # CKIM: this is more of that awful pseudo-recursive code.
#     if ($recursive) {                                                                                                             #
#     my $children;                                                                                                                 #
#     my $items = DBAccess->sqlSelectMany("i.iid, i.parentiid", "instance i, instance j", "i.iid = j.parentiid", "order by i.iid"); #
#                                                                                                                                   #
#                                                                                                                                   #
#     while (my ($iid, $parentiid) = $items->fetchrow()) {                                                                          #
#         push @{$children->{$parentiid}}, $iid;                                                                                    #
#     }                                                                                                                             #
#     $self->_apply_children($node, $children, $function);                                                                          #
#     }                                                                                                                             #

}

# sub _apply_children {
#     my $self = shift || croak "no self";
#     my ($node, $children, $function) = @_;
#
#     my $items = DBAccess->sqlSelectMany("iid, parentiid", "instance", "parentiid = $node");
#
#     while (my ($iid, $parentiid) = $items->fetchrow()) {
#
#         my $gizmo = Gizmo->restore($iid);
#         $function->($gizmo);
#     }
#
#     foreach my $child (@{$children->{$node}}) {
#
#         $self->_apply_children($child, $children, $function);
#     }
# }


sub path_qualified_name {
    my $self = shift;

    my $iid = $self->id();

    my $name = "";
    my $id = "";
    my $isa = '';
    my $content = "";

    my @parent_list;

    #CKIM: always use get_visible_parents_list
    @parent_list = $self->get_visible_parents_list();

    my $entry = {};
    for (my $x=0; $parent_list[$x]; $x++){

        $entry = $parent_list[$x];
        $id = $entry->{id};
        $name = $entry->{name};
        $isa = $entry->{isa};
    #($id, $name, $isa) = split(/:/,$parent_list[$x]);

    my @nameparts = split(/ /, $name);

    if (($#nameparts + 1) > 2){
        $name = $nameparts[0]." ".$nameparts[1]."...";
    }

    $content .= "/ $name ";

    }

    return $content;
}


sub user_has_scheduled_content {
    my $self = shift;
    my $user = shift || $USER;

    return ( ($self->{show_from} || $self->{show_end}) && $self->is_allowed_to_do('modify', $user) ) ? 1 : 0;
}

sub new_children_appear_first {
    # return a true value if you want new children of this instance
    # to precede its elders by instanceorder
    return 0;
}


# returns the # of seconds since last update
sub time_since_last_update {
    my $self = shift;
    return time - Utility->unix_time_format($self->{update_date});
}


# for backward compatibility with the non-hash constructor
sub normalize_constructor_args {
    my $class = shift;
    my $args  = shift;
    $args = {} unless defined $args;

    if(defined $args && !ref($args)) {
        #old calling method $id is only param
        $args = { id => $args };
    }

    #$class->debug_msg(5, caller()."-->".__PACKAGE__."/$class\->normalize_constructor_args has ".Dumper($args));
    return $args;
}





#####################################################
###                   THE END                     ###
#####################################################

1;


__END__

=head1 NAME

Gizmo -

=head1 SYNOPSIS

    use Gizmo;
    $gizmo= Gizmo->new ($args);

    $gizmo->get_version;
    $gizmo->lists_gizmos ;
    $gizmo->filtered_gizmos ;
    $gizmo->get_is_public ;
    $gizmo->get_gizmo_name ;
    $gizmo->get_AC;
    $gizmo->get_parent ;
    $gizmo->get_owner_uid ;
    $gizmo->not_logged_in_callback ;
    $gizmo->restore ($id);
    $gizmo->www_show ;
    $gizmo->www_change_owner ;
    $gizmo->www_edit_permissions ;
    &go_back_to_parent_after_editing_permissions ;
    $gizmo->process_edit_form;  [interface]
    $gizmo->www_modify ([$errormsg]);
    $gizmo->www_save ;
    $gizmo->www_delete ;
    $gizmo->www_delete_ok ;
    $gizmo->www_up ;
    $gizmo->www_down ;
    $gizmo->www_cut ;
    $gizmo->www_paste ;
    $gizmo->www_download ;
    &show_search_results ($gizmo_name, $thingtosearch,
              [$startfrom=0, $maxitem=50]);
    $gizmo->last_position ;
    $gizmo->delete ;
    $gizmo->reorder_parent;
    $gizmo->save ;
    $gizmo->cid ;
    $gizmo->down;
    $gizmo->up ;
    $gizmo->print_attributes;
    $gizmo->add ;
    $gizmo->DESTROY;
    $gizmo->url ([$url]);
    $gizmo->list_ascendants ([$id, $nbcalls=0]);
    $gizmo->show ;
    $gizmo->show_edit_form ($title);
    $gizmo->show_title ([$title]);
    $gizmo->show_path;
    $gizmo->show_panel ([$id, $isa]);
    $gizmo->show_up_down_buttons ([$id, $isa]);
    $gizmo->show_edit_button ([$id, $isa]);
    $gizmo->show_del_button ([$id, $isa]);
    $gizmo->show_cut_button ([$id, $isa]);
    $gizmo->show_ac_button ([$id, $isa]);
    $gizmo->parentCheck ([$id]);
    $boolean = $gizmo->delete_attachment();


=head1 DESCRIPTION

Gizmo is an abstract class from which concrete gizmos (eg. Item and Category) are derived. This class provides
a default implementation for many methods that concrete gizmos will seldom need to override. Methods in this
class are essentially infrastructure that makes life easier for Gizmo developers.

=head2 Methods

$self = ConcreteGizmo->SUPER::new([ HASHREF | SCALAR ]);

Object constructor for this class. The main function of this method is to load concrete gizmo's instance data from database storage. Must always be called from concrete gizmos own constructors, so infrastructure initializations (such data loading and Permissions initialization) can take place on instance creation.
Two forms of paramters can be passed to it: a) Named parameters can in the form of a hashref to override default values on object creation (eg. if we want to specify a parent id when creating gizmos programatically). b) A single numeric scalar to signify the instance id of an already existing gizmo.This form is mostly used when rendering gizmos from within the Category gizmo.


  @gizmo_list = Gizmo->lists_gizmos()

Classwide method that returns a list containing the names of gizmos installed in the system. It is used in Category to determine what gizmos it should be rendering. Should be of use to implement generic container gizmos.


  $is_public = $gizmo->get_is_public()

Determines whether the current gizmo instance is searchable by the built in search infrastructure. Currently it is a classwide method, so it allows to specify whether a whole set of concrete gizmos is searchable. Gizmos that globally are not meant to be searched should override this method to return 0.


  @gizmo_list = Gizmo->filtered_gizmos();

This method returns a list of gizmos that are meant to be searchable. It's used within Portal.pm to implement search infrastructure.

    Gizmo->get_version;

This method returns "No Version Information Available" and is only used when one of the subclasses of Gizmo has not defined this method


  Gizmo->get_gizmo_name;

This is a classwide method that returns the name of the gizmo.

  $gizmo->get_AC;

  $gizmo->get_parent;

  $gizmo->get_owner_uid;

  $gizmo->not_logged_in_callback;

These methods are required by the Auditable interface. See documentation in Auditable.pm.


  Gizmo->restore($id);

Classwide method. Will restore a gizmo object back from the database. The gizmo will be blessed to the class of the concrete Gizmo it represents, even if called from the Gizmo parent class.


  $gizmo->www_show ;

Will render the content of the concrete gizmo. Will be invoked by the system as users browswe the gizmo tree. Most concrete gizmos will need to override this method.


  $gizmo->www_change_owner ;

Will be invoked by the system when a user wants to change the owner of a gizmo. Concrete gizmos will not ususally need to override this method.


  $gizmo->www_edit_permissions ;

Will be invoked by the system when a user wants to change permissions of a gizmo. Concrete gizmos will not ususally need to override this method.


  $gizmo->go_back_to_parent_after_editing_permissions ;

Determines if the system should jump to the parent object after editing permissions of an object. The default implementation returns true, which is the behavior most gizmos need.


  $gizmo->www_modify ([$errormsg]);

This method is invoked by the system when a user wants to modify the content of a gizmo (usually by clicking in the "Edit" icon). This method will generally not be overridden in derived classes, instead the show_edit_form method should be. If an error message is supplied this will appear before the html from show_edit_form.


  $gizmo->show_edit_form ($title);

Must be overridden in concrete classes (ie. the ones derived from Gizmo) to generate the HTML specific to the data fields required in the concrete class. Will be invoked by the system from the www_modify method and from Category::www_add_child.


  $gizmo->process_edit_form;  [interface]

This method will read gizmo specific data from the posted form when an gizmo is being created.It is invoked from www_create_child in Category.pm. Most gizmos will need to override this to process their specific data.

  $gizmo->www_save;

This method will be invoked by the system after www_modify to commit field changes to the database. Some gizmos will not use this method and instead save directly to the database (eg. Poll.pm.). Some gizmos override this method (eg. Table).

  $gizmo->save;

This method will commit new gizmo data to the instance table. It's called from www_save and from other save mathods in concrete gizmos.

  $gizmo->www_delete ;

Will be called by the system when users request a gizmo to be deleted.

  $gizmo->www_delete_ok ;

Will be called by the system when users confirm that they want a gizmo to be deleted.

  $gizmo->www_up ;
  $gizmo->www_down ;

Will be called by the system when users request that a gizmo is moved up or down in the rendering order.

  $gizmo->www_cut ;
  $gizmo->www_paste ;

Will be called by the system when users request to cut or paste a gizmo.

  $gizmo->www_download ;

Will be called by the system when users request to download a file contained in a gizmo.(eg. from an Item gizmo).

  &show_search_results ($gizmo_name, $thingtosearch,
               [$startfrom=0, $maxitem=50]);

Will perform a search for matching words in gizmos of the specified type.

  $gizmo->last_position ;

Utility method that returns the next position number for a gizmo class. The position number is used to determine the order in which gizmos are rendered.

  $gizmo->delete ;

Will delete a gizmo instance from the instance table.

  $gizmo->reorder_parent;

Will update the instance order numbers of a gizmo type in the table. It is called when gizmos are added or removed or cut and when their order is changed.

  $gizmo->save;

Will commit a gizmo to the database.

  $gizmo->cid;

Returns the channel id associated with a gizmo.

  $gizmo->down;

  $gizmo->up ;

Will change the position in which a gizmo is rendered with respect to gizmos of the same kind under the same parent.

  $gizmo->print_attributes;

Will print the values in the gizmo data fields. Not used anywhere. Could be used for debugging.

  $gizmo->add;

Will insert a newly created gizmo into the database.

  $gizmo->DESTROY;

Not used but to comply with a mod_perl requirement.

  $gizmo->url ([$url]);

Accessor method to the url field. This method is not taken care of by AUTOLOAD because the CGI.pm module exports a method with the same name.

  $gizmo->list_ascendants ([$id, $nbcalls=0]);

Will return a list of the ids of all Categories above a gizmo.  This is use to generate links to parent categories when rendering a gizmo.

  $gizmo->show ;

Will generate HTML to render the content of the gizmo. Must be overidden in concrete classes.

  $gizmo->show_title ([$title]);

Will render a title when rendering the gizmo. For the case of Categories and Tables it will also call show_panel(). It will be called by the rendering infrastructure and will seldom need to be invoked by gizmo writers.

  $gizmo->show_path;

Will return the HTML for the clickable path that allows to go to go to back to the parent categories under which a gizmo is found. It's used by rendering infrastructure.

  $gizmo->show_panel ([$id, $isa]);

Will return the html for the edit, delete, cut, shortcut up, down and edit permissions icons when the user is in edit mode. It's used by the rendering infrastructure.

  $gizmo->show_edit_button ([$id, $isa]);

  $gizmo->show_del_button ([$id, $isa]);

  $gizmo->show_cut_button ([$id, $isa]);

  $gizmo->show_shortcut_button ([$id, $isa]);

  $gizmo->show_up_down_buttons ([$id, $isa]);

  $gizmo->show_ac_button ([$id, $isa]);

Will generate html to display icons for edit, delete, cut, up, down, and edit permissions. These methods are used by show_panel() and would rarely need to be used or overridden in derived classes.

  $gizmo->parentCheck ([$id]);

Utility function used by the search infrastructure to determine if an item is hiden or in a clipboard, in which case it should nto be displayed as a search result.

Back end method for deleting a file, called by www_delfileok. Will attempt to delete a file attachment whose name is stored in the file1 field of the gizmo.

    $boolean = $gizmobuilder->delete_attachment();


=cut




      while( (my ($iid, $name, $description, $isa)=$c->fetchrow()) && ($i < $maxitem)) {
      if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC
      my $gizmo = $isa->new($iid);
      next unless $gizmo->is_allowed_to_do('show', $USER);
      next unless $gizmo->parentCheck(); # don't show hidden items
      $totalrows++;
      # shorten the description
      $description="" if (!$description || !defined($description));
      $name="null" if (!$name || !defined($name));
      $description=substr($description, 0, 150) ."...";
      ### remove tags that could break the page.
      $description = Verify->stripBadHTML($description);
      $oneresult = $gizmo_name->format_search_result($iid, $name, $description, $isa);
      ### put the results in list tag
      $oneresult="<li>" . "$oneresult" . "<br /><br /></li>\n\n";
      $s .= "$oneresult\n\n";;
      $i++;
      }
