#!/usr/bin/perl

#
# Test file for Gizmo.pm
# $Id: Gizmo.100.t,v 1.3 2005/02/02 14:24:11 daniel Exp $
#

# tests that Gizmos check their page permissions before revealing themselves

use strict;
use Test::More tests=>8;

use SmokeMetadot;
use Metadot::Testing::Mechanizer;
use DBAccess;
use MetadotConfig;
use Metadot::PortalFactory;

# set up info
my $mech = Metadot::Testing::Mechanizer->new();
my $home_page = Metadot::PortalFactory->get_home_object();
my $admin_uid = MetadotConfig->get_reserved_uid('ADMIN');

my $test_page_name = '_FOO_TEST_PAGE_64654';
my $page;

my $test_class_name    = 'Item';
my $test_instance_name = '_FOO_TEST_INSTANCE_64654';
my $obj;

################################################################################
# Test: Make sure the category does not yet exist in navigation
################################################################################
ok(!$mech->instance_retrievable(iid=>$home_page->id(), expr=>$test_page_name),
   'Category does not yet exist in navigation');


################################################################################
# Setup: Create the test object and category in the db
################################################################################
use_ok('Gizmo::Category');
use_ok('Gizmo::'.$test_class_name);

$page = Category->new({
    uid => $admin_uid,
    parent_id => $home_page->id(),
    name => $test_page_name,
});
$page->add;

$obj = $test_class_name->new({
    uid => $admin_uid,
    parent_id => $page->id(),
    name => $test_instance_name,
});
$obj->add;


################################################################################
# Test: Check that the instance is accessible
################################################################################
ok($mech->instance_retrievable(iid=>$obj->id(), expr=>$test_instance_name),
   'Instance was retrievable');


################################################################################
# Test: Restrict access to the test page
################################################################################
$page->set_bundle_level('DISP', 2); # two means login
$page->save();
ok(!$mech->instance_retrievable(iid=>$page->id(), expr=>$test_page_name),
   'Instance was retrievable');


################################################################################
# Test: Instance is not accessible
################################################################################
ok(!$mech->instance_retrievable(iid=>$obj->id(), expr=>$test_instance_name),
   'Instance was NOT retrievable');


################################################################################
# Test: Restore access to the test page
################################################################################
$page->set_bundle_level('DISP', 0); # zero means public
$page->save();
ok($mech->instance_retrievable(iid=>$page->id(), expr=>$test_page_name),
   'Instance was retrievable');


################################################################################
# Test: Check that the instance is accessible again
################################################################################
ok($mech->instance_retrievable(iid=>$obj->id(), expr=>$test_instance_name),
   'Instance was retrievable');


################################################################################
# Tear-down: Remove the object and category from the db
################################################################################
$obj->delete();
$page->delete();



#################################################
# That's it!!!!
#################################################



