###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package AbstractEvent;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT);

use Carp;
use Gizmo;
use Time::Local;
use Date::Calc qw(check_date);

use DBAccess;

# added import of escapeHTML
use CGI qw/escape unescape escapeHTML/;
use vars qw(@ISA);
@ISA=qw(GizmoBuilder);
my $version = "4.5";

sub get_version {
    return $version;
}

my $DATE_FIELD      = 'i1'; # stores the timestamp
my $TIMED_FIELD     = 'c1'; # stores whether the time-of-day is relevant
my $DURATION_FIELD  = 'i2'; # stores the duration of the event
my $is_visible_in_add_child_lists = 0;


sub is_visible_in_add_child_lists {
    return $is_visible_in_add_child_lists;
}

sub customize_permissions {
    my $self = shift;
    #if set, this text will be displayed at the top of the gizmo edit form.

    my $ac   = shift;
    $self->SUPER::customize_permissions($ac);

        #modify the ac here
        $ac->add_bundle('DISP', 'View', '0', '0');
        $ac->modify_bundle('DISP', 'DEFAULTS_TO', 'on' );
        $ac->add_op('DISP','show');

        $ac->add_bundle('MOD', 'Edit', '8', '2');
        $ac->add_op('MOD', 'modify');
        $ac->add_op('MOD', 'save');
        $ac->add_op('MOD', 'up');
        $ac->add_op('MOD', 'down');

        $ac->add_bundle('DEL', 'Delete and Cut', '8', '8');
        $ac->add_op('DEL', 'delete');
        $ac->add_op('DEL', 'delete_ok');
    }

my $MONTHS = {
                0 => 'January',
                1 => 'February',
                2 => 'March',
                3 => 'April',
                4 => 'May',
                5 => 'June',
                6 => 'July',
                7 => 'August',
                8 => 'September',
                9 => 'October',
               10 => 'November',
               11 => 'December'
             };

#####################################################
######                                        #######
######            FRONT-END METHODS           #######
######                                        #######
#####################################################

sub www_delete {
  my $self = shift;
  my $isa = $self->is_a();
  my $gizmo_name = $isa->get_gizmo_name();
  $self->debug_msg(3, "AbstractEvent::www_delete()");
  $self->print_http_header;
  $self->two_choices("Do you want to delete this $gizmo_name? <br />",
                     "caution",
             "Yes please", "index.pl?iid=".$self->id."&isa=$isa&op=delete_ok",
             "No!", "$SESSION->{referer}");
}

sub www_delete_ok {
  my $self = shift;
  my $url  = $self->_get_url_of_parent();
  $self->debug_msg(3, "AbstractEvent::www_delete_ok()");
  $self->delete();
  $self->md_redirect($url);
}

###############

# returns the url of the AbstractEvent's parent, including the necessary
# op, isa and form variables
sub _get_url_of_parent {
    my $self = shift;
    my $parent_obj = shift || $self->get_parent_obj();
    my $result;

    my $parent_iid = $parent_obj->id();
    my $parent_isa = $parent_obj->is_a();
    my $op = 'show';
    my $dt = $self->date();

    $result = "index.pl?iid=$parent_iid&amp;isa=$parent_isa&amp;op=$op&amp;dt=$dt";
    return $result;
}

sub get_parent_obj {
    my $self = shift;
    my $result;

    my $parent_iid = shift || $self->parent_id();
    $result = Gizmo->restore($parent_iid);
    return $result;
}

sub date_field {
    my $self = shift;
    my $input = shift;
    if (defined $input) {
        $DATE_FIELD = $input;
    }
    return $DATE_FIELD;
}

sub timed_field {
    my $self = shift;
    my $input = shift;
    if (defined $input) {
        $TIMED_FIELD = $input;
    }
    return $TIMED_FIELD;
}

sub duration_field {
    my $self = shift;
    my $input = shift;
    if (defined $input) {
        $DURATION_FIELD = $input;
    }
    return $DURATION_FIELD;
}

sub date {
    my $self = shift;
    my $input = shift;
    my $result;

    my $field_name = $self->date_field();
    if (defined $input) {
        $result = $self->$field_name($input);
    } else {
        $result = $self->$field_name();
    }
    return $result;
}

sub timed {
    my $self = shift;
    my $input = shift;
    my $result;

    my $field_name = $self->timed_field();
    if (defined $input) {
        $result = $self->$field_name($input);
    } else {
        $result = $self->$field_name();
    }
    return $result;
}

sub duration {
    my $self = shift;
    my $input = shift;
    my $result;

    my $field_name = $self->duration_field();
    if (defined $input) {
        $result = $self->$field_name($input);
    } else {
        $result = $self->$field_name();
    }
    return $result;
}

sub months {
    my $self = shift;
    my $input = shift;

    if (defined $input) {
        $MONTHS = $input;
    }
    return $MONTHS;
}

##############

sub process_edit_form {
    my $self = shift;
    my $isa = ref $self;

    $self->debug_msg(3, "AbstractEvent::process_edit_form()");

    my $errormsg = $self->_input_form_fields();

    if ($errormsg) {
        my $parentid = $self->parent_id();
        my $parent = Gizmo->restore($parentid);
        $parent->www_add_child("<font color=red>$errormsg</font>", $self->is_a, $self);
    } else {
        $self->_show_edit_success();
    }
}

# This method will process only the common form fields for AbstractInstance. This method should
# be extended. This method returns an error if any form validation fails
sub _input_form_fields {
    my $self = shift;
    my $errormsg = '';

    $self->name($FORM{name});
    $self->description($FORM{description});

    # input the date if it is defined
    my $date_defined = (defined($FORM{day}) && defined($FORM{month}) && defined($FORM{year}));

    if ($date_defined) {
        if ($FORM{timed} eq 'yes') {
          $self->date(timelocal(0, ($FORM{timed_min}||0), ($FORM{timed_hour}||0), $FORM{day}, $FORM{month}, $FORM{year}));
          ## seconds
          my $duration = ($FORM{duration_hour} * 60 * 60) + ($FORM{duration_min} * 60);
          $self->duration($duration);
        } else {
          $self->date(timelocal(0, 0, 0, $FORM{day}, $FORM{month}, $FORM{year}));
        }
        $self->timed($FORM{timed});
    }

    # check if everything is ok
    $errormsg .= "<br />Title cannot be empty." if $self->name() eq '';
    $errormsg .= "<br />Invalid Date: ".$self->months()->{$FORM{month}}." $FORM{day} " . ($FORM{year} + 1900) unless (!$date_defined || Date::Calc::check_date($FORM{year}+1900,$FORM{month}+1,$FORM{day}));

    return $errormsg;
}

# This method will define the Gizmo's behavior when an edit is successful
sub _show_edit_success {
    my $self = shift;

    $self->add();

    my $parent_id   = $self->parent_id();
    my $parent      = Gizmo->restore($parent_id);
    my $parent_isa  = $parent->is_a();
    my $child_isa   = $self->is_a();
    my $gizmo_name  = $self->get_gizmo_name();

    $self->two_choices(
        "<Big>New $gizmo_name added!</big><br /><br />What do you want to do?",
        "confirm",
        "Add another $gizmo_name", "index.pl?iid=$parent_id&isa=$parent_isa&op=add_child&child_type=$child_isa&dt=".$self->date(),
        "Enough for now!", "index.pl?iid=$parent_id&dt=".$self->date());
}
##############

sub www_save {
  my $self = shift;
  $self->debug_msg(3, "AbstractEvent::www_save()");
  my $url = $FORM{target};

  my $iid = $self->defined_or_exit($self->id,"id is missing.");
  my $errormsg = $self->_input_form_fields();
  if ($errormsg) {
    $self->www_modify("<font color=red>$errormsg</font>");
  } else {
    my $parentid = $self->parent_id();
    $self->save();
    $self->md_redirect("index.pl?iid=$parentid&dt=".$self->date());
  }
}

#####################################################
######                                        #######
######            BACK-END METHODS            #######
######                                        #######
#####################################################

#####################################################
###         WEB DISPLAY BACK-END METHODS          ###
#####################################################

sub show {
  my $self  = shift;
  $self->debug_msg(4, "AbstractEvent::show()");

  #my $s = "<p /><b>Title:</b> " . $self->name();
  my $s = "<table border=\"0\" width=\"100%\" cellspacing=\"0\" cellpadding=\"0\"><tr><td>\n";
  $s .= "<table border=\"0\" width=\"100%\" cellspacing=\"1\" cellpadding=\"5\">\n";
  $s .= "<tr><th colspan='2'>" . $self->name(). "</th></td></tr>";
  $s .= "<tr><td><b>Date";
  if ($self->c1() eq 'yes') {
      $s .= " and Time:</b></td><td>";
      $s .= $self->format_datetime($self->date(), 'M e, Y T');
      $s .= "-" . $self->format_datetime($self->date() + $self->duration(), 'T');
      $s .= "</td></tr>\n";
  } else {
      $s .= ":</b></td><td>";
      $s .= $self->format_datetime($self->date(), 'M e, Y') .
            "&nbsp;&nbsp;<i>(All Day Event)</i>";
      $s .= "</td></tr>\n";
  }
  my $user = Metadot::UserFactory->get_user( $self->uid() );
  my $parent_obj  = $self->get_parent_obj();

  $s .= "<tr><td><b>Details:</b></td><td>" . ($self->description() || '&nbsp') . "</td></tr>\n";
  $s .= "<tr><td><b>Author:</b></td><td>" . ($user->fullname() || $user->nickname() || '&nbsp') . "</td></tr>\n";
  $s .= "</table>\n</table>\n";
  $s .= "<p /><a href=\"".$self->_get_url_of_parent($parent_obj)."\">Back to ".$parent_obj->get_gizmo_name()."</a>\n";

  return $s;
}

##############

sub _generate_form_fields {
    my $self  = shift;
    my $result = shift || {};

    my $id = $self->id();

    my $dt              = $self->date() || $FORM{dt} || time;
    my $duration_hour   = 0;
    my $duration_min    = 0;
    my $timed           = $self->timed();
    my $duration        = $self->duration();

    my ($year, $month, $day, $timed_hour, $timed_min) = (localtime($dt))[5, 4, 3, 2, 1];

    if ($id > 0 || $self->date()) { ## editing or fixing error on add
      use integer;
      $duration_hour = $duration / 3600;
      $duration_min  = ($duration / 60) % 60;
    } else { ## adding new
      $timed_min = 0;
      $timed = 'yes';
      $duration_hour = 1;
      $duration_min  = 0;
    }

    ###
    ### BUILD DROP-DOWNS
    ###

    ## build day dropdown
    my $day_dropdown = "<select name = \"day\">\n";;
    for my $d (1..31) {
      $day_dropdown .= "    <option value=\"$d\"" .
                       ($d == $day ? " selected" : "") . ">$d\n";
    }
    $day_dropdown .= "</select>";
    ##

    ## build month dropdown
    my $month_dropdown = "<select name = \"month\">\n";;
    for my $m (0..11) {
      $month_dropdown .= "    <option value=\"$m\"" .
                       ($m == $month ? " selected" : "") . ">".$self->months()->{$m}."\n";
    }
    $month_dropdown .= "</select>";
    ##

    ## build year dropdown
    my $year_dropdown = "<select name = \"year\">\n";;
    for my $y (($year-1)..($year+5)) {
      $year_dropdown .= "    <option value=\"$y\"" .
                        ($y == $year ? " selected" : "") . ">" . ($y+1900) . "\n";
    }
    $year_dropdown .= "</select>";
    ##

    ## build hour dropdown
    my $hour_dropdown = "<select name = \"timed_hour\">\n";
    my $display_hour = '';
    my $ampm = '';
    for my $h (0..23) {
      $ampm = $h > 11 ? 'pm': 'am';
      if ($h == 0) {
          $display_hour = '12';
      } elsif ($h > 12) {
          $display_hour = $h - 12;
      } else {
          $display_hour = $h;
      }
      $display_hour .= " $ampm";
      $hour_dropdown .= "    <option value=\"$h\"" .
                        ($h == $timed_hour ? " selected" : "") . ">$display_hour\n";
    }
    $hour_dropdown .= "</select>";
    ##

    my $min_dropdown = "
    <select name=timed_min>
        <option value=0" . ($timed_min ==  0?' selected':'') . ">:00
        <option value=15". ($timed_min == 15?' selected':'') . ">:15
        <option value=30". ($timed_min == 30?' selected':'') . ">:30
        <option value=45". ($timed_min == 45?' selected':'') . ">:45
    </select>
    ";

    ## duration hour dropdown
    my $dh_dropdown = "<select name = \"duration_hour\">\n";;
    #my $dh_value = '';
    for my $dh (0..12) {
      #$dh_value = $dh < 10 ? "0$dh" : $dh;
      $dh_dropdown .= "    <option value=\"$dh\"" .
                        ($dh == $duration_hour ? " selected" : "") . ">$dh\n";
    }
    $dh_dropdown .= "</select>";

    my $dm_dropdown = "
    <select name=duration_min>
        <option value=0" . ($duration_min ==  0?' selected':'') . ">:00
        <option value=15". ($duration_min == 15?' selected':'') . ">:15
        <option value=30". ($duration_min == 30?' selected':'') . ">:30
        <option value=45". ($duration_min == 45?' selected':'') . ">:45
    </select>
    ";

    # aju 04-22-02 - escape notes and name HTML
    ## build the result hash
    $result->{name} = '<input type=text name=name size=55 maxlength=255 value="'.
        escapeHTML($self->name()) .'">';
    $result->{description} = '<textarea name=description rows=5 cols=60 WRAP=VIRTUAL>'.
        escapeHTML($self->description()) . '</textarea>';
    $result->{timed_no} = '<input type=radio name=timed value=no' . ($timed eq 'no'?' checked':'') . '>';
    $result->{timed_yes} = '<input type=radio name=timed value=yes' . ($timed eq 'yes'?' checked':'') . '>';

    $result->{year}             = $year_dropdown;
    $result->{month}            = $month_dropdown;
    $result->{day}              = $day_dropdown;
    $result->{timed_hour}       = $hour_dropdown;
    $result->{timed_min}        = $min_dropdown;

    $result->{duration_hour}    = $dh_dropdown;
    $result->{duration_min}     = $dm_dropdown;

    return $result;
}


# an alias for Calendar->format_datetime
sub format_datetime {
    my $class = shift;
    return Calendar->format_datetime(@_);
}

#####################################################
###                   THE END                     ###
#####################################################


1;


__END__

=head1 NAME

Gizmo::AbstractEvent -

=head1 SYNOPSIS

    use Gizmo::AbstractEvent;

    $event = Event->new ($id);

    $AC = $event->get_AC ();
    $event->www_show ();
    $event->www_delete ();
    $event->www_delete_ok ();
    $event->process_edit_form ();
    $event->www_save ();
    $event->show ();
    $html = $event->show_edit_form ($title);
    $version = Event->get_version();

=head1 DESCRIPTION

=cut


