###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package AggregatedCalendar;

use strict;
use CGI;
use CGI qw(escapeHTML);
use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY);

use Gizmo::Calendar;
use DBAccess; 
use Time::Local;
use Utility;
use vars qw(@ISA);

@ISA=qw(Calendar);

sub get_gizmo_name {
   my $self = shift;
   return "Aggregated Calendar View";
}


sub exec_show_summary {
    my $self = shift;

    return $self->SUPER::show_in_column(@_);
}

sub show_edit_form {
    my $self  = shift;
    my $title = shift;

    my $name = $self->name();

    my $clist = $self->get_calendar_selection_list();

    # We have to construct the edit form to get a list of all the calendars available
    my $html = "<h1>$title</h1>\n";

    $html .= "<b>Aggregated Calendar View name</b><br/>\n"
             ."<input type='text' name='name' size='55' maxlength='255' value='$name' />\n"
             ."<input type='hidden' name='c5' value='1' />\n"
             ."<input type='hidden' name='c3' value='on' />\n"
             ."<div style='margin-top: 40px'>The following is a list of available calendars</div>\n"
             ."<div id='aggregated_calendar_view_list'>\n"
             ."  <table border='2' cellspacing='2' cellpadding='2' width='45%' >\n"
             ."    <tr>\n"
             ."      <td><b>Add</b></td>\n"
             ."      <td><b>Calendar Name</b></td>\n"
             ."      <td><b>Description</b></td>\n"
             ."    </tr>\n";

    my $calendars = $self->get_all_calendars();

    foreach my $cal( @$calendars ){
       $DISPLAY->debug_msg(5,"got calendar id[".$cal->{iid}."]");

       my $cid   = $cal->id();
       my $cname = $cal->name();
       my $desc  = $cal->description() || '&nbsp;';

       my $checked = $self->is_calendar_selected($cid,$clist);

       my $cal_url = $self->make_url("index.pl?iid=$cid&amp;isa=Calendar&amp;op=show");

       $html .=  "    <tr>\n"
                ."      <td><input type='checkbox' name='calendar_$cid' value='$cid' $checked /></td>\n"
                ."      <td><a href='$cal_url'>$cname</a></td>\n"
                ."      <td>$desc</td>\n"
                ."    </tr>\n";
    }
    $html .=  "  </table>\n"
            . "</div>\n";

    return $html;
}

sub get_all_calendars {
    my $self = shift;

    $DISPLAY->debug_msg(5,"IN get all calendars");
    my @all_calendars = $self->get_object_accessor()->retrieve_where(where=>"isa='Calendar'");
    my @calendars = ();

    foreach my $calendar (@all_calendars) {
      if ($calendar->is_allowed_to_do('show')) {
        $DISPLAY->debug_msg(5,"IN get all calendars calendar[".$calendar->id()."] will be added");
        push @calendars, $calendar;
      }
    }

    return \@calendars;
}

sub get_calendar_selection_list {
    my $self = shift;

    my @list = split(/,/,$self->t10());
    return \@list;
}

sub get_calendar_selection_list_string {
    my $self = shift;
 
    return join(',',@{$self->get_calendar_selection_list()});
}

sub is_calendar_selected {
    my $self  = shift;
    my $cid   = shift;
    my $clist = shift;

    my $result = '';

    foreach my $cal(@$clist){
       if( $cid == $cal ){
           $DISPLAY->debug_msg(5,"Calendar[$cid] is selected!");
           return "checked='true'";
       }
    }

    return $result;
}

sub www_save {
    my $self = shift;

    my $calendar_selection;
    foreach my $key( keys %FORM){
       if( $key =~ "calendar_" ){
           $calendar_selection .= "$FORM{$key},";
       }
    }

    $self->t10($calendar_selection);

    $self->SUPER::www_save();
}

sub _get_parent_list {
    my $self = shift;
    $DISPLAY->debug_msg(5,"IN _get_parent_list");
    return $self->get_calendar_selection_list();
}

sub www_add_child {
    my $self = shift;
    my $args = shift || {};

    ## We do not want to allow adding events to an
    ## aggregated calendar
    $DISPLAY->debug_msg(5,"In add_child"); 

    my $dt = $args->{datetime};
   
    my $html = "<div><b>Please click on the calendar to which you want to add an event</b></div><br/>";

    foreach my $cal(@{$self->get_calendar_selection_list}){
       my $c = Calendar->new($cal);
       $html .= '<div>'.$c->make_add_event_link({datetime=>$dt, text=>$c->name(),view_mode=>'day'}).'</div>';
    } 
  
    $self->print_form($html,"Adding new event");

}

sub get_monthly_events_for_view {
    my $self = shift;

    $DISPLAY->debug_msg(5,"-IN get_monthly_events_for_view");
    my $start = '1122786000';
    my $end   = '1125809999';
    my $parents = $self->get_calendar_selection_list_string();

    my ($start_datetime, $end_datetime) = $self->_get_datetimes_of_month_display();

    my @start_date_array = (localtime($start_datetime))[3,4,5];
    my @end_date_array = (localtime($end_datetime))[3,4,5];
  
    $start = timelocal(0, 0, 0, @start_date_array);
    $end   = timelocal(0, 0, 0, @end_date_array);
       
    my $events = DBAccess->sqlSelectMany(
                    "distinct iid, i1",
                    "instance",
                    "parentiid in ($parents) AND
                     isa = 'Event' AND
                     i1 >= $start AND i1 <= $end",
                    "order by i1"
                 );
    $DISPLAY->debug_msg(5,"Get_montly_events_for_view will return events".$DISPLAY->dumper($events));        
    return $events; 
}

sub generate_day_view {
    my $self = shift;
    my $args = shift || {};

    $DISPLAY->debug_msg(5,"IN generate_day_view");
    $args->{disable_add_event} = 0;

    return $self->SUPER::generate_day_view($args);
}

sub get_day_event_listing {
    my $self = shift;
 
    $DISPLAY->debug_msg(5,"IN get_day_event_listing");
    my $selected_calendars = $self->get_calendar_selection_list();
    my $events_hash = {};
    my @column_maxes;

    # We will build a has as follows:
    # $events_hash->{event_hour}->{event_id}->{event_details}

    # Get events for selected calendars and build the
    # event hash
    foreach my $cal (@$selected_calendars){
       ## For each calendar, traverse down the event hash
       my $c = Calendar->new($cal);

       foreach my $hour (keys (%{$c->{events}})){
         my $event_id = $c->{events}->{$hour};
         foreach my $event_id (keys(%{$c->{events}->{$hour}}) ){
             $DISPLAY->debug_msg(5," I HAVE event[$event_id]");
    
             # Alter the description of the event to add calendar name
             my $desc = $c->{events}->{$hour}->{$event_id}->{'desc'};
             $desc .= "&nbsp;(".$c->name.")";
             $c->{events}->{$hour}->{$event_id}->{'desc'} = $desc;

             #$DISPLAY->debug_msg(5,"WILL add HOUR[$hour] event_id[$event_id] into events hash".$DISPLAY->dumper($c->{events}->{$hour}->{$event_id}));
             $events_hash->{$hour}->{$event_id} = $c->{events}->{$hour}->{$event_id};

             # Keep track of the maximum number of columns per event
             push @column_maxes, $c->{'MAXCOLS'};
         }
       }
    }

    #Determine the max column_maxa
    @column_maxes = sort @column_maxes;
    my $maxcol = pop @column_maxes;
    $events_hash->{MAXCOLS} = $maxcol + 1;

    # Add a list of calendars that were included in this event list
    $events_hash->{calendar_list} = $self->get_calendar_selection_list_string();

    $self->{events} = $events_hash;

    #$DISPLAY->debug_msg(5,"<br><br><br>++++++++++++++++++Final Hash is<br>".$DISPLAY->dumper($events_hash));

    return $self->SUPER::get_day_event_listing($events_hash);
}

