###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Application;

use Metadot qw($USER %FORM $PARAMS);

use GizmoBuilder;
use strict;
use vars qw(@ISA);
use CGI qw/escapeHTML unescapeHTML/;
@ISA=qw(GizmoBuilder);
my $version = "4.5";


sub get_version {
    return $version;
}


my $AC =
  {
   BUNDLES => {
           MOD => { DESC => 'Edit',
            OPS => [qw(create modify save up down paste delfile delfileok)],
            DEFAULT_LEVEL => 8,
            MIN_LEVEL => 8,
              },
           DEL => { DESC => 'Delete and Cut',
            OPS => [qw(delete delete_ok cut)],
            DEFAULT_LEVEL => 8,
            MIN_LEVEL => 8,
            PARENT_DEP => {cut => 'MOD'},
              },
           DISP => { DESC => 'View',
             OPS => [qw(show)],
             DEFAULT_LEVEL => 2,
             MIN_LEVEL => 2,
             DEFAULTS_TO => 'on',
               },
           EDITP => { DESC => 'Change<br />Permissions',
              OPS => [qw(edit_permissions set_permissions change_owner)],
              DEFAULT_LEVEL => 8,
              MIN_LEVEL => 8,
            },
          },
   DISPLAY_ORDER => [qw(DISP MOD DEL EDITP)],
  };

###
### ACCESS CONTROL METHODS
###
sub get_AC {
  my $self = shift;
  my $class = ref $self || $self;

  die "&get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

  return $AC;
}


####################################
sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

  $self->{cool}='No';

  $self->set_field_info (
        "name",        "<b>Application Name</b>", 1,
        "description", "<b>Description</b>", 0,
        "keywords",    "<b>Keywords and Synonyms</b>", 0,
        "cool",        "<b>Star this Item</b>", 0,
       );

  my $info_string = "This is a temporary interface, which in subsequent releases will
be made more automated and conceal Application creation from non-admin users.
Only developers/system admins should use this feature.
<p>The name of the application must correspond to the name of
an existing App module (as described in the Metadot Developers' Guide).
The name is case-sensitive.  Do not add the &quot;.pm&quot;
suffix for the module.  For example, &quot;Basic&quot; is a module name.";
  $self->set_intro_text($info_string);
}


sub get_is_public {
    my $class = shift;

    if ($PARAMS->{use_application_gizmo}) {
    return "Yes";
    }
    else {
    return "No";
    }
}

##################################
sub process_edit_form() {
    my $self  = shift;
    my $error = undef;

    my $name = $FORM{name};
    $self->require_app($name);

    ## check for required fields
    foreach my $field (keys (%{$self->{fields}})) {
        if ($self->{fields}->{$field}->{required} &&
        (!$FORM{$field} || !($FORM{$field} =~ /\S/)) # field is not just whitespace
        ) {
            $error .= $self->{fields}->{$field}->{label} .
        " is a required field.\n";
        }
    }

    my $iid = $self->get_iid();

    if (defined($error) && $iid){ ## if error and updating
        $self->www_modify($error);
    }
    elsif (defined($error)) {   ## if error and saving new
        my $parentid = $self->parent_id();
        my $parent = Gizmo->restore($parentid);
#   print STDERR "in gizmo::process_edit_form, calling parent add child on error\n";
        $parent->www_add_child($error, $self->is_a);
    }
    else {          ## no errors
    my $checked_reqs = 1;
        $self->www_save($checked_reqs);
    }
}

###################################
sub show_application_bar {
    my $class = shift;

    my $output = undef;
    my $app_list = undef;
    my $categorytitlebgcolor = $PARAMS->{'categorytitlebgcolor'};
#    my $fileuploaddir = $PARAMS->{'fileuploaddir'};
    my $fileuploaddir = undef;
    my $appcount = 0;

    my ($result) = DBAccess->sqlSelectMany("iid", "instance", "isa='Application'");

    $output = "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" width=\"100%\" "
                 ."bgcolor='#$categorytitlebgcolor'><tr>\n";

    while (my ($iid) = $result->fetchrow()) {
    $appcount++;
        my $app = $class->new($iid);
        my $edit = $app->show_edit_button();
        my $perm = $app->show_ac_button();
        my $delete = $app->show_del_button();
        my $app_root = $app->parent_id();
        my $name = $app->name();
        my $img = $app->file1();

        if ($app->is_allowed_to_do('show', $USER)){
            $app_list .= "<td valign='bottom' align='center'> "
                        ."<a class='catheader' href='index.pl?iid=$app_root&amp;app=$name'>";
            if($img){
                $app_list .="<img src='$fileuploaddir/$iid/$img' border=\"0\"><br />";
            }

            $ app_list.= "$name</a> $edit $perm $delete </td>";
        }
    }

    $output .= ($app_list)? $app_list : "<td>&nbsp;</td>";

    $output .= "</tr></table>";
    if ($appcount) {
    return $output;
    }
    else {          # if no apps, don't return the containing table.
    return "";
    }

}

1;

__END__

=head1 NAME

Application - gizmo for creating a link to an application. This link will appear above the
              category listing on the main page.

=head1 SYNOPSIS

    $application = Application->new($id)

    $AC = $application->get_AC();
    $boolean = $application->get_is_public();
    $html = $application->show();
    $html = Application->show_application_bar();
    Application->get_version();

=head1 DESCRIPTION

This gizmo creates a link for Applications to appear in an application bar above the category
bar on the default layout of the system.

=head1 Methods

constructor

    Gizmo::Application->new($id);

Called by the metadot framework, this method should not be called.

    $AC = $application->get_AC();

Called by the metadot framework. This method is used to determine if a gizmo should appear in
the dropdown list of gizmos.

    $boolean = $application->get_is_public();

This class method gets all applications that have been created and builds the application
bar out of them.

    $html = Application->show_application_bar();

This class method retrieves the version of the gizmo:

    Application->get_version();


=cut
