###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Calendar;

use strict;
use CGI;
use CGI qw(escapeHTML);
use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY);


use Gizmo;
use GizmoBuilder;
use Gizmo::Calendar::Event;
use Utility;
use Metadot::SystemApp::Util;
use Metadot::Controllable;
use Time::Local;
use vars qw(@ISA);

@ISA=qw(GizmoBuilder Metadot::Controllable);
my $version = "4.5";

sub get_version {
    return $version;
}

use constant EVENT_TYPE => 'Event';
use constant EVENT_TYPE_NAME => EVENT_TYPE->get_gizmo_name();

use constant VIEW_MONTH     => 'month';
use constant VIEW_DAY       => 'day';
use constant VIEW_MINI      => 'mini';
use constant VIEW_DEFAULT   => VIEW_MINI;


my $COLOR1 = '#CCCCCC';
my $COLOR2 = '#EEEEEE';
my $COLOR3 = '#FFFFFF';
my $COLOR5 = '#AAAAAA';
my $COLOR6 = '#FFFFCC'; # highlight

my $TASKLIST_SCOPE_NONE   = 0;
my $TASKLIST_SCOPE_LOCAL  = 1;
my $TASKLIST_SCOPE_GLOBAL = 2;

use constant EVENT_LISTING_TYPE_DAYS   => 0;
use constant EVENT_LISTING_TYPE_EVENTS => 1;

sub customize_permissions {
   my $class = shift;
   my $AC = shift;

   $class->SUPER::customize_permissions($AC);

   $AC->add_op('MOD','add_child');
   $AC->add_op('MOD','create_child');
   $AC->add_op('MOD','choose_colors');
}



{
    my $search_params = {
             searchable_fields => [ qw( name description keywords) ],
             displayed_fields  => [ qw( description) ],
#            child_isa => 'Event',
#            child_searchable_fields => [ qw( name description t1 t2 t3 c2 ) ],
            };

    sub get_search_params {
    my $self = shift;

    return $search_params;
    }
}

####################################
sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

    $self->{c3} = '1';
    $self->{i1} = $TASKLIST_SCOPE_LOCAL;
    $self->{c4} = '5';

    # if using 'themes', suppress color controls (but keep the vars so
    # settings are maintained)
    if (Metadot::Utils::Theme->use_theme_info()) {
    $self->set_field_info (
          "name",        "<b>".$self->get_gizmo_name." Name</b>",              1,
          "description", "<b>Description</b>",                0,
          "keywords",    "<b>Keywords and Synonyms</b>",      0,
          "c3",          "<b>Show ".$self->EVENT_TYPE_NAME." Quick List</b>",      0,
          "i1",          "<b>Show Task List content in ".$self->EVENT_TYPE_NAME." Quick List</b>", 0,
          "c4",          "<b>Number of days or events to show in ".$self->EVENT_TYPE_NAME." Quick List</b>", 0,
          "c5",          "", 0, ## days or events
          "t1",          "<b>Default view mode</b>", 0, ## default view
          "c1",          "", 0,
          "c2",          "", 0,
        );

        $self->set_field_type('c1', 'hidden');
        $self->set_field_type('c2', 'hidden');
    } else {            # regular usage
        $self->set_field_info (
              "name",        "<b>".$self->get_gizmo_name." Name</b>",              1,
              "description", "<b>Description</b>",                0,
              "keywords",    "<b>Keywords and Synonyms</b>",      0,
              "c3",          "<b>Show ".$self->EVENT_TYPE_NAME." Quick List</b>",      0,
              "i1",          "<b>Show Task List content in ".$self->EVENT_TYPE_NAME." Quick List</b>", 0,
              "c4",          "<b>Number of days or events to show in ".$self->EVENT_TYPE_NAME." Quick List</b>", 0,
              "c5",          "", 0, ## days or events
              "t1",          "<b>Default view mode</b>", 0, ## default view
              "c1",          "<b>Title row background color</b>", 0,
              "c2",          "<b>Footer background color</b>",    0,
            );
        $self->set_field_form_text('c1', $self->_get_form_text('c1'));
        $self->set_field_form_text('c2', $self->_get_form_text('c2'));
    }

    my $num_days = $self->_get_num_days();
    if (!defined($num_days) || $num_days eq '') {
        $self->_set_num_days(3);
        $self->_set_listing_type(EVENT_LISTING_TYPE_EVENTS);
    }

    $self->set_field_type('c3', 'checkbox');
    $self->set_field_type('i1', 'select');
    $self->{fields}->{i1}->{selectoptions} = $self->_get_task_selector_text();
    #$self->{fields}->{$locale_field}->{field_value} = $self->get_locale_name();
    $self->set_field_type('c5', 'select');
    $self->{fields}->{c5}->{selectoptions} = $self->_get_event_listing_type_text();

    $self->set_field_type('t1', 'select');
    $self->{fields}->{t1}->{selectoptions} = $self->_get_default_view_type();

    $self->{display_wrapped} = { show => 1, };

    ## total number of columns used in event listing - will change
    ## depending on events
    $self->{MAXCOLS} = 1;
    ## minimum hour to display in event listing - will change
    ## depending on events
    $self->{MINHOUR} = 7;
    ## maximum hour to display in event listing - will change
    ## depending on events
    $self->{MAXHOUR} = 19;

    ## will initialize calendar's time values
    $self->_initialize_calendar();
}


# Calendar needs to load its events every time. This can
# probably be improved in the future.
sub restore {
    my $class = shift;
    my @args = @_;
    my $self = $class->SUPER::restore(@args);
    #$class->debug_msg(4, "Calendar\->restore(@args) got $self");

    if ($self) {
        $self->_initialize_calendar();
    }

    return $self;
}


#####################################################
######                                        #######
######            FRONT-END METHODS           #######
######                                        #######
#####################################################
sub www_add_child {
  my $self = shift;
  my $errormsg = shift;
  my $child_type = shift;
  my $child_object = shift;

  $self->debug_msg(3, "Calendar::www_add_child()");

  my $class = ref($self) || $self;
  my $child_isa = $FORM{child_type} || $child_type;
  my $gizmo_name = $child_isa->get_gizmo_name();

  my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
  # we re-instanciate the object because we want a brand new one
  # unless there is an error message (we want to keep his data)
  #my $child = $child_isa->new({ parent_id => $parentid }) unless ($errormsg);
  my $child = $errormsg && defined($child_object) ? $child_object : $child_isa->new({ parent_id => $parentid });

    $errormsg = $errormsg ?
                "<p /><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p />\n" :
                "";
  my $form = $child->show_edit_form("Adding new $gizmo_name");
  my $content =
Components->color_palette_code()."
<form name=fm action='index.pl' method='post' ENCTYPE=\"multipart/form-data\">
<input type='hidden' name='isa' value='$class'>\n
<input type='hidden' name='iid' value='$parentid'>\n
<input type='hidden' name='op' value='create_child'>\n
<input type='hidden' name='child_type' value='".$self->EVENT_TYPE."'>\n
$errormsg
$form
<table align='right'><tr><td><input type='submit' value='Create'></td></tr></table>
<br />&nbsp;
</form>
";

  $child->print_form($content, "Adding new $child_isa");
}

sub www_create_child {
  my $self = shift;
  my $child_type = shift;

  $self->debug_msg(3, "Calendar::www_create_child()");

  my $child_isa = $FORM{child_type} || $child_type;

  my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
  # we re-instanciate the object because we want a brand new one
  my $child = $child_isa->new({
                         uid => $USER->uid(),
                         parent_id => $parentid,
                        });

  # this method should ALWAYS be overridden in derived classes.
  my $result = $child->process_edit_form();
}

sub show_edit_form {
    my $self = shift;
    my $title = shift;

    my $id = $self->id();
    my $isa = $self->is_a();
    ## fix because values not being populated
    $self->set_field_form_text('c1', $self->_get_form_text('c1'));
    $self->set_field_form_text('c2', $self->_get_form_text('c2'));

  my $js = "
<SCRIPT LANGUAGE=\"JavaScript\" SRC=\"/js/ColorPicker2.js\"></SCRIPT>
<SCRIPT LANGUAGE=\"JavaScript\" SRC=\"/js/AnchorPosition.js\"></SCRIPT>
<SCRIPT LANGUAGE=\"JavaScript\" SRC=\"/js/PopupWindow.js\"></SCRIPT>


<SCRIPT LANGUAGE=\"JavaScript\">
// Create a new ColorPicker object using Window Popup
var cp = new ColorPicker('window');

function setFieldValue(fieldname, value) {
        document.fm[fieldname].value = value;
    }

</SCRIPT>
";

    return $js . $self->SUPER::show_edit_form($title);
}

###################################
sub show {
    my $self = shift;

    $DISPLAY->debug_msg(3, "In $self\->show()");

    my $dt = $self->determine_datetime();

    # check for view mode
    my $view_mode = $FORM{view_mode};
    my $session_key = 'CALENDAR_VIEW_MODE_'.$self->id();
    if (defined $view_mode)
    {
        # save the view mode in the session
        $SESSION->set_value($session_key, $view_mode);
    }
    else
    {
        # if no view mode defined, check the session
        # if still no view mode defined, check the object

        $view_mode = $SESSION->get_value($session_key) || $self->get_view_mode();
    }

    return $self->show_base({ datetime=>$dt, view_mode=>$view_mode});
}


# determines the correct datetime to show
sub determine_datetime
{
    my $self = shift;

    # first check the form
    my $result = $FORM{dt};
    my $session_key = $self->generate_datetime_session_key();
    my $id = $self->id();

    if (defined $result)
    {
        if (defined $id)
        {
            # save it to the session
            $SESSION->set_value($session_key, $result);
            $DISPLAY->debug_msg(4, "$self $id set the datetime to ".localtime($result));
        }
    }
    else
    {
        # check the session
        if (defined $id)
        {
            $result = $SESSION->get_value($session_key);
        }
        if ($result)
        {
            $DISPLAY->debug_msg(4, "$self $id retrieved datetime from session ".localtime($result));
        }
        else
        {
            $result = $self->{current_datetime};
            $DISPLAY->debug_msg(4, "$self ".($id || 'undef')." used current datetime ".localtime($result));
        }
    }

    $self->apply_datetime($result);
    return $result;
}


# this is a mess, but the original writer of this gizmo f**ked it up
sub apply_datetime
{
    my $self     = shift;
    my $datetime = shift;

    if (defined $datetime)
    {
        ## object's datetime broken down
        (
         $self->{sec},   #0-60
         $self->{min},   #0-59
         $self->{hour},  #0-23
         $self->{day},   #1-31
         $self->{month}, #0-11
         $self->{year},  #>1 since 1900
         $self->{wday},  #0-6
         $self->{yday},  #1-366
         $self->{isdst}  #0 or 1 daylight savings in effect
        ) = localtime($datetime);
    }
}


# for backwards compatibility
sub datetime {
    my $self = shift;
    if (@_) {
        return $self->apply_datetime(@_);
    } else {
        return $self->determine_datetime();
    }
}



sub generate_datetime_session_key
{
    my $self = shift;
    my $id   = $self->id() || 'undef';
    return 'CALENDAR_DATETIME_'.$id;
}

sub show_base
{
    my $self = shift;
    my $args = shift || {};

    my $dt        = $args->{datetime};
    my $view_mode = $args->{view_mode};

    my $name        = $self->get_name();
    my $description = $self->get_description();

    $self->debug_msg(4, "Calendar::show has view mode $view_mode; current time $dt; prev: ".$self->get_previous_day_datetime($dt)." next: ".$self->get_next_day_datetime($dt));


    my $display_name = $name;
    if ($description)
    {
        $display_name .= ':';
    }

    my $buttons = $self->get_buttons();

    my $html .= "$buttons<h2 class='calendar'>$display_name</h2>";
    $html .= "<span class='element_description'>$description</span>\n" if ($description);
    $html .= "<br />";

    if ($view_mode eq VIEW_MONTH)
    {
        $html .= $self->generate_month_view({datetime => $dt});
    }
    else
    {
        $html .= $self->generate_day_view({datetime => $dt});
    }

    return $html;
}

=head2

view_mode

Gets or sets the calendar's default view mode. Can be overridden with user settings

=cut
sub get_view_mode
{
    my $self = shift;
    return $self->t1() || VIEW_DEFAULT;
}


sub show_in_column {
    my $self = shift;
    my $color = shift;
    my $caller_id = shift;
    my $caller_op = shift;

    $DISPLAY->debug_msg(3, $self->name()." show_in_column; view mode is ".$self->get_view_mode());

    if ($self->get_view_mode() eq VIEW_MINI)
    {
        $DISPLAY->debug_msg(4, "Displaying in column");
        my $buttons = $self->get_buttons();
        $buttons .= "&nbsp;\n" if $buttons;
        my $name    = $self->name();

        my $event_quick_list = $self->c3() ? Utility->get_spacer(1,4).$self->get_event_quick_list() : '';
        return "
          <table width=\"157\" border=\"0\" cellspacing=\"0\" cellpadding=\"1\" bgcolor=\"$COLOR1\"><tr><td>
          <table width=\"155\" border=\"0\" cellspacing=\"0\" cellpadding=\"2\" bgcolor=\"$COLOR3\">
            <tr><td align='center'><b><font class=\"calendar\">" . $buttons . $self->name() . "</font></b></td></tr>
            <tr><td align='center'>"
            . $self->get_month_navagation({
                color       => $color,
                caller_id   => $caller_id,
                caller_op   => $caller_op,
                datetime    => $self->determine_datetime(),
                })
            .$event_quick_list
            . "</td></tr>
          </table></td></tr></table>\n";
    }
}

sub show_summary {
    my $self        = shift;

    $DISPLAY->debug_msg(3, $self->name()." show_summary; view mode is ".$self->get_view_mode());

    # Check if my view mode is mini
    my $view_mode = $self->get_view_mode();
    if ($view_mode eq VIEW_MINI) {
        # do not show the mini-mode if my parent is a Category; this is part of
        # the messy show_calendar_children code
        my $parent = $self->get_parent();
        unless ($parent && $parent->isa('Category')) {
            return $self->show_in_column();
        }
    } else {
        $DISPLAY->debug_msg(4, "Displaying in summary");

        my $dt = $self->determine_datetime();

        return $self->show_base({
            datetime =>$dt,
            view_mode=>$view_mode});
    }


}

sub show_line {
    my $self        = shift;
    return $self->show_summary();
}

sub gizmo_list_decorator {
   return 'HTML::Decorator::NullDecorator';
}


sub generate_day_view
{
    my $self = shift;
    my $args = shift || {};

    my $dt = $args->{datetime};

    $DISPLAY->debug_msg(4, "generate_day_view(datetime => $dt)");

    my $prev_day_link = $self->make_calendar_link({datetime=>$self->get_previous_day_datetime($dt), text=>'<', view_mode=>VIEW_DAY});
    my $next_day_link = $self->make_calendar_link({datetime=>$self->get_next_day_datetime($dt), text=>'>', view_mode=>VIEW_DAY});
    my $month_link    = $self->make_calendar_link({datetime=>$dt, text=>'Month view', view_mode=>VIEW_MONTH});

    my $html = "
    <table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%\">
      <tr>
        <td>
          <table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%\">
            <tr>
              <td align='left'><b>$prev_day_link " . $self->format_datetime($dt, 'W M e, Y') . " $next_day_link</b></td>
              <td>$month_link</td>
              <td align='right'>"
              . $self->make_add_event_link({datetime=>$dt, text=>'Add '.$self->EVENT_TYPE_NAME}) . "</td>
            </tr>
          </table>
        </td>
      </tr>
      <tr valign=\"top\">
        <td>" . $self->get_day_event_listing() . "</td>\n";

        $DISPLAY->debug_msg(5, "globallook: ".($PARAMS->{globallook} || ''));

if ($PARAMS->{globallook} eq "usetemplate") {
    $html .= "
        <td width=\"1%\">&nbsp;&nbsp;</td>
        <td width=\"10%\">" . $self->show_in_column($PARAMS->{'header1_color'}) . "</td>\n";
}
$html .= "
      </tr>
    </table>\n";

    return $html;
}


sub generate_month_view
{
    my $self = shift;
    my $args = shift || {};

    my $dt = $args->{datetime};

    my $color       = $PARAMS->{'header1_color'};
    my $caller_id   = $FORM{id};
    my $caller_op   = $FORM{op};

    ## today's formatted date
    my $todays_fdate = $self->format_datetime($self->{current_datetime}, 'M e, Y');

    ## month formatted date
    my $month_fdate = $self->format_datetime($dt, 'M Y');


    my $this_month_link = $self->make_calendar_link({
        datetime=>$dt, text=>$month_fdate,
        id=>$self->id(), op=>'show',
        view_mode=>VIEW_MONTH});

    my $prev_month_link = $self->make_calendar_link({
        datetime=>$self->_get_previous_month_datetime(),
        text=>'&lt;', id=>$caller_id, op=>$caller_op,
        view_mode=>VIEW_MONTH});
    my $next_month_link = $self->make_calendar_link({
        datetime=>$self->_get_next_month_datetime(), text=>'&gt;',
        id=>$caller_id, op=>$caller_op,
        view_mode=>VIEW_MONTH});


    my $prev_month_link_2 = $self->make_calendar_link({
        datetime=>$self->_get_previous_month_datetime(),
        text=>'&lt; previous', id=>$caller_id, op=>$caller_op,
        view_mode=>VIEW_MONTH});

    my $next_month_link_2 = $self->make_calendar_link({
        datetime=>$self->_get_next_month_datetime(), text=>'next &gt;',
        id=>$caller_id, op=>$caller_op,
        view_mode=>VIEW_MONTH});

    my $today_link      = $self->make_calendar_link({
        datetime=>$self->{current_datetime},
        text=>'Today',
        id=>$caller_id, op=>$caller_op,
        view_mode=>VIEW_DAY});


    my $month_table = $self->_get_days_of_month_display({
        # TODO: move the styles into a stylesheet
        container_properties    => 'class="calendar_month" style="width: 100%; border-collapse: collapse"',
        cell_properties         => 'class="calendar_month_cell" style="height: 70px;"',
        cell_day_properties     => 'class="calendar_month_cell_day" style="height: 50px; padding: 5px"',
        cell_date_container_properties  => 'class="calendar_month_cell_date" style="padding: 5px"',
        cell_event_container_properties => 'class="calendar_month_cell_event" style="padding: 5px"',

        expanded                => 1,
    });


    my $event_quick_list = $self->c3() ? $self->get_event_quick_list() : '';

    my $out = <<EOM
      <table width="100%"
             border="0"
             cellspacing="0"
             cellpadding="0"
             class="calendar_month">
        <tr>
          <td class="calendar_summary" colspan="2">
            <table border="0" cellspacing="0" cellpadding="0" width="100%" height="100%">
              <tr valign="bottom">
                  <td align="left">
                    <h3>$prev_month_link $this_month_link $next_month_link</h3>
                  </td>
                  <td align="right">
                    $today_link is $todays_fdate
                  </td>
              </tr>
            </table>
        </tr>
        <tr>
          <td colspan="2">
          $month_table
          </td>
        </tr>
        <tr><td colspan="2">&nbsp;<!-- spacer row --></td></tr>
        <tr valign="top">
          <td>$event_quick_list</td>
          <td align='right'>$prev_month_link_2\&nbsp;|&nbsp;$next_month_link_2</td>
        </tr>
      </table>
EOM
;

    return $out;
}



## builds calendar display for month
sub get_month_navagation {
    my $self = shift;
    my $args = shift || {};

    $DISPLAY->debug_msg(4, "get_month_navagation begins");

    my $color       = $args->{color};
    my $caller_id   = $args->{caller_id};
    my $caller_op   = $args->{caller_op};
    my $datetime    = $args->{datetime} || $self->datetime();

    my $self_id     = $self->id();
    my $self_op     = 'show';
    
    ## today's formatted date
    my $todays_fdate = $self->format_datetime($self->{current_datetime}, 'M e, Y');
    ## month formatted date
    my $month_fdate = $self->format_datetime($datetime, 'M Y');

    my $header_color; my $footer_color;
    if ($color && ($color ne '')) {
        $header_color = $color;
        $footer_color = $color;
    }
    else
    {
        $header_color = $self->c1();
        $footer_color = $self->c2();
    }

    my $month_table = $self->_get_days_of_month_display({
        cell_day_properties => 'class="calendar"',
    });

    my $prev_month_link = $self->make_calendar_link_title({
        datetime=>$self->_get_previous_month_datetime(),
        text=>'&lt;', id=>$self_id, op=>$self_op,
        view_mode=>VIEW_MONTH});

    my $next_month_link = $self->make_calendar_link_title({
        datetime=>$self->_get_next_month_datetime(),
        text=>'&gt;', id=>$self_id, op=>$self_op,
        view_mode=>VIEW_MONTH});

    my $this_month_link  = $self->make_calendar_link_title({
        datetime=>$self->{current_datetime},
        text=>$month_fdate,
        id=>$self_id, op=>$self_op,
        view_mode=>VIEW_MONTH});

    my $today_link      = $self->make_calendar_link_title({
        datetime=>$self->{current_datetime},
        text=>'<span class="smaller">Today</span>',
        id=>$self_id, op=>$self_op,
        view_mode=>VIEW_DAY});

    my $out = "
<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"$COLOR1\">
<tr><td>\n
    <table border=\"0\" cellspacing=\"1\" cellpadding=\"1\" align='center'>
      <tr><td bgcolor=\"$header_color\"><center><b><font class=\"calendartitle\">$prev_month_link $this_month_link $next_month_link</font></b></center></td></tr>
      <tr><td bgcolor=\"$COLOR3\">$month_table</td></tr>
      <tr><td bgcolor=\"$footer_color\" align='center'><font class=\"calendartitle\"><b>$today_link</b><span class=\"smaller\"> is <span style=\"white-space: nowrap\">$todays_fdate</span></span></font></td></tr>
    </table>
</td></tr>
</table>
";

    $DISPLAY->debug_msg(5, "get_month_navagation returns '$out'");

    return $out;
}

## displays event quick list
sub get_event_quick_list {
    my $self = shift;
    my %args = @_;

    my $event_listing_type = $self->get_c5();
    if ($event_listing_type == EVENT_LISTING_TYPE_EVENTS) {
        return $self->_get_events_quick_list($args{color});
    } else {
        return $self->_get_days_quick_list($args{color});
    }
}

sub _get_days_quick_list {
    my $self = shift;
    my $color = shift;

    # if info not passed in as method arg, use default.
    my $number_of_days = $self->_get_num_days();    #  get the default info from the object
    if (!defined($number_of_days) || ($number_of_days eq '')) {
        $number_of_days = 3;
    }

    $DISPLAY->debug_msg(3,"in _get_days_quick_list; got number of days[$number_of_days]");
    my $id = $self->id();
    my $header_color;
    if ($color) {
    $header_color = $color;
    }
    else {
    $header_color = $self->c1();
    }

    return '' unless ($number_of_days && $id);

    my $quick_list = '';
    my $original_start = timelocal(0, 0, 0, $self->{current_day}, $self->{current_month}, $self->{current_year});
    #my $original_start = $self->{current_datetime};
    my $end = 0;
    my $day_seconds = 60 * 60 * 24;
    my $events;
    my $name_link = '';
    my $week_day  = 'Today';

    # get all parents, in order to include tasks_____________________________
    # generate the list to be used in the query
    my $parent_list = join(',',@{$self->_get_parent_list()});

    for (my $start = $original_start; $start < ($original_start + ($day_seconds * $number_of_days)); $start += $day_seconds) {
        $end = $start + $day_seconds - 1;
        #print STDERR "\n\n**** start:$start end:$end ****\n\n";

        if ($start != $original_start) {
            $week_day = $self->format_datetime($start, 'a');
        }

        $quick_list .= "
            <table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">
              <tr bgcolor='$header_color'>
                <td>
                  <b>" . $self->make_calendar_link_title({
                            datetime=>$start,
                            text=>"<span class=\"smaller\">$week_day</span>",
                            view_mode=>VIEW_DAY})
            . "</b>
                </td>
                <td align='right'>
                  " . $self->make_add_event_link_title({datetime=>$start, text=>"<span class=\"smaller\">Add Event</span>"}) . "
                </td>
              </tr>
            </table>
            <table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">\n";

        my $where_clause = "parentiid in ($parent_list) and isa in ('".$self->EVENT_TYPE."','Task') and i1 >= $start and i1 <= $end"
                         . " AND ".Task->status_field()."<".Task->get_status_value('complete');

        $events = DBAccess->sqlSelectMany("distinct isa, iid, name, c1, i1, i2", "instance", $where_clause, "order by c1, i1, i2 desc, iid");
        while (my ($isa, $iid, $name, $timed, $start_time) = $events->fetchrow) {
            #print STDERR "\n\n** EVENT iid:$iid**\n\n";

            # determine the image to display with this link
            my $image;
            if ($timed ne 'yes') {
                my $image_name;
                if ($isa eq 'Task') {
                    $image_name = 'task_icon.gif';
                } else {
                    $image_name = 'event_icon.gif';
                }
                $image = "<img src=\"$PARAMS->{imageslocation}/obj/$image_name\" border=\"0\" alt=\"$isa\" />";
            } else {
                $image = '';
            }

            $name_link = $self->_make_event_link({
                id=>$iid, datetime=>$start_time, text=>"<span class=\"smaller\">$image$name</span>", isa=>$isa
                });
            if ($timed eq 'yes') { ## timed event
                $quick_list .= "<tr valign='middle'><td align='left' class='calendar'><div style=\"white-space: nowrap\"><span class=\"smaller\">" . $self->format_datetime($start_time, 'T') . ":&nbsp; </span></div></td><td>$name_link</td></tr>";
            } else { ## all day event
                $quick_list .= "<tr valign='middle'><td align='left' class='calendar' colspan='2' valign='top'>$name_link</td></tr>";
            }
        } ## end while
        $quick_list .= "<tr><td class='calendar'><i><span class=\"smaller\">No events scheduled.</span></i></td></tr>" if $events->rows == 0; #rows is ok because all the rows have been fetched
        $quick_list .= "</table>\n";
    } ## end for

    return $quick_list;
}

sub _get_events_quick_list {
    my $self = shift;
    my $color = shift;

    # if info not passed in as method arg, use default.
    my $number_of_events = $self->_get_num_events();    #  get the default info from the object
    if (!defined($number_of_events) || ($number_of_events eq '')) {
        $number_of_events = 3;
    }
#     print STDERR "in get_event_quick_list; got number of events: $number_of_events\n";
    my $id = $self->id();
    my $header_color;
    if ($color) {
    $header_color = $color;
    }
    else {
    $header_color = $self->c1();
    }

    return '' unless ($number_of_events && $id);

    my $quick_list = '';
    my $day_seconds = 60 * 60 * 24;
    my $events;
    my $name_link = '';
    my $week_day  = 'Today';

    # get all parents, in order to include tasks_____________________________
    # generate the list to be used in the query
    my $parent_list = join(',',@{$self->_get_parent_list()});
    
    # provide safety net for bug no.593
    # If parent list is null then a malformed SQL request
    # is executed causing a nasty looking error box and no
    # way to remove the AggCalendar causing it.
    if (length($parent_list) <= 0) { $parent_list = 1; }

        my $original_start = timelocal(0, 0, 0, $self->{current_day}, $self->{current_month}, $self->{current_year});
        my $where_clause = "parentiid in ($parent_list) and isa in ('".$self->EVENT_TYPE."','Task') and i1 >= $original_start"
                         . " AND ".Task->status_field()."<".Task->get_status_value('complete');

        $events = DBAccess->sqlSelectMany("distinct isa, iid, name, c1, i1, i2", "instance", $where_clause, "order by i1, i2 desc, iid");
        my $last_start_time = 0;
        my $count = 0;
        while (my ($isa, $iid, $name, $timed, $start_time) = $events->fetchrow) {
            last if ++$count > $number_of_events;
            #print STDERR "\n\n** EVENT iid:$iid**\n\n";

            if ( !Utility->same_day($start_time, $last_start_time) ) {
                $week_day = Utility->is_today($start_time) ? 'Today' : $self->format_datetime($start_time, 'a (m/e/y)');
                $quick_list .= "</table>\n" if $last_start_time > 0;
                $quick_list .= "\n
            <table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">
              <tr bgcolor='$header_color'>
                <td>
                  <b>" . $self->make_calendar_link_title({datetime=>$start_time, text=>"<span class=\"smaller\">$week_day</span>", view_mode=>VIEW_DAY}) . "</b>
                </td>
                <td align='right'>
                  " . $self->make_add_event_link_title({datetime=>$start_time, text=>"<span class=\"smaller\">Add ".$self->EVENT_TYPE_NAME."</span>"}) . "
                </td>
              </tr>
            </table>
            <table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">\n";
            }

            # determine the image to display with this link
            my $image;
            if ($timed ne 'yes') {
                my $image_name;
                if ($isa eq 'Task') {
                    $image_name = 'task_icon.gif';
                } else {
                    $image_name = 'event_icon.gif';
                }
                $image = "<img src=\"$PARAMS->{imageslocation}/obj/$image_name\" border=\"0\" alt=\"$isa\" />";
            } else {
                $image = '';
            }

            $name_link = $self->_make_event_link({
                id=>$iid, datetime=>$start_time, text=>"<span class=\"smaller\">$image$name</span>", isa=>$isa});
            if ($timed eq 'yes') { ## timed event
                $quick_list .= "<tr valign='middle'><td align='left' class='calendar'><div style=\"white-space: nowrap\"><span class=\"smaller\">" . $self->format_datetime($start_time, 'T') . ":&nbsp; </span></div></td><td>$name_link</td></tr>";
            } else { ## all day event
                $quick_list .= "<tr valign='middle'><td align='left' class='calendar' colspan='2' valign='top'>$name_link</td></tr>";
            }
            $last_start_time = $start_time;
        } ## end while
        $quick_list .= "</table>\n" if $events->rows;

    return $quick_list;
}

sub _get_parent_list {
    my $self = shift;
    my $id = $self->id();
    my @parent_list = ($id);
    # determine tasklist scope
    my $scope = $self->i1();
    if ($scope > $TASKLIST_SCOPE_NONE) {
        # add tasks to this list
        my $where_clause = "isa='TaskList'";
        if ($scope < $TASKLIST_SCOPE_GLOBAL) {
            # only tasks in my category
            $where_clause .= " AND parentiid=".$self->parent_id();
        }
        # filter completed tasks
        my $tasks_rs = DBAccess->sqlSelectMany('iid','instance', $where_clause);
        # check the perms for each task list
        while (my ($tasklist_id) = $tasks_rs->fetchrow()) {
            my $tasklist = TaskList->restore($tasklist_id);
            if ($tasklist->is_allowed_to_do('show', $USER)) {
                push @parent_list, $tasklist->id();
            }
        }
    }

    return \@parent_list;
}


## builds event listing display for day of month
sub get_day_event_listing {
    my $self = shift;

    my $table = '';
    my $count = 0;
    my $empty_blocks = 0;
    my $event;
    my $rowspan = 0;
    $DISPLAY->debug_msg(4,"get_day_event_listing() MAX COLS:" . $self->{MAXCOLS});

    use Data::Dumper;
    #$self->debug_msg(4, "Calendar\->get_day_event_listing has events:<br />\n".Dumper($self->{events}));

    $table .= "<table border=\"0\" width=\"100%\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"$COLOR1\"><tr><td>\n";
    $table .= "<table border=\"0\" cellspacing=\"1\" cellpadding=\"2\" width=\"100%\">\n";

    ## first print all-day events
    my $hour = -1;
    if (scalar(keys (%{$self->{events}->{$hour}}))) {
        $table .= "<tr><td width=\"2%\" align='right' bgcolor=\"$COLOR2\">&nbsp;</td>";
        #$table .= "<td colspan=\"" . ($self->{MAXCOLS}||1) .
        #          "\" bgcolor=\"$COLOR3\" valign='top'>\n";
        foreach my $event_id (sort{$self->{events}->{$hour}->{$a}->{order}<=>$self->{events}->{$hour}->{$b}->{order}}(keys %{$self->{events}->{$hour}})) {
            $event = $self->EVENT_TYPE->new($event_id);
            $table .= $event->get_panel() . "<font class=\"calendar\">" .
                      $self->{events}->{$hour}->{$event_id}->{desc} .
                      "</font><br />";
        }
        $table .= "</td></tr>\n";
    }

    ## print timed events
    foreach $hour ($self->{MINHOUR}..$self->{MAXHOUR}){
         $table .= "<tr><td width=\"2%\" align='right' bgcolor=\"$COLOR2\"><b><font class=\"calendar\">" . $self->make_add_event_link({datetime=>timelocal($self->{sec}, $self->{min}, $hour, $self->{day}, $self->{month}, $self->{year}), text=>$self->_format_hour($hour) . ":00", ignore_permissions=>1}) . "</font></b></td>";

         $count = 0;
         foreach my $event_id (sort{$self->{events}->{$hour}->{$a}->{order}<=>$self->{events}->{$hour}->{$b}->{order}}(keys %{$self->{events}->{$hour}})) {
             $event = $self->EVENT_TYPE->new($event_id);
             $rowspan = $self->{events}->{$hour}->{$event_id}->{block_hours};
             $table .= "<td colspan=" .
                       $self->_calculate_col_span($hour, ++$count, $rowspan) .
                       " rowspan=" . $rowspan .
                       " bgcolor=\"$COLOR3\" valign='top'>" .
                       $event->get_panel() .
                       "<div style=\"white-space: nowrap\"><b><font class=\"calendar\">" .
                       $self->{events}->{$hour}->{$event_id}->{start_time} .
                       "-" .
                       $self->{events}->{$hour}->{$event_id}->{end_time} .
                       "</b></div><br />" .
                       $self->{events}->{$hour}->{$event_id}->{desc} .
                       "</font></td>";
         }

         ## fill in empty blocks with non-breaking spaces
         my $colspan = 0;
         for (my $col = 0; $col <= $self->{MAXCOLS}; $col++) {
             ## if not last column and column not taken
             if ($col != $self->{MAXCOLS} && !$self->{grid}->{$hour}->{$col}) {
                 $colspan++; ## increase column span
                 next;
             }

             if ($colspan) {
                 $table .= "<td bgcolor=\"$COLOR2\" colspan=\"$colspan\">&nbsp;</td>";
             }
             $colspan = 0; ## reset
         }


         $table .= "</tr>\n";
    }

    $table .= "</table>\n";
    $table .= "</td></tr></table>\n";

    return $table;
}

## used to calculate the column span for an event in the
## event listing display for day of month
sub _calculate_col_span {
    my $self = shift;
    my $hour = shift;
    my $order = shift;
    my $rowspan = shift;

    my $colspan = 1; ## default
    my $last_event = 0;

    if ($self->{MAXCOLS} == $self->{hours}->{$hour}) {
        ## if the max cols used in event listing is
        ## equal to the hour count for $hour
        return $colspan;
    } elsif ($order <= $self->{hours}->{$hour}) {
        ## if event is not the last event for $hour
        $colspan = 1;
    } else {
        return $colspan;
    }

    my $count = 0;
    my $nexthour = 0;
    my $start_col = 0;
    EXPAND: for (my $col = 0; $col < $self->{MAXCOLS}; $col++) {
        ## start of new event or overlapping event
        if ($self->{grid}->{$hour}->{$col} == 1) {
            $count++;
            if ($count == $order) {
                $start_col = $col;
            }
        } elsif ($self->{grid}->{$hour}->{$col} == 2) {## expanded event
            next;
        } elsif ($count > $order) {## stop
            last EXPAND;
        } elsif ($count == $order) {## do more checks
            ## see if columns to left are taken
            for (my $col2 = $start_col + 1; $col2 < $col; $col2++) {
                last EXPAND if $self->{grid}->{$hour}->{$col2};
            }

            ## see if hours(rows) below column are taken
            for (my $row = 1; $row < $rowspan; $row++) {
                $nexthour = $hour + $row;
                ## hour below is taken so exit
                last EXPAND if $self->{grid}->{$nexthour}->{$col};
            }
            ## hours(rows) below not taken so occupy
            for (my $row2 = 0; $row2 < $rowspan; $row2++) {
                $nexthour = $hour + $row2;
                $self->{grid}->{$nexthour}->{$col} = 2;
            }
            $colspan++;        ## all checks met so increment
            $start_col = $col; ## set start to current column
        }

    } ## end of EXPAND

    return $colspan;
}

## creates link for navigating calendar
sub make_calendar_link {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);

    my $link_class  = $args->{class} || 'calendar';
    my $datetime    = $args->{datetime};
    my $text        = $args->{text};
    my $link_op     = $args->{op} || 'show';
    my $link_id     = $args->{id} || $self->id();
    my $link_isa    = $args->{isa} || '';
    my $view_mode   = $args->{view_mode} || '';

    my $calendar_id = $self->id();

    return qq~<a class="$link_class" href="index.pl?iid=$link_id&amp;isa=$link_isa&amp;op=$link_op&amp;dt=$datetime&amp;calendar_id=$calendar_id&amp;view_mode=$view_mode">$text</a>~;
}

sub make_calendar_link_title {
    my $self = shift;
    my $args = shift || {};

    $args->{link_class} = 'calendartitle';
    return $self->make_calendar_link($args);
}


## creates link for navigating calendar
sub _make_event_link {
    my $self = shift;
    my $args = shift || {};

    my $event_id    = $args->{id};
    my $datetime    = $args->{datetime};
    my $text        = $args->{text};
    my $isa         = $args->{isa} || $self->EVENT_TYPE;
    my $style_class = $args->{style_class} || 'calendar';

    return "<a class='$style_class' href=\"index.pl?iid=$event_id&amp;isa=$isa&amp;op=show&amp;dt=$datetime\">$text</a>";
}


## creates link for adding event
sub make_add_event_link {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);

    $DISPLAY->debug_msg(4, "make_add_event_link begins");

    my $datetime    = $args->{datetime};
    my $text        = $args->{text} || '';
    my $style_class = $args->{style_class} || 'calendar';
    my $attributes  = $args->{attributes} || '';
    my $always_show_text = $args->{ignore_permissions} || 0;

    my $op       = 'add_child';
    my $result;
    if ($self->is_allowed_to_do('modify', $USER)) {
        my $id   = $self->id;
        my $isa  = $self->is_a();

        my $href = "index.pl?iid=$id&amp;isa=$isa&amp;op=$op&amp;child_type=".$self->EVENT_TYPE."&amp;dt=$datetime" ;

        $result = "<a class='$style_class' href=\"".$href."\" $attributes>$text</a>";
    } else {
        ## user is not allowed to add event
        if ($always_show_text) { ## if set then show only text
            $result = $text;
        } else { ## otherwise return space
            $result = "&nbsp;";
        }
    }

    $DISPLAY->debug_msg(5, "make_add_event_link returns '$result'");
    return $result;
}

# aju -- need to dist. style class, otherwise the same
sub make_add_event_link_title {
    my $self     = shift;
    my $args = shift || {};

    my $datetime    = $args->{datetime};
    my $text        = $args->{text} || '';
    my $style_class = $args->{style_class} || 'calendar';
    my $attributes  = $args->{attributes} || '';
    my $always_show_text = $args->{ignore_permissions} || 0;

    my $op       = 'add_child';

    ## if user is not allowed to add event
    unless ($self->is_allowed_to_do('modify', $USER)) {
        if ($always_show_text) { ## if set then show only text
            return $text;
        } else { ## otherwise return space
            return "&nbsp;";
        }
    }

    my $id   = $self->id;
    my $isa  = $self->is_a();

    my $href = "index.pl?iid=$id&amp;isa=$isa&amp;op=$op&amp;child_type=".$self->EVENT_TYPE."&amp;dt=$datetime" ;

    return "<a class=\"$style_class\" href=\"".$href."\" $attributes>$text</a>";
}


## public method to re-initialize the calendar object based on given datetime
sub set_datetime {
    my $self     = shift;
    my $datetime = shift;

    return unless defined($datetime) && $datetime > 0;

    $self->_initialize_calendar($datetime);
}

## initializes calendar object
sub _initialize_calendar {
    my $self     = shift;
    my $datetime = shift;
    my $id       = $self->id;

    ## hash that stores number of events stored for each hour
    $self->{hours}   = undef;

    ## hash that stores event display
    ## value of:
    ##  1 = start of event
    ##  2 = occupied by event (non-starting place)
    $self->{grid}    = undef;


    ###
    ### initialize the current datetime the object will be using
    ###
    $self->{current_datetime} = time;

    ## current datetime broken down
    (
     $self->{current_sec},   #0-60
     $self->{current_min},   #0-59
     $self->{current_hour},  #0-23
     $self->{current_day},   #1-31
     $self->{current_month}, #0-11
     $self->{current_year},  #>1 since 1900
     $self->{current_wday},  #0-6
     $self->{current_yday},  #1-366
     $self->{current_isdst}  #0 or 1 daylight savings in effect
    ) = localtime($self->{current_datetime});


    ###
    ### initialize the datetime the object will be using

    if (defined $datetime)
    {
        # save it to the session
        $SESSION->set_value($self->generate_datetime_session_key(), $datetime);
        $self->apply_datetime($datetime);

        $DISPLAY->debug_msg(4, "$self $id set the datetime to ".localtime($datetime));
    }
    else
    {
        # retrieve it however I can
        $datetime = $self->determine_datetime();
    }

    ###
    ### initialize hour hash
    ###   look at _update_block_hour_hash() to see how set
    for my $i (0..23) {
        $self->{hours}->{$i} = 0;
    }


    ###
    ### load all events for datetime
    ###

    if (defined($id) && ($id > 0)) {

        # first clear all the events, in case this object has already been populated
        $self->{events} = {};

        my $start = timelocal(0, 0, 0, $self->{day}, $self->{month}, $self->{year});
        my $end = timelocal(59, 59, 23, $self->{day}, $self->{month}, $self->{year});
        #print STDERR "\n\n**** start:$start end:$end ****\n\n";

        ## order by showfrom then duration
        my $events = DBAccess->sqlSelectMany("distinct
                   iid,
                   name,
                   c1,
                   i1,
                   (i1 + i2),
                   i2",
                   , "instance", "parentiid=$id and isa='".$self->EVENT_TYPE."' and i1 >= $start and i1 <= $end", "order by i1, i2 desc, iid");

        my $shour = 0;
        my $order = 0;
        my $name_link = '';
        while (my($iid, $name, $timed, $start_time, $end_time) = $events->fetchrow) {
        #print STDERR "\n\n** NEW EVENT iid:$iid**\n\n";
            $name_link = $self->_make_event_link({id=>$iid, datetime=>$start_time, text=>$name});
            if ($timed eq 'yes') { ## timed event
                $shour = (localtime($start_time))[2];
                $shour+=0;
                $self->{events}->{$shour}->{$iid}->{start_time} =
                  $self->format_datetime($start_time, 'T');
                $self->{events}->{$shour}->{$iid}->{end_time} =
                  $self->format_datetime($end_time, 'T');
                $self->{events}->{$shour}->{$iid}->{desc} = $name_link;
                ## returns total number of hours that event occupies on
                ## calendar day display
                $self->{events}->{$shour}->{$iid}->{block_hours} = $self->_update_block_hour_hash($start_time, $end_time);
            } else { ## all day event
                $shour = -1;
                $self->{events}->{$shour}->{$iid}->{desc}  = $name_link;
            }
            $self->{events}->{$shour}->{$iid}->{order} = $order++;
        } ## end while
    } ## end if
}

## object hash that keeps track of the individual hours of the day
## that are used by all the day's events - also keeps a count for each
## hour (i.e. if hour 13 is used 5 times then $self->{hours}->{13} = 5)
sub _update_block_hour_hash {
    my $self = shift;
    my $start_time = shift;  # hh:mm  23:59
    my $end_time   = shift;  # hh:mm  23:59
    my $total_block_hours = 0;

    my ($start_hour, $start_min) = (localtime($start_time))[2,1];
    my ($end_hour, $end_min) = (localtime($end_time))[2,1];

    ## make sure they're integers
    $start_hour+=0;
    $end_hour+=0;
    $start_min+=0;
    $end_min+=0;
    ## for cases where passes over 12am
    $end_hour = $end_hour < $start_hour ? 23 : $end_hour;
    #print STDERR "\n\n*** start_hour:$start_hour start_min:$start_min end_hour:$end_hour end_min:$end_min ***\n\n";

    for (my $h = $start_hour; $h <= $end_hour; $h++) {
        last if $h > 23;

        if ($h == $start_hour || (($h * 100) < ($end_hour * 100 + $end_min))) {
            $self->{hours}->{$h}++;
            $self->{MAXCOLS} = $self->{hours}->{$h} if ($self->{MAXCOLS} < $self->{hours}->{$h});
            $self->{MINHOUR} = $h if ($h < $self->{MINHOUR});
            $self->{MAXHOUR} = $h if ($h > $self->{MAXHOUR});
            $total_block_hours++;
        }
    }


    my $col   = 0;
    my $next_hour = $start_hour + 1;
    ## find next unoccupied column of hour
    while ($self->{grid}->{$start_hour}->{$col}) {
            $col++;
    }
    ## assign column to event
    ## 1 indicates start of event
    $self->{grid}->{$start_hour}->{$col} = 1;
    ## 2 indicates that the event occupies space
    for (my $row = 1; $row < $total_block_hours; $row++) {
        $self->{grid}->{$next_hour}->{$col} = 2;
        $next_hour++;
    }

    return $total_block_hours;
}

## will replace the html produced by GizmoBuilder for given instance variable
sub _get_form_text {
      my $self = shift;
      my $field = shift;
      my $id = $self->id();

      return unless defined $field;

      my $value = $FORM{$field} || $self->{$field} || $COLOR1;
      my $space = '&nbsp' x 10;

      my $info = '';
      $info .= "
                <table border=\"0\" cellspacing=\"2\" cellpadding=\"2\">
                  <tr>
                    <td><input type=\"text\" name=\"$field\" size=\"13\" maxlength=\"10\" value=\"$value\"></td>
                    <td><input type=\"button\" onClick=\"javascript:setFieldValue('$field','$value');return false;\" value=\"Reset\"</td>
                    <td><input type=\"button\" onClick=\"javascript:setFieldValue('$field','$COLOR1');return false;\" value=\"Default\"</td>
                    <td><input type=\"button\" onClick=\"cp.select(document.fm.$field,'pick$field');return false;\" NAME=\"pick$field\" ID=\"pick$field\" value=\"Pick Color\"</td>
                  </tr>
                </table>";

      return $info;
}

## formats a unix timestamp
sub format_datetime {
    my $self  = shift;
    my $datetime  = shift;
    my $format = shift || 'M e, Y T';

    return Utility->format_unix_timestamp($datetime, $format);
}

# sets the form text for the tasklist selector; determines what
# tasks are displayed in this calendar's event list
sub _get_task_selector_text {
    my $self = shift;
    my $html = '';

    # determine the name of the calendar's parent
    #my $parent_id = $self->parent_id() || $FORM{iid};
    #my $parent_name = Gizmo->restore($parent_id)->name();

    # the above lines can cause a problem because this method is called from the constructor!
    my $parent_id = $self->parent_id();
    # only use FORM{iid} if I'm being created as a new child
    if (!defined $parent_id && defined($FORM{child_type}) && $FORM{child_type} eq ref($self)) {
        $parent_id = $FORM{iid};
    }
    my $parent_name;
    if (defined $parent_id) {
        my $parent_obj = Gizmo->restore($parent_id);
        if ($parent_obj) {
            $self->debug_msg(5, "Calendar got parent_obj '$parent_obj' ($parent_id, $parent_obj->{is_a})");
            $parent_name = $parent_obj->name();
        }
    }
    $parent_name ||= '';

    my @selections = ();
    # add the three scopes: none, local, global
    push @selections, $TASKLIST_SCOPE_NONE   . $self->get_value_label_separator() . "Don't show any tasks";
    push @selections, $TASKLIST_SCOPE_LOCAL  . $self->get_value_label_separator() . "Only show the tasks contained in the $parent_name Page";
    push @selections, $TASKLIST_SCOPE_GLOBAL . $self->get_value_label_separator() . "Show all the tasks on this website";

    $html = join(',', @selections);
    return $html;
}

sub _get_event_listing_type_text {
    my $self = shift;
    my $html = '';

    my @selections = ();
    # add the three scopes: none, local, global
    push @selections, EVENT_LISTING_TYPE_EVENTS . $self->get_value_label_separator() .$self->EVENT_TYPE_NAME()."s";
    push @selections, EVENT_LISTING_TYPE_DAYS . $self->get_value_label_separator() . "Days";
    $html = join(',', @selections);
    return $html;
}


sub _get_default_view_type {
    my $self = shift;
    my $html = '';

    my @selections = ();
    # add the three view types: mini, monthly, daily
    push @selections, VIEW_MINI     . $self->get_value_label_separator() . "Miniature view";
    push @selections, VIEW_MONTH    . $self->get_value_label_separator() . "Monthly view";
    push @selections, VIEW_DAY      . $self->get_value_label_separator() . "Daily view";
    $html = join(',', @selections);
    return $html;
}


## formats hour (returns 1-12)
sub _format_hour {
    my $self  = shift;
    my $hour  = shift;

    return Utility->_format_hour($hour);
}

## displays days of month navigation
sub _get_days_of_month_display {
    my $self  = shift;
    my $args  = shift || {};

    my $container_properties    = $args->{container_properties} || '';
    my $cell_properties         = $args->{cell_properties} || '';
    my $cell_day_properties     = $args->{cell_day_properties} || '';
    my $cell_date_container_properties
                                = $args->{cell_date_container_properties};
    my $cell_event_container_properties
                                = $args->{cell_event_container_properties};

    my $expanded                = $args->{expanded};

    my $table = '';
    my $id    = $self->id;


    my $cellspacing = my $cellpadding = $expanded ? 0 : 1;

    $table .= "
<table border=\"0\" cellspacing=\"$cellspacing\" cellpadding=\"$cellpadding\" align='center' $container_properties>
<tr align='right'>";

    # write out the days
    my $day_of_week_display_length = $expanded ? 3 : 2;
    for (my $i=0; $i<7; $i++)
    {
        my $day_of_week = substr(Metadot::SystemApp::Util::DAYS_OF_WEEK()->{$i}, 0, $day_of_week_display_length);
        $table .= qq~<td $cell_day_properties width=\"15%\" align="left">$day_of_week</td>~;
    }

    $table .= "</tr>";

    my ($start_datetime, $end_datetime) = $self->_get_datetimes_of_month_display();

    my $month_events = ();
    ## mark each day of calendar that an event occurs
    if ($id > 0) {
        my @start_date_array = (localtime($start_datetime))[3,4,5];
        my @end_date_array = (localtime($end_datetime))[3,4,5];
        my $start = timelocal(0, 0, 0, @start_date_array);
        my $end = timelocal(59, 59, 23, @end_date_array);
        #print STDERR "\n\n**** start:$start end:$end ****\n\n";

        my $parent_list = join(',',@{$self->_get_parent_list()});
        # provide safety net for bug no.593
        # If parent list is null then a malformed SQL request
        # is executed causing a nasty looking error box and no
        # way to remove the AggCalendar causing it.
        if (length($parent_list) <= 0) { $parent_list = 1; }

        my $events = DBAccess->sqlSelectMany("distinct iid, i1", "instance", "parentiid in ($parent_list) and isa='".$self->EVENT_TYPE."' and i1 >= $start and i1 <= $end", "order by i1");

        my $month =0;
        my $day   =0;
        while (my($iid, $start_time) = $events->fetchrow) {
            ($day, $month) = (localtime($start_time))[3,4];
            my $existing_events = $month_events->{$month}->{$day} || [];
            push @$existing_events, $iid;
            $month_events->{$month}->{$day} = $existing_events;
        }
    }

    my $day_seconds = 60 * 60 * 24; ## number of seconds in day
    my $datetime = $start_datetime;
    my $datetime_day = 0;
    my $datetime_month = 0;
    my $today_day=0;
    my $today_month=0;
    while ($datetime <= $end_datetime) {

        $table .= "<tr align='right' valign='top'>\n";

        for my $col (0..6) {
            ($datetime_day, $datetime_month) = (localtime($datetime))[3, 4];

            my $daily_events = $month_events->{$datetime_month}->{$datetime_day};
            my $day_link = $self->make_calendar_link({datetime=>$datetime, text=>$datetime_day, view_mode=>VIEW_DAY});

            my $cell_within_month = ($datetime_month == $self->{month});
            my $cell_is_current_day = ($datetime_month == $self->{current_month} && $datetime_day == $self->{current_day});

            my $cell_bg_color;
            if ($cell_is_current_day)
            {
                $cell_bg_color = " bgcolor=\"$COLOR6\"";
            }
            elsif (!$cell_within_month)
            {
                $cell_bg_color = " bgcolor=\"$COLOR2\"";
            }
            else
            {
                $cell_bg_color = '';
            }


            $table .= "<td $cell_properties $cell_bg_color>\n";

            if ($expanded)
            {
                # display the expanded view
                my $day_cell = qq~<table width="100%" height="100%" cellspacing="0" cellpadding="3">\n~;

                # day row
                $day_cell .= qq~<tr $cell_date_container_properties><td align="left">$day_link</td>\n<td align="right" width="100%">~;
                if ($SESSION->admin_mode())
                {
                    $day_cell .= $self->make_add_event_link({
                        datetime=>$datetime, text=>"[add]",
                        attributes=>'style="font-weight: normal; font-size: smaller"'});
                }
                $day_cell .= qq~</td></tr>\n~;

                # event row
                $day_cell .= qq~<tr $cell_event_container_properties><td align="left" colspan="2" height="100%">\n~;
                    if ($daily_events)
                    {
                        $day_cell .= qq~<table border="0" cellspacing="0" cellpadding="1" width="100%">\n~;
                        my $event_count = 0;
                        foreach my $event_id (@$daily_events)
                        {
                            my $event = Metadot::PortalFactory->get_gizmo_object($event_id);
                            if ($event_count++ < 3)
                            {
                                my $event_time = $self->_make_event_link({
                                        id=>$event_id, datetime=>$datetime,
                                        text=>$self->format_datetime($event->date(), 'T'),
                                        isa=>$event->is_a()
                                    });

                                my $name_text = $event->name();
                                my $max_length = 15;
                                if (length($name_text) > $max_length)
                                {
                                    $name_text = substr($name_text, 0, ($max_length-3)).'...';
                                }

                                my $event_link = $self->_make_event_link({
                                        id=>$event_id, datetime=>$datetime, text=>$name_text, isa=>$event->is_a()
                                    });
                                $day_cell .= qq~<tr valign="baseline"><td nowrap="nowrap">$event_time</td><td>&nbsp;</td><td width="100%">$event_link</td></tr>\n~;
                            }
                            else
                            {
                                # stop displaying events; link to day
                                my $day_link = $self->make_calendar_link({datetime=>$datetime, text=>'More...', view_mode=>VIEW_DAY});
                                $day_cell .= qq~<tr><td colspan="2">$day_link</td></tr>\n~;
                                last;
                            }
                        }
                        $day_cell .= "</table>\n";
                    }
                    else
                    {
                        $day_cell .= "&nbsp;";
                    }

                $day_cell .= "</td></tr>\n";

                $day_cell .= "</table>\n";
                $table .= $day_cell;
            }
            else
            {
                # display the miniature view
                ## is selected day
                if (($datetime_day == $self->{current_day}) &&
                    ($datetime_month == $self->{current_month})){
                    $table .= $day_link;
                #} elsif ($datetime_day == $self->{day} && $datetime_month == $self->{month}) {
                #    $table .= "<font class=\"calendarselected\"><b>"
        #           . "$datetime_day</b></font>";
                } else {
                    if ($daily_events)
                    {
                        $day_link = "<b>$day_link</b>";
                    }
                    $table .= $day_link;
                }

                $table .= "</td>\n";
            }

            $datetime += $day_seconds; ## add full day's seconds
        }
        $table .= "</tr>\n";
    }

    $table .= "
</table>
";

   return $table;
}

## will return datetimes for first day and last day for the
## calendar month navagation display (note: start and end times
## may be different months)
sub _get_datetimes_of_month_display {
    my $self = shift;

    my $day_seconds = 60 * 60 * 24;

    ###
    ### Calculate first Sunday
    ###
    ## datetime for first day of month
    my $first_datetime = timelocal($self->{sec}, $self->{min}, $self->{hour}, 1, $self->{month}, $self->{year});
    my ($fsec, $fmin, $fhour, $fday, $fmonth, $fyear, $fwday, $fyday, $fisdst) = localtime($first_datetime);
    ### if first day not Sunday
    if ($fwday) {
        ## subtract to get to Sunday
        $first_datetime = $first_datetime - ($fwday * $day_seconds);
    }

    ###
    ### Calculate last Saturday
    ###
    my $next_month = $self->{month} + 1;
    my $next_year  = $self->{year};

    if ($next_month > 11) {
        $next_month =  0;
        $next_year++;
    }

    ## datetime for first day of next month
    my $last_datetime = timelocal($self->{sec}, $self->{min}, $self->{hour}, 1, $next_month, $next_year);
    my ($lsec, $lmin, $lhour, $lday, $lmonth, $lyear, $lwday, $lyday, $lisdst) = localtime($last_datetime);
    ### if first day of next month Sunday
    if ($lwday == 0) {
        $last_datetime = $last_datetime - $day_seconds;
    } else {
        ## add to get to Sunday
        $last_datetime = $last_datetime + ((6 - $lwday) * $day_seconds);
    }

    return ($first_datetime, $last_datetime);
}

## returns datetime for first day of previous month
sub _get_previous_month_datetime {
    my $self = shift;

    my $prev_month = $self->{month} - 1;
    my $prev_year  = $self->{year};

    if ($prev_month < 0) {
        $prev_month = 11;
        $prev_year--;
    }

    return timelocal($self->{sec}, $self->{min}, $self->{hour}, 1, $prev_month, $prev_year);
}

## returns datetime for first day of next month
sub _get_next_month_datetime {
    my $self = shift;

    my $next_month = $self->{month} + 1;
    my $next_year  = $self->{year};

    if ($next_month > 11) {
        $next_month =  0;
        $next_year++;
    }

    return timelocal($self->{sec}, $self->{min}, $self->{hour}, 1, $next_month, $next_year);
}

## returns datetime for previous day or -24 hours
sub get_previous_day_datetime {
    my $self        = shift;
    my $datetime    = shift || $self->{current_datetime};

    my $day_seconds = 60 * 60 * 24;
    return $datetime - $day_seconds;
}

## returns datetime for next day or +24 hours
sub get_next_day_datetime {
    my $self        = shift;
    my $datetime    = shift || $self->{current_datetime};

    my $day_seconds = 60 * 60 * 24;
    return $datetime + $day_seconds;
}

sub _get_num_days {
    my $self = shift;
    my $num = $self->get_c4() || 0;
    if ($num =~ /\D/) {     # quick check if chars not numbers-- just return 0 in that case
    return 0;
    } else {
#     print STDERR "in _get_num_days, returning: ", $self->get_c4(), "\n";
        return $num;
    }
}

sub _set_num_days {
    my ($self, $num) = @_;
    $self->set_c4($num);
}

sub _set_listing_type {
    my ($self, $type) = @_;
    $self->set_c5($type);
}

sub _get_num_events {
    my $self = shift;
    my $num = $self->get_c4();
    if ($num =~ /\D/) {     # quick check if chars not numbers-- just return 0 in that case
    return 0;
    }
#     print STDERR "in _get_num_events, returning: ", $self->get_c4(), "\n";
    return $self->get_c4();
}


## overrides parent's delete function in order to delete events
sub delete{
  my $self = shift;
  $self->debug_msg(3, "Calendar::delete()");
  my $id = $self->id;

  my ($childs)=DBAccess->sqlSelectMany("iid", "instance", "parentiid=$id and isa='".$self->EVENT_TYPE."'");
  while(my ($iid)=$childs->fetchrow()) {
    my $e = $self->EVENT_TYPE->new($iid);
    $e->delete();
  }

  # delete from instance
  $self->SUPER::delete();
  return 1;
}

1;

__END__

=head1 NAME

Gizmo::Calendar - can either create a new Calendar (returing a pointer to the new object) or control the way existing ones are displayed (returning the formatted HTML).


=head1 SYNOPSIS

    use Gizmo::Calendar;

    $calendar = Calendar->new ($id);
    $html = $calendar->show ();
    $html = $calendar->show_as_bullet ();
    $html = $calendar->show_as_dash ();
    $html = $calendar->show_edit_form ();
    $html = $calendar->show_in_column ();
    $html = $calendar->get_month_navagtion ();
    $html = $calendar->get_event_quick_list ();
    $text = $calendar->format_datetime ();
    $calendar->set_datetime ();
    $version = Calendar->get_version();

=head1 DESCRIPTION

Calendar are used to keep track of day's events.


=head1 Methods

Calling the new function instantiates a new calendar or retrive its persistent info from the backend database.

    $calendar->set_datetime () will set the datetime of the calendar
        object and initialize based off this datetime

    $calendar->format_datetime () returns formatted date



This class method retrieves the version oh
   ->get_version();

The remaining functions control how an Calendar is displayed:

    $calendar->show() displays a calendar's events for a day.
    $calendar->show_as_bullet() displays the calendar in a bulleted list.
    $calendar->show_as_dash() same as show_as_bullet()
    $calendar->show_edit_form() displays form for adding/editing calendar
    $calendar->get_month_navagtion() displays month navagation
    $calendar->get_event_quick_list() displays event summary list

=cut
