#!/usr/bin/perl
#
# Test file for Calendar.pm
# $Id: Calendar.t,v 1.3.2.1 2005/09/02 17:24:35 cory Exp $
#

use strict;
use Metadot::Testing::Mechanizer; # wraps LWP::UserAgentand HTML::Form

use Test::More tests=>15;

#load Metadot test suite and create a new portal
use SmokeMetadot;


#first we create a new page

#################################################
# Test: first we create a new page where will 
# put the stuff we had
#################################################
my $page = Category->new({
			 uid => 0,
			 parent_id => -0,
			 name => "smoke",
			 description => "you can safely delete this page",
			 keywords => '',
			 display_look => 1,
			 created_by => -1,
			});

isa_ok( $page, "Category") ;

my $page_id = $page->add();
cmp_ok($page_id, '>', 0, "page created");


#################################################
# Test: Create a new instance of the calendar
#################################################
my $cal = Calendar->new();
isa_ok( $cal, "Calendar") ;

#################################################
# Test: Set calendar name and description
#################################################
$cal->set_name("Vacation calendar");
$cal->set_description("Vacation description");

my $name = $cal->name();
ok( $name eq 'Vacation calendar', 'testing set_name() and get_name()');

my $description = $cal->description();
ok( $description eq 'Vacation description', 'testing set_description() and get_description()');

#################################################
# Test: Adding a calendar (saving in DB)
#################################################
$cal->set_uid(-1);
$cal->set_parent_id($page_id);
my $id = $cal->add();

# Load the newly saved calendar
my $newcal = Calendar->restore($id);
my $id_of_new_cal = $newcal->id();
ok( $id == $id_of_new_cal, 'testing add()' );

#################################################
# Test: Deleting an instance of the calendar
#################################################
$cal->delete();

#testing if really deleted
#we try to load the one we just deleeted and we should get an error

my $newcal2 = Calendar->restore($id);
$id = defined($newcal2) ? $newcal2->id() : undef;

ok( ((!defined($id)) or ($id=='')), 'testing delete()');

#################################################
# Test: testing if the calendar appears on the 
# front page
#################################################

# adding a calendar
my $tcal = Calendar->new();
isa_ok( $tcal, "Calendar") ;

my $calendar_name = "Vacation-calendar-for-testing";
$tcal->set_name("$calendar_name");
$tcal->set_uid(-1);
$tcal->set_parent_id($page_id); 
$tcal->add();
ok($tcal > 0, "test calendar added");

# fetching the webpage
my $m = Metadot::Testing::Mechanizer->new();
isa_ok($m, "Metadot::Testing::Mechanizer");

$m->get_page( iid=>$page_id ); #fetch the page
ok( ($m->status == 200), "testing fetch page iid $page_id");
like($m->content(), qr/$calendar_name/, "Is cal on  page?");

#################################################
# Test: testing if the calendar diappears 
# from the front page if we delete it
#################################################

$tcal->delete(); #getting rid of the calendar
$m->get_page(iid=>$page_id); #fetching the new page
ok(($m->status == 200), "testing fetch page again");

unlike($m->content(), qr/$calendar_name/, "deleting cal from front page");

#################################################
# Delete the page we created at first
#################################################

$page->delete();
my $exists = $page->exists($page->id());
ok($exists == 0, "page deleted");

#################################################
# That's it!!!!
#################################################
