###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Event;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT);
use Metadot::Controllable;

use Carp;
use Gizmo;
use Time::Local;
use Date::Calc qw(check_date);

use DBAccess;

use CGI qw/escape unescape escapeHTML/;
use vars qw(@ISA);
@ISA=qw(AbstractEvent Metadot::Controllable);
my $version = "4.5";

sub get_version {
    return $version;
}

sub customize_permissions {
    my $self = shift;
    my $AC   = shift;

    $self->SUPER::customize_permissions($AC);

    $AC->remove_op('DISP', 'create_shortcut');
    $AC->remove_op('DEL', 'cut');
}


use constant EVENT_TYPES => {
                     1 => 'Other',
                     2 => 'Anniversary',
                     3 => 'Appointment',
                     4 => 'Bill Payment',
                     5 => 'Birthday',
                     6 => 'Breakfast',
                     7 => 'Call',
                     8 => 'Chat',
                     9 => 'Class',
                    10 => 'Club Event',
                    11 => 'Concert',
                    12 => 'Dinner',
                    13 => 'Graduation',
                    14 => 'Happy Hour',
                    15 => 'Holiday',
                    16 => 'Interview',
                    17 => 'Lunch',
                    18 => 'Meeting',
                    19 => 'Movie',
                    20 => 'Net Event',
                    21 => 'Party',
                    22 => 'Performance',
                    23 => 'Reunion',
                    24 => 'Sports Event',
                    25 => 'Travel',
                    26 => 'TV Show',
                    27 => 'Vacation',
                    28 => 'Wedding'
                  };


# returns the list of event types
sub get_event_types {
    return EVENT_TYPES();
}

my $COLOR1 = '#CCCCCC';
my $COLOR2 = '#EEEEEE';
my $COLOR3 = '#FFFFFF';



#####################################################
######                                        #######
######            FRONT-END METHODS           #######
######                                        #######
#####################################################

sub _input_form_fields {
    my $self = shift;
    my $errormsg = $self->SUPER::_input_form_fields();

    $self->i3($FORM{type});
    $self->t1($FORM{location});
    $self->t2($FORM{address});
    $self->t3($FORM{city_state_zip});
    $self->c2($FORM{phone});

    return $errormsg;
}

#####################################################
######                                        #######
######            BACK-END METHODS            #######
######                                        #######
#####################################################

#####################################################
###         WEB DISPLAY BACK-END METHODS          ###
#####################################################

sub show {
  my $self  = shift;
  $self->debug_msg(4, "Event::show()");

  #my $s = "<p /><b>Title:</b> " . $self->name();
  my $s = "<br /><br /><table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"$COLOR1\" width=\"100%\"><tr><td>\n";
  $s .= "<table border=\"0\" cellspacing=\"1\" cellpadding=\"5\" width=\"100%\">\n";
  $s .= "<tr><th colspan='2' bgcolor=\"$COLOR2\">" . $self->name(). "</th></td></tr>";
  $s .= "<tr><td bgcolor=\"$COLOR2\"><b>Date";
  if ($self->timed() eq 'yes') {
      $s .= " and Time:</b></td><td bgcolor=\"$COLOR2\">";
      $s .= Calendar->format_datetime($self->date(), 'M e, Y T');
      $s .= "-" . Calendar->format_datetime($self->date() + $self->duration(), 'T');
      $s .= "</td></tr>\n";
  } else {
      $s .= ":</b></td><td bgcolor=\"$COLOR2\">";
      $s .= Calendar->format_datetime($self->date(), 'M e, Y') .
            "&nbsp;&nbsp;<i>(All Day ".$self->get_gizmo_name().")</i>";
      $s .= "</td></tr>\n";
  }
  my $user = Metadot::UserFactory->get_user( $self->uid() );

  $s .= "<tr><td bgcolor=\"$COLOR2\"><b>Type:</b></td><td bgcolor=\"$COLOR2\">" . $self->get_event_types()->{$self->i3()} . "</td></tr>\n";
  $s .= "<tr><td bgcolor=\"$COLOR2\"><b>".$self->get_description_label()."</b></td><td bgcolor=\"$COLOR2\">" . ($self->description() || '&nbsp') . "</td></tr>\n";
  $s .= "<tr><td bgcolor=\"$COLOR2\"><b>Author:</b></td><td bgcolor=\"$COLOR2\">" . ($user->fullname() || $user->nickname() || '&nbsp') . "</td></tr>\n";
  $s .= "<tr><td bgcolor=\"$COLOR2\"><b>".$self->get_gizmo_name()." Location:</b></td><td bgcolor=\"$COLOR2\">" . ($self->t1() || '&nbsp') . "</td></tr>\n";
  $s .= "<tr><td bgcolor=\"$COLOR2\"><b>Street:</b></td><td bgcolor=\"$COLOR2\">" . ($self->t2() || '&nbsp'). "</td></tr>\n";
  $s .= "<tr><td bgcolor=\"$COLOR2\"><b>City, State, Zip:</b></td><td bgcolor=\"$COLOR2\">" . ($self->t3() || '&nbsp') . "</td></tr>\n";
  $s .= "<tr><td bgcolor=\"$COLOR2\"><b>Phone:</b></td><td bgcolor=\"$COLOR2\">" . ($self->c2() || '&nbsp') . "</td></tr>\n";
  $s .= "</table>\n</table>\n";

  $s .= "<p /><a href=\"index.pl?isa=Calendar&amp;op=show&amp;iid=" . $self->parent_id() . "&amp;dt=" . $self->date() . "\">Back to Calendar</a>\n";


  return $s;
}

##############

sub show_edit_form {
  my $self  = shift;
  my $title = shift;
  $self->debug_msg(4, "Event::show_edit_form($title)");

  my $id = $self->id();

  my $type = $self->i3() || $FORM{type} || 18;
  my $location = escapeHTML( $self->t1() );
  my $address = escapeHTML( $self->t2() );
  my $city_state_zip = escapeHTML( $self->t3() );
  my $phone = escapeHTML( $self->c2() );


  ###
  ### BUILD DROP-DOWNS
  ###

  ## event type dropdown
  my $et_dropdown = "<select name = \"type\">\n";
  foreach my $key (sort{$a <=> $b}keys(%{$self->get_event_types()})) {
      $et_dropdown .= "    <option value=\"$key\"" .
                      ($key == $type ? " selected" : "") . ">"
                      . $self->get_event_types()->{$key} . "\n";
  }
  $et_dropdown .= "</select>";
  ##

  my $fields = $self->_generate_form_fields();

  ###
  ### BUILD HTML
  ###

  my $description_label = $self->get_description_label();
  my $s = "<h1>$title</h1>

          <p /><b>Title</b> (*)<br />
          $fields->{name}

          <p /><b>Date and Time</b> (*)<br />
          $fields->{month}
          $fields->{day}
          $fields->{year}

          <br />
          <table border=\"0\" cellspacing=\"1\" cellpadding=\"1\">
            <tr>
              <td>$fields->{timed_no} All Day ".$self->get_gizmo_name()." </td>
            </tr>
            <tr>
              <td>
                 $fields->{timed_yes} Timed ".$self->get_gizmo_name().": $fields->{timed_hour}
                 $fields->{timed_min} minutes
             </td>
            </tr>
            <tr>
              <td align='right'>
                 Duration: $fields->{duration_hour} hours
                 $fields->{duration_min} minutes
             </td>
            </tr>
          </table>

          <p /><b>Type</b><br />
          $et_dropdown

          <p /><b>$description_label</b><br />
          $fields->{description}

          <p /><b>".$self->get_gizmo_name()." Location</b> <br />
          <input type='text' name='location' size='55' maxlength='255' value=\"$location\">

          <p /><b>Street</b> <br />
          <input type='text' name='address' size='55' maxlength='255' value=\"$address\">

          <p /><b>City, State, Zip</b> <br />
          <input type='text' name='city_state_zip' size='55' maxlength='255' value=\"$city_state_zip\">

          <p /><b>Phone</b> <br />
          <input type='text' name='phone' size='55' maxlength='255' value=\"$phone\">

          <br />
          <br /><br /><br /><i>(*) Mandatory field(s)</i><br />\n";

  return $s;
}

sub get_gizmo_name {
    return 'Event';
}

sub show_properties_button {
    return '';
}


# allow this to be overridden
sub get_description_label {
    return 'Notes';
}


#####################################################
###                   THE END                     ###
#####################################################


1;


__END__

=head1 NAME

Gizmo::Calendar::Event -

=head1 SYNOPSIS

    use Gizmo::Calendar::Event;

    $event = Event->new ($id);

    $AC = $event->get_AC ();
    $event->www_show ();
    $event->www_delete ();
    $event->www_delete_ok ();
    $event->process_edit_form ();
    $event->www_save ();
    $event->show ();
    $html = $event->show_edit_form ($title);
    $version = Event->get_version();

=head1 DESCRIPTION

=cut


