###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Category;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $CACHE $DISPLAY $ACCESS_BROKER %REQUEST_VARS);

use Carp;
use Gizmo;
use Channel;

use DBAccess;
use Gizmo::Poll;
use Gizmo::Calendar;
use Gizmo::Category::Clipboard;
use Gizmo::LayoutArea;
use Data::Dumper;
use ErrorHandling;
use Utility;

use Metadot::Utils::Theme;
use Metadot::TemplateMgr;
use HTML::Components;
use HTML::Decorator::BulletDecorator;
use HTML::Decorator::TableRowDecorator;
use HTML::Decorator::NullDecorator;

use Metadot::GizmoContainer;
use Metadot::Navigable;
use Metadot::Printable;
use Metadot::SkinManager;
use Metadot::Invitable;
use Metadot::Controllable;
use Metadot::Subscribable;
use Metadot::UpdateNotification::UpdateNotifier;
use Gizmo::NewsPaperView;

use CGI qw/escapeHTML unescapeHTML/;
use vars qw(@ISA $AC);
# aju 08-21-01 make Category a GizmoBuilder  -- needs get_iid method.
@ISA=qw(
    Metadot::GizmoContainer
    Metadot::Controllable
    Metadot::Navigable
    Metadot::Invitable
    Metadot::Printable
    Metadot::Subscribable
    Metadot::UpdateNotification::UpdateNotifier
);

my $version = "4.5";
my $is_visible_in_add_child_lists = 0;
my @EXCLUDE_FROM_CONTENTS = qw( Newsitem Poll ); # Metadot::Invitable was erroneously placed here

sub get_version {
    return $version;
}

sub get_gizmo_name {
   my $self = shift;
   return "Page";
}


sub customize_permissions {
    my $self = shift;
    my $ac   = shift;

    $self->SUPER::customize_permissions($ac);

    $ac->add_op('DISP', 'show_news');
    $ac->add_op('DISP', 'show_image');
    $ac->add_op('DISP', 'show_printer_friendly');

    $ac->add_op('MOD', 'clipboard_delete');

    Metadot::Subscribable->customize_permissions($ac);
    Metadot::UpdateNotification::UpdateNotifier->customize_permissions($ac);
    Metadot::Invitable->customize_permissions($ac);

    return $ac;
}


sub get_layout_definitions {
    my $class = shift;


    my $layouts = [
                                {
                                  name => 'layout1',
                                  areas =>  1,
                                  small_image => '
<table border="1" cellspacing="0" cellpadding="0">
  <tr>
    <td>' . Utility->get_spacer(60,60) . '</td>
  </tr>
</table>
',
                                },
                                {
                                  name => 'layout2',
                                  areas =>  2,
                                  small_image => '
<table border="1" cellspacing="0" cellpadding="0">
  <tr>
    <td>' . Utility->get_spacer(15,60) . '</td>
    <td>' . Utility->get_spacer(45,60) . '</td>
  </tr>
</table>
',
                                },
                                {
                                  name => 'layout3',
                                  areas =>  2,
                                  small_image => '
<table border="1" cellspacing="0" cellpadding="0">
  <tr>
    <td>' . Utility->get_spacer(45,60) . '</td>
    <td>' . Utility->get_spacer(15,60) . '</td>
  </tr>
</table>
',
                                },
                                {
                                  name => 'layout4',
                                  areas =>  3,
                                  small_image => '
<table border="1" cellspacing="0" cellpadding="0">
  <tr>
    <td>' . Utility->get_spacer(15,60) . '</td>
    <td>' . Utility->get_spacer(30,60) . '</td>
    <td>' . Utility->get_spacer(15,60) . '</td>
  </tr>
</table>
',
                                },
                                {
                                  name => 'layout5',
                                  areas =>  3,
                                  areas =>  3,
                                  small_image => '
<table border="1" cellspacing="0" cellpadding="0">
  <tr>
    <td>' . Utility->get_spacer(20,60) . '</td>
    <td>' . Utility->get_spacer(20,60) . '</td>
    <td>' . Utility->get_spacer(20,60) . '</td>
  </tr>
</table>
',
                                },
                                {
                                  name => 'layout6',
                                  areas =>  4,
                                  small_image => '
<table border="1" cellspacing="0" cellpadding="0">
  <tr>
    <td colspan="2">' . Utility->get_spacer(40,30) . '</td>
    <td rowspan="2">' . Utility->get_spacer(20,60) . '</td>
  </tr>
  <tr>
    <td>' . Utility->get_spacer(20,30) . '</td>
    <td>' . Utility->get_spacer(20,30) . '</td>
  </tr>
</table>
',
                                },
                                {
                                  name => 'layout7',
                                  areas =>  5,
                                  small_image => '
<table border="1" cellspacing="0" cellpadding="0">
  <tr>
    <td colspan="3">' . Utility->get_spacer(60,20) . '</td>
  </tr>
  <tr>
    <td colspan="2">' . Utility->get_spacer(40,20) . '</td>
    <td rowspan="2">' . Utility->get_spacer(20,40) . '</td>
  </tr>
  <tr>
    <td>' . Utility->get_spacer(20,20) . '</td>
    <td>' . Utility->get_spacer(20,20) . '</td>
  </tr>
</table>
',
                                },
                          ];

    unless (MetadotConfig->get_param_value('enable_complex_layouts')) {
        pop(@$layouts);
        pop(@$layouts);
    }

    return $layouts;
}

sub www_show_printer_friendly {
    my $self = shift;

    $DISPLAY->print_http_header();

    print $self->get_printer_friendly_content();
}

sub get_printer_friendly_content {
    my ($self, %params) = @_;

    my $id = $self->id();

    my $location_url = $PARAMS->{basename} . "/index.pl?iid=$id";

    my $name = $self->get_name();
    my $abs_name = $self->path_qualified_name();

    my ( $news_col, $poll_col, $cal_col, $content_col, $style_sheet );
    my $img_loc_rel = $PARAMS->{imageslocation} ;
    my $basename = $PARAMS->{basename};
    my $uri = $PARAMS->{uri_scheme};
    my $hostname = $PARAMS->{hostname};
    if ($params{absolute_path}) {
        $PARAMS->{imageslocation} = $uri . "://" . $hostname . $img_loc_rel ;
    }

    if( $SESSION->admin_mode() ) {

    $SESSION->edit_off();

    ( $news_col, $poll_col, $cal_col, $content_col, $style_sheet ) =
      ( $self->show_news_column(),
        $self->show_poll_column(),
        $self->show_calendar_column('', $id, 'show'),
        $self->show_simple(),
        $DISPLAY->show_style_sheet() );

    $SESSION->edit_on();

    } else {

    ( $news_col, $poll_col, $cal_col, $content_col, $style_sheet ) =
      ( $self->show_news_column(),
        $self->show_poll_column(),
        $self->show_calendar_column('', $id, 'show'),
        $self->show_simple(),
        $DISPLAY->show_style_sheet() );
    }

    my $right_col = "
<!-- News column -->
<td align='right' valign='top' width=\"155\">
$cal_col

$news_col

$poll_col
</td>
<!-- End of News column -->
";
    my $now = gmtime();
    my $out = <<"END_TOK";
<html>
$style_sheet
<body bgcolor="white">
<!-- Outer Table -->
<table border="0" cellspacing="10" cellpadding="10" width="600">
<tr><td>

<!-- Category Table -->
<table border="0" cellpadding="0" cellspacing="10" width='100%'>
  <tr>
    <td colspan='2'>
      <h1>$name</h1>
      <h3>$abs_name<br />$now</h3>
      $location_url
    </td>
  </tr>
<tr valign='top'>
<!-- Category column -->
<td valign='top' width="100%">
$content_col
</td>
<!-- End of Category column -->
$right_col
</tr></table>
<!-- End of Category Table -->

</td></tr></table>
<!-- End of Outer Table -->
</body></html>
END_TOK

    if ($params{absolute_path}) {
             # changing <a href > and <img src > to absolute:
             $out =~ s/href ?= ?(['"])\/?/href=$1$basename\//gi ;
             $out =~ s/img src ?= ?(['"])\.\.\/?/img src=$1$uri:\/\/$hostname\//gi ;
             $out =~ s/img src ?= ?(['"])index/img src=$1$basename\/index/gi ;
             $PARAMS->{imageslocation} = $img_loc_rel; ## reset to original
    }

    return $out;
}


#########################################
# WWW FUNCTIONS

sub www_show {
  my $self = shift;
  $self->debug_msg(3, "Category::www_show()");

  #CKIM-REDIRECT: if this is a redirect category, don't bother with anything else
  my $content;
  if ($self->id() && $self->get_redirect_url()) {
      my $redirect_url = $self->get_redirect_url();

      unless ($SESSION->admin_mode() && $self->is_allowed_to_do('modify')) {
        # do not redirect in admin mode, or there will be no way to edit this category
        return print $self->md_redirect($redirect_url);
      }
  }

  my $iid = $self->defined_or_exit($self->id,"id is missing.");

  my $title = $self->name();


  my ($global_look) = $PARAMS->{'globallook'};

  #CKIM-REDIRECT: $content is declared earlier
#    print STDERR "in Category::www_show: global_look is: $global_look\n";
  # aju 02-27-02: don't need to run any of the 'show' methods if in template mode; when
  # print_portal is called, template will not use the content param passed in, but will
  # generate its own content from the template & gizmotag expansion.
  if (($global_look eq "new" || $global_look eq "latest")) {
      $content .= $self->show_simple();
  }
  elsif ($global_look eq "usetemplate") {
      # add nothing to content string
  }
  else {            # if look is 'old', or otherwise, default
                # to 'show'  (I guess this is the best default?..)
      $content .= $self->show();
  }
#    $content .= ($global_look eq "new" || $global_look eq "latest") ? $self->show_simple() : $self->show();

  $self->print_portal($content);
}

sub www_show_news {
  my $self = shift;
  $self->debug_msg(3, "Category::www_show_news()");

  my $iid = $self->defined_or_exit($self->id,"id is missing.");

  my $content = $self->show_news_list($FORM{from}, $FORM{nb});
  $self->print_portal($content,"News List");
}

sub process_edit_form {
    my $self = shift;
    $self->debug_msg(3, "Category::process_edit_form()");

    my $parentid = $self->parent_id();
    my $printname = $self->get_gizmo_name();

    # fill the object with form values
    $self->name($FORM{name});
    $self->description($FORM{description});
    $self->keywords($FORM{keywords});
    $self->display_look($FORM{display_look});
    $self->skin_layout($FORM{skin_layout} || $PARAMS->{default_page_layout} || '');
    #TODO: add support for PARAMS->default_page_layout to the config UI

    #CKIM-REDIRECT
    $self->check_redirect_url($FORM{redirect_url});
    $self->set_redirect_url($FORM{redirect_url});

    $self->is_hidden($FORM{is_hidden});

    $self->is_update_notification_enabled($PARAMS->{default_update_notification_enablement});

    # check if everything is ok
    my $errormsg = "";
    $errormsg .= "<li>Too bad! Name field cannot be empty...</li>" if (!$self->name()) ;

    if ($errormsg) {

        my $parent = Gizmo->restore($parentid);
        $parent->www_add_child($errormsg, $self->is_a);

    } else {
        my $newid;
        if ($newid = $self->add()){        #### add successful

            my $uploads_err = $self->_process_roll_over_images() ;
                my $area_err    = $self->_process_page_areas();

            if( $uploads_err || $area_err) {
            my $parentid = $self->parent_id();
            my $parent = Gizmo->restore($parentid);

            # backtrack object creation
            $self->delete();

            $parent->www_add_child( $uploads_err . "<br />" .$area_err, $self->is_a );
            exit;
            }

            if ( (uc($FORM{createchannel}) eq 'YES') || $PARAMS->{default_page_channel_creation} ){
                unless ($self->create_channel()) {
                    $self->two_choices("Could not create the channel for this $printname",
                               "error",
                               "Enough for now!", "index.pl?id=$newid");
                }
            }

            $self->two_choices("<Big>New $printname added!</big><br /><br />What do you want to do?",
                                "confirm",
                                "Add another $printname", "index.pl?iid=$parentid&isa=Category&op=add_child&child_type=Category",
                                "Enough for now!", "index.pl?id=$newid");

        } else { ### add fails
            $self->debug_msg(3, "Item::www_create : error when adding Item to DB");
            $self->two_choices("Sorry it completely failed, what do you want to do?",
                       "error",
                       "Try again", "index.pl?iid=$parentid&isa=Category&op=add_child&child_type=Category",
                       "Enough for now!", "index.pl?id=$parentid");
        }
    }
}

sub www_save {
    my $self = shift;
    $self->debug_msg(3, "Category::www_save()") ;

    my $iid = $self->defined_or_exit($self->id,"id is missing.") ;

    # fill the object with form values
    $self->name($FORM{name});
    #    print STDERR "in Category::www_save, with description\n" . $FORM{description} . "\n";
    $self->description($FORM{description});
    $self->keywords($FORM{keywords});
    $self->display_look($FORM{display_look});

    #
    #CKIM-REDIRECT
    $self->check_redirect_url($FORM{redirect_url});
    $self->set_redirect_url($FORM{redirect_url});

    $self->is_hidden($FORM{is_hidden});

    # check if everything is ok
    my $errormsg = "";
    $errormsg .= "<li>Too bad! Name field cannot be empty...</li>" if (!$self->name()) ;

    if ($errormsg) {

        $self->www_modify($errormsg);

    } else {

        $self->save();

        # we redirect to itself not to parent
        my $target = $FORM{delete_image} ? $SESSION->{referer} : "index.pl?iid=$iid" ;
        $DISPLAY->md_redirect($target);
    }
}



sub save_properties {
    my $self = shift;
    $self->debug_msg(3, "Category::save_properties()") ;

    my $iid = $self->defined_or_exit($self->id,"id is missing.") ;

    my $using_user_defined_templates = $DISPLAY->using_user_defined_templates();
    # fill the object with form values

    if ($using_user_defined_templates) {
        $self->page_template_enabled($FORM{page_template_enabled}); ## use page templates
        if ($self->page_template_enabled()) {
            $self->{page_template} = $FORM{page_template};
            Metadot::TemplateMgr->save_template(obj => $self, op => 'show', template => $self->{page_template});
        }
    }

    $self->skin_layout($FORM{skin_layout} || $PARAMS->{default_page_layout} || '');
    $self->set_page_styles($FORM{page_styles});

    # [vadim] moved to separate private method
    my $uploads_err = $self->_process_roll_over_images() ;
    my $area_err    = $self->_process_page_areas();

    if( $uploads_err || $area_err ) {
        $self->www_modify_properties( $uploads_err . $area_err);
        exit;
    }

    if( $uploads_err || $area_err ) {
        return $uploads_err . $area_err;
    }

    #
    #CKIM-REDIRECT
    $self->check_redirect_url($FORM{redirect_url});
    $self->set_redirect_url($FORM{redirect_url});

    # Why does this change when saving properties?
    # there is no form[is_hidden] here
    #$self->is_hidden($FORM{is_hidden});

    if(uc($FORM{createchannel}) eq "YES"){
        $self->create_channel();
    }

    $self->is_update_notification_enabled($FORM{enable_update_notifications});


    $self->save();

    ## hand over control to SUPER class
    return $self->SUPER::save_properties();
}

sub www_add_child {
    my $self = shift;
    my $errormsg = shift;
    my $child_type = shift;
    $self->debug_msg(3, "Category::www_add_child()") ;

    my $child_isa = $FORM{child_type} || $child_type;
    $self->SUPER::www_add_child($errormsg, $child_isa);
}

sub select_skin_layout {
    my $self = shift;
    my %args = @_;
    $self->debug_msg(3, "Category::select_skin_layout()") ;

    my $errormsg   = $args{errormsg};
    my $child_type = $args{child_type};

    my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
    my $form = $self->_select_skin_layout_html();

  my $content =
"
<form name=fm action='index.pl' method='get' ENCTYPE=\"multipart/form-data\">
<input type='hidden' name='isa' value='Category'>\n
<input type='hidden' name='iid' value='$parentid'>\n
<input type='hidden' name='op' value='add_child'>\n
<input type='hidden' name='child_type' value='$child_type'>\n
$errormsg
$form
<table align='right'><tr><td><input type='submit' name='layout_chosen' value='Continue'></td></tr></table>
<br />&nbsp;
</form>
";

  $self->print_form($content, "Select Page Layout");
}

sub _select_skin_layout_html {
    my $self = shift;
    my %args = @_;
    $self->debug_msg(3, "Category::_select_skin_layout_html()") ;

    my $skin_layout = $args{skin_layout};
    $self->debug_msg(4, "Category::_select_skin_layout_html() skin_layout:$skin_layout") ;

    my $layout_area_definitions = $self->get_layout_definitions();
    my $layout_count = 0;

    my $html = "<b>Select Page Layout</b><br />\n";
    $html .= "<table border=\"0\" cellspacing=\"5\" cellpadding=\"5\">\n";
    $html .= "<tr valign=borderline align=center>\n";
    $html .= "<td><font size=+1>" . ($layout_count++) . "</font></td>\n";
    foreach (@$layout_area_definitions) {
        $html .= "<td><font size=+1>" . ($layout_count++) . "</font></td>\n";
    }
    $html .= "</tr>\n";

    my $selected = '';

    $selected = $skin_layout eq '' ? 'checked' : '';
    $html .= "<tr valign=borderline>\n";
    $html .= "<td><table border=\"0\" cellspacing=\"0\" cellpadding=\"0\"><tr><td><input type=\"radio\" name=\"skin_layout\" value=\"\" $selected></td><td>&nbsp;Automatic layout</td></tr></table></td>\n";
    foreach my $layout_area_definition (@$layout_area_definitions) {
        $selected = $skin_layout eq $layout_area_definition->{name} ? 'checked' : '';
        $html .= "<td><table border=\"0\" cellspacing=\"0\" cellpadding=\"0\"><tr><td><input type=\"radio\" name=\"skin_layout\" value=\"" . $layout_area_definition->{name} . "\" $selected></td><td>" . $layout_area_definition->{small_image} . "</td></tr></table></td>\n";
    }

    $html .= "</tr></table>\n";
    $html .= "<b>Automatic layout</b> manages the layout of each page. Calendars, Polls, and News will appear in a sidebar on the right.<br />";

    return $html;
}

sub _process_roll_over_images {
    my $self = shift ;

    my $result = $self->process_upload( {
                     isa => 'Category',
                     id => $self->id,
                     field_name => 'display_image',
                     field_desc => 'Display Image',
                     is_required => 0,
                     type => Metadot::UploadsManager::UPLOAD_TYPE_PUBLIC(),
                    } );

    if($result) { $result .= '<br />' }

    $result .= $self->process_upload( {
                       isa => 'Category',
                       id => $self->id,
                       field_name => 'highlight_image',
                       field_desc => 'Highlight Image',
                       is_required => 0,
                       type => Metadot::UploadsManager::UPLOAD_TYPE_PUBLIC(),
                      } );

    if($result) { $result .= '<br />' }

    $result .= $self->process_upload( {
                       isa => 'Category',
                       id => $self->id,
                       field_name => 'active_image',
                       field_desc => 'Active Image',
                       is_required => 0,
                       type => Metadot::UploadsManager::UPLOAD_TYPE_PUBLIC(),
                      } );


    return $result;

}



sub copy_uploads {
    my ($self, $target) = @_;

    $self->SUPER::copy_uploads($target);

    foreach my $field qw(display_image highlight_image active_image) {
        my $error = $self->copy_upload(
            {
                 isa  => $self->is_a(),
                 id   => $self->id(),
                 field_name => $field,
            },
            {
                 isa  => $target->is_a(),
                 id   => $target->id(),
                 uid  => $target->uid(),
                 field_name  => $field,
                 is_required => 0,
            },
        );

        if ($error) {
            print STDERR "Error occurred while copying file: $error\n";
        }
    }
}






sub _process_page_areas {
    my $self = shift;
    my %args = @_;

    my $id = $self->id();
    $self->debug_msg(3, "Category::_process_page_areas(id=$id)");

    return unless ($PARAMS->{use_controller});

    return 'ID not defined' unless defined $id;  #CKIM

    my $new_skin_layout = $self->skin_layout()||1; ## layout1, layout2, ...
#print STDERR "tseting.....". $new_skin_layout;

    ##### PRE-PROCESSING #####

    ## STEP 1 - build a hash (%number_of_areas) that will store each
    ##   layout name and the number of areas that layout contains
    my %number_of_areas = ();
    my $layout_definitions = $self->get_layout_definitions();
    my $layout_name = '';
    foreach my $layout_definition (@$layout_definitions) {
        $layout_name = $layout_definition->{name}; ## layout1, layout2, ...
        $number_of_areas{$layout_name} = $layout_definition->{areas};
    }
    ## END OF STEP 1

    ## STEP 2 - build a hash (%current_defined_areas) that will store
    ##  info on the areas of the current category and determine the
    ##  largest area_id ($previous_largest_area_id)
    ##  Note: each area contains an area_id and iid
    ##  Note: area_ids are numbered starting from 1
    my %current_defined_areas = ();

    ## instance table field name where area id is stored
    my $area_id_field_name = LayoutArea->get_area_id_field_name();
    my $list = DBAccess->sqlSelectMany("$area_id_field_name, iid", "instance", "parentiid = $id and isa='LayoutArea'", "order by $area_id_field_name");

    my $previous_largest_area_id = 0;
    ## compile a list of all current areas
    while (my($area_id, $iid) = $list->fetchrow) {
        next unless $area_id > 0;
        if ($area_id == $previous_largest_area_id) { ## duplicates - should not happen
            ## move children of $area_id into $previous_largest_area_id
            my $parent_object = Gizmo->restore($current_defined_areas{$previous_largest_area_id} );
            my $err_msg = $parent_object->move_children_into_and_delete_old_parent(current_iid => $iid);
            die $err_msg if $err_msg ne '';
        } else {
            $current_defined_areas{$area_id} = $iid;
            $previous_largest_area_id = $area_id;
        }
    }
    ## END OF STEP 2

    $self->debug_msg(4, "Category::_process_page_areas():end");


    ##### END OF PRE-PROCESSING #####

    ## get the number of areas that the new skin layout has
    my $new_number_of_areas = ($number_of_areas{$new_skin_layout} || 0);

    ## create new areas if not defined
    ## i.e. if the number of areas increases from 3 to 5
    for (1 .. $new_number_of_areas) {
        unless ($current_defined_areas{$_}) {
            ## area does not exist -> create
            my $new_area = LayoutArea->new({
                         uid => $USER->uid(),
                         parent_id => $id,
                         $area_id_field_name => $_,
                         name => "Area $_",
                        });
            $new_area->add();
            ## add new area to hash
            $current_defined_areas{$_} = $new_area->id();
            $self->debug_msg(4, "Category::_process_page_areas(): creating new area $_ with iid " . $current_defined_areas{$_});
        }
    }

    my $err_msg = '';
    ## if the number of areas is less than the previous number of areas
    ## i.e. if the number of areas decreases from 5 to 3
    my $new_parent_gizmo_container;
    if ($previous_largest_area_id > $new_number_of_areas) {
        $self->debug_msg(4, "Category::_process_page_areas(): must remove areas with ids greater than: $new_number_of_areas");


        ## must determine who new parent will be
        if ($new_number_of_areas == 0) {
            ## removing all areas - must move to parent category
            $new_parent_gizmo_container = $self;
            $self->debug_msg(4, "Category::_process_page_areas(): removing all areas must move all children of areas to this category:" . $new_parent_gizmo_container->id());
        } else {
            ## the new parent will be the area with the largest id
            $new_parent_gizmo_container = Gizmo->restore($current_defined_areas{$new_number_of_areas} );
            $self->debug_msg(4, "Category::_process_page_areas(): new gizmo container will be:" . $new_parent_gizmo_container->id());
        }
        ##

        ## must move the children of the soon-to-be-deleted areas to the
        ## area with the largest area_id
        ## i.e. if number of areas decreasing from 5 to 3, then all
        ##   children of areas 4 & 5 will be moved to area 3
        for (($new_number_of_areas + 1) .. $previous_largest_area_id) {
            $self->debug_msg(4, "Category::_process_page_areas(): moving children of Area $_ with iid:" . $current_defined_areas{$_} . " to area $new_number_of_areas with new iid:" . $new_parent_gizmo_container->id());
            ## move children to new parent
            $err_msg = $new_parent_gizmo_container->move_children_into_and_delete_old_parent(current_iid => $current_defined_areas{$_});
            if ($err_msg) {
                $self->debug_msg(4, "Category::_process_page_areas(): error moving children of Area $_ with iid:" . $current_defined_areas{$_} . " error:$err_msg");
                return $err_msg;
            } else {
                ## delete the old area from the hash of current areas
                delete($current_defined_areas{$_});
            }
        }
    }


    ## make sure all children of Category are in area if a single area exists
    ## i.e. if the category previously had no areas and now does, then all
    ##   those area-less children are moved to the first area
    if ($current_defined_areas{1} && $new_number_of_areas > 0) {
        my $children = DBAccess->sqlSelectMany("iid", "instance", "parentiid = $id and isa != 'Category' and isa != 'LayoutArea'");
        while (my($child_iid) = $children->fetchrow) {
            my $child = Gizmo->restore( $child_iid );
            $child->{parent_id} = $current_defined_areas{1};
            $self->debug_msg(4, "Gizmo::Category::_process_page_areas(iid=$id) moving child $child_iid to new parent " . $current_defined_areas{1}) ;
            $child->save();
        }
    }
}

## will move all children of instance to instance's parent
## overrides GizmoContainer
sub promote_children {
    my $self = shift;
    $self->debug_msg(3, "Category::promote_children()");
    my $id = $self->id;

    my $error_msg = '';
    ## delete all the areas and move all gizmos to page level
    $self->skin_layout(0);
    $error_msg = $self->_process_page_areas();
    if ($error_msg) {
        $self->debug_msg(4, "Category::promote_children() error_msg:$error_msg");
        return 0;
    }

    my $parent = $self->get_parent();
    $error_msg = $parent->move_children_into(current_iid => $id);
    if ($error_msg) {
        $self->debug_msg(4, "Category::promote_children() error_msg:$error_msg");
        return 0;
    }
    $error_msg = $parent->_process_page_areas();
    if ($error_msg) {
        $self->debug_msg(4, "Category::promote_children() error_msg:$error_msg");
        return 0;
    }
    return 1;
}

# adds a class to exclude. this can be called from the initial code
# of a Gizmo class.
# NOTE: THIS METHOD HAS A SITE-WIDE EFFECT.
sub exclude_gizmo_class {
    my $class = shift;
    my @exclude_list = @_;

    foreach my $exclude (@exclude_list) {
        unless (Utility->array_contains( $exclude, \@EXCLUDE_FROM_CONTENTS )) {
            push @EXCLUDE_FROM_CONTENTS, $exclude;
        }
    }
}

###CLEVAND
############################################
# Returns a list of all non excluded gizmos.
# Uses @EXCLUDE_FROM_CONTENTS
############################################

sub get_non_excluded_gizmo_list {
    my %excluded;
    foreach (@EXCLUDE_FROM_CONTENTS) {
        $excluded{$_} = 1;
    }

    my @included;
    foreach (Gizmo->get_gizmo_list()) {
        unless (exists ($excluded{$_})) {push @included, $_;}
    }
    return @included;
}


#########################################
# BACKEND FUNCTIONS

sub go_back_to_parent_after_editing_permissions {
        return 0;
}

# delete image and the folder (if it's empty)
sub delete_image {
        my $self = shift ;
        my $iid = $self->id() ;
        my $uid = $USER->uid() ;
        my $file = shift ;
        $self->debug_msg(4, "Category::delete_image(iid=$iid, filename=$file)") ;  # delete from file system

        my $absolutepathfileuploaddir = $PARAMS->{'absolutepathfileuploaddir'} ;

        my $directory_name = "$absolutepathfileuploaddir/$iid" ;
        my $filename = "$directory_name/$file" ;

        $self->debug_msg(5, "Category::delete_image: $filename") ;
        if (-e $filename) {
                unlink("$filename") ;# || $self->print_click_back("Error", "Enable to delete file ($!): $filename") ;
        }
        else {
                $self->print_click_back("Error", "File does not exist ($!): $filename") ;
        }

        # test if directory exists and is empty and delete it if it is
        if (-d $directory_name) {
                my $directory_is_empty = 1 ;
                if (opendir(MYDIR, $directory_name)) {
                        my $filename ;
                        while ($directory_is_empty && ($filename = readdir(MYDIR))) {
                                $directory_is_empty = $filename eq '.' || $filename eq '..' ;
                        }
                        closedir MYDIR ;
                        if ($directory_is_empty) {
                                $self->debug_msg(5, "Category::delete_image: deleting dir: $directory_name") ;
                                rmdir ($directory_name) || $self->print_click_back("Error", "Enable to delete empty directory ($!): $directory_name") ;
                        }
                }
        }
        #

        # delete link from DB
        $self->{file1} = "";
        $self->{file1_size} = "";
        $self->save();
        #DBAccess->sqlUpdate("instance", "iid=$iid", (file1=>"", file1_size=>"")) ;

        return 1 ;
}


sub show_edit_form {
  my $self = shift;
  my $title =shift;
  $self->debug_msg(4, "Category::show_edit_form($title)");
  my $iid = $self->id;
  my $name = $self->name;
  $name = escapeHTML($name);
  my $description = $self->description;

  # CKIM: let the html editor class take care of this
  # $description = escapeHTML($description);


  my $keywords = $self->keywords;
  $keywords = escapeHTML($keywords);
  my $skin_layout = $FORM{skin_layout};
  #CKIM-REDIRECT
  my $redirect_url = escapeHTML($self->get_redirect_url());

  my $display_look = $self->display_look || "LULI";

  my $printname = $self->get_gizmo_name();
  my ($global_look) = $PARAMS->{'globallook'};

  my $look = "";
  if ($global_look eq "old") {
      # the 'old' display look is the only circumstance under which the following information
      # will be acutually used in the display.
      # prepare radio buttons for display look
      $look = "Category/Item two-level list\n <input type='radio' name='display_look' value='LULI'";
      $look .= " checked " if ($display_look eq "LULI");
      $look .= ">\n\n Two-column table\n<input type='radio' name='display_look' value='twocol' ";
      $look .= " checked " if ($display_look eq "twocol");
      $look .= ">\n\n <br />Categories first then items\n<input type='radio' name='display_look' value='catfirst' ";
      $look .= " checked " if ($display_look eq "catfirst");
      $look .= ">\n\n Dashes (no BR) \n<input type='radio' name='display_look' value='Dashes' ";
      $look .= " checked " if ($display_look eq "Dashes");
      $look .= ">\n\n";
  }

  #CKIM-REDIRECT: added input field for redirects
  my $s = "<h1>$title</h1>
           <input type='hidden' name='skin_layout' value=\"$skin_layout\">
           <br /><b>" . $printname . "Name </b> (*)<br />
           <input type='text' name='name' size='55' maxlength='255' value=\"$name\">" ;

     # JSForm
     $self->{jsform}->add_check_simple("name", "$printname Name", "notempty") if defined($self->{jsform}) ;
     # JSForm

   $s .= "<br /><br /><b>Description</b><br />";

   $s .= $DISPLAY->generate_wysiwyg_field({field_name=>'description', field_value=>$description});

   $s .= " <br />
           <br /><b>Redirect to URL</b><br />
           <input type='text' name='redirect_url' size='55' maxlength='255' value=\"$redirect_url\">
           <br />
           <i>Typing in a URL, such as 'http://www.metadot.com', will open that website whenever a user
           clicks on this $printname.</i><br />

           <br /><b>Hide this $printname in navigation</b>&nbsp;
           <input type='checkbox' name='is_hidden' value='1' ".($self->is_hidden() ? 'checked':'')."/>
           <br />
           <i>If checked, this page will not appear in navigation. The page can still be viewed if the user has the URL.</i><br />

           <br /><b>Keywords and Synonyms</b><br />
           <textarea name='keywords' rows=1 cols=80 WRAP='VIRTUAL'>$keywords</textarea>
           <br />";

   $s .= "<br /><b>Display Look</b><br />$look<br />" if(lc($global_look) eq 'old');

  $s .= "\n<br /><br /><br /><i>(*) Mandatory field(s)</i><br />";
  return $s;
}

sub show_edit_properties_form {
  my $self = shift;
  my $title =shift;
  $self->debug_msg(4, "Category::show_edit_properties_form($title)");
  my $iid = $self->id;
  my $page_template = $self->get_page_template(); ## template
  # if there is no template, we're going to use portal_sub.tt2 from the SkinManager.
  $page_template ||= Metadot::SkinManager->get_instance()->get_file_contents({name=>'portal_sub.tt2'});

  $page_template = escapeHTML($page_template);
  my $use_page_template = $self->page_template_enabled;
  #CKIM-REDIRECT
  my $redirect_url = escapeHTML($self->get_redirect_url());

  my $printname = $self->get_gizmo_name();

  # prepare field for channel information
  my $channel = "<br />";
  if ($FORM{op} =~ m/(add|create)/ || !$self->cid) {
    $channel .= "<h3>Create a news channel for this $printname?</h3>\n";
    $channel .= "Yes&nbsp;<input type='radio' name='createchannel' value='YES' >&nbsp;&nbsp;\n";
    $channel .= "No&nbsp;<input type='radio' name='createchannel' value='NO' checked><br />\n";
  } else {
    $channel .= "<h3>$printname Channel</h3>\n";
    $channel .= "<a href='".$self->make_url("index.pl?cid=".$self->cid."&amp;isa=Channel&amp;op=modify")."'>Edit news channel information</a> <br />\n";
    $channel .= "<a href='".$self->make_url("index.pl?cid=".$self->cid."&amp;isa=Channel&amp;op=delete")."'>Delete news channel</a> <br />\n";
  }

  # field to enable update notifications
  my $update_notification_check = $self->is_update_notification_enabled() ? 'checked' : '';
  my $update_notification = <<EOM
  <br />
  <h3>
    Content update notifications
  </h3>
  <div>
    <input type="checkbox" name="enable_update_notifications" value="1" $update_notification_check />&nbsp;Allow editors of this $printname to
    submit notifications to users when content is updated. Users can subscribe to these notifications via channels.
    <br />
    <strong>Note: enabling this feature will also create a channel if one has not yet been created.</strong>
  </div>
EOM
;



  #CKIM-REDIRECT: added input field for redirects
  my $s = "<h1>$title</h1>\n";

  my $isa = $self->is_a;
  my $return_url = $self->make_url( ( $FORM{op} eq 'add_child' ) ? "index.pl?iid=$iid&amp;isa=Category&amp;op=add_child&amp;child_type=Category" : "index.pl?id=$iid&amp;isa=Category&amp;op=modify_properties" );


    if ($PARAMS->{use_controller}) {
        my $skin_layout_html = $self->_select_skin_layout_html(skin_layout => $self->skin_layout());
        $s .= "<br /><a name=\"skin_layout\"><h3>Page Layouts</h3><table border=\"1\" cellpadding=\"10\" cellspacing=\"0\" width=\"100%\"><tr><td> " ;
        $s .= $skin_layout_html;
        $s .= "<br /><br />" ;
        $s .= "</td></tr></table><p />" ;
    }

  # 10-16-02: only add this part of the form if the iid is not 0 (e.g. page
  # is not the home page).  For the home page, one never sees these images, and they have
  # a naming conflict with the themes image browser images.
  if ( (!defined $self->id()) || !($self->id() == 0) || MetadotConfig->get_param_value('allow_root_navigation_images') ) {
      $s .= "<br /><a name=\"images\"><h3>Images</h3><table border=\"1\" cellpadding=\"10\" cellspacing=\"0\"><tr><td> " ;
      $s .= "<p /><b>Display Image</b><br /><br /><i>" ;
      $s .= "If provided, this image will appear instead of the name of the $printname in the left column navigation bar. ";
      $s .= "The name of the $printname will be used for the ALT tag of this image." ;
      $s .= "</i><br /><br />" ;


      $s .= $self->get_upload_input_form_line( {
      isa => $isa,
      id => $iid,
      field_name => 'display_image',
      field_desc => 'Display Image',
      is_required => 0,
      type => Metadot::UploadsManager::UPLOAD_TYPE_PUBLIC(),
      return_url => $return_url,
      } );

      $s .= "<hr height=\"1\"><p /><b>Highlight Image</b><br /><br /><i>" ;
      $s .= "If provided, this image will appear when the user places mouse over this $printname in the left column navigation bar. ";
      $s .= "Note: To use it you should provide \"Display Image\" also. You can but don't have to provide \"Active Image\"." ;
      $s .= "</i><br /><br />" ;

      $s .= $self->get_upload_input_form_line( {
      isa => $isa,
      id => $iid,
      field_name => 'highlight_image',
      field_desc => 'Highlight Image',
      is_required => 0,
      type => Metadot::UploadsManager::UPLOAD_TYPE_PUBLIC(),
      return_url => $return_url,
      } );

      $s .= "<hr height=\"1\"><p /><b>Active Image</b><br /><br /><i>" ;
      $s .= "If provided, this image will appear when user clicks on this $printname in the left column navigation bar. ";
      $s .= "Note: To use it you should provide \"Display Image\" also. You can but don't have to provide \"Highlight Image\"." ;
      $s .= "</i><br /><br />" ;


      $s .= $self->get_upload_input_form_line( {
      isa => $isa,
      id => $iid,
      field_name => 'active_image',
      field_desc => 'Active Image',
      is_required => 0,
      type => Metadot::UploadsManager::UPLOAD_TYPE_PUBLIC(),
      return_url => $return_url,
      } );

      $s .= "</td></tr></table>" ;
  }

  if ( $DISPLAY->using_user_defined_templates()
       && $self->user_may_modify_local_templates()
       && ((!defined $self->id()) || !($self->id() == 0))
      ) {
      $s .= "<h3>$printname Template</h3>
         <table border=\"1\" cellpadding=\"10\" cellspacing=\"0\"><tr><td>
           <input type = 'checkbox' name='page_template_enabled' value='on'";

        if ($use_page_template){
            $s .= " checked ";
        }

        $s .= ">&nbsp;<b>Use Page Template below?</b> <i>(Allows this page & subpages to have a specific look and feel.)</i><br />
           <textarea name='page_template' rows=20 cols=80 WRAP='VIRTUAL'>$page_template</textarea>
         </td></tr></table>\n";
  }

  if ($PARAMS->{use_controller}) {
      my $page_styles = $self->get_page_styles({local=>1});
      $s .= "<br />\n";
      $s .= "<div><a name=\"page_styles\"><h3>Page Styles</h3>(allows a page to define or override the CSS stylesheet for the current skin)</div>\n";
      $s .= "<br />\n";
      $s .= "<table border='1'><tr><td>\n"; # for consistency with the other page elements
      $s .= "<div><textarea name='page_styles' rows=20 cols=80 WRAP='VIRTUAL'>$page_styles</textarea></div>\n";

      # see if this page inherits styles from a parent
      my $parent = Utility->get_structural_ancestor($self->get_parent(), __PACKAGE__);
      if (defined $parent) {
          my $parent_styles = $parent->get_page_styles();
          if (defined $parent_styles && $parent_styles ne '') {
              $parent_styles =~ s/\n/<br \/>\n/g;
              $s .= "<div><b>This page will inherit styles from its parent(s):</b></div>\n";
              $s .= "<div>$parent_styles</div>\n";
          }
      }
      $s .= "</td></tr></table>\n";
  }


  $s .= " $channel\n";

  $s .= $update_notification."\n";

  $s .= "<p />" . $self->SUPER::show_edit_properties_form();

  $s .= "<br /><br /><br /><i>(*) Mandatory field(s)</i><br />";
  return $s;
}


sub get_page_styles {
    my $self = shift;
    my $args = shift || {};
    my $result = $self->t1();

    # prevent infinite recursion
    return if (defined $args->{visited}{$self->id()});
    $args->{visited}{$self->id()}=1;

    # get the inherited styles unless suppressed
    unless ($args->{local}) {
        $result &&= "/* page styles for ".$self->get_name()." */\n".$result;
        my $parent = Utility->get_structural_ancestor($self->get_parent(), __PACKAGE__);
        if (defined $parent) {
            my $parent_style = $parent->get_page_styles();
            if (defined $parent_style) {
                $result = $parent_style."\n\n".$result;
            }
        }
    }

    return $result;
}

sub set_page_styles {
    my $self = shift;
    return $self->t1(@_);
}


sub user_may_modify_local_templates {
    my $class = shift;

    # In the case of MetadotController, we check for user permission. This
    # is because templates give too much power to regular end users.
    return (!$PARAMS->{use_controller} || $ACCESS_BROKER->is_allowed_to_do('admin_tasks'));
}


###################################

sub create{
    my $self = shift;
    my ($uid, $parentiid, $name, $description, $keywords, $displaylook, $createdby )=@_;

    $self->debug_msg(3, "Category::create(" . ($parentiid||"") . ", " . ($name||"") . "," . ($description||"") . ", " . ($keywords||"") . ", " . ($displaylook||"") . ")");

    # fill object with passed data
    $self->uid($uid);
    $self->parent_id($parentiid);
    $self->name($name);
    $self->description($description);
    $self->keywords($keywords);
    $self->display_look($displaylook);
    $self->created_by($createdby);

    # add the object to DB (take care about 'iid' and 'instanceorder')
    my $iid = $self->add();

    $self->debug_msg(4, "Category::create() : new category created with iid=$iid");
    return $iid;
}


#########################################
# show_simple
# for new_look
sub show_simple {
  my $self = shift ;
#    print STDERR "in Category::show_simple\n";
  $self->debug_msg(3, "Category::show_simple()");

  my $name = $self->name();
  my $description = $self->description();

  my $str = "";
  #my $panel = $self->show_panel();

  #$str = "<table width="100%"><tr><td align='right'></td></tr></table>\n" if ($panel);
  $str .= "$description";

  $str .= $self->show_all_childs_list();

  return $str;

}
#########################################
sub show {
  my $self = shift;
  my $id= $self->{id};
  my $str="";

  $self->debug_msg(3,"Category::show($id)");
#  return "" if (!defined($id));

  my $title_bar = $self->show_title();
  my $cate_col  = $self->show_category_column();
  my $news_col  = $self->show_news_column();
  my $poll_col  = $self->show_poll_column();
  # aju 04-23-02 pass in info to allow calendar to stay rendered in parent envir when shift
  # forw/backw a month
  my $cal_col   = $self->show_calendar_column('', $id, 'show');
  my $right_col = '';

  $right_col = "
<!-- News column -->
<td align='right' valign='top' width=\"155\">
$cal_col

$news_col

$poll_col
</td>
<!-- End of News column -->
";

  return "

<!-- Outer Table -->
<table border=\"0\" cellspacing=\"0\" width=\"100%\"><tr><td>

<!-- Category Table -->
<table border=\"0\" cellpadding=\"0\" cellspacing=\"10\" width='100%'><tr>

<!-- Category column -->
<td valign='top' width=\"100%\">
$cate_col
</td>
<!-- End of Category column -->
$right_col
</tr></table>
<!-- End of Category Table -->

</td></tr></table>
<!-- End of Outer Table -->

";

}



#########################################
# Yes, it's possible to place a page inside of a
# section.
#
sub show_summary {
    my $self        = shift;

    my $name        = $self->get_name();
    my $buttons     = $self->get_buttons();
    my $url         = $self->make_url("index.pl?iid=".$self->id());

    my $html        = "<a href=\"$url\">$name</a>$buttons";
    return $html;
}



#########################################
sub show_category_column {
  my $self = shift;
  my $id = $self->{id};
  $self->debug_msg(3, "Category::show_category_column()");
  return "" if (!defined($id));

  my $str = "";
  my $name = $self->name();

  #my $panel = $self->show_panel();
  #$str .= "<table width="100%"><tr><td align='right'>$panel</td></tr></table>\n" if ($panel);

  ### Render the category
  ### XXX [claudio] is this query necessary?
  my ($look) = DBAccess->sqlSelect("displaylook", "instance", "iid=$id");

  $look="twocol" if(!defined $look);
  if (uc($look) eq "TWOCOL") {
    $str .= $self->show_list_twocol();
  } elsif (uc($look) eq "LULI") {
    $str .= $self->show_list_luli();
  } elsif (uc($look) eq "CATFIRST") {
    $str .= $self->show_list_cat_first();
  } elsif (uc($look) eq "DASHES") {
    $str .= $self->show_list_dashes();
  } else {
    $str .= "I don't know what to show with this look: $look!";
  }

  $str .= $self->show_all_childs_list();

  return $str;
}


sub get_main_column_children {
    my $self = shift;
    $self->debug_msg(4, 'get_main_column_children('.$self->id().')');
    my @excluded = (Gizmo->list_navigation_gizmos(), @EXCLUDE_FROM_CONTENTS);
    my $args = {excluded=>\@excluded};

    my $childs = $self->get_childs_list($args);
    my @results;

    # If page layout is auto, filter out calendars who have mini-mode
    if ($self->skin_layout()) {
        @results = @$childs;
    } else {
        foreach my $child (@$childs)
        {
            unless ($child->isa('Calendar') && $child->get_view_mode() eq Calendar::VIEW_MINI())
            {
                push @results, $child;
            }
        }
    }

    return \@results;
}

#########################################
sub show_all_childs_list {
    my $self = shift;

    my $messages = '';
    if ($SESSION->admin_mode() && $self->is_allowed_to_do('modify')) {
        my $redirect_msg;
        if($self->id() && $self->get_redirect_url()) {
            my $redirect_url = $self->get_redirect_url();
            $messages .= "<br />This page will redirect users to:<br />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"$redirect_url\">$redirect_url</a>\n";
        }

        # if this is a hidden page, indicate that
        if ($self->is_hidden) {
            $messages .= "<br />This page is hidden from navigation.";
        }
    }

    my @excluded = (Gizmo->list_navigation_gizmos(), @EXCLUDE_FROM_CONTENTS);
    return $messages . $self->show_childs_list(excluded=>\@excluded);
}


#########################################
sub show_my_list {

  my $self = shift;
  my $gizmotype = shift;

  $self->debug_msg(3, "Category::show_my_list");
  #print STDERR "calling show_my_list with gizmo - $gizmotype\n";

  my $name = $self->name();
  my $str = "";
  my @gizmos_list = grep { /$gizmotype/; } (Gizmo->lists_gizmos()) ;
  my $head;
  my $decorator;

  foreach my $gizmo (@gizmos_list) {
    my $gizmo_name = $gizmo->get_gizmo_name();
    $decorator = $gizmo->gizmo_list_decorator || 'HTML::Decorator::BulletDecorator';

    my $childs_list = $self->show_childs_list( type=>$gizmo, decorator=>$decorator);
    if ($gizmo->isa("NewsPaperView") || $gizmo_name eq "Poll" || $gizmo_name eq "Contractor") {
            $head = "";
    }
    else {
    # split name...
    my @namewords = split(/\ /, $name);
    # get last word in name
    my $lastword = $namewords[$#namewords];
    if (lc $lastword eq lc($gizmo_name . "s") ) {
        $head = "<h3>$name</h3>";
    }
    elsif  (lc $lastword eq lc $gizmo_name) {
        $head = "<h3>$name" . "s</h3>";
    }
    else {
            $head = "<h3>$name ".$gizmo_name."s</h3>";
    }
    }
    if ($childs_list){
    if ($gizmo->isa("NewsPaperView")) {
        $str .= "<!-- $gizmo List -->\n$head\n$childs_list\n<!-- End of $gizmo List -->\n\n"
        } elsif ($gizmo_name eq "Discussion") {
        $str .= "<!-- $gizmo List -->\n$head\n$childs_list\n<!-- End of $gizmo List -->\n\n"
                }else{
                    $str .= "<!-- $gizmo List -->\n<p /><br />\n<div  class='catlistsub'> $head</div>\n$childs_list\n<!-- End of $gizmo List -->\n\n"
            }
    }

 }

 return $str ;
}

#########################################
# Must reorder navigation gizmos separately
#########################################
sub reorder_children {
    my $self = shift;
    my $id=$self->id();
    my $isa = $self->is_a();

    my @navigation_gizmos = $self->list_navigation_gizmos();
    my $nav_gizmo_list = Utility->convert_list_to_sql_list(@navigation_gizmos);

    # this is used to separate the various kinds of Gizmos I can contain
    my $instance_page = $self->get_instance_order_address_page('Category');

    # order the navigation gizmos
    my $i = $instance_page * Metadot::GizmoContainer::INSTANCE_ORDER_ADDRESS_PAGE_SIZE();
    my ($c) = DBAccess->sqlSelectMany("iid", "instance", "parentiid=$id AND isa IN ($nav_gizmo_list)", "order by instanceorder");
#print STDERR "\n\nWHERE1:parentiid=$id AND isa IN ($nav_gizmo_list)\n\n";
    while(my ($sonid) = $c->fetchrow()) {
        $self->get_object_accessor()->update_where( where=>"iid=$sonid", updates=>{instance_order=>$i});
        $i++;
    }

    # order the NON-navigation gizmos
    my $exclude_list = Utility->convert_list_to_sql_list(@navigation_gizmos, @EXCLUDE_FROM_CONTENTS);

    $instance_page = $self->get_instance_order_address_page(); # get the default
    $i = $instance_page * Metadot::GizmoContainer::INSTANCE_ORDER_ADDRESS_PAGE_SIZE();

    ($c) = DBAccess->sqlSelectMany("iid", "instance", "parentiid=$id AND isa NOT IN ($exclude_list)", "order by instanceorder");
#print STDERR "\n\nWHERE2:parentiid=$id AND isa NOT IN ($exclude_list)\n\n";
    while(my ($sonid) = $c->fetchrow()) {
        $self->get_object_accessor()->update_where( where=>"iid=$sonid", updates=>{instance_order=>$i});
        $i++;
    }
    $c->finish();

    # order the exclude gizmos
    foreach my $exclude_item (@EXCLUDE_FROM_CONTENTS) {
        $instance_page = $self->get_instance_order_address_page($exclude_item);
        $i = $instance_page * Metadot::GizmoContainer::INSTANCE_ORDER_ADDRESS_PAGE_SIZE();

        ($c) = DBAccess->sqlSelectMany("iid", "instance", "parentiid=$id AND isa = '$exclude_item'", "order by instanceorder");
        while(my ($sonid) = $c->fetchrow()) {
            $self->get_object_accessor()->update_where( where=>"iid=$sonid", updates=>{instance_order=>$i});
            $i++;
        }
        $c->finish();
    }
}


sub show_as_roll_over {
    my ($self, $args) = @_;

#   print STDERR "in show_as_roll_over for " . $self->id() . "\n";
    # CKIM: removed the comment tags, because we need to strip all \n's in some cases
    srand ;
    my $img_name = "img_".lc($self->is_a)."_".$self->id()."_".('a'..'z')[26*rand].('a'..'z')[26*rand].('a'..'z')[26*rand] ;
    my $temp = "<script language=\"JavaScript\">\n";
    $temp   .= "var ".$img_name."_on = new Image(); ".$img_name."_on.src = \"" . $self->get_image_url('highlight_image') . "\" ;\n" if $self->get_image_url('highlight_image') ;
    $temp   .= "var ".$img_name."_off = new Image(); ".$img_name."_off.src = \"" . $self->get_image_url('display_image') . "\" ;\n" if $self->get_image_url('display_image') ;
    $temp   .= "var ".$img_name."_click = new Image(); ".$img_name."_click.src = \"" . $self->get_image_url('active_image') . "\" ;\n" if $self->get_image_url('active_image') ;
    $temp   .= "//</script>\n";

    my $javascript = "" ;
    $javascript .= " onmouseover=\"document.".$img_name.".src = ".$img_name."_on.src; ".$args->{onmouseover}."; return true;\"" if $self->get_image_url('highlight_image') ;
    $javascript .= " onmouseout=\"document.".$img_name.".src = ".$img_name."_off.src; return true;\"" if $self->get_image_url('display_image') ;
    $javascript .= " onmousedown=\"document.".$img_name.".src = ".$img_name."_click.src; return true;\"" if $self->get_image_url('active_image') ;

    $temp   .= "<A HREF=\"" . $self->make_url("index.pl?iid=" . $self->id()) . "\" $javascript>";

    my $name = $self->name() ;
    # [vadim] excluding the html code and quotes from name to avoid bad look
    $name =~ s/<.*?>//g ;
    $name =~ s/"/\\"/g ;
    $name =~ s/\n//g ;
    # [vadim] excluding the name to enable Cory's shoutcuts
    # in the case of regular Category (not shotcut) this string will be just empty
    my $shotcut = $self->name() ;
    $shotcut =~ s/\Q$name\E//g ;
    # if image is set for this category - show image, otherwise - show the name of the category
    if ($self->get_image_url('display_image')) {
        $temp   .= "<IMG SRC=\"" . $self->get_image_url('display_image') . "\" ALT=\"" . $name . "\" BORDER=\"0\" NAME=\"".$img_name."\">".$shotcut ;
    } else {
        $temp   .= $self->name() ;
    }
    $temp   .= "</a>\n";

    return $temp;
}

sub show_as_image {
    my ($self, $args) = @_;
    #  print STDERR "in show_as_image for " . $self->id() . "\n";
    my $image_url = $self->get_image_url('active_image') || $self->get_image_url('display_image');
    my $temp = "<A HREF=\"" . $self->make_url("index.pl?iid=" . $self->id()) . "\" onmouseover=\"$args->{onmouseover}\"";
    $temp   .= "><IMG SRC='" . $image_url . "' NAME='img_" . $self->id() . "' ALT='" . $self->name() . "' BORDER=\"0\"></a>\n";
    #  print STDERR "built string: $temp\n";

    return $temp;
}

sub get_image_url {
    my ($self, $img_id) = @_;

    # try to cache the image_url
    my $private_property = "_image_url_$img_id";
    my $url = $self->{$private_property};
    my $nullflag = '_NULL_';

    unless (defined $url && $url ne $nullflag) {
        my ($isa, $id) = ( $self->is_a(), $self->id() );
        $url = $self->get_upload_url( {
                          isa => $self->is_a,
                          id => $self->id,
                          field_name => $img_id,
                         } );
        if (defined $url) {
            $self->{$private_property} = $url;
        } else {
            $self->{$private_property} = $nullflag;
        }
    }

    return $url;


#    my $image = "index.pl?id=";
#    $image   .= $self->id();
#    $image   .= "&amp;isa=Category&amp;op=show_image&amp;image=";
#    $image   .= $img_id;
#    return $image;
}

#CKIM-REDIRECT: accessors & mutators for redirect url
sub get_redirect_url_field {
    return 'c1';
}


sub set_redirect_url {
    my ($self, $value) = @_;
    $self->{$self->get_redirect_url_field()} = $value;
}

sub get_redirect_url {
    my $self = shift;
    return $self->{$self->get_redirect_url_field()};
}

# checks that the redirect url does not point to self, to avoid silly mistakes
sub check_redirect_url {
    my ($self, $value) = @_;

    my $error;

    if ($value) {
        # check that I am not redirecting to myself

        # check that I'm not redirecting to self. Check for id=[some number] and
        # compare that number to myself
        $value =~ /id=(\d+)/;
        my $id_var = $1;

        $DISPLAY->debug_msg(4, "check_redirect_url has value '$value', found id_var '".(defined $id_var ? $id_var : 'undef')."'");
        if (defined $id_var && $id_var ne '' && $id_var == $self->id()) {
            $error = "You have set the page to redirect to itself. Please correct this.";
        }
    }

    if ($error) {
        $DISPLAY->print_click_back("Error", $error);
        exit();
    } else {
        return 1;
    }
}

###########################################

sub is_hidden_field {
    my $self = shift;
    return 'i1';
}

sub is_hidden {
    my $self = shift;
    my $field = $self->is_hidden_field();
    return @_ ? $self->{$field} = shift : $self->{$field};
}



###########################################
sub show_category_list {
  my $self = shift;
  my $id = $self->{id};
  $self->debug_msg(3, "Category::show_category_list()");

  return "" if (!defined($id));

  my ($look) = DBAccess->sqlSelect("displaylook", "instance", "iid=$id");
  $look="twocol" if(!defined $look);

  return $self->show_list_twocol() if (uc($look) eq "TWOCOL");
  return $self->show_list_luli()   if (uc($look) eq "LULI");
  return $self->show_list_cat_first()   if (uc($look) eq "CATFIRST");
  return $self->show_list_dashes() if (uc($look) eq "DASHES");


  return "I don't know what to show with this look: $look!";
}

##########################################
### show_list_dashes
### format category in dashes look
##########################################
sub show_list_dashes {
  my $self = shift;
  my ($id) = $self->id;
  my $s=""; my $nb=0;
  $self->debug_msg(4, "Category::show_list_dashes($id)");

  my $c=DBAccess->sqlSelectMany("Name, iid",
                               "instance",
                               "parentiid=$id and isa='Category'",
                               " order by instanceOrder"
                              );

  while(my ($name, $sonid)=$c->fetchrow()) {
    my $son = Gizmo->new($sonid);
    $nb++;
    $s .= "<li><a  class='catlistsub' href='".$self->make_url("index.pl?id=$sonid")."'>$name</a>\n";
    #$s .= $son->show_ac_button();
    #$s .= $son->show_cut_button();
    #$s .= $son->show_up_down_buttons();
    $s  .= $son->get_buttons();
    $s .= "\n<blockquote>\n";

    $s .= "\n</blockquote>\n";
    $s .= "\n<blockquote>\n";
    $s .= $son->show_childs_list(type=>'Item',decorator=>'HTML::Decorator::DashDecorator');
    $s .= "\n</blockquote>\n</li>\n";
  }
  $c->finish();

  $s = "" if ($nb ==0);
  return ($s);
}

##########################################
### show_list_cat_first
### format category in catfirst look
##########################################
sub show_list_cat_first {
  my $self = shift;
  my ($id) = $self->id;
  my $s="";
  my $sons;
  my $nb=0;
  $self->debug_msg(3, "Category::show_list_cat_first($id)");

  # number of item to display
  my ($cnt) = DBAccess->sqlSelect("count(*)", "instance",
                                 "parentiid=$id and isa='Category'"
                                );
  $s.="<table border=\"0\" cellspacing=\"0\" width=\"100%\" valign='top'>\n";
  $s .= "<tr><td valign='top' width=\"50%\">\n";
  ## get the records
  my $c=DBAccess->sqlSelectMany("name, iid",
                               "instance",
                               "parentiid=$id and isa='Category'",
                               " order by instanceorder"
                              );
  #$s .= "$nb<br />";
  while(my ($name, $sonid)=$c->fetchrow()) {
    my $son = Gizmo->new($sonid);
    if($son->is_allowed_to_do('show', $USER)) {
      if($nb >= ($cnt/2)){
        $cnt=10000;
        $s.="\n\n</td><td valign='top' width=\"50%\">\n\n";
      }
      $nb++;
      $s.= "<table cellspacing=\"7\" cellpading=7 border=\"0\"><tr><td valign='top'>";
      $s.="<a class='catlistsub' href='".$self->make_url("index.pl?id=$sonid")."'>$name</a>\n";
      #$s .= $son->show_ac_button();
      #$s .= $son->show_cut_button();
      #$s.= $son->show_up_down_buttons();
      $s  .= $son->get_buttons();
      $s.= "</td></tr></table>\n";
    }
  }
  $c->finish();
  $s.="\n\n</ul></td></tr></table>\n<HR width=\"100%\">\n\n";

  $s = "" if($nb==0);
  return ($s);
}

##########################################
### show_list_luli
### format category in luli look
##########################################
sub show_list_luli {
  my $self = shift;
  my $id = $self->id;
  my $s="";
  my $nb=0;

  $self->debug_msg(3, "Category::showcategorylist($id)");

  my $c=DBAccess->sqlSelectMany("name, iid, description",
                               "instance",
                               "parentiid=$id and isa='Category'",
                      " order by instanceOrder");
  while(my ($name, $sonid, $description)=$c->fetchrow()) {
    my $cat = Gizmo->new($sonid);
    if($cat->is_allowed_to_do('show', $USER)) {
      $nb++;
      $s.= "<li><a class='catlistsub' href='".$self->make_url("index.pl?id=$sonid")."'>$name</a>\n";
      $s.= $cat->get_buttons();
      $s.=  "- " if ($description);
      $s.= $description;
      #$s.= "<blockquote>\n";
      $s.= "<blockquote>\n";
      $s .= $cat->show_childs_list(type=>'Item',decorator=>'HTML::Decorator::BulletDecorator');
      #$s.= "\n</blockquote>\n";
      $s.= "\n</blockquote>\n</li>\n";
    }
  }
  $c->finish();

  $s = "" if ($nb ==0);
  return ($s);
}

##########################################
### show_list_twocol
### format category in twocol look
##########################################
sub show_list_twocol {
  my $self = shift;
  my $s="";
  my $sons;
  my $nb=0;
  $self->debug_msg(3, "Category::showcategorytwocol()");
  my $id = $self->id;

  ## number of item to display
  my ($cnt) = DBAccess->sqlSelect("count(*)",
                                 "instance",
                                 "parentiid=$id and isa='Category'"
                                );
  $s.="<table width=\"100%\" border=\"0\" cellspacing=\"10\" ><tr><td  valign='top'>\n\n";

  ## get the records
  my $c=DBAccess->sqlSelectMany("name, iid",
                               "instance",
                               "parentiid=$id and isa='Category'",
                               " order by instanceorder"
                              );
  while(my ($name, $iid)=$c->fetchrow()) {
    #$s .= "$nb<br />";
    if($nb >= ($cnt/2)){
      $cnt=10000;
      $s.="\n\n</td><td valign='top'>\n\n";
    }
    my $cat = Gizmo->new($iid);
    if($cat->is_allowed_to_do('show', $USER)) {
      $nb++;
      $s.= "<a class='catlistmain' href='".$self->make_url("index.pl?id=$iid")."'>$name</a>\n";
      #$s .= $cat->show_ac_button();
      #$s .= $cat->show_cut_button();
      #$s.= $cat->show_up_down_buttons();
      $s.= $cat->get_buttons();
      $s.= "<br />\n";
      $s.= $cat->show_sons();
      $s.= "<br />\n";
    }
  }
  $c->finish();
  $s.="</td></tr></table>\n\n";

  $s = "" if ($nb ==0);
  return ($s);
}

######################################
### show_sons
### format childs for show_list_twocol
######################################
sub show_sons {
  my $self = shift;
  my $iid = $self->id;
  my($inputquery, $nb, $MAXTOSHOW, $name, $curiid, $s, $points);
  $self->debug_msg(3,"Category::showsons(iid=$iid)");

  $s = "";
  # max link to show
  $MAXTOSHOW=4;

  # query db
  my ($c)=DBAccess->sqlSelectMany("name, iid", "instance",
                                 "parentiid=$iid and isa='Category'",
                                 " order by instanceOrder"
                                );
  # dressing
  $s.= "<font size=small>\n";
  $nb = 0;

  # loop around all elements
  while((my ($name, $curiid)=$c->fetchrow()) && ($nb<4)) {
    # XXX [claudio] how much of a performance hit we get from son thawing?
    my $son = Gizmo->new($curiid);
    if($son->is_allowed_to_do('show', $USER)) {
      $s.= ", " if ($nb > 0);
      $s.= "\n<a href='".$self->make_url("index.pl?id=$curiid")."'>$name</a>";
      $nb++;
    }
  }
  $c->finish();
  $points = "";
  $points = ",..." if ($nb >= 4);
  $s .= "$points<br />\n" if ($s);
  $s .= "</font>\n\n<!-- end of show_sons -->\n\n";
  $s = "" if ($nb ==0);
  return ($s);
}

sub get_news_children {
    my $self = shift;

    ## following extra_shere not needed, but there for effiency
    ##  if not in edit mode then restrict with show from
    ##  if edit mode is on won't restrict because might be owner
    #my $db_now = Utility->db_now();
    #my $extra_where = ($SESSION->admin_mode()) ? "" : "(showfrom is NULL OR showfrom <= '$db_now')";

    my @gizmo_list = @{$self->build_gizmo_list(
                           type=>'Newsitem',
  #                                additional_statements=>'order by showfrom desc, iid desc',
                           #where_conditions=>$extra_where,
                      )};
# MIKE - sort by instance order
#    # must sort by creation or show_from date
#    # schwartzian transform
#    @gizmo_list =
#      map{ $_->[0] }
#    sort{ $b->[1] cmp $a->[1] }
#      map{ [ $_,
#             ( $_->{show_from} && $_->{show_from} !~ /0000/ ) ? $_->{show_from} : $_->{creation_date} ] }
#      @gizmo_list;
    if (wantarray) {
        return @gizmo_list;
    } else {
        return \@gizmo_list;
    }
}

###########################################################
#### show_news_column
#### show the list of news for this category (news column)
###########################################################
sub show_news_column {
  my $self = shift;
  my $maxitem = shift;
  my $color = shift;

  my $id = $self->id;
  my $isa = $self->is_a;
  $self->debug_msg(3, "Category::show_news_column($id)");

  ## let's build the title
  #my ($cname) = DBAccess->sqlSelect("name", "instance", "iid=$id");
  # use the channel name if it exists
  my ($cname) = DBAccess->sqlSelect("name", "channel", "isa='Internal' AND iid=$id");
  $cname ||= $self->name();

  $cname = ($cname) ? $cname : "News";
#    print STDERR "in Category::show_news_column-- got cname $cname and iid $id\n";

  my $newscoltitlebgcolor;
  if ($color) {
      $newscoltitlebgcolor = $color;
  }
  else {
      $newscoltitlebgcolor = $PARAMS->{'newscoltitlebgcolor'};
  }
  $maxitem = ($maxitem)? $maxitem : $PARAMS->{'maxnewsitems'};

  my $title = $self->frame_this($cname, $newscoltitlebgcolor, "newscattitle");

  ## let's build the add to my page logo
#  my $channel = Channel->get_channel_for_iid($id);
#    print STDERR "calling show_add_to_my_page with args id $id\n";
#  my $logo = $channel->show_add_to_my_page($id);
  my $logo = $self->get_page_subscribe_button();

  #CKIM-SENDABLE
  if ($self->can('show_invite_button')) {
      #make sure I'm inviting to the category, not to a child
      my $invitable = Utility->get_structural_ancestor($self, __PACKAGE__);
      my $invite_button = $invitable->show_invite_button();
      if ($invite_button) {
          $invite_button = "<br />".Utility->get_spacer(5,1)."<br />".$invite_button;
          $logo .= $invite_button;
      }
  }

  ## let's build the items
  use Gizmo::Newsitem;
  my $i=0;
  my $items = "";
  ## following extra_shere not needed, but there for effiency
  ##  if not in edit mode then restrict with show from
  ##  if edit mode is on won't restrict because might be owner
  #my $db_now = Utility->db_now();
  #my $extra_where = ($SESSION->admin_mode()) ? "" : "(showfrom is NULL OR showfrom <= '$db_now')";


  my @gizmo_list = $self->get_news_children();

  # aju
  my $orig_list_size = $#gizmo_list + 1;
#   print STDERR "orig_list_size is $orig_list_size\n";
  while ( (my $nitem = shift(@gizmo_list)) && ($i < $maxitem) ) {
    # print STDERR "got newsitem: " . $nitem->show_in_column() . "\n";

    ## if you are allowed to see news item and showfrom is today or before
    ##      OR
    ## if you are allowed to modify the news item
    #
    # some of the following logic is moved to the class
    #if( ($nitem->is_allowed_to_do('show', $USER) && ($showfrom <= $now) ) ||
    #if( ($nitem->is_allowed_to_do('show', $USER) && $nitem->is_show_time() ) ||
    #    ($nitem->is_allowed_to_do('modify', $USER))
    if( $nitem->is_allowed_to_do('show', $USER) ) {
          $i++;
          $items.= $nitem->show_in_column();
    }
  }

  unless ($items) { return ''; }

  #$c->finish();
  $self->debug_msg(4, "Category::show_news_column: found $i newsitems");
#   print STDERR "i is $i and maxitem is $maxitem\n";
  return "" if (!$items && !$logo);
  $items.= "\n<a href='".$self->make_url("index.pl?iid=$id&amp;isa=Category&amp;op=show_news")."'>More...</a>\n" if ($orig_list_size > $maxitem);
  $items ||= "&nbsp;";

  #aju
  my $newscolbgcolor;
  if (Metadot::Utils::Theme->use_theme_info()) {
      $newscolbgcolor = Metadot::Utils::Theme->get_theme_default_bg();
  }
  else {
    $newscolbgcolor = $PARAMS->{'newscolbgcolor'};
  }

  ## let's render the news column
  my $rs = "
<table width=\"155\" border=\"0\" bgcolor='#$newscolbgcolor' cellpadding=\"3\" cellspacing=\"0\">
<tr><td valign='top' align='center'>
$title
</td></tr>
<tr><td valign='top'>
 $items
</td></tr>
<tr><td valign='top' align='center'>
$logo
</td></tr>
</table>
";

  return $rs;
}

sub get_page_subscribe_button {
  my $self = shift;
  my $id = $self->id;
  ## let's build the add to my page logo
  my $channel = $self->get_page_channel();
#    print STDERR "calling show_add_to_my_page with args id $id\n";
  if ($channel) {
      return $channel->show_add_to_my_page($id);
  }
}

sub get_page_channel {
  my $self = shift;
  my $id = $self->id;
  return Channel->get_channel_for_iid($id);
}


# Gets the page's channel. Creates a channel if one does not exist.
sub create_channel {
    my $self = shift;
    my $args = shift || {};

    my $id   = $args->{iid} || $self->id();
    my $name = $args->{name} || $self->name();


    my $channel = $self->get_page_channel();
    unless ($channel) {
        $channel = Channel->new();
        my $cid = $channel->create($id, "Internal", $name, "$name channel");
    }

    return $channel;
}



########## POLLS ###############
#sub get_poll_children {
#    my $self = shift;
#    return @{$self->build_gizmo_list(type=>'Poll', additional_statements=>'order by showfrom desc, iid desc')};
#}

sub get_poll_children {
    my $self = shift;
    return @{$self->build_gizmo_list(type=>'Poll')};
}

sub show_poll_column {
  my $self = shift;
  my $maxitem = shift || 20;
  my $color = shift;

  my $id = $self->id;
  $self->debug_msg(3, "Category::show_polls_column($id)");

  ## let's build the title
  #my ($poll_cname) = DBAccess->sqlSelect("name", "instance", "iid=$id");
  #$poll_cname = ($poll_cname) ? "$poll_cname Poll" : "Poll";
  my $poll_cname = $self->name() . " Poll";

  my $newscoltitlebgcolor;
  if ($color) {
      $newscoltitlebgcolor = $color;
  }
  else {
     $newscoltitlebgcolor = $PARAMS->{'newscoltitlebgcolor'};
  }

#  print STDERR "in show poll w/ color $newscoltitlebgcolor\n";
  my $poll_title = $self->frame_this($poll_cname, $newscoltitlebgcolor, "newscattitle");

  my $channel = Channel->new();
  my $logo = ""; #$channel->show_add_to_my_page($id);

  # XXX iterate on all polls in this category
  my $parent_id = $FORM{iid} || '0';;

  ## let's build the items
  my $i=0;
  my $poll_items = "";

  my @gizmo_list = $self->get_poll_children();
  while ( (my $polly = shift(@gizmo_list)) && ($i < $maxitem) ) {
    if($polly->is_allowed_to_do('show', $USER)) {
      if ($i++ > 0) { $poll_items .= "<hr>"; }
       $poll_items .= $polly->show_in_column();
    }
  }

  unless ($poll_items) { return ''; }

  my $rs;
  #aju
  my $newscolbgcolor;
  if (Metadot::Utils::Theme->use_theme_info()) {
      $newscolbgcolor = Metadot::Utils::Theme->get_theme_default_bg();
  }
  else {
    $newscolbgcolor = $PARAMS->{'newscolbgcolor'};
  }

  $rs = "
<br /><table width=\"155\" border=\"0\" bgcolor=$newscolbgcolor cellpadding=\"3\" cellspacing=\"0\">
<tr><td valign='top' align='center'>
$poll_title
</td></tr>
<tr><td valign='top'>
$poll_items
</td></tr>
<tr><td valign='top' align='center' colspan='2'>
$logo
</td></tr>
</table>
";

  return $rs;
}

sub get_calendar_children {
    my $self = shift;

    $DISPLAY->debug_msg(4, "get_calendar_children");

    # get all calendar types
    my @types = ();
    foreach my $type ($self->get_gizmo_list()) {
        if (UNIVERSAL::isa($type, 'Calendar')) {
            $DISPLAY->debug_msg(5, "$type is a calendar");
            push @types, $type;
        }
    }

    my $children = $self->build_gizmo_list(types=>\@types);
    my @results;

    # filter out calendars that are not in mini mode
    foreach my $child (@$children)
    {
        if ($child->get_view_mode() eq Calendar::VIEW_MINI())
        {
            push @results, $child;
        }
    }

    return @results;
}

sub show_calendar_column {
  my $self = shift;
  my $color = shift;
  my $caller_id = shift;
  my $caller_op = shift;

  my $id = $self->id;
  $self->debug_msg(3, "Category::show_calendar_column($id)");


  # XXX iterate on all calendars in this category
  my $parent_id = $id || '0';

  ## let's build the items
  my $i=0;
  my $cal_items = "";

  my @gizmo_list = $self->get_calendar_children();
  for my $cal (@gizmo_list) {
      $self->debug_msg(5, "Showing calendar child $cal");
    if($cal->is_allowed_to_do('show', $USER)) {
       $i++;
       $cal_items .= $cal->show_in_column($color, $caller_id, $caller_op);
    }
  }
  $cal_items .= "<br />" if $cal_items;

  return $cal_items;
}

###########################################################
#### show_news_list
#### show the list of news for this category (page format)
###########################################################
sub show_news_list  {
  my $self = shift;
  my ($startfrom, $maxitem) = @_;
  $self->debug_msg(4, "Category::show_news_list(start=$startfrom, max=$maxitem)");
  my $id = $self->id;
  my $s="";

  $maxitem = ($maxitem)? $maxitem : 20; #$params->{maxnewsitems};        #PREVIOUS NEXT MAGIC
  $startfrom   =0  if(!$startfrom); #PREVIOUS NEXT MAGIC
  my $from = $startfrom;        #PREVIOUS NEXT MAGIC

  my $select_from = 'instance';
  my $where = "parentiid=$id and isa='Newsitem'";
  my $other = "order by showfrom desc";

  my $c = DBAccess->sqlSelectMany("iid, showfrom,'%b %e, %Y'", $select_from, $where, $other);

  my $i=0;
  #my ($totalrows)=$c->rows;        #PREVIOUS NEXT MAGIC
  # rows function is not reliable for counting the rows retrieved by SELECT
  my ($totalrows) = DBAccess->sqlSelect("count(*)",$select_from, $where, $other);

  my $nitem;
  my @row_colors = ('#FFFFFF','#F4F4F4');
  $s .= '<table CELLPADDING="3" CELLSPACING="0" border="0" WIDTH="100%">';
  while((my ($iid, $showfrom)=$c->fetchrow()) && ($i < $maxitem)) {
    if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC
    $nitem = Newsitem->new($iid);
    # CKIM: check permissions before adding to list
    if ($nitem->is_allowed_to_do('show')) {
        my $nitem_bgcolor = $row_colors[$i%2];
    # aju 07-24-02: fix newsitem count
    #      print STDERR "in show_news_list, showfrom is: $showfrom\n";
        my $item_show = $nitem->show_in_list(undef, Utility->format_datetime($showfrom, 'M d, Y'));
        if ($item_show ne '') {
            $s.= $item_show;
            $i++;
        }
        $s.= "\n";
    }
  }
  $s .= '</table>';
  $c->finish();

  my $url = "index.pl?iid=$id&amp;isa=Category&amp;op=show_news";
  my ($nf) = Components->get_previous_next_button($maxitem, $i, $totalrows, $startfrom, $url); #PREVIOUS NEXT MAGIC
  my $page = "$nf\n$s$nf\n\n";
  return ($page);
}

################################################
### frame_this
### return title in frame (for show_news_column)
################################################
sub frame_this {
  my $self = shift;
  my ($str, $color, $class)=@_;
  if (substr($color,0,1) ne "#") {
      $color = "#$color";
  }
  $self->debug_msg(5, "framethis(str=$str. color=$color)");
  return "\n\n<table border='0' cellspacing=\"0\" cellpadding=\"5\" width=\"150\">\n<tr>\n<td bgcolor='$color' class='$class' align='center'>\n$str</td></tr></table>\n\n";
}
#########################################
sub show_navigation_column {
    my $self = shift;
    my $depth = shift || 0;
    my $expanded_cat_hash = shift;
    my $expanded_cat = defined($expanded_cat_hash) ? $expanded_cat_hash->{id} : 0;
    my @expanded_childs = @_;
    my $id = $self->id();
    my $indent = "&nbsp;&nbsp;";
    my $s = "" ;

    ErrorHandling->printError("Hey man, I'm looping like crazzy here (Category::show_navigation_column)!!") if ($depth > 100);

    my $shift = $indent x $depth ;

    # Allow this to work with gizmo shortcuts
    #my $cats = DBAccess->sqlSelectMany("iid, name","instance","parentiid=$id and isa='Category'","order by instanceorder");  #get all the kids
    #while (my ($ciid, $cname) = $cats->fetchrow()) {                                                                    #if the category has kids
    #  my $cat = Gizmo->new($ciid);                                                                                                  #create an instance of the child category
    my @cat_list = @{ $self->get_navigation_children() };
    for my $cat (@cat_list) {
        my $ciid = $cat->get_iid();
        my $cname = $cat->get_name();

        if ($cat->is_allowed_to_do('show', $USER)) { #make sure that the person can view the child
            #aju
            my $up_down = $cat->show_up_down_buttons($ciid,'Category');#create up and down buttons
        my $del_button = $cat->show_del_button;
            my $buttons = $up_down.$del_button;
        my $row ; my $link;

        if ($self->id() == $ciid && $cat->get_image_url('active_image')) {
        $link = $cat->show_as_image();
        }
        elsif ($cat->get_image_url('display_image')) {
        $link = $cat->show_as_roll_over();
        }
        else {
            my $style;
            if ($ciid == $expanded_cat) {
                $style = 'catlistmain';
            } else {
                $style = 'catlistsub';
            }
            $link = $cat->show_as_nav_line(style=>$style);
        }
        my $level = $depth;
        while ($level) {
        $link = $self->link_indent($link);
        $level-- ;
        }
        $row = $self->base_row($link, $buttons);
            $s .= $row;
            if ($ciid == $expanded_cat) {#if the child is the one we are working on,
        $s .= $cat->show_navigation_column($depth+1,@expanded_childs);
        }
        }#if the user can't view it, we don't do anything
    }#if it doesn't have kids, we don't do anything
    #$cats->finish();

    return $s;
}

sub show_navigation_column_flex {
    my $self = shift;
    my %args = @_;
    my $s = "" ;

    # process args
    #print STDERR "CKIM: column_flex RECEIVED args: depth='$args{depth}', path='@{$args{path}}', unconditional='$args{unconditional_expansion_depth}', pad=$args{pad}\n";

    my $unconditional_expansion_depth = $args{unconditional_expansion_depth} || 0;
    my $depth = $args{depth} || 0;
    $args{depth} = $depth+1;

    # get the category path
    my $cat_hash = shift @{ $args{path} };
    my $cat_id = $cat_hash->{id};
    return unless defined $cat_id;
    my $cat = Gizmo->new($cat_id);

    #print STDERR "CKIM: column_flex PROCESSED args: depth='$args{depth}', path='@{$args{path}}', unconditional='$args{unconditional_expansion_depth}'\n";

    # determine the children
    my @child_list = @{ $cat->get_navigation_children() };

    # special case: if the next level in the path does NOT exist in the child_list, then this
    # path is artificial. in this case, just display the next level in the path
    if (defined $args{path}->[0]->{id}) {
        my $next_id = $args{path}->[0]->{id};
        my $next_was_found = 0;
        for (my $i=0; (!$next_was_found && $child_list[$i]); $i++) {
            $next_was_found ||= ($child_list[$i]->id() == $next_id);
        }
        # the special case occurred, therefore replace the child_list
        unless ($next_was_found) {
            @child_list = (Gizmo->restore($next_id));
        }
    }


        ## generate a list of all cat ids in path
        my @path_id_array = ();
        my $entry = {};
        foreach $entry (@{ $args{path} }) {
            push (@path_id_array, $entry->{id});
        }

    # process each child
    foreach my $child (@child_list) {
        my $child_id = $child->id();
        # determine whether the child should be shown
        # - if visible to the user
        if ($child->is_allowed_to_do('show', $USER)) {
            my $up_down = $child->show_up_down_buttons($child_id,'Category');                  #create up and down buttons
            my $del_button = $child->show_del_button;
            my $buttons = $up_down.$del_button;
            my $row;

            if (($self->id() == $child_id || Utility->array_contains($child_id, $args{'path'})) && $child->get_image_url('highlight_image')) {
                $row = $child->show_as_image();
            }
            elsif ($child->get_image_url('display_image')) {

                $row = $child->show_as_roll_over();
            }
            else {
                my $style;
                if (Utility->array_contains($child_id, \@path_id_array)) {
                    $style = 'catlistmain';
                } else {
                    $style = 'catlistsub';
                }
                $row = $child->show_as_nav_line(style=>$style);
            }
            $row = $self->link_indent($row, $depth, $args{pad});
            $row = $self->base_row($row, $buttons);
            $s .= $row;
        }

        # determine whether I should recurse through this child
        # - if the child is part of the path
        # - if the depth < unconditional_expand_depth
        if (Utility->array_contains($child_id, \@path_id_array)) {
            # recurse normally
            #print STDERR "Expanding PATH $child (".$child_id.")\n";
            $s .= $self->show_navigation_column_flex(%args);
        } elsif ($depth < $unconditional_expansion_depth) {
            # change the path to JUST the child, and recurse
            my %local_args = %args;
            $local_args{path} = [ { id => $child_id } ];
            #print STDERR "Expanding UNCOND $child (".$child_id.")\n";
            $s .= $self->show_navigation_column_flex(%local_args);
        }
    }
    return $s;
}

sub show_latest_navigation_column {
  my $self = shift;
  my $depth = shift || 0;
  my $id = $FORM{iid} || 0;
  my $s = "" ;

  ErrorHandling->printError("Hey man, I'm looping like crazy here (Category::show_latest_navigation_column)!!") if ($depth > 100);

  # CKIM-SHORTCUTS: allow this to work with shortcuts
  #my $cats = DBAccess->sqlSelectMany("iid, name","instance","parentiid=$id and isa='Category'","order by instanceorder");  #get all the kids
  #while (my ($ciid, $cname) = $cats->fetchrow()) {                                                              #if the category has kids
  #  my $cat = Gizmo->new($ciid);                                                                                             #create an instance of the child category

  my @cat_list = @{ $self->get_navigation_children() };
  for my $cat (@cat_list) {
    my $ciid = $cat->get_iid();
    my $cname = $cat->get_name();

    if ($cat->is_allowed_to_do('show', $USER)) {                                    #make sure that the person can view the child
      my $up_down = $cat->show_up_down_buttons($ciid,'Category');                  #create up and down buttons
      my $del_button = $cat->show_del_button;
      my $buttons = $up_down.$del_button;
                my $row;
                if ($self->id() == $ciid && $cat->get_image_url('active_image')) {
                    $row = $self->base_row($cat->show_as_image(), $buttons);
                }
                elsif ($cat->get_image_url('display_image')) {
                    $row = $self->base_row($cat->show_as_roll_over(), $buttons);
                }
                else {
                    my $style;
                    if ($self->id() == $ciid) {
                        $style = 'catlistmain';
                    } else {
                        $style = 'catlistsub';
                    }
                    $row = $self->base_row($cat->show_as_nav_line(style=>$style), $buttons);
                }
                $s .= $row;
    }                                                                               #if the user can't view it, we don't do anything
  }                                                                                                                                                             #if it doesn't have kids, we don't do anything
  #$cats->finish();

  return $s;
}













#########################################
sub base_row {
    my $self = shift;

    my $impl = $self->get_implementation('CategoryImplementation','Metadot::Implementations::Gizmo::Category::Default');
    return $impl->base_row($self, @_);
#          my $self = shift;
#          my $link = shift;
#          my $up_down = shift;
#  #          my $up_down = shift || "";
#          my $s .="<tr><td nowrap=\"nowrap\" valign = top>&nbsp;</td><td><table cellpadding=\"0\" cellspacing=\"2\" border=\"0\"><tr><td valign = top><div style=\"white-space: nowrap\">$up_down</div></td><td>$link</td></tr></table></td></tr>";
#          return $s;
}

sub link_indent {
    my $self = shift;
    my $link = shift;
    my $depth = shift;
    my $pad = shift || '&nbsp;&nbsp;&nbsp;&nbsp;';

    my $linkpad = '';
    if (!$link) {
    $link = '';
    }
    unless (defined $depth) {
    $depth = 1;
    }
    for (my $i=0; $i<$depth; $i++) {
    $linkpad .= $pad;
    }
    return "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\"><tr><td>$linkpad</td><td>$link</td></tr></table>";
}

sub show_category_bar {
  my $self  = shift;
  my $highlighted_id = shift ;
  my $iid = shift; #iid is the iid of the page being viewed
  my $color = shift;
  $self->debug_msg(4, "Category::show_category_bar()");
#  print STDERR "in show_category_bar with highlighted id $highlighted_id and iid $iid and color $color\n";
  my $s = "";

  #CKIM-SHORTCUTS: need to allow GizmoShortcut to comment on this list.
  #                so, i'm using build_gizmo_list instead of a direct
  #                DBAccess call
  #my $cats = DBAccess->sqlSelectMany("iid, name","instance","parentiid=$id and isa='Category'","order by instanceorder");
  #my $count = $cats->rows() || 1;
  #while (my ($ciid, $cname) = $cats->fetchrow()) {
  #  my $cat = Gizmo->new($ciid);

  my @cat_list = @{$self->build_gizmo_list(
                                type=>'Category',
                    )};
  my $count = scalar(@cat_list) || 1;
  #print STDERR "CKIM: Category->show_category_bar for $self\n";
  for my $cat(@cat_list) {
      my $ciid = $cat->get_iid();
      my $cname = $cat->get_name();
      #print STDERR "CKIM: Category->show_category_bar processing ".$cat->{is_a}." $ciid ($cname)\n";

      #my $width = int(100/$count) ;
      if ($cat->is_allowed_to_do('show', $USER)) {
          my $link = "<a class='catheader' href='".$self->make_url("index.pl?iid=$ciid")."'>$cname</a>";
          if ($ciid == $iid) {
              $s .= "<b class='catheader'>$cname</b>&nbsp;&nbsp;&nbsp;&nbsp;\n";
          } elsif ($ciid == $highlighted_id) {
              $s .= "<b >$link</b>&nbsp;&nbsp;&nbsp;&nbsp;\n";
          } else {
              $s .= "$link&nbsp;&nbsp;&nbsp;&nbsp;\n";
          }
      }
  }
  $s ||= "<td>&nbsp;</td>";

  my $categorytitlebgcolor;
  if ($color) {
      $categorytitlebgcolor = $color;
  }
  else {
      $categorytitlebgcolor = $PARAMS->{'categorytitlebgcolor'};
  }
  if (substr($categorytitlebgcolor,0,1) ne "#") {
      $categorytitlebgcolor = "#$categorytitlebgcolor";
  }


  $s = "
<!-- Category Bar -->
<table border=\"0\"  bgcolor='$categorytitlebgcolor' width='100%' cellpadding=\"0\" cellspacing=\"0\"><tr><td align='center'>
<table border=\"0\" cellpadding=\"4\" cellspacing=\"0\"><tr><td>
$s
</td></tr></table>
</td></tr></table>
<!-- End ofCategory Bar -->
";

  $self->debug_msg(4, "Category::show_category_bar() FINISHED");
  return $s ;
}

sub show_latest_category_bar {
  my $self  = shift;
  my $highlighted_id = shift ;
  $self->debug_msg(4, "Category::show_latest_category_bar()");
  my $id = $self->id();
  my $current_iid = $FORM{iid};
  $current_iid = 0 unless Utility->is_valid_integer($current_iid); # secu_200401

  my $s = "";

  my ($cname) = DBAccess->sqlSelect("name","instance","iid=$current_iid");

    my $cat = Gizmo->new($current_iid);
    if ($cat->is_allowed_to_do('show', $USER)) {
      my $link = "<a class='catheader' href='".$self->make_url("index.pl?iid=$id")."'>$cname</a>";
        $s .= "<b class='catheader'>$cname</b>&nbsp;&nbsp;&nbsp;&nbsp;\n";
      }
  $s ||= "<td>&nbsp;</td>";

  my $categorytitlebgcolor = $PARAMS->{'categorytitlebgcolor'};

  $s = "
<!-- Category Bar -->
<table border=\"0\"  bgcolor='#$categorytitlebgcolor' width='100%' cellpadding=\"0\" cellspacing=\"0\"><tr><td align='center'>
<table border=\"0\" cellpadding=\"4\" cellspacing=\"0\"><tr><td nowrap=\"nowrap\">
$s
</td></tr></table>
</td></tr></table>
<!-- End ofCategory Bar -->
";

  return $s ;
}

sub show_category_bar_dropdown {
    my $self = shift;
    my %args = @_;
    $self->debug_msg(4, "Category::show_category_bar_dropdown()");

    my $caller = $args{caller};

    my $iid;
    if (defined($caller)) {
        $iid = $caller->{id};
    }
    $iid = $iid || $self->id() || 0;

    my $instance = Gizmo->restore($iid);

    my $s = $self->get_dropdown_javascript();

    $s .= "\n<script>\n";

    my $home_cat   = Gizmo->restore(0);
    my @child_list = @{ $home_cat->build_gizmo_list(type=>'Category') };
    my $count = 0;
    my $subcount = 0;
    my $textclass = '';
    my $textclassover = '';
    my $ancestor_found = 0;
    foreach my $child (@child_list) {
        $subcount = 0;
        if ($child->is_allowed_to_do('show', $USER)) {
            if ( !$ancestor_found && $instance->can('is_descendant_of') && $instance->is_descendant_of($child) ) { ## show as selected
                $textclass     = "clLevel0selected";
                $textclassover = "clLevel0overselected";
                $ancestor_found = 1; ## no more lookups
            } else {
                $textclass = "";
                $textclassover = "";
            }
           ## Menu item creation:
           ## myCoolMenu.makeMenu(name, parent_name, text, link, target, width, height, regImage, overImage, regClass, overClass , align, rows, nolink, onclick, onmouseover, onmouseout)
            $s .= "oCMenu.makeMenu('top" . $count ."','','" . quotemeta($child->name()) . "','index.pl?iid=" . $child->id() . "','','','','','','$textclass','$textclassover')\n";
            foreach my $sub_child (@{ $child->build_gizmo_list(type=>'Category') }) {
                if ($sub_child->is_allowed_to_do('show', $USER)) {
                    $s .= "oCMenu.makeMenu('sub" . $count . "x" . $subcount . "','top" . $count ."','" . quotemeta($sub_child->name()) . "','index.pl?iid=" . $sub_child->id() . "')\n";
                    $subcount++;
                }
            }
            $count++;
        }
    }

    $s .= "\noCMenu.construct()\n";
    $s .= "\n</script>\n";
}


sub get_dropdown_javascript {
    my $self = shift;

    my $public_directory = MetadotConfig->get_public_uploads_location();
    my $js =<<EOF
<script language="JavaScript1.2" src="/js/coolmenus4.js" type="text/javascript">
/*****************************************************************************
Copyright (c) 2001 Thomas Brattli (webmaster\@dhtmlcentral.com)

DHTML coolMenus - Get it at coolmenus.dhtmlcentral.com
Version 4.0_beta
This script can be used freely as long as all copyright messages are
intact.

Extra info - Coolmenus reference/help - Extra links to help files ****
CSS help: http://coolmenus.dhtmlcentral.com/projects/coolmenus/reference.asp?m=37
General: http://coolmenus.dhtmlcentral.com/reference.asp?m=35
Menu properties: http://coolmenus.dhtmlcentral.com/properties.asp?m=47
Level properties: http://coolmenus.dhtmlcentral.com/properties.asp?m=48
Background bar properties: http://coolmenus.dhtmlcentral.com/properties.asp?m=49Item properties: http://coolmenus.dhtmlcentral.com/properties.asp?m=50
******************************************************************************/
</script>
<script language="JavaScript1.2" src="$public_directory/category_dropdown.js" type="text/javascript"></script>
EOF
;

    return $js;
}

########################################

sub show_bottom {
    my $c = shift;                #self
    my $width= "100%";
    my $id= $c->{id};

    #### Clipboard
    my $s .= "\n\n<center>\n";
    if($SESSION->admin_mode()){
        $s .= Clipboard->show_select($id, undef, ['File']);
    }
    $s .= "\n</center>";

    # edit button
  my ($global_look) = $PARAMS->{globallook};
  if ($global_look ne 'usetemplate') {
    $s .= "<center>". $DISPLAY->get_traffic_lights_display_if_allowed( subject => $c) . $c->show_admin_button() . "</center>\n";
    }
    # footer
    $s .= $c->show_footer($id);
    $s .= $c->close_html;
    return $s;
}


# XXX [claudio] following methods used to enable recursive perms
sub allows_recursive_permissions {
    return 1;
}


my $ac_hashes;
sub _read_all_ac_hashes {
    my $self = shift;

    my @gizmo_classes = Gizmo->lists_gizmos();

    foreach my $gizmo_class ( @gizmo_classes ) {

    $ac_hashes->{$gizmo_class} = $gizmo_class->get_AC();

    }
}

my $all_bundles;
sub get_all_possible_bundles {
    my $self = shift;

    if( $all_bundles ) { return @$all_bundles }

    unless($ac_hashes) { $self->_read_all_ac_hashes() }

    my $ac = $self->get_AC();
    my %found_bundles;
    foreach my $bundle_id ( @{ $ac->{DISPLAY_ORDER} } ) {

    $found_bundles{$bundle_id}++;
    push @$all_bundles, $bundle_id;

    }

    foreach my $class_name ( keys %$ac_hashes ) {

    foreach my $bundle_id ( @{ $ac_hashes->{$class_name}->{DISPLAY_ORDER} } ) {

        unless( $found_bundles{$bundle_id} ) {

        push @$all_bundles, $bundle_id;
        }

        $found_bundles{$bundle_id}++;
    }
    }

    return ( @$all_bundles );
}


my $bundle_descriptions;
sub get_bundle_description_from_all_possible_bundles {
    my $self = shift;
    my $wanted_bundle_id =  shift || die 'must include bundle_id';


    if( $bundle_descriptions->{$wanted_bundle_id} ) {

    return $bundle_descriptions->{$wanted_bundle_id}
    }


    my $ac = $self->get_AC();

    my %found_bundles;
    foreach my $bundle_id ( @{ $ac->{DISPLAY_ORDER} } ) {

    $bundle_descriptions->{$bundle_id} = $ac->{BUNDLES}{$bundle_id}{DESC};
    $found_bundles{$bundle_id}++;
    }


    unless($ac_hashes) { $self->_read_all_ac_hashes() }

    foreach my $class_name ( keys %$ac_hashes ) {

    foreach my $bundle_id ( @{ $ac_hashes->{$class_name}->{DISPLAY_ORDER} } ) {

        unless( $found_bundles{$bundle_id} ) {

        $bundle_descriptions->{$bundle_id} = $ac_hashes->{$class_name}{BUNDLES}{$bundle_id}{DESC};
        $found_bundles{$bundle_id}++;
        }
    }
    }


    ( $bundle_descriptions->{$wanted_bundle_id} )
      || die "$wanted_bundle_id bundle was not found in any gizmo";

}




sub get_bundle_allowed_levels_from_all_possible_bundles {
    my $self = shift;
    my $bundle_id = shift || die 'must include bundle_id';
    my $user = shift || die "must provide reference to user object";

#    print STDERR "AT get_bundle_allowed_levels_from_all_possible_bundles -bundle id is $bundle_id\n";

    my $mini = 10;

    unless($ac_hashes) { $self->_read_all_ac_hashes() }

    foreach my $class_name ( keys %$ac_hashes ) {

    if( defined $ac_hashes->{$class_name}{BUNDLES}{$bundle_id}{MIN_LEVEL} ) {

        if( $mini > $ac_hashes->{$class_name}{BUNDLES}{$bundle_id}{MIN_LEVEL} ) {

        $mini = $ac_hashes->{$class_name}{BUNDLES}{$bundle_id}{MIN_LEVEL};
        }

    } else {

        # must do this to remove auto-vivified bundle
        delete $ac_hashes->{$class_name}{BUNDLES}{$bundle_id};
    }
    }

    my $maxi = $self->_get_bundle_maximum_level( $bundle_id, $user );

    my @result = grep {$_ >= $mini and $_ <= $maxi} $self->_get_level_numbers();

    return @result;
}

sub is_visible_in_add_child_lists {
    my $self = shift;
    return $is_visible_in_add_child_lists;
}


# CKIM: damnit! Yuri copied and pasted this code from elsewhere, and it doesn't quite work
#       in this case. Hasn't he ever heard of the Visitor pattern?
sub show_site_map {
    my $self = shift;
    my %args = @_;

    my $s = "<ul>\n";

    $DISPLAY->debug_msg(3, "$self\->show_site_map, id=".$self->get_id());

    # process args
    #print STDERR "CKIM: column_flex RECEIVED args: depth='$args{depth}', path='@{$args{path}}', unconditional='$args{unconditional_expansion_depth}', pad=$args{pad}\n";

    my $unconditional_expansion_depth = $args{unconditional_expansion_depth} || 0;
    my $depth = $args{depth} || 0;
    $args{depth} = $depth+1;

    my $bold_first_level = $args{bold_first_level};
    delete $args{bold_first_level};

    # get the category path
    my $cat_id = shift @{ $args{path} };
    return unless defined $cat_id;
    my $cat = Gizmo->new($cat_id);

    #print STDERR "CKIM: column_flex PROCESSED args: depth='$args{depth}', path='@{$args{path}}', unconditional='$args{unconditional_expansion_depth}'\n";

    # determine the children
    my @child_list = @{ $self->get_navigation_children() };

    # special case: if the next level in the path does NOT exist in the child_list, then this
    # path is artificial. in this case, just display the next level in the path
    if (defined $args{path}->[0]) {
        my $next_id = $args{path}->[0];
        my $next_was_found = 0;
        for (my $i=0; (!$next_was_found && $child_list[$i]); $i++) {
            $next_was_found ||= ($child_list[$i]->id() == $next_id);
        }
        # the special case occurred, therefore replace the child_list
        unless ($next_was_found) {
            @child_list = (Gizmo->restore($next_id));
        }
    }

    # process each child
    foreach my $child (@child_list) {
        my $child_id = $child->id();
        #print STDERR "Processing child $child, id='$child_id'\n";

        # determine whether the child should be shown
        # - if visible to the user
        if ($child->is_allowed_to_do('show', $USER)) {

            my $style;
            if (Utility->array_contains($child_id, $args{'path'})) {
                $style = 'sitemap';
        #       $style = 'catlistmain';
            } else {
                $style = 'sitemap';
        #       $style = 'catlistsub';
            }


            my $child_link = $child->show_as_nav_line(style=>$style);

            if ( $bold_first_level ) {
                $child_link = '<b>' . $child_link . '</b>';
            }

            $s .= "<li>$child_link\n";
        }

        # determine whether I should recurse through this child
        # - if the depth < unconditional_expand_depth
        if($depth < $unconditional_expansion_depth) {
            # change the path to JUST the child, and recurse
            my %local_args = %args;
            $local_args{path} = [$child_id];
            #print STDERR "Expanding UNCOND $child (".$child_id.")\n";
            #$s .= $self->show_site_map(%local_args);

            # CKIM: had to change this because the child may not be a Category
            $s .= Category::show_site_map($child, %local_args);
        }

        $s .= "</li>\n";
    }

    $s .= "</ul>\n";

    return $s;
}


sub page_template_enabled {
    my $self = shift;
    return $self->c2(@_);
}

sub page_has_custom_template {
    my $self = shift;
    my $args = shift || {};
    my $visited = $args->{visited} || {};

    my $result = $self->c2();
    unless ($result) {
        my $next_parent = $self->get_navigation_parent({exclude_self=>1});
        if (defined $next_parent && !$visited->{$next_parent->id()}++ && $next_parent->can('page_has_custom_template')) {
            $result = $next_parent->page_has_custom_template({ visited => $visited });
        }
    }

    return $result;
}


sub get_page_template {
    my $self = shift;

    # this actually doesn't work if the parent template changes but current page is in cache
#     unless ($self->{page_template}) {
#         if ($PARAMS->{use_controller}) {
#             my $template = Metadot::TemplateMgr->get_template(obj => $self, op => 'show');
#             $self->{page_template} = $template;
#         } else {
#             $self->{page_template} = Metadot::TemplateMgr->get_template(obj => $self, op => 'show') || $self->get_template($self->id) || '';
#         }
#     }
#
#     return $self->{page_template};

    # this way is slower but doesn't have the above problem
    my $result;
    if ($PARAMS->{use_controller}) {
        $result = Metadot::TemplateMgr->get_template(obj => $self, op => 'show');
    } else {
        $result = Metadot::TemplateMgr->get_template(obj => $self, op => 'show') || $self->get_template($self->id) || '';
    }

    return $result;
}





BEGIN {

    use Utility;

    if( Utility->is_module_available('CategoryExtensions') ) {
    eval { require CategoryExtensions };
    }

}


{
    my $search_params = {
             searchable_fields => [ qw( name description keywords) ],
             displayed_fields  => [ qw( description) ],
             child_isa => 'Separator',
             child_searchable_fields => [ qw( name description t1 ) ],
             child_displayed_fields => [ qw( description t1 ) ],
            };

    sub get_search_params {
    my $self = shift;

    return $search_params;
    }
}


# allow local templates in show
sub exec_show {
    my $self = shift;
    my $context = shift;

    # if this is a redirect category, return the redirect response
    if ($self->id() && $self->get_redirect_url()) {
        my $redirect_url = $self->get_redirect_url();

        if ($SESSION->admin_mode()) {
            return 'show_sub';
        } else {
            # do not redirect in admin mode, or there will be no way to edit this category
            return print $self->md_redirect($redirect_url);
        }
    }

    # if we are using a page template, return that
    if ($self->page_template_enabled() || $self->page_has_custom_template() ) {
        my $page_template = $self->get_page_template();
        if ($page_template) {
            return $page_template;
        }
    }

    # if we are using a skin layout
    if ($PARAMS->{use_controller} && $self->skin_layout()) {
        return 'show_skin_layout';

    # the default
    } else {
        return $self->SUPER::exec_show($context, @_);
    }

}


sub exec_show_printer_friendly {
    my $self = shift;
    if ($PARAMS->{use_controller} && $self->skin_layout()) {
        return 'show_printer_friendly_layout';
    } else {
        return $self->SUPER::exec_show_printer_friendly();
    }

}


sub skin_layout {
    my $self = shift;
    my $value = shift;

    if (defined($value)) {
        $self->c3($value);
    }

    return $self->c3();
}

sub get_category_areas {
    my $self = shift;
    my $args = @_;

    return unless ($PARAMS->{use_controller} && $self->skin_layout());

    my $iid = $self->id();
    my $area_id_field_name = LayoutArea->get_area_id_field_name();
    my @areas = ();

    my $areas_list = DBAccess->sqlSelectMany("iid", "instance", "parentiid = $iid and isa='LayoutArea'", "order by $area_id_field_name");
    while (my($area_iid) = $areas_list->fetchrow) {
            push (@areas, LayoutArea->new($area_iid));
    }

    wantarray ? return @areas : return \@areas;
}



sub generate_add_child_selection_list {
    my $self = shift;

    my $result = '';

    # pages under layout management cannot have gizmos directly added.
    unless ($self->skin_layout()) {
        $result .= $self->SUPER::generate_add_child_selection_list(@_);
    }

    return $result;
}


# need to make sure the paste contents are consistent with page areas
sub paste {
    my ($self, @args) = @_;

    $self->SUPER::paste(@args);

    if ($self->skin_layout()) {
        $self->_process_page_areas();
    }
}

sub prohibited_child_types {
    my $self = shift;
    my @result;

    # if using skin layouts, then separator is not allowed
    if ($self->skin_layout()) {
        push @result, 'Separator';
    }

    if (wantarray) {
        return @result;
    } else {
        return \@result;
    }
}

# Category separates Navigables, normal Gizmos, Calendars, Polls, and NewsItems into separate
# ordering spaces
sub get_instance_order_address_page {
    my $class = shift;
    my $child = shift;

    my $result;
    if (defined $child) {
        if ($child->isa('Metadot::Navigable')) {
            $result = 1;
        } elsif ($child->isa('Newsitem')) {
            $result = 3
        } elsif ($child->isa('Poll')) {
            $result = 4;
        } elsif ($child->isa('Calendar') && $child->get_view_mode() eq Calendar::VIEW_MINI()) {
            $result = 5;
        }
    }

    # the default
    $result = 2 unless (defined $result);

    #print STDERR "$class\->get_instance_order_address_page($child) returns '$result'\n";
    return $result;
}

sub get_highlight_image {
    my $self = shift;
    return $self->{highlight_image};
}

sub get_active_image {
    my $self = shift;
    return $self->{active_image};
}

sub get_display_image {
    my $self = shift;
    return $self->{display_image};
}

##### INVITE overwritten methods
sub get_invite_interface_message {
    my $self = shift;
    my $gizmo_name = $self->get_display_name();
    my $s = '';
        if ($USER->is_logged_in()) {
            $s .= <<EOM
<h1>Mail users to view this $gizmo_name.</h1>
Below, you can search for users you would like to mail
to view this $gizmo_name. You may also mail entire groups or type in the
email address of the user. Once
you\'ve finished, click on the 'Next' button to send an e-mail
to the people you have selected.
EOM
;
        } else {
            $s .= <<EOM
<h1>Mail users to view this $gizmo_name.</h1>
Below, you can type in the email address of the user. Once
you\'ve finished, click on the 'Next' button to send an e-mail
to the people you have selected.
EOM
;
        }
    return $s;
}

sub get_invite_title_keyword {
    return 'mail';
}

sub get_invite_title_keyword_proper {
    return 'Mail';
}

sub _get_invitation_subject_html_default_field_text {
    my $self = shift;
    my $gizmo_name = $self->get_display_name();
    return $PARAMS->{hostname} . " $gizmo_name";
}



### Override the method from UpdateNotification

sub is_update_notification_enabled {
    my $self = shift;
    my $field = 'i2';

    if (@_) {
        my $value = (shift) ? 1 : 0;  # store boolean as an int
        $DISPLAY->debug_msg(4, "is_update_notification_enabled is being set to '$value'");
        $self->$field($value);
        if ($value) {
            $self->get_cid();  # this guarantees I have a channel, allowing
                               # people to subscribe to my updates
        }
    } else {
        return $self->$field();
    }
}


















##### END INVITE overwritten methods
1;































__END__

=head1 NAME

Gizmo::Category - The basic container Gizmo.

=head1 SYNOPSIS

Note: this list is not yet complete in showing the method
return values.

    use Gizmo::Category;
    $category = Gizmo::Category->new ($id);

    $AC = $category->get_AC();
    $category->www_show;
    $category->www_show_news ;
    $category->process_edit_form ;
    $category->www_save ;
    $category->www_delete ;
    $category->www_delete_ok ;
    $form_html = $category->show_edit_form ($title);
    $boolean = $category->delete ($deleteson);
    $html = $category->show_simple ;
    $category_html = $category->show ;
    $html = $category->show_category_column;
    $html = $category->show_all_childs_list ;
    $html = $category->show_childs_list ($type, $decorator);
    $html = $category->show_category_list ;
    $html = $category->show_list_dashes ;
    $html = $category->show_list_cat_first ;
    $html = $category->show_list_luli ;
    $html = $category->show_list_twocol;
    $html = $category->show_sons ;
    $news_html = $category->show_news_column ([$maxitem]);
    $poll_html = $category->show_poll_column ([$maxitem]);
    $html = $category->show_news_list  ([$startfrom, $maxitem]);
    $html = $frame_this ($str, $color, $class);
    $html = $show_navigation_column  ([$depth, $expanded_cat, @expanded_childs]);
    $html = $category->show_latest_navigation_column ([$depth]);
    $html = $category->base_row ($link, [$up_down]);
    $html = $category->link_indent ($baserow);
    $html = $category->show_category_bar ($highlighted_id);
    $html = $category->show_latest_category_bar ($highlighted_id);
    $html = $category->show_bottom ;
    $category->www_add_child ($error_msg, $child_type);
    $category->www_create_child ($child_type);
    $version = Category->get_version();


=head1 DESCRIPTION

The Category gizmo contains instances of other gizmos (including other Category classes). It also handles some of the layout of the web interface to Metadot.
Category is descended from L<Gizmobuilder>.

=head2 Methods

Constructor; instantiates a new item or reinstates an existing item from the database.
$id is the unique identifier of the existing instance in the database:

        $category = Gizmo::Category->new([$id]);

Get the access control hash for this object:

        $AC = $category->get_AC();

Class method to return the version of the gizmo

        $version = Category->get_version();

Get the html and display the contents of the Category instance in the Portal's look and feel:

        $category->www_show;

Get the news items associated with this category instance, build and display the news page in the Portal's look and feel:

        $category->www_show_news;

Receive the form variables from the new Category edit form, validate their contents, store them in the data fields of the object, and call the save method on the object:

        $category->process_edit_form;

Receive the form variables from the modify Category form, validate their contents, store them in the data fields of the object, and call the save method on the object:

        $category->www_save;

Render and display the html to verify a user's request to delete the Category object from the database. This method also prompts the user for what to do with children of this Category:

        $category->www_delete;

Interface to the delete method for processing delete object requests:

        $category->www_delete_ok;

Generate and return the html form used to edit and add new Category objects.
$title is the title that should be displayed at the top of the form page:

        $form_html = $category->show_edit_form($title);

Delete the Category object from the database and handle its children in the manner specified by the user (from the www_delete method)
$deleteson contains a either "yes" or "no" and stores the user's choice on wether to delete children of this category or move them to its parent:

        $category->delete($deleteson);

Generate and return the html used to display the main content area and list of children for this Category. This method is only called if the category's display type setting is set to "new" or "latest":

        $html = $category->show_simple;

Generate and return the html used to display the main content area and list of children for this Category. This method is only called if the category's display type setting is not set to "new" or "latest":

        $category_html = $category->show;

Determine what layout to use for listing sub-categories, combine the sub-categories list with thelist of child objects for this object and return the html to display it:

        $html = $category->show_category_column;

Generate a list of children gizmos for the current Category object, build and return the html to present them in the category:

        $html = $category->show_all_childs_list;

Generate and return the html layout for each individual gizmo of the same type in the category.
$type is the gizmo type that we are handling
$decorator is the layout decorator we wish to use for this gizmo type:

        $html = $category->show_childs_list(type=>type, decorator=>decorator);

Generate the layout and html output for child Categories of the parent in dash format:

        $html = $category->show_list_dashes;

Generate the layout and html output for child Categories of the parent in catfirst format:

        $html = $category->show_list_catfirst;

Generate the layout and html output for child Categories of the parent in luli format:

        $html = $category->show_list_luli;

Generate the layout and html output for child Categories of the parent in twocol format:

        $html = $category->show_list_twocol

Generate the layout and html output for grandchild Categories listed under the child Categories in the twocol format:

        $html = $category->show_sons;

Build the News display box and return the html to render it:
$maxitem is the maximum number of news items to show in the display box.

        $html = $category->show_news_column([$maxitem]);

Build the Poll display box and return the html to render it:
$maxitem is the maximum number of Poll items to show in the display box.

        $html = $category->show_poll_column([$maxitem]);

Generate the html for the news items in the news page for this Category object.
$startfrom specifies which news item to begin the list with.
$maxitem specifies how many news items to show per page:

        $html = $category->show_news_list([$startfrom], [$maxitem]);

Generate html to put a colored title bar on the news column.
$str is the title text.
$color is the color of the background for the framed area.
$class is the style sheet class to apply to the title text:

        $html = $category->frame_this($str, $color, $class);

Generate html for the list of sub-categories to be used as the left side navigation column.
$depth is an integer describing the current recursion depth of the method call.
$expanded_cat is the sub-category we are currently processing.
@expanded_childs is an array of the remaining sub-categories to process:

        $html = $category->show_navigation_column([$depth],[$expanded_cat],[@expanded_childs]);

Generate html for the list of sub-categories to be used as the navigation column. The difference between this and "show_navigation_column()" is that this one is only used for categories whose layout type is set to "latest".
$depth is an integer describing the current recursion depth of the method call:

        $html = $category->show_latest_navigation_column([$depth]);

Generate html to properly display a base row in the navigation column.
$link is the url to be used in the href created by the base row.
$up_down is the html to draw the up/down buttons when edit is on:

        $html = $category->base_row($link, [$up_down]);

Generate html to properly indent the base row in the navigation column.
$baserow is the output from the base_row method:

        $html = $category->link_indent($baserow);





=cut

