#!/usr/bin/perl

#
# Test file for Category.pm
# $Id: Category.001.t,v 1.2.2.2 2005/07/01 22:41:27 nico Exp $
#

use strict;
use Test::More tests=>12;

use SmokeMetadot;
use Metadot::Testing::Mechanizer;
use DBAccess;
use MetadotConfig;
use Metadot::PortalFactory;

# set up info
my $class_name   = 'Category';
my $mech = Metadot::Testing::Mechanizer->new();
my $home_page = Metadot::PortalFactory->get_home_object();
my $admin_uid = MetadotConfig->get_reserved_uid('ADMIN');
my $test_page_name = '_FOO_TEST_PAGE_';


################################################################################
# Test: Make sure the category does not yet exist in navigation
################################################################################
ok(!$mech->instance_retrievable(iid=>$home_page->id(), expr=>$test_page_name),
   'Category does not yet exist in navigation');


################################################################################
# Test: Create a category
################################################################################
use_ok('Gizmo::'.$class_name);

my $page = $class_name->new({
    uid => $admin_uid,
    parent_id => $home_page->id(),
    name => $test_page_name,
});

isa_ok($page, $class_name);


################################################################################
# Test: Add the category to the db
################################################################################
my $page_id = $page->add();
ok(Utility->is_valid_integer($page_id), "Got a valid iid $page_id");

$page = $class_name->restore($page_id);
ok(defined($page));
isa_ok($page, $class_name);
ok($page->id() == $page_id);


################################################################################
# Test: Category is visible in navigation
################################################################################
ok($mech->instance_retrievable(iid=>$home_page->id(), expr=>$test_page_name),
   'Category was found in navigation');

################################################################################
# Test: Hide the category in navigation
################################################################################
$page->is_hidden(1);
$page->save();
ok(!$mech->instance_retrievable(iid=>$home_page->id(), expr=>$test_page_name),
   'Category was hidden in navigation');


################################################################################
# Test: Unhide the category in navigation
################################################################################
$page->is_hidden(0);
$page->save();
ok($mech->instance_retrievable(iid=>$home_page->id(), expr=>$test_page_name),
   'Category was revealed in navigation');


################################################################################
# Test: Remove the category from the db
################################################################################
$page->delete();
$page = $class_name->restore($page_id);
ok(!defined($page), "Page was removed from db");


################################################################################
# Test: Instance is visible from browser
################################################################################
ok(!$mech->instance_retrievable(iid=>$home_page->id(), expr=>$test_page_name),
   'Category was removed from navigation'
);

#################################################
# That's it!!!!
#################################################


