#!/usr/bin/perl

#
# Test file for Category.pm
# $Id: Category.002.t,v 1.4.2.2 2005/07/01 22:41:27 nico Exp $
#

use strict;
use Test::More tests=>34;

use SmokeMetadot;
use Metadot::Testing::Mechanizer;
use DBAccess;
use MetadotConfig;
use Metadot::PortalFactory;

# set up info
my $class_name   = 'Category';
my $mech = Metadot::Testing::Mechanizer->new();
my $home_page = Metadot::PortalFactory->get_home_object();
my $admin_uid = MetadotConfig->get_reserved_uid('ADMIN');
my $test_page_name = '_FOO_TEST_PAGE_002.t';


################################################################################
# Test description: this test checks if the security of a gizmo located in
# a public  but with a parent page that is not public. We check that the
# access to the gizmo is denied...
################################################################################

################################################################################
# Test: Create a category #1
################################################################################
#use_ok('Gizmo::'.$class_name, "we certainly can use this!");
use_ok('Gizmo::'.$class_name);

my $num = 1;
my $page = $class_name->new({
    uid => $admin_uid,
    parent_id => $home_page->id(),
    name => $test_page_name . $num,
});

isa_ok($page, $class_name, "page has the rigt class");

################################################################################
# Test: Add the category to the db
################################################################################
my $page_id = $page->add();
ok(Utility->is_valid_integer($page_id), "Got a valid iid $page_id");

#make it invisible to anonymous
$page->set_bundle_level( 'DISP', 8 );
$page->set_bundle_level( 'MOD', 8 );
$page->save();

$page = $class_name->restore($page_id);
ok(defined($page), "page 1 restored OK");
isa_ok($page, $class_name, "page 1 has the correct class");
ok($page->id() == $page_id, "page 1 has the corrrect id");




################################################################################
# Test: Create a second category with the first as its parent
################################################################################
$num = 2;
my $page2 = $class_name->new({
    uid => $admin_uid,
    parent_id => $page_id,
    name => $test_page_name . $num,
});

isa_ok($page2, $class_name, "page 2 has the right class");

################################################################################
# Test: Add page 2 to the db
################################################################################
my $page_id2 = $page2->add();
ok(Utility->is_valid_integer($page_id2), "Got a valid iid $page_id2");

#Make the access public
$page2->set_bundle_level( 'DISP', 0 );
$page2->set_bundle_level( 'MOD', 0);
$page2->save();

$page2 = $class_name->restore($page_id2);
ok(defined($page2), "page 2 restored properly");
isa_ok($page2, $class_name, "page 2 has the right class");
ok($page2->id() == $page_id2, "page 2 id is correct");


################################################################################
# Test: Create a 3rd category with the 2nd as its parent
################################################################################

$num = 3;
my $page3 = $class_name->new({
    uid => $admin_uid,
    parent_id => $page_id2,
    name => $test_page_name . $num,
});

isa_ok($page3, $class_name, "page 3 has the right class");


################################################################################
# Test: Add page 3 to the db
################################################################################
my $page_id3 = $page3->add();
ok(Utility->is_valid_integer($page_id3), "Got a valid iid for p $page_id2");

#Make the access public
$page3->set_bundle_level( 'DISP', 0 );
$page3->set_bundle_level( 'MOD', 0);
$page3->save();

$page3 = $class_name->restore($page_id3);
ok(defined($page3), "p3 is restored");
isa_ok($page3, $class_name, "p3 has the right class");
ok($page3->id() == $page_id3, "p3 id matches");

################################################################################
#
#
#
# The real testing starts here
#
#
################################################################################

################################################################################
# Test: login in as admin
################################################################################
ok($mech->login(account_name=>'admin', password=>'123456'), "logging in as admin");
$mech->get_page(iid=>0);
ok($mech->content() =~ 'Not Admin?', "Login was successful");

################################################################################
# Test: Can we see page 2: should be yes
################################################################################
#fetching page 2
$num = 2;
ok($mech->get_page(iid=>$page_id2), "fetching page 2 ($page_id2)");
ok( ($mech->status == 200), "fetching p2 status == 200");
like($mech->content(), qr/$test_page_name$num/, "testing name of p2 available");



################################################################################
# Test: Can we see page 3: should be yes
################################################################################
#fetching page 3
$num = 3;
ok($mech->get_page(iid=>$page_id3), "fetching page 3 ($page_id3)");
ok( ($mech->status == 200), "fetching p3 status == 200");
like($mech->content(), qr/$test_page_name$num/, "testing name of p3 available");


################################################################################
#
#
#
#
# Test: NOT LOGGED IN
#
#
#
#
###############################################################################
ok($mech->logout, "login out now...");

################################################################################
# Test: Can we see page 2: should fail
#
###############################################################################
$num = 2;
ok($mech->get_page(iid=>$page_id2), "fetching page 2 ($page_id2)");
ok( ($mech->status == 200), "fetching status p2 == 200");
unlike($mech->content(), qr/$test_page_name$num/, "testing page name 2 is unavailable");

################################################################################
# Test: Can we see page 3: should fail
#
################################################################################

$num = 3;
ok($mech->get_page(iid=>$page_id3), "fetching page 3 ($page_id3)");
ok( ($mech->status == 200), "fetching page 3 status == 200");
unlike($mech->content(), qr/$test_page_name$num/, "testing page name 3 is unavailable");





################################################################################
#
#
#
# Cleaning up behind us before we leave...
#
#
################################################################################

################################################################################
# Test: Remove the category from the db
################################################################################
$page->delete();
$page = $class_name->restore($page_id);
ok(!defined($page) || !defined($page->id()), "Page was removed from db");

################################################################################
# Test: Remove the category from the db
################################################################################
$page2->delete();
$page2 = $class_name->restore($page_id2);
ok(!defined($page2), "Page 2 was removed from db");

################################################################################
# Test: Remove the category from the db
################################################################################
$page3->delete();
$page3 = $class_name->restore($page_id3);
ok(!defined($page3), "Page 3 was removed from db");


#################################################
# That's it!!!!
#################################################


