###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Clipboard;

use strict;

use Metadot qw(%FORM $USER $PARAMS);
use Carp;
use Gizmo;
use Channel;
use Gizmo::Category;
use DBAccess;
use Utility;

use vars qw(@ISA);
@ISA=qw(Category);
my $version = "4.5";

sub get_version {
    return $version;
}


sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

    $self->{display_look}='twocol';
}


#########################################
# WWW FUNCTIONS


###########################################################
#### show_select
#### show the list of items as select list for pasting
###########################################################
sub show_select {
    my $self = shift;
    my $paste_into_iid = shift;
    my $allowed_isa_list = shift;
    my $denied_isa_list = shift;

    my $id = $USER->get_clipboard_iid();

    my ($cname) = DBAccess->sqlSelect("name", "instance", "iid=$id");
    $cname = "" if (!defined $cname);

    my @items = $self->get_items($paste_into_iid, $allowed_isa_list, $denied_isa_list);
    my $i=0;
    my @cols = ();

    foreach my $child (@items) {
        my $child_iid = $child->get_id();
        my $name = $child->get_name() || Utility->html2text($child->get_description()) || $child->get_gizmo_name() || $child->get_isa() || 'Unnamed Gizmo';

        #print STDERR "Clipboard item: name=$name\n\n";

        my $max_len = 50;
        if (length($name) > $max_len) {
            $name = Utility->trim_clean($name, $max_len);
        }

        $cols[$i % 4] .= "<input type=\"checkbox\" name=\"box_$child_iid\" value=\"$child_iid\">&nbsp;$name<br />\n";
        $i++;

    }

    return "" if ($i==0);
    $i = 4 if ($i>4);
    #  my $width = 100 / $i ;
    my $columns = join("\n",map("<td align=\"left\" valign=\"top\">$_</td>",@cols));

    my $imageslocation = $PARAMS->{'imageslocation'};

    # instead of show_standalone mode, which was clunky, we
    # now allow paste directly into a container
    my $s = <<EOM
<script type="text/javascript">
  function pasteInto(targetId) {
    document.pasteform.id.value = targetId;
    document.pasteform.submit();
  }
</script>
EOM
;

    $s .= "
<FORM name=\"pasteform\" action=\"index.pl\" method=\"POST\">
<input type=\"hidden\" name=\"id\" value=\"$paste_into_iid\"/>
<input type=\"hidden\" name=\"op\" value=\"paste\"/>
".$self->make_hidden()."
<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\">
  <tr>
    <td class=\"clipboard_container_outer\">
      <table border=\"0\" cellspacing=\"1\" cellpadding=\"0\">
        <tr>
          <td align=\"center\" bgcolor=\"#FCCA2C\">
            <table width=\"100%\">
              <tr>
                <td align=\"left\">
                  <b>$cname</b>
                </td>
                <td align=\"right\">
                  <input border=\"0\" type=\"image\" src=\"$imageslocation/obj/paste.gif\" alt=\"Paste selection\"/>
                  <input border=\"0\" name=\"del\" type=\"image\" src=\"$imageslocation/obj/del.gif\" alt=\"Delete selection\"/>
                </td>
              </tr>
            </table>
          </td>
        </tr>
        <tr>
          <td align=\"center\" class=\"clipboard_container\">
            <table width=\"100%\">
              <tr>
$columns
              </tr>
            </table>
          </td>
        </tr>
      </table>
    </td>
  </tr>
</table>
</FORM>
";

  return $s;
}






sub generate_paste_into_link {
    my $class = shift;
    my $target_iid = shift;
    my $allowed_isa_list = shift;
    my $denied_isa_list = shift || ['File'];

    my $result;
    my @items = $class->get_items($target_iid, $allowed_isa_list, $denied_isa_list);
    if (defined $target_iid && @items) {
        $result = "javascript:pasteInto($target_iid);";
    } else {
        $result = '';
    }

    return $result;
}




# returns the items the clipboard
sub get_items {
    my $self = shift;
    my $paste_into_iid = shift;
    my $allowed_isa_list = shift;
    my $denied_isa_list = shift;

    my @items;
    # hack for caching the items list
    my $cache_var = '__ClipboardItems';
    if ($FORM{$cache_var}) {
        @items = @{$FORM{$cache_var}};
        #print STDERR "Clipboard got items from cache\n";
    } else {
        @items = ();
        $FORM{$cache_var} = \@items;

        my $id = $USER->get_clipboard_iid();

        my %allowed_isa = map {$_, 1} @$allowed_isa_list;
        my %denied_isa = map {$_, 1} @$denied_isa_list;

        my ($c) = DBAccess->sqlSelectMany("iid, isa", "instance", "parentiid=$id", " order by showfrom desc, iid desc");

        while(my ($child_iid, $isa) = $c->fetchrow()) {
            next if %allowed_isa && !$allowed_isa{$isa};
            next if %denied_isa && $denied_isa{$isa};

            push @items, $isa->restore($child_iid);
        }

        $c->finish();
    }


    if (wantarray) {
        return @items;
    } else {
        return \@items;
    }
}




1;

__END__

=head1 NAME

Gizmo::Category::Clipboard -

=head1 SYNOPSIS

    use Gizmo::Category::Clipboard;

    $clipboard = Clipboard->new ($id);
    $html = $clipboard->show_select ($paste_into_iid,
                 $allowed_isa_list, $denied_isa_list);

    $version = Clipboard->get_version();

=head1 DESCRIPTION

=cut


