###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package ContactUs;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT);
use Metadot::Controllable;

use vars qw(@ISA);
@ISA=qw(GizmoBuilder Metadot::Controllable);

use CGI qw/escapeHTML/;

my $version = "4.5";

sub get_version {
    return $version;
}


{
    my $search_params = {
             searchable_fields => [ qw( name description keywords t1 t2 c1 c2 c3 c4 c5 t4 t5 ) ],
             displayed_fields  => [ qw( description t1 t2 c1 c2 c3 c4 c5 t4 t5 ) ],
            };

    sub get_search_params {
    my $self = shift;

    return $search_params;
    }
}

####################################

sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

  $self->_form_initialize();
  $self->set_field_info (
    "name", "<b>Corporation/Location</b>", 1,
    "t1", "<b>Address</b>", 0,
    "t2", "<b>City</b>", 0,
    "c1", "<b>State</b>", 0,
    "c2", "<b>Zip code</b>", 0,
    "c3", "<b>Country</b>", 0,
    "c4", "<b>Phone Number</b>", 0,
    "c5", "<b>Fax Number</b>", 0,

    "t4", "<b>Title</b>",  0,
    "t5", "<b>Address</b>", 0,
    );


  $self->{display_wrapped} = { show => 1,
                             };
}

sub get_gizmo_name {
   my $self = shift;
   return "Contact Us";
}

sub gizmo_list_header {
  my $class = shift;
  my $name = shift;

  my $output = "<h3>Contact Us</h3><hr width=\"100%\"><br />\n";

  return $output;
}

sub gizmo_list_decorator {
    return 'HTML::Decorator::NullDecorator';
}

#override one of the default form types.
sub _form_initialize {
    my $self = shift;

    $self->{fields}->{t1}->{formtype} = "text";
    $self->{fields}->{t2}->{formtype} = "text";
    $self->{fields}->{c3}->{formtype} = "text";
    $self->{fields}->{c4}->{formtype} = "text";
    $self->{fields}->{c5}->{formtype} = "text";
    $self->{fields}->{t3}->{formtype} = "text";
    $self->{fields}->{t4}->{formtype} = "text";
    $self->{fields}->{t5}->{formtype} = "text";
    $self->{fields}->{t6}->{formtype} = "text";
    $self->{fields}->{t7}->{formtype} = "text";
    $self->{fields}->{Name}->{formtype} = "text";

    my $ts = "<!-- intro text would go here -->
";
    $self->set_intro_text($ts);

}

##########################################
### show
### return one newsitem (page format)
##########################################
sub show {
    my $self = shift;
    $self->debug_msg(1, "ContactUs::show()");
    my $s="";
    #  print STDERR "in ContactUs:show()\n";

    my $location    = $self->get_name();
    my $address = $self->get_t1();
    my $city = $self->get_t2();
    my $state = $self->get_c1();
    my $zip = $self->get_c2();
    my $country = $self->get_c3();
    my $phone = $self->get_c4();
    my $fax = $self->get_c5();

    my $emails_ref = $self->build_email_list_array($self->{t4}, $self->{t5});
    my @emails = @{$emails_ref};

    my $email_list;
    foreach my $email_ref (@emails) {
    my %email_hash = %{$email_ref};
    $email_list .= "<b>$email_hash{title}</b>: <a href=\"mailto:$email_hash{addr}\">"
      .  "$email_hash{addr}</a><br />";
    }

    $s .= "<b>$location</b><br />";

    $s .= "<font size=\"-1\">";

    if( $address ) { $s .= "$address<br />" }
    if( $city ) { $s .= "$city," }
    if( $state or $zip ) { $s .= "$state $zip<br />" }
    if( $country ) { $s .= "$country<br />" }
    if( $phone ) { $s .= "Ph: $phone<br />" }
    if( $fax ) { $s .= "Fx: $fax<br />" }
    $s .= "<br />";
    if( $email_list ) { $s .= "$email_list<br /><br />" }

    $s .= "</font>";

    return $s;
}

##########################################
### show_in_list
### return one newsitem (list format)
##########################################
# aju: do I need this method?
sub show_in_list {
  my $self = shift;
  $self->debug_msg(1, "ContactUs::show_in_list()");
#    print STDERR "in ContactUs:show_in_list()\n";

  my $name        = $self->get_name();
  my $description = $self->get_description();
  my $url         = $self->get_url();
  my $panel       = $self->get_panel();

  $name = "<img src=$url>"; #if ($url) ;

  return "<b>$name</b> - $panel<br />\n$description<br />\n";
}

# I don't need this... do I?
##########################################
### show_in_column
### return one newsitem (news column format)
##########################################
#  sub show_in_column {
#    my $self = shift;
#    $self->debug_msg(1, "ContactUs::show_in_column()");
#    print STDERR "in ContactUs:show_in_column()\n";

#    my $iid             = $self->get_iid();
#    my $name            = $self->get_name();
#    my $description     = $self->get_description();
#    my $url             = $self->get_url();
#    my $longdescription = $self->get_long_description();
#    my $panel           = $self->get_panel();
#    my $s               = "";

#    if($longdescription || $url){
#      $name = "<a class='newsitemtitle' href='index.pl?id=$iid&amp;isa=ContactUs&amp;op=show'>$name</a>";
#    }

#    $s.= $panel;
#    $s.= "<table cellpadding=\"0\" cellspacing=\"0\"><tr><td class='newsitemtitle'>- $name\n</td></tr>\n";
#    $s.= "<tr><td class='newsitem'>$description<br />&nbsp;</td></tr></table>\n\n";

#    return $s;
#  }


sub show_summary {
    my $self = shift;
    my @args = @_;

    my $buttons     = $self->get_buttons();
    my $text = $self->show(@args);

    return $buttons.$text;
}


#  sub get_Height {
#   my $self = shift;

#   return $self->{height};
#  }

#  sub get_Width {
#   my $self = shift;

#   return $self->{width};
#  }

sub build_email_list_array {
    my $self = shift;
    my $titles = $self->t4() || shift;
    my $addrs = $self->t5() || shift;

    my @emails;
    my @email_titles = split(",",$titles);
    my @email_addr = split(",",$addrs);
    for (my $ctr = 0; $email_titles[$ctr]; $ctr++) {
        my %email_hash;
        $email_hash{'title'} = $email_titles[$ctr];
        $email_hash{'addr'} = $email_addr[$ctr];
        push(@emails, \%email_hash);
    }

    return \@emails;
}

sub build_email_list_strings {
    my $self = shift;
    my $array_ref = shift;
    my @email_array = @{$array_ref};

    my %strings;
    my @titles;
    my @emails;

    foreach my $hash_ref (@email_array) {
        my %data_hash = %{$hash_ref};
        push(@titles, $data_hash{title});
        push(@emails, $data_hash{addr});
    }

    $strings{titles} = join(",", @titles);
    $strings{addr}   = join(",", @emails);

    return \%strings;
}

sub show_edit_form {
    my $self = shift;
    my $title = shift;
    my $output = "";
    my $order = 0;

    my $titles = $self->t4();
    my $addrs = $self->t5();

    $self->debug_msg(4, "ContactUs::show_edit_form($title)");

    my $emails_ref = $self->build_email_list_array();
    my @emails = @{$emails_ref};

    $output .= "<p /><h1>" . $title . "</h1>\n";
    my $first_time = 1;
    $self->{t4} = "";
    $self->{t5} = "";

    $FORM{t4} = "";
    $FORM{t5} = "";

    foreach my $field (sort {$self->{fields}->{$a}->{order} <=> $self->{fields}->{$b}->{order}} keys (%{$self->{fields}})) {

        $order = $self->{fields}->{$field}->{order};
        next if !defined($order) || $order <= 0;

        if( $field eq 't4' ) {
            $output .= "<hr width=\"500\"><br /><b>Emails: </b><p /><table border=\"0\"><tr><td>";
        }

        $output .= $self->get_form_box($field);

#       if ($self->{fields}->{$field}->{required})
#           $output .= $self->get_form_box($field);
#       } else {
#           if ($first_time) {
#               $output .= "<hr width=\"500\"><br /><b>Emails: </b><p /><table border=\"0\"><tr><td>";
#               $first_time = 0;
#           }
#           $output .= $self->get_form_box($field);
#       }
    }

    $self->t4($titles);
    $self->t5($addrs);

    $output .= "</td><td align='center' valign='center'>"
            .  "<input type=submit name=add_email value=\"Add\">"
            .  "</td></tr></table>";

#   $output .= "<input type=hidden name=\"email_titles\" value=\"$email_title_list\">\n";
#   $output .= "<input type=hidden name=\"email_addr\" value=\"$email_addr_list\">\n";

    $output .= "<table border=\"0\">";

    my $cnt = 0;
    foreach my $email (@emails) {
        $output .= "<tr>";

        my $email_title = escapeHTML( $email->{title} );
        my $email_addr = escapeHTML( $email->{addr} );

        $output .= "<td><b>Title:</b></td>"
                .  "<td><input type=text name=\"emailtitle_$cnt\" value=\"$email_title\">"
                .  "</td><td><input type=submit name=\"save_$cnt\" value=\"Save\"></td>"
                .  "</tr><tr>"
                .  "<td><b>Address:</b></td>"
                .  "<td><input type=text name=\"emailaddr_$cnt\" value=\"$email_addr\">"
                .  "</td><td><input type=submit name=\"delete_$cnt\" value=\"Delete\"><br />"
                .  "</td></tr>";
        $cnt++;
    }
    $output .= "</table>\n";

    $output .= "<br /><br /><br /><i>(*) Mandatory field(s)</i><br />";

    return $output;
}

1;

sub process_edit_form {
    my $self = shift;
    my $submit_type;
    my $submit_pos;
    my $error;

    if ($FORM{add_email} eq "Add") {
        $submit_type = "add";
    } else {
        GRAB_BUTTON:
        foreach my $key (keys(%FORM)) {
            $key = lc($key);
            if (($key =~ m/^save_[\d]+/) || ($key =~ m/^delete_[\d]+/)) {
                my @submit_value = split("_", $key);
                $submit_type = lc($submit_value[0]);
                $submit_pos = $submit_value[1];
                last GRAB_BUTTON;
            }
        }
    }

    my $email_list_ref = $self->build_email_list_array();
    my @email_list = @{$email_list_ref};

    if (!$submit_type) {
#print STDERR "NO SUBMIT\n\n";
        if (($FORM{t4}) && ($FORM{t5})) {
            my %new_email;
            $new_email{title} = $FORM{t4};
            $new_email{addr}  = $FORM{t5};
            push(@email_list, \%new_email);
        } elsif (($FORM{t4}) && !($FORM{t5})) {
            $error = "You cannot provide an email title without an address.\n";
        } elsif (!($FORM{t4}) && ($FORM{t5})) {
            $error = "You cannot provide an email address without a title.\n";
        }
    } elsif ($submit_type eq "add") {
#print STDERR "ADD\n\n";
        if (($FORM{t4}) && ($FORM{t5})) {
            my %new_email;
            $new_email{title} = $FORM{t4};
            $new_email{addr}  = $FORM{t5};
            push(@email_list, \%new_email);
        } else {
            $error = "You must supply a title and an email address.\n";
        }
    } elsif ($submit_type eq "save") {
#print STDERR "SAVE\n\n";
        my $title = "emailtitle_".$submit_pos;
        my $addr  = "emailaddr_".$submit_pos;
        if (($FORM{$title}) && ($FORM{$addr})) {
            my %updated_email;
            $updated_email{title} = $FORM{$title};
            $updated_email{addr} = $FORM{$addr};
            $email_list[$submit_pos] = \%updated_email;
        } else {
            $error = "You must supply a title and an email address.\n";
        }
    } elsif ($submit_type eq "delete") {
#print STDERR "DELETE\nsubmit_pos = $submit_pos\n\n\n";
        splice(@email_list, $submit_pos, 1);
    }
    my $email_ref = \@email_list;
    my $strings_ref = $self->build_email_list_strings($email_ref);
    my %strings = %{$strings_ref};
    $FORM{t4} = $strings{titles};
    $FORM{t5} = $strings{addr};

    $self->save_me($submit_type, $error);
}

sub www_save {
    my $self = shift;

    $self->process_edit_form();
}

sub save_me {
    my $self = shift;
    my $type = shift;
    my $error = shift || undef;

    my $uid = $self->uid();
    my $iid = $self->get_iid();

    foreach my $field (keys (%{$self->{fields}})) {
        if ($self->{fields}->{$field}->{required} &&
            (!$FORM{$field} || !($FORM{$field} =~ /\S/)) #field isnt just whitespace.
            ) {
            $error .= "<br />".$self->{fields}->{$field}->{label}
                    . " is a required field.\n";
        }
    }

    if ((defined($error)) && ($iid)){ # error while updating
        $self->www_modify($error);
        exit;
    } elsif (defined($error)) { # error while creating new object
        my $parentid = $self->parent_id();
        my $parent = Gizmo->restore($parentid);
        $parent->www_add_child($error, $self->is_a);
        exit;
    }

    # handle adding/updating gizmo
    my $order;
    foreach my $field (keys (%{$self->{fields}})) {
        $order = $self->{fields}->{$field}->{order};
        next if !defined($order) || $order <= 0;
        $self->{$field} = $FORM{$field};
    }

    if (!$iid) { # new gizmo
        $iid = $self->add();

        my $isa = $self->is_a();
        my $parentid = $self->parent_id();

        if (!$type){ # done editing.
            $self->two_choices ("<Big>New $isa added!</big><br /><br /> What do you want to do?",
                                "confirm",
                                "Add another $isa",
                                "index.pl?child_type=$isa&isa=Category&iid=$parentid&op=add_child&app=",
                                "Enough for now!",
                                "index.pl?id=$parentid");
        } else {
            my $obj = ContactUs->new($iid);
            $obj->www_modify();
            exit;
        }

    } else {    # updating existing gizmo
        $self->save();

        if (!$type) { #done editing
            $self->md_redirect("index.pl?iid=".$self->parent_id);
        } else {
            $self->www_modify();
        }
    }
}

__END__

=head1 NAME

Gizmo::ContactUs - A Gizmo subclass to display ContactUs.

=head1 SYNOPSIS

    use Gizmo::ContactUs;
    $ContactUs = ContactUs->new([$id]);

    $name = $ContactUs->get_gizmo_name();
    $item_display_html = $ContactUs->show();
    $bullet_display_html = $ContactUs->show_as_bullet();
    $version = Discussion->get_version();



=head1 DESCRIPTION

The Contractor class supports the handling of images as 'items'.
Contractor is descended from L<GizmoBuilder>.

It uses the form editing capabilities of its parent class,
GizmoBuilder, to allow the following info about the image to be
specified:

Image files may be be specified in the edit form via either a URL, or
a file to upload as an attachment.  If both are specified, the
attachment is used.  The size of the image in the 'bulleted list' view
can be specified by entering height and width info into the edit form.
In the bulleted list view, the "abstract" text for the image may be
optionally hidden via an option in the edit form.  The full
descriptive text for the image is not generated in list view.

Full information about the image may be accessed by clicking on the
image displayed in a list.  In the "full" view for an image, the image
is displayed at full size, and its descriptive text is included.  The
edit form allows the image alignment to be indicated, and if alignment
is specified, it is used to place the image in the "full" view area.
The image alignment specification is not used in the "bulleted list"
view.

Note: Attached images won't be displayed inline if debugging is turned on.


=head2 Methods

Constructor;  instantiates a new item or retrive its persistent info from the backend
database.
$id is the $id of the Contractor:

    $ContactUs = Gizmo::ContactUs->new([$id]);


Class method to retrieve the version of the gizmo

    $version = Discussion->get_version();

Get the name of the item for pretty-printing:

    $name = $ContactUs->get_gizmo_name();

Generate html to show the image full-size and include its descriptive text:

    $item_display_html = $ContactUs->show();

Generate html to  'list', or give  abbreviated display of the image information:

    $bullet_display_html = $ContactUs->show_as_bullet();

=cut


