###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
#
# Special thanks to Brent Parish for contributing much of the source code
# for this Gizmo.
###
#$Id: ContentScraper.pm,v 1.3.2.4 2005/07/01 22:41:26 nico Exp $
###
package ContentScraper;

use strict;

use GizmoBuilder;
use Metadot::Controllable;
use vars qw(@ISA);

@ISA=qw(GizmoBuilder Metadot::Controllable);

my $version = "1.0";

sub get_version {
    return $version;
}


sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

    $self->set_field_info (
        "name",        "<b>Name of this ".$self->get_gizmo_name()."</b>", 1,
        "c1",         "<b>Target URL</b>", 1,
        "c2",         "<b>Start Text</b> <I>key phrase to begin scrape</I>", 0,
        "c3",         "<b>Stop Text</b> <I>key phrase to end scrape</I>", 0,
        "c5",         "<b>Show contents inside of the page</b>", 0,
        "c4",         "<b>Allow regex interpolation</b>", 0,
        "description", "<b>Description</b>", 0,
        "keywords",    "<b>Keywords and Synonyms</b>", 0,
        );

    $self->set_field_type('c1', 'text', {size=>80, maxlength=>500});
    $self->set_field_type('c4', 'checkbox');
    $self->set_field_type('c5', 'checkbox');
}

sub get_gizmo_name {
    return 'URL Scraper';
}

sub scrape_contents {
    my $self = shift;

    my $url         = $self->get_c1();
    my $html;

    $self->debug_msg(1, "$self getting content from [$url]");
    # FETCH CONTENT FROM REMOTE SITE ($url) with 3 second timeout
    my $response = Utility->fetch_url($url, 3);
    if ($response) {
        if ($response->is_success()) {
            $self->debug_msg(2, "$self successfully got content");
            $html = $response->content();
            my $startkey    = $self->get_c2();
            my $stopkey     = $self->get_c3();
            my $interpolate = $self->get_c4();

            $self->debug_msg(3, "$self interpolates? [$interpolate]");
            # remove leading content, interpolating if desired
            if (defined $startkey && $startkey ne '') {
                $self->debug_msg(3, "$self stripping leading content [$startkey]");
                if ($interpolate) {
                    $html =~ s/^.*?$startkey//s;
                } else {
                    $html =~ s/^.*?\Q$startkey\E//s;
                }
            }

            # remove trailing content, interpolating if desired
            if (defined $stopkey && $stopkey ne '') {
                $self->debug_msg(3, "$self stripping trailing content [$stopkey]");
                if ($interpolate) {
                    $html =~ s/$stopkey.*?$//s;
                } else {
                    $html =~ s/\Q$stopkey\E.*?$//s;
                }
            }
        } else {
            $html = $response->error_as_HTML();
            $self->debug_msg(2, "$self failed with [$html]");
        }
    }

    return $html;
}


sub exec_show {
    return 'content_scraper.tt2';
}

sub exec_show_summary {
    return 'content_scraper_summary.tt2';
}


1;


__END__

=head1 NAME

Gizmo::ContentScraper - can either create a new Item (returing a pointer to the new object) or control
    the way existing ones are displayed (returning the formatted HTML).


=head1 SYNOPSIS

    use Gizmo::ContentScraper;

    $gizmo = ContentScraper->new ($id);
    $html = $gizmo->show ();
    $html = $gizmo->show_as_bullet ();
    $version = ContentScraper->get_version();


=head1 DESCRIPTION

This Gizmo "scrapes" content from another web page and embeds it in the local Metadot page.


=head1 Methods

Calling the new function instantiates a new item or retrive its persistent info from the backend database.

Class method to retrieve the version number of the gizmo

    $version = ContentScraper->get_version();



The remaining functions control how an Item is displayed:

    $gizmo->show() displays an item.
    $gizmo->show_as_bullet() displays the item as an item in a  bulleted list.

=cut

