###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Contractor;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT);
use Metadot::Controllable;

use vars qw(@ISA);
@ISA=qw(GizmoBuilder Metadot::Controllable);
my $version = "4.5";

sub get_version {
    return $version;
}

{
    my $search_params = {
             searchable_fields => [ qw( name url description keywords t1 t2 c3 c4 c5 t3 longdescription file1 c1 c2 t4 t5 t6 t7 ) ],
             displayed_fields  => [ qw( url description t1 t2 c3 c4 c5 t3 longdescription file1 c1 c2 t4 t5 t6 t7 ) ],
            };

    sub get_search_params {
    my $self = shift;

    return $search_params;
    }
}

####################################

sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

  $self->_form_initialize();
  $self->set_field_info (
    "name", "<b>Company Name</b>", 1,
    "url", "<b>Company URL</b>", 0,
    "t1", "<b>Address line 1</b>",0,
    "t2", "<b>Address line 2</b>",0,
    "c3", "<b>City</b>", 0,
    "c4", "<b>State</b>", 0,
    "c5", "<b>Zip code</b>", 0,
    "t3", "<b>Country</b>", 0,
    "long_description", "<b>Company Description</b>", 0,

    "file1",       "<b>Logo Image</b>",  0,
    "c1", "<b>Logo height in pixels</b>", 0,
    "c2", "<b>Logo width in pixels</b>", 0,
    "description", "<b>Image information ('alt' tag)</b>", 0,

    "t4", "<b>Contact name</b>",0,
    "t5", "<b>Contact email</b>",0,
    "t6", "<b>Contact phone number</b>",0,
    "t7", "<b>Contact fax number</b>",0,
    );


  $self->{display_wrapped} = { show => 1,
                             };

}


sub gizmo_list_decorator {
   return 'HTML::Decorator::NullDecorator';
}

sub get_gizmo_name {
   my $self = shift;
#    return "Contractor";
   return "Organization";
}

sub gizmo_list_header {
    my $class = shift;

    return "";
}

#override one of the default form types.
sub _form_initialize {
    my $self = shift;

    $self->{fields}->{t1}->{formtype} = "text";
    $self->{fields}->{t2}->{formtype} = "text";
    $self->{fields}->{c3}->{formtype} = "text";
    $self->{fields}->{c4}->{formtype} = "text";
    $self->{fields}->{t3}->{formtype} = "text";
    $self->{fields}->{t4}->{formtype} = "text";
    $self->{fields}->{t5}->{formtype} = "text";
    $self->{fields}->{t6}->{formtype} = "text";
    $self->{fields}->{t7}->{formtype} = "text";
    $self->{fields}->{description}->{formtype} = "text";

    my $ts = "<!-- intro text would go here -->
";
    $self->set_intro_text($ts);

}

##########################################
### show
### return one newsitem (page format)
##########################################
sub show {
  my $self = shift;
  $self->debug_msg(1, "Contractor::show()");
  my $s="";
  #print STDERR "in Contractor:show()\n";

  my $company_name    = $self->get_name();
  my $alt_tag         = $self->get_description();
  my $company_description = $self->get_long_description();
  my $url             = $self->get_url();
  my $image_url    = $self->get_f1_attachment_url();

  my $height = $self->get_c1();
  my $width = $self->get_c2();
  my $company_city = $self->get_c3();
  my $company_state = $self->get_c4();
  my $company_zip = $self->get_c5();
  my $company_address_1 = $self->get_t1();
  my $company_address_2 = $self->get_t2();
  my $company_country = $self->get_t3();

  my $contact_name = $self->get_t4();
  my $contact_email = $self->get_t5();
  my $contact_phone = $self->get_t6();
  my $contact_fax = $self->get_t7();

  my $text_width = 270;
  my $table_width = "";

  $company_address_1 &&= $company_address_1.'<br />';
  $company_address_2 &&= $company_address_2.'<br />';
  $company_city &&= $company_city.',';
  $company_country &&= $company_country.'<br />';

  my $company_line_end;
  if ($company_city || $company_state || $company_zip) {
      $company_line_end = '<br />';
  } else {
      $company_line_end = '';
  }

  my $contact_label;
  if ($contact_name || $contact_email || $contact_phone || $contact_fax) {
      $contact_label = '<b>Contact</b><br />';
  } else {
      $contact_label = '';
  }
  my $contact_line_end;
  if ($contact_label) {
      $contact_line_end = '<br />';
  } else {
      $contact_line_end = '';
  }

  $contact_name &&= $contact_name.'<br />';
  $contact_email = "<a href=\"mailto:$contact_email\">$contact_email</a><br />" if $contact_email;
  $contact_phone &&= 'Ph: '.$contact_phone.'<br />';
  $contact_fax &&= 'Fx: '.$contact_fax.'<br />';

  my $url_text;
  if ($url) {
      $url_text = "<a href=\"$url\" class='centerNewsLink'>$url</a><br /><br />"
  }

  my ($height_text, $width_text);
  if ($height) {
      $height_text = "height=\"$height\"";
  } else {
      $height_text = "";
  }
  if ($width) {
      $width_text = "width=\"$width\"";
  } else {
      $width_text = "";
  }

  my $image_text;
  if ($image_url) {
      $image_text = "<img src=\"$image_url\" border=\"0\" align=\"right\" $height_text $width_text alt=\"$alt_tag\">";
      if ($url) {
        $image_text = "<a href=\"$url\">$image_text</a>";
      }
  } else {
      $image_text = '';
  }

  my $spacer_image = Utility->get_spacer(5,1);

  # layout was revised based on feedback from J Kugler
  $s .= <<EOM
         $image_text
            <p /><b>$company_name</b>
                <br />
                <font size="-1">
                    $company_address_1
                    $company_address_2
                    $company_city $company_state $company_zip $company_line_end
                    $company_country
                </font>
                $url_text
            <p />
                <font size="-1">
                    $company_description
                </font>
            <p />
                <font size="-1">
                    $contact_label
                    $contact_name
                    $contact_email
                    $contact_phone
                    $contact_fax
                    $contact_line_end
                </font>
            <BR clear="all">
EOM
;

  return $s;
}

##########################################
### show_in_list
### return one newsitem (list format)
##########################################
# aju: do I need this method?
sub show_in_list {
  my $self = shift;
  $self->debug_msg(1, "Contractor::show_in_list()");
#    print STDERR "in Contractor:show_in_list()\n";

  my $name        = $self->get_name();
  my $description = $self->get_description();
  my $url         = $self->get_url();
  my $panel       = $self->get_panel();

  $name = "<img src=$url>"; #if ($url) ;

  return "<b>$name</b> - $panel<br />\n$description<br />\n";
}

# I don't need this... do I?
##########################################
### show_in_column
### return one newsitem (news column format)
##########################################
#  sub show_in_column {
#    my $self = shift;
#    $self->debug_msg(1, "Contractor::show_in_column()");
#    print STDERR "in Contractor:show_in_column()\n";

#    my $iid             = $self->get_iid();
#    my $name            = $self->get_name();
#    my $description     = $self->get_description();
#    my $url             = $self->get_url();
#    my $longdescription = $self->get_long_description();
#    my $panel           = $self->get_panel();
#    my $s               = "";

#    if($longdescription || $url){
#      $name = "<a class='newsitemtitle' href='index.pl?id=$iid&amp;isa=Contractor&amp;op=show'>$name</a>";
#    }

#    $s.= $panel;
#    $s.= "<table cellpadding=\"0\" cellspacing=\"0\"><tr><td class='newsitemtitle'>- $name\n</td></tr>\n";
#    $s.= "<tr><td class='newsitem'>$description<br />&nbsp;</td></tr></table>\n\n";

#    return $s;
#  }


sub show_summary {
    my $self = shift;
    my @args = @_;

    my $buttons     = $self->get_buttons();
    my $text = $self->show(@args);

    return $buttons.$text;
}


#  sub get_Height {
#   my $self = shift;

#   return $self->{height};
#  }

#  sub get_Width {
#   my $self = shift;

#   return $self->{width};
#  }

1;

__END__

=head1 NAME

Gizmo::Contractor - A Gizmo subclass to display Contractors.

=head1 SYNOPSIS

    use Gizmo::Contractor;
    $Contractor = Contractor->new([$id]);

    $name = $Contractor->get_gizmo_name();
    $item_display_html = $Contractor->show();
    $bullet_display_html = $Contractor->show_as_bullet();
    $version = Discussion->get_version();



=head1 DESCRIPTION

The Contractor class supports the handling of images as 'items'.
Contractor is descended from L<GizmoBuilder>.

It uses the form editing capabilities of its parent class,
GizmoBuilder, to allow the following info about the image to be
specified:

Image files may be be specified in the edit form via either a URL, or
a file to upload as an attachment.  If both are specified, the
attachment is used.  The size of the image in the 'bulleted list' view
can be specified by entering height and width info into the edit form.
In the bulleted list view, the "abstract" text for the image may be
optionally hidden via an option in the edit form.  The full
descriptive text for the image is not generated in list view.

Full information about the image may be accessed by clicking on the
image displayed in a list.  In the "full" view for an image, the image
is displayed at full size, and its descriptive text is included.  The
edit form allows the image alignment to be indicated, and if alignment
is specified, it is used to place the image in the "full" view area.
The image alignment specification is not used in the "bulleted list"
view.

Note: Attached images won't be displayed inline if debugging is turned on.


=head2 Methods

Constructor;  instantiates a new item or retrive its persistent info from the backend
database.
$id is the $id of the Contractor:

    $Contractor = Gizmo::Contractor->new([$id]);


Class method to retrieve the version of the gizmo

    $version = Discussion->get_version();

Get the name of the item for pretty-printing:

    $name = $Contractor->get_gizmo_name();

Generate html to show the image full-size and include its descriptive text:

    $item_display_html = $Contractor->show();

Generate html to  'list', or give  abbreviated display of the image information:

    $bullet_display_html = $Contractor->show_as_bullet();

=cut


