###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Discussion;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY);

use DBAccess;
use Utility;
use Message;
use Carp;
use Gizmo;
use GizmoBuilder;
use Metadot::EmailToDiscussion;
use HTML::Components;
use HTML::Parser;
use Metadot::Subscribable;
use Metadot::Invitable;
use Metadot::Controllable;
use Metadot::PopupMgr;

use CGI qw/escapeHTML unescapeHTML/;
use CGI qw/:standard/;
use vars qw(@ISA);

@ISA=qw(GizmoBuilder Metadot::Subscribable Metadot::Invitable Metadot::Controllable);
my $version = "4.5";

use constant DEFAULT_MESSAGE_CONTENT_TYPE => 'text/plain';


sub get_version {
    return $version;
}

#######################################
#aju 08-21-01 -- needed to add some bundles & ops to the default.

sub customize_permissions {
    my $self = shift;
    my $ac   = shift;

    $self->SUPER::customize_permissions($ac);
    Metadot::Subscribable->customize_permissions($ac);
    Metadot::Invitable->customize_permissions($ac);

    $ac->add_bundle('POST', 'Post', '2','2');
    $ac->add_bundle('MODERATE', 'Moderate', '8','8');
    $ac->add_bundle('PURGE', 'Purge', '8','8');

    $ac->add_op('MOD','modify_message');
    $ac->add_op('MOD','create_shortcut');
    $ac->add_op('MOD','save_message');

    $ac->add_op('POST','reply');
    $ac->add_op('POST','send');
    $ac->add_op('POST','compose');
    $ac->add_op('POST','modify_own_message');
    $ac->add_op('POST','save_own_message');
    $ac->add_op('POST','download_file');

    $ac->add_op('MODERATE','moderate');
    $ac->add_op('MODERATE','delete_message');
    $ac->add_op('MODERATE','delete_message_ok');
    $ac->add_op('MODERATE','approve');

    $ac->add_op('PURGE','purge');
    $ac->add_op('PURGE','purge_ok');
    $ac->add_op('PURGE','purge_messages');

    return $ac;
}



####################################

sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

  my $textInst = "The Discussion name should be very clear, since";
  $textInst .= " other users may read the the name in their context (in their user's <i>My Page</i>";
  $textInst .= " for example) E.g.,  <i>Laser" ;
  $textInst .= " Computer product suggestions</i>.";

  my @field_info = (
                    "name",        "<b>Discussion Name</b>",                                  1,
                    "text",        $textInst,                                                 0,
                    "description", "<b>Brief Description</b>",                                0,
                    "discussion_attachment_file",       "<b>Attachment .gif, .jpg, or .png Image</b>",             0,
                    "keywords",    "<b>Keywords and Synonyms</b>",                            0,
                    "c3",          "<b>Hide author picture</b>",                              0,
                    "c1",          "<b>Moderated?</b>",                                       0,
  );

  if ($PARAMS->{allow_discussion_email_alias}) {
      my $alias_domain = Metadot::EmailToDiscussion->get_alias_domain();
      my $domain = Metadot::EmailToDiscussion->get_alias_domain();
      push @field_info, (
          'c2',
          "<b>Email address for this discussion</b><br /><input type=text name=\"c2\" value=\"<<VALUE>>\" size=20>\@$domain<br />
          <br />Define an email address if you want to allow users to submit new posts to this discussion via email",
          0
      );
  }
  $self->set_field_info(@field_info);

  $self->set_field_type('discussion_attachment_file', Metadot::UploadsManager::UPLOAD_TYPE_PUBLIC());
  $self->set_field_type('c2', 'other');

  #override the c3 and c1 to be a checkbox
  $self->set_field_type('c3', 'checkbox');
  $self->set_field_type('c1', 'checkbox');

  $self->{display_wrapped} = { show => 1,
                             };

}

sub gizmo_list_layout {
  my $class = shift;
  my $head = shift;
  my $child_list = shift;

  my $name = $class->get_gizmo_name();

  my $output = "<!-- $name -->\n$head\n $child_list\n<!-- End of $name List -->\n\n";

  return $output;
}

###################################
# WWW FUNCTIONS






###################################
# BACK END FUNCTIONS


# specialize the Gizmo delete method to remove the channel if it exists too.
sub delete {
  my $self = shift;

  my $id = $self->{id};
  $self->debug_msg(3, "Discussion::delete(id=$id)");

  if(!$id){
    $self->debug_msg(3, "Error Discussion::delete(id=$id): I don't know what id to delete");
    return 0;
  }

  ### delete channel for discussion if it exists
  ### first get cid
  my $channel = $self->get_channel_object();
  if (defined $channel) {
      $channel->delete();
  }
  ### Delete db records for attachments
  my $attch_list = DBAccess->sqlSelectMany("mid","message","iid = $id");
  while( my ($attch) = $attch_list->fetchrow() ){
    DBAccess->sqlDelete("uploads","id = $attch");
  }

  my $upload_dir = $PARAMS->{absolutepathfileuploaddir};
  $upload_dir .= "/$id";
  Utility->_delete_directory($upload_dir);

  DBAccess->sqlDelete("message", "iid=$id");

  #CKIM-SHORTCUTS - the remainder is is a copy of SUPER, should instead invoke SUPER
  return($self->SUPER::delete());

#  # Auditable requirement
#  $self->delete_permissions();
#
#  DBAccess->sqlDelete ("instance", "iid=$id");
#  $self->debug_msg(4, "Gizmo deleted (id=$id)");
#  $self->reorder_parent();
#  return 1;

}


# Discussion needs to save its channel information if it exists
sub save {
    my $self = shift;
    my @args = @_;
    $self->SUPER::save(@args);

    my $channel = $self->get_channel_object();
    if (defined $channel) {
        $channel->name($self->get_name());
        $channel->description($self->get_description());
        $channel->save();
    }
}

###################################
sub get_channel_name {
 my $self = shift;
 my $id   = shift || $self->id();
 return undef if (!defined $id);
 my ($name) = DBAccess->sqlSelect ("name",  " channel", " channel.iid=$id");
 return $name;
}

sub get_channel_id {
    my $self = shift;
    my $id = $self->id;
    return undef if (!$id);
    my ($cid) = DBAccess->sqlSelect("cid", "channel", "iid=$id");
    return $cid;
}

sub get_channel_object {
    my $self = shift;
    my $cid = $self->get_channel_id();
    my $channel;
    if (defined $cid) {
        $channel = Channel->new($cid);
    }
    return $channel;
}

sub get_icon {
    my $self = shift;
    my ( $file, $file_icon, $file_url ) = $self->get_upload_filename_icon_and_url( {
                                            id => $self->id(),
                                            isa => $self->is_a(),
                                            field_name => 'discussion_attachment_file',
                                           } );
    return $file_url || '';
}

###################################
sub show_summary {
    my $self = shift;

    my $id = $self->{id} || $self->get_id();
    $self->debug_msg(3, "Discussion::show_as_bullet($id)");

    my $panel = $self->get_buttons();
    my $name = $self->name();
    my $description = $self->description();
    my $nbitems = $self->nb_items();
    my $moderated = $self->c1() ? " <i>moderated</i>" : "";

    my $str="$panel\n<a href=".$self->make_url("index.pl?iid=$id&amp;isa=Discussion&amp;op=show").">$name</a>($nbitems)$moderated";
    $str .= ": $description\n" if ($description);
    $str .= "<br />\n";
    return $str;
}


sub get_user_threading_preference {
    my $self = shift;
    my $result = $FORM{threads};
    my $session_value = $SESSION->get_value('discussion_thread_preference') || 0;
    if (defined($result)) {
        if ($session_value ne $result) {
            $SESSION->set_value('discussion_thread_preference', $result);
        }
    } else {
        $result = $session_value;
    }
    $self->debug_msg(4, "Discussion: User threading preference='$result'");
    return $result;
}

sub get_user_show_content_preference {
    my $self = shift;
    my $result = $FORM{content};
    my $session_value = $SESSION->get_value('discussion_content_preference') || 0;
    if (defined($result)) {
        if ($session_value ne $result) {
            $SESSION->set_value('discussion_content_preference', $result);
        }
    } else {
        $result = $session_value;
    }
    $self->debug_msg(4, "Discussion: User show content preference='$result'");
    return $result;
}


###################################
sub show {
  my $self = shift;
  my $Threads = shift || $self->get_user_threading_preference() || 0;
  my $show_content = $self->get_user_show_content_preference() || 0;
  my $iid = $self->id();

  my $c1 = $self->c1() || 0;
  my $moderated = $c1  ? "<b>Moderated</b>" : "";

  my $startfrom = $FORM{from} || 0;
  my $max = $FORM{nb} || 50 ;

  my $s="";
  my @arr=();

  $self->debug_msg(3,"Discussion:show(id=$iid, startfrom=$startfrom, maxitem=$max)");

  $s .= "
<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">
<tr class='discussionheader' bgcolor=\"CCCCCC\">
<td><font class='discussionheader'><b>Subject</b></font></td>
<td><font class='discussionheader'><b>Author</b></font></td>
<td><font class='discussionheader'><b>Date</b></font></td>
</tr>";


  if ($Threads) {

      #Check for obsolete messages
      my ($c)=DBAccess->sqlSelectMany("mid, message.creationdate", "message", "(iid=$iid) and (updatedate is NULL)", "");
      while(my ($mid, $creationdate)=$c->fetchrow()) {
          # The following is not compatible w/ Oracle
          #DBAccess->sqlUpdate ("message", "mid = '$mid'", -updatedate=>'creationdate');
          DBAccess->sqlSimple("update message set updatedate=creationdate where mid=$mid");
      }

    # Start getting Threads recursively
    @arr = @{$self->_show_thread_child(0,0)};

  } else {
    @arr = @{$self->_show_by_date()};
  }

  my $i=0;
  my $totalrows = @arr;
  if ($totalrows) {

    # Count number of rows to display on page
    my $enditem = $startfrom + $max;
    $enditem = $totalrows if($enditem > $totalrows);

    for (my $j=$startfrom;$j<$enditem;$j++){
      $s.= $arr[$j];
      $i++;
    }
  } else {
    $s .= "<tr><td valign='middle'><font size=-1><b>No messages in this discussion</b></font></td>\n";
  }

  #PREVIOUS NEXT MAGIC
  my $pn = "" ;
  my $url = "index.pl?iid=$iid&amp;isa=Discussion&amp;op=show&amp;threads=$Threads&amp;content=$show_content";
  $pn = Components->get_previous_next_button($max, $i, $totalrows, $startfrom, $url) if ($totalrows>$max);

  my $button .= $self->_show_compose_button($Threads,$show_content);

  my $alias_text = '';
  if ($self->get_email_alias()) {
      my $alias = $self->get_email_alias({full=>1});
      $alias_text .= "You can post to this discussion by sending an email to: <a href=\"mailto:$alias\">$alias</a><br />";
  }

  $s .="\n</table></center>\n\n";

  $s = "$moderated<br />$button<br />\n$alias_text\n$pn<br />\n\n$s<br />\n\n$pn\n\n";
  return ($s);
}

#################################
sub _show_thread_child {
 my $self = shift;
 my $parentmid = shift;
 my $indent = shift;
 my $highlighted_mid = shift || 0;
 my $iid = $self->id();
 my @result;
 my $s;
 my $arref;

 # Selecting all messages by parentmid
 my $c = $self->get_messages_by_parent($parentmid);

 while(my @row=$c->fetchrow()) {
   my $mid = $row[0] ;
   my $highlight = ($mid == $highlighted_mid) ? 1 : 0 ;
   $s = $self->_show_one_row(\@row,$indent,$highlight);

   # Save result and start myself recursively
   if ($s ne "") {
       $arref = $self->_show_thread_child($mid,$indent+1,$highlighted_mid);
       push  (@result,$s,@$arref);
   }

 }
 $c->finish();

 return \@result;

}
##############################
sub _show_one_row {
  my $self = shift;
  my $show_content = $FORM{content} || 0;
  my $Threads = $FORM{threads} || 0;
  my ($array_ref, $indent, $highlight, $row_color) = @_;
  my ($mid, $name, $authorname, $uid, $creationdate, $status, $iid, $content, $updatedby) = @$array_ref;

  ##check to see if message not approved 'OK' and if not allowed to moderate
  return "" if ($status ne 'OK' && !$self->is_allowed_to_do('moderate', $USER));

  my $imageslocation = $PARAMS->{'imageslocation'};

  $indent ||= 0;
  my $shift = "&nbsp;&nbsp;&nbsp;" x $indent;
  my $time_stamp_creation_date = Utility->unix_time_format($creationdate);
  my $new = (time < ($time_stamp_creation_date + 3600)) ? " <img src = \"$imageslocation/my/animated_new.gif\">" : "";
  my $modified_message = (defined($updatedby) &&  $updatedby ne '') ? "<font color=red>Modified</font>" : "";
  my $class = ($highlight) ? "highlightedmessage" : "othermessage" ;
  my $current = ($highlight) ? "bgcolor='#D3D3D3'" : ($row_color ? "bgcolor=$row_color" : "");
  my $id = $self->id();

  my $panel = '';
  ## edit message button
  my $edit_op;
  if ($SESSION->admin_mode && $self->is_allowed_to_do('modify', $USER)) {
      $edit_op = 'modify_message';
  } elsif (($uid == $USER->uid() && $PARAMS->{allow_discussion_message_editing} && $self->is_allowed_to_do('modify_own_message'))) {
      $edit_op = 'modify_own_message';
  }
  if ($edit_op) {
      $panel .= "<a href = \"".$self->make_url("index.pl?mid=$mid&amp;isa=Message&amp;op=$edit_op")."\"><img src = \"$imageslocation/obj/edit.gif\" alt='Edit Me!' border=\"0\"></a>";
  }

  ## delete message button
  ## added the 'delete_message' op
  $panel .= (   ($SESSION->admin_mode && $self->is_allowed_to_do('delete', $USER))
             || $self->is_allowed_to_do('delete_message')
             )
             ?  " <a href = \"".$self->make_url("index.pl?mid=$mid&amp;isa=Message&amp;op=delete_message")."\" class='icon'><img src = \"$imageslocation/obj/del.gif\" border=\"0\"></a>"
             :  "";

  ## moderate message buttons
  $panel .= ($status ne 'OK' && $self->is_allowed_to_do('moderate', $USER)) ?
   " <a href = \"".$self->make_url("index.pl?mid=$mid&amp;isa=Message&amp;op=approve&amp;threads=$Threads&amp;content=$show_content")."\"><img src = \"$imageslocation/obj/accept.gif\" border=\"0\"></a>
    <a href = \"".$self->make_url("index.pl?mid=$mid&amp;isa=Message&amp;op=delete_message&amp;threads=$Threads&amp;content=$show_content")."\"><img src = \"$imageslocation/obj/reject.gif\" border=\"0\"></a>" :
   "";
  if ($show_content) {

      Utility->strip_xss($content);

      $content =~ s/\<br\>/\<br\>$shift/gi;        # pp breaks
      $content = "<tr><td colspan='3'>$shift$content</td></tr>";
  } else {
      $content = "";
  }
  return "
<tr class='$class' $current>
<td nowrap='nowrap'>$shift<font class='$class' size=-1><b><a class='$class' href='"
.$self->make_url("index.pl?mid=$mid&amp;isa=Message&amp;op=show")
."'>$name</a>$new</b> $modified_message</font> $panel</td>
<td><i><div style=\"white-space: nowrap\">$authorname</i></div></td>
<td><div style=\"white-space: nowrap\">" . Utility->format_datetime($creationdate, 'b. e, Y') . "</div></td>
</tr>
$content
";
}


sub get_messages_by_date {
    my $self = shift;
    my $iid = $self->get_id();
    return $self->get_messages_where("iid=$iid AND ".$self->get_approval_filter(),'ORDER BY creationdate DESC');
}

sub get_messages_by_parent {
    my $self = shift;
    my $parentmid = shift;
    $parentmid = undef unless Utility->is_valid_integer($parentmid); # secu_200401

    my $iid = $self->get_id();

    return $self->get_messages_where("iid=$iid AND parentmid=$parentmid AND ".$self->get_approval_filter(), 'ORDER BY creationdate DESC');
}

sub get_message_by_mid {
    my $self = shift;
    my $mid = shift;
    $mid = 0 unless Utility->is_valid_integer($mid); # secu_200401

    my $iid = $self->get_id();
    my $rs = $self->get_messages_where("iid=$iid AND mid=$mid AND ".$self->get_approval_filter());
    my $result = $rs->fetchrow_hashref;
    $rs->finish;
    return $result;
}

sub get_messages_where {
    my $self = shift;
    my $where = shift || 'iid='.$self->get_id();
    my $other = shift;
    my $rs = DBAccess->sqlSelectMany(
        "mid, name, authorname, userid, creationdate, status, iid, description, updatedby",
        "message", $where," $other");
    return $rs;
}

# if the user is not a moderator, than this only retrieves approved messages
sub get_approval_filter {
    my $self = shift;
    my $approver = $self->is_allowed_to_do('moderate',$USER);
    my $result;
    if ($approver) {
        # CKIM: Oracle doesn't like a unary '1'
        $result = '1=1';
    } else {
        $result = 'status=\'OK\'';
    }
    return $result;
}


##############################
sub _show_by_date {
  my $self = shift;
  my $iid = $self->id();
  my @result;
  my $s;
    my $row_color_on = "#EEEEEE";
    my $row_color_off = "#FFFFFF";
    my $row_color = "";

  my $c = $self->get_messages_by_date();

  while (my @row=$c->fetchrow())  {
            $row_color = ($row_color eq $row_color_on) ? $row_color_off : $row_color_on;
      $s = $self->_show_one_row(\@row,0,0,$row_color);

      if ($s ne "") {
        push  (@result,$s);
      }
  }
  $c->finish();

  return \@result;
}


sub _format_search_result{
  my $gizmo_name = shift ;
  my ($iid, $name, $description, $isa)=@_;

  if (!$iid || !$isa ){
    return "";
  }

  $name = "No title" if (!$name);

  my $s="";
  $s = "<a href=".$gizmo_name->make_url("index.pl?iid=$iid&amp;isa=$isa&amp;op=show")."><b>$name</b></a>\n- $description";

  return $s;
}

##################################
sub show_search_results  {
  my $gizmo_name = shift ;
  my $thingtosearch = shift;
  my $startfrom = shift || 0;
  my $maxitem = shift || 50;

  my $s="";
  my $oneresult="";

  my $filter = $thingtosearch;
  my $wherestring = "";

  ################################################################
  ####
  #### we build the query
  ####
  #### clean quote and other chars
  ################################################################

  $thingtosearch = uc($thingtosearch);
  $thingtosearch = DBAccess->quote("%$thingtosearch%");

  $wherestring = join(" or ", map("upper($_) like $thingtosearch", ("name", "description", "keywords")));

  my $whereclause="isa='$gizmo_name' AND ($wherestring)";
  my $i=0;
  my $totalrows=0;
  my $from = $startfrom;

  my ($c)=DBAccess->sqlSelectMany("instance.iid, instance.name, instance.description, instance.isa, instance.parentiid", "instance", $whereclause." and instance.iid >= 0 and instance.parentiid >= 0", "order by name");

  while( (my ($iid, $name, $description, $isa, $pid)=$c->fetchrow()) && ($i < $maxitem)) {


      if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC
      my $gizmo = $isa->new($iid);
      next unless $gizmo->is_allowed_to_do('show', $USER);
      next unless $gizmo->parentCheck(); # don't show hidden items
      $totalrows++;
        # shorten the description
      $description="" if (!$description || !defined($description));
      $name="null" if (!$name || !defined($name));
      $description=substr($description, 0, 150) ."...";
      ### remove tags that could break the page.
      $description = Verify->stripBadHTML($description);
      $oneresult = $gizmo_name->_format_search_result($iid, $name, $description, $isa);
      ### put the results in list tag
      $oneresult="<li>" . "$oneresult" . "<br/><br/></li>\n\n";
      $s .= "$oneresult\n\n";;
      $i++;
  }
  $c->finish();


  $whereclause="instance.isa='$gizmo_name' AND instance.iid = message.iid AND status = 'OK' AND (UPPER(message.name) like $thingtosearch or UPPER(message.description) like $thingtosearch or UPPER(message.authorname) like $thingtosearch or UPPER(message.email) like $thingtosearch)";

  ($c)=DBAccess->sqlSelectMany("instance.iid, instance.isa, instance.name, message.mid, message.name", "instance, message", $whereclause, "order by instance.name, message.name");

  while( (my ($discussion_iid, $isa, $discussion_name, $message_mid, $message_name)=$c->fetchrow()) && ($i < $maxitem)) {
      if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC
        my $gizmo = $isa->new($discussion_iid);
      next unless $gizmo->is_allowed_to_do('show', $USER);
      next unless $gizmo->parentCheck(); # don't show hidden items
    # shorten the description
        $totalrows++;
      $discussion_name="null" if (!$discussion_name || !defined($discussion_name));
      ### remove tags that could break the page.
      $oneresult = $gizmo_name->_format_message_search_result($discussion_iid, $discussion_name, $message_mid, $message_name);
      ### put the results in list tag
      $oneresult="<li>" . "$oneresult" . "<br/><br/></li>\n\n";
      $s .= "$oneresult\n\n";;
      $i++;
  }
  $c->finish();

  $s = $gizmo_name->show_search_result_group({
      gizmo_name    => $gizmo_name,
      total_rows    => $totalrows,
      max_rows      => $maxitem,
      start_from    => $startfrom,
      cursor        => $i,
      filter        => $filter,
      results       => $s,
  });

  return ($s);

}

sub nb_items {
    my $self = shift;
    my $id   = $self->id() || 0;

    my ($count) = DBAccess->sqlSelect("count(*)", "message", "iid=$id AND ".$self->get_approval_filter());

    return $count;
}

#######################

# fail if an illegal or duplicate alias was specified
sub www_save {
    my $self  = shift;
    my $error = undef;

    my $alias = $FORM{c2};
    my $iid = $self->get_iid();

    if (defined($alias) && ($alias ne '')) {
        # check for illegal alias
        (my $illegal = $alias) =~ s/(\w|_|-|\.)*//;
        if ($illegal) {
            $error .= "The email address contains one or more illegal characters. Only alphanumeric characters, \".\", \"-\", and \"_\" are permitted<br />";
        }

        # check for duplicate alias
        unless ($error) {
            my $existing_iid = $self->get_iid_for_alias($alias);
            if ($existing_iid) {
                # existing one found, now see if it is my own
                if (!$iid || $existing_iid != $iid) {
                    $error .= "The email address '$alias' is already in use by another ".$self->get_gizmo_name()."<br />";
                }
            }
        }
    }
    if ($error) {
        if (defined($iid)) {
            # error when modifying
            return $self->www_modify($error);
        } else {
            # error when adding
            my $parentid = $self->parent_id();
            my $parent = Gizmo->restore($parentid);
            $parent->www_add_child($error, $self->is_a);
        }
    } else {
        return $self->SUPER::www_save(@_);
    }
}


#######################
sub _format_message_search_result{
  my $gizmo_name = shift ;
  my ($discussion_iid, $discussion_name, $message_mid, $message_name)=@_;

  if (!$discussion_iid){
    return "";
  }

  $discussion_name = "No title" if (!$discussion_name);

  my $s="";
  $s = "<a href=".$gizmo_name->make_url("index.pl?iid=$discussion_iid&amp;isa=Discussion&amp;op=show")."><b>$discussion_name</b></a> - <a href=".$gizmo_name->make_url("index.pl?mid=$message_mid&amp;isa=Message&amp;op=show")."><b>$message_name</b></a>\n ";

  return $s;
}


############################################################

sub _show_compose_button {
    my $self = shift;
    my $threads = shift;
    my $show_content = shift;
    my $iid = $self->id();


    my ( $file, $file_icon, $file_url ) = $self->get_upload_filename_icon_and_url( {
                                            id => $self->id(),
                                            isa => $self->is_a(),
                                            field_name => 'discussion_attachment_file',
                                           } );

    my $s;
    $self->debug_msg(5,"_show_compose_button(id=$iid)");
    $s  = "<table border=\"0\" width=\"100%\"><tr><td>\n";

    # will show post link only when user has permissions to post
    if( $self->is_allowed_to_do( 'send', $USER ) ){
    $s .= "<a href=".$self->make_url("index.pl?isa=Message&amp;op=compose&amp;discussion_id=$iid").">Post a new message</a> | ";
    }

    if ($threads) {
      $s .= "<a href=".$self->make_url("index.pl?iid=$iid&amp;isa=Discussion&amp;op=show&amp;threads=0&amp;content=$show_content").">Sort by date</a>";
    } else {
       $s .= "<a href=".$self->make_url("index.pl?iid=$iid&amp;isa=Discussion&amp;op=show&amp;threads=1&amp;content=$show_content").">Sort by thread</a>";
    }
    if ($show_content) {
      $s .= " | <a href=".$self->make_url("index.pl?iid=$iid&amp;isa=Discussion&amp;op=show&amp;threads=$threads&amp;content=0").">Hide Content</a>";
    } else {
       $s .= " | <a href=".$self->make_url("index.pl?iid=$iid&amp;isa=Discussion&amp;op=show&amp;threads=$threads&amp;content=1").">Show Content</a>";
    }

    $s .= "</td><td align='right'>";

    my $chan = $self->get_channel_object();
    if ($chan) {
        $s .= $chan->show_add_to_my_page($iid);
    }

    #CKIM-SENDABLE
    if ($self->can('show_invite_button')) {
        my $invite_button = $self->show_invite_button();
        if ($invite_button) {
            $invite_button = "<br />".Utility->get_spacer(5,1)."<br />".$invite_button;
            $s .= $invite_button;
        }
    }
    $s .= "</td>";


    if($file){
        $s .="<td align='right'><img BORDER=\"0\" src=\"$file_url\"></td>";
    }
    $s .="</tr></table>\n\n";
    return $s;
}

sub is_author_image_enabled {
    my $self = shift;

    # show image if 'disable author image' is not checked
    return ($self->c3() ne 'on');
}

sub is_moderated {
    my $self = shift;
    my $iid = shift;

    # allow the caller to pass iid
    unless (ref $self) {
        $self = Discussion->restore($iid);
    }
    return ($self->c1());
}


# pass 'full' as an arg to get the fully qualified email address
sub get_email_alias {
    my ($self, $args) = @_;
    my $iid = $args->{iid};
    my $result;

    unless (ref $self) {
        if (defined $iid) {
            $self = Discussion->restore($iid);
        } else {
            return;
        }
    }

    if ($PARAMS->{allow_discussion_email_alias}) {
        $result = $self->c2();

        if ($result && $args->{full}) {
            $result .= '@'.Metadot::EmailToDiscussion->get_alias_domain();
        }
    }

    return $result;
}

sub get_iid_for_alias {
    my $class = shift;
    my $alias = shift;

    my $iid;
    if (defined($alias)) {
        ($iid) = DBAccess->sqlSelect( 'iid', 'instance', 'LOWER(c2)='.DBAccess->quote(lc($alias)) );
    }
    return $iid;
}
sub get_upload_filename_icon_and_url{
    my $self = shift;
    my $params = shift;

    my $message = Message->new($params->{id});

    return $message->get_upload_filename_icon_and_url($params);
}

######
# This method returns whether or not a user is subscribed to a discussion
#####
sub user_is_subscribed {
    my $self = shift;
    my $userid = shift;
    my $did    = $FORM{iid} || shift || die "Must specify Discussion id";

    my ($result) = DBAccess->sqlSelect("count(*)","subscription s, channel c",
                                       "c.iid = $did and c.cid = s.cid and userid = $userid");

    return $result;

}

sub discussion_message_attachment {
    my $self = shift;
    my $params = shift;

    #print STDERR "in discussion_message_attachment\n\n";

    my $message = Message->new($params->{id});
    return $message->get_upload_filename_icon_and_url($params);
}



# return true if a gizmo class allows processing of user template directives
sub processes_user_directives {
    return 0;
}


sub exec_purge {
    my $self = shift;
    
    return "discussion_purger.tt2";
}

sub www_purge_messages {
    my $self = shift;
    my $did = $FORM{iid} || die "Could not get discussion id";
    my $isa = $FORM{isa} || die "Could not get isa field";
    my $date_filter = $FORM{date_from};

    my ($msg_count) = DBAccess->sqlSelect('count(*)','message',"iid = $did and updatedate <= \"$date_filter\"");

    if( $msg_count ){

       $DISPLAY->two_choices("You are about to delete $msg_count messages from this discussion",
                 "warning",
                 "I'm sure, delete them","index.pl?iid=$did&filter=$date_filter&op=purge_ok",
                 "Nevermind", "index.pl?iid=$did&isa=$isa&op=show",
       );
       exit(0);
    }

    $DISPLAY->two_choices("Your search did not return any records",
                 "confirm",
                 "Search again","index.pl?iid=$did&isa=$isa&op=purge",
                 "Return to portal","index.pl?iid=$did&isa=$isa",
    );
    return 1;
}
sub www_purge_ok {
    my $self = shift;
    my $iid  = $FORM{iid} || die "Must pass iid!";
    my $date_filter = $FORM{filter} || die "Must pass date filter";

    my $message_list = DBAccess->sqlSelectMany('mid','message',"iid=$iid and updatedate <= \"$date_filter\"");

    while( my ($mid) = $message_list->fetchrow() ){
         Message->_delete_message($mid,$iid);
    }

    my $url = "index.pl?iid=$iid&amp;op=show";

    $self->md_redirect($url);
}




sub get_message_content_type {
    my $self = shift;
    return $PARAMS->{discussion_message_content_type} || DEFAULT_MESSAGE_CONTENT_TYPE();
}

sub message_content_type_is_html {
    my $self = shift;
    my $content_type = $self->get_message_content_type();
    return Utility->content_type_is_html($content_type);
}
















1;

__END__

=head1 NAME

Gizmo::Discussion - descended from L<Gizmo>; controls category threaded discussions

=head1 SYNOPSIS

    use Gizmo::Discussion;
    $discussion = Discussion->new ($id);

    $name = $discussion->get_channel_name();
    $bullet_html = $discussion->show_as_bullet ();
    $detail_html = $discussion->show ();
    $count = $discussion->nb_items;
    $search_res_html = &show_search_results ($gizmo_name,
            $thingtosearch, [$startfrom, $maxitem]);
    $boolean = $discussion->delete();
    $version = Discussion->get_version();


=head1 DESCRIPTION

B<Discussion> allows users to post messages to threads within categories.
Controls adding and modifing of discussion properties.

Constructor. if $id specified then creates Discussion object based on
database info.

    $discussion = Discussion->new ($id);

Class method to retrieve the version of the gizmo

    $version = Discussion->get_version();

Returns channel name of discussion

    $name = $discussion->get_channel_name();

Format when Discussion is displayed as 'bullet'

    $bullet_html = $discussion->show_as_bullet ();

Returns html of threaded discussion

    $detail_html = $discussion->show ();

Returns count of messages in discussion

    $message_count = $discussion->nb_items;

Returns html of search results for a discussion

    $search_res_html = &show_search_results ($gizmo_name,
            $thingtosearch, [$startfrom, $maxitem]);

Deletes discussion, and deletes associated internal channel if it exists.  Returns boolean
status: 1 if okay, 0 if there was an error.

    $boolean = $discussion->delete();


=cut



