###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Faq;
use strict;

use Gizmo::Faq::Question;
use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY);
use Metadot::Controllable;
use Metadot::GizmoContainer;

use vars qw(@ISA);
@ISA=qw(GizmoBuilder Metadot::Controllable Metadot::GizmoContainer);
my $version = "4.5";

sub get_version {
    return $version;
}

sub customize_permissions {
   my $class = shift;
   my $AC = shift;

   $class->SUPER::customize_permissions($AC);

   $AC->add_op('MOD', 'add_child');
   $AC->add_op('MOD', 'create_child');
}


{
    my $search_params = {
             searchable_fields => [ qw( name description keywords ) ],
             displayed_fields  => [ qw( description) ],
             child_isa => 'Question',
             child_searchable_fields => [ qw( name t1 description keywords ) ],
            };

    sub get_search_params {
    my $self = shift;

    return $search_params;
    }
}

####################################
sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

  $self->set_field_info (
    "name", "<b>FAQ Section Name</b>", 1,
    "description", "<b>Description</b>",0,
    );

}


sub get_gizmo_name {
   my $self = shift;
   return "FAQ Section";
}

sub gizmo_list_header {
    my $class = shift;

    return "";
}

sub gizmo_list_decorator {
    return 'HTML::Decorator::NullDecorator';
}

sub www_add_child {
  my $self = shift;
  my $errormsg = shift;
  my $child_type = shift;
  $self->debug_msg(3, "Faq::www_add_child()");

  my $child_isa = $FORM{child_type} || $child_type;
  my $gizmo_name = $child_isa->get_gizmo_name();

  my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
  # we re-instanciate the object because we want a brand new one
  # unless there is an error message (we want to keep his data)
  my $child = $child_isa->new({ parent_id => $parentid });

    $errormsg = $errormsg ?
                "<p /><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p />\n" :
                "";

  my $form = $child->show_edit_form("Adding new $gizmo_name");
  my $content =
  Components->color_palette_code().
"
<form name=fm action='index.pl' method='post' ENCTYPE=\"multipart/form-data\">
<input type='hidden' name='isa' value='Faq'>\n
<input type='hidden' name='iid' value='$parentid'>\n
<input type='hidden' name='op' value='create_child'>\n
<input type='hidden' name='child_type' value='Question'>\n
$errormsg
$form
<table align='right'><tr><td><input type='submit' value='Create'></td></tr></table>
<br />&nbsp;
</form>
";

  $child->print_form($content, "Adding new $child_isa");
}

sub www_create_child {
  my $self = shift;
  my $child_type = shift;

  $self->debug_msg(3, "Gizmo::www_create_child()");

  my $child_isa = $FORM{child_type} || $child_type;

  my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
  # we re-instanciate the object because we want a brand new one
  my $child = $child_isa->new({
                         uid => $USER->uid(),
             parent_id => $parentid,
            });

  # this method should ALWAYS be overridden in derived classes.
  $child->process_edit_form();
}


# returns a set of Question objects
sub get_questions {
    my $self = shift;
    my @results;

    my $iid = $self->get_iid();
    my $rs = DBAccess->sqlSelectMany('iid', "instance", "parentiid = $iid ORDER BY instanceorder");
    while (my ($question_id) = $rs->fetchrow) {
        push @results, Question->restore($question_id);
    }
    $rs->finish();
    return \@results;
}

##########################################
### show
### return one newsitem (page format)
##########################################
sub _show_questions {
  my $self = shift;
  $self->debug_msg(1, "Faq::show()");
  my $s="";
#  print STDERR "in Faq:show()\n";

  my $id = $self->id();
  my $isa = $self->is_a();
  my $name = $self->name();
  my $description = $self->description();
  my $child = "Question";

  my $top_anchor = $self->_generate_top_anchor;
  my $page_id = ($self->{is_inline})? $self->parent_id() : $self->id();

  my $admin_panel;
  if (($self->is_allowed_to_do('modify', $USER)) && ($SESSION->admin_mode())) {
    $admin_panel .= "[ <a href=\""
                 .  $self->make_url("index.pl?iid=$id&amp;isa=$isa&amp;op=add_child&amp;child_type=$child")
                 .  "\">Add Question</a> ]";
  } else {
    $admin_panel .= "";
  }
  $admin_panel .= "<br />" unless (!$admin_panel);
  $name &&= "<b>$name</b><br />";
  $description .= "<br />" unless (!$description);

    my $show_panels;
    if (($self->is_allowed_to_do('modify', $USER)) && ($SESSION->admin_mode())) {
        $show_panels = 1;
    }

    my $question_list_ref =  Question->get_questions_and_answers($self->id(), $show_panels);
    my @question_arr = @{$question_list_ref};

    my $question_list = "<ul>";
    foreach my $question_ref (@question_arr) {
        my %question = %{$question_ref};

        # building question list section
        $question_list .= "<li>$question{panel}<a href=\"index.pl?iid=$page_id#$question{id}\">"
                        . "$question{question}</a>";
    }
    $question_list .= "</ul>";

  $s .= <<EOM
    <a name="$top_anchor"></a>
    $name
    $description
    $admin_panel
    $question_list
EOM
;

  return $s;
}

sub _show_questions_and_answers {
    my $self = shift;
    $self->debug_msg(1, "Faq::show()");
    my $s="";
#   print STDERR "in Faq:show()\n";

    my $id = $self->id();
    my $isa = $self->is_a();
    my $child = "Question";

    my $top_anchor = $self->_generate_top_anchor;
    my $page_id = ($self->{is_inline})? $self->parent_id() : $self->id();

    my $question_list_ref =  Question->get_questions_and_answers($self->id());
    my @question_arr = @{$question_list_ref};

    my $questions_and_answers;

    foreach my $question_ref (@question_arr) {
        my %question = %{$question_ref};
                my $q = Question->new($question{id});
                my $related_url = $q->get_related_url();

        # building question list section
        $questions_and_answers .= "<hr width=\"50%\">"
                                . "<a name=\"$question{id}\"></a><b>$question{question}</b>"
                                . "<p />$question{answer}<p />"
                                                                . "<p />More information on this topic at:<br /><a href='$related_url'>$related_url</a>"
                                . "<table width=\"100%\" border=\"0\"><tr><td align='right'>"
                                . "<a href=\"#$top_anchor\">"
                                . "<span class=\"smaller\">Back to Questions</span></a></td></tr></table>";

    }

  $s .= <<EOM
    $questions_and_answers
    <br />
EOM
;

  return $s;
}

sub _generate_top_anchor {
    my $self = shift;
    my $result = 'anchor_'.$self->get_iid();
    return $result;
}


sub show {
    my $self = shift;

    $self->md_redirect("index.pl?iid=".$self->parent_id);

    exit;
}


sub show_summary {
    my $self = shift;
    my @args = @_;

    my $buttons     = $self->get_buttons();
    my $id = $self->id();
    my $isa = $self->is_a();

    $self->{is_inline} = 1;
    my $text = $self->_show_questions();
    return $buttons.$text;
}

sub show_postscript {
    my $self = shift;
    my @args = @_;

    my $id = $self->id();
    my $isa = $self->is_a();

    $self->{is_inline} = 1;
    my $text = $self->_show_questions_and_answers();
    return $text;
}

sub www_delete_ok {
    my $self = shift;

    # sub www_delete_ok method is called if the FAQ gizmo is being deleted 
    $self->delete_children();

    return $self->SUPER::www_delete_ok(@_);
}

sub delete {
    my $self = shift;

    # sub delete method is called if the page/section is being deleted
    $self->delete_children();
    return $self->SUPER::delete(@_);
}

sub delete_children {
    my $self = shift;

    ## Need to delete all of my children before I go away
    my $children = $self->get_questions();
    foreach my $child(@$children){
       $child->delete();
    }
    return 1; 
}

1;

__END__

=head1 NAME

Gizmo::Faq - A Gizmo subclass to display Faqs and contain FAQ questions.

=head1 SYNOPSIS

    use Gizmo::Faq;
    $Faq = Faq->new([$id]);

    $name = $Faq->get_gizmo_name();
    $item_display_html = $Faq->show();
    $bullet_display_html = $Faq->show_as_bullet();
    $version = Faq->get_version();



=head1 DESCRIPTION

This is a container and interface for viewing FAQ questions and organising them.

=head2 Methods

Constructor;  instantiates a new item or retrive its persistent info from the backend
database.
$id is the $id of the Faq:

    $Faq = Gizmo::Faq->new([$id]);


Class method to retrieve the version of the gizmo

    $version = Discussion->get_version();

Get the name of the item for pretty-printing:

    $name = $Faq->get_gizmo_name();

Generate html to show the image full-size and include its descriptive text:

    $item_display_html = $Faq->show();

Generate html to  'list', or give  abbreviated display of the image information:

    $bullet_display_html = $Faq->show_as_bullet();

=cut


