###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Question;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $DISPLAY);

use vars qw(@ISA);
@ISA=qw(GizmoBuilder);
my $version = "4.5";

sub get_version {
    return $version;
}

####################################
sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

  # aju 11-18-02: use t1 field for question
  $self->set_field_info (
    "t1", "<b>Question</b>", 1,
    "description", "<b>Answer</b>", 1,
    "t2", "<b>Related URL</b>",0,

    );

  $self->set_field_type('t2','text');
}

sub restore {
    my $class = shift;
    my @args = @_;
    my $self = $class->SUPER::restore(@args);

    # for Questions that have not been converted, copy name field
    # contents to t1 field
    if (!$self->get_t1()) {
        $self->set_t1($self->get_name());
    }

    return $self;
}

sub get_gizmo_name {
   my $self = shift;
   return "Question";
}


##########################################
### show
### return one newsitem (page format)
##########################################
sub show {
  my $self = shift;
  $self->debug_msg(1, "Question::show()");
  my $s="";
  #print STDERR "in Question:show()\n";


  # CKIM: this now redirects to the parent page
  my $parent_page = $self->get_navigation_parent();
  my $url = $parent_page->make_gizmo_url();

  # add the anchor to the page
  unless ($url =~ m/#/) {
      $url .= '#'.$self->id();
  }

  # redirect the browser
  $DISPLAY->md_redirect($url);
}


##########################################
### show_in_list
### return one newsitem (list format)
##########################################
# aju: do I need this method?
sub show_in_list {
  my $self = shift;
  $self->debug_msg(1, "Question::show_in_list()");
#    print STDERR "in Question:show_in_list()\n";

  # aju - changed to use t1 field
  my $name        = $self->get_t1();
  my $description = $self->get_description();
  my $url         = $self->get_url();
  my $panel       = $self->get_panel();

  $name = "<img src=$url>"; #if ($url) ;

  return "<b>$name</b> - $panel<br />\n$description<br />\n";
}


sub get_question {
    my $self = shift;
    return $self->t1() || $self->name();
}

sub get_response {
    my $self = shift;
    return $self->description();
}

sub get_related_url {
    my $self = shift;

    return $self->t2();
}

# I don't need this... do I?
##########################################
### show_in_column
### return one newsitem (news column format)
##########################################
#  sub show_in_column {
#    my $self = shift;
#    $self->debug_msg(1, "Question::show_in_column()");
#    print STDERR "in Question:show_in_column()\n";

#    my $iid             = $self->get_iid();
#    my $name            = $self->get_t1();
#    my $description     = $self->get_description();
#    my $url             = $self->get_url();
#    my $longdescription = $self->get_long_description();
#    my $panel           = $self->get_panel();
#    my $s               = "";

#    if($longdescription || $url){
#      $name = "<a class='newsitemtitle' href='index.pl?id=$iid&amp;isa=Question&amp;op=show'>$name</a>";
#    }

#    $s.= $panel;
#    $s.= "<table cellpadding=\"0\" cellspacing=\"0\"><tr><td class='newsitemtitle'>- $name\n</td></tr>\n";
#    $s.= "<tr><td class='newsitem'>$description<br />&nbsp;</td></tr></table>\n\n";

#    return $s;
#  }


sub show_summary {
    my $self = shift;
    my @args = @_;

    my $buttons     = $self->get_buttons();
    my $text = $self->show(@args);

    return $buttons.$text;
}

sub get_questions_and_answers {
    my $class = shift;
    my $parentid = shift;
    my $panels = shift || 0;
    my $order = shift || "InstanceOrder";
    my @data;

    $parentid = undef unless Utility->is_valid_integer($parentid); # secu_200401

    # aju 11-18-02 - changed to use t1 field
    # for backwards compatibility, get name field too, and use it if necessary to populate
    # t1 field.
    my $params = "iid, t1, name, description";
    my $result = DBAccess->sqlSelectMany($params, "instance",
                                        "parentiid = $parentid ORDER BY $order");

    while (my ($id, $question, $qbackup, $answer) = $result->fetchrow()) {
        my %data;

        if ($panels) {
            my $obj = Question->new($id);
            $data{panel} = $obj->get_buttons();
        } else {
            $data{panel} = "";
        }

        $data{id} = $id;
        if ($question) {
            $data{question} = $question;
        }
        else {
            $data{question} = $qbackup;
        }
        $data{answer} = $answer;

        my $data_ref = \%data;
        push (@data, $data_ref);
    }

    my $list = \@data;

    return $list;
}


sub _show_create_success {
    my $self = shift;

    use Gizmo::Faq;
    my $parent = Faq->new($self->parent_id());
    my $catid = $parent->parent_id();

    my $isa = $self->is_a();
    my $parentid = $self->parent_id();

    $self->two_choices("<big>New " . $self->get_gizmo_name() .
            " added!</big><br /><br />What do you want to do?",
            "confirm",
            "Add another " . $self->get_gizmo_name(),
            "index.pl?child_type=$isa&isa=Category&iid=$parentid&op=add_child&app=",
            "Enough for now!",
            "index.pl?id=$catid");

}

sub _show_edit_success {
    my $self = shift;

    # This redirects to the parent page
    my $parent_page = $self->get_navigation_parent();
    my $url = $parent_page->make_gizmo_url();
    $DISPLAY->md_redirect($url);

}

sub show_panel {
    my $self = shift;
    my $id = shift || $self->id();
    my $isa = shift || $self->is_a();
    $self->debug_msg(5, "Question::show_panel(id=$id, isa=$isa)");

    return '' unless ($SESSION->admin_mode());

    my $str = "";
    $str .= $self->show_edit_button();
    $str .= $self->show_ac_button();
    $str .= $self->show_del_button();

    return $str;
}


# returns the FAQ's name
sub get_search_display_name {
    my $self = shift;
    my $parent = $self->get_parent();
    return $parent->get_search_display_name();
}


1;

__END__

=head1 NAME

Gizmo::Question - A Gizmo subclass to display Questions.

=head1 SYNOPSIS

    use Gizmo::Question;
    $Question = Question->new([$id]);

    $name = $Question->get_gizmo_name();
    $item_display_html = $Question->show();
    $bullet_display_html = $Question->show_as_bullet();
    $version = Discussion->get_version();



=head1 DESCRIPTION

=head2 Methods

Constructor;  instantiates a new item or retrive its persistent info from the backend
database.
$id is the $id of the Question:

    $Question = Gizmo::Question->new([$id]);


Class method to retrieve the version of the gizmo

    $version = Discussion->get_version();

Get the name of the item for pretty-printing:

    $name = $Question->get_gizmo_name();

Generate html to show the image full-size and include its descriptive text:

    $item_display_html = $Question->show();

Generate html to  'list', or give  abbreviated display of the image information:

    $bullet_display_html = $Question->show_as_bullet();

=cut


