###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name$
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package FormMaker;


use strict;
use Sendmail;
use Text::CSV_XS;
use CGI qw(escapeHTML);


use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $DISPLAY $PARAMS $CACHE %REQUEST_VARS);

use DBAccess;
use Gizmo::FormMaker::FormMakerElement;
use Gizmo::FormMaker::FieldElement;
use Metadot::Controllable;
use Metadot::UploadsManager;
use Metadot::MIME::Types;
use Form::Verify;
use Metadot::GizmoContainer;

use MIME::Entity;

use vars qw(@ISA);
@ISA=qw(GizmoBuilder Metadot::Controllable Metadot::GizmoContainer);
my $version = "1.2";

sub get_version {
  return $version;
}


my $AC =
  {
   BUNDLES => {
               MOD => { DESC => 'Edit',
                        #CKIM-SHORTCUTS: added a new op 'create_shortcut'
                        OPS => [qw(add_child create_child modify save up down paste create_shortcut delfile delfileok download_file)],
                        DEFAULT_LEVEL => 8,
                        MIN_LEVEL => 2,
                      },
               DEL => { DESC => 'Delete and Cut',
                        OPS => [qw(delete delete_ok cut)],
                        DEFAULT_LEVEL => 8,
                        MIN_LEVEL => 8,
                        PARENT_DEP => {cut => 'MOD'},
                      },
               DISP => { DESC => 'View',
                         OPS => [qw(show show_summary process_form)],
                         DEFAULT_LEVEL => 0,
                         MIN_LEVEL => 0,
                         DEFAULTS_TO => 'on',
                       },
               EDITP => { DESC => 'Change<br />Permissions',
                          OPS => [qw(edit_permissions set_permissions change_owner)],
                          DEFAULT_LEVEL => 8,
                          MIN_LEVEL => 8,
                        },
              },
   DISPLAY_ORDER => [qw(DISP MOD DEL EDITP)],
  };


###
### ACCESS CONTROL METHODS
###
sub get_AC {
  my $self = shift;
  my $class = ref $self || $self;

  die "&get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

  return $AC;
}

use constant FORMAT_PLAIN_TEXT => 0;
use constant FORMAT_HTML => 1;
use constant FORMAT_CSV  => 2;
use constant FORMAT_TEMPLATE => 3;
use constant FORMAT_DB => 4;

use constant FORMATS => [
    FORMAT_PLAIN_TEXT,
    FORMAT_HTML,
    FORMAT_CSV,
    FORMAT_TEMPLATE,
    FORMAT_DB
    ];

use constant EXTERNAL_TABLE_PROPERTY => 't6';

####################################
sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

  $self->_form_initialize();
  $self->set_field_info (
                         "name", "<b>Form Title</b>", 1,
                         "description", "<b>Description</b>", 0,
                         "t1", "<b>Email To</b>", 1,
                         "i5", "<b>Set form submitter as sender and reply-to address</b>", 0,
                         "t2", "<b>Subject Template</b>", 0,
                         "t3", "<b>Body Template</b>", 0,
                         "t4", "<b>Body Template for Submitter</b>", 0,
                         "t5", "<b>Body Template for Submitter</b>", 0,
                         "i3", "<b>Email Format</b>", 0,
                         "c4", "<b>Custom 'thank you' message</b>", 0,
                         "c5", "<b>Landing URL after form submission</b>", 0,
                         "i4", "Enable/Disable FormMaker email-to-submitter", 0,
                         EXTERNAL_TABLE_PROPERTY(), "External table to save data", 0,
                        );

  $self->set_field_type('i3', 'select');
}


sub get_field_names {
  my $self = shift;
  #     print STDERR "in get_field_names\n";
  #return the list of set field used by the gizmo object
  my @flist = ();
  foreach my $field (keys (%{$self->{fields}} ) ) {
    push @flist, $field;
  }
  return @flist;
}

## will replace the html produced by GizmoBuilder for given instance variable
sub _get_alt_text {
  my $self = shift;
  my $field = shift;
  my $id = $self->id();

  return unless defined $field;

  my $value = $FORM{$field} || $self->{$field};
  my $info = "<input type=text size=\"20\" maxlength=\"30\" name=\"$field\" value=\"$value\" />";
  return $info;
}

sub get_gizmo_name {
  my $self = shift;
  return "FormMaker";
}

#override one of the default form types.
sub _form_initialize {
  my $self = shift;

  $self->{fields}->{i3}->{formtype} = "select";
  $self->{fields}->{i3}->{selectsize} = "1";
  $self->{fields}->{i3}->{selectoptions} =
      join(
      ',',
      FORMAT_PLAIN_TEXT, FORMAT_HTML, FORMAT_CSV, FORMAT_TEMPLATE,
      );
  my $ts = "

";
  $self->set_intro_text($ts);
}


sub get_submitter_email_field {
  my $self = shift;
  return $self->{i5};
}

sub get_email_to {
  my $self = shift;
  return $self->{t1};
}

sub get_format{
  my $self = shift;
  return $self->{i3};
}

sub get_subject_template {
  my $self = shift;
  return $self->{t2};
}

sub get_landing_url {
  my $self = shift;
  return $self->{c5};
}


sub get_body_template {
  my $self = shift;
  return $self->{t3};
}

sub get_subject_template_submitter {
  my $self = shift;
  my $str = $self->{t4} || "Thank you for your submission!";
  return $str;
}

sub get_body_template_submitter {
  my $self = shift;
  my $str = $self->{t5};

  if ( $str eq '' ) {
    $str  = "This message is to confirm your submission, and let you know that we ";
    $str .= "have received your message.";
  }
  return $str;
}

sub get_default_message {
  return "Thank you for your submission.";
}

sub get_form_maker_message {
  my $self = shift;
  if ( defined($PARAMS->{custom_form_maker} ) && $PARAMS->{custom_form_maker} ne '' ) {
    return $PARAMS->{custom_form_maker};
  }
  return $self->get_default_message();
}

sub external_table {
    my $self = shift;
    my $property = EXTERNAL_TABLE_PROPERTY();

    if (@_) {
        return $self->$property(@_);
    } else {
        return $self->$property();
    }
}



###############################################
#
# This function display the 'thank you' msg after the user submits the form
#
sub get_success_page {
  my $self = shift;
  my $thank;
  my $local_message = $FORM{c4} || $self->get_c4();


  if ( $local_message ne '' ) {
    $thank = $local_message;
  } elsif ( defined($PARAMS->{custom_form_maker} ) && $PARAMS->{custom_form_maker} ne '' ) {
    $thank = $PARAMS->{custom_form_maker};
    my $translate_flag = 1;
    $thank = escapeHTML($thank);
    $thank = $self->translate_white_space($thank,$translate_flag);
  } else {
    $thank = $self->get_default_message();
  }

  $DISPLAY->print_choices(
      "$thank",
      'readme', "Return to the portal", "index.pl?iid=".($FORM{caller_iid} || 0)
     );
}


sub show_edit_form {
  my $self  = shift;
  my $title = shift;

  $self->debug_msg(4, $self->{is_a}."::show_edit_form($title)");
  my $id   = $self->id;
  my $name = $self->name || $FORM{name} ;
  $name = escapeHTML($name);
  my $description = $self->description  || $FORM{description} ;
  my $email_to = $self->get_email_to()  || $FORM{t1} ;
  my ($txt_format, $html_format, $csv_format, $custom_format);
  my $format = $self->get_format();
  my $childs_list = '<table>';
  if ($format==FORMAT_HTML) {
    $html_format="checked";
  } elsif ($format==FORMAT_CSV) {
    $csv_format="checked";
  } elsif ($format==FORMAT_TEMPLATE) {
    $custom_format="checked";
  } else {
    $txt_format="checked";
  }

  my $submitter_reply_to = $self->get_submitter_email_field();
  if( $submitter_reply_to == 1 ){
      $submitter_reply_to = 'checked';
  } else {
      $submitter_reply_to = '';
  }

  if ($id) {
    $childs_list .= $self->show_childs_list( type=>'FormMakerElement', decorator=>'HTML::Decorator::NullDecorator');
  }
  $childs_list .= '</table>';

  my $output = "\n<h1>$title</h1>";
  $output .=  "
        <p />The form Name will be used as its title when the form is displayed in a page.<br />
        The optional Description will be displayed under the name and may
        contain additional instructions for filling out your Form.<br />
     The information in the submitted form will be emailed to the indicated address.
     The 'Email format' is the format that will be used when emailing the submitted form information.
     Comma-separated 'csv' format will allow you to import data to MS Excel or similar programs.<br />
     Click 'Add new field' to start defining your form fields, once you have filled out
     the required information <br />

        <p /><b>Name</b> (*)<br />
      <input type='text' name='name' size='55' maxlength='255' value=\"$name\" /><br />
      <p /><b>Description</b><br />
      <textarea name='description' rows=2 cols=46 WRAP='VIRTUAL'>$description</textarea>

      <br /><hr />
                <p><h2>Email form submissions to FormMaker manager</h2></p>
        <p><b>Mail form submissions to this email address</b> (*)<br />
      <input type='text' name='t1' size='55' maxlength='255' value=\"$email_to\" />
      <br />
          Add a comma between email addresses.
          <p /><input type='checkbox' name='i5' value='1' $submitter_reply_to /> <b>Set form submitter
          as the form sender and reply-to address</b><br />Example: When Joe (joe\@somecompany.com) submits
          a form and this feature is enabled, the email received by the form manager will display joe\@somecompany.com
          as the sender of the form. Therefore allowing the form manager to reply to the submitter easily.<br /><br />
          <b>Important notes</b>:
          <ul>
           <li>A field of type 'email' must be present in the form for this feature to work
          (e.g. for the customer email address choose <i>Add a field</i> of 'email' type).
           <li>The FormMaker will use the first email address field in the form
          </ul>
          <br />
          </p>
        <p><b>Custom subject line for emails</b><br />
      <input type='text' name='t2' size='80' maxlength='255' value=\"".$self->get_t2()."\" />
      <br />
          Optionally specify a custom subject line for emails. If you do not
          specify the subject line, a standard subject line will be used.".
            $self->get_template_field_explanation()."</p>";

  $output .= "<p /><b>Email Format</b><br />
      <input type='radio' name='i3' value='".FORMAT_PLAIN_TEXT."' $txt_format /> Human-readable format plain-text <br />" .
        #       "<input type='radio' name='i3' value='".FORMAT_HTML."' $html_format /> Human-readable format html <br />" .
        "<input type='radio' name='i3' value='".FORMAT_CSV."' $csv_format /> CSV (comma-separated) format<br />
         <input type='radio' name='i3' value='".FORMAT_TEMPLATE."' $custom_format /> Custom format (define the template below)<br />
        <p><b>Custom body for emails</b> (choose Custom format above)<br />
      <textarea name='t3' cols='80' rows='8' wrap='virtual'>".escapeHTML($self->get_t3())."</textarea>
      <br />
          Optionally specify a custom body for emails. This is required if you
          selected 'Custom format' above. If you do not specify the custom body,
          then a standard plain-text email will be sent.".
            $self->get_template_field_explanation()."</p>";

  $output .= "<p><b>Custom \"thank you message\" for FormMaker gizmo submissions</b><br />
                 This message will be displayed when a user submits a form created by the FormMaker gizmo.
                 If the field below is left blank, the message <b><i>".  $self->get_form_maker_message() . "</i></b>
                 will be displayed.<br />
                 <textarea rows='4' cols='40' wrap='physical' name='c4'>". $self->get_c4() ."</textarea>
                 </p><hr />\n\n";

################################
# Email reply to submiter
################################
  $output .= "<h2>Email reply to submitter</h2><p />


This option allows to send an email to the form submitter when s/he submits the form. The form manager will get a copy of it appended to his/her own email notification.

<p><b>Important note</b>: an 'email' field must be present in the form for this feature to work (e.g. for the customer email address choose 'Add a field of 'email' type)</p>

If you are not able to select enable/disable it is because your Metadot install has not been configured to do so. Please ask your tech support to enable it for you.\n\n<br /><br />";


  my $email_disabled_selected = 'checked';
  my $email_enabled_selected = '';

  my $disable_radio = 'DISABLED';

  if ( !defined($PARAMS->{email_to_submitter}) ||  !$PARAMS->{email_to_submitter} ) {
    $output .= "<b>FormMaker email-to-submitter capability is disabled in this system. Please contact your
                          system administrator to change this setting.</b><br />";

  } else {
    $disable_radio = '';
    #
    #
    #
    if ( $self->send_email_to_submitter()) {
      $email_disabled_selected = '';
      $email_enabled_selected = 'checked';
    }
  }
  $output .= "<p><input type='radio' name='i4' value='1' $email_enabled_selected $disable_radio>
                      Enable FormMaker to send a reply to the submitter of this form<br />
                      <input type='radio' name='i4' value='0' $email_disabled_selected $disable_radio>
                      Disable email reply</p>";

  $output .= "<p><b>Subject line for emails sent to submitter</b><br />
               <input type='text' name='t4' size='80' maxlength='255' value='" . $self->get_t4() ."' /><br />
               Optionally specify a subject line for emails. If you do not
               specify the subject line, a standard subject line will be
               used.". $self->get_template_field_explanation() ."</p>";

  $output .= "<p /><b>Message for emails sent to submitter</b><br />
                <textarea rows='8' cols='80' wrap='physical' name='t5'>" . escapeHTML($self->get_t5()) ."</textarea><br />";



  #############################################
  # Save to external table
  #############################################

  my $external_table_property = EXTERNAL_TABLE_PROPERTY();
  my $external_table_value    = $self->external_table() || '';
  $output .= <<EOM
  <hr />
  <h2>Save form data to external table</h2>
  <p>
    FormMaker can save your data to an external table. In order to do this, you
    must:
    <ul>
      <li>Create a table within the existing metadot database</li>
      <li>
        Create a column for each form field that you define. FormMaker will not
        check that the column is the correct data type; that is the
        responsibility of the person creating the external table.
      </li>
    </ul>
  </p>
  <p>
    <b>Name of external table</b><br/>
      <input type="text" name="$external_table_property" value="$external_table_value" />
  </p>
EOM
;




  $output .= "<p /><hr /><h2>Form field definitions</h2><b>Form Fields</b><br />
          $childs_list<br />

      <input type='submit' name='new_field'  value='Add New Field' />
      <p /><i>(*) Mandatory field(s)</i><br />";

  ### Landing URL
  $output .= "<p /><b>Landing URL after form submission</b><br />
      <input type='text' name='c5' size='80' maxlength='111' value=\"" . $self->get_landing_url()."\" />
      <br />" . "Warning: having a landing URL will redirect the user directly to this url after submission";
  ### Attachments
  $output .=  "<p /><hr /><h2>Form attachments to be sent to the form submitter</h2>Optionally you can upload attachments here, as many as needed. They will be sent to the form submitter along with the email. The FormMaker manager will not get these by email, only the form submitter.<br /><br /><br /> <p/>
  <input type='submit' name='add_attachment'  value='Add attachment' />";

  ####
  $childs_list = $self->show_childs_list( type=>'Item', decorator=>'HTML::Decorator::NullDecorator');
  $output .= $childs_list . "<br /><br />\n\n";

  return $output;
}



######################################################
#
#
#
sub get_upload_filename_icon_and_url {
  my $self = shift;
  return $self->SUPER::get_upload_filename_icon_and_url( {
                                  id => $self->id(),
                                  isa => $self->is_a(),
                                  field_name => 'item_attachment_file',
                                 });
}

#####################################
## called before saving or updating
#####################################
sub process_edit_form {
  my $self  = shift;
  my $error = shift || undef;
  $FORM{name} = escapeHTML( $FORM{name} );

  ## check for required fields
  foreach my $field (keys (%{$self->{fields}})) {

    # upload fields do their own "is_required" checking
    next if ( $self->{fields}{$field}{formtype} =~ /_upload$/ );

    if ($self->{fields}->{$field}->{required} &&
        (!$FORM{$field} || !($FORM{$field} =~ /\S/)) # field is not just whitespace
       ) {
      $error .= "<br />".$self->{fields}->{$field}->{label} .
        " is a required field.\n";
    }
  }

  # if email format 3 is selected, then a custom body is required
  if ($FORM{i3} == FORMAT_TEMPLATE && !$FORM{t3}) {
    $error .= "<br />You have selected a custom email format, but did not specify the custom body.\n";
  }

  my $iid = $self->get_iid();

  if (defined($error) && $iid) { ## if error and updating

    $self->www_modify($error);
  } elsif (defined($error)) {   ## if error and saving new
    my $parentid = $self->parent_id();
    my $parent = Gizmo->restore($parentid);
    #          print STDERR "in gizmo::process_edit_form, calling parent add child on error\n";
    $parent->www_add_child($error, $self->is_a);
  } else {                      ## no errors
    my $checked_reqs = 1;
    $self->www_save($checked_reqs);
    $FORM{child_type} = 'FormMakerElement';
  }
}



##############################
# this method handles what to do when you are finished successfully creating a new gizmo
##############################
sub _show_create_success {
  my $self = shift;

  if ($FORM{add_attachment}) {
    $FORM{child_type} = 'Item';
    $FORM{name} = '';
    $FORM{description} = '';
    $self->www_add_child('', 'Item');
  } else {
    if ($FORM{new_field}) {
      $FORM{child_type} = 'FormMakerElement';
      $FORM{name} = '';
      $FORM{description} = '';
      $self->www_add_child('', 'FormMakerElement');
    } else {
      #$self->www_modify();
      my $parentid = $self->parent_id();
      my $url = "index.pl?id=$parentid";
      $self->md_redirect($url);
    }
  }
}


########################################
# _show_edit_success
########################################
sub _show_edit_success {
  my $self = shift;


  #
  # Adding attachments (many possible via Item gizmo)
  #
  if ($FORM{add_attachment}) {
    $FORM{child_type} = 'Item';
    $FORM{name} = '';
    $FORM{description} = '';
    $self->www_add_child('', 'Item');
  } else {
    #
    # Adding a field to the FormMaker
    #
    if ($FORM{new_field}) {
      $FORM{name} = '';
      $FORM{description} = '';
      $self->www_add_child('', 'FormMakerElement');
    } else {
      $self->md_redirect("index.pl?iid=".$self->parent_id);
    }
  }
}




#########################################
#
# show_childs_list: return HTML with a bullet point type of list
# for the type of childs sent as arg.
#########################################
sub show_childs_list {

  my $self = shift;
  my %args = @_;

  my $id = $self->{id};
  $self->debug_msg(3, "show_childs_list(id=$id)");

  my $str = "";
  return "" unless defined($id);

  my @childs_list = @{$self->build_gizmo_list(%args)};
  foreach my $gizmo (@childs_list) {
    $str .= $gizmo->show_summary();
  }

  $self->debug_msg(4, "show_childs_list completes");
  $self->debug_msg(9, "show_childs_list completes with <xmp>$str</xmp>");
  return $str;

}



##########################################
### show
###
###
sub show {
  my $self        = shift;
  my $isa         = $self->is_a();
  my $iid         = $self->get_iid();
  my $name        = $self->get_name();
  my $description = $self->get_description();
  my $buttons     = $self->get_buttons();

  $self->debug_msg(3, "FormMaker::show()");

  my $form = "<form name='form$iid' action='index.pl' method='post'>";

  #just do inline alignment for the 'bullet' view.
  my $default_bullet_align = "top"; # default alignment for bullet list is 'top'.
  # The actual set alignment is used in
  # the more detailed view.
  my $childs_list .= $self->show_childs_list( type=>'FormMakerElement',
                            decorator=>'HTML::Decorator::NullDecorator'
                          );
  my $html = "$buttons";

  # show any errors
  if (@{$REQUEST_VARS{formmaker_errors}})
  {
    $html .= qq~<div class="error_message">\n~;
    foreach my $error (@{$REQUEST_VARS{formmaker_errors}})
    {
      $html .= qq~  <div class="error_item">$error</div>\n~;
    }
    $html .= qq~</div>\n~;
  }

  $html .= "
        <b>$name</b></td></tr>$form
        <tr><td></td><td>&nbsp;</td></tr>
        <tr><td></td><td>$description</td></tr>
        <tr><td></td><td>&nbsp;</td></tr>
        <tr><td></td><td><table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">$childs_list</table></td></tr>
        <tr><td></td><td align='center' height='60' valign='middle'>
        <input type='reset' value='Clear' />
        <input type='submit' name='Submit' value='Submit' />
        <input type='hidden' name='isa' value='$isa' />
        <input type='hidden' name='iid' value='$iid' />
        <input type='hidden' name='caller_iid' value='".($FORM{iid} || $FORM{id})."' />
        <input type='hidden' name='op' value='process_form' />
        </form>
        ";


  return $html;
}


##########################################
### show_in_list
### return one newsitem (list format)
##########################################
# aju: do I need this method?
sub show_in_list {
  my $self        = shift;
  my $isa         = $self->is_a();
  my $iid         = $self->get_iid();
  my $name        = $self->get_name();
  my $description = $self->get_description();
  my $buttons     = $self->get_buttons();

  $self->debug_msg(1, "FormMaker::show_in_list()");

  my $form = "<form name='form$iid' action='index.pl' method='post'>";

  #just do inline alignment for the 'bullet' view.
  my $default_bullet_align = "top"; # default alignment for bullet list is 'top'.
  # The actual set alignment is used in
  # the more detailed view.
  my $childs_list .= $self->show_childs_list( type=>'FormMakerElement', decorator=>'HTML::Decorator::NullDecorator');
  my $html        = "$buttons";

  $html .= "
        <b>$name</b></td></tr>$form
        <tr><td></td><td>&nbsp;</td></tr>
        <tr><td></td><td>$description</td></tr>
        <tr><td></td><td>&nbsp;</td></tr>
    <tr><td></td><td><table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">$childs_list</table></td></tr>
        <tr><td></td><td align='center' height='60' valign='middle'>
        <input type='reset' value='Clear' />
        <input type='submit' name='Submit' value='Submit' />
        <input type='hidden' name='isa' value='$isa' />
        <input type='hidden' name='iid' value='$iid' />
        <input type='hidden' name='caller_iid' value='".($FORM{iid} || $FORM{id})."' />
        <input type='hidden' name='op' value='process_form' />
        </form>
        ";

  return $html;
}

####################
# show_summary
####################
sub show_summary {
  my $self        = shift;
  my $isa         = $self->is_a();
  my $iid         = $self->get_iid();
  my $name        = $self->get_name();
  my $description = $self->get_description();
  my $buttons     = $self->get_buttons();

  $self->debug_msg(1, "FormMaker::show_summary()");

  my $form = "<form name='form$iid' action='index.pl' method='post'>";
  my $send_user_info='';
  #         my $logged_in = $USER->is_logged_in() || 0;
  #         if ( $logged_in >= 1) {  # if logged in
  #              $send_user_info = "
  #              <tr><td></td><td colspan='2'><input type='checkbox' name='send_user_info' value='1'>
  #               Submit my name and email with this form.
  #              </td></tr>
  #              ";
  #         }

  #just do inline alignment for the 'bullet' view.
  my $default_bullet_align = "top"; # default alignment for bullet list is 'top'.
  # The actual set alignment is used in
  # the more detailed view.
  my $childs_list .= $self->show_childs_list( type=>'FormMakerElement', decorator=>'HTML::Decorator::NullDecorator');
  my $html        = "$buttons";


  $html .= "
         $form
         <b>$name</b><br />
         $description<br />
         <table border=\"0\" cellspacing=\"0\" cellpadding=\"2\">
         $childs_list
          <tr>
            <td colspan='2'></td>
            <td align=\"left\">
                 <input type='reset' value='Clear' />
                 <input type='submit' name='Submit' value='Submit' />
                 <input type='hidden' name='isa' value='$isa' />
                 <input type='hidden' name='iid' value='$iid' />
                 <input type='hidden' name='caller_iid' value='".($FORM{iid} || $FORM{id})."' />
                 <input type='hidden' name='op' value='process_form' />
             </td>
           </tr>
         </table>
         </form>
         $send_user_info
         ";


  return $html;
}

####################
# _removeH
####################
sub _removeH {
  my ($self, $value) = @_;
  return $value unless defined($value);
  $value =~ s/<(?:[^>'"]*|(['"]).*?\1)*>//gs;
  return $value;
}



#####################################################
######                                        #######
######            FRONT-END METHODS           #######
######                                        #######
#####################################################
sub www_process_form {
  my $self = shift;

  $self->debug_msg(3, "In $self\->www_process_form");


  my $name = $self->_removeH($self->get_name());
  my $btitle = (uc $name) . " FORM SUBMISSION";
  my $btitle_text = "######\n#\n# $btitle\n#\n######\n\n";
  my $format = $self->get_format();
  my $custom_submitter_subject = $self->get_subject_template_submitter();
  my $custom_submitter_body = $self->get_body_template_submitter();
  my $email_to= $self->get_email_to();
  my $cc;
  my ($field_name,$field_type,$field_text,$field_value);
  my (@childs_list, @allowed_list);
  my $gizmo;
  my $subj;
  my $subj_submitter;
  my $body_submitter;
  my @recipient_list;
  my $email_from;
  my @attachments;
  my $msg;
  my $csv = Text::CSV_XS->new({binary => 1});
  my $str;
  my $basename = $PARAMS->{basename} ;
  my $parentid = $self->parent_id();
  my $link = $basename . "/index.pl?iid=$parentid";
  my $number_of_attachments=0;
  my $filename_msg="";

  @childs_list = @{$self->build_gizmo_list(type=>'FormMakerElement',
                         decorator=>'HTML::Decorator::NullDecorator'
                        )
           };

  #
  # Get the list of custom fields
  #
  foreach $gizmo (@childs_list) {
    if ($gizmo->is_allowed_to_do('show', $USER) ) {
      #print STDERR "pushing $gizmo\n";
      push @allowed_list, $gizmo;
    }
  }


  #
  # Collect all the field values
  #
  my %field_values = ();
  my %field_types  = ();
  my @form_errors = ();
  foreach $gizmo (@allowed_list) {
      my $field_name = $gizmo->get_name();
      $field_types{$field_name}     = $gizmo->get_type();

      my $field_value               = $gizmo->get_form_value();
      $field_values{$field_name}    = $field_value;
      if ($gizmo->is_mandatory() && (!defined $field_value || $field_value eq ''))
      {
        my $field_label = $gizmo->description();
        push @form_errors, qq~<em>$field_label</em> is a required field. Please check the form and re-submit.~;
      }
  }


  # if there was an error, send the user back to the form
  if (@form_errors)
  {
    $REQUEST_VARS{formmaker_errors} = \@form_errors;
    return $self->www_show();
  }


  #
  # Save to an external table
  #
  if (my $external_table = $self->external_table()) {
      my %row_values = ();
      while (my ($field_name, $field_value) = each %field_values) {
          $row_values{$field_name} = $self->_format_value(
              value     => $field_value,
              'format'  => FORMAT_DB(),
          );
      }

      eval {
          DBAccess->sqlInsert($external_table, %row_values);
      };
      if ($@) {
          ErrorHandling->printError("Error writing data to external table '$external_table'<br/>Values are ".$DISPLAY->dumper(\%row_values));
      }
  }



  #
  # Get customer email fields (recipients for a reply)
  #
  while (my ($field_name, $field_value) = each %field_values) {
      $field_type = $field_types{$field_name};
      if ($field_type eq 'Email') {
        if ($field_value ne '') {
          push @recipient_list, $field_value;
        }
      }
  }


  #print STDERR "RECIPIENT_LIST[@recipient_list]+++++++++++++++++++\n\n\n";
  if ($self->get_subject_template()) {
    $subj = $self->process_template($self->get_subject_template(), \%field_values);
  } else {
    $subj = "[$name] - Form Submission";
  }

  #####################
  # PREPARE EMAIL TO THE FORM MANAGER
  #####################
  $subj_submitter = $self->process_template($custom_submitter_subject, \%field_values);
  $body_submitter = $self->process_template($custom_submitter_body, \%field_values);

  # get user info
  my $intro = "This is an automatically-generated email sent from $basename" . ".\n  Please do not reply.\n";
  my $user_info ='';
  my $footer_text = "\n--\nEmail generated by Metadot FormMaker v"
    . $self->get_version() . " on " . localtime(time) . "\n"
      . "From: $link\n";
  # we need to add the IP# of submitter

  my $footer_html = "--<br />\nEmail generated by Metadot FormMaker v"
    . $self->get_version() . " on " . localtime(time) . "<br />\n" .
      "From: $link<br />\n" .
        "<br />\n";


  $self->debug_msg(5, "$self\->www_process_form: about to process body using format '$format'");
  $self->debug_msg(5, "$self\->www_process_form: my body template is: ".$self->get_body_template());


  ##################################
  # PREPARING EMAIL MSG
  ##################################
  if ($format == FORMAT_TEMPLATE && $self->get_body_template()) {
    $msg = $self->process_template($self->get_body_template(), \%field_values);
  } else {
    # standard msg processing

    if ($format == FORMAT_HTML) {
      $msg .= "<html><body>$btitle<br /><br /><table border=\"0\">";
      $msg .= "<tr><td width='30%'></td><td width='70%'></td></tr>\n";
    } elsif ($format == FORMAT_CSV) {
      $msg .= "$btitle_text\n\nField Name,Field Value\n";
    } else {
      $msg .= "$btitle_text\n\n";
    }

    $self->debug_msg(5, "$self\->www_process_form -  about to build element list");


    foreach $gizmo (@allowed_list) {
      $field_text = $self->_removeH($gizmo->get_description());
      $field_name = $gizmo->get_name();
      $field_type  = $gizmo->get_type();
      $field_value = $self->_format_value( value => $field_values{$field_name}, 'format' => $format);
      my $field_id = $gizmo->get_id();
      if ($field_type eq 'TextBlock') {
        next;
      }
      if ($field_type eq 'Checkbox') {
        if ($field_value) {
          $field_value='Yes';
        }
      }
      if ($format == FORMAT_HTML) {
        $msg .= "<tr><td>$field_text</td><td>$field_value</td></tr>\n";
      } elsif ($format == FORMAT_CSV) {
        #
        # CSV Format
        #
        my @cols = ($field_name, $field_value);
        my $status = $csv->combine(@cols); # combine columns into a string
        my $str = $csv->string(); # get the combined string
        #     print STDERR "got csv string $str from $field_value\n";
        $msg .= $str."\n";
      } else {
        $msg .= "### $field_text\n$field_value\n\n";
      }
    }
  }


  ##################################
  #END OF PREP MSG
  ##################################


  #print $msg; # debug
  $self->debug_msg(5, "In $self\->www_process_form, about to send email with subject: $subj<br />message: $msg");


  ### check to see if email_to_submitter is set
  my $local_email_to_submitter = $self->send_email_to_submitter();
  if ( !defined($local_email_to_submitter) ) {
    $local_email_to_submitter = 1;
  }



  ##############################################################
  # SEND AN EMAIL TO THE SUBMITTTER OF THE FORM (if needed only)
  ##############################################################


  # print STDERR "Recipient list: @recipient_list\n\n";

  if ( (defined($PARAMS->{email_to_submitter})
        && $PARAMS->{email_to_submitter}) && $local_email_to_submitter == 1 ) {
    my @child_attachments = @{
        $self->get_childs_list({ type=>'Item', decorator=>'HTML::Decorator::NullDecorator'})
        };
    $number_of_attachments = scalar @child_attachments;
    my $mail_type = '';
    my $attach_path = '';

    #      print STDERR "Will send email to submitter with $number_of_attachments\n";
    #########################################
    ### build message to be sent to the submitter of the form
    #########################################


    $email_from = $PARAMS->{mailsenderaddress};

    my $i=0;
    #    print STDERR "# of recipient in the submitter list: " . scalar @recipient_list . "\n";
    foreach my $recipient (@recipient_list) {
      #      print STDERR $i++ . "-- will send to RECIPIENT [$recipient] FROM  email_from[$email_from]\n";
      # print STDERR "mail_type[$mail_type] TO email_to[$email_to]\n";

      # Verify email address of recipient
      my ($valid_email,$err_message) = $self->Form::Verify::email($recipient);
      unless( $valid_email ){
        $self->invalid_email($recipient);
        exit(0);
      }

      my %email_params;

      #
      if ($number_of_attachments==0) { ### No attachments
        ### simple text container
        my $mail_type = 'text/plain';
        %email_params = (
                         From    => $email_from,
                         To      => $recipient,
                         #Bcc     => $email_to,
                         Subject => $subj_submitter,
                         Data    => $body_submitter,
                         Type    => $mail_type
                        );

      } else {                  ### There are attachments
        ### multi-type container
        $mail_type = 'multipart/mixed';
        %email_params = (
                         From    => $email_from,
                         To      => $recipient,
                         #Bcc     => $email_to,
                         Subject => $subj_submitter,
                         Type    => $mail_type
                        );

      }

      ##########################################
      # Create mail container
      # USING MIME::Entity (part of MIME::Tools)
      ##########################################
      my $emsg = MIME::Entity->build(%email_params);
      # Add the text message part


      ########################################
      # Adding each attachment to the email
      ########################################
      my ($attch_name, $xattach, $url);
      my $i=0;
      # print STDERR "before attach foreach\n";
      if ($number_of_attachments > 0) {
          $emsg->attach (
                     Type => 'TEXT',
                     Data => $body_submitter,
                 );
      }
      foreach my $item (@child_attachments) {
        $i++;
        #     print STDERR "$i-$item\n";
        ($attch_name, $xattach, $url)=$item->get_upload_filename_icon_and_url();
        if ($attch_name) {
                  $filename_msg .= "  o  $attch_name\n";

          # TODO: NICO - delete this and use the MIME::Types class. Example is in UploadsManager
          my $attach_mime_type = $self->get_mime_type($attch_name);
          $attach_path = $PARAMS->{absolutepathfileuploaddir} . "/" . $item->get_id() . "/$attch_name";
          #xxxxxxxxxxxxxxx
          #     print STDERR "attaching $attch_name, $attach_path\n, mime type=$attach_mime_type\n\n\n";
          $emsg->attach (
                         Type     => $attach_mime_type,
                         Path     => $attach_path,
                         Filename => $attch_name,
                         Encoding => 'base64'
                        );
        }
      }                         ### end for each attachment

      ######################################
      # Send the message to the recipient
      ######################################
      my $mail_host = $PARAMS->{mailserveraddress};
      my ($retcode, $result);
      eval{
        ($retcode,$result) = $emsg->smtpsend(
                        Host      => $mail_host,
                        MailFrom  => $email_from
                       );
      };

      if($@){
        $DISPLAY->debug_msg (2, "Error in eval ==> $@ ### going to use onfailure. retcode='".($retcode||'')."', result='".($result||'')."'\n");
      }

      # print STDERR "Send email to submitter with $i attachments\n";
      $DISPLAY->debug_msg(2,"Sent email to submitter with $i attachments");
    }

    ##########################################
    # Finishing email to the FormMaker manager
    ##########################################
    # Let's add what we sent to the submitter.....
    my $noun = 'was';
    if( $number_of_attachments > 1 ){
        $noun = 'were';
    }
    $msg .= "\n\n###########################\n\n";
    $msg .= "The following message has been sent to the form submitter(s): @recipient_list\n\n--\n";
    $msg .= "Subject: $subj_submitter\n\n\n $body_submitter\n--\n\n\n";
    $msg .= "Attachments: There $noun $number_of_attachments attachment(s) sent along with that email (the email sent to the submitter, not the one you just read!). ";
    if ($number_of_attachments > 0) {
      $msg .= "Here are the filename(s):\n$filename_msg\n\n";
    }
  }

  $msg .= $footer_text;
  #print STDERR "send email to manager\n";
  ##########################################
  # SENDING EMAIL
  ##########################################
  my $sm = Sendmail->new($email_to, $cc, "", $subj, $msg);
  $sm->set_from($email_from);

  #########################################
  ### Check to see if submitter should be set as
  ### sender and reply-to address
  #########################################
  #print STDERR "submitter_email_field[".$self->get_submitter_email_field."]\n\n";
  $DISPLAY->debug_msg(5,"submitter_email_field[".$self->get_submitter_email_field."]");
  if( $self->get_submitter_email_field() == 1 ){
      #print STDERR "will attempt to set submitter as sender of form\n";
      foreach my $recipient(@recipient_list){
         #print STDERR "verifying[$recipient]\n\n";
         # Verify email address of recipient
         my ($valid_email,$err_message) = $self->Form::Verify::email($recipient);
         unless( $valid_email ){
             #print STDERR "recipient[$recipient] is not a valid email address\n";
             next;
         }
         #print STDERR "will set[$recipient] as submitter and reply-to address\n";

         #only one address can be set as "from" and "reply-to"
         $sm->set_from($recipient);
         $sm->set_reply_to($recipient);
         last;
      }
  }

  if ($format == FORMAT_HTML) {
    # NZ This method was removed. No longer needed?
    #$sm->set_encoding('html');
  }

  if ($sm->sendMail()) {
    ### if there is a landing URL we redirect to it now and
    ### skip the thank you msg.
    if (my $url = $self->get_landing_url()) {
      $self->md_redirect($url);
    } else {
      my $content = $self->get_success_page(); # show success page
    }
  } else {
    $DISPLAY->print_click_back("Error: An error occured while trying to send an email notification.<p />Please contact the system administrator of this portal.");
  }
}




#
#
###############################
sub invalid_email {
  my $self = shift;
  my $email = shift;
  my $url = "index.pl?iid=" . $self->get_id() . "&amp;isa=FormMaker&amp;op=show";

  $DISPLAY->print_click_back("Error: '$email' does not appear to be a valid email address!");

}

#
#
###############################
sub www_download_file {
  my $self = shift;
  my $id = $self->get_id() || $FORM{iid};
  my ($fname) = DBAccess->sqlSelect("file1","instance","iid=$id");
  my $location = $PARAMS->{absolutepathfileuploaddir} . "/$id/$fname";
  my $file_type = Metadot::UploadsManager::UPLOAD_TYPE_USER();

  my %args = (
              file_name     => $fname,
              file_location => $location,
              file_type     => $file_type
             );
  #print STDERR "will download file[$fname] from[$location]\n";
  return $self->transmit_file( %args );

}

# TODO: NICO - delete this and use the MIME::Types class. Example is in UploadsManager
sub get_mime_type {
  my $self = shift;
  my $attch_file = shift;
  $attch_file = lc($attch_file);
  my ($junk,$file_type) = split('.', lc($attch_file));

  my $mtype = '';

  if ( $attch_file =~ /jpg$/ || $attch_file =~ /jpeg$/ ) {
    $mtype = 'image/jpeg';
  } elsif ( $attch_file =~ /gif$/ ) {
    $mtype = 'image/gif';
  } elsif ( $attch_file =~ /zip$/ ) {
    $mtype = 'application/zip';
  } elsif ( $attch_file =~ /pdf$/ ) {
    $mtype = 'application/pdf';
  } elsif ( $attch_file =~ /doc$/ ) {
    $mtype = 'application/msword';
  } else {
    # attempt to guess from the filename
    $mtype = 'AUTO';
  }

  return $mtype;
}

#############################
#
# Tell whether the submitter of the form should get an automated email
#
#############################
sub send_email_to_submitter {
  my $self = shift;

  return $self->get_i4();
}


#
# Returns a formatted version of the value
#
################################
sub _format_value {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);

    my $value = $args->{value};
    my $format = $args->{'format'};

    my $result;

    if (defined $value) {
        # split arrays according to the format
        if (UNIVERSAL::isa($value, 'ARRAY')) {
            if ($format == FORMAT_HTML) {
              $result = join("<br />", @$value);
            } elsif ($format == FORMAT_CSV) {
              $result = join("\n", @$value);
            } elsif ($format == FORMAT_DB) {
              $result = join("; ", @$value);
            } else {
              $result = join("\n", @$value);
            }
        } else {
            $result = $value;
        }
    }

    return $result;
}


#
# DEPRECATED
# Gets the formatted value from the FORM hash
#
###############################
sub _format_from_FORM {
  my ($self, $field_name, $format) = @_;
  my $fvalue = $FORM{$field_name};

  return $self->_format_value(
      value     => $fvalue,
      'format'  => $format,
  );
}



#
#
###############################
sub www_create_child {
  my $self = shift;
  my $child_type = shift;

  $self->debug_msg(3, "Gizmo::www_create_child()");

  my $child_isa = $FORM{child_type} || $child_type;

  my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
  # we re-instanciate the object because we want a brand new one
  my $child = $child_isa->new({
                               uid => $USER->uid(),
                               parent_id => $parentid,
                              });

  # this method should ALWAYS be overridden in derived classes.
  $child->process_edit_form();
}



sub www_add_child {
  my $self = shift;
  my $errormsg = shift;
  my $child_type = shift;
  $self->debug_msg(3, $self->{is_a}."::www_add_child()");

  my $child_isa = $FORM{child_type} || $child_type;
  my $gizmo_name = $child_isa->get_gizmo_name();

  my $parentid = $self->defined_or_exit($self->get_iid(), "parent id is missing.");
  # we re-instanciate the object because we want a brand new one
  # unless there is an error message (we want to keep its data)
  #my $child = $child_isa->new({ parent_id => $parentid }) unless ($errormsg);
  my $child = $child_isa->new({ parent_id => $parentid });
  $errormsg = $errormsg ?
    "<p /><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p />\n" :
      "";
  my $form = $child->show_edit_form("Adding new $gizmo_name");

  # Force the upload to be access controlled
  $FORM{'force_access_controlled'} = 1;

  if( $FORM{add_attachment} ){
      $form = $child->show_edit_form("Adding new email attachment");
  }
  my $content =
    Components->color_palette_code().
      "
    <form name=fm action='index.pl' method='post' ENCTYPE=\"multipart/form-data\">
    <table align='right'><tr><td><input type='submit' value='Create'></td></tr></table>
    <input type='hidden' name='isa' value='".$self->is_a()."'>\n
    <input type='hidden' name='iid' value='$parentid'>\n
    <input type='hidden' name='op' value='create_child'>\n
    <input type='hidden' name='child_type' value='$child_isa'>\n
    $errormsg
    $form
    <table align='right'><tr><td><input type='submit' value='Create'></td></tr></table>
    <br />&nbsp;
    </form>
    ";

  $child->print_form($content, "Adding new $gizmo_name");
}


###############################
# delete: delete the FormMaker, fields and attachments
###############################
sub delete {
  my $self = shift;

  my $id = $self->{id};
  $self->debug_msg(3, __PACKAGE__.": $self delete; id=$id");

  if (!$id) {
    $self->debug_msg(3, "Error Gizmo::delete(id=$id): I don't know what id to delete");
    return 0;
  }

  ########### Delete child fields
  my @childs_list = @{$self->build_gizmo_list(type=>'FormMakerElement', decorator=>'HTML::Decorator::NullDecorator')};
  foreach my $gizmo (@childs_list) {
    $self->debug_msg(3, "$self deleting child $gizmo");
    $gizmo->delete();
  }

  #### CHECK TO SEE IF THERE ARE ATTACHMENTS FOR THIS FORM AND DELETE THEM
  @childs_list = @{$self->build_gizmo_list(type=>'Item', decorator=>'HTML::Decorator::NullDecorator')};
  foreach my $gizmo (@childs_list) {
    $self->debug_msg(3, "$self deleting child $gizmo");
    $gizmo->delete();
  }

  return $self->SUPER::delete();
}

#
#
###########################
sub get_template_field_explanation {
  return <<EOM
        You can include form fields by enclosing the corresponding field name
        in double brackets, such as: &lt;&lt;requester_name&gt;&gt;
EOM
    ;
}


#########################
# process_template - expands <<field_name>>
#########################
sub process_template {
  my $self = shift;
  my $text = shift;
  my $values_hash = shift;

  my @tags = ($text =~ m/<<\s*.*?\s*>>/g);
  foreach my $field_tag (@tags) {
    my $field_name;
    my $replace_text;

    if ($field_tag =~ m/<<\s*(.*?)\s*>>/) {
      $field_name = $1;
      $replace_text = $self->_format_value( value => $values_hash->{$field_name}, 'format' => FORMAT_TEMPLATE);
    } else {
      $replace_text = '';
    }
    $text =~ s/\Q$field_tag\E/$replace_text/g;
    $self->debug_msg(5, "Replaced tag '$field_tag', field '$field_name' with '$replace_text'");
  }

  return $text;
}


1;

__END__

