###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package FieldElement;
use strict;
use Data::Dumper;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $DISPLAY);
use Metadot::AccessBroker::FormMakerEltBroker;

use vars qw(@ISA);
@ISA=qw(GizmoBuilder);
my $version = "4.5";

my $ACCESS_BROKER = Metadot::AccessBroker::FormMakerEltBroker->new();

sub _get_access_broker {
    my $self = shift;
    return $ACCESS_BROKER;
}

sub get_version {
    return $version;
}


####################################
sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

  $self->_form_initialize();
  $self->set_field_info (
    "description", "<b>Element Value</b>", 1,
        "name", "<b>Element Prompt Text</b>", 0,
        "t1", "<b>Selected</b>", 0,
    );

    $self->set_field_type('t1', 'checkbox');
}


sub is_allowed_to_do {
   my $self = shift;
   my $op = shift || die "must provide operation name";
   my $user = shift || $USER;

   my $ab = $self->_get_access_broker();
   return $ab->is_allowed_to_do(op => $op, user => $user, form_element => $self);
}

sub get_parent_form {
    my $self = shift;
    # get parent formmaker element:
    my $fm_elt = $self->get_parent();
    if ($fm_elt) {
    return $fm_elt->get_parent();
    }
    else {
    return undef;
    }
}

#override one of the default form types.
sub _form_initialize {
    my $self = shift;
    $self->{fields}->{description}->{formtype} = "text";
        $self->{fields}->{t1}->{formtype} = "checkbox";

        my $ts = "
        Enter option value, prompt text, and whether the option is selected.";
#     $ts .= "<br />field type: " . $self->{field_type};
    $self->set_intro_text($ts);

}



sub get_gizmo_name {
   my $self = shift;
   return "FieldElement";
}


## will replace the html produced by GizmoBuilder for given instance variable
sub _get_alt_text {
      my $self = shift;
      my $field = shift;
      my $id = $self->id();

      return unless defined $field;

      my $value = $FORM{$field} || $self->{$field};
      my $info = "<input type=text size=\"20\" maxlength=\"30\" name=\"$field\" value=\"$value\">";
      return $info;
}


sub get_type {
   my $self = shift;
   return $self->{keywords};
}

sub get_selected {
   my $self = shift;
   return $self->{t1};
}


# non standard edit panel (w/out key, cut and shortcut buttons)
sub show_panel{
  my $self = shift;
  my $id   = shift || $self->id();
  my $isa  = shift || $self->is_a();
  $self->debug_msg(5, "FieldElement::show_panel(id=$id,isa=$isa)");

  return '' unless($SESSION->admin_mode());

  my $str = "";
  $str .= $self->show_edit_button();
  $str .= $self->show_del_button();

  return $str;
}


# switch area is not supported for this gizmo
sub show_switch_area_panel {
  return '';
}


sub show_edit_form {
    my $self  = shift;
    my $title = shift;
    $self->debug_msg(4, "FieldElement::show_edit_form($title)");
        my $id   = $self->id;
#     print STDERR "in FieldElement::show_edit_form: ", Dumper($self), "\n";
    my $field_text  = $self->get_name() || $FORM{name};
    my $field_name  = $self->get_description()  || $FORM{description};
        my $selected  = $self->get_selected();
        $selected = "checked" if ($selected);

        my $output = "\n<h1>$title</h1>";
        $output .=  "
        <p />Enter option value, display text, and whether the option is selected.<br />
        <p /><b>Option Value</b> (*)<br />
      <input type='text' name='description' size='55' maxlength='255' value='$field_name'><br />
      <p /><b>Option Display Text</b><br />
      <input type='text' name='name' size='55' maxlength='255' value='$field_text'><br />
      <p /><b>Selected</b><br />
        <input type='checkbox' name='t1' value='1' $selected>
      <br />
        ";

    return $output;
}


## called before saving or updating
sub process_edit_form {
    my $self  = shift;
    my $error = shift || undef;

    ## check for required fields
    foreach my $field (keys (%{$self->{fields}})) {

    # upload fields do their own "is_required" checking
    next if ( $self->{fields}{$field}{formtype} =~ /_upload$/ );

        if ($self->{fields}->{$field}->{required} &&
            (!$FORM{$field} || !($FORM{$field} =~ /\S/)) # field is not just whitespace
            ) {
            $error .= "<br />".$self->{fields}->{$field}->{label} .
                " is a required field.???\n";
        }
    }

    my $iid = $self->get_iid();

    if (defined($error) && $iid){ ## if error and updating

        $self->www_modify($error);
    }
    elsif (defined($error)) {   ## if error and saving new
        my $parentid = $self->parent_id();
        my $parent = Gizmo->restore($parentid);
#          print STDERR "in gizmo::process_edit_form, calling parent add child on error\n";
        $parent->www_add_child($error, $self->is_a);
    }
    else {
                ## no errors
        my $checked_reqs = 1;
        $self->www_save($checked_reqs);

    }
}



# this method handles what to do when you are finished successfully creating a new gizmo
sub _show_create_success {
    my $self = shift;
  my $parentid = $self->parent_id();
    my $url = "index.pl?id=$parentid&isa=FormMakerElement&op=modify";
    $self->md_redirect($url);

}

sub _show_edit_success {
    my $self = shift;
  my $parentid = $self->parent_id();
    my $url = "index.pl?id=$parentid&isa=FormMakerElement&op=modify";
    $self->md_redirect($url);
}


##########################################
### show
### return one newsitem (page format)
##########################################
sub show {
    my $self        = shift;
    my $iid         = $self->get_iid();
    my $field_text  = $self->get_name();
    my $field_name  = $self->get_description();
        my $field_type  = $self->get_type();

    my $buttons     = $self->get_buttons();
    my $show_border = $self->get_c5();

        my ($value1, $value2, $selected) = $self->show_option();
    #just do inline alignment for the 'bullet' view.
    my $default_bullet_align = "top"; # default alignment for bullet list is 'top'.
                      # The actual set alignment is used in
                      # the more detailed view.

    my $html .= "$buttons  ";
    $html .= "<b>$value1 $value2 <font color='red'>$selected</font></b><br />\n";
    return $html;
}

#
##########################################
### show_in_list
### return one newsitem (list format)
##########################################
sub show_in_list {
    my $self        = shift;
    my $iid         = $self->get_iid();
    my $field_text  = $self->get_name();
    my $field_name  = $self->get_description();
        my $field_type  = $self->get_type();

    my $buttons     = $self->get_buttons();
    my $show_border = $self->get_c5();

        my ($value1, $value2, $selected) = $self->show_option();
    #just do inline alignment for the 'bullet' view.
    my $default_bullet_align = "top"; # default alignment for bullet list is 'top'.
        # The actual set alignment is used in
        # the more detailed view.

    my $html .= "$buttons  ";
    $html .= "<b>$value1 $value2 <font color='red'>$selected</font></b><br />\n";
    return $html;
}


sub show_summary {
    my $self        = shift;
    my $iid         = $self->get_iid();
    my $field_text  = $self->get_name();
    my $field_name  = $self->get_description();
        my $field_type  = $self->get_type();

    my $buttons     = $self->get_buttons();
    my $show_border = $self->get_c5();

        my ($value1, $value2, $selected) = $self->show_option();
    #just do inline alignment for the 'bullet' view.
    my $default_bullet_align = "top"; # default alignment for bullet list is 'top'.
                      # The actual set alignment is used in
                      # the more detailed view.

    my $html .= "$buttons  ";
    $html .= "<b>$value1 $value2 <font color='red'>$selected</font></b><br />\n";
    return $html;
}


sub show_option {
    my $self        = shift;
    my $iid         = $self->get_iid();
    my $field_text  = $self->get_name();
    my $field_name  = $self->get_description();

    # CKIM: allow existing form value to set selected
    my $form_field_name = $self->get_parent()->get_name();
    my $selected    = ($FORM{$form_field_name} eq $field_name) || $self->get_selected();
    my $str;
    my $parentid = $self->parent_id();

    my $parent = Gizmo->restore($parentid);
        my $field_type  = $parent->get_type();

        if ($field_type eq 'SelectionList') {
             if ($selected) {$selected = "selected";} else { $selected = ""; }
             return ($field_name,$field_text,$selected);

        }   elsif ($field_type eq 'Radio') {
             if ($selected) {$selected = "checked";} else { $selected = ""; }
             return ($field_name,$field_text,$selected);

        }   else {

        }
}



## called for saving and updating
sub www_save {
    my $self  = shift;
    my $checked_reqs = shift; #optional
    my $order = 0;
    my $uid = $USER->uid();
    my $error = undef;

#    print STDERR "AT WWW_SAVE $FORM{show_from}\n";

    foreach my $field (keys (%{$self->{fields}})) {

        if ( $self->{fields}{$field}{formtype} =~ /_upload$/ ||
         $self->{fields}{$field}{formtype} =~ /text$/ ) {

#       print STDERR "APPLYING ARRAY FIX TO FIELD $field\n";

        # added here for the mess IE 5.0 does when submitted form
        # contains special characters
        if( ref( $FORM{$field} ) eq 'ARRAY' ) {
        $FORM{$field} = $FORM{$field}->[0];
        }
    }
    }

    # XXX remove?
    if(ord substr($FORM{file1},0,1) < 32) {
        $FORM{file1} = "" ;
    }

    if (!$checked_reqs) {
        ## check for required fields
        foreach my $field (keys (%{$self->{fields}})) {

        # upload fields do their own "is_required" checking
        next if ($self->{fields}{$field}{formtype} =~ /_upload$/ );

            if ($self->{fields}->{$field}->{required} &&
                (!$FORM{$field} || !($FORM{$field} =~ /\S/)) # field is not just whitespace
           ) {
                $error .= "<br />".$self->{fields}->{$field}->{label} .
          " is a required field.\n";
            }
        }
    }


        if ( $FORM{t1} ) {               ## reset 'selected' flag for all options if selected defined
             my $parentid = $self->parent_id();
             my $parent = Gizmo->restore($parentid);
             my @childs_list = @{$parent->build_gizmo_list(type=>'FieldElement', decorator=>'HTML::Decorator::NullDecorator')};

             foreach my $gizmo (@childs_list) {
                        my $selected  = $gizmo->get_selected();
                        if ($selected) { $gizmo->{t1} = ''; $gizmo->save(); }
             }
        }

    my $iid = $self->get_iid();

    if (defined($error) && $iid){ ## if error and updating

        $self->www_modify($error);

        exit;

    } elsif (defined($error)) {   ## if error and saving new

        my $parentid = $self->parent_id();
        my $parent = Gizmo->restore($parentid);
        $parent->www_add_child($error, $self->is_a);
        exit;
    }

    foreach my $field (keys (%{$self->{fields}})) {
        $order = $self->{fields}->{$field}->{order};
        next if !defined($order) || $order <= 0;
        if ('mediumtext' eq $self->{fields}->{$field}->{formtype} || 'longtext' eq $self->{fields}->{$field}->{formtype}) {

            my $text = $FORM{$field};
            if ($text && ('text' eq $FORM{"type$field"})) {
                $text = escapeHTML($text);
                $text = $self->translate_white_space($text);
                $text = "<!-- form type is text -->" . $text;
                $FORM{$field} = $text;
            }
        }

    if ( $self->{fields}{$field}{formtype} eq 'show_from' ) {
        if( $FORM{$field} ) { $FORM{$field} = $FORM{$field} . ' 00:00:01' }
    }

    # we make an exception with file1 because we won't save it unless the
    # upload is successful
    unless($field eq 'file1') {
        $self->{$field} = $FORM{$field};
    }
    }

    if (!$iid){ ## we are creating new

        $iid = $self->add();

        if ($iid) {
            my $isa = $self->is_a();
            my $parentid = $self->parent_id();

            $self->_show_create_success();
        }
    } else {            ## we are updating

    $self->save();
    $self->_show_edit_success();
        #$self->md_redirect("index.pl?iid=".$self->parent_id);
    }

}



1;

__END__

=head1 NAME


=head1 SYNOPSIS


=head1 DESCRIPTION


=head2 Methods


=cut


