###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package FormMakerElement;
use strict;


use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $CACHE $PARAMS $DISPLAY $CGI);
use Metadot::AccessBroker::FormMakerEltBroker;

use vars qw(@ISA);
@ISA=qw(GizmoBuilder);
my $version = "4.5";

my $ACCESS_BROKER = Metadot::AccessBroker::FormMakerEltBroker->new();

sub _get_access_broker {
    my $self = shift;
    return $ACCESS_BROKER;
}

sub get_version {
    return $version;
}


my $AC =
  {
   BUNDLES => {
           MOD => { DESC => 'Edit',
            #CKIM-SHORTCUTS: added a new op 'create_shortcut'
            OPS => [qw(add_child create_child modify save up down paste create_shortcut)],
            DEFAULT_LEVEL => 8,
            MIN_LEVEL => 2,
              },
           DEL => { DESC => 'Delete and Cut',
            OPS => [qw(delete delete_ok cut)],
            DEFAULT_LEVEL => 8,
            MIN_LEVEL => 8,
            PARENT_DEP => {cut => 'MOD'},
              },
           DISP => { DESC => 'View',
             OPS => [qw(show)],
             DEFAULT_LEVEL => 0,
             MIN_LEVEL => 0,
             DEFAULTS_TO => 'on',
               },
           EDITP => { DESC => 'Change<br />Permissions',
              OPS => [qw(edit_permissions set_permissions change_owner)],
              DEFAULT_LEVEL => 8,
              MIN_LEVEL => 8,
            },
          },
   DISPLAY_ORDER => [qw(DISP MOD DEL EDITP)],
  };


###
### ACCESS CONTROL METHODS
###
sub get_AC {
  my $self = shift;
  my $class = ref $self || $self;

  die "&get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

  return $AC;
}





####################################

sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

    $self->_form_initialize();
    $self->set_field_info (
      "name", "<b>Field Name</b>", 1,
                  "description", "<b>Field Prompt Text</b>", 0,
                  "keywords", "<b>Field Type</b>", 0,
                  "i1", "<b>Size</b>", 0,
                  "i2", "<b>Rows</b>", 0,
                  "t1", "<b>Title Text</b>", 0,
                  "t2", "<b>Value</b>", 0,
                  "i3", "<b>Checked by Default</b>", 0,
                  "i4", "<b>Multiple</b>", 0,
                  "i5", "<b>Email to submitter</b>", 0,
                  "file1","<b>FormMaker attachment</b>", 0,
                  "c1", "<b>Mandatory field</b>", 0,

      );

    $self->set_field_type('keywords', 'select');

    $self->{display_wrapped} = { show => 1,
                               };
}

sub is_allowed_to_do {
   my $self = shift;
   my $op = shift || die "must provide operation name";
   my $user = shift || $USER;

   my $ab = $self->_get_access_broker();
   return $ab->is_allowed_to_do(op => $op, user => $user, form_element => $self);
}

sub get_parent_form {
    my $self = shift;
    return $self->get_parent();
}

#override one of the default form types.
sub _form_initialize {
    my $self = shift;
        $self->{fields}->{keywords}->{formtype} = "select";
        $self->{fields}->{keywords}->{selectsize} = "1";
    $self->{fields}->{keywords}->{selectoptions} = "Text,Password,Email,TextArea,SelectionList,Radio,Checkbox,TextBlock,File,UserEmail,UserID";
    $self->{fields}->{description}->{formtype} = "text";

        my $ts = "
        Instructions text
";
    $self->set_intro_text($ts);

}

## will replace the html produced by GizmoBuilder for given instance variable
sub _get_alt_text {
      my $self = shift;
      my $field = shift;
      my $id = $self->id();

      return unless defined $field;

      my $value = $FORM{$field} || $self->{$field};
      my $info = "<input type=text size=\"20\" maxlength=\"30\" name=\"$field\" value=\"$value\" />";
      return $info;
}

sub get_gizmo_name {
   my $self = shift;
   return "FormMakerElement";
}


sub get_type {
   my $self = shift;
   return $self->{keywords};
}

sub get_size {
   my $self = shift;
   return $self->{i1};
}

sub get_rows {
   my $self = shift;
   return $self->{i2};
}

sub get_title {
   my $self = shift;
   return $self->{t1};
}

sub get_checked {
   my $self = shift;
   return $self->{i3};
}

sub get_value {
   my $self = shift;
   return $self->{t2};
}

sub get_multiple {
   my $self = shift;
   return $self->{i4};
}

sub get_email_to_submitter {
   my $self = shift;
   return $self->{i5};
}


# is the form field mandatory
sub is_mandatory
{
  my $self = shift;
  return $self->c1(@_);
}


# returns the form value. Some special cases exist for UserEmail and UserID
sub get_form_value {
    my $self = shift;

    my $type = $self->get_type();
    my $result;

    $DISPLAY->debug_msg(4, "get_form_value begins, my type is '$type'");

    if ($type eq 'UserID') {
        $result = $USER->uid();
    } elsif ($type eq 'UserEmail') {
        $result = $USER->email();
    } else {
        $result = $FORM{$self->get_name()};
    }

    $DISPLAY->debug_msg(4, "get_form_value finishes with result '$result'");
    return $result;
}


# non standard edit panel (w/out key, cut and shortcut buttons)
sub show_panel{
  my $self = shift;
  my $id   = shift || $self->id();
  my $isa  = shift || $self->is_a();
  $self->debug_msg(5, "FormMakerElementshow_panel(id=$id,isa=$isa)");

  return '' unless($SESSION->admin_mode());

  my $str = "";
  $str .= $self->show_edit_button();
  $str .= $self->show_del_button();

  return $str;
}


# switch area is not supported for this gizmo
sub show_switch_area_panel {
  return '';
}


sub get_options {
    my $self = shift;
    my $type = $self->get_type();
    my $size = $self->get_size();
    my $rows = $self->get_rows();
    my $title = $self->get_title();
    my $value = $self->get_value();
    my $checked = $self->get_checked();
    my $multiple = $self->get_multiple();

    my $str ;

    if ($type eq 'SelectionList') {
        if (!$size) {
            $size = 5;
        }
        $multiple = 'checked' if ($multiple) ;
        my $childs_list = $self->show_childs_list( type=>'FieldElement', decorator=>'HTML::Decorator::NullDecorator');
        $str .= "<table border='0'><tr><td>Size</td><td><input type='text' name='i1' size='3' value='$size' /></td></tr>";
        $str .= "<tr><td>Multiple</td><td><input type='checkbox' name='i4' value='1' $multiple /></td></tr></table><br />";
        $str .= "<b>List Elements</b><br />".$childs_list."<br /><input type='submit' name='new_option'  value='Add List Element' />";

    } elsif (($type eq 'Text') || ($type eq 'Password')) {
        if (!$size) {
            $size = 30;
        }
        $str = "Size <input type='text' name='i1' size='3' value='$size' />";

    } elsif ($type eq 'Email') {
        my $state = $self->get_email_to_submitter();
        if( $state ){
            $state = "checked";
        } else{
            $state = '';
        }
        $str = "Email to submitter: <input type='checkbox' name='i5' value='1' $state /> yes";

    } elsif ($type eq 'TextArea') {
        if (!$size) {
            $size = 30;
        }
        if (!$rows) {
            $rows = 5;
        }
        $str = "<table border=\"0\"><tr><td>Columns</td><td><input type='text' name='i1' size='3' value='$size' /></td></tr>";
        $str .="<tr><td>Rows</td><td><input type='text' name='i2' size='3' value='$rows' /></td></tr></table>";

    } elsif ($type eq 'Radio') {
        my ($multi_line, $single_line);
        if ($size==1) { $multi_line='checked'; } else { $single_line='checked'; }
        my $childs_list = $self->show_childs_list( type=>'FieldElement', decorator=>'HTML::Decorator::NullDecorator');
        $str .= "<input type='radio' name='i1' value='0' $single_line />display in single line  ";
        $str .= "<input type='radio' name='i1' value='1' $multi_line />display multi-line<br /><br />";
        $str .= "<b>Options List</b><br />";
        $str .= $childs_list . "<br /><input type='submit' name='new_option'  value='Add New Option' />";

    } elsif ($type eq 'Checkbox') {
        $checked = 'checked' if ($checked) ;
        $str = "Checked by Default <input type='checkbox' name='i3' value='1' $checked />";

    } elsif ($type eq 'TextBlock') {
        my $fchecked = '';
        $fchecked = 'checked="checked"' if ($multiple) ;
        $str = "Bigger Font <input type='checkbox' name='i4' value='1' $fchecked /><br />";
    } else {
        # return an empty
    }

    return $str;

}



sub show_edit_form {
    my $self  = shift;
    my $title = shift;

    $self->debug_msg(4, $self->{is_a}."::show_edit_form($title)");

    my $field_text      = $self->get_description()  || $FORM{description};
    my $field_name      = $self->get_name() || $FORM{name};

    my $field_options   = $self->get_options();
    my $field_type      = $self->get_type();
    my $field_type_html;

    if (!$field_type) {
         $field_type_html = "
         <select name='keywords'>
         <option>Text</option>
         <option>Password</option>
         <option>Email</option>
         <option>TextArea</option>
         <option>SelectionList</option>
         <option>Radio</option>
         <option>Checkbox</option>
         <option>TextBlock</option>
         <option>File</option>
         <option>UserEmail</option>
         <option>UserID</option>
         </select>
         <br /><font size=\"-1\"><i>The 'TextBlock' option generates a text label.</i></font>
         ";
    } else {
        $field_type_html = $field_type;
        # display options if they exist
        if ($field_options) {
            $field_type_html .= "<br /><p /><b> Field Properties</b><br />$field_options<br />";
        }
        $field_type_html .= "<input type='hidden' name='keywords'  value='$field_type' />";
    }


    my $field_mandatory_html = qq~<p><b>Mandatory field</b> <input type="checkbox" name="c1" value="1"~;
    if ($self->is_mandatory())
    {
      $field_mandatory_html .= qq~ checked="checked"~;
    }
    $field_mandatory_html .= qq~/>~;


        my $output = "\n<h1>$title</h1>";
        $output .=  "
        <p /><b>Field Name</b> (*)<br />
      <input type='text' name='name' size='55' maxlength='255' value=\"$field_name\" /><br /><font size=\"-1\"><i>The form field name should contain only alpha-numeric characters, no spaces or special characters.</i></font><br />
      <p /><b>Field Prompt Text</b><br />
      <textarea name='description' cols=40 rows=5>$field_text</textarea><br />
      $field_mandatory_html
      <p /><b>Field Type</b><br />
        $field_type_html
        ";

    return $output;
}



## called before saving or updating
sub process_edit_form {
    my $self  = shift;
    my $error = shift || undef;
        my $gizmo;

    ## check for required fields
    foreach my $field (keys (%{$self->{fields}})) {

        # upload fields do their own "is_required" checking
        next if ( $self->{fields}{$field}{formtype} =~ /_upload$/ );

        if ($self->{fields}->{$field}->{required} &&
            (!$FORM{$field} || !($FORM{$field} =~ /\S/)) # field is not just whitespace
            ) {
            $error .= "<br />".$self->{fields}->{$field}->{label} .
                " is a required field!\n";
        }

    }


    my $dupes = ($self->check_field_name($FORM{name})) ;   ## check dups and fieldname
    if ($dupes)
    {
        $error .= $dupes;
    }

    my $iid = $self->get_iid();

    if (defined($error) && $iid){ ## if error and updating

        $self->www_modify($error);
    }
    elsif (defined($error)) {   ## if error and saving new
#          print STDERR "in gizmo::process_edit_form, calling parent add child on error\n";
        my $parentid = $self->parent_id();
        my $parent = Gizmo->restore($parentid);
        $parent->www_add_child($error, $self->is_a);
    }
    else {
        ## no errors
        my $checked_reqs = 1;
        $self->www_save($checked_reqs);

        if ($FORM{new_option}) {
            $FORM{name} = '';
            $FORM{description} = '';
            $FORM{child_type} = 'FormMakerElement';
            $self->www_add_child($error, 'FieldElement');
        }

    }
}



# this method handles what to do when you are finished successfully creating a new gizmo
sub _show_create_success {
    my $self = shift;

    # If the field has additional options, then modify the field. Otherwise,
    # we return to our parent
    if ($self->get_options()) {
        $self->www_modify();
    } else {
        $self->_redirect_to_parent();
    }
}

sub _show_edit_success {
    my $self = shift;
  if ($FORM{new_option}) {
         $FORM{name} = '';
         $FORM{description} = '';
         $self->www_add_child('', 'FieldElement');
    }
    else {
        $self->_redirect_to_parent();
    }

}


sub _redirect_to_parent {
    my $self = shift;
    my $parentid = $self->parent_id();
    my $url = "index.pl?id=$parentid&op=modify";
    $self->md_redirect($url);
}


sub check_field_name {
    my $self = shift;
    my $name = shift;
    my $error = undef;
    my $dups = 0;
    my @allowed_list;
    my $gizmo;

# check fieldname:
    if ($name =~ /[^a-zA-Z0-9_]/)  {
    $error = "<br />You enter a bad Field Name<br />Please use only alpha-numeric characters\n";
    }

# check duplicate fieldname:
    my $parentid = $self->parent_id();
    my $parent = Gizmo->restore($parentid);
    my @childs_list = @{$parent->build_gizmo_list(type=>'FormMakerElement', decorator=>'HTML::Decorator::NullDecorator')};
# first determine which ones I'm allowed to display
    foreach $gizmo (@childs_list) {
    if ($gizmo->is_allowed_to_do('show', $USER) && (($gizmo->get_id()) != ($self->get_id())) ) {    push @allowed_list, $gizmo; }
    }
# get parent field list -- don't allow the name to match one of these.
    my $parent_isa = $parent->is_a();
    my $parent_full = $parent_isa->new($parentid);
    my @flist = $parent_full->get_field_names();
    if (@flist && (Utility->member(lc $name, @flist))) {
    $error .= "<br />The field name '$name' you entered is reserved.<br />Please choose another field name\n";
    }
    foreach $gizmo (@allowed_list) {
    my $field_name  = $gizmo->get_name();
    if ($field_name eq $name) { $dups++; }
    }

    if ( $dups )  {
    $error .= "<br />The field name '$name' you entered is already used in the form.<br />Please choose another field name\n";
    }

    return $error;


}


##########################################
### show
### return one newsitem (page format)
##########################################
sub show {
  my $self = shift;
  $self->debug_msg(1, "FormMakerElement::show()");

  return 'FormMakerElement does not support op \'show\'';
}


sub show_summary {
    my $self        = shift;

    $DISPLAY->debug_msg(3, 'show_summary begins');

    my $field = $self->show_field();
    my $html;

    if ($field) {
        my $buttons     = $self->get_buttons();
        $html = "<tr><td valign='top' nowrap='nowrap'>$buttons</td> ";
        $html .= "$field</tr>\n";
        $DISPLAY->debug_msg(5, "show_summary got field value <xmp>$field</xmp>");
    } else {
        $DISPLAY->debug_msg(4, 'show_summary got empty field value');
        $html = '';
    }

    $DISPLAY->debug_msg(5, "show_summary will return html <xmp>$html</xmp>");
    return $html;
}


sub show_field {
    my $self        = shift;
    my $iid         = $self->get_iid();
    my $field_text  = $self->get_description();
    my $field_name  = $self->get_name();
    my $field_type  = $self->get_type();
    my $field_align = 'center';
    my $size = $self->get_size();
    my $rows = $self->get_rows();
    my $title = $self->get_title();

    # CKIM: allow the fields to get existing form input
    my $value = $FORM{$field_name} || $self->get_value();
    my $checked = ($FORM{$field_name}) || $self->get_checked();

    my $multiple = $self->get_multiple();
    my $email_to_submitter = $self->get_email_to_submitter();


    my $str;
    my ($value1, $value2, $selected, @childs_list, @allowed_list, $gizmo, $options);
    # aju -- the following was setting a ref to a hash. Fixed, but don't
    # know where this is used anyway.
    my $args = {type=>'FieldElement', decorator=>'HTML::Decorator::NullDecorator'};

    my $is_modify_mode = ($FORM{op} eq 'modify');

    # if we are in the modify screen, show the field_name as well
    if ($is_modify_mode && defined($field_name)) {
        $field_text .= "  &lt;&lt;$field_name&gt;&gt; ";
    }

    # if the field is mandatory, indicate the required field
    if ($self->is_mandatory())
    {
       $field_text .= $self->generate_mandatory_indicator();
    }

    if ($field_type eq 'SelectionList') {
        $multiple = "multiple" if ($multiple);
        @childs_list = @{$self->build_gizmo_list(type=>'FieldElement', decorator=>'HTML::Decorator::NullDecorator')};
        # first determine which ones I'm allowed to display
        foreach $gizmo (@childs_list) {
            if ($gizmo->is_allowed_to_do('show', $USER) ) { push @allowed_list, $gizmo; }
        }
        foreach my $gizmo (@allowed_list) {
            ($value1, $value2,$selected) = $gizmo->show_option();
            if (!$value2) {
            $value2 = $value1;
            }
            $options .= "<option value=\"$value1\" $selected>$value2</option>\n";
        }
        $str .= "<td class='smaller' valign='top' align='right' nowrap='nowrap'>$field_text</td><td class='smaller' valign='top'><select name='$field_name' size='$size' $multiple>\n$options</select></td>\n";

    }   elsif ($field_type eq 'TextArea'){
        $str = "<td class='smaller' valign='top' align='right' nowrap='nowrap'>$field_text</td><td class='smaller' valign='top'><textarea name='$field_name' rows='$rows' cols='$size'>".$CGI->escapeHTML($value)."</textarea></td>";
    }
    elsif ($field_type eq 'Email') {
       $field_type = 'Text';
       $str = "<td class='smaller' valign='top' align='right'>$field_text</td>
               <td class='smaller' valign='top'><input type='".lc($field_type)."'
               name='$field_name' size='30' value=\"".$CGI->escapeHTML($value)."\"/>";

       if( $is_modify_mode ){
           $str .= " Email to submitter";
       }
       $str .= "</td>";
    }
    elsif ($field_type eq 'Radio') {

        my $separator ='&nbsp;';
    #   if ($size==1) { $separator ='<br />'; } else { $separator ='&nbsp;'; }
        @childs_list = @{$self->build_gizmo_list(type=>'FieldElement', decorator=>'HTML::Decorator::NullDecorator')};
        # first determine which ones I'm allowed to display
        foreach $gizmo (@childs_list) {
            if ($gizmo->is_allowed_to_do('show', $USER) ) { push @allowed_list, $gizmo; }
        }
    #   print STDERR "radio size is: $size\n";
        if ($size == 0) {
            $options .= "<table><tr valign='top'>\n";
        }
        else {
            $options .= "<table border=\"0\" cellspacing=\"0\">\n";
        }
        foreach my $gizmo (@allowed_list) {
            ($value1, $value2, $selected) = $gizmo->show_option();
            if ($size == 0) {
            $options .= "<td><input type='".lc($field_type)."' name='$field_name' value=\"$value1\" $selected /></td><td class='smaller'>$value2</td> \n";
            }
            else {
            $options .= "<tr><td valign='baseline'><input type='".lc($field_type)."' name='$field_name' value=\"$value1\" $selected /></td><td valign='top' class='smaller'>$value2</td></tr>\n";
            }
        }
        if ($size == 0) {
            $options .= "</tr>";
        }
        $options .= "</table>\n";

        $str = "<td class='smaller' valign='top' align='right' nowrap='nowrap'>$field_text</td><td class='smaller' valign='top'>$options</td> ";

    } elsif ($field_type eq 'Checkbox'){
        $checked = 'checked="checked"' if ($checked) ;
        #$str = "<td class='smaller' valign='top'><input type='".lc($field_type)."' name='$field_name' value='1' $checked></td><td class='smaller' valign='top' nowrap='nowrap'>$field_text</td> ";
        $str = "<td class='smaller' valign='top' align='right' nowrap='nowrap'>$field_text</td><td class='smaller' valign='top'><input type='".lc($field_type)."' name='$field_name' value='1' $checked /></td> ";

    } elsif ($field_type eq 'TextBlock'){
        $multiple = "class='smaller'" if (!$multiple);
        $str = "<td $multiple valign='top' colspan='2'>$field_text</td> ";
    } elsif ($field_type eq 'File'){
        $str = "<td class='smaller' valign='top' align='right'>$field_text</td>
                <td class='smaller' valign='top'><input name='file_attachment' type='".lc($field_type)."' size='30'></td>";
    } elsif ($field_type eq 'UserID' || $field_type eq 'UserEmail') {
        # only display something if we're in modify mode
        my $display_text;
        if ($is_modify_mode) {
            $str = "<td class='smaller' valign='top' align='right'>$field_text</td><td class='smaller' valign='top'><i>(This $field_type field is not visible)</i></td>";
        } else {
            $str = '';
        }
    } else {
        $str = "<td class='smaller' valign='top' align='right'>$field_text</td><td class='smaller' valign='top'><input type='".lc($field_type)."' name='$field_name' size='$size' value=\"".$CGI->escapeHTML($value)."\"/></td> ";
    }

    return $str;
}


#########################################
sub show_childs_list {

    my $self = shift;
    my %args = @_;

    my $id = $self->{id};
    $self->debug_msg(3, "Category::show_childs_list(id=$id)");

            my $row_color_on = "#FFFFFF";
            my $row_color_off = "#EEEEEE";
            my $row_color = $row_color_on;

    #print STDERR "Category->show_childs_list\n",Dumper \%args;

    my $str = "";
    my $renderer = "show_summary";
    my @childs_list;

#   if( $args{type} && $args{$type}->can('self_render') and $args{$type}->self_render() ) {
#
#            my $gizmo = $type->new();
#            @childs_list = @{$self->build_gizmo_list(type=>$type)};
#
#            #XXX [claudio] terribly inefficient --must find better solution
#            unless( scalar( @childs_list ) == 0 ) {
#
#                $str .= $decorator->new( component=>$gizmo, background_color => $row_color )->$renderer($id);
#            }
#   } else {
        @childs_list = @{$self->build_gizmo_list(%args)};
#   }

    # first determine which ones I'm allowed to display
    my @allowed_list = ();
    foreach my $gizmo (@childs_list) {
        if ($gizmo->is_allowed_to_do('show', $USER) ) {
            push @allowed_list, $gizmo;
        }
    }

    # run the renderer for each gizmo
    foreach my $gizmo (@allowed_list) {

        if ($gizmo->is_a() eq 'Category' && $gizmo->get_image_url('display_image')) {
        $renderer = "show_as_roll_over";
        }

        my $decorator = 'HTML::Decorator::NullDecorator';

        $row_color = ($row_color eq $row_color_on) ? $row_color_off : $row_color_on;
        my $str_from_decorator = $decorator->new(component=>$gizmo, background_color=>$row_color)->$renderer($id);

        $str .= $str_from_decorator if (defined $str_from_decorator);
    }

    # if gizmos wish to display postscript text, allow them to do so afterwards
    foreach my $gizmo (@allowed_list) {
        $str .= $gizmo->show_postscript();
    }

    return $str;

}



####################################################
######                                        #######
######            FRONT-END METHODS           #######
######                                        #######
#####################################################
sub www_create_child {
  my $self = shift;
  my $child_type = shift;

  $self->debug_msg(3, "Gizmo::www_create_child()");

  my $child_isa = $FORM{child_type} || $child_type;

  my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
  # we re-instanciate the object because we want a brand new one
  my $child = $child_isa->new({
             uid => $USER->uid(),
             parent_id => $parentid,
            });

  # this method should ALWAYS be overridden in derived classes.
  $child->process_edit_form();
}



sub www_add_child {
    my $self = shift;
    my $errormsg = shift;
    my $child_type = shift;
    $self->debug_msg(3, $self->{is_a}."::www_add_child()");

    my $child_isa = $FORM{child_type} || $child_type;
    my $gizmo_name = $child_isa->get_gizmo_name();

    my $parentid = $self->defined_or_exit($self->get_iid(), "parent id is missing.");
    # we re-instanciate the object because we want a brand new one
    # unless there is an error message (we want to keep its data)
    #my $child = $child_isa->new({ parent_id => $parentid }) unless ($errormsg);
    my $child = $child_isa->new({ parent_id => $parentid });

    $errormsg = $errormsg ?
    "<p /><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p />\n" :
    "";

    my $form = $child->show_edit_form("Adding new $gizmo_name");
    my $content =
    Components->color_palette_code().
    "
    <form name=fm action='index.pl' method='post' ENCTYPE=\"multipart/form-data\">
    <table align='right'><tr><td><input type='submit' value='Create' /></td></tr></table>
    <input type='hidden' name='isa' value='".$self->is_a()."' />\n
    <input type='hidden' name='iid' value='$parentid' />\n
    <input type='hidden' name='op' value='create_child' />\n
    <input type='hidden' name='child_type' value='$child_isa' />\n
    $errormsg
    $form
    <table align='right'><tr><td><input type='submit' value='Create' /></td></tr></table>
    <br />&nbsp;
    </form>
    ";

    $child->print_form($content, "Adding new $gizmo_name");
}


##############
## called for saving and updating
sub www_save {
    my $self  = shift;
    my $checked_reqs = shift; #optional
    my $order = 0;
    my $uid = $USER->uid();
    my $error = undef;
    my $err = undef;


    foreach my $field (keys (%{$self->{fields}})) {

        if ( $self->{fields}{$field}{formtype} =~ /_upload$/ ||
             $self->{fields}{$field}{formtype} =~ /text$/ ) {

    #       print STDERR "APPLYING ARRAY FIX TO FIELD $field\n";

            # added here for the mess IE 5.0 does when submitted form
            # contains special characters
            if( ref( $FORM{$field} ) eq 'ARRAY' ) {
            $FORM{$field} = $FORM{$field}->[0];
            }
        }
    }

    # XXX remove?
    if(ord substr($FORM{file1},0,1) < 32) {
        $FORM{file1} = "" ;
    }

    if (!$checked_reqs) {
        ## check for required fields
        foreach my $field (keys (%{$self->{fields}})) {

        # upload fields do their own "is_required" checking
        next if ($self->{fields}{$field}{formtype} =~ /_upload$/ );

            if ($self->{fields}->{$field}->{required} &&
                (!$FORM{$field} || !($FORM{$field} =~ /\S/)) # field is not just whitespace
           ) {
                $error .= "<br />".$self->{fields}->{$field}->{label} .
          " is a required field.\n";
            }
        }
    }

    my $dupes = ($self->check_field_name($FORM{name})) ;   ## check dups and fieldname
    if ($dupes)
    {
        $error .= $dupes;
    }

    my $iid = $self->get_iid();

    if (defined($error) && $iid){ ## if error and updating

        $self->www_modify($error);
        exit;

    } elsif (defined($error)) {   ## if error and saving new

        my $parentid = $self->parent_id();
        my $parent = Gizmo->restore($parentid);
        $parent->www_add_child($error, $self->is_a);
        exit;
    }

    foreach my $field (keys (%{$self->{fields}})) {
        $order = $self->{fields}->{$field}->{order};

        next if !defined($order) || $order <= 0;
        if ('mediumtext' eq $self->{fields}->{$field}->{formtype} || 'longtext' eq $self->{fields}->{$field}->{formtype}) {

            my $text = $FORM{$field};
            if ('text' eq $FORM{"type$field"}) {
                $text = escapeHTML($text);
                $text = $self->translate_white_space($text);
                $text = "<!-- form type is text -->" . $text;
                $FORM{$field} = $text;
            }
        }

        if ( $self->{fields}{$field}{formtype} eq 'show_from' ) {
            if( $FORM{$field} ) { $FORM{$field} = $FORM{$field} . ' 00:00:01' }
        }

        # we make an exception with file1 because we won't save it unless the
        # upload is successful
        unless($field eq 'file1') {
            $self->{$field} = $FORM{$field};
        }
    }

    if (!$iid) { ## we are creating new

        $iid = $self->add();

        if ($iid) {
            my $isa = $self->is_a();
            my $parentid = $self->parent_id();

            ## handle situation when file upload is required

            if( $err ) {
                my $parentid = $self->parent_id();
                my $parent = Gizmo->restore($parentid);

                # backtrack object creation
                $self->delete();

                $parent->www_add_child( $err, $self->is_a );
                exit;
            }

            $self->_show_create_success();
        }
    } else {
        ## we are updating

        if( $err ) {

            $self->www_modify( $err );
            exit;
        }

        if($FORM{file1}){

            my $filename;

            $filename = $self->upload_file($FORM{file1}, $iid, $uid );

            if($filename eq 'USER_QUOTA_EXCEEDED' ||
               $filename eq 'GLOBAL_QUOTA_EXCEEDED' ) {

            $self->file1('');
            $self->save();
            $FORM{file1} = '';

            if( $filename eq 'USER_QUOTA_EXCEEDED' ) {

                $self->www_modify('File upload has exceeded allowed user quota, please request a larger quota to the site administrator or delete some of your files to free up space');
            } else {

                $self->www_modify('File upload has exceeded allowed site-wide quota, please request a larger quota to the site administrator or delete some of your files to free up space');
            }

            } elsif ($filename){

            $self->file1($filename);
            $self->save();
            }
        } else {

            $self->save();
        }

        $self->_show_edit_success();
        #$self->md_redirect("index.pl?iid=".$self->parent_id);
    }

}



##############

sub delete {
  my $self = shift;

  my $id = $self->{id};
  $self->debug_msg(3, __PACKAGE__.": $self delete; id=$id");

  if(!$id){
    $self->debug_msg(3, "Error Gizmo::delete(id=$id): I don't know what id to delete");
    return 0;
  }


  # Delete childs
  $self->debug_msg(3, __PACKAGE__.": $self about to delete childs");

  my @childs_list = @{$self->build_gizmo_list(type=>'FieldElement', decorator=>'HTML::Decorator::NullDecorator')};
  foreach my $gizmo (@childs_list) {
      $self->debug_msg(3, __PACKAGE__.": $self deleting child $gizmo");
      $gizmo->delete();
  }

  $self->debug_msg(3, __PACKAGE__.": $self finished deleting children. Now calling super.");
  return $self->SUPER::delete();
}




# generates the html to indicate a mandatory field
sub generate_mandatory_indicator
{
  return qq~<super><span style="color: red">*</span></super>~;
}




1;

__END__



