##############################################################################
# <LICENSEINFO>
# </LICENSEINFO>
##############################################################################
package Iframe;

use Metadot qw($USER $PARAMS);

use strict;
use vars qw (@ISA);
@ISA=qw (GizmoBuilder);

my $version = "1.0";

sub get_version { return $version; }

# ============================================================================
sub new
{
        my $proto = shift;
        my $class = ref ($proto) || $proto;
        my $args = $class->normalize_constructor_args (shift);
        my $id = $args->{id};

        my $self;

        if ( defined ($id) )
        {
                return $class->restore ($id);
        }

        $self = $class->SUPER::new ($args);
        $self->{is_a} = __PACKAGE__;

        # Set up select items

        # Align
        $self->{fields}->{c1}->{formtype} = "select";
        $self->{fields}->{c1}->{selectsize} = 1;
        $self->{fields}->{c1}->{selectoptions} = "Top,Bottom,Middle,Left,Right";

        # Border
        $self->{fields}->{c2}->{formtype}= "select";
        $self->{fields}->{c2}->{selectsize} = 1;
        $self->{fields}->{c2}->{selectoptions} = "None,1";

        # Scrolling
        $self->{fields}->{c3}->{formtype}= "select";
        $self->{fields}->{c3}->{selectsize} = 1;
        $self->{fields}->{c3}->{selectoptions} = "Auto,Yes,No";

        # Values are percentage or pixels
        $self->{fields}->{c4}->{formtype}= 'select';
        $self->{fields}->{c4}->{selectsize} = 1;
        $self->{fields}->{c4}->{selectoptions} = 'Pixels,Percentage';

        $self->set_field_info
                (
                 "name", "<b>Name</b>", 1,
                 "url", "<b>URL</b>", 1,
                 "description", "<b>Description</b> - For non-iframe browsers", 0,
                 "i1", "<b>Frame Width</b>", 1,
                 "i2", "<b>Frame Height</b>", 1,
                 "c4", "<b>Width value is defined as</b>", 1,
                 "c1", "<b>Align</b>", 1,
                 "c2", "<b>Border</b>", 1,
                 "c3", "<b>Scrolling</b>", 1
                 );

        bless ($self, $class);
        return $self;
}

# ============================================================================
sub exec_show
{
        my $self = shift;
        my $context = shift;
        return $self->show ();
}

# ============================================================================
sub exec_show_summary
{
        my $self = shift;
        my $context = shift;
        return $self->show_summary ();
}

# ============================================================================
sub show
{
        my $self = shift;

        # Initialize the html string.
        my $html = $self->get_buttons ().'<br/>';

        my ($open, $content, $close);

        # See if frame width and height are pixels or percentages
        my $metrics = $self->get_metric_type();
        if( $metrics =~ /ercent/ ){
            #Make the values percentages
            $metrics = '%';
        } else {
            $metrics = undef;
        }

        # Draw the open tag
        $open = "<iframe";
        $open .= " width=\"" . $self->get_i1() . "$metrics\"";
        $open .= " height=\"" . $self->get_i2() . "\"";
        $open .= " src=\"" . $self->get_url() . "\"";
        $open .= " align=\"" . $self->get_c1() . "\"";

        if( $self->get_c2() eq 'None' ){
            $open .= " frameborder=\"0\"";
        } else {
            $open .= " frameborder=\"" . $self->get_c2() . "\"";
        }

        $open .= " scrolling=\"" . $self->get_c3() . "\"";
        $open .= ">";

        # Draw the content
        $content = "<a href=\"" . $self->get_url() . "\">";
        $content .= $self->get_description ();
        $content .= "</a>";

        # Close tag
        $close = "</iframe>";

        # Add to the html buffer
        $html .= $open . $content. $close;

        return $html;
}

# ============================================================================
# This is the primary display for the gizmo
sub show_summary
{
        my $self = shift;

        return $self->show ();
}

# ============================================================================
# Bypass template processing
sub response_may_contain_content
{
        my $self = shift;
        return 1;
}

# ============================================================================
sub get_gizmo_name
{
        return "Inline Frame";
}

sub get_metric_type {
    my $self = shift;

    return $self->c4();
}

