###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package ImageItem;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $DISPLAY);
use Metadot::Controllable;

use vars qw(@ISA);
@ISA=qw(GizmoBuilder Metadot::Controllable);
my $version = "4.5";

sub get_version {
    return $version;
}


####################################
sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

  $self->_form_initialize();
  $self->set_field_info (
    "name", "<b>Image Item Title</b>", 1,
    "description", "<b>Abstract</b>", 0,
    "long_description", "<b>Long Description</b>", 0,
    "url", "<b>Image location URL</b> <i>e.g. http://www.yourcompany.com/abc.gif</i>", 0,
    "item_attachment_file",       "<b>Select Image</b>",  0,
    "c1", "<b>Image height in pixels</b>", 0,
    "c2", "<b>Image width in pixels</b>", 0,
    "c3", "<b>Image's 'alt' tag</b>", 0,
    "t1", "<b>Image's alignment</b>", 0,
    "t2", "<b>Link URL for this image</b>",0,
    "c4", "<b>Include abstract in summary view?</b>", 0,
    "c5", "<b>Show border around image?</b>", 0,
    );

  $self->set_field_form_text('c3', $self->_get_alt_text('c3'));
  $self->set_field_type( 'item_attachment_file', Metadot::UploadsManager::UPLOAD_TYPE_USER_SELECTED() );

}

## will replace the html produced by GizmoBuilder for given instance variable
sub _get_alt_text {
      my $self = shift;
      my $field = shift;
      my $id = $self->id();

      return unless defined $field;

      my $value = $FORM{$field} || $self->{$field};
      my $info = "<input type=text size=\"20\" maxlength=\"30\" name=\"$field\" value=\"".(defined($value) ? $value : '')."\">";
      return $info;
}

sub get_gizmo_name {
   my $self = shift;
   return "Image Item";
}

#override one of the default form types.
sub _form_initialize {
    my $self = shift;

    $self->{fields}->{t1}->{formtype} = "select";
    $self->{fields}->{t1}->{selectsize} = "1";
    $self->{fields}->{t1}->{selectoptions} = "Left,Right,Top,Middle,Bottom";

    $self->{fields}->{t2}->{formtype} = 'text';

    $self->{fields}->{c4}->{formtype} = "select";
    $self->{fields}->{c4}->{selectsize} = "1";
    $self->{fields}->{c4}->{selectoptions} = "Yes,No";

    $self->{fields}->{c5}->{formtype} = "select";
    $self->{fields}->{c5}->{selectsize} = "1";
    $self->{fields}->{c5}->{selectoptions} = "No,Yes";


    my $ts = "Image files may be be specified via either a URL, or
a file to upload as an attachment.  If both are specified, the
attachment is used.  The size of the image in the 'bulleted list' view
can be specified by entering height and width below.  Both height and width must be filled for the
information to be used.
In the bulleted list or 'short' view, the &quot;Abstract&quot; text for the image may be
optionally hidden.
<br />The full
descriptive text for the image is not displayed in list view.
Full information about the image may be accessed by clicking on the
image when it is displayed in a list.  In the &quot;full&quot; view for an image, the image
is <i>displayed at full size</i>, and its descriptive text is included.  The
edit form allows the image alignment to be indicated, and if alignment
is specified, it is used to place the image in the &quot;full&quot; view area.
The image alignment specification is not used in the &quot;bulleted list&quot;
view.
";
    $self->set_intro_text($ts);

}

sub get_upload_filename_icon_and_url {
    my $self = shift;
    return $self->SUPER::get_upload_filename_icon_and_url( {
        id => $self->id(),
        isa => $self->is_a(),
        field_name => 'item_attachment_file',
    });
}

sub get_upload_filename_iconsrc_and_url {
    my $self = shift;
    return $self->SUPER::get_upload_filename_iconsrc_and_url( {
        id => $self->id(),
        isa => $self->is_a(),
        field_name => 'item_attachment_file',
    });
}


sub save {
    my $self = shift;

    my $url = $self->get_url();
    if (defined($url) && ($url ne '') && ($url ne "http://"))
    {
        my $not_ok = $self->_img_src_url_check($url);
        if ($not_ok)
        {
            $DISPLAY->print_click_back('Error', $DISPLAY->UNALLOWED_HTML_MESSAGE());
            $self->set_url(undef);
            exit();
        }
    }

    $self->SUPER::save();
}


##########################################
### show
### return one newsitem (page format)
##########################################
sub show {
  my $self = shift;
  $self->debug_msg(1, "ImageItem::show()");
  my $s="";

  my $name            = $self->get_name();
  my $description     = $self->get_description();
  my $longdescription = $self->get_long_description();
  my $url             = $self->get_url();
  my $align       = $self->get_t1();
  my $link_url    = $self->link_url();

  my ( $file, $file_icon, $file_url ) = $self->get_upload_filename_icon_and_url();

#  print STDERR "AT IMAGEITEM::SHOW $file === $file_icon === $file_url\n";

  my $alt       = $self->get_c3();
  my $show_border = $self->get_c5();

  my $imageurl = '';
  #use downloaded url if it exists, else go with specified url
  if (defined($file) && ($file ne '') && defined($file_url)) {
      $imageurl = $file_url;
  }
  elsif (defined($url) && ($url ne '') && ($url ne "http://")) {
      my $not_ok = $self->_img_src_url_check($url);
      if ($not_ok) {
      $s.= "<b>$name</b><i>\n<br /><br />$description</i>\n";
      $s .= "<p />Error: URL was unsafe.  CGI image source not allowed.\n";
      return $s;
      }
      $imageurl = $url;
  }

  $s.= "<b>$name</b>\n";
#  $s .= "<br /><font size=-2>$imageurl</font>\n";
  $s .= "<br /><br />$description\n";
  # show image full-size here.

  if( $imageurl ) {

      my $img_border = ( lc $show_border eq 'yes' ) ? '1' : '0';

      $s.= "<br /><br />";
      my $img = "<img border=$img_border src=\"$imageurl\" ";
      # use alignment info if available.
      if (defined($align)) {
      $img .= "align='" . lc $align . "' ";
      }
      $img .= " alt=\"$alt\">\n";
      if ($link_url ne '') {
          $s .= "<a href=\"$link_url\">$img</a>";
      } else{
          $s .= $img;
      }
  } else {

      $s.= "<br /><br />";
  }

  $s.= "$longdescription\n" if ($longdescription);
  $s.= "<br />\n";

  return $s;
}

##########################################
### show_in_list
### return one newsitem (list format)
##########################################
# aju: do I need this method?
sub show_in_list {
  my $self = shift;
  $self->debug_msg(1, "ImageItem::show_in_list()");
#    print STDERR "in ImageItem:show_in_list()\n";

  my $name        = $self->get_name();
  my $description = $self->get_description();
  my $url         = $self->get_url();
  my $panel       = $self->get_panel();

  $name = "<img src=$url>"; #if ($url) ;

  return "<b>$name</b> - $panel<br />\n$description<br />\n";
}


sub show_summary {
    my $self        = shift;
    my $iid         = $self->get_iid();
    my $name        = $self->get_name();
    my $description = $self->get_description();
    my $url         = $self->get_url();

    my ( $file, $file_icon, $file_url ) = $self->get_upload_filename_icon_and_url();
    my $buttons     = $self->get_buttons();
    my $height       = $self->get_c1();
    my $width       = $self->get_c2();
    my $alt       = $self->get_c3();
    my $show_desc       = $self->get_c4();

    my $show_border = $self->get_c5();

    #just do inline alignment for the 'bullet' view.
    my $default_bullet_align = "top"; # default alignment for bullet list is 'top'.
                      # The actual set alignment is used in
                      # the more detailed view.
    my $html        = '';
    my $imageurl = '';

    #use downloaded url if it exists, else go with specified url
    if (defined($file) && ($file ne '') && defined($file_url)) {
    $imageurl = $file_url;
    }
    elsif (defined($url) && ($url ne '') && ($url ne "http://")) {
      my $not_ok = $self->_img_src_url_check($url);
      if ($not_ok) {
      $imageurl = '';
      }
      else {
      $imageurl = $url;
      }
    }

    $html .= "$buttons ";

    if ($imageurl ne '' && $imageurl ne "http://") {
    # wrap image with link to display more detailed information
    $html .= "<a href='".
      $self->make_url("index.pl?id=$iid&amp;isa=ImageItem&amp;op=show")."' CLASS='centerNewsTitle'>$name</a> \n";

    if ( ( lc $show_desc eq "yes" ) and
         ( ( $description =~ /\S/ ) and ( not ( $description =~ /^\s*<!-- form type is text -->\s*$/ ) ) ) ) {

        $html .= "<br />$description";
    }

    $html .= "<br />";

        $html .= "<a href='".
      $self->make_url("index.pl?id=$iid&amp;isa=ImageItem&amp;op=show")."'>";

    my $img_border = ( lc $show_border eq 'yes' ) ? '1' : '0';

    $html .= "<img src=\"$imageurl\" border=$img_border align=$default_bullet_align vspace=5 ";
    if (defined($height)  && $height =~ /\d/ ) {
        $html .= " height=$height ";
    }
    if ( defined($width) && $width =~ /\d/ ) {
        $html .= " width=$width ";
    }
    if (defined($alt)) {
        $html .= " alt=\"$alt\" ";
    }

    #close image and link tags
    $html .= "></a>\n";

    } else {
        $html .= "<b>$name</b>\n";
    }

    $html .= "<br />\n";

    return $html;
}


sub link_url {
    my $self = shift;
    return $self->t2(@_);
}

#  sub get_Height {
#   my $self = shift;

#   return $self->{height};
#  }

#  sub get_Width {
#   my $self = shift;

#   return $self->{width};
#  }

1;

__END__

=head1 NAME

Gizmo::ImageItem - A Gizmo subclass to display image items.

=head1 SYNOPSIS

    use Gizmo::ImageItem;
    $imageitem = ImageItem->new([$id]);

    $name = $imageitem->get_gizmo_name();
    $item_display_html = $imageitem->show();
    $bullet_display_html = $imageitem->show_as_bullet();
    $version = Discussion->get_version();



=head1 DESCRIPTION

The ImageItem class supports the handling of images as 'items'.
ImageItem is descended from L<GizmoBuilder>.

It uses the form editing capabilities of its parent class,
GizmoBuilder, to allow the following info about the image to be
specified:

Image files may be be specified in the edit form via either a URL, or
a file to upload as an attachment.  If both are specified, the
attachment is used.  The size of the image in the 'bulleted list' view
can be specified by entering height and width info into the edit form.
In the bulleted list view, the "abstract" text for the image may be
optionally hidden via an option in the edit form.  The full
descriptive text for the image is not generated in list view.

Full information about the image may be accessed by clicking on the
image displayed in a list.  In the "full" view for an image, the image
is displayed at full size, and its descriptive text is included.  The
edit form allows the image alignment to be indicated, and if alignment
is specified, it is used to place the image in the "full" view area.
The image alignment specification is not used in the "bulleted list"
view.

Note: Attached images won't be displayed inline if debugging is turned on.


=head2 Methods

Constructor;  instantiates a new item or retrive its persistent info from the backend
database.
$id is the $id of the ImageItem:

    $imageitem = Gizmo::ImageItem->new([$id]);


Class method to retrieve the version of the gizmo

    $version = Discussion->get_version();

Get the name of the item for pretty-printing:

    $name = $imageitem->get_gizmo_name();

Generate html to show the image full-size and include its descriptive text:

    $item_display_html = $imageitem->show();

Generate html to  'list', or give  abbreviated display of the image information:

    $bullet_display_html = $imageitem->show_as_bullet();

=cut


