###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name$
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package ImageTable;
use strict;

use Gizmo::ImageTable::ImageRow;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS);

use vars qw(@ISA);
@ISA=qw(GizmoBuilder Category);
my $version = "4.5";

sub get_version {
    return $version;
}


sub customize_permissions {
    my $class = shift;
    my $AC = shift;

    $class->SUPER::customize_permissions($AC);

    $AC->add_op('MOD', 'add_child');
    $AC->add_op('MOD', 'create_child');

    $AC->add_op('DISP', 'process_order');
}


####################################
sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

  $self->_form_initialize();

  $self->set_field_info (
    "name", "<b>Name</b>", 1,
    "c5", "<b>Description</b>",0,
    "c1", "<b>Order Recipient Email Address</b>",1,
    "description", "<b>Order Email Body</b>",0,
    "t10", "<b>Acknowledgement Email Body</b>",0,
    "t1", "<b>Column One Title</b>",1,
    "t2", "<b>Column Two Title</b>",1,
    "t3", "<b>Column Three Title</b>",1,
    "t4", "<b>Column Four Title</b>",1,
    "t5", "<b>Column Five Title</b>",0,
    "t6", "<b>Column Six Title</b>",0,
    "t7", "<b>Column Seven Title</b>",0,
    "t8", "<b>Column Eight Title</b>",0,
    "t9", "<b>Column Nine Title</b>",0,
    );

    $self->t1("Image ID") if (!$self->t1());
    $self->t2("Description") if (!$self->t2());
    $self->t3("Thumbnail") if (!$self->t3());
    $self->t4("Download") if (!$self->t4());
    $self->t5("Order") if (!$self->t5());
    $self->t6();
    $self->t7();
    $self->t8();
    $self->t9();

}

sub _form_initialize {
    my $self = shift;

    $self->{fields}->{c5}->{formtype} = "mediumtext";
    $self->{fields}->{c1}->{formtype} = "text";
    $self->{fields}->{t1}->{formtype} = "text";
    $self->{fields}->{t2}->{formtype} = "text";
    $self->{fields}->{t3}->{formtype} = "text";
    $self->{fields}->{t4}->{formtype} = "text";
    $self->{fields}->{t5}->{formtype} = "text";
    $self->{fields}->{t6}->{formtype} = "text";
    $self->{fields}->{t7}->{formtype} = "text";
    $self->{fields}->{t8}->{formtype} = "text";
    $self->{fields}->{t9}->{formtype} = "text";

}

sub set_field_default_data {
    my $self = shift;

    my $field;

    while ($#_ > 0) {
        $field = shift;

        $self->{fields}->{$field}->{default} = shift;
    }
}

sub get_additional_fields {
    my $self = shift;
    my @output;

    for (my $xx=6; $xx <= 9 ; $xx++) {

        my $fieldname = "t".$xx;
        if ($self->${fieldname}()) {
            push(@output, $self->${fieldname}());
        }
    }

    return @output;
}

sub gizmo_list_header {
    my $class = shift;
    my $cat_name = shift;
    my $output;

    return "";
}

sub get_gizmo_name {
   my $self = shift;
   return "Image Table";
}

sub gizmo_list_decorator {
    return 'HTML::Decorator::NullDecorator';
}

sub www_add_child {
  my $self = shift;
  my $class = ref($self) || $self;
  my $errormsg = shift;
  my $child_type = shift;
  $self->debug_msg(3, "ImageTable::www_add_child()");

  my $child_isa = $FORM{child_type} || $child_type;
  my $gizmo_name = $child_isa->get_gizmo_name();

  my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
  # we re-instanciate the object because we want a brand new one
  # unless there is an error message (we want to keep his data)
  my $child = $child_isa->new({ parent_id => $parentid });

    $errormsg = $errormsg ?
                "<p /><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p />\n" :
                "";

  my $form = $child->show_edit_form("Adding new $gizmo_name");
  my $content =
  Components->color_palette_code().
"
<form name=fm action='index.pl' method='post' ENCTYPE=\"multipart/form-data\">
<input type='hidden' name='isa' value='ImageTable'>\n
<input type='hidden' name='iid' value='$parentid'>\n
<input type='hidden' name='op' value='create_child'>\n
<input type='hidden' name='child_type' value='ImageRow'>\n
$errormsg
$form
<table align='right'><tr><td><input type='submit' value='Create'></td></tr></table>
<br />&nbsp;
</form>
";

  $child->print_form($content, "Adding new $child_isa");
}


sub www_create_child {
  my $self = shift;
  my $child_type = shift;

  $self->debug_msg(3, "Gizmo::www_create_child()");

  my $child_isa = $FORM{child_type} || $child_type;

  my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
  # we re-instanciate the object because we want a brand new one
  my $child = $child_isa->new({
             uid => $USER->uid(),
             parent_id => $parentid,
            });

  # this method should ALWAYS be overridden in derived classes.
  $child->process_edit_form();
}

sub show_edit_form {
    my $self = shift;
    $self->debug_msg(4, "ImageTable::show()");

    my $title = shift;

    my $s = "";
    my $order = 0;

    $s .= "<p /><h1>" . $title . "</h1>\n";

    $s .= "<p>" . $self->get_intro_text() . "</p>";

    foreach my $field (sort {$self->{fields}->{$a}->{order} <=> $self->{fields}->{$b}->{order}}
                        keys(%{$self->{fields}})) {

        my $order = $self->{fields}->{$field}->{order};
        if ((!(defined($order))) || ($order <= 0)) {  next; }

        $s .= $self->get_form_box($field);

    }

    return $s;
}


sub _show_create_success {
    my $self = shift;
    my $isa = $self->is_a();

    my $parentid = $self->parent_id();

    my $parent_obj = Gizmo->new($parentid);

    my $grandparent = $parent_obj->parent_id();

    $self->two_choices("<Big>New $isa added!</big><br /><br />What do you want to do?",
               "confirm",
               "Add another $isa",
               "index.pl?child_type=$isa&isa=Category&iid=$grandparent&op=add_child&app=",
               "Enough for now!",
               "index.pl?id=$grandparent");
}


##########################################
### show
### return one newsitem (page format)
##########################################
sub show {
  my $self = shift;
  $self->debug_msg(1, "ImageTable::show()");
  my $s="";
  #print STDERR "in ImageTable:show()\n";

  my $id = $self->id();
  my $isa = $self->is_a();
  my $name = $self->name();
#    my $description = $self->description();
  my $description = $self->get_c5();
  my $child = "ImageRow";

  my $headbg = "#C0C0C0";
  my $bodybg = "#F0F0F0";

  my $top_anchor = int rand(999999);
  my $page_id = ($self->{is_inline})? $self->parent_id() : $self->id();

  my $spacer = "<table border=\"0\" width=\"100%\"><tr><td>&nbsp;</td></tr></table>";

  my $admin_panel;
  my $show_panels;
  if (($self->is_allowed_to_do('modify', $USER)) && ($SESSION->admin_mode())) {
    $admin_panel .= "[ <a href=\""
                 .  $self->make_url("index.pl?iid=$id&amp;isa=$isa&amp;op=add_child&amp;child_type=$child")
                 .  "\">Add ImageRow</a> ]";
    $show_panels = 1;
  } else {
    $admin_panel .= "";
  }

    my @tmparr = ();
    my $row_list_ref =  ImageRow->get_rows($self->id(), $show_panels);
    my @row_arr = @{$row_list_ref};

    my $rows;
    $rows .= "<table align='center' border=\"0\" cellpadding=\"4\" cellspacing=\"2\"><tr bgcolor=$headbg>\n";
    for(my $xx = 1; $xx <= 9; $xx++){
        my $val_name = "t".$xx;
        my $val = ${val_name};
        if ($self->$val()){
            $rows .= "<td>".$self->$val()."</td>\n";
        }
    }
    $rows .= "</tr>\n";

    my $mime_type = "null";
    foreach my $row_ref (@row_arr) {
        my %row = %{$row_ref};

        my $url_path = $self->make_url("index.pl?id=$row{id}&amp;isa=ImageRow&amp;op=show_image&amp;image=c3");
        my $img_path = $self->make_url("index.pl?id=$row{id}&amp;isa=ImageRow&amp;op=show_image&amp;image=$row{thumbnail}");

        my $img_big = $self->make_url("index.pl?id=$row{id}&amp;isa=ImageRow&amp;op=show_image&amp;image=c1&amp;type=$mime_type");

        my $img_med = $self->make_url("index.pl?id=$row{id}&amp;isa=ImageRow&amp;op=show_image&amp;image=c2&amp;type=$mime_type");

        my $img_small = $self->make_url("index.pl?id=$row{id}&amp;isa=ImageRow&amp;op=show_image&amp;image=c3&amp;type=$mime_type");

        # building row list section

        $rows .= "<tr bgcolor=$bodybg>";

        my $images = "<a href=\"$img_big\">$row{img1_desc}</a><br /><br />" if (($row{img1}) && ($row{img1_desc}));
        $images .= "<a href=\"$img_med\">$row{img2_desc}</a><br /><br />" if (($row{img2}) && ($row{img2_desc}));
        $images .= "<a href=\"$img_small\">$row{img3_desc}</a><br /><br />" if (($row{img3}) && ($row{img3_desc}));

        $rows .= "<td valign='top'>$row{name}</td>"
                ."<td valign='top'>$row{description}</td>"
                ."<td valign='top'><a href=\"$url_path\"><img src=\"$img_path\" border=\"0\"></a>"
                ."</td>"
                ."<td valign='top'>"
                . $images
                ."</td>"
                ."<td valign='top'>$row{order}<br />"
                ."<input type=checkbox name=orderbox value=$row{name}></td>";

        $rows .= "<td valign='top'>$row{extra_1}</td>\n" if ($row{extra_1});
        $rows .= "<td valign='top'>$row{extra_2}</td>\n" if ($row{extra_2});
        $rows .= "<td valign='top'>$row{extra_3}</td>\n" if ($row{extra_3});
        $rows .= "<td valign='top'>$row{extra_4}</td>\n" if ($row{extra_4});
        $rows .= "<td valign='top'>$row{extra_5}</td>\n" if ($row{extra_5});

        if ($row{panel}) {
            $rows .= "<td>$row{panel}</td>\n";
        }

        $rows .= "</tr>";
    }

    $rows .= "</table>\n";

    my $address = $SESSION->get_value('address');

  $s .= <<EOM
        <p />$description <p />
    $admin_panel<br />
    <form method=post action=index.pl>
    <input type=hidden name=isa value=ImageTable>
    <input type=hidden name=op value=process_order>
    <input type=hidden name=id value=$id>
    $rows
    <table width="100%"><tr><td valign='bottom' align='left'>
    Enter Mailing Address: <textarea name=address wrap=virtual>$address</textarea>
    </td>
    <td valign='bottom' align='right'>
    <input type=submit name=submit value="Order Images">
    </td></tr></table>
    </form>

    <p />
    $spacer
EOM
;

  return $s;
}

sub www_process_order {
    my $self = shift;
    my @images;

    my $id = $self->id();

    my $images_ref = $FORM{orderbox};
    if (ref($FORM{orderbox}) eq "ARRAY"){
        @images = @{$images_ref};
    } elsif ($FORM{orderbox}) {
        push (@images, $FORM{orderbox});
    } else {
        $self->print_click_back("You did not choose any images.");
        exit;
    }

    if ($FORM{'address'}) {
        $SESSION->set_value("address", $FORM{'address'});
    } elsif (my $address = $SESSION->get_value('address')) {
        #still fine.
    } else {
        $self->print_click_back("You did not include a mailing address.");
        exit;
    }

    my $parentid = $self->parent_id();
    my $email = $self->c1();
    my $order_text = $self->description();
    my $ack = $self->t10();
    my $user_email = $USER->email();
    my $user_name = $USER->fullname();

    my $user_ordering = "\n\n$user_name ($user_email) has placed the following order:";
    my $user_address_order = "\n\n$user_name supplied the following mailing address:\n";
    my $user_address_ack = "\n\nThe following address will be used for this order:\n";
    my $user_address .= $SESSION->get_value('address');

    $order_text .= $user_ordering . "\n\n".join(", ", @images)."\n\n$user_address_order$user_address\n";
    $ack        .= "\n\n".join(", ", @images)."\n\n$user_address_ack $user_address\n";

    use Sendmail;

#print STDERR "id=$id\nemail=$email\nuser=$user_email\n\n";
    my $sm; my $result;
    if ($email) {
        $sm = Sendmail->new($email, '', '', 'Images Order', $order_text);
        ($result) = $sm->sendMail();
    }
    else {
        $self->print_click_back("Error: no email address to which to send the order has been defined.  Please contact the site administrator.");
        exit;
    }

    if ($user_email) {
        $sm = Sendmail->new($user_email, '', '', 'Images Order', $ack);
        ($result) = $sm->sendMail();
    }
    else {
        $self->print_click_back("Error: could not get user email address; confirmation message not sent.");
    }

    $self->two_choices("Your order has been placed. A confirmation letter has been sent to your email address",
                        "confirm",
                        "Return to the images",
                        "index.pl?iid=$parentid"
                        );
}

##########################################
### show_in_list
### return one newsitem (list format)
##########################################
# aju: do I need this method?
sub show_in_list {
  my $self = shift;
  $self->debug_msg(1, "ImageTable::show_in_list()");
#    print STDERR "in ImageTable:show_in_list()\n";

  my $name        = $self->get_name();
  my $description = $self->get_description();
  my $url         = $self->get_url();
  my $panel       = $self->get_panel();

  $name = "<img src=$url>"; #if ($url) ;

  return "<b>$name</b> - $panel<br />\n$description<br />\n";
}

# I don't need this... do I?
##########################################
### show_in_column
### return one newsitem (news column format)
##########################################
#  sub show_in_column {
#    my $self = shift;
#    $self->debug_msg(1, "ImageTable::show_in_column()");
#    print STDERR "in ImageTable:show_in_column()\n";

#    my $iid             = $self->get_iid();
#    my $name            = $self->get_name();
#    my $description     = $self->get_description();
#    my $url             = $self->get_url();
#    my $longdescription = $self->get_long_description();
#    my $panel           = $self->get_panel();
#    my $s               = "";

#    if($longdescription || $url){
#      $name = "<a class='newsitemtitle' href='index.pl?id=$iid&amp;isa=ImageTable&amp;op=show'>$name</a>";
#    }

#    $s.= $panel;
#    $s.= "<table cellpadding=\"0\" cellspacing=\"0\"><tr><td class='newsitemtitle'>- $name\n</td></tr>\n";
#    $s.= "<tr><td class='newsitem'>$description<br />&nbsp;</td></tr></table>\n\n";

#    return $s;
#  }


sub show_summary {
    my $self = shift;
    my @args = @_;

    my $buttons     = $self->get_buttons();
    my $id = $self->id();
    my $isa = $self->is_a();
#    my $text = "<a href=\"index.pl?isa=$isa&amp;op=show&amp;id=$id\">".$self->name()."</a>";

    $self->{is_inline} = 1;
    my $text = $self->show();

   return $buttons.$text;
}

sub www_save {
    my $self = shift;

    if (!$self->get_iid()) {
        use Gizmo::Category;
        my $cat = Category->new();
        $cat->name($FORM{name});
        $cat->uid($self->uid());
        $cat->description("");
        $cat->parent_id($self->parent_id);
        $cat->is_a("Category");
        $cat->keywords("");
        $cat->display_look("");
        $cat->created_by("");
        $cat->t1("");
        $cat->t2("");
        $cat->t3("");
        my $newcatid = $cat->add();

        $self->{parent_id} = $newcatid;
    }

    $self->SUPER::www_save();

}




1;

__END__

=head1 NAME

Gizmo::ImageTable - A Gizmo subclass to display ImageTables and contain FAQ questions.

=head1 SYNOPSIS

    use Gizmo::ImageTable;
    $ImageTable = ImageTable->new([$id]);

    $name = $ImageTable->get_gizmo_name();
    $item_display_html = $ImageTable->show();
    $bullet_display_html = $ImageTable->show_as_bullet();
    $version = ImageTable->get_version();



=head1 DESCRIPTION

This is a container and interface for viewing FAQ questions and organising them.

=head2 Methods

Constructor;  instantiates a new item or retrive its persistent info from the backend
database.
$id is the $id of the ImageTable:

    $ImageTable = Gizmo::ImageTable->new([$id]);


Class method to retrieve the version of the gizmo

    $version = Discussion->get_version();

Get the name of the item for pretty-printing:

    $name = $ImageTable->get_gizmo_name();

Generate html to show the image full-size and include its descriptive text:

    $item_display_html = $ImageTable->show();

Generate html to  'list', or give  abbreviated display of the image information:

    $bullet_display_html = $ImageTable->show_as_bullet();

=cut


