###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package ImageRow;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS);

use vars qw(@ISA);
@ISA=qw(GizmoBuilder);
my $version = "5";

sub get_version {
    return $version;
}

sub customize_permissions {
    my $class = shift;
    my $AC = shift;

    $class->SUPER::customize_permissions($AC);

    $AC->add_op('DISP', 'show_image');
}


####################################
sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

  my @other_fields = $self->get_other_fields();

  $self->_form_initialize();

  $self->set_field_info (
    "name", "<b>ID#</b>", 1,
    "description", "<b>Description</b>", 1,
    "item_attachment_file", "<b>Thumbnail Attachment File</b>", 1,
    "c1", "<b>Image Attachment File</b>", 0,
    "t1", "<b>Link Description</b>", 0,
    "c2", "<b>Image Attachment File</b>", 0,
    "t2", "<b>Link Description</b>", 0,
    "c3", "<b>Image Attachment File</b>", 0,
    "t3", "<b>Link Description</b>", 0,
    "t4", "<b>Order Text</b>", 0,
    @other_fields,
    );


  $self->{display_wrapped} = { show => 1,
                             };
}


sub _form_initialize {
    my $self = shift;

    $self->{fields}->{item_attachment_file}->{formtype} = "user_selected_upload";
    $self->{fields}->{c1}->{formtype} = "user_selected_upload";
    $self->{fields}->{c2}->{formtype} = "user_selected_upload";
    $self->{fields}->{c3}->{formtype} = "user_selected_upload";

}

sub get_other_fields {
    my $self = shift;
    my @output;

    my $parent = $self->parent_id();
    my $parent_obj = Gizmo->new($parent);

    my @fields = $parent_obj->get_additional_fields();

    my $counter = 5;
    foreach my $field (@fields) {
        my $tag = "t".$counter;
        push (@output, $tag);
        my $field_string = "<b>$field</b>";
        push (@output, $field_string);
        push (@output, "0");
        $counter++;
    }

    return @output;
}

sub get_gizmo_name {
   my $self = shift;
   return "ImageRow";
}


##########################################
### show
### return one newsitem (page format)
##########################################
sub show {
  my $self = shift;
  $self->debug_msg(1, "ImageRow::show()");
  my $s="";
#  print STDERR "in ImageRow:show()\n";

  my $question    = $self->get_name();
  my $answer         = $self->get_description();

  $s .= <<EOM


EOM
;

  return $s;
}

##########################################
### show_in_list
### return one newsitem (list format)
##########################################
# aju: do I need this method?
sub show_in_list {
  my $self = shift;
  $self->debug_msg(1, "ImageRow::show_in_list()");
#    print STDERR "in ImageRow:show_in_list()\n";

  my $name        = $self->get_name();
  my $description = $self->get_description();
  my $url         = $self->get_url();
  my $panel       = $self->get_panel();

  $name = "<img src=$url>"; #if ($url) ;

  return "<b>$name</b> - $panel<br />\n$description<br />\n";
}

# I don't need this... do I?
##########################################
### show_in_column
### return one newsitem (news column format)
##########################################
#  sub show_in_column {
#    my $self = shift;
#    $self->debug_msg(1, "ImageRow::show_in_column()");
#    print STDERR "in ImageRow:show_in_column()\n";

#    my $iid             = $self->get_iid();
#    my $name            = $self->get_name();
#    my $description     = $self->get_description();
#    my $url             = $self->get_url();
#    my $longdescription = $self->get_long_description();
#    my $panel           = $self->get_panel();
#    my $s               = "";

#    if($longdescription || $url){
#      $name = "<a class='newsitemtitle' href='index.pl?id=$iid&amp;isa=ImageRow&amp;op=show'>$name</a>";
#    }

#    $s.= $panel;
#    $s.= "<table cellpadding=\"0\" cellspacing=\"0\"><tr><td class='newsitemtitle'>- $name\n</td></tr>\n";
#    $s.= "<tr><td class='newsitem'>$description<br />&nbsp;</td></tr></table>\n\n";

#    return $s;
#  }

sub www_save {
    my $self = shift;
    $self->debug_msg(3, "ImageRow::www_save()");

    my $parentid = $self->parent_id();
    my $type = $self->is_a();
    my $printname = $self->get_gizmo_name();

        $self->name($FORM{name}) if ($FORM{name});
        $self->description($FORM{description}) if ($FORM{description});
        $self->t1($FORM{t1}) if ($FORM{t1});
        $self->t2($FORM{t2}) if ($FORM{t2});
        $self->t3($FORM{t3}) if ($FORM{t3});
        $self->t4($FORM{t4}) if ($FORM{t4});
        $self->t5($FORM{t5}) if ($FORM{t5});
        $self->t6($FORM{t6}) if ($FORM{t6});
        $self->t7($FORM{t7}) if ($FORM{t7});
        $self->t8($FORM{t8}) if ($FORM{t8});
        $self->t9($FORM{t9}) if ($FORM{t9});

    if (!$self->id()) {
        # new object
        my $newid;
        if($newid = $self->add()) {
            $self->process_uploads($newid);
            $self->save();
            $self->_show_create_success();
        } else {
            $self->debug_msg(3, "ImageRow::www_save : error when adding to DB.");
            $self->two_choices("Sorry. The $printname failed to be saved for some reason."
                        . " What do you want to do?",
                        "error",
                        "Try again",
                        "index.pl?iid=$parentid&isa=ImageTable&op=add_child&child_type=$type",
                        "Enough for now",
                        "index.pl?id=$parentid");
        }


    } else {
        # saving an already existing object
        $self->process_uploads($self->id());
        $self->save();
        $self->_show_edit_success();
    }

}

sub save_images {
    my $self = shift;
    my $iid = $self->id();
    my $uid = $USER->uid();
    my $filename = "";
    my $currfile = $self->file1();

    if ($FORM{'c1'}) {
        $filename = $self->upload_file($FORM{'c1'}, $iid, $uid);
        $self->c1($filename);
        $self->i1($self->file1_size());
    }
    if ($FORM{'c2'}) {
        $filename = $self->upload_file($FORM{'c2'}, $iid, $uid);
        $self->c2($filename);
        $self->i2($self->file1_size());
    }
    if ($FORM{'c3'}) {
        $filename = $self->upload_file($FORM{'c3'}, $iid, $uid);
        $self->c3($filename);
        $self->i3($self->file1_size());
    }
    if ($FORM{'file1'}) {
        $filename = $self->upload_file($FORM{'file1'}, $iid, $uid);
        $self->i4($self->file1_size());
        $self->file1($filename);
    }

    my $full_size = $self->i1() + $self->i2() + $self->i3() + $self->i4();
    $self->{file1_size} = $full_size;

    $self->get_object_accessor()->update(object=>$self);
    #DBAccess->sqlUpdate("instance", "iid=$iid", file1_size => $full_size);

    return;
}

sub www_delfileok {
    my $self = shift;
    my $iid = $self->id();
    my $isa = $self->is_a();

    my %size_field = ('c1' => 'i1', 'c2' => 'i2', 'c3' => 'i3', 'file1' => 'i4');

    my $size_var = $size_field{$FORM{name}};
    $self->$size_var(0);

    my $total = 0;
    foreach my $key (keys (%size_field)) {
        my $field = $size_field{$key};
        $total += $self->$field();
    }
    $self->delete_attachment($FORM{name}, $total);

    $self->get_object_accessor()->update(object=>$self);
    #DBAccess->sqlUpdate("instance", "iid=$iid", ($size_var=>0));

    $self->md_redirect($self->make_url("index.pl?id=$iid&isa=$isa&op=modify"));

}

sub show_summary {
    my $self = shift;
    my @args = @_;

    my $buttons     = $self->get_buttons();
    my $text = $self->show(@args);

    return $buttons.$text;
}

sub get_rows {
    my $class = shift;
    my $parentid = shift;
    my $panels = shift || 0;
    my $order = shift || "InstanceOrder";
    my @data;

    #print STDERR "in ImageRow:get_rows($parentid)\n";

    my $params = "iid, name, description, t1, t2, t3, t4, t5, t6, t7, t8";
    my $result = DBAccess->sqlSelectMany($params, "instance",
                                        "parentiid = $parentid ORDER BY $order");

    while (my ($id, $name, $desc, $t1, $t2, $t3, $t4, $t5, $t6, $t7, $t8)
        = $result->fetchrow()) {
        my %data;

        if ($panels) {
            my $obj = Gizmo->new($id);
            $data{panel} = $obj->get_buttons();
        } else {
            $data{panel} = "";
        }

        $data{id}           = $id;
        $data{name}         = $name;
        $data{description}  = $desc;
        $data{thumbnail}    = "item_attachment_file";
        $data{img1}         = "c1";
        $data{img1_desc}    = $t1;
        $data{img2}         = "c2";
        $data{img2_desc}    = $t2;
        $data{img3}         = "c3";
        $data{img3_desc}    = $t3;
        $data{order}        = $t4;
        $data{extra_1}      = $t5;
        $data{extra_2}      = $t6;
        $data{extra_3}      = $t7;
        $data{extra_4}      = $t8;

        my $data_ref = \%data;
        push (@data, $data_ref);
    }

    my $list = \@data;

    return $list;
}

sub www_show_image {
    my $self = shift;

    $self->debug_msg(3, "ImageTable::ImageRow::www_show_image");

    my $uid = $self->uid;
    my $iid = $self->id;
    my $image_var = $FORM{'image'};
    chomp $image_var;

    my %params;
    $params{id} = $iid;
    $params{isa} = $self->is_a();
    $params{field_name} = $image_var;
        if($FORM{type}){
        $params{type} = $FORM{type};
        }

    my $fields = \%params;
    $self->www_download_file($fields);

}


sub _show_create_success {
    my $self = shift;

    #use Gizmo::ImageTable;
    my $parent = Gizmo->new($self->parent_id());
    my $catid = $parent->parent_id();

    my $isa = $self->is_a();
    my $parentid = $self->parent_id();

    $self->two_choices("<big>New " . $self->get_gizmo_name() .
            " added!</big><br /><br />What do you want to do?",
            "confirm",
            "Add another " . $self->get_gizmo_name(),
            "index.pl?child_type=$isa&isa=Category&iid=$parentid&op=add_child&app=",
            "Enough for now!",
            "index.pl?id=$catid");

}

sub _show_edit_success {
    my $self = shift;

    #use Gizmo::ImageTable;
    my $parent = Gizmo->new($self->parent_id());
    my $catid = $parent->parent_id();

    $self->md_redirect("index.pl?iid=".$catid);

}

sub show_panel {
    my $self = shift;
    my $id = shift || $self->id();
    my $isa = shift || $self->is_a();
    $self->debug_msg(5, "ImageRow::show_panel(id=$id, isa=$isa)");

    return '' unless ($SESSION->admin_mode());

    my $str = "";
    $str .= $self->show_edit_button();
    $str .= $self->show_ac_button();
    $str .= $self->show_del_button();

    return $str;
}


1;

__END__

=head1 NAME

Gizmo::ImageRow - A Gizmo subclass to display ImageRows.

=head1 SYNOPSIS

    use Gizmo::ImageRow;
    $ImageRow = ImageRow->new([$id]);

    $name = $ImageRow->get_gizmo_name();
    $item_display_html = $ImageRow->show();
    $bullet_display_html = $ImageRow->show_as_bullet();
    $version = Discussion->get_version();



=head1 DESCRIPTION

=head2 Methods

Constructor;  instantiates a new item or retrive its persistent info from the backend
database.
$id is the $id of the ImageRow:

    $ImageRow = Gizmo::ImageRow->new([$id]);


Class method to retrieve the version of the gizmo

    $version = Discussion->get_version();

Get the name of the item for pretty-printing:

    $name = $ImageRow->get_gizmo_name();

Generate html to show the image full-size and include its descriptive text:

    $item_display_html = $ImageRow->show();

Generate html to  'list', or give  abbreviated display of the image information:

    $bullet_display_html = $ImageRow->show_as_bullet();

=cut


