###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Item;

use strict;

use Metadot qw($PARAMS);

use Gizmo;
use GizmoBuilder;
use Metadot::Rateable;
use Metadot::Controllable;
use vars qw(@ISA);
use Metadot::Notifiable;

@ISA=qw(GizmoBuilder Metadot::Rateable Metadot::Controllable Metadot::Notifiable);

my $version = "4.5";

sub get_version {
    return $version;
}

my $rateable_ac;

sub customize_permissions {
    my $self = shift;
    my $ac   = shift;

    $self->SUPER::customize_permissions($ac);

    # handle rateable
    if( $PARAMS->{items_are_rateable} && $self->can('www_rate') ) {
        $ac->add_bundle('RATE', 'Rate', '2','2');
        $ac->add_op('RATE', 'rate');
        $ac->add_op('RATE', 'delete_rating');
        $ac->add_bundle('RATING_DETAILS',
                        'View Rating Details', '8','0');
        $ac->add_op('RATING_DETAILS', 'rating_details');
    }

    Metadot::Notifiable->customize_permissions($ac);

    return $ac;
}


####################################
sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

    $self->{cool}='No';

    #make the field a checkbox
    $self->set_field_type('c3', 'checkbox');
    $self->set_field_type('c4', 'checkbox');

    $self->set_field_info (
        "name",        "<b>Name of this ".$self->get_gizmo_name()."</b>",                                  1,
        "description", "<b>Description</b>",                                0,
        "url",         "<b>URL</b> <i>e.g. http://www.yourcompany.com</i>", 0,
        "c3",          "<b>Open URL in new window</b>", 0,
        "c4",          "<b>Do not show URL</b>", 0,
        "item_attachment_file",   "<b>Attachment file</b>",                      0,
        "keywords",    "<b>Keywords and Synonyms</b>",                      0,
        "cool",        "<b>Star this ".$self->get_gizmo_name()."</b>",                             0,
              );

    $self->set_field_type( 'item_attachment_file', Metadot::UploadsManager::UPLOAD_TYPE_USER_SELECTED() );

    $self->{display_wrapped} = { show => 1,
                   };
}


sub get_upload_filename_icon_and_url {
    my $self = shift;
    return $self->SUPER::get_upload_filename_icon_and_url( {
        id => $self->id(),
        isa => $self->is_a(),
        field_name => 'item_attachment_file',
    });
}

sub get_upload_filename_iconsrc_and_url {
    my $self = shift;
    $self->debug_msg(4, "in get_upload_filename_iconsrc_and_url");

    return $self->SUPER::get_upload_filename_iconsrc_and_url( {
        id => $self->id(),
        isa => $self->is_a(),
        field_name => 'item_attachment_file',
    });
}

###################################
sub show {
    my $self = shift;

    my $name            = $self->get_name();
    my $description     = $self->get_description();
    my $new_window      = $self->get_c3();
    my $url             = $self->get_url();
    my $keywords        = $self->get_keywords();

    my ( $file, $file_icon ) = $self->get_upload_filename_icon_and_url();
    my $cool_icon       = $self->get_cool_icon();
    my $longdescription = $self->get_long_description();
    my $html            = '';
    my $blank;

    $html .= ($url) ? "<a href='".$self->make_url($url)."' $blank>$name</a>\n" : "<b>$name</b>\n";
    $html .= $cool_icon;
    $html .= "<br />$description\n" if ($description);
    $html .= "<p />$longdescription\n" if ($longdescription);
    $html .= "<br /><br />File attached: $file $file_icon\n" if ($file);
    $html .= "<br /><br />Synonyms: $keywords\n" if ($keywords);

    return $html;
}


sub show_summary {
    my $self        = shift;

    my $name        = $self->get_name();
    my $description = $self->get_description();
    my $new_window  = $self->get_c3();
    my $suppress    = $self->get_c4();
    my $url         = $self->get_url();

    my ( $file, $file_icon ) = $self->get_upload_filename_icon_and_url();

    my $cool_icon   = $self->get_cool_icon();
    my $buttons     = $self->get_buttons();

    my $html        = '';
    my $i = 0 ;
    my $maxiLen = 50 ;      # aju- make a little smaller (need more general soln)
    my $nameLen = length($name) ;
    my $linkLen = length($url) ;
    my $coolLen = ($cool_icon ne '') ? 10 : 0 ;
    my $fileLen = ($file_icon ne '') ? 10 : 0 ;
    my $fullLen = $nameLen + $linkLen + $coolLen + $fileLen ;

    my $flgUrl = (defined($url) && $url ne '' && $url ne "http://") ;
    my $blank;
    my $popup_img;

    $html .= "<table border=\"0\" cellspacing=\"0\" width=\"100%\">";

    $html .= "<tr valign='BOTTOM'>";

    if($buttons ne '') { $html .= "<td nowrap='nowrap'>$buttons</td>"; $i+=1; }

    $html .= "<td nowrap='nowrap'>";
    if ($flgUrl) {
        if( $new_window ){
            $blank = 'target="_blank"';
            $popup_img = '<img src="' . $PARAMS->{'imageslocation'} . '/new_window.gif" border="0" align="bottom"  alt="Opens new window"/>';
        }
        $html .= "<a href=\"".$self->make_url($url)."\" $blank class='centerNewsTitle'>$name</a> ";
        $html .= "<a href=\"".$self->make_url($url)."\" $blank class='centerNewsTitle'>$popup_img</a>";
        $html .= "</td>"; $i+=1;

    } else {
        $html .= "<b>$name</b>"; $i+=1 ;
        $html .= "</td>";

    } ;

    if($cool_icon ne '') { $html .= "<td align='LEFT' valign='BOTTOM'>$cool_icon</td>"; $i+=1 ; }

    if($file_icon ne '') { $html .= "<td nowrap='nowrap' align='LEFT' valign='BOTTOM'>$file_icon</td>"; $i+=1 ; }

    unless( $suppress ){
        if ($flgUrl) {
           $html .= "<td class='centerNewsLink' nowrap='nowrap'>";
           $html .= "-&nbsp;" . substr($url,0, ($maxiLen >= $fullLen) ? $fullLen : $maxiLen - $nameLen - $coolLen - $fileLen);
           if($fullLen>$maxiLen) { $html .= "..." ;} ;
           $html .= "</td>"; $i+=1 ;
        }
    }



    $html .= "<td width=\"100%\">&nbsp;</td>"; $i+=1;

    $html .= "</tr>\n";

    if($description ne '') {
        $html .= "<tr valign='TOP'>";
        $html .= "<td colspan='$i' class='centerNewsItem'>";
        $html .= "$description";
        $html .= "</td>";
        $html .= "</tr>\n";
    }

    if( $PARAMS->{items_are_rateable} && $self->can('www_rate') ) {

    $html .= "<tr valign='TOP'><td colspan=$i>" . $self->show_rate_link() . "</td></tr>\n";
    }

    $html .= "</table>\n";

    return $html;
}


sub show_line {
    my $self        = shift;
    my $name        = $self->get_name();
    my $description = $self->get_description();
    my $url         = $self->get_url();

    my ( $file, $file_icon ) = $self->get_upload_filename_icon_and_url();

    my $cool_icon   = $self->get_cool_icon();
    my $buttons     = $self->get_buttons();

    my $html        = '';

    $html .= "$buttons $cool_icon $file_icon";
    if (defined($url) && $url ne '' && $url ne "http://") {
        $html .= "<a href=\"".$self->make_url($url)."\">$name</a>\n";
    } else {
        $html .= $name . "\n";
    }
    $html .= $description;

    return $html;
}


sub get_gizmo_name {
    return 'Attachment / Link';
}


BEGIN {

    use Utility;

    if( Utility->is_module_available('ItemExtensions') ) {
#   eval { require ItemExtensions };
    }
}

1;

__END__

=head1 NAME

Gizmo::Item - can either create a new Item (returing a pointer to the new object) or control
    the way existing ones are displayed (returning the formatted HTML).


=head1 SYNOPSIS

    use Gizmo::Item;

    $item = Item->new ($id);
    $html = $item->show ();
    $html = $item->show_as_bullet ();
    $html = $item->show_as_dash ();
    $version = Item->get_version();


=head1 DESCRIPTION

Item is the basic building block for the Portal.   Adding an item allows the
user to create a link to a URL, and the user may also attach a file to an
item.


=head1 Methods

Calling the new function instantiates a new item or retrive its persistent info
from the backend database.

Class method to retrieve the version number of the gizmo

    $version = Item->get_version();



The remaining functions control how an Item is displayed:

    $item->show() displays an item.
    $item->show_as_bullet() displays the item as an item in a  bulleted list.
    $item->show_as_dash() show the item with a dash in front of its name.

=cut
