###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
#
#
############################################################################
package JavascriptHTML;

use strict;
use GizmoBuilder;
use vars qw(@ISA);
use CGI qw/escapeHTML unescapeHTML/;

my $version = "1.0";

sub get_version {
    return $version;
}

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS);

use constant NOTE=>'<ul><li><b><i>Name</i></b> and <b><i>description</i></b> fields will not appear in normal viewing mode</li>'
    .'<li>The script code will only be executed in normal viewing mode, or by clicking on the "Run" button,<br />'
    .'and not in editing mode, therefore, you must disable editing to see the result of the code</li></ul>';

use constant SAMPLE=>'<b>The following example code displays an alert box when the page loads:<br /><br /></b>'
    . '&lt;script language="JavaScript"&gt;<br />&nbsp;&nbsp;&nbsp;&nbsp;alert("I am alive!");<br />'
    . '&lt;/script&gt;<br />';

@ISA=qw(GizmoBuilder);


sub customize_permissions {
    my $class = shift;
    my $AC = shift;

    $class->SUPER::customize_permissions($AC);

    $AC->add_op('MOD',  'run_it');
}


sub get_gizmo_name {
    return "HTML/Javascript";
}

sub gizmo_list_decorator {
    return 'HTML::Decorator::NullDecorator';
}

sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

    $self->set_field_info (
        'name', NOTE.'<br /><b>Name</b>', 1,
        'description', '<b>Description</b>', 0,
        't1',   SAMPLE.'<br /><b>Code</b>', 1,
    );

    $self->set_field_type( 't1', 'variabletext', {rows=>35, cols=>120, html_only=>1} );
}


sub get_script_code {
    my $self = shift;
    return $self->get_t1();
}

sub show_summary {
    my $self  = shift;
    my $iid             = $self->get_iid();
    my $name            = $self->get_name();
    my $description     = $self->get_description();
    my $script      = $self->get_script_code();
    my $buttons           = $self->get_buttons();
    my $run_it          = $FORM{'run_it'};
    my $html            = "\n";

    ## Only show name and description when in edit mode
    if ($USER->is_logged_in() && ($SESSION->admin_mode()) && $self->is_allowed_to_do('run_it', $USER)) {
       $html = "<p><b>$buttons <b><font size=\"3\">&lt; &gt;</font></b> $name</b>\n";
       $html .= "<br />$description\n" if ($description);
       $html .= "</p>\n";

       unless( $run_it ){
          my $script_size = length $script;

          $script = escapeHTML($script);

          if( $script_size > 100 ){
          $script = substr($script,0,100);
              $script .= "<br />...\n";
          }

          $script =~ s/\n/<br \/>/g;

       }

       $html .= "<pre><i><font face=\"courier\">$script</font></i></pre>";

    }
    else{

       $html .= "$script\n";
    }
    return $html;
}

sub show {
    return show_summary;
}

sub exec_show {
    my $self = shift;
    my $context = shift;
    return $self->show();
}

sub exec_show_summary {
    my $self = shift;
    my $context = shift;
    return $self->show_summary();
}


# add a 'run code' button.
sub get_buttons {
    my $self = shift;
    my $id = $self->get_id();
    my $buttons;

    if (defined $id) {
        $buttons .= $self->SUPER::get_buttons( {add_panel=>$self->get_run_button()});
        } else {
        $buttons = $self->SUPER::get_buttons();
    }
    return $buttons;
}

sub get_run_button {
    my $self = shift;
    my $parent = $self->get_navigation_parent();
    return unless defined($parent);

    my $parent_id = $parent->get_id();
    my $parent_isa = $parent->get_is_a();

    my $id = $self->get_id();
    my $isa = $self->get_is_a();

    my $result;
    my $run_it          = $FORM{'run_it'};
    my $flag = 1;
    my $image = "run.jpg";

    my $win = "";
    my $href;

    $href    = "?id=$parent_id&isa=$parent_isa&op=show&run_it=$flag";
    $win = "target=\"_blank\"";

    if( $run_it ){
        $flag = 0;
    }

    if ($SESSION->admin_mode()) {
        if ($href) {
            my $img_src = "$PARAMS->{'imageslocation'}/obj/$image";
            $result = "<a href=\"$href\" $win><img src=\"$img_src\" class=\"icon\" border=\"0\" /></a>";
        }
    }

    return $result;
}

1;
