###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package JobRequest;

use strict;
use CGI;
use CGI qw(escapeHTML);
use Metadot qw($USER %FORM $SESSION $PARAMS $DISPLAY);

use Gizmo;
use GizmoBuilder;
use Time::Local;
use vars qw(@ISA);

@ISA=qw(GizmoBuilder);
my $version = "1.0";

sub get_version {
    return $version;
}

use Gizmo::JobRequest::JobModule;
my $CHILD_TYPE = 'JobModule';


sub customize_permissions {
    my $class = shift;
    my $AC = shift;

    $class->SUPER::customize_permissions($AC);

    $AC->add_op('MOD','add_child');
    $AC->add_op('MOD','create_child');

    $AC->add_bundle('PUBLICVIEW', 'View customer report', '0', '0');
    $AC->add_op('PUBLICVIEW','view_customer_report');
}




{
    my $search_params = {
             searchable_fields => [ qw( name description keywords) ],
             child_isa => 'JobModule',
             child_searchable_fields => [ qw( name description c2 keywords ) ],
            };

    sub get_search_params {
    my $self = shift;

    return $search_params;
    }
}


####################################
sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

    $self->set_field_info (
          "name",        "<b>".get_gizmo_name()." Name</b>",              1,
          "description", "<b>Description</b>",                0,
     "c1", "Project manager name",1,
     "c2", "Project manager email",1,
      "c3", "<b>Priority (normal, rushed, top priority)</b>", 0,
      "c4", "<b>Status (New, open, under construction, post-processing,closed, action required, cancelled </b>", 0,
          "t1", "<b>Customer information</b> (company, name, email address, phone#)", 0,
          "d1",    "<b>Date created</b>",      0,
          "d2",    "<b>To be delivered on or before</b>",      0,
          "t2", "<b>Remarks</b> (internal comments, won't be visible by customer)", 0,
          "t3", "<b>PM comments</b>", 0,
          "t4", "<b>Action needed (if any)</b>", 0,
         "keywords",    "<b>Keywords and Synonyms</b>",      0,
         "c5",    "<b>Magic words</b> (e.g. eat!steak)",      0,

        );


    $self->{display_wrapped} = { show => 1, };
    $self->{sort_field} = $SESSION->get_value('jobmodule_sort_field') || JobModule->due_date_field();
    $self->{sort_order} = $SESSION->get_value('jobmodule_sort_order') || -1;
}

#####################################################
######                                        #######
######            FRONT-END METHODS           #######
######                                        #######
#####################################################
sub www_view_customer_report {

  my $self        = shift;
  my $name        = $self->get_name();
  my $description = $self->get_description();
  my $buttons     = $self->get_buttons();
  my $id              = $self->get_iid();
  my $isa             = $self->get_is_a();
  my $pm_name         = $self->get_c1();
  my $pm_email        = $self->get_c2();
  my $priority        = $self->get_c3();
  my $status          = $self->get_c4();
  my $date_created    = $self->get_d1();
  my $delivery_date   = $self->get_d2();
  my $remarks         = $self->get_t2();
  my $cust_info       = $self->get_t1();
  my $pm_comments     = $self->get_t3();
  my $action_needed   = $self->get_t4();
  my $magic_words     = $self->get_c5();

 $self->debug_msg(3, "JobRequest::view_customer_report()");

  ### test if the url contains the magic words and if not
  ### prevent access to the report
  my $form_magic_words = $FORM{magic_words};
  if ($form_magic_words ne $magic_words){
    $self->debug_msg(5, "JobRequest::view_customer_report($form_magic_words /  $magic_words");
    $DISPLAY->print_click_back("Nice try...", "Sorry you need to know the magic words to get access to this report");
    return;
  }

  my $str_delivery_date = Utility->format_datetime($delivery_date, 'M e, Y');
  my $str_date_created = Utility->format_datetime($date_created, 'M e, Y');

    my $html = "";

    $html .= "<html><title>Project summary for $name</title>\n";
    $html .= "<body><h1>Project summary for $name</h1>\n";
  $html .= "<i>Report generated on " . `date` . "</i><br /><br />\n";


  $html .= "<b>### Priority:</b> $priority<br />\n";
  $html .= "<b>### Status:</b> $status (*)<br />\n";
  $html .= "<b>### Delivery:</b> on or before $str_delivery_date<br /><br />\n";
    $html .= "<b>Project manager: </b>" . $pm_name . ", " . $pm_email . "<br />\n";

    $html .= "<b>Project description</b>: $description<br />\n";
    $html .= "<b>Date created</b>: $str_date_created <br />\n";

    $html .= "<b>Customer information:</b> $cust_info <br />\n";


    if($pm_comments){
      $html .= "<b>Project manager's commments:</b> $pm_comments<br />\n";
    }

    if($action_needed){
      $html .= "<b>Action needed:</b> $action_needed<br />\n";
    }

  $html .= "<h2>Project modules to be delivered</h2>\n";

    my $bgcolor = 'bgcolor="#FFFFFF"';

    my %args;
    $args{order_by} = $CHILD_TYPE->due_date_field().' ASC,'
                    . $CHILD_TYPE->priority_field().' DESC,'
                    . $CHILD_TYPE->assigned_to_field().' ASC,'
                    . 'name ASC';
    #$args{where_clause} = $CHILD_TYPE->status_field().' != '.$CHILD_TYPE->get_status_value('complete');
  $args{where_clause} = "1=1";

    my @children = $self->_get_my_children(%args);
    my $summary = $self->_generate_summary();

    # if childs, display row data message
    if (@children) {
        $html .= <<EOM
       <p>
        <table cellspacing="0" cellpadding="0" border="0" bgcolor="#CCCCCC" width="450"><tr><td>
        <table cellspacing="1" cellpadding="3" border="0" width="100%">
          <tr valign='bottom'>
        <th class='smaller' nowrap="nowrap" align='left' $bgcolor>Title &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class='smaller' nowrap="nowrap" align='left' $bgcolor>Due Date</th>
        <th class='smaller' nowrap="nowrap" align='center' $bgcolor>Assigned To</th>
        <th class='smaller' nowrap="nowrap" align='center' $bgcolor>Priority</th>
        <th class='smaller' nowrap="nowrap" align='center' $bgcolor>Status</th>
      </tr>
EOM
;
        my $col_count = 4;

        foreach my $child (@children) {
            $html .= $self->_generate_table_row_summary(
                object      => $child,
            )."\n";
        }
        #my $detail_link = "<a class='smaller' href=\"".$self->make_url("index.pl?iid=$id&amp;isa=$isa&amp;op=show")."\">View this entire jobmodule list</a>";

        my $cols_left = 1;
        my $cols_right = $col_count - $cols_left;
        $html .= <<EOM
      <tr valign="baseline">
        <td class='smaller' align='left' colspan=$cols_left>
          &nbsp;<br />
        </td>
        <td align='right' colspan=$cols_right>
          &nbsp;
        </td>
      </tr>
    </table>
    </td></tr></table></div>
EOM
;
    } else {
        # no childs, just display summary
        $html .= "<span class=\"smaller\">&nbsp;</span>\n";
    }

    $html .= "</p>\n";

$html .= <<EOT;
<h2>(*) What does my project status message mean?</h2>
<ul>
<li>New: Your order has been received but has not begun processing. In most cases, Metadot can begin processing your order in approximately 1 day from the time you place your order.
<li>Open: Your project has begun processing and a project manager has been assigned. Once your order is open, Metadot will provide you the estimated dates that your project modules will delivered and schedule a kick-off teleconference meeting.
<li>Under construction: the project is scheduled in our project queue and the kickoff meeting has been done. Delivery will occur module by module if possible or all at once at the end of the project.
<li>Delivered: All items on your project have been delivered by Metadot.
<li>Post-processing: Your project has been delivered and the Metadot team will now discuss what went well and what went not so well during the project. You will receive a customer satisfaction survey so we know how satisfied you are. We also archive the source code and project documents in the Metadot project repository. This is important for maintenance purpose and also for continuous quality improvement.
<li>Completed: After delivery and post-processing we file everything and move on to maintenance/support mode.
</ul>
<p />Other statuses that can happen:
<ul><LI>Cancelled: Your project has been cancelled.
<li>Action Required: Your order/project has been received but it cannot be processed due to incomplete payment or lack of other important information. Once you have provided this information, Metadot can continue processing it and will provide you with the estimated dates your project will be delivered.
</ul><br /><br />
EOT
;

    $DISPLAY->print_http_header();
    print "$html \n</body></html>";
}

sub show {
    my $self = shift;

    my $name        = $self->get_name();
    my $description = $self->get_description();
    my $keywords    = $self->get_keywords();
    my $id          = $self->get_iid();
    my $isa         = $self->get_is_a();
    my $dt          = $FORM{dt} || $self->{$self->{current_datetime}};

    my $magic_words = $self->get_c5();

    $self->debug_msg(3, "JobRequest::show()");
    my $editable = $self->is_allowed_to_do('modify', $USER);

    my $html .= "<b>$name</b>";
    $html .= "<b>:</b> $description\n" if ($description);

    # show the jobmodules
    my $bgcolor = 'bgcolor="#FFFFFF"';
    my $col_count = 0;

    # generate the headers that allow me to sort each
    $self->_process_sort_input();
    my $complete_header = $self->_generate_sort_header(
        text            => 'Complete',
        sort_field      => JobModule->status_field(),
        default_order   => 1,
    );
    my $title_header = $self->_generate_sort_header(
        text            => 'Title',
        sort_field      => 'name',
        default_order   => 1,
    );
    my $due_date_header = $self->_generate_sort_header(
        text            => 'Due Date',
        sort_field      => JobModule->due_date_field(),
        default_order   => 1,
    );
    my $status_header = $self->_generate_sort_header(
        text            => 'Status',
        sort_field      => JobModule->status_field(),
        default_order   => 1,
    );
    my $priority_header = $self->_generate_sort_header(
        text            => 'Priority',
        sort_field      => JobModule->priority_field(),
        default_order   => -1,
    );
    my $assigned_to_header = $self->_generate_sort_header(
        text            => 'Assigned To',
        sort_field      => JobModule->assigned_to_field(),
        default_order   => 1,
    );

    $html .= <<EOM
    <p />
    <table cellspacing="0" cellpadding="0" border="0" bgcolor="#777777" width="100%"><tr><td>
    <table cellspacing="1" cellpadding="3" border="0" width="100%">
      <tr valign='bottom'>
        <th nowrap="nowrap" align='center' $bgcolor>$complete_header&nbsp;</th>
        <th nowrap="nowrap" align='left'   $bgcolor>$title_header &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th nowrap="nowrap" align='left'   $bgcolor>$due_date_header</th>
        <th nowrap="nowrap" align='center'   $bgcolor>$assigned_to_header</th>
        <th nowrap="nowrap" align='center'   $bgcolor>$status_header</th>
        <th nowrap="nowrap" align='center'   $bgcolor>$priority_header</th>
      </tr>
EOM
;
    $col_count += 6;

    my @children = $self->_get_my_children();
    foreach my $child (@children) {
        $html .= $self->_generate_table_row(
            object      => $child,
            bgcolor     => $bgcolor,
            editable    => $editable,
        )."\n";
    }

    # if no childs, display a message
    unless (@children) {
        $html .= "<td colspan=$col_count align='center' $bgcolor><br /><i>- There are no jobmodules in this list -</i><br /><br /></td>\n";
    }

    $html .= <<EOM
    </table>
    </td></tr></table>
EOM
;

    # show the 'add jobmodule' button
    $html .= $self->_render_add_jobmodule();
    $html .= "<div style='text-align:right; font-size:small'><a style=\"font-weight: bold;\" href=\"".$self->make_url("index.pl?iid=$id&amp;isa=$isa&amp;op=view_customer_report&amp;magic_words=$magic_words")."\">[View customer report]</a><br /></div>";

}


sub www_add_child {

  my $self = shift;
  my $errormsg = shift;
  my $child_type = shift;
  my $child_object = shift;

  $self->debug_msg(3, "JobRequest::www_add_child()");

  my $child_isa = $FORM{child_type} || $child_type;
  my $gizmo_name = $child_isa->get_gizmo_name();

  my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
  # we re-instanciate the object because we want a brand new one
  # unless there is an error message (we want to keep his data)
  #my $child = $child_isa->new({ parent_id => $parentid }) unless ($errormsg);
  my $child = $errormsg && defined($child_object) ? $child_object : $child_isa->new({ parent_id => $parentid });

    $errormsg = $errormsg ?
                "<p /><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p />\n" :
                "";
  my $form = $child->show_edit_form("Adding new $gizmo_name");
  my $content =
Components->color_palette_code()."
<form name=fm action='index.pl' method='post' ENCTYPE=\"multipart/form-data\">
<input type='hidden' name='isa' value='JobRequest'>\n
<input type='hidden' name='iid' value='$parentid'>\n
<input type='hidden' name='op' value='create_child'>\n
<input type='hidden' name='child_type' value=\"$CHILD_TYPE\">\n
$errormsg
$form
<table align='right'><tr><td><input type='submit' value='Create'></td></tr></table>
<br />&nbsp;
</form>
";

  $child->print_form($content, "Adding new $gizmo_name");
}

sub www_create_child {
  my $self = shift;
  my $child_type = shift || $CHILD_TYPE;

  $self->debug_msg(3, "JobRequest::www_create_child()");

  my $child_isa = $FORM{child_type} || $child_type;

  my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
  # we re-instanciate the object because we want a brand new one
  my $child = $child_isa->new({
                         uid => $USER->uid(),
                         parent_id => $parentid,
                        });

  # this method should ALWAYS be overridden in derived classes.
  $child->process_edit_form();
}


###################################
sub show_summary {
    my $self        = shift;
    my $name        = $self->get_name();
    my $description = $self->get_description();
    my $buttons     = $self->get_buttons();
    my $id              = $self->get_iid();
    my $isa             = $self->get_is_a();
    my $pm_name         = $self->get_c1();
    my $pm_email        = $self->get_c2();
    my $priority        = $self->get_c3();
    my $status          = $self->get_c4();
    my $magic_words     = $self->get_c5();
    my $date_created    = $self->get_d1();
    my $delivery_date   = $self->get_d2();
    my $pm_comments     = $self->get_t3();
    my $remarks         = $self->get_t2();
    my $cust_info       = $self->get_t1();

    $self->debug_msg(3, "JobRequest::show_summary()");

    my $html = "<p />\n";

    $html .= "$buttons <a style=\"font-weight: bold;\" href=\"".$self->make_url("index.pl?iid=$id&amp;isa=$isa&amp;op=show")."\">$name</a><br />";
    $html .= "<div style='text-align:right; font-size:small'><a style=\"font-weight: bold;\" href=\"".$self->make_url("index.pl?iid=$id&amp;isa=$isa&amp;op=view_customer_report&amp;magic_words=$magic_words")."\">[View customer report]</a><br /></div>";



    $html .= "<b>### Priority: " . $priority . " ### Status: " . $status . "\n";
    $html .= " ### Delivery on or before $delivery_date</b><br /><br />\n";
    $html .= "<b>Project manager: </b>" . $pm_name . ", " . $pm_email . "<br />\n";

    $html .= "<b>Project description</b>: $description<br />\n";
    $html .= "<b>Date created</b>: $date_created <br />\n";

    $html .= "<b>Customer information:</b> $cust_info <br />\n";
    $html .= "<b>Remarks</b>: $remarks<br />\n";


    my $bgcolor = 'bgcolor="#FFFFFF"';

    my %args;
    $args{order_by} = $CHILD_TYPE->due_date_field().' ASC,'
                    . $CHILD_TYPE->priority_field().' DESC,'
                    . $CHILD_TYPE->assigned_to_field().' ASC,'
                    . 'name ASC';
    #$args{where_clause} = $CHILD_TYPE->status_field().' != '.$CHILD_TYPE->get_status_value('complete');
    $args{where_clause} = "1=1";

    my @children = $self->_get_my_children(%args);
    my $summary = $self->_generate_summary();

    # if childs, display row data message
    if (@children) {
        $html .= <<EOM
      <p>
        <div class='smaller'><br />Module list:</div>
        <table cellspacing="0" cellpadding="0" border="0" bgcolor="#CCCCCC" width="450"><tr><td>
        <table cellspacing="1" cellpadding="3" border="0" width="100%">
          <tr valign='bottom'>
        <th class='smaller' nowrap="nowrap" align='left' $bgcolor>Title &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class='smaller' nowrap="nowrap" align='left' $bgcolor>Due Date</th>
        <th class='smaller' nowrap="nowrap" align='center' $bgcolor>Assigned To</th>
        <th class='smaller' nowrap="nowrap" align='center' $bgcolor>Priority</th>
        <th class='smaller' nowrap="nowrap" align='center' $bgcolor>Status</th>
      </tr>
EOM
;
        my $col_count = 4;

        foreach my $child (@children) {
            $html .= $self->_generate_table_row_summary(
                object      => $child,
            )."\n";
        }
        my $detail_link = "<a class='smaller' href=\"".$self->make_url("index.pl?iid=$id&amp;isa=$isa&amp;op=show")."\">View this entire jobmodule list</a>";

        my $cols_left = 1;
        my $cols_right = $col_count - $cols_left;
        $html .= <<EOM
      <tr valign="baseline">
        <td class='smaller' align='left' colspan=$cols_left>
          $summary<br />
        </td>
        <td align='right' colspan=$cols_right>
          $detail_link
        </td>
      </tr>
    </table>
    </td></tr></table></div>
EOM
;
    } else {
        # no childs, just display summary
        $html .= "<span class=\"smaller\">$summary</span>\n";
    }

    $html .= "</p>\n";
    return $html;
}


sub _generate_summary {
    my $self = shift;
    my $html;

    my $late_count     = $self->count_late_children;
    my $complete_count = $self->count_complete_children;
    my $child_count    = $self->count_all_children;
    my $child_plural    = ($child_count==1)? '' : 's';
    $self->debug_msg(3, "JobRequest::_generate_summary");
    $html = "$child_count jobmodule$child_plural ($late_count late, $complete_count complete)\n";
    return $html;
}

sub _generate_sort_header {
    my $self = shift;
    my %args = @_;
    my $text = $args{text};
    my $sort_field = $args{sort_field};
    my $sort_order = $args{default_order} || 1;
    my $result;

    my $iid = $self->id();
    my $isa = $self->is_a();
    my $op  = $FORM{op} || 'show';

    $self->debug_msg(3, "JobRequest::_generate_sort_header()");
    # determine what image to use for indicate sort order
    my $sort_image;
    if ($self->sort_field() eq $sort_field) {
        if ($self->sort_order() >= 0) {
            $sort_image = "<img src=\"$PARAMS->{imageslocation}/obj/triangle_up.gif\" border=\"0\">";
        } else {
            $sort_image = "<img src=\"$PARAMS->{imageslocation}/obj/triangle_down.gif\" border=\"0\">";
        }
        $sort_order = -1 * $self->sort_order(); # toggle the sort
    } else {
        $sort_image = '';
    }

    $result = "<a href=\"index.pl?iid=$iid&amp;isa=$isa&amp;op=$op&amp;jobmodule_sort_order=$sort_order&amp;jobmodule_sort_field=$sort_field\">$text</a> $sort_image";
    return $result;
}

sub _process_sort_input {
    my $self = shift;
    $self->debug_msg(3, "JobRequest::_process_sort_input()");
    if (defined $FORM{jobmodule_sort_field}) {
        $self->sort_field($FORM{jobmodule_sort_field});
    }
    if (defined $FORM{jobmodule_sort_order}) {
        $self->sort_order($FORM{jobmodule_sort_order});
    }
}

sub _generate_table_row {
    my $self = shift;
    my %args = @_;
    my $html = '';

    my $object = $args{object};
    my $iid = $object->id();
    my $isa = $object->is_a() || $CHILD_TYPE;
    my $name = $object->name();
    my $due_date = $object->due_date();
    my $complete = $object->is_complete();
    my $priority = $object->priority_name() || '-';
    my $assigned_to = $object->assigned_to() || '-';
    my $status = $object->status_name() || '-';

    my $editable = $args{editable};
    $self->debug_msg(3, "JobRequest::_generate_table_row");
    # generate the complete_box check
    my $complete_box;
    if ($complete) {
        $complete_box = "<img src=\"$PARAMS->{imageslocation}/checked.gif\" border=\"0\">";
    } else {
        $complete_box = "<img src=\"$PARAMS->{imageslocation}/unchecked.gif\" border=\"0\">";
    }

    # if the user can edit this, we'll make the complete_box an active element
    if ($editable) {
        $complete_box = "<a href=\"index.pl?iid=$iid&amp;isa=$isa&amp;op=toggle_complete\">$complete_box</a>";
    }

    # format the due date
    my $date_string="";
    if ($due_date) {
        $date_string = Calendar->format_datetime($due_date, 'M e, Y');
        if ($object->is_late()) {
            $date_string = "<font color=red>$date_string</font>";
        }
    } else {
        $date_string = '&nbsp;';
    }

    # generate a link for the jobmodule, using the name text
    my $name_string = "<a href=\"index.pl?iid=$iid&amp;isa=$isa&amp;op=show\">$name</a>&nbsp;".$object->show_panel();

    my $bgcolor = $args{bgcolor} || 'bgcolor="#FFFFFF"';

    $html .= <<EOM
        <tr valign="baseline">
          <td align='center' $bgcolor>$complete_box</td>
          <td align='left' $bgcolor>$name_string</td>
          <td align='left' $bgcolor nowrap>$date_string</td>
          <td align='center' $bgcolor>$assigned_to</td>
          <td align='center' $bgcolor>$status</td>
          <td align='center' $bgcolor>$priority</td>
        </tr>
EOM
;


}

sub _generate_table_row_summary {
    my $self = shift;
    my %args = @_;
    my $html = '';

    my $object = $args{object};
    my $iid = $object->id();
    my $isa = $object->is_a() || $CHILD_TYPE;
    my $name = $object->name();
    my $due_date = $object->due_date();
    my $complete = $object->is_complete();
    my $priority = $object->priority_name() || '-';
    my $assigned_to = $object->assigned_to() || '-';
    my $status = $object->status_name() || '-';

    # format the due date
    my $date_string;
    if ($due_date) {
        $date_string = Calendar->format_datetime($due_date, 'M e, Y');
        if ($object->is_late()) {
            $date_string = "<font color=red>$date_string</font>";
        }
    } else {
        $date_string = '&nbsp;';
    }

    my $bgcolor = $args{bgcolor} || 'bgcolor="#FFFFFF"';

    # generate a link for the jobmodule, using the name text
    # my $name_string = "<a href=\"index.pl?iid=$iid&amp;isa=$isa&amp;op=show\" style=\"font-size: 100%;\">$name</a>&nbsp;".$object->show_panel();
    my $name_string = $name;


    $html .= <<EOM
        <tr valign="baseline">
          <td class='smaller' nowrap="nowrap" align='left' $bgcolor>$name_string</td>
          <td class='smaller' nowrap="nowrap" align='left' $bgcolor nowrap>$date_string</td>
          <td class='smaller' nowrap="nowrap" align='center' $bgcolor>$assigned_to</td>
          <td class='smaller' nowrap="nowrap" align='center' $bgcolor>$priority</td>
          <td class='smaller' nowrap="nowrap" align='center' $bgcolor>$status</td>
        </tr>
EOM
;


}

sub show_line {
    my $self        = shift;
    return $self->show_summary();
}

## creates link for adding event
sub _render_add_jobmodule {
    my $self = shift;
    my $html = '';

    my $op       = 'add_child';

    ## if user is not allowed to add event
    unless ($self->is_allowed_to_do('add_child', $USER)) {
        return "&nbsp;";
    }

    my $id   = $self->id;
    my $isa  = $self->is_a();

    $html .= <<EOM
    <FORM name="form_add_jobmodule" action="index.pl" method="post">
      <INPUT type=hidden name=iid value="$id">
      <INPUT type=hidden name=isa value="$isa">
      <INPUT type=hidden name=op  value="$op">
      <INPUT type=hidden name=child_type value=$CHILD_TYPE>
      <INPUT type=submit name=add_child value="Add a jobmodule">
    </FORM>
EOM
;

    return $html;
}

sub _get_my_children {
    my $self = shift;
    my %args = @_;
    my $child_type = $args{child_type} || $CHILD_TYPE;
    my @results;

    # determine the sort
    my $order_by = $args{order_by};

    if (!$order_by && $self->sort_field()) {
        $order_by = $self->sort_field();
        if ($self->sort_order() >= 0) {
            $order_by .= ' asc';
        } else {
            $order_by .= ' desc';
        }
    }
    my $id = $self->id;
    my $where_clause = "parentiid=$id and isa='$child_type'";
    if ($args{where_clause}) {
        $where_clause .= " and $args{where_clause}";
    }

    my ($childs)=DBAccess->sqlSelectMany("iid", "instance", $where_clause, "order by $order_by");
    while(my ($iid)=$childs->fetchrow()) {
      my $e = $CHILD_TYPE->restore($iid);
      push @results, $e;
    }
    return @results;
}

# get count of all children
sub count_all_children {
    my $self = shift;
    return $self->_count_children_where('1=1');
}

# get count of children whose due dates are past, and are not complete
# late is computed if 24 hours past first second of due date
sub count_late_children {
    my $self = shift;
    return $self->_count_children_where($CHILD_TYPE->due_date_field.'<'.$CHILD_TYPE->late_time.' AND '.$CHILD_TYPE->status_field.'!='.$CHILD_TYPE->get_status_value('complete'));
}

# get count of children whose status is complete
sub count_complete_children {
    my $self = shift;
    return $self->_count_children_where($CHILD_TYPE->status_field.'='.$CHILD_TYPE->get_status_value('complete'));
}

sub _count_children_where {
    my $self = shift;
    my $where_clause = shift || '1=1';

    my ($count) = DBAccess->sqlSelect('count(iid)', 'instance', "isa='$CHILD_TYPE' AND parentiid=".$self->id." AND $where_clause");
    return $count;
}

sub _show_create_success {
    my $self = shift;

#   my $isa = lc($self->is_a());
    my $isa = $self->is_a();
    my $parentid = $self->parent_id();

    $self->two_choices("<Big>New " . $self->get_gizmo_name() .
                       " added!</big><br /><br />What do you want to do?",
                       "confirm",
                       "Add another " . $self->get_gizmo_name(),
                       "index.pl?child_type=$isa&isa=".($FORM{isa} || 'Category')."&iid=$parentid&op=add_child&app=",
                       "Enough for now!",
                       "index.pl?id=".$self->id());

}

sub _show_edit_success {
    my $self = shift;

    if ($FORM{op} =~ m/delete/) {
        $self->md_redirect("index.pl?iid=".$self->parent_id);
    } else {
        $self->md_redirect("index.pl?iid=".$self->id);
    }
}

sub gizmo_list_header {
    my $class = shift;
    my $cat_name = shift;
    my $out = '';
    return $out;
}

sub gizmo_list_decorator {
    return 'HTML::Decorator::NullDecorator';
}

## overrides parent's delete function in order to delete events
sub delete{
    my $self = shift;
    $self->debug_msg(3, "JobRequest::delete()");

    foreach my $child ($self->_get_my_children()) {
        $child->delete();
    }

    # delete from instance
    $self->SUPER::delete();
    return 1;
}

sub get_gizmo_name {
    return 'Job Request';
}

sub sort_field {
    my $self = shift;
    my $input = shift;

    if (defined $input) {
        $self->{sort_field} = $input;
        $SESSION->set_value('jobmodule_sort_field', $input);
    }
    return $self->{sort_field};
}

sub sort_order {
    my $self = shift;
    my $input = shift;

    if (defined $input) {
        $self->{sort_order} = $input;
        $SESSION->set_value('jobmodule_sort_order', $input);
    }
    return $self->{sort_order};
}

1;

__END__

=head1 NAME

Gizmo::JobRequest - can either create a new JobRequest (returing a pointer to the new object) or control the way existing ones are displayed (returning the formatted HTML).


=head1 SYNOPSIS

    use Gizmo::JobRequest;

    $calendar = JobRequest->new ($id);
    $html = $calendar->show ();
    $html = $calendar->show_as_bullet ();
    $html = $calendar->show_as_dash ();
    $html = $calendar->show_edit_form ();
    $html = $calendar->show_in_column ();
    $html = $calendar->get_month_navagtion ();
    $html = $calendar->get_event_quick_list ();
    $text = $calendar->format_datetime ();
    $calendar->set_datetime ();
    $version = JobRequest->get_version();

=head1 DESCRIPTION

JobRequest are used to keep track of jobmodules, such as to-dos or action items.


=head1 Methods

Calling the new function instantiates a new calendar or retrive its persistent info from the backend database.

    $calendar->set_datetime () will set the datetime of the calendar
        object and initialize based off this datetime

    $calendar->format_datetime () returns formatted date



This class method retrieves the version oh
   ->get_version();

The remaining functions control how a JobRequest is displayed:

    $calendar->show() displays a calendar's events for a day.
    $calendar->show_as_bullet() displays the calendar in a bulleted list.
    $calendar->show_as_dash() same as show_as_bullet()
    $calendar->show_edit_form() displays form for adding/editing calendar
    $calendar->get_month_navagtion() displays month navagation
    $calendar->get_event_quick_list() displays event summary list

=cut
