###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Revision$
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package LayoutArea;

use strict;

use Metadot qw($PARAMS $SESSION %FORM $USER);

use Gizmo;
use GizmoBuilder;
use Metadot::GizmoContainer;
use Metadot::Controllable;
use vars qw(@ISA);

@ISA=qw(Metadot::GizmoContainer Metadot::Controllable);

my $version = "4.5";


sub customize_permissions {
    my $self = shift;
    my $AC   = shift;

    $self->SUPER::customize_permissions($AC);

    $AC->remove_op('DISP', 'create_shortcut');
    $AC->remove_bundle('DEL');
    $AC->remove_op('MOD', 'create');
    $AC->remove_op('MOD', 'modify');
    $AC->remove_op('MOD', 'save');
    $AC->remove_op('MOD', 'up');
    $AC->remove_op('MOD', 'down');
    $AC->remove_op('MOD', 'delfile');
    $AC->remove_op('MOD', 'delfileok');
    $AC->remove_op('MOD', 'confirm_upload_deletion');
    $AC->remove_op('MOD', 'delete_upload');
    $AC->add_op('MOD', 'change_area_of_child');
    $AC->remove_bundle('EDITP');
}


sub delete_permissions {
    my $self = shift;
}

sub save_permissions {
    my $self = shift;
}

sub _get_permissions {
    my $self = shift;
    my $parent = $self->get_parent();
    my $result;
    if ($parent) {
        $result = $parent->_get_permissions();
    } else {
        $result = $self->SUPER::_get_permissions();
    }

    return $result;
}


sub get_version {
    return $version;
}

sub gizmo_list_decorator {
    return 'HTML::Decorator::NullDecorator';
}

####################################
sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

    $self->{c2}='on';

    $self->set_field_info (
        "name", "<b>Area Name</b>", 0,
    );

    ## i1 is area id
    $self->{display_wrapped} = { show => 1 };

    return $self;
}



# never display ones' self as a standalone page; only show the parent
sub show {
    my $self = shift;
    my $iid = $self->get_iid();
    $self->debug_msg(3, "Gizmo::LayoutArea::show(iid=$iid)") ;

    my $parent = $self->get_parent();
    my $parent_iid = $parent->get_iid();
    my $parent_isa = $parent->get_is_a();

    Utility->md_redirect("index.pl?iid=$parent_iid&isa=$parent_isa");
}



###################################
sub show_summary {
    my $self = shift;
    my $iid = $self->get_iid();
    $self->debug_msg(3, "Gizmo::LayoutArea::show(iid=$iid)") ;

    my $title           = $self->get_title;
    my $description     = $self->description();
    my $buttons         = $self->get_buttons();

    my $isa = $self->get_is_a();

    my $html = '';
    $html .= $buttons;

    my $admin_mode = $SESSION->admin_mode();

    $html .= $self->show_childs_list();

    if ($admin_mode) {
        # draw a light box around the section for clarity
        $html = <<EOM
<div>LayoutArea fully supported only in Style 6 - edit page properties to change page layout</div>
<table border="0" cellspacing="1" cellpadding="0" bgcolor="#DDDDDD"><tr><td>
  <table border="0" cellspacing="0" cellpadding="3" bgcolor=white><tr><td>
$html
  </td></tr></table>
</td></tr></table>
EOM
;
    }

    return $html;
}

sub generate_add_child_selection_list {
    my $self = shift;
    my $iid = $self->get_iid();
    $self->debug_msg(3, "Gizmo::LayoutArea::generate_add_child_selection_list(iid=$iid)") ;
    my $out;

    if ($self->is_allowed_to_do('add_child')) {
        $out = $self->SUPER::generate_add_child_selection_list();
        my $submit_script = $self->generate_submit_me_script();
        $out = <<EOM
    <FORM name="section_add_child_$iid" action="index.pl">
        <input type=hidden name=iid>
        <input type=hidden name=isa>
        <input type=hidden name=op>
    $submit_script
    <b><span class='smaller'>This area:</span></b>
    <a href="index.pl?id=$iid&amp;isa=LayoutArea&amp;op=add_child&amp;child_type=Separator" class='text_button'>New section</a>&nbsp;
    $out
    </FORM>
EOM
;
    }

    return $out;
}

sub create_change_area_selection {
    my $self = shift;
    my $iid = $self->get_iid();
    $self->debug_msg(3, "Gizmo::LayoutArea::create_change_area_selection(iid=$iid)") ;
    my %args = @_;

    my $child = $args{child};
    return unless $child;

    my $parent = $self->get_parent();

    return unless $parent;

    my $areas = $parent->get_category_areas();
    return unless $areas->[0];

    my $isa = $self->is_a();
    my $child_iid = $child->id();
    my $option_list = "<option value=\"0\">Move to...\n";
    $option_list .= join("\n", map("<option value=\"" . $_->id() . "\">" . $_->name() . "</option>", @$areas));

    my $submit_me_script = $self->generate_submit_me_script();

    my $area_dropdown = <<EOA
<form method="get" action="index.pl" name="change_area_$child_iid">
  $submit_me_script
  <input type="hidden" name="isa">
  <input type="hidden" name="iid">
  <input type="hidden" name="op">
  <input type="hidden" name="child_iid" value="$child_iid">
  <select name ="new_area_iid" style="font-size: smaller;" onChange="javascript:submitMe(this, '$iid', '$isa', 'change_area_of_child')">
  $option_list
  </select>
</form>
EOA
;
  return $area_dropdown;

}

sub www_change_area_of_child {
    my $self = shift;
    $self->debug_msg(3, "Gizmo::LayoutArea::www_change_area_of_child") ;

    my $child_iid = $FORM{child_iid};
    my $new_area_iid  = $FORM{new_area_iid};

    unless ($child_iid && $new_area_iid) {
        $self->print_click_back("Error", "Error with Selection. Please try again.");
        exit(0);
    }
    my $child = Gizmo->restore($child_iid);
    my $new_area = Gizmo->restore($new_area_iid);

    ## MAKE SURE child exists, the new area is really a LayoutArea, and that the child currently has a Layout Area
    unless ($child && $new_area && $new_area->is_a() eq 'LayoutArea' && $child->get_layout_area()) {
        $self->print_click_back("Error", "Invalid Selection. Please try again.");
        exit(0);
    }

    ## MAKE SURE using style 6, user can modify child, and user can add to new area
    unless ($PARAMS->{use_controller} && $child->is_allowed_to_do('modify', $USER) && $new_area->is_allowed_to_do('change_area_of_child', $USER)) {
        $self->print_click_back("Error", "You do not have sufficient access to make the change.");
        exit(0);
    }

    ## EVERYTHING CHECKS OUT SO CHANGE AREA OF CHILD
    $child->{parent_id} = $new_area_iid;
    $child->save();

    $child->can('reorder_parent') && $child->reorder_parent();
    $self->md_redirect("index.pl?iid=".$self->parent_id);
}


sub get_name {
    my $self = shift;
    return $self->name();
}

sub get_gizmo_name {
    return 'Page Area';
}

sub get_is_public {
    return 'no';
}

sub get_title {
    my $self = shift;
    return $self->name();
}

sub set_area_id {
    my $self = shift;
    my $value = shift;
    my $id_field_name = $self->get_area_id_field_name();

    $self->{$id_field_name} = $value;
}

sub get_area_id {
    my $self = shift;
    my $id_field_name = $self->get_area_id_field_name();

    return $self->$id_field_name();
}

sub get_area_id_field_name {
    my $self = shift;
    return 'i1';
}


{
    my $search_params = {
             searchable_fields => [ qw( description keywords) ],
             displayed_fields  => [ qw( description) ],
             child_isa => 'Separator',
             child_searchable_fields => [ qw( name description t1 keywords ) ],
            };

    sub get_search_params {
    my $self = shift;

    return $search_params;
    }
}


# defines the name to be displayed in search results
sub get_search_display_name {
    my $self = shift;
    my $parent = $self->get_parent();
    my $result;

    if ($parent) {
        $result = $parent->get_search_display_name();
    } else {
        $result = $self->SUPER::get_search_display_name();
    }
    return $result;
}


# never show myself directly; show my parent
sub exec_show {
    my $self = shift;
    my $context = shift;
    $context->set_subject($self->get_parent());

    return $context;
}


1;

__END__

=head1 NAME

Gizmo::LayoutArea - can either create a new LayoutArea (returing a pointer to the new object) or control
    the way existing ones are displayed (returning the formatted HTML).


=head1 SYNOPSIS

    use Gizmo::LayoutArea;

    $LayoutArea = LayoutArea->new ($id);
    $html = $LayoutArea->show ();
    $html = $LayoutArea->show_as_bullet ();
    $html = $LayoutArea->show_as_dash ();
    $version = LayoutArea->get_version();


=head1 DESCRIPTION

LayoutArea creates a separation between different Gizmos.


=head1 Methods

Calling the new function instantiates a new item or retrive its persistent info
from the backend database.

Class method to retrieve the version number of the gizmo

    $version = Item->get_version();



The remaining functions control how an Item is displayed:

    $item->show() displays an item.
    $item->show_as_bullet() displays the item as an item in a  bulleted list.
    $item->show_as_dash() show the item with a dash in front of its name.

=cut
