###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package NavigationBar;

use vars qw(@ISA);

use Metadot qw($PARAMS);
use GizmoBuilder;
use Metadot::Gizmo2;
use Metadot::Controllable;
use Metadot::PortalFactory;

@ISA=qw(Metadot::Gizmo2 Metadot::Controllable);


my $version = "1.0";
use constant GIZMO_NAME => 'Navigation Bar';
use constant HELP_MESSAGE => << 'END_TOK';

The Navigation Bar gizmo will display a site navigation bar wherever it is added. It is especially useful
in Gizmo layouts that do not include a default navigation bar.

END_TOK


use constant SCOPE_GLOBAL => 'global';
use constant SCOPE_LOCAL  => 'local';
use constant SCOPE_PARENT => 'parent';

use constant FIELDS_CONFIG => {
             FIELDS => [
                 {
                  field_name => 'global_or_local',
                  field_type => 'select_box',
                  description => 'Start navigation from',
                  select_options => {
                             SCOPE_GLOBAL() => 'Home page',
                             SCOPE_LOCAL()  => 'This page',
                             SCOPE_PARENT() => 'This page\'s parent',
                            },
                  display_order => [ (SCOPE_GLOBAL, SCOPE_LOCAL, SCOPE_PARENT) ],
                  default_option => SCOPE_GLOBAL,
                  is_required => 1,
                  store_at_column => "c2",
                  help_message => 'Select Global if you want this '.GIZMO_NAME.' to encompass all pages in the Site. Select Local if you want this '.GIZMO_NAME.' to encompass only those pages below its parent page.',
                 },
                 {
                  field_name => 'levels_deep',
                  field_type => 'select_box',
                  description => 'Levels Deep',
                  select_options => {
                             'all' => 'All levels',
                             1 => '1',
                             2 => '2',
                             3 => '3',
                             4 => '4',
                             5 => '5' },


                  display_order => [ qw( all 1 2 3 4 5 ) ],
                  default_option => '2',
                  is_required => 1,
                  store_at_column => "c1",
                  help_message => 'Select the number of levels that you want to be displayed on this '.GIZMO_NAME.'.',
                 },
                   ],

             EDIT_HELP => HELP_MESSAGE,

            };



sub get_fields_config {
    my $self = shift;

    return FIELDS_CONFIG;
}

sub get_version {
    return $version;
}

sub get_gizmo_name {
    return GIZMO_NAME;
}

sub show {
    my $self = shift;
    return $self->get_gizmo_name()." is only supported in Skins Mode";
}


sub levels_deep {
    my $self = shift;
    return $self->c1(@_);
}

sub scope {
    my $self = shift;
    return $self->c2(@_);
}

sub is_scope_global {
    my $self = shift;
    return ($self->scope() eq SCOPE_GLOBAL);
}

sub is_scope_local {
    my $self = shift;
    return ($self->scope() eq SCOPE_LOCAL);
}

sub is_scope_parent {
    my $self = shift;
    return ($self->scope() eq SCOPE_PARENT);
}

# based on the selected scope, returns the appropriate
# root object for the navigation bar
sub navigation_root {
    my $self = shift;
    my $root;

    if ($self->is_scope_global()) {
        $root = Metadot::PortalFactory->get_home_object();
    } elsif ($self->is_scope_parent()) {
        $root = $self->get_navigation_parent()->get_navigation_parent({exclude_self=>1});
    } else {
        $root = $self->get_navigation_parent();
    }

    return $root;
}



1;
