###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Revision$
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package NewsArticleImage;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT);
use Metadot::Controllable;

use vars qw(@ISA);
@ISA=qw(GizmoBuilder Metadot::Controllable);
my $version = "4.5";

sub get_version {
    return $version;
}

####################################

sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

  $self->set_field_info (
    "name", "<b>Title</b>", 0,
    "t1", "<b>Sub Title</b>", 0,
    "description", "<b>Description</b>", 0,
    "item_attachment_file",       "<b>Select Image</b>",  0,
#    "long_description", "<b>Long Description</b>", 0,
#    "show_from", "<b>Begin showing on the following date:</b> - [YYYY-MM-DD]", 0,
    "url", "<b>Story location URL</b> <i>e.g. http://www.yourcompany.com</i>", 0,
    "c2", "<b>Choose Layout</b>", 0,
    "keywords", "<b>Keywords and Synonyms</b>", 0,
    );

#  $self->set_field_type( 'show_from', 'show_from' );
  $self->set_field_type( 't1', 'text');
  $self->set_field_type( 'item_attachment_file', Metadot::UploadsManager::UPLOAD_TYPE_USER_SELECTED() );

  $self->set_field_type('c2', 'radio');
  $self->set_field_form_text('c2', $self->_get_radio_button_html());


  $self->{display_wrapped} = { show => 1,
                             };
}

sub _get_radio_button_html {
    my $self = shift;

    my $layout = $FORM{c2} || $self->get_layout() || 'layout1';

    my $layout1_checked = $layout eq 'layout1' ? 'checked' : '';
    my $layout2_checked = $layout eq 'layout2' ? 'checked' : '';

my $html = "
<table border=\"0\" border=\"0\" cellspacing=\"2\" cellpadding=\"2\">
  <tr>
    <td>
      <input type=\"radio\" name=\"c2\" value=\"layout1\" $layout1_checked>
    </td>
    <td>
      <table border=\"1\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">
        <tr>
          <td rowspan=2 align='center' " . Utility->get_spacer(30, 60, 1) . ">IMG</td>
          <td>" . Utility->get_spacer(30, 20) . "</td>
        </tr>
        <tr>
          <td>" . Utility->get_spacer(30, 40) . "</td>
        </tr>
      </table>
    </td>
    <td>
      <input type=\"radio\" name=\"c2\" value=\"layout2\" $layout2_checked>
    </td>
    <td>
      <table border=\"1\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">
        <tr>
          <td align='center' " . Utility->get_spacer(30, 30, 1) . ">IMG</td>
          <td>" . Utility->get_spacer(30, 30) . "</td>
        </tr>
        <tr>
          <td colspan='2'>" . Utility->get_spacer(60, 30) . "</td>
        </tr>
      </table>
    </td>
  </tr>
</table>\n";

    return $html;
}

my %FORM_FIELDS;
sub get_form_fields {
    my $class = shift;
    unless (%FORM_FIELDS) {
        %FORM_FIELDS = $class->SUPER::get_form_fields();
    }
}

sub get_gizmo_name {
   my $self = shift;
   return "News Article Image";
}

sub gizmo_list_decorator {
    return 'HTML::Decorator::NullDecorator';
}


sub get_title {
    my $self = shift;
    return $self->name();
}

sub get_subtitle {
    my $self = shift;
    return $self->t1();
}

sub get_layout {
    my $self = shift;
    return $self->c2();
}

sub get_image_url {
    my $self = shift;
    my ( $file, $file_icon, $file_url ) = $self->SUPER::get_upload_filename_icon_and_url( {
        id => $self->id(),
        isa => $self->is_a(),
        field_name => 'item_attachment_file',
    });
    return $file_url;
}

1;

__END__

=head1 NAME

Gizmo::NewsArticleImage - supports the creation and display of news items in an
internal news channel.

=head1 SYNOPSIS

    use Gizmo::NewsArticleImage;
    $newsitem = NewsArticleImage->new ($id);

    $name = $newsitem->get_gizmo_name ();
    $html = $newsitem->show ();
    $html = $newsitem->show_in_list ();
    $html = $newsitem->show_in_column ();
    $version = NewsArticleImage->get_version();


=head1 DESCRIPTION

Supports the creation of a new NewsArticleImage (returning a pointer to the
new object), and controls the way existing NewsArticleImages are displayed
(returning the formatted HTML).

Calling the new function generates a new NewsArticleImage. GizmoBuilder acquires the
next available id and assigns it to the new NewsArticleImage and new returns a pointer
to the new NewsArticleImage.

    get_gizmo_name()

tells anyone who calls it that this gizmo is a "News Item".

Class Method to retrieve the version of the gizmo

    $version = NewsArticleImage->get_version();




The remaining functions control how a NewsArticleImage is displayed.
  $newsitem->show()

shows one NewsArticleImage.

  $newsitem->show_in_list()

shows one NewsArticleImage in a list format.

  $newsitem->show_in_column()

shows one NewsArticleImage in a table
    (news column)format.

=cut


