###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package NewsPaperView;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS);
use Metadot::Controllable;
use DBAccess;
use Carp;
use Gizmo;
use HTML::Parser;
use Gizmo::Discussion ;
use CGI qw/escapeHTML unescapeHTML/;
use Utility;

use vars qw(@ISA);

@ISA=qw(GizmoBuilder Metadot::Controllable);
my $version = "4.5";

sub get_version {
    return $version;
}

####################################
sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

  $self->set_field_info ("name",
                         "<b>".$self->get_gizmo_name()." Name</b>",
                         1,
                         "description",
                         "<b>Description</b>",
                         0,);


  $self->{display_wrapped} = { show => 1, };

}

sub get_gizmo_name {
   my $class = shift;
   return "Aggregated Discussion View";
}

sub gizmo_list_header {
  my $class = shift;

  return "";
}

sub gizmo_list_layout {
  my $class = shift;
  my $head = shift;
  my $child_list = shift;

  my $name = $class->get_gizmo_name();

  my $output = "<!-- $name List -->\n$head\n$child_list\n<!-- End of $name List -->\n\n";

  return $output;
}

sub gizmo_list_decorator {
    return 'HTML::Decorator::NullDecorator';
}

sub www_save {
  my $self = shift;
  $self->debug_msg(3, "NewsPaperView::www_save()");

  my $iid = $self->defined_or_exit($self->id,"id is missing.");
  my $discussion_mids;
  # fill the object with form values

  foreach my $key (keys %FORM) {
    if ($key =~ "mid") {
     $discussion_mids .= "$FORM{$key},";
    }
  }
  $self->c1($discussion_mids);
  $self->name($FORM{name});
  $self->c2($FORM{all_msgs});

  # check if everything is ok
  my $errormsg = "";
  $errormsg .= "<li>Too bad! Name field cannot be empty...</li>" if (!$self->name()) ;


  if ($errormsg) {
    $self->www_modify($errormsg);
  } else {
    $self->save();
    $self->md_redirect("index.pl?iid=".$self->parent_id);
  }
}

sub process_edit_form {
  my $self = shift;
  my $isa = ref $self;
  my $id = $FORM{'iid'};
  my @discussion_mids;

  foreach my $key (keys %FORM) {
    if ($key =~ "mid") {
     push(@discussion_mids, $FORM{$key});
    }
  }
  my $discussion_mids_string = join(",", @discussion_mids);
  $self->c1($discussion_mids_string);

  # fill the object with form values
  $self->name($FORM{name});
  $self->c2($FORM{all_msgs});

  # check if everything is ok
  my $errormsg = "";
  $errormsg .= "<li>Too bad! Name field cannot be empty...</li>" if (!$self->name()) ;

  my $parentid = $self->parent_id();
  if ($errormsg) {

    my $parent = Gizmo->restore($parentid);
    $parent->www_add_child($errormsg, $self->is_a);

  } else {
    my $newid = $self->add();
    if ($newid != 0){   #### add successful
      $self->two_choices("<Big>New " . $self->get_gizmo_name() . " added!</big><br /><br />What do you want to do?",
                         "confirm",
             "Add another " . $self->get_gizmo_name(),
             "index.pl?iid=$parentid&isa=Category&op=add_child&child_type=NewsPaperView",
             "Enough for now!", "index.pl?id=$parentid");
    } else {
      $self->debug_msg(3, "NewsPaperView::process_edit_form : error when adding ".$self->get_gizmo_name()." to DB");
    }
  }
}




###################################
# BACK END FUNCTIONS


sub show_edit_form {
    my $self  = shift;
    my $title = shift;

    my $s = "";
    my $order = 0;
    my $name = $self->name;
    $name = escapeHTML($name);
    my $description = $self->description;
    $description = escapeHTML($description);
    my $old_value;
    my $iid = $self->id;


    my $checked = "";
    my $message = DBAccess->sqlSelectMany("name, IID, description ", "instance", "IsA='Discussion'", "order by name");

    if ($self->c2() eq "true") {
      $checked = "checked";
    }

    $s = "<h1>$title</h1>" .
#         "<font size=-3><i>This gizmo was inspired by the look and feel of Slashdot.org </i><br />
#          <p />"
        "This view displays all selected discussions in a newspaper-style format. Clicking on the
        image or number of comments will take you to the top level of the discussion thread.</font><br />
          <p />
        <b>".$self->get_gizmo_name()."</b><br />
          <input type=\"text\" name=\"name\" size=\"55\" maxlength=\"255\" value=\"$name\">
        <p />
        <input type=\"checkbox\" name=\"all_msgs\" value=\"true\" $checked>&nbsp;<b>Show All Messages</b>
        <p />
        Choose Discussions To Monitor<br />
        <table border=\"2\" cellpadding=\"2\" cellspacing=\"2\">
          <tr>
            <td><b>Add</b></td>
            <td><b>Discussion Name</b></td>
            <td><b>Description</b></td>
          </tr>";

  while (my ($name, $discussion_IID, $description)=$message->fetchrow) {
        if (length($description) >100) {
            $description = Utility->trim_clean($description, 100);
            $description .= "...";
        }

        if ($iid) {
            $checked = "";
            my ($already_selected) = DBAccess->sqlSelect ("c1","instance","iid=$iid");
            my @mid_numbers = split(/,/,$already_selected);
            foreach my $mid_number (@mid_numbers) {
                  if ($discussion_IID == $mid_number) {
                  $checked = "checked";
                  }
            }
      }

      if (!$iid) {
            $checked = "";
      }

      # AC enforcement
      my $disc = Discussion->new($discussion_IID);
      $description .= "&nbsp;";
      if ($disc->is_allowed_to_do('show', $USER)) {
        $s .= "<tr><td><input type=checkbox name=mid$discussion_IID value=\"$discussion_IID\" $checked></td><td> <a href=\""
              .$self->make_url("index.pl?iid=$discussion_IID&amp;isa=Discussion&amp;op=show").
              "\" STYLE=\"text-decoration: none\"> $name </a> </td><td>$description </td></tr>";
      }
    }

    # aju 09-1-0: should this be removed?  I think it is an artifact of a previous way of doing things, since
    # the Gizmo::www_add op is now commented out.
    $s .= "</table>" ;



    #foreach my $field (sort {$self->{fields}->{$a}->{order} <=> $self->{fields}->{$b}->{order}} keys (%{$self->{fields}})) {

      #  $order = $self->{fields}->{$field}->{order};

      #  next if !defined($order) || $order <= 0;

       # $s .= $self->get_form_box($field);
    #}

    #$s .= "<br /><br /><br /><i>(*) Mandatory field(s)</i><br />";

    return $s;

}

sub get_discussion_iids {
    my $self = shift;
    my @result = split(/,/,$self->c1());
    return @result;
}

sub get_show_all_messages {
    my $self = shift;
    return $self->c2();
}

sub get_messages {
    my $self = shift;

    my @where_array;
    my $creationdate;
    my $where_string = '';
    my $parent_check;
    my @iid_numbers = $self->get_discussion_iids();

    foreach my $iid(@iid_numbers) {
        push(@where_array, "iid=$iid") ;
    }

    if (scalar(@where_array) != 0) {
        $where_string = "AND ("
        .join(" OR ", @where_array)
        .")";
    } else {
        $where_string = "and iid=0";
    }

    $where_string .= " and status = 'OK'";

    if ($self->get_show_all_messages() eq "true") {
        $parent_check = "parentmid IS NOT NULL ";
    } else {
        $parent_check = "parentmid = 0 ";
    }

    my $from = "message";
    my $where = $parent_check.$where_string;
        #aju: oracle does not support 'limit' (sigh)
    my $other = "order by creationdate desc "; #limit $start, $INCREMENT";

    my $rs;
    eval {
         $rs = DBAccess->sqlSelectMany("name, mid, iid, description, creationdate, authorname, authoremail, userid",
            $from, $where, $other);
        };
    if ($@) {
        print STDERR "got db error: $@\n";
    }

    my $result = {};
    $result->{rows} = $rs;
    ($result->{count})        = DBAccess->sqlSelect("count(*)",$from, $where, $other);
    ($result->{thread_count}) = DBAccess->sqlSelect("count(*)", "message", "parentmid = 0 ".$where_string);

    return $result;
}


sub show {
    my $self = shift;
    return $self->show_summary();
}

###################################
sub show_summary {
    my $self = shift;
    #my $category_iid = shift; ## actually the parent id
    #$category_iid = undef unless Utility->is_valid_integer($category_iid); # secu_200401
    my $category_iid = $self->id();

    my $INCREMENT = 20;
#     my $INCREMENT = 3;
    my $s = "";
    my $uid = $self->uid();
    my $all_msgs = $self->get_show_all_messages();

    $self->debug_msg(4, 'NewsPaperView::show_as_bullet()');
    #    print STDERR "in NewsPaperView::show_as_bullet-- iid is " . $self->id() . "\n";

    my $buttons = $self->show_panel().$self->show_up_down_buttons();
    ($buttons) && ($buttons .= "&nbsp;\n");
    $s .= "<h3>" . $buttons . $self->name . "</h3>";

    my @iid_numbers = $self->get_discussion_iids();

    if (exists($FORM{post_mess})) {
        $s .= "<b>Select A Discussion To Post A Message</b><p />";
        $s .= "<table border=\"2\" cellpadding=\"2\" cellspacing=\"2\"><tr><td>Discussion Name</td><td>Description</td><tr>";
        foreach my $iid_number(@iid_numbers) {

        my $disc = Gizmo->restore( $iid_number );

        # do not how those discussions that are not
        # accessible to the user
        unless( $disc->is_allowed_to_do( 'send' ) ) { next }

            my ($discussion) = DBAccess->sqlSelectMany("name, description","instance","iid=$iid_number","order by name");
            my ($discussion_name, $discussion_description) = $discussion->fetchrow;
            $s .= "<tr><td width=\"100%\"><a href=\"".
            $self->make_url("index.pl?isa=Message&amp;op=compose&amp;discussion_id=$iid_number").
            "\">$discussion_name</a></td><td>  $discussion_description &nbsp;</td></tr>";
        }
        $s .= "</table>";
    } else {
        my $start = $FORM{start} || 0;

# aju: actually, I don't think these variables are even used here.. so I will comment
# out my upgrade for now.
#   my $mypageelementboxcolor; my $mypagetitleboxcolor;
#   #aju
#   my $theme = $PARAMS->{'template_theme'};
#   if ($theme && ($theme ne 'null')) {
#       # get the color
#       my $boxcolor = $PARAMS->{'theme_mypage_color'};
#       $mypageelementboxcolor = $boxcolor;
#       $mypageelementboxcolor = $boxcolor;
#   }
#   else {
#       $mypageelementboxcolor = $PARAMS->{'mypageelementboxcolor'};
#       $mypagetitleboxcolor = $PARAMS->{'mypagetitleboxcolor'};
#   }
#   if (substr($mypageelementboxcolor,0,1) ne "#") {
#       $mypageelementboxcolor = "#$mypageelementboxcolor";
#   }
#   if (substr($mypagetitleboxcolor,0,1) ne "#") {
#       $mypagetitleboxcolor = "#$mypagetitleboxcolor";
#   }
#         my ($mypagetitleboxcolor, $mypageelementboxcolor) =
#         ($PARAMS->{'mypagetitleboxcolor'}, $PARAMS->{'mypageelementboxcolor'});

        my $messages = $self->get_messages();
        my $message  = $messages->{rows};
        my $message_count = $messages->{count};
        my $comments;

        if (!$message_count) {
            $s .= "<br /><b>" . $self->get_gizmo_name() . " " . $self->name()
            ." :</b> No discussions selected or the discussions selected have no messages.\n";
        }

        my $num_messages;
        $s .= '<table CELLPADDING="3" CELLSPACING="0" border="0" WIDTH="100%">';

        my @row_colors = ('#FFFFFF','#F4F4F4');

        my $mcount = 0;
        my %discussion_cache = ();
        while (my ($name, $mid, $discussion_iid, $description, $creationdate, $authorname, $authoremail) = $message->fetchrow) {
            my $disc = $discussion_cache{$discussion_iid};
            unless ($disc) {
                $disc = Discussion->restore($discussion_iid);
                $discussion_cache{$discussion_iid} = $disc;
            }
            next unless ($disc->is_allowed_to_do('show', $USER));

            if ($mcount < $start) {
                $mcount++;
                next;
            }
            if ($mcount >= ($start + $INCREMENT)) {
                last;
            }
            # AC enforcement


            $mcount++;  # moved this after the AC check
            $num_messages++;

            my $not_ok;

            my $tag_open_sub =

            sub {
                my ($tagname, $attr) = @_;

                if($tagname eq 'img') {
                    # will remove the img tag altogether if its src attribute
                    # contains any & and ? characters
                    #      use this line instead if you want to go all the way to
                    #      prohibiting all image types but the ones mentioned in
                    #      the regexp:
                    #         if( $attr->{src} && ( ($attr->{src} =~ /\&|\?/) ||
                    #           ($attr->{src} !~ /(gif|jpg|png|jpeg)$/i) ) ) {
                    if( $attr->{src} &&  ($attr->{src} =~ /\&|\?/) ) {
                            $not_ok = 1;
                    }
                }
            };

            my ($parsedDescription) = new HTML::Parser (api_version => 3,
                start_h => [ $tag_open_sub, 'tagname,attr,offset,length' ], );
            $parsedDescription->parse($description);

            my $discussion_name = $disc->name();

        my ( $img_file, $img_file_icon, $img_file_url ) = $disc->get_upload_filename_icon_and_url( {
                                                    id => $disc->id(),
                                                    isa => $disc->is_a(),
                                                    field_name => 'discussion_attachment_file',
                                                       } );

            my $discussion_url = $self->make_url("index.pl?iid=$discussion_iid&amp;isa=Discussion&amp;op=show");
            my $message_url = $self->make_url("index.pl?mid=$mid&amp;isa=Message&amp;op=show");
            my $read_more_link = "" ;
            my $max_len = 250;
            $description = Utility->remove_tags($description);
            if (length ($description) > $max_len) {
                $description = Utility->trim_clean($description, $max_len).'. . .';
                $read_more_link = "<a class='centerNewsItem' href=\"".
                $self->make_url("index.pl?mid=$mid&amp;isa=Message&amp;op=show").
                "\">Read More</a> | ";
            }

            my $image_html = "&nbsp;" ;

        if ($img_file) {
        $image_html = "<a href=\"".$self->make_url("index.pl?iid=$discussion_iid&amp;isa=Discussion&amp;op=show")."\">" ;
        $image_html .= "<img src=\"$img_file_url\" width=\"30\" border=\"0\">" ;
        $image_html .= "</a>" ;

            }

            my $total_messages;
            if ($all_msgs eq "true") {
                $total_messages = $self->count_messages_in_thread($mid);
        if ($total_messages == 1) {
            $comments = "message in thread";
        }
        else {
            $comments = "messages in thread";
        }
            } else {
                $total_messages = $self->count_messages($mid);
                if ($total_messages == 1) {
                    $comments = "comment";
                } else {
                    $comments = "comments"
                }
            }
           ### print # of comments/msgs in thread
            my $thread_link = "<a class='centerNewsItem' href=\""
                . $self->make_url("index.pl?mid=$mid&amp;isa=Message&amp;op=show")
                . "\">$total_messages</a> $comments";

            my $bgcolor = $row_colors[$num_messages%2];

            my $cut ;
            my $max_length = 20 ;
            $cut = length($discussion_name) > $max_length && length($discussion_name.$name) > 2*$max_length;
            $discussion_name = Utility->trim_clean($discussion_name, $max_length-5)."..." if ($cut) ;
            $cut = length($name) > $max_length && length($discussion_name.$name) > 2*$max_length;
            $name = Utility->trim_clean($name, $max_length+5)."..." if ($cut) ;

            my $my_style_tag = "" ;
            my $my_class_tag = "" ;

            $s .= "<tr bgcolor=\"$bgcolor\">" ;
            $s .= "<td align='center' valign='top' nowrap=\"nowrap\" class='DATE'>" . Utility->format_datetime($creationdate, 'b e, Y') . "</td>" ;
            $s .= "<td valign='top' $my_style_tag>" ;
            $s .= "<a href=\"$discussion_url\" $my_class_tag><b $my_class_tag>$discussion_name:</b></a> " ;
            $s .= "<a href=\"$message_url\" $my_class_tag>$name</a>" ;
            $s .= "<br />from&nbsp;<b $my_class_tag>$authorname</b>" ;
            $s .= "</td>" ;
            $s .= "<td align='center' valign='top'>$image_html</td>" ;
            $s .= "</tr>" ;

            $s .= "<tr bgcolor=\"$bgcolor\">" ;
            $s .= "<td>&nbsp;</td>" ;
            $s .= "<td colspan='2'  $my_class_tag>" ;
            $s .= "<i>$description</i>" ;
            $s .= "<br />" ;
            $s .= "($read_more_link$thread_link)" ;
            $s .= "</td>" ;
            $s .= "</tr>" ;

        }
        $s .= '</table>';

        my $total_messages = $messages->{thread_count};
        my $next_start = $start + $INCREMENT;
        my $next_number;

        if($start >= $INCREMENT) {
            $s .= "[ <a href=\"".
            $self->make_url("index.pl?iid=$category_iid&amp;start=".
            ($start-$INCREMENT))."\">Previous $INCREMENT</a> ] \&nbsp\; ";
        }

        if($next_start < $total_messages) {
            my $messages_left = $total_messages - $next_start;

            if ($messages_left > $INCREMENT) {
                $next_number = $INCREMENT;
            } else {
                $next_number = $messages_left;
            }

            $s .= "[ <a href=\"".
            $self->make_url("index.pl?iid=$category_iid&amp;start=".$next_start).
            "\">Next $next_number</a> ]\&nbsp\;";
        }
        $s .= "[ <a href=\"".
        $self->make_url("index.pl?iid=$category_iid&amp;post_mess=yes").
        "\">Post New Message</a> ]" if($num_messages > 0);
    }

    $s ="<p />".$s;
    return $s;
}

###################################
sub get_top_parent {
  my $self = shift;
  my $mid = shift;

  $mid = undef unless Utility->is_valid_integer($mid); # secu_200401
  my ($parent) = DBAccess->sqlSelect("parentmid", "message", "mid = ".$mid);

  if ($parent != 0) {
    $mid = $self->get_top_parent($parent);
  }

  return $mid;
}


###################################
sub count_messages_in_thread {
  my $self = shift;
  my $mid = shift;

  my $result = $self->count_messages($self->get_top_parent($mid)) + 1;
  return $result;
}


###################################
sub count_messages {
  my $self = shift;
  my $mid = shift;

  my $ctr = 0;

  $mid = undef unless Utility->is_valid_integer($mid); # secu_200401
  my $rows = DBAccess->sqlSelectMany("mid","message","parentmid=$mid");

  while(my $m_id = $rows->fetchrow()) {

    $ctr++;

    $ctr += $self->count_messages($m_id);
  }

  return $ctr;
}

###################################
sub show_line {
  my $self = shift;

  my ($attach, $coolicon, $s);

  # in case there is an attached file
  $attach = "";     #erase previous file setting
  $attach = $self->attachment_icon($self->file1) if ($self->file1);
  if (uc($self->cool) eq "YES"){
      my $imageslocation = $PARAMS->{'imageslocation'};
      my $star = "<img SRC=\"$imageslocation/star.gif\" width=\"15\" height=\"16\" border=\"0\">";

      $coolicon = "&nbsp;$star\n";

  } else {
      $coolicon="";
  }
  if ($self->url){
    $s .= "<a href=\"".$self->make_url($self->url) . "\">" . $self->name . "</a>$coolicon $attach\n";
  } else {
    $s .= $self->name ." $coolicon $attach\n";
  }
  $s .= $self->show_panel();
  #$s .= " - $description" if ($description);
  $s .= "\n\n";

  return $s;
}


1;

__END__

=head1 NAME

Gizmo::NewsPaperView - can either create a new NewsPaperView (returing a pointer to
    the new object) or control the way existing ones are displayed (returning the
    formatted HTML).

=head1 SYNOPSIS

    use Gizmo::NewsPaperView;

    $newpaperview = NewsPaperView->new ($id);
    $newspaperview->www_save ();
    $newspaperview->process_edit_form ();
    $html = $newspaperview->show_edit_form ($title);
    $html = $newspaperview->show_as_bullet ($category_iid, $all_msgs);
    $html = $newspaperview->show_as_dash ();
    $version = NewsPaperView->get_version();


=head1 DESCRIPTION

NewsPaperView allows the user to add NewsPaperViews and to modify
the way existing ones display information.

=head1 Methods

Calling the new function generates a new NewsPaperView. GizmoBuilder acquires
the next available id and assigns it to the new NewsPaperView, and new returns
a pointer to it.

   $newspaperview->www_save()

is the public method for saving the NewsPaperView object,
either when it is being created or when it is being modified. This function saves the
HTML form values that the user entered.

   $newspaperview->process_edit_form()

is a private method that validates the data
the user has entered. This is called when the user presses the "Save" button.

   $newspaperview->show_edit_form()

is a private method that loads information from
the database and presents it to the user in the edit form.

    $version = NewsPaperView->get_version();

is a class method that retrieves the version of the gizmo


The reminaining functions control how the NewsPaperView is displayed. show_as_bullet
shows a bulleted list with the option of showing all messages or just the top level,
and show_as_dash shows each item with a dash preceding it.


=cut


