#!/usr/bin/perl

#
# Test file for Newsitem.pm
# $Id: Newsitem.001.t,v 1.2.2.2 2005/07/01 22:41:27 nico Exp $
#

use strict;
use Test::More tests=>11;

use SmokeMetadot;
use Metadot::Testing::Mechanizer;
use DBAccess;
use MetadotConfig;
use Metadot::PortalFactory;

# set up info
my $class_name   = 'Newsitem';
my $mech = Metadot::Testing::Mechanizer->new();
my $parent_page = Metadot::PortalFactory->get_home_object();
my $admin_uid = MetadotConfig->get_reserved_uid('ADMIN');

my $test_instance_name = '_FOO_TEST_INSTANCE_';
my $test_instance_description = '_FOO_TEST_DESCRIPTION_';

################################################################################
# Test: Make sure the instance does not yet exist
################################################################################
ok(!$mech->instance_retrievable(iid=>$parent_page->id(), expr=>$test_instance_name),
   'Instance does not yet exist');


################################################################################
# Test: Create an instance 
################################################################################
use_ok('Gizmo::'.$class_name);

my $instance = $class_name->new({
    uid         => $admin_uid,
    parent_id   => $parent_page->id(),
    name        => $test_instance_name,
    description => $test_instance_description,
});

isa_ok($instance, $class_name);


################################################################################
# Test: Add the instance to the db
################################################################################
my $instance_id = $instance->add();
ok(Utility->is_valid_integer($instance_id), "Got a valid iid $instance_id");

$instance = $class_name->restore($instance_id);
ok(defined($instance));
isa_ok($instance, $class_name);
ok($instance->id() == $instance_id);


################################################################################
# Test: Instance is visible from parent page
################################################################################
ok($mech->instance_retrievable(iid=>$parent_page->id(), expr=>$test_instance_name),
   'Instance was found in parent page');

################################################################################
# Test: Instance can be shown standalone
################################################################################
ok($mech->instance_retrievable(
    iid=>$instance->id(), 
    expr=>"${test_instance_name}.*$test_instance_description"),
   'Instance was shown in standalone');

################################################################################
# Test: Remove the instance from the db
################################################################################
$instance->delete();
$instance = $class_name->restore($instance_id);
ok(!defined($instance), "Instance was removed from db");


################################################################################
# Test: Instance is visible from browser
################################################################################
ok(!$mech->instance_retrievable(iid=>$parent_page->id(), expr=>$test_instance_name),
   'Instance was removed from database'
);

#################################################
# That's it!!!!
#################################################


