###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Poll;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY);

use strict;

use DBAccess;
use Gizmo;
use Form::Verify;
use Data::Dumper;
use GizmoBuilder;
use Metadot::Invitable;
use Metadot::Controllable;
use CGI qw/escapeHTML unescapeHTML/;

use vars qw(@ISA);
@ISA=qw(GizmoBuilder Metadot::Invitable Metadot::Controllable);
my $version = "4.5";

sub get_version {
    return $version;
}


#######################################




sub customize_permissions {
    my $class = shift;
    my $AC = shift;

    $class->SUPER::customize_permissions($AC);
    Metadot::Invitable->customize_permissions($AC);

    $AC->add_bundle('VOTE', 'Vote', '0','0');
    $AC->add_op('VOTE','vote');
    $AC->add_op('VOTE','add_statistic');

    $AC->add_op('MOD', 'modify_poll');
    $AC->add_op('MOD', 'add_option');
    $AC->add_op('MOD', 'create_shortcut');
}


sub show_search_results {
    my $gizmo_name = shift ;
    my $thingtosearch = shift;
    my $startfrom = shift || 0;
    my $maxitem = shift || 50;

    my $s="";
    my $oneresult="";

    my $filter = $thingtosearch;

    ################################################################
    ####
    #### we build the query
    ####
    #### clean quote and other chars
    ################################################################

    $thingtosearch = uc($thingtosearch);
    $thingtosearch = DBAccess->quote("%$thingtosearch%");

    my $portal_parent = MetadotConfig->get_reserved_iid('PORTAL_PARENT');

    my $i=0;
    my $totalrows=0;
    my $from = $startfrom;
    my $entries;

    my $c;

    ($c)=DBAccess->sqlSelectMany( "distinct i.iid, i.isa, i.description, j.response", "instance i, poll_response j", "i.isa='$gizmo_name' AND i.iid=j.parent_iid and ( i.parentiid >= 0 OR i.parentiid = $portal_parent) AND (upper(i.description) LIKE $thingtosearch OR upper(j.response) LIKE $thingtosearch)" );


    while( ( my ( $iid, $isa, $name, $response ) = $c->fetchrow() ) && ($i < $maxitem)) {
    if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC
    my $gizmo = $isa->new($iid);
    next unless $gizmo->is_allowed_to_do('show', $USER);
    next unless $gizmo->parentCheck(); # don't show hidden items
    $totalrows++;

    $name="null" if (!$name || !defined($name));
    ### remove tags that could break the page.

    #      print STDERR "ABOUT TO CALL format_search_result: $iid, $isa, $name @fields_text\n";
    $oneresult = $gizmo_name->format_search_result($iid, $isa, $name, [ $response ] );
    ### put the results in list tag
    $oneresult="<li>" . "$oneresult" . "<br /><br /></li>\n\n";
    $s .= "$oneresult\n\n";;
    $i++;
    }

    $c->finish();

    $s = $gizmo_name->show_search_result_group({
        gizmo_name    => $gizmo_name,
        total_rows    => $totalrows,
        max_rows      => $maxitem,
        start_from    => $startfrom,
        cursor        => $i,
        filter        => $filter,
        results       => $s,
    });

    return ($s);

}


sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

  $self->set_field_info (
                        "description",        "<b>Question</b>",                                         1,
                        "t1",           "enter answers one per line",                                    0,
                    );

  $self->{display_wrapped} = { show => 1,
                             };

}

sub gizmo_list_header {
    my $class = shift;

    return "";

}

sub name {
    my $self = shift;
    return $self->{description};
}

sub process_edit_form {
    my $self = shift;
    my $id = $FORM{'iid'};

    #$self->c1($FORM{name});
    #$self->c2($FORM{question});
    $self->description($FORM{'description'});
    $self->t1($FORM{'t1'});

    my $verify = new Form::Verify();
#    my $responses = $FORM{'t1'};
    my $responses = $FORM{'responses'};

    $verify->verify('non_empty' , 'Question', $FORM{'description'});

    foreach (split "\n", $responses) {

        #$verify->verify('non_empty', $temp2, $_);
        $verify->verify('non_empty' , 'Responses', $_);
        if ($verify->errors) {
            last;
        }
    }

    if (my $errorstring = $verify->errors) {
        $errorstring = "The following field(s) are invalid: $errorstring - cannot have blank lines.";
    #$self->print_click_back("Sorry", "The following field(s) are invalid: $errorstring\n");

    my $parentid = $self->parent_id();
    my $parent = Gizmo->restore($parentid);
    $parent->www_add_child($errorstring, $self->is_a);

    } else {
    my $newid = $self->add();
    foreach (split "\n", $responses) {
        $_ =~ s/\r//g;
        DBAccess->sqlInsert("poll_response", ('response'   => "$_",
                   'parent_iid' => "$newid"));
    }
    $self->two_choices("<Big>Poll Added!</Big><br /><br /><br />What do you want to do?",
                             "confirm",
                 "Add another poll", "index.pl?iid=$id&isa=Category&op=add_child&child_type=Poll",
#                "I'm done for now.", "index.pl?iid=". $id);
                 "I'm done for now.", "index.pl?iid=". $self->parent_id());
    }
}


sub _make_session_id_hash {
    my $self = shift;

    my $session_id = $SESSION->{session_id};

    my $result;
    while( my $char = chop($session_id) ) {

    my $x = ord($char) % 10;
    $result .= $x;
    }

    # keep size at 7 digits
    $result = substr( $result, 0, 7 );

#    print STDERR "\nJUST MADE A SESSION HASH Sess_id is $SESSION->{session_id}; --hash: $result\n";

    return $result;
}

sub show_in_column {
    my $self = shift;

    my $user = $USER->{'uid'};

    my $rs = "";

    my $iid = $self->id();

    my $question = $self->{description};

    if( defined($USER->{uid}) && $USER->{uid} == MetadotConfig->get_reserved_uid('ANONYMOUS_USER') ) {

    $user = $self->_make_session_id_hash();

#   print STDERR "RENDRING POLL: $user\n";
    }

    if ($self->is_allowed_to_do('vote', $USER) && (defined $user) && ($user ne '')) {
    my ($sth) = DBAccess->sqlSelectMany("userid",
                  "poll_response, poll_vote",
                  "poll_response.parent_iid = $iid
                          and poll_response.poll_response_id = poll_vote.poll_response_id
                                  and userid = '$user'" );

    unless ($sth->fetchrow_hashref) {
        #$rs .= $self->show_panel() . $self->show_scheduling_buttons();
        $rs .= $self->get_buttons();
        $rs .= $self->_show_question($iid);
    } else {
        my @results = $self->_get_results($iid);
        #$rs .= $self->show_panel() . $self->show_scheduling_buttons() ."<div class=\"question\">$question</div>";
        $rs .= $self->get_buttons() ."<div class=\"question\">$question</div>";
        my $graph = $self->draw_graph(@results);
        $rs .=  $graph;
        $rs .= $self->draw_invite_button();
    }
    } else {
    #$rs .= $self->show_panel() . $self->show_scheduling_buttons() ."<div class=\"question\">$question</div>";
    $rs .= $self->get_buttons() ."<div class=\"question\">$question</div>";
    my @results = $self->_get_results($iid);
    my $graph = $self->draw_graph(@results);
    $rs .=  $graph;
    $rs .= $self->draw_invite_button();
    }
    return $rs;
}

sub _show_question {
   my $self = shift;
    my $iid = shift;
    $iid = undef unless Utility->is_valid_integer($iid); # secu_200401

    $self->debug_msg(4, "Poll::show_question()");

    my $poll_id = $iid;
    my ($question) = DBAccess->sqlSelect('description', 'instance', "iid=$poll_id");


    my $rs = "";
    $rs .= "<span class=\"question\">$question</span>\n";
    $rs .= "<form action=\"index.pl\" method=\"post\">\n";
    $rs .= "<input type=\"hidden\" name=\"isa\" value=\"Poll\" />\n";
    $rs .= "<input type=\"hidden\" name=\"op\" value=\"add_statistic\" />\n";
    $rs .= "<input type=\"hidden\" name=\"id\" value=\"" . $poll_id . "\" />\n";

    $rs .= "<table border=\"0\" cellspacing=\"2\" cellpadding=\"0\" width=\"100%\">\n";
    my $sth = DBAccess->sqlSelectMany("response, poll_response_id", "poll_response", "parent_iid=$poll_id");

    while (my ($response, $rid) = $sth->fetchrow()) {

      $self->debug_msg(5, "row $rid $response");
      $rs .= "<tr><td>\n";
      $rs .= "<input type=\"radio\" name=\"response\" value=\"" . $rid . "\" />\n";
      $rs .= "</td><td width=\"100%\">\n";
      $rs .= "<span class=\"response\">".$response . "</span>\n";
      $rs .= "</td></tr>\n";
    }


    $rs .= "<tr><td class=\"poll\" colspan=\"2\" align=\"center\">\n";
    $rs .= "<input type=\"submit\" value=\"Vote\" name=\"Vote\" /><br />\n";
    $rs .= "<div><a class=\"poll\" href=\"".$self->make_url("index.pl?isa=Poll&amp;op=add_statistic&amp;id=$poll_id")."\">View Results</a></div>\n";
    #CKIM-SENDABLE
    if ($self->can('show_invite_button')) {
        my $invite_button = $self->show_invite_button();
        if ($invite_button) {
            $invite_button = "<br />".Utility->get_spacer(5,1)."<br />".$invite_button;
            $rs .= $invite_button;
        }
    }
    $rs .= "</td></tr>\n";
    $rs .= "</table></form>\n";

    return $rs;

}

sub show {
    my $self = shift;
    $self->debug_msg(4, "Poll::show()");

    my $iid = $FORM{'iid'};
    my $rs = '<br />';

    $iid = undef unless Utility->is_valid_integer($iid); # secu_200401
    my ($question) = DBAccess->sqlSelect('description', 'instance', "iid=$iid");
    my $sth = DBAccess->sqlSelectMany('*', 'poll_response', "parent_iid=$iid");


#    $rs .= "<h1>$topic</h1>";
    $rs .= "<div class='question'>$question</div>\n";
    $rs .= "<form action=\"index.pl\" method=\"post\">";
    $rs .= "<input type=\"hidden\" name=\"isa\" value=\"Poll\" />\n";
    $rs .= "<input type=\"hidden\" name=\"op\" value=\"add_statistic\" />\n";
    $rs .= "<input type=\"hidden\" name=\"id\" value=\"$FORM{'iid'}\" />\n";

    $rs .= "<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n";
    while (my $row = $sth->fetchrow_hashref) {
    $rs .= "<tr><td>";
    $rs .= "<input type=\"radio\" name=\"response\" value=\"" . $row->{'poll_response_id'} . "\" />";
    $rs .= "</td>\n<td>";
    $rs .= $row->{'response'};
    $rs .= "</td></tr>";
    }
    $rs .= "<tr><td colspan=\"2\" align=\"right\">";
    $rs .= "<input type=\"submit\" value=\"See Results\" name=\"Vote\" />";
    $rs .= "<input type=\"submit\" value=\"Vote\" name=\"Vote\" />";

    $rs .= "</td></tr>";
    $rs .= "</table>";

    #return $self->show_panel() .  $self->show_scheduling_buttons() . $rs;
    return $self->get_buttons() . $rs;
}

sub show_summary {
    my $self = shift;

    my $user = $USER->{'uid'};

    my $rs = "";

    my $iid = $self->id();
    my $question = $self->{description};
    if( $USER->{uid} == MetadotConfig->get_reserved_uid('ANONYMOUS_USER') ) {
    $user = $self->_make_session_id_hash();
    }

    if ($self->is_allowed_to_do('vote', $USER) && (defined $user) && ($user ne '')) {
    my ($sth) = DBAccess->sqlSelectMany("userid",
                        "poll_response, poll_vote",
                        "poll_response.parent_iid = $iid
                          and poll_response.poll_response_id = poll_vote.poll_response_id
                                  and userid = '$user'" );

    unless ($sth->fetchrow_hashref) {
        $rs .= $self->get_buttons();
        $rs .= $self->_show_question($iid);
    } else {
        my @results = $self->_get_results($iid);
        $rs .= $self->get_buttons() . "<div class=\"question\">$question</div>";
        my $graph = $self->draw_graph(@results);
        $rs .=  $graph;
        $rs .= $self->draw_invite_button();
    }
    } else {
    $rs .= $self->get_buttons() . "<div class=\"question\">$question</div>";
    my @results = $self->_get_results($iid);
    my $graph = $self->draw_graph(@results);
    $rs .=  $graph;
    $rs .= $self->draw_invite_button();
    }

    $rs = <<EOM
    <table width="160" border="0" cellspacing="1" cellpadding="0" bgcolor="#CCCCCC"><tr><td>
    <table width="100%" border="0" cellspacing="0" cellpadding="5" bgcolor=white><tr><td>
    $rs
    </td></tr></table>
    </td></tr></table>
EOM
    ;
    return $rs;
}

#sub www_modify_poll {
sub www_save {
    my $self = shift;

    my $iid = $FORM{iid};
    $iid = undef unless Utility->is_valid_integer($iid); # secu_200401

    my $verify = new Form::Verify();

    $verify->verify('non_empty', 'Question' , $FORM{'description'});

    ## verify the form data
    foreach my $key (sort keys %FORM) {
    if ($key =~ m/response_id/) {
            $verify->verify('non_empty', 'Responses', $FORM{$key});
        }
        if ($verify->errors) {
            last;
        }
    }
    foreach (split "\n", $FORM{'responses'}) {
        $verify->verify('non_empty', 'Responses', $_);
        if ($verify->errors) {
            last;
        }
    }

    if (my $errorstring = $verify->errors) {
        $errorstring = "The following field(s) are invalid: $errorstring - cannot have blank lines.";
        $self->www_modify($errorstring);
    } else {

#        DBAccess->sqlUpdate("instance", "iid = $FORM{'iid'}", (c1 => $FORM{'name'}));
#        DBAccess->sqlUpdate("instance", "iid = $FORM{'iid'}", (name => $FORM{'description'}));
#        DBAccess->sqlUpdate("instance", "iid = $FORM{'iid'}", (description => $FORM{'description'}));
        my $instance = Gizmo->restore($iid);
        $instance->description($FORM{'description'});
        $instance->name($FORM{'description'});
        $instance->save();

        foreach my $key (sort keys %FORM) {
        if ($key =~ m/response_id/) {
            my $response_id = $key;
            $response_id =~ s/response_id_//o;
            DBAccess->sqlUpdate("poll_response","poll_response_id = $response_id",('response' => $FORM{$key}));
        }
        }

        foreach (split "\n", $FORM{'responses'}) {
        $_ =~ s/\r//g;
        DBAccess->sqlInsert("poll_response", ('response'   => "$_",
                    'parent_iid' => $iid));
        }
        if($SESSION->get_value('GB_status') eq '0') {
                $self->print_choices("<Big>Poll Modified!</Big><br /><br /><br />What do you want to do?",
                                "confirm",
                                "View poll", "index.pl?iid=$iid&isa=Poll&op=show",
        #            "I'm done for now.", "index.pl?iid=". $FORM{'iid'});
                                "I'm done for now.", "index.pl?iid=". $self->parent_id());
        } else {
            $self->md_redirect("index.pl?iid=". $self->parent_id());
        }
    }

}

#sub www_modify {
sub show_edit_form {
    my $self = shift;
    my $errormsg = shift;
    $self->debug_msg(4, "Poll::www_modify()");

#    $errormsg = $errormsg ?
#                 "<p /><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p />\n" :
#                 "";
#
#     my $title = 'Modify Poll';
    my $title = $errormsg;
    my $rs = '';

    my $id        = $FORM{'iid'};
    $id = undef unless Utility->is_valid_integer($id); # secu_200401

    my $responses = $FORM{'t1'} || '';

    if($title eq "Adding New Poll") {
        $id = '-666666' ;
    }

    my ($question) = DBAccess->sqlSelect("description", "instance", "iid = $id");
    my $sth = DBAccess->sqlSelectMany("poll_response_id, response", "poll_response", "parent_iid = $id");

    $rs .= "<h1>$title</h1>\n";


    $rs .= "<table>";

#    $rs .= "<tr><td>Topic</td><td>";
#    $rs .= "<input type=\"text\" name=\"name\" value=\"$topic\">";
#    $rs .= "</td></tr>";

    # must convert double quotes to html entities to preserve syntax
    $question = escapeHTML( $question );

    $rs .= "<tr><td>Question</td><td>";
    $rs .= "<input type=\"text\" name=\"description\" value=\"$question\" />";
    $rs .= "</td></tr>";

    my $count = 0;

    while(my $row = $sth->fetchrow_hashref) {
    $rs .= "<tr><td>";
    $rs .= ($count++ == 0) ? 'Responses' : "&nbsp;";
    $rs .= "</td><td>";

    my $value = $row->{response};

    # must convert double quotes to html entities to preserve syntax
    $value = escapeHTML( $value );

    $rs .= "<input type=\"text\" name=\"" .
           'response_id_' . $row->{'poll_response_id'} .
               "\" value=\"" .
               $value . "\" />";

    $rs .= "</td></tr>";
    }


    $rs .= "<tr><td valign=\"top\">Responses</td><td>";
    $rs .= "<textarea cols=\"30\" rows=\"5\" wrap=\"yes\" name=\"responses\">$responses</textarea><br />";
    $rs .= "Note: Enter one response on each line";
    $rs .= "</td></tr>";

#     $rs .= "<tr><td colspan='2' align='right'>";
#     $rs .= "<input type=\"submit\" name=\"Update\" value=\"Update\">";
#     $rs .= "</td></tr>";

    $rs .= "</table>\n";

#    $rs .= "</form>";

#     my $html = $self->show_top();
#     $html .= $self->show_title("Poll results");
#     $html .= $rs;
#     $html .= $self->show_bottom();
#     $self->print_http_header();
#     print $html;

    return $rs;
}

sub www_add_statistic {
    my $self = shift;
    $self->debug_msg(4, "Poll::www_add_statistic()");

    my $choice = $FORM{'response'};
    my $id = $FORM{'id'};
    $id = undef unless Utility->is_valid_integer($id); # secu_200401

    my $user = $USER->{'uid'};
    my $global_look = $PARAMS->{'globallook'};

    if( $USER->{uid} == MetadotConfig->get_reserved_uid('ANONYMOUS_USER') ) {

    $user = $self->_make_session_id_hash();

#   print STDERR "ADDING VOTE: $user\n";
    }

    if ($FORM{'Vote'} eq 'Vote') {
    if ($choice) {
        my ($sth) = DBAccess->sqlSelectMany("userid",
                        "poll_response, poll_vote",
                        "poll_response.parent_iid = $id
                       and poll_response.poll_response_id = poll_vote.poll_response_id
                                       and userid = $user" );
        unless ($sth->fetchrow) {
                my $db_now = Utility->db_now();
        DBAccess->sqlInsert("poll_vote", ('poll_response_id' => "$choice",
                          'userid'              => "$user",
                          'creation_date'    => $db_now));
        } else {
        $self->print_click_back("Sorry", "You have already voted in this poll.");
        exit 0;
        }
    }
    }

    if ($FORM{'Vote'} eq 'Vote') {

    $self->md_redirect($ENV{'HTTP_REFERER'});

    }
    else {

    my @presults = $self->_get_results($id);
    my $graph = $self->draw_graph(@presults);
    my $results = $self->show_title("Poll results");

    $results .= "<br /><br />";
    $results .= "<center>";
    $results .= "<b>" . $self->description . "</b>";
    $results .= "<table width=\"200\"><tr><td>";
    $results .= $graph;
    $results .= $self->draw_invite_button();
    $results .= "</td></tr></table></center>";

    if ($global_look eq "usetemplate") {
        $self->print_http_header(); #CKIM: ensure http header is sent
        print  $DISPLAY->simple_expand($id, $self, $self->show_template(-1), $results);
    }
    else {
        $self->print_portal($results);
    }

    }
}

sub _get_results {
    my $self = shift;
    my $id = shift;
    my @results = ();

    $id = undef unless Utility->is_valid_integer($id); # secu_200401
    my $sth = DBAccess->sqlSelectMany("poll_response_id, response", "poll_response", "parent_iid = $id");

    while (my $row = $sth->fetchrow_hashref) {
    my %hash = %$row;
    ($hash{'count'}) = DBAccess->sqlSelect('count(poll_response_id)', 'poll_vote', "poll_response_id = $hash{'poll_response_id'}");
    push @results, \%hash;
    }

    return @results;
}

sub draw_graph {
    my $self = shift;
    my @results = @_;
    my $sum;

    foreach (@results) {
    $sum += $_->{'count'};
    }

    my $rs = "<table width=\"100%\"><tr><td class=\"graph\">";

    my $rowcount = 0;

    foreach (@results) {
    my $color = ($rowcount++ % 2) == 1 ? "#eeeeee" : "#dddddd";
    $rs .= "<table bgcolor='$color' border=\"0\" cellspacing=\"0\" cellpadding=\"3\" width=\"100%\">";
    my $percent = $_->{'count'} == 0 ? 0 : ($_->{'count'} / $sum) * 100;
    $rs .= "<tr><td>";
    $rs .= '<span class="response">'.$_->{'response'}.'</span>';

    my $text ='';
    if ($_->{'count'} == 1) {
        $text = 'vote';
    } else {
        $text = 'votes';
    }

    $rs .= "</td><td align='right'>" . $_->{'count'} . " $text</td></tr>";

    $rs .= "<tr><td colspan='2'>";
    unless ($percent == 0) {

        my $imageslocation = $PARAMS->{'imageslocation'};

        $rs .= "<img class=\"icon\" src=\"$imageslocation/bluedotstrt.gif\" height=\"10\" width=\"2\" border=\"0\" alt='' />";
        $rs .= "<img class=\"icon\" src=\"$imageslocation/bluedot.gif\" height=\"10\" width=\"";
        $rs .= sprintf("%0.0f", ($percent));
        $rs .= "\" border=\"0\" alt='$percent percent' />";
            $rs .= "<img class=\"icon\" src=\"$imageslocation/bluedotend.gif\" height=\"10\" width=\"4\" border=\"0\" alt='' />";
        $rs .= "\&nbsp;";
    }
    $rs .= sprintf("%3.0f", $percent) . "\%";
    $rs .= "</td></tr></table>";
    }

    $rs .= "</td></tr><tr><td colspan='2' align='right'>Total Votes: $sum</td></tr></table>";
    return $rs;
}


sub draw_invite_button {
    my $self = shift;
    my $rs = '';
    if ($self->can('show_invite_button')) {
        my $invite_button = $self->show_invite_button();
        if ($invite_button) {
            $invite_button = "<br />".Utility->get_spacer(5,1)."<br />".$invite_button;
            $rs .= "<center>" . $invite_button . "</center>";
        }
    }
    return $rs;
}


sub gizmo_list_decorator {
    return 'HTML::Decorator::NullDecorator';
}


sub get_name {
    my $self = shift;
    return $self->description();
}


1;

__END__

=head1 NAME

Gizmo::Poll -

=head1 SYNOPSIS

    use Gizmo::Poll;
    $poll = Poll->new ($id);

    $permissions= &get_default_permissions ();
    $poll->process_edit_form () ;
    $html = $poll->show_in_column ();
    $html = $poll->show ();
    $poll->www_modify_poll ();
    $poll->www_modify ($errormsg);
    $poll->www_add_statistic ();
    $version = Poll->get_version();


=head1 DESCRIPTION

The Poll gizmo allows for users to create and add polls to their categories. It provides a simple interface for entering poll questions and their possible answers. It will allow users to vote via a voting form that gets rendered in the right column. For users that have already voted it will render the poll results instead of the voting form.

=head2 Methods

$poll = Poll->new ($id);

Standard Gizmo constructor. If an id is provided it will restore an existing gizmo from the datab

$permissions= &get_default_permissions ();

Returns the default permissions Operations object for this class. See GizmoBuilder.pm documentation.

$poll->process_edit_form () ;

This method will read gizmo specific data from the posted form when an gizmo is being created.It is invoked from www_create_child in Category.pm and Table.pm. See Gizmo.pm documentation.


$version = Poll->get_version();

This class method will retrieve the version number of the gizmo

$html = $poll->show_in_column ();

Will generate HTML to render the content of this gizmo in a format suitable for displaying in column form.

$html = $poll->show ();

Will generate HTML to render the content of this gizmo. This method is not currently called from anywhere in the system, but may be if use in teh future.

$poll->www_modify ($errormsg);

This method is invoked by the system when a user wants to modify the content of a gizmo (usually by clicking in the "Edit" icon). See Gizmo.pm documentation.

$poll->www_modify_poll ();

This method is invoked by the system in sequence after www_modify to commit changes to the Poll instance to the database.

$poll->www_add_statistic ();

This method processes the input of votes from users. It is invoked by the user interface rendered by the show and show_in_column methods.


=cut

