###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name$
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package PressRelease;
use strict;

use Metadot::GizmoContainer;
use Gizmo::PressRelease::Release;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS);

use vars qw(@ISA);
@ISA=qw(GizmoBuilder Metadot::GizmoContainer);
#use AbstractCategoryGizmoHack;
#@ISA=qw(AbstractCategoryGizmoHack);
my $version = "4.5";

sub get_version {
    return $version;
}


{
    my $search_params = {
             searchable_fields => [ qw( name description keywords ) ],
             child_isa => 'Release',
             child_searchable_fields => [ qw( name d1 longdescription keywords ) ],
            };

    sub get_search_params {
    my $self = shift;

    return $search_params;
    }
}


####################################
sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

  $self->{is_inline} = "";

  $self->set_field_info (
    "name", "<b>Press Release Page Title</b>", 1,
    "description", "<b>Description</b>",0,
    );

  # NZ added htmlarea field_type to the description field
  $self->set_field_type( 'description', 'htmlarea' );

  return $self;
}

sub customize_permissions {
    my ($class, $ac) = @_;

    $class->SUPER::customize_permissions($ac);
    Metadot::GizmoContainer->customize_permissions($ac);

    return $ac;
}


sub get_gizmo_name {
   my $self = shift;
   return "Press Release Section";
}

sub gizmo_list_header {
    my $class = shift;

    return "";
}

sub gizmo_list_decorator {
   return 'HTML::Decorator::NullDecorator';
}


sub www_add_child {
  my $self = shift;
  my $errormsg = shift;
  my $child_type = shift;
  $self->debug_msg(3, "PressRelease::www_add_child()");

  my $child_isa = $FORM{child_type} || $child_type;
  my $gizmo_name = $child_isa->get_gizmo_name();

  my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
  # we re-instanciate the object because we want a brand new one
  # unless there is an error message (we want to keep his data)
  my $child = $child_isa->new({ parent_id => $parentid });

    $errormsg = $errormsg ?
                "<p /><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p />\n" :
                "";

  my $form = $child->show_edit_form("Adding new $gizmo_name");
  my $content =
  Components->color_palette_code().
"
<form name=fm action='index.pl' method='post' ENCTYPE=\"multipart/form-data\">
<input type='hidden' name='isa' value='PressRelease'>\n
<input type='hidden' name='iid' value='$parentid'>\n
<input type='hidden' name='op' value='create_child'>\n
<input type='hidden' name='child_type' value='Release'>\n
$errormsg
$form
<table align='right'><tr><td><input type='submit' value='Create'></td></tr></table>
<br />&nbsp;
</form>
";

  $child->print_form($content, "Adding new $child_isa");
}

sub www_create_child {
    my $self = shift;
    my $child_type = shift;

    $self->debug_msg(3, "Gizmo::www_create_child()");

    my $child_isa = $FORM{child_type} || $child_type;

    my $parentid = $self->defined_or_exit($self->id, "parent id is missing.");
    # we re-instanciate the object because we want a brand new one
    my $child = $child_isa->new({
                            uid => $USER->uid(),
                            parent_id =>$parentid,
                           });

    # this method should ALWAYS be overridden in derived classes.
    $child->process_edit_form();
}


##########################################
### show
### return one newsitem (page format)
##########################################
sub show {
  my $self = shift;
  $self->debug_msg(1, "PressRelease::show()");
  my $s="";
#  print STDERR "in PressRelease:show()\n";

  my $id = $self->id();
  my $isa = $self->is_a();
  my $name = $self->name();
  my $description = $self->description();
  my $child = "Release";

  my $top_anchor = int rand(999999);

  my $page_id = ($self->is_inline())? $self->parent_id : $self->id();

  my $spacer = "<table border=\"0\" width=\"100%\"><tr><td>&nbsp;</td></tr></table>";

  my $admin_panel;
  my $show_panels;
  if (($self->is_allowed_to_do('modify', $USER)) && ($SESSION->admin_mode())) {
    $admin_panel .= "[ <a href=\""
                 .  $self->make_url("index.pl?iid=$id&amp;isa=$isa&amp;op=add_child&amp;child_type=$child")
                 .  "\">Add Press Release</a> ]";
    $show_panels = 1;
  } else {
    $admin_panel .= "";
  }
  $admin_panel .= "<br />" unless (!$admin_panel);
  $name &&= "<b>$name</b><br />";
  $description .= "<br />" unless (!$description);

    my $release_list_ref =  Release->get_releases($self->id(), $show_panels);
    my @release_arr = @{$release_list_ref};

    my $release_list = "<ul>";
    my $release_descriptions = "";
    foreach my $release_ref (@release_arr) {
        my %release = %{$release_ref};

        # building release list section
        $release_list .= "<li>$release{panel}<a href=\"#$release{id}\">"
                        . "$release{title}</a><br />posted on: $release{date}";

        $release_descriptions .= "<a name=\"$release{id}\"></a><b>$release{title}</b><p />";

        $release_descriptions .= "$release{description}<p />";

        $release_descriptions .= "<table width=\"100%\" border=\"0\"><tr><td align='right'>"
                            . "<a href=\"#$top_anchor\">"
                            . "Back to Top</a></td></tr></table>"
                            . "<br /><hr width=\"50%\"><br />";

    }
    $release_list .= "</ul>";

  $s .= <<EOM
    <a name="$top_anchor"></a>
    $name
    $description
    $admin_panel
    $release_list

    <hr width="50%"><br />

    $release_descriptions

    <p />
    $spacer

EOM
;

  return $s;
}

sub show_summary {
    my $self = shift;
    my @args = @_;

    my $buttons = $self->get_buttons();
    my $id = $self->id();
    my $isa = $self->is_a();

    $self->{is_inline} = 1;
    my $text = $self->show();
    return $buttons.$text;
}


##########################################
### show_in_list
### return one newsitem (list format)
##########################################
# aju: do I need this method?
sub show_in_list {
  my $self = shift;
  $self->debug_msg(1, "PressRelease::show_in_list()");
#    print STDERR "in PressRelease:show_in_list()\n";

  my $name        = $self->get_name();
  my $description = $self->get_description();
  my $url         = $self->get_url();
  my $panel       = $self->get_panel();

  $name = "<img src=$url>"; #if ($url) ;

  return "<b>$name</b> - $panel<br />\n$description<br />\n";
}




1;

__END__

=head1 NAME

Gizmo::PressRelease - A Gizmo subclass to display PressReleases and contain FAQ questions.

=head1 SYNOPSIS

    use Gizmo::PressRelease;
    $PressRelease = PressRelease->new([$id]);

    $name = $PressRelease->get_gizmo_name();
    $item_display_html = $PressRelease->show();
    $bullet_display_html = $PressRelease->show_as_bullet();
    $version = PressRelease->get_version();



=head1 DESCRIPTION

This is a container and interface for viewing FAQ questions and organising them.

=head2 Methods

Constructor;  instantiates a new item or retrive its persistent info from the backend
database.
$id is the $id of the PressRelease:

    $PressRelease = Gizmo::PressRelease->new([$id]);


Class method to retrieve the version of the gizmo

    $version = Discussion->get_version();

Get the name of the item for pretty-printing:

    $name = $PressRelease->get_gizmo_name();

Generate html to show the image full-size and include its descriptive text:

    $item_display_html = $PressRelease->show();

Generate html to  'list', or give  abbreviated display of the image information:

    $bullet_display_html = $PressRelease->show_as_bullet();

=cut


