###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Separator;

use strict;

use Metadot qw($PARAMS $SESSION);

use Gizmo::Category::Clipboard;
use Gizmo;
use GizmoBuilder;
use Metadot::GizmoContainer;
use Metadot::Controllable;
use Metadot::CyclingContainer;
use vars qw(@ISA);

@ISA=qw(Metadot::GizmoContainer Metadot::CyclingContainer Metadot::Controllable);

my $version = "4.5";

sub get_version {
    return $version;
}

sub gizmo_list_decorator {
    return 'HTML::Decorator::NullDecorator';
}

####################################
sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

    $self->set_field_info (
        "t1",           "<b>Section Heading</b>",   0,
        "c4",           "<b>Add line below section heading</b>",   0,
        "description",  "<b>Section Content</b>",   0,
        "c1",           "<b>Display a horizontal rule below the section</b>",         0,
        "c2",           "<b>Leave space above the section</b>",   0,
        "c3",           "<b>Leave space below the section</b>",   0,
        # c5 is reserved for cycling
    );

    $self->set_field_type( 't1', 'text' );
    $self->set_field_type( 'description', 'htmlarea' );
    $self->set_field_type( 'c1', 'checkbox' );
    $self->set_field_type( 'c2', 'checkbox' );
    $self->set_field_type( 'c3', 'checkbox' );
    $self->set_field_type( 'c4', 'checkbox' );
    $self->{display_wrapped} = { show => 1 };
}


# never display ones' self as a standalone page; only show the parent
sub show {
    my $self = shift;

    my $parent = $self->get_parent();
    my $parent_iid = $parent->get_iid();
    my $parent_isa = $parent->get_is_a();

    Utility->md_redirect("index.pl?iid=$parent_iid&isa=$parent_isa");
}


###################################
sub show_summary {
    my $self = shift;

    my $title           = $self->get_title;
    my $description     = $self->description();
    my $hr_below        = $self->is_hr_below;
    my $br_above        = $self->is_br_above;
    my $br_below        = $self->is_br_below;
    my $hr_heading      = $self->is_hr_heading;
    my $buttons         = $self->get_buttons();

    my $iid = $self->get_iid();
    my $isa = $self->get_is_a();

    my $html = '';
    $html .= "<br />\n" if $br_above;
    $html .= $buttons;

    my $admin_mode = $SESSION->admin_mode();

    # in admin mode, use a default title if none is defined
    if (!$title && $admin_mode) {
        $title = '[ Unnamed Section ]';
    }

    $title &&= "<span style=\"font-size: 120%; font-weight: bold;\">$title</span>";
    $html .= $title;

    # if there was a section heading, either add a line break or a horizontal rule
    if ($self->t1() || $admin_mode) {
        if ($hr_heading) {
            $html .= "<hr>\n";
        } else {
            $html .= "<br />\n";
        }
    }

    if ($description) {
        $html .= "$description<br />\n";
    }

    if ($SESSION->admin_mode()) {
        $html .= "<br />\n".$self->generate_add_child_selection_list()."\n";
    }

    $html .= $self->show_childs_list();

    $html .= "<hr>\n" if $hr_below;
    $html .= "<br />\n" if $br_below;

    if ($admin_mode) {
        # draw a light box around the section for clarity
        $html = <<EOM
<table border="0" cellspacing="1" cellpadding="0" bgcolor="#DDDDDD"><tr><td>
  <table border="0" cellspacing="0" cellpadding="3" bgcolor=white><tr><td>
$html
  </td></tr></table>
</td></tr></table>
EOM
;
    }

    return $html;
}

sub generate_add_child_selection_list {
    my $self = shift;
    my $iid = $self->get_iid();
    my $out;

    if ($self->is_allowed_to_do('add_child')) {
        $out = $self->SUPER::generate_add_child_selection_list();
        my $submit_script = $self->generate_submit_me_script();
        $out = <<EOM
<FORM name="section_add_child_$iid" action="index.pl">
    <input type=hidden name=iid>
    <input type=hidden name=isa>
    <input type=hidden name=op>
$submit_script
<b><span class='smaller'>This section:</span></b>
$out
</FORM>
EOM
;
    }
    return $out;
}


sub get_name {
    my $self = shift;
    return $self->name();
}

sub get_gizmo_name {
    return 'Page Section';
}

sub get_is_public {
    return 'no';
}

sub name {
    my $self = shift;
    my @args = @_;
    return $self->t1(@args);
}

sub get_title {
    my $self = shift;
    return $self->t1();
}

sub is_hr_below {
    my $self = shift;
    return $self->c1();
}

sub is_br_above {
    my $self = shift;
    return $self->c2();
}

sub is_br_below {
    my $self = shift;
    return $self->c3();
}

sub is_hr_heading {
    my $self = shift;
    return $self->c4();
}

sub number_of_elements_to_display {
    my $self = shift;
    return $self->c5(@_);
}


# never show myself directly; show my parent
sub exec_show {
    my $self = shift;
    my $context = shift;
    $context->set_subject($self->get_parent());

    return $context;
}




1;

__END__

=head1 NAME

Gizmo::Separator - can either create a new Separator (returing a pointer to the new object) or control
    the way existing ones are displayed (returning the formatted HTML).


=head1 SYNOPSIS

    use Gizmo::Separator;

    $Separator = Separator->new ($id);
    $html = $Separator->show ();
    $html = $Separator->show_as_bullet ();
    $html = $Separator->show_as_dash ();
    $version = Separator->get_version();


=head1 DESCRIPTION

Separator creates a separation between different Gizmos.


=head1 Methods

Calling the new function instantiates a new item or retrive its persistent info
from the backend database.

Class method to retrieve the version number of the gizmo

    $version = Item->get_version();



The remaining functions control how an Item is displayed:

    $item->show() displays an item.
    $item->show_as_bullet() displays the item as an item in a  bulleted list.
    $item->show_as_dash() show the item with a dash in front of its name.

=cut
