package SiteLibrary;
use strict;
use vars qw(@ISA);

use Portal;
use Metadot qw( $SESSION $USER %FORM $USER $CGI $DISPLAY );
use MetadotConfig;
use Metadot::GizmoContainer;
use Metadot::GizmoSingleton;
use Metadot::PortalFactory;
use Metadot::Controllable;


@ISA = qw( Metadot::GizmoContainer Metadot::GizmoSingleton Metadot::Controllable );
my $SESSION_KEY_FILTER = 'SiteLibraryFilter';


sub customize_permissions {
    my $class = shift;
    my $AC = shift;

    $class->SUPER::customize_permissions($AC);

    $AC->set_default_level('DISP', 0);
    $AC->set_min_level('DISP', 0);

    $AC->add_op('DISP', 'toggle_filter');
    $AC->add_op('MOD',  'manage');
    $AC->add_op('MOD',  'clipboard_delete');
}


# SiteLibrary is a reserved instance. You should access an instance
# of SiteLibrary using SiteLibrary->get_instance();

sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);
    
    $self->set_parent_iid(MetadotConfig->get_reserved_iid('HIDDEN_CATEGORY'));
    $self->set_uid(MetadotConfig->get_reserved_uid('ADMIN'));
    $self->set_name('Site Image Library');
}


###################################
# Force this to operate in controller mode
sub www_show {
    my $self = shift;

    my $controller = Metadot::PortalFactory->get_controller();
    my $context = Metadot::PortalFactory->build_context({
        subject => $self,
        op      => $FORM{op},
        user    => $USER,
        form    => \%FORM,
        session => $SESSION,
    });
    my $response = $controller->do($context);
    if ($response) {
        $DISPLAY->print_http_header();
        print $response;
        exit;
    }    
}


# toggles whether we filter for the images owned by the user
sub www_toggle_filter {
    my $self = shift;
    $self->filter_images(!$self->filter_images());
    $self->md_redirect($self->make_url("index.pl?iid=".$self->get_iid()));
}


sub generate_add_child_selection_list {
    my $self = shift;
    my $iid = $self->get_iid();
    my $out = $self->SUPER::generate_add_child_selection_list();
    my $submit_script = $self->generate_submit_me_script();
    $out = <<EOM
<FORM name="section_add_child_$iid" action="index.pl">
    <input type=hidden name=iid>
    <input type=hidden name=isa>
    <input type=hidden name=op>
$submit_script
<b><span class='smaller'>Add an element:</span></b> $out
</FORM>    
EOM
;
    return $out;
}


sub get_name {
    return 'Image Library';
}


sub get_description {
    return <<EOM
This is the image library for the portal. You can copy an image in this library
and paste it into the description of a page.
EOM
;
}


# returns true if images are being filtered
sub filter_images {
    my $self = shift;
    my $value = shift;
    my $result;
    if (defined $value) {
        $SESSION->set_value($SESSION_KEY_FILTER, $value);
        $result = $value;
    } else {
        $result = $SESSION->get_value($SESSION_KEY_FILTER);
    }
    return $result;
}


# can filter images
sub get_childs_list {
    my $self = shift;
    my @args = @_;

    if ($self->filter_images()) {
        my $uid = $USER->uid();
        my @temp_list = @{$self->SUPER::get_childs_list(@args)};
        my @result = ();
        foreach my $child (@temp_list) {
            if ($child->uid() == $uid ) {
                push @result, $child;
            }
        }
        return \@result;
    } else {
        return $self->SUPER::get_childs_list(@args);
    }

}


sub get_gizmo_name {
    return 'Image Library';
}


sub get_is_public {
    return 'no';
}

sub go_back_to_parent_after_editing_permissions {
	return 0;
}

sub www_manage {
    my $self = shift;
    #my $content = $self->show();
    #return $DISPLAY->print_simple_layout({content=>$content});

    return $self->www_show(@_);
}

sub exec_manage {
    my $self = shift;
    return $self->exec_show(@_);
}


1;
