##########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package SiteMap;

use strict;

use Sendmail;

use Metadot qw( $PARAMS %FORM $DISPLAY $USER );

use Metadot::Gizmo2;

use vars qw(@ISA);

@ISA=qw( Metadot::Gizmo2 );

use constant HELP_MESSAGE => << 'END_TOK';

The SiteMap gizmo will display a clickable list of pages available in the site or at the section of the site where added.

END_TOK

my $FIELDS_CONFIG = {
             FIELDS => [
                {
                 field_name => 'levels_deep',
                 field_type => 'select_box',
                 description => 'Levels Deep',
                 select_options => {
                            'all' => 'All levels',
                            1 => '1',
                            2 => '2',
                            3 => '3',
                            4 => '4',
                            5 => '5' },


                 display_order => [ qw( all 1 2 3 4 5 ) ],
                 default_option => '2',
                 is_required => 1,
                 store_at_column => "c1",
                 help_message => 'Select the number of levels that you want to be displayed on this Site Map.',
                },
                {
                 field_name => 'global_or_local',
                 field_type => 'select_box',
                 description => 'Coverage',
                 select_options => {
                            'global' => 'Global',
                            'local' => 'Local',
                           },
                 display_order => [ qw( global local ) ],
                 default_option => 'global',
                 is_required => 1,
                 store_at_column => "c2",
                 help_message => 'Select Global if you want this Site Map to encompass all pages in the Site. Select Local if you want this Site Map to encompass only those pages below its parent page.',
                },
                   ],

             EDIT_HELP => HELP_MESSAGE,

            };

sub get_fields_config {
    my $self = shift;

    return $FIELDS_CONFIG;
}

sub get_gizmo_name {

    return 'Site Map';
}

sub gizmo_list_decorator {
    return 'HTML::Decorator::NullDecorator';
}

sub show_summary {
    my $self = shift;

    $DISPLAY->debug_msg(3, "in show_summary()");

    my $levels_deep = $self->get_field_value( 'levels_deep' );

    my $levels_label;

    if( $levels_deep eq 'all' ) {
        $levels_deep = 99;
        $levels_label = ''
    } else {
        my $number = [qw( Zero One Two Three Four Five Six Seven Eight)]->[$levels_deep];
        my $s = '';
        if( $levels_deep > 1 ) { $s = 's' }
        $levels_label = " - $number level$s deep";
    }

    my $global_or_local = $self->get_field_value( 'global_or_local' );

    my ( $parent_cat, $site_word );

    if( $global_or_local eq 'local' ) {
        $site_word = 'Section';
        $parent_cat = $self->get_navigation_parent();

    } else {
        $site_word = 'Site';
        $parent_cat = Gizmo->restore( 0 );
    }

    $DISPLAY->debug_msg(4, "show_summary has parent_cat $parent_cat, levels_deep=$levels_deep; global_or_local: $global_or_local");

    my @path = ( $parent_cat->get_id() );


    my $buttons = $self->get_buttons();

    my %args = (
                path => \@path,
                unconditional_expansion_depth => $levels_deep - 1,
                #depth => 2,
                bold_first_level => 1,
    );

    my $html = $parent_cat->show_site_map(%args);
    # aju - put div around html so that it doesn't ever go on the same line
    # as the output of the previous child in the page.
    my $navigation_column = "\n<div>$buttons$html</div>";

#
# removed title for the moment
#
#    my $navigation_column = "
#<H2>$buttons$site_word Map$levels_label</H2>
#$html";

    $DISPLAY->debug_msg(4, "Finished show_summary");

    return $navigation_column;
}


1;
