###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package SubmissionItem;

use strict;

use Metadot qw($PARAMS $USER %FORM $DISPLAY);

use Gizmo;
use GizmoBuilder;
use Gizmo::SubmissionSystem;
use Metadot::Metagram;
use Utility;
use Metadot::AccessBroker::SubmissionItemBroker;
use MetadotConfig;
use Metadot::Controllable;
use Metadot::Notifiable;
use vars qw(@ISA);

@ISA=qw(GizmoBuilder Metadot::Controllable Metadot::Notifiable);

my $version = "4.5";

sub get_version {
    return $version;
}

sub customize_permissions {
    my ($self, $ac) = @_;
    $self->SUPER::customize_permissions($ac);
    Metadot::Notifiable->customize_permissions($ac);

    return $ac;
}



## DEFINE LABEL BELOW
use constant PENDING => 0;
use constant APPROVED => 100;
use constant REJECTED => 200;

my %STATUS_LABELS = (
                     0  => 'Pending',
                     100 => 'Approved',
                     200 => 'Rejected',
                    );

####################################
sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

    my $status_field_name = $self->get_status_field_name();
    my $fullname_field_name = $self->get_fullname_field_name();
    my $email_field_name = $self->get_email_field_name();

    my $last_comments_field_name = $self->get_last_comments_field_name();

    $self->set_field_info (
        "name",                 "<b>Name of your submission</b>", 1,
        "description",          "<b>Description</b>",             0,
        "item_attachment_file", "<b>Attachment file</b>",         0,
        "keywords",             "<b>Keywords and Synonyms</b>",   0,
        $fullname_field_name,   "<b>Your Full Name (Submitter)</b>",          1,
        $email_field_name,      "<b>Your Email (Submitter)</b>",              1,
        #"t1",                   "<b>Comments</b>",               0,
        $last_comments_field_name, "<b>Enter New Comments</b>",   0,
        "xyz", $self->get_edit_form_comments(), 0,
        #$status_field_name,     "<b>Status</b>",                 0,
              );

    $self->set_field_type( 'item_attachment_file', Metadot::UploadsManager::UPLOAD_TYPE_USER() );
    $self->set_field_type($last_comments_field_name, 'mediumhtml');
    $self->set_field_type( 'description', 'htmlarea' );
    #$self->set_field_type($status_field_name, 'radio');
    #$self->set_field_form_text($status_field_name, $self->_get_radio_button_html());

    $self->{display_wrapped} = { show => 1,
                   };

    $self->_determine_intro_text();
}


sub restore {
    my $proto = shift;
    my $self = $proto->SUPER::restore(@_);

    if ($self) {
        $self->_determine_intro_text();
    }
    return $self;
}

sub _determine_intro_text {
    my $self = shift;

    my $intro_text = '';

    if ($self->is_approved()) {
        $intro_text = '';
    } elsif ($self->is_approval_required()) {
        $intro_text = "You are about to submit an item for publishing that requires approval. The approver may require that you make changes before your item is approved. Make sure you enter a correct email address so that you can receive status updates and approver feedback regarding your submission item.";
    } else {
        $intro_text = "You are about to submit an item for publishing that does not require approval. This item will automatically be published on the site.";
    }

    $self->set_intro_text($intro_text);
}

sub get_upload_filename_icon_and_url {
    my $self = shift;
    return $self->SUPER::get_upload_filename_icon_and_url( {
        id => $self->id(),
        isa => $self->is_a(),
        field_name => 'item_attachment_file',
    });
}

sub get_www_modify_buttons {
    my $self = shift;
    return "<input type='submit' value='Save' class='button'> <input type='checkbox' name='save_and_email' value='1'> <b>Email</b> <input type='checkbox' name='save_and_metagram' value='1'> <b>".Metadot::Metagram->get_label()."</b>\n";
}

sub notify_parties {
    my ($self, %params) = @_;
    $self->debug_msg(3, "Gizmo::SubmissionSystem::SubmissionItem::notify_parties()");


    ##### PRE-PROCESSING

    my $parent = $self->get_parent();
    my $status_label    = $self->get_status_field_label();

    my $mystuff_link = $PARAMS->{'basename'} . "/index.pl?isa=Metadot::SystemApp::MyStuff&amp;op=show";
    my $body = $params{body_intro} . "\n" . $self->show(suppress_for_email => 1);
    my $last_comments = $self->get_last_comments();
    if ($last_comments) {
         $body .= "<p /><b>Most recent comment</b><br />$last_comments\n";
    }
    $body .= "<p />You can view this submission by going to your <a href=\"$mystuff_link\">MyStuff</a> area.<br />" . $PARAMS->{'basename'} . "/index.pl\n";

    ##### END PRE-PROCESSING

    ### set params
    $params{subject} = "[$status_label] submission: " . $self->get_name();
    if ($params{body_intro}) {
        $params{subject} .= " - " . substr($params{body_intro}, 0, 12) . "...";
    }
    $params{approver_uids} = $parent->get_all_who_can('APPROVE'); ## get all users who can approve
    $params{body} = $body;


    ## create metagrams
    $self->send_metagrams_to_parties(%params) if $params{metagram_parties};


    ## email if necessary
    $self->email_parties(%params) if $params{email_parties};
}

sub send_metagrams_to_parties {
    my ($self, %params) = @_;
    $self->debug_msg(3, "Gizmo::SubmissionSystem::SubmissionItem::send_metagrams_to_parties()");


    return unless $USER->is_logged_in();

    my $subject    = $params{subject};
    my $body       = $params{body};
    my $uids       = $params{approver_uids}; ## get all users who can approve
    my $authorname = $params{authorname} || $USER->get_display_name();
    my $authoremail = $params{authoremail} || $USER->email();

    my $id         = $self->id();
    my $isa        = $self->get_is_a();

    ## add submitter
    push (@$uids, $self->get_uid());
    $uids = Utility->uniquify_array($uids);

    foreach my $uid (@$uids) {
        ## create metagram
        Metadot::Metagram->add_message(
                  userid        => $USER->uid(),
                  authorname    => $authorname,
                  authoremail   => $authoremail,
                  iid           => $id,
                  name          => $subject,
                  description   => $body,
                  recipient_uid => $uid,
            );
    }

}

sub email_parties {
    my ($self, %params) = @_;
    $self->debug_msg(3, "Gizmo::SubmissionSystem::SubmissionItem::email_parties()");

    my $isa = $self->get_is_a();
    my $id  = $self->id();

    my $subject    = $params{subject};
    my $body       = $params{body};
    my $uids       = $params{approver_uids}; ## get all users who can approve

    $body .= "<br /><i>This is an automated message - please do not reply.</i>\n";

    my @emails = "";
    foreach my $uid (@$uids) {
        push (@emails, $USER->get_email_from_uid($uid));
    }
#    ## add submitter email
#    push (@emails, $self->get_email());
#    @emails = @{ Utility->filter_valid_emails($self->uniquify_array(@emails)) };

    my $recipient_list = join(',',@emails);
    #my $sender = $USER->{email};
    my $sender = $PARAMS->{mailsenderaddress};
    my $sm = Sendmail->new('null','', $recipient_list, $subject, $body);
    $sm->{to} = $self->get_email();
    #$sm->{to} = $PARAMS->{mailsenderaddress};
    #$sm->{cc} = $params{cc_me} ? $sender : '';
    $sm->{bcc} = $recipient_list;
    $sm->{subject} = $subject;
    $sm->{body} = $body;
    #NZ no method found. No longer needed?
    #$sm->set_encoding('html');
    #$sm->{from} = $sm->{reply_to} = $sender;
    $sm->{from} = $sender;

    my ($result) = $sm->sendMail();
    #print STDERR "CKIM: Invitable->send_invitations FINISHED SENDMAIL\n";
}

###################################

sub www_add_comments {
    my $self = shift;
    $self->debug_msg(3, "Gizmo::SubmissionSystem::SubmissionItem::www_add_comments()");

    my $email_parties = defined($FORM{save_and_email}) ? 1 : 0;
    my $metagram_parties = defined($FORM{save_and_metagram}) ? 1 : 0;
    my $comments_field_name = $self->get_last_comments_field_name();

    if (defined($FORM{$comments_field_name}) && $FORM{$comments_field_name} ne '') {
        $self->set_last_comments($FORM{$comments_field_name});
        $self->save();
        $self->notify_parties(body_intro => "New comments have been added.", email_parties => $email_parties, metagram_parties => $metagram_parties);
        $self->print_choices("<Big>New comments added!</big>",
                       "confirm",
                       "Continue",
                       "index.pl?id=" . $self->id() . "&isa=" . $self->is_a() . "&op=show");
    } else {
        ## no comments
        $DISPLAY->print_click_back('You did not enter any comments.');
    }
}

sub www_approve {
    my $self = shift;
    $self->debug_msg(3, "Gizmo::SubmissionSystem::SubmissionItem::www_approve()");

    my $email_parties = defined($FORM{save_and_email}) ? 1 : 0;
    my $metagram_parties = defined($FORM{save_and_metagram}) ? 1 : 0;
    my $comments_field_name = $self->get_last_comments_field_name();
    my $navigable_parent = $self->get_navigation_parent();

    $self->set_last_comments($FORM{$comments_field_name}) if (defined($FORM{$comments_field_name}) && $FORM{$comments_field_name} ne '');
    $self->set_approval_values();
    $self->init_permissions(); ## make sure permissions are reset
    $self->save();
    $self->notify_parties(body_intro => 'Status changed to ' . $self->get_status_label(APPROVED), email_parties => $email_parties, metagram_parties => $metagram_parties);

    $self->print_choices("<Big>Status updated to " . $self->get_status_label(APPROVED) . " for submission " . $self->get_name() . "</big>",
                   "confirm",
                   "Continue",
                   "index.pl?id=" . $self->id() . "&isa=" . $self->is_a() . "&op=show",
                   );

}


sub www_reject {
    my $self = shift;
    $self->debug_msg(3, "Gizmo::SubmissionSystem::SubmissionItem::www_reject()");

    my $email_parties = defined($FORM{save_and_email}) ? 1 : 0;
    my $metagram_parties = defined($FORM{save_and_metagram}) ? 1 : 0;
    my $comments_field_name = $self->get_last_comments_field_name();

    $self->set_last_comments($FORM{$comments_field_name}) if (defined($FORM{$comments_field_name}) && $FORM{$comments_field_name} ne '');
    $self->set_rejection_values();
    $self->save();
    $self->notify_parties(body_intro => 'Status changed to ' . $self->get_status_label(REJECTED), email_parties => $email_parties, metagram_parties => $metagram_parties);

    $self->print_choices("<Big>Status updated to " . $self->get_status_label(REJECTED) . " for submission " . $self->get_name() . "</big>",
                   "confirm",
                   "Continue",
                   "index.pl?id=" . $self->id() . "&isa=" . $self->is_a() . "&op=show");
}

###################################
sub show {
    my $self = shift;
    my %args = @_;

    my $name            = $self->get_name();
    my $description     = $self->get_description();
    my $keywords        = $self->get_keywords();
    my $fullname        = $self->get_fullname();
    my $email           = $self->get_email();
    my $submitdate      = Utility->format_datetime($self->get_creation_date(), 'm/e/y T');
    my $status_label    = $self->get_status_field_label();
    my $status_date     = Utility->format_datetime($self->get_status_date(), 'm/e/y T');

    my ( $file, $file_icon ) = $self->get_upload_filename_icon_and_url();

    my $not_registered = '';

    unless ($self->is_submission_user_registered()) {
        $not_registered .= "&nbsp;<font color=\"red\">(Not registered)</font>";
    }

    my $html            = '';

    $html .= "<center><h1>Submission Information</h1></center>\n";
    $html .= "<p /><b>Submitted by:</b> $fullname, $email $not_registered\n";
    $html .= "<br /><b>Submission Name:</b> $name\n";
    $html .= "<br /><b>Description:</b> $description\n" if ($description);
    $html .= "<br /><b>File attached: </b>$file\n" if ($file);
    $html .= " $file_icon\n" if (!$args{suppress_for_email});
    $html .= "<br /><b>Synonyms: </b>$keywords\n" if ($keywords);

    $html .= "<p />Submitted on $submitdate";
    $html .= "<br />Status: <b>$status_label</b> set on $status_date";

    if (!$args{suppress_for_email} && $self->is_allowed_to_do('modify', $USER) && $self->is_pending()) {
        $html .= $self->get_action_screen();
    }
    return $html;
}

sub get_action_screen {
    my $self = shift;

    my $comments = $self->get_comments();

    my $isa = $self->is_a();
    my $iid = $self->id();
    my $op  = 'handle_action';
    my $html = '';
    my $approve_label = $self->get_status_label(APPROVED);
    my $reject_label  = $self->get_status_label(REJECTED);

    $html .= "<p /><hr size=0>\n";

    $html .= <<EOM
<form method="post" action="index.pl">
  <input type="hidden" name="isa" value="$isa">
  <input type="hidden" name="iid" value="$iid">
<table border="0" width="100%" cellspacing="2" cellpadding="2">
EOM
;

    if ($self->is_allowed_to_do('can_update_status')) {
        $html .= <<EOM
  <tr>
    <td><b>Action:</b></td>
    <td>
          <input type="radio" name="op" value="add_comments" checked>Just Add Comments
          <input type="radio" name="op" value="approve">$approve_label
          <input type="radio" name="op" value="reject">$reject_label
<!--
          <input type="radio" name="op" value="delete">Delete
        <br />
-->
    </td>
  </tr>
EOM
;
    } else {
        $html .= <<EOM
          <input type="hidden" name="op" value="add_comments">
EOM
;
    }

    my $comments_field_name = $self->get_last_comments_field_name();

    my $modify_string = $self->get_www_modify_buttons();
    $html .= <<EOM
  <tr>
    <td><b>New Comments:</b></td>
    <td><textarea name="$comments_field_name" rows=4 cols=50 WRAP="VIRTUAL"></textarea></td>
  <tr>
    <td colspan='2' align='center'>$modify_string</td>
  </tr>
</table>
</form>
<p /><b>Previous Comments</b><br />$comments
EOM
;

    return $html;
}

sub show_summary {
    my $self        = shift;

    my $name        = $self->get_name();
    my $description = $self->get_description();

    my ( $file, $file_icon ) = $self->get_upload_filename_icon_and_url();
    my $buttons     = $self->get_buttons();

    my $html        = '';

    $html .= "\n$buttons " if $buttons;
    $html .= "<b>$name</b>";
    $html .= " $file_icon" if $file_icon;
    $html .= "<br />$description" if $description;
    return $html;
}

sub show_line {
    my $self        = shift;
    return $self->show_summary();
}

sub get_is_public {
    return 'yes';
}

sub get_gizmo_name {
    return 'Submission Item';
}

sub set_status {
    my $self = shift;
    my $value = shift;

    my $status_field_name = $self->get_status_field_name();

    $self->{$status_field_name} = $value;
}

sub get_status {
    my $self = shift;
    my $status_field_name = $self->get_status_field_name();

    return $self->{$status_field_name};
}

sub get_status_field_label {
    my $self = shift;
    my $status = $self->get_status();

    return $self->get_status_label($status);
}

sub get_status_label {
    my $self = shift;
    my $status = shift;

    return $STATUS_LABELS{$status};
}

sub get_status_field_name {
    my $self = shift;
    return 'i1';
}

sub get_status_date_field_name {
    my $self = shift;
    return 'd1';
}

sub get_status_date {
    my $self = shift;
    my $status_date_field_name = $self->get_status_date_field_name();

    return $self->{$status_date_field_name};
}

sub get_fullname {
    my $self = shift;
    my $fullname_field_name = $self->get_fullname_field_name();
    return $self->{$fullname_field_name};
}

sub get_email {
    my $self = shift;
    my $email_field_name = $self->get_email_field_name();
    return $self->{$email_field_name};
}

sub get_fullname_field_name {
    my $self = shift;
    return 'c1';
}

sub get_email_field_name {
    my $self = shift;
    return 'c2';
}

sub _get_access_broker {
    my $self = shift;
    return Metadot::AccessBroker::SubmissionItemBroker->new();
}

sub is_allowed_to_do {
   my $self = shift;
   my $op = shift || die "must provide operation name";
   my $user = shift || $USER;

   ## Have to handle this here because Accessbroker was complaining
   return $self->SUPER::is_allowed_to_do($op, $user) if $self->is_approved();

   my $ab = $self->_get_access_broker();
   return $ab->is_allowed_to_do($op, $user, $self);
}

sub get_approved_status_code {
    my $self = shift;
    return APPROVED;
}

sub get_pending_status_code {
    my $self = shift;
    return PENDING;
}

sub get_rejected_status_code {
    my $self = shift;
    return REJECTED;
}

sub is_approved {
    my $self = shift;
    return $self->get_status() == APPROVED ? 1 : 0;
}

sub is_pending {
    my $self = shift;
    return $self->get_status() == PENDING ? 1 : 0;
}

sub _show_create_success {
    my $self = shift;

    my $approval_required = $self->is_approval_required();

    my $parent   = $self->get_parent();
    if ($approval_required) {
        ## first email and metagram users
        $self->notify_parties(body_intro => 'Thank you for your new submission. An approver will review your submission now.', email_parties => 1, metagram_parties => 1, authorname => $self->get_fullname(), authoremail=> $self->get_email());
    }

    my $isa = $self->is_a();
    my $parentid = $self->{sub_id} || $self->parent_id();
    my $navigable_parent = $self->get_navigation_parent();
    my $message  = $approval_required ? '<p />This submission item requires approval. You will be contacted if further information is needed.' : '';

    if( !$message ){
         $self->two_choices("<Big>Submission accepted!</big>" .
                       "$message<br /><br />What do you want to do?",
                       "confirm",
                       "Add another " . $self->get_gizmo_name(),
                       "index.pl?child_type=$isa&isa=" . $parent->is_a() . "&iid=$parentid&op=add_child&app=",
                       "Enough for now!",
                       "index.pl?id=" . $navigable_parent->id() );
    }
    else{
         $self->two_choices("<Big>Submission accepted!</big>" .
                       "$message<br /><br />What do you want to do?",
                       "confirm",
                       "Add another " . $self->get_gizmo_name(),
                       "index.pl?child_type=$isa&isa=" . $parent->is_a() . "&iid=$parentid&op=add_child&app=",
                       "Enough for now!",
                       "index.pl?id=" . $navigable_parent->id());
   }
}
sub _show_edit_success {
    my $self = shift;

    if (!$self->is_approved() && (defined($FORM{save_and_email}) || defined($FORM{save_and_metagram})) ) {
        ## first email and/or users
        $self->notify_parties(body_intro => 'Submission updated.', email_parties => $FORM{save_and_email}, metagram_parties => $FORM{save_and_metagram});
    }

    return $self->SUPER::_show_edit_success(@_);
}

#sub c1 {
#    my $self = shift;
#    my $value = shift;
#
#    if ($value ne '') {
#        $self->{c1} = $value;
#    }
#
#    if ($self->{c1} eq '') {
#        $self->{c1} = $USER->fullname();
#    }
#
#    return $self->{c1};
#}

sub show_edit_form {
    my $self = shift;

    my $fullname_field_name = $self->get_fullname_field_name();
    my $email_field_name = $self->get_email_field_name();

    if ($FORM{$fullname_field_name} eq '' && $self->{$fullname_field_name} eq '' && $USER->is_logged_in()) {
        $FORM{$fullname_field_name} = $USER->get_display_name();
    }

    if ($FORM{$email_field_name} eq '' && $self->{$email_field_name} eq '' && $USER->is_logged_in()) {
        $FORM{$email_field_name} = $USER->email();
    }

    my $field_name = $self->get_last_comments_field_name();
    $self->{$field_name} = '';

    return $self->SUPER::show_edit_form(@_);
}

#sub www_save {
#    my $self = shift;
#    $self->debug_msg(3, "Gizmo::SubmissionSystem::SubmissionItem::www_save()");
#
#
#    $self->SUPER::www_save(@_);
#}

sub add {
    my $self = shift;

    my $fullname_field_name = $self->get_fullname_field_name();
    my $fullname = $FORM{$fullname_field_name} || $self->$fullname_field_name();
    my $sort_flag = 0;
    my $submission;
    $self->handle_new_comments(fullname => $fullname);

    ## DETERMINE APPROPRIATE STATUS
    my $status_field_name = $self->get_status_field_name();
    if( $self->is_approval_required() ) {
        $self->set_pending_values();
    } else { ## set to approve
        $self->set_approval_values();
        $submission = Gizmo->restore($self->{'sub_id'});
        $sort_flag = 1;
        #$self->move_obstacles();
    }
    $self->init_permissions(); ## make sure permissions are reset
    my $newid = $self->SUPER::add(@_);
    my $parent = $self->get_parent;

    if( $sort_flag ){
        $submission->post_process_ordering($newid);
        $self->SUPER::save(@_);
    }
    return $newid;

    #return $self->SUPER::add(@_);
}

sub set_pending_values {
    my $self = shift;

    my $status_field_name = $self->get_status_field_name();
    my $status_date_field_name = $self->get_status_date_field_name();
    $self->{$status_field_name} = PENDING;
    $self->{$status_date_field_name} = Utility->db_now();

}

sub set_approval_values {
    my $self = shift;
    my $approver = shift || $USER;

    my $sort_order = $self->get_parent()->get_c4();
    my $status_field_name = $self->get_status_field_name();
    my $status_date_field_name = $self->get_status_date_field_name();
    $self->{$status_field_name} = APPROVED;
    $self->{$status_date_field_name} = Utility->db_now();

    # CKIM; the approver becomes the owner
    #$self->{uid} = MetadotConfig->get_reserved_uid('ADMIN');
    $self->{uid} = $approver->uid();

    #$self->{parent_id} = $self->get_navigation_parent()->id() || 0;
    my $sub_instance = $self->get_parent()->instance_order();

    $self->{sub_id} = $self->get_parent()->get_id();
    $self->{parent_id} = $self->get_parent()->parent_id() || 0;

    $self->{sub_sort_order} = $sort_order;
    $self->{sub_instance} = $sub_instance;

}

sub set_rejection_values {
    my $self = shift;

    my $status_field_name = $self->get_status_field_name();
    my $status_date_field_name = $self->get_status_date_field_name();
    $self->{$status_date_field_name} = Utility->db_now();
    $self->{$status_field_name} = REJECTED;
    #$self->{uid} = MetadotConfig->get_reserved_uid('ADMIN');
}

sub is_approval_required {
    my $self = shift;

    my $parent = $self->get_parent();

    return defined($parent) && $parent->can('is_approval_required') && $parent->is_approval_required();
}

## This method should be called after the latest comments
##  field has been updated with the most recent value and prior
##  to saving or updating the instance object
sub handle_new_comments {
    my $self = shift;
    my %args = @_;

    my $field_name = $self->get_last_comments_field_name();

    my $last_comments = $args{$field_name} || $FORM{$field_name};

    if ($last_comments ne '') {
        ## get the header with the user's name and timestamp
        my $comments_header = $self->get_comments_header(%args);
        my $comments_field_name = $self->get_comments_field_name();
        ## prepend the header and latest comments to the comments' field
        $self->{$comments_field_name} = "<p />" . $comments_header . $last_comments . $self->{$comments_field_name} . "\n";
        ## prepend the header to the latest comments' field
        my $last_comments_field_name = $self->get_last_comments_field_name();
        $self->{$last_comments_field_name} = $comments_header . $last_comments . "\n";
    }
}

sub save {
    my $self = shift;

    my $fullname = '';
    my $id = $self->id();
    my $parent = $self->get_parent();

    my $subid = $self->{sub_id};
    my $submission;
    if( defined($subid) ){
        $submission = Gizmo->restore($subid)
    }

    #print STDERR "\n\nin save subid[$subid]\n";
    if ($USER->is_owner($self)) {
        my $fullname_field_name = $self->get_fullname_field_name();
        $fullname = $FORM{$fullname_field_name} || $self->$fullname_field_name();
    }

    $self->handle_new_comments(fullname => $fullname);
    if( defined($submission) ){
        #print STDERR "about to call process_ordering\n\n";
        $submission->post_process_ordering($id);
    }
    #######$self->last_position();
    #######$self->move_obstacles($subid);
    my $save = $self->SUPER::save(@_);

    return $save;
}

sub get_comments_field_name {
    my $self = shift;
    return 't1';
}

sub get_comments {
    my $self = shift;
    my $field_name = $self->get_comments_field_name();
    return $self->$field_name();
}

sub get_last_comments {
    my $self = shift;
    my $field_name = $self->get_last_comments_field_name();
    return $self->$field_name();
}

sub set_last_comments {
    my $self = shift;
    my $value = shift;

    if ($value ne '') {
        my $field_name = $self->get_last_comments_field_name();
        $self->{$field_name} = $value;
    }
}

sub get_last_comments_field_name {
    my $self = shift;
    return 't2';
}

sub get_comments_header {
    my $self = shift;
    my %args = @_;

    #my $fullname_field_name = $self->get_fullname_field_name();
    #my $full_name = $FORM{$fullname_field_name} || $self->$fullname_field_name();
    my $full_name = $args{fullname} || $USER->get_display_name();
    my $date = Utility->format_unix_timestamp(time, 'm/e/y r');
    return "<i>### Submitted by $full_name on $date</i><br />\n";
}

sub get_edit_form_comments {
    my $self = shift;

    my $comments = $self->get_comments();
    if ($comments ne '') {
        return "<b>Previous Comments</b>" . $self->get_comments();
    }

    return '';
}

sub get_action_links {
    my $self = shift;
    my %args = @_;

    my $hide_review = $args{hide_review} || 0;

    my $isa  = $self->get_is_a();
    my $id   = $self->id();

    my $action_links   = "";
    $action_links .= "<a href=\"index.pl?isa=$isa&amp;iid=$id&amp;op=show\"><span class=\"centerNewsLink\">Review</span></a>" unless $hide_review;
    if ($self->is_allowed_to_do('modify', $USER)) {
        if ($action_links ne '') {
            $action_links .= "-";
        }
        $action_links .= "<a href=\"index.pl?isa=$isa&amp;iid=$id&amp;op=modify\"><span class=\"centerNewsLink\">Edit</span></a>";
    }

    if ($self->is_allowed_to_do('delete', $USER)) {
        if ($action_links ne '') {
            $action_links .= "-";
        }
        $action_links .= "<a href=\"index.pl?isa=$isa&amp;iid=$id&amp;op=delete\"><span class=\"centerNewsLink\">Delete</span></a>";
    }

    #if ($self->is_allowed_to_do('change_owner', $USER)) {
    #    if ($action_links ne '') {
    #        $action_links .= "-";
    #    }
    #    $action_links .= "<a href=\"index.pl?op=change_owner&amp;isa=$isa&amp;iid=$id\"><span class=\"centerNewsLink\">Change Owner</span></a>";
    #}

    return $action_links;
}

sub is_submission_user_registered {
    my $self = shift;
    my $uid  = $self->get_uid();

    return $uid > 0 ? 1 : 0;
}

1;

__END__
