###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
#
#
############################################################################
package SuperGizmo;


use strict;
use Metadot::GizmoContainer;
use MetadotObject;
use Metadot::Controllable;
use XML::Simple;
#use Data::Dumper;
use Metadot::SuperGizmoUploadsManager;

use vars qw(@ISA);
my $version = "4.5";

sub get_version {
    return $version;
}

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CGI);

@ISA=qw(Metadot::GizmoContainer MetadotObject Metadot::Controllable);

## DATABASE FIELD TYPES
my $DB_FILE  = 'file';
my $DB_CHAR  = 'character';
my $DB_TEXT  = 'text';
my $DB_INT   = 'int';
my $DB_DATE  = 'date';
my $DB_NAME  = 'name';
my $DB_DESC  = 'description';

## FORM FIELD TYPES
my $FORM_FILE     = 'file';
my $FORM_STRING   = 'string';
my $FORM_TEXT     = 'text';
my $FORM_NUMBER   = 'number';
my $FORM_RADIO    = 'radio';
my $FORM_CHECKBOX = 'checkbox';
my $FORM_SELECT   = 'select';
my $FORM_DATE     = 'date';
my $FORM_NAME     = 'name';
my $FORM_DESC     = 'description';

sub get_is_public {
    return 'no';
}

sub get_gizmo_name {
    return "Supergizmo";
}

sub gizmo_list_decorator {
    return 'HTML::Decorator::NullDecorator';
}


sub customize_permissions {
    my $class = shift;
    my $AC = shift;

    $class->SUPER::customize_permissions($AC);

    $AC->add_bundle('BUILD_SGIZMO', 'Build ' . $class->get_gizmo_name(), '9', '9');
    $AC->add_op('BUILD_SGIZMO', 'build_supergizmo');
    $AC->add_op('BUILD_SGIZMO', 'import_supergizmo');
    $AC->add_op('BUILD_SGIZMO', 'manage_supergizmo');
    $AC->add_op('BUILD_SGIZMO', 'manage_all_supergizmos');
    $AC->add_op('BUILD_SGIZMO', 'show');
}




sub restore {
    my $class = shift;
    my @args = @_;
    my $self = $class->SUPER::restore(@args);
    $class->debug_msg(4, "SuperGizmo\->restore(@args) got $self");

    if ($self) {
        $self->_initialize_supergizmo();
    }

    return $self;
}

sub get_constructor_properties {
   my $self = shift;
   return &get_supergizmo_properties();
}

sub get_supergizmo_properties {
    return {
#        id          => {value => undef},
        name              => {value => ''},
        short_name        => {value => ''},
        author            => {value => ''},
        revision          => {value => ''},
        website           => {value => ''},
        description       => {value => ''},
        license           => {value => ''},
        read_only         => {value => 'false'},
        enabled           => {value => 'false'},
        template          => {value => ''},
        minimum_version   => {value => ''},
        supergizmo_fields => {value => {}},
        };
}



sub _initialize_supergizmo {
    use Data::Dumper;
    my $self = shift;
    $self->debug_msg(4, "SuperGizmo::_initialize_supergizmo()");

    #$DISPLAY->debug_msg(5,"supergizmo1:".Dumper($self));
    my $descriptor = $self->get_descriptor();


    $DISPLAY->debug_msg(4,"descriptor[$descriptor]");

    my $result;
    if ($descriptor) {
        #my @keys = keys %{$self->{db_field_defs}};

       my @keys = keys %{$self->{db_field_defs}};

        eval {
            $result = XMLin($descriptor, ForceArray => 1);
        };
        $result ||= {};

        ## set properties - overwrite

        #$DISPLAY->debug_msg(5,"result:".Dumper($result));


        $self->MO_set_properties($result);
    }

    $DISPLAY->debug_msg(5,"fields:".Dumper($self->get_supergizmo_fields()));


    ###################################################
    ### Defined the database fields in the instance ###
    ### table that can be used by the supergizmo    ###
    ###################################################
    $self->{db_field_defs} = {
        name     => $DB_NAME,
        description => $DB_DESC,
        file1    => $DB_FILE,
        t1       => $DB_TEXT,
        t2       => $DB_TEXT,
        t3       => $DB_TEXT,
        t4       => $DB_TEXT,
        t5       => $DB_TEXT,
        t6       => $DB_TEXT,
        t7       => $DB_TEXT,
        t8       => $DB_TEXT,
        t9       => $DB_TEXT,
        t10      => $DB_TEXT,
        c1       => $DB_CHAR,
        c2       => $DB_CHAR,
        c3       => $DB_CHAR,
        c4       => $DB_CHAR,
        c5       => $DB_CHAR,
        d1       => $DB_DATE,
        d2       => $DB_DATE,
        d3       => $DB_DATE,
        d4       => $DB_DATE,
        d5       => $DB_DATE,
        i4       => $DB_INT,
        i5       => $DB_INT,
    };
}

sub get_field_type_mapping {
    my $self = shift;
    my $mapping_type = shift || 1;

    my $mapping;
    if ($mapping_type == 1) {
        ## order important in array because preference will be
        ##  given to those db types first
        $mapping = {
            $FORM_FILE     => [$DB_FILE],
            $FORM_STRING   => [$DB_CHAR, $DB_TEXT],
            $FORM_TEXT     => [$DB_TEXT],
            $FORM_NUMBER   => [$DB_INT, $DB_TEXT],
            $FORM_RADIO    => [$DB_CHAR, $DB_TEXT],
            $FORM_CHECKBOX => [$DB_CHAR, $DB_TEXT],
            $FORM_SELECT   => [$DB_CHAR, $DB_TEXT],
            $FORM_DATE     => [$DB_DATE],
            $FORM_NAME     => [$DB_NAME],
            $FORM_DESC     => [$DB_DESC],
        };
    } elsif ($mapping_type == 2) {
        $mapping = {
            $DB_FILE     => [$FORM_FILE],
            $DB_CHAR     => [$FORM_STRING, $FORM_NUMBER, $FORM_RADIO, $FORM_CHECKBOX, $FORM_SELECT],
            $DB_TEXT     => [$FORM_TEXT, $FORM_STRING, $FORM_NUMBER, $FORM_RADIO, $FORM_CHECKBOX, $FORM_SELECT],
            $DB_DATE     => [$FORM_DATE],
            $DB_INT      => [$FORM_NUMBER],
            $DB_NAME     => [$FORM_NAME],
            $DB_DESC     => [$FORM_DESC],
        };
    }

    return $mapping;
}

sub get_all_form_field_types {
    my $self = shift;

    if (defined($self->{all_form_field_types})) {
        return $self->{all_form_field_types};
    }

    my $mapping = $self->get_field_type_mapping(2);
    my @form_types_array = ();
    foreach my $db_field_class (keys(%$mapping)) {
        push @form_types_array, @{$mapping->{$db_field_class}};
    }

    $self->{all_form_field_types} = Utility->uniquify_array(sort @form_types_array);
    return $self->{all_form_field_types};
}

sub get_used_supergizmo_db_fields {
    my $self = shift;
    $self->debug_msg(3, "SuperGizmo::get_used_supergizmo_db_fields()");

    if (defined($self->{used_supergizmo_db_fields})) {
        return $self->{used_supergizmo_db_fields};
    }

    my $fields = $self->get_supergizmo_fields();
#print STDERR "\n\nget_used_supergizmo_db_fields() fields:" . Dumper($fields) . "\n\n";
    my %hash = ();
    my $db_field = '';
    foreach my $id (keys(%$fields)) {
        #next if $id eq 'name';
        $db_field = $fields->{$id}->{instance_field};
        if ($db_field ne '') {
            $hash{$db_field} = 1;
        }
    }
    $self->{used_supergizmo_db_fields} = \%hash;
    return $self->{used_supergizmo_db_fields};
}

sub get_available_supergizmo_db_fields {
    my $self = shift;
    $self->debug_msg(3, "SuperGizmo::get_available_supergizmo_db_fields()");

    if (defined($self->{available_supergizmo_db_fields})) {
        return $self->{available_supergizmo_db_fields};
    }

    my $used_fields = $self->get_used_supergizmo_db_fields();
#print STDERR "\n\nused_fields:" . Dumper($used_fields) . "\n\n";

    my %available_fields = ();
    foreach my $db_field (keys(%{$self->{db_field_defs}})) {
        unless (exists($used_fields->{$db_field})) {
            $available_fields{$db_field} = $self->{db_field_defs}->{$db_field};
        }
    }

    $self->{available_supergizmo_db_fields} = \%available_fields;

    return $self->{available_supergizmo_db_fields};
}

sub get_available_form_field_types {
    my $self = shift;
    $self->debug_msg(3, "SuperGizmo::get_available_form_field_types()");

    if (defined($self->{available_form_field_types})) {
        return $self->{available_form_field_types};
    }

    my $available_supergizmo_db_fields = $self->get_available_supergizmo_db_fields();
#print STDERR "\n\navailable_supergizmo_db_fields:" . Dumper($available_supergizmo_db_fields) . "\n\n";
    my $mapping = $self->get_field_type_mapping(2);
#print STDERR "\n\nmapping:" . Dumper($mapping) . "\n\n";

    my @form_types_array = ();
    my $previous_value = '';
    foreach my $db_field_value (sort values(%$available_supergizmo_db_fields)) {
#print STDERR "\n\ndb_field_value:$db_field_value\n\n";
        next if $db_field_value eq $previous_value; ## already handled
        push @form_types_array, @{$mapping->{$db_field_value}};
        $previous_value = $db_field_value;
    }

    $self->{available_form_field_types} = Utility->uniquify_array(sort @form_types_array);

    return $self->{available_form_field_types};
}

########################################################
#
# Builder/Importer Methods
#
########################################################

sub www_build_supergizmo {
    my $self = shift;
    $self->debug_msg(3, "SuperGizmo::www_build_supergizmo()");
    return $self->build_supergizmo(@_);
}

sub www_import_supergizmo {
    my $self = shift;
    $self->debug_msg(3, "SuperGizmo::www_import_supergizmo()");
    return $self->import_supergizmo(@_);
}

sub build_supergizmo {
    my $self = shift;
    $self->debug_msg(3, "SuperGizmo::build_supergizmo()");

    my $isa = $self->is_a();
    my $manage_all_link = "index.pl?isa=$isa&op=manage_all_supergizmos";
    if ($FORM{'build_supergizmo_confirm'}) {
        if($FORM{name}) {
            #print STDERR "creating supergizmo named $FORM{name}\n";
            my $supergizmo = SuperGizmo->new();
            $self->_update_supergizmo_info_from_form($supergizmo);

            my $supergizmo_id = $self->add_supergizmo( { supergizmo => $supergizmo });
            if ($supergizmo_id) {
                $self->create_default_supergizmo_fields_and_template(SuperGizmo->restore($supergizmo_id));
                $self->print_choices("Sucessfully created " . $self->get_gizmo_name() . ": <br />" . $self->name, "confirm",
             "Return to manage all supergizmo", $manage_all_link);
            } else {
                $DISPLAY->print_click_back("There was an error creating your " . $self->get_gizmo_name() . ".");
            }
        } else {
            $DISPLAY->print_click_back("Please provide a name for the " . $self->get_gizmo_name() . ".");
        }
    } else {

        my $template_name = 'build_supergizmo.tt2';
        my $template = $self->get_supergizmo_template_object();
        my $data = {
                hidden_fields => [
                    { name=>'isa',              value=>'SuperGizmo'},
                    { name=>'op',               value=>'build_supergizmo'},
                    ],
                self          => $self,
        };

        my $output = '';
        $template->process($template_name, $data, \$output)
                || die $template->error(), "\n";

        return $self->print_form($output);
    }
}

sub import_supergizmo {
    my $self = shift;
    $self->debug_msg(3, "SuperGizmo::import_supergizmo()");

    my $isa = $self->is_a();
    my $manage_all_link = "index.pl?isa=$isa&op=manage_all_supergizmos";
    if ($FORM{'import_supergizmo_confirm'}) {
        if($FORM{import_supergizmo_name}) {
            my $result = Metadot::SuperGizmoUploadsManager->upload_supergizmo_import_file({filepath=>$FORM{import_supergizmo_name}, overwrite => $FORM{overwrite}});
            if (UNIVERSAL::isa($result, 'HASH') && $result->{error}) {
                $DISPLAY->print_click_back($result->{error});
                exit;
            } else {
                $self->print_choices("Sucessfully imported " . $self->get_gizmo_name() . ": <br />" . $self->name, "confirm",
                                     "Return to manage all supergizmo", $manage_all_link);
            }
        } else {
            $DISPLAY->print_click_back("Please provide an upload zip file for the " . $self->get_gizmo_name() . ".");
        }
    } else {

        my $template_name = 'import_supergizmo.tt2';
        my $template = $self->get_supergizmo_template_object();
        my $isa = $self->is_a();
        my $manage_all_link = "index.pl?isa=$isa&op=manage_all_supergizmos";
        my $data = {
                hidden_fields => [
                    { name=>'isa',              value=>'SuperGizmo'},
                    { name=>'op',               value=>'import_supergizmo'},
                    ],
                self => $self,
                manage_supergizmos_link => $manage_all_link,
        };

        my $output = '';
        $template->process($template_name, $data, \$output)
                || die $template->error(), "\n";

        return $self->print_form($output);
    }
}


sub create_default_supergizmo_fields_and_template {
    my $self = shift;
    my $supergizmo = shift;
    $self->debug_msg(4, "SuperGizmo::create_default_supergizmo_fields_and_template()");

    my $fields = $supergizmo->get_supergizmo_fields();

    {
        my $label = 'Name';
        my $type  = $FORM_NAME;
        my $required = 'true';
        my $field_id = $supergizmo->_generate_field_id($label);
        my $instance_field = $supergizmo->get_next_available_instance_field($type) || die('create_default_supergizmo_fields_and_template():could not get an instance field for $label');

    #print STDERR "\n\ncreate_default_supergizmo_fields_and_template() label:$label type:$type required:$required field_id:$field_id instance_field:$instance_field\n\n";
        $fields->{$field_id} = {
            id    => $field_id,
            label => $label,
            order => 1,
            type  => $type,
            instance_field  => $instance_field,
            required        => $required,
        };
    }

    {
        my $label = 'Description';
        my $type  = $FORM_DESC;
        my $required = 'false';
        my $field_id = $supergizmo->_generate_field_id($label);
        my $instance_field = $supergizmo->get_next_available_instance_field($type) || die('create_default_supergizmo_fields_and_template():could not get an instance field for $label');
    #print STDERR "\n\ncreate_default_supergizmo_fields_and_template() label:$label type:$type required:$required field_id:$field_id instance_field:$instance_field\n\n";

        $fields->{$field_id} = {
            id    => $field_id,
            label => $label,
            order => 2,
            type  => $type,
            instance_field  => $instance_field,
            required        => $required,
        };
    }

    $supergizmo->set_supergizmo_fields($fields);
    $supergizmo->set_template($supergizmo->rebuild_template());
    $supergizmo->update_supergizmo();
}

# updates supergizmo's info from %FORM
sub _update_supergizmo_info_from_form {
    my $class = shift;
    $class->debug_msg(4, "SuperGizmo::_update_supergizmo_template_from_form()");
    my $supergizmo = shift || die("Must provide a " . $class->get_gizmo_name());

    $supergizmo->set_name($FORM{name});
    $supergizmo->set_revision($FORM{revision});
    $supergizmo->set_author({
        fullname => $FORM{author},
        email    => $FORM{email},
        });
    $supergizmo->set_website($FORM{website});
    $supergizmo->set_description($FORM{description});
    $supergizmo->set_license($FORM{license});
    $supergizmo->set_minimum_version($FORM{minimum_version});
    $supergizmo->set_enabled($FORM{enabled}) if $supergizmo->id() > 0;

}

# builds a new supergizmo.
# one will be computed
sub add_supergizmo {
    my $class = shift;
    $class->debug_msg(3, "SuperGizmo::add_supergizmo()");
    my $params = shift;
    my $supergizmo = $params->{supergizmo} || SuperGizmo->new();
    my $descriptor = $params->{descriptor} || '';
    (defined($supergizmo) && ref($supergizmo) && $supergizmo->isa('SuperGizmo')) || die("Must pass a SuperGizmo object");
    #print STDERR "in add_supergizmo\n";

#    my $supergizmo_id = $class->_generate_id($supergizmo->get_short_name() || $supergizmo->get_name());
#    $supergizmo->set_id($supergizmo_id);
#    #print STDERR "generated supergizmo_id $supergizmo_id\n";

    $supergizmo->set_parent_iid(MetadotConfig->get_reserved_iid('HIDDEN_CATEGORY'));
    $supergizmo->set_uid(MetadotConfig->get_reserved_uid('ADMIN'));
#    $supergizmo->set_is_a($class);
#    $supergizmo->set_name('Site Image Library');

    if ($descriptor eq '') {
        $supergizmo->set_short_name($supergizmo->_generate_short_name());
        $descriptor = $class->create_descriptor($supergizmo);
    }

    $supergizmo->set_descriptor($descriptor);

    $supergizmo->_initialize_supergizmo();

    if ($supergizmo->get_short_name() eq '') {
        $class->click_back ("Error:", "add_supergizmo(): could not generate short_name.");
            exit();
    }

    $supergizmo->add();


    return $supergizmo->get_iid();
}

# create the supergizmo's descriptor
sub create_descriptor {
    my $class = shift;
    $class->debug_msg(3, "SuperGizmo::create_descriptor()");
    my $supergizmo = shift || $class;
    (defined($supergizmo) && ref($supergizmo) && $supergizmo->isa('SuperGizmo')) || die("Must pass a SuperGizmo object");

    my $get_supergizmo_properties = $class->get_supergizmo_properties();

    my $attributes = {};

    my $method = '';
    foreach my $key (keys (%$get_supergizmo_properties)) {
        $method = 'get_' . $key;
        $attributes->{$key} = $supergizmo->$method();
    };

#print STDERR "\n\nattributes:" . Dumper($attributes) . "\n\n";
    #my @keys = keys %{$supergizmo->{db_field_defs}};
    #my $result = XMLout($attributes, RootName=>'supergizmo', KeyAttr=>\@keys);
    my $result = XMLout($attributes, RootName=>'supergizmo');
    #use Data::Dumper;
    #print STDERR "generated descriptor:\n\n$result\n\n from\n",Dumper($supergizmo);

    return $result;
}


########################################################
#
# Modify / Manage Methods
#
########################################################

sub www_manage_all_supergizmos {
    my $class = shift;
    $class->debug_msg(3, "SuperGizmo::www_manage_all_supergizmo()");
    return $class->manage_all_supergizmos(@_);
}

sub manage_all_supergizmos {
    my $class = shift;
    $class->debug_msg(3, "SuperGizmo::manage_all_supergizmos()");
    my $template_name = 'manage_all_supergizmos.tt2';
    my $template = $class->get_supergizmo_template_object();
    my $supergizmos = $class->get_all_supergizmos();
    my $manage_link = "index.pl?isa=SuperGizmo&op=manage_supergizmo";
    my $add_link    = "index.pl?isa=SuperGizmo&op=build_supergizmo";
    my $import_link    = "index.pl?isa=SuperGizmo&op=import_supergizmo";

    my $isa = $class->is_a();
    my $manage_all_link = "index.pl?isa=$isa&op=manage_all_supergizmos";
    my $debug_link = $manage_all_link;
    if ($FORM{debug} == 1) {
        $debug_link .= "&debug=0";
        $SESSION->debug_level(4);
        $SESSION->save();
        $class->print_choices("Debug on", "confirm", "Return to manage all " . $class->get_gizmo_name(), $manage_all_link);
        exit(0);
    } elsif (defined($FORM{debug})) {
        $debug_link .= "&debug=1";
        $SESSION->debug_level(0);
        $SESSION->save();
        $class->print_choices("Debug off", "confirm", "Return to manage all " . $class->get_gizmo_name(), $manage_all_link);
        exit(0);
    } elsif ($SESSION->debug_level() > 0) {
        $debug_link .= "&debug=0";
    } else {
        $debug_link .= "&debug=1";
    }

    my $data = {
            self          => $class,
            supergizmos   => $supergizmos,
            manage_link   => $manage_link,
            add_link      => $add_link,
            import_link   => $import_link,
            session       => $SESSION,
            debug_link    => $debug_link,
    };

    my $output = '';
    $template->process($template_name, $data, \$output)
        || die $template->error(), "\n";
    return $DISPLAY->print_simple_layout({ content => $output });
}

sub www_manage_supergizmo {
    my $self = shift;
    $self->debug_msg(3, "SuperGizmo::www_manage_supergizmo()");
    return $self->manage_supergizmo(@_);
}

sub manage_supergizmo {
    my $self = shift;
    $self->debug_msg(3, "SuperGizmo::manage_supergizmo()");


    my $iid = $self->id();
    my $isa = $self->is_a();
#print STDERR "\n\nmanage_supergizmo:iid:#$iid# isa:$isa\n\n";
#print STDERR "\n\nsupergizmo:" . Dumper($self) . "\n\n";
    if (!defined $iid){
      $self->click_back ("Error:", "manage_supergizmo(): id is missing.");
      exit(0);
    }
    my $manage_link = "index.pl?isa=$isa&amp;op=manage_supergizmo&amp;id=$iid";
    my $manage_all_link = "index.pl?isa=$isa&amp;op=manage_all_supergizmos";

    if ($self->is_read_only()) {
        $DISPLAY->print_click_back('You cannot edit this read-only ' . $self->get_gizmo_name());
        exit;
    } elsif (defined($FORM{update_supergizmo_info})) {
        my $template_name = 'update_supergizmo.tt2';
        my $template = $self->get_supergizmo_template_object();
        my $data = {
                hidden_fields => [
                    { name=>'iid',   value=>$iid},
                    { name=>'isa',   value=>'SuperGizmo'},
                    { name=>'op',    value=>'manage_supergizmo'},
                    ],
                 supergizmo          => $self,
                 self                => $self,
        };

        my $output = '';
        $template->process($template_name, $data, \$output)
                || die $template->error(), "\n";
        return $self->print_form($output);
    } elsif (defined($FORM{update_supergizmo_info_confirm})) {
        if($FORM{name}) {
            $DISPLAY->debug_msg(3,"Updating supergizmo named $FORM{name}");
            $self->_update_supergizmo_info_from_form($self);

            if ($self->update_supergizmo()) {
                $self->print_choices("Sucessfully updated " . $self->get_gizmo_name() . " info: <br />" . $self->name, "confirm",
             "Return to manage all " . $self->get_gizmo_name(), $manage_all_link);
            } else {
                $DISPLAY->print_click_back("There was an error updating your " . $self->get_gizmo_name() . ".");
            }
        } else {
            $DISPLAY->print_click_back("Please provide a name for the " . $self->get_gizmo_name() . ".");
        }
    } elsif (defined($FORM{enable_supergizmo})) {
            $self->set_enabled('true');

            if ($self->update_supergizmo()) {
                $self->print_choices("Sucessfully enabled " . $self->get_gizmo_name() . ": <br />" . $self->name, "confirm",
             "Return to manage all " . $self->get_gizmo_name() . "s", $manage_all_link);
            } else {
                $DISPLAY->print_click_back("There was an error updating your " . $self->get_gizmo_name() . ".");
            }
    } elsif (defined($FORM{disable_supergizmo})) {
            $self->set_enabled('false');

            if ($self->update_supergizmo()) {
                $self->print_choices("Sucessfully disabled " . $self->get_gizmo_name() . ": <br />" . $self->name, "confirm",
             "Return to manage all " . $self->get_gizmo_name() . "s", $manage_all_link);
            } else {
                $DISPLAY->print_click_back("There was an error updating your " . $self->get_gizmo_name() . ".");
            }
    } elsif (defined($FORM{delete_supergizmo_confirm})) {
        if (defined($FORM{confirm})) { ## delete it
            $self->delete_supergizmo();

            #TODO: make sure this manage_link is not escaped
            Utility->md_redirect($manage_all_link);
            exit(0);
        } else {
            $self->print_choices("Are you sure you want to delete \"" . $self->name . "\"?<br \/>Doing so will delete every gizmo of type \"" . $self->name . "\".", "caution",
             "Just disable \"" . $self->name . "\"", $manage_link . "&disable_supergizmo=1",
             "Delete \"" . $self->name . "\" and all its gizmos.", $manage_link . "&delete_supergizmo_confirm=1&confirm=1",
             "Cancel", $manage_all_link);
        }
    } elsif (defined($FORM{manage_supergizmo_fields})) {
        my $template_name = 'manage_supergizmo_fields.tt2';
        my $template = $self->get_supergizmo_template_object();
        my $data = {
                 supergizmo          => $self,
                 add_supergizmo_field_link => $manage_link . "&add_supergizmo_field=1",
                 update_supergizmo_field_link => $manage_link . "&update_supergizmo_field=1",
                 delete_supergizmo_field_link => $manage_link . "&delete_supergizmo_field_confirm=1",
                 move_up_supergizmo_field_link => $manage_link . "&move_up_supergizmo_field=1",
                 move_down_supergizmo_field_link => $manage_link . "&move_down_supergizmo_field=1",
                 preview_supergizmo_link => $manage_link . "&show_preview=1",
                 manage_supergizmos_link => $manage_all_link,
        };

        my $output = '';
        $template->process($template_name, $data, \$output)
                || die $template->error(), "\n";
        return $self->print_form($output);
    } elsif (defined($FORM{update_supergizmo_template_confirm})) {
        if($FORM{template}) {
            $self->set_template($FORM{template});

            if ($self->update_supergizmo()) {
                $self->print_choices("Sucessfully updated " . $self->get_gizmo_name() . " template: <br />" . $self->name, "confirm",
             "Return to manage all " . $self->get_gizmo_name(), $manage_all_link);
            } else {
                $DISPLAY->print_click_back("There was an error updating the template of your " . $self->get_gizmo_name() . ".");
            }
        } else {
            $DISPLAY->print_click_back("Please provide a template for the " . $self->get_gizmo_name() . ".");
        }
    } elsif (defined($FORM{manage_supergizmo_template}) || defined($FORM{rebuild_supergizmo_template})) {
        my $template_name = 'manage_supergizmo_template.tt2';
        my $template = $self->get_supergizmo_template_object();

        my $supergizmo_template = '';
        ## rebuild template
        if (defined($FORM{rebuild_supergizmo_template})) {
            $supergizmo_template = $self->rebuild_template();
        }

        my $data = {
                hidden_fields => [
                    { name=>'iid',   value=>$iid},
                    { name=>'isa',   value=>'SuperGizmo'},
                    { name=>'op',    value=>'manage_supergizmo'},
                    ],
                 supergizmo          => $self,
                 manage_supergizmos_link => $manage_all_link,
                 supergizmo_template => $supergizmo_template,
        };

        my $output = '';
        $template->process($template_name, $data, \$output)
                || die $template->error(), "\n";
        return $self->print_form($output);
    } elsif (defined($FORM{download_supergizmo})) {
        return $self->_download_supergizmo_file();
    } elsif (defined($FORM{manage_supergizmo_images}) || defined($FORM{upload_image_button}) || defined($FORM{delete_image})) {
        my $template_name = 'manage_supergizmo_images.tt2';
        my $template = $self->get_supergizmo_template_object();
        my $file_text = '';

        #process an image upload
        if ($FORM{upload_image_button} && $FORM{upload_image_name}) {
            my $upload_result = Metadot::SuperGizmoUploadsManager->upload_image_file({supergizmo => $self, name=>$FORM{upload_image_name}});
            $FORM{file_selector} = $upload_result->{file_name};
            $file_text = Metadot::SuperGizmoUploadsManager->get_file_url({supergizmo => $self, name=>$FORM{file_selector}});
        } elsif ($FORM{delete_image} && $FORM{file_selector}) {
            Metadot::SuperGizmoUploadsManager->delete_file({supergizmo => $self, name=>$FORM{file_selector}});
        } elsif ($FORM{file_selector}) {
            $file_text = Metadot::SuperGizmoUploadsManager->get_file_url({supergizmo => $self, name=>$FORM{file_selector}});
        }

        my $supergizmo_images = Metadot::SuperGizmoUploadsManager->get_image_file_list( { supergizmo => $self } );
        my $data = {
                hidden_fields => [
                    { name=>'iid',   value=>$iid},
                    { name=>'isa',   value=>'SuperGizmo'},
                    { name=>'op',    value=>'manage_supergizmo'},
                    { name=>'manage_supergizmo_images',    value=>'1'},
                    ],
                 supergizmo              => $self,
                 manage_supergizmos_link => $manage_all_link,
                 supergizmo_images       => $supergizmo_images,
                 form                    => \%FORM,
                 portal_params           => $PARAMS,
                 file_text               => $file_text,
        };

        my $output = '';
        $template->process($template_name, $data, \$output)
                || die $template->error(), "\n";
        return $self->print_form($output);
    } elsif (defined($FORM{add_supergizmo_field})) {
        my $template_name = 'add_supergizmo_field.tt2';
        my $template = $self->get_supergizmo_template_object();
        my $data = {
                hidden_fields => [
                    { name=>'iid',   value=>$iid},
                    { name=>'isa',   value=>'SuperGizmo'},
                    { name=>'op',    value=>'manage_supergizmo'},
                    ],
                supergizmo          => $self,
        };

        my $output = '';
        $template->process($template_name, $data, \$output)
                || die $template->error(), "\n";
        return $self->print_form($output);
    } elsif (defined($FORM{add_supergizmo_field_confirm})) {
            #print STDERR "updating supergizmo named $FORM{name}\n";
            $self->_add_supergizmo_field_from_form();

            if ($self->update_supergizmo()) {
                $self->print_choices("Sucessfully added " . $self->get_gizmo_name() . " field \"$FORM{field_label}\" to " . $self->name . ".", "confirm",
             "Add another field", $manage_link . "&add_supergizmo_field=1",
             "Return to manage "   . $self->get_gizmo_name() . " fields", $manage_link . "&manage_supergizmo_fields=1");
            } else {
                $DISPLAY->print_click_back("There was an error updating your " . $self->get_gizmo_name() . ".");
            }
    } elsif (defined($FORM{update_supergizmo_field}) && defined($FORM{field_id})) {
        $DISPLAY->debug_msg(5,"Updating supergizmo field[$FORM{field_id}]");
        my $template_name = 'update_supergizmo_field.tt2';
        my $template = $self->get_supergizmo_template_object();
        my $data = {
                hidden_fields => [
                    { name=>'iid',   value=>$iid},
                    { name=>'isa',   value=>'SuperGizmo'},
                    { name=>'op',    value=>'manage_supergizmo'},
                    { name=>'previous_field_id',    value=>"$FORM{field_id}"},
                    ],
                supergizmo          => $self,
                field_id            => $FORM{field_id},
        };

        my $output = '';
        $template->process($template_name, $data, \$output)
                || die $template->error(), "\n";
        return $self->print_form($output);
    } elsif (defined($FORM{update_supergizmo_field_confirm}) && defined($FORM{field_id})) {
            $self->_update_supergizmo_field_from_form();

            if ($self->update_supergizmo()) {
                $self->print_choices("Sucessfully updated " . $self->get_gizmo_name() . " field: <br />" . $self->name, "confirm",
             "Return to manage "   . $self->get_gizmo_name() . " fields", $manage_link . "&amp;manage_supergizmo_fields=1");
            } else {
                $DISPLAY->print_click_back("There was an error updating your " . $self->get_gizmo_name() . ".");
            }
    } elsif (defined($FORM{delete_supergizmo_field_confirm}) && defined($FORM{field_id})) {
        my $field = $self->get_supergizmo_field($FORM{field_id});
        unless (defined($field)) {
            $DISPLAY->print_click_back("The field does not exist.");
            exit(0);
        }
        if (defined($FORM{confirm})) { ## delete it
            $self->delete_supergizmo_field($FORM{field_id});

            #TODO: make sure this manage_link is not escaped
            Utility->md_redirect($manage_link . "&manage_supergizmo_fields=1");
            exit(0);
        } else {
            $self->print_choices("Are you sure you want to delete field " . $field->{label}, "confirm",
             "Yes", $manage_link . "&delete_supergizmo_field_confirm=1&field_id=" . $FORM{field_id} . "&confirm=1",
             "No", $manage_link . "&manage_supergizmo_fields=1");
        }
    } elsif (defined($FORM{move_up_supergizmo_field}) && defined($FORM{field_id})) {
            $self->move_supergizmo_field_up($FORM{field_id});

            #TODO: make sure this manage_link is not escaped
            Utility->md_redirect($manage_link . "&manage_supergizmo_fields=1");
    } elsif (defined($FORM{move_down_supergizmo_field}) && defined($FORM{field_id})) {
            $self->move_supergizmo_field_down($FORM{field_id});

            #TODO: make sure this manage_link is not escaped
            Utility->md_redirect($manage_link . "&manage_supergizmo_fields=1");
    } elsif (defined($FORM{show_preview})) {
        my $template_name = 'preview_supergizmo.tt2';
        my $template = $self->get_supergizmo_template_object();

        my $supergizmo_parent_iid_field = SuperGizmoRecord->get_supergizmo_parent_iid_field();
        my $child_constructor_args = {};
        $child_constructor_args->{$supergizmo_parent_iid_field} = $iid;
        #$child_constructor_args->{parent_id} = $parentid;
        my $child = SuperGizmoRecord->new($child_constructor_args);
        my $data = {
                supergizmo          => $self,
                supergizmo_child    => $child,
        };


        my $output = '';
        $template->process($template_name, $data, \$output)
                || die $template->error(), "\n";
        return $self->print_form($output);
    } else {
         $DISPLAY->print_click_back("Nothing to do for " . $self->get_gizmo_name() . ".");
    }

}

sub _download_supergizmo_file {
    my $self = shift;
    $self->debug_msg(4, "SuperGizmo::_download_supergizmo_file()");

    my $clean_name = $self->clean_string($self->get_name());
    my $file = $clean_name . '.zip';

    my $read_file = Metadot::SuperGizmoUploadsManager->export_zip({ supergizmo => $self, file => $file, name => $clean_name });
    if (defined $read_file) {
        $self->transmit_file(
            file        => $file,
            read_file   => $read_file,
        );
    } else {
        $self->print_click_back('Could not find the requested supergizmo.');
        exit;
    }
}

sub rebuild_template {
    my $self = shift;
    $self->debug_msg(4, "SuperGizmo::rebuild_template()");

        my $template_name = 'rebuild_template.tt2';
        my $template = $self->get_supergizmo_template_object();
        my $data = {
                supergizmo          => $self,
        };

        my $output = '';
        $template->process($template_name, $data, \$output)
                || die $template->error(), "\n";

        return $CGI->unescapeHTML($output);

}

sub reorder_supergizmo_fields {
    my $self = shift;
    $self->debug_msg(4, "SuperGizmo::reorder_supergizmo_fields()");

    my $fields = $self->get_supergizmo_fields();

    my $order = 1;
    foreach my $field_id (sort {$fields->{$a}->{order} <=> $fields->{$b}->{order}} keys (%$fields)) {
        $fields->{$field_id}->{order} = $order++;
    }
    $self->set_supergizmo_fields($fields);
    $self->update_supergizmo();
}

sub move_supergizmo_field_down {
    my $self = shift;
    my $field_id = shift;

    my $fields = $self->get_supergizmo_fields();
    unless (exists($fields->{$field_id})) {
        $DISPLAY->print_click_back("move_supergizmo_field_down(): Field does not exist.");
        exit(0); ## do not change
    }

    my $current_field_order = $fields->{$field_id}->{order};
    my $total_fields        = scalar keys %$fields;

    if ($current_field_order == $total_fields) { ## wrap
        $fields->{$field_id}->{order} = -1;
    } else {
        $fields->{$field_id}->{order} += 1.5;
    }


    $self->reorder_supergizmo_fields();
}

sub move_supergizmo_field_up {
    my $self = shift;
    my $field_id = shift;

    my $fields = $self->get_supergizmo_fields();
    unless (exists($fields->{$field_id})) {
        $DISPLAY->print_click_back("move_supergizmo_field_up(): Field does not exist.");
        exit(0); ## do not change
    }

    my $current_field_order = $fields->{$field_id}->{order};
    my $total_fields        = scalar keys %$fields;

    if ($current_field_order == 1) { ## wrap
        $fields->{$field_id}->{order} = $total_fields + 1;
    } else {
        $fields->{$field_id}->{order} -= 1.5;
    }


    $self->reorder_supergizmo_fields();
}

sub _add_supergizmo_field_from_form {
    my $self = shift;
    $self->debug_msg(4, "SuperGizmo::_add_supergizmo_field_from_form()");

    return $self->_update_supergizmo_field_from_form(adding_new => 1);
}

sub _update_supergizmo_field_from_form {
    my $self = shift;
    my $adding_new = shift || 0;
    $self->debug_msg(4, "SuperGizmo::_update_supergizmo_field_from_form()");

    unless (defined($FORM{field_label}) && $FORM{field_label} ne '') {
        $DISPLAY->print_click_back("Error: Field label not defined.");
        exit(0); ## do not change
    }

    unless (defined($FORM{field_id}) && $FORM{field_id} ne '') {
        $DISPLAY->print_click_back("Error: Field programming name not defined.");
        exit(0); ## do not change
    }

    unless ($self->verify_programming_name($FORM{field_id})) {
        $DISPLAY->print_click_back("Error: Field programming name contains illegal characters.");
        exit(0); ## do not change
    }
    $self->debug_msg(4, "SuperGizmo::_update_supergizmo_field_from_form() field_type:$FORM{field_type}");

    my $fields = $self->get_supergizmo_fields();
    my $field_id = $FORM{field_id};
    my $prepend = '';
    my $instance_field = '';

    if ($adding_new) {
        $prepend = $FORM{field_type};
        $self->debug_msg(4, "SuperGizmo::_update_supergizmo_field_from_form() adding new field");
        unless (defined($prepend) && $prepend ne '') {
            $DISPLAY->print_click_back("Error: Field type not defined.");
            exit(0); ## do not change
        }
        #$field_id = $self->_generate_field_id($FORM{field_label});
        $instance_field = $self->get_next_available_instance_field($FORM{field_type});
        $self->debug_msg(4, "SuperGizmo::_update_supergizmo_field_from_form() adding new field with instance field:$instance_field");

        unless (defined($instance_field) && $instance_field ne '') {
            $DISPLAY->print_click_back("No database field available for new field.");
            exit(0); ## do not change
        }
    } else {
        my $previous_field_id = $self->clean_string($FORM{previous_field_id});
        unless (exists($fields->{$previous_field_id})) {
            $DISPLAY->print_click_back("_update_supergizmo_field_from_form(): Field does not exist.");
            exit(0); ## do not change
        }
        $prepend = $fields->{$previous_field_id}->{type};
        unless (defined($prepend)) {
            $DISPLAY->print_click_back("Error: Field type not defined.");
            exit(0); ## do not change
        }
    }

    ## CHECKS FOR RADIO AND SELECT FIELDS
    my $possible_values = $FORM{$prepend . "_field_values"} || '';
    my @possible_values = ();
    foreach (split "\n", $possible_values) {
        $_ =~ s/\r//g;
        push @possible_values, $_;
    }

    if (($prepend eq 'radio' || $prepend eq 'select') && $possible_values[0] eq '') {
        $DISPLAY->print_click_back("Error: You must specify field values.");
        exit(0); ## do not change
    }
    ##

    my $field_order = 1;
    my $field_type  = '';
    ## CHECKS FOR FIELD ID
    if ($adding_new) {

        ## check if field_id exists
        if (exists($fields->{$field_id})) {
            $DISPLAY->print_click_back("Programming name \"$field_id\" already exists.");
            exit(0); ## do not change
        }

        $field_order = $self->get_new_field_order();
        $field_type  =  $FORM{field_type};
#        $instance_field  = $instance_field;

#        $fields->{$field_id} = {
#            id    => $field_id,
#            label => $FORM{field_label},
#            order => $self->get_new_field_order(),
#            default_value => $FORM{$prepend . "_field_default_value"} || '',
#            possible_values => \@possible_values,
#            type            => $FORM{field_type},
#            instance_field  => $instance_field,
#            #size            => $FORM{$prepend . "_field_size"} || '',
#            required        => $FORM{$prepend . "_field_required"} ? 'true' : 'false',
#        };
    } else { ## updating

        my $previous_field_id = $self->clean_string($FORM{previous_field_id});

        if ($previous_field_id ne $FORM{previous_field_id} || !exists($fields->{$previous_field_id})) {
            $DISPLAY->print_click_back("Error processing.");
            exit(0); ## do not change
        }

        $field_order = $fields->{$previous_field_id}->{order};
        $field_type  = $fields->{$previous_field_id}->{type};
        $instance_field  = $fields->{$previous_field_id}->{instance_field};

        ## check if field_id exists
        if ($FORM{previous_field_id} ne $field_id) {
            if(exists($fields->{$field_id})) {
                $DISPLAY->print_click_back("Programming name \"$field_id\" already exists.");
                exit(0); ## do not change
            } else {
                delete($fields->{$previous_field_id});
            }
        }

    }
    ## END CHECKS FOR FIELD ID

    ## okay, add or update field
    $fields->{$field_id}->{id} = $field_id;
    $fields->{$field_id}->{label} = $FORM{field_label};
    $fields->{$field_id}->{help} = $FORM{field_help};
    $fields->{$field_id}->{order} = $field_order;
    $fields->{$field_id}->{default_value} = $FORM{$prepend . "_field_default_value"} || '';
    $fields->{$field_id}->{possible_values} = \@possible_values;
    #$fields->{$field_id}->{size} = $FORM{$prepend . "_field_size"} || '';
    $fields->{$field_id}->{type} = $field_type;
    $fields->{$field_id}->{instance_field} = $instance_field;
    $fields->{$field_id}->{required} = $FORM{$prepend . "_field_required"} ? 'true' : 'false';

    #NZ:Add image/file fields to the set of supergizmo fields
    $fields->{$field_id}->{show_image} = $FORM{"file_show_image"} ? 'true' : 'false';
    $fields->{$field_id}->{image_width_max} = $FORM{"file_constraint_width"} ? $FORM{"file_constraint_width"} : '';
    $fields->{$field_id}->{image_height_max} = $FORM{"file_constraint_height"} ? $FORM{"file_constraint_height"} : '';

    $self->set_supergizmo_fields($fields);
}

sub delete_supergizmo_field {
    my $self = shift;
    $self->debug_msg(4, "SuperGizmo::delete_supergizmo_field()");

    my $fields = $self->get_supergizmo_fields();
    my $field_id = $FORM{field_id};
    my $instance_field = $fields->{$field_id}->{instance_field};
    my $type           = $fields->{$field_id}->{type};
    unless (exists($fields->{$field_id})) {
        $DISPLAY->print_click_back("delete_supergizmo_field(): Field does not exist.");
        exit(0); ## do not change
    }
    delete($fields->{$field_id});
    $self->set_supergizmo_fields($fields);
    $self->reorder_supergizmo_fields(); ## will update supergizmo

    $self->clear_out_record_field_values({ instance_field => $instance_field, type => $type });
}

sub clear_out_record_field_values {
    my $self = shift;
    my $args = shift;
    $self->debug_msg(4, "SuperGizmo::clear_out_record_field_values()");

    my $id = $self->id();
    return unless $id;

    my $instance_field = $args->{instance_field};
    my $type           = $args->{type};

    return unless defined($instance_field) && defined($type);

    my $records = $self->get_supergizmo_children_record_set();
    return unless defined($records);

    while(my ($riid)=$records->fetchrow()) {
        # try to instantiate and delete the instance
        eval {
            my $gizmo = SuperGizmoRecord->new($riid);
            if ($type eq 'file') {
                $gizmo->_delete_upload(
                    id          => $gizmo->id(),
                    isa         => $gizmo->is_a(),
                    field_name  => 'item_attachment_file',
                    write_dir   => $self->get_write_dir_namespace($args),
                );
            } else {
                $gizmo->{$instance_field} = $gizmo->get_constructor_properties()->{$instance_field}->{value}; ## set to default value
                $gizmo->save();
            }
        };
        if ($@) {
            $self->print_click_back("Error", "There was a problem deleting SuperGizmoRecord ($riid)");
        }
    }
    $records->finish();
}

sub get_supergizmo_children_record_set {
    my $self = shift;
    my $args = shift;
    $self->debug_msg(3, "SuperGizmo::get_supergizmo_children_record_set()");

    my $id = $self->id();
    return unless $id;

    my $select_fields = $args->{select_fields} || 'iid';

    my $parent_field = SuperGizmoRecord->get_supergizmo_parent_iid_field();
    my $records = DBAccess->sqlSelectMany($select_fields, "instance", "isa='SuperGizmoRecord' and $parent_field = $id");
    return $records;
}

sub delete {
    my $self = shift;
    $self->debug_msg(4, "SuperGizmo::delete()");

    my $id = $self->id();
    return unless $id;

    my $records = $self->get_supergizmo_children_record_set();
    return unless defined($records);

    my $dir = Metadot::SuperGizmoUploadsManager->get_supergizmo_dir_path( { supergizmo => $self } );

    while(my ($riid)=$records->fetchrow()) {
        # try to instantiate and delete the instance
        eval {
            my $gizmo = SuperGizmoRecord->new($riid);
            $gizmo->delete("YES");
        };
        if ($@) {
            $self->print_click_back("Error", "There was a problem deleting SuperGizmoRecord ($riid)");
        }
    }
    $records->finish();


    my $result = $self->_delete_directory($dir);

    ## delete the supergizmo itself
    $self->SUPER::delete();
}

sub delete_supergizmo {
    my $self = shift;
    $self->debug_msg(4, "SuperGizmo::delete_supergizmo()");
    return $self->delete();
}

# generates a mostly unique id for a field. Provide field name as the base (optional).
sub _generate_field_id {
    my $self = shift;
    my $name = shift;
    my $id_base = $name || '';

    #$id_base = $name . "_" if ($name eq 'name');

    my $fields = $self->get_supergizmo_fields();

    # first transform all whitespace and non-word characters
    #$id_base =~ s/\s/_/g;
    #$id_base =~ s/\W//g;
    $id_base = $self->clean_string($id_base);

    # make sure there is some value by the end of this
    $id_base ||= 'field';
    $id_base = lc($id_base); ## lower case since easier to handle as key

    #print STDERR "generating id against '$name'; got base '$id_base'\n";
    # see if this id is in use, or keep trying to find an available id
    my $id = $id_base;
    my $sequence = 1;
    while (exists($fields->{$id})) {
        $id = $id_base.'_'.$sequence++;
    }

    return $id;
}

sub _generate_short_name {
    my $self = shift;
    my $name = shift || $self->get_name();
    my $id_base = $name || '';

    $id_base = $self->clean_string($id_base);

    # make sure there is some value by the end of this
    $id_base ||= 'supergizmo';

    #print STDERR "generating short name against '$name'; got base '$id_base'\n";
    # see if this id is in use, or keep trying to find an available id
    my $id = $id_base;
    my $sequence = 1;
    while (-e Metadot::SuperGizmoUploadsManager->get_image_path({ supergizmo => $self, short_name => $id })) {
        $id = $id_base.'_'.$sequence++;
    }

    $DISPLAY->debug_msg(5, "$self generated short name '$id' using name '$name' and base '$id_base'");
    return $id;
}

sub clean_string {
    my $self = shift;
    my $string = shift;

    # first transform all whitespace and non-word characters
    $string =~ s/\s/_/g;
    $string =~ s/\W//g;

    return $string;
}


sub verify_programming_name {
    my $self = shift;
    my $string = shift;

    return if !defined($string) || $string eq '';

    my $clean_string = $self->clean_string($string);

    return $string eq $clean_string;
}



sub get_next_available_instance_field {
    my $self = shift;
    $self->debug_msg(4, "SuperGizmo::get_next_available_instance_field()");
    my $field_type = shift || die('get_next_available_instance_field():must provide field type');

use Data::Dumper;

    my $temp_mapping = $self->get_field_type_mapping(1);
#print STDERR "\n\ntemp_mapping:" . Dumper($temp_mapping) . "\n\n";
    my @possible_db_types = @{$temp_mapping->{$field_type}};
#print STDERR "\n\npossible_db_types:" . Dumper(\@possible_db_types) . "\n\n";

    my $current_type = '';
    my $used_fields = $self->get_used_supergizmo_db_fields();
#print STDERR "\n\nused_fields:" . Dumper($used_fields) . "\n\n";

    my $unused_db_field = undef;
#print STDERR "\n\ndb_fields_defs:" . Dumper($self->{db_field_defs}) . "\n\n";
    while (!defined($unused_db_field) && $possible_db_types[0] ne '') {
        $current_type = shift @possible_db_types;
#print STDERR "\n\ncurrent_type:$current_type\n\n";
        $self->debug_msg(4, "SuperGizmo::get_next_available_instance_field() current_type:$current_type");
        foreach my $db_field (sort keys(%{$self->{db_field_defs}})) {
            next unless $self->{db_field_defs}->{$db_field} eq $current_type;
#print STDERR "\n\ndb_field:$db_field\n\n";
            $self->debug_msg(4, "SuperGizmo::get_next_available_instance_field() db_field:$db_field");
            if (!exists($used_fields->{$db_field})) {
                $unused_db_field = $db_field;
                last;
            }
        }
    }
#print STDERR "\n\nunused_db_field:$unused_db_field\n\n";

    return $unused_db_field;
}


# updates an existing supergizmo.
sub update_supergizmo {
    my $self = shift;
    my $params = shift;
    $self->debug_msg(3, "SuperGizmo::update_supergizmo()");

    my $descriptor = $params->{descriptor} || '';

    if ($descriptor eq '') {
        $descriptor = $self->create_descriptor($self);
    }
    $self->_initialize_supergizmo();

    if ($self->get_short_name() eq '') {
        $self->click_back ("Error:", "update_supergizmo(): short_name does not exist.");
            exit();
    }
    $self->set_descriptor($descriptor);
    $self->save();

    return $self->get_iid();
}

########################################################
#
# Template Object
#
########################################################

my $SUPERGIZMO_TEMPLATE_OBJECT = undef;

# returns a class variable that holds a Template processing object
sub get_supergizmo_template_object {
    unless(defined $SUPERGIZMO_TEMPLATE_OBJECT) {
        #my @INCLUDE_PATH = Metadot::SkinManager->get_instance()->get_template_include_path();
        $SUPERGIZMO_TEMPLATE_OBJECT = Template->new({
            #INCLUDE_PATH => ['Metadot/templates/supergizmo','Metadot/templates',@INCLUDE_PATH],
            INCLUDE_PATH => ['Metadot/templates/supergizmo','Metadot/templates'],
            }) || die $Template::ERROR, "\n";
    }
    return $SUPERGIZMO_TEMPLATE_OBJECT;
}


########################################################
#
# Accessors and mutators
#
########################################################

sub author {
    my $self = shift;
    my $result = $self->_author();

    if (ref($result) eq 'ARRAY') {
        $result = $result->[0];
    }

    return $result;
}




sub get_template_code {
    my $self = shift;
    my $val = $self->access_property('template', @_);
    if (ref($val) eq 'ARRAY') {
        $val = $val->[0];
    }

    return $val;
}

sub get_template {
    my $self = shift;
    return $self->access_property('template', @_);
}

sub set_template {
    my $self = shift;
    return $self->mutate_property('template', [@_]);
}





sub is_read_only {
    my $self = shift;
    my $read_only_value = $self->get_read_only();
    return ( $read_only_value && lc($read_only_value) ne 'false');
}

sub set_enabled {
    my $self = shift;
    my $enabled = shift;

    $enabled = defined($enabled) && $enabled ne 'false' ? 'true' : 'false';
    return $self->mutate_property('enabled', $enabled);
}

sub is_enabled {
    my $self = shift;
    my $enabled_value = $self->get_enabled();
    return ( $enabled_value && lc($enabled_value) eq 'true');
}

sub set_descriptor {
    my $self = shift;
    my $value = shift;
    $self->set_t1($value);
}

sub get_descriptor {
    my $self = shift;
    $self->get_t1();
}

sub supergizmo_fields {
    my $self = shift;
    my $result = $self->_supergizmo_fields(@_);
    unless (ref($result) eq 'HASH') {
        $result = {};
    }

    return $result;
}


########################################################
#
# END Accessors and mutators
#
########################################################



sub get_supergizmo_field {
    my $self = shift;
    my $field_id = shift;

    return {} unless defined($field_id);

    my $fields = $self->get_supergizmo_fields();
    if (exists($fields->{$field_id})) {
        return $fields->{$field_id};
    } else {
        return {};
    }
}



sub get_supergizmo_fields_sorted {
    my $self = shift;
    my $fields = $self->get_supergizmo_fields();

    my @fields = ();
    foreach my $field_id (sort {$fields->{$a}->{order} <=> $fields->{$b}->{order}} keys (%$fields)) {
        push (@fields, $fields->{$field_id});
    }
#use Data::Dumper;
#print STDERR "\n\nsorted fields:" . Dumper(\@fields) . "\n\n";
    return \@fields;
}

sub get_new_field_order {
    my $self = shift;
    my $fields = $self->get_supergizmo_fields_sorted();

    if ($fields->[0]) {
       return 1 + $fields->[$#$fields]->{order};
    } else {
       return 1;
    }
}

sub get_name_field_programming_name {
    my $self = shift;
    my $fields = $self->get_supergizmo_fields();
#use Data::Dumper;
#print STDERR "\n\get_name_field_programming_name() fields:" . Dumper($fields) . "\n\n";
    my $pname;
    foreach my $field (keys(%$fields)) {
#print STDERR "\n\nget_name_field_programming_name() field:$field type:" . $fields->{$field}->{type} . "\n\n";
        next unless $fields->{$field}->{type} eq $FORM_NAME;
        $pname = $fields->{$field}->{id};
        last;
    }
#print STDERR "\n\nget_name_field_programming_name() pname:$pname\n\n";

    return $pname;
}

sub get_description_field_programming_name {
    my $self = shift;
    my $fields = $self->get_supergizmo_fields();
    my $pname;
    foreach my $field (keys(%$fields)) {
        next unless $fields->{$field}->{type} eq $FORM_DESC;
        $pname = $fields->{$field}->{id};
        last;
    }
#print STDERR "\n\nget_description_field_programming_name() pname:$pname\n\n";

    return $pname;
}

sub does_field_exist {
    my $self = shift;
    my $field_id = shift;

    return 0 unless (defined($field_id) && $field_id ne '');

    #my $fields = $self->get_supergizmo_fields();
    my $field_hash = $self->get_supergizmo_field($field_id);

    return (defined($field_hash->{id}));
}

## used to create "Add a gizmo..." Supergizmo list
sub get_add_supergizmos_hash {
    my $class = shift;

    my $isa = ref($class) || $class;
    my $r = DBAccess->sqlSelectMany("iid", "instance", "isa='$isa'");

    my %supergizmos;
    my $supergizmo;
    my $name = '';
    my $key  = '';
    while (my ($iid) = $r->fetchrow) {
        $supergizmo = $isa->restore($iid);
        if ($supergizmo->is_enabled()) {
            $name = $supergizmo->get_name();
            $key  = $class->get_add_new_interface($iid);
            $supergizmos{$key} = $name;
        }
    }

    return \%supergizmos;
}

## interface used when creating a new supergizmo record of type supergizmo
sub get_add_new_interface {
    my $self = shift;
    my $iid   = shift || $self->id();
    return unless $iid;

    return "SuperGizmoRecord_$iid";
}

sub get_number_of_supergizmo_records {
    my $self = shift;
    $self->debug_msg(3, "SuperGizmo::get_number_of_supergizmo_records()");
    my $iid = $self->id();
    return unless $iid;

    my $count_rs = $self->get_supergizmo_children_record_set( {select_fields => 'count(*)'} );

    my $count = $count_rs->fetchrow();
    $count_rs->finish();

    return $count || 0;
}


sub get_www_add_child_args {
    my $class  = shift;
    my $args   = shift;

    my $param_child_isa        = $args->{child_isa};
    my $child_constructor_args = $args->{child_constructor_args} || {};

    $class->debug_msg(3, "SuperGizmo::get_www_add_child_args()");

    my ($child_isa, $supergizmo_id) = split('_', $param_child_isa);
    my $supergizmo_parent_iid_field = $child_isa->get_supergizmo_parent_iid_field();
    $supergizmo_id ||= $FORM{$supergizmo_parent_iid_field};

    unless (Utility->is_valid_integer($supergizmo_id)) {
        $DISPLAY->print_click_back("There was an error creating a record for " . $class->get_gizmo_name() . ".");
        exit(0);
    }

    $child_constructor_args->{$supergizmo_parent_iid_field} = $supergizmo_id;
    return ($child_isa, $child_isa->get_gizmo_name($supergizmo_id), $child_constructor_args);
}



# recursively deletes the named directory
sub _delete_directory {
    my $class = shift;
    my $dir_name = shift || return; #die("Must specify the directory name");
    my $result = 1;

    # get the files and subdirectories
    opendir DIRECTORY, $dir_name;
    my @subdirs = ();
    foreach my $entry (grep !/^\.\.?$/, readdir DIRECTORY) {
        #print STDERR "evaluating entry: $entry\n";
        if (-d "$dir_name/$entry") {
            push @subdirs, $entry;
        } else {
            (unlink "$dir_name/$entry");
        }
    }
    closedir DIRECTORY;

    # delete the subdirectories
    foreach my $entry (@subdirs) {
        $result &&= $class->_delete_directory($dir_name.'/'.$entry);
    }

    $result &&= (rmdir $dir_name);
    return $result;
}

sub does_supergizmo_exist {
    my $class = shift;
    $class->debug_msg(3, "SuperGizmo::get_number_of_supergizmo_records()");
    my $params = shift;

    my $match_short_name = $params->{short_name};
    return unless $match_short_name;

    my $supergizmos = $class->get_all_supergizmos();

    my $found_iid = 0;
    foreach my $supergizmo (@$supergizmos) {
        if ($supergizmo->get_short_name() eq $match_short_name) {
            $found_iid = $supergizmo->get_iid();
            last;
        }
    }

    return $found_iid;
}

sub get_all_supergizmos {
    my $class = shift;
    $class->debug_msg(3, "SuperGizmo::get_all_supergizmos()");
    my $hidden_category = Gizmo->restore(MetadotConfig->get_reserved_iid('HIDDEN_CATEGORY'));
    my $supergizmos = $hidden_category->get_childs_list( {type => 'SuperGizmo', additional_statements => 'order by name'} );
    return $supergizmos;
}


1;
