###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
#
#
############################################################################
package SuperGizmoRecord;


use strict;
use Metadot::Controllable;
use Gizmo;
use GizmoBuilder;
use Gizmo::SuperGizmo;
use vars qw(@ISA);
my $version = "4.5";

sub get_version {
    return $version;
}

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY);

@ISA=qw(GizmoBuilder Metadot::Controllable);

my $is_public = "no";

sub get_gizmo_name {
    my $self          = shift;
    my $supergizmo_id = shift;

    $supergizmo_id = undef unless Utility->is_valid_integer($supergizmo_id);
    my $name = 'Supergizmo Record';

    if (!defined($supergizmo_id) && ref($self)) {
        $supergizmo_id = $self->get_supergizmo_parent_iid();
    }

    if ($supergizmo_id) {
        $name = SuperGizmo->restore($supergizmo_id)->get_name();
    }

    return $name;
}



sub customize_permissions {
    my $class = shift;
    my $AC = shift;

    $class->SUPER::customize_permissions($AC);

    $AC->add_op('MOD', 'show_edit_form');
}


sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

    $self->set_supergizmo_record_fields();

}

sub set_supergizmo_record_fields {
    my $self = shift;
    $self->debug_msg(4, "SuperGizmoRecord::set_supergizmo_record_fields()");

    my $supergizmo = $self->get_supergizmo_parent();
    return unless defined($supergizmo);

    my @field_info = ($self->get_supergizmo_parent_iid_field(), undef, 1);
    my $instance_field = '';
    my $programming_name = '';
    my %programming_name_hash = ();
    foreach my $field (@{$supergizmo->get_supergizmo_fields_sorted()}) {
        $instance_field = $self->get_proper_instance_field($field->{instance_field});
        $programming_name = $field->{id};
        $programming_name_hash{$programming_name} = $instance_field;
        push @field_info, ($instance_field, "<b>" . $field->{label} . "</b>", $field->{required}  eq 'true' ? 1 : 0);
    }
    $self->set_field_info (@field_info);
    ## defined mapping between supergizmo field ids and instance fields
    $self->set_programming_name_hash(\%programming_name_hash);

    ## set form types && help buttons
    $self->set_field_type($self->get_supergizmo_parent_iid_field(), 'hidden');
    my $type = '';
    my $mappings = $self->get_field_form_mappings();
    foreach my $field (@{$supergizmo->get_supergizmo_fields_sorted()}) {
        $instance_field = $self->get_proper_instance_field($field->{instance_field});
        $type = $mappings->{$field->{type}} || $field->{type};
        $self->set_field_type($instance_field, $type);
        if ($field->{type} eq 'radio' && defined($field->{possible_values})) {
            my @radiooptions = ();
            foreach my $value (@{$field->{possible_values}}) {
                push @radiooptions, {value => $value, label=> $value};
            }
            $self->{fields}->{$field->{instance_field}}->{radiooptions} = \@radiooptions;
        } elsif ($field->{type} eq 'select' && defined($field->{possible_values})) {
            my @selectoptions = ();
            foreach my $value (@{$field->{possible_values}}) {
                push @selectoptions, $value;
            }
            $self->{fields}->{$field->{instance_field}}->{selectoptions} = join(',', @selectoptions);
        }

        ## set help
        if (defined($field->{help}) && $field->{help} ne '') {
            $self->{fields}->{$instance_field}->{help} = $field->{help};
        }
    }
}

sub get_proper_instance_field {
    my $self = shift;
    my $field = shift;

    my $instance_field = $field;

    if ($field eq 'file1') {
        $instance_field = 'item_attachment_file';
    }

    return $instance_field;
}

sub set_programming_name_hash {
    my $self = shift;
    my $hash = shift;

    $self->{programming_name_hash} = $hash || {};
}

sub get_programming_name_hash {
    my $self = shift;
    return $self->{programming_name_hash};
}


sub get_instance_field {
    my $self = shift;
    my $field_name = shift;
    my $hash = $self->get_programming_name_hash();

    my $instance_field = $self->get_proper_instance_field($hash->{$field_name});

    return $instance_field;
}

sub get_field_value {
    my $self = shift;
    my $field_name = shift;

    my $hash = $self->get_programming_name_hash();

    my $instance_field = $self->get_proper_instance_field($hash->{$field_name});

    if (defined($instance_field)) {
        return $self->$instance_field;
    } else {
        return '';
    }
}

sub get_field_label {
    my $self = shift;

    my $field_name = shift;

    my $hash = $self->get_programming_name_hash();

    my $instance_field = $self->get_proper_instance_field($hash->{$field_name});

    if (defined($instance_field)) {
        return $self->{fields}->{$instance_field}->{label};
    } else {
        return '';
    }
}

sub get_field_form_mappings {
    my $self = shift;

    return {
       name  => 'text',
       description => 'mediumtext',
       date  => 'smalltext',
       file  => Metadot::UploadsManager::UPLOAD_TYPE_USER_SELECTED(),
       number=> 'smalltext',
       string=> 'smalltext',
       text  => 'longtext',
    };
}

sub restore {
    my $class = shift;
    my @args = @_;
    my $self = $class->SUPER::restore(@args);
    $class->debug_msg(4, "SuperGizmoRecord\->restore(@args) got $self");
#print STDERR "\n\nrestore() self:" . Dumper($self) . "\n\n";

    if ($self) {
        $self->set_supergizmo_record_fields();
    }

    return $self;
}

sub get_upload_filename_icon_and_url {
    my $self = shift;
    return $self->SUPER::get_upload_filename_icon_and_url( {
        id => $self->id(),
        isa => $self->is_a(),
        field_name => 'item_attachment_file',
    });
}

sub get_upload_filename_iconsrc_and_url {
    my $self = shift;

    return $self->SUPER::get_upload_filename_iconsrc_and_url( {
        id => $self->id(),
        isa => $self->is_a(),
        field_name => 'item_attachment_file',
    });
}


# this method handles what to do when you are finished successfully creating a new gizmo
sub _show_create_success {
    my $self = shift;

    my $supergizmo_parent = $self->get_supergizmo_parent();
    my $add_isa = $supergizmo_parent->get_add_new_interface();
    my $parentid = $self->parent_id();

    $self->two_choices("<Big>New " . $self->get_gizmo_name() .
                       " added!</big><br /><br />What do you want to do?",
                       "confirm",
                       "Add another " . $self->get_gizmo_name(),
                       "index.pl?child_type=$add_isa&isa=".($FORM{isa} || 'Category')."&iid=$parentid&op=add_child&app=",
                       "Enough for now!",
                       "index.pl?id=$parentid");

}


sub get_supergizmo_parent {
    my $self = shift;
    $self->debug_msg(3, "SuperGizmoRecord::get_supergizmo_parent()");

    my $supergizmo_iid = $self->get_supergizmo_parent_iid();
    return undef unless defined($supergizmo_iid);
    return SuperGizmo->restore($supergizmo_iid);
}

sub get_supergizmo_parent_iid_field {
    return 'i1';
}

sub get_supergizmo_parent_iid {
    my $self = shift;
    my $field = $self->get_supergizmo_parent_iid_field();
    return $self->{$field};
}

sub get_supergizmo_template {
    my $self = shift;

    my $supergizmo = $self->get_supergizmo_parent();
    return unless defined($supergizmo);

    return $supergizmo->get_template_code();
}

sub exec_show {
    my $self = shift;


    my $template = $self->get_supergizmo_template();

    if (defined($template) && $template ne '') {
        return $template;
    } else {
        return 'supergizmo_record.tt2';
    }
}

sub exec_show_summary {
    my $self = shift;

    return $self->exec_show();
}



1;
