###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Table;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS);

use Carp;
use Gizmo;
use DBAccess;
use Gizmo::Table::Row;
use HTML::Verify;
use HTML::Components;
use Metadot::Report;
use Metadot::Controllable;
use Metadot::GizmoContainer;

use CGI qw/escapeHTML unescapeHTML/;
use vars qw(@ISA);
@ISA=qw(GizmoBuilder Metadot::Report Metadot::Controllable Metadot::GizmoContainer);
my $version = "4.5";

sub get_version {
    return $version;
}


my $AC =
  {
   BUNDLES => {
           MOD => { DESC => 'Edit',
            #CKIM-SHORTCUTS: added a new op 'create_shortcut'
            OPS => [qw(add_child create_child modify save up down paste create_shortcut modify_properties save_properties)],
            DEFAULT_LEVEL => 8,
            MIN_LEVEL => 2,
              },
           DEL => { DESC => 'Delete and Cut',
            OPS => [qw(delete delete_ok cut)],
            DEFAULT_LEVEL => 8,
            MIN_LEVEL => 8,
            PARENT_DEP => {cut => 'MOD'},
              },
           DISP => { DESC => 'View',
             OPS => [qw(show show_summary)],
             DEFAULT_LEVEL => 0,
             MIN_LEVEL => 0,
             DEFAULTS_TO => 'on',
               },
           EDITP => { DESC => 'Change<br />Permissions',
              OPS => [qw(edit_permissions set_permissions change_owner)],
              DEFAULT_LEVEL => 8,
              MIN_LEVEL => 8,
            },
          },
   DISPLAY_ORDER => [qw(DISP MOD DEL EDITP)],
  };


###
### ACCESS CONTROL METHODS
###
sub get_AC {
  my $self = shift;
  my $class = ref $self || $self;

  die "&get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

  return $AC;
}


#####################################################
######                                        #######
######               CONSTRUCTOR              #######
######                                        #######
#####################################################

sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

  $self->{display_wrapped} = { show => 1,
                             };


}

#####################################################
######                                        #######
######            FRONT-END METHODS           #######
######                                        #######
#####################################################
sub www_add_child {
  my $self = shift;
  my $errormsg = shift;
  my $child_type = shift;
  $self->debug_msg(3, "Table::www_add_child()");

  my $child_isa = $FORM{child_type} || $child_type;
  my $gizmo_name = $child_isa->get_gizmo_name();

  my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
  # we re-instanciate the object because we want a brand new one
  # unless there is an error message (we want to keep his data)
  my $child = $child_isa->new({ parent_id => $parentid }) unless ($errormsg);

    $errormsg = $errormsg ?
                "<p /><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p />\n" :
                "";

  my $form = $child->show_edit_form("Adding new $gizmo_name");
  my $content =
  Components->color_palette_code().
"
<form name=fm action='index.pl' method='post' ENCTYPE=\"multipart/form-data\">
<input type='hidden' name='isa' value='Table'>\n
<input type='hidden' name='iid' value='$parentid'>\n
<input type='hidden' name='op' value='create_child'>\n
<input type='hidden' name='child_type' value='Row'>\n
$errormsg
$form
<table align='right'><tr><td><input type='submit' value='Create'></td></tr></table>
<br />&nbsp;
</form>
";

  $child->print_form($content, "Adding new $child_isa");
}


sub www_create_child {
  my $self = shift;
  my $child_type = shift;

  $self->debug_msg(3, "Gizmo::www_create_child()");

  my $child_isa = $FORM{child_type} || $child_type;

  my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
  # we re-instanciate the object because we want a brand new one
  my $child = $child_isa->new({
             uid => $USER->uid(),
             parent_id => $parentid,
            });

  # this method should ALWAYS be overridden in derived classes.
  $child->process_edit_form();
}

sub www_modify {
  my $self = shift;
  my $errormsg = shift;
  $self->debug_msg(3, "Gizmo::www_modify()");
  my $iid = $self->id();
  my $isa = $self->is_a();
  my $target = $SESSION->{referer};
  if (!defined $iid){
    $self->click_back ("Error:", "id is missing.");
    exit(0);
  }

    $errormsg = $errormsg ?
                "<p /><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p />\n" :
                "";

  my $form = $self->show_edit_form("Customizing $isa");
  my $content =
#  Components->color_palette_code().
"
<form name=fm action='index.pl'  method='post' ENCTYPE=\"multipart/form-data\">
<input type='hidden' name='op' value='save'>
<input type='hidden' name='isa' value='$isa'>
<input type='hidden' name='iid' value='$iid'>
<input type='hidden' name='target' value='$target'>
$errormsg
$form
<table align='right'><tr><td><input type='submit' value='Save'></td></tr></table>
<br />&nbsp;
</form>
";

  $self->print_form($content);
}

sub process_edit_form {
  my $self = shift;
  my $isa = ref $self;

  $self->debug_msg(3, "Table::process_edit_form()");

  # fill the object with form values
  $self->name($FORM{name});
  $self->description($FORM{description});
  $self->keywords($FORM{keywords});
  $self->show_childs($FORM{showincat});

  $self->i1($FORM{maxrows});
  $self->i3($FORM{showsearch});
  $self->i4($FORM{showsort});
  $self->c1($FORM{titlecolor});
  $self->c2($FORM{rowcolor});

  $self->t1($FORM{column1});
  $self->t2($FORM{column2});
  $self->t3($FORM{column3});
  $self->t4($FORM{column4});
  $self->t5($FORM{column5});
  $self->t6($FORM{column6});
  $self->t7($FORM{column7});
  $self->t8($FORM{column8});
  $self->t9($FORM{column9});
  $self->t10($FORM{column10});

  # check if everything is ok
  my $errormsg = "";
  $errormsg .= "<li>Too bad! Name field cannot be empty...</li>" if (!$self->name()) ;
  $errormsg .= "<li>Too bad! Wrong number of displayed rows...</li>" if ($self->i1 !~ m/\d+/) ;
  $errormsg .= "<li>Too bad! All column names cannot be empty...</li>"
    if (!$self->t1 && !$self->t2 && !$self->t3 && !$self->t4 && !$self->t5 && !$self->t6 && !$self->t7 && !$self->t8 && !$self->t9 && !$self->t10) ;

  my $parentid = $self->parent_id();
  if ($errormsg) {

    my $parent = Gizmo->restore($parentid);
    $parent->www_add_child($errormsg, $self->is_a);

  } else {
    $self->add();
    $self->two_choices("<Big>New Table added!</big><br /><br />What do you want to do?",
                       "confirm",
               "Add another Table", "index.pl?iid=$parentid&isa=Category&op=add_child&child_type=Table",
               "Enough for now!", "index.pl?iid=".$self->id);
  }
}

##############

sub www_save {
  my $self = shift;
  $self->debug_msg(3, "Table::www_save()");

  my $iid = $self->defined_or_exit($self->id,"id is missing.");

  # fill the object with form values
  $self->name($FORM{name});
  $self->description($FORM{description});
  $self->keywords($FORM{keywords});
  $self->show_childs($FORM{showincat});

  $self->i1($FORM{maxrows});
  $self->i2($FORM{hide_name_in_lists});
  $self->i3($FORM{showsearch});
  $self->i4($FORM{showsort});
  $self->c1($FORM{titlecolor});
  $self->c2($FORM{rowcolor});

  my @to_delete;
  $self->t1($FORM{column1});
  push(@to_delete,'t1') if (!$self->t1) ;
  $self->t2($FORM{column2});
  push(@to_delete,'t2') if (!$self->t2) ;
  $self->t3($FORM{column3});
  push(@to_delete,'t3') if (!$self->t3) ;
  $self->t4($FORM{column4});
  push(@to_delete,'t4') if (!$self->t4) ;
  $self->t5($FORM{column5});
  push(@to_delete,'t5') if (!$self->t5) ;
  $self->t6($FORM{column6});
  push(@to_delete,'t6') if (!$self->t6) ;
  $self->t7($FORM{column7});
  push(@to_delete,'t7') if (!$self->t7) ;
  $self->t8($FORM{column8});
  push(@to_delete,'t8') if (!$self->t8) ;
  $self->t9($FORM{column9});
  push(@to_delete,'t9') if (!$self->t9) ;
  $self->t10($FORM{column10});
  push(@to_delete,'t10') if (!$self->t10) ;
  $self->delete_columns(@to_delete) if (@to_delete) ;


  # check if everything is ok
  my $errormsg = "";
  $errormsg .= "<li>Too bad! Name field cannot be empty...</li>" if (!$self->name()) ;
  $errormsg .= "<li>Too bad! Wrong number of displayed rows...</li>" if ($self->i1 !~ m/\d+/) ;
  $errormsg .= "<li>Too bad! All column names cannot be empty...</li>"
    if (!$self->t1 && !$self->t2 && !$self->t3 && !$self->t4 && !$self->t5 && !$self->t6 && !$self->t7 && !$self->t8 && !$self->t9 && !$self->t10) ;

  if ($errormsg) {
    $self->www_modify($errormsg);
  } else {
    $self->save();

    $self->md_redirect("index.pl?iid=" . $self->parent_id);
  }
}

#####################################################
######                                        #######
######            BACK-END METHODS            #######
######                                        #######
#####################################################

#####################################################
###          DATABASE BACK-END METHODS            ###
#####################################################

sub _format_search_result{
  my $gizmo_name = shift ;
  my ($iid, $name, $description, $isa)=@_;

  if (!$iid || !$isa ){
    return "";
  }

  $name = "No title" if (!$name);

  my $s="";
  $s = "<a href=".$gizmo_name->make_url()."><b>$name</b></a>\n- $description";

  return $s;
}

sub show_search_results {
  my $gizmo_name = shift ;
  my $thingtosearch = shift;
  my $startfrom = shift || 0;
  my $maxitem = shift || 50;

  my $s="";
  my $oneresult="";

  my $filter = $thingtosearch;
  my $wherestring="";

  ################################################################
  ####
  #### we build the query
  ####
  #### clean quote and other chars
  ################################################################

  $thingtosearch = uc($thingtosearch);
  $thingtosearch = DBAccess->quote("%$thingtosearch%");

  my $i=0;
  my $totalrows=0;
  my $from = $startfrom;
  my $iid_list;
  my $entries;

  $wherestring = join(" OR ", map("upper($_) like $thingtosearch", ("i2.t1", "i2.t2", "i2.t3", "i2.t4", "i2.t5", "i2.t6", "i2.t7", "i2.t8", "i2.t9", "i2.t10")));
  my $whereclause="i1.isa='Table' AND i2.isa='Row' AND i2.parentiid = i1.iid and ($wherestring)";
  my ($c)=DBAccess->sqlSelectMany("i1.iid, i1.name, i1.description, i1.isa, count(i2.iid) ", "instance i1, instance i2", $whereclause, "group by i1.iid, i1.isa, i1.name, i1.description");

  while( (my ($i1_iid, $i1_name, $i1_description, $i1_isa, $i2_count)=$c->fetchrow()) && ($i < $maxitem)) {
      if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC
      my $gizmo = $i1_isa->new($i1_iid);
      next unless $gizmo->is_allowed_to_do('show', $USER);
      next unless $gizmo->parentCheck(); # don't show hidden items
    $totalrows++,
      # shorten the description
      $i1_description="" if (!$i1_description || !defined($i1_description));
      $i1_description=substr($i1_description, 0, 150) ."...";
      ### remove tags that could break the page.
      $i1_description = Verify->stripBadHTML($i1_description);
      $iid_list .= $i1_iid;
      $oneresult = $gizmo_name->_format_row_search_result($i1_iid, $i1_name, $i1_description, $i2_count, $filter);
      ### put the results in list tag
      $oneresult="<li>" . "$oneresult" . "<br /><br /></li>\n\n";
      $s .= "$oneresult\n\n";;
      $i++;
  }
  $c->finish();

  $wherestring = join(" or ", map("upper($_) like $thingtosearch", ("name", "description", "t1", "t2", "t3", "t4", "t5", "t6", "t7", "t8", "t9", "t10","Keywords")));

  $whereclause="isa='$gizmo_name' AND ($wherestring)";

  ($c)=DBAccess->sqlSelectMany("iid, name, description, isa ", "instance", $whereclause);

  while( (my ($iid, $name, $description, $isa)=$c->fetchrow()) && ($i < $maxitem)) {
      if (grep(!/$iid/, $iid_list)) {
      if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC
      my $gizmo = $isa->new($iid);
      next unless $gizmo->is_allowed_to_do('show', $USER);
      next unless $gizmo->parentCheck(); # don't show hidden items
    $totalrows++;
      # shorten the description
      $description="" if (!$description || !defined($description));
      $name="null" if (!$name || !defined($name));
      $description=substr($description, 0, 150) ."...";
      ### remove tags that could break the page.
      $description = Verify->stripBadHTML($description);
      $oneresult = $gizmo_name->_format_search_result($iid, $name, $description, $isa);
      ### put the results in list tag
      $oneresult="<li>" . "$oneresult" . "<br /><br /></li>\n\n";
      $s .= "$oneresult\n\n";;
      $i++;
      }
  }
  $c->finish();

  $s = $gizmo_name->show_search_result_group({
      gizmo_name    => $gizmo_name,
      total_rows    => $totalrows,
      max_rows      => $maxitem,
      start_from    => $startfrom,
      cursor        => $i,
      filter        => $filter,
      results       => $s,
  });

  return ($s);

}

sub _format_row_search_result{
  my $gizmo_name = shift ;
  my ($i1_iid, $i1_name, $i1_description, $i2_count, $filter)=@_;

  if (!$i1_iid){
    return "";
  }

  $i1_name = "No title" if (!$i1_name);

  my $s="";
  $s = "<a href="
       .$gizmo_name->make_table_url('',{ iid=>$i1_iid, op=>'show' })
       ."><b>$i1_name</b></a> - <a href="
       .$gizmo_name->make_table_url('',{ iid=>$i1_iid, filter=>$filter, op=>'show' })
       ."><b>\[$i2_count matching row\(s\)\]</b></a>";

  return $s;
}


sub delete{
  my $self = shift;
  $self->debug_msg(3, "Table::delete()");
  my $id = $self->id;

  my ($childs)=DBAccess->sqlSelectMany("iid", "instance", "parentiid=$id and isa='Row'");
  while(my ($iid)=$childs->fetchrow()) {
    my $row = Row->new($iid);
    $row->delete();
  }

  # delete discussion from instance
  $self->SUPER::delete();
  return 1;
}

sub delete_columns {
  my $self = shift;
  my @to_delete = @_;
  $self->debug_msg(5, "Table::delete_column()");
  return 0 if (!@to_delete) ;
  my $id = $self->id;
  my %changes ;
  $self->debug_msg(5, "deleting columns ".join(', ',@to_delete));
  foreach my $field (@to_delete) { $changes{$field}=""; }
  $self->get_object_accessor()->update_where(class_field=>'is_a', where=>"parentiid=$id and isa='Row'", updates=>\%changes);
  #DBAccess->sqlUpdate("instance","parentiid=$id and isa='Row'",%changes);
  return 1;
}

#####################################################
###         WEB DISPLAY BACK-END METHODS          ###
#####################################################

sub show {
  my $self  = shift;
  $self->debug_msg(4, "Table::show()");

  my $id = $self->id;
  my $name = $self->name||"";
  my $description = $self->description||"";

  ## if admin
  my $admin_panel = "";
  if (($self->is_allowed_to_do('modify', $USER)) && ($SESSION->admin_mode())){
    $admin_panel .= "[ <a href='".$self->make_table_url('', { op=>'add_child', child_type=>'Row'})."'>Add a Row</a> ]\n";
  }

  if ($self->get_filter()){
    $admin_panel .= "[ <a href='".$self->make_table_url('', { filter=>'' })."'>"
                    ."Show all Rows</a> ]\n";
  }
  my $table = $self->show_table();

  # display filter form if i3 flag set#
  my $filter_form ;
  if ($self->{i3}) {
    $filter_form = "
    <form action='index.pl' method='post'>
    <td align='right'>
    <input type=hidden name='id' value='$id'>
    <input size='15' name='filter' value=\"" . ($self->get_filter()) . "\">
    <input type='submit' value='Search'></td>
    </td>
    </form>
";
    }
  else {
    $filter_form = "<td></td>";
  }

  my $s = "
<table width='100%' border='0'>
<tr><td align='left'>
$admin_panel<br />
<b>Description:</b> $description
$filter_form
</tr></table>
$table
<br /><br />
";
  return $s;
}

##############

sub gizmo_list_header {
  my $class = shift;
  my $name = shift;

  my $output = "<!-- Beginning of table list -->\n";

  return $output;
}

sub gizmo_list_decorator {
    return 'HTML::Decorator::NullDecorator';
}

sub get_filter {
    return $FORM{filter} || '';
}

sub get_sort_column {
    return $FORM{sort_column};
}

sub get_sort_order {
    return $FORM{sort_order} || 1;
}

sub get_start_row {
    return $FORM{start_row};
}

sub get_max_rows {
    my $self = shift;
    return $self->i1() || 20;
}

sub get_columns {
    my $self = shift;

    my @columns = ();

    my @all_fields = ("t1","t2","t3","t4","t5","t6","t7","t8","t9","t10");

    # select the good fields and column names
    foreach my $x (@all_fields) {
      if($self->$x()) {
          my $column = {
              name  => $self->$x(),
              field => $x,
              sort_order => 1,
              sort_priority => undef,
          };
          push @columns, $column;
      }
    }
    return \@columns;
}


sub get_data {
    my $self = shift;
    my $args = shift;

    my $id = $self->get_iid();
    my $filter = $self->get_filter();
    my $columns = $self->get_columns;

    # build a list of fields from the column data
    my @attr = ();
    foreach my $column (@$columns) {
        push @attr, $column->{field};
    }


    my $where = "parentiid=$id and isa='Row'";
    if ($filter) {
      my $filter = uc(DBAccess->quote("%$filter%"));
      $where  .= " and (".join(" or ",map("upper($_) like $filter",@attr)).")";
    }
    my $select_from = "instance";

    my $other = "order by ".($self->get_sort_column() || 'instanceorder');
    if ($self->get_sort_order() < 0) {
        $other .= ' DESC';
    } else {
        $other .= ' ASC';
    }

    my $row_set = DBAccess->sqlSelectMany("iid, ".join(', ',@attr), $select_from, $where, $other);
    my ($count) = DBAccess->sqlSelect("count(*)",$select_from, $where, $other);

    return { columns=>$columns, row_set=>$row_set, row_count=>$count };
}

sub get_up_arrow {
    return '<img class="icon" src="'.$PARAMS->{'imageslocation'}.'/obj/triangle_up.gif" border="0" alt="Sort Ascending">';
}

sub get_down_arrow {
    return '<img class="icon" src="'.$PARAMS->{'imageslocation'}.'/obj/triangle_down.gif" border="0" alt="Sort Descending">';
}

sub show_table {
  my $self  = shift;
  $self->debug_msg(4, "Table::show_table()");

  my $id = $self->id;
  my $titlecolor = $self->c1 || "CCCCCC";
  my $rowcolor = $self->c2 || "FFFFFF";

  my @arrow;

  my $data = $self->get_data();
  my $columns = $data->{columns};
  my $results = $data->{row_set};
  my $total   = $data->{row_count};

  #PREVIOUS NEXT MAGIC
  my $max = $self->get_max_rows();

  my $from   = $FORM{from} || 0;
  my $nbdisp = $total - $from ;
  $nbdisp = $max if ($nbdisp>$max);
  my $PNbuttons = ($total>$max) ? Components->get_previous_next_button($max, $nbdisp, $total, $from, $self->make_table_url()) : "&nbsp;";

  # build the column titles
  my $titles_row = "";
  my $column_count = scalar(@$columns);
  my $sort_column = $self->get_sort_column() || 'instanceorder';

  for ( my $i=0 ; $i<$column_count ; $i++) {
    my $column = $columns->[$i];
    if ($self->{i4}) { # if sorting is allowed
        if ($sort_column eq $columns->[$i]->{field}) {
            $titles_row .= "<td><a href=\"".$self->make_table_url('', { sort_column=>$column->{field}, sort_order=>(-1 * $self->get_sort_order()) })."\">";
            if ($self->get_sort_order < 0) {
                $titles_row .= $self->get_down_arrow();
            } else {
                $titles_row .= $self->get_up_arrow();
            }
            $titles_row .=$column->{name}.'</a></td>';
        } else {
            $titles_row .= "<td><a href=\"".$self->make_table_url('', { sort_column=>$column->{field}, sort_order=>1 })."\">$column->{name}</a></td>";
        }
    } else {
        $titles_row .= "<td><b>$column->{name}</b></td>" ;
    }
  }

  # build main rows
  my $rows = "";
  my $panel = "";
  while (($nbdisp>0) && (my @row = $results->fetchrow)) {
    if ($from) {$from--; next;}
    $nbdisp--;
    my $tid = shift @row; # to get the id
    $rows .= "<tr bgcolor='#$rowcolor' valign='top'>\n";
    $rows .= join("\n",map("<td>$_&nbsp;</td>",@row));

    my $row = Row->new($tid);
    $panel = $row->show_panel();
    $panel .= $row->show_up_down_buttons()if(!$self->get_filter());
    ## can't set at row level
    #$panel .= $row->show_scheduling_buttons();
    #$panel .= $self->show_up_down_buttons() if(!$FORM{order} && !$FORM{filter});

    $rows .= "\n<td align='center'>\n$panel\n</td>" if ($panel);
    $rows .= "\n</tr>\n";
  }
  $results->finish;

  ## if up_down we have to add a column
  if ($panel) {
    $titles_row .= "\n<td>Edit</td>";
  }

  return "
<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\">
    <tr><td>$PNbuttons</td></tr>
    <tr><td>
    <table border='0' cellspacing=\"1\" cellpadding=\"3\" bgcolor=\"#$titlecolor\">
    <tr>
    $titles_row
    </tr>
    $rows
    </table></td></tr>
    <tr><td>$PNbuttons</td></tr>
</table>

";

}

##############

sub show_edit_form {
  my $self  = shift;
  my $title = shift;
  $self->debug_msg(4, "Table::show_edit_form($title)");

  my $id   = $self->id;
  my $name = $self->name;
  $name = escapeHTML($name);
  my $description = $self->description;
  $description = escapeHTML($description);
  my $keywords = $self->keywords;
  $keywords = escapeHTML($keywords);
  my $checked  = ($self->show_childs || !defined($id)) ? "checked" : "" ;
  my $checked1 = (!($self->i3) || !defined($id)) ? "" : "checked" ;
  my $checked2 = ($self->i4 || !defined($id)) ? "checked" : "" ;

  my $max = $self->i1 || 20;
  my $hide_name_in_lists = $self->i2;
  my $hide_name_check = ($hide_name_in_lists? 'checked' : '');

  my $titlecolor = $self->c1 || "CCCCCC";
  my $rowcolor = $self->c2 || "FFFFFF";

  my $column1 = $self->t1;
  my $column2 = $self->t2;
  my $column3 = $self->t3;
  my $column4 = $self->t4;
  my $column5 = $self->t5;
  my $column6 = $self->t6;
  my $column7 = $self->t7;
  my $column8 = $self->t8;
  my $column9 = $self->t9;
  my $column10 = $self->t10;

  ## if admin
  my $admin_panel = "";
#   if (($self->is_allowed_to_do('modify', $USER)) && ($SESSION->admin_mode())) {
#     $admin_panel .= "[ <a href='".$self->make_url("index.pl?iid=" .$self->id . "&amp;isa=Table&amp;op=add_child&amp;child_type=Row")."'>Add a Row</a> ]\n";
#   }

  my $s = Components->color_palette_code(). "\n<h1>$title</h1>

      <p /><b>Name</b> (*)<br />
      <input type='text' name='name' size='55' maxlength='255' value='$name'><br />
      <input type='checkbox' name='hide_name_in_lists' $hide_name_check value='1'>Hide this name when table is shown as part of a page

      <p /><b>Description</b><br />
      <textarea name='description' rows=2 cols=46 WRAP='VIRTUAL'>$description</textarea>
      <br />

      <p /><b>Keywords and Synonyms</b><br />
      <textarea name='keywords' rows=2 cols=46 WRAP='VIRTUAL'>$keywords</textarea>

      <p /><input type='checkbox' name='showincat' $checked>&nbsp;&nbsp;<b>Show in parent category</b>
          <p /><input type='checkbox' name='showsearch' $checked1 value='1'>&nbsp;&nbsp;<b>Allow search</b>
           <p /><input type='checkbox' name='showsort' $checked2 value='1'>&nbsp;&nbsp;<b>Allow sorting</b>
          <hr>
          <h2>Table Column Names</h2>

      <p />
          <table>
          <tr><td><b>Column #1</b></td><td><input type='text' name='column1' size='40' maxlength='255' value='$column1'><br /></td></tr>
          <tr><td><b>Column #2</b></td><td><input type='text' name='column2' size='40' maxlength='255' value='$column2'><br /></td></tr>
          <tr><td><b>Column #3</b></td><td><input type='text' name='column3' size='40' maxlength='255' value='$column3'><br /></td></tr>
          <tr><td><b>Column #4</b></td><td><input type='text' name='column4' size='40' maxlength='255' value='$column4'><br /></td></tr>
          <tr><td><b>Column #5</b></td><td><input type='text' name='column5' size='40' maxlength='255' value='$column5'><br /></td></tr>
          <tr><td><b>Column #6</b></td><td><input type='text' name='column6' size='40' maxlength='255' value='$column6'><br /></td></tr>
          <tr><td><b>Column #7</b></td><td><input type='text' name='column7' size='40' maxlength='255' value='$column7'><br /></td></tr>
          <tr><td><b>Column #8</b></td><td><input type='text' name='column8' size='40' maxlength='255' value='$column8'><br /></td></tr>
          <tr><td><b>Column #9</b></td><td><input type='text' name='column9' size='40' maxlength='255' value='$column9'><br /></td></tr>
          <tr><td><b>Column #10</b></td><td><input type='text' name='column10' size='40' maxlength='255' value='$column10'><br /></td></tr>
          </table>

          <hr>
          <h2>Table Display</h2>

      <p /><b>Number of displayed rows</b> (*)
      <input type='text' name='maxrows' size='12' maxlength='20' value='$max'>

           <p /><font color=red>Instructions:  Choose a color from the palette.<br />For advanced users:  \"hexadecimal\" values (0-9, A-F) allowed in the following color selections.</font><br />

           <HR align=\"left\" Size=3  width=\"600\">
          <p /><table width=\"95%\"><tr><td width=\"55%\"><b>Title row background color</b></td><td width=\"10\">&nbsp;</td><td bgcolor=\"#ffffff\"><font size=-1>Current Applied Color</font>
          <table width=\"100%\"><tr><td bgcolor='#$titlecolor'>&nbsp;</td></tr></table></td><td>$titlecolor</td>
          <td><input type=radio name=\"ttitlecolor\" value=\"$titlecolor\" onclick='javascript:setTxt(\"ttitlecolor\",\"titlecolor\")' checked></td></tr></table><br />".
          Components->color_palette("titlecolor","ttitlecolor")."
          <input type=text name=\"titlecolor\" value=\"$titlecolor\" size=7 maxlength=6>(e.g. CCCCCC)
          <HR align=\"left\" Size=3  width=\"600\">

          <p /><table width=\"95%\"><tr><td width=\"55%\"><b>Data rows background color</b></td><td width=\"10\">&nbsp;</td><td bgcolor=\"#ffffff\"><font size=-1>Current Applied Color</font>
          <table width=\"100%\"><tr><td bgcolor='#$rowcolor'>&nbsp;</td></tr></table></td><td>$rowcolor</td>
          <td><input type=radio name=\"rrowcolor\" value=\"$rowcolor\" onclick='javascript:setTxt(\"rrowcolor\",\"rowcolor\")' checked></td></tr></table><br />".
          Components->color_palette("rowcolor","rrowcolor")."
         <input type='text' name=\"rowcolor\" value=\"$rowcolor\" size=7 maxlength=6>(e.g. FFFFFF)
         <HR align=\"left\" Size=3  width=\"600\">

      <p /><b>$admin_panel</b><br />

          <br /><br /><br /><i>(*) Mandatory field(s)</i><br />";
  return $s;
}

######

sub show_summary {
  my $self = shift;
  my $id              = $self->get_iid();
  my $isa             = $self->get_is_a();
  $self->debug_msg(5, "Table::show_as_bullet($id)");

  my $str = "<p />\n";
  my $panel = $self->show_panel() . $self->show_up_down_buttons() . $self->show_scheduling_buttons();
  my $name = $self->name();
  my $description = $self->description();

  $str = "$panel\n";
  $str .= "<a style=\"font-size: 14px; font-weight: bold; color:black;\" href=\"".$self->make_table_url()."\">$name</a><br />" unless $self->i2;

  if ($description) {
    $str .= "$description<br />\n";
  }

  $str.= $self->show_table."\n" if ($self->show_childs);

  return $str;
}


sub make_table_url {
    my $proto = shift;
    my $url = shift || '';
    my $args = shift || {};

    # get all the defaults
    $args->{op}             ||= $FORM{op} || 'show';
    if (ref($proto)) {
        $args->{iid} = $proto->get_iid()        unless defined($args->{iid});
        $args->{isa} = $proto->get_is_a()       unless defined($args->{isa});
        $args->{filter} = $proto->get_filter()  unless defined($args->{filter});
        $args->{sort_column} = $proto->get_sort_column() unless defined($args->{sort_column});
        $args->{sort_order} = $proto->get_sort_order()   unless defined($args->{sort_order});
        $args->{start_row} = $proto->get_start_row()     unless defined($args->{start_row});
    }

    # default url if none
    $url ||= 'index.pl?';

    # if there's no question mark, add one
    unless ($url =~ m/\?/) {
        $url .= '?';
    }

    # write any optional values
    foreach my $key (keys %$args) {
        unless ($url =~ /$key=/) {
            $url .= "&$key=$args->{$key}";
        }
    }

    return $proto->SUPER::make_url($url);
}



sub title_color {
    my $self = shift;
    return $self->c1(@_);
}

sub row_color {
    my $self = shift;
    return $self->c2(@_);
}


#####################################################
###                   THE END                     ###
#####################################################

1;

__END__


=head1 NAME

Gizmo::Table -

=head1 SYNOPSIS

    use Gizmo::Table;

    $table = Table->new ($id);
    $AC = $table->get_AC ();
    $table->www_add_child ($errormsg, $child_type);
    $table->www_create_child ($child_type);
    $table->www_modify ($error_msg);
    $table->process_edit_form ();
    $table->www_save ();
    $html = &show_search_results ($gizmo_name, $thingtosearch,;
              [$startfrom, $maxitem]);
    $status = $table->delete ();
    $status = $table->delete_columns ();
    $html = $table->show ();
    $html = $table->show_table ();
    $html = $table->show_edit_form ($title);
    $html = $table->show_as_bullet ();

    $version = Table->get_version();


=head1 DESCRIPTION

Table is gizmo that allows to create custom html tables from user supplied data. Each table is parent of a number of Row gizmos. See also documentation for Row.pm.

=head2 Methods

$table = Table->new ($id);

Standard Gizmo constructor. If an id is provided it will restore an existing gizmo from the database.

$AC = $table->get_AC();

Auditable.pm required method. See Auditable documentation.

$table->www_add_child ($errormsg, $child_type);

Will be called by the user interface to generate the user interface for adding a new row to the table. This method will call show_edit_form() in the Row class to generate the form for entering the row's data.

$version = Table->get_version();

Class method to retrieve the version number of the document


$table->www_create_child ($child_type);

Will be called after www_add_child to create and insert a new gizmo into the database. This method will call process_edit_form in the child class to read and validate form data, and then insert it in the database.

$table->www_modify ($error_msg);

This method is invoked by the system when a user wants to modify the content of a gizmo (usually by clicking in the "Edit" icon). See Gizmo.pm documentation.

$table->process_edit_form ();

This method will read gizmo specific data from the posted form when an gizmo is being created.It is invoked from www_create_child in Category.pm and Table.pm. See Gizmo.pm documentation.

$table->www_save ();

This method will be invoked by the system after www_modify to commit field changes to the database. See Gizmo.pm documentation.

$html = &show_search_results ($gizmo_name, $thingtosearch,;
              [$startfrom, $maxitem]);

Will perform a search for matching words in gizmos of the specified type.

$status = $table->delete ();

Will delete a gizmo instance from the instance table.

$status = $table->delete_columns ( @columns );

Will delete columns for this table. Columns should be specified as a list of column identifiers, from 't1' to 't10'.

$html = $table->show ();

Will generate HTML to render the content of the table. See Gizmo.pm documentation.

$html = $table->show_as_bullet ();

Will generate HTML to render the content of this gizmo in a manner suitable for rendering as a bullet point within a Category gizmo.

$html = $table->show_table ();

Will generate HTML to render the table. This method is called from show and show_as_bullet methods.

$html = $table->show_edit_form ($title);

Will generate HTML to render the table editing form. This method is called from the www_modify method.

=cut
