###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Row;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT);

use Carp;
use Gizmo;


use DBAccess;

use CGI qw/escape unescape/;
use vars qw(@ISA);
@ISA=qw(GizmoBuilder);
my $version = "4.5";

sub get_version {
    return $version;
}


my $AC =
  {
   BUNDLES => {
           MOD => { DESC => 'Edit',
            OPS => [qw(modify save up down)],
            DEFAULT_LEVEL => 0,
            MIN_LEVEL => 0,
            PARENT_DEP => {
                       modify => 'MOD',
                       save => 'MOD',
                       up => 'MOD',
                       down => 'MOD',
                      },
            NO_PARENT_INHERITANCE => 1,
            DEFAULTS_TO => 'off',
              },
           DEL => { DESC => 'Delete',
            OPS => [qw(delete delete_ok)],
            DEFAULT_LEVEL => 0,
            MIN_LEVEL => 0,
            PARENT_DEP => { delete => 'DEL', delete_ok => 'DEL' },
            NO_PARENT_INHERITANCE => 1,
            DEFAULTS_TO => 'off',
              },
           DISP => { DESC => 'View',
             OPS => [qw(show)],
             DEFAULT_LEVEL => 0,
             MIN_LEVEL => 0,
             PARENT_DEP => { show => 'DISP' },
             NO_PARENT_INHERITANCE => 1,
             DEFAULTS_TO => 'on',
               },
          },
   DISPLAY_ORDER => [qw(DISP MOD DEL)],
  };

###
### ACCESS CONTROL METHODS
###
sub get_AC {
  my $self = shift;
  my $class = ref $self || $self;

  die "&get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

  return $AC;
}




#####################################################
######                                        #######
######            FRONT-END METHODS           #######
######                                        #######
#####################################################

sub www_show {
  my $self = shift;
  $self->debug_msg(3, "Row::www_show()");

  my $iid = $self->defined_or_exit($self->id,"id is missing.");
  my $name = "Row #".$self->instance_order();

  my $content = $self->show();
  $self->print_portal($content,$name);
}

sub www_delete {
  my $self = shift;
  my $target = $SESSION->{referer};
  $target = escape($target);
  my $isa = $self->is_a();
  my $gizmo_name = $isa->get_gizmo_name();
  my $row = $self->get_row_content();
  $self->debug_msg(3, "Row::www_delete()");
  $self->print_http_header;
  $self->two_choices("Do you want to delete this $gizmo_name? <br />".$row,
                     "caution",
             "Yes please", "index.pl?iid=".$self->id."&target=$target&isa=$isa&op=delete_ok",
             "No!", "$SESSION->{referer}");
}

sub www_delete_ok {
  my $self = shift;
  my $target=$FORM{'target'};
  $self->debug_msg(3, "Row::www_delete_ok()");
  $self->delete();
  $self->md_redirect($target);
}
##############

sub process_edit_form {
  my $self = shift;
  my $isa = ref $self;

  $self->debug_msg(3, "Row::process_edit_form()");

  # fill the object with form values
  $self->t1($FORM{field1});
  $self->t2($FORM{field2});
  $self->t3($FORM{field3});
  $self->t4($FORM{field4});
  $self->t5($FORM{field5});
  $self->t6($FORM{field6});
  $self->t7($FORM{field7});
  $self->t8($FORM{field8});
  $self->t9($FORM{field9});
  $self->t10($FORM{field10});

  # check if everything is ok
  my $errormsg = "";
  #$errormsg .= "<li>Too bad! All  fields cannot be empty..."
  #  if (!$self->t1 && !$self->t2 && !$self->t3 && !$self->t4 && !$self->t5 && !$self->t6 && !$self->t7 && !$self->t8 && !$self->t9 && !$self->t10) ;

  my $parentid = $self->parent_id();
  if ($errormsg) {

    my $parent = Gizmo->restore($parentid);
    $parent->www_add_child($errormsg, $self->is_a);

  } else {
    $self->add();
    # XXX parent is not a Category!
    $self->two_choices("<Big>New Row added!</big><br /><br />What do you want to do?",
                       "confirm",
               "Add another Row", "index.pl?iid=$parentid&isa=Table&op=add_child&child_type=Row",
               "Enough for now!", "index.pl?iid=$parentid");
  }
}

##############

sub www_save {
  my $self = shift;
  $self->debug_msg(3, "Row::www_save()");
  my $url = $FORM{target};

  my $iid = $self->defined_or_exit($self->id,"id is missing.");

  # fill the object with form values
  $self->t1($FORM{field1});
  $self->t2($FORM{field2});
  $self->t3($FORM{field3});
  $self->t4($FORM{field4});
  $self->t5($FORM{field5});
  $self->t6($FORM{field6});
  $self->t7($FORM{field7});
  $self->t8($FORM{field8});
  $self->t9($FORM{field9});
  $self->t10($FORM{field10});

  # check if everything is ok
  my $errormsg = "";
  $errormsg .= "<li>Too bad! All fields cannot be empty...</li>"
    if (!$self->t1 && !$self->t2 && !$self->t3 && !$self->t4 && !$self->t5 && !$self->t6 && !$self->t7 && !$self->t8 && !$self->t9 && !$self->t10) ;

  if ($errormsg) {
    $self->www_modify($errormsg);
  } else {
    $self->save();

    $self->md_redirect($url);
  }
}

#####################################################
######                                        #######
######            BACK-END METHODS            #######
######                                        #######
#####################################################

#####################################################
###         WEB DISPLAY BACK-END METHODS          ###
#####################################################

sub show {
  my $self  = shift;
  $self->debug_msg(4, "Row::show()");

  my @fields = ('t1','t2','t3','t4','t5','t6','t7','t8','t9','t10');

  my $parentid = $self->parent_id;
  my $table = Table->new($parentid);

  my $s = "<p /><b>Table: <a href='".$self->make_url("index.pl?iid=$parentid")."'>".$table->name."</a></b>\n<p />\n<table>\n";
  for(my $i=0; $i<10; $i++) {
    my $title = $table->{$fields[$i]};
    my $field = $self->{$fields[$i]};
    $s .= "<tr><td><b>$title</b></td><td>$field</td></tr>\n" if ($title) ;
  }
  $s .= "</table>\n<br /><br />";
}

##############

sub show_edit_form {
  my $self  = shift;
  my $title = shift;
  $self->debug_msg(4, "Row::show_edit_form($title)");

  my @numbers = (1 .. 10) ;

  my $parentid = $self->parent_id;
  # get the column titles
  my @titles = DBAccess->sqlSelect(join(', ',map("t$_",@numbers)),"instance","iid=$parentid");
  # get fields
  my @fields = map { my $var="t$_"; $self->$var() } @numbers ;

  my $s = "<h1>$title</h1>\n\n";

  for(my $i=0; $i<10; $i++) {
    $s .= "<p /><b>$titles[$i]</b> <br /><textarea name='field$numbers[$i]' rows=3 cols=46 WRAP='VIRTUAL'>$fields[$i]</textarea><br />\n\n"
      if ($titles[$i]) ;
  }

  #$s .= "<br /><br /><br /><i>(*) Mandatory field(s)</i><br />";

  return $s;
}

sub get_row_content {
        my $self = shift;
        my $iid = $FORM{id};
        my @attr;
        my @col_name;
        my $nb_col = 0 ;
        my @cols = ("t1","t2","t3","t4","t5","t6","t7","t8","t9","t10");
        my $col = Table->new($self->parent_id());
        my $titlecolor = $col->c1 || "C0C0C0";
        my $rowcolor = $col->c2 || "F0F0F0";

        $iid = undef unless Utility->is_valid_integer($iid); # secu_200401

        # select the good fields and column names
        foreach my $x (@cols) {
                if($col->$x()) {
                        push(@col_name,$col->$x());
                        push(@attr,$x);
                        $nb_col++;
                }
        }

#   print STDERR "============" . join(', ',@attr) . "=============\n";
        my @row = DBAccess->sqlSelect("".join(', ',@attr), "instance", "iid=$iid and isa='Row'");
        my $row = join("\n",map("<td>$_&nbsp;</td>",@row));
        my $col_name = join("\n",map("<td>$_&nbsp;</td>",@col_name));
        $row ="<br /><center><table border=\"1\"><tr bgcolor=$titlecolor>$col_name</tr><tr bgcolor=$rowcolor>$row</tr></table></center>";

        return $row;
}

#####################################################
###                   THE END                     ###
#####################################################


1;


__END__

=head1 NAME

Gizmo::Table::Row -

=head1 SYNOPSIS

    use Gizmo::Table::Row;

    $row = Row->new ($id);

    $AC = $row->get_AC ();
    $row->www_show ();
    $row->www_delete ();
    $row->www_delete_ok ();
    $row->process_edit_form ();
    $row->www_save ();
    $row->show ();
    $html = $row->show_edit_form ($title);
    $html = $row->get_row_content ();
    $version = Row->get_version();


=head1 DESCRIPTION

=cut


