###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package TaskList;

use strict;
use CGI;
use CGI qw(escapeHTML);
use Metadot qw($USER %FORM $SESSION $PARAMS);

use Gizmo;
use GizmoBuilder;
use Time::Local;
use Gizmo::Calendar;
use vars qw(@ISA);
use Metadot::Report;
use Metadot::Controllable;

@ISA=qw(GizmoBuilder Metadot::Report Metadot::Controllable);
my $version = "1.0";

sub get_version {
    return $version;
}

use Gizmo::TaskList::Task;
my $CHILD_TYPE = 'Task';



sub customize_permissions {
    my $class = shift;
    my $AC = shift;

    $class->SUPER::customize_permissions($AC);

    $AC->add_op('MOD','add_child');
    $AC->add_op('MOD','create_child');
}


{
    my $search_params = {
             searchable_fields => [ qw( name description keywords) ],
             displayed_fields  => [ qw( description) ],
             child_isa => 'Task',
             child_searchable_fields => [ qw( name description c2 keywords ) ],
            };

    sub get_search_params {
    my $self = shift;

    return $search_params;
    }
}


####################################
sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

    $self->set_field_info (
          "name",        "<b>".get_gizmo_name()." Name</b>",              1,
          "description", "<b>Description</b>",                0,
          "keywords",    "<b>Keywords and Synonyms</b>",      0,
        );
}

#####################################################
######                                        #######
######            FRONT-END METHODS           #######
######                                        #######
#####################################################
sub www_add_child {
  my $self = shift;
  my $errormsg = shift;
  my $child_type = shift;
  my $child_object = shift;

  $self->debug_msg(3, "TaskList::www_add_child()");

  my $child_isa = $FORM{child_type} || $child_type;
  my $gizmo_name = $child_isa->get_gizmo_name();

  my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
  # we re-instanciate the object because we want a brand new one
  # unless there is an error message (we want to keep his data)
  #my $child = $child_isa->new({ parent_id => $parentid }) unless ($errormsg);
  my $child = $errormsg && defined($child_object) ? $child_object : $child_isa->new({ parent_id => $parentid });

    $errormsg = $errormsg ?
                "<p /><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p />\n" :
                "";
  my $form = $child->show_edit_form("Adding new $gizmo_name");
  my $content =
Components->color_palette_code()."
<form name=fm action='index.pl' method='post' ENCTYPE=\"multipart/form-data\">
<input type='hidden' name='isa' value='TaskList'>\n
<input type='hidden' name='iid' value='$parentid'>\n
<input type='hidden' name='op' value='create_child'>\n
<input type='hidden' name='child_type' value=\"$CHILD_TYPE\">\n
$errormsg
$form
<table align='right'><tr><td><input type='submit' value='Create'></td></tr></table>
<br />&nbsp;
</form>
";

  $child->print_form($content, "Adding new $gizmo_name");
}

sub www_create_child {
  my $self = shift;
  my $child_type = shift || $CHILD_TYPE;

  $self->debug_msg(3, "TaskList::www_create_child()");

  my $child_isa = $FORM{child_type} || $child_type;

  my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
  # we re-instanciate the object because we want a brand new one
  my $child = $child_isa->new({
                         uid => $USER->uid(),
                         parent_id => $parentid,
                        });

  # this method should ALWAYS be overridden in derived classes.
  $child->process_edit_form();
}


###################################
sub show_summary {
    my $self        = shift;
    my $name        = $self->get_name();
    my $description = $self->get_description();
    my $buttons     = $self->get_buttons();
    my $id              = $self->get_iid();
    my $isa             = $self->get_is_a();


    my $html = "<p />\n";

    $html .= "$buttons <a style=\"font-size: 14px; font-weight: bold; color:black;\" href=\"".$self->make_url("index.pl?iid=$id&amp;isa=$isa&amp;op=show")."\">$name</a><br />";

    if ($description) {
        $html .= "$description<br />\n";
    }

    my $bgcolor = 'bgcolor="#FFFFFF"';

    my %args;
    $args{where_clause} = $CHILD_TYPE->status_field().' != '.$CHILD_TYPE->get_status_value('complete');

    my $data = $self->get_data(\%args);
    my $summary = $self->_generate_summary();

    # if childs, display row data message
    if ($data->{row_count} > 0) {
        $html .= <<EOM
      <p>
        <div class='smaller'><br />Summary of upcoming tasks:</div>
        <table cellspacing="0" cellpadding="0" border="0" bgcolor="#CCCCCC" width="450"><tr><td>
        <table cellspacing="1" cellpadding="3" border="0" width="100%">
          <tr valign='bottom'>
        <th class='smaller' nowrap="nowrap" align='left' $bgcolor>Title &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class='smaller' nowrap="nowrap" align='left' $bgcolor>Due Date</th>
        <th class='smaller' nowrap="nowrap" align='center' $bgcolor>Assigned To</th>
        <th class='smaller' nowrap="nowrap" align='center' $bgcolor>Priority</th>
      </tr>
EOM
;
        my $col_count = 4;

        my $rs = $data->{row_set};
        while (my ($child_iid) = $rs->fetchrow()) {
            my $child = $CHILD_TYPE->restore($child_iid);
            $html .= $self->_generate_table_row_summary(
                object      => $child,
            )."\n";
        }
        $rs->finish();

        my $detail_link = "<a class='smaller' href=\"".$self->make_url("index.pl?iid=$id&amp;isa=$isa&amp;op=show")."\">View this entire task list</a>";

        my $cols_left = 1;
        my $cols_right = $col_count - $cols_left;
        $html .= <<EOM
      <tr valign="baseline">
        <td class='smaller' align='left' colspan=$cols_left>
          $summary<br />
        </td>
        <td align='right' colspan=$cols_right>
          $detail_link
        </td>
      </tr>
    </table>
    </td></tr></table></div>
EOM
;
    } else {
        # no childs, just display summary
        $html .= "<span class=\"smaller\">$summary</span>\n";
    }

    $html .= "</p>\n";
    return $html;
}

sub show {
    my $self = shift;

    my $name        = $self->get_name();
    my $description = $self->get_description();
    my $keywords    = $self->get_keywords();
    my $id          = $self->get_iid();
    my $isa         = $self->get_is_a();
    my $dt          = $FORM{dt} || $self->{$self->{current_datetime}};

    my $editable = $self->is_allowed_to_do('modify', $USER);

    my $html .= "<b>$name</b>";
    $html .= "<b>:</b> $description\n" if ($description);

    # show the tasks
    my $bgcolor = 'bgcolor="#FFFFFF"';
    my $col_count = 0;

    # generate the headers that allow me to sort each
    $self->_process_sort_input();

    $html .= <<EOM
    <p />
    <table cellspacing="0" cellpadding="0" border="0" bgcolor="#777777" width="100%"><tr><td>
    <table cellspacing="1" cellpadding="3" border="0" width="100%">
      <tr valign='bottom'>
EOM
;
    foreach my $column (@{$self->get_columns()}) {
        $html .= '<td bgcolor=white nowrap>'.$self->_generate_sort_header($column)."</th>\n";
    }

    $html .= "</tr>\n";
    $col_count += 6;

    my $data = $self->get_data();
    my $rs = $data->{row_set};
    while (my ($child_iid) = $rs->fetchrow()) {
        my $child = $CHILD_TYPE->restore($child_iid);
        $html .= $self->_generate_table_row(
            object      => $child,
            bgcolor     => $bgcolor,
            editable    => $editable,
        )."\n";
    }
    $rs->finish();

    # if no childs, display a message
    unless ($data->{row_count} > 0) {
        $html .= "<td colspan=$col_count align='center' $bgcolor><br /><i>- There are no tasks in this list -</i><br /><br /></td>\n";
    }

    $html .= <<EOM
    </table>
    </td></tr></table>
EOM
;

    # show the 'add task' button
    $html .= $self->_render_add_task();
    return $html;
}

sub _generate_summary {
    my $self = shift;
    my $html;

    my $late_count     = $self->count_late_children;
    my $complete_count = $self->count_complete_children;
    my $child_count    = $self->count_all_children;
    my $child_plural    = ($child_count==1)? '' : 's';

    $html = "$child_count task$child_plural ($late_count late, $complete_count complete)\n";
    return $html;
}

sub _generate_sort_header {
    my $self = shift;
    my $column = shift;
    my $text = $column->{name};
    my $sort_field = $column->{field};
    my $sort_order = $column->{sort_order} || 1;
    my $result;

    my $iid = $self->id();
    my $isa = $self->is_a();
    my $op  = $FORM{op} || 'show';

    # determine what image to use for indicate sort order
    my $sort_image;
    if ($self->sort_field() eq $sort_field) {
        if ($self->sort_order() >= 0) {
            $sort_image = "<img class=\"icon\" src=\"$PARAMS->{imageslocation}/obj/triangle_up.gif\" border=\"0\">";
        } else {
            $sort_image = "<img class=\"icon\" src=\"$PARAMS->{imageslocation}/obj/triangle_down.gif\" border=\"0\">";
        }
        $sort_order = -1 * $self->sort_order(); # toggle the sort
    } else {
        $sort_image = '';
    }

    $result = "<a href=\"index.pl?iid=$iid&amp;isa=$isa&amp;op=$op&amp;sort_order=$sort_order&amp;sort_field=$sort_field\">$text</a> $sort_image";
    return $result;
}

sub _process_sort_input {
    my $self = shift;
    if (defined $FORM{sort_field}) {
        $self->sort_field($FORM{sort_field});
    }
    if (defined $FORM{sort_order}) {
        $self->sort_order($FORM{sort_order});
    }
}

sub _generate_table_row {
    my $self = shift;
    my %args = @_;
    my $html = '';

    my $object = $args{object};
    my $iid = $object->id();
    my $isa = $object->is_a() || $CHILD_TYPE;
    my $name = $object->name();
    my $due_date = $object->due_date();
    my $complete = $object->is_complete();
    my $priority = $object->priority_name() || '-';
    my $assigned_to = $object->assigned_to() || '-';
    my $status = $object->status_name() || '-';

    my $editable = $args{editable};

    # generate the complete_box check
    my $complete_box;
    if ($complete) {
        $complete_box = "<img class=\"icon\" src=\"$PARAMS->{imageslocation}/checked.gif\" border=\"0\">";
    } else {
        $complete_box = "<img class=\"icon\" src=\"$PARAMS->{imageslocation}/unchecked.gif\" border=\"0\">";
    }

    # if the user can edit this, we'll make the complete_box an active element
    if ($editable) {
        $complete_box = "<a href=\"index.pl?iid=$iid&amp;isa=$isa&amp;op=toggle_complete\">$complete_box</a>";
    }

    # format the due date
    my $date_string;
    if ($due_date) {
        $date_string = Calendar->format_datetime($due_date, 'M e, Y');
        if ($object->is_late()) {
            $date_string = "<font color=red>$date_string</font>";
        }
    } else {
        $date_string = '&nbsp;';
    }

    # generate a link for the task, using the name text
    my $name_string = "<a href=\"index.pl?iid=$iid&amp;isa=$isa&amp;op=show\">$name</a>&nbsp;".$object->show_panel();

    my $bgcolor = $args{bgcolor} || 'bgcolor="#FFFFFF"';

    $html .= <<EOM
        <tr valign="baseline">
          <td align='center' $bgcolor>$complete_box</td>
          <td align='left' $bgcolor>$name_string</td>
          <td align='left' $bgcolor nowrap='nowrap'>$date_string</td>
          <td align='center' $bgcolor>$assigned_to</td>
          <td align='center' $bgcolor>$status</td>
          <td align='center' $bgcolor>$priority</td>
        </tr>
EOM
;


}

sub _generate_table_row_summary {
    my $self = shift;
    my %args = @_;
    my $html = '';

    my $object = $args{object};
    my $iid = $object->id();
    my $isa = $object->is_a() || $CHILD_TYPE;
    my $name = $object->name();
    my $due_date = $object->due_date();
    my $complete = $object->is_complete();
    my $priority = $object->priority_name() || '-';
    my $assigned_to = $object->assigned_to() || '-';
    my $status = $object->status_name() || '-';

    # format the due date
    my $date_string;
    if ($due_date) {
        $date_string = Calendar->format_datetime($due_date, 'M e, Y');
        if ($object->is_late()) {
            $date_string = "<font color=red>$date_string</font>";
        }
    } else {
        $date_string = '&nbsp;';
    }

    my $bgcolor = $args{bgcolor} || 'bgcolor="#FFFFFF"';

    # generate a link for the task, using the name text
    my $name_string = "<a href=\"index.pl?iid=$iid&amp;isa=$isa&amp;op=show\" style=\"font-size: 100%;\">$name</a>&nbsp;".$object->show_panel();

    $html .= <<EOM
        <tr valign="baseline">
          <td class='smaller' align='left' $bgcolor>$name_string</td>
          <td class='smaller' align='left' $bgcolor nowrap='nowrap'>$date_string</td>
          <td class='smaller' align='center' $bgcolor>$assigned_to</td>
          <td class='smaller' align='center' $bgcolor>$priority</td>
        </tr>
EOM
;


}

sub show_line {
    my $self        = shift;
    return $self->show_summary();
}

## creates link for adding event
sub _render_add_task {
    my $self = shift;
    my $html = '';

    my $op       = 'add_child';

    ## if user is not allowed to add event
    unless ($self->is_allowed_to_do('add_child', $USER)) {
        return "&nbsp;";
    }

    my $id   = $self->id;
    my $isa  = $self->is_a();

    $html .= <<EOM
    <FORM name="form_add_task" action="index.pl" method="post">
      <INPUT type=hidden name=iid value="$id">
      <INPUT type=hidden name=isa value="$isa">
      <INPUT type=hidden name=op  value="$op">
      <INPUT type=hidden name=child_type value=$CHILD_TYPE>
      <INPUT type=submit name=add_child value="Add a task">
    </FORM>
EOM
;

    return $html;
}


sub get_columns {
    my $self = shift;
    my @columns = (
        {
            name            => 'Complete',
            field           => Task->status_field(),
            sort_order      => 1,
            sort_priority   => 0,
        },
        {
            name            => 'Title',
            field           => 'name',
            sort_order      => 1,
            sort_priority   => 0,
        },
        {
            name            => 'Due Date',
            field           => Task->due_date_field(),
            sort_order      => 1,
            sort_priority   => 0,
        },
        {
            name            => 'Assigned To',
            field           => Task->assigned_to_field(),
            sort_order      => 1,
            sort_priority   => 0,
        },
        {
            name            => 'Status',
            field           => Task->status_field(),
            sort_order      => 1,
            sort_priority   => 0,
        },
        {
            name            => 'Priority',
            field           => Task->priority_field(),
            sort_order      => 1,
            sort_priority   => 0,
        },
    );
    return \@columns;
}

sub get_data {
    my $self = shift;
    my $args = shift;

    # determine the sort
    my $order_by;

    if ($self->sort_field()) {
        $order_by = $self->sort_field();
        if ($self->sort_order() >= 0) {
            $order_by .= ' asc';
        } else {
            $order_by .= ' desc';
        }
    } else {
        # define the default
        $order_by =  $CHILD_TYPE->due_date_field().' ASC,'
                     . $CHILD_TYPE->priority_field().' DESC,'
                     . $CHILD_TYPE->assigned_to_field().' ASC,'
                     . 'name ASC';
    }

    my $id = $self->id;
    my $where_clause = "parentiid=$id and isa=".DBAccess->quote($CHILD_TYPE);
    if ($args->{where_clause}) {
        $where_clause .= " and $args->{where_clause}";
    }

    my $rs = DBAccess->sqlSelectMany("iid", "instance", $where_clause, "order by $order_by");
    my ($count) = DBAccess->sqlSelect('count(*)', "instance", $where_clause);

    my $result = {
        row_set     => $rs,
        row_count   => $count,
    };

    #use Data::Dumper;
    #print STDERR "get_data with $where_clause returns\n",Dumper $result;
    return $result;
}

sub get_max_rows {
    return 0;
}

# get count of all children
sub count_all_children {
    my $self = shift;
    return $self->_count_children_where('1=1');
}

# get count of children whose due dates are past, and are not complete
# late is computed if 24 hours past first second of due date
sub count_late_children {
    my $self = shift;
    return $self->_count_children_where($CHILD_TYPE->due_date_field.'<'.$CHILD_TYPE->late_time.' AND '.$CHILD_TYPE->status_field.'!='.$CHILD_TYPE->get_status_value('complete'));
}

# get count of children whose status is complete
sub count_complete_children {
    my $self = shift;
    return $self->_count_children_where($CHILD_TYPE->status_field.'='.$CHILD_TYPE->get_status_value('complete'));
}

sub _count_children_where {
    my $self = shift;
    my $where_clause = shift || '1=1';

    my ($count) = DBAccess->sqlSelect('count(iid)', 'instance', "isa='$CHILD_TYPE' AND parentiid=".$self->id." AND $where_clause");
    return $count;
}

sub _show_create_success {
    my $self = shift;

#   my $isa = lc($self->is_a());
    my $isa = $self->is_a();
    my $parentid = $self->parent_id();

    $self->two_choices("<Big>New " . $self->get_gizmo_name() .
                       " added!</big><br /><br />What do you want to do?",
                       "confirm",
                       "Add another " . $self->get_gizmo_name(),
                       "index.pl?child_type=$isa&isa=".($FORM{isa} || 'Category')."&iid=$parentid&op=add_child&app=",
                       "Enough for now!",
                       "index.pl?id=".$self->id());

}

sub _show_edit_success {
    my $self = shift;

    if ($FORM{op} =~ m/delete/) {
        $self->md_redirect("index.pl?iid=".$self->parent_id);
    } else {
        $self->md_redirect("index.pl?iid=".$self->id);
    }
}

sub gizmo_list_header {
    my $class = shift;
    my $cat_name = shift;
    my $out = '';
    return $out;
}

sub gizmo_list_decorator {
    return 'HTML::Decorator::NullDecorator';
}

## overrides parent's delete function in order to delete events
sub delete{
    my $self = shift;
    $self->debug_msg(3, "TaskList::delete()");

    my $data = $self->get_data();
    my $rs = $data->{row_set};
    while (my ($child_iid) = $rs->fetchrow()) {
        my $child = $CHILD_TYPE->restore($child_iid);
        $child->delete();
    }

    # delete from instance
    $self->SUPER::delete();
    return 1;
}

sub get_gizmo_name {
    return 'Task List';
}

sub sort_field {
    my $self = shift;
    my $input = shift;

    #secu_200401: sort field is used for queries. make sure it's valid
    if (defined $input && exists($self->{$input})) {
        $self->{sort_field} = $input;
        $SESSION->set_value('task_sort_field', $input);
    }
    return $input || $SESSION->get_value('task_sort_field') || Task->due_date_field();
}

sub sort_order {
    my $self = shift;
    my $input = shift;

    if (defined $input) {
        $SESSION->set_value('task_sort_order', $input);
    }
    return $input || $SESSION->get_value('task_sort_order') || -1;
}


sub get_child_object {
    my $self = shift;
    my $child_iid = shift;
    return $CHILD_TYPE->new($child_iid);
}

sub get_child_type {
    return $CHILD_TYPE;
}

1;

__END__

=head1 NAME

Gizmo::TaskList - can either create a new TaskList (returing a pointer to the new object) or control the way existing ones are displayed (returning the formatted HTML).


=head1 SYNOPSIS

    use Gizmo::TaskList;

    $calendar = TaskList->new ($id);
    $html = $calendar->show ();
    $html = $calendar->show_as_bullet ();
    $html = $calendar->show_as_dash ();
    $html = $calendar->show_edit_form ();
    $html = $calendar->show_in_column ();
    $html = $calendar->get_month_navagtion ();
    $html = $calendar->get_event_quick_list ();
    $text = $calendar->format_datetime ();
    $calendar->set_datetime ();
    $version = TaskList->get_version();

=head1 DESCRIPTION

TaskList are used to keep track of tasks, such as to-dos or action items.


=head1 Methods

Calling the new function instantiates a new calendar or retrive its persistent info from the backend database.

    $calendar->set_datetime () will set the datetime of the calendar
        object and initialize based off this datetime

    $calendar->format_datetime () returns formatted date



This class method retrieves the version oh
   ->get_version();

The remaining functions control how an TaskList is displayed:

    $calendar->show() displays a calendar's events for a day.
    $calendar->show_as_bullet() displays the calendar in a bulleted list.
    $calendar->show_as_dash() same as show_as_bullet()
    $calendar->show_edit_form() displays form for adding/editing calendar
    $calendar->get_month_navagtion() displays month navagation
    $calendar->get_event_quick_list() displays event summary list

=cut
