###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Task;

use strict;

use Metadot qw($PARAMS %FORM $USER);

use Gizmo;
use GizmoBuilder;
use vars qw(@ISA);
use Time::Local;
use Metadot::Controllable;

use CGI qw/escapeHTML/;

use Gizmo::AbstractEvent;
@ISA=qw(AbstractEvent Metadot::Controllable);


my $version = "4.5";

sub get_version {
    return $version;
}


my $COMPLETE_STATUS = 1000;
my $PROGRESS_STATUS = 100;
my $DEFERRED_STATUS = 200;
my %STATI = (
    $PROGRESS_STATUS    => 'In progress',
    $COMPLETE_STATUS    => 'Complete',
    $DEFERRED_STATUS    => 'Deferred',
);

my %PRIORITIES = (
    300 => 'High',
    200 => 'Medium',
    100 => 'Low',

);


sub customize_permissions {
    my $class = shift;
    my $AC = shift;

    $class->SUPER::customize_permissions($AC);

    $AC->add_op('MOD', 'toggle_complete');
}



####################################
sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

    $self->set_field_info (
        "name",        "<b>Task Name</b>",                                  1,
        "description", "<b>Description</b>",                                0,
        $self->date_field(), "<b>Due Date</b>",                             0,

        );


    # prevent null errors in DBI
    $self->status(0);
}

###################################

my $COLOR1 = '#CCCCCC';
my $COLOR2 = '#EEEEEE';
my $COLOR3 = '#FFFFFF';

sub show {
  my $self  = shift;
  $self->debug_msg(4, "Task::show()");

  my $date_string = Calendar->format_datetime($self->due_date(), 'M e, Y');
  if ($self->is_late()) {
    $date_string = "<font color=red>$date_string</font>";
  }
  my @widths = (100,400);

  my $s = "<br /><br /><table border=\"0\" width=\"500\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"$COLOR1\"><tr><td>\n";
  $s .= "<table border=\"0\" width=\"100%\" cellspacing=\"1\" cellpadding=\"5\">\n";
  $s .= "<tr><th colspan='2' bgcolor=\"$COLOR2\">" . $self->name(). "</th></td></tr>";

  $s .= "<tr><td width=$widths[0] bgcolor=\"$COLOR2\"><b>Due Date";
  $s .= ":</b></td><td width=$widths[1] bgcolor=\"$COLOR2\">";
  $s .= $date_string;
  $s .= "</td></tr>\n";

  $s .= "<tr><td width=$widths[0] bgcolor=\"$COLOR2\"><b>Start Date";
  $s .= ":</b></td><td width=$widths[1] bgcolor=\"$COLOR2\">";
  $s .= Calendar->format_datetime($self->start_date(), 'M e, Y');
  $s .= "</td></tr>\n";

  my $user = Metadot::UserFactory->get_user( $self->uid() );

  $s .= "<tr><td width=$widths[0] bgcolor=\"$COLOR2\"><b>Status:</b></td><td width=$widths[1] bgcolor=\"$COLOR2\">" . ($self->status_name() || '&nbsp') . "</td></tr>\n";
  $s .= "<tr><td width=$widths[0] bgcolor=\"$COLOR2\"><b>Priority:</b></td><td width=$widths[1] bgcolor=\"$COLOR2\">" . ($self->priority_name() || '&nbsp'). "</td></tr>\n";
  $s .= "<tr><td width=$widths[0] bgcolor=\"$COLOR2\"><b>Assigned to:</b></td><td width=$widths[1] bgcolor=\"$COLOR2\">" . ($self->assigned_to() || '&nbsp') . "</td></tr>\n";
  $s .= "<tr><td width=$widths[0] bgcolor=\"$COLOR2\"><b>Created by:</b></td><td width=$widths[1] bgcolor=\"$COLOR2\">" . ($user->fullname() || $user->nickname() || '&nbsp') . "</td></tr>\n";
  $s .= "<tr><td width=$widths[0] bgcolor=\"$COLOR2\"><b>Notes:</b></td><td width=$widths[1] bgcolor=\"$COLOR2\">" . ($self->description() || '&nbsp') . "</td></tr>\n";
  $s .= "</table>\n</table>\n";

  $s .= "<p /><a href=\"index.pl?isa=TaskList&amp;op=show&amp;iid=" . $self->parent_id() . "\">Back to Task List</a>\n";


  return $s;
}

sub _generate_form_fields {
    my $self = shift;
    my $result = $self->SUPER::_generate_form_fields();

    ###
    ### BUILD DROP-DOWNS
    ###

    my $dt = $self->start_date() || time;
    my ($year, $month, $day, $timed_hour, $timed_min) = (localtime($dt))[5, 4, 3, 2, 1];

    ## build day dropdown
    my $day_dropdown = "<select name = \"start_day\">\n";;
    for my $d (1..31) {
      $day_dropdown .= "    <option value=\"$d\"" .
                       ($d == $day ? " selected" : "") . ">$d\n";
    }
    $day_dropdown .= "</select>";
    ##

    ## build month dropdown
    my $month_dropdown = "<select name = \"start_month\">\n";;
    for my $m (0..11) {
      $month_dropdown .= "    <option value=\"$m\"" .
                       ($m == $month ? " selected" : "") . ">".$self->months()->{$m}."\n";
    }
    $month_dropdown .= "</select>";
    ##

    ## build year dropdown
    my $year_dropdown = "<select name = \"start_year\">\n";;
    for my $y (($year-1)..($year+5)) {
      $year_dropdown .= "    <option value=\"$y\"" .
                        ($y == $year ? " selected" : "") . ">" . ($y+1900) . "\n";
    }
    $year_dropdown .= "</select>";

    ## build status selections
    my $status_select = '';
    foreach my $value (sort(keys(%STATI))) {
        my $checked;
        if ($self->status() == $value) {
            $checked = 'checked';
        } else {
            $checked = '';
        }
        $status_select .= "<input type=radio name=status $checked value=$value>$STATI{$value}&nbsp;&nbsp;&nbsp;";
    }

    ## build priority selections
    my $priority_select = '';
    foreach my $value (sort(keys(%PRIORITIES))) {
        my $checked;
        if ($self->priority() == $value) {
            $checked = 'checked';
        } else {
            $checked = '';
        }
        $priority_select .= "<input type=radio name=priority $checked value=$value>$PRIORITIES{$value}&nbsp;&nbsp;&nbsp;";
    }

    my $assigned_to = "<input type=text size=40 name=\"assigned_to\" value=\""
                    . escapeHTML( $FORM{assigned_to} || $self->assigned_to() )
                    . "\">";

    $result->{start_day}        = $day_dropdown;
    $result->{start_month}      = $month_dropdown;
    $result->{start_year}       = $year_dropdown;
    $result->{status}           = $status_select;
    $result->{'priority'}       = $priority_select;
    $result->{'assigned_to'}    = $assigned_to;

    return $result;
}

sub www_toggle_complete {
    my $self = shift;
    $self->is_complete(!$self->is_complete());
    my $parentid = $self->parent_id();
    $self->save();
    $self->md_redirect("index.pl?iid=$parentid");
}

sub _input_form_fields {
    my $self = shift;
    my $errormsg = $self->SUPER::_input_form_fields();

    # input the date if it is defined
    my $date_defined = (defined($FORM{start_day}) && defined($FORM{start_month}) && defined($FORM{start_year}));

    if ($date_defined) {
        $self->start_date(timelocal(0, 0, 0, $FORM{start_day}, $FORM{start_month}, $FORM{start_year}));
    }

    $self->priority($FORM{'priority'});
    $self->status($FORM{status});
    $self->assigned_to($FORM{assigned_to});

    # check if everything is ok
    $errormsg .= "<br />Invalid Date: $self->months()->{$FORM{start_month}} $FORM{start_day} " . ($FORM{year} + 1900) unless (!$date_defined || Date::Calc::check_date($FORM{start_year}+1900,$FORM{start_month}+1,$FORM{start_day}));

    return $errormsg;
}

sub show_edit_form {
  my $self  = shift;
  my $title = shift;
  $self->debug_msg(4, "Event::show_edit_form($title)");

  my $id = $self->id();

  ###
  ### BUILD DROP-DOWNS
  ###

  my $fields = $self->_generate_form_fields();

  ###
  ### BUILD HTML
  ###

  my $s = "<h1>$title</h1>

          <p /><b>Title</b> (*)<br />
          $fields->{name}

          <p /><b>Due Date</b> (*)<br />
          $fields->{month}
          $fields->{day}
          $fields->{year}

          <p /><b>Start Date</b> (*)<br />
          $fields->{start_month}
          $fields->{start_day}
          $fields->{start_year}

          <p /><b>Status</b><br />
          $fields->{status}

          <p /><b>Priority</b><br />
          $fields->{'priority'}

          <p /><b>Assigned to</b><br />
          $fields->{assigned_to}

          <br />
          <table border=\"0\" cellspacing=\"1\" cellpadding=\"1\">
          </table>

          <p /><b>Notes</b><br />
          $fields->{description}


          <br />
          <br /><br /><br /><i>(*) Mandatory field(s)</i><br />\n";

  return $s;
}


sub show_summary {
    my $self        = shift;
}

sub due_date_field {
    my $self = shift;
    return $self->SUPER::date_field();
}

sub start_date_field {
    my $self = shift;
    return 'i2';
}

sub status_field {
    my $self = shift;
    return 'i3';
}

sub priority_field {
    my $self = shift;
    return 'i4';
}

sub assigned_to_field {
    my $self = shift;
    return 'c2';
}

sub due_date {
    my $self = shift;
    my @input = @_;
    return $self->SUPER::date(@input);
}

sub start_date {
    my $self = shift;
    my $input = shift;
    my $result;

    my $field_name = $self->start_date_field();

    if (defined $input) {
        $result = $self->$field_name($input);
    } else {
        $result = $self->$field_name();
    }
    return $result;
}

sub status {
    my $self = shift;
    my $input = shift;
    my $result;

    my $field_name = $self->status_field();

    if (defined $input) {
        $result = $self->$field_name($input);
    } else {
        $result = $self->$field_name();
    }
    return $result;
}

sub status_name {
    my $self = shift;
    my $result = $STATI{$self->status()};
    return $result;
}
sub priority {
    my $self = shift;
    my $input = shift;
    my $result;

    my $field_name = $self->priority_field();

    if (defined $input) {
        $result = $self->$field_name($input);
    } else {
        $result = $self->$field_name();
    }
    return $result;
}

sub priority_name {
    my $self = shift;
    my $result = $PRIORITIES{$self->priority()};
    return $result;
}

sub assigned_to {
    my $self = shift;
    my $input = shift;
    my $result;

    my $field_name = $self->assigned_to_field();

    if (defined $input) {
        $result = $self->$field_name($input);
    } else {
        $result = $self->$field_name();
    }
    return $result;
}

sub is_complete {
    my $self = shift;
    my $input = shift;
    if (defined $input) {
        if ($input) {
            $self->status($COMPLETE_STATUS);
        } else {
            $self->status($PROGRESS_STATUS);
        }
    }
    my $result = ( $self->status() == $COMPLETE_STATUS );
    return $result;
}

sub get_status_value {
    my $self = shift;
    my $status = lc(shift);
    my $result;

    if ($status eq 'complete') {
        $result = $COMPLETE_STATUS;
    } elsif ($status eq 'progress') {
        $result = $PROGRESS_STATUS;
    } elsif ($status eq 'deferred') {
        $result = $DEFERRED_STATUS;
    }
    return $result;
}

sub is_late {
    my $self = shift;
    my $result = (!$self->is_complete() && $self->due_date() < $self->late_time());
    return $result;
}

# the timestamp value to compare a task to to see if it's late. It's set
# for 24 hours ago.
sub late_time {
    my $self = shift;
    return time-(86400);
}


1;

