###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name$
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
#
#
############################################################################
package WhatsNew;


use strict;
use Gizmo;
use GizmoBuilder;
use Metadot::Report;
use vars qw(@ISA);
my $version = "4.5";

sub get_version {
    return $version;
}

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY);

use Data::Dumper;

@ISA=qw(GizmoBuilder Metadot::Report);
my $is_public = "no";

sub get_gizmo_name {
    return "What's New";
}

sub gizmo_list_decorator {
    return 'HTML::Decorator::NullDecorator';
}

sub initialize {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);
    $self->SUPER::initialize(@_);

    $self->set_field_info (
        "name", "<b>Name</b>", 1,
        "description",   "<b>Description</b>", 0,
        $self->get_number_of_days_name(), "<b>Number of days to look back</b>", 1,
        $self->get_number_of_lines_name(), "<b>Number of lines to display</b>", 1,
        $self->get_whats_new_on_option_name(), "<b>What's new on</b>", 0,
    );


    $self->set_field_type($self->get_whats_new_on_option_name(), 'radio');
    $self->set_field_form_text($self->get_whats_new_on_option_name(), $self->_get_radio_button_html());

}

sub show_edit_form {
    my $self  = shift;
    my $title = shift;

    my $name = $self->name();
    my $desc = $self->description();
    my $numd = $self->get_number_of_days();
    my $numl = $self->get_number_of_lines();

    my $radio = $self->_get_radio_button_html();

    my $html = "<h1>$title</h1>\n";
  
    $html .= "<b>Name</b> (*)<br/>\n"
            ."<input type='text' name='name' size='55' maxlenght='255' value='$name' /><br/><br/>\n"
            ."<b>Description</b><br/>\n"
            ."<textarea name='description' rows='4' cols='80' wrap='virtual' />$desc</textarea><br/><br/>\n"
            ."<b>Number of days to look back</b> (*)<br/>"
            ."<input type='text' name='i1' size='25' maxlenght='100' value='$numd' /><br/><br/>\n"
            ."<b>Number of lines to look back</b> (*)<br/>"
            ."<input type='text' name='i2' size='25' maxlenght='100' value='$numl' /><br/><br/>\n"
            ."<b>What's new on</b><br/>\n"
            ."$radio\n";

    return $html; 
    #return $self->SUPER::show_edit_form(@_);
}

sub show {
    my $self  = shift;
    my $iid             = $self->get_iid();
    my $name            = $self->get_name();
    my $description     = $self->get_description();
    my $buttons           = $self->get_buttons();

    #($buttons) && ($buttons .= "&nbsp;\n");
    my $html            = "<p />\n";

    $html .= "<b>$buttons $name</b>\n";
    $html .= "<br />$description\n" if ($description);

    return $html . $self->display_whats_new_table(-1);
}

sub show_summary {
    my $self  = shift;

    my $iid             = $self->get_iid();
    my $name            = $self->get_name();
    my $description     = $self->get_description();
    my $buttons           = $self->get_buttons();

    #($buttons) && ($buttons .= "&nbsp;\n");
    my $html            = "<p />\n";

    $html .= "<b>$buttons $name</b>\n";
    $html .= "<br />$description\n" if ($description);

    return $html . $self->display_whats_new_table();
}

sub display_whats_new_table {
    my $self = shift;
    my $number_of_lines = shift || $self->get_number_of_lines();

    my $iid             = $self->get_iid();
    my $isa             = $self->get_is_a();
    my $html = '';


    my %args;
    #$args{where_clause} = $CHILD_TYPE->status_field().' != '.$CHILD_TYPE->get_status_value('complete');

    my $data = $self->get_data(\%args);

    # if childs, display row data message
    if ($data->{row_count} > 0) {
        $html .= <<EOM
        <table cellspacing="0" cellpadding="0" border="0" class="report" width="450">
          <thead class="report">
            <tr class="report">
              <th class="report">Name</th>
              <th class="report">Type</th>
              <th class="report">Date Added</th>
            </tr>
          </thead>
          <tbody class="report">
EOM
;
        my $col_count = 3;

        my $rs = $data->{row_set};
        my $totalrows = 0;
        my $more_to_show = 0;
        my $whats_new_on   = $self->get_whats_new_on_option();
        my $parent = $self->get_parent;
        while (my ($child_iid, $child_isa) = $rs->fetchrow()) {
            #print STDERR "\n\nchild_iid:$child_iid child_isa:$child_isa\n\n";
            my $child = $child_isa->new($child_iid);
            #if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC
            next unless $child->is_allowed_to_do('show', $USER);

            # TODO: this check is not very good. need to replace it with something better.
            next unless ($child->parentCheck()); # don't show hidden items

            next if ($whats_new_on eq 'page_and_sub' && $child->can('is_descendant_of') && !$child->is_descendant_of($parent));
            $totalrows++;
            if ($number_of_lines > 0 && $totalrows > $number_of_lines) {
                $more_to_show = 1;
                last;
            }
            $html .= $self->_generate_table_row_summary(
                object      => $child,
            )."\n";
        }
        $rs->finish();


        my $cols_left = 1;
        my $cols_right = $col_count - $cols_left;
        if ($more_to_show) {
            my $more_link = "<a class='smaller' href=\"".$self->make_url("index.pl?iid=$iid&amp;isa=$isa&amp;op=show")."\">View All</a>";
            $html .= <<EOM
      <tr  class="report">
        <td  class="report" colspan="3">
          $more_link
        </td>
      </tr>
EOM
;
        }
        $html .= <<EOM
      </tbody>
    </table>
EOM
;
    } else {
        my $days = $self->get_number_of_days();
        my $day_label = $days == 1 ? 'day': 'days';
        # no childs, just display summary
        $html .= "<br /><span class=\"smaller\">There has been nothing added to the site for the last $days $day_label.</span>\n";
    }

    $html .= "<p />\n";

    return $html;
}

sub get_data {
    my $self = shift;
    my $args = shift;

    # determine the sort
    my $order_by;

    #if ($self->sort_field()) {
    #   $order_by = $self->sort_field();
    #   if ($self->sort_order() >= 0) {
    #       $order_by .= ' asc';
    #   } else {
    #       $order_by .= ' desc';
    #   }
    #} else {
        # define the default
        $order_by =  'CreationDate DESC,'
                     . 'name ASC,'
                     . 'isa ASC';
    #}

    my $id = $self->id;
    my $parentiid = $self->get_parent_iid;
    my $days           = $self->get_number_of_days();
    my $whats_new_on   = $self->get_whats_new_on_option();
    my @exclude_list = qw(Version Question Release FormMakerElement ImageRow ImageTable Job);
    my $where_clause = join(' and ', map("isa !='$_'", @exclude_list));
    if ($days > 0) {
        my $datetime = time - ( 86400 * $days );
        $datetime = Utility->db_time_format($datetime);
        $where_clause .= " and CreationDate >= '$datetime'";
    }
    if ($whats_new_on eq 'page') {
        $where_clause .= " and parentiid=$parentiid";
    }
    #my $where_clause = "parentiid=$id";
    if ($args->{where_clause}) {
        $where_clause .= " and $args->{where_clause}";
    }

    my $rs = DBAccess->sqlSelectMany("iid,isa", "instance", $where_clause, "order by $order_by");
    my ($count) = DBAccess->sqlSelect('count(*)', "instance", $where_clause);

    my $result = {
        row_set     => $rs,
        row_count   => $count,
    };

    #use Data::Dumper;
    #print STDERR "get_data with $where_clause returns\n",Dumper $result;
    return $result;
}

sub _get_radio_button_html {
  my $self = shift;

  my $radio_name = $self->get_whats_new_on_option_name();
  my $display_option = $FORM{$radio_name} || $self->get_whats_new_on_option();
  my $page_checked = '';
  my $page_sub_checked = '';
  my $whole_checked = '';

  if ($display_option eq 'page') {
      $page_checked = 'checked';
  } elsif ($display_option eq 'page_and_sub') {
      $page_sub_checked = 'checked';
  } else {
      $whole_checked = 'checked';
  }

  my $str = <<EOM
<input type="radio" name="$radio_name" value="page" $page_checked>that page only<br />
<input type="radio" name="$radio_name" value="page_and_sub" $page_sub_checked>that page and all subpages<br />
<input type="radio" name="$radio_name" value="whole" $whole_checked>the whole site
EOM
;

  return $str;
}

sub _generate_table_row_summary {
    my $self = shift;
    my %args = @_;
    my $html = '';

    my $object = $args{object};
    my $iid = $object->id();
    my $isa = $object->is_a();
    my $type = $object->get_gizmo_name() || $isa;
    my $name = $object->name() || 'No Title';
    my $creation_date = $object->get_creation_date();

    my $editable = $args{editable};

    # format the due date
    my $date_string;
    if ($creation_date) {
        $date_string = Utility->format_datetime($creation_date, 'M e, Y');
    } else {
        $date_string = '&nbsp;';
    }

    # generate a link for the task, using the name text
    my $name_string = $object->show_panel() . "<a href=\"index.pl?iid=$iid&amp;isa=$isa&amp;op=show\">$name</a>";

    $html .= <<EOM
        <tr class="report" valign="baseline">
          <td class="report">$name_string</td>
          <td class="report">$type</td>
          <td class="report">$date_string</td>
        </tr>
EOM
;

    return $html;
}

sub sort_field {
    my $self = shift;
    my $input = shift;

    if (defined $input) {
        $SESSION->set_value('whatsnew_sort_field', $input);
    }
    return $input || $SESSION->get_value('whatsnew_sort_field') || 'CreationDate';
}

sub sort_order {
    my $self = shift;
    my $input = shift;

    if (defined $input) {
        $SESSION->set_value('whatsnew_sort_order', $input);
    }
    return $input || $SESSION->get_value('whatsnew_sort_order') || -1;
}

sub get_whats_new_on_option_name {
    return 'c1';
}

sub get_whats_new_on_option {
    my $self = shift;

    my $test = $self->get_whats_new_on_option_name();
    my $test2 = $self->c1;
    my $test3 = $self->{'c1'};
    my $isa = $self->is_a();

    return $self->{$self->get_whats_new_on_option_name()};

}

sub get_number_of_days_name {
    return 'i1';
}

sub get_number_of_days {
    my $self = shift;
    return $self->{$self->get_number_of_days_name()};
}

sub get_number_of_lines_name {
    return 'i2';
}

sub get_number_of_lines {
    my $self = shift;
    return $self->{$self->get_number_of_lines_name()};
}






1;
