###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Gizmo3000::Newsletter::Article;

use strict;
use vars qw(@ISA);

use Metadot qw($DISPLAY);
use Metadot::Gizmo3000;
use Metadot::Navigable;
@ISA = qw(Metadot::Gizmo3000 Metadot::Navigable);

use constant GIZMO_CONFIG => {
    GIZMO_INFO   => {
        name                => 'Newsletter Article',
        version             => '1.0',
        description         => 'An article that appears in a Newsletter',
        author              => 'Cory Kim',
        info                => 'cory@metadot.com',
        need_portal_version => 6.4,
        debug               => 3,
        edit_help           => '',
        is_dedicated_child  => 1,  # this gizmo is a child only, no properties etc.
    },
    GIZMO_FIELDS  => [
        {
            name            => 'title',
            label           => 'Title',
            is_mandatory    => 1,
            inline_help     => 'The title of the article',
            store_at_column => 'name',
            size            => 80,
        },
        {
            name            => 'author',
            label           => 'Author',
            is_mandatory    => 0,
            inline_help     => 'The author of the article',
            size            => 80,
            store_at_column => 'c2',
        },
        {
            name            => 'content',
            label           => 'Content',
            type            => 'htmlarea',
            store_at_column => 'description',
        }
    ],
};


# newletter articles redirect to their parent, but add the article_id
sub do_show {
    my $self    = shift;
    my $context = shift;

    my $parent = $self->get_parent();
    my $parent_id = $parent->id();
    if (defined($parent) && defined($parent_id)) {
        $DISPLAY->debug_msg(4, "do_show redirects to navigation parent $parent_id ($parent_id)");
        $DISPLAY->md_redirect("index.pl?iid=$parent_id&article_id=".$self->id());
    } else {
        die("Could not resolve my parent");
    }

}


sub do_show_summary {
    my $self    = shift;
    my $context = shift;

    return 'newsletter3000_article.tt2';
}



1;
