#!/usr/bin/perl

#
# Test file for Newsletter.pm
# $Id: Newsletter.001.t,v 1.3 2005/03/02 05:51:51 cory Exp $
#


use strict;
use Test::Simple tests=>8;

# sets up the environment
use SmokeMetadot;

# this is the class being tested
my $test_class = 'Gizmo3000::Newsletter::Newsletter';



#################################################
# Test: Create a new instance
#################################################

my $obj = $test_class->new();
ok((defined($obj) and ref $obj eq $test_class), "testing new()");



#################################################
# Test: Set name
#################################################

my $test_name = "Test Case Foo";
my $test_description = "My test description";

$obj->set_name($test_name);
$obj->set_description($test_description);

my $name = $obj->name();
my $description = $obj->description();

ok( $name eq $test_name, 'testing set_name() and get_name()');
ok( $description eq $test_description, 'testing set_description() and get_description()');



#################################################
# Test: Saving the instance to the DB
#################################################
$obj->set_uid(-1);
$obj->set_parent_id(-1);


my $id = $obj->add();
ok (defined $id && Utility->is_valid_integer($id), 'testing add()');



#################################################
# Test: Retrieving the instance to the DB
#################################################

# Load the newly saved instance
my $newobj = $test_class->restore($id);
my $restored_id = $newobj->id();
my $restored_name = $newobj->name();
my $restored_description = $newobj->description();

ok( $id == $restored_id, 'testing restore()' );
ok( $test_name eq $restored_name, 'testing restored name');
ok( $test_description eq $restored_description, 'testing restored description');


#################################################
# Test: Deleting an instance
#################################################

$obj->delete();

#testing if really deleted
#we try to load the one we just deleeted and we should get an error

my $newobj2 = $test_class->new($id);
$id = (defined $newobj2) ? $newobj2->id() : undef;

ok( ((!defined($id)) or ($id=='')), 'testing delete()');



#################################################
# That's it!!!!
#################################################
