#!/usr/bin/perl

#
# Test file for Newsletter.pm
# $Id: Newsletter.002.t,v 1.2.2.2 2005/07/01 22:41:27 nico Exp $
#

use strict;
use Test::More tests=>12;

use SmokeMetadot;
use Metadot::Testing::Mechanizer;
use DBAccess;
use MetadotConfig;
use Metadot::PortalFactory;

# set up info
my $mech = Metadot::Testing::Mechanizer->new();
my $parent_page = Metadot::PortalFactory->get_home_object();
my $admin_uid = MetadotConfig->get_reserved_uid('ADMIN');

my $test_class_name             = 'Gizmo3000::Newsletter::Newsletter';
my $test_instance_name          = '_FOO_TEST_INSTANCE_';
my $test_instance_description   = '_FOO_TEST_DESCRIPTION_';

my $child_class_name             = 'Gizmo3000::Newsletter::Article';
my $child_instance_name          = '_FOO_TEST_INSTANCE_';
my $child_instance_description   = '_FOO_TEST_DESCRIPTION_';



################################################################################
# Test: Make sure the instance does not yet exist
################################################################################
ok(!$mech->instance_retrievable(iid=>$parent_page->id(), expr=>$test_instance_name),
   'Instance does not yet exist');


################################################################################
# Test: Create an instance 
################################################################################
use_ok($test_class_name);

my $instance = $test_class_name->new({
    uid         => $admin_uid,
    parent_id   => $parent_page->id(),
    name        => $test_instance_name,
    description => $test_instance_description,
});

isa_ok($instance, $test_class_name);


################################################################################
# Test: Add the instance to the db
################################################################################
my $instance_id = $instance->add();
ok(Utility->is_valid_integer($instance_id), "Got a valid iid $instance_id");

$instance = $test_class_name->restore($instance_id);
ok(defined($instance));
isa_ok($instance, $test_class_name);
ok($instance->id() == $instance_id);


################################################################################
# Test: Instance is visible from parent page
################################################################################
ok($mech->instance_retrievable(iid=>$parent_page->id(), expr=>$test_instance_name),
   'Instance was found in parent page '.$parent_page->id());

################################################################################
# Test: Instance redirects to parent when shown in standalone
################################################################################
ok($mech->instance_retrievable(
       iid=>$instance->id(), 
       expr=>"$test_instance_name\.*$test_instance_description"
   ),
   'Instance was found when requested in standalone');


################################################################################
# Test: Instance is displayed in search results
################################################################################
$mech->get_page(
    isa     => 'Portal',
    op      => 'search',
    filter  => $test_instance_name,
);

ok($mech->content() =~ m/\Q$test_instance_description\E/s,
   "Instance was found in search results");

################################################################################
# Test: Remove the instance from the db
################################################################################
$instance->delete();
$instance = $test_class_name->restore($instance_id);
ok(!defined($instance), "Instance was removed from db");


################################################################################
# Test: Instance is no longer visible from browser
################################################################################
ok(!$mech->instance_retrievable(iid=>$parent_page->id(), expr=>$test_instance_name),
   'Instance was removed from database'
);

#################################################
# That's it!!!!
#################################################


