###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package GizmoTool;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS);

use DBAccess;
use Carp;

use MyPageElement;
use Utility;
use ErrorHandling;

use vars qw(@ISA);
@ISA=qw(MyPageElement);


my @TOOL_LIST = ();
load_and_initialize();

################################################################
####                     INITIALIZE                        #####
################################################################

### load all GizmoTool and initialize if necessary
sub load_and_initialize {

    my $found_modules = Utility->read_modules_in_directory("GizmoTool");

    while (my ($module, $rel_filename) = each %{$found_modules} ) {
    # load the module
    require "$rel_filename";
    # check if the channel exists

    my ($cid) = DBAccess->sqlSelect("cid","channel","isa='$module'");

    $module->initialize() unless (defined $cid);
    }

    @TOOL_LIST = ( keys %{$found_modules} );
}

sub lists_tools {
    my $class = shift;

    return @TOOL_LIST;
}


###
### ACCESS CONTROL METHODS (Auditable Interface Methods)
###

# should always be overridden in derived classes
sub get_AC {
  my $self = shift;
  my $class = ref $self || $self;

  croak "&get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

  confess "Thou shalt never instantiate GizmoTool, yadda yadda yadda.";

}

sub get_parent {
  my $self = shift;

  my $container;
  my $parent_id = $self->parent_id();

  if(defined $parent_id) {
    $container = Gizmo->restore($parent_id);
  }

  return $container;

}

sub get_owner_uid {
  my $self = shift;

  return $self->uid();
}


################################################################
####                     CONSTRUCTOR                       #####
################################################################

sub new {
    my $proto = shift;
    my $cid   = shift;
    my $uid   = shift;

    my $class = ref($proto) || $proto;
    my $self  = $class->SUPER::new(undef,$uid);

    $cid = undef unless Utility->is_valid_integer($cid); # secu_200401
    $uid = undef unless Utility->is_valid_integer($uid); # secu_200401

    if ((defined $cid) && (defined $uid)) {
      # load channel & mypage information
      ($self->{cid}, $self->{mpid}, $self->{name}, $self->{description}, $self->{url}, $self->{home_url}, $self->{html}, $self->{col}, $self->{mypageorder})
    = DBAccess->sqlSelect("cid, mpid, name, description, url, homeurl, html, col, mypageorder",
            "channel, mypage", "mypage.id=$cid and mypage.userid=$uid and mypage.id=channel.cid");
    }
    if ((defined $cid) && !(defined $self->{cid})) {
      # load only channel information
      ($self->{cid}, $self->{name}, $self->{description}, $self->{url}, $self->{home_url}, $self->{html})
    = DBAccess->sqlSelect("cid, name, description, url, homeurl, html", "channel", "cid=$cid");
    }


    $self->{is_a} = "GizmoTool";
    $self->{items} = undef;
    $self->{_intro_text_string} = "";


    bless ($self, $class);

    return $self;
}

################################################################
####                 EDIT PREVIEW ADD                      #####
################################################################

sub www_edit {
    my $self = shift;
    $self->debug_msg(3, "GizmoTool::www_edit()");

    my $cid = $self->defined_or_exit($self->cid(),"GizmoTool::www_edit() - cid is missing.");
    my $isa = $self->is_a();

    #load the params for the mpid into hash
    $self->_load_tool_params();

    my $forms = $self->show_edit_form($isa);

    $self->print_form($forms, "Parameters");
}

sub www_info {
  my $self = shift ;
  $self->debug_msg(3, "GizmoTool::www_info()");

  my $class = ref $self || $self ;
  my $info = $self->_show_info();
  $self->print_form($info,$class);
}

sub www_preview {
  my $self = shift ;
  $self->debug_msg(3, "GizmoTool::www_preview()");

  my $ciid = $self->_get_item($self->{default});
  $self->_get_item_content($ciid);

  $self->SUPER::www_preview();
}

sub www_add {
  my $self = shift ;
  my $isa = $self->is_a();
  $self->debug_msg(3, "GizmoTool::www_add()");

  my $id = $self->defined_or_exit($FORM{id},"GizmoTool::www_add() - id is missing.");
  my $col = $FORM{col} ;
  my $row = $FORM{row} ;

  $self->{mpid} = $self->_add($id,$isa,$col,$row);

  $self->_add_item($self->{default});

  my $redirect_url = $FORM{cancelurl} || $SESSION->{referer} || 'userchannel.pl';
  $self->md_redirect($redirect_url);
}

################################################################
####       ADD_ITEM  -  DELETE_ITEM  - MODIFY_ITEM         #####
################################################################

sub www_save_item {
    my $self = shift;
    my $ciid = $FORM{ciid};
    $self->debug_msg(3, "GizmoTool::www_save_item()");

    my $cid = $self->defined_or_exit($self->cid(),"GizmoTool::www_save_item() - cid is missing.");

    $self->_save_item($ciid);
    $self->md_redirect($SESSION->{referer});
}

sub www_delete_item {
    my $self = shift;
    $self->debug_msg(3, "GizmoTool::www_delete_item()");
    my $ciid = $self->defined_or_exit($FORM{ciid},"GizmoTool::www_delete_item() - ciid is missing.");

    $self->_delete_item($ciid) || confess "GizmoTool.pm: Can't delete item.";

    $self->md_redirect($SESSION->{referer});
}

sub www_add_item() {
    my $self = shift;
    $self->debug_msg(3, "GizmoTool::www_add_item()");
    my $cid = $self->defined_or_exit($self->cid(),"GizmoTool::www_add_item() - cid is missing.");

    $self->_add_item($self->_load_form_params());

    $self->md_redirect($SESSION->{referer});
}

sub url {
    my $self = shift;
    return $self->{url};
}

################################################################
####                     ON SCHEDULE                       #####
################################################################

sub on_schedule {
  my $self = shift;
  my $cid = $self->cid();

  $self->cleanup();

  my ($ciids) = DBAccess->sqlSelectMany("ciid, description", "channelitem", "cid=$cid");

  while (my($ciid, $old_content) = $ciids->fetchrow()) {
      my $params = $self->_get_item_params($ciid);
      my ($name, $content) = $self->refresh_item($params, $old_content);
      DBAccess->sqlUpdate("channelitem", "ciid=$ciid", (name => $name, description => $content)) if (defined $content);
  }
}

sub cleanup {
  my $self = shift;
  my $cid = $self->cid();

  my ($ciids) = DBAccess->sqlSelectMany("ciid", "channelitem", "channelitem.cid=$cid");

  while (my($ciid) = $ciids->fetchrow()) {
      unless (DBAccess->sqlSelect("ciid", "gizmotoolitem", "ciid=$ciid")) {
          DBAccess->sqlDelete("gizmoitemparam", "ciid=$ciid");
          DBAccess->sqlDelete("channelitem", "ciid=$ciid");
      }
  }
  $ciids->finish();
}

sub refresh_item {
  my $self = shift;
  my $isa  = $self->is_a();

  die("A refresh_item function must be implemented in module $isa");
}


################################################################
####              CONTENT  -  DISPLAY_ITEM                 #####
################################################################

# this function overload the AUTOLOAD method in base class
sub content {
  my $self = shift;
  my $index = shift;
  my %names;
  my @items;
  my $total = $self->_load_tool_content();
  $self->_load_tool_params();
  return "" if (!defined $self->{items});

  foreach my $ciid (keys %{$self->{items}}) {
    $names{$self->{items}->{$ciid}->{name}}{$ciid}=1;
  }
  my $j=0;
  foreach my $name (sort keys %names) {

    foreach my $ciid (sort keys %{$names{$name}}) {
      push(@items,$self->display_item($ciid,$index,$j,$total))if($self->display_item($ciid,$index,$j,$total));
      $j = @items;
    }
  }

  my $content = $self->format_list_of_items(@items);
  $content .= $self->{html};
  return $content;
}

sub format_list_of_items {
  my $self = shift ;

  return join("\n",@_);
}

sub display_item {
  my $self = shift ;
  my $ciid = shift ;

  my $display = $self->{items}->{$ciid}->{content};
  return $display;

}

sub get_item_intro_text {
    my $self = shift;
    return $self->{_intro_text_string};
}

sub set_item_intro_text {
    my $self = shift;
    my $ts = shift;
    $self->{_intro_text_string} = $ts;
}

################################################################
####              NEW PARAM & EDIT PARAM FORMS             #####
################################################################

sub show_edit_form {
    my $self = shift;
    my $isa = shift;
    my $forms = "
<a href = 'userchannel.pl'>Back to MyPage</a>
<br />
";
    $forms .= "<p>" . $self->get_item_intro_text() . "</p>";
    $forms .= $self->new_item_params_form() . "
<!-- Start of Param Forms -->

";
    foreach my $ciid (keys %{$self->{items}}){
        $forms .= $self->edit_item_params_form($ciid);
    }

$forms .= "

<!-- End of Param Forms -->
";

    return $forms;
}

sub show_edit_form_for_add {
    my $self = shift;
    my $isa = shift;
    my $ciid_comp =  shift;
    my $forms = "
<a href = 'userchannel.pl'>Back to MyPage</a>
<br />
";

    $forms .= "<!-- Start of Param Forms -->
";
    foreach my $ciid (keys %{$self->{items}}){
        $forms .= $self->edit_item_params_form($ciid) if($ciid==$ciid_comp);
    }

$forms .= "

<!-- End of Param Forms -->
";

    return $forms;
}

sub new_item_params_form {
    my $self = shift;
    my $cid = $self->cid();
    my $isa = $self->is_a();

    my $form = "
<p />
<table border=\"0\" cellspacing = 5 cellpadding = 5>
<tr><td>
<table border=\"0\">
<!-- New Param Form -->
<FORM action='userchannel.pl' method='post'>
";
        foreach my $name (keys %{$self->default()}){
            $form .= "
<tr><th align='right'>$name:</th>
<td><INPUT type='text' name='$name' value=''></td></tr>
";
        }
        $form .= "
</table></td>
<td valign='middle' align='center'>
<INPUT type='hidden' name='isa' value='$isa'>
<INPUT type='hidden' name='op' value='add_item'>
<INPUT type='hidden' name='id' value='$cid'>
<INPUT type='submit' value='Add'>
</FORM>
<!-- End New Param Form -->
</td></tr>
</table>
";

    return $form;
}

sub edit_item_params_form {
    my $self = shift;
    my $ciid = shift;
    my $cid = $self->cid();
    my $isa = $self->is_a();
    my $forms = "
<p />
<table border=\"0\" cellspacing = 5 cellpadding = 5>
<tr><td>
<table border=\"0\">
<!-- Edit Form -->
<FORM action='userchannel.pl' method='post'>
";
    my $value = "";
        foreach my $name (keys %{$self->default()}){
        #aju 08-21-01: escape any single quotes in value field
        $value = $self->{items}->{$ciid}->{params}->{$name};
        $value =~ s/\'/&#146;/g;

            $forms .= "
<tr><th align='right'>$name:</th>
<td><INPUT type='text' name='$name' value='$value'></td></tr>
";
        }
        $forms .= "
</table></td>
<td valign='middle' align='center'>
<INPUT type='hidden' name='isa' value='$isa'>
<INPUT type='hidden' name='op' value='save_item'>
<INPUT type='hidden' name='ciid' value='$ciid'>
<INPUT type='hidden' name='id' value='$cid'>
<INPUT type='submit' value='Save'>
</FORM>
<!-- End Edit Form -->

<!-- Delete Button/Form -->
<FORM action='userchannel.pl' method='post'>
<INPUT type='hidden' name='isa' value='$isa'>
<INPUT type='hidden' name='op' value='delete_item'>
<INPUT type='hidden' name='ciid' value='$ciid'>
<INPUT type='hidden' name='id' value='$cid'>
<INPUT type='submit' value='Delete'>
</FORM>
<!-- End Delete Button/Form -->
</td></tr>
</table>
";
    return $forms;
}

################################################################
####                       PANEL                           #####
################################################################

#mypage element panel
sub _panel {
  my $self = shift ;
  my $isa = $self->is_a();
  my $cid = $self->cid();

  my $images_location = $PARAMS->{'imageslocation'};

  my $s =  "<a href='userchannel.pl?id=$cid&amp;isa=$isa&amp;op=edit'><img class=\"icon\" src=$images_location/obj/edit.gif border=\"0\" valign='top' alt='Edit Me!' class='icon'></a>&nbsp;";
  $s .= "<a href='userchannel.pl?id=$cid&amp;isa=$isa&amp;op=info'><img class=\"icon\" src=$images_location/obj/info.gif border=\"0\" valign='top' alt='Info' class='icon'></a>&nbsp;";
  $s .= $self->SUPER::_panel($cid);
  return $s;
}

################################################################
####                     DATABASE METHODS                  #####
################################################################

sub _get_item_params {
  my $self = shift;
  my $ciid = shift;

  $ciid = undef unless Utility->is_valid_integer($ciid); # secu_200401

  my ($params) = DBAccess->sqlSelectMany("name, value", "gizmoitemparam", "ciid=$ciid");
  my %namehash;
  while (my ($name, $value) = $params->fetchrow)  {
    $namehash{$name} = $value ;
  }
  $params->finish();

  return \%namehash;
}

sub _load_tool_params {
  my $self = shift;
  my $cid = $self->cid() || shift;
  my $mpid = $self->mpid();

  return unless defined $mpid;

  my ($params) = DBAccess->sqlSelectMany("gizmoitemparam.ciid, gizmoitemparam.name, gizmoitemparam.value",
                   "gizmoitemparam, gizmotoolitem", "gizmotoolitem.mpid=$mpid and gizmotoolitem.ciid=gizmoitemparam.ciid");
  while (my ($ciid, $name, $value) = $params->fetchrow)  {
    $self->{items}->{$ciid}->{params}->{$name} = $value ;
  }
  $params->finish();
}

sub _load_tool_content {
  my $self = shift;
  my $cid = shift || $self->cid();
  my $mpid = $self->mpid();
  my $total = 0;
  return unless defined $mpid;

  my ($contents) = DBAccess->sqlSelectMany("channelitem.ciid, channelitem.name, channelitem.description", "channelitem, gizmotoolitem", "gizmotoolitem.mpid=$mpid and gizmotoolitem.ciid=channelitem.ciid ORDER BY creationdate DESC");

  while (my ($ciid, $name, $description) = $contents->fetchrow)  {
    $self->{items}->{$ciid}->{name} = $name ;
    $self->{items}->{$ciid}->{content} = $description ;
    $total++;
  }
  $contents->finish();
  return $total;
}

sub _get_item_content {
  my $self = shift ;
  my $ciid = shift ;

  $ciid = undef unless Utility->is_valid_integer($ciid); # secu_200401
  my ($name, $content) = DBAccess->sqlSelect("name, description", "channelitem", "ciid=$ciid");

  $self->{items}->{$ciid}->{name} = $name;
  $self->{items}->{$ciid}->{content} = $content;

}

sub _load_form_params{
    my $self = shift;
    my %params;

    foreach my $name (keys %{$self->default()}){
        $params{$name} = $FORM{$name};
    }

    return \%params;
}

sub _exists_item_param {
    my $self = shift;
    my $params = shift;
    my $cid = $self->cid();
    my @tables;
    my @where;
    my $tablecount = 1;

    foreach my $name (keys %{$self->default()}){
        # Oracle does not like AS alias here
        #push(@tables, "gizmoitemparam AS g$tablecount");
        push(@tables, "gizmoitemparam g$tablecount");

        push(@where, "g" . ($tablecount - 1) . ".ciid = g$tablecount.ciid") if ($tablecount > 1);
    # aju 08-21-01 allow single quotes in field values.
    my $value = $$params{$name};
    $value =~ s/\'/\\\'/g;
#          push(@where, "g$tablecount.name = '$name' AND g$tablecount.value = '$$params{$name}'");
        push(@where, "g$tablecount.name = '$name' AND g$tablecount.value = '$value'");

        $tablecount++;
    }

    my ($ciid) = DBAccess->sqlSelect("channelitem.ciid", "channelitem, " . join(", ", @tables), "channelitem.cid = $cid AND channelitem.ciid = g1.ciid AND " . join(" AND ", @where));
    return $ciid;
}

sub _create_item_param {
    my $self = shift;
    my $params = shift;
    my $cid = $self->cid();
    my $db_now = Utility->db_now();

    my ($ciid) = DBAccess->sqlInsert("channelitem",
                     (cid           => $cid,
                      creationdate => $db_now)
                    );

    if ($ciid > 0) {
        foreach my $name (keys %{$self->default()}){
            DBAccess->sqlInsert("gizmoitemparam",
                (ciid  => $ciid,
                 name  => $name,
                 value => $$params{$name})
                   );
        }
    }

    return $ciid;
}

sub _get_item {
    my $self = shift;
    my $params = shift;

    my $ciid = $self->_exists_item_param($params);

    unless (defined $ciid) {
        $ciid = $self->_create_item_param($params);

        my ($name, $content) = $self->refresh_item($params);
        DBAccess->sqlUpdate("channelitem", "ciid=$ciid", (name => $name, description => $content)) if (defined $content);

    }

    return $ciid;
}

sub _add_item {
    my $self = shift;
    my $params = shift;
    my $mpid = $self->mpid();

    # check to see if form params must pass any validation
    $self->_check_params($params);

    my $ciid = $self->_get_item($params);

    ErrorHandling->printError("GizmoTool::www_add_item() - ciid($ciid) is invalid.") unless ($ciid > 0);
    DBAccess->sqlInsert("gizmotoolitem",
              (mpid => $mpid,
               ciid => $ciid)
             );
}

sub _save_item {
    my $self = shift;
    my $ciid = shift;

    # check to see if form params must pass any validation
    $self->_check_params($self->_load_form_params());

    $self->_delete_item($ciid) || confess "GizmoTool.pm: Can't edit item.";

    $self->_add_item($self->_load_form_params());
}

# check to see if form params must pass any validation
sub _check_params {
    my $self = shift;
    my $params = shift;

    return 1;
}

sub _delete_item {
    my $self = shift;
    my $ciid = shift;
    my $mpid = $self->mpid();

    $ciid = undef unless Utility->is_valid_integer($ciid); # secu_200401
    if (defined $ciid) {
        DBAccess->sqlDelete("gizmotoolitem", "mpid=$mpid AND ciid=$ciid");
        return 1;
    } else {
        return 0;
    }
}

#overrides base class method
sub _delete {
    my $self = shift;
    my $mpid = $self->mpid();

    DBAccess->sqlDelete("gizmotoolitem", "mpid=$mpid");

    $self->SUPER::_delete();
}

1;

__END__

=head1 NAME

GizmoTool - base API for personal My Page elements - all Gizmo Tools
 are found in the GizmoTool directory.

=head1 SYNOPSIS

    use GizmoTool;

    $gizmotool = GizmoTool->new ([$cid], [$uid]);
    GizmoTool->&load_and_initialize ();
    @TOOL_LISTS = GizmoTool->&lists_tools ();
    $gizmotool->get_AC () : i<interface>
    $uid = $gizmotool->get_owner_uid ();
    $gizmotool->not_logged_in_callback ();
    $gizmotool->www_edit ();
    $gizmotool->www_info ();
    $gizmotool->www_preview ();
    $gizmotool->www_add ();
    $gizmotool->www_save_item ();
    $gizmotool->www_delete_item ();
    $gizmotool->www_add_item() ();
    $url = $gizmotool->url ();
    $gizmotool->on_schedule ();
    $content = $gizmotool->content ($index);
    $content = $gizmotool->format_list_of_items ();
    $display = $gizmotool->display_item ($ciid);
    $intro_string = $gizmotool->get_item_intro_text ();
    $gizmotool->set_item_intro_text ($text_string);
    $forms = $gizmotool->show_edit_form ($isa);
    $forms = $gizmotool->show_edit_form_for_add ($isa, $ciid_comp);
    $form = $gizmotool->new_item_params_form ();
    $forms = $gizmotool->edit_item_params_form ($ciid);

=head1 DESCRIPTION

B<GizmoTool> is the base API for all GizmoTools which are personal My Page
Elements. GizmoTool is descended from L<MyPageElement>. GizmoTool will
handle all database backend function such as saving items that are
associated with the GizmoTool. GizmoTool also contains the on_schedule ()
subroutine which updates the GizmoTool through the MetaDot cron. GizmoTool
also can build the generic input forms for GizmoTools.

=head2 Methods

Constructor; $cid is the Channel ID of the GizmoTool from the channel table.
$uid is the User ID of the user which owns the instance of the GizmoTool as
a L<MyPageElement>. If $cid and $uid are passed, then the GizmoTool's data
is loaded from the channel and mypage tables.

    $gizmotool = GizmoTool->new ([$cid], [$uid]);

Class method that initializes all Gizmo Tools in GizmoTool directory.

    GizmoTool->&load_and_initialize ();

Class method that returns array of all GizmoTools.

    @TOOL_LISTS = GizmoTool->&lists_tools ();


Access control abstract subroutine that must be overwritten in derived
classes. Derived classes will return reference to %AC which defines its
permissions.

    $gizmotool->get_AC ();

Access control subroutine that returns $uid which is the owner of GizmoTool.

    $uid = $gizmotool->get_owner_uid ();

Access control subroutine that defines the behavior of the system when
a signed-out user performs an action that requires being signed-on.

    $gizmotool->not_logged_in_callback ();

Builds the edit form for the paramaters of a GizmoTool.

    $gizmotool->www_edit ();

Displays GizmoTool information.

    $gizmotool->www_info ();

Displays a preview of the GizmoTool's default content.

    $gizmotool->www_preview ();

Adds a new set of parameters of a GizmoTool.

    $gizmotool->www_add ();

Updates a set of parameters of a GizmoTool.

    $gizmotool->www_save_item ();

Deletes a set of paramers of a GizmoTool.

    $gizmotool->www_delete_item ();

Adds a new set of parameters of a GizmoTool.

    $gizmotool->www_add_item() ();

Returns $url which is a GizmoTool data member

    $url = $gizmotool->url ();

Called by MetaDot cron which updates all parameters of a GizmoTool.

    $gizmotool->on_schedule ();

Returns the display of the results of all parameters.

    $content = $gizmotool->content ($index);

Returns formatted $content of GizmoTool paramaters

    $content = $gizmotool->format_list_of_items ();

Returns the content of a single paramter of the GizmoTool

    $display = $gizmotool->display_item ($ciid);

Returns GizmoTool's introduction text; displayed when adding or editing
paramters

    $intro_string = $gizmotool->get_item_intro_text ();

Sets the introdution text of a GizmoTool

    $gizmotool->set_item_intro_text ($text_string);


Returns the display of the forms that add and edit paramters

    $forms = $gizmotool->show_edit_form ($isa);

Deprecated

    $forms = $gizmotool->show_edit_form_for_add ($isa, $ciid_comp);

Returns form for adding a new parameter

    $form = $gizmotool->new_item_params_form ();

Returns forms for editing all exiting parameters

    $forms = $gizmotool->edit_item_params_form ($ciid);

=cut


