###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
################################################################################
# WEATHERTOOL v1.1 - MetaDot Portal Server Gizmo Tool                          #
#                                                                              #
# created by: Mike Saenz & Pierre Renaldo                                      #
#                                                                              #
# This tool allows a user to monitor the current weather of his/her prefered   #
# cities within the US. It fetches the data from Weather24 server.             #
#                                                                              #
################################################################################

package WeatherTool;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $CGI);

use Carp;
use GizmoTool;
use Channel;
use Utility;

use vars qw(@ISA);
@ISA=qw(GizmoTool);

################################################################
####                     INFORMATION                       #####
################################################################

my $NAME          = "Weather Tool";
my $ABSTRACT      = "This tool allows a user to monitor the current weather of his/her prefered cities within the US.";
my $VERSION       = "1.1";
my $LICENSE       = "";
my $AUTHOR        = "Mike Saenz & Pierre Renaldo" ;
my $DOWNLOAD      = "http://www.metadot.com";
my $SERVER        = "http://www.wunderground.com";
my $FETCH_URL     = "http://banners.wunderground.com/banner/infoboxtr";
my $REFRESH_EVERY = 60 ; #minutes
my $DESCRIPTION   = "<a href='http://www.wunderground.com'>Weather Underground</a>";

my %UNITS = (
us => 'Fahrenheit',
metric => 'Celsius');

my %STATES = (

'AL' => 'Alabama',
'AK' => 'Alaska',
'AZ' => 'Arizona',
'AR' => 'Arkansas',
'CA' => 'California',
'CO' => 'Colorado',
'CT' => 'Connecticut',
'DC' => 'District of Columbia',
'DE' => 'Delaware',
'FL' => 'Florida',
'GA' => 'Georgia',
'HI' => 'Hawaii',
'ID' => 'Idaho',
'IL' => 'Illinois',
'IN' => 'Indiana',
'IA' => 'Iowa',
'KS' => 'Kansas',
'KY' => 'Kentucky',
'LA' => 'Louisiana',
'ME' => 'Maine',
'MD' => 'Maryland',
'MA' => 'Massachusetts',
'MI' => 'Michigan',
'MN' => 'Minnesota',
'MS' => 'Mississippi',
'MO' => 'Missouri',
'MT' => 'Montana',
'NE' => 'Nebraska',
'NV' => 'Nevada',
'NH' => 'New Hampshire',
'NJ' => 'New Jersey',
'NM' => 'New Mexico',
'NY' => 'New York',
'NC' => 'North Carolina',
'ND' => 'North Dakota',
'OH' => 'Ohio',
'OK' => 'Oklahoma',
'OR' => 'Oregon',
'PA' => 'Pennsylvania',
'PR' => 'Puerto Rico',
'RI' => 'Rhode Island',
'SC' => 'South Carolina',
'SD' => 'South Dakota',
'TN' => 'Tennessee',
'TX' => 'Texas',
'UT' => 'Utah',
'VT' => 'Vermont',
'VA' => 'Virginia',
'WA' => 'Washington',
'WV' => 'West Virginia',
'WI' => 'Wisconsin',
'WY' => 'Wyoming');


my $AC =
  {
   BUNDLES => {
           MOD => { DESC => 'Edit',
            OPS => [qw(add edit add_item add_listitem save_item save_listitem)],
            DEFAULT_LEVEL => 2,
            MIN_LEVEL => 2,
              },
           DEL => { DESC => 'Delete and Cut',
            OPS => [qw(delete_item delete_gizmotool delete_gizmotool_ok delete)],
            DEFAULT_LEVEL => 8,
            MIN_LEVEL => 8,
            PARENT_DEP => {cut => 'MOD'},
              },
           DISP => { DESC => 'View',
             OPS => [qw(preview info show)],
             DEFAULT_LEVEL => 0,
             MIN_LEVEL => 0,
             DEFAULTS_TO => 'on',
               },
          },
   DISPLAY_ORDER => [qw(DISP MOD DEL EDITP)],
  };

###
### ACCESS CONTROL METHODS
###
sub get_AC {
  my $self = shift;
  my $class = ref $self || $self;

  die "&get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

  return $AC;
}


################################################################
####                     INITIALIZE                        #####
################################################################

sub initialize {
  my $proto = shift;

  my $channel = Channel->new();

  # package name (mandatory)
  $channel->{is_a} = __PACKAGE__;
  # the on_schedule function will be called by the daemon every... (mandatory)
  $channel->{fetch_every} = $REFRESH_EVERY*60; # in seconds
  # name of the Gizmo Tool (mandatory)
  $channel->{name} = $NAME;
  # small description of the Gizmo Tool (mandatory)
  $channel->{description} = $DESCRIPTION;
  # server url where you fetch information
  $channel->{url} = $FETCH_URL;
  # home page of the server
  $channel->{home_url} = $SERVER;
  # tool bottom html
  $channel->{html} = "";

  $channel->add();
}

################################################################
####                     CONSTRUCTOR                       #####
################################################################

sub new {
  my $proto = shift;
  my $cid   = shift;
  my $uid   = shift;

  my $class = ref($proto) || $proto;
  my $self  = $class->SUPER::new($cid,$uid);

  $self->{is_a} = __PACKAGE__;

  $self->{default}->{city}  = "austin";
  $self->{default}->{state} = "TX";
  $self->{default}->{units} = "us" ;     # US or METRIC

  bless ($self, $class);

  return $self;
}

################################################################
####                      REFRESH ITEM                     #####
################################################################

sub refresh_item {
  my $self = shift;
  my $params = shift;

  my $city = $params->{city};
  my $state = $params->{state};
  my $units = $params->{units};
  my $url = $self->url();
  my $home_url = $self->home_url();
  $url .= "_metric" if lc($units) eq 'metric';
  $url .= "/US/$state/$city.gif";
  #my $response = Utility->fetch_url($url);
  #my $status = $response->code;

  #return if ($status != 200);
  #my $content = $response->content;
  #$content =~ s/.*\(\'(.*)\'\)\;/$1/;
  my $content = "<a href=\"$home_url/US/$state/$city.html\" target=\"_new\"><img src=\"$url\" border=\"0\" height=\"108\" width=\"144\"></a>";
  return("Weather for $city, $state", $content);
}

################################################################
####                      DISPLAY                          #####
################################################################

sub display_item {
  my $self = shift;
  my $ciid = shift;

  my $display = $self->{items}->{$ciid}->{content};
  $display = "<center>$display</center>";
  return $display;
}

sub new_item_params_form {
    my $self = shift;
    my $cid = $self->cid();
    my $isa = $self->is_a();
    my $spopup = $self->state_popup();
    my $upopup = $self->unit_popup();

    my $form = "
<p />
<table border=\"0\" cellspacing = 5 cellpadding = 5>
<tr><td>
<table border=\"0\">
<!-- New Param Form -->
<FORM action='userchannel.pl' method='post'>
<tr><th align='right'>City:</th>
<td><input type=text name='city'></td></tr>
<tr><th align='right'>State:</th>
<td>$spopup</td></tr>
<tr><th align='right'>Units:</th>
<td>$upopup</td></tr>
</table></td>
<td valign='middle' align='center'>
<INPUT type='hidden' name='isa' value='$isa'>
<INPUT type='hidden' name='op' value='add_item'>
<INPUT type='hidden' name='id' value='$cid'>
<INPUT type='submit' value='Add'>
</FORM>
<!-- End New Param Form -->
</td></tr>
</table>
";

    return $form;
}

sub edit_item_params_form {
    my $self = shift;
    my $ciid = shift;
    my $cid = $self->cid();
    my $isa = $self->is_a();
    my $city = $self->{items}->{$ciid}->{params}->{city};
    my $spopup = $self->state_popup($self->{items}->{$ciid}->{params}->{state});
    my $upopup = $self->unit_popup($self->{items}->{$ciid}->{params}->{units});

    my $forms = "
<p />
<table border=\"0\" cellspacing = 5 cellpadding = 5>
<tr><td>
<table border=\"0\">
<!-- Edit Form -->
<FORM action='userchannel.pl' method='post'>
<tr><th align='right'>City:</th>
<td><input type=text name='city' value = '$city'></td></tr>
<tr><th align='right'>State:</th>
<td>$spopup</td></tr>
<tr><th align='right'>Units:</th>
<td>$upopup</td></tr>
</table></td>
<td valign='middle' align='center'>
<INPUT type='hidden' name='isa' value='$isa'>
<INPUT type='hidden' name='op' value='save_item'>
<INPUT type='hidden' name='ciid' value='$ciid'>
<INPUT type='hidden' name='id' value='$cid'>
<INPUT type='submit' value='Save'>
</FORM>
<!-- End Edit Form -->

<!-- Delete Button/Form -->
<FORM action='userchannel.pl' method='post'>
<INPUT type='hidden' name='isa' value='$isa'>
<INPUT type='hidden' name='op' value='delete_item'>
<INPUT type='hidden' name='ciid' value='$ciid'>
<INPUT type='hidden' name='id' value='$cid'>
<INPUT type='submit' value='Delete'>
</FORM>
<!-- End Delete Button/Form -->
</td></tr>
</table>
";
    return $forms;
}

sub state_popup {
    my $self = shift;
    my $state = shift;

    my $popup = $CGI->popup_menu(-name => 'state', -default => $state, -values => [sort{ $STATES{$a} cmp $STATES{$b} }(keys(%STATES))], -labels => \%STATES);

    return $popup;
}

sub unit_popup {
    my $self = shift;
    my $unit = shift || 'us';

    my $popup = $CGI->popup_menu(-name => 'units', -default => $unit, -values => [sort(keys(%UNITS))], -labels => \%UNITS);

    return $popup;
}

sub _load_form_params{
    my $self = shift;
    my %params;

    $params{state} = $FORM{state};
    $params{city} = $FORM{city};
    $params{city} =~ s/\s/_/g;
    $params{city} = lc($params{city});
    $params{units} = $FORM{units};

    return \%params;
}

sub _check_params {
    my $self = shift;
    my $params = shift;

    foreach my $key (keys %$params) {
        if ($$params{$key} eq '') {
            $self->print_click_back("Error",
                "The $key value is blank");
            exit;
        }
    }
}

sub _show_info {

return "
<H1>$NAME</H1>
<p />
<table border=\"1\">
<tr>
<th align='right' valign='top'>Abstract:</th>
<td align='left'>$ABSTRACT</td>
</tr>
<tr>
<th align='right' valign='top'>Version:</th>
<td align='left'>$VERSION</td>
</tr>
<tr>
<th align='right' valign='top'>Author:</th>
<td align='left'>$AUTHOR</td>
</tr>
<tr>
<th align='right' valign='top'>License:</th>
<td align='left'>$LICENSE</td>
</tr>
<tr>
<th align='right' valign='top'>Download:</th>
<td align='left'><a href='$DOWNLOAD'>$DOWNLOAD</a></td>
</tr>
<tr>
<th align='right' valign='top'>Server:</th>
<td align='left'><a href='$SERVER'>$SERVER</a></td>
</tr>
<tr>
<th align='right' valign='top'>Refresh every:</th>
<td align='left'>$REFRESH_EVERY minutes</td>
</tr>
</table>
<p />
<br />
<center><a href='userchannel.pl'>Back to MyPage</a></center>
<br />
";
}

1;

__END__

=head1 NAME

GizmoTool::WeatherTool - personal city weather tool

=head1 SYNOPSIS

    use GizmoTool::WeatherTool;
    $weathertool = WeatherTool->new ($cid, $uid);

    $AC = $weathertool->get_AC;
    $weathertool->refresh_item ($params);
    $display_html = $weathertool->display_item ($ciid);
    $form = $weathertool->new_item_params_form;
    $form = $weathertool->edit_item_params_form ($ciid) ;

=head1 DESCRIPTION

B<WeatherTool> is a L<GizmoTool> that allows users to enter a city, state,
and either Fahrenheit or Celsius. WeatherTool return a GIF image with
the city's current weather. A user can enter as many cities as he or
she wishes.

Constructor; Loads the WeatherTool's channel information then calls the
L<GizmoTool> constructor. $cid is the Channel ID and $uid is the current
user attempting to access the WeatherTool.

    $weathertool = WeatherTool->new ($cid, $uid);

Access control subroutine that returns reference to %AC which defines its
permissions.

    $AC = $weathertool->get_AC ();

Updates the content of a set of parameters by fetching the url of the GIF
image. $params is the hash reference to the parameters.

    $weathertool->refresh_item ($params);

Returns display of the content of an individual set of parameters. $ciid is
the Channelitem ID of the channelitem which contains the content.

    $display_html = $weathertool->display_item ($ciid);


Returns form for adding a new parameter - city, state, and either
Fahrenheit or Celsius.

    $form = $weathertool->new_item_params_form;


Returns forms for editing all exiting parameters

    $form = $weathertool->edit_item_params_form ($ciid) ;

=cut

