###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
################################################################################
# WEBMONITOR v1.2 - MetaDot Portal Server Gizmo Tool                           #
#                                                                              #
# created by: Pierre Renaldo                                                   #
#                                                                              #
# This tool allows a user to monitor web servers by testing there HTTP status. #
# The user can be alerted by email if the status of one server is changing.    #
# It has been inspired by Daniel Guermeur's Defcon.                            #
#                                                                              #
################################################################################
package WebMonitor;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $CGI $DISPLAY);

use DBAccess;
use Carp;
#  use Data::Dumper;

use GizmoTool;
use Channel;
use Utility;

use vars qw(@ISA);
@ISA=qw(GizmoTool);

################################################################
####                     INFORMATION                       #####
################################################################

my $NAME          = "Web Monitor";
my $ABSTRACT      = "This tool allows a user to monitor web servers by testing their HTTP status. If you decide to be alerted, you will receive an email when the status of the server changes.";
my $VERSION       = "1.2";
my $LICENSE       = "";
my $AUTHOR        = "Pierre Renaldo" ;
my $DOWNLOAD      = "http://www.metadot.com";
my $SERVER        = "NaN";
my $FETCH_URL     = "NaN";
my $REFRESH_EVERY = 5 ; #minutes
my $DESCRIPTION   = "";

my %ALERTS = (
  "none" => "None",
  "email" => "Email Me"
);


my %HTTP_STATUS = (0     , "The server is not responding",
           1   , "The server is online but the delivered page does not match the given specification",
           201   , "The server is online and has no errors",
           202   , "The server is online and has no errors",
           203   , "The server is online and has no errors",
           204   , "The server is online but returned no content",
           206   , "The server is online but only returned a portion of the page",
           300   , "The server is online and has no errors",
           301   , "The server is online but the page has moved to a different location",
           302   , "The server is online but the page has temporarily moved",
           304   , "The server is online and has no errors",
           400   , "The server is online but you have entered a bad URL to check",
           401   , "The server is online but requires authentication",
           402   , "The server is online but requires payment to access",
           403   , "The server is online but you are not allowed to access this document",
           404   , "The server is online but the page you entered was not found",
           405   , "The server is online but the method you used to request it was not allowed",
           406   , "The server is online but the request was not acceptable",
           407   , "The server is online but your proxy server must request authentication",
           408   , "Request to the server timed-out",
           410   , "The server is online but the page has moved to a different location",
           412   , "The server is online but a conditional retrieval of the document has failed",
           413   , "The server is online but the request URL is too large",
           414   , "The server is online but the request URL is too larg",
           415   , "The server is online but the mime-type your specified is not supported",
           500   , "The server is online but encountered an unexpected error, this is probably due to an error in a CGI script",
           501   , "The server is online but the feature you requested is not implemented",
           502   , "The server encountered an error during a proxy request",
           503   , "The server is temporarily down",
           504   , "The server is online but timed out on a remote proxy request",
           505   , "The server is online but does not accept this HTTP version",
           -1    , "The server either does not exist or there is a network error"
  );


my $AC =
  {
   BUNDLES => {
           MOD => { DESC => 'Edit',
            OPS => [qw(add edit add_item add_listitem save_item save_listitem)],
            DEFAULT_LEVEL => 2,
            MIN_LEVEL => 2,
              },
           DEL => { DESC => 'Delete and Cut',
            OPS => [qw(delete_item delete_gizmotool delete_gizmotool_ok delete)],
            DEFAULT_LEVEL => 8,
            MIN_LEVEL => 8,
            PARENT_DEP => {cut => 'MOD'},
              },
           DISP => { DESC => 'View',
             OPS => [qw(preview info show)],
             DEFAULT_LEVEL => 0,
             MIN_LEVEL => 0,
             DEFAULTS_TO => 'on',
               },
          },
   DISPLAY_ORDER => [qw(DISP MOD DEL EDITP)],
  };

###
### ACCESS CONTROL METHODS
###
sub get_AC {
  my $self = shift;
  my $class = ref $self || $self;

  die "&get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

  return $AC;
}


################################################################
####                     INITIALIZE                        #####
################################################################

sub initialize {
  my $proto = shift;

  my $channel = Channel->new();

  # package name (mandatory)
  $channel->{is_a} = __PACKAGE__;
  # the on_schedule function will be called by the daemon every... (mandatory)
  $channel->{fetch_every} = $REFRESH_EVERY*60; # in seconds
  # name of the Gizmo Tool (mandatory)
  $channel->{name} = $NAME;
  # small description of the Gizmo Tool (mandatory)
  $channel->{description} = $DESCRIPTION;
  # server url where you fetch information
  $channel->{url} = $FETCH_URL;
  # home page of the server
  $channel->{home_url} = $SERVER;
  # tool bottom html
  $channel->{html} = "";

  $channel->add();
}

################################################################
####                     CONSTRUCTOR                       #####
################################################################

sub new {
  my $proto = shift;
  my $cid   = shift;
  my $uid   = shift;

  my $class = ref($proto) || $proto;
  my $self  = $class->SUPER::new($cid,$uid);

  $self->{is_a} = __PACKAGE__;

  $self->{default}->{name} = "Metadot";
  $self->{default}->{url} = "http://www.metadot.com";
  $self->{default}->{alert} = "none";  # none | email |
  $self->{default}->{kwords} = "none";

  bless ($self, $class);
  my $intro_string = "<p /> For each URL you are monitoring, you may add optional keywords that must be\n
      present in the delivered page content in order for the server to be considered 'up'.  Leave the\n
      <i>keywords</i> field blank if you do not need to check for any particular content.";
  $self->set_item_intro_text ($intro_string);

  return $self;
}

################################################################
####                      REFRESH ITEM                     #####
################################################################

sub refresh_item {
  my $self = shift;
  my $params = shift;
  my $content = shift;

  my $url   = $params->{url};
  my $name  = $params->{name};
  my $alert = $params->{alert};
  my $kwords = $params->{kwords};

  # we ask for a 3 second timeout
  my $response = Utility->fetch_url($url, 3);

  #$DISPLAY->debug_msg(5,"URL[$url] IN refresh_item, got response code:[".$response->code."]");
  #$DISPLAY->debug_msg(5,"URL[$url] IN refresh_item, got response:".$DISPLAY->dumper($response));

  my $response_content = $response->content;
  #print STDERR "in refresh_item with old content = $content for $name and new content $response_content\n";

  my $new_status = 0;
  #my $uri = '';
  if (defined ($response)) {
      $new_status = $response->code;
      #$uri = $response->base;
  }

#  #    print STDERR "in WebMonitor refresh_item: got response code: " . $response->code . ", and response as string: " .
#        $response->as_string . "\n";

  if (($response->code == 500) &&
      (($response->as_string =~ m/\(Bad hostname \'/) ||
       ($response->as_string =~ m/\(Timeout/))
       ) {
      $new_status = 0;
  }
  # if code nonzero but keywords don't match page content, status is still bad.
  elsif ($response->code && (!($self->_matches_keywords($response_content, $params->{kwords})))) {
      $new_status = 1;
  }

  my ($old_status, $change, $time, $code) = split(/\|/, $content);
#    print STDERR "old status is: $old_status, new status is: $new_status\n";

  $change = ($new_status == $old_status) ? "no" : "yes" ;
  $time = time;
  $content = join("|",($new_status, $change, $time, $response->code));

#    print STDERR "change for $name: $change\n";
  $self->_warn_users($params, $new_status, $time) if ($change eq "yes" && $alert ne "none");
#  $self->_log_changes($params, $new_status, $time) if ($change eq "yes");

  return("Status for $name", $content);
}

# ...
sub _matches_keywords {
    my $self = shift;
    my $rcontent = shift;
    my $keywords_string = shift;

    # look for all the keywords in the page, return status 0 if they don't all match.
    # that is, in boolean terms, this is an AND, not an OR.
    # make the check case-insensitive.
    my @keywords = ();
    if ($keywords_string) {
    @keywords = split(' ', $keywords_string);
    }
    my $match_status = 1;
    foreach my $keyword (@keywords) {
    if (!($rcontent =~ /$keyword/i)) {
#       print STDERR "did not find $keyword in content\n";
        return 0;
    }
    else {
#       print STDERR "found $keyword in content\n";
    }
    }
    return 1;
}

sub _warn_users {
    my $self = shift;
    my $params = shift;
    my $status = shift;
    my $time = shift;
    my $formatted_time = localtime($time);

    my $ciid = $self->_exists_item_param($params);
    my $url   = $params->{url};
    my $name  = $params->{name};

    my ($users) = DBAccess->sqlSelectMany("mypage.userid","mypage, gizmotoolitem",
                      "mypage.mpid=gizmotoolitem.mpid and ciid=$ciid");

    my $subject;
    if ($status =~ m/^2/) {
    $subject = "Up: $name";
    } else {
    $subject = "Down: $name";
    }

    my $basename = $PARAMS->{'basename'};

    while (my ($uid) = $users->fetchrow ) {

    my $email = $USER->get_email_from_uid( $uid );
    my $mesg = "
$HTTP_STATUS{$status} (HTTP status: $status) as of $formatted_time.
Server URL: $url


You are receiving this email because you have been chosen to monitor this channel.

You can modify your subscription at any time by going to your personal subscription

$basename/userchannel.pl and click EDIT in the WebMonitor frame to change your preferences.

>-> This email was sent to $email
";
    my $sm = Sendmail->new($email, "", "", $subject, $mesg);
    $sm->sendMail();
    #      Sendmail::sendMail($email, "", "", $subject, $mesg, "yes");
    }
}

#sub _log_changes {
#  my $self = shift;
#  my $params = shift;
#  my $status = shift;
#  my $time = shift;

#  my $ciid = $self->_exists_item_param($params);

#}
################################################################
####                      DISPLAY                          #####
################################################################

sub display_item {
  my $self = shift;
  my $ciid = shift;
  my $message;
  my $name = $self->{items}->{$ciid}->{params}->{name};
  my $url  = $self->{items}->{$ciid}->{params}->{url};
  my $information = $self->{items}->{$ciid}->{content};
  my ($status, $change, $time, $code) = split(/\|/, $information);
  $time = localtime($time);
  $time =~ s/\s\d\d\d\d$//;

  my $imageslocation = $PARAMS->{'imageslocation'};

  my $logo = "$imageslocation/my/GizmoTool/WebMonitor/";
  if ($status =~ m/^2/) {
    $logo .= "up.gif";
    $message = "The server is up";
  } elsif ($change eq "yes"){
    $logo .= "unstable.gif";
    $message = $HTTP_STATUS{$status};
  } elsif ($status =~ m/^-1/) {
    $logo .= "down.gif";
    $message = $HTTP_STATUS{$status};
  } else {
    $logo .= "down.gif";
    $message = $HTTP_STATUS{$status};
 }
  return "
<tr>
<td><img src='$logo' border=\"0\"></img></td>
<td><a href='$url'><b>$name</b></a><br />
$message<br />
$time</td>
<tr>
";
}


sub format_list_of_items {
  my $self = shift ;

  return "<center><table>".join("\n",@_)."</table></center>";
}

sub new_item_params_form {
    my $self = shift;
    my $cid = $self->cid();
    my $isa = $self->is_a();

     my $alert_popup = $CGI->popup_menu(-name => 'alert', -default => 'none', -values => [sort(keys(%ALERTS))], -labels => \%ALERTS);

    my $form = "
<p />
<table border=\"0\" cellspacing = 5 cellpadding = 5>
<tr><td>
<table border=\"0\">
<!-- New Param Form -->
<FORM action='userchannel.pl' method='post'>
<tr><th align='right'>Server Name:</th>
<td><input type=text name='name'></td></tr>
<tr><th align='right'>URL:</th>
<td><input type=text name='url'></td></tr>
<tr><th align='right'>Keywords to match:</th>
<td><input type=text name='kwords'></td></tr>
<tr><th align='right'>Alert:</th>
<td>$alert_popup</td></tr>
</table></td>
<td valign='middle' align='center'>
<INPUT type='hidden' name='isa' value='$isa'>
<INPUT type='hidden' name='op' value='add_item'>
<INPUT type='hidden' name='id' value='$cid'>
<INPUT type='submit' value='Add'>
</FORM>
<!-- End New Param Form -->
</td></tr>
</table>
";

    return $form;
}

sub edit_item_params_form {
    my $self = shift;
    my $ciid = shift;
    my $cid = $self->cid();
    my $isa = $self->is_a();

    my $name  = $self->{items}->{$ciid}->{params}->{name};
    #aju 08-21-01: escape any single quotes in name value field.
    $name =~ s/\'/&#146;/g;
#      print STDERR "got name: $name\n";
    my $url   = $self->{items}->{$ciid}->{params}->{url};
    my $alert = $self->{items}->{$ciid}->{params}->{alert};
    my $kwords = $self->{items}->{$ciid}->{params}->{kwords};
    #aju 08-21-01: escape any single quotes in keyword value field.
    $kwords =~ s/\'/&#146;/g;

    my $alert_popup = $CGI->popup_menu(-name => 'alert', -default => $alert, -values => [sort(keys(%ALERTS))], -labels => \%ALERTS);

    my $forms = "
<p />
<table border=\"0\" cellspacing = 5 cellpadding = 5>
<tr><td>
<table border=\"0\">
<!-- Edit Form -->
<FORM action='userchannel.pl' method='post'>
<tr><th align='right'>Server Name:</th>
<td><input type=text name='name' value='$name'></td></tr>
<tr><th align='right'>URL:</th>
<td><input type=text name='url' value='$url'></td></tr>
<tr><th align='right'>Keywords to match:</th>
<td><input type=text name='kwords' value='$kwords'></td></tr>
<tr><th align='right'>Alert:</th>
<td>$alert_popup</td></tr>
</table></td>
<td valign='middle' align='center'>
<INPUT type='hidden' name='isa' value='$isa'>
<INPUT type='hidden' name='op' value='save_item'>
<INPUT type='hidden' name='ciid' value='$ciid'>
<INPUT type='hidden' name='id' value='$cid'>
<INPUT type='submit' value='Save'>
</FORM>
<!-- End Edit Form -->

<!-- Delete Button/Form -->
<FORM action='userchannel.pl' method='post'>
<INPUT type='hidden' name='isa' value='$isa'>
<INPUT type='hidden' name='op' value='delete_item'>
<INPUT type='hidden' name='ciid' value='$ciid'>
<INPUT type='hidden' name='id' value='$cid'>
<INPUT type='submit' value='Delete'>
</FORM>
<!-- End Delete Button/Form -->
</td></tr>
</table>
";
    return $forms;
}

sub _load_form_params{
    my $self = shift;
    my %params;

    $params{name}  = $FORM{name};
    if( $FORM{url} =~ "^http://" ){
        $params{url} = $FORM{url};
    } elsif( $FORM{url} =~ "^https://" ){
        $params{url} = $FORM{url};
    } else {
        $params{url} = "http://$FORM{url}";
    }

    $params{alert} = $FORM{alert};
    $params{kwords} = $FORM{kwords};

    return \%params;
}

sub _show_info {

return "
<H1>$NAME</H1>
<p />
<table border=\"1\">
<tr>
<th align='right' valign='top'>Abstract:</th>
<td align='left'>$ABSTRACT</td>
</tr>
<tr>
<th align='right' valign='top'>Version:</th>
<td align='left'>$VERSION</td>
</tr>
<tr>
<th align='right' valign='top'>Author:</th>
<td align='left'>$AUTHOR</td>
</tr>
<tr>
<th align='right' valign='top'>License:</th>
<td align='left'>$LICENSE</td>
</tr>
<tr>
<th align='right' valign='top'>Download:</th>
<td align='left'><a href='$DOWNLOAD'>$DOWNLOAD</a></td>
</tr>
<tr>
<th align='right' valign='top'>Refresh every:</th>
<td align='left'>$REFRESH_EVERY minutes</td>
</tr>
</table>
<p />
<br />
<center><a href='userchannel.pl'>Back to MyPage</a></center>
<br />
";
}

1;

__END__

=head1 NAME

GizmoTool::WebMonitor - allows monitoring and displaying the status of specified web sites

=head1 SYNOPSIS

    use GizmoTool::WebMonitor;
    $webmonitor = WebMonitor->new ($cid, $uid);

    $AC = $webmonitor->get_AC;
    $webmonitor->refresh_item ($params, $content);
    $display_html = $webmonitor->display_item ($ciid);
    $form = $webmonitor->new_item_params_form;
    $form = $webmonitor->edit_item_params_form ($ciid);

=head1 DESCRIPTION

GizmoTool::WebMonitor extends L<GizmoTool>.  GizmoTools can be MyPage
elements.  The WebMonitor allows a user to define a set of URLs to
monitor every N minutes.  Each URL can have an associated set of keywords.
If keywords are defined, they must all be present for the site's status to
be considered "OK".  If a site undergoes a change in status, and if the user has
specified that they be alerted to change, a notification will be emailed.

N, the monitoring frequency, is determined by the frequency at which
metadot's cron daemon runs, but its lower limit is the value of
$REFRESH_EVERY, which is currently set to 5 minutes.

    $webmonitor = WebMonitor->new ($cid, $uid);

Constructor: Calls
the L<GizmoTool> constructor, which will call the WebMonitor's intialization method.

    $AC = $webmonitor->get_AC;

Access control subroutine.  Returns reference to hash which defines its
permissions.

    $webmonitor->refresh_item ($params, $content);

Check the status of a given URL.  $params is a reference to a hash containing values for the url,
its name, the alert method to use if there is a status change, and a white-space-separated
list of keywords to look for in the retrieved URL content.  The page content for the URL is fetched.
If there is an error fetching the page, or if not all of the keywords are present in the
page content (case-insensitive) then a non-OK status code is assigned.  If there has been a change in
the status from the previous poll of the URL, and the user has elected to be alerted, a notification
is sent.

    $display_html = $webmonitor->display_item ($ciid);

Generates html to display status information about a given server (URL).

    $form = $webmonitor->new_item_params_form;

Generates html to build a form into which the user can input info to define a new
web server monitor.

    $form = $webmonitor->edit_item_params_form ($ciid);

Generates html to build a form allowing the user to modify the specifications of an
existing web server monitor.

=cut

