###########################################################################
# Metadot MetaCenter Portal v. 4.0
# Copyright (C) 2001 Metadot Corporation
# Copyright (C) 1999-2000 Daniel Guermeur
#
# Refer to the README file in the top-level directory of this distribution for
# license information.
#
############################################################################
################################################################################
# SAMPLETOOL v1.0 - MetaDot Portal Server Gizmo Tool                           #
#                                                                              #
# created by: Pierre Renaldo                                                   #
#                                                                              #
# This sample tool is not a real Gizmo Tool but just a sample file to help     #
# developers to build their own Gizmo Tools.                                   #
#                                                                              #
################################################################################
package SampleTool;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT);

use Carp;

use GizmoTool;
use Channel;
use Utility;

use vars qw(@ISA);
@ISA=qw(GizmoTool);

################################################################
####                     INFORMATION                       #####
################################################################

my $NAME          = "Sample Tool";
my $ABSTRACT      = "This tool does nothing it is just a sample file";
my $VERSION       = "1.0";
my $LICENSE       = "";
my $AUTHOR        = "Pierre Renaldo" ;
my $DOWNLOAD      = "http://www.metadot.com";
my $SERVER        = "http://www.sampletool.com";
my $FETCH_URL     = "http://www.sampletool.com/download";
my $REFRESH_EVERY = 60 ; #minutes
my $DESCRIPTION   = "This is a Fake Tool";

################################################################
####                     INITIALIZE                        #####
################################################################

sub initialize {
  my $proto = shift;

  my $channel = Channel->new();

  # package name (mandatory)
  $channel->{is_a} = __PACKAGE__;
  # the on_schedule function will be called by the daemon every... (mandatory)
  $channel->{fetch_every} = $REFRESH_EVERY*60; # in seconds
  # name of the Gizmo Tool (mandatory)
  $channel->{name} = $NAME;
  # small description of the Gizmo Tool (mandatory)
  $channel->{description} = $DESCRIPTION;
  # server url where you fetch information
  $channel->{url} = $FETCH_URL;
  # home page of the server
  $channel->{home_url} = $SERVER;
  # tool bottom html
  $channel->{html} = "";

  $channel->add();
}

################################################################
####                     CONSTRUCTOR                       #####
################################################################

sub new {
  my $proto = shift;
  my $cid   = shift;
  my $uid   = shift;

  my $class = ref($proto) || $proto;
  my $self  = $class->SUPER::new($cid,$uid);

  $self->{is_a} = __PACKAGE__;

  $self->{default}->{param1} = "default value of param1";
  $self->{default}->{param2} = "default value of param2";

  bless ($self, $class);

  return $self;
}

################################################################
####                      REFRESH ITEM                     #####
################################################################

sub refresh_item {
  my $self = shift;
  my $params = shift;
  my $old_content = shift;

  my $param1 = $params->{param1};
  my $param2 = $params->{param2};

  my $content = "";
  # this function needs to return the content
  # of the item matching the parameters
  # if you need to know something about
  # the previous state of the item you
  # the old content is passed as argument

  # store the url
  # my $url = $self->url();
  # and then
  # if you need to fetch a URL use
  # my $response = Utility->fetch_url($url);
  # and then
  # my $status = $response->code;
  # my $content = $response->content;

  # return the name and the content of the item
  return("item name", $content);
}

################################################################
####                      DISPLAY                          #####
################################################################

sub display_item {
  my $self = shift;
  my $ciid = shift;

  my $name    = $self->{items}->{$ciid}->{name};
  my $content = $self->{items}->{$ciid}->{content};
  my $params  = $self->{items}->{$ciid}->{params};

  my $param1 = $params->{param1};
  my $param2 = $params->{param2};

  my $display = "";
  # here you need to format the display of one item
  # you have access to the name and the content of this item
  # and also the parameters

  return $display;
}

sub format_list_of_items {
  my $self = shift ;
  my @items = @_;

  my $display = "";
  # you receive here a list of items displays
  # so that you can format them globally

  return $display;
}

### You don't need to re-implement the next couple functions
### but you may want to if you want the interface to look good

#sub new_item_params_form {
#    my $self = shift;
#    my $cid = $self->cid();
#    my $isa = $self->is_a();

#    my $default_param1 = $self->{default}->{param1};
#    my $default_param2 = $self->{default}->{param2};

#    my $form = "
#<FORM action='userchannel.pl' method='post'>
#<INPUT type='hidden' name='id' value='$cid'>
#<INPUT type='hidden' name='isa' value='$isa'>
#<INPUT type='hidden' name='op' value='add_item'>
#...
#</FORM>
#";
#    # format the HTML form for a new item
#    # you can use the default parameters

#    return $form;
#}

#sub edit_item_params_form {
#    my $self = shift;
#    my $ciid = shift;
#    my $cid = $self->cid();
#    my $isa = $self->is_a();

#    my $param1 = $self->{items}->{$ciid}->{params}->{param1};
#    my $param2 = $self->{items}->{$ciid}->{params}->{param2};

#    my $forms = "
#<!-- Edit Form -->
#<FORM action='userchannel.pl' method='post'>
#<INPUT type='hidden' name='id' value='$cid'>
#<INPUT type='hidden' name='isa' value='$isa'>
#<INPUT type='hidden' name='op' value='save_item'>
#<INPUT type='hidden' name='ciid' value='$ciid'>
#....
#</FORM>
#<!-- End Edit Form -->

#<!-- Delete Button/Form -->
#<FORM action='userchannel.pl' method='post'>
#<INPUT type='hidden' name='id' value='$cid'>
#<INPUT type='hidden' name='isa' value='$isa'>
#<INPUT type='hidden' name='op' value='delete_item'>
#<INPUT type='hidden' name='ciid' value='$ciid'>
#....
#</FORM>
#<!-- End Delete Form -->
#";
#    return $forms;
#}

sub _load_form_params{
  my $self = shift;
  my %params;

  $params{param1} = $FORM{param1};
  $params{param2} = $FORM{param2};

  # Here you need to gather the parameters return by the POST form
  # you may want to check the righteousness or make them not case
  # sensitive.

  return \%params;
}

sub _show_info {

  # this function format the informations of the Gizmo Tools

return "
<H1>$NAME</H1>
<p />
<table border=\"1\">
<tr>
<th align='right' valign='top'>Abstract:</th>
<td align='left'>$ABSTRACT</td>
</tr>
<tr>
<th align='right' valign='top'>Version:</th>
<td align='left'>$VERSION</td>
</tr>
<tr>
<th align='right' valign='top'>Author:</th>
<td align='left'>$AUTHOR</td>
</tr>
<tr>
<th align='right' valign='top'>License:</th>
<td align='left'>$LICENSE</td>
</tr>
<tr>
<th align='right' valign='top'>Download:</th>
<td align='left'><a href='$DOWNLOAD'>$DOWNLOAD</a></td>
</tr>
<tr>
<th align='right' valign='top'>Server:</th>
<td align='left'><a href='$SERVER'>$SERVER</a></td>
</tr>
<tr>
<th align='right' valign='top'>Refresh every:</th>
<td align='left'>$REFRESH_EVERY minutes</td>
</tr>
</table>
<p />
<br />
<center><a href='userchannel.pl'>Back to MyPage</a></center>
<br />
";
}

1;
