###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Group;

use strict;

use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $CACHE $CGI $ACCESS_BROKER $DISPLAY);

use Digest::MD5 qw(md5_base64);

use Metadot::UserFactory;
use Metadot::User::Default;
use Metadot::ObjectAccess::Restorable;
use DBAccess;
use Carp;
use Metadotclass;
use Utility;
use Portal;
use Auditable;
use Gizmo;
use Sendmail;
use Session;
use Utility;
use Data::Dumper;
use HTML::TableFrame;
use HTML::Components;
use vars qw(@ISA);
@ISA=qw(Portal Auditable Metadot::ObjectAccess::Restorable);

my $HIDDEN_GROUPS = {
             ADMIN => 1,
            };

sub get_AC {
    my $self = shift;
    return $self->get_implementation()->get_AC($self, @_);
}

sub is_a {
    my $self = shift;

    return ref($self);
}

sub save {
    my $self = shift;

    $self->save_permissions;
}

sub get_owner_uid {
    my $self = shift;

    return $self->{uid};
}

sub parent_id {
    my $self = shift;
        return 0;
}

sub id {
    my $self = shift;
    return 1;
}

# Class variables are evil! This one causes huge concurrency problems
# my %groups;

#### GROUP PERMISSION VARIABLES
my $IS_OWNER = 0;
my $IS_ADMIN = 0;
my $IS_MANAGER = 0;
####

###################################################
sub validate_or_refresh_cache {
    my $class = shift;

    my $impl = $class->get_implementation();
    return $impl->validate_or_refresh_cache($class,@_);
}

sub get_groups_hash {
    my $class = shift;
    $class->validate_or_refresh_cache();

    return $CACHE->get_value('GROUPS_HASH') || {};
}


sub get_group_name_by_grp_id {
    my $class = shift;
    my $grp_id = shift;
    my $get_organization_info = shift || 0;

    $grp_id = undef unless Utility->is_valid_integer($grp_id); # secu_200401

    if (defined $grp_id) {
        my $groups = $class->get_groups_hash();

        my $result1 = $groups->{BY_GRP_ID}{$grp_id}{GROUP_NAME};
        my $result2 = $groups->{BY_GRP_ID}{$grp_id}{IS_ORG};
        unless (defined $result1) {
            #print STDERR "Group: CACHE MISS for get_group_name_by_grp_id($grp_id)\n";
            ($result1, $result2) = DBAccess->sqlSelect('group_name, is_organization', 'grp', "grp_id=$grp_id");
            $groups->{BY_GRP_ID}{$grp_id}{GROUP_NAME} = $result1 || '';
            $groups->{BY_GRP_ID}{$grp_id}{IS_ORG} = $result2 || '';
            $CACHE->set_value('GROUPS_HASH', $groups);
        }

        if ($get_organization_info) {
            return ($result1, $result2);
        } else {
            return $result1;
        }
    }
}

sub get_group_id_by_name {
    my $class = shift;
    my $group_name = shift;

    my $groups = $class->get_groups_hash();
    my $result = $groups->{BY_GRP_NAME}{$group_name}{GRP_ID};
    unless (defined $result) {
        #print STDERR "Group: CACHE MISS for get_group_id_by_name($group_name)\n";
        ($result) = DBAccess->sqlSelect('grp_id', 'grp', "group_name=".DBAccess->quote($group_name)); #secu_200401
        $groups->{BY_GRP_NAME}{$group_name}{GRP_ID} = $result || '';
        $CACHE->set_value('GROUPS_HASH', $groups);
    }
    return $result;
}

sub get_subgroups_for_group {
    my $class = shift;
    my $grp_id = shift;

    $grp_id = undef unless Utility->is_valid_integer($grp_id); # secu_200401

    my $groups = $class->get_groups_hash();
    my $result = $groups->{BY_GRP_ID}{$grp_id}{SUBGROUPS};
    ##print STDERR "Group->get_subgroups_for_group($grp_id) checking cache, got\n",Dumper $result;

    if (defined $result) {
        if (!$result) {
            $result = [];
        } else {
            $result = [ keys( %{ $result } ) ];
        }
    } elsif ($grp_id) {
        #print STDERR "Group: CACHE MISS for get_subgroups_for_group($grp_id)\n";
        my $sth = DBAccess->sqlSelectMany('distinct subgrp_id', 'grpmembers', "grp_id=$grp_id AND subgrp_id != 0");
        my @results = ();
        while (my ($subgrp_id) = $sth->fetchrow()) {
            $groups->{BY_GRP_ID}{$grp_id}{SUBGROUPS}{$subgrp_id} = 1;
            push @results, $subgrp_id;
        }
        if (@results) {
            $result = \@results;
        } else {
            $groups->{BY_GRP_ID}{$grp_id}{SUBGROUPS} = 0;
            $result = [];
        }
        $sth->finish();
        $CACHE->set_value('GROUPS_HASH', $groups);
        ##print STDERR "Group->get_subgroups_for_group($grp_id) yields\n",Dumper($groups->{BY_GRP_ID}{$grp_id}{SUBGROUPS});
    }
    return $result;
}

#CKIM-SENDABLE
sub get_emails_by_grp_id {
    my $class = shift;
    my $grp_id = shift || die("Must provide a group ID to query against.\n");
    my $recurse = shift;    # aju

    return $USER->get_emails_by_grp_id( $grp_id, $recurse );

}

sub get_emails_by_grp_name {
    my $class = shift;
    my $grp_name = shift || die("Must provide a group name to query against.\n");
    my $recurse = shift;    # aju

    return $USER->get_emails_by_grp_name( $grp_name, $recurse );
}

sub new {
    my $proto = shift;
    my $grp_id = shift;
    my $class = ref($proto) || $proto;
    my $self = $class->SUPER::new;

    $grp_id = undef unless Utility->is_valid_integer($grp_id); # secu_200401

    $self->{grp_id}            = undef;
    $self->{group_name}        = undef;
    $self->{group_description} = undef;
    $self->{is_organization}   = undef;
    $self->{organization_homepage} = undef;
    $self->{organization_post_registration} = undef;
    $self->{uid}               = undef;
    $self->{iid}               = undef;

    $self->{welcome_letter_heading} = undef;
    $self->{welcome_letter_body}    = undef;


    if (defined($grp_id)) {
        (
         $self->{grp_id},
         $self->{group_name},
         $self->{group_description},
         $self->{is_organization},
         $self->{organization_homepage},
         $self->{organization_post_registration},
         $self->{uid},
         $self->{iid},
         $self->{welcome_letter_heading},
         $self->{welcome_letter_body},
        )
         = DBAccess->sqlSelect("grp_id,
                      group_name,
                      group_description,
                      is_organization,
                      organization_homepage,
                      organization_post_registration,
                      userid,
                      iid,
                      welcome_letter_heading,
                      welcome_letter_body",
                     "grp",
                     "grp_id=$grp_id"
                    );
    }

    bless ($self, $class);

    return $self;
}

sub get_value {
    my $self = shift;
    my $property = shift || die("Must supply a property key");
    return $self->{$property};
}

sub set_value {
    my $self = shift;
    my $property = shift || die("Must supply a property key");
    my $value = shift;
    $self->{$property} = $value;
}

sub set_properties {
    my $self = shift;
    my %properties = @_;

    # CKIM: this is not the right place for this! set_properties is called when
    #       editing groups, too. So someone edits another person's group, the
    #       ownership is changed!
    # $properties{uid} ||= $USER->{uid};

    foreach my $key ( keys(%properties) ) {
        $self->set_value($key, $properties{$key});
    }
    return %properties;
}


sub get_group_properties {
    my $self = shift;
    return $self->get_implementation->get_group_properties();
}


# Restorable methods
sub extract_state {
    my ($self) = shift;
    ref($self) || ($self = $self->new());

    my $memento = Metadot::ObjectAccess::Memento->new();
    foreach my $key ($self->get_group_properties()) {
        $memento->set_state($key, $self->get_property($key));
    }
    return $memento;
}

sub restore_state {
    my ($self, $memento) = @_;
    foreach my $key ($self->get_group_properties()) {
        $self->set_value($key, $memento->get_state($key));
    }
    return $self;
}



# restores the group's properties from the database
sub load {
   my $self = shift;
   my $impl = $self->get_implementation();
   return $impl->load($self, @_);
}

# saves the group to the database
sub update {
    my $self = shift;
    my %properties = @_;
    unless (%properties) {die("Must supply the new properties");}

    # update the object
    %properties = $self->set_properties(%properties);
    %properties = $self->translate_properties_to_db_fields(%properties);
    DBAccess->sqlUpdate( 'grp', "grp_id='$self->{grp_id}'", %properties );
    $CACHE->set_as_dirty('GROUPS_HASH');
}


sub translate_properties_to_db_fields {
    my $self = shift;
    my %properties = @_;

    if (!defined($properties{userid}) && defined($properties{uid})) {
        # map uid to userid
        $properties{userid} = $properties{uid};
        delete ($properties{uid});
    }

    return %properties;
}

# returns a set of group objects that follow a set of SQL select criteria
sub get_groups_where {
   my $self = shift;
   my $impl = $self->get_implementation();
   return $impl->get_groups_where($self, @_);
}

# aju -- mod for nested groups
sub is_member {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my ($uid, $grp) = @_;

    my $grp_id = $class->get_group_id_by_name($grp);
    return $class->is_member_of_grp_id($uid, $grp_id);
}

# this one does not recur into nested groups
sub is_local_member_of_grp_id {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my ($uid, $grp_id) = @_;

    $uid = undef unless Utility->is_valid_integer($uid); # secu_200401
    $grp_id = undef unless Utility->is_valid_integer($grp_id); # secu_200401

    my $groups = $class->get_groups_hash();
    my $result = $groups->{BY_GRP_ID}{$grp_id}{MEMBERS}{$uid};

    if (defined $result) {
        $result = ($result);
    } elsif ($grp_id && $uid) {
        #print STDERR "Group: CACHE MISS for is_local_member_of_grp_id($uid, $grp_id)\n";
        # first check the owner of the group
        my ($owner) = DBAccess->sqlSelect( 'userid', 'grp', "grp_id=$grp_id" );
        if ($owner == $uid) {
            $result = 1;
        }
        # then check if the user is a member of the group
        unless ($result) {
            ($result) = DBAccess->sqlSelect( 'count(*)', 'grpmembers', "grp_id=$grp_id AND userid = $uid" );
        }
        $groups->{BY_GRP_ID}{$grp_id}{MEMBERS}{$uid} = $result || 0;
        $CACHE->set_value('GROUPS_HASH', $groups);
    }
    #print STDERR "Group->is_local_member_of_grp_id($uid, $grp_id) returns ($result)\n";
    return ($result);
}

# aju -- mod for nested groups
# same as above except uses grp id instead of name
sub is_member_of_grp_id {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my ($uid, $grp_id) = @_;
    my $result = undef;

    if ($uid && $grp_id) {
        $DISPLAY->debug_msg(5, "in is_member for user[$uid] and grp id[$grp_id]");

        if ($class->is_local_member_of_grp_id($uid, $grp_id)) {
            $result = 1;
        } else {
            #if didn't find user listed explicitly as a group member, check recursively if
            #they are a member of one of the group's subgroups.

            $DISPLAY->debug_msg(5, "checking to see if $uid is member of group's subgroups");
            my $subgroups = $class->get_subgroups_for_group($grp_id);
            #print STDERR "subgroups of $grp_id are: ", Dumper($subgroups), "\n";

            my $member_grps = $class->get_group_ids_where_member($uid);
            #print STDERR "looking at grps for this member; got\n", Dumper $member_grps;

            my $intersection = Utility->intersection($subgroups, $member_grps) || [];
            #print STDERR "Intersection is\n",Dumper $intersection;
            $result = (defined $intersection->[0]);

        }

    }

    $DISPLAY->debug_msg(5, "is_member for user[$uid] and grp id[$grp_id] returns $result");
    return $result;
}


sub is_manager {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my ($uid, $grp) = @_;

    my $grp_id = $class->get_group_id_by_name($grp);
    return $class->is_manager_of_grp_id($uid, $grp_id);
}

sub is_manager_of_grp_id {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my ($uid, $grp_id) = @_;

    $uid = undef unless Utility->is_valid_integer($uid); # secu_200401
    $grp_id = undef unless Utility->is_valid_integer($grp_id); # secu_200401

    my $groups = $class->get_groups_hash();
    my $result = $groups->{BY_GRP_ID}{$grp_id}{MANAGERS}{$uid};
    if (defined $result) {
        $result = ($result);
    } elsif ($uid && $grp_id) {
        #print STDERR "Group: CACHE MISS for is_manager_of_grp_id($uid, $grp_id)\n";
        ($result) = DBAccess->sqlSelect('count(*)', 'grpmembers', "grp_id=$grp_id AND userid=$uid AND is_manager != 0");
        $groups->{BY_GRP_ID}{$grp_id}{MANAGERS}{$uid} = $result || 0;
        $CACHE->set_value('GROUPS_HASH', $groups);
    }
    return $result;
}

sub number_of_members {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $grp = shift || die "must provide grp";

    my $groups = $class->get_groups_hash();
    my $result = $groups->{BY_GROUP}{$grp}{MEMBERS};
    if (defined $result) {
        $result = scalar(keys %{ $result });
    } elsif ($grp) {
        $groups->{BY_GROUP}{$grp}{MEMBERS} = {};
        $result = 0;
        my $grp_id = $class->get_group_id_by_name($grp);
        if ($grp_id) {
            #print STDERR "Group: CACHE MISS for number_of_members($grp)\n";
            my $sth = DBAccess->sqlSelectMany('distinct userid', 'grpmembers', 'userid IS NOT NULL');
            while (my ($userid) = $sth->fetchrow()) {
                $groups->{BY_GROUP}{$grp}{MEMBERS}{$userid} = 1;
                $result++;
            }
            $sth->finish();
            $CACHE->set_value('GROUPS_HASH', $groups);
        }
    }

    return $result;
}


# this processes the list of groups for a given member
sub _get_groups_data_where_member {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $uid = shift || die "must provide uid";
    #print STDERR "IN _get_groups_data_where_member($uid)\n";

    $uid = undef unless Utility->is_valid_integer($uid); # secu_200401
    my $result = {};

    if (defined $uid) {
        my $groups = $class->get_groups_hash();
        if (defined $groups) {
            $result = $groups->{BY_USER}{$uid};
        }
        if (!defined($result) && defined($uid)) {
            $result = {
                'NAME' => {},
                'ID'   => {},
            };

            #print STDERR "Group: CACHE MISS for get_groups_where_member($uid)\n";
                $groups->{BY_USER}{$uid} = {};
                my $sth = DBAccess->sqlSelectMany('distinct group_name, gm.grp_id', 'grpmembers gm, grp', "gm.userid=$uid AND gm.grp_id=grp.grp_id");
                while (my ($group_name, $grp_id) = $sth->fetchrow()) {
                    $result->{'NAME'}{$group_name} = 1;
                    $result->{'ID'}{$grp_id} = 1;
                }
            $sth->finish();
            $groups->{BY_USER}{$uid} = $result;
            $CACHE->set_value('GROUPS_HASH', $groups);
        }
    }

    #print STDERR "_get_groups_data_where_member($uid) returns\n",Dumper $result;
    return $result;
}


# this returns a list of group ids for a given userid
sub get_group_ids_where_member {
    my $class = shift;
    my $uid = shift || die "must provide uid";
    return [keys %{$class->_get_groups_data_where_member($uid)->{'ID'}}];
}

# this returns a list of group ids for a given userid
sub get_group_names_where_member {
    my $class = shift;
    my $uid = shift || die "must provide uid";
    #print STDERR "IN get_group_names_where_member($uid)\n";
    return [keys %{$class->_get_groups_data_where_member($uid)->{'NAME'}}];
}

# This returns a list of group id's or objects. Pass instantiate=>1 to get objects.
# Pass recursive=>1 to get look recursively. This method can be somewhat expensive,
# particularly if instantiate and recursive are both turned on
sub get_groups_where_member {
    my $class = shift;
    my $args  = shift || {};

    my $uid         = $args->{uid};
    my $grp_id      = $args->{grp_id};
    my $recursive   = $args->{recursive};
    my $visited     = $args->{visited} || {};
    my $instantiate = $args->{instantiate};

    die("Must provide either a uid or grp_id") unless (defined($uid) || defined($grp_id));

    $DISPLAY->debug_msg(5, 'IN get_group_names_where_member(uid=>'.($uid||'').', grp_id=>'.($grp_id||'').', recursive=>'.($recursive||'').')');
    my @grp_ids = ();

    if (defined $uid) {
        push @grp_ids, keys %{$class->_get_groups_data_where_member($uid)->{'ID'}};
        $DISPLAY->debug_msg(5, "Based on uid, got group ids @grp_ids");
    }

    if (defined $grp_id) {
        my $sth = DBAccess->sqlSelectMany('grp_id', 'grpmembers', 'subgrp_id='.$grp_id);
        while (my ($grp_id) = $sth->fetchrow()) {
            push @grp_ids, $grp_id;
        }
        $sth->finish();
    }

    if ($recursive) {
        my @base_grp_ids = @grp_ids;
        foreach my $grp_id (@base_grp_ids) {
            $DISPLAY->debug_msg(5, "Recursing from grp_id $grp_id");
            unless ($visited->{$grp_id}++) {
                push @grp_ids, @{$class->get_groups_where_member({ grp_id=>$grp_id, visited=>$visited, recursive=>$recursive})}
            }
        }
    }

    if ($instantiate) {
        my @results = ();
        foreach my $grp_id (@grp_ids) {
            push @results, Group->new($grp_id);
        }
        return \@results;
    } else {
        return \@grp_ids;
    }
}



sub get_groups {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $groups = $class->get_groups_hash();
    my $result = $groups->{BY_GROUP};
    if (defined $result) {
        @$result = (keys %{ $result });
    } else {
        $result = [];
        #print STDERR "Group: CACHE MISS for get_groups()\n";
        $groups->{BY_GROUP} = {};
        my $sth = DBAccess->sqlSelectMany('group_name', 'grp', '1=1');
        while (my ($group_name) = $sth->fetchrow()) {
            $groups->{BY_GROUP}{$group_name} = 1;
            push @$result, $group_name;
        }
        $sth->finish();
        $CACHE->set_value('GROUPS_HASH', $groups);
    }

    return $result;
}

# CKIM: this method just sets a class variable. We're deleting this class
#       variable because of concurrency issues
# sub set_group_cache {
#     my $proto = shift;
#     my $class = ref($proto) || $proto;
#         my $groups_ref = shift;
#         %groups = %{$groups_ref};
# }

sub get_grp_ids {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $groups = $class->get_groups_hash();
    my @results = ();
    my $sth = DBAccess->sqlSelectMany('grp_id', 'grp', '1=1');
    while (my ($grp_id) = $sth->fetchrow()) {
        $groups->{BY_GRP_ID}{$grp_id} ||= {};
        push @results, $grp_id;
    }
    $sth->finish();
    @results = sort {_get_grp_ids_sort_by_name($a, $b)} @results;

    return \@results;
}

sub _get_grp_ids_sort_by_name {

    my $groups = __PACKAGE__->get_groups_hash();

    my $a = $groups->{BY_GRP_ID}{shift(@_)}{GROUP_NAME} ;
    my $b = $groups->{BY_GRP_ID}{shift(@_)}{GROUP_NAME} ;
    return ($a cmp $b) ;
}


sub www_show_groups {
    my $self = shift;
    $self->debug_msg(3, "Group::www_show_groups()");

    my $rs = '';

    $rs .= "<table border=\"0\"><tr><td valign=\"top\">";
    $rs .= $self->_show_groups_table;
    $rs .= $self->_show_groups_table(1, 'My Managed Groups');
    $rs .= "</td><td valign=\"top\">";
    $rs .= $self->show_add_group;
    $rs .= "</td></tr>";
    $rs .= "</table>";
    if ( Group->new()->is_allowed_to_do('show_all_groups')) {
        $rs .= "<a href=\"".$self->make_url("index.pl?isa=Group&amp;op=show_all_groups&amp;iid=0")."\">See All Groups</a>";
    }

    my ($filter_property, $filter_value) = $self->get_filter_settings;
    my $title;
    if ($filter_value) {
        $title = "Manage $filter_value Groups";
    } else {
        $title = "Manage Groups";
    }
    $self->_output_form($rs, $title);
}

# defines the fields used in www_show_all_groups
sub show_groups_fields {
    my $self = shift;
    return $self->get_implementation()->show_groups_fields($self, @_);
}

sub www_show_all_groups {
    my $self = shift;
    my $show_emails = 0;

    if ( $PARAMS->{trusted_site} || $ACCESS_BROKER->is_allowed_to_do('admin_tasks') ) {
        $show_emails = 1;
    }

    my ($filter_property, $filter_value) = $self->get_filter_settings();

    my @fields = $self->show_groups_fields();

    my $sort_field = $FORM{'field'} ? $FORM{'field'} : 'group_name';
    my $order = $FORM{'order'} ? $FORM{'order'} : 'asc';

    # a reference to an array of hashes
    my $groups_by_owner = $USER->get_groups_data_by_owner( $sort_field, $order );

    my $images = $PARAMS->{'imageslocation'};

    my $url_string_prefix = "index.pl?"
      ."isa=Group&amp;op=show_all_groups&amp;field=";
    my $rs = "<table>";
    $rs .= "<tr bgcolor=\"#bbbbbb\">";

    foreach my $field (@fields) {
        my $field_name = $field->{field_name};
        next if ($field_name eq 'email' && !$show_emails);
         $rs .= "<th><b>";

        my $myurl .= "$url_string_prefix" . "$field_name&";
        $myurl .= "field=$field_name&";

        if (($field eq $field_name) && ($order eq 'asc')) {
            $myurl .= "order=desc";
        } else {
            $myurl .= "order=asc";
        }
        $myurl = $self->make_url($myurl);

        $rs .= "<a href=\"$myurl\">$field->{label}</a>&nbsp;";
        if (($field eq $field_name) && ($order eq 'asc')) {   #### FIX IMAGE ################
            $rs .= "<img class=\"icon\" src=\"$images/obj/triangle_up.gif\" border=\"0\" "
              ."alt=\"Sorted Ascending\">";
        } elsif ($field eq $field_name) {
            $rs .= "<img class=\"icon\" src=\"$images/obj/triangle_down.gif\" border=\"0\" "
              ."alt=\"Sorted Descending\">";
        }
        $rs .="</b></th>";
    }

    my $count = 0;

    foreach my $row ( @$groups_by_owner ) {
        # do not display hidden groups
        next if( $self->is_hidden( $row->{'grp_id'} ) );

        my $rownum = ($count++ % 2);
        my $color = $rownum ? "#eeeeee" : "#ffffff";

        $rs .= "<tr bgcolor=\"$color\">";
        foreach my $field (@fields) {
            my $field_name = $field->{field_name};
            if ($field_name eq 'group_name') {
                if ( $ACCESS_BROKER->is_allowed_to_do('admin_tasks') ) {
                    my $grp_id = $row->{'grp_id'};
                    unless ( $row->{'group_name'} eq "SITE_MGR" ||
                             $row->{'group_name'} eq "ADMIN" ||
                             $row->{'group_name'} eq Metadot::User::Default::REGISTRATION_MGR_GROUP_NAME()) {
                    $rs .= "<td><a href=\""
                        .$self->make_url("index.pl?isa=Group&amp;op=edit_groups&amp;grp_id=$grp_id")."\" class='icon'><img alt='Edit Me!' border=\"0\" src=\"$images/obj/edit.gif\" alt=\"Edit Me!\"></a>&nbsp;<a href=\""
                        .$self->make_url("index.pl?isa=Group&amp;op=delete_group&amp;grp_id=$grp_id")
                        ."\" class='icon'><img border=\"0\" src=\"$images/obj/del.gif\" alt=\"Delete Me!\" class='icon'></a>&nbsp;" . "$row->{'group_name'}&nbsp;</td>";
                    } else {
                        $rs .= "<td>$row->{'group_name'}&nbsp;</td>";
                    }
                } else {
                    $rs .= "<td>$row->{'group_name'}&nbsp;</td>";
                }
            } elsif ($field_name eq 'email') {
                if ($show_emails) {
                    $rs .= "<td><a href=\"mailto:$row->{'email'}\">$row->{'email'}</a></td>";
                } else {
                    next;
                }
            } else {
                $rs .= "<td>$row->{$field_name}&nbsp;</td>";
            }
        }
        $rs .= "</tr>";
    }

    $rs .= "<tr>"
      ."<td><img src=\"$images/pixel.gif\" width=\"100\" height=\"1\"></td>"
    ."<td><img src=\"$images/pixel.gif\" width=\"200\" height=\"1\"></td>"
      ."<td><img src=\"$images/pixel.gif\" width=\"100\" height=\"1\"></td>"
        ."<td><img src=\"$images/pixel.gif\" width=\"100\" height=\"1\"></td>"
          ."<td><img src=\"$images/pixel.gif\" width=\"100\" height=\"1\"></td>"
        ."</tr>";

    $rs .= "</table>";

    $rs .= "<a href=\"".$self->make_url("index.pl?isa=Group&amp;op=show_groups")."\">"
      ."Manage My Groups</a>";

    $self->_output_form($rs, 'All Groups');
}

sub _output_form {
    my $self = shift;
    my $content = shift;
    my $title = shift || $FORM{title} || '';

    #$self->print_http_header;
    # print $self->show_top($FORM{iid});
    # print $self->show_title($title);
    # print $content;
    # print $self->show_bottom;
    $DISPLAY->print_form($content);
}

sub _show_groups_table {
    my $self = shift;
    my $show_managed_groups = shift || 0;
    my $title = shift || 'My Groups';
    $self->debug_msg(3, "Group::show_group_table()");

    my $images = $PARAMS->{'imageslocation'};

    my %names = ('group_name'        => 'Name',
                 'group_membership'  => 'Member(s)',
                 'group_description' => 'Description');

    my @groups = $show_managed_groups
                 ? $self->groups_managed_by_user($USER->{'uid'})
                 : $self->groups_owned_by_user($USER->{'uid'});

    # filter the list of groups if requested
    my ($filter_property, $filter_value) = $self->get_filter_settings();
    if ($filter_property) {
        my @filtered = ();
        #print STDERR "Group filter by '$filter_property' = '$filter_value'\n";
        foreach my $group (@groups) {
            #print STDERR "Group has value '",$group->get_value($filter_property),"\n",Dumper $group;
            if ($group->get_value($filter_property) eq $filter_value) {
                push @filtered, $group;
            }
        }
        @groups = @filtered;
    }
    return unless (@groups);

    my $field = $FORM{'field'} ? $FORM{'field'} : 'group_name';
    my $order = $FORM{'order'} ? $FORM{'order'} : 'asc';

    my $num_rows = scalar(@groups);

    my $rs = "<h3>$title ($num_rows)</h3>";

    $rs .= "<table border=\"0\">";
    $rs .= "<tr bgcolor=\"#bbbbbb\"><th>&nbsp;</th>";

    foreach my $field_name ('group_name', 'group_membership', 'group_description') {
            $rs .= "<th><b>";


            my $myurl = "index.pl?isa=Group&amp;op=show_groups&";
            $myurl .= "field=$field_name&";

            if (($field eq $field_name) && ($order eq 'asc')) {
                $myurl .= "order=desc";
            } else {
                $myurl .= "order=asc";
            }

            $myurl = $self->make_url($myurl);

            $rs .= "<a href=\"$myurl\">$names{$field_name}</a>&nbsp;";
            if (($field eq $field_name) && ($order eq 'asc')) {   #### FIX IMAGE #######
                $rs .= "<img src=\"$images/obj/triangle_up.gif\" border=\"0\" "
                                ."alt=\"Sorted Ascending\">";
            } elsif ($field eq $field_name) {
                    $rs .= "<img src=\"$images/obj/triangle_down.gif\" border=\"0\" "
                                ."alt=\"Sorted Descending\">";
            }
              $rs.="</b></th>";
    }

    $rs .= "</tr>";

    my $count = 0;


    if ($field eq 'group_membership') {
            @groups = $self->_sort_by_numeric_hash_element($field, @groups);
    } else {
            @groups = $self->_sort_by_hash_element($field, @groups);
    }

    if ($order eq 'desc') {
            @groups = reverse @groups;
    }

    foreach my $group (@groups) {
            my $rownum = ($count++ % 2);
            my $color = $rownum ? "#eeeeee" : "#ffffff";
            my $grp_id = $group->{'grp_id'};

            $rs .= "<tr bgcolor=\"$color\">";

            if ( $group->{'group_name'} eq "SITE_MGR" ||
                 $group->{'group_name'} eq "ADMIN" ||
                 $group->{'group_name'} eq Metadot::User::Default::REGISTRATION_MGR_GROUP_NAME() ||
                 $show_managed_groups ) {
                $rs .= "<td>&nbsp;</td>";
            } else {
                $rs .= "<td><a href=\"".$self->make_url("index.pl?isa=Group&amp;op=edit_groups&amp;grp_id=$grp_id")."\" class='icon'><img alt='Edit Me!' border=\"0\" src=\"$images/obj/edit.gif\" alt=\"Edit Me!\"></a>&nbsp;<a href=\"".$self->make_url("index.pl?isa=Group&amp;op=delete_group&amp;grp_id=$grp_id")."\"><img border=\"0\" src=\"$images/obj/del.gif\" alt=\"Delete Me!\"></a></td>";
            }

            $rs .= "<td>$group->{'group_name'}&nbsp;</td>"
                        ."<td align=\"center\">"
                        ."<a href=\"".$self->make_url("index.pl?isa=Group&amp;op=show_group_members&amp;grp_id=$grp_id")."\" class='icon'>$group->{'group_membership'}</a>"
                        ."</td>"
                        ."<td>" . ($group->{'group_description'} || "") . "&nbsp;</td>"
                        ."</tr>"
    }

    $rs .= "<tr>"
                ."<td>&nbsp;</td>"
                ."<td><img src=\"$images/pixel.gif\" width=\"30\" height=\"1\"></td>"
                ."<td><img src=\"$images/pixel.gif\" width=\"10\" height=\"1\"></td>"
                ."<td><img src=\"$images/pixel.gif\" width=\"200\" height=\"1\"></td>"
                ."</tr>";

    $rs .= "</table>";

    return $rs;
}

sub show_group_search_form {
    my $self = shift;
    return $self->get_implementation()->show_group_search_form($self, @_);
}

# aju
sub _show_hidden_fields {
    my ($self, %params) = @_;
    my $out;

#      my %hidden = %{ $self->_get_standard_form_vars( %params ) };
    for my $var (keys(%params)) {
    $out .= "<input type=\"hidden\" name=\"$var\" value=\"$params{$var}\">\n";
    }
    return $out;
}



sub www_show_group_members {
    my $self = shift;
    $self->debug_msg(3, "Group::www_show_group_members()");
    my $grp_id = $self->{grp_id};
    my $group_name = $self->{group_name};
    my $rs = '';
    my $formupdate = $FORM{'update'} || $FORM{'remove_groups'} || "";
    my $formadd    = $FORM{'add'}    || "";
    my $owner_uid = $USER->{uid};

#     print STDERR "in www_show_group_members, with group name $group_name\n";
    if ($group_name eq 'ADMIN') {
        my $ok = $ACCESS_BROKER->is_allowed_to_do('edit_admin_group');
        if (!$ok) {
            $DISPLAY->print_click_back("You do not have permissions to edit this group.");
            exit;
        }
    }

    $self->set_group_permission_variables();

    unless (ref($USER))     {
    warn("USER is not set up");
    &printError("USER is not set up");
    exit (0);
    }
    unless ($IS_OWNER || $IS_ADMIN || $IS_MANAGER) {
    warn("UNAUTHORIZED ACCESS attempt to group id " . $grp_id .
         " by " . $USER->email);
    $DISPLAY->print_click_back ("You are not the owner or manager of this group!");
    exit (0);
    }
    if ($formupdate eq 'Update') {## remove users or add/remove managers
        if ($IS_OWNER || $IS_ADMIN || $IS_MANAGER) {
            $self->remove_all_managers() if ($IS_OWNER || $IS_ADMIN);
        foreach my $key (keys %FORM) {
        if($key =~ /^remove_uid(.*)/) {
            my $uid = $1;
            $self->remove_member($uid);

            # delete orphaned users if specified in params,
            # UNLESS the user is the current one!
            if ($uid && $PARAMS->{delete_orphaned_users} && ($uid != $USER->uid())) {
                #print STDERR "checking for orphaned users\n";
                my $u = Metadot::UserFactory->get_user($uid);
                if ($u->is_orphan()) {
                    #print STDERR "deleting an orphaned user\n";
                    $u->delete();
                }
            }
        } elsif($key =~ /^manager_uid(.*)/ && ($IS_OWNER || $IS_ADMIN) ) {
            $self->make_manager($1);
        }
        }
    } else {
        $DISPLAY->print_click_back ("You are not the owner or manager of the group to which you wish to remove users.");
        exit (0);
    }
    }
    elsif ($formupdate eq 'Remove Groups') {
    #print STDERR "got instruction to remove groups\n";
        foreach my $key (keys %FORM) {
        if($key =~ /^remove_grp(.*)/) {
            $self->remove_group($1);
        } }
    }
    elsif ($formadd eq 'Add') {
        $self->add_remove_group_members($grp_id);
    }
    elsif ($formadd eq 'add_subgroup') {
    #print STDERR "in www_show_group_members, add-subgroup case\n";
        my ($status, $errmsg) = $self->add_group_subgroup($grp_id);
        if (!$status) {
        $self->print_click_back ($errmsg);
        exit (0);
        }
    }

    $rs .= "<table border=\"0\" width=\"100%\"><tr><td valign=\"top\">";
    $rs .= $self->_show_group_members_table();
    $rs .= "<br /><br /><a href=\"".$self->make_url("index.pl?isa=Group&amp;op=show_groups")."\">"
                    ."Manage My Groups</a>";

    if ( Group->new()->is_allowed_to_do('show_all_groups')) {
        $rs .= "<br /><a href=\"".$self->make_url("index.pl?isa=Group&amp;op=show_all_groups")."\">"
                        ."See All Groups</a>";
    }

    $rs .= "</td>&nbsp;<td valign=\"top\">";
    if ($ACCESS_BROKER->is_allowed_to_do('register_users')) {
        $rs .= $self->_show_register_new_group_member().'<br />';
    }
    if ($self->is_allowed_to_do('search_add_group_members')) {
        $rs .= $self->show_group_search_form();
        $rs .= $self->_show_add_group_members();
    }

    if ($FORM{'search'} eq 'Search') {

    $rs .= <<EOM
    <form method=post action = "index.pl">
    <input type=hidden name = "isa" value = "$FORM{isa}">
    <input type=hidden name = "iid" value = "$FORM{iid}">
    <input type=hidden name = "grp_id" value = "$grp_id">
    <input type=hidden name = "op" value = "show_group_members">
EOM
;

    $rs .= $USER->get_search_user_results({
                           EMAIL => "$FORM{email}",
                           FULLNAME => "$FORM{fullname}",
                           NICKNAME => "$FORM{nickname}",
                          });

    $rs .= "<input type=submit value ='Add' name = 'add'></form>";

    }

    $rs .= "</td></tr></table>";

    $self->_output_form($rs, 'Manage Group Members');
}

sub remove_member {
    my $self = shift;
    my $uid = shift || die("Must specify uid of user to remove\n");
    my $grp_id = shift || $self->{grp_id};

    $grp_id = undef unless Utility->is_valid_integer($grp_id); # secu_200401

    ## lit bit of extra security
    my $extra_where = $IS_OWNER || $IS_ADMIN ? "" : "and is_manager != 1";

    DBAccess->sqlDelete("grpmembers", "userid=$uid and grp_id=$grp_id $extra_where");
    if (($IS_OWNER || $IS_ADMIN) && $PARAMS->{allow_organization_registration}) {
        # this is not used?
        #my $config = $USER->get_fields_config();
        my $user_obj = Metadot::UserFactory->get_user($uid);
        if ($user_obj->can('remove_user_primary_organization_choice')) {
            $user_obj->remove_user_primary_organization_choice($grp_id);
        }
    }

    $CACHE->set_as_dirty('GROUPS_HASH');
}

sub remove_all_managers {
    my $self = shift;
    my $grp_id = shift || $self->{grp_id};

    $grp_id = undef unless Utility->is_valid_integer($grp_id); # secu_200401

    DBAccess->sqlUpdate("grpmembers", "grp_id=$grp_id", is_manager => 0);
    $CACHE->set_as_dirty('GROUPS_HASH');
}

sub make_manager {
    my $self = shift;
    my $uid = shift || die("Must specify uid of user to remove\n");
    my $grp_id = shift || $self->{grp_id};

    $grp_id = undef unless Utility->is_valid_integer($grp_id); # secu_200401

    DBAccess->sqlUpdate("grpmembers", "userid=$uid and grp_id=$grp_id", is_manager => 1);
    $CACHE->set_as_dirty('GROUPS_HASH');
}

#aju
sub remove_group {
    my $self = shift;
    my $subgrp = shift;
    my $grp_id = shift || $self->{grp_id};

    $grp_id = undef unless Utility->is_valid_integer($grp_id); # secu_200401
    $subgrp = undef unless Utility->is_valid_integer($subgrp); # secu_200401
    defined($subgrp) || die("Must specify id of subgroup to remove\n");

    DBAccess->sqlDelete("grpmembers", "subgrp_id=$subgrp and grp_id=$grp_id");
    $CACHE->set_as_dirty('GROUPS_HASH');
}

sub _show_group_members_table {
    my $self = shift;

    my $rs = '';
    $rs .= $self->_show_view_groups_page();

    return $rs;
}

sub _show_add_group_members {
    my $self   = shift;
    my $grp_id = $self->{grp_id};
    my $header = "Search and Add Members";

    my $rs = '';


     $rs = Components->get_search_users({HEADER => $header,
                    ACTION => "index.pl",
                    PARAMS => {grp_id => $grp_id, op => 'show_group_members', isa => 'Group', iid => "$FORM{iid}"},
                    WIDTH => "100%",
                    });
    return $rs;
}

sub _show_register_new_group_member {
    my $self   = shift;
    my $grp_id = $self->{grp_id};
    my $header = "Register a new user for this group";

    my $rs = '';


     $rs = Components->get_register_new_group_member({HEADER => $header,
                    ACTION => "index.pl",
                    PARAMS => { enroll_groups   => $grp_id,
                                admin_mode      => 1,
                                op              => 'register_form',
                                isa             => 'Register',
                                refererlink     => 'index.pl?isa=Group&amp;op=show_group_members&amp;grp_id='.$grp_id,
                               },
                    WIDTH => "100%",
                    });
    return $rs;
}

sub www_verify_change_group_information {
    my $self = shift;
    my $grp_id = $self->{grp_id};

    my $grp_name = $self->get_group_name_by_grp_id( $grp_id );

    if( uc $grp_name eq 'SITE_MGR' || uc $grp_name eq 'ADMIN' || $grp_name eq Metadot::User::Default::REGISTRATION_MGR_GROUP_NAME()) {
        $DISPLAY->print_click_back( "Special group $grp_name cannot be edited" );
        exit;
    }


    my %properties = $self->input_group_edit_form();
    my $group_name = $properties{group_name};

    my ($old_group_name) = DBAccess->sqlSelect('group_name', 'grp', "grp_id = $grp_id");

    my $group_update_error_message;
    if ($group_name =~ /[^\w\s\.\,]/)   {
        # if the group contains something other than numbers, letters, or spaces
        $group_update_error_message = "The group name may only contain numbers, letters, and/or spaces";
    } elsif (!$group_name){
        $group_update_error_message = "You must put in a group name"
    } elsif (($group_name ne $old_group_name) && (DBAccess->sqlSelect ("group_name, grp_id", "grp", "group_name = '$group_name'", ""))) {
        $group_update_error_message = "The group $group_name already exists";
    }

    if ($group_update_error_message) {
        $DISPLAY->print_click_back ("Sorry", "The group could not be changed for the following reason:  $group_update_error_message");
    } else {
        $self->update(%properties);
        my $referer = $FORM{referer} || $SESSION->{referer};
        if ($referer) {
            Utility->md_redirect($referer);
        } else {
            $self->www_show_groups;
        }
    }
}


sub www_verify_add_group {
    my $self = shift;

    my %properties = $self->input_group_edit_form();

    # the current user becomes the owner
    $properties{uid} ||= $USER->{uid};

    my $group_name = $properties{group_name};
    my $group_add_error_message;

    $self->debug_msg(3, "Group::www_verify_add_group()");
    if ($group_name =~ /[^\w\s\.\,]/) { # if the group contains something other than numbers, letters, or spaces
        $group_add_error_message = "The group name may only contain numbers, letters, and/or spaces";
    } elsif (!$group_name){
        $group_add_error_message = "You must put in a group name"
    }  else {
        # [vadim] group security bug fix starts here
        my ($count) = DBAccess->sqlSelect("count(*)", "grp", "lower(group_name)=lower('".$group_name."')") ;
        if ($count > 0) { #if the group already exists
            $group_add_error_message = "The group $group_name already exists";
        }
        # [vadim] group security bug fix ends here
    }

    if ($group_add_error_message) {
        $DISPLAY->print_click_back ("Sorry", "The group could not be added for the following reason: $group_add_error_message");
    } else {
        $self->add_group( %properties );
        $self->www_show_groups;
    }
}

sub input_group_edit_form {
    my $self = shift;
    my %properties;

    $properties{group_description}  = $CGI->escapeHTML($FORM{'group_description'});
    $properties{group_name}         = $FORM{group_name};
    if ($PARAMS->{allow_organization_registration}) {
        $properties{is_organization}    = $FORM{is_organization};
        if ($properties{is_organization}) {
            # removed the escapeHTML call here
            $properties{organization_homepage}          = $FORM{organization_homepage};
            $properties{organization_post_registration} = $FORM{organization_post_registration};

        } else {
            $properties{organization_homepage} = '';
            $properties{organization_post_registration} = '';
        }
    }

    if ($self->group_welcome_letters_allowed()) {
        $properties{welcome_letter_heading} = Utility->trim($FORM{welcome_letter_heading});
        $properties{welcome_letter_body}    = Utility->trim($FORM{welcome_letter_body});
    }

    %properties = $self->get_implementation()->input_custom_form_fields($self, %properties);

    return %properties;
}

sub www_edit_groups {
  my $self = shift;
  #my $grp_id = shift || $FORM{grp_id};  #if we are passed a group from another function, we use that.  Otherwise, we use the group obtained from the form
  $self->debug_msg(3, "Group::www_edit_group()");
  $self->_output_form($self->show_edit_groups_page(), 'Edit Groups');
}

sub add_group {
    my ($self, %properties) = @_;

    my $group = $self->new();

    # set the properties in the group object, and retrieve
    # any that were modified
    %properties = $group->set_properties(%properties);

    # prepare the properties for insertion to the database
    %properties = $self->translate_properties_to_db_fields(%properties);

    ## update object database info
    $group->{grp_id} = DBAccess->sqlInsert("grp", %properties) ;
    $CACHE->set_as_dirty('GROUPS_HASH');
    return $group;
}


###################################################
#########The back end functions to display the
#########group manager form
###################################################


###################################################
sub show_add_group {
    my $self = shift;
    my $impl = $self->get_implementation();
    return $impl->show_add_group($self, @_);
}



##################################################
##################################################
#############################################
## This actually shows group members table
# aju
sub _show_view_groups_page {
    my $self   = shift;
    my $impl = $self->get_implementation();
    return $impl->_show_view_groups_page($self,@_);
}

sub _build_subgroups_table {
    my ($self, $groups_in_group, $group_name, $grp_id,
    $gfield, $order, $aref) = @_;
    my %args = %{$aref};
    my $count = 0;

    my $rs = '';
    my %names = ('subgrp_id'    => 'Subgroup ID',
         'group_name' => 'Subgroup Name',
         'fullname' => 'Subgroup Owner',
         'owner' => 'Owner UID');

    return '' unless ($groups_in_group);

    foreach my $grp ( @$groups_in_group ) {
    my ($subgrp_id, $group_name, $owner, $fullname) = ( $grp->{subgrp_id},
                                $grp->{group_name},
                                $grp->{owner}, $grp->{fullname});
    my $rownum = ($count++ % 2);
    my $color = $rownum ? "#eeeeee" : "#ffffff";

    $rs .= "<tr bgcolor=\"$color\">
        <td align='center'>";
    if ((lc $group_name) eq 'admin') {
        $rs .= "&nbsp;"
        } else {
        $rs .="<input type=checkbox name=remove_grp$subgrp_id>"
        }
    $rs .=" </td>";
    $rs .=" <td>$subgrp_id\n</td>" .
         "<td><a href=\"index.pl?isa=Group&amp;op=edit_groups&amp;grp_id=$subgrp_id\">$group_name</a>\n</td>\n" .
         "<td>$fullname\n</td>\n" .
         "<td>$owner\n</td>\n</tr>";
    }
    my $title = $args{title} || "Current Groups in $group_name";
    my $return_link = $args{return_link} || "<p /><a href=\"".$self->make_url("index.pl?isa=Group&amp;op=show_groups")."\">Manage My Groups</a>";

    if ( Group->new()->is_allowed_to_do('show_all_groups')) {
        $return_link .= "<br /><a href=\"".
        $self->make_url("index.pl?isa=Group&amp;op=show_all_groups") . "\">See All Groups</a>";
    }

    my $s = "\n<H2>$title</H2>\n" .
    "<form method=post action=index.pl>
    <input type=hidden name=remove value=remove_groups>
    <input type=hidden name=isa value=Group>
    <input type=hidden name=grp_id value='$grp_id'>
    <input type=hidden name=op value='show_group_members'>
    <input type=hidden name=iid value=$FORM{iid}>" .
        $self->make_hidden().
    "<table border=\"0\">
    <tr bgcolor=\"#bbbbbb\">
        <th>
                 Remove
        </th>";
    foreach my $field_name ('subgrp_id', 'group_name', 'fullname', 'owner') {
    $s .= "<th><b>";
    my $myurl = "index.pl?grp_id=$grp_id&amp;isa=Group&amp;op=show_group_members&amp;gfield=$field_name&amp;order=";
    if (($gfield eq $field_name) && ($order eq 'asc') ){
        $myurl .= "desc";
    } else {
        $myurl .= "asc";
    }
    $myurl = $self->make_url($myurl);
    $s .= "<a href=\"$myurl\">$names{$field_name}</a></b></th>";
    }
    $s .= "</tr>";

    my $images = $PARAMS->{'imageslocation'};
    $s .= $rs;
    $s .= "<tr>";
    $s .= "<td>&nbsp;</td>";
    $s .= "<td><img src=\"$images/pixel.gif\" width=\"30\" height=\"1\"></td>\n";
    $s .= "<td><img src=\"$images/pixel.gif\" width=\"30\" height=\"1\"></td>\n";
    $s .= "<td><img src=\"$images/pixel.gif\" width=\"30\" height=\"1\"></td>\n";
    $s .= "<td><img src=\"$images/pixel.gif\" width=\"30\" height=\"1\"></td>\n";
    $s .= "</tr>";

    $s .= "</table><input type=submit name=\"remove_groups\" value=\"Remove Groups\"></form>";
    $s .= $return_link;
}

sub _build_user_table {
    my ($self, $users_in_group, $group_name, $grp_id,
    $show_emails, $field, $order, $aref) = @_;

    my %args = %{$aref};
    my $rs = '';
    my $i;
    my $images = $PARAMS->{'imageslocation'};
    my %names = (
         'email'    => 'E-mail',
         'fullname' => 'Full Name',
         'nickname' => 'Nickname / Screenname',
         'is_manager'  => 'Manager',
         );
    my $number_of_users = 0;
    if ($users_in_group){
    $number_of_users = scalar(@$users_in_group);
    }
    my $count = 0;
    foreach my $usr ( @$users_in_group ) {

    my ($email, $fullname, $nickname, $uid, $is_user_manager) = (
        $usr->{email},
        $usr->{fullname},
        $usr->{nickname},
        $usr->{userid},
        $usr->{is_manager}
                                                  );
  my $checkbox_value = $is_user_manager ? "checked" : "";
    my $rownum = ($count++ % 2);
    my $color = $rownum ? "#eeeeee" : "#ffffff";

    $rs .= "<tr bgcolor=\"$color\">
        <td align='center'>";
    if ($uid == '8') {
        $rs .= "&nbsp;"
    } else {
      if ($IS_OWNER || $IS_ADMIN || ($IS_MANAGER && !$is_user_manager)) {
          $rs .="<input type=checkbox name=remove_uid$uid>"
        } else {
          $rs .= "&nbsp;";
      }
    }
    $rs .=" </td>";

    if ($show_emails) {
        $rs .= "<td>
            $email&nbsp;
        </td>";
    }
    $rs .="     <td>";
    my $full_name;
    if ($is_user_manager) {
        $full_name = "<b>$fullname&nbsp</b>";
    } else {
        $full_name = "$fullname&nbsp";
    }
    if ($ACCESS_BROKER->is_allowed_to_do('show_details')) {
        $full_name = "<a href=\"index.pl?isa=User&amp;op=show_details&amp;uid=$uid\">$fullname</a>";
    }
    if ($USER->is_admin()) {
        my $user_obj = Metadot::UserFactory->get_user($uid);
        $full_name = $user_obj->show_panel().$full_name;
    }

    $rs .= $full_name;
        $rs .="
        </td>
        <td>
            $nickname&nbsp;
        </td>
        <td align='center'>\n";
    if ($uid == '8') {
      $rs .= "&nbsp;";
  } else {
      if ($IS_OWNER || $IS_ADMIN) {
          $rs .="<input type=checkbox name=manager_uid$uid $checkbox_value>"
        } else {
            $rs .= $is_user_manager ? "Yes" : "No";
      }
  }

    $rs .=" </td>
    </tr>";
    }

    my $title = $args{title} || "Managing Group Members for $group_name";
    my $return_link = ""; #put this in after the subgroups table instead

    my $s = "<H2>$title<br /><font size=\"-1\"><i>Currently $number_of_users member(s).</i></font></H2>
    <form method=post action=index.pl>
    <input type=hidden name=isa value=Group>
    <input type=hidden name=grp_id value='$grp_id'>
    <input type=hidden name=op value='show_group_members'>
    <input type=hidden name=iid value=$FORM{iid}>
    ".$self->make_hidden()."
<table border=\"0\">
    <tr bgcolor=\"#bbbbbb\">
        <th>
                        Remove
        </th>";

    ## name as it appears in database
    foreach my $field_name ('email', 'fullname', 'nickname', 'is_manager') {
        next if ($field_name eq  'email' && !$show_emails);
        $s .= "<th><b>";
        my $myurl = "index.pl?grp_id=$grp_id&amp;isa=Group&amp;op=show_group_members&amp;field=$field_name&amp;order=";
        if (($field eq $field_name) && ($order eq 'asc') ){
            $myurl .= "desc";
        } else {
            $myurl .= "asc";
        }
        $myurl = $self->make_url($myurl);
        $s .= "<a href=\"$myurl\">$names{$field_name}</a></b></th>";
    }
    $s .= "</tr>";

    $s .= $rs;
    $s .= "<tr>";
    $s .= "<td>&nbsp;</td>";
    $s .= "<td><img src=\"$images/pixel.gif\" width=\"30\" height=\"1\"></td>" if $show_emails;
    $s .= "<td><img src=\"$images/pixel.gif\" width=\"30\" height=\"1\"></td>";
    $s .= "<td><img src=\"$images/pixel.gif\" width=\"30\" height=\"1\"></td>";
    $s .= "</tr>";

    $s .= "</table><input type=submit name=\"update\" value=\"Update\"></form>";
    $s .= $return_link;
    return $s;
}


###################################################
sub add_remove_group_members {
    my $self = shift;
    my $grp_id = $self->{grp_id};
    foreach my $key (keys %FORM) {
            if ($key =~ /^user(.*)/) {
            # if this value is indeed checked, we also know that $key at this point
            # is of the form 'old' ####
            my $uid = $1;
            if (!DBAccess->sqlSelect ("grp_id", "grpmembers", "grp_id = $grp_id and userid = '$uid'")) {
                $self->_add_a_member_to_a_group($grp_id, $uid);
                $self->_send_group_welcome_letter(Metadot::UserFactory->get_user($uid));
            }
      }
    }
}

sub add_group_subgroup {
    my $self = shift;
    #print STDERR "in add_group_subgroup\n";
    my $grp_id = $self->{grp_id};
    # get subgroup info
    my $subgroup_id = $FORM{add_group};
    if (!defined($subgroup_id) || ($subgroup_id eq '')) {
    return (0, "Please select a group");
    }
    my ($status, $errmsg) =
    $self->_add_subgroup_to_group($grp_id, $subgroup_id);
    return ($status, $errmsg);
}

###################################################
sub _add_a_member_to_a_group {
    my ($self, $grp_id, $uid) = @_;

    $grp_id = undef unless Utility->is_valid_integer($grp_id); # secu_200401
    $uid = undef unless Utility->is_valid_integer($uid); # secu_200401

    unless (DBAccess->sqlSelect("userid", "grpmembers", "userid = $uid AND grp_id = $grp_id")) {
    DBAccess->sqlInsert("grpmembers",   #we add them into the group
                (grp_id => "$grp_id",
                 userid    => "$uid")
            );
     $CACHE->set_as_dirty('GROUPS_HASH');
    }
}

#aju
sub _add_subgroup_to_group {
    my ($self, $grp_id, $subgrp_id) = @_;

    $grp_id = undef unless Utility->is_valid_integer($grp_id); # secu_200401
    $subgrp_id = undef unless Utility->is_valid_integer($subgrp_id); # secu_200401

    #print STDERR "attempting to add $subgrp_id to $grp_id.\n";
    if ($grp_id eq $subgrp_id) {
    my $errmsg = "Can't add a group to itself\n";
    return (0, $errmsg);
    }
    if (DBAccess->sqlSelect("subgrp_id", "grpmembers", "subgrp_id = $subgrp_id AND grp_id = $grp_id")) {
    # group already added
    #print STDERR "$subgrp_id already subgroup of $grp_id\n";
    return 1;       #  operation okay
    }
    else {
    my $anc = $self->_ancestor_of($subgrp_id, $grp_id);
    if (!$anc) { # if the would-be sub group is not already an
        # ancestor group of the parent group
        #print STDERR "ancestor check okay -- doing insert\n";
        DBAccess->sqlInsert("grpmembers", #we add them into the group
                (grp_id => "$grp_id",
                 subgrp_id    => "$subgrp_id")
                );
        $CACHE->set_as_dirty('GROUPS_HASH');
        return 1;       #  operation okay
    }
    else {
        # group IS ancestor; not allowed to add
        # return 0 status, indicating operation not completed.
        #print STDERR "Group with id $subgrp_id is ancestor of group $grp_id because of group with id $anc -- can't add\n";
        my $errmsg = "Group with id $subgrp_id is ancestor of group $grp_id because of group with id $anc -- can't add\n";
        return (0, $errmsg);
    }
    }
}


#aju
sub _ancestor_of {
    my ($self, $group1_id, $group2_id) = @_;

    #if group1 is an immediate parent of group 2, return 1 (true)
    my @pgroups = $self->_parent_groups_of($group2_id);
    if (Utility->member($group1_id, @pgroups)) {
    #print STDERR "$group1_id is an ancestor of $group2_id\n";
    return $group1_id;
    }
    else {
    #otherwise, return 1 if group1 is an ancestor of any parent of group 2
    foreach my $parent_id (@pgroups) {
        #print STDERR "checking that $group1_id is an ancestor of $parent_id\n";
        if ($self->_ancestor_of($group1_id, $parent_id)) {
        #print STDERR "$group1_id is an ancestor of $group2_id\n";
        return $parent_id;
        }
    }
    # if finished for loop w/out returning, then answer is false
    return 0;
    }
}

#aju
sub _parent_groups_of {
    my ($self, $subgrp_id) = @_;

    $subgrp_id = undef unless Utility->is_valid_integer($subgrp_id); # secu_200401

    my $rows =
    DBAccess->sqlSelectMany("grp_id", "grpmembers", "subgrp_id = $subgrp_id");
#      print STDERR "rows: ", Dumper($rows), "\n";
    my @result;
    while (my $row = $rows->fetchrow_hashref() ) {
    push @result, $row->{grp_id};
    }
#      print STDERR "parent groups of $subgrp_id are: " , Dumper(\@result), "\n";
    return @result;
}

#aju
sub _get_subgroups_in_group {
    my ($self, $grp_id, $gfield, $order) = @_;

    #print STDERR "in _get_subgroups_in_group\n";
    my $rows = DBAccess->sqlSelectMany('grpmembers.subgrp_id subgrp_id, group_name, grp.userid owner, fullname',
               'grp, grpmembers, users',
               "grpmembers.grp_id = $grp_id and grpmembers.subgrp_id = grp.grp_id and grp.userid = users.userid",
               "ORDER BY $gfield $order ");
    my $result;
    while (my $row = $rows->fetchrow_hashref() ) {
    #print STDERR "in _get_subgroups_in_group, with row: ", Dumper($row), "\n";
        my $group = $self->new();
        $group->set_properties(%$row);
    push @$result, $group;
    }
    return $result;
}


sub add_member {
    my $self = shift;
    my $uid = shift || die("Must provide a uid\n");

    my $grp_id = $self->{grp_id} || die("This group does not have an id\n");
    $self->_add_a_member_to_a_group($grp_id, $uid);
}

sub show_edit_groups_page {
    my $self = shift;
    my $grp_id = $self->{grp_id};
    my $group_name = $self->{group_name};

#     print STDERR "in show_edit_groups_page, with group name $group_name\n";
    if ($group_name eq 'ADMIN') {
    my $ok = $ACCESS_BROKER->is_allowed_to_do('edit_admin_group');
    if (!$ok) {
        $DISPLAY->print_click_back("You do not have permissions to edit this group.");
        exit;
    }
    }
    my $group_description = $self->{group_description};

    my $referer = $FORM{referer} || $SESSION->{referer};

    my $frame_manage_users = new HTML::TableFrame('#333333', '#efefef');
    $frame_manage_users->add("<a href=\"".$self->make_url("index.pl?isa=Group&amp;op=show_group_members&amp;grp_id=$grp_id")."\">Manage users for this group</a>");

    my $custom_fields = $self->get_implementation()->show_custom_form_fields($self);
# begin big quoted string section.
my $edit_group = "<form method=post action=index.pl><table border=\"0\">
        <tr>
                <td colspan='2'><h2>Edit Group</h2></td>
        </tr>
    <tr>
        <td valign='top'>
            <input type=hidden name=isa value=Group>
            <input type=hidden name = op value = verify_change_group_information>
            <input type=hidden name=referer value=\"$referer\">
            Name:
        </td>
        <td valign='top'>
            <input type=text size=25 value = '$group_name' name = 'group_name'>
            <input type=hidden value = '$grp_id' name = \"grp_id\">
        </td>
    </tr>
    <tr>
        <td valign='top'>
                    Description:
        </td>
        <td valign='top'>
            <textarea name=\"group_description\" rows=4 cols=25 wrap>$group_description</textarea>
        </td>
    </tr>\n";

    if ($PARAMS->{allow_organization_registration}) {
        $edit_group .= $self->generate_organization_input_fields();
    }

    if ($self->group_welcome_letters_allowed()) {
        $edit_group .= $self->generate_welcome_letter_input_fields();
    }



    $edit_group .= "
    $custom_fields
    <tr><td colspan='2' align='right'><input type=submit value = \"Save\"><br /><br /></td></tr>
        <tr>
                <td colspan='2' align='center'>" . $frame_manage_users->print() . "</td>
        </tr>

    ";

    $edit_group .= "</table></form>";

    my $frame_edit_group = new HTML::TableFrame("#333333", "#efefef");
    $frame_edit_group->add($edit_group);

    my $rs = "<br /><br />" . $frame_edit_group->print;

#    $rs .= $frame_manage_users->print;

    $rs .= "<br /><br /><a href=\"".$self->make_url("index.pl?isa=Group&amp;op=show_groups")."\">"
                    ."Manage My Groups</a>";

    if ( Group->new()->is_allowed_to_do('show_all_groups')) {
        $rs .= "<br /><a href=\"".$self->make_url("index.pl?isa=Group&amp;op=show_all_groups")."\">"
                        ."See All Groups</a>";
    }

    return $rs;
}


# generates the form input for organizations
sub generate_organization_input_fields {
    my $self = shift;

    my $is_organization;
    my $organization_homepage;
    my $organization_post_registration;

    # if called in an object context, we check existing values
    if (ref $self) {
        $is_organization = $self->{is_organization};
        $organization_homepage = $CGI->escapeHTML($self->{organization_homepage});
        $organization_post_registration = $CGI->escapeHTML($self->{organization_post_registration});
    } else {
        $is_organization = '';
        $organization_homepage = '';
        $organization_post_registration = '';
    }

    my $is_checked_yes = $is_organization ? 'checked' : '';
    my $is_checked_no  = $is_organization ? '' : 'checked';

    my $result = "
<tr>
    <td valign='top'>
      Is this group an Organization?
    </td>
    <td valign='top'>
      <input type=\"radio\" name=\"is_organization\" value=\"1\" $is_checked_yes>Yes
      <input type=\"radio\" name=\"is_organization\" value=\"0\" $is_checked_no>No
    </td>
</tr>
<tr>
    <td valign='top'>
      Organization first page after login:
    </td>
    <td valign='top'>
      <input type=\"text\" size=50 name=\"organization_homepage\" value=\"$organization_homepage\">
    </td>
</tr>
<tr>
    <td valign='top'>
      Organization first page after registration:
    </td>
    <td valign='top'>
      <input type=\"text\" size=50 name=\"organization_post_registration\" value=\"$organization_post_registration\">
    </td>
</tr>\n";


    return $result;
}


# generates the form input for welcome letters
sub generate_welcome_letter_input_fields {
    my $self = shift;

    my $heading_form_value;
    my $body_form_value;

    # if called in an object context, we check existing values
    if (ref $self) {
        my $welcome_letter_heading = $self->welcome_letter_heading();
        my $welcome_letter_body    = $self->welcome_letter_body();
        $heading_form_value     = $CGI->escapeHTML($welcome_letter_heading);
        $body_form_value        = $CGI->escapeHTML($welcome_letter_body);
    } else {
        $heading_form_value     = '';
        $body_form_value        = '';
    }

    my $broadcast_check = Portal->is_broadcast_email_enabled()
      ? ''
      : '<p><b>Note: This site has disabled broadcast of emails. Only server administrators, site managers, and registration managers will cause a welcome letter to be sent.</b></p>';

    my $content_type_notice = Sendmail->mail_content_type_is_html()
      ? 'The email will be sent as HTML. Please make sure that any links you create a valid HTML'
      : 'The email will be send as plain text. Please avoid using HTML.';

    my $result = <<EOM
    <tr><td colspan="2">&nbsp;</tr>
    <tr valign="top">
      <td>
        Welcome letter for this group:<br/><br/>
        $broadcast_check
        <p>
            $content_type_notice
        </p>
        <p>
            <i>
              If you would like to send users an email whenever they are added to
              this group, type a welcome letter. You can add dynamic text to the
              letter using the following special tags:<br/><br/>
              &lt;&lt;fullname&gt;&gt;<br/>
              &lt;&lt;email&gt;&gt;<br/>
              &lt;&lt;nickname&gt;&gt;<br/>
              &lt;&lt;group_name&gt;&gt;<br/>
              &lt;&lt;group_description&gt;&gt;<br/>
            </i>
          </p>
      </td>
      <td>
        Subject: <input type="text" size="70" name="welcome_letter_heading" value="$heading_form_value"/><br/><br/>
        <textarea name="welcome_letter_body" rows="15" cols="80">$body_form_value</textarea>
      </td>
    </tr>
EOM
;

    return $result;
}


sub get_filter_settings {
    my $class = shift;
    my $filter_property = $FORM{filter_property};
    my $filter_value;

    if (defined $filter_property) {
        $filter_value = $FORM{filter_value};
        if ($SESSION) {
            $SESSION->set_value('group_filter_property', $filter_property || '');
            $SESSION->set_value('group_filter_value', $filter_value || '');
        }
    } elsif ($SESSION) {
        $filter_property = $SESSION->get_value('group_filter_property');
        $filter_value = $SESSION->get_value('group_filter_value');
    }

    #print STDERR "Group->get_filter_settings returns '$filter_property', '$filter_value'\n";
    return ($filter_property, $filter_value);
}

sub www_delete_group {
    my $self = shift;

    my $grp_id = $self->{grp_id};
    my $uid = $USER->{'uid'};

    if ($FORM{'verify'} eq 'yes') {
    if ( (DBAccess->sqlSelect("grp_id, group_name", "grp", "userid=$uid and grp_id='$grp_id'")) ||
         ( $ACCESS_BROKER->is_allowed_to_do('admin_tasks') ) ){

        $self->delete($grp_id);
        $self->www_show_groups;
    } else {

        $DISPLAY->print_click_back("Sorry", "You are not the owner of this group");
    }
    }
    else {
    my ($group) = $self->{group_name};
    $self->two_choices("Do you really want to delete $group?",
                      "caution",
              "Yes",
              $self->make_url("index.pl?isa=Group&op=delete_group&verify=yes&grp_id=$grp_id"),
              "No",
              $self->make_url("index.pl?isa=Group&op=show_groups"));
    }
}

sub delete {
    my $self = shift;
    my $grp_id = shift || $self->{grp_id};
    $grp_id = undef unless Utility->is_valid_integer($grp_id); # secu_200401
    defined($grp_id) || die("Must supply a grp_id\n");

    my $grp_name = $self->get_group_name_by_grp_id( $grp_id );

    if( uc $grp_name eq 'SITE_MGR' || uc $grp_name eq 'ADMIN' || $grp_name eq Metadot::User::Default::REGISTRATION_MGR_GROUP_NAME()) {

    $DISPLAY->print_click_back( "Special group $grp_name is not deletable" );
    exit;
    }

    DBAccess->sqlDelete("grpmembers", "grp_id='$grp_id'");
    #aju - delete subgroup entries also
    DBAccess->sqlDelete("grpmembers", "subgrp_id='$grp_id'");
    DBAccess->sqlDelete("grp", "grp_id=$grp_id");
    $CACHE->set_as_dirty('GROUPS_HASH');
}

sub groups_owned_by_user {
    my $self = shift;
    my $uid = shift || $USER->uid();

    $uid = undef unless Utility->is_valid_integer($uid); # secu_200401
    return unless defined $uid;

    my $grp;
    my @groups_list;
    my $where = "userid = $uid";
    my $sth = DBAccess->sqlSelectMany("*", "grp", $where, "");
    while (my $rows = $sth->fetchrow_hashref()) {
            my %hash = %$rows;

            my ($count) = DBAccess->sqlSelect("count(*)","grpmembers","grp_id = '" . $rows->{'grp_id'} . "'");
            $hash{'group_membership'} = $count;
            my $grp = $self->new();
            $grp->set_properties(%hash);
            push @groups_list, $grp;
    }
    return @groups_list;
}

sub groups_managed_by_user {
    my $self = shift;
    my $uid = shift || $USER->uid();

    $uid = undef unless Utility->is_valid_integer($uid); # secu_200401
    return unless defined $uid;

    my $grp;
    my @groups_list;
    my $where = "gm.userid = $uid and is_manager = 1 and g.grp_id = gm.grp_id";
    my $sth = DBAccess->sqlSelectMany("*", "grp g, grpmembers gm", $where, "");
    while (my $rows = $sth->fetchrow_hashref()) {
            my %hash = %$rows;

            my ($count) = DBAccess->sqlSelect("count(*)","grpmembers","grp_id = '" . $rows->{'grp_id'} . "'");
            $hash{'group_membership'} = $count;
            my $grp = $self->new();
            $grp->set_properties(%hash);
            push @groups_list, $grp;
    }
    return @groups_list;
}

sub DESTROY {
    my $self = shift;

}

# returns the group with the name supplied
sub get_group_by_name {
    my $self = shift;
    my $name = shift;
    my @groups = Group->get_groups_where("group_name='$name'");
    return $groups[0];
}

sub get_member_uids {
    my $self = shift;
    my $grp_id = shift || $self->{grp_id};
    my $recurse = shift || 0;
    my @results;

    $grp_id = undef unless Utility->is_valid_integer($grp_id); # secu_200401
    my $rows = DBAccess->sqlSelectMany('userid',
                       'grpmembers',
                       "grpmembers.grp_id='$grp_id' AND userid != 0",
                        # added check to make sure uid is a non-zero value; otherwise grp rows
                        # will be retrieved also
                       "ORDER BY userid");

    while ( my ($uid) = $rows->fetchrow() ) {
       push @results, $uid;
    }
    $rows->finish();

    if ($recurse) {
        push @results, $self->get_subgroup_member_uids($grp_id);
    }

    return @results;
}


# get the member uids of ONLY the subgrps. This is useful for knowing exactly
# who is in a subgroup (used by WebEPU)
sub get_subgroup_member_uids {
    my $self = shift;
    my $grp_id = shift;
    my @results;

        my $subgroups =
            DBAccess->sqlSelectMany ('subgrp_id', 'grpmembers', "grp_id = '$grp_id' and subgrp_id > 0");
        while (my ($sg) = $subgroups->fetchrow) {
        my @sg_user_list = $self->get_member_uids($sg, 1);
            push @results, @sg_user_list;
        }
    $subgroups->finish();

    return @results;
}


sub set_group_permission_variables {
    my $self = shift;
    my $grp_id = $self->{grp_id};
    my $owner_uid = $USER->{uid};

    # reset the permission variables to be safe!
    $IS_OWNER = $IS_ADMIN = $IS_MANAGER = 0;

    if ($grp_id)
    {
        $DISPLAY->debug_msg(4, "checking group permission variables for user $owner_uid, grp $grp_id");

        if (DBAccess->sqlSelect ("group_name", "grp", "grp_id = $grp_id and userid = '$owner_uid'")) {
            $IS_OWNER = 1;
        } elsif ( $ACCESS_BROKER->is_allowed_to_do('manage_registrations') ) {
            $IS_ADMIN = 1;
        } elsif (DBAccess->sqlSelect ("is_manager", "grpmembers", "grp_id = $grp_id and userid = '$owner_uid' and is_manager = 1")) {
            $IS_MANAGER = 1;
        }

        $DISPLAY->debug_msg(4, "IS_OWNER: $IS_OWNER; IS_ADMIN: $IS_ADMIN; IS_MANAGER: $IS_MANAGER");
    }
}

sub is_hidden {
    my $self = shift;
    my $grp_id = shift;

    if ( $ACCESS_BROKER->is_allowed_to_do('server_setup') ) { return '' }

    my $group_name = $self->get_group_name_by_grp_id( $grp_id );

    if( $HIDDEN_GROUPS->{ $group_name } ) {
    return 1;
    }

    return '';
}

sub get_implementation {
    my $class = shift;
    my $implementation_class = shift || 'GroupImplementation';
    my $default = shift || 'Metadot::Implementations::Group::Default';
    my $impl = $class->SUPER::get_implementation($implementation_class, $default);
    return $impl;
}

sub get_organization_groups {
    my $class = shift;
    my $list = DBAccess->sqlSelectMany("grp_id, group_name", "grp", "is_organization=1", "order by group_name");


    my $groups;
    while (my($grp_id, $group_name) = $list->fetchrow) {
         $groups->{$grp_id} = $group_name;
    }

    return $groups;
}

sub get_organization_info {
    my $class = shift;
    my $grp_id = shift;
    my $get_organization_info = 1;

    my ($name, $is_organization) = $class->get_group_name_by_grp_id($grp_id, $get_organization_info);

    return ($name, $is_organization);
}

sub is_group_an_organization {
    my $class = shift;
    my $grp_id= shift;

    return unless $grp_id;
    #my $is_organization = DBAccess->sqlSelect("is_organization", "grp", "grp_id=$grp_id");
    my ($name, $is_organization) = $class->get_organization_info($grp_id);
    return $is_organization;
}


sub group_welcome_letters_allowed {
    my $class = shift;
    return $PARAMS->{allow_group_welcome_letters};
}


# send the group welcome letter
sub _send_group_welcome_letter {
    my $self = shift;
    my $user = shift;

    $DISPLAY->debug_msg(4, "in _send_group_welcome_letter");
    if (defined $user && $self->group_welcome_letters_allowed()) {
        # Check if the user has permission. If broadcast emails are allowed, then
        # anyone has permission. If not, then only registration_mgr or above
        if (Portal->is_broadcast_email_enabled() || $ACCESS_BROKER->is_allowed_to_do('manage_registrations')) {
            $self->_do_send_group_welcome_letter($user);
        } else {
            $DISPLAY->debug_msg(4, "current user is not permitted to send broadcast emails");
        }
    } else {
        $DISPLAY->debug_msg(4, "group welcome letters not allowed for this site");
    }
}


# sends the group welcome letter, without checking permissions
sub _do_send_group_welcome_letter {
    my $self = shift;
    my $user = shift || die("Must pass a user object");

    my $email = $user->email();

    if ($email) {
        $DISPLAY->debug_msg(4, "in _do_send_group_welcome_letter");

        my $subject = $self->welcome_letter_heading();
        my $body = $self->welcome_letter_body();

        if ($subject || $body) {
            # process the content
            my $map = {
                fullname            => $user->fullname(),
                email               => $user->email(),
                nickname            => $user->nickname(),
                group_name          => $self->group_name(),
                group_description   => $self->group_description(),
            };

            $subject = Utility->process_simple_template({
                template => $subject,
                map      => $map,
            });


            $body = Utility->process_simple_template({
                template => $body,
                map      => $map,
            });


            # send the email
            $DISPLAY->debug_msg(4, "in _do_send_group_welcome_letter, sending the email.");
            my $sm = Sendmail->new($email, "", "", $subject, $body);
            if ($sm) {
                $sm->sendMail();
            } else {
                $DISPLAY->debug_msg(1, "Error getting a sendmail object");
            }
        } else {
            $DISPLAY->debug_msg(4, "No welcome message to send!");
        }
    }

}




1;



__END__

=head1 NAME

Group -

=head1 SYNOPSIS

    use Group;
    $group = Group->new ($grp_id);

    $AC =  $group->get_AC;
    $isa = $group->is_a ;
    $group->save ;
    $uid = $group->get_owner_uid ;
    $id = $group->parent_id ;
    $id = $group->id ;
    $group_name = $group->get_group_name_by_grp_id ($grp_id);
    $uid = $group->is_member ($uid, $grp);
    $uid = $group->is_member_of_grp_id ($uid, $grp_id);
    $num = $group->number_of_members ($grp);
    $groups_list_ref = $group->get_group_names_where_member ($uid);
    $groups_list_ref = $group->get_groups ;
    $groups_list_ref = $group->get_grp_ids ;
    $group->www_show_groups ;
    $group->www_show_all_groups ;
    $group->www_show_group_members ;
    $group->www_verify_change_group_information ;
    $group->www_verify_add_group ;
    $group->www_edit_groups ;
    $group_update_error_message = $group->change_group_name ($group_name, $old_group_name);
    $group_update_error_message = $group->change_group_description ($group_name, $group_description, $uid);
    $group->add_remove_group_members ;
    $html = $group->show_edit_groups_page ;
    $group->www_delete_group ;
    $group->DESTROY ;


=head1 DESCRIPTION

The groups class provides for creating groups, modifying groups, and deleting groups, as well as adding and removing members from groups.

=head2 Methods

Constructor:  instantiates a group, filling it with information from the database if a group id is passed to the constructor.

    $group = Group->new([$group_id])


Return the operations associated with each access control bundle, as well as the default, min, and max level for those bundles.

    $AC =  Group->get_AC;


Return the isa of the class (this will always return group)

    $isa = $group->is_a


Saves the permissions associated with the group class

    $group->save ;


Returns the uid of the owner of the group

    $uid = $group->get_owner_uid ;


Returns 0  the parent_id subroutine is a requirement of classes implementing the Auditable interface

    $id = $group->parent_id ;


Returns 1   the id subroutine is a requirement of classes implementing the Auditable interface

    $id = $group->id ;

Returns the name of the group, given the id of a group.

    $group_name = Group->get_group_name_by_grp_id ($grp_id);


Returns true if the uid passed in is a member of the group passed in, otherwise it returns undef

    $uid = $group->is_member ($uid, $grp);

Returns true if the uid passed in is a member of the group corresponding to the group id passed in
    $uid = $group->is_member_of_grp_id ($uid, $grp_id);


Returns the number of members of the group corresponding to the group name passed in
    $num = $group->number_of_members ($grp);


Return a reference to an array containing the names of all the groups that the uid passed in is a member of
    $groups_list_ref = $group->get_group_names_where_member ($uid);


Return a reference to an array containing the names of all the groups
    $groups_list_ref = $group->get_groups ;


Return a reference to an array containing the ids of all the groups
    $groups_list_ref = $group->get_grp_ids ;


Prints to a brower a page that shows the current user's groups, and a form that allows the user to add new gorups
    $group->www_show_groups ;


Prints to a browser a page that shows all the groups
    $group->www_show_all_groups ;


Prints to a broswer a page that shows the current members of a given group, and a form that allows the user to search for additional members to add
    $group->www_show_group_members ;


Takes in a group name and description information, and the group id, which stays the same, all via form parameters.
Then, it checks the group name and description.  If the information is good, it makes the change, and then calls
www_show_groups.  Otherwise, it prints to the browser a message detailing the nature of the problem with the newly
proposed group name or description.

    $group->www_verify_change_group_information ;


Takes in new group name and description information, all via form parameters.
Then, it checks the group name and description.  If the information is good, it creates the group, and then calls
www_show_groups.  Otherwise, it prints to the browser a message detailing the nature of the problem with the newly
proposed group name or description.

    $group->www_verify_add_group ;


Prints to a browser a page that shows the current name and description of a group, and allows a person to edit that information

    $group->www_edit_groups ;


Takes information about members of a group that have been submitted via a form, and adds those members to a group

    $group->add_remove_group_members ;

Provides the html used in the www_edit_groups page


    $html = $group->show_edit_groups_page ;


Prints a page that promts the user to confirm that a given group should be deleted.  If the user clicks yes, this function
also processes that information, and deletes the group.
    $group->www_delete_group ;

Required for mod_perl
    $group->DESTROY ;





=cut
