###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package HTML::Decorator;
use strict;
use vars qw(@ISA);
@ISA=();

sub new {
  my $proto = shift;
  my %parameters = @_;

  my $class = ref($proto) || $proto;
  my $self = {};

  #instance variables
  $self->{_component} = undef;
  $self->{_parameters} = \%parameters;

  bless ($self, $class);

  # if a component was provided, assign it
  my $component = $self->get_parameter('component');
  if ($component) {
    $self->set_component($component);
  }

  return $self;
}


# methods being decorated_______________________________________________
sub show_summary {
    my ($self, @args) = @_;
    my $component = $self->get_component();
    my $method = 'show_summary';
    if ($component->can($method)) {
        return $self->get_component()->$method(@args);
    }
}

sub show {
    my ($self, @args) = @_;
    my $component = $self->get_component();
    my $method = 'show';
    if ($component->can($method)) {
        return $self->get_component()->$method(@args);
    }
}

sub show_as_roll_over {
    my ($self, @args) = @_;
    my $component = $self->get_component();
    my $method = 'show_as_roll_over';
    if ($component->can($method)) {
        return $self->get_component()->$method(@args);
    }
}

# utility methods_______________________________________________________
sub accepts_parameters {
    my $self = shift;
    my $results = {};
    $results->{component} = 'SCALAR';

    return $results;
}

sub accepts_parameter {
    my $self = shift;
    my $parameter = shift;
    my $result = $self->accepts_parameters()->{$parameter};

    return $result;
}


# accessors and mutators________________________________________________
sub set_component {
    my ($self, $component_ref) = @_;
    $self->{_component} = $component_ref;
}

sub get_component {
    my $self = shift;
    my $recursive = shift;
    my $result = $self->{_component};

    if ($recursive) {
        # see if my component can do get_component
        if ($result->can('get_component')) {
            return $result->get_component($recursive);
        }
    }
    return $result;
}

sub get_parameter {
    my $self = shift;
    my $key = shift;
    return $self->{_parameters}->{$key};
}


1;
