###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::AbstractMessage;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CGI);

use Carp;
use vars qw(@ISA $AUTOLOAD);

use HTML::Parser;
use DBAccess;
use Text::Wrap;
use Portal;
#use Metadot::Utils::Theme;
#use Gizmo;
#use Gizmo::Discussion;
use Metadot::UserFactory;
use Utility;
use Operations;

# CKIM: testing to see where this breaks
#@ISA = qw(Discussion Portal);
@ISA = qw(Portal);


sub get_owner_uid {
    my $self = shift;

    return $self->{recipient_uid};
}

sub get_AC {
    my $self = shift;
    return $self->get_default_permissions();
}

sub get_default_permissions {
    my $self = shift;

    my $ac;
    if (ref($self) && $self->{_default_permissions}) {
        $ac = $self->{_default_permissions};
    } else {
        $ac = Operations->new();
        $ac->add_bundle('DISP', 'View', 8, 8);
        $ac->add_op('DISP', 'show');
    }

    return $ac;
}

sub initialize {
    my $self = shift;
}

sub new {
    my $proto = shift;
    my $mid = shift;
    my $class = ref($proto) || $proto;

    my $self;

    $mid = undef unless Utility->is_valid_integer($mid); # secu_200401

    $self->{mid}             = undef;
    $self->{iid}             = "";
    $self->{parent_mid}      = "";
    $self->{postuid}         = "";
    $self->{name}            = "";
    $self->{description}     = "";
    $self->{creation_date}   = "";
    $self->{update_date}     = "";
    $self->{updated_by}      = "";
    $self->{status}          = "";
    $self->{priority}        = "";
    $self->{email}           = "";
    $self->{author_name}     = "";
    $self->{author_email}    = "";
    $self->{recipient_uid}   = "";
    $self->{is_a}            = undef;

    # [vadim] runtime flag
    $self->{editing}    = undef ;

    if(defined ($mid)){

      ($self->{mid}            ,
       $self->{postuid}            ,
       $self->{parent_mid}     ,
       $self->{iid}            ,
       $self->{name}           ,
       $self->{description}    ,
       $self->{creation_date}  ,
       $self->{update_date}    ,
       $self->{updated_by}     ,
       $self->{status}         ,
       $self->{priority}       ,
       $self->{email}          ,
       $self->{author_name}    ,
       $self->{author_email}   ,
       $self->{recipient_uid}  ,
       $self->{is_a}           ,
      )
    = DBAccess->sqlSelect(
            "mid           ,
                    userid         ,
            parentmid      ,
            iid            ,
            name           ,
            description    ,
            creationdate ,
            updatedate ,
            updatedby      ,
            status         ,
            priority       ,
            email          ,
            authorname     ,
            authoremail    ,
            recipient_uid  ,
            isa",
            "message",
            "mid=$mid"
           );

    }
    $self->{creation_date} = Utility->format_datetime($self->{creation_date}, 'b e, Y, r');
    $self->{update_date}   = Utility->format_datetime($self->{update_date}, 'b e, Y, r');
    unless (defined($self->{is_a}) || $self->{is_a} ne '') {
        $self->{is_a} = $class;
    }

    bless ($self, $class);
    $self->initialize();

    return $self;
}

# aju 03-9-02
sub get_id {
    my $self = shift;
    return $self->get_mid();
}

sub id {
    my $self = shift;
    return $self->get_mid();
}

sub get_mid {
    my $self = shift;
    return $self->{mid};
}

sub authorname {
    my $self = shift;
    return $self->{author_name};
}

sub authoremail {
    my $self = shift;
    return $self->{author_email};
}

sub creationdate {
    my $self = shift;
    return $self->{creation_date};
}

#####################################################
# WWW FUNCTIONS


sub www_delete_message {
        my $self = shift;
        my $mid = $self->mid();
        my $referrer = $SESSION->{referer};
        # [vadim] do not forget to escape the URL !
         my $target = $CGI->escape($referrer);
        #

        $self->debug_msg(3, "Message::www_delete()");
        $self->print_http_header;
        $self->two_choices("Do you want to delete this Message?",
                "caution",
                "Yes please",
                $self->make_url("index.pl?mid=$mid&isa=Message&op=delete_message_ok&target=$target"), "No!", $referrer);
}


#####################################################
# BACKEND FUNCTIONS

sub tag_open_sub{

    my $not_ok;
    my ($tagname, $attr) = @_;

    if(defined($tagname) && $tagname eq 'img') {

        # will remove the img tag altogether if its src attribute
        # contains any & and ? characters
        #      use this line instead if you want to go all the way to
        #      prohibiting all image types but the ones mentioned in
        #      the regexp:
        #         if( $attr->{src} && ( ($attr->{src} =~ /\&|\?/) ||
        #           ($attr->{src} !~ /(gif|jpg|png|jpeg)$/i) ) ) {
        if( $attr->{src} &&  ($attr->{src} =~ /\&|\?/) ) {

            $not_ok = 1;

        }

    }

};



sub add_message {
    my $class = shift;
    my %args = @_;

    my $db_now = Utility->db_now();
    my $isa    = ref($class) || $class;

    my %fields = (
                  userid        => $args{userid}        || $USER->uid(),
                  authorname    => $args{authorname}    || $USER->fullname(),
                  authoremail   => $args{authoremail}   || $USER->{email},
                  iid           => $args{iid},
                  parentmid     => $args{parentmid}     || 0,
                  name          => $args{name},
                  description   => $args{description},
                  priority      => $args{priority},
                  status        => $args{status}        || $class->_get_default_status(),
                  creationdate  => $args{creationdate}  || $db_now,
                  updatedate    => $args{updatedate}    || $db_now,
                  recipient_uid => $args{recipient_uid},
                  isa           => $args{is_a}          || $isa,
    );

    #use Data::Dumper;
    #print STDERR "Inserting new message with args\n",Dumper(\%args),"\nInsert fields are:\n",Dumper(\%fields);

    my $mid = DBAccess->sqlInsert("message", %fields);
    #print STDERR "Message was inserted\n";
    return $mid;
}

sub _get_default_status{
    return 'OK';
}

sub _delete_message {
  my $self = shift;
  my $mid  = shift || $self->mid();
  $mid = undef unless Utility->is_valid_integer($mid); # secu_200401
  my $iid  = $self->iid();

  # Selecting all messages that mid is parentmid
  my ($c)=DBAccess->sqlSelectMany("mid", "message", "iid=$iid AND parentmid=$mid");

  while(my ($childmid) =$c->fetchrow()) {
      $self->_delete_message($childmid);
  }
  DBAccess->sqlDelete("message", "mid=$mid") if defined($mid);
}

sub user_has_messages_with_status {
  my $class = shift;
  my $args  = shift;

  my $uid  = $args->{uid};
  my $status = $args->{status};

  return unless Utility->is_valid_integer($uid); # secu_200401
  return if $status eq '';

  my $isa    = ref($class) || $class;

  my ($count) = DBAccess->sqlSelect("count(*)", "message", "recipient_uid = $uid and isa= '$isa' and status = '$status'");

  return $count;
}

# modify object attribute for reply
sub prepare_reply {
  my $self = shift;
  $self->debug_msg(4, "Message::prepare_reply()");

  ### prepare the Re: message
  $self->name("Re: ".$self->name) if ($self->name !~ m/^Re: /);

  ### Copy the orginal msg and add > in front of each line.
  $Text::Wrap::columns=65;
  my $pre1="> ";
  my $pre2="> ";
  $self->description(wrap($pre1, $pre2, $self->description));

  ### [vadim] Set the new flag (to recognize the difference between editing message and new one)
  $self->{editing} = undef ;
}

#####################################################

1;


__END__
