###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::AccessBroker::SubmissionItemBroker;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CACHE);

# this is a singleton;
my $self = {};

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    bless ($self, $class);

    return $self;
}

my %SUBMITTER_METHODS = (
         modify => 1,
         show   => 1,
         save   => 1,
         download_file => 1,
         confirm_upload_deletion => 1,
         delete_upload => 1,
         add_comments => 1,
         );

my %SUBMITTER_METHODS_WHEN_REJECTED = (
         show   => 1,
         download_file => 1,
         );

my %APPROVER_METHODS = (
         modify  => 1,
         delete  => 1,
         delete_ok  => 1,
         show    => 1,
         can_update_status => 1,
         save   => 1,
         download_file => 1,
         confirm_upload_deletion => 1,
         delete_upload => 1,
         add_comments => 1,
         approve => 1,
         reject  => 1,
         );

## ADMINS CAN PERFORM THESE METHODS IN ADDITION TO THE APPROVER METHODS
my %ADMIN_METHODS = (
#         change_owner => 1, ## don't know if we want to allow this capablity
         );


sub is_allowed_to_do {
    my $self = shift;
    my $op   = shift || die "must provide operation name";
    my $user = shift || die "must provide user";
    my $submissionitem = shift || die "must provide submission item object";

    my $parent = $submissionitem->get_parent();
    if ($user->is_logged_in()) {
        my $rejected_status_code = $submissionitem->get_rejected_status_code();
        my $pending_status_code = $submissionitem->get_pending_status_code();
        if ( $user->is_admin() && ( $ADMIN_METHODS{$op} || $APPROVER_METHODS{$op} ) ) { ## admin case
            return 1;
        } elsif ( $parent->is_allowed_to_do('approve', $user) && $APPROVER_METHODS{$op} ) { ## approver case
            return 1;
        } elsif ($submissionitem->get_status() == $rejected_status_code && $user->is_owner($submissionitem) && $SUBMITTER_METHODS_WHEN_REJECTED{$op}) { ## case for submitter when submission is rejected
            return 1;
        } elsif ($submissionitem->get_status() == $pending_status_code && $user->is_owner($submissionitem) && $SUBMITTER_METHODS{$op}) { ## case for submitter when submission is pending
            return 1;
        }
    }
    return '';
}

1;
