###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::AccessBrokerFactory;
use strict;

use DBAccess;
use MetadotConfig;

#we store in this hash the different instantiated reg types.
#if we need a particular reg type, we first check to see
#if it is in this hash
my %broker_types;

sub get_access_broker {
    my $class = shift;

    my $access_broker_type = MetadotConfig->get_access_broker_type() || 'Default';

    if ( $broker_types{$access_broker_type} ) {

        return $broker_types{$access_broker_type};

    } else {
        require "Metadot/AccessBroker/$access_broker_type.pm";

        my $broker = "Metadot::AccessBroker::$access_broker_type"->new();
        $broker_types{$access_broker_type} = $broker;
        return $broker;
    }
}

1;


=head1 NAME

RegisterFactory -

=head1 SYNOPSIS

    use Metadot::RegisterFactory;
    $register = Metadot::RegisterFactory->get_register();


=head1 DESCRIPTION

RegisterFactory is responsible for returning an appropriate subclass of
Metadot::Register.  It does that by first examining the type of register
that it should return, and then checking to see if it has a previously
instantiated register of that type available to return.  If it does,
it returns that instantiated register.  If not, it instantiates
the appropriate register type, adds it to its cache, and then returns it.

Returns the appropriate register type
    $register = Metadot::RegisterFactory->get_register();


