###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Authenticator;

use strict;

use DBAccess;
use Metadot::SessionHandlerFactory;
use Metadot qw(%FORM $PARAMS $CGI $SESSION);
use Portal;

my $CREATE_SESSION = 1;

sub new {
    my $proto = shift;
    my $class = ref{$proto} || $proto;
    my $self = {};

    bless ($self, $class);
    return $self;
}


sub authenticate {
    my $self = shift;
    my $session_id = $FORM{key};

    my $user = undef;
    my $session = undef;

    my $action = $self->determine_action();

    my $session_handler = Metadot::SessionHandlerFactory->get_session_handler();

    if (defined($action) && ($action == $CREATE_SESSION)) {
        $session = $self->create_session();
        $session_handler->persist_session($session);
        $user = $self->restore_user( $session );
        if ($user && $user->uid()) {
            $user->last_online(time);
            $user->save();
        }
    }

    if ($session_id) {
        $session = $session_handler->restore_session($session_id);
    }

    unless ($session) {

    $session = $session_handler->restore_session();

    unless ($session) {
        # create new session for an anonymous user
        return $self->create_anonymous_session_and_user();
    }
    }

    $user = $self->restore_user($session);

    if( $user->is_logged_in( $session ) ) {
    $session->refresh_session($ENV{'REMOTE_ADDR'});
    }

    return ($user, $session);
}


sub create_anonymous_session_and_user {
    my $self = shift;

    my $anonymous_uid = MetadotConfig->get_reserved_uid('ANONYMOUS_USER');

    my $session = Session->new();
    $session->add( $anonymous_uid );

    my $user = $self->restore_user( $session );
    $user->logout( $session );

    my $session_handler = Metadot::SessionHandlerFactory->get_session_handler();
    $session_handler->persist_session( $session );

    return ($user, $session);
}


sub determine_action {
    die "abstract, must be overridden";
}


sub create_session {
    die "abstract, must be overridden";
}


sub restore_user {
    die "abstract, must be overridden";
}

sub show_my_page_login_box {
    die "abstract, must be overridden";
}

sub show_login_form {
    die "abstract, must be overridden";
}

sub process_login_form {
    die "abstract, must be overridden";
}

sub get_CREATE_SESSION {
    my $self = shift;

    return $CREATE_SESSION;
}

sub authorize {
# go to the target after the identification - the target must be provided by the subclasses
# can be redefined in the subclasses.
    my $self = shift;
    my $target = shift or die "missing target";

    #print "<META HTTP-EQUIV='Refresh' CONTENT='0; url=$target'>";
    Portal->md_redirect($target);
}

### THIS METHOD MUST BE CALLED FROM EACH AUTHENTICATOR SUBCLASS
### Defines and returns password form parameters based on password mode
sub get_password_form_parameters {
    my ($self, %params) = @_;

    my $form_type     = $params{form_type} || 'login';
    my $login_mode = $params{login_mode} || ''; ## desired login mode
    my $target = $params{target} || $PARAMS->{post_login} || $SESSION->referer() || $PARAMS->{home_page} || 'index.pl';
    $target = Utility->get_qualified_url($target);
    my $refererlink = $params{refererlink};

    my $form_action = '';
    my $password_submessage = '';
    my $force_secure_login = MetadotConfig->get_param_value('force_secure_login');

    if ($PARAMS->{login_mode} eq 'https_standard') {
        if ($login_mode eq 'standard' && !$force_secure_login) {
            $form_action = Utility->get_unsecure_url('index.pl');
            $password_submessage .= "The secure server will encrypt your information. ";
            if ($form_type eq 'register') {
                $password_submessage .= "If you wish, you can register using our <a href=\"" . Utility->get_unsecure_url('index.pl') . "?isa=Register&amp;op=register_form&amp;login_mode=secure&amp;refererlink=" . $CGI->escape($refererlink) . "\">secure server</a>.";
            } elsif ($form_type eq 'profile_self') {
                $password_submessage .= "If you wish, you can modify your profile using our <a href=\"" . Utility->get_unsecure_url('index.pl') . "?isa=User&amp;op=modify_self&amp;login_mode=secure&amp;refererlink=" . $CGI->escape($refererlink) . "\">secure server</a>.";
            } elsif ($form_type eq 'profile_other') {
                $password_submessage .= "If you wish, you can modify the profile using our <a href=\"" . Utility->get_unsecure_url('index.pl') . "?isa=User&amp;op=modify&amp;uid=" . $SESSION->get_value('User_uid_to_modify') . "&amp;login_mode=secure&amp;refererlink=" . $CGI->escape($refererlink) . "\">secure server</a>.";
            } else {
                $password_submessage .= "If you wish, you can sign in using our <a href=\"" . Utility->get_unsecure_url('index.pl') . "?isa=Session&amp;op=login&amp;login_mode=secure&amp;target=" . $CGI->escape($target) . "\">secure server</a>.";
            }
        } else {
            $login_mode = 'secure';
            $form_action = Utility->get_secure_url('index.pl');
            $password_submessage .= $force_secure_login ? '' : "If you received an error message when you tried to use our secure server, ";
            if ($form_type eq 'register') {
                $password_submessage .= $force_secure_login ? '' : "register using our <a href=\"" . Utility->get_unsecure_url('index.pl') . "?isa=Register&amp;op=register_form&amp;login_mode=standard&amp;refererlink=" . $CGI->escape($refererlink) . "\">standard server</a>.";
            } elsif ($form_type eq 'profile_self') {
                $password_submessage .= $force_secure_login ? '' : "modify your profile using our <a href=\"" . Utility->get_unsecure_url('index.pl') . "?isa=User&amp;op=modify_self&amp;login_mode=standard&amp;refererlink=" . $CGI->escape($refererlink) . "\">standard server</a>.";
            } elsif ($form_type eq 'profile_other') {
                $password_submessage .= $force_secure_login ? '' :"modify the profile using our <a href=\"" . Utility->get_unsecure_url('index.pl') . "?isa=User&amp;op=modify&amp;uid=" . $SESSION->get_value('User_uid_to_modify') . "&amp;login_mode=standard&amp;refererlink=" . $CGI->escape($refererlink) . "\">standard server</a>.";
            } else {
                $password_submessage .= $force_secure_login ? '' : "sign in using our <a href=\"" . Utility->get_unsecure_url('index.pl') . "?isa=Session&amp;op=login&amp;login_mode=standard&amp;target=" . $CGI->escape($target) . "\">standard server</a>.";
            }
        }
    #} elsif ($PARAMS->{login_mode} eq 'https') {
    #    $login_mode = 'secure';
    #    $form_action = Utility->get_secure_url('index.pl');
    } else {## standard default - no special treatment
        $login_mode = 'standard';
        $form_action = 'index.pl';
    }

    return ($form_action, $login_mode, $password_submessage, $target);
}


sub get_logout_url {
    my $class    = shift;
    my $url_args = shift;

    my $result = "index.pl?isa=Session&amp;op=logout";
    if (defined $url_args) {
        $result .= "&amp;$url_args";
    }
    return $result;
}

sub get_login_url {
    my $class    = shift;
    my $url_args = shift;

    my $result = "index.pl?isa=Session&amp;op=login";
    if (defined $url_args) {
        $result .= "&amp;$url_args";
    }
    return $result;
}


1;

__END__


=head1 NAME

Metadot::Authenticator - Abstract base class for implementing different authentication strategies in the Metadot framework.


=head1 SYNOPSIS

  my $authenticator = Metadot::AuthenticatorFactory->get_authenticator();

  ($USER, $SESSION) = $authenticator->authenticate();


=head1 DESCRIPTION

Method Metadot::Authenticator::authenticate acts as a template that
should be applicable for all kinds of concrete Authenticator
subclasses. That is, authenticate is not meant to be overridden in
concrete subclasses. Concrete subclasses will override the methods
invoked by authenticate, to customize its behavior as needed.

Method authenticate will return user and session objects, to be used
by the framework to keep track of a users status.


=head1 METHODS

  authenticate

A template method to enforce an order of execution for each of the
methods implemented by concrete Metadot::Authenticator subclasses to
authenticate and return user and session objects.

  get_CREATE_SESSION

A concrete method that returns a token to signal that a new session
must be created for a user, to be returned by method
"determine_action", which must be overridden by concrete subclasses.

  show_login_form

  show_my_page_login_box

Methods to display login forms for the general portal and for My
Page. These will be automatically called by the framework when login
forms rendering is required (Invoked from Session::www_login and
MyPage::show).

  create_session

Should authenticate the user by verifying login information. This
method is only invoked when methoid "detemine_action" returns a
CREATE_SESSION token.

  process_login_form

Used to process $FORM values from login forms. Will be invoked as a
result of login form submits, from Session::www_login_check. This
method is not in charge of authenticating but of validating login form
contents. The framework will always invoke this method after
create_session is invoked.

  determine_action

Should return a CREATE_SESSION token (that it gets from method
get_CREATE_SESSION) when it detects the appropriate values in %FORM,
that is, when a login form is being processed.

 restore_user

Should restore the User object belonging to the user, or the anonymous
user, depending on success of authentication.



