###########################################################################
# <LICENSEINFO>
# Metadot SmartSite Server                                                                      
# Copyright (C) 2001 Metadot Corporation                                                        
#                                                                                               
# For more information go to                                                                    
# http://www.metadot.com                                                                        
# or contact us at info@metadot.com                                                             
#                                                                                               
#</LICENSEINFO>
############################################################################

# An interface that defines a Metadot::Authenticator class as using an
# external Single-Signon (SSO) mechanism, such as RSA ClearTrust or
# Netegrity SiteMinder
package Metadot::Authenticator::BaseAuthenticator;

use strict;
use MetadotConfig;
use Metadot::Authenticator;
use Metadotclass;

use vars qw(@ISA);
@ISA = qw( Metadot::Authenticator Metadotclass);

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self  = $class->SUPER::new();
    return bless($self, $class);
}


sub restore_user {
    my $self = shift;
    my $session = shift;

    my $uid = $session->uid();

    my $user;

    # CKIM: added a check for session expiration
    Metadotclass->debug_msg(5, "restore_user checking if uid '$uid' session expired? ".$session->is_expired());
    if(($uid || $uid == 0) && !$session->is_expired()) {
        $user = Metadot::UserFactory->get_user( $uid );
        Metadotclass->debug_msg(5, "restore_user restore user uid='$uid' $user");
    } else {
        $user = Metadot::UserFactory->get_user( MetadotConfig->get_reserved_uid('ANONYMOUS_USER') );	
        Metadotclass->debug_msg(5, "restore_user restore ANONYMOUS user $user");
    }

    return $user;
}



1;
