###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Authenticator::LDAPAuthenticator;

use strict;

use Metadot::Authenticator::UserPassAuthenticator;
use Portal;
use DBAccess;

use Metadot::LDAPSubsystemFactory;
use Metadot::Utils::Display;
use Metadot::Utils::Theme;

use Metadot qw(%FORM $PARAMS $USER $SESSION $DISPLAY);

use vars qw(@ISA);
@ISA=qw( Metadot::Authenticator::UserPassAuthenticator );



sub create_session {
    my $self = shift;


    my $ldap_uid = $FORM{email};
    my $password = $FORM{password};
    my $registering = $FORM{registering};

    $self->debug_msg(4, "Metadot::Authenticator::LDAPAuthenticator::create_session with ldap_uid:$ldap_uid");

    my $result;

    my $ldap = Metadot::LDAPSubsystemFactory->get_ldap_subsystem();

    unless ( $USER->is_default_account( $ldap_uid ) ) {

        unless( $ldap->initialize() ) {
            $DISPLAY->print_click_back( "LDAP authentication is not properly configured or the LDAP server is down. Please contact your site administrator. " );
        }

        $result = $ldap->check_ldap_password( $ldap_uid, $password );
        if (my $ldap_error = $ldap->get_error()) {
            $DISPLAY->print_click_back('Error', "<p><b>There was an error processing your LDAP credentials:</b></p><p>$ldap_error</p>");
            exit();
        }

    } else {
        # we have to use the standard "UserPassAuthenticator" for default accounts
        return $self->SUPER::create_session();
    }

    my $session = Session->new();

    if( defined $result ) {

        if( $result ) {

            $self->debug_msg(4, "Metadot::Authenticator::LDAPAuthenticator::create_session ldap password verified - process user account");

            $self->process_user_account(ldap_uid => $ldap_uid, password => $password, registering => $registering);

            # authentication success
            my $uid = $USER->get_uid_from_ldap_uid( $ldap_uid );

            if( defined $uid) {
                $self->debug_msg(4, "Metadot::Authenticator::LDAPAuthenticator::create_session uid exists $uid");
                $session->add($uid);
            } else {
                $self->debug_msg(4, "Metadot::Authenticator::LDAPAuthenticator::create_session uid does not exist");

                # authentication failure
                $session->add();
            }

        } else {
            $self->debug_msg(4, "Metadot::Authenticator::LDAPAuthenticator::create_session ldap password invalid");
            # authentication failure
            $session->add();
        }

    } else {

        # LDAP is not properly configured or
        # server is not reachable
        $DISPLAY->print_click_back( "LDAP authentication is not properly configured or the LDAP server is down. Please contact your site administrator. " );

        exit();

    }

    return $session;

}


sub show_my_page_login_box {
    my $self = shift;
    my $error_message=shift;

    my $email = $PARAMS->{sessions_are_deleted_on_logout} ? '' : $USER->email();

    my $target = Utility->get_qualified_url("userchannel.pl");
    my $form_action = '';

    if ($PARAMS->{login_mode} eq 'https_standard' || $PARAMS->{login_mode} eq 'https') {
        $form_action = Utility->get_secure_url('index.pl');
    } else {
        $form_action = Utility->get_unsecure_url('index.pl');
    }

    my $rs = "<FORM action='$form_action' method='POST'>";
    $rs .= "<input type='hidden' name='target' value='$target'>";
    $rs .= "<input type='hidden' name='isa' value='Session'>";
    $rs .= "<input type='hidden' name='op'  value='login_check'>";

    if($error_message){
    $rs .= " <font color='red'>Sorry! Please enter an email address.</font><br /><br /> ";
    }

    if (!$PARAMS->{registration_closed}) {
        $rs .= "Not a member yet? <br /><a href=\"index.pl?isa=Register&amp;op=register_form\">Click here to register</a><br /><br />\n";
    }

    $rs .= "<table border=\"0\" cellspacing=\"1\">";
    $rs .= "<tr><td bgcolor=\"#000000\">";

    $rs .= "<table border=\"0\" cellspacing=\"0\" cellpadding=\"5\">";
    $rs .= "<tr bgcolor=\"#eeeeee\"><td>";

    my $ldap_uid_name = $PARAMS->{'ldap_uid_label'} || $PARAMS->{'ldap_uid'};
    my $prompt = "Your $ldap_uid_name";
    my $ldap_uid = $USER->ldap_dn();

    $rs .= "<b>$prompt:</b><br />";
    $rs .= "<INPUT type=text name=email size=20 value='$ldap_uid'>";

    $rs .= "</td></tr>";

    $rs .= "<tr bgcolor=\"#eeeeee\"><td>";
    $rs .= "<b>Password:</b><br />";
    $rs .= "<INPUT type=password name=password size=20><br />";

  #aju
  my $mypageelementboxcolor; my $mypagetitleboxcolor;
  if (Metadot::Utils::Theme->use_theme_info() ) {
      # get the color
      my $boxcolor = $PARAMS->{'theme_mypage_color'};
      $mypageelementboxcolor = $boxcolor;
      $mypagetitleboxcolor = $boxcolor;
  }
  else {
      $mypageelementboxcolor = $PARAMS->{'mypageelementboxcolor'};
      $mypagetitleboxcolor = $PARAMS->{'mypagetitleboxcolor'};
    }
  if (substr($mypageelementboxcolor,0,1) ne "#") {
      $mypageelementboxcolor = "#$mypageelementboxcolor";
  }
  if (substr($mypagetitleboxcolor,0,1) ne "#") {
      $mypagetitleboxcolor = "#$mypagetitleboxcolor";
  }
#      my $mypageelementboxcolor = $PARAMS->{'mypageelementboxcolor'};
#      my $mypagetitleboxcolor = $PARAMS->{'mypagetitleboxcolor'};

    $rs .= "</td></tr>";

    $rs .= "</table>";

    $rs .= "</td></tr>";
    $rs .= "<tr><td align='right'>";
    $rs .= "<INPUT type=submit name=action value='Login' class='button'>";
    $rs .= "</FORM>\n";
    $rs .= "</td></tr>";
    $rs .= "</table>";


    return "

                <TABLE CLASS='FRAME' CELLPADDING=\"2\" CELLSPACING=\"0\" border=\"0\" WIDTH=\"100%\" BGCOLOR='$mypageelementboxcolor'><TR><TD>

                <TABLE CLASS='FRAMETITLE' CELLPADDING=\"0\" CELLSPACING=\"0\" border=\"0\" WIDTH=\"100%\" BGCOLOR='$mypagetitleboxcolor'><TR>
                <TD HEIGHT=\"15\">
                <font class='FRAMETITLE'>Login</FONT>
                </TD></TR></TABLE>

                <TABLE CLASS='FRAMECONTENT' CELLPADDING=\"2\" CELLSPACING=\"0\" border=\"0\" WIDTH=\"100%\" BGCOLOR=white>
                <TR><TD align='center'>
                $rs
                </TD></TR>
                </TABLE>
                </TD></TR>
                <TR><TD HEIGHT=\"0\" BGCOLOR=white></TD></TR>
                </TABLE>
        ";

}


sub show_login_form {
    my $self = shift;
    my $email = shift;
    my $error_message = shift || undef;
    my $style = shift || undef;

#    my $target = $FORM{target} || $PARAMS->{post_login} || $SESSION->referer() || $PARAMS->{home_page} || 'index.pl';

    ## will set form parameters depending on site's login mode
    my ($form_action, $login_mode, $login_submessage, $target) = $self->get_password_form_parameters(login_mode => $FORM{login_mode}, target => $FORM{target});

    my $registering = $FORM{registering} || 0;
    #print STDERR "\n\nshow_login_form(): form_action:$form_action login_mode:$login_mode target:$target login_submessage:$login_submessage\n\n";

#      my $rs = "<SCRIPT LANGUAGE=\"JavaScript\" SRC=\"/js/SubmitRollover.js\"></SCRIPT>";
    my $rs = "<p /><FORM name=\"login_form\" action='$form_action' method='POST'>\n";

    $rs .= qq~<div id="tagLoginBox">~;

    $rs .= "<input type='hidden' name='target' value='$target'>\n";
    $rs .= "<input type='hidden' name='isa' value='Session'>\n";
    $rs .= "<input type='hidden' name='op'  value='login_check'>\n";
    $rs .= "<input type='hidden' name='registering'  value='$registering'>\n";
    $rs .=  $DISPLAY->show_title_bar("Login");

    if($error_message){
    $rs .= " <font color='red'>Sorry! Please enter an email address.</font><br /><br /> ";
    }

    if (!$PARAMS->{registration_closed}) {
        $rs .= "Not a member yet? <a href=\"" . $DISPLAY->make_url("index.pl?isa=Register&amp;op=register_form")."\">Click here to register</a><br /><br />\n";
    }

    #$rs .= "<table border=\"0\" cellspacing=\"1\">";
    $rs .= "<table class=\"$style\" border=\"0\" cellspacing=\"1\">";

    #$rs .= "<tr><td bgcolor=\"#000000\">";
    $rs .= "<tr><td>";

    $rs .= "<table border=\"0\" cellspacing=\"0\" cellpadding=\"5\">";
    #$rs .= "<tr bgcolor=\"#ffffff\"><td class=$style>";
    $rs .= "<tr><td>";

    my $ldap_uid_name = $PARAMS->{ldap_uid_label} || $PARAMS->{ldap_uid};
    my $prompt = "Your $ldap_uid_name";
    my $ldap_uid = $USER->ldap_dn();

    $rs .= "<b>$prompt:</b>";
    #$rs .= "</td><td class=$style>";
    $rs .= "</td><td>";

    unless( $USER->is_default_account( $email ) ) {

    $rs .= "<INPUT type=text name=email size=20 value='$ldap_uid'>";

    } else {

    $rs .= "<INPUT type=text name=email size=20 value='$email'>";

    }

    $rs .= "</td></tr>";

    #$rs .= "<tr bgcolor=\"#ffffff\"><td class=$style>";
    $rs .= "<tr><td>";

    $rs .= "<b>Password:</b>";
    #$rs .= "</td><td class=$style>";
    $rs .= "</td><td>";
    $rs .= "<INPUT type=password name=password size=20><br />";

    $rs .= "</td></tr>";

    $rs .= "</table>";

    $rs .= "</td></tr>";
    $rs .= "</table>";

    my $iloc = $PARAMS->{'imageslocation'};
    my $login_image1 = $login_mode eq 'standard' ? 'login2_01.gif' : 'login2_02.gif';
    my $login_image2 = $login_mode eq 'standard' ? 'login2_01-over.gif' : 'login2_02-over.gif';
    $rs .= "<p />" .
  "<SCRIPT type=text/javascript>
  <!--
  function newImage(arg) {
      if (document.images) {
          rslt = new Image();
          rslt.src = arg;
          return rslt;
      }
  }
  var submit_image = newImage('$iloc/$login_image1') ;
  var submit_image_over = newImage('$iloc/$login_image2') ;
  //-->
  </SCRIPT>

  <input type=image border=\"0\" name=submit src=\"$iloc/$login_image1\"
   onMouseOut=\"this.src = submit_image.src\"
   onmouseover=\"this.src = submit_image_over.src\"
  >"
#       "<SCRIPT TYPE=\"text/javascript\">
#         <!--
#         var sr = new submitroll(\"$iloc/login2_01.gif\",\"$iloc/login2_01-over.gif\",\"mysubmit\");
#         sr.write();
#         //-->
#         </SCRIPT>" .
#        "<NOSCRIPT>
#         <INPUT TYPE=SUBMIT VALUE=\"Login\">
#         </NOSCRIPT>"
    . "<br />";

    $rs .= "</div>";

    $rs .= "</FORM>\n";
    $rs .= "<p />$login_submessage</center>\n";

}


sub process_login_form {
    my $self = shift;

    my $target = ($FORM{target} !~ /logout/) ? $FORM{target} : 'index.pl';

    #if couldn't get target , go to start page
    if (!$target) {
    $target = 'index.pl';
    }

    $DISPLAY->print_http_header();

    unless( $FORM{email} ) {

        my $ldap_uid_name = $PARAMS->{'ldap_uid_label'} || $PARAMS->{'ldap_uid'};

        $SESSION->www_login("Sorry! You need to enter your $ldap_uid_name ");
        exit 0;
    }

    if ($FORM{form_action} ne 'forgot_pwd') {

        # need to answer to a login request
        if(!$FORM{password}) {
            $DISPLAY->print_click_back("Sorry!", "You need to enter a password");
            exit 0;
        }

        # needed to check approvalneeded in user info.
        my $u = Metadot::UserFactory->get_user($SESSION->uid );

        if ( !$SESSION->uid ) {

            ## allow subclassess to handle any extra duties - may not return
            ## For some reason, not working here. Moved back to create_session()
            ##$self->process_user_account(ldap_uid => $FORM{email}, password => $FORM{password});
            $DISPLAY->print_click_back("Error", "<b>Sorry we could not identify you.</b><br /><br />
                        You entered either a wrong email address or a wrong password.");

        } elsif ($u->approval_needed eq "required" ) {

            # if login and password are right but user isnt approved yet...
            $DISPLAY->print_click_back("Thank you!", "<b>Due to access controls, your account is awaiting administrator approval.<br />
                            You will be notified via email as soon as your account is approved.</b><br /><br />");
            # getting rid of session created during login attempt.
            $SESSION->erase;
            exit 0;

        } elsif ($u->approval_needed eq "rejected") {
            # if login has been rejected
            $DISPLAY->print_click_back("Sorry!", "<b>Your account request has been rejected.</b><br /><br />");
            #remove session created during login attempt.
            $SESSION->erase;
            exit 0;

        } else {
            $self->authorize($target);
        }

    } else {

    $DISPLAY->print_click_back("Sorry!", "<b>Authentication configuration problem.</b><br /><br />");

    }

}

sub process_user_account {
    my $self = shift;
}

1;

__END__


=head1 NAME

Metadot::Authenticator::LDAPAuthenticator - Concrete subclass of
Metadot::Authenticator to provide authentication against an LDAP
server.


=head1 SYNOPSIS

  my $authenticator = Metadot::AuthenticatorFactory->get_authenticator();

  ($USER, $SESSION) = $authenticator->authenticate();


=head1 DESCRIPTION

This concrete Authenticator will query an LDAP server for user
id/password authentication. This LDAP server must be set via the
system configuration user interface methods in module
Metadot::Register::LDAP, or similar ones.

See Metadot::Authenticator for explanation of each of methods in this module.

