###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name$
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Authenticator::ManagedLDAPAuthenticator;

use strict;

use Metadot qw(%FORM $PARAMS $USER $SESSION $DISPLAY);
use Portal;
use Metadot::Authenticator::LDAPAuthenticator;
use Metadot::RegisterFactory;

use vars qw(@ISA);
@ISA=qw( Metadot::Authenticator::LDAPAuthenticator );


## method called from process_login_form() to register a user if
## doesn't exist - will not return if registering
sub process_user_account {
    my $self = shift;
    my %args = @_;

    my $ldap_uid = $args{ldap_uid};
    my $password = $args{password};
    my $registering = $args{registering};

    $self->debug_msg(4, "Metadot::Authenticator::ManagedLDAPAuthenticator::process_user_account() with ldap_uid:$ldap_uid");

    return unless defined ($ldap_uid);

    my $uid = $USER->get_uid_from_ldap_uid( $ldap_uid );

    ## if user not in db then register
    if (defined($uid) && $registering) {
        $DISPLAY->print_click_back( "You are already registered with $ldap_uid." );
        exit;
    } elsif (!defined($uid)) {
        $self->debug_msg(4, "Metadot::Authenticator::ManagedLDAPAuthenticator::process_user_account() will create account for ldap_uid:$ldap_uid");
        my $registrator = Metadot::RegisterFactory->get_register();
        ## password will be verified one more time before registering
        $registrator->www_register(
            ldap_uid => $ldap_uid,
            password => $password,
        );
        exit; ## exit to allow register to handle
    }
}

1;

__END__
