###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Authenticator::UserPassAuthenticator;

use strict;

use Metadot::Authenticator::BaseAuthenticator;
use Metadot::Utils::Theme;
use Portal;
use DBAccess;
use Metadotclass;

use Metadot qw(%FORM $USER $SESSION $DISPLAY $PARAMS $CGI);

use vars qw(@ISA);
@ISA=qw(Metadot::Authenticator::BaseAuthenticator);

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};

    bless ($self, $class);
    return $self;
}


sub determine_action {
    my $self = shift;

    if ( defined $FORM{op} && $FORM{op} eq 'login_check' && $FORM{isa} eq 'Session' ) {

        return $self->SUPER::get_CREATE_SESSION;

    }

    return undef;
}


sub create_session {
    my $self = shift;
    my $email = $FORM{email};
    my $password = $FORM{password};


    my ( $uid ) = $self->check_login($email, $password);

    my $session = Session->new();

    if( defined $uid ) {

    $session->add($uid);

    } else {

    $session->add();
    }

    return $session;
}


sub check_login {
    my $self = shift;
    my ($email, $password) = @_;

    $password = Portal->encrypt_password_if_necessary($password);

    my $resuid = $USER->check_password( $email, $password );

    if ( defined $resuid ) {

        return $resuid;

    } else {

        return undef;
    }
}

sub show_my_page_login_box {
    my $self = shift;
    my $error_message=shift;

    my $email = $PARAMS->{sessions_are_deleted_on_logout} ? '' : $USER->email();

    my $target = Utility->get_qualified_url("userchannel.pl");
    my $form_action = '';

    if ($PARAMS->{login_mode} eq 'https_standard' || $PARAMS->{login_mode} eq 'https') {
        $form_action = Utility->get_secure_url('index.pl');
    } else {
        $form_action = Utility->get_unsecure_url('index.pl');
    }

    my $rs = "<FORM action='$form_action' method='POST'>";
    $rs .= "<input type='hidden' name='target' value='$target'>";
    $rs .= "<input type='hidden' name='isa' value='Session'>";
    $rs .= "<input type='hidden' name='op'  value='login_check'>";

    if($error_message){
    $rs .= " <font color='red'>Sorry! Please enter an email address.</font><br /><br /> ";
    }

    if (!$PARAMS->{registration_closed}) {
        $rs .= "Not a member yet? <br /><a href=\"index.pl?isa=Register&amp;op=register_form\">Click here to register</a><br /><br />\n";
    }

    $rs .= "<table border=\"0\" cellspacing=\"1\">";
    $rs .= "<tr><td bgcolor=\"#000000\">";

    $rs .= "<table border=\"0\" cellspacing=\"0\" cellpadding=\"5\">";
    $rs .= "<tr bgcolor=\"#eeeeee\"><td>";

    $rs .= "<b>Email address:</b><br />";
    $rs .= "<INPUT type=text name=email size=20 value='$email'>";

    $rs .= "</td></tr>";

    $rs .= "<tr bgcolor=\"#eeeeee\"><td>";
    $rs .= "<b>Password:</b><br />";
    $rs .= "<INPUT type=password name=password size=20><br />";

  #aju
  my $mypageelementboxcolor; my $mypagetitleboxcolor;
  if (Metadot::Utils::Theme->use_theme_info() ) {
      # get the color
      my $boxcolor = $PARAMS->{'theme_mypage_color'};
      $mypageelementboxcolor = $boxcolor;
      $mypagetitleboxcolor = $boxcolor;
  }
  else {
      $mypageelementboxcolor = $PARAMS->{'mypageelementboxcolor'};
      $mypagetitleboxcolor = $PARAMS->{'mypagetitleboxcolor'};
    }
  if (substr($mypageelementboxcolor,0,1) ne "#") {
      $mypageelementboxcolor = "#$mypageelementboxcolor";
  }
  if (substr($mypagetitleboxcolor,0,1) ne "#") {
      $mypagetitleboxcolor = "#$mypagetitleboxcolor";
  }
#     my $mypageelementboxcolor = $PARAMS->{'mypageelementboxcolor'};
#     my $mypagetitleboxcolor = $PARAMS->{'mypagetitleboxcolor'};

    $rs .= "<a href=\"index.pl?isa=Session&amp;op=forgot_password\">Forgot your password</a>?";

    $rs .= "</td></tr>";

    $rs .= "</table>";

    $rs .= "</td></tr>";
    $rs .= "<tr><td align='right'>";
    $rs .= "<INPUT type=submit name=action value='Login' class='button'>";
    $rs .= "</FORM>\n";
    $rs .= "</td></tr>";
    $rs .= "</table>";


    return "

                <TABLE CLASS='FRAME' CELLPADDING=\"2\" CELLSPACING=\"0\" border=\"0\" WIDTH=\"100%\" BGCOLOR='$mypageelementboxcolor'><TR><TD>

                <TABLE CLASS='FRAMETITLE' CELLPADDING=\"0\" CELLSPACING=\"0\" border=\"0\" WIDTH=\"100%\" BGCOLOR='$mypagetitleboxcolor'><TR>
                <TD HEIGHT=\"15\">
                <font class='FRAMETITLE'>Login</FONT>
                </TD></TR></TABLE>

                <TABLE CLASS='FRAMECONTENT' CELLPADDING=\"2\" CELLSPACING=\"0\" border=\"0\" WIDTH=\"100%\" BGCOLOR=white>
                <TR><TD align='center'>
                $rs
                </TD></TR>
                </TABLE>
                </TD></TR>
                <TR><TD HEIGHT=\"0\" BGCOLOR=white></TD></TR>
                </TABLE>
        ";

}


sub show_login_form {
    my $self = shift;
    my $email = shift;
    my $error_message = shift || undef;
    my $style = shift || undef;

    #print STDERR "\n\nshow_login_form() form target:$FORM{target} post_login:" . $PARAMS->{post_login} . " session_referer:" . $SESSION->referer() . " home_page:" . $PARAMS->{home_page} . "\n\n";

    #Remove HTML code from URL
    my $target = $FORM{target};
    $target =~ s/\%3A/:/g;
    $target =~ s/\%2F/\//g;

    ## will set form parameters depending on site's login mode
    my ($form_action, $login_mode, $login_submessage, $target) = $self->get_password_form_parameters(login_mode => $FORM{login_mode}, target => $target);

    #print STDERR "\n\nshow_login_form(): form_action:$form_action login_mode:$login_mode target:$target login_submessage:$login_submessage\n\n";

    my $rs = "<center><table border=\"0\" cellspacing=\"0\" cellpadding=\"0\">
    <tr><td id=\"tagLoginBox\" ><p /><FORM name=\"login_form\" action=\"$form_action\" method=\"POST\">\n";
    $rs .= "<input type='hidden' name='target' value='$target'>\n";
    $rs .= "<input type='hidden' name='isa' value='Session'>\n";
    $rs .= "<input type='hidden' name='op'  value='login_check'>\n";
    $rs .= "<br /><b><font size=\"+2\">Login</font></b><br /><br />";
#      $rs .=  $DISPLAY->show_title_bar("Login");

    if($error_message){
    $rs .= " <font color='red'>Sorry! Please enter an email address.</font><br /><br /> ";
    }

    if (!$PARAMS->{registration_closed}) {
        #$rs .= "<span class=\"$style\">Not a member yet? <a href=\"" . $DISPLAY->make_url("index.pl?isa=Register&amp;op=register_form")."\">Click here to register</a><br /><br /></span>\n";
        $rs .= "<span>Not a member yet? <a href=\"" . $DISPLAY->make_url("index.pl?isa=Register&amp;op=register_form")."\">Click here to register</a><br /><br /></span>\n";
    }

    #$rs .= "<table border=\"0\" cellspacing=\"1\">";
    $rs .= "<table class=\"$style\" border=\"0\" cellspacing=\"1\">";

    #$rs .= "<tr><td bgcolor=\"#000000\">";
    $rs .= "<tr><td>";

    $rs .= "<table border=\"0\" cellspacing=\"0\" cellpadding=\"5\">";
    #$rs .= "<tr bgcolor=\"#ffffff\"><td class=$style>";
    $rs .= "<tr><td>";

    $rs .= "<b>Email address:</b>";
    #$rs .= "</td><td class=$style>\n";
    $rs .= "</td><td>\n";
    $rs .= "<INPUT type=text name=email size=20 value='$email'>\n";
    $rs .= "</td></tr>\n";

    #$rs .= "<tr bgcolor=\"#ffffff\"><td class=$style>";
    $rs .= "<tr><td>";
    $rs .= "<b>Password:</b>";
    #$rs .= "</td><td class=$style>\n";
    $rs .= "</td><td>\n";

    $rs .= "<INPUT type=password name=password size=20></td></tr>\n";

    #$rs .= "<tr bgcolor=\"#ffffff\"><td>&nbsp;</td><td class=$style><a href=\"" . $DISPLAY->make_url("index.pl?isa=Session&amp;op=forgot_password"). "\">Forgot your password</a>?";
    $rs .= "<tr><td>&nbsp;</td><td><a href=\"" . $DISPLAY->make_url("index.pl?isa=Session&amp;op=forgot_password"). "\">Forgot your password</a>?";
    $rs .= "</td></tr>\n";

    $rs .= "</table>";

    $rs .= "</td></tr>\n";

    my $iloc = $PARAMS->{'imageslocation'};
    my $login_image1 = $login_mode eq 'standard' ? 'login2_01.gif' : 'login2_02.gif';
    my $login_image2 = $login_mode eq 'standard' ? 'login2_01-over.gif' : 'login2_02-over.gif';
    $rs .= "<tr><td colspan='2' align='center'><br />" .
  "<SCRIPT type=text/javascript>
  <!--
  function newImage(arg) {
      if (document.images) {
          rslt = new Image();
          rslt.src = arg;
          return rslt;
      }
  }

  var submit_image = newImage('$iloc/$login_image1') ;
  var submit_image_over = newImage('$iloc/$login_image2') ;
  //-->
  </SCRIPT>

  <input type=image border=\"0\" name=submit src=\"$iloc/$login_image1\"
   onMouseOut=\"this.src = submit_image.src\"
   onmouseover=\"this.src = submit_image_over.src\"
  >"
    . "</td></tr>\n";
        #. "<tr><td colspan='2' align='center'>$login_submessage</td></tr>\n";
#        $rs .= "</table><br />&nbsp;";
    $rs .= "</table>\n";
    $rs .= "</FORM></td></tr></table><p />$login_submessage</center>\n";

}


sub process_login_form {
    my $self = shift;

    my $target = ($FORM{target} !~ /logout/) ? $FORM{target} : 'index.pl';

    #if couldn't get target , go to start page
    if (!$target) {
        $target = 'index.pl';
    }

    ##make sure target is in correct format
    #print STDERR "\n\ntarget1:$target\n\n";
    $target = Utility->get_qualified_url($target);
    #print STDERR "\n\ntarget2:$target\n\n";

    $DISPLAY->print_http_header();

    unless( $FORM{email} ) {
        $SESSION->www_login("Sorry! You need to enter an email address");
        exit 0;
    }

    if ($FORM{form_action} ne 'forgot_pwd') {

        # need to answer to a login request
        if(!$FORM{password}) {
            $DISPLAY->print_click_back("Sorry!", "You need to enter a password");
            exit 0;
        }

        # needed to check approvalneeded in user info.
        my $u = Metadot::UserFactory->get_user( $SESSION->uid );
        #print STDERR "processing login for user $u\n";

        if ( !$SESSION->uid ) {
            #print STDERR "login failed\n";
            return $self->handle_failed_login(@_);

        } elsif ($u->approval_needed eq "required" ) {

            # if login and password are right but user isnt approved yet...
            $DISPLAY->print_click_back("Thank you!", "<b>Due to access controls, your account is awaiting administrator approval.<br />
                            You will be notified via email as soon as your account is approved.</b><br /><br />");
            # getting rid of session created during login attempt.
            $SESSION->erase;
            exit 0;

        } elsif ($u->approval_needed eq "rejected") {
            # if login has been rejected
            $DISPLAY->print_click_back("Sorry!", "<b>Your account request has been rejected.</b><br /><br />");
            #remove session created during login attempt.
            $SESSION->erase;
            exit 0;

        } else {
              ## check to see if password is expired
              ## only check for passwords expired if user_management is INTERNAL
              my $passwords_expire = $PARAMS->{password_expire};
              #print STDERR "check for expired passwords? '$passwords_expire'\n";

              if( $passwords_expire eq 'on' &&  !MetadotConfig->get_param_value('user_management_is_external') ){

                  if( $u->password_expired($u->uid()) ){
                     $target = Utility->get_qualified_url("index.pl?isa=Session&amp;op=change_password");

                     my $uid = $u->uid();
                     ## erase the created session when user was authenticated
                     $SESSION->erase;

                     my $reset_session     = $u->build_reset_session();
                     my $reset_session_key = $reset_session->{'session_id'};

                     $target = Utility->get_qualified_url("index.pl?isa=User&op=reset_password&key=$reset_session_key&refererlink=index.pl#password");

                     Utility->md_redirect($target);
                  }

              }
              if ($u->can('get_user_defined_homepage')) {
                  my $user_defined_homepage = $u->get_user_defined_homepage();
                  if ($user_defined_homepage) {
                      ## if user specified homepage
                      # CKIM: this was commented out between 5.7 and 6.0, don't know why.
                      $target = Utility->get_qualified_url($user_defined_homepage);
                  }
              }

              #print STDERR "Authorizing with $target\n";
              $self->authorize($target);
        }

    } else { #form_action was 'forgot_pwd'

        # need to answer to a 'mail my password' request
        my $u = Metadot::UserFactory->get_user();

        my $uid = $u->email_address_exists($FORM{email});

        # Check if the email corresponds to a member
        if(!$uid){
            $DISPLAY->print_click_back("Error","Sorry there is no member with the email address you provided: $FORM{email}");
            return 0;
        }


        # let's load user information
        $u = Metadot::UserFactory->get_user( $uid );

        # user not approved yet
        if ($u->approval_needed eq "rejected" ||
            $u->approval_needed eq "required") {
        $DISPLAY->print_click_back("Sorry!", "<b>You are not allowed to perform this operation .</b><br /><br />");
        return 0;
        }

        my $session_reset = $u->build_forgot_password_session();
        my $sessionid     = $session_reset->{'session_id'};

        # build message
        my $fullname = $u->webname();

        my $to=$FORM{email};
        my $cc="";
        my $bcc="";
        my $urgent="no";
        my $subject="Password information";

        my $basename = $PARAMS->{'basename'};

        my $msg="Dear $fullname,\n\n"
          ."You requested to have an email sent to you regarding your password information."
            ."\n\nTo enter a new password, click on the link: \n"
              ."$basename/index.pl?isa=Session&amp;op=auto_login&amp;new_user=$FORM{new_user}&amp;key=$sessionid\n\n\n\n"
            ."This link will be valid for 8 hours. After that time, you will need to submit "
              ."another request to receive a new link.\n" .
             "If you have trouble clicking on the URL above from within your mail client, cut and paste it\ndirectly into your browser.\n\n";

        $msg .= "If you didn't request this information, then someone else requested it for you "
          ."and you can just ignore this mail and delete it. Your account settings will "
            ."not be changed."
              ."\n\nThis message was generated automatically.\nThank you.\n\n"
            .">-> This email was sent to: $to\n\n\n\n";

        use Sendmail;

        my $sm = Sendmail->new($to, $cc, $bcc, $subject, $msg);
        $sm->set_from("noreply");
        my $result = $sm->sendMail();

        if($result){
            $DISPLAY->print_send_to_front("Mail Sent!", "An email has been sent to you at:<br /><br /><b>$FORM{email}</b><br /><br />Instructions on resetting your password are included in the message.");
        } else {
            ## No mail has been sent, there is a mail server problem
            $DISPLAY->print_click_back("Error", "Sorry there is a mail server problem! Please let us know this is happening.");
        }
    }

}

sub handle_failed_login {
    my $self = shift;
    $DISPLAY->print_click_back("Error", "<b>Sorry we could not identify you.</b><br /><br />
                You entered either a wrong email address or a wrong password.");
    return 0;
}

1;

__END__


=head1 NAME

Metadot::Authenticator::UserPassAuthenticator - Concrete subclass of Metadot::Authenticator
to provide standard user/password authentication.


=head1 SYNOPSIS

  my $authenticator = Metadot::AuthenticatorFactory->get_authenticator();

  ($USER, $SESSION) = $authenticator->authenticate();


=head1 DESCRIPTION


This concrete Authenticator provides standard user/password
authentication and it provides the additional service of a password
recovery mechanism through email. See Metadot::Authenticator for
explanation of each of its methods.
