###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
# $Id: X509Authenticator.pm,v 1.1.2.7 2006/09/25 18:15:03 mel Exp $
#  
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Authenticator::X509Authenticator;

use strict;

use Metadot::Authenticator::BaseAuthenticator;
use Metadot::Authenticator::UserPassAuthenticator;
use Metadot::Register::FlexRegister;
use Metadot::Utils::Theme;
use Portal;
use DBAccess;
use Metadotclass;

use Data::Dumper;

use Metadot qw(%FORM $USER $SESSION $DISPLAY $PARAMS $CGI);

use vars qw(@ISA);
@ISA=qw(Metadot::Authenticator::UserPassAuthenticator);

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};

    bless ($self, $class);
    return $self;
}

# Since the header variable is a constant we need to alter this check to keep from constantly creating connections
sub create_anonymous_session_and_user {
    my $self = shift;
    my $uid = undef;
    my $user = undef;

  # If the user is verified restore the user and keep the session persistent
   if($ENV{'SSL_CLIENT_VERIFY'} && $ENV{'SSL_CLIENT_S_DN_Email'}){
      $uid = Metadot::User::Default->get_uid_from_email($ENV{'SSL_CLIENT_S_DN_Email'});
      $user = Metadot::UserFactory->get_user($uid);
      
      my $session = Session->new();
      $session->add($uid);
      
      $user = $self->restore_user($session);
      my $session_handler = Metadot::SessionHandlerFactory->get_session_handler();
      $session_handler->persist_session( $session );
      
      
      return ($user, $session);
   
   # If not setup the default user
   } else {
   
    my $anonymous_uid = MetadotConfig->get_reserved_uid('ANONYMOUS_USER');

    my $session = Session->new();
    $session->add( $anonymous_uid );

    my $user = $self->restore_user( $session );
    $user->logout( $session );

    my $session_handler = Metadot::SessionHandlerFactory->get_session_handler();
    $session_handler->persist_session( $session );

    return ($user, $session);
  }
}

# Decide what to do based on what we're given / what state we're currently in
sub determine_action {
    my $self = shift;

   # Check if the user is verified and there is no session 
    if(($ENV{'SSL_CLIENT_VERIFY'}) && !$SESSION ) {
   	   return $self->SUPER::get_CREATE_SESSION;
    }
	
   # Check to see if the user is logged in
    if ( defined $FORM{op} && $FORM{op} eq 'login_check' && $FORM{isa} eq 'Session' ) {
        return $self->SUPER::get_CREATE_SESSION;
    }
	 
    return undef;
}

# Create the session
sub create_session {
    my $self = shift;
	my $email = $FORM{email};
    my $password = $FORM{password};
      
	my $uid = undef;

   # If verified create a session - the 'constant' problem is remedied by create_anonymous_session_and_user
   if($ENV{'SSL_CLIENT_VERIFY'} && $ENV{'SSL_CLIENT_S_DN_Email'}){
      $uid = Metadot::User::Default->get_uid_from_email($ENV{'SSL_CLIENT_S_DN_Email'});
   }

   # If the user is trying to login override the above variable with the user trying to login
   if ( defined $FORM{op} && $FORM{op} eq 'login_check' && $FORM{isa} eq 'Session' ) {
       $uid = $self->check_login($email, $password);
   } 
   
   
    my $session = Session->new();
	
    if(defined $uid){
      $session->add($uid);
    } else {
      $session->add();
    }
    
    return $session;
}

sub show_login_form {
    my $self = shift;
    my $email = shift;
    my $error_message = shift || undef;
    my $style = shift || undef;

    #print STDERR "\n\nshow_login_form() form target:$FORM{target} post_login:" . $PARAMS->{post_login} . " session_referer:" . $SESSION->referer() . " home_page:" . $PARAMS->{home_page} . "\n\n";

    #Remove HTML code from URL
    my $target = $FORM{target};
    $target =~ s/\%3A/:/g;
    $target =~ s/\%2F/\//g;

    ## will set form parameters depending on site's login mode
    my ($form_action, $login_mode, $login_submessage, $target) = $self->get_password_form_parameters(login_mode => $FORM{login_mode}, target => $target);

    #print STDERR "\n\nshow_login_form(): form_action:$form_action login_mode:$login_mode target:$target login_submessage:$login_submessage\n\n";

    my $rs = "<center><table border=\"0\" cellspacing=\"0\" cellpadding=\"0\">
    <tr><td id=\"tagLoginBox\" ><p /><FORM name=\"login_form\" action=\"$form_action\" method=\"POST\">\n";
    $rs .= "<input type='hidden' name='target' value='$target'>\n";
    $rs .= "<input type='hidden' name='isa' value='Session'>\n";
    $rs .= "<input type='hidden' name='op'  value='login_check'>\n";
    $rs .= "<br /><b><font size=\"+2\">Login</font></b><br /><br />";
#      $rs .=  $DISPLAY->show_title_bar("Login");

    if($error_message){
    $rs .= " <font color='red'>Sorry! Please enter an email address.</font><br /><br /> ";
    }

    if (!$PARAMS->{registration_closed}) {
        #$rs .= "<span class=\"$style\">Not a member yet? <a href=\"" . $DISPLAY->make_url("index.pl?isa=Register&amp;op=register_form")."\">Click here to register</a><br /><br /></span>\n";
        $rs .= "<span>Not a member yet? <a href=\"" . $DISPLAY->make_url("index.pl?isa=Register&amp;op=register_form")."\">Click here to register</a><br /><br /></span>\n";
    }

    #$rs .= "<table border=\"0\" cellspacing=\"1\">";
    $rs .= "<table class=\"$style\" border=\"0\" cellspacing=\"1\">";

    #$rs .= "<tr><td bgcolor=\"#000000\">";
    $rs .= "<tr><td>";

    $rs .= "<table border=\"0\" cellspacing=\"0\" cellpadding=\"5\">";
    #$rs .= "<tr bgcolor=\"#ffffff\"><td class=$style>";
    $rs .= "<tr><td>";

    $rs .= "<b>Email address:</b>";
    #$rs .= "</td><td class=$style>\n";
    $rs .= "</td><td>\n";
    $rs .= "<INPUT type=text name=email size=20 value='$email'>\n";
    $rs .= "</td></tr>\n";

    #$rs .= "<tr bgcolor=\"#ffffff\"><td class=$style>";
    $rs .= "<tr><td>";
    $rs .= "<b>Password:</b>";
    #$rs .= "</td><td class=$style>\n";
    $rs .= "</td><td>\n";

    $rs .= "<INPUT type=password name=password size=20></td></tr>\n";

    #$rs .= "<tr bgcolor=\"#ffffff\"><td>&nbsp;</td><td class=$style><a href=\"" . $DISPLAY->make_url("index.pl?isa=Session&amp;op=forgot_password"). "\">Forgot your password</a>?";
    $rs .= "<tr><td>&nbsp;</td><td><a href=\"" . $DISPLAY->make_url("index.pl?isa=Session&amp;op=forgot_password"). "\">Forgot your password</a>?";
    $rs .= "</td></tr>\n";

    $rs .= "</table>";

    $rs .= "</td></tr>\n";

    my $iloc = $PARAMS->{'imageslocation'};
    my $login_image1 = $login_mode eq 'standard' ? 'login2_01.gif' : 'login2_02.gif';
    my $login_image2 = $login_mode eq 'standard' ? 'login2_01-over.gif' : 'login2_02-over.gif';
    $rs .= "<tr><td colspan='2' align='center'><br />" .
  "<SCRIPT type=text/javascript>
  <!--
  function newImage(arg) {
      if (document.images) {
          rslt = new Image();
          rslt.src = arg;
          return rslt;
      }
  }

  var submit_image = newImage('$iloc/$login_image1') ;
  var submit_image_over = newImage('$iloc/$login_image2') ;
  //-->
  </SCRIPT>

  <input type=image border=\"0\" name=submit src=\"$iloc/$login_image1\"
   onMouseOut=\"this.src = submit_image.src\"
   onmouseover=\"this.src = submit_image_over.src\"
  >"
    . "</td></tr>\n";
        #. "<tr><td colspan='2' align='center'>$login_submessage</td></tr>\n";
#        $rs .= "</table><br />&nbsp;";
    $rs .= "</table>\n";
    $rs .= "</FORM></td></tr></table><p />$login_submessage</center>\n";

}

#sub get_modify_settings_form_field_values {
#   my $self = shift;
#   
#   return undef;   
#}

1;