###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::AuthenticatorFactory;
use strict;

use DBAccess;

#we store in this hash the different instantiated reg types.
#if we need a particular reg type, we first check to see
#if it is in this hash
my %authenticator_types;

sub get_authenticator {
    my $class = shift;

    my $auth_type = MetadotConfig->get_authenticator_type();

    if ($authenticator_types{$auth_type}) {

        return $authenticator_types{$auth_type};

    } else {

        require "Metadot/Authenticator/$auth_type.pm";

        my $auth = "Metadot::Authenticator::$auth_type"->new();

        $authenticator_types{$auth_type} = $auth;

        return $auth;
    }
}


1;


__END__


=head1 NAME

Metadot::AuthenticatorFactory - Middleman class to provide instances
of different Metadot::Authenticator subclasses depending on
configuration.

=head1 SYNOPSIS

  my $authenticator = Metadot::AuthenticatorFactory->get_authenticator();

  ($USER, $SESSION) = $authenticator->authenticate();


=head1 DESCRIPTION

Middleman class to provide instances of different
Metadot::Authenticator subclasses depending on configuration. This
class relies on method get_authenticator_type in MetadotConfig, which
reads in the value of the variable "authenticaor_type" in metadot.conf
(the config file), from where different authenticators can be
selected. Note that for backward compatibility authenticators will be
selected fromn the system config page Enable/Disable LDAP toggle
unless the variable disable_selecting_registration_modules_from_GUI is
set.


=head1 METHODS

get_authenticator

  Returns an instance of a Metadot::Authenticator subclass depending on system configuration.
