###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Metadot::ClassUtilities;

# a set of utility methods for working with classes

use strict;
no strict 'refs';

use Data::Dumper;
use Utility;

# returns a list of classes that a given class descends from
sub ancestor_hierarchy {
    my $class   = shift;
    my %args    = @_;

    #print STDERR "ancestor_hierarchy with args\n",Dumper \%args;

    my $thing_to_check = $args{'class'} || die("Must specify the class to check");
    my $base_class     = $args{'base_class'};
    my $visited        = $args{'visited'}; # prevents cycling
    my $inclusive      = $args{'inclusive'};

    my $class_to_check = ref($thing_to_check) || $thing_to_check;
    #print "thing_to_check '$thing_to_check'; class_to_check '$class_to_check'\n";

    unless (defined $visited) {
        $visited = { $class_to_check => 1 };
        $args{visited} = $visited;
    }

    my @ancestor_list = ();
    my $ref_name = "${class_to_check}\::ISA";

    #print STDERR "Checking for $ref_name\n";
    if (defined (@{$ref_name})) {
        foreach my $ancestor_class (@{$ref_name}) {
            if ((!defined($base_class) || $ancestor_class->isa($base_class)) && !$visited->{$ancestor_class}++) {
                push @ancestor_list, $class->ancestor_hierarchy(
                    'class'      => $ancestor_class,
                    'base_class' => $base_class,
                    'visited'    => $visited,
                    '_inner'     => 1,
                );

                #print STDERR "I have an ancestor: $ancestor_class\n";
                push @ancestor_list, $ancestor_class;
            } else {
                #print STDERR "Ignoring ancestor: $ancestor_class\n";
            }
        }
    }

    # add this class to the hierarchy if 'inclusive' requested
    if ($inclusive) {
        push @ancestor_list, $class_to_check;
    }

    # uniquify the list. this only needs to happen once
    unless ($args{_inner}) {
        @ancestor_list = Utility->uniquify_array(@ancestor_list)
    }

    return @ancestor_list;
}



# implements the Visitor pattern on a list of classes
sub visit {
    my $class = shift;
    my %args  = shift;

    my $class_list = $args{class_list} || die('Must specify a class_list');
    my $method     = $args{method}     || die('Must specify a method');
    my $args       = $args{args}       || [];

    foreach my $visitee (%$class_list) {
        if ($visitee->can($method)) {
            $visitee->$method(%$args);
        }
    }
}





1;
