###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Config;

use strict;
use FileHandle;
use Metadot::Config::IniFiles;

####
# Authors: Marcus Semanek & Michael Vogel
# Created: 06/08/01
# Purpose: To configure stuff
#
# <-- INSERT STANDARD DISCLAIMER HERE -->
#
# Copyright: Metadot, Inc.
#
# this example is no longer accurate.
# EXAMPLE CONF FILES:
#
# metadot.conf
# ----------------------------------------
# [metadot]
# file = /usr/metadot/another_metadot.conf
# servers = <<SERVERS
# metadot.com
# metadot.net
# www.metadot.com
# www.metadot.net
# 192.168.0.1
# SERVERS
#
# [localhost]
# file = /usr/metadot/localhost.conf
# servers = <<SERVERS
# localhost
# 127.0.0.1
# SERVERS
#
# another_metadot.conf
# -----------------------------------------
# [metadotdb]
# name = dbi:mysql:metadotdb
# user = metadot
# pass = changeme
#
# [metadotdb2]
# name = dbi:mysql:some_other_db
# user = metadot
# pass = me_too
#
#
# EXAMPLE CODE:
#
# my $config = new Metadot::Config;
# my $dbconfig = $config->get_db_info('metadot')
#
###
sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $server_name = shift;

    my $self = {};

    if ($server_name){
         $self->{server_name} = $server_name;
         $self->{age} = undef;
    }

    bless ($self, $class);

    $self->load_values();
    return $self;
}

sub get_conf_file {
    my $self = shift;
    my $num_confs = 0;
    my $first_section = undef;

    my $cfg = new Metadot::Config::IniFiles ( -file => "../etc/metadot.conf" );

    unless ($cfg) {
    print STDERR "Could not open config file!\n";
    }
    my $conf_file;

    my $server_name = $ENV{'HTTP_HOST'} || $self->{server_name};
#    print STDERR "In Metadot::Config::get_conf_file: using web server name: $server_name\n";

    foreach my $section ($cfg->Sections) {
        $first_section = ($num_confs)? $first_section : $section;
        $num_confs++;
    my @values = $cfg->val($section, 'servers');
    foreach my $line_in_the_server_list(@values) {
#        print STDERR "line:  $line_in_the_server_list" . "...";
        $line_in_the_server_list =~ s/ //g;
#        print STDERR "line:  $line_in_the_server_list" . "...";
        if ($_ eq $server_name) {
        $conf_file = $cfg->val($section, 'file');
        }
    }
    }

    if ($num_confs == 1){
        $conf_file = $cfg->val($first_section, 'file');
    }

    if ($conf_file) {
    return $conf_file;
    } else {
    return undef;
    }
}

sub load_values {
    my $self = shift;

    #my $conf_file = $self->get_conf_file || return undef;

    #my $cfg = new Metadot::Config::IniFiles ( -file => $conf_file );
    my $cfg = new Metadot::Config::IniFiles ( -file => "../etc/metadot.conf" );

    # XXX [vadim] hack for the shareM startup.pl script successful loading,
    # doesn't affect normal operation
    $cfg = new Metadot::Config::IniFiles ( -file => $ENV{'METADOT_CONF'} ) unless ($cfg);

    unless ($cfg) {
    print STDERR "Could not open config file: metadot.conf\n";
    }

    my $server_name = $self->{server_name} || $ENV{'HTTP_HOST'};
#    print STDERR "In Metadot::Config::load_values: using web server name: $server_name\n";

    my $config;
    my $sections;

    my ( $found, $localhost_config, $localhost_section );

    foreach my $section ($cfg->Sections) {
        my @server_vals = $cfg->val($section, 'servers');
        foreach my $line_in_the_server_list (@server_vals){
            $line_in_the_server_list =~ s/ //g;

            if (lc $line_in_the_server_list eq lc $server_name) {

        $found++;

            foreach my $paramater ($cfg->Parameters($section)) {
                $config->{$section}->{$paramater} = $cfg->val($section, $paramater);
            }

                $config->{$section}->{'servers'} = $server_name;

            } elsif( lc $line_in_the_server_list eq 'localhost' ) {

        # we save localhost params to use in case
        # the hostname does not have an entry in the config file
        $localhost_section = $section;

            foreach my $paramater ($cfg->Parameters($section)) {
                $localhost_config->{$section}->{$paramater} = $cfg->val($section, $paramater);
            }

                $localhost_config->{$section}->{'servers'} = 'localhost';
        }

        $self->{$line_in_the_server_list}->{'SECTION'} = $section;
        }
    }

    unless( defined $found ) {

    unless( $localhost_section ) {
        die "server name $server_name is not defined in metadot.conf and a section is not present for localhost"
    }

    $self->{$server_name}->{SECTION} = $localhost_section;
    $config->{$localhost_section}->{'servers'} = $server_name;

    $config->{$localhost_section} = $localhost_config->{$localhost_section};
    }


    $self->{'CONFIG'} = $config;

    return 1;
}

sub get_section_name {
    my ($self, $sv) = @_;

    my $section  = $self->{$sv}->{'SECTION'};
    if ($section) {
    return $section;
    }
    else {
    return "";
    }
}

sub get_log_file {
    my ($self, $sv) = @_;

    my $sections  = $self->{$sv}->{'SECTION'};

    if (defined $self->{'CONFIG'}->{$sections}) {
    return $self->{'CONFIG'}->{$sections}->{'log_file'};
    } else {
    print STDERR "I can't get config info for $sections log_file\n";
    return undef;
    }
}

sub get_section_info {
    my ($self, $sv) = @_;
    if (!$sv || (!$self->{$sv}->{'SECTION'})) {
        $sv = "localhost";
    }

    if (!$sv){ $sv = "localhost";}

    my $sections  = $self->{$sv}->{'SECTION'};
#      print STDERR "sv = $sv\n\n";

    if (defined $self->{'CONFIG'}->{$sections}) {
    return $self->{'CONFIG'}->{$sections};
    } else {
    print STDERR "I can't get config info for $sections\n";
    return undef;
    }
}

sub get_db_info {
    my ($self, $sv) = @_;
    return $self->get_section_info($sv);
}

sub get_all_config_sections {
    my $class = shift;

    my @sections;

    my $cfg = new Metadot::Config::IniFiles( -file => "../etc/metadot.conf" );

    unless ($cfg) {
        print STDERR "Could not open config file!\n";
    }

    foreach my $section ($cfg->Sections) {
        push(@sections, $section);
    }

    return (@sections);
}

sub get_all_server_names {
    my $class = shift;

    my $cfg = new Metadot::Config::IniFiles ( -file => "../etc/metadot.conf" );

    unless ($cfg) {
        print STDERR "Could not open config file!\n";
    }

    my $conf_file;
    my $num_confs = 0;
    my $first_section = undef;
    my @result;
    foreach my $section ($cfg->Sections) {
        $first_section = ($num_confs) ? $first_section : $section;
        $num_confs++;
        my @values = $cfg->val($section, 'servers');
        foreach (@values) {
            push @result, $_;
        }
    }

    return (@result);
}


sub get_console_server_name {
    my $self = shift;
    my $num_confs = 0;
    my $first_section = undef;

    my $console_cfg = new Metadot::Config::IniFiles ( -file => "../etc/console.conf" );

    unless ($console_cfg) {
        print STDERR "Could not open console config file!\n";
        return "";
    }

    my $section = ($console_cfg->Sections)[0];
    my $server_name = $console_cfg->val($section, 'server_name');
    return $server_name;
}


sub age {
    my $self = shift;
    (@_) ? return $self->{age} = shift : return $self->{age};
}



1;



__END__

=head1 NAME

Metadot::Config - A class that acts as an interface for the data in the Metadot config files.

=head1 SYNOPSIS

    use Metadot::Config;
    $md = Config->new ;

    $md->get_log_file ($sv);
    $md->get_db_info ($db);


=head1 DESCRIPTION

The Config class handles the interface loading and accessing the configuration information in the config files. The config files are located at metadot/etc/ and in metadot/etc/DBConfig.

This class uses the Config::IniFiles module from CPAN to handle the format the config files are in.

=head2 Methods

Constructor: instantiates a copy of the config object and loads the config data.

    $md = new Config;
    or
    $md = Config->new();

Get the log file to write metadot debug information to:
B<Note: $sv is the Metadot server name under which you wish to look for the log file.>

    $log_file = $md->get_log_file($sv);

Get the database connection information for a specific database.
B<Note: $db is the name of the database for which you want the connection information.>

    $get_db_info($DB);

=cut
