###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::ConfigParser;
use strict;

use Metadot::XML::Parser::Lite;


my $metadot_conf;
my $current_server;
my @current_tag;

my $start_sub = sub {
    shift;
    my $tag_id = shift;

    push @current_tag, $tag_id;

    if(lc $tag_id eq 'server') {
    my %attr = (@_);

    unless(exists $attr{name}) {
        die "config file must contain server name";
    }
    $current_server = $attr{name};
    $metadot_conf->{SERVER}{$current_server} = {};
    }
};

my $char_sub = sub {
    shift;
    my $text = shift;
    my $current_tag = $current_tag[$#current_tag];

    if(lc $current_tag eq 'name') {
    $metadot_conf->{SERVER}{$current_server}{DATABASE}{NAME} = $text;
    }

    if(lc $current_tag eq 'user') {
    $metadot_conf->{SERVER}{$current_server}{DATABASE}{USER} = $text;
    }

    if(lc $current_tag eq 'password') {
    $metadot_conf->{SERVER}{$current_server}{DATABASE}{PASSWORD} = $text;
    }
};

my $end_sub = sub {

    pop @current_tag;

};



sub parse_file {
    my $class = shift;
    my $file_name = shift || die "must include filename";

    open CONFIG, "$file_name" || die "Couldn't open configuration file: $file_name: $!";
    my @config_file = <CONFIG>;
    close CONFIG;
    my $config_xml = join '', @config_file;

    my $parser = Metadot::XML::Parser::Lite->new;

    $parser->setHandlers(
             Start => $start_sub,
             Char => $char_sub,
             End => $end_sub,
            );

    $parser->parse($config_xml);

    return $metadot_conf;
}


1;


__END__

=head1 NAME

Metadot::ConfigParser - A class for handling the XML based config files of Metadot. (deprecated)

=head1 SYNOPSIS

    use Metadot::ConfigParser;
    ConfigParser->parse_file ($file_name);


=head1 DESCRIPTION

This file was for use with an XML based config file system and is now deprecated.
Use Metadot::Config instead.

=cut
