###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::CyclingContainer;

# an interface to implement cycling
use Metadot qw($SESSION %FORM $DISPLAY);
use Metadot::AbstractGizmoContainer;
use Utility;

use vars qw(@ISA);

@ISA = qw(Metadot::AbstractGizmoContainer);

use constant ELEMENT_INDEX_SESSION_KEY => '_CyclingContainerElementIndex';
use constant PROPERTIES_FORM_KEY => '_CyclingContainerNumber';


# the method used to read and write the cycling number
sub number_of_elements_to_display {
    die ('Abstract method');
}


# determine the next index. send it a child list to save some work
sub next_element_index {
    my $self = shift;
    my $child_list = shift;
    my $update_counter = shift;

    my $result;

    $update_counter = 1 unless (defined($update_counter));


    # avoid this whole exercise if I don't do cycling
    my $number_of_elements_to_display = $self->number_of_elements_to_display();
    if (defined $number_of_elements_to_display && $number_of_elements_to_display > 0) {
        my $list_size;
        unless (defined $child_list) {
            $child_list = $self->get_children();
        }

        $list_size = scalar(@$child_list);

        if ($list_size < 1) {
            $result = undef;
        } else {
            $result = $SESSION->get_value($self->_generate_session_key());

            # if there was no result, compute one randomly
            if (!defined $result) {
                $result = int(rand($list_size));
            }

            # calculate the next index and store it in the session.
            if ($update_counter) {
                my $next_value = $result + $self->number_of_elements_to_display();
                $SESSION->set_value($self->_generate_session_key(), $next_value);
            }
        }

        # wrap at end. I do this here and not to next value because the
        # list may grow before the next page load.
        if ($list_size > 0) {
            $result = $result % $list_size;
        }
    }

    $result = 0 unless Utility->is_valid_integer($result);

    return $result;
}


sub _generate_session_key {
    my $self = shift;
    return ELEMENT_INDEX_SESSION_KEY().'_'.$self->id();
}


# typically called within show_edit_properties_form

sub show_cycling_properties {
    my $self = shift;

    my $number_of_elements_to_display = $self->number_of_elements_to_display();
    my $form_field = PROPERTIES_FORM_KEY();

    my $out = "<h1>Content Cycling</h2>\n";
    $out .= <<EOM
<b>Number of gizmos to display at once:</b>&nbsp;<input type="text" name="$form_field" size="3" value="$number_of_elements_to_display" />
&nbsp;<i>(Leave blank to display all gizmos)</i>
<br />
This container will cycle through its content, displaying a subset of its
gizmos each time. Typical values are 0 or 1.<br />
EOM
;
    return $out;
}


sub process_cycling_properties {
    my $self  = shift;
    my $input = shift || \%FORM;

    my $value = $input->{PROPERTIES_FORM_KEY()};
    $value = undef unless Utility->is_valid_integer($value);

    $self->number_of_elements_to_display($value);

    $DISPLAY->debug_msg(5, "process_cycling_properties found value $value, saved ".$self->number_of_elements_to_display());
}









1;

