###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::DataCollection;
use strict;

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    $self->{data_objects} = [];
    $self->{current_position} = 0;
    bless ($self, $class);
    return $self;
}

sub add {
    my $self = shift;
    push @{$self->{data_objects}}, @_;
}

sub addAll {
    my ($self, $collection) = @_;
    $collection->restart();
    while (my $instance = $collection->get_next()) {
        $self->add($instance);
    }
}

sub pop {
    my $self = shift;
    return pop @{$self->{data_objects}};
}

sub get_next {
    my $self = shift;
    my $return_value;
    if ($self->{current_position} >= $self->data_array_length()) {
        $return_value = undef;
    } else {
        $return_value = @{$self->{data_objects}}[($self->{current_position})];
        ($self->{current_position})++;
    }
    return $return_value;
}

sub data_array_length {
    my $self = shift;
    return 0 unless $self->{data_objects};
    return scalar @{$self->{data_objects}};
}

sub array {
    my $self = shift;
    return @{$self->{data_objects}};
}

sub restart {
    my $self = shift;
    $self->{current_position} = 0;
    return $self;
}

sub save {
    my $self = shift;
    $self->restart();
    while (my $instance = $self->get_next()) {
        my $isa = ref $instance;
        eval {
            $instance->save();
        };
    }
    return 1;
}

sub delete {
    my $self = shift;
    $self->restart();
    while (my $instance = $self->get_next()) {
        eval {
            $instance->delete();
        };
    }
    return 1;
}

# try to set value for all objects in the collection
sub set_value {
    my ($self, $name, $value) = @_;
    while (my $instance = $self->get_next()) {
        eval {
            $instance->set_value($name, $value);
        };
    }
    return 1;
}

1;
