###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Extensions;

use Data::Dumper;

my %registry = ();

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $self = {};

    bless ($self, $class);
    return $self;
}

sub get_registry {
    return \%registry;
}

sub register_extension {
    my ($self, $extended_method) = @_;
    $registry{$extended_method} = {};
}

sub _get_extensions {
    my ($self, $extended_method) = @_;
    my $extensions;

    # make sure the extension exists
    if (defined($registry{$extended_method})) {
        $extensions = $registry{$extended_method};
    }
    # coerce this to a hash
    if (ref($extensions) ne "HASH") {
        $extensions = %{$extensions};
    }
    return $extensions;
}

sub _set_extensions {
    my ($self, $extended_method, $extensions) = @_;
    $registry{$extended_method} = $extensions;
}

sub register_extender {
    my ($self, $extended_method, %params) = @_;
    # valid params
    # extender_method => name of the extender method, if different than the name of the method being extended

    my $extensions = $self->_get_extensions($extended_method);

    # don't register the extender unless the extension has been defined
    if ($extensions) {
        my ($extending_class ) = caller;

        $extensions->{$extending_class} = $params{extender_method} || $extended_method;
        $self->_set_extensions($extended_method, $extensions);
    }
}

sub process {
    my ($self, $extended_method, $calling_object, $params, $results) = @_;
    my ($extended_class) = caller;
    my %extensions = %{$self->_get_extensions($extended_method)};

    # process the extension against all extending classes
    my @classes = keys(%extensions);
    foreach $extending_class (@classes) {
        my $extending_method = $extensions{$extending_class};
        $results = $extending_class->$extending_method($calling_object, $params, $results);
    }

    # finally, return the modified StateBall
    return $results;
}


1;

__END__

