###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################


# ContentFilter's purpose is to take a list of instances and apply Filters
# against the list. The filters are set using add_filter() and set_local_filters().
# Filtering action is performed with filter().

package Metadot::Filters::ContentFilter;

use strict;
use vars qw(@ISA);

@ISA=qw();

# class variables ________________________________________________________
# base filters are always run, and can't be omitted
my @base_filters = ();

####################################
sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $self = {};
    $self->{local_filters} = [];

    bless ($self, $class);
    return $self;
}

###################################

# accesses the base filters, that cannot ever be omitted
sub get_base_filters {
    my $self = shift;
    return @base_filters;
}

# accesses the local filters, which are specific to this instance
sub get_local_filters {
    my $self = shift;
    return @{$self->{local_filters}};
}

# gets the list of all filters which will be used by this instance,
# including base_filters and local_filters
sub get_filters {
    my $self = shift;
    return ($self->get_base_filters(), $self->get_local_filters());
}

# sets the list of local_filters to the list passed as arguments
sub set_local_filters {
    my ($self, @filters) = @_;
    $self->{local_filters} = \@filters;
}

# appends the list of filters passed to this method to the list of
# local filters
sub add_filter {
    my ($self, @filters) = @_;
    $self->set_local_filters( $self->get_local_filters(), @filters );
}

# applies the content filters against the content provided
sub filter {
    my $self = shift;
    my @instances = @_;

    # apply each filter to each instance
    my @filters = $self->get_filters();
    foreach my $filter (@filters) {
        # short-circuit if there are no more instances
        last unless(scalar(@instances));
        @instances = $filter->filter(@instances);
    }

    return @instances;
}

1;
