package Metadot::Form3000::Fields::AbstractSelectionField;

use Metadot::Form3000::Fields::BaseFormField;
use Metadot qw($DISPLAY);

use strict;
use vars qw(@ISA);

@ISA = qw(Metadot::Form3000::Fields::BaseFormField);


use constant MO_INSTANCE_PROPERTIES => {
    select_options  => undef,
};


# given a stored value, returns the label
sub get_label_for_value {
    my $self  = shift;
    my $value = shift;

    my $result = undef;

    my $select_options = $self->select_options();
    if (defined $select_options) {
        my @options = @$select_options;
        while (@options) {
            my $option_value = shift @options;
            my $option_label = shift @options;

            if ($option_value eq $value) {
                $result = $option_label;
                last;
            }
        }
    }

    return $result;
}


sub select_options {
    my $self  = shift;

    if (@_)
    {
        my $value = shift;

        if (defined $value)
        {
            # unflatten the select options if they are flat
            if (!ref $value) {
                $DISPLAY->debug_msg(5,"select_options must unflatten '$value'");
                $value = $self->unflatten_select_options($value);
            }
        }
        $self->_select_options($value);
    }
    else
    {
        return $self->_select_options();
    }

}


# returns a true if the option_value matches the field_value.
# if field_value is an array, it will scan the array
sub value_is_selected
{
    my ($class, $option_value, $field_value) = @_;
    my $result;

    if (UNIVERSAL::isa($field_value, 'ARRAY'))
    {
        $result = Utility->array_contains($option_value, $field_value);
    }
    else
    {
        $result = ($field_value eq $option_value);
    }

    $DISPLAY->debug_msg(5, "value_is_selected($option_value, $field_value) returns $result");
    return $result;
}

use constant OPTION_VALUE_SEPARATOR => ' ';
sub flatten_select_options {
    my $self    = shift;
    my $options = shift || $self->select_options() || [];
    my $separator = shift || OPTION_VALUE_SEPARATOR();

    my $flat;

    my ($value, $label);

    my @option_list = @$options;
    while (@$options) {
        $value = shift @$options;
        $label = shift @$options;

        if (defined $value) {
            $flat .= $value . $separator . (defined $label ? $label : $value) . "\n";
        }
    }
    # remove trailing \n
    chomp($flat);

    return $flat;
}


sub unflatten_select_options {
    my $self  = shift;
    my $flat  = shift || $self->select_options();
    my $separator = shift || OPTION_VALUE_SEPARATOR();

    my @select_options = ();
    my @lines = split("\n", $flat);
    foreach my $line (@lines) {
        my ($value, $label) = split(/\Q$separator\E/, $line, 2);

        $value = Utility->trim($value);
        $label = Utility->trim($label);

        # add the option if we have something
        if (defined $value) {

           # in case we only got one value
            unless (defined $label) {
                $label = $value;
            }

            push @select_options, ($value, $label);
        }
    }

    return \@select_options;
}



1;
