#######################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Metadot::Form3000::Fields::BaseFormField;

use strict;

use MetadotObject;
use Metadot qw($DISPLAY);
use Metadot::Form3000::Fields::FormField;

use vars qw(@ISA);
@ISA = qw(Metadot::Form3000::Fields::FormField);


# Renders the form field as HTML. For backwards compatibility with
# Model 1 code
# Args:
# - value: Optional, tells the field what value to use
# - input: Optional, a hashtable containing key/value pairs. If no value is
#          provided, the form field will look for a key with the same name
#          as the field to use as its value

sub render_html {
    my $self   = shift;
    my $args   = shift || {};

    my $form_object = $args->{form_object};
    my $values      = $args->{values};

    if (!defined $values && defined $form_object) {
        $values = $form_object->values();
    }

    my $field_value = $values->{$self->form_key()};
    if (!defined $field_value) {
        $field_value = $self->default_value();
    }

    my $result = '';
    if (defined $self->template()) {
        my $tt   = Portal->get_portal_template_object();

        my $vars = {
            %$args,
            field       => $self,
            form_values => $values,
            field_value => $field_value,
            };

        $tt->process($self->template(), $vars, \$result);
        if ($tt->error()) {
            $DISPLAY->debug_msg(0, "Error processing template:\n".$tt->error());
        }

    } else {
        $result = "$self must either define a template or override render_html";
    }

    return $result;
}


# Processes form input.
# Args:
# - value: Optional, tells the input processor what value to use
# - input: Optional, a hashtable containing key/value pairs. If no value is
#          provided, the processor will look for a key with the same name
#          as the field to use as its value
# - field: Optional, tells the input processor what the field is. If not
#          specified, we assume that the field and input processor are the
#          same object

sub process_input {
    my $self = shift;
    my $args = shift || {};

    my $input = $args->{input} || {};
    my $field = $args->{field} || $self;
    my $value = $self->get_raw_input($input);

    if (!defined $value) {
        $value = $field->default_value();
    }

    return $value;
}


# returns the raw, unprocessed, unvalidated input
sub get_raw_input {
    my $self  = shift;
    my $input = shift || die('Must provide input');
    my $key   = shift || $self->form_key();

    my $result = $input->{$key};
    return $result;
}

# Processes form input.
# Args:
# - value: Optional, tells the field what value to use
# - input: Optional, a hashtable containing key/value pairs. If no value is
#          provided, the processor will look for a key with the same name
#          as the field to use as its value
# - field: Optional, tells the input processor what the field is. If not
#          specified, we assume that the field and input processor are the
#          same object

sub validate_input {
    my $self = shift;
    my $args = shift || {};

    my $input = $args->{input} || {};
    my $value = $args->{value};
    my $field = $args->{field} || $self;

    if (!defined $value) {
        $value = $self->get_raw_input($input);
    }
    if (!defined $value) {
        $value = $field->default_value();
    }

    my $error = undef;
    if ($field->is_mandatory() && (!defined($value) || $value eq '')) {
        $error = $field->label().' is mandatory.';
    }

    $DISPLAY->debug_msg(5, "Validated $self ("
                        .($self->label() || '')
                        ."), mandatory? '".($self->is_mandatory() || '')
                        ."', value='".($value||'')."'; got '".($error||'')."'");

    return $error;
}


# label falls back to name
sub label {
    my $self = shift;
    @_ ? $self->SUPER::label(@_) : $self->SUPER::label() || $self->name();
}


# the key value that this form checks for input. By default, this
# is the "name" property
sub form_key {
    my $self = shift;
    return $self->name();
    #return $self->name() || die("$self has no form_key!");
}





1;





