#######################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

# $Id: FormField.pm,v 1.3.2.2 2005/07/01 22:41:28 nico Exp $

package Metadot::Form3000::Fields::FormField;

use strict;

use MetadotObject;
use Metadot qw($DISPLAY);

use vars qw(@ISA);
@ISA = qw(MetadotObject);


use constant MO_INSTANCE_PROPERTIES => {
    name            => undef,   # the internal name of the field
    label           => undef,   # the label to display, otherwise the name is used
    default_value   => undef,   # the default value that will be used at all times
    is_mandatory    => undef,   # indicates whether the form field is mandatory
    heading_text    => undef,   # text that is displayed as a heading above the field
    inline_help     => undef,   # help text to be displayed along with this field
    template        => undef,   # the template to use when displaying this field
    input_processor => undef,   # the object or class that is responsible for
                                # processing input
    input_validator => undef,   # the object or class that is responsible for
                                # validating input
    property_mapping => undef,  # the PropertyMapping object which created this
                                # instance
};



# renders the form field as HTML
# Args:
# - value: Optional, tells the field what value to use
# - input: Optional, a hashtable containing key/value pairs. If no value is
#          provided, the form field will look for a key with the same name
#          as the field to use as its value

sub render_html {
    my $self  = shift;
    my $args  = shift || {};

    die('Abstract method');
}


# Processes form input.
# Args:
# - value: Optional, tells the field what value to use
# - input: Optional, a hashtable containing key/value pairs. If no value is
#          provided, the form field will look for a key with the same name
#          as the field to use as its value

sub process_input {
    my $self = shift;
    my $args = shift || {};

    die('Abstract method');
}


# the key value that this form checks for input. By default, this
# is the "name" property
sub form_key {
    my $self = shift;
    die('Abstract method');
}



# The object which is responsible for validating input. By
# default, that is the FormField itself.
sub input_processor {
    my $self    = shift;
    my $result;

    if (@_) {
        $result = $self->_input_processor(@_);
    } else {
        my $object  = $self->_input_processor();

        if (ref($object)) {
            $result = $object;
        } elsif(defined $object) {
            $result = $object->new();
        } else {
            $result = $self;
        }
    }
    return $result;
}

# The object which is responsible for validating input. By
# default, that is the FormField itself.
sub input_validator {
    my $self = shift;
    my $result;

    if (@_) {
        $result = $self->_input_validator(@_);
    } else {
        my $object  = $self->_input_validator();

        if (ref($object)) {
            $result = $object;
        } elsif(defined $object) {
            $result = $object->new();
        } else {
            $result = $self;
        }
    }
    return $result;
}




1;





