# Builds a Form3000::FormField object as needed

package Metadot::Form3000::Fields::FormFieldFactory;

use strict;
use MetadotObject;
use Utility;
use Metadot qw($DISPLAY);

use constant DEFAULT_MAPPINGS => {
    simple_text => 'Metadot::Form3000::Fields::SimpleTextField',
    select_list => 'Metadot::Form3000::Fields::SelectListField',
    select_box  => 'Metadot::Form3000::Fields::SelectBoxField',
    hidden      => 'Metadot::Form3000::Fields::HiddenField',
    radio       => 'Metadot::Form3000::Fields::RadioField',
    password    => 'Metadot::Form3000::Fields::PasswordField',
    file        => 'Metadot::Form3000::Fields::FileField',
    textarea    => 'Metadot::Form3000::Fields::TextAreaField',
    htmlarea    => 'Metadot::Form3000::Fields::HTMLAreaField',
    checkbox    => 'Metadot::Form3000::Fields::CheckboxField',
    predefined  => 'Metadot::Form3000::Fields::PredefinedFormField',

    # for backwards compatibility w/ FlexUser fields
    select      => 'Metadot::Form3000::Fields::SelectListField',
    user_upload => 'Metadot::Form3000::Fields::FileField',        
};

# initialize all the default mappings
{
    foreach my $classname (values(%{DEFAULT_MAPPINGS()})) {
        require_class($classname);
    }
}


# require a class by name. converts the class name to a file name
# and then attempts to run the require function.
sub require_class {
    my $classname = shift;
    my $filename = Utility->package_to_filename($classname);
    
    eval {
        require $filename;
    };
    if ($@) {
        die("FormFieldFactory encountered an error evaluating '$classname'\n",$@);
    }
}


# the factory method
sub get_instance {
    my $class = shift;
    my $args  = MetadotObject->normalize_constructor_args(@_);

    my $target_class = $class->get_mapped_class($args->{type});
    require_class($target_class);

    my $result = $target_class->new($args);
    return $result;
}


# given a mapping name, returns the Metadot::Form3000::Fields::FormField class
sub get_mapped_class {
    my $class = shift;
    my $mapping_name = shift || '';

    return DEFAULT_MAPPINGS()->{$mapping_name} || $mapping_name || 'Metadot::Form3000::Fields::SimpleTextField';
}







1;





