###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Form3000::PropertyMapping;

# Maps a Gizmo3000 property to a FormField object

use strict;
use MetadotObject;
use Metadot qw($DISPLAY);
use Metadot::Form3000::PropertySettings::PropertySettingFactory;

use vars qw(@ISA);
@ISA = qw(MetadotObject);


use constant MO_INSTANCE_PROPERTIES => {
    property_name       => undef,   # the external name for this field mapping
    mapped_property     => undef,   # the internal Gizmo3000 property used to store this
                                    # property
    display_order       => undef,   # the order in which this field should be displayed
    label               => undef,   # the user-friendly label for this property
    field_definition    => undef,   # the definition of form field this property prefers
    is_searchable       => 1,       # defines whether the property is searchable
    display_in_search   => undef,   # defines whether the property is displayed
                                    # in a list of search results
};


# generate a form field based on the PropertyMapping object.
sub generate_form_field {
    my $self = shift;
    my $args = shift || {};
    my $result;

    if (defined $self->field_definition()) {
        $result = Metadot::Form3000::Fields::FormFieldFactory->get_instance(
                $self->field_definition()
        );

        # associate the new form field with the instance that created it
        $result->property_mapping($self);
    }

    return $result;
}


sub get_property_settings_control {
    my $self = shift;
    return Metadot::Form3000::PropertySettings::PropertySettingFactory->get_instance({
        property_mapping=>$self
    });
}




1;



