###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::FormCalPopup;

use Metadot qw( $PARAMS );
#========================================

# FormCalPopup is a javascript-lib-specific helper library.  It
# generates HTML to support a popup calendar window for a given
# element descriptor object, compatible with specific javascript to
# implement the popups.  (see below for details).  If the javascript
# lib changes, the methods in this class must be modified also.

# tbd: parameterize all the javascript info so that a change in the
# javascript lib is not tied to a change in the perl code.

# The element descriptor obj passes the following info to the methods
# below: the name of the input element (that is, the html element's
# 'name' attribute), and the name of the form.  Both are necessary in
# building javascript fcn calls that work properly.  The formname is
# needed for pages in which > 1 form is present.

# The methods below are specific to a particular javascript library.
# This class assumes that the following javascript lib:
#   http://www.mattkruse.com/javascript/anchorposition/AnchorPosition.js
# and its helper libraries:
#   http://www.mattkruse.com/javascript/popupwindow/PopupWindow.js
#   http://www.mattkruse.com/javascript/calendarpopup/CalendarPopup.js
# will have been loaded by the html page in which the popups will be
# used.

# The methods below also assume that the following javascript function
# definitions are present in the html page.  If they are not, this
# code won't work. (In the context of the FormBuilder functionality,
# it is the page templates that have this code included; see
# e.g. <metadot>/metadot/Metadot/templates/fb_msg.tt2 for an example.


#  var selectedInput; var cal1 = new CalendarPopup();
#  cal1.setReturnFunction("selectDate");
#  var cal2 = new CalendarPopup();
#  cal2.setReturnFunction("selectDateTime");

#  // format expected by db insert
#  function selectDate(y,m,d) {
#      if (m < 10) {
#   m = "0"+ m;
#      }
#      if (d < 10) {
#   d = "0" + d;
#      }
#      if (selectedInput!="") {
#   selectedInput.value=y + "-" + m + "-" + d;
#      }
#  }

#  // format expected by db insert
#  function selectDateTime(y,m,d) {
#      if (m < 10) {
#   m = "0"+ m;
#      }
#      if (d < 10) {
#   d = "0" + d;
#      }
#      if (selectedInput!="") {
#   selectedInput.value=y + "-" + m + "-" + d + " 00:00:00";
#      }
#  }

#  function getDate(input,linkname) {
#      selectedInput=input;
#      cal1.showCalendar(linkname);
#  }

#  function getDateTime(input,linkname) {
#      selectedInput=input;
#      cal2.showCalendar(linkname);
#  }


#========================================

use strict;

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};

    $self->{_DATE_IMAGE} = "<img border=0 align=middle height=27 width=25 src=\"" . $PARAMS->{imageslocation} . "/date2.gif\">";

    bless ($self, $class);
    return $self;
}


# construct a string that will include an image that when clicked on will generate
# a calendar popup.  For this to work, requires that the form has the
# compatible calendar javascript in it:
# http://www.mattkruse.com/javascript/calendarpopup/CalendarPopup.js.
# and its helper scripts.
# if the javascript changes, this method and the one below need to change.
# The javascript determines the date FORMAT and should be
# appopriate to the date format used by the application.
sub getCalendarPopup {
    my ($self, $name, $formname) = @_;

    return "<a href=\"#\" onClick=\"getDate(document.forms." . $formname . ".$name" .
        ",'" . $name . "cal'); return false;\" NAME=\"" .
#       $name. "cal\" ID=\"" . $name . "cal\"><img border=0 src=\"/images/calendar.gif\"</a>";
        $name. "cal\" ID=\"" . $name . "cal\">" . $self->{_DATE_IMAGE} . "</a>";
}

sub get_calendar_popup_scripts {
    my $self = shift;
    my %args = @_;

  return "<script type=\"text/javascript\" src=\"/js/CalendarPopup.js\"></script>
<script type=\"text/javascript\" src=\"/js/AnchorPosition.js\"></script>
<script type=\"text/javascript\" src=\"/js/PopupWindow.js\"></script>
<script type=\"text/javascript\" src=\"/js/date.js\"></script>\n";
}

sub get_calendar_popup {
    my $self = shift;
    my %args = @_;

    my $name        = $args{name};
    my $value       = $args{value};
    my $date_format = $args{date_format} || 'yyyy-MM-dd';
    my $cal_name    = "cal_$name";
    my $anchor_name = "anchor_$name";
    my $div_name    = "div_name";
    my $form_name   = $args{form_name} || 'properties_form';

    my $s = '';

    $s .= "<script type=\"text/javascript\">var $cal_name = new CalendarPopup(\"$div_name\"); </script>\n";

    $s .= "<input type=\"text\" name=\"".($name||'')."\" value=\"".(defined($value) ? $value : '')."\" size=\"25\"> ";
    $s .= "<a href=\"#\" onclick=\"$cal_name.select(document.$form_name.$name,'$anchor_name','$date_format'); return false;\" title=\"$cal_name.select(document.$form_name.$name,'$anchor_name','$date_format'); return false;\" name=\"$anchor_name\" id=\"$anchor_name\">" . $self->{_DATE_IMAGE} . "</a>\n";

     $s .= "<DIV ID=\"$div_name\" STYLE=\"position:absolute;visibility:hidden;background-color:white;layer-background-color:white;\"></DIV>";


     return $s;
}

# same as above, but builds a popup that will generate a default time string as well as
# the date selected.
# For this to work, requires that the form has the
# compatible calendar javascript in it:
# http://www.mattkruse.com/javascript/calendarpopup/CalendarPopup.js
# and its helper scripts.
# if the javascript changes, this method and the one below need to change.
# The javascript determines the date FORMAT and should be
# appopriate to the date format used by the application.
sub getCalendarPopupT {
    my ($self, $name, $formname) = @_;

    return "<a href=\"#\" onClick=\"getDateTime(document.forms." . $formname . ".$name" .
        ",'" . $name . "cal'); return false;\" NAME=\"" .
        $name. "cal\" ID=\"" . $name . "cal\">" .
            $self->{_DATE_IMAGE} . "</a>";
}


1;
