###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::FormProcess;

use strict;
use Template;
use Data::Dumper;
#use CGI;
#use XML::Simple;

use MetadotConfig;
use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT);
use DBAccess;
use Metadot::FormProcessUID;
use Metadot::TableInfoFactory;
use ErrorHandling;


#assumes form contains necessary bookkeeping info in hidden fields...

use constant SEPARATOR => "__";

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};

    bless ($self, $class);
    return $self;
}

sub saveAllTableFieldValues {
    my ($self) = @_;

    #print STDERR "all query info: ", Dumper(\%FORM), "\n";
    # get the necessary info about the tables and primary keys used.
    my $mode = $FORM{'mode'};
    my $tableref = $FORM{'table'};
    my @tables; my @prim_keys;
    if (ref($tableref) eq 'ARRAY') {
    @tables = @$tableref;
    }
    else {
    @tables = ($tableref);
    }
    my $pkeyref = $FORM{'primary_key'};
    if (ref($pkeyref) eq 'ARRAY') {
    @prim_keys = @$pkeyref;
    }
    else {
    @prim_keys = ($pkeyref);
    }
    #print STDERR "got primary keys: ", Dumper(\@prim_keys), "and tables: ",
    #  Dumper(\@tables), "\n";
    foreach my $table (@tables) {
    my $mpkey = "";
    foreach my $pkey (@prim_keys) {
        if ($pkey =~ /^$table\./) {
        $mpkey = $pkey;
        last;
        }
    }
    # convert to format that will be used for accessing hidden var info about
    # the primary key values
    my $sep = SEPARATOR;
#   print STDERR "got primary keystring $mpkey for $table\n";
    $mpkey =~ s/\./$sep/g;
#   print STDERR "got primary keystring $mpkey for $table\n";
    my $pkval = "";
    if (lc $mode eq 'edit') {
        $pkval = $FORM{$mpkey};
        #print STDERR "got primary key value for $mpkey of $pkval\n";
    }
    $self->saveTableFieldValues($table, $mode, $mpkey, $pkval);
    }
}


sub saveTableFieldValues {
    my ($self, $table, $mode, $prim_key, $pkval) = @_;

    #find all the query params holding info for that
    #table, make a big attr/value hash of the info.
    #handle single values only ... need to write special-purpose method
    #to handle insertion of multiple values into a database table 'object'.
#      print STDERR "in saveTableFieldValues with table $table\n";
    my $param_value;
    my $tablehash = {};
    my $prim_key_table;
#      print STDERR "in saveTableFieldValues with prim_key $prim_key\n";
    my $sep = SEPARATOR;
    if ($prim_key =~/^$table($sep)(.*)/) {
    $prim_key_table = $2;
    }
    else {
#   die "error getting actual primary key string from $prim_key\n";
    ErrorHandling->printError( "error getting actual primary key string from $prim_key\n");
    }
    foreach my $key (%FORM) {
    # if key is a field of the table
    if ($key =~ /^$table($sep)(.*)/) {
        $param_value = $FORM{$key};
        $tablehash->{$2} =  $param_value;
    }
    }
#      print STDERR "for $table, got hash: ", Dumper($tablehash), "\n";
    #get hash of field type info
    my $col_info_hash =
        Metadot::TableInfoFactory->getTableInfoObj()->getFieldInfo($table);
#      print STDERR "col_info_hash: ", Dumper($col_info_hash), "\n";
    #make field hash, using type info to determine whether or not to quote
    #an element.
    if ($mode eq 'edit') {
    # then do an update
    my %attrhash = ();
    foreach my $field (keys %$tablehash) {
        if (($col_info_hash->{$field}->{'type'} eq 'number')
        && ($tablehash->{$field} ne "")){
        # don't add this attr if it's of type 'number' and the value is empty
        $attrhash{"-$field"} = $tablehash->{$field};
        }
        else {
        $attrhash{"$field"} = $tablehash->{$field};
        }
    }
    my $where_clause;
    if ($col_info_hash->{$prim_key_table}->{'type'} eq 'number') {
        $where_clause = "$prim_key_table=$pkval";
    }
    else {
        $where_clause = "$prim_key_table='$pkval'";
    }
    if (keys(%attrhash)) {
        DBAccess->sqlUpdate($table, $where_clause, %attrhash);
    }
    }
    elsif ($mode eq 'create') {
    # then do an insert of a new row in the table
    # get new uid first
    my $new_uid = Metadot::FormProcessUID->getUID($table, $prim_key_table);
    # build query hash
    my %attrhash = ();
    foreach my $field (keys %$tablehash) {
        if (lc $field eq lc $prim_key_table) { # if primary key
        if ($new_uid eq 'auto-increment') { # if auto increment
            # then don't add to insert query
        }
        else {      # not auto increment
            if (($col_info_hash->{$field}->{'type'} eq 'number')
            && ($tablehash->{$field} ne "")){
            # don't add this attr if it's of type 'number' and the value is empty
            $attrhash{"-$field"} = $new_uid;
            }
            else {
            $attrhash{"$field"} = $new_uid;
            }
        }
        }
        else {      # not primary key
        if (($col_info_hash->{$field}->{'type'} eq 'number')
            && ($tablehash->{$field} ne "")){
            # don't add this attr if it's of type 'number' and the value is empty
            $attrhash{"-$field"} = $tablehash->{$field};
        }
        else {
            $attrhash{"$field"} = $tablehash->{$field};
        }
        }
    }
#   print STDERR "in create mode, ended up with attr hash ", Dumper(\%attrhash), "\n";
    if (keys(%attrhash)) {  # if nonempty
#       print STDERR "calling sqlInsert\n";
        DBAccess->sqlInsert($table, %attrhash);
    }
    }
    else {
#   die "error: unknown mode $mode.";
    ErrorHandling( "error: unknown mode $mode.");
    }
}



1;
