###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Gizmo2;

use strict;

use Metadot qw( $PARAMS %FORM $DISPLAY $USER $CGI $DISPLAY );

use Template;

use Gizmo;
use GizmoBuilder;
use vars qw(@ISA);

use CGI qw/escapeHTML/;

@ISA=qw(GizmoBuilder);

my $version = "4.5";

#
#submit_handler   -- will return a value for storage PLUS an error message, if any. Will have %FORM available.
#
#display_handler -- will return HTML to display the contents of this field
#
#edit_handler -- will return HTML to edit the contents of this field
#
#on_change_handler -- will get called whenever the field changes from its previous value.
#                     This includes the first time the field gets a value. It doesn't have to
#                     return any value.
#
#save_handler -- will be invoked to save the contents of this field to secondary storage
#                If not provided, the normal instance table save will happen
#
#restore_handler -- will be invoked to restore the contents of this field from secondary storage
#


sub customize_permissions {
    my $class = shift;
    my $AC = shift;

    $class->SUPER::customize_permissions($AC);

    $AC->add_op('MOD','show_help_popup');
}


sub get_version {
    return $version;
}


sub get_fields_config {
    my $self = shift;

    die "ABStrACT, must override";
}


# Gizmo2 does not require this any more
sub initialize_fields_config {
}

sub show_edit_form {
    my $self = shift;
    my $title = shift;

    my $config = $self->get_fields_config();

    my $html;

    $html .= "
<table width=\"450\">";

    foreach my $field ( @{ $config->{FIELDS} } ) {

    my $edit_handler = $field->{edit_handler};

    if( $edit_handler ) {

        $html .= $self->$edit_handler( $field );

    } elsif( lc $field->{field_type} eq 'textarea' ) {

        $html .= $self->_text_area_edit_handler( $field );

    } elsif (lc $field->{field_type} eq 'text') {

        $html .= $self->_text_edit_handler( $field );

    } elsif (lc $field->{field_type} eq 'select_box' ) {

        $html .= $self->_select_box_edit_handler( $field );

    } elsif( lc $field->{field_type} eq 'text_log' ) {

        $html .= $self->_text_log_edit_handler( $field );

    } elsif( lc $field->{field_type} eq 'user_selector' ) {

        $html .= $self->_user_selector_edit_handler( $field );

    } elsif( lc $field->{field_type} eq 'time_select' ) {

        $html .= $self->_time_select_edit_handler( $field );
    }
    }

    $html .= "</table>";

    my $s;
    $s .= "<p /><h1>" . $title . "</h1>\n";

    if( $config->{EDIT_HELP} ) { $s .= $config->{EDIT_HELP} }

    # if there is any intro text to be displayed at the top, add it.
    $s .= "<p>" . $self->get_intro_text() . "</p>";

    $s .= $html;

    return $s;
}


sub _select_box_edit_handler {
    my $self = shift;
    my $field = shift;

    my $field_name = $field->{field_name} || die "must specify field_name";
    my $field_desc = $field->{description} || die "must specify field description";
    my $required = ( $field->{is_required} ) ? "<b><font color=\"#FF0000\">* </font></b>" : '';

    my $value = exists( $FORM{$field_name} ) ?
      $FORM{$field_name} : $self->{ $field->{store_at_column} } || $field->{default_option};

    my $html = "
<tr>
  <td>
    $required<b>$field_desc</b>
  </td>
  <td>";

    $html .= "<select name='$field_name'>";

    foreach my $val ( @{ $field->{display_order} } ) {

    my $tag = $field->{select_options}{$val};

    my $selected = '';

    if( $val eq $value ) {
        $selected = ' selected';
    }

    $html .= "<option value=\"$val\"$selected> $tag </option>";
    }

    my $help_icon = $self->get_help_icon( $field );

    $html .= "</select>$help_icon";
    $html .= "
  </td>
</tr>";

    return $html;
}


sub _user_selector_edit_handler {
    my $self = shift;
    my $field = shift;

    my $field_name = $field->{field_name} || die "must specify field_name";
    my $field_desc = $field->{description} || die "must specify field description";
    my $required = ( $field->{is_required} ) ? "<b><font color=\"#FF0000\">* </font></b>" : '';

    my $value = exists( $FORM{$field_name} ) ?
      $FORM{$field_name} : $self->{ $field->{store_at_column} } || $field->{default_option};

    my $html = "
<tr>
  <td>
    $required<b>$field_desc</b>
  </td>
  <td>";

    $html .= "<select name='$field_name'>";

    my @options;

    if( $field->{show_groups} ) {

    my @show_groups = split( /,/, $field->{show_groups} );

    foreach my $group_name ( @show_groups ) {

        my %options;
        foreach my $usr_email ( @{ $USER->get_emails_by_grp_name( $group_name ) } ) {

        $options{$usr_email}++;
        }

        @options = keys %options;
    }

    } else {

    # include all users
    @options = map{ $_->email()  } @{ $USER->get_all_active_users() };
    }

    # include a null option
    $html .= "<option value=\"\"> -- </option>";

    foreach my $email ( @options ) {

    my $uid = $USER->get_uid_from_email( $email );

    my $selected = '';

    if( $uid eq $value ) {
        $selected = ' selected';
    }

    $html .= "<option value=\"$uid\"$selected> $email </option>";
    }

    $html .= "</select>\n";
    $html .= "
  </td>
</tr>";

    return $html;
}


sub _text_log_edit_handler {
    my $self = shift;
    my $field = shift;

    my $field_name = $field->{field_name} || die "must specify field_name";
    my $field_desc = $field->{description} || die "must specify field description";
    my $required = ( $field->{is_required} ) ? "<b><font color=\"#FF0000\">* </font></b>" : '';

    my $submitted_val = $FORM{$field_name} || '';
    my $value = $self->{ $field->{store_at_column} };

    my $rows = $field->{rows} || 2;
    my $cols = $field->{cols} || 30;
    my $wrap = $field->{wrap} ? 'wrap' : '';
    my $max_length = $field->{max_length} || 255;

    my $explanation = '';
    if( $value ) {
    $explanation = '<br /><i>(this comment will be added to the text log below)</i><br /><br />Text log:';
    $value .= '<br />';
    }

    my $html = "
<tr>
  <td>
    $required<b>$field_desc</b>";

    $html .= "
  </td>
  <td>Add comment:<br />
    <textarea name=$field_name rows=$rows cols=$cols wrap=$wrap maxlength=$max_length>$submitted_val</textarea>$explanation
    <br />$value
  </td>
</tr>";

    return $html;
}

sub _text_edit_handler {
    my $self = shift;
    my $field = shift;

    my $field_name = $field->{field_name} || die "must specify field_name";
    my $field_desc = $field->{description} || die "must specify field description";
    my $required = ( $field->{is_required} ) ? "<b><font color=\"#FF0000\">* </font></b>" : '';

    my $size = $field->{size} || 25;
    my $max_length = $field->{max_length} || 64;

    my $value = exists( $FORM{$field_name} ) ?
      $FORM{$field_name} : $self->{ $field->{store_at_column} };

    $value = escapeHTML( $value );

    my $help_icon = $self->get_help_icon( $field );

    my $html = "
<tr>
  <td>
    $required<b>$field_desc</b>";

    $html .= "
  </td>
  <td>
<INPUT type=text name=$field_name size=$size value=\"$value\" maxlength=$max_length>$help_icon
  </td>
</tr>";

    return $html;
}

sub _text_area_edit_handler {
    my $self = shift;
    my $field = shift;

    my $field_name = $field->{field_name} || die "must specify field_name";
    my $field_desc = $field->{description} || die "must specify field description";
    my $required = ( $field->{is_required} ) ? "<b><font color=\"#FF0000\">* </font></b>" : '';

    my $value = exists( $FORM{$field_name} ) ?
      $FORM{$field_name} : $self->{ $field->{store_at_column} };

    my $rows = $field->{rows} || 3;
    my $cols = $field->{cols} || 30;
    my $wrap = $field->{wrap} ? 'wrap' : '';
    my $max_length = $field->{max_length} || 255;

    my $help_icon = $self->get_help_icon( $field );

    my $html = "
<tr>
  <td>
    $required<b>$field_desc</b>";

    $html .= "
  </td>
  <td>
    <table><tr><td><textarea name=$field_name rows=$rows cols=$cols wrap=$wrap maxlength=$max_length>$value</textarea></td><td valign='bottom'>$help_icon</td></tr></table>
  </td>
</tr>";

    return $html;
}


sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    # Some gizmos pass only an $id to the constructor, while
    # others send a hashref of arguments. This causes the
    # constructor to always see a hashref of arguments, no
    # matter what the actual parameters were.
    my $args = $class->normalize_constructor_args(shift);
    my $id = $args->{id};

    my $self;

    if (defined ($id)){
        # I'm retrieving an instance from the db. Gizmo->restore() takes
        # care of all instantiation duties by calling new() if needed.
        # Therefore, we can just return the result.
        return $class->restore($id);
    } else {
        $self  = $class->SUPER::new($args);
    }

    $self->debug_msg(4, "Instantiating new $self");
    $self->{jsform} = undef;

    bless ($self, $class);

    # reconsecrate
    return $self;
}

###################################

sub show {
    my $self = shift;

    my $config = $self->get_fields_config();

    my $data = {};

    foreach my $field ( @{ $config->{FIELDS} } ) {

    my $display_handler = $field->{display_handler};

    if( $display_handler ) {

        push @{ $data->{display_fields} }, $self->$display_handler( $field );

    } elsif( lc $field->{field_type} eq 'select_box' ) {

        push @{ $data->{display_fields} }, $self->_select_box_display_handler( $field );

    } elsif( lc $field->{field_type} eq 'creation_datetime' ) {

        push @{ $data->{display_fields} }, $self->_datetime_display_handler( $field );

    } elsif( lc $field->{field_type} eq 'user_selector' ) {

        push @{ $data->{display_fields} }, $self->_user_selector_display_handler( $field );

    } elsif( lc $field->{field_type} eq 'time_select' ) {

        push @{ $data->{display_fields} }, $self->_time_select_display_handler( $field );

    } else {

        push @{ $data->{display_fields} }, $self->_plain_field_display_handler( $field );
    }
    }

    $data->{parent_id} = $self->parent_id();

    my $template = Template->new({
                  INCLUDE_PATH => 'Metadot/templates',
                 }) || die $Template::ERROR, "\n";

    my $out;
    $template->process("gizmo2_show.tt2",  $data, \$out)
      || die $template->error(), "\n";

    return $out;
}

sub _user_selector_display_handler {
    my $self = shift;
    my $field = shift;

    my $uid = $self->{ $field->{store_at_column} };

    my $value = $USER->get_email_from_uid( $uid );

    my $text = "<td width=\"100\" bgcolor=\"#EEEEEE\"><b>$field->{description}<b></td><td width=\"400\" bgcolor=\"#EEEEEE\">$value</td>\n";

    return $text;
}

sub _plain_field_display_handler {
    my $self = shift;
    my $field = shift;

    my $value = $self->{ $field->{store_at_column} } || '&nbsp;';

    my $text = "<td width=\"100\" bgcolor=\"#EEEEEE\"><b>$field->{description}<b></td><td width=\"400\" bgcolor=\"#EEEEEE\">$value</td>\n";

    return $text;
}

sub _select_box_display_handler {
    my $self = shift;
    my $field = shift;

    my $value = $field->{select_options}{ $self->{ $field->{store_at_column} }} || '&nbsp;';

    my $text = "<td width=\"100\" bgcolor=\"#EEEEEE\"><b>$field->{description}</b></td><td width=\"400\" bgcolor=\"#EEEEEE\">$value</td>\n";

    return $text;
}

sub _datetime_display_handler {
    my $self = shift;
    my $field = shift;

    my $value = $self->_format_datetime( $self->{ $field->{store_at_column} } ) || '&nbsp;';

    my $text = "<td width=\"100\" bgcolor=\"#EEEEEE\"><b>$field->{description}</b></td><td width=\"400\" bgcolor=\"#EEEEEE\">$value</td>\n";

    return $text;
}


sub process_edit_form {
    my $self  = shift;
    my $error = shift || undef;

    my $config = $self->get_fields_config();

    my $iid = $self->get_iid();

    my $values;

    my $error_msg = '';

    foreach my $field ( @{ $config->{FIELDS} } ) {

    my ( $value, $error );

    my $submit_handler = $field->{submit_handler};

    if( $submit_handler ) {

        ( $value, $error ) = $self->$submit_handler( $field );

    } elsif( $field->{field_type} eq 'auto_increment' ) {

        ( $value, $error ) =
          $self->_process_auto_increment_field( $field );

    } elsif( $field->{field_type} eq 'creation_datetime' ) {

        ( $value, $error ) =
          $self->_process_creation_datetime_field( $field );

    } elsif( $field->{field_type} eq 'text_log' ) {

        ( $value, $error ) =
          $self->_process_text_log_field( $field );

    } elsif( $field->{field_type} eq 'time_select' ) {

        ( $value, $error ) =
          $self->_time_select_submit_handler( $field );

    } else {

        ( $value, $error ) =
          $self->_process_plain_field( $field );
    }

    if( $error ) { $error_msg .= "$error <br />\n" };

    $values->{ $field->{store_at_column} } = $value;
    }

    if ( $error_msg && $iid ){ ## if error and updating

        $self->www_modify( $error_msg );

    } elsif ( $error_msg ) {   ## if error and saving new

        my $parentid = $self->parent_id();
        my $parent = Gizmo->restore( $parentid );

        $parent->www_add_child( $error_msg, $self->is_a );

    } else {                        ## no errors

    # fill the right fields for www_save to work
    while( my ( $key, $val ) = each %$values ) { $FORM{$key} = $val; }

    # XXX [claudio] a hack to address sthe fact that name column cannot be null
    unless (exists $FORM{name}) { $FORM{name} = '' }

    $self->_invoke_on_change_handlers();

        $self->www_save( 'values_checked' );
    }
}


sub _invoke_on_change_handlers {
    my $self = shift;

    my $config = $self->get_fields_config();

    my $iid = $self->get_iid();

    foreach my $field ( @{ $config->{FIELDS} } ) {

    my $old_val = $self->{ $field->{store_at_column} };
    my $new_val = $FORM{ $field->{store_at_column} };
    my $on_change_handler = $field->{on_change_handler};

    if( ( $old_val ne $new_val ) and $on_change_handler ) {

        $self->$on_change_handler( $field );
    }
    }
}


sub _process_text_log_field {
    my $self = shift;
    my $field = shift;

    my ( $value, $error ) = $self->_process_plain_field( $field );

    my $old = $self->{ $field->{store_at_column} };

    if( $value ) {

    my $date = $self->_format_datetime( time );

    my $username = $USER->fullname();

    $old .= '<br />' if( $old );

    $value = "$old\[$date\] Added by $username:<br />$FORM{ $field->{field_name} }<br />\n";

    } else {

    $value = $old;
    }

    return ( $value, $error );
}


sub _process_created_by {
    my $self = shift;
    my $field = shift;

    my $value;

    if( $self->get_iid() ) {

    $value = $self->{ $field->{store_at_column} };

    } else {

    $value = $USER->fullname();
    }

    return ( $value, '' );
}


sub _process_auto_increment_field {
    my $self = shift;
    my $field = shift;

    my $value;

    my $XXX = $self->get_iid();

    if( $self->get_iid() ) {

    $value = $self->{ $field->{store_at_column} };

    } else {

    # XXX[claudio] must get a lock and release it when this datum is saved
    my ($max) = DBAccess->sqlSelect("MAX($field->{store_at_column})", "instance", "isa='Ticket'" );
    $value = $max + 1;
    }

    return ( $value, '' );
}

sub _process_creation_datetime_field {
    my $self = shift;
    my $field = shift;

    my $value;

    if( $self->get_iid() ) {

    # item is being modified
    $value = $self->{ $field->{store_at_column} };

    } else {

    # item is being created
    $value = time;
    }

    return ( $value, '' );
}


sub _process_plain_field {
    my $self = shift;
    my $field = shift;

    my ( $error, $value );

    if( $FORM{ $field->{field_name} } || $field->{is_required} ) {

    # if the field is required or has been filled
    my $verify_type = $field->{verify_type} || 'non_empty';

    if( $verify_type eq 'non_empty' ) {

        my $verify = Form::Verify->new;

        $verify->verify( $verify_type,
                 $field->{description},
                 $FORM{ $field->{field_name} },
                 $field->{date_format} );

        $error = $verify->errors();
        $error && ( $error = "$error field must be present<br />" );
    }
    }

    $value = $FORM{ $field->{field_name} };

    return ( $value, $error );
}


## called for saving and updating
sub www_save {
    my $self  = shift;
    my $values_checked = shift;

    unless( $values_checked ) {
    $self->process_edit_form();
    return
    }

    my $order = 0;
    my $uid = $USER->uid();

    my $error = undef;

    my $iid = $self->get_iid();

    my $config = $self->get_fields_config();

    foreach my $field ( @{ $config->{FIELDS} } ) {

    my $column_name = $field->{store_at_column};

    $self->{$column_name} = $FORM{$column_name};
    }

    if ( !$iid ) { ## we are creating new

        $iid = $self->add();

        if ($iid) {

            my $isa = $self->is_a();
            my $parentid = $self->parent_id();

        # XXX[claudio] this method must be made public
        $self->_show_create_success();
        }

    } else {            ## we are updating

    $self->save();

    # XXX[claudio] this method must be made public
    $self->_show_edit_success();
    }
}



my %MONTHS = (
               0 => 'January',
               1 => 'February',
               2 => 'March',
               3 => 'April',
               4 => 'May',
               5 => 'June',
               6 => 'July',
               7 => 'August',
               8 => 'September',
               9 => 'October',
              10 => 'November',
              11 => 'December'
             );

my %DAYS_OF_WEEK = (
                    0 => 'Sunday',
                    1 => 'Monday',
                    2 => 'Tuesday',
                    3 => 'Wednesday',
                    4 => 'Thursday',
                    5 => 'Friday',
                    6 => 'Saturday'
                   );


## formats a unix timestamp
## M - month (January)
## D - day   (1-31)
## Y - year  (YYYY)
## T - time  (1:00 pm)
## H - hour  (1-12)
sub _format_datetime {
    my $self  = shift;
    my $datetime  = shift;
    my $format = shift || 'M D, Y T';

    my ($sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst) = localtime($datetime);
    $year+=1900;
    my $date = '';

    my @format_array = split(//, $format);

    foreach my $char (@format_array) {
        if ($char eq 'W') {
            $date .= $DAYS_OF_WEEK{$wday};
        } elsif ($char eq 'M') {
            $date .= $MONTHS{$month};
        } elsif ($char eq 'D') {
            $date .= $day;
        } elsif ($char eq 'Y') {
            $date .= $year;
        } elsif ($char eq 'T') {
            $date .= $self->_format_time($datetime);
        } elsif ($char eq 'H') {
            $date .= $self->_format_hour($hour);
        } else {
            $date .= $char;
        }
    }

    return $date;
}


## formats hour portion of a unix time stamp (i.e. 1:00 pm)
sub _format_time {
    my $self  = shift;
    my $time  = shift;

    my ($hour, $min) = (localtime($time))[2, 1];
    my $ampm = $hour > 11 ? 'pm': 'am';
    $min = $min < 10 ? "0$min" : $min;

    return $self->_format_hour($hour) . ":$min $ampm";
}


## formats hour (returns 1-12)
sub _format_hour {
    my $self  = shift;
    my $hour  = shift;

    my $display_hour = '';

    if ($hour == 0) {
        $display_hour = '12';
    } elsif ($hour > 12) {
        $display_hour = $hour - 12;
    } else {
        $display_hour = $hour;
    }

    $display_hour+=0;

    return $display_hour;
}


{
    my $fields_by_field_name;

    sub _make_fields_by_field_name {
    my $self = shift;

    my $isa = $self->is_a();

    unless( $fields_by_field_name->{$isa} ) {

        my $config = $self->get_fields_config();

        foreach my $field ( @{ $config->{FIELDS} } ) {

        $fields_by_field_name->{$isa}{ $field->{field_name} } = $field;
        }
    }
    }

    sub get_field_value {
    my $self = shift;
    my $field_name = shift;

    $self->_make_fields_by_field_name();

    my $isa = $self->is_a();

    my $iid = $self->get_iid();
    my $val;

#   if( $field_name eq 'name' ) {
#       use Data::Dumper;
#       print StdERR "===========> AT GET_FIELD_VALUE:\n";
#       print StdERR $self->{name} . "\n";
#   }

    if( $iid ) {

        my $column_name = $fields_by_field_name->{$isa}{$field_name}{store_at_column};

        $val = $self->{ $fields_by_field_name->{$isa}{$field_name}{store_at_column} };

    } else {

        unless( $fields_by_field_name->{$isa}{$field_name}{field_type} eq 'time_select' ) {

        $val = $FORM{$field_name};

        } else {

        $val = $self->_get_time_select_from_form( $field_name );
        }
    }

    #    print StdERR "GETTING FIELD $field_name: $val\n";

    return $val;
    }

    sub set_field_value {
    my $self = shift;
    my $field_name = shift;
    my $value = shift;

    $self->_make_fields_by_field_name();

    my $isa = $self->is_a();

    $self->{ $fields_by_field_name->{$isa}{$field_name}{store_at_column} } =

    $value;
    }

    sub get_field_attribute {
    my $self = shift;
    my $field_name = shift;
    my $attribute_name = shift;

    $self->_make_fields_by_field_name();

    my $isa = $self->is_a();

    my $attribute = $fields_by_field_name->{$isa}{$field_name}{$attribute_name} ;

    return $attribute;
    }
}


sub save {
    my $self = shift;

    $self->SUPER::save();
}


sub _time_select_edit_handler {
    my $self = shift;
    my $field = shift;

    my $field_name = $field->{field_name} || die "must specify field_name";
    my $field_desc = $field->{description} || die "must specify field description";
    my $required = ( $field->{is_required} ) ? "<b><font color=\"#FF0000\">* </font></b>" : '';

    my ( $stored_hour, $stored_minutes, $stored_am_pm ) = split( /\s/, $self->{ $field->{store_at_column} } );

    my $hour = $FORM{ $field_name . '_hour' } || $stored_hour || $field->{default_hour};
    my $minutes = $FORM{ $field_name . '_minutes' }  || $stored_minutes || $field->{default_minutes};
    my $am_pm = $FORM{ $field_name . '_am_pm' }  || $stored_am_pm || $field->{default_am_pm};

    my $html = "
<tr>
  <td>
    $required<b>$field_desc</b>
  </td>
  <td>";

    $html .= "<select name='$field_name\_hour'>";
    foreach( 1..12 ) {

    my $selected = ( $_ == $hour ) ? 'selected' : '';

    $html .= "<option value=\"$_\"$selected> $_ </option>";
    }
    $html .= "</select>\n";

    $html .= ' : ';
    $html .= "<select name='$field_name\_minutes'>";
    foreach my $val ( qw( 00 15 30 45 ) ) {

    my $selected = ( $val  == $minutes ) ? 'selected' : '';

    $html .= "<option value=\"$val\"$selected> $val </option>";
    }
    $html .= "</select>\n";

    $html .= ' ';
    $html .= "<select name='$field_name\_am_pm'>";
    foreach my $val ( qw( AM PM ) ) {

    my $selected = ( $val eq $am_pm ) ? 'selected' : '';

    $html .= "<option value=\"$val\"$selected> $val </option>";
    }
    $html .= "</select>\n";

    $html .= "
  </td>
</tr>";

    return $html;
}

sub _time_select_submit_handler {
    my $self = shift;
    my $field = shift;

    my ( $error, $value ) = ('', '');

    if( $FORM{ $field->{field_name} . '_hour' } ||
    $FORM{ $field->{field_name} . '_minutes' } ||
    $FORM{ $field->{field_name} . '_am_pm' } ||
    $field->{is_required} ) {

    unless( $FORM{ $field->{field_name} . '_hour' } &&
        $FORM{ $field->{field_name} . '_minutes' } &&
        $FORM{ $field->{field_name} . '_am_pm' } ) {

        $error = "$field->{description} field must be present<br />";

    } else {

        $value = $self->_get_time_select_from_form( $field->{field_name} );
    }
    }

    return ( $value, $error );
}


sub _get_time_select_from_form {
    my $self = shift;
    my $field_name = shift;

    my ( $hour, $min, $am_pm ) = ( $FORM{ $field_name . '_hour' },
                   $FORM{ $field_name . '_minutes' },
                   $FORM{ $field_name . '_am_pm' } );

    my $value = "$hour $min $am_pm";

    return $value;
}


sub _time_select_display_handler {
    my $self = shift;
    my $field = shift;

#    use Carp;
#    Carp::cluck("AT _TIME_SELECT_DISPLAY_HANDLER");

    my ( $stored_hour, $stored_minutes, $stored_am_pm ) = split( /\s/, $self->{ $field->{store_at_column} } );

    my $value = "$stored_hour:$stored_minutes $stored_am_pm";

    my $text = "<td width=\"100\" bgcolor=\"#EEEEEE\"><b>$field->{description}</b></td><td width=\"400\" bgcolor=\"#EEEEEE\">$value</td>\n";

    return $text;
}



sub show_summary {
    my $self        = shift;

    my $buttons     = $self->get_buttons();
    my $html        = '';

    $html .= "<TABLE BORDER=\"0\" CELLSPACING=\"0\" width=\"100%\">";

    my $config = $self->get_fields_config();

    my $showed_buttons;
    foreach my $field ( @{ $config->{FIELDS} } ) {

    if( $field->{show_in_summary} ) {

        my $field_desc = $field->{description};
        my $field_value = $self->get_field_value( $field->{field_name} );

        $html .= "<tr Valign='BOTTOM'>";

        if( $buttons ) {
        unless( $showed_buttons ) {
            $showed_buttons++;
            $html .= "<td nowrap='nowrap'>$buttons</td>"
        } else {
            $html .= "<td nowrap='nowrap'>&nbsp;</td>"
        }
        }

        $html .= "<td nowrap='nowrap'><b>$field_desc</b>&nbsp;</td>";
        $html .= "<td nowrap='nowrap'>$field_value</td>";
        $html .= "<td width=\"100%\">&nbsp;</td>";

        $html .= "</tr>\n";
    }
    }
    $html .= "<tr Valign='BOTTOM'>";
    if( $buttons ) { $html .= "<td nowrap='nowrap'> &nbsp; </td>" }

    my $isa = $self->is_a();
    my $id = $self->id();

    $html .= "<td nowrap='nowrap'><a href=\"" .
      $self->make_url( "index.pl?isa=$isa&amp;id=$id&amp;op=show" ) .
    "\">Details</a></td>";
    $html .= "<td nowrap='nowrap'> &nbsp; </td>";
    $html .= "<td width=\"100%\">&nbsp;</td>";
    $html .= "</tr>\n";
    $html .= "<tr></tr>\n";

    $html .= "</TABLE>\n";

    return $html;
}


sub www_show_help_popup {
    my $self = shift;

    my $field_name = $FORM{field_name};

    my $help_message = $self->get_field_attribute( $field_name, 'help_message');

    my $field_desc = $self->get_field_attribute( $field_name, 'description' );

    my $template = Template->new({
                  INCLUDE_PATH => 'Metadot/templates',
                 }) || die $Template::ERROR, "\n";


    my $page_top = $DISPLAY->show_page_top();

    my $out;
    $template->process( "gizmo2_help_popup.tt2",
            {
             page_top => $page_top,
             help_message => $help_message,
             field_desc => $field_desc,
             header_bg_color => $PARAMS->{header1_color} || '#003366',
             header_text_color => $PARAMS->{title_text_color} || '#FFFFFF',
            },
            \$out ) || die $template->error(), "\n";

    $DISPLAY->print_http_header();
    print $out;

}


sub get_help_icon {
    my $self = shift;
    my $field = shift;

    return unless( $field->{help_message} );

    my $field_name = $field->{field_name};

    my $class_name = $self->is_a();
    my $id = $self->id();
    my $images_loc = $PARAMS->{imageslocation};

    my $script = "<a href=\"javascript:popUp( \'index.pl?id=$id&amp;isa=$class_name&amp;op=show_help_popup&amp;field_name=$field_name\' )\"><img src=\"$images_loc/question_mark.gif\" border=\"0\"></a>";

    return $script;

}


1;

__END__

