#######################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Metadot::Gizmo3000;

=head1 NAME

Metadot::Gizmo3000

=head1 VERSION

Version $Revision: 1.32.2.5 $

=head1 SYNOPSIS

package MyGizmo;


use Metadot::Gizmo3000;

@ISA = qw(Metadot::Gizmo3000);


use constant GIZMO_CONFIG => {
    GIZMO_INFO   => {
        name                => 'Book',
        version             => '1.0',
        description         => 'Allows to add book gizmos',
        author              => 'Jean Vacance',
        info                => 'jean@mail.com +1 512 323 3232',
        need_portal_version => 6.5,
        debug               => 3,
        edit_help           => 'blablablah',
    },
    GIZMO_FIELDS  => [
        {
            name            => 'title',
            label           => 'Title',
            is_mandatory    => 1,
            inline_help     => 'The title of the book',
            store_at_column => 'name',
        },
        {
            name            => 'author',
            label           => 'Author',
            is_mandatory    => 1,
            inline_help     => 'The author of the book',
            store_at_column => 'c2',
        },
        {
            name            => 'publisher',
            label           => 'Publisher',
            type            => 'select_list',
            store_at_column => 'c3',
            select_options  => [
                'knopf', 'Knopf',
                'rhouse','Random House'
            ],
        },
        {
            name            => 'cover_photo',
            label           => 'Cover Photo',
            type            => 'file',
            store_at_column => 'file1',
        },
        {
            name            => 'download',
            label           => 'File Attachment',
            type            => 'file',
            store_at_column => 'c5',
        },
    ],
};




sub do_show {
    my $self    = shift;
    my $context = shift;

    return 'my_template.tt2';
}


1;

=head1 DESCRIPTION

Gizmo3000 is the base class for a new generation of Gizmos. Gizmo3000 provides
methods that allow one to easily declare new Gizmos while writing minimal code.

=cut





use strict;

use Metadot qw(%FORM $USER $DISPLAY $PARAMS $SESSION %REQUEST_VARS);
use Metadot::ClassUtilities;
use Metadot::Controllable;
use Metadot::GizmoBase;
use Metadot::Gizmo3000::ClassRegistry;
use Metadot::Gizmo3000::IconEditControl;
use Metadot::Form3000::Fields::FormFieldFactory;
use Metadot::Form3000::Form;
use Metadot::UploadsManager;
use Utility;


use vars qw(@ISA);
@ISA = qw(Metadot::GizmoBase Metadot::Controllable);

# some class-wide constants
use constant GIZMO_INFO     => 'GIZMO_INFO';
use constant GIZMO_FIELDS   => 'GIZMO_FIELDS';

# MetadotObject stuff
use constant MO_INSTANCE_PROPERTIES => {
    _descriptor => undef,
};
# customize the initializer method
use constant MO_INITIALIZER_METHOD => 'do_initialize';
use constant EXEC_METHOD_BASE      => 'do_';


# each Gizmo class should populate this
use constant GIZMO_CONFIG => {
    GIZMO_INFO   => {
        name                => 'Gizmo3000 Base Class',
        version             => '1.0',
        description         => 'This is the base class for all Gizmo3000 instances',
        author              => 'Cory Kim',
        info                => 'cory@metadot.com',
        need_portal_version => 6.5,
        debug               => 0,
        is_searchable       => 1,
        edit_help           => '',
        is_dedicated_child  => 0,  # indicates whether the gizmo is always a child.
                                   # this affects permissions & "add child" lists,
        allowed_containers      => undef,   # by default all containers are permitted
                                            # to hold this gizmo.
        prohibited_containers   => undef,   # by default no containers are prohibited
                                            # from holding this gizmo
    },
    GIZMO_FIELDS  => undef,
};

=head1 METHODS

=cut

=head2 do_initialize()

base method for initializing individual instances

=cut
sub do_initialize {
    my $self = shift;
    my $args = shift || {};

    my $class = ref($self) || die('Instance method');

    $self->SUPER::initialize($args);

    #print STDERR "In $self\->do_initialize()\n";

    my $descriptor = $self->_descriptor($class->get_descriptor());
    #print STDERR "Set my descriptor '$descriptor'\n";
}






# Convenience methods to access the gizmo's descriptor
# ----------------------------------------------------

=head2 get_gizmo_name()

returns the name of the gizmo class

=cut
sub get_gizmo_name {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    return $proto->get_descriptor()->get_name() || $class;
}

=head2 get_gizmo_version()

returns the version of the gizmo class

=cut
sub get_gizmo_version {
    my $class = shift;
    return $class->get_descriptor()->get_version();
}

=head2 get_gizmo_author()

returns the author of the gizmo class

=cut
sub get_gizmo_author {
    my $class = shift;
    return $class->get_descriptor()->get_author();
}

=head2 get_gizmo_info()

returns the info of the gizmo class

=cut
sub get_gizmo_info {
    my $class = shift;
    return $class->get_descriptor()->get_info();
}



=head2 get_class_registry()

returns the Metadot::Gizmo3000::ClassRegistry instance

=cut
sub get_class_registry {
    return Metadot::Gizmo3000::ClassRegistry->get_instance();
}


=head2 get_descriptor()

returns the Metadot::Gizmo3000::ClassDescriptor object

=cut
sub get_descriptor {
    my $self = shift;
    my $result;
    if (ref($self)) {
        # instances already have a reference to the descriptor
        $result = $self->_descriptor();
    } else {
        # packages need to lookup the descriptor from the registry
        $result = $self->get_class_registry()->get_descriptor_for($self);

        #my ($package, $filename, $line) = caller;
        #print STDERR "$self\->get_descriptor called in class context by $package:$line, returned [$result]\n";
    }

    return $result;
}


=head2 get_property_mappings()

gets the form fields of this class and all its parents

=cut
sub get_property_mappings {
    my $class = shift;

    my @mappings = ();
    foreach my $module ($class->class_hierarchy({inclusive => 1})) {
        my $descriptor = $module->get_descriptor();
        if (defined $descriptor) {
            my $mapping = $descriptor->get_property_mappings();
            if (defined $mapping) {
                push @mappings, @{$mapping};
            }
        }
    }

    # order the fields
    @mappings = sort {($a->display_order() || 0) <=> ($b->display_order() || 0)} @mappings;

    return \@mappings;
}


=head2 get_property_map()

gets the form fields of this class as a hashtable

=cut
sub get_property_map {
    my $class = shift;

    my %map = ();
    foreach my $module ($class->class_hierarchy({inclusive => 1})) {
        my $descriptor = $module->get_descriptor();
        if (defined $descriptor) {
            Utility->merge_hashes(\%map, $descriptor->get_property_map());
        }
    }

    #use Data::Dumper;
    #print STDERR "$class\->get_property_map produces:\n",Dumper(\%map);

    return \%map;
}




# default event handlers
#-----------------------------------------------------------------

=head2 do_show()

Generates the output to display when the show operation is called. You can
output HTML here or the name of a template file that exists somewhere in the
template path.

=cut
sub do_show {
    my $self = shift;
    my $context = shift;

    # Most gizmos only want to show myself in the context of my page

    # redirect to parent unless I am a navigable gizmo
    if ($self->isa('Metadot::Navigable')) {
        # replace this
        my $content = "This is a navigable page. Please implement the 'do_show' method.";
        return $self->generate_standard_wrapper($content);
    } else {
        # redirect to navigable parent
        my $navigable_parent = $self->get_navigation_parent();
        my $navigable_parent_id = $navigable_parent->id();
        if (defined($navigable_parent) && defined($navigable_parent_id)) {
            $DISPLAY->debug_msg(4, "do_show redirects to navigation parent $navigable_parent ($navigable_parent_id)");
            $DISPLAY->md_redirect("index.pl?iid=".$navigable_parent_id);
        }
    }
}


=head2 do_show_summary()

Generates the output to display when the show operation is called. You can
output HTML here or the name of a template file that exists somewhere in the
template path.

=cut
sub do_show_summary() {
    my $self    = shift;
    my $context = shift;

    return $self->do_show(@_);
}


=head2 do_modify()

Generates the form for modifying an instance.

=cut
sub do_modify {
    my $self    = shift;
    my $context = shift;

    $DISPLAY->debug_msg(3, "$self\->do_modify()");

    my $iid = $self->id();
    my $isa = $self->is_a();

    # If I don't have an iid, there is a problem
    unless (defined $iid){
        $DISPLAY->click_back ("Error:", "id is missing.");
        exit(0);
    }

    my $landing_url = $FORM{landing_url} || $SESSION->generate_referer();

    my ($fields, $values) = $self->define_form_fields();


    # build the form object
    my $form = Metadot::Form3000::Form->new(
        'action'    => 'index.pl',
        'method'    => 'post',
        'enctype'   => 'multipart/form-data',
        'name'      => 'fm',
        'title'     => 'Customizing '.$self->get_gizmo_name(),
        'fields'    => $fields,
        'values'    => $values,
        'mappings'  => $self->get_property_mappings,
        'submit_actions' => [
            {
                name  => 'submit_form',
                value => 'Save changes',
            }
        ],
    );


    $form->generate_form_field(
        type  => 'hidden',
        name  => 'op',
        value => 'save' );

    $form->generate_form_field(
        type  => 'hidden',
        name  => 'isa',
        value => $self->is_a() );

    $form->generate_form_field(
        type  => 'hidden',
        name  => 'iid',
        value => $self->iid() );

    $REQUEST_VARS{form_object} = $form;

    my $template_name = 'form_3000.tt2';
    $DISPLAY->debug_msg(3, "$self\->will process template '$template_name'");

    return $template_name;
}


=head2 do_save()

Persists an instance.

=cut
sub do_save {
    my $self    = shift;
    my $context = shift;

    my $form = $self->process_edit_form();
    $self->process_uploads({ form=>$form });

    if ($form->has_errors()) {
        return $self->do_modify(@_);
    } else {
        $self->save();
        $DISPLAY->md_redirect( $form->landing_url()
                               || "index.pl?iid=".$self->iid());
    }
}


=head2 do_download_file()

Transmits a file attachment

=cut
sub do_download_file {
    my $self = shift;

    my $field_name = $FORM{field_name};

    my $file_info = $self->get_file_info({ field_name=>$field_name });
    my $file_dir  = $self->get_write_dir_namespace($field_name);
    my $file_name = $file_info->{file_name};
    if ($file_name) {
        Metadot::UploadsManager->transmit_file(
            file_name     => $file_name,
            file_location => $file_dir.'/'.$file_name,
        );
    } else {
        $DISPLAY->click_back("No file exists for $field_name");
    }

    # we don't want the portal to output anything else.
    exit;
}





# internal methods to be used by the event handlers


=head2 get_edit_controls({ args })

Returns a collection of EditControl objects that are valid
for the given user and time

subject: the object for whom the edit controls are being created

=cut
sub get_edit_controls {
    my $self = shift;
    my $args = shift || {};

    my $requested_ops = $args->{ops};

    $DISPLAY->debug_msg(4, "IN $self\->get_edit_controls");

    if (defined $requested_ops) {
        $DISPLAY->debug_msg(5, "Requested ops: ".join(' ', @$requested_ops));
    }

    my @controls = ();

    my $edit_control_class = 'Metadot::Gizmo3000::IconEditControl';

    my $images_location = $PARAMS->{imageslocation};

    if ($SESSION->admin_mode()) {
        foreach my $operation (@{ $self->_BASIC_OPS() }) {
            my $op = $operation->{op};

            # checking if this op is requested
            if (!defined $requested_ops || Utility->array_contains($op, $requested_ops)) {
                $DISPLAY->debug_msg(5, "$self checking if user is allowed to $op");
                if ($self->is_allowed_to_do($op)) {
                    my $control = $edit_control_class->new(
                            operation   => $op,
                            actor_iid   => $self->id(),
                            icon_url    => $images_location.'/obj/'.$operation->{icon},
                            description => $operation->{description},
                        );
                    push @controls, $control;
                    $DISPLAY->debug_msg(5, "$self added control $control for $op");
                }
            }
        }
    }

    # get parental controls
    my $parent = $self->get_parent();
    if ($parent->can('get_edit_controls_for_child')) {
        $DISPLAY->debug_msg(5, "IN $self\->get_edit_controls, checking conrols in $parent");
        push @controls, $parent->get_edit_controls_for_child($self, $args);
    }
    $DISPLAY->debug_msg(5, "IN $self\->get_edit_controls, finished processing parents");


    return @controls;
}


use constant _BASIC_OPS => [
    {
      op   => 'modify',
      icon => 'edit.gif',
      description => 'Edit Me',
    },
    {
      op   => 'modify_properties',
      icon => 'property.gif',
      description => 'Edit My Properties',
    },
    {
      op   => 'edit_permissions',
      icon => 'ac.gif',
      description => 'Change Access Rights',
    },
    {
      op   => 'create_shortcut',
      icon => 'shortcut.gif',
      description => 'Create Shortcut',
    },
    {
      op   => 'delete',
      icon => 'del.gif',
      description => 'Delete Me',
    },
];


=head2 get_resource_directory()

returns the directory where Gizmo3000 resources are stored

=cut
sub get_resource_directory {
    my $class = shift;
    return MetadotConfig->get_web_resource_dir().'/Gizmo3000';
}


=head2 get_resource_location()

returns the URI path where Gizmo3000 resources are accessed

=cut
sub get_resource_location {
    my $class = shift;
    return MetadotConfig->get_web_resource_location().'/Gizmo3000';
}



=head2 get_edit_control_html()

Outputs the edit controls as HTML

=cut
sub get_edit_control_html {
    my $self = shift;
    my $args = shift;

    my $html = '';
    foreach my $control ($self->get_edit_controls($args)) {
        $html .= $control->render_html()."\n";
    }

    return $html;
}


# for backwards compatibility with Gizmo
sub show_up_down_buttons{
  my $self   = shift;
  return $self->get_edit_control_html({ops=>['move_child_up_down']});
}

# for backwards compatibility with Gizmo
sub show_del_button {
    my $self   = shift;
    return $self->get_edit_control_html({ops=>['delete']});
}



# Generates the form fields. This is for backwards compatibility with
# Model 1 code
sub generate_form_field_html {
    my $self = shift;

    my ($fields) = $self->define_form_fields();
    return Metadot::Form3000::Form->render_form_fields_html({
        'fields' => $fields,
    });
}




=head2 process_edit_form()

processes form input and stores the values to self
returns the form object that resulted.

=cut
sub process_edit_form {
    my $self = shift;
    my $input = \%FORM;

    my @mappings = @{$self->get_property_mappings()};

    my ($fields, $values) = $self->define_form_fields();
    my $form = Metadot::Form3000::Form->new(
        fields      => $fields,
        values      => $values,
        mappings    => $self->get_property_mappings,
    );

    $form->process_input($input);

    if ($form->has_errors()) {
        $self->debug_msg(3, "Form processed an error.");
    } else {
        # apply the values to myself
        my $values = $form->values();
        foreach my $mapping (@mappings) {
            my $form_field = $mapping->generate_form_field();
            my $property   = $mapping->mapped_property();
            my $form_key   = $form_field->form_key();

            if ($self->can($property)) {
                my $value = $form->get_value($form_key);
                $self->debug_msg(5, "process_edit_form sets $property to $value");
                $self->$property($value);
            }
        }
    }

    return $form;
}



# Access control methods
#-----------------------------------------------------------------

# to conform with the Auditable interface. get_AC returns a permissions
# hash that can be used for the default permissions of all instances
sub get_AC {
    my $self = shift;

    unless (ref $self) {
        #my ($package, $filename, $line) = caller;
        #print STDERR "$self\->get_AC called in class context by $package:$line\n";
    }

    return $self->get_descriptor()->default_permissions();
}


=head2 customize_permissions()

This method allows any Gizmo3000 to create custom bundles and operations
for itself. The first argument is a Permissions object which the Gizmo
should customize. This Permissions object will then be used to determine
the default permissions of a Gizmo.



Notes:
- this is a class method
- the ClassBuilder will automatically call initialize_permission on SUPER
- always return the initialized permissions object

=cut
sub customize_permissions {
    my $class = shift;
    my $perms = shift || Operations->new();

    #print STDERR '<'.__PACKAGE__ ."> $class\->customize_permissions($perms)\n";

    $perms->add_bundle('DISP', 'View', '0', '0');
    $perms->modify_bundle('DISP', 'DEFAULTS_TO', 'on' );
    $perms->add_op('DISP','show');
    $perms->add_op('DISP','show_summary');
    $perms->add_op('DISP','show_in_column');
    $perms->add_op('DISP','download_file');
    $perms->add_op('DISP', 'create_shortcut');

    $perms->add_bundle('MOD', 'Edit', '8', '2');
    $perms->add_op('MOD', 'create');
    $perms->add_op('MOD', 'modify');
    $perms->add_op('MOD', 'save');
    $perms->add_op('MOD', 'delfile');
    $perms->add_op('MOD', 'delfileok');
    $perms->add_op('MOD', 'modify_properties');
    $perms->add_op('MOD', 'save_properties');

    $perms->add_bundle('DEL', 'Delete and Cut', '8', '8');
    $perms->add_op('DEL', 'delete');
    $perms->add_op('DEL', 'delete_ok');
    $perms->add_op('DEL', 'cut');
    $perms->add_op_parent_dependency('DEL', 'cut', 'MOD');

    $perms->add_bundle('EDITP', 'Change<br />Permissions', '8', '8');
    $perms->add_op('EDITP', 'edit_permissions');
    $perms->add_op('EDITP', 'set_permissions');
    $perms->add_op('EDITP', 'change_owner');

    return $perms;
}



=head2 can_be_added_to()

Determines if a Gizmo3000 class can be added to a particular class of container.
By default, any Gizmo3000 class can be added to any container. This is limited
by prohibited_containers and allowed_containers in the descriptor. If neither
has a value, we also check is_dedicated_child

=cut
sub can_be_added_to {
    my $class       = shift;
    my $container   = shift || die('Must specify the container');

    my $container_class         = ref($container) || $container;
    my $descriptor              = $class->get_descriptor();
    my $allowed_containers      = $descriptor->allowed_containers();
    my $prohibited_containers   = $descriptor->prohibited_containers();

    my $result = 1;

    # check if the container is prohibited
    if (defined $prohibited_containers) {
        $result = !Utility->array_contains($container_class, $prohibited_containers);
    }

    if ($result) {
        # check if the container is not in the allowed containers. If that is
        # not defined, then by default all containers are allowed
        if (defined $allowed_containers) {
            $result = Utility->array_contains($container_class, $allowed_containers);
        }
    }

    return $result;
}





=head2 class_hierarchy()

returns the Gizmo3000 class hierarchy for a given class

=cut
sub class_hierarchy {
    my $class       = shift;
    my $args  = shift || {};

    my $inclusive   = $args->{inclusive};
    my $base_class  = $args->{base_class} || __PACKAGE__;

    return Metadot::ClassUtilities->ancestor_hierarchy(
        'class'      => $class,
        'base_class' => $base_class,
        'inclusive'  => $inclusive,
    );
}


=head2 customize_descriptor()

at initialization time, each class has a chance to customize its descriptor

=cut
sub customize_descriptor {
    my $class       = shift;
    my $descriptor  = shift;
}





=head2 generate_standard_wrapper()

creates a standard Metadot wrapper compatible with Metadot skins

=cut
sub generate_standard_wrapper {
    my $self    = shift;
    my $content = shift || '';

    my $result = <<EOM
[% WRAPPER gizmo3000.tt2 %]
$content
[% END %]
EOM
;

    return $result;
}





=head2 build_columns()

builds column hashes for use in reports and tables

=cut
sub build_columns {
    my $self            = shift;
    my $property_names  = shift || [];
    my @results = ();

    $DISPLAY->debug_msg(4, "$self\->build_columns with ".join(',',@$property_names));

    my $property_map = $self->get_property_map();
    $DISPLAY->debug_msg(5, "$self\->build_columns has property map $property_map");

    my $sort_order = 1;
    foreach my $property_name (@$property_names) {
        my $mapping = $property_map->{$property_name};
        $DISPLAY->debug_msg(5, "$self\->build_columns has examines mapping $mapping for property name '$property_name'");

        if (defined $mapping) {
            my $column = {
                name            => $mapping->label(),
                field           => $property_name,
                sort_order      => undef,
                sort_priority   => undef,
            };

            push @results, $column;
            $DISPLAY->debug_msg(5, "$self\->build_columns built column $column for field $property_name");
        }
    }

    return \@results;
}







# file attachment methods
#-----------------------------------------------------------------------

=head2 process_uploads()

process all uploads in a form

=cut
sub process_uploads {
    my $self = shift;
    my $args = shift || {};
    my $input = $args->{input} || \%FORM;
    my $form  = $args->{form} || die('Please supply a form variable');

    foreach my $mapping (@{$self->get_property_mappings()}) {
        my $form_field = $mapping->generate_form_field();
        my $property   = $mapping->mapped_property();
        my $field_name = $mapping->property_name();

        if ($self->can($property)) {
            # if the field was of type 'file', then we may need to
            # process an upload
            $DISPLAY->debug_msg(4, "Checking if we need to process an upload with $form_field");
           if ($form_field->isa('Metadot::Form3000::Fields::FileField')) {
               # prepare these args for sending to uploads manager
               my $upload_manager_args = {
                   write_dir    => $self->get_write_dir_namespace($field_name),
                   isa          => $self->is_a(),
                   id           => $self->id(),
                   field_name   => $field_name,
               };

               # if a delete was requested
               if ($form_field->is_delete_requested($input)) {
                   Metadot::UploadsManager->delete_upload($upload_manager_args);
                   $DISPLAY->debug_msg(4, "Deleted upload for $form_field");
               }

               # if a new upload was submitted
               if ($form_field->has_new_upload($input)) {
                   # determine the upload type (public, access-controlled)
                   my $type = $upload_manager_args->{type}
                            = $form_field->upload_type($input);




                   # transmit the filename
                   my $file_path = $upload_manager_args->{file_path}
                                 = $self->$property();

                   my $err = Metadot::UploadsManager->process_upload($upload_manager_args);

                   ## This block should have used UploadsManager->process_upload(), which
                   ## has all the same logic.

                   #if ($type eq Metadot::UploadsManager::UPLOAD_TYPE_PUBLIC()) {
                   #    # upload to the public dir
                   #    $err = Metadot::UploadsManager->upload_public_file( $upload_manager_args );
                   #} else {
                   #    # upload to the access controlled dir
                   #    $err = Metadot::UploadsManager->upload_access_controlled_file($upload_manager_args);
                   #}







                   $DISPLAY->debug_msg(4, "Uploading $type file for $field_name, $file_path; error? $err");

                   if ($err) {
                       $form->add_error($err);
                   }
               }
            }
        }
    }
}



sub copy_uploads {
    my ($self, $target) = @_;

    $self->SUPER::copy_uploads($target);

    foreach my $mapping (@{$self->get_property_mappings()}) {
        my $form_field = $mapping->generate_form_field();
        my $property   = $mapping->mapped_property();
        my $field_name = $mapping->property_name();

        if ($self->can($property)) {
            # if the field was of type 'file', then we may need to
            # process an upload
            $DISPLAY->debug_msg(4, "Checking if we need to process an upload with $form_field");
           if ($form_field->isa('Metadot::Form3000::Fields::FileField')) {

               my $error = $self->copy_upload(
                   {
                        write_dir    => $self->get_write_dir_namespace($field_name),
                        isa          => $self->is_a(),
                        id           => $self->id(),
                        field_name   => $field_name,
                   },
                   {
                       write_dir    => $target->get_write_dir_namespace($field_name),
                       isa          => $target->is_a(),
                       id           => $target->id(),
                       field_name   => $field_name,
                   },
               );

               if ($error) {
                   print STDERR "Error occurred while copying file: $error\n";
               }
            }
        }
    }
}


=head2 get_write_dir_namespace(field_name)

returns the directory where we store a file attachment for a given field

=cut
sub get_write_dir_namespace {
    my $self       = shift;
    my $field_name = shift;

    my $dir_separator = '/';

    my $result = $PARAMS->{absolutepathfileuploaddir}
                 .$dir_separator
                 .$self->id();

    if ($field_name) {
        $result .= $dir_separator . $field_name;
    }

    return $result;
}

=head2 get_file_info({ field_name=> 'some_field_name'})

returns an information hash about a file attachment

=cut
sub get_file_info {
    my $self = shift;
    my $args = shift || {};

    defined ($args->{field_name}) || die('Must specify a field_name');
    $args->{id}  = $self->id();
    $args->{isa} = $self->is_a();

    return Metadot::UploadsManager->get_file_info($args);
}

=head2 get_file_url(field_name)

returns the url of the file

=cut
sub get_file_url {
    my $self = shift;
    my $info = $self->get_file_info(@_);
    return $info->{url};
}



=head2 delete_attachments()

deletes all attachments for an instance

=cut
sub delete_attachments {
    my $self = shift;

    # delete all my attachment fields
    foreach my $mapping (@{$self->get_property_mappings()}) {
        my $form_field = $mapping->generate_form_field();
        if ($form_field->isa('Metadot::Form3000::Fields::FileField')) {
            my $property = $mapping->property_name();
            Metadot::UploadsManager->delete_upload({
                field_name  => $property,
                write_dir   => $self->get_write_dir_namespace($property),
                isa         => $self->is_a(),
                id          => $self->id(),
            });
            $self->$property(undef);
        }
    }

    # delete my upload directory
    Utility->_delete_directory($self->get_write_dir_namespace());
}










# search methods
#-----------------------------------------------------------------------

# return a record that tells the gizmo what data to search against.
# here is an example record:
# {
#   searchable_fields   => [ qw( name description keywords) ],
#   displayed_fields    => [ qw( description) ],
#   searchable_children => [
#       {
#           class              => 'SomeChildClass',
#           searchable_fields  => [ qw( name description t1 ) ],
#           displayed_fields   => [ qw( description t1 ) ],
#       },
#   ],
# }

sub get_search_params {
    my $class = shift;
    my $result;
    my $class_descriptor = $class->get_descriptor();

    $DISPLAY->debug_msg(3, "$class\->get_search_params starts with descriptor $class_descriptor");
    if ($class_descriptor->is_searchable()) {
        # get all the searchable and displayable properties
        my @properties = @{$class_descriptor->get_property_mappings()};
        my @searchable_fields = ();
        my @displayed_fields  = ();
        foreach my $property (@properties) {
            $DISPLAY->debug_msg(4, "checking if '$property' is searchable");
            if ($property->is_searchable()) {
                my $field_name = $property->mapped_property();
                $DISPLAY->debug_msg(4, "'$property' is searchable. added field '$field_name'");

                push @searchable_fields, $field_name;
                if ($property->display_in_search()) {
                    push @displayed_fields, $field_name;
                }
            }
        }

        # if no displayed properties were defined, go with the default
        unless (@displayed_fields) {
          @displayed_fields = qw(description);
        }

        # define the result
        $result = {
            class => ref($class) || $class,
            searchable_fields => \@searchable_fields,
            displayed_fields  => \@displayed_fields,
        };


        # if I have child types, add them
        if (defined $class_descriptor->child_types()) {
            my @searchable_children = ();
            foreach my $child_type (@{$class_descriptor->child_types()}) {
                $DISPLAY->debug_msg(4, "recursively getting search params for child $child_type");
                push @searchable_children, $child_type->get_search_params();
            }
            $result->{searchable_children} = \@searchable_children;
        }
    }

    return $result;
}






# private methods
#-----------------------------------------------------------------------

# Uses the property mappings to create a set of new form fields.
sub define_form_fields {
    my $self = shift;

    my @fields = ();
    my %values = ();

    foreach my $mapping (@{$self->get_property_mappings()}) {
        my $property_name = $mapping->property_name();
        my $form_field    = $mapping->generate_form_field();

        push @fields, $form_field;

        # if the field was of type 'file', then we may need to
        # add file info
        if ($form_field->isa('Metadot::Form3000::Fields::FileField')) {
            $form_field->file_info($self->get_file_info({ field_name => $property_name }));
        }

        my $form_key = $form_field->form_key();
        if (defined $property_name && $self->can($property_name)) {
            # TT returns a false 'defined' if the key exists but is
            # defined, so I just leave the key out if I don't want it
            if (exists $FORM{$form_key}) {
                $values{$form_key} = $FORM{$form_key};
                $DISPLAY->debug_msg(5, "using existing form value $form_key = $FORM{$form_key}");
            } elsif (defined $self->$property_name()) {
                $values{$form_key} = $self->$property_name();
                $DISPLAY->debug_msg(5, "using existing property $property_name = ".$self->$property_name());
            }
        }
    }

    return \@fields, \%values;
}










# Aliases for backwards compatibility with older Gizmos
#----------------------------------------------------------
sub get_buttons {
    my $self = shift;
    my $result = $self->get_edit_control_html()
                 . $self->show_scheduling_buttons;

    return $result;
}

sub show_edit_properties_form {
    return Gizmo::show_edit_properties_form(@_);
}

sub save_properties {
    return Gizmo::save_properties(@_);
}

sub do_create_shortcut {
    return GizmoBuilder::www_create_shortcut(@_);
}


# forward all unsupported operations to Gizmo (crossing fingers!)
use Gizmo;
sub handle {
    my $self = shift;
    my $op   = shift;
    my $suppress_error = shift;

    $DISPLAY->debug_msg(3, "$self\->handle($op)");
    ## for HTML RENDERING
    my $handler = "www_$op";

    # aju - add check that the op method is defined to trap a possible server error.
    if (Gizmo->can($handler)) {
        no strict 'refs';
        &{"Gizmo::${handler}"}($self);
        return 1;
    }
    elsif (!$suppress_error) {
        $DISPLAY->print_click_back('Operation not supported');
        exit;
    }
}










1;
