package Metadot::Gizmo3000::ClassDescriptor;

use strict;

use MetadotObject;
use vars qw(@ISA);
@ISA = qw(MetadotObject);


use constant MO_INSTANCE_PROPERTIES => {
    name                    => undef,
    version                 => undef,
    description             => undef,
    author                  => undef,
    info                    => undef,
    need_portal_version     => undef,
    debug                   => undef,
    edit_help               => undef,
    default_permissions     => undef,
    class_name              => undef,
    is_searchable           => 1,
    _property_mappings      => undef,
    
    child_types             => undef,
    prohibited_child_types  => undef,

    allowed_containers      => undef,   # by default all containers are permitted
                                        # to hold this gizmo.

    prohibited_containers   => undef,   # by default no containers are prohibited
                                        # from holding this gizmo

    is_dedicated_child      => 0,       # causes the gizmo to lose some of its
                                        # privileges, like access control

};


sub initialize {
    my $self = shift;
    my $args = shift || {};

    $self->SUPER::initialize($args);
    # initialize the internal fields array
    $self->_property_mappings([]);
}

# this is a list of hashtables where
#  key   = 'store_at_column', and
#  value = associated Metadot::Form3000::FormField object
sub get_property_mappings {
    my $self = shift;
    return $self->_property_mappings();
}


# this is a hashtable where
#  key   = property name
#  value = PropertyMapping object for that property
sub get_property_map {
    my $self = shift;
    my $mappings = $self->get_property_mappings();
    my %result = ();

    foreach my $mapping (@$mappings) {
        my $key = $mapping->property_name();
        #print STDERR "Adding mapping $mapping with key '$key'\n";
        $result{$key} = $mapping;
    }

    return \%result;
}



# adds a Metadot::Form3000::PropertyMapping object to this class
sub add_property_mapping {
    my $self    = shift;
    my $mapping = shift;

    (defined $mapping && $mapping->isa('Metadot::Form3000::PropertyMapping'))
        || die('Must provide an instance of Metadot::Form3000::PropertyMapping');


    push @{$self->_property_mappings()}, $mapping;
}






1;





