#######################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Metadot::Gizmo3000::IconEditControl;

use strict;

use Metadot qw($CGI);
use Metadot::Gizmo3000::EditControl;

use vars qw(@ISA);
@ISA = qw(Metadot::Gizmo3000::EditControl);

use constant MO_INSTANCE_PROPERTIES => {
    icon_url     => undef,  # the URL, if any, of the icon to be used
    };


# renders the control as an HTML icon
sub render_html {
    my $self = shift;

    my $icon_url = $self->icon_url();
    my $alt_text = $self->description();
    my $href     = $self->generate_href();

    my $html = <<EOM
    <a href="$href" class="icon">
      <img class="icon" src="$icon_url" alt="$alt_text" />
    </a>
EOM
;
}


sub generate_href {
    my $self = shift;

    my $iid = $self->actor_iid();
    my $op  = $self->operation();
    my $target_iid = $self->target_iid();

    my $href = qq~index.pl?iid=$iid&amp;op=$op~;
    if (defined $target_iid) {
        $href .= qq~&amp;target_iid=$target_iid~;
    }

    while (my ($key, $value) = each %{ $self->arguments() }) {
        $key   = $CGI->escape($key);
        $value = $CGI->escape($value);
        $href .= qq~&amp;$key=$value~;
    }

    return $href;
}



1;
