###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::GizmoContainer;

use vars qw(@ISA);

use Gizmo;
use GizmoBuilder;
use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY
                $CACHE $CGI $ACCESS_BROKER %REQUEST_VARS );
use Metadot::PortalFactory;
use Metadot::BaseGizmoContainer;
use Gizmo::Category::Clipboard;
use Gizmo::SuperGizmo::SuperGizmoRecord;
use strict;

@ISA = qw(GizmoBuilder Metadot::BaseGizmoContainer);

# this is an unfortunate thing, but it allows Gizmo containers to maintain different
# address spaces for their children
use constant INSTANCE_ORDER_ADDRESS_PAGE_SIZE => 10000;

sub customize_permissions {
   my $class = shift;
   my $AC    = shift;

   $class->SUPER::customize_permissions($AC);

   $AC->add_op('MOD', 'add_child');
   $AC->add_op('MOD', 'create_child');
   $AC->add_op('MOD', 'paste');
   $AC->add_op('MOD', 'move_child_up_down');
   $AC->add_op('MOD', 'cut_child');
}



sub www_add_child {
    my $self = shift;
    my $errormsg = shift;
    my $child_type = shift;
    $self->debug_msg(3, "GizmoContainer::www_add_child()");

    my $child_isa = $FORM{child_type} || $child_type;
    my $parent_isa = $FORM{isa} || $self->is_a();
    my $gizmo_name = '';

    my $child_constructor_args = {};
    my $supergizmo_id;
    if ($child_isa =~ /^SuperGizmoRecord/) {
        ($child_isa, $gizmo_name, $child_constructor_args) = SuperGizmo->get_www_add_child_args( { child_isa => $child_isa, child_constructor_args => $child_constructor_args } );
    } else {
        $gizmo_name = $child_isa->get_gizmo_name();
    }

    $self->debug_msg(4, "GizmoContainer::www_add_child() has parent_isa $parent_isa, child_isa $child_isa");


    my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");
    # we re-instanciate the object because we want a brand new one
    # unless there is an error message (we want to keep his data)
    $child_constructor_args->{parent_id} = $parentid;

    my $child = $child_isa->new($child_constructor_args);

    $self->debug_msg(4, "GizmoContainer::www_add_child() has created child object: $child");


    # different handling for Gizmo3000
    if ($child->isa('Metadot::Gizmo3000')) {
        $DISPLAY->debug_msg(4, "Passing control to output_add_child_form($child, $errormsg)");
        return Metadot::Gizmo3000::Container::output_add_child_form($self, $child);
    } else {
        # the default handling for legacy gizmos
        my $form;

        $errormsg = $errormsg ?
          "<p /><h2>Please correct the following error(s):</h2>\n<font color=red>$errormsg</font><p />\n" :
            "";

        $form = $child->show_edit_form("Adding New $gizmo_name");
        # JSForm
        if (defined($self->{jsform}) && defined($child->{jsform})) {
            $self->{jsform}->add_check_array($child->{jsform}->get_check_array()) ;
        }
        my $form_tag = "<form name=fm action='index.pl' method='post' ENCTYPE=\"multipart/form-data\">" ;
        my $cancel_button = "<input type=\"button\" onClick=\"location.href='index.pl?iid=$parentid&amp;isa=$parent_isa'\" value=\"Cancel\">";
        my $content =  defined($self->{jsform}) ? $self->{jsform}->get_form($form_tag, $self->{jsform}->get_ALL_ERRORS()) : $form_tag ;
        # JSForm
        $content .= "
    <input type='hidden' name='isa' value='$parent_isa'>\n
    <input type='hidden' name='iid' value='$parentid'>\n
    <input type='hidden' name='op' value='create_child'>\n
    <input type='hidden' name='child_type' value='$child_isa'>\n
    ".$self->make_hidden()."
    <table align='right'><tr><td><input type='submit' value='Create' class='button'>&nbsp;$cancel_button</td></tr></table>
    $errormsg
    $form
    <table align='right'><tr><td><input type='submit' value='Create' class='button'>&nbsp;$cancel_button</td></tr></table>
    <br />&nbsp;
    </form>
    ";


      $DISPLAY->print_form($content, "Adding new $gizmo_name");
    }
}


sub www_create_child {
  my $self = shift;
  my $child_type = shift;

  $DISPLAY->debug_msg(3, "GizmoContainer::www_create_child(); container $self;  id=".$self->id());

  my $child_isa = $FORM{child_type} || $child_type;

  my $parentid = $self->defined_or_exit($self->id,"parent id is missing.");

  my $child_constructor_args = {};
  if ($child_isa eq 'SuperGizmoRecord') {
      my $supergizmo_parent_iid_field = $child_isa->get_supergizmo_parent_iid_field();
      $child_constructor_args->{$supergizmo_parent_iid_field} = $FORM{$supergizmo_parent_iid_field};
  }
  $child_constructor_args->{uid} = $USER->uid();
  $child_constructor_args->{parent_id} = $parentid;
  # we re-instanciate the object because we want a brand new one
  my $child = $child_isa->new($child_constructor_args);


  # Gizmo3000 doesn't do all the post-processing stuff.
  if ($child->isa('Metadot::Gizmo3000')) {
      my $form = $child->process_edit_form();
      if ($form->has_errors()) {
          $DISPLAY->debug_msg(4, "www_create_child processed an error in the form");
          return Metadot::Gizmo3000::Container::output_add_child_form($self, $child);
      } else {
          $DISPLAY->debug_msg(4, "www_create_child successfully processed the form");
          $child->instance_order($self->next_vertical_position());
          $child->add();

          # have to process uploads as a second phase, because UploadsManager
          # requires an existing iid
          $child->process_uploads({form => $form});
          if ($form->has_errors()) {
              $child->delete();
              return Metadot::Gizmo3000::Container::output_add_child_form($self, $child);
          } else {
              return $self->show_create_child_success($child);
          }
      }
  } else {
      # this method should ALWAYS be overridden in derived classes.
      $child->process_edit_form();
      return $child;
  }

}





# this gets all children, regardless of permission
sub get_all_childs_list {
    my $self = shift;
    my $args = shift;

    my @result = $self->build_gizmo_list(%{$args});

    if (wantarray) {
        return @result;
    } else {
        return \@result;
    }
}



# this is now an alias for get_children
sub get_childs_list {
    my $self = shift;
    my $args = shift;

    return $self->get_children($args);
}


# recursively checks if this container has any content that allows
# a given op for a given user
#
# NOTE: the recursion does not follow Navigable children
sub user_has_permission_for_children {
    my $self    = shift;
    my $op      = shift || die ("Must specify op");
    my $user    = shift || $USER;

    return $self->recursive_boolean_check_on_children({
        'method'        => 'is_allowed_to_do',
        'method_args'   => [ 'modify', $user ],
        });
}


# recursively checks for a boolean TRUE for any of a container's children
# against a specific method and \@method_args.
#
# NOTE: the recursion does not follow Navigable children
sub recursive_boolean_check_on_children {
    my $self = shift;
    my $args = shift || {};
    my $visited = shift || {};

    my $method      = $args->{'method'}      || die("Must specify arg method");
    my $method_args = $args->{'method_args'} || [];


    my $result = 0;
    # check all the childs. short-circuit as soon as one is found
    my $child_list = $self->get_childs_list();
    foreach my $child (@$child_list) {

        # prevent deep recursion
        next if (defined $visited->{$child->get_id()});
        $visited->{$child->get_id()} = 1;

        # check permission
        $result ||= ($child->can($method) && $child->$method(@$method_args));
        last if ($result);

        # check recursively if the child is a container, but NOT if
        # the child is a navigable
        if ($child->isa('Metadot::GizmoContainer') && !$child->isa('Metadot::Navigable')) {
            $result ||= $child->recursive_boolean_check_on_children($args, $visited);
        }
    }

    return $result;

}

# recursively checks if this container has any scheduled content that allows
# a given op
#
# NOTE: the recursion does not follow Navigable children
sub children_have_scheduled_content_for_user {
    my $self    = shift;
    my $user    = shift || $USER;

    return $self->recursive_boolean_check_on_children({
        'method'        => 'user_has_scheduled_content',
        'method_args'   => [ $user ],
        });
}

#########################################
sub show_childs_list {
    my $self = shift;
    my %args = @_;

    my $id = $self->{id};
    $self->debug_msg(3, "GizmoContainer::show_childs_list(id=$id)");

    my $row_color_on = "#FFFFFF";
    my $row_color_off = "#EEEEEE";
    my $row_color = $row_color_on;

    my $str = "";
    my @allowed_list = @{$self->get_childs_list(\%args)};
    # run the renderer for each gizmo
    foreach my $gizmo (@allowed_list) {
        my $renderer = "show_summary";

        if ($gizmo->is_a() eq 'Category' && $gizmo->get_image_url('display_image')) {
            $renderer = "show_as_roll_over";
        }

        $self->debug_msg(4, "GizmoContainer::show_childs_list renders $gizmo ($gizmo->{id} with '$renderer'");

        my $decorator = $gizmo->gizmo_list_decorator || 'HTML::Decorator::BulletDecorator';
        $row_color = ($row_color eq $row_color_on) ? $row_color_off : $row_color_on;
        my $str_from_decorator = $decorator->new(component=>$gizmo, background_color=>$row_color)->$renderer($id);
        $str .= $str_from_decorator if (defined $str_from_decorator);

        $self->debug_msg(4, "GizmoContainer::show_childs_list finished rendering $gizmo");
    }

    # if gizmos wish to display postscript text, allow them to do so afterwards
    foreach my $gizmo (@allowed_list) {
        if ($gizmo->can('show_postscript')) {
            $str .= $gizmo->show_postscript();
        }
    }

    return $str ;

}


sub www_delete {
  my $self = shift;
  my $name = $self->get_gizmo_name();
  my $isa = $self->get_is_a();
  $self->debug_msg(3, "GizmoContainer::www_delete()");

  my $iid = $self->defined_or_exit($self->id,"id is missing.");

  $self->three_choices("Delete This $name:".$self->name()."?",
                       "",
                       "Yes and all items under it as well",
                       "index.pl?iid=$iid&isa=$isa&op=delete_ok&deleteson=YES",
                       "Yes but only this $name, promote all other elements",
                       "index.pl?iid=$iid&isa=$isa&op=delete_ok&deleteson=NO",
                       "No!", $SESSION->{referer});
}

sub www_delete_ok {
  my $self = shift;
  $self->debug_msg(3, "GizmoContainer::www_delete_ok()");
  my $parent_id = $self->parent_id;
  unless ($self->delete($FORM{deleteson})) {
      $self->click_back('Error deleting.');
      exit(0);
  }

  $self->md_redirect("index.pl?iid=" . $parent_id);
}

#########################################
sub delete {
    my $self = shift;
    my $deleteson=shift;
    my $id = $self->id;
    $self->debug_msg(3, "Metadot::GizmoContainer::delete(id=$id)") ;

    if ( !defined($id) || $id==0 ){
        #cannot delete top cat=0);
        $self->debug_msg(4, "Oops: cannot delete container $id");
        return 0;
    }



    ### delete channel
    ### first get cid
    my ($cid) = DBAccess->sqlSelect("cid", "channel", "iid=$id and isa='Internal'");
    if ($cid) {
        my $channel = Channel->new($cid);
        $channel->delete();
    }

    if (uc($deleteson) eq "YES"){
        $self->delete_children();
    } else {
        ### upgrade sons to new parent
        unless ($self->promote_children()) {
            $self->debug_msg(4, "Metadot::GizmoContainer::delete(id=$id) could not promote children");
            return 0;
        }
        #my ($parentiid) = $self->parent_id;
        #DBAccess->sqlUpdate("instance", "parentiid=$id", parentiid => $parentiid);
    }

    $self->reorder_parent;
    $self->SUPER::delete();
    return 1;
}



## will move all children of instance to instance's parent
sub promote_children {
    my $self = shift;
    my $id = $self->id;
    $self->debug_msg(3, "Metadot::GizmoContainer::promote_children(id=$id)") ;

    my $parent = $self->get_parent();
    my $err_msg = $parent->move_children_into(current_iid => $id);
    if ($err_msg ne '') {
        $self->debug_msg(4, "Metadot::GizmoContainer::promote_children - err_msg:$err_msg") ;
        return 0;
    } else {
        return 1;
    }
}




# add a 'paste into' button.
sub get_buttons {
    my $self = shift;
    my $id = $self->get_id();

    my $buttons;

    if (defined $id) {
        $buttons .= $self->SUPER::get_buttons( {add_panel=>$self->get_paste_into_button()});
        } else {
        $buttons = $self->SUPER::get_buttons();
    }
    return $buttons;
}



sub get_paste_into_button {
    my $self = shift;
    my $id = $self->get_id();

    my $result;

    if ($SESSION->admin_mode()) {
        my $href    = Clipboard->generate_paste_into_link($id);

        if ($href) {
            my $img_src = "$PARAMS->{'imageslocation'}/obj/paste.gif";
            $result = "<a href=\"$href\"><img src=\"$img_src\" class=\"icon\" border=\"0\" /></a>";
        }
    }

    #print STDERR "get_paste_into_button returns: $result\n";
    return $result;
}


sub generate_add_child_selection_list {
    my $self = shift;

    my $result = '';

    my $iid = $self->get_iid();
    my $isa = $self->get_is_a();

    my @gizmo_list = $self->get_add_child_list();
    my %gizmo_name_list = ();
    foreach my $gizmo (@gizmo_list) {
        my $gizmo_name = $gizmo->get_gizmo_name();
        $DISPLAY->debug_msg(5, "Adding $gizmo_name to child selection list");
        $gizmo_name_list{$gizmo} = $gizmo_name;
    }

    ## GET THE SUPERGIZMOS
    $DISPLAY->debug_msg(5, "Merging gizmo name list with supergizmos hash");
    Utility->merge_hashes(\%gizmo_name_list, SuperGizmo->get_add_supergizmos_hash());

    ## GET THE GIZMO3000'S
    $DISPLAY->debug_msg(5, "Merging gizmo name list with gizmo3000 hash");
    Utility->merge_hashes(\%gizmo_name_list, $self->get_add_gizmo3000_hash());



    $result .= <<EOM
    <select name = "child_type" style="font-size: smaller;" onChange="javascript:submitMe(this, '$iid', '$isa', 'add_child')">
      <option value="0" selected>Add a gizmo...
EOM
;


    $DISPLAY->debug_msg(5, "Sorting and outputting child list");
    foreach my $gizmo (sort { $gizmo_name_list{$a} cmp $gizmo_name_list{$b} } keys %gizmo_name_list) {
        $result .= "<option value=\"$gizmo\">" . $gizmo_name_list{$gizmo}  . "\n";
    }
    $result .= "</select>";

    return $result;
}


sub get_add_gizmo3000_hash {
    my $self = shift;
    my $registered_modules = Metadot::Gizmo3000::ClassRegistry->get_instance()->get_registered_modules();
    my $result = {};

    while (my ($class, $descriptor) = each %$registered_modules) {
        if (!$descriptor->is_dedicated_child() && $class->can_be_added_to($self)) {
            $result->{$class} = $descriptor->name();
        }
    }

    return $result;
}


sub get_add_child_list {
    my $self = shift;
    my @result;

    # first get the list of available gizmo types
    my @types = Gizmo->filtered_gizmos();

    # add the type to the list if both child and container agree
    for my $type (@types) {
        if ($type->is_visible_in_add_child_lists() && $self->is_child_type_allowed($type)) {
            push @result, $type;
        }
    }

    if (wantarray) {
        return @result;
    } else {
        return \@result;
    }
}


## move children of Gizmo container into another container
sub move_children_into {
    my $self = shift;
    my %args = @_;

    my $iid = $self->id();
    $self->debug_msg(3, "Metadot::GizmoContainer::move_children_into(iid=$iid)") ;

    return 'Parent Gizmo Container id not defined' unless defined($iid);

    ## iid of current parent of children you want to move
    my $current_iid = $args{current_iid};

    return 'Gizmo Container current id not defined:$current_iid' unless defined($current_iid);

    $self->debug_msg(4, "Metadot::GizmoContainer::move_children_into(iid=$iid) moving children from old parent $current_iid to new parent $iid") ;
    my $children = DBAccess->sqlSelectMany("iid", "instance", "parentiid = $current_iid");
    while (my($child_iid) = $children->fetchrow) {
        #my $child = $self->get_object_accessor()->retrieve( key=> $child_iid );
        my $child = Gizmo->restore($child_iid);
        $child->{parent_id} = $iid;
        $self->debug_msg(4, "Metadot::GizmoContainer::move_children_into(iid=$iid) moving child $child_iid from old parent $current_iid to new parent $iid") ;
        $child->save();
    }

    #DBAccess->sqlUpdate("instance", "parentiid = $current_iid", parentiid => $iid);
    return '';
}

sub move_children_into_and_delete_old_parent {
    my $self = shift;
    my %args = @_;

    my $current_iid = $args{current_iid};
    return 'Gizmo Container current id not defined:$current_iid' unless defined($current_iid);
    my $iid = $self->id();
    $self->debug_msg(3, "Metadot::GizmoContainer::move_children_into_and_delete_old_parent(iid=$iid)") ;
    my $err_msg = $self->move_children_into(%args);

    if ($err_msg) {
        return $err_msg;
    } else {
        ## if children where moved successfully then delete old parent
        $self->debug_msg(4, "Metadot::GizmoContainer::move_children_into_and_delete_old_parent(): will delete old parent $current_iid");
        #$self->get_object_accessor()->delete(key=>$current_iid);
        my $gizmo = Gizmo->restore($current_iid);
        $gizmo->delete();
        return '';
    }
}



# handle the paste operation
sub paste {
    my $self = shift;
    my $paste_iids = shift;

    foreach my $iid (@$paste_iids) {
        $self->debug_msg(3, "paste: $iid" );

        my $gizmo = Gizmo->restore($iid);

        # shouldn't allow pasting categories onto themselves
        unless( $iid == $self->id() ) {
            $gizmo->parent_id($self->id());
            $gizmo->{instance_order} = $gizmo->{instance_order} + 1000; # try to get it to the bottom of the page
            $gizmo->save();
        }
    }
    $self->reorder_children();
}


# used by some containers, particularly Category, to separate different Gizmo types.
# returns the address page index for a particular gizmo instance or class
sub get_instance_order_address_page {
    my $self  = shift;
    my $child = shift;

    # default is undef
    return undef;
}

# used by some containers, particularly Category, to separate different Gizmo types
sub get_instance_order_address_page_filter {
    my $self  = shift;
    my $child = shift;

    my $result;
    my $address_page = $self->get_instance_order_address_page($child);
    if (defined $address_page) {
        $result = 'instanceorder >= '.($address_page * INSTANCE_ORDER_ADDRESS_PAGE_SIZE).' AND instanceorder < '.(($address_page+1) * INSTANCE_ORDER_ADDRESS_PAGE_SIZE);
    }

    #print STDERR "$self\->get_address_space_filter($child) returns '$result'\n";
    return $result;
}





sub www_move_child_up_down {
    my $self = shift;

    my $delta = $FORM{delta};
    my $child_id = $FORM{target_iid};

    $self->move_child_up_down(Gizmo->restore($child_id), $delta);
    $DISPLAY->md_redirect($SESSION->referer());
}

# when reordering children, allow child classes to adjust the filter
sub _adjust_child_reordering_filter {
    my $self   = shift;
    my $child  = shift;
    my $filter = shift;

    # this makes sure that last_one and first_one are in a given address space
    my $address_page_filter = $self->get_instance_order_address_page_filter($child);

    if ($address_page_filter) {
        $filter .= ' AND '.$address_page_filter;
    }

    return $filter;
}



sub www_cut_child {
  my $self = shift;

  my $child_id = $FORM{target_iid};

  my $child = Metadot::GizmoBase->restore($child_id);

  my $return_url;
  # Determine the return URL.
  # If the child is a navigable, we redirect to the parent, in order to
  # avoid re-displaying the child
  if ($child->isa('Metadot::Navigable'))
  {
      my $parent_id = $FORM{iid};
      if (defined $parent_id)
      {
          $DISPLAY->debug_msg(5, "Will redirect to FORM value of $parent_id");
      }
      else
      {
          $parent_id = $child->parent_id();
          $DISPLAY->debug_msg(5, "Will redirect to parent_id of $parent_id");
      }
      $return_url = "index.pl?iid=$parent_id";
  }
  else
  {
      $return_url = $SESSION->referer();
      $DISPLAY->debug_msg(5, "Will redirect to referrer url $return_url");
  }



  # cut the child
  $self->cut_child($child);

  # redirect the user
  $DISPLAY->md_redirect($return_url);
}




sub cut_child {
    my $self = shift;
    my $child = shift || die('Must specify the child to move');

    # make sure child is mine
    ($child->parent_id() == $self->id())
        || die('This is not my child! My iid is '.$self->id()
               .' and child\'s parent is '.$child->parent_id());

    # clipboard is a Category object
    $child->parent_id($USER->get_clipboard_iid());
    $child->instance_order(100000);
    $child->save();

    $self->reorder_children();
}












1;
