# This package allows Gizmo classes to implement the Singleton design pattern.
# It's not strictly a Singleton in terms of objects in memory, but it
# tries to ensure that only one particular instance of a gizmo exists in any given
# database. Note that it's possible for two server instances to simultaneously
# create their own Singletons.

package Metadot::GizmoSingleton;

use strict;


# gets a new or existing singleton instance. Can specify a target class as well.
sub get_instance {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $target_class = shift || $class;

    my $iid = $class->get_instance_iid($target_class);
    my $result;

    if (defined $iid) {
        $result = Gizmo->restore($iid);
    } else {
        $result = $class->new();
        $result->add();
    }
    return $result;
}


# gets an existing instance_iid for a given class.
sub get_instance_iid {
        my $proto = shift;
        my $class = ref($proto) || $proto;
        my $target_class = shift || $class;
        my ($iid) = DBAccess->sqlSelect("iid", "instance", "isa='$target_class'");
        return $iid;
}



1;
