package Metadot::Implementations::Group::Default;

use strict;
use vars qw(@ISA);

use Metadot::Implementations::Group::AbstractGroupImplementation;
use MetadotConfig;
use Metadot::ObjectAccess::ObjectAccessor;

@ISA=qw(Metadot::Implementations::Group::AbstractGroupImplementation );

use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $CACHE $CGI $ACCESS_BROKER $DISPLAY);

my $AC = {
       BUNDLES => {
                   MOD =>    { DESC =>          'Edit',
                               OPS =>            [qw(edit_groups)],
                               DEFAULT_LEVEL =>  8,
                               MIN_LEVEL =>      8,
                             },
                   DEL =>    { DESC =>          'Delete and Cut',
                               OPS =>            [qw(delete_group)],
                               DEFAULT_LEVEL =>  8,
                               MIN_LEVEL =>      8,
                               PARENT_DEP =>     {cut => 'MOD'},
                             },
                   DISP =>   { DESC =>          'View',
                               OPS =>            [qw(show_groups show_all_groups verify_change_group_information verify_add_group show_group_members search_add_group_members)],

                               DEFAULT_LEVEL =>  2,
                               MIN_LEVEL =>      2,
                               DEFAULTS_TO =>    'on',
                             },
                   EDITP =>  { DESC =>          'Change<br />Permissions',
                               OPS =>            [qw(edit_permissions set_permissions change_owner)],
                               DEFAULT_LEVEL =>  8,
                               MIN_LEVEL =>      8,
                             },
                  },
          DISPLAY_ORDER => [qw(DISP MOD DEL EDITP)],
        };

sub get_AC {
    return $AC;
}

###################################################
sub validate_or_refresh_cache {
    my $class = shift;
    my $implementor = shift;

    my $db_timestamp = $CACHE->is_dirty('GROUPS_HASH');
    #return if the cache is up to date
    unless ($db_timestamp) {
        # CKIM: this method just sets a class variable. We're deleting this class
        #       variable because of concurrency issues
        # $implementor->set_group_cache($CACHE->get_value('GROUPS_HASH'));
        return undef;
    }

    $DISPLAY->debug_msg(5, "Refreshing the cache");
    my %local_groups = ();
    my $groupinfo;

    # If the cache is defined incrementally, then the system will not load all the group/user
    # information at once. This is useful when this operation is very expensive, such as when
    # over 1,000 users exist.
    my $incremental_cache = MetadotConfig->get_param_value('incremental_group_cache');

    # get grpmembers information
    unless ($incremental_cache) {
    my $groupinfo = DBAccess->sqlSelectMany (
                            "grp.group_name, grp.grp_id, grpmembers.userid, grpmembers.subgrp_id, grpmembers.is_manager",
                            "grpmembers, grp",
                            "grp.grp_id = grpmembers.grp_id");

        while (my ($group_name, $grp_id, $uid, $subgrp_id, $is_manager) = $groupinfo->fetchrow) {
            if ($uid && $uid ne '') {
                $local_groups{BY_GRP_ID}{$grp_id}{MEMBERS}{$uid} = 1;
                $local_groups{BY_GROUP}{$group_name}{MEMBERS}{$uid} = 1;
                $local_groups{BY_USER}{$uid}{'NAME'}{$group_name} = 1;
                $local_groups{BY_USER}{$uid}{'ID'}{$grp_id} = 1;

                #CKIM: Add managers to the hash
                if ($is_manager) {
                    $local_groups{BY_GRP_ID}{$grp_id}{MANAGERS}{$uid} = 1;
                    $local_groups{BY_GROUP}{$group_name}{MANAGERS}{$uid} = 1;
                }
            }
            if ($subgrp_id and $subgrp_id ne '') {
                #print STDERR "got subgrp_id $subgrp_id\n";
                $local_groups{BY_GRP_ID}{$grp_id}{SUBGROUPS}{$subgrp_id} = 1;
                $local_groups{BY_GROUP}{$group_name}{SUBGROUPS}{$subgrp_id} = 1;
                $local_groups{BY_SUBGROUP}{$subgrp_id}{$group_name} = 1;
            }
        }
        $groupinfo->finish();
    }

    # get group information
    unless ($incremental_cache) {
        # also read in group descriptions
        $groupinfo = DBAccess->sqlSelectMany ("group_name, grp_id, userid, is_organization", "grp", "1=1");
        while (my ($group_name, $grp_id, $uid, $is_organization) = $groupinfo->fetchrow) {
            #the group owner is always a member of the group
            if ($uid && $uid ne '') {
                $local_groups{BY_GRP_ID}{$grp_id}{MEMBERS}{$uid} = 1;
                $local_groups{BY_GROUP}{$group_name}{MEMBERS}{$uid} = 1;
                $local_groups{BY_USER}{$uid}{'NAME'}{$group_name} = 1;
                $local_groups{BY_USER}{$uid}{'ID'}{$grp_id} = 1;

                #CKIM: the group owner is always a manager of the group
                $local_groups{BY_GRP_ID}{$grp_id}{MANAGERS}{$uid} = 1;
                $local_groups{BY_GROUP}{$group_name}{MANAGERS}{$uid} = 1;
            }
            $local_groups{BY_GRP_ID}{$grp_id}{GROUP_NAME} = $group_name;
            $local_groups{BY_GRP_ID}{$grp_id}{IS_ORG} = $is_organization;
        }
        $groupinfo->finish();
    }
    $CACHE->set_value('GROUPS_HASH', \%local_groups);
    $CACHE->set_timestamp('GROUPS_HASH', $db_timestamp);

    # CKIM: this method just sets a class variable. We're deleting this class
    #       variable because of concurrency issues
    # $implementor->set_group_cache(\%local_groups);
}


sub _show_view_groups_page {
    my $self   = shift;
    my $implementor = shift;

    my %args = @_;

    my $grp_id = $implementor->{grp_id};
    my $group_name = $implementor->{group_name};

    my $field = $FORM{'field'} ? $FORM{'field'} : 'fullname';
    my $gfield = $FORM{'gfield'} ? $FORM{'gfield'} : 'subgrp_id';
    my $order = $FORM{'order'} ? $FORM{'order'} : 'asc';

    my $trusted_site = $PARAMS->{trusted_site};
    my $show_emails = 0;
    if ($trusted_site || $USER->belongs_to_grp("ADMIN", $USER->{uid})
        || $ACCESS_BROKER->is_allowed_to_do('view_private_email')) {
        $show_emails = 1;
    }

    my $users_in_group = $USER->get_users_in_group( $grp_id, $field, $order );
    my $groups_in_group = $implementor->_get_subgroups_in_group($grp_id, $gfield, $order);

    my $s = $implementor->_build_user_table($users_in_group, $group_name,
                     $grp_id, $show_emails, $field, $order, \%args);
    $s .= $implementor->_build_subgroups_table($groups_in_group, $group_name, $grp_id,
                    $gfield, $order, \%args);
    return $s;

}

my @GROUP_PROPERTIES = qw ( grp_id group_name group_description userid iid );
sub get_group_properties {
    return @GROUP_PROPERTIES;
}

my $OBJECT_ACCESSOR = Metadot::ObjectAccess::ObjectAccessor->new(
   properties    => \@GROUP_PROPERTIES,
   tables        => [ 'grp' ],
   key_property  => 'grp_id',
   owner         => __PACKAGE__,
);

sub get_object_accessor {
   return $OBJECT_ACCESSOR;
}

sub load {
   my ($self, $implementor, $grp_id) = @_;
   my $object = $self->get_object_accessor()->retrieve(key=>$grp_id, class=>ref($self));
}

sub save {
    my ($self, $implementor, $grp_id) = @_;
    $self->get_object_accessor()->update($grp_id, $self);
}

sub add {
    my ($self, $implementor, $grp_id) = @_;
    my $key_value = $self->get_object_accessor()->create($self);
    $implementor->set_property('grp_id', $key_value);
}


sub get_groups_where {
   my ($self, $implementor, $where) = @_;
   my @objects = $self->get_object_accessor()->retrieve_where(where=>$where, class=>'Group');

   # this is leftover from the older ObjectAccess
#    my @objects = ();
#    foreach my $memento (@mementos) {
#        my $object = $implementor->new();
#        $object->restore_state($object, $memento);
#        push @objects, $object;
#    }

   #use Data::Dumper;
   #print STDERR "get_groups_where \"$where\" returns:\n",Dumper(\@objects);

   return @objects;
}

sub show_add_group {
    my $self = shift;
    my $implementor = shift;

    return '' unless $implementor->is_allowed_to_do('verify_add_group');

    my $hidden = $implementor->make_hidden();
    my $custom_fields = $self->show_custom_form_fields($implementor) || '';
    my $organization_fields = '';
    my $welcome_letter_fields = '';

    if ($PARAMS->{allow_organization_registration}) {
        $organization_fields = $implementor->generate_organization_input_fields();
    }

    if ($implementor->group_welcome_letters_allowed()) {
        $welcome_letter_fields = $implementor->generate_welcome_letter_input_fields();
    }

    my $s = <<EOM
<form method="post" action=index.pl>
<input type="hidden" name="isa" value="Group">
<input type="hidden" name="op" value=verify_add_group>
<input type="hidden" name="iid" value=$FORM{iid}>
$hidden
<table border="0">
    <tr>
        <td colspan="2"><H2>Add a new group</H2></td>
    </tr>
    <tr>
        <td>Group Name</td>
        <td>
            <input type="text" name=group_name maxlength="30" value="">
        </td>
    </tr>
    <tr>
        <td valign="top">Group Description</td>
        <td><textarea name="group_description" rows="3" cols="20" wrap></textarea></td>
    </tr>
$organization_fields
$welcome_letter_fields
$custom_fields
    <tr>
        <td colspan="2" align="right">
            <input type="submit" value="Add">
        </td>
    </tr>
</table>
</form>
EOM
;
    my $add_group_frame = new HTML::TableFrame("#333333", "#efefef");

    $add_group_frame->add($s);

    return $add_group_frame->print();
}


# this is used for specifying what fields are displayed in www_show_all_groups.
# note: the field_name must match the fields returned by the result set in
# User->get_groups_data_by_owner

my @SHOW_GROUPS_FIELDS = (
    {
        field_name => 'group_name',
        label => 'Name'
        },
    {
        field_name => 'description',
        label => 'Description'
        },
    {
        field_name => 'owner',
        label => 'Owner'
        },
    {
        field_name => 'nick',
        label => 'Nickname / Screenname'
        },
    {
        field_name => 'email',
        label => 'E-mail'
        },
);

# defines the fields used in www_show_all_groups
sub show_groups_fields {
    my $self = shift;
    my $impl = shift;
    return @SHOW_GROUPS_FIELDS;
}

# aju
sub show_group_search_form {
    my ($self, $impl, %params) = @_;
    my $out = '';

    my $form_name = 'group_memb_add_group';
    my $action = $params{action} || 'index.pl';

    $out = "<form name=\"$form_name\" action=\"$action\" method=\"POST\">\n";
    $out .= $impl->_show_hidden_fields(op=>'show_group_members', add => 'add_subgroup',
                       grp_id => $FORM{grp_id}, isa => 'Group');
    $out .= Auditable->_show_add_group_form( message_text => '<h2>Add a group to the membership list</h2>',
                         submit_button => '<input type="submit" name="add_new_group" value="Add Group">',
                         no_admin => 1
                         );
    $out .= "</form>\n";

    return $out;
}


1;
