package Metadot::Implementations::Portal::Default;

use strict;
use vars qw(@ISA);

use Metadot::Implementations::Portal::AbstractPortalImplementation;
@ISA=qw(Metadot::Implementations::Portal::AbstractPortalImplementation);

use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $CACHE $CGI $ACCESS_BROKER $DISPLAY %REQUEST_VARS);
use Metadot::AuthenticatorFactory;

###################################################

sub initialize {
    my $self = shift;
    my $impl = shift;
    #my %hash = (); #@_;   #XXXXXXXXX changed this
    my %hash = @_;  # need this

    #print STDERR "Initializing portal...\n";
    $HTTP_HEADER_SENT=0;

    my $on_the_web = $hash{on_the_web};
    if (!defined $on_the_web) {
        $on_the_web = $impl->on_the_web();
    }
    #print STDERR "on the web? $on_the_web\n";

    # unless we're really on the web, don't create a standard CGI object,
    # because it makes the request headers unusable for SOAP
    if($on_the_web) {
        #print STDERR "Creating CGI object...\n";
        $CGI = CGI->new();
    } else {
        #print STDERR "Setting up empty CGI object...\n";
        $CGI = CGI->new('');
    }

    # ParamsHash must be initialized early because DISPLAY requires it
    $CACHE = Metadot::Cache->get_cache();
    ParamsHash->validate_or_refresh_cache();

    # these need to happen early because it's called by many classes
    $DISPLAY = $impl->get_display_object();
    #print STDERR "DISPLAY: $DISPLAY\n";

    $SESSION = Session->new();
    #print STDERR "SESSION: $SESSION\n";

    Group->validate_or_refresh_cache();
    # user riding the server (instance)
    $ACCESS_BROKER = Metadot::AccessBrokerFactory->get_access_broker();
    $USER = Metadot::UserFactory->get_user();
    %REQUEST_VARS = ();

    # server is running :-)
    $impl->{running} = 1;

    # FORM user input parameter are here
    undef %FORM;
    #foreach my $key (keys %FORM){
    #  undef $FORM{$key};
    #}
    $impl->{FORM} = \%FORM;

    # debug level set to 0 or to input val
    (exists($hash{debug_level}))
        ? ($SESSION->{debug_level}=$hash{debug_level})
            : ($SESSION->{debug_level}=0);

    (exists($hash{force_http_header}))
        ? ($impl->{force_http_header}=$hash{force_http_header})
            : ($impl->{force_http_header}=0);

    (exists($hash{id}))
      ? ($impl->{id}=$hash{id})
          : ($impl->{id}=0);


  # if we are on the web then...
    if($on_the_web){
        $impl->_web_init();
    }

    # we need to do this for the 2nd time since
  # web init may have disabled it.
    if(exists($hash{debug_level})){
        # force to input debug level
        $SESSION->{debug_level}=$hash{debug_level};
    }

}

# The original
sub show_v_sub_navigation_flex {
    my $class = shift;
    my $implementor = shift;
    my %args = @_;
#     print STDERR "in show_v_sub_navigation_flex\n";
    $implementor->debug_msg(3, "Portal::show_v_sub_navigation()");

    my @asc;

    if ($args{show_parents}) {
       @asc = $implementor->get_visible_parents_list($implementor->id, 'id');
       @asc = ( { id => 0 } ) unless (@asc);
    } else {
       @asc = ( { id => $implementor->id } );
    }
    $implementor->debug_msg(4, "Portal::show_v_sub_navigation() \@asc:" . join(',', @asc));
    $args{path} = \@asc;
    # application bar

    my $application_bar = Application->show_application_bar();

    # navigation colum
    if ($implementor->can('show_navigation_column_flex')) {
    my $navigation_column = $implementor->show_navigation_column_flex( %args ) ;
    $navigation_column = "<table border = 0 cellpadding = 0 cellspacing = 0>$navigation_column</table>";

    return $navigation_column;
    }
    else {
    return '';
    }

}

# The original
sub show_welcome {
    my $class = shift;
    my $implementor = shift;
    my %args = @_;

    my $s="";
    my $dot="";
    my ($fullname, $nickname, $name);

    $implementor->debug_msg(5, "In show_welcome");

    my $logged_in_count = $USER->is_logged_in() || 0;

    my $webname;
    if ( $logged_in_count >= 1) { # if logged in
        ($webname) = $USER->webname();
        $implementor->debug_msg(3, "show_welcome(webname=$webname)");
    }
    #else, not logged in.
    else {
        #if not deleting session info on logout then
        #go ahead and get webname.
        if (!($PARAMS->{sessions_are_deleted_on_logout}) )  {
            ($webname) = $USER->webname();
            $implementor->debug_msg(3, "show_welcome(webname=$webname)");
        }
    }

    if ($webname) {
        $dot = "." x $logged_in_count;
        my $metagram = '';
        if ($USER->can('has_unread_metagrams') && $USER->has_unread_metagrams()) {
            $metagram = "<a href=\"index.pl?isa=Metadot::SystemApp::MetagramCenter&op=show\"><img src=\"" . $PARAMS->{imageslocation} . "/obj/env.gif\" border=\"0\" alt=\"You have new metagrams!\"></a>";
        }

        #aju
        #          $s .= "<div style="white-space: nowrap"><font class='welcome'>Hello, <b>$webname</b>$dot</font></div><br />\n";
        $s .= "<font class=\"welcome\">Hello, <b>$webname</b>$dot</font>$metagram<br />\n";

        if($logged_in_count < 1) {
            $s .= "<font class=\"welcomesub\"><div style=\"white-space: nowrap\">You are not currently logged in.</div><br /><div style=\"white-space: nowrap\">Click <a class=\"welcomesub\" href='".$implementor->make_url(Metadot::AuthenticatorFactory->get_authenticator()->get_login_url())."'>here</a> to log back in.</div><br /></font>\n";
            #              $s .= "<font class=\"welcomesub\">You are not currently logged in.<br />Click <a class='welcomesub' href='".$implementor->make_url("index.pl?isa=Session&amp;op=login")."'>here</a> to log back in.<br /></font>\n";
        } else {

        # CKIM: disabling this feature; do not know why it's there, and it
        #       should use a different test
        #unless( MetadotConfig->get_param_value('user_management_is_external') ) {
        #aju
        #       $s .= "<div style="white-space: nowrap"><font class=\"welcomesub\">(Not $webname? Click <a class='welcomesub' href='index.pl?isa=Session&amp;op=login&amp;notyou=1'>here</a>)</font></div>\n";
        $s .= "<font class=\"welcomesub\">(Not $webname? Click <a class=\"welcomesub\" href='".Metadot::AuthenticatorFactory->get_authenticator()->get_login_url('notyou=1')."'>here</a>)</font>\n";
        }
        #}

        if ( $ACCESS_BROKER->is_allowed_to_do('manage_registrations') && $logged_in_count ) {
            my $pending = $USER->get_num_accounts_pending_approval();
            if ($pending){
                $s .= "<br /><span class=\"welcomesub\"><b><div style=\"white-space: nowrap\">There are <a class=\"welcomesub\" href='".$implementor->make_url("index.pl?iid=0&amp;isa=User&amp;op=show_users&amp;pending=true")."'>new user accounts</a></div><br /><div style=\"white-space: nowrap\">pending approval.</div></b></span>\n";
            }
        }
        $s .= $implementor->pending_messages() if $logged_in_count;
    } else {
        $s .= "<font class=\"welcomesub\"><span style=\"white-space: nowrap\">Already registered? <a class=\"welcome\" href='".$implementor->make_url(Metadot::AuthenticatorFactory->get_authenticator()->get_login_url())."'>Login</a></span></font>\n";
    }

    return ($s);
}

sub show_h_dropdown {
    my $class = shift;
    my $implementor = shift;
    my %args = @_;
#     print STDERR "in show_h_dropdown\n";
    $implementor->debug_msg(3, "Portal::show_h_dropdown()");

    # navigation colum
    if ($implementor->can('show_category_bar_dropdown')) {
        return $implementor->show_category_bar_dropdown( %args ) ;
    } else {
        return '';
    }
}

1;
