###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Metadot::Invitable;

use strict;
use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $ACCESS_BROKER);

use DBAccess;
use Carp;
use Gizmo;
use vars qw(@ISA);
use CGI qw(escapeHTML);
use Storable qw(freeze thaw);
use HTML::TableFrame;
use Utility;
use Data::Dumper;

use constant SESSION_KEY  => '_invite_list';
my $DELIMITER = ', *';  # a comma followed by any number of spaces
                      # for some reason, this doesn't work as a constant.


sub customize_permissions {
    my ($class, $perms) = @_;

    if (defined $perms) {
        $perms->add_bundle('BROADCAST', 'Send Broadcast Messages', '8','0');
        $perms->set_broadcast('BROADCAST', 1);

        $perms->add_op('BROADCAST','invite');
    }
}


sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $self = {};

    bless ($self, $class);
    return $self;
}

sub get_display_name {
    my $self = shift;
    my $result;

    my ($class, $descriptor);

    if ($self->can('get_gizmo_name')) {
        $class = $self->get_gizmo_name();
    } else {
        $class = ref($self) || $self;
    }

    $descriptor = $self->{name} || $self->{description};
    if ($descriptor) {
        $descriptor = ": $descriptor";
    }

    $result = $class.$descriptor;

    # CKIM-DEFECT-INVITE-IMAGE strip out any tags
    $result =~ s/<.*?>//g;

    return $result;
}

sub invite {
    my $self = shift;

    my $out;
    my $message = $self->process_invite_input();
    if ($message) {
        $out = $message;
    } else {
        $out = $self->show_invite_interface();
    }
    return $out;
}

sub www_invite {
    my $self = shift;

    if ($self->is_invite_enabled() || $ACCESS_BROKER->is_allowed_to_do('admin_tasks')) {
        my $out = $self->invite();
        $self->print_form($out);
    } else {
        $self->print_click_back('Sorry', 'The broadcast feature has been disabled by the site administrator.');
        exit(0);
    }
}


# adapt Invitable to Gizmo3000
sub do_invite {
    my $self = shift;

    if ($self->is_invite_enabled() || $ACCESS_BROKER->is_allowed_to_do('admin_tasks')) {
        my $out = $self->invite();
        return $self->generate_standard_wrapper($out);
    } else {
        $self->print_click_back('Sorry', 'The broadcast feature has been disabled by the site administrator.');
        exit(0);
    }
}


sub show_invite_button {
    my $self = shift;
    my $out;

    if ($self->is_invite_enabled() && $self->is_allowed_to_do('invite', $USER)) {
        my %form_vars = %{ $self->_get_standard_form_vars(op=>'invite') };

        $out = "<a href=\"".$DISPLAY->escapeHREF($self->_create_href(%form_vars))."\">";
        my $imageslocation = $PARAMS->{'imageslocation'};

        $out .= '<img border="0" src="'.$imageslocation.'/invite.gif" alt="Invite others to this '.$self->get_display_name().'" />';
        $out .= "</a>";
    }
    return $out;
}


sub show_invite_panel_button {
    my $self = shift;

    return '' unless($self->is_invite_enabled() && $SESSION->admin_mode());

    my $out = '';

    if ($self->is_allowed_to_do('invite', $USER)) {

        my %form_vars = %{ $self->_get_standard_form_vars(op=>'invite') };

        $out = "<a href=\"".$DISPLAY->escapeHREF($self->_create_href(%form_vars))."\">";

        my $imageslocation = $PARAMS->{'imageslocation'};


        $out .= '<img border="0" src="'.$imageslocation.'/obj/invite_panel_button.gif" alt="Invite others to this '.$self->get_display_name().'" />';
        $out .= "</a>";
    }

    return $out;
}


sub show_invite_interface {
    my $self = shift;
    my $s = '';

    my $add_user_form = $self->show_invite_input_form();
    my $recipient_form = $self->show_invite_recipients();
    my $gizmo_name = $self->get_display_name();

    $s .= $self->get_invite_interface_message();

    $s .= <<EOM
<hr>
<table width="100%" cellspacing="0" cellpadding="5" border="0">
    <tr><td width="500" valign="top">
        $add_user_form
    </td><td valign="top">
        $recipient_form
    </td></tr>
</table>

EOM
;
    return $s;
}

sub get_invite_interface_message {
    my $self = shift;
    my $gizmo_name = $self->get_display_name();
    my $s .= <<EOM
<h1>Invite users to view this $gizmo_name.</h1>
Below, you can search for users you would like to invite
to view this $gizmo_name. You may also invite entire groups or type in the
email address of the user. Once
you\'ve finished, click on the 'Next' button to send an e-mail
inviting the people you have selected.
EOM
;
    return $s;
}

sub process_invite_input {
    my $self = shift;
    my $out = '';

    #print STDERR "CKIM: inviteable->process gets\n".Dumper \%FORM;

    my $new_uid = $FORM{new_uid};
    if ($new_uid && $USER->is_logged_in()) {
        $self->add_invite_recipient(uid=>$new_uid);
    }
    my $new_group = $FORM{add_group};
    if ($new_group && $USER->is_logged_in()) {
        $self->add_invite_recipient(group=>$new_group);
    }
    my $new_typed_email = $FORM{typed_email};
    if ($new_typed_email && Utility->is_valid_delimited_email_string($new_typed_email, $DELIMITER)) {
        my @emails = split(/$DELIMITER/, $new_typed_email);
        $self->debug_msg(3, "Invitable adds email recipients (using '$DELIMITER'): @emails");
        $self->add_invite_recipient( typed_email => \@emails );
    }
    if ($FORM{delete_recipient}) {
        $self->remove_invite_recipient({ id => $FORM{delete_recipient}, type =>  $FORM{type} });
    }
    if ($FORM{invite_compose_button}) {
        $out .= $self->show_invitation_compose_form();
    }
    if ($FORM{invite_finished_button}) {
        $self->send_invitations(text=>$FORM{invitation_text}, subj_text=> $FORM{subj_text}, cc_me => $FORM{cc_me}, show_full_text => $FORM{show_full_text});
        $out .= $self->show_invitation_confirmation();
        # delete this from the session
        $self->set_invite_recipients();
    }
    return $out;
}

sub show_invitation_confirmation {
    my ($self, %params) = @_;
    my $out;
    my $return_link = $DISPLAY->escapeHREF($self->_create_href(%{ $self->_get_standard_form_vars(op=>'show') }));
    my $title_keyword = $self->get_invite_noun_keyword();

    $out .= "Your $title_keyword has been sent!<br /><a href=\"$return_link\">Click here to return to the ".$self->get_display_name()."</a>\n";
    return $out;
}

sub get_invite_title_keyword {
    return 'invite';
}

sub get_invite_title_keyword_proper {
    return 'Invite';
}

sub get_invite_noun_keyword {
    return 'invitation';
}

sub _get_return_link_text {
    my ($self, %params) = @_;
    my $result;

    my $basename = $PARAMS->{'basename'};
    my $return_link = $basename."/".$self->_create_href(%{ $self->_get_standard_form_vars(op=>'show') });
    my $gizmo_name = $self->get_display_name();
    my $sender = $USER->fullname();
    my $sender_email = $USER->email();

    $result = "This message was sent by $sender ($sender_email). To view the $gizmo_name, browse to:\n    $return_link\n";
    return $result;
}

sub _get_invitation_subject_prefix {
    my $self = shift;
#      return 'You have been invited ';
    return '';
}

sub _get_invitation_subject_html {
    my ($self, %params) = @_;
    my $result;

    my $gizmo_name = $self->get_display_name();
    $result = $self->_get_invitation_subject_prefix();

    my $default_field_text = $self->_get_invitation_subject_html_default_field_text();
    my $subj_field_text = $default_field_text;
    if ($FORM{subj_text}) {
        $subj_field_text = $FORM{subj_text};
    }
    return "$result <input type=text value=\"$subj_field_text\" size=60 name=subj_text>";
}

sub _get_invitation_subject_html_default_field_text {
    my $self = shift;
    my $gizmo_name = $self->get_display_name();
    return 'You have been invited to a '.$gizmo_name;
}

sub _get_invitation_subject_text {
    my ($self, $subj_text) = @_;
    if (!$subj_text) {
        $subj_text = $FORM{subj_text};
    }
    return $self->_get_invitation_subject_prefix() . $subj_text;
}

sub _get_invitation_body_html_default_field_text {
    my $self = shift;
    my $gizmo_name = $self->get_display_name();
    my $default_text = <<EOM
I would like to invite you to view a $gizmo_name that I think you will find very interesting.
EOM
;
    return $default_text;
}

sub show_invitation_compose_form {
    my ($self, %params) = @_;
    my $out = '';

    my $form_name = 'InvitableCompose';
    my $action = $params{action} || 'index.pl';
    my $gizmo_name = $self->get_display_name();

    my $default_text = $self->_get_invitation_body_html_default_field_text();
    my $button_keyword = $self->get_invite_noun_keyword();
    my $op             = 'invite';
    my $isa            = $self->is_a();

    $out = "<form name=\"$form_name\" action=\"$action\" method=\"POST\">\n";
    $out .= $self->_show_hidden_fields(op=>$op);
    $out .= "<h2>The recipients you have selected will receive the following $button_keyword:</h2>When you are finished, press the 'Send $button_keyword' button<hr>\n";
    $out .= '<table border="0" cellspacing="0" cellpadding="0" width="100%">';
    $out .= '<tr valign="baseline"><td><b>Subject:</b>&nbsp;</td><td>'.$self->_get_invitation_subject_html()."<br />&nbsp</td></tr>\n";
    $out .= "<tr valign=\"baseline\"><td colspan='2'>";
    $out .= "<b>Send full text of story:</b>&nbsp;<input type=checkbox name=show_full_text>&nbsp;&nbsp;" if $isa eq 'Category';
    $out .= $USER->is_logged_in() ? "<b>Send me a copy:</b>&nbsp;<input type=checkbox name=cc_me checked>" : "";
    $out .= "<br />&nbsp;</td></tr>\n";
    $out .= '<tr valign="baseline"><td valign="top"><b>Message:</b>&nbsp;</td>'."\n";
    $out .= '<td>';
    $out .= "<textarea rows=10 cols=72 name=\"invitation_text\">\n".escapeHTML($default_text)."</textarea>\n";
    $out .= "<br /><br />".$self->_get_return_link_text()."\n</td></tr>";
    $out .= "</table>\n";
    $out .= "<br /><input type=\"submit\" name=\"invite_finished_button\" value=\"Send $button_keyword\">\n";
    $out .= '</form>';

    my $frame_add = new HTML::TableFrame("#333333", "#efefef", "100%");
    $frame_add->add($out);

    return $frame_add->print;

    #return $out;
}

sub send_invitations {
    my ($self, %params) = @_;
    my $body = $params{text} || '';
    my $out='';

    #print STDERR "CKIM: Invitable->send_invitations sending invite with\n$body\n";
    my %recipients = %{$self->get_invite_recipients};
    my @uids = @{ $recipients{uid} };
    my $gizmo_name = $self->get_display_name();
    my @emails = "";

    # process groups
    for my $group (@{ $recipients{group} } ) {
#          @emails = (@emails, @{Group->get_emails_by_grp_id($group) });
        # aju - recurse to get subgroup members also
        @emails = (@emails, @{Group->get_emails_by_grp_id($group, 1) });
    }
    foreach my $uid (@uids) {
        push (@emails, $USER->get_email_from_uid($uid));
    }

    push @emails, @{ $recipients{typed_email} };


    @emails = @{ Utility->filter_valid_emails($self->uniquify_array(@emails)) };

    # send the emails
    my $subject = $self->_get_invitation_subject_text($params{subj_text});
    my $is_html = 0;
    if ($params{show_full_text}) {
        $body .= "<p />" . $self->get_printer_friendly_content(absolute_path => 1);
        #$body .= $self->www_show();
        $is_html = 1;
    }
    $body = $body."\n\n".$self->_get_return_link_text();

    $self->send_invitation_notification(
            recipient_email_list => join(',',@emails),
            subject              => $subject,
            body                 => $body,
            cc_me                => $params{cc_me},
            is_html              => $is_html,
        );
}

## email
sub send_invitation_notification {
    my ($self, %params) = @_;

    my $recipient_email_list = $params{recipient_email_list};
    my $subject              = $params{subject};
    my $body                 = $params{body};
    my $cc_me                = $params{cc_me};
    my $is_html              = $params{is_html};

    my $sender = $USER->{email};
    $self->debug_msg(3, "Invitable->send invitations FROM '$sender' TO '$recipient_email_list'");
    #print STDERR "CKIM: Invitable->send_invitations STARTING SENDMAIL\n";
    my $sm = Sendmail->new('null','',$recipient_email_list,$subject,$body);
    #$sm->{to} = $sender;
    $sm->{to} = $PARAMS->{mailsenderaddress};
    $sm->{cc} = $cc_me ? $sender : '';
    $sm->{bcc} = $recipient_email_list;
    $sm->{subject} = $subject;
    $sm->{body} = $body;
    $is_html && $sm->set_content_type('html');
    $sm->{from} = $sm->{reply_to} = $sender;

        my ($result) = $sm->sendMail();
    #print STDERR "CKIM: Invitable->send_invitations FINISHED SENDMAIL\n";
}


sub show_invite_input_form {
    my ($self, %params) = @_;
    my $out;

    $out .= $USER->is_logged_in() ? $self->show_group_search_form(%params) : '';
    $out .= "<br />\n";
    $out .= $USER->is_logged_in() ? $self->show_user_search_form(%params) : '';
    $out .= $self->show_add_email_address_form(%params);

    return $out;
}

sub show_group_search_form {
    my ($self, %params) = @_;
    my $out = '';

    my $form_name = 'InvitableAddGroup';
    my $action = $params{action} || 'index.pl';
    my $title_keyword = $self->get_invite_title_keyword_proper();

    $out = "<form name=\"$form_name\" action=\"$action\" method=\"POST\">\n";
    my $op = 'invite';
    $out .= $self->_show_hidden_fields(op=>$op);
    $out .= Auditable->_show_add_group_form( message_text => "<h2>$title_keyword a group</h2>",
            );
    $out .= "</form>\n";

    return $out;
}

sub show_user_search_form {
    my ($self, %params) = @_;

    my $action = $params{action} || 'index.pl';
    #print STDERR "CKIM: Invitable->show_user_search_form action='$action'\n";

    my $out = '';
    my $title_keyword = $self->get_invite_title_keyword();
    my $op = 'invite';
    my $config = {
            TITLE  => "Search users to $title_keyword",
            HEADER => "",
            ACTION => $action,
            PARAMS => { EMAIL=>$FORM{email},
                        FULLNAME=>$FORM{fullname},
                        NICKNAME=>$FORM{nickname},
                        %{ $self->_get_standard_form_vars( op=>$op) },
            },
            WIDTH  => "100%",
    };
    if ($FORM{email} || $FORM{fullname} || $FORM{nickname}) {
        $config->{ACTION} = $action;
        $config->{EMAIL} = $FORM{email};
        $config->{FULLNAME} = $FORM{fullname};
        $config->{NICKNAME} = $FORM{nickname};
        $config->{SELECT_ONE} = 0;
        $config->{RETURN_VAL} = "uid";
        $out .= "<br />"
             . $USER->get_search_user_results($config)
             . "<br />";
    }

    $out .= Components->get_search_users($config);
    return $out;
}

sub show_add_email_address_form {
    my ($self, %params) = @_;
    my $out = '';

    my $form_name = 'InvitableAddEmail';
    my $action = $params{action} || 'index.pl#AddEmailAnchor';

    ##check if any problems with form submission
    my ($typed_email, $error_msg);
    if (defined($FORM{add_typed_user_button})) {
        $typed_email = $FORM{typed_email};
        if (!$typed_email){
            $error_msg   = "Please Enter an E-mail address(es).";
        } elsif (!Utility->is_valid_delimited_email_string($typed_email, $DELIMITER)) {
            $error_msg   = "Please Enter a Valid E-mail address(es).";
        } else {## no errors submitting - clear out values
            $typed_email = "";
        }
        $error_msg   = "<font color=\"red\">$error_msg</font>";
    }
    my $title_keyword = $self->get_invite_title_keyword();
    my $op = 'invite';

    $out .= "<form name=\"$form_name\" action=\"$action\" method=\"POST\">\n";
    $out .= $self->_show_hidden_fields(op=>$op);
    $out .= "<H2>Enter email address(es) to $title_keyword</H2>
    <table border=\"0\" cellspacing=\"1\" cellpadding=\"1\">
      <tr>
        <td><b>E-mail address</b></td>
        <td><input type=\"text\" name=\"typed_email\" value=\"$typed_email\" size=45 maxsixe=100></td>
      </tr>
      <tr>
        <td colspan='2'><font size=1><i>You may enter multiple email addresses by separating them with a comma (e.g. joe\@abc.com,mike\@xyz.com).</i></font></td>
      </tr>
      <tr>
        <td><input type=\"submit\" name=\"add_typed_user_button\" value=\"Add\"></td>
      </tr>
    </table>
    </form>\n";
    my $frame_add = new HTML::TableFrame("#333333", "#efefef", "100%");
    $frame_add->add($out);

    return "<p /><a name=\"AddEmailAnchor\">$error_msg<p />" . $frame_add->print;
}


sub _show_single_recipient {
    my $self=shift;
    my %params = @_;

    my $recipient = $params{recipient} || die ("Must specify recipient\n");
    my $row_count = $params{row_count} || 0;
    my $display_name = $params{display_name} || $recipient;
    my $type      = $params{type};
    my $imageslocation = $PARAMS->{'imageslocation'};

    my $result='';

    my @colors = ('FFFFFF', 'CCCCCC');
    my $op = 'invite';
    my %form_vars = %{ $self->_get_standard_form_vars(op=>$op) };

    my $out = "<a href=\"".$DISPLAY->escapeHREF($self->_create_href(%form_vars))."\">";

    $result .= '<tr bgcolor="#'.$colors[$row_count%2]."\">\n";
    $result .= "<td>$display_name</td>\n";
    $result .= "<td align='right'>&nbsp;<a href=\"".$DISPLAY->escapeHREF($self->_create_href(%form_vars))."&amp;delete_recipient=".escapeHTML($recipient)."&amp;type=$type\">";
    $result .= "<img border=\"0\" src=\"$imageslocation/obj/del.gif\" alt=\"Delete this recipient\" /></a></td></tr>\n";

    return $result;
}

sub show_invite_recipients {
    my ($self, %params) = @_;
    my $out = '';

    my %recipients = %{$self->get_invite_recipients};
    my $form_name = 'InvitableRecipients';
    my $action = $params{action} || 'index.pl';

    if (@{$recipients{uid}} || @{$recipients{group}} || @{$recipients{typed_email}}) {
        my $op = 'invite';
        $out = "<form name=\"$form_name\" action=\"$action\" method=\"POST\">\n";
        $out .= $self->_show_hidden_fields(op=>$op);
        $out .= "<h2>You have selected the following recipients:</h2>\n";
        $out .= '<table border="0" cellspacing="0" cellpadding="0" width="100%">';

        # sort recipients alphabetically, groups first
        $recipients{uid} = [sort {$a cmp $b} @{$recipients{uid}}];
        $recipients{group} = [sort {$a cmp $b} @{$recipients{group}}];
        $recipients{typed_email} = [sort {$a cmp $b} @{$recipients{typed_email}}];

        my $row_count = 0;
        for my $r (@{$recipients{group}}) {
            my $group .= Group->get_group_name_by_grp_id($r)." group\n";
            $out .= $self->_show_single_recipient(recipient=>$r, row_count=>$row_count, display_name=>$group, type=>'group');
            $row_count++;
        }
        for my $r (@{$recipients{uid}}) {
            my $fullname = $USER->get_fullname_from_uid($r);
            $out .= $self->_show_single_recipient(recipient=>$r, row_count=>$row_count, display_name=>$fullname, type=>'uid');
            $row_count++;
        }
        for my $r (@{$recipients{typed_email}}) {
            my $fullname = $r;
            $out .= $self->_show_single_recipient(recipient=>$r, row_count=>$row_count, display_name=>$fullname, type=>'typed_email');
            $row_count++;
        }
        $out .= "</table>\n";
        $out .= '<br /><input type="submit" name="invite_compose_button" value="Next">'."\n";
        $out .= '</form>';
    } else {
        $out .= "<h2>No recipients selected.</h2>Please add a user or group.";
    }
    return $out;
}

sub remove_invite_recipient {
    my ($self, @recipients) = @_;

    # create a hash of the recipients for easy lookup
    my %removals;
    my ($id, $type);
    for my $val (@recipients) {## $val is a hash
        $type = $val->{type};
        $id = $val->{id};
        $removals{$type}{$id} = 1;
    }

    # get the existing values
    my %recipients = %{$self->get_invite_recipients};
    my %out;

    # cycle through the types of recipients
    for my $key (keys(%recipients)) {
        my @invite_list = @{ $recipients{$key} };
        my @out_list;
        # cycle through the invite list
        for my $val (@invite_list) {
            unless ($removals{$key}{$val}) {
                push @out_list, $val;
            }
        }
        # store it back
        $recipients{$key} = \@out_list;
    }
    # save the modified recipients
    $self->set_invite_recipients(\%recipients);
}

sub add_invite_recipient {
    my ($self, %params) = @_;
    # to add users, send param uid=>[one or more uids]
    # to add groups, send param group=>[one or more groups]

    # get the existing values
    my %recipients = %{$self->get_invite_recipients};

    # coerce uids to an array
    my $uid = $params{uid};
    if ($uid) {
        if (ref($uid) eq "ARRAY") {
            #print STDERR (Dumper \%recipients), (Dumper $uid);
            $recipients{uid} = [ @{$recipients{uid}}, @{$uid} ];
        } else {
            $recipients{uid} = [ @{$recipients{uid}}, $uid ];
        }
        $recipients{uid} = $self->uniquify_array($recipients{uid});
    }

    # coerce groups to an array
    my $group = $params{group};
    if ($group) {
        if (ref($group) eq "ARRAY") {
            $recipients{group} = [ @{$recipients{group}}, @{$group} ];
        } else {
            $recipients{group} = [ @{$recipients{group}}, $group ];
        }
        $recipients{group} = $self->uniquify_array($recipients{group});
    }

    # coerce typed users to an array
    my $typed_email = $params{typed_email};
    if ($typed_email) {
        if (ref($typed_email) eq "ARRAY") {
            $recipients{typed_email} = [ @{$recipients{typed_email}}, @{$typed_email} ];
        } else {
            $recipients{typed_email} = [ @{$recipients{typed_email}}, $typed_email];
        }
        $recipients{typed_email} = $self->uniquify_array($recipients{typed_email});
    }

    # save the modified recipients
    #$self->debug_msg(4, "Invitable added recipients:\n".Dumper \%recipients);
    $self->set_invite_recipients(\%recipients);
}

sub uniquify_array {
    my ($self, @array) = @_;
    my %keys;
    my @out;

    # in case I got a ref to an array instead of an actual array
    if (@array && ref($array[0]) eq "ARRAY") {
        @array = @{ $array[0] };
    }

    for my $val (@array) {
        unless ($keys{$val}) {
            $keys{$val} = 1;
            push @out, $val;
        }
    }
    return \@out;
}

sub filter_valid_emailsXXX {
    my ($self, @array) = @_;
    my @out;

    # in case I got a ref to an array instead of an actual array
    if (@array && ref($array[0]) eq "ARRAY") {
        @array = @{ $array[0] };
    }

    for my $email (@array) {
        if ($email =~ m/.*@.*\..*/) {
            push @out, $email;
        }
    }
    return \@out;
}

sub get_invite_recipients {
    my $self = shift;
    my $session_value = $SESSION->get_value(SESSION_KEY);
    my $out;
    if ($session_value) {
        $out = thaw($session_value);
    } else {
        $out = {
            uid => [],
            group => [],
            typed_email => [],
        };
    }
    return $out;
}

sub set_invite_recipients {
    my $self = shift;
    my $recipients = shift;
    # recipients must be a hash reference
    my $save_value;
    if ($recipients) {
        $save_value = freeze($recipients);
    } else {
        $save_value = 0;
    }
    $SESSION->set_value(SESSION_KEY, $save_value);
}

sub _get_standard_form_vars {
    my ($self, %settings) = @_;
    my %result = %settings;

    my @keys = ('iid', 'id', 'isa', 'op', 'app');
    my %key_attribute_map = (
        id  => 'id',
        iid => 'id',
        isa => 'is_a',
    );

    for my $key (@keys) {
        # try to get the value from self
        unless ( defined($result{$key}) ) {
            if ( $key_attribute_map{$key} ) {
                $result{$key} = $self->{$key_attribute_map{$key}};
            }
        }
        # try to get the value from the form
        unless ( defined($result{$key}) ) {
            $result{$key} = $FORM{$key};
        }
    }
    return \%result;
}

sub _show_hidden_fields {
    my ($self, %params) = @_;
    my $out;

    my %hidden = %{ $self->_get_standard_form_vars( %params ) };
    for my $var (keys(%hidden)) {
        $out .= "<input type=\"hidden\" name=\"$var\" value=\"$hidden{$var}\">\n";
    }
    return $out;
}

sub _create_href {
    my ($self, %params) = @_;
    my $base = 'index.pl';

    my @url_params;
    for my $key (keys(%params)) {
        if (defined($params{$key})) {
            push @url_params, escapeHTML("$key=$params{$key}");
        }
    }
    my $param_string = join '&', @url_params;
    my $out = join '?', ($base, $param_string);

    return $out;
}

# returns true if the $PARAMS option for invitations is enabled
sub is_invite_enabled {
    my $class = shift;

    #return !($PARAMS->{disable_invitable});
    return Portal->is_broadcast_email_enabled();
}



1;

__END__

