###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::JSForm::Chain ;
use strict;

use Metadot qw ($PARAMS) ;

use vars qw(@ISA) ;
@ISA=qw(Metadot::JSForm) ;

#########################################################
##  Constructor
#########################################################
sub new {
    my $proto = shift ;
    my $class = __PACKAGE__ ;
    my $self = {} ;
    $self  = $class->SUPER::new(@_) ;

    bless ($self, $class) ;
    return $self ;
}

#########################################################
##  Private Methods
#########################################################

#### overridden methods
sub _apply_filter {
    my $self = shift ;
    my $data = shift || return undef ;
    my $jsform_filter = shift || "Default" ;
    return undef unless ref($data) ;

    my @filter_types = split(/[^A-z]/, $jsform_filter) ;
#    print STDERR "\n\n".$jsform_filter."\n\n" ;
#    print STDERR join(",", @filter_types) ;
    my $filter_index = 0 ;
    foreach my $filter_type (@filter_types) {
        my $filter = Metadot::JSForm::FilterFactory->get_filter($filter_type) ;
        # skip Default (empty) filter or filter that cannot be in this position in the chain
        # Note: skiped filter is not counted to avoid problems like in the case
        # of the following filter: Default|Checker|Dialog that otherwise doesn't work
        if ($filter_type ne "Default" && $filter->CAN_BE_NUMBER_N_IN_CHAIN($filter_index)) {
            $data = $filter->handle_data($data)  ;
            $filter_index++ ;
        }
    }
    return $data ;
}

sub _construct_html {
    my $self = shift ;
    my $data = shift || return undef ;
    return "" unless ref($data) ;

    my $html = "" ;
    # include library_js for the first form on the page only
    $html .= $data->{library_js} if ($data->{form_index} == 0);
    $html .= $data->{initialize_js} ;
    my $checks = $data->{checks} ;
    foreach my $check (@{$checks}) {
        $html .= $check->{start_js} ;
        $html .= $check->{end_js} ;
    }
    $html .= $data->{final_js} ;
    $html .= $data->{form_html} ;

    return $html ;
}

1;
