###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::JSForm::Filter ;
use strict;

use Metadot qw ($PARAMS) ;
# for class variables
use Metadot::JSForm ;

#########################################################
##  Class Methods
#########################################################
sub get_ONE_ERROR {
    my $class = shift ;
    return Metadot::JSForm->get_ONE_ERROR() ;
}

sub get_ALL_ERRORS {
    my $class = shift ;
    return Metadot::JSForm->get_ALL_ERRORS() ;
}

#########################################################
##  Constructor
#########################################################
sub new {
    my $proto = shift ;
    my $p = shift ;
    my $class = __PACKAGE__ ;
    my $self = {};

    # getting external parameters
    $self->{params} = $PARAMS ;
    # getting parameters of the constructor
    $self->{p} = $p if defined($p) && ref($p) ;

    bless ($self, $class) ;
    return $self ;
}

#########################################################
##  Public Methods
#########################################################
sub handle_data {
    my $self = shift ;
    my $data = shift ;
    return undef unless defined($data) && ref($data) ;

    $data->{library_js}     = $self->_handle_library_js($data->{library_js}, $data) ;
    $data->{initialize_js}  = $self->_handle_initialize_js($data->{initialize_js}, $data) ;
    $data->{checks}         = $self->_handle_checks($data->{checks}, $data) ;
    $data->{final_js}       = $self->_handle_final_js($data->{final_js}, $data) ;

    $data->{form_html}      = $self->_handle_form_html($data->{form_html}, $data) ;

    return $data ;
}

#########################################################
##  Private Methods
#########################################################
sub _handle_library_js {
    my $self = shift ;
    my $js = shift || "" ;
    my $data = shift ;
    my $add_js = $self->_get_library_js($data) ;
    # add additional library js if it's not contained in the library js already
    $js .= $add_js if ($js !~ /\Q$add_js\E/) ;
    return $js ;
}
sub _handle_initialize_js {
    my $self = shift ;
    my $js = shift || "" ;
    my $data = shift ;
    my $add_js = $self->_get_initialize_js($data) ;
    $js .= $add_js  ;
    return $js ;
}
sub _handle_checks {
    my $self = shift ;
    my $checks = shift || return undef ;
    my $data = shift ;
    my $check_index = 0 ;
    foreach my $check (@{$checks}) {
        $check->{start_js} = "" unless defined($check->{start_js}) ;
        $check->{start_js} .= $self->_get_start_js_for_check($check, $data, $check_index) ;
        $check->{end_js} = "" unless defined($check->{end_js}) ;
        $check->{end_js} = $self->_get_end_js_for_check($check, $data, $check_index).$check->{end_js} ;
        $check_index++ ;
    }
    return $checks ;
}
sub _handle_final_js {
    my $self = shift ;
    my $js = shift || "" ;
    my $data = shift ;
    my $add_js = $self->_get_final_js($data) ;
    $js .= $add_js ;
    return $js ;
}

#### abstract methods, must be overridden
sub CAN_BE_NUMBER_N_IN_CHAIN {
    my $class = shift ;
    my $number = shift ;
    die "abstract, must be overridden" ;
    return 0 ;
}
sub _handle_form_html {
    my $self = shift ;
    my $form_html = shift ;
    my $data = shift || return $form_html ;
    my $params = $self->{params} ;
    my $p = $self->{p} ;
    die "abstract, must be overridden" ;
    return "" ;
}

sub _get_library_js {
    my $self = shift ;
    my $params = $self->{params} ;
    my $p = $self->{p} ;
    die "abstract, must be overridden" ;
    return "" ;
}
sub _get_initialize_js {
    my $self = shift ;
    my $params = $self->{params} ;
    my $p = $self->{p} ;
    die "abstract, must be overridden" ;
    return "" ;
}
sub _get_start_js_for_check {
    my $self = shift ;
    my $check = shift || return "" ;
    my $data = shift ;
    my $check_index = shift ;
    my $params = $self->{params} ;
    my $p = $self->{p} ;
    die "abstract, must be overridden" ;
    return "" ;
}
sub _get_end_js_for_check {
    my $self = shift ;
    my $check = shift || return "" ;
    my $data = shift ;
    my $check_index = shift ;
    my $params = $self->{params} ;
    my $p = $self->{p} ;
    die "abstract, must be overridden" ;
    return "" ;
}
sub _get_final_js {
    my $self = shift ;
    my $data = shift ;
    my $params = $self->{params} ;
    my $p = $self->{p} ;
    die "abstract, must be overridden" ;
    return "" ;
}

#### filter-specific methods
# none

1;
