###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::JSForm::Filter::Checker ;
use strict;

use Metadot qw ($PARAMS) ;

use vars qw(@ISA) ;
@ISA=qw(Metadot::JSForm::Filter) ;

#########################################################
##  Class Variables
#########################################################
my $CONFIG = {
    # http://, https:// or imageslocation-relative path
    libfile_location => "js/jsform/filter/checker.js",
} ;

# mapping between element attribute and FormCheck.js call
# (see FormCheck.js for function definitions)
my $CHECKER_MAPPINGS = {
                        DEFAULT         => '!isWhitespace',
                        'notempty'      => '!isWhitespace',
                        'required'      => '!isWhitespace',
                        'email'         => 'isEmail',
                        'url'           => 'isUrl',
} ;

#########################################################
##  Constructor
#########################################################
sub new {
    my $proto = shift ;
    my $class = __PACKAGE__ ;
    my $self = {} ;
    $self  = $class->SUPER::new(@_) ;

    bless ($self, $class) ;
    return $self ;
}

#########################################################
##  Private Methods
#########################################################

#### overridden methods
sub CAN_BE_NUMBER_N_IN_CHAIN {
    my $class = shift ;
    my $number = shift ;
    return ($number == 0) ? 1 : 0 ;
}
sub _handle_form_html {
    my $self = shift ;
    my $form_html = shift ;
    my $data = shift ;
    my $form_index = $data->{form_index} ;

    my $add_on = "" ;
    $add_on .= "\nif (isError) { return false ; }" ;
    $add_on .= "\nreturn true ;\n" ;
    $add_on .= "\n}" ;
    $add_on .= "\n// end hide-->\n</script>" ;

    $form_html = $self->_insert_js_event_handle_into_form_tag($form_html, $form_index, $data->{random_name}) ;

    return $add_on.$form_html ;
}
sub _get_library_js {
    my $self = shift ;
    my $library_js = $self->_get_libfiles_js() ;
    return $library_js ;
}
sub _get_initialize_js {
    my $self = shift ;
    my $data = shift ;
    my $is_all_errors = $data->{mode} eq $self->get_ALL_ERRORS() ? 1 : 0 ;
    my $function_name = $self->_get_js_function_name($data->{form_index}, $data->{random_name}) ;
    my $output = "" ;

    $output .= "\n<script language=\"JavaScript\">\n<!-- hide\n" ;
    $output .= " function ".$function_name."(theForm) {\n" ;
    $output .= " if (theForm == null) return true ;\n" ;
    $output .= " var isError = false ;\n " ;

    return $output ;
}
sub _get_final_js {
    my $self = shift ;
    return "" ;
}

sub _get_start_js_for_check {
    my $self = shift ;
    my $check = shift || return "" ;
    my $data = shift ;
    my $check_index = shift ;
    my $name = $check->{name} ;
    my $check_type = $check->{check_type} ;
    my $is_all_errors = $data->{mode} eq $self->get_ALL_ERRORS() ? 1 : 0 ;
    my $check_function = $CHECKER_MAPPINGS->{$check_type} || $CHECKER_MAPPINGS->{DEFAULT} ;

    my $output = "" ;
    $output .= "\n if (false) {}" if ($check_index == 0 && !$is_all_errors) ;
    if (!$is_all_errors) {
        $output .= "\n else if "  ;
    } else {
        $output .= "\n if " ;
    }
    $output .= "(theForm.".$name." != null && !(".$check_function."(theForm.".$name.".value))) {" ;
    $output .= "\n isError = true ;" ;
    return $output ;
}

sub _get_end_js_for_check {
    my $self = shift ;
    return "\n }" ;
}

#### filter-specific methods
sub _get_libfiles_js {
    my $self = shift ;
    my $imageslocation = $PARAMS->{'imageslocation'} ;
    my $libfile_location = $CONFIG->{libfile_location} ;

    if ($libfile_location !~ /^http:\/\//i && $libfile_location !~ /^https:\/\//i) {
        $libfile_location = $imageslocation."/".$libfile_location ;
    }

    my $output = "" ;

    $output .= "\n<script language=\"JavaScript\" src=\"" ;
    $output .= $libfile_location ;
    $output .= "\"></script>" ;

    return $output ;
}

sub _get_js_function_name {
    my $self = shift ;
    my $form_index = shift ;
    my $isa = shift ;
    my $name = lc("form_".$isa."_check_function".$form_index) ;

    return $name ;
}

sub _insert_js_event_handle_into_form_tag {
    my $self = shift ;
    my $form_tag = shift ;
    my $form_index = shift ;
    my $random_name = shift ;
    my $event_handle = $self->_get_js_event_handle($form_index, $random_name) ;

    return "" unless defined($form_tag) ;
    return $form_tag if ($form_tag =~ /\sonsubmit=/i) ;

    $form_tag =~ s/<form\s/<form $event_handle /i ;

    return $form_tag ;
}

sub _get_js_event_handle {
    my $self = shift ;
    my $form_index = shift ;
    my $random_name = shift ;
    my $function_name = $self->_get_js_function_name($form_index, $random_name) ;
    my $output = "" ;

    $output .= "onsubmit=\"javascript: return $function_name(this);\"" ;

    return $output ;
}

1;
