###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::JSForm::Filter::Dialog ;
use strict;

use Metadot qw ($PARAMS) ;

use vars qw(@ISA) ;
@ISA=qw(Metadot::JSForm::Filter) ;

#########################################################
##  Class Variables
#########################################################
my $CONFIG = {
    # http://, https:// or imageslocation-relative path
    libfile_location => "js/jsform/filter/dialog.js",
} ;

# mapping between element attribute and FormCheck.js call
# (see FormCheck.js for function definitions)
my $DIALOG_MAPPINGS = {
                        DEFAULT         => 'warnInvalid',
                        ALL_ERRORS      => 'warnAllErrors',
                        'notempty'      => 'warnEmpty',
                        'required'      => 'warnEmpty',
                        'email'         => 'warnInvalid',
                        'url'           => 'warnInvalid',
} ;

#########################################################
##  Constructor
#########################################################
sub new {
    my $proto = shift ;
    my $class = __PACKAGE__ ;
    my $self = {} ;
    $self  = $class->SUPER::new(@_) ;

    bless ($self, $class) ;
    return $self ;
}

#########################################################
##  Private Methods
#########################################################

#### overridden methods
sub CAN_BE_NUMBER_N_IN_CHAIN {
    my $class = shift ;
    my $number = shift ;
    return ($number == 0) ? 0 : 1 ;
}
sub _handle_form_html {
    my $self = shift ;
    my $form_html = shift ;
    return $form_html ;
}
sub _get_library_js {
    my $self = shift ;
    my $library_js = $self->_get_libfiles_js() ;
    return $library_js ;
}
sub _get_initialize_js {
    my $self = shift ;
    my $output = "" ;

    $output .= "\n var dialog_error_text = \"\" ; " ;

    return $output ;
}
sub _get_final_js {
    my $self = shift ;
    my $data = shift ;
    my $js_function = $DIALOG_MAPPINGS->{ALL_ERRORS} ;
    my $output = "\n ".$js_function."(dialog_error_text) ; " ;
    return $output ;
}

sub _get_start_js_for_check {
    my $self = shift ;
    my $check = shift || return "" ;
    my $label_text = $check->{label_text} ;
    my $check_type = $check->{check_type} ;
    my $js_function = $DIALOG_MAPPINGS->{$check_type} || $DIALOG_MAPPINGS->{DEFAULT} ;

    my $output = "\n dialog_error_text += ".$js_function."('".$label_text."', true) ; " ;
    return $output ;
}

sub _get_end_js_for_check {
    my $self = shift ;
    return "" ;
}

#### filter-specific methods
sub _get_libfiles_js {
    my $self = shift ;
    my $imageslocation = $PARAMS->{'imageslocation'} ;
    my $libfile_location = $CONFIG->{libfile_location} ;

    if ($libfile_location !~ /^http:\/\//i && $libfile_location !~ /^https:\/\//i) {
        $libfile_location = $imageslocation."/".$libfile_location ;
    }

    my $output = "" ;

    $output .= "\n<script language=\"JavaScript\" src=\"" ;
    $output .= $libfile_location ;
    $output .= "\"></script>" ;

    return $output ;
}

1;
