###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Metadot::MetadotController;
use strict;
use Data::Dumper;

#$Id: MetadotController.pm,v 5.69.2.13 2005/09/02 22:16:02 nico Exp $

use Metadotclass;
use Metadot qw(
    $USER %FORM $SESSION $HTTP_HEADER_SENT
    $PARAMS $CACHE $ACCESS_BROKER $DISPLAY %REQUEST_VARS $CGI
    );
use Portal;
use Metadot::PortalFactory;
use Metadot::SkinManager;
use Metadot::RequestContext;
use Template;

my $SINGLETON = undef;


# factory method
sub get_instance {
    my $class = shift;
    my @args  = @_;

    my $type = MetadotConfig->get_param_value('controller_type');
    my $result;

    if (defined $type) {
        my $file = Utility->package_to_filename($type);
        eval {
            require $file;
            $result = $type->new(@args);
        };
        if ($@) {
            print STDERR "Error instantiating MetadotController of type '$type'\n";
        }
    }

    $result ||= $class->new(@args);

    return $result;
}


sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = $SINGLETON;
    unless ($self) {
        $self = {};
        bless $self, $class;
        $SINGLETON = $self;
    }
    return $self;
}

sub test {
    return "This is the MetadotController";
}

use constant _FAILSAFE_  => '_failsafe';


my %MAPPING = (
    _FAILSAFE_ => 'generic.tt2',
    Calendar => {
        mappings => {
            show_full       => 'calendar.tt2',
            show_summary    => 'calendar_small.tt2',
        },
        default_op => 'show',
    },
    AggregatedCalendar => {
        mappings => {
            show_full       => 'calendar.tt2',
            show_summary    => 'calendar_small.tt2',
        },
        default_op => 'show',
    },
    Category => {
        mappings => {
            show_home               => 'portal_home.tt2',
            show_sub                => 'portal_sub.tt2',
            show_full               => 'portal_chooser.tt2',
            show_summary            => 'category_summary.tt2',
            show_skin_layout        => 'portal_chooser.tt2',
            show_printer_friendly   => 'portal_chooser.tt2',
            show_printer_friendly_layout  => 'portal_chooser.tt2',
        },
        default_op => 'show',
    },
    CMSDocument => {
        mappings => {
            show_full       => 'document.tt2',
            show_summary    => 'document_summary.tt2',
        },
        default_op => 'show',
    },
    Contractor => {
        mappings => {
            show_full       => 'organization.tt2',
            show_summary    => 'organization.tt2',
        },
        default_op => 'show',
    },
    ContactUs => {
        mappings => {
            show_full       => 'contact_us.tt2',
            show_summary    => 'contact_us.tt2',
        },
        default_op => 'show',
    },
    Discussion => {
        mappings => {
            show_full       => 'discussion.tt2',
            show_summary    => 'discussion_summary.tt2',
        },
        default_op => 'show',
    },
    Event => {
        mappings => {
            show_full       => 'event.tt2',
        },
        default_op => 'show',
    },
    Faq => {
        mappings => {
            show_full       => 'faq.tt2',
            show_summary    => 'faq.tt2',
        },
        default_op => 'show',
    },
    FormMaker => {
        mappings => {
            show_full       => 'form_maker.tt2',
            show_summary    => 'form_maker.tt2',
        },
        default_op => 'show',
    },
    GizmoShortcut => {
        mappings => {
            show_full       => 'gizmoshortcut.tt2',
            show_summary    => 'gizmoshortcut.tt2',
        },
        default_op => 'show',
    },
    ImageItem => {
        mappings => {
            show_full       => 'image_item.tt2',
            show_summary    => 'image_item.tt2',
        },
        default_op => 'show',
    },
    Item => {
        mappings => {
            show_full       => 'item_full.tt2',
            show_summary    => 'item.tt2',
        },
        default_op => 'show',
    },
    SQLGizmo => {
        mappings => {
            show_full         => 'dbtoolkit/sql_gizmo.tt2',
            show_summary      => 'dbtoolkit/sql_gizmo_summary.tt2',
            #show_search_table => 'dbtoolkit/sql_gizmo_search.tt2',
        },
        default_op => 'show',
    },
    JobRequest => {
        mappings => {
            show_full       => 'job_request.tt2',
            show_summary    => 'job_request.tt2',
        },
        default_op => 'show',
    },
    LayoutArea => {
        mappings => {
            show_full       => 'layout_area.tt2',
            show_summary    => 'layout_area.tt2',
        },
        default_op => 'show',
    },
    Message => {
        mappings => {
            show_full       => 'message.tt2',
            purge           => 'discussion_purger.tt2',
        },
        default_op => 'show',
    },
    "Metadot::SystemApp::MetagramCenter" => {
        mappings => {
            show_full       => 'metagrams/metagram_center.tt2',
            show_metagram   => 'metagrams/metagram.tt2',
            compose         => 'metagrams/compose_metagram.tt2',
        },
        default_op => 'show',
    },
    NavigationBar => {
        mappings => {
            show_full       => 'navigation_bar.tt2',
            show_summary    => 'navigation_bar.tt2',
        },
        default_op => 'show',
    },
    Newsitem => {
        mappings => {
            show_full       => 'newsitem_full.tt2',
            show_summary    => 'newsitem.tt2',
        },
        default_op => 'show',
    },
    NewsArticleImage => {
        mappings => {
            show_full       => 'newsarticleimage_full.tt2',
            show_summary    => 'newsarticleimage.tt2',
        },
        default_op => 'show',
    },
    NewsPaperView => {
        mappings => {
            show_full       => 'scrollview.tt2',
            show_summary    => 'scrollview.tt2',
        },
        default_op => 'show',
    },
    Poll => {
        mappings => {
            show_full       => 'poll.tt2',
            show_summary    => 'poll.tt2',
        },
        default_op => 'show',
    },
    Separator => {
        mappings => {
            show_full       => 'section.tt2',
            show_summary    => 'section.tt2',
        },
        default_op => 'show',
    },
    Session => {
        mappings => {
            login           => 'login.tt2',
        },
        default_op => 'login',
    },
    SiteLibrary => {
        mappings => {
            show_full       => 'site_library.tt2',
            show_summary    => 'site_library.tt2',
        },
        default_op => 'show',
    },
    SiteMap => {
        mappings => {
            show_full       => 'sitemap.tt2',
            show_summary    => 'sitemap.tt2',
        },
        default_op => 'show',
    },
    SubmissionSystem => {
        mappings => {
            show_full       => 'submission_system.tt2',
            show_summary    => 'submission_system_summary.tt2',
        },
        default_op => 'show',
    },
    SubmissionItem=> {
        mappings => {
            show_full       => 'submission_item.tt2',
            show_summary    => 'submission_item_summary.tt2',
        },
        default_op => 'show',
    },
    SuperGizmoRecord => {
        mappings => {
            show_full       => 'supergizmo_record_full.tt2',
            show_summary    => 'supergizmo_record.tt2',
        },
        default_op => 'show',
    },
    Table => {
        mappings => {
            show_full       => 'table.tt2',
            show_summary    => 'table_summary.tt2',
        },
        default_op => 'show',
    },
    Task => {
        mappings => {
            show_full       => 'task.tt2',
        },
        default_op => 'show',
    },
    TaskList => {
        mappings => {
            show_full       => 'task_list.tt2',
            show_summary    => 'task_list_summary.tt2',
        },
        default_op => 'show',
    },
);



# checks a form parameter 'disable_skins', and returns true if present
sub is_disabled {
    my $class = shift;
    my $session_key = 'disable_skins';

    # look in the form first
    my $result = $FORM{$session_key};

    # use the session if not found in the form
    if (defined $SESSION && UNIVERSAL::isa($SESSION, 'Session')) {
       if (defined $result) {
           $SESSION->set_value($session_key, $result);
       } else {
           $result = $SESSION->get_value($session_key);
       }
    }
    return $result;
}


# checks a form parameter 'force_default_skin', and returns true if present
sub force_default_skin {
    my $class = shift;
    my $session_key = 'force_default_skin';

    # look in the form first
    my $result = $FORM{$session_key};

    # use the session if not found in the form
    if (defined $SESSION && UNIVERSAL::isa($SESSION, 'Session')) {
       if (defined $result) {
           $SESSION->set_value($session_key, $result);
       } else {
           $result = $SESSION->get_value($session_key);
       }
    }

    #print STDERR "force_default_skin? '$result'\n";
    return $result;
}


sub do {
    my $class = shift;
    my $context = shift || die("Must supply the application context");
    my $args = shift || {};

    my $logged_in_callback = $args->{logged_in_callback};
    if (!defined $logged_in_callback) {
        $logged_in_callback = 1;
    }

    if ($class->is_disabled()) {
        return undef;
    }

    my $subject = $context->get_subject();
    return undef unless (defined $subject);

    return if $class->detect_deep_recursion($subject);

    my $op      = $context->get_op();

    my $isa;
    if ($subject->can('is_a')) {
        $isa = $subject->is_a();
    } else {
        $isa = ref($subject) || $subject;
    }

    my $user    = $context->get_user();
    my $result;


    Metadotclass->debug_msg(4,"MetadotController\->do with subject: $subject, op: $op, isa: $isa, user: $user; logged_in_callback: $logged_in_callback\n");

    # check permissions
    if (!$subject->can('is_allowed_to_do') || $subject->is_allowed_to_do($op, $user, $logged_in_callback)) {
        Metadotclass->debug_msg(5, "Metadotcontroller confirmed permission.");
        my $subject_class = ref($subject) || $subject;
        my $default_op  = $MAPPING{$subject_class}{default_op};

        # figure out the base name of the method to be called for this op
        my $exec_base;
        if ($subject->can('EXEC_METHOD_BASE')) {
            $exec_base = $subject->EXEC_METHOD_BASE();
        }
        $exec_base ||= 'exec_';

        # if no op was defined, use the default op
        if (!defined $op) {
            $op = $default_op;
        }

        my $exec_method = $exec_base.$op;
        my $exec_response;
        my $mapping;

        Metadotclass->debug_msg(5, "MetadotController tries to exec '$exec_method'");

        # now try to run the exec method
        if ($subject->can($exec_method)) {
            $exec_response = $subject->$exec_method($context);
            Metadotclass->debug_msg(5, "MetadotController got exec response '$exec_response'");

            if (defined $exec_response) {
                # see if the response is a Context object
                if (ref($exec_response) && $exec_response->isa('Metadot::RequestContext')) {
                    Metadotclass->debug_msg(5, "Will execute $exec_response");
                    $result = $class->do($exec_response);
                } elsif ($exec_response =~ /\.tt2$/) {
                    # the response is a direct reference to a tt2 file
                    $mapping = $exec_response;
                } else {
                    $mapping = $MAPPING{$subject_class}{mappings}{$exec_response};
                }
            }

            Metadotclass->debug_msg(5, "got mapping ".($mapping||'[undef]'));

            # finally, map to the generic one if the mapping still is not found.
            unless ($exec_response || $mapping) {
                $mapping = $MAPPING{_FAILSAFE_};
                Metadotclass->debug_msg(5, "Used the FAILSAFE mapping $mapping");
            }


            Metadotclass->debug_msg(3, "Using method  '$subject_class\->$exec_method', got response '".($exec_response||'[undef]')."' and mapping '".($mapping||'[undef]'."'\n"));

            # process the mapping. If I got a mapping, then I will process that as a template file.
            # If I did not get a mapping, then I will process the result as a template source.
            unless ($result) {
                if ($mapping) {
                    # currently, only .tt2 mapping is specifically supported
                    $result = $class->view({context=>$context, template=>$mapping});
                } elsif ($exec_response) {
                    # all other return values assume that the template source was provided
                    $result = $class->view({context=>$context, template_source=>$exec_response});
                }
            }

        } else {
            #BACKWARDS COMPATIBILITY WITH NON-TEMPLATE ENABLED GIZMOS
            if ($subject->isa('Gizmo')) {
                Metadotclass->debug_msg(4, "Attempted backward compatibility mode");
                if($subject->handle($op, 1)) {
                    $result = "<!-- MetadotController passed $subject/$op to handler -->";
                    Metadotclass->debug_msg(4, "Attempted $subject\->handle($op), result: ".($result ? 1 : 0));
                } elsif (!$result && $subject->can($op)) {
                    $result = $subject->$op();
                    Metadotclass->debug_msg(4, "Attempted $subject\->$op(), result: ".($result ? 1 : 0));
                }
            }
        }
    } else {
#        print STDERR "Operation not allowed\n";
    }
    return $result;
}


# stores template objects in a cache. all this will be moved to a Viewer class
#
# send the args:
#   Metadot::RequestContext  context            : specifies the context of the request
#   filename                 template           : specifies the template filename to process
#   string                   template_source    : specifies the template source to process

my $TEMPLATE_CACHE = {};

# increase the iteration limit
$Template::Directive::WHILE_MAX = 10000;

sub view {
    my $class = shift;
    my $args  = shift;

    if ($class->is_disabled()) {
        return undef;
    }

    my $sm = Metadot::SkinManager->get_instance();
    my $context = $args->{context} || Metadot::PortalFactory->build_context();
    my $subject = $context->get_subject();

    # this data will be sent to the template
    my $template_data = $args->{data} || {};

    #Metadotclass->debug_msg(4, "MetadotController view has template: '$args->{template}'; template_source: '$args->{template_source}'; context: '$context'; subject: '$subject'");

    return if $class->detect_deep_recursion($subject);

    # either send argument template, to refer to an existing template
    # or send string template_source, to provide the source code of the template
    my $template_source;
    if ($args->{template}) {
        $template_source = $args->{template};
    } elsif ($args->{template_source}) {
        $template_source = \$args->{template_source};
    } else {
        return '';
    }
    Metadotclass->debug_msg(5, "Will send template source '$template_source'; ref=".(ref($template_source) || 'null'));

    my @INCLUDE_PATH = (
                            $sm->get_template_include_path(),
                            Metadot::Gizmo3000->get_resource_directory(),
                            MetadotConfig->get_portal_extension_dir(),
                            @{ Portal::PORTAL_TEMPLATE_INCLUDE_PATH()},
                            );

    Metadotclass->debug_msg(4, "Template include path='@INCLUDE_PATH'");

    # check if the template object can be found in cache.
    my $template = $TEMPLATE_CACHE->{"@INCLUDE_PATH"};
    #print STDERR "Got from template from cache? $template\n";

    unless ($template) {
        $template = Template->new({
                      INCLUDE_PATH  => \@INCLUDE_PATH,
                      PRE_CHOMP     => 1,
                      POST_CHOMP    => 1,
                      RECURSION     => 1,
                     }) || die $Template::ERROR, "\n";
        $TEMPLATE_CACHE->{"@INCLUDE_PATH"} = $template;
        #print STDERR "Created template instance $template\n";
    }

    my $data = {
        %$template_data,
        self            => $subject,
        portal_context  => $context,
        portal_factory  => Metadot::PortalFactory->get_instance(),
        portal_params   => $PARAMS,
        display         => $DISPLAY,
        user            => $context->get_user(),
        form            => $context->get_form(),
        session         => $context->get_session(),
        skin_manager    => $sm,
        request_vars    => \%REQUEST_VARS,
    };

    #print STDERR Dumper $subject;
    #print STDERR Dumper $data;

    my $out;

    if ($template->process($template_source,  $data, \$out)) {
        Metadotclass->debug_msg(5, "Successfully processed template.");
        # for Category instances, do post_processing of user templates, unless the op is 'modify*'
        my $op = $context->get_op();

        # only process the user tag if the op is not actually modifying the content
        if (($op !~ '^(modify|save)')) {
            my $subject = $context->get_subject();

            if ($subject && $subject->can('processes_user_directives')) {
                $DISPLAY->debug_msg(5, "$subject can process user directives");
                if ($subject->processes_user_directives()) {
                    $DISPLAY->debug_msg(5, "$subject will process user directives");

                    my $reprocessed;
                    $class->process_user_tags(
                        input   => \$out,
                        output  => \$reprocessed,
                        context => $context,
                    );

                    $out = $reprocessed;
                } else {
                    $DISPLAY->debug_msg(5, "$subject will NOT process user directives");
                    # This was an affirmative 'no'.
                    # Escape the user tags so they aren't processed by an outer container
                    $out = $class->escape_user_tags($out);
                }
            }
        }

        return $out."\n";

    } elsif ($template->error()) {
        $class->handle_view_error($template);
    }
}


sub output {
    my $self = shift;
    my $args = shift;

    $DISPLAY->debug_msg(5, "Doing output, generating response");
    my $response = $self->view($args);

    if ($response) {
        $DISPLAY->print_http_header();
        print $response;
    } else {
        $DISPLAY->print_error("Error generating output");
    }
    exit;

}


# allows a template to call a custom page template as its wrapper
sub wrap_page_template {
    my $class = shift;
    my $args = shift || {};

    $DISPLAY->debug_msg(5, "wrapping page template");
    my $context = $args->{context} || die('Must supply the context');
    my $content = $args->{content};
    (defined $content) || die('Must supply content');

    my $subject     = $args->{self} || $context->get_subject();
    my $container   = $args->{container} || $subject->get_navigation_parent();

    my $page_template = $args->{page_template} || $container->get_page_template();

    my $template_source = <<EOM
[% WRAPPER page_template %]
$content
[% END %]

[% BLOCK page_template %]
$page_template
[% END %]
EOM
;

    #print STDERR "wrap_page_template will process:\n\n$template_source\n\n";
    return $class->view({
        context         => $context,
        template_source => $template_source,
    });
}


# process user tags
my $USER_TEMPLATE_PROCESSOR = Template->new({}) || die("Could not instantiate user template processor:\n", $Template::ERROR);
sub process_user_tags {
    my $class = shift;
    my %args = @_;

    my $input   = $args{input}      || die("Must specify input reference");
    my $output  = $args{output}     || die("Must specify output reference");
    my $context = $args{context};

    # if there are no tags, then don't bother.
    if ($$input =~ /\$\{/) {

        # as a safety precaution, first make sure there are no [% or %] tags
        $$input =~ s/\[\%/&#91;&#37;/g;
        $$input =~ s/\%\]/&#37;&#93;/g;

        # unescape text within template tags
        my @tags = ($$input =~ /\$\{.*?\}/gs);   # in case the tag is broken into two lines
        foreach my $tag (@tags) {
            my $replace = $tag;

            $replace =~ s/^\$\{/\[%/;
            $replace =~ s/\}$/%\]/;

            $replace =~ s/&quot;/"/g;
            $replace =~ s/&lt;/</g;
            $replace =~ s/&gt;/>/g;

            $$input =~ s/\Q$tag\E/$replace/gs;
        }

        # build some data structures for users to access
        my %data = ();

        # create some read-only representations (data transfer hashes) of context objects
        if ($context && ref($context)) {
            my (%user_dth, %form_dth, %self_dth);

            if ($context->get_user()) {
                %user_dth    = %{$context->get_user()};
                # password is a public field, but we don't want people to
                # have access to it. Even though it is hashed, it can
                # be broken by brute force.
                delete $user_dth{password};
            }
            if ($context->get_form()) {
                %form_dth    = %{$context->get_form()};
            }
            if ($context->get_subject && ref($context->get_subject())) {
                %self_dth    = %{$context->get_subject()};
            }

            $data{user}     = \%user_dth;
            $data{form}     = \%form_dth;
            $data{self}     = \%self_dth;
        }
        $data{display}       = $DISPLAY;

        unless ($USER_TEMPLATE_PROCESSOR->process($input, \%data, $output)) {
            $class->handle_view_error($USER_TEMPLATE_PROCESSOR  );
            $$output = $$input;
        }
    } else {
        # just pass the output
        $$output = $$input;
    }

}


# escapes user tags so they are not processed by other invocations of process_user_tags
sub escape_user_tags {
    my $class = shift;
    my $text  = shift;

    $text =~ s/\$\{/\$<!--escaped user tag -->{/g;

    return $text;
}



# this will eventually be moved to Viewer->handle_error
sub handle_view_error {
    my $class = shift;
    my $template = shift;

    my $error = "An error has occurred while processing the template:<br />\n".$template->error();

    Metadotclass->debug_msg(0, $error);

    return undef;
}


sub detect_deep_recursion {
    my $class = shift;
    my $subject = shift || return();
    my $limit = shift || 25;

    # avoid deep recursion
    my $run_counter = "$class\_RUNTIME_COUNTER";
    $REQUEST_VARS{$run_counter}->{$subject}++;
    return ($REQUEST_VARS{$run_counter}->{$subject} > $limit);
}


1;
