###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Metagram;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CGI);

use vars qw(@ISA $AUTOLOAD);
use Metadot::AbstractMessage;
use Utility;

@ISA = qw(Metadot::AbstractMessage);

sub new {
    my $proto = shift;
    my $mid = shift;
    my $class = ref($proto) || $proto;

    my $self;

    $mid  = undef unless Utility->is_valid_integer($mid); # secu_200401
    $self = $class->SUPER::new($mid);

    bless ($self, $class);

    return $self;
}

sub _get_default_status{
    my $self = shift;
    return $self->get_unread_status();
}

sub get_unread_status {
    return 'UNREAD';
}

sub get_read_status {
    return 'READ';
}

sub get_delete_code {
    return 'DELETE';
}

sub is_read {
    my $self = shift;
    return $self->{status} eq $self->get_read_status();
}

sub is_unread {
    my $self = shift;
    return !$self->is_read();
}

sub mark_metagrams_read {
    my $class = shift;
    my $args  = shift;

    $args->{status} = $class->get_read_status();
    return $class->modify_metagrams_status($args);
}

sub mark_metagrams_unread {
    my $class = shift;
    my $args  = shift;

    $args->{status} = $class->get_unread_status();
    return $class->modify_metagrams_status($args);
}

sub delete_metagrams {
    my $class = shift;
    my $args  = shift;

    $args->{status} = $class->get_delete_code();
    return $class->modify_metagrams_status($args);
}

sub modify_metagrams_status {
    my $class = shift;
    my $args  = shift;

    my $mids   = Utility->arrayify($args->{mids});
    my $uid    = $args->{uid};
    my $status = $args->{status};
    my $isa    = ref($class) || $class;

    $uid = undef unless Utility->is_valid_integer($uid); # secu_200401

    return unless $mids->[0] > 0 && defined($uid) && defined($status);

    my @update_mids = (); ## good list of mids
    foreach my $mid (@$mids) {
        next unless Utility->is_valid_integer($mid); # secu_200401
        push @update_mids, $mid;
    }

    return unless $update_mids[0] > 0;

    my $where  = "recipient_uid = '$uid' and isa='$isa' and "
               . "mid in (" . join(",", @update_mids) .")";

    if ($status eq $class->get_delete_code()) {
        DBAccess->sqlDelete("message", $where);
    } else {
        DBAccess->sqlUpdate("message", $where, status => $status);
    }
}

sub has_unread_metagrams {
    my $self = shift;
    my $args = shift;

    my $uid  = $args->{uid};
    return undef unless Utility->is_valid_integer($uid); # secu_200401

    return $self->user_has_messages_with_status({
               uid => $uid,
               status => $self->get_unread_status(),
              });
}

sub get_formatted_description {
    my $self = shift;
    my $description = $self->{description};
    $description =~ s/\n/<br \/>/g;
    return $description;
}

sub get_label {
    return MetadotConfig->get_param_value('metagram_label') || 'Metagram';
}


sub get_label_proper {
    return MetadotConfig->get_param_value('metagram_label_proper') || 'Metagram';
}


1;


__END__
