###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::MySQLTableInfo;

#========================================
#This module allows the form-building and form-processing modules to
#get info about the fields in the db tables.  It is mysql-specific.  It needs
#some robustness improvements; it uses a hand-generated mapping of field type codes
#to type strings.  This mapping probably does not include all the field
#types that we will require.  There is probably a way to build the
#mapping automatically; this needs to be pursued.
#========================================


use strict;
use Data::Dumper;

use MetadotConfig;
use DBAccess;

# tbd: this list of mappings needs to be further fleshed out.
use constant FIELD_TYPES => { 12 => 'char', 11 => 'datetime', 9 => 'date', -1 => 'text', 4 => 'number', 3 => 'number', -6 => 'number'};

#constructor
sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};

    bless ($self, $class);
    return $self;
}

# get info about the fields and values of the given database table.
# return info as a hash.
sub getDataInfo {
    my ($self, $select, $from, $where) = @_;

    #print STDERR "in getDataInfo with select $select, from $from, and where $where\n";
    my $sth;
    eval {
    $sth = DBAccess->sqlSelectMany($select, $from, $where);
    };
    my $hash_ref = $sth->fetchrow_hashref('NAME_lc'); #  just take first result row

    # if got no results, then just get field info instead.
    if (! $hash_ref) {
    return $self->getFieldInfo($from);
    }
    my $fields = $sth->{NAME_lc};
    my $types  = $sth->{TYPE};
    my $prec  = $sth->{PRECISION};

#      print STDERR "in getDataInfo: types: ", Dumper($types), ", fields " , Dumper($fields),
#        ", and precision: ", Dumper($prec), "\n";
    my $field = ""; my $ftype = ""; my $fvalue = "";
    my $col_info_hash = {};
    for ( my $i = 0; $i < scalar @{ $fields }; $i++ ) {
    my %col_info = ();
    $field = $fields->[$i];
    $ftype = $types->[$i];
#   $col_info{'name'} = $field;
    $col_info{'value'} = $hash_ref->{$field};
    $col_info{'precision'} = $prec->[$i];
    if (exists (FIELD_TYPES->{$ftype})) {
        $col_info{'type'} = FIELD_TYPES->{$ftype};
    }
    else {
        $col_info{'type'} = 'text'; # tbd...
    }
    $col_info_hash->{$field} = \%col_info;
    }
    return $col_info_hash;
}

# get info about the fields of the given database table.
# Return info as a hash.
sub getFieldInfo {
    my ($self, $from) = @_;
    my $col_info_hash = {};
    my $sth;

    # [vadim] if table does not exist, return empty hash
    return $col_info_hash unless (DBAccess->existsTable($from));

    eval {
       $sth = DBAccess->sqlSelectMany("*", $from, "1=0");
    };

    my $fields = $sth->{NAME_lc};
    my $types  = $sth->{TYPE};
    my $prec  = $sth->{PRECISION};

#      print STDERR "in FieldInfo: types: ", Dumper($types), ", fields " , Dumper($fields),
#        ", and precision: ", Dumper($prec), "\n";
    my $field = ""; my $ftype = "";
    for ( my $i = 0; $i < scalar @{ $fields }; $i++ ) {
    my %col_info = ();
    $field = $fields->[$i];
    $ftype = $types->[$i];
    $col_info{'precision'} = $prec->[$i];
#   print STDERR "got type $ftype for $field of $from\n";
    if (exists (FIELD_TYPES->{$ftype})) {
        $col_info{'type'} = FIELD_TYPES->{$ftype};
    }
    else {
        #print STDERR "didn't know type for $ftype for $field of $from\n";
        $col_info{'type'} = 'text'; # tbd...
    }
    $col_info_hash->{$field} = \%col_info;
    }
    return $col_info_hash;
}


1;
