###########################################################################
# <LICENSEINFO>
# Metadot SmartSite Server                                                                      
# Copyright (C) 2001 Metadot Corporation                                                        
#                                                                                               
# For more information go to                                                                    
# http://www.metadot.com                                                                        
# or contact us at info@metadot.com                                                             
#                                                                                               
#</LICENSEINFO>
############################################################################

# An interface to specify that a gizmo should appear in navigation.
# You should use this if you want a gizmo to appear in site maps and navigation lists

package Metadot::Navigable;

use strict;
use Metadot qw($DISPLAY);
use Metadot::PortalFactory;


# returns the level of a page in the site hierarchy, where home page has level 1
sub get_navigation_level {
    my $self = shift;
    return $self->get_distance_from_home() + 1;
}


# returns the navigational distance from home
sub get_distance_from_home {
    my $self = shift;
    my $home_obj = Metadot::PortalFactory->get_home_object();
    
    my $distance = 0;
    my $cursor = $self;
    my %visited = ();       # track visited pages to avoid infinite loops

    while (defined $cursor && $cursor->get_id() >= 0 && !$visited{$cursor->get_id()} && $cursor->get_id() != $home_obj->get_id()) {
        $distance++;
        $visited{$cursor->get_id()} = 1;
        $cursor = $cursor->get_parent();
    }

    return $distance;
}


# indicates whether a navigable gizmo is hidden in navigation
sub is_hidden {
    return 0;
}


# Allows any Navigable subclass to work with page templates. Override this if
# you want to implement page templates in the class.
sub page_template_enabled {
    my $self = shift;
    
    # kick this upstairs
    my $next_parent = $self->get_navigation_parent({exclude_self=>1});
    return ($next_parent ? $next_parent->page_template_enabled() : 0);
}


# Allows any Navigable subclass to work with page templates. Override this if
# you want to implement page templates in the class.
sub page_has_custom_template {
    my $self = shift;
    
    # kick this upstairs
    my $next_parent = $self->get_navigation_parent({exclude_self=>1});
    return ($next_parent ? $next_parent->page_has_custom_template() : 0);
}


# Allows any Navigable subclass to work with page templates. Override this if
# you want to implement page templates in the class.
sub get_page_template {
    my $self = shift;

    # kick this upstairs
    my $next_parent = $self->get_navigation_parent({exclude_self=>1});
    return ($next_parent ? $next_parent->get_page_template() : undef);
}




# for backwards compatibility with non-skin mode, also needed by SiteMap gizmo
sub show_as_nav_line {
    my $self = shift;
    my %args = @_;

    my $iid = $self->id();
    my $name = $self->name();
    my $isa = $self->is_a();
    my $style = $args{style} || 'catlistmain';

    my $result = "<a class=$style href=\"".$DISPLAY->make_url("index.pl?iid=$iid&amp;isa=$isa")."\">$name</a>";
    return $result;
}


# for backwards compatibility with non-skin mode, also needed by SiteMap gizmo
sub get_image_url {
    return undef;
}







1;
