###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################


# Notifiable is a subclass of Invitable, with the intent of allowing
# the owner of a Gizmo to broadcast the fact that the Gizmo has been
# created.
package Metadot::Notifiable;

use strict;
use vars qw(@ISA);

use Metadot qw($PARAMS);
use Metadot::Notifiable;

@ISA = qw(Metadot::Invitable);


# default changes to permissions for Notifiable gizmos
sub customize_permissions {
    my ($self, $ac) = @_;

    # handle invitable - only owner can invite
    $ac->add_bundle('BROADCAST', 'Send Broadcast Messages', '8','8');
    $ac->set_broadcast('BROADCAST', 1);

    $ac->add_op('BROADCAST','invite');

    return $ac;
}


# typically called within show_edit_properties_form
sub show_notification_properties {
    my $self = shift;

    my $display_name;
    if ($self->isa('Metadot::GizmoBase')) {
        $display_name = $self->get_gizmo_name();
    } else {
        $display_name = ref($self) || $self;
    }

    my $id  = $self->id();
    my $isa = $self->is_a();
    my $op  = 'invite';

    my $out = qq~
<div>
<b>Click this button to send an ad-hoc email to notify users of this $display_name:
<a class="text_button"
   href="index.pl?isa=$isa&amp;iid=$id&amp;op=$op"
   target="notify_popup"
   >Notify by ad-hoc email</a>
<img src="$PARAMS->{imageslocation}/new_window.gif" />
</div>
~;

    return $out;
}


sub get_invite_interface_message {
    my $self = shift;
    my $gizmo_name = $self->get_display_name();
    my $s .= <<EOM
<h1>Email users to view this $gizmo_name.</h1>
Below, you can search for users you would like to notify
to view this $gizmo_name. You may also invite entire groups or type in the
email address of the user. Once
you\'ve finished, click on the 'Next' button to send an e-mail
inviting the people you have selected.
EOM
;
    return $s;
}

sub get_invite_title_keyword {
    return 'notify';
}

sub get_invite_title_keyword_proper {
    return 'Notify';
}

sub get_invite_noun_keyword {
    return 'notification';
}

sub _get_invitation_subject_html_default_field_text {
    my $self = shift;
    my $gizmo_name = $self->get_display_name();
    return "A new $gizmo_name has been created for your review.";
}


sub _get_invitation_body_html_default_field_text {
    my $self = shift;
    my $gizmo_name = $self->get_display_name();
    my $default_text = <<EOM
I have created a new $gizmo_name. Please review it at your earliest convenience. Thank you.
EOM
;
    return $default_text;
}


# override the normal invite behavior - we are closing a popup
sub show_invitation_confirmation {
    my ($self, %params) = @_;
    my $out;
    my $return_link = 'javascript:window.close()';
    my $title_keyword = $self->get_invite_noun_keyword();

    $out .= "Your $title_keyword has been sent!<br /><a href=\"$return_link\">Click here to close this window</a>";
    return $out;
}




1;

__END__

