#!/usr/bin/perl -w

#
# Test file for Permissions.pm
# $Id: Permissions.001.t,v 1.1.2.2 2005/07/01 22:41:28 nico Exp $
#

use strict;
use Test::More tests=>200;

use SmokeMetadot;
use GizmoBuilder;
use Metadot::Permissions;


#################################################
# Test: Ensure that Permissions can serialize and
#       deserialize an object without defects
#################################################

test_random_permissions(100);





#################################################
# That's it!!!!
#################################################


sub test_random_permissions
{
    my $runs = shift || 100;
    my $isa = 'PermissionTestScript';

    # delete the permissions entries
    DBAccess->sqlDelete('permissions', "isa='$isa'");
    
    for (my $run = 0; $run < $runs; $run++)
    {
        my $operations = Operations->new();
        GizmoBuilder->customize_permissions($operations);

        my $loops;
        my $i;

        # add some random bundles
        $loops = int(rand(10));
        for ($i = 0; $i<$loops; $i++)
        {
            my $bundle_name         = generate_random_name();
            my $bundle_description  = generate_random_name();
            $operations->add_bundle(
                $bundle_name,
                $bundle_description,
                generate_random_level(),
                generate_random_level()
                );

            # add some random ops
            my $loops2 = int(rand(20))+1;
            for (my $j = 0; $j<$loops2; $j++)
            {
                $operations->add_op($bundle_name, generate_random_name());
            }
        }


        # keep a record of all my bundles
        my @bundles = keys(%{$operations->{BUNDLES}});

        # generate a permissions object from this
        my $perms = Metadot::Permissions->generate_permissions_object($operations);

        # add some random group permissions
        $loops = int(rand(20));
        for ($i = 0; $i<$loops; $i++)
        {
            my $bundle = $bundles[ int(rand(scalar(@bundles)))];
            $perms->add_group_acl_pair(int(rand(1000)+1), $bundle);
        }

        # add some random user permissions
        $loops = int(rand(20));
        for ($i = 0; $i<$loops; $i++)
        {
            my $bundle = $bundles[ int(rand(scalar(@bundles)))];
            $perms->add_user_acl_pair(int(rand(1000)+1), $bundle);
        }


        # serialize it
        my $serialized = Metadot::Permissions->serialize($perms);

        # save it to the database
        DBAccess->sqlInsert(
            'permissions',
            id=>$run, isa=>$isa, permissions=>$serialized
        );

        # retrieve it from the database
        my ($retrieved) = DBAccess->sqlSelect(
            'permissions', 'permissions', "id=$run AND isa='$isa'"
            );

        is($serialized, $retrieved) || diag("Failed the round trip from the database");

        my $deserialized = Metadot::Permissions->deserialize($retrieved);
        is_deeply($perms, $deserialized) || diag("Failed the round trip from serialization");
    }

    # delete the permissions entries
    DBAccess->sqlDelete('permissions', "isa='$isa'");
}


# generates a random name
sub generate_random_name
{
    my $max_length = shift || 20;
    my $min_length = shift || 1;

    my $out = '';
    my $length = int(rand($max_length-$min_length)) + 1;
    if ($length > 0)
    {
        while (length($out) < $length)
        {
            $out .= chr(int(rand(52))+65);
            if (rand(6)<1)
            {
                $out .= '_';
            }
        }
    }

    return $out;
}


sub generate_random_level
{
    return int(rand(11));
}
