package Metadot::PortalFactory;
use strict;
use Data::Dumper;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $CACHE $ACCESS_BROKER $DISPLAY);
use Portal;
use Subscription;
use Metadot::MetadotController;
use Metadot::RequestContext;
use Metadot::AuthenticatorFactory;
use MetadotConfig;
use Metadot::UserFactory;
use Metadot::GizmoBase;
use Metadot::ChannelProcessor;

my $SINGLETON = undef;

# this now an alias for get_instance
sub new {
    return get_instance(@_);
}

sub get_instance {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = $SINGLETON;
    unless ($self) {
        $self = {};
        bless $self, $class;
        $SINGLETON = $self;
    }

    return $self;
}

sub test {
    return "This is a PortalFactory";
}

sub get_user {
    my $class  = shift;
    my $userid = shift;

    my $result;
    if (defined $userid) {
        $result = Metadot::UserFactory->get_user($userid);
    } else {
        $result = $USER;
    }

    #print STDERR "PortalFactory gets user with uid '$userid' $result (".Dumper($result).")\n";
    return $result;
}

sub get_display {
    return $DISPLAY;
}

sub get_controller {
    return Metadot::MetadotController->get_instance();
}

sub build_context {
    my $class = shift;
    my $args = shift;


    my $subject = $args->{subject};
    my $user    = $args->{user} || $USER;
    my $form    = $args->{form} || \%FORM;
    my $op      = $args->{op}   || $form->{op};
    my $session = $args->{session} || $SESSION;
    my $extra_args = $args->{extra_args};

    # if no subject was provided, try to determine
    # based on iid and isa
    unless ($subject) {
        my $iid;
        if (defined $args->{iid}) {
            $iid = $args->{iid};
        } else {
            $iid = $form->{iid} || $form->{id};
        }
        my $isa;
        if (defined $args->{isa}) {
            $isa = $args->{isa};
        } else {
            $isa = $form->{isa};
        }

        if ($isa && $isa->can('restore')) {
            $subject = $isa->restore($iid);
        } elsif(defined $iid) {
            $subject = Metadot::GizmoBase->restore($iid);
        }
    }

    my $context = Metadot::RequestContext->new({
        subject => $subject,
        op      => $op,
        user    => $user,
        form    => $form,
        session => $session,
        extra_args => $extra_args
    });

    return $context;
}

sub get_home_object {
    my $class = shift;
    my $iid = MetadotConfig->get_reserved_iid('ROOT_CATEGORY');
    Metadotclass->debug_msg(5, "$class\->get_home_object seeks object using iid '$iid'");
    my $object = Metadot::GizmoBase->restore($iid);
    Metadotclass->debug_msg(5, "get_home_object finds $object with iid '".$object->id()."'");

    return $object;
}

sub get_hidden_category {
    my $class = shift;
    my $object = Metadot::GizmoBase->restore(MetadotConfig->get_reserved_iid('HIDDEN_CATEGORY'));
    return $object;
}

sub get_parent_of_root {
    my $class = shift;
    my $object = Metadot::GizmoBase->restore(MetadotConfig->get_reserved_iid('PARENT_OF_ROOT'));
    return $object;
}

sub get_portal_parent {
    my $class = shift;
    my $object = Metadot::GizmoBase->restore(MetadotConfig->get_reserved_iid('PORTAL_PARENT'));
    return $object;
}

sub get_basic_tool_parent {
    my $class = shift;
    my $object = Metadot::GizmoBase->restore(MetadotConfig->get_reserved_iid('PARENT_BASIC_TOOL'));
    return $object;
}

sub get_clipboard_parent {
    my $class = shift;
    my $object = Metadot::GizmoBase->restore(MetadotConfig->get_reserved_iid('PARENT_CLIPBOARD'));
    return $object;
}


sub get_gizmo_object {
    my $class = shift;
    my $iid = shift;
    my $object;
    if (Utility->is_valid_integer($iid)) {
        $object = Metadot::GizmoBase->restore($iid);
    }
    return $object;
}

sub get_site_library {
    return SiteLibrary->get_instance();
}

sub get_authenticator {
    return Metadot::AuthenticatorFactory->get_authenticator();
}

# Get an instance of Utility
sub get_utility_instance {
    return Utility->new();
}

# Get an instance of Portal
sub get_portal_instance {
    return Portal->new();
}

# an alias for get_utility_instance (get_utility_object is now deprecated)
sub get_utility_object {
    return &get_utility_instance();
}

# Gets an instance of channel processor
sub get_channel_processor {
    my $class = shift;

    return Metadot::ChannelProcessor->get_instance(@_);
}

















# TODO:  WHY ARE THESE HERE? Nico, these are not factory methods. Please
# use Factory classes ONLY for factory methods. When are you going to move
# these somewhere sensible?  - CKIM

sub get_subscription_list {
    my $self = shift;
    my $uid = shift;
    $DISPLAY->debug_msg(4, "calling get_user_subscriptions with uid[$uid]");
    return Subscription->get_user_subscriptions($uid);
}


# CKIM: refactored the other stuff I asked Nico to refactor!





1;
