###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name$
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Rateable;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $DISPLAY $PARAMS);
use CGI;
use DBAccess;

my $CONFIG = {
          show_average_on_rate_link => 'yes',
          edit_rating_allowed => 'yes',
          show_rate_link_only_in_edit_mode => 'no',
          non_logged_in_users_can_see_average_rating => 'yes',
          rate_comment_is_optional => 'yes',
          raters_remain_anonymous => 'no',
         };

# warning: do not change unless existing database entries are first
# converted to new scale
# aju 06-6-02 -- changed to 10 so that ilpf can use out-of-the-box code base
my $MAX_RATE = 10;

sub show_rate_link {
    my $self = shift;

    unless ( $USER->is_logged_in() ) {
    if( lc $CONFIG->{non_logged_in_users_can_see_average_rating} eq 'yes' ) {

        my $details = $self->_get_rates_detail();

        if( defined $details->{AVERAGE} ) {
        return '(Average Rating: ' . $details->{AVERAGE} . ') ';
        } else {
        return '(Not Rated) ';
        }

    } else {
        return '';
    }
    }

    if( lc $CONFIG->{show_rate_link_only_in_edit_mode} eq 'yes' ) {
    return '' unless($SESSION->admin_mode());
    }

    my $id = $self->id();
    my $class_name = $self->is_a();

    my $result_html = "<SCRIPT TYPE=\"text/javascript\" SRC=\"/js/rate_this.js\"></SCRIPT>\n";

    if( $self->_is_rated_by_user() ) {

    if( lc $CONFIG->{show_average_on_rate_link} eq 'yes' ) {

        my $details = $self->_get_rates_detail();

        if( defined $details->{AVERAGE} ) {
        $result_html .= "<b>Average Rating:</b> " . $details->{AVERAGE} . ' ';
        } else {
        $result_html .= "Not Rated ";
        }

    }

    my ($rate) = $self->_get_rate_and_comment_by_user();

    $result_html .= "<b>Your Rating:</b> " . $rate . '<br />';

    if( $self->is_allowed_to_do('rate', $USER) ) {

        $result_html .= " <a href=\"javascript:RateThis( \'index.pl?id=$id&amp;isa=$class_name&amp;op=rate\' )\">";
        $result_html .= (lc $CONFIG->{edit_rating_allowed} eq 'yes') ?
          '<i>(change or delete my rating)</i>' : '<i>(view my rating)</i>';
        $result_html .= '</a> ';

    }

    } elsif( $self->is_allowed_to_do('rate', $USER) ) {

        $result_html .= " <a href=\"javascript:RateThis( \'index.pl?id=$id&amp;isa=$class_name&amp;op=rate\' )\">";
        $result_html .= 'Add your rating';
        $result_html .= '</a> ';
    }

    if( $self->is_allowed_to_do('rating_details', $USER) ) {

    $result_html .= "&nbsp;<a href='index.pl?id=$id&amp;isa=$class_name&amp;op=rating_details'>";
    $result_html .= '<i>(view comments)</i>';
    $result_html .= '</a> ';

    }

    return "$result_html<br /><br />";
}

sub rate_op {
    my $self = shift;
    my $title = shift;

    unless(defined $title) { die "must provide title\n" };

    my $return_html = "<h1> Your Rate for $title </h1><p />";

    my ( $rate, $comment );

    my $is_rated_by_user = $self->_is_rated_by_user() || '';

    if( $is_rated_by_user ) {

    ($rate, $comment) = $self->_get_rate_and_comment_by_user();

    unless ( lc $CONFIG->{edit_rating_allowed} eq 'yes' ) {

        $return_html .= "Your Rate: $rate<p />\n";
        $return_html .= "Your Comment:<p />$comment<p />\n";

        # XXX [claudio] add a return button

        return $return_html;
    }
    }

    $rate = ( $FORM{rate} ) ? $FORM{rate} : $rate;
    $comment = ( $FORM{comment} ) ? $FORM{comment} : $comment ;

    my $errormsg;
    my $isa = $self->is_a();
    my $abs_name = $self->path_qualified_name();

    if ($FORM{rate_action} eq 'delete_rate') {

    my $class_name = $self->is_a();

    my $confirm_url = "index.pl?iid=" . $self->id .
      "&amp;isa=$class_name&amp;op=delete_rating";

    return "
<BODY BGCOLOR=\"FFFFFF\" LINK=\"#0033CC\" ALINK=\"RED\" VLINK=\"#0033CC\">
<CENTER>
<p /><br />
<FONT SIZE=\"5\" FACE=\"Arial, Helvetica, sans-serif\">Please confirm deletion of your rate for this $isa</FONT>
<br /><p />
<FONT SIZE=\"2\" FACE=\"Arial, Helvetica, sans-serif\">
<a href=\"$confirm_url\">Yes, I want to delete my rate</a><br /><br />
<a href=\"#\" onClick='self.close();'>Cancel</a><br /><br />
</CENTER>
</FONT>
</BODY>";

    }
    elsif($FORM{rate_action} eq 'save_rate') {
    # save and redirect or generate error message

    unless( (defined $rate) && ($rate >= 0) && ($rate <= $MAX_RATE) ) {
        $errormsg .= "<LI>Please select a rate";
    }

    unless ( lc $CONFIG->{rate_comment_is_optional} eq 'yes' ) {
        unless ( $comment ) {
        $errormsg .= "<LI>Comment field must not be empty";
        }
    }

    unless ($errormsg) {
        $comment = $comment || '';
        $self->_update_rate($rate, $comment);

        return "
<BODY BGCOLOR=\"FFFFFF\" LINK=\"#0033CC\" ALINK=\"RED\" VLINK=\"#0033CC\">
<CENTER>
<p /><br />
<FONT SIZE=\"5\" FACE=\"Arial, Helvetica, sans-serif\">Thank you</FONT>
<br /><p />
<FONT SIZE=\"2\" FACE=\"Arial, Helvetica, sans-serif\">
<a href=\"#\" onClick='self.close();'>Close This Window</a></CENTER>
</FONT>
</BODY>";
    }
    }

    my ($dial);
    for( 0 .. $MAX_RATE ) {

    $dial .= "<td><input type='radio' name='rate' value='$_'";
    if ( defined $rate && ( $_ == $rate ) ) { $dial .= " checked " }
    $dial .= "><br />&nbsp;$_</td>";
    }

    $dial = "
<table>
   <tr>
      <td valign=\"middle\">Poor</td>
      <td valign=\"middle\"><table CELLPADDING=\"0\" CELLSPACING=\"8\"> <tr align='center'>$dial</tr></table>
      </td>
      <td valign=\"middle\">Excellent</td>
   </tr>
</table>";

    my $comment_is_mandatory = (lc $CONFIG->{rate_comment_is_optional} eq 'yes' ) ?
      '(optional)' : '(mandatory)';

    my $target = $FORM{target} || $SESSION->{referer};
    my $iid = $self->id();

    my ( $header_bg_color, $header_text_color ) = ( '#003366', '#FFFFFF' );

    if ( Metadot::Utils::Theme->use_theme_info() ) {

    $header_bg_color = $PARAMS->{header1_color} || $header_bg_color;
    $header_text_color = $PARAMS->{title_text_color} || $header_text_color;
    }

    my $page_top = $DISPLAY->show_page_top();

    my $delete_rating_form = ( $is_rated_by_user ) ? "
<td>
<form action='index.pl'  method='post' ENCTYPE=\"multipart/form-data\">
<input type='hidden' name='op' value='rate'>
<input type='hidden' name='isa' value='$isa'>
<input type='hidden' name='iid' value='$iid'>
<input type='hidden' name='target' value='$target'>
<input type='hidden' name='rate_action' value='delete_rate'>
<input type='submit' value='Delete Rating'>
</form></td>
" : '';

    $return_html = "
$page_top
<HTML>
<HEAD>
<TITLE>Rate this $isa</TITLE>
</HEAD>
<BODY bgcolor=\"#ffffff\" LEFTMARGIN=\"0\" TOPMARGIN=\"0\" MARGINWIDTH=\"0\" MARGINHEIGHT=\"0\">

<table width=\"110%\" border=\"0\" cellpadding=\"12\" cellspacing=\"0\">
  <tr>
     <td width=\"100%\" valign=\"middle\" align=\"left\"
       bgcolor=\"$header_bg_color\"><b><font color=\"$header_text_color\">

How do you rate this $isa?

</font></b>
<br /><br /><font color=\"$header_text_color\">$abs_name</font>

     </td>
  </tr>

<tr>
<td bgcolor=\"#FFFFFF\">
  <table>
    <tr>
      <td>
<form action='index.pl'  method='post' ENCTYPE=\"multipart/form-data\">
<input type='hidden' name='op' value='rate'>
<input type='hidden' name='isa' value='$isa'>
<input type='hidden' name='iid' value='$iid'>
<input type='hidden' name='target' value='$target'>
<input type='hidden' name='rate_action' value='save_rate'>
$errormsg
$dial

      </td>
    </tr>

    <tr>
      <td align=\"center\">
        <table><tr><td><b>Comment</b> $comment_is_mandatory</td></tr>
           <tr><td>
<textarea name='comment' rows=2 cols=20 WRAP='VIRTUAL'>$comment</textarea>
           </tr></td>
        </table>
      </td>
    </tr>
    <tr>
      <td align=\"center\">
         <table>
           <tr>
              <td>
<input type='submit' value='Save'>
</form>
              </td>
$delete_rating_form
              <td>
<form ><input type=\"button\" value=\"Cancel\" onClick=\"window.close();\" name=\"button\"></form>
             </td>
           </tr>
        </table>
      </td>
    </tr>
  </table>
</td></tr></table>
</BODY>
</HTML>
";

    return $return_html;
}

sub rating_details_op {
    my $self = shift;

    my $title = shift;
    unless(defined $title) { die "must provide title\n" };


    my $return_html = "<h1>Rating Details for $title</h1><p />";

    my $details = $self->_get_rates_detail();

    if ($details->{NUM_RATERS} == 0) {

    $return_html .= '<b>Not yet rated</b> Please come back later.';

    return $return_html;
    }

    $return_html .= "<b>Average rating: </b> $details->{AVERAGE}<p />\n";
    if ($details->{NUM_RATERS} == 1) {
    $return_html .= "<b>Has been rated by: </b> $details->{NUM_RATERS} user<p />\n";
    }
    else {
    $return_html .= "<b>Has been rated by: </b> $details->{NUM_RATERS} users<p />\n";
    }
    $return_html .= "<b>User comments: </b> <p />\n";

    foreach my $uid (keys %{$details->{BY_UID}}) {
    my $user_name;
    unless(lc $CONFIG->{raters_remain_anonymous} eq 'yes') {

        my $rater = Metadot::UserFactory->get_user( $uid );

        $user_name = $rater->fullname();

        $user_name = "From <b>$user_name</b>: \n";
    }

    my $comment = $details->{BY_UID}{$uid}{COMMENT};
    my $rate = $details->{BY_UID}{$uid}{RATE};

    $return_html .= "$user_name<br />$comment<br />Rate: $rate<p />";

    }

    return $return_html;
}

sub _delete_check {
    my $self = shift;
    my $new_target = shift;
    $new_target = escape($new_target);

    my $class_name = $self->is_a();
    my $id = $self->id();

    $self->two_choices("Do you want to delete this rating?", "confirm",
             "Yes", "index.pl?iid=" . $self->id .
               "&isa=$class_name&op=delete_rating&target=$new_target",
             "No", $SESSION->{referer} . "&target=$new_target");
    exit;
}

sub www_delete_rating {
    my $self = shift;

    my $uid = $USER->uid();
    my $class_name = $self->is_a();
    my $id = $self->id();

    if( $self->_is_rated_by_user() ) {
        DBAccess->sqlDelete( "rates",
         "rater_uid=$uid and rated_class='$class_name' and rated_id=$id");
    }

    $DISPLAY->print_http_header();

    print "
<BODY BGCOLOR=\"FFFFFF\" LINK=\"#0033CC\" ALINK=\"RED\" VLINK=\"#0033CC\">
<CENTER>
<p /><br />
<FONT SIZE=\"5\" FACE=\"Arial, Helvetica, sans-serif\">Your rate has been deleted</FONT>
<br /><p />
<FONT SIZE=\"2\" FACE=\"Arial, Helvetica, sans-serif\">
<a href=\"#\" onClick='self.close();'>Close This Window</a></CENTER>
</FONT>
</BODY>";

}

sub _update_rate {
    my $self = shift;
    my $rate = shift;
    my $comment = shift;

    unless(defined $rate && defined $comment) {
    die "rate and comment must be supplied";
    }

    my $uid = $USER->uid();
    my $class_name = $self->is_a();
    my $id = $self->id();

    # will create the record unless it is already there
    if( $self->_is_rated_by_user() ) {

        DBAccess->sqlUpdate( "rates",
                 "rater_uid=$uid and rated_class='$class_name' and rated_id=$id",
                 ( rate => $rate,
                   comments => $comment ) );

    } else {

    DBAccess->sqlInsert( "rates", ( rate => $rate,
                  comments => $comment,
                  rater_uid => $uid,
                  rated_class => $class_name,
                  rated_id => $id ) );
    }
}

sub _get_rates_detail {
    my $self = shift;

    my $class_name = $self->is_a();
    my $id = $self->id();

    my $rates_struct = {};

    my $rates = DBAccess->sqlSelectMany( "rate, comments, rater_uid", "rates",
                     "rated_class='$class_name' and rated_id=$id" );

    my ($rates_num, $rates_sum) = (0, 0);
    while (my ($rate, $comment, $rater_uid) = $rates->fetchrow()) {
    $rates_struct->{BY_UID}{$rater_uid}{RATE} = $rate;
    $rates_struct->{BY_UID}{$rater_uid}{COMMENT} = $comment;

    $rates_num++;
    $rates_sum += $rate;
    }

    if($rates_num > 0) {
    $rates_struct->{AVERAGE} = $rates_sum / $rates_num;
    }

    $rates_struct->{NUM_RATERS} = $rates_num;

    return $rates_struct;
}

sub _get_rate_and_comment_by_user {
    my $self = shift;

    my $uid = $USER->uid();
    my $class_name = $self->is_a();
    my $id = $self->id();

    unless( (defined $uid) && (defined $class_name) && (defined $id) ) {
    return undef;
    }

    my ($rate, $comment) =
      DBAccess->sqlSelect("rate, comments", "rates",
              "rater_uid=$uid and rated_class='$class_name' and rated_id=$id" );

    return ($rate, $comment);
}

sub _is_rated_by_user {
    my $self = shift;

    my $uid = $USER->uid();
    my $class_name = $self->is_a();
    my $id = $self->id();

    unless( (defined $uid) && (defined $class_name) && (defined $id) ) {
    return undef;
    }

    my ($cnt) = DBAccess->sqlSelect("count(*)", "rates",
                    "rater_uid=$uid and rated_class='$class_name' and rated_id=$id"
                   );

    return ($cnt > 0) ? 1 : undef;

}

sub www_rate {
    my $self = shift;

    my $title = $self->name();

    $DISPLAY->print_http_header();

    print $self->rate_op( $title );
}


sub www_rating_details {
    my $self = shift;

    my $title = $self->name();

    $self->print_portal( $self->rating_details_op( $title ) );
}



1;

__END__

# This is the SQL for keeping Rate information

CREATE TABLE rates (
rater_uid      INT NOT NULL,
rated_class    VARCHAR(128) NOT NULL,
rated_id       INT NOT NULL,
rate           INT NOT NULL,
comments       TEXT,
PRIMARY KEY (rater_uid, rated_class, rated_id)
)




