###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Register::Default;

=head1 NAME

Metadot::Register::Default -

=head1 SYNOPSIS

    use Metadot::Register::Default.pm
    $register  = Metadot::Register::Default->new();
    $register->send_new_user_notification();
    $register->www_register_form
    $register->www_register
    $register->www_show_params
    $register->www_edit_params
    $register->www_save_params
    $register->save_modify_settings_form
    $register->show_modify_settings_form
    $register->send_approval_message
    $register->is_a
    $register->id
    $register->save
    $register->get_owner_uid
    $register->parent_id


=head1 DESCRIPTION

Metadot::Register::Default is a concrete subclass of the Metadot::Register class.
This subclass is instantiated and returned by Metadot::RegisterFactory, if the
factory determines that this subclass is the one that it wants to return.

This particualar subclass allows people to register by entering their name
and password.


Show the form that people use to register on the site
    $register->www_register_form

Process the register form

    $register->www_register

Show the registration parameters

    $register->www_show_params

Form to edit the registration paramaters

    $register->www_edit_params

Code to process the changed parameters

    $register->www_save_params

Code to process the changed registration parameters

    $register->save_modify_settings_form

Form to edit the registration parameters

    $register->show_modify_settings_form

Sends an approval message to the new user

    $register->send_approval_message


=head1 METHODS

=cut



use strict;

use Metadot::SessionHandlerFactory;
use Metadot::Register;
use Form::Verify;
use CGI qw(escapeHTML);
use Utility;
use Session;
use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CACHE $ACCESS_BROKER );
use Metadot::AuthenticatorFactory;

use vars qw(@ISA);
@ISA=qw(Metadot::Register);


my $AC = {
       BUNDLES => {

                   CREATE => { DESC =>          'Create Account',
                               OPS =>            [qw(register_form register show_params edit_params modify_params save_params)],
                               DEFAULT_LEVEL =>  0,
                               MIN_LEVEL =>      0,
                             },
                   DISP =>   { DESC =>          'View',
                               OPS =>            [qw(password_change_complete)],

                               DEFAULT_LEVEL =>  0,
                               MIN_LEVEL =>      0,
                               DEFAULTS_TO =>    'on',
                             }
                  },
          DISPLAY_ORDER => [qw(CREATE)],
};

sub get_AC {
    my $self = shift;
    my $register_level = $self->get_register_level();
    $AC->{BUNDLES}->{CREATE}->{DEFAULT_LEVEL} = $AC->{BUNDLES}->{CREATE}->{DEFAULT_LEVEL} = $register_level;
        return $AC;
}


sub get_register_level {
    my $self = shift;
    my $default = shift || 0;

    my $register_level = $default;
    if ($PARAMS->{registration_closed}) {
        $register_level = 8;
    }
    return $register_level;
}

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $self = {};
    $self  = $class->SUPER::new();

    bless ($self, $class);
    return $self;
}

sub www_register_form {
    my $self = shift;

    if (!$USER->is_admin() && !$FORM{admin_mode} ) {
        $USER->logout();
    }

    if( $USER->max_number_of_users_quota_exceeded() ) {

        $DISPLAY->print_click_back( 'User registration forbidden. Maximum number of users has been reached.' );
        exit;
    }

    if( $PARAMS->{registration_closed} && ! $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks' ) ){

        $DISPLAY->print_click_back( 'User registration forbidden. Only Administrators can register new users.' );
        exit;

    }

    #$FORM{'homepage'} = $FORM{'homepage'} ? $FORM{'homepage'} : 'http://';

    my $topform = "";

    $topform .= "<table width=\"450\">";

    $topform .= "<tr><td colspan='2'>";
    $topform .= "<h2><font color=\"#ff0000\">Required Fields</font></h2>";
    $topform .= "</td></tr>";

    $topform .= "<tr><td>";
    $topform .= "<b>Full Name:</b>";
    $topform .= "</td><td>";
    $topform .= "<INPUT type=text name=fullname size=30 value=\"$FORM{'fullname'}\" maxlength=50><br />";
    $topform .= "<font size=\"-2\"><i>(e.g.  Mike Saenz)</i></font>";
    $topform .= "</td></tr>";

    $topform .= "<tr><td>";
    $topform .= "<b>E-mail:</b><br />";
    $topform .= "</td><td>";
    $topform .= "<font size=\"-2\"><i>your email will be used as your username.</i></font><br />";
    $topform .= "<INPUT type=text name=email size=30 value=\"$FORM{'email'}\" maxlength=50><br />";
    $topform .= "<font size=\"-2\"><i>(e.g. metadotmike\@hotmail.com)</i></font>";
    $topform .= "</td></tr>";

    $topform .= "<tr><td>";
    $topform .= "<b>Password:</b>";
    $topform .="<br /><font color=\"red\">Long Passwords Required (6 characters min.)</font>" if($PARAMS->{'password_checker'} == 5);
    $topform .="<br /><font color=\"red\">Strong Passwords Required</font><br />Click <a href=\"javascript:popUp('/html/passwdchecker.html')\">Here</a> for more information." if($PARAMS->{'password_checker'} == 1);
    $topform .= "</td><td>";
    $topform .= "<INPUT type=password name=password size=20><br />";
    $topform .= "<font size=\"-2\"><i>(e.g.  sm2azp)</i></font>";
    $topform .= "</td></tr>";

    $topform .= "<tr><td>";
    $topform .= "<b>Verify Password: </b>";
    $topform .= "</td><td>";
    $topform .= "<INPUT type=password name=password2 size=20>";
    $topform .= "</td></tr>";

    $topform .= "</table>";



    my $bottomform .= "<table width=\"450\">";

    $bottomform .= "<tr><td colspan='2'>";
    $bottomform .= "<h2>Optional Fields</h2>";
    $bottomform .= "</td></tr>";

    $bottomform .= "<tr><td>";
    $bottomform .= "<b>Nickname:</b>\n";
    $bottomform .= "</td><td>";
    $bottomform .= "<INPUT type=text name=nickname size=30 value=\"$FORM{'nickname'}\" maxlength=50><br />";
    $bottomform .= "<font size=\"-2\"><i>(e.g.  Mike)</i></font>";
    $bottomform .= "</td></tr>";



    $bottomform .= "<tr><td>";
    $bottomform .= "<b>ICQ UIN:</b>";
    $bottomform .= "</td><td>";
    $bottomform .= "<INPUT type=text name=icqnumber size=15 value=\"$FORM{'icqnumber'}\">";
    $bottomform .= "</td></tr>";

    $bottomform .= "<tr><td>";
    $bottomform .= "<b>Pager address:</b>";
    $bottomform .= "</td><td>";
    $bottomform .= "<INPUT type=text name=pager size=15 value=\"$FORM{'pager'}\">";
    $bottomform .= "</td></tr>";

    $bottomform .= "<tr><td>";
    $bottomform .= "<b>Homepage:</b>";
    $bottomform .= "</td><td>";
    $bottomform .= "<INPUT type=text name=homepage size=30 value=\"$FORM{'homepage'}\" maxlength=50>";
    $bottomform .= "</td></tr>";
    $bottomform .= "<tr><td>";
    $bottomform .= "&nbsp;";
    $bottomform .= "</td><td>";
    $bottomform .= "<font size=\"-2\"><I>(e.g. http://www.myhomepage.com)</I></font>";
    $bottomform .= "</td></tr>";

    $bottomform .= "<tr><td>";
    $bottomform .= "<b>Quote:</b>";
    $bottomform .= "</td><td>";
    $bottomform .= "<textarea cols=30 rows=3 wrap name=quote>$FORM{'quote'}</textarea>";
    $bottomform .= "</td></tr>";

    $bottomform .= "</table>";

    my $auth = Metadot::AuthenticatorFactory->get_authenticator();
    ## will set form parameters depending on site's login mode
    my ($form_action, $login_mode, $submessage) = $auth->get_password_form_parameters(login_mode => $FORM{login_mode}, form_type => 'register');

    my $window_popup = $DISPLAY->popup_window();
    my  $form = $window_popup;
    $form .= "<table><tr><td><b>Register as a New User</b><br />";
    $form .= $self->_generate_regapproval_text() if($PARAMS->{'regapproval'});
    $form .= "<FORM action='$form_action'  method=post>\n";
    $form .= "<INPUT type='hidden' name='isa' value='Register'>\n";
    $form .= "<INPUT type='hidden' name='op' value='register'>\n";
    $form .= "<INPUT type='hidden' name='this_op' value='$FORM{'op'}'>";
    $form .= "<INPUT type='hidden' name='this_isa' value='$FORM{'isa'}'>";
    $form .= "<INPUT type='hidden' name='this_iid' value='$FORM{'iid'}'>";
    $form .= "<INPUT type='hidden' name='admin_mode' value='$FORM{'admin_mode'}'>";

    my $frame = new HTML::TableFrame("", "#ffffff");
    $frame->add($topform);
    $form .= $frame->print;
    $form .= "</td></tr>";

    $form .= "<tr><td><br /></td></tr>";

    $form .= "<tr><td>";

    my $frame2 = new HTML::TableFrame("", "#ffffff");
    $frame2->add($bottomform);
    $form .= $frame2->print;

    $form .= "</td><tr>";
    $form .= "<tr><td align='right'>";
    if ($login_mode eq 'standard') {
        $form .= "<br /><INPUT type=submit name=action value=\"Register\">";
    } else {
        $form .= "<br /><INPUT type=submit name=action value=\"Register Securely\">";
    }
    $form .= "</FORM>\n";
    $form .= "</td></tr></table>";
    $form .= "<p />" . $submessage;


    $DISPLAY->print_form($form, "Registration");

    #  $self->print_form($form,"Registration");
}


# generates the standard text to be displayed when registration approval
# is required
sub _generate_regapproval_text {
    return <<EOM
<font color='red'>* This system requires administrator approval for new registrations.
 You will receive notification via email when your registration has been approved.</font>
EOM
;
}


sub www_register {
    my $self = shift;

    if( $USER->max_number_of_users_quota_exceeded() ) {

        $DISPLAY->print_click_back( 'User registration forbidden. Maximum number of users has been reached.' );
        exit;
    }

    if( $PARAMS->{registration_closed} && ! $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks' ) ) {

        $DISPLAY->print_click_back( 'User registration forbidden. Only Administrators can register new users.' );
        exit;

    }

    $FORM{nickname} = $FORM{nickname} || $FORM{fullname};
    my $verify = new Form::Verify;

    $verify->verify_multiple([
                              {'name'     => 'fullname',
                               'msg_name' => 'full name',
                               'value'    => $FORM{'fullname'},
                               'type'     => 'empty'},
                              {'name'     => 'email',
                               'msg_name' => 'email',
                               'value'    => $FORM{'email'},
                               'type'     => 'email'}
                             ]);

    if ($PARAMS->{password_checker} == 1){
        $verify->verify("password", "password", $FORM{'password'}, "", $FORM{'fullname'}, $FORM{'nickname'});
    } elsif ($PARAMS->{password_checker} == 5){
        $verify->verify("password_long", "password", $FORM{'password'}, "", $FORM{'fullname'}, $FORM{'nickname'});
    } else {
        $verify->verify("others", "password", $FORM{'password'}, "");
    }

    if ($verify->errors) {
        if($PARAMS->{password_checker} > 0){
            $DISPLAY->print_click_back("Sorry", "The following field(s) were invalid:<br /> " . $verify->errors
                                .$self->_password_error_addendum($verify->errors));
            exit;
        }
        $DISPLAY->print_click_back("Sorry", "The following field(s) were invalid:<br /> " . $verify->errors);
        exit;
    }

    if ( $FORM{password} ne $FORM{password2}) {
        $DISPLAY->print_click_back("Too bad!", "Your passwords didn't match!");
        exit;
    }

    # Check if User Exists
    my $cnt = $self->email_already_registered( $FORM{email} );

    if($cnt==0) {
        my $password  =  $FORM{password};
        my $user = Metadot::UserFactory->get_user();

        $user->nickname( escapeHTML($FORM{nickname}) );
        $user->fullname( escapeHTML($FORM{fullname}) );
        $user->email($FORM{email});
        $user->pager_number( escapeHTML($FORM{pager}) );
        $user->icq_number( escapeHTML($FORM{icqnumber}) );
        $user->homepage( escapeHTML($FORM{homepage}) );
        $user->quote( escapeHTML($FORM{quote}) );

        ## changing the user's password
        if (defined($FORM{password}) && $FORM{password} ne '') {
            my $new_password = $FORM{password};

            # encrypt password if necessary
            $new_password = Portal->encrypt_password_if_necessary($new_password);

            if( $user->{password} ne $FORM{password} ){
               # we will set the password_changed_time field whenever
               # a new password is being set
               $user->password_changed_time(time);

               $user->password($FORM{password});
            }

        }

        # get registration approval setting from database
        my $att = "regapproval";

        my ($regval) = $PARAMS->{$att};

        # also check blacklist
        if ($regval == 1 || $self->is_email_blacklisted($user->email())) {
            $user->{approval_needed} = "required";
        }
        else {
            #aju 09-27-01: if approval is not required, mark the new user's status as 'accepted'.
            #This addresses cases where a mail needs to go out to all registered
            #users (as in the Surveys).
            $user->{approval_needed} = "accepted";
        }
        ##########################

        my $uid = $user->add();

        ##########################

        ($USER->create_mypage($uid))  || die "can't create mypage";


        # return admins back to user management

        if ($USER->is_admin() && $FORM{admin_mode}) {
            $DISPLAY->two_choices ("You have successfully registered a new user.",
                    "confirm",
                    "You can register another user",
                    Utility->get_qualified_url("index.pl?isa=Register&op=register_form&admin_mode=true"),
                    "You can go back to Users List",
                    Utility->get_qualified_url("index.pl?manage=Users&isa=Portal&op=modify_site_params")
                       );
        } elsif((!$user->{approval_needed}) || ($user->{approval_needed} eq "accepted")) {
            $SESSION = Session->new();
            my $session_id = $SESSION->add($uid);
            $USER = Metadot::UserFactory->get_user($uid);

            #using new persistence mechanism
            my $session_handler = Metadot::SessionHandlerFactory->get_session_handler();
            $session_handler->persist_session($SESSION);
            #end of using new persistence mechanism


            ## we refresh his session
            $user->last_online(time);

            my $name = $user->webname;
            my $hpiid = $user->has_portal;

            # send email approval notification
            $self->send_approval_message($user, "auto");

            if( $hpiid ) {
                $DISPLAY->two_choices ("Success, $name, you are now registered.",
                                       "confirm",
                                       "You can go to the front page",
                                       Utility->get_qualified_url("index.pl"),
                                       "You can visit your public website",
                                       Utility->get_qualified_url("index.pl?iid=$hpiid")
                                      );
            } else {
                $DISPLAY->print_choices ("Success, $name, you are now registered.",
                                         "confirm",
                                         "You can go to the front page",
                                         Utility->get_qualified_url("index.pl"),
                                        )
            }


        } else {
            #CKIM-ADMIN-NOTIFY
            $self->send_new_user_notification($user);
            my $name = $user->webname;
            $DISPLAY->two_choices ("Success, $name, you are now registered.<br />This system requires administrator approval for new registrations. You will receive notification via email when your registration has been approved",
                    "readme",
                    "Return to the front page",
                    Utility->get_qualified_url("index.pl")
                       );
        }

    } else {

        $DISPLAY->print_click_back(
                                "Error",
                                "A user already exists with that email address:<br /><br />"
                                . $FORM{email} . "<br /><br />"
                                . "If this is your email address then go ahead and \n"
                                . "<a href="
                                .$self->make_url(Metadot::AuthenticatorFactory->get_authenticator()->get_login_url("email=$FORM{email}"))
                                . ">"
                                . "Log in now!</a>!"
                               );
    }



    return;
}


sub www_show_params {
    my $self = shift;

    my ($approval_status, $password_checker);
    if ($PARAMS->{regapproval}) {
        $approval_status = "On";
    } else {
        $approval_status = "Off";
    }

    if ($PARAMS->{password_checker} == 0) {
        $password_checker = "Not Active";
    } elsif ($PARAMS->{password_checker} == 5) {
        $password_checker = "Low Security (Minimum Password Length is required)";
    } else {
        $password_checker = "High Security (Strong Password is required)";
    }
    #CKIM-NOTIFY-ADMIN
    my $new_user_notification_enabled;
    if ($PARAMS->{new_user_notification_enabled}) {
        $new_user_notification_enabled = "On";
    } else {
        $new_user_notification_enabled = "Off";
    }

    #CKIM-CLOSED-REG
    my $registration_closed;
    if ($PARAMS->{registration_closed}) {
        $registration_closed = "On";
    } else {
        $registration_closed = "Off";
    }


    #CKIM-REGISTER-USERS
    my $allow_group_mgr_register_users;
    if ($PARAMS->{allow_group_mgr_register_users}) {
        $allow_group_mgr_register_users = "On";
    } else {
        $allow_group_mgr_register_users = "Off";
    }

    my $allow_organization_registration;
    if ($PARAMS->{allow_organization_registration}) {
        $allow_organization_registration = "On";
    } else {
        $allow_organization_registration = "Off";
    }

    my ($admin_marker, $msg_for_super_admins) = ('', '');

    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ) {

        $admin_marker =  '<font color=red size=-1>*</font>';
        $msg_for_super_admins = "$admin_marker <i><font size=-1>indicates fields available to site managers.</font></i>";
    }

    my $blacklisted_emails = $PARAMS->{blacklisted_emails};
    $blacklisted_emails =~ s/\n/<br \/>/sg;


    my $information = "
<br /><br /><center>
<TABLE BGCOLOR=FFFF WIDTH=\"450\" CELLSPACING=\"0\" CELLPADDING=\"5\" BORDER=\"0\">
        <tr bgcolor = #bbbbbb>
                <td colspan = 2><b>Registration Parameter Administration</td>
        <TR bgcolor = #ffffff>
            <TD>$admin_marker Users Cannot Register Themselves</TD>
            <TD>$registration_closed</TD>
        </TR>
        <TR bgcolor = #efefef>
            <TD>$admin_marker Users Can Register Into an Organization</TD>
            <TD>$allow_organization_registration</TD>
        </TR>
        <TR bgcolor = #ffffff>
            <TD>$admin_marker Allow Group Managers to Register users</TD>
            <TD>$allow_group_mgr_register_users</TD>
        </TR>
        <TR bgcolor = #efefef>
            <TD>$admin_marker Registration Approval Required:</TD>
            <TD>$approval_status</TD>
        </TR>
        <TR bgcolor = #ffffff>
            <TD>$admin_marker New User Notification Enabled</TD>
            <TD>$new_user_notification_enabled</TD>
        </TR>
        <TR bgcolor = #efefef>
            <TD>$admin_marker Strong Password Checker</TD>
            <TD>$password_checker</TD>
        </TR>
        <TR bgcolor = #ffffff>
            <TD>$admin_marker Group Managers can register new users into their group</TD>
            <TD>$allow_group_mgr_register_users</TD>
        </TR>

        <TR bgcolor = #efefef>
            <TD>$admin_marker Blacklisted emails</TD>
            <TD>$blacklisted_emails</TD>
        </TR>

      </TABLE><br />$msg_for_super_admins<br /><br />
<a href = ".$self->make_url("index.pl?isa=Register&amp;op=modify_params").">Modify Registration Params</a><br /><a href=".$self->make_url("index.pl?isa=Portal&amp;op=show_admin").">Show Administration Page</a></center><br /><br />
";
     $DISPLAY->print_form($information, "Registration Parameters");

}


sub www_modify_params {
    my $self = shift;

     my ($admin_marker, $msg_for_super_admins) = ('', '');

    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ) {

        $admin_marker =  '<font color=red size=-1>*</font>';
        $msg_for_super_admins = "$admin_marker <i><font size=-1>indicates fields available to site managers.</font></i>";
    }

    #CKIM-NOTIFY-ADMIN: add field
    my $information = "

    <center>
        <form action='index.pl' method='post'>
        <input type='hidden' name='isa' value='Register'>
        <input type='hidden' name='op' value='save_params'>


        <TABLE BGCOLOR='#FFFFFF' WIDTH=\"600\" CELLSPACING=\"0\" CELLPADDING=\"5\" BORDER=\"0\" valign = top>
             <TR bgcolor = #bbbbbb>
                   <td colspan = 3 align = center><b>Registration Parameters</b></td>
             </tr>
               <tr bgcolor = #ffffff>
                    <td>$admin_marker <b>Users Cannot Register Themselves</b></td>";
                    if ($PARAMS->{registration_closed}) {
                        $information .= "<TD><input type = radio name = registration_closed value = \"1\" checked>Yes <input type = radio name = registration_closed value = \"0\">No</TD>";
                    } else {
                        $information .= "<TD><input type = radio name = registration_closed value = \"1\">Yes <input type = radio name = registration_closed value = \"0\" checked>No</TD>";
                    }
                    $information .= "
                    <td>This sets whether new users can register themselves, or site managers and the admin are the only ones who can register new users</td>
               </tr>
               <tr bgcolor = #efefef>
                    <td>$admin_marker <b>Users Can Register Into an Organization</b></td>";
                    if ($PARAMS->{allow_organization_registration}) {
                        $information .= "<TD><input type = radio name = allow_organization_registration value = \"1\" checked>Yes <input type = radio name = allow_organization_registration value = \"0\">No</TD>";
                    } else {
                        $information .= "<TD><input type = radio name = allow_organization_registration value = \"1\">Yes <input type = radio name = allow_organization_registration value = \"0\" checked>No</TD>";
                    }
                    $information .= "
                    <td>This sets whether new users can register themselves into an organization</td>
               </tr>
               <tr bgcolor = #ffffff>
                    <td>$admin_marker <b>Group Managers Can Register New Users</b></td>";
                    if ($PARAMS->{allow_group_mgr_register_users}) {
                        $information .= "<TD><input type = radio name = allow_group_mgr_register_users value = \"1\" checked>Yes <input type = radio name = allow_group_mgr_register_users value = \"0\">No</TD>";
                    } else {
                        $information .= "<TD><input type = radio name = allow_group_mgr_register_users value = \"1\">Yes <input type = radio name = allow_group_mgr_register_users value = \"0\" checked>No</TD>";
                    }
                    $information .= "
                    <td>This sets whether users that own or manage groups are permitted to register new users directly into their groups. Some notes:
                      <ul>
                        <li>This feature is in progress. Later, this option will be addressable per group, and not sitewide.
                        <li>Generally, any user can create a group. Therefore this option will allow any user to gain the
                            ability to register new users
                        <li>This feature is in addition to the option above. If both are turned on, then any user can register
                            new users into their group.
                      </ul>
                    </td>
               </tr>
              <tr bgcolor = #efefef>
                   <td>$admin_marker <b>Admin Approval</b></td>";
                   if ($PARAMS->{regapproval}) {
                       $information .= "<TD ><div style=\"white-space: nowrap\"><input type = radio name = admin value = yes checked>Yes <input type = radio name = admin value = no>No</div></TD>";
                   }  else {
                       $information .= "<TD ><div style=\"white-space: nowrap\"><input type = radio name = admin value = yes>Yes <input type = radio name = admin value = no checked>No</div></TD>";
                   }
                    $information .= "
                    <td>This sets whether or not individuals need to be approved by you, the admin, before their registration process is complete</td>
               </tr>
               <tr bgcolor = #ffffff>
                    <td>$admin_marker <b>Admin Notification of New Users</b></td>";
                    if ($PARAMS->{new_user_notification_enabled}) {
                        $information .= "<TD><input type = radio name = new_user_notification_enabled value = \"1\" checked>Yes <input type = radio name = new_user_notification_enabled value = \"0\">No</TD>";
                    } else {
                        $information .= "<TD><input type = radio name = new_user_notification_enabled value = \"1\">Yes <input type = radio name = new_user_notification_enabled value = \"0\" checked>No</TD>";
                    }
                    $information .= "
                    <td>This sets whether or admins will be notified whenever new users are awaiting approval</td>
               </tr>";
#               <tr bgcolor = #efefef>
#                    <td>$admin_marker <b>Strong Passwords</b></td>";
#                    if ($PARAMS->{password_checker}) {
#                        $information .= "<TD><input type = radio name = strong_passwords value = yes checked>Yes <input type = radio name = strong_passwords value = no>No</TD>";
#                    } else {
#                        $information .= "<TD><input type = radio name = strong_passwords value = yes>Yes <input type = radio name = strong_passwords value = no checked>No</TD>";
#                    }
#                    $information .= "
#                    <td>This sets whether or not individuals can only pick strong passwords (passwords that are hard to break)</td>
#               </tr>
               $information .= "
               <tr bgcolor = #efefef>
                    <td>$admin_marker <b>Password Checker</b></td>
                    <td colspan='2'>
                    <table border=\"0\">";
                    $information .= "<tr><td><input type = radio name = strong_passwords value=0 ".($PARAMS->{password_checker} == 0 ? "checked" : "")."></td><td>No password checker (individuals can pick any password)</td></tr>";
                    $information .= "<tr><td><input type = radio name = strong_passwords value=5 ".($PARAMS->{password_checker} == 5 ? "checked" : "")."></td><td>Simple password checker (the length of the password should be at least 6 characters)</td></tr>";
                    $information .= "<tr><td><input type = radio name = strong_passwords value=1 ".($PARAMS->{password_checker} == 1 ? "checked" : "")."></td><td>Individuals can only pick strong passwords (passwords that are hard to break)</td></tr>";
                    $information .= "
                    </table>
                    </td>
               </tr>";


                # blacklist
                $information .= "
                <tr valign='top' bgcolor = #ffffff>
                     <td width='30%'><b>Blacklisted email</b></td>
                     <td colspan='2'>
                     <table border=\"0\">";
                     $information .= qq~<tr><td><textarea name="blacklisted_emails" scroll="auto" cols="40" rows="5">$PARAMS->{blacklisted_emails}</textarea></td></tr>~;
                     $information .= qq~<tr><td>The portal will check all new registrations for a match for any text entered in this box.
                                                Any registration whose email matches this text will require administrative approval, regardless of
                                                whether Admin Approval is turned on. You can enter more than one line in the blacklist, and the portal
                                                will check against each line.
                                        </td></tr>~;
                     $information .= "
                     </table>
                     </td>
                </tr>";


               $information .= "
        </TABLE><br />$msg_for_super_admins<br /><br />
        <input type=submit value = Save><br /><br /><br />
        </form>

    </center> ";

     $DISPLAY->print_form($information, "Registration Parameters");
}


sub www_save_params {
    my $self = shift;
    my $admin_value;
    $self->debug_msg(5, "Portal::www_save_registration_params");

    if ($FORM{admin} eq "yes") {
        $admin_value = 1
    } else {
        $admin_value = 0
    }

    my $strong_passwords_value = 1;
    if ($FORM{strong_passwords} =~ /[0|5|1]{1}/) {
           $strong_passwords_value = $FORM{strong_passwords};
    }

    $PARAMS->set_regapproval($admin_value);
    $PARAMS->set_password_checker($strong_passwords_value);

    #CKIM-NOTIFY-ADMIN: add field
    $PARAMS->set_new_user_notification_enabled($FORM{'new_user_notification_enabled'});

    #CKIM-CLOSED-REG
    $PARAMS->set_registration_closed($FORM{'registration_closed'});

    #CKIM-REGISTER-USERS
    $PARAMS->set_property('allow_group_mgr_register_users', $FORM{'allow_group_mgr_register_users'});
    $PARAMS->set_property('allow_organization_registration', $FORM{'allow_organization_registration'});

    $PARAMS->set_property('blacklisted_emails', $FORM{'blacklisted_emails'});

    my $location = "index.pl?isa=Register&op=show_params";
    $self->debug_msg(5, "Portal::www_ldap_registration_params: will redirect to $location");
    Utility->md_redirect($self->make_url($location));
}


sub show_new_password_form {
    my $self = shift;
    my $uid = shift;

    my $action;
    my $op = 'save_self';

    if(!$uid){
                $DISPLAY->print_click_back("Error", "Sorry I did not get an UID to work on!");
                exit;
    }

        my $imagesloc = $PARAMS->{'imageslocation'};
    my $u = Metadot::UserFactory->get_user($uid);
    $SESSION->set_value('User_uid_to_modify', $uid);
        my $email = $u->email();
        my $site_name = $PARAMS->{rootdomain};

        $action="Save Settings";
        $op = 'save_self';

    my $form = <<EOM

<script type="text/javascript">

        function initializeFocus() {

                document.new_password.password.focus();

        }

</script>

<table border="0" cellspacing="0" cellpadding="0" align="center" width="0">
  <tr>
    <td>
      <table width="100%" border="0" cellspacing="0" cellpadding="0">
        <tr>
          <td>
<FORM name="new_password" action="index.pl" method=post>
            <table width="100%" border="0" cellspacing="5" cellpadding="10">
              <tr>
                <td colspan="2">
                  <div align="center"><font face="Arial, Helvetica, sans-serif"><font face="Arial, Helvetica, sans-serif"><b><font size="+1">Welcome
                    to Metadot Portal Server<br />
                    <br />
                    </font></b><font size="-1">An account on the site <b>$site_name</b>
                    with the login <b>$email</b> has just been created
                    for you.</font></font></font></div>
                </td>
              </tr>
              <tr>
                <td colspan="2">
                  <table width="100%" border="0" cellspacing="0" cellpadding="2">
                    <tr>
                      <td bgcolor="#2D3F6D" valign="top"><font face="Arial, Helvetica, sans-serif" size="-1"><b><font color="#FFFFFF">&nbsp;<img src="$imagesloc/orangedottiny.gif" width="12" height="12">&nbsp;&nbsp;<font size="+1">Enter
                        Your Password</font></font></b></font></td>
                    </tr>
                  </table>
                  <table width="100%" border="0" cellspacing="0" cellpadding="2">
                    <tr>
                      <td bgcolor="#CCCCCC" valign="top"><font face="Arial, Helvetica, sans-serif" size="-1"><font color="#FFFFFF">&nbsp;</font><font face="Verdana, Arial, Helvetica, sans-serif" size="-2">Please
                        enter your very own password in the text fields below
                        and then click on the button '<b>Save and go to my site</b>'
                        to start using it.</font></font></td>
                    </tr>
                  </table>
                </td>
              </tr>
              <tr>
                <td width="50%">
                  <div align="right"><b><font face="Arial, Helvetica, sans-serif" size="-1">Choose your own password:
                    </font></b></div>
                </td>
                <td><font face="Arial, Helvetica, sans-serif" size="-1"><b><font face="Arial, Helvetica, sans-serif" size="-1">
                  <input type="password" name="password">
                  </font></b></font></td>
              </tr>
              <tr>
                <td width="50%">
                  <div align="right"><b><font face="Arial, Helvetica, sans-serif" size="-1">Password
                    confirmation: </font></b></div>
                </td>
                <td><font face="Arial, Helvetica, sans-serif" size="-1"><b><font face="Arial, Helvetica, sans-serif" size="-1">
                  <input type="password" name="password2">
                  </font></b></font></td>
              </tr>
              <tr>
                <td colspan="2">
                  <div align="center">
                    <hr width="70%">
                                        <font face="Arial, Helvetica, sans-serif" size="-2">By clicking the '<b><font size="-1">Save and go

                                        to my site</font></b>' button, <br />

                                        you indicate that you agree with the

                                        <a href="#" onClick="window.open('http://www.metadot.com/terms.htm','terms','scrollbars=yes,resizable=yes,width=400,height=300');return false;">

                                        <font size="-1">Terms of Service</font></a>.</font><br /><br />

                    <font face="Arial, Helvetica, sans-serif" size="-1"><input type=image src="$imagesloc/gotosite.gif" width="126" height="19"></font></font></div>
                </td>
              </tr>
            </table>
<INPUT type="hidden" name="isa" value="User">
<INPUT type="hidden" name="op" value="$op">
<INPUT type="hidden" name="new_user" value=1>
</form>
          </td>
        </tr>
      </table>
    </td>
  </tr>
</table>

EOM
;

        return $form;

}




################################################################
# www_change_password
# shows a form where users are allowed to change their password
################################################################
sub www_change_password {
    my $self = shift;


    # NICO-PASS
    # If the uid is not a valid id, check the session to see if a
    # uid is stored under RESET_PASSWORD_KEY
    my $session_id = $FORM{key};
    my $session_obj = Session->restore($session_id);
    my $user;

    if( defined( $session_obj ) ){
        my $uid = $USER->get_reset_password_uid_from_session($session_obj);
        if (defined $uid) {
           $user = Metadot::UserFactory->get_user($uid);
        }
    }

    # Verify the session is not expired
    if (defined($session_obj) && $session_obj->is_expired()) {
       $session_obj->erase();
       $session_obj = undef;
       $DISPLAY->print_click_back(
                  "Error",
                  "Sorry, you are not authorized to access this operation."
                );
       exit (0);
    }

    # kick out the user and delete the session if this attempt is
    # unauthorized
    unless (defined $user) {
       $DISPLAY->print_click_back(
                   "Error",
                   "Sorry, you are not authorized to access this operation."
                 );
       exit (0);
    }


    my $forgot_password_mode = $user->is_forgot_password_mode($session_obj);


    my $title_message = undef;
    my $duration_message = undef;

    my $window_message = undef;

    if( $forgot_password_mode ){
       $title_message = 'Resetting your password';
       $window_message = 'Please enter a new password';

    } else {
       $title_message = $user->password_expiration_message();
       my $password_duration = $USER->password_expiration_duration();

       $duration_message = "Your password will expire every
                            $password_duration days.";

       $window_message =  "To reset your password enter your current password, and a new password
                           in the fields provided.";
    }

    my $button_message = 'Reset password';



    my $target = $SESSION->referer();


    my $rs = "<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\"><tr><td><FORM action='index.pl' method='POST'>";
    $rs .= "<input type='hidden' name='target' value='$target'>";
    $rs .= "<input type='hidden' name='isa' value='Register'>";
    $rs .= "<input type='hidden' name='op'  value='password_change_complete'>";
    $rs .= "<input type='hidden' name='form_action'  value='changed_password'>";
    $rs .= "<input type='hidden' name='key' value='$session_id'>";

    $rs .=  Session->title_bar($title_message);

    $rs .= "<h4>$duration_message</h4><br />\n";

    $rs .= "$window_message<br /><br />\n";

    $rs .= "<table border=\"0\" cellspacing=\"1\">";
    $rs .= "<tr><td bgcolor=\"#000000\">";
    $rs .= "<table border=\"0\" cellspacing=\"0\" cellpadding=\"5\">";


    unless( $forgot_password_mode ){
       $rs .= "<tr bgcolor=\"#ffffff\"><td>";
       $rs .= "<b>Current password:</b>";
       $rs .= "</td><td>";
       $rs .= "<INPUT type=password name=current_password size=20>";
       $rs .= "</td></tr>";
    }

    $rs .= "<tr bgcolor=\"#ffffff\"><td>";
    $rs .= "<b>New password:</b>";
    $rs .= "</td><td>";
    $rs .= "<INPUT type=password name=new_password size=20>";
    $rs .= "</td></tr>";

    $rs .= "<tr bgcolor=\"#ffffff\"><td>";
    $rs .= "<b>Confirm new password:</b>";
    $rs .= "</td><td>";
    $rs .= "<INPUT type=password name=confirm_new_password size=20>";
    $rs .= "</td></tr>";

    $rs .= "</table>";

    $rs .= "</td></tr>";
    $rs .= "</table>";

    $rs .= "<p /><INPUT type=submit name=action value=\"$button_message\" class='button'><br />";
    $rs .= "</FORM></td></tr></table>";

    #aju
    my $formbgcolor;
    if (Metadot::Utils::Theme->use_theme_info()) {
        # for themes, hardwire form bg to light gray -- for now.
        $formbgcolor = Metadot::Utils::Theme->get_theme_default_bg();
    }
    else {
        $formbgcolor = $PARAMS->{'formbgcolor'};
    }

    $DISPLAY->print_form($rs);

    exit(0);

}
################################################################
# www_password_change_complete
# resets a user password, and sets the 'changepassword' field
# with a timestamp of time of change.
################################################################
sub www_password_change_complete {
    my $self       = shift;

    my $entered_current_password = $FORM{'current_password'};
    my $new_password = $FORM{'new_password'};
    my $confirm_password = $FORM{'confirm_new_password'};
    my $session_id = $FORM{'key'};

    my $session_obj = Session->restore($session_id);
    my $uid         = $USER->get_reset_password_uid_from_session($session_obj);

    ## restore the user that will be processed
    my $u = Metadot::UserFactory->get_user($uid);

    my $forgot_password_mode = $u->is_forgot_password_mode($session_obj);

    my $current_password = $u->{password};

    ## make sure a current password has been entered
    my $error = undef;
    if( $entered_current_password eq '' && !$forgot_password_mode ){
        $error = 'You must enter your current password to continue.';
        $DISPLAY->print_click_back($error);
        exit 0;
    } else {
        $entered_current_password = Portal->encrypt_password_if_necessary($entered_current_password);
    }

    ## make sure current password matches the one entered in the form
    if( $current_password ne $entered_current_password && !$forgot_password_mode ){
       $error = 'Your current password does not match the one entered.';
       $DISPLAY->print_click_back($error);
       exit 0;
    }

    ## check that a new password was entered
    if( $new_password eq '' && !$forgot_password_mode ){
       $error = 'You must enter a new password to continue';
       $DISPLAY->print_click_back($error);
       exit 0;
    }

    ## check that a password confirmation was entered
    if( $confirm_password eq '' ){
       $error = 'You must confirm your new password!';
       $DISPLAY->print_click_back($error);
       exit 0;
    }

    ## check that new password matches the confirmation
    if( $new_password ne $confirm_password ){
       $error = 'Your new password does not match the confirmation.';
       $DISPLAY->print_click_back($error);
       exit 0;
    }

    ## check that the new password is not the same as the one in file
    $new_password = Portal->encrypt_password_if_necessary($new_password);
    if( $new_password eq $current_password && !$forgot_password_mode ){
       $error = 'Your new password must be different!';
       $DISPLAY->print_click_back($error);
       exit 0;
    }

    my $fullname = $u->{fullname};
    my $nickname = $u->{nickname};
    my $verify = new Form::Verify;

    if($PARAMS->{password_checker} == 1){
        $verify->verify("password", "password", $new_password, "", $fullname, $nickname );
    } elsif($PARAMS->{password_checker} == 5){
        $verify->verify("password_long", "password", $new_password, "", $fullname, $nickname);
    } else{
        $verify->verify("others", "password", $new_password, "");
    }

    if ($verify->errors) {
        $error = " Your password is invalid: " . $verify->errors . $self->_password_error_addendum($verify->errors);
        $DISPLAY->print_click_back($error);
        exit 0;
    }


    ### uff, if we got here, reset password and update the
    ### changepassword field in the users table
    my $timestamp = time();
    my $db_now = Utility->db_now();

    DBAccess->sqlUpdate("users","userid=$uid",
                        (updatedate=>$db_now, password => $new_password,
                         password_changed_time=>$timestamp
                        )
                       );

    # now become the user
    $session_obj->_change_user($uid);
    my $session_handler = Metadot::SessionHandlerFactory->get_session_handler();
    $session_handler->persist_session($session_obj);

    $self->debug_msg(5,"new[$new_password] confirm[$confirm_password]
                         entered[$entered_current_password]
                         oldp[$current_password]");

    $DISPLAY->print_choices("Password was reset successfuly!","confirm",
                            "Go back to the portal","index.pl");

}


sub show_modify_settings_form {
    my $self = shift;
    my ($uid, $showadminlevel)=@_;

    # NICO-PASS: make sure this method is not dependent on the $USER object
    ## NICO(says): not dependent

    my $action;
    my $op = "";

    if(!$uid){
        $DISPLAY->print_click_back("Error", "Sorry I did not get an UID to work on!");
        exit;
    }

    my $u = Metadot::UserFactory->get_user($uid);

    my $admin_switch = "" ;

    my $form_type = '';
    if( $showadminlevel ){

        $action="Save User Settings";
        $op = 'save';
        my $not_admin = (! $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks', $u ) )
          ? "checked" : "" ;

        my $is_admin = ( $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks', $u ) ) ? "checked" : "" ;

        $admin_switch = "
<b>Person Role:</b></td><td>
User <input type='radio' name='adminlevel' value='1' $not_admin> &nbsp;&nbsp;
Site Manager&nbsp;<input type='radio' name='adminlevel' value='9' $is_admin><br />
";
        $form_type = 'profile_other';

    } elsif( $FORM{op} eq 'reset_password' ){
        $action='Save settings';
        $op = 'reset_password';
        $form_type = 'profile_self';

    } else {
        $action="Save Settings";
        $op = 'save_self';
        $form_type = 'profile_self';
    }

    my $not_default_size = ($u->{upload_size} ne "-1")? "checked": "";
    my $default_size = ($u->{upload_size} eq "-1")? "checked": "";

    my $max_total_upload_size = $PARAMS->{'max_total_upload_size'};
    my $max_total_number_upload = $PARAMS->{'max_total_number_upload'};

    my $uploadsize = "<input type='radio' name=upload_size_bn value='-1' $default_size>Default: $max_total_upload_size<br />";
    if($u->{upload_size} ne "-1") {
        $uploadsize .= "<input type='radio' name=upload_size_bn value='1' $not_default_size>Custom: &nbsp;<INPUT type=text name=upload_size size=15 value=\"".$u->{upload_size}."\" maxlength=50>";
    }
    else {
        $uploadsize .= "<input type='radio' name=upload_size_bn value='1' $not_default_size>Custom: &nbsp;<INPUT type=text name=upload_size size=15 value=\"\" maxlength=50>";
    }

    $not_default_size = ($u->{upload_size} ne "-1")? "checked": "";
    $default_size = ($u->{upload_size} eq "-1")? "checked": "";

    my $not_default_number = ($u->{upload_number} ne "-1")? "checked": "";
    my $default_number = ($u->{upload_number} eq "-1")? "checked": "";

    my $uploadnumber = "<input type='radio' name=upload_number_bn value='-1' $default_number>Default: $max_total_number_upload<br />";
    if($u->{upload_number} ne "-1") {
        $uploadnumber .= "<input type='radio' name=upload_number_bn value='1' $not_default_number>Custom:&nbsp;<INPUT type=text name=upload_number size=15 value=\"".$u->{upload_number}."\" maxlength=50>";
    }
    else {
        $uploadnumber .= "<input type='radio' name=upload_number_bn value='1' $not_default_number>Custom:&nbsp;<INPUT type=text name=upload_number size=15 value=\"\" maxlength=50>";
    }


    my $topform = "";

    my $user_management_is_external =
      MetadotConfig->get_param_value('user_management_is_external');
    if ( $user_management_is_external ) {
        $topform .= "<table width=\"500\"><tr><td><b>User Management is external. Only the following information is managed by this site.</b></td></tr></table>\n";
    } else {
        $topform .= "<table width=\"500\">";

        $topform .= "<tr><td colspan='2'>";
        $topform .= "<h2><font color=\"#ff0000\">Required Fields</font></h2>";
        $topform .= "</td></tr>";

        $topform .= "<tr><td>";
        $topform .= "<b>Full Name:</b>";
        $topform .= "</td><td>";

        # we don't allow admin and default look names to change
        if( $USER->is_default_account( $u->email() ) ) {

        $topform .= "<INPUT type=hidden name=fullname size=30 value=\"".$u->fullname()."\" maxlength=50>";
        $topform .= "<b>" . $u->fullname . "</b><br />";

        } else {

        $topform .= "<INPUT type=text name=fullname size=30 value=\"".$u->fullname()."\" maxlength=50><br />";
        $topform .= "<font size=\"-2\"><i>(e.g.  Mike Saenz)</i></font>";

        }

        $topform .= "</td></tr>";

        $topform .= "<tr><td>";
        $topform .= "<b>E-mail:</b><br />";
        $topform .= "</td><td>";

        $topform .= "<font size=\"-2\"><i>your email will be used as your username.</i></font><br />";

        $topform .= "<INPUT type=text name=email size=30 value=\"".$u->email()."\" maxlength=50>";
        $topform .= "<br />\n";

        $topform .= "</td></tr>";

        $topform .= "<tr><td>";
        $topform .= "<b>Password:</b>";
        $topform .="<br /><font color=\"red\">Long Passwords Required (6 characters min)</font>" if($PARAMS->{'password_checker'} == 5);
        $topform .="<br /><font color=\"red\">Strong Passwords Required</font><br />Click <a href=\"javascript:popUp('/html/passwdchecker.html')\">Here</a> for more information." if($PARAMS->{'password_checker'} == 1);
        $topform .= "</td><td>";
        $topform .= "<INPUT type=password name=password size=20><br />";
        $topform .= "<font size=\"-2\"><i>(e.g.  sm2azp)</i></font>";
        $topform .= "</td></tr>";

        $topform .= "<tr><td>";
        $topform .= "<b>Verify Password: </b>";
        $topform .= "</td><td>";
        $topform .= "<INPUT type=password name=password2 size=20>";
        $topform .= "</td></tr>";


        $topform .= "</table>";
    }

    my $bottomform .= "<table width=\"500\">";

    $bottomform .= "<tr><td colspan='2'>";
    $bottomform .= "<h2>Optional Fields</h2>";
    $bottomform .= "</td></tr>";

    $bottomform .= "<tr><td>";
    $bottomform .= "<b>Nickname:</b>\n";
    $bottomform .= "</td><td>";
    $bottomform .= "<INPUT type=text name=nickname size=30 value=\"".$u->nickname()."\" maxlength=50><br />";
    $bottomform .= "<font size=\"-2\"><i>(e.g.  Mike)</i></font>";
    $bottomform .= "</td></tr>";

    $bottomform .= "<tr><td>";
    $bottomform .= "<b>ICQ UIN:</b>";
    $bottomform .= "</td><td>";
    $bottomform .= "<INPUT type=text name=ICQNumber size=15 value=\"".$u->icq_number()."\">";
    $bottomform .= "</td></tr>";

    $bottomform .= "<tr><td>";
    $bottomform .= "<b>Pager address:</b>";
    $bottomform .= "</td><td>";
    $bottomform .= "<INPUT type=text name=pager size=15 value=\"".$u->pager_number()."\">";
    $bottomform .= "</td></tr>";

    $bottomform .= "<tr><td>";
    $bottomform .= "<b>Homepage:</b>";
    $bottomform .= "</td><td>";
    $bottomform .= "<INPUT type=text name=homepage size=30 value=\"".$u->homepage()."\" maxlength=50>";
    $bottomform .= "</td></tr>";
    $bottomform .= "<tr><td>";
    $bottomform .= "&nbsp;";
    $bottomform .= "</td><td>";
    $bottomform .= "<font size=\"-2\"><I>(e.g. http://www.myhomepage.com)</I></font>";
    $bottomform .= "</td></tr>";

    $bottomform .= "<tr><td>";
    $bottomform .= "<b>Quote:</b>";
    $bottomform .= "</td><td>";
    $bottomform .= "<textarea cols=30 rows=3 wrap name=quote>".$u->quote()."</textarea>";
    $bottomform .= "</td></tr>";

    $bottomform .= "</table>";

    my $adminform = "";

    $adminform .= "<table width=\"500\">";
    $adminform .= "<tr><td colspan='2'>";
    $adminform .= "<h2>Admin params</h2>";
    $adminform .= "</td></tr>";

    $adminform .= "<tr valign='baseline'><td>";
    $adminform .= "<b>Max total upload size</b> (MB)<b>:</b><br />\n";
    $adminform .= "</td><td>";
    $adminform .= "$uploadsize";
    $adminform .= "</td></tr>";

    $adminform .= "<tr><td>&nbsp;</td></tr>\n";

    $adminform .= "<tr valign='baseline'><td>";
    $adminform .= "<b>Max total number of upload:</b><br />";
    $adminform .= "</td><td>";
    $adminform .= "$uploadnumber";
    $adminform .= "</td></tr>";

    if( $FORM{uid} && ( $FORM{uid} != MetadotConfig->get_reserved_uid('ADMIN') ) ) {
        $adminform .= "<tr><td>";
        $adminform .= "$admin_switch";
        $adminform .= "</td></tr>";
    }

    $adminform .= "</table>";
    my $auth = Metadot::AuthenticatorFactory->get_authenticator();
    ## will set form parameters depending on site's login mode
    my ($form_action, $login_mode, $submessage) = $auth->get_password_form_parameters(login_mode => $FORM{login_mode}, form_type => $form_type);

    my $window_popup = $DISPLAY->popup_window();
    my $form = $window_popup."<table><tr><td><b>Change Profile</b><br />";
    $form .= "<FORM action='$form_action' method=post>\n";
    $form .= "<INPUT type='hidden' name='isa' value='User'>\n";
    $form .= "<INPUT type='hidden' name='op' value='$op'>\n";

    $SESSION->set_value('User_uid_to_modify', $uid);

    my $frame = new HTML::TableFrame("", "#ffffff");
    $frame->add($topform);
    $form .= $frame->print;
    $form .= "</td></tr>";

    $form .= "<tr><td><br /></td></tr>";

    $form .= "<tr><td>";

    my $frame2 = new HTML::TableFrame("", "#ffffff");
    $frame2->add($bottomform);
    $form .= $frame2->print;

    $form .= "</td></tr>";

    if( $showadminlevel || $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks', $u ) ){
        $form .= "<tr><td><br /></td></tr>";

        $form .= "<tr><td>";

        my $frame3 = new HTML::TableFrame("", "#ffffff");
        $frame3->add($adminform);
        $form .= $frame3->print;

        $form .= "</td></tr>";
    }

    $form .= "<tr><td align='right'>";

    unless ($login_mode eq 'standard') {
        $action .= " Securely";
    }

    $form .= "<p /><INPUT type=submit name='action' value='$action'>\n";
    $form .= "</FORM>\n";
    $form .= "</td></tr></table>";
    $form .= "<p />$submessage";
    return $form;
}

sub _password_error_addendum {

        my $self = shift;

        my $error_string = shift;

        my $result = '';



        if ($error_string =~ m/pass/i) {

                $result .= "<br /><br />Please <a href=\"javascript:popUp('/html/passwdchecker.html')\">click here</a> for additional information about valid passwords.\n";

        }

        return $result;

}



sub save_new_password_form {
    my $self = shift;

    my $basename = $PARAMS->{basename};
    $DISPLAY->print_http_header;

    my $errormsg= '';
    my $verify = new Form::Verify;

    if ( ($FORM{password} ne $FORM{password2}) ) {
                $errormsg .= "Your passwords didn't match<p />";
    }

    if ($FORM{password}) {
        if($PARAMS->{password_checker} == 1){
            $verify->verify("password", "password", $FORM{'password'}, "", $FORM{'fullname'}, $FORM{'nickname'});
        } elsif($PARAMS->{password_checker} == 5){
            $verify->verify("password_long", "password", $FORM{'password'}, "", $FORM{'fullname'}, $FORM{'nickname'});
        } else{
            $verify->verify("others", "password", $FORM{'password'}, "");
        }
    } else {
                $errormsg .= "You did not type in your password.<p />";
    }

    if ($verify->errors) {
            $errormsg .= "Your password is invalid: " . $verify->errors . $self->_password_error_addendum($verify->errors)."<br /><p />";
    }

    if( $errormsg ){
                #####      ERROR     #####
                $DISPLAY->print_click_back("Sorry there is a slight problem", "$errormsg<br />");
                return 0;

    } else {
            my $u = Metadot::UserFactory->get_user( $SESSION->get_value('User_uid_to_modify') );
                $u->save_password($FORM{password});

                my $name = $u->webname();

                my ($title);

                $title="Success $name, your settings are saved. <br /><br />You can go to the front page<br /><br />\n";
                $DISPLAY->two_choices ($title, "confirm", "Home", "index.pl");
        }
}

###############################################################
### save user's settings : 'backend' function for
### User::www_save and User::www_save_self
### the uid being modifyied is in FORM{uid}
### the user modifying is in $USER->{uid}
###
### if the keep_same_adminlevel==1 then the user is modifying
### himself and we don't want to loose his current privilege
### so we will read the db first.
###############################################################
sub save_modify_settings_form {
    my $self = shift;

    my ($keep_same_adminlevel) = @_;
    my $basename = $PARAMS->{basename};

    $DISPLAY->print_http_header;

    $DISPLAY->debug_msg(4, "save_modify_settings_form begins");

    my $errormsg= '';
    my $verify = new Form::Verify;
    my $u = Metadot::UserFactory->get_user( $SESSION->get_value('User_uid_to_modify') );

    unless (MetadotConfig->get_param_value('user_management_is_external')) {
        if ( ($FORM{password} ne $FORM{password2}) ) {

        $errormsg .= "<h3> Your passwords didn't match</h3><p />";
        }

        if ($FORM{password}) {
            if($PARAMS->{password_checker} == 1){
                $verify->verify("password", "password", $FORM{'password'}, "", $FORM{'fullname'}, $FORM{'nickname'});
            } elsif($PARAMS->{password_checker} == 5){
                $verify->verify("password_long", "password", $FORM{'password'}, "", $FORM{'fullname'}, $FORM{'nickname'});
            } else{
                $verify->verify("others", "password", $FORM{'password'}, "");
            }
        }
        if ($verify->errors) {
            $errormsg .= "<h3> Your password is invalid: " . $verify->errors . $self->_password_error_addendum($verify->errors) ."</h3><br /><p />";
        }

        if(!($FORM{fullname})) {
        $errormsg .= "<h3>You need to enter your fullname.</h3><br /><p />";
        }

        # if the nickname is null then we use the name
        $FORM{nickname} = $FORM{nickname}|| $FORM{fullname};

        if (!$FORM{email} || $FORM{email} =~ m/\<|\>|\&/) {
        $errormsg .= "<h3>You need to provide a real email address</h3><br /><p />";
        }

        # check if the email address exist for another user with a different uid
        unless ( $u->can_use_email_address( $FORM{email} ) ){
        $errormsg .=  "<h3>The email address you entered is already used by another user. Please verify your email address. If this 'another user' is you then try to login or ask your system admin to help you.</h3><br /><p />\n";
        }
    }


    if( $errormsg ){
        #####      ERROR     #####
        $DISPLAY->print_click_back("Sorry there is a slight problem", "$errormsg<br />");
        return 0;

    } else {

        ##### NO ERROR we do the update #####
        ### set privilege to NORMAL USER BY DEFAULT
        my $admin_level = 1;
        if( $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks' ) ) {

            if(defined $FORM{adminlevel} && $FORM{adminlevel}) {

                $admin_level = $FORM{adminlevel};

            }
        }

    my $old_email = $u->{email};

        unless (MetadotConfig->get_param_value('user_management_is_external')) {
        $u->fullname( escapeHTML($FORM{fullname}) );
        $u->nickname( escapeHTML($FORM{nickname}) );
        $u->email($FORM{email});
    }
        $u->homepage( escapeHTML($FORM{homepage}) );
        $u->email($FORM{email});
        $u->pager_number( escapeHTML($FORM{pager}) );
        $u->icq_number( escapeHTML($FORM{ICQNumber}) );
        $u->quote( escapeHTML($FORM{quote}) );

        if($FORM{upload_size_bn} && ($FORM{upload_size_bn} >= 0) && $FORM{upload_size}) {

            $u->upload_size( escapeHTML($FORM{upload_size}) );
        }else{

            $u->upload_size( "-1");
        }

        if($FORM{upload_number_bn} && ($FORM{upload_number_bn} >= 0) && $FORM{upload_number}){

            $u->upload_number( escapeHTML($FORM{upload_number}) );

        }else{

            $u->upload_number( "-1");
        }


        unless ($keep_same_adminlevel){

            $u->admin_level( $admin_level );
        }

        ### UPDATE with password (if there is one)
        if ($FORM{password}){
            $u->save_password($FORM{password});
        }

        $u->save();

    my $to = $u->{email};
    my $cc = "";
    my $bcc = "";
    my $subject = "Your Profile Has Been Changed";
    my $message = "Your profile has been changed.  If you wish to view your new profile, please click here:  $basename/index.pl?isa=User&amp;op=modify_self";
    my $sm = Sendmail->new($to, $cc, $bcc, $subject, $message);
    $sm->sendMail();

    if ($old_email ne $to) {
        $sm = Sendmail->new($old_email, $cc, $bcc, $subject, $message);
        $sm->sendMail();
    }




        my $grp_id = DBAccess->sqlSelect("grp_id", "grp", "group_name='ADMIN'");

        # XXX move all this code to Groups.pm
        if($admin_level == 9){

            Group->_add_a_member_to_a_group($grp_id, $SESSION->get_value('User_uid_to_modify') );

        } elsif ( ! $keep_same_adminlevel) { #if not keeping admin level

            my $uid = $SESSION->get_value('User_uid_to_modify');

            if(DBAccess->sqlSelect("userid", "grpmembers", "userid=$uid AND grp_id=$grp_id")){
                DBAccess->sqlDelete("grpmembers", "userid=$uid AND grp_id=$grp_id");

       $CACHE->set_as_dirty('GROUPS_HASH');
            }
        }

        #################

        my $name = $u->webname();

        my ($title);

        if( $SESSION->get_value('User_uid_to_modify') != $USER->{uid}){
            # an admin is updating a user

            $title="Success: settings are saved.<br /><br />\n";
            $DISPLAY->two_choices ($title, "confirm", "Home", Utility->get_qualified_url("index.pl"), "Back to Edit Users", Utility->get_qualified_url("index.pl?manage=Users&isa=Portal&app=&iid=0&op=modify_site_params"));

        } else {

            # self modif

            $title="Success $name, your settings are saved. <br /><br />You can go to the front page<br /><br />\n";
            $DISPLAY->two_choices ($title, "confirm", "Home", Utility->get_qualified_url("index.pl"));
        }

    }
}


####################################################################
#       Send Accept/Reject noticed via email
####################################################################
sub send_approval_message {
    my $self = shift;
    my $user = shift;
    my ($mail_type) = shift;

    my $to = $user->email;
    my $cc = "";
    my $bcc = "";
    my $fullname = $user->fullname || $user->webname;
    my $nickname = $user->nickname || $user->webname;

    my $password_string = $FORM{password} ;
    my $username_string = $user->email ;

    my $subject = "Registration Confirmation";
    my $mesg;

    if ($mail_type eq "auto"){

        $mesg = $PARAMS->{defregmessage};
        $mesg = $user->substitute($mesg, $fullname, $username_string, $password_string, $nickname);

    } elsif ($mail_type eq "accepted"){

        $mesg = $PARAMS->{aprregmessage};
        $mesg = $user->substitute($mesg, $fullname, $username_string, $password_string, $nickname);

    } else {

        $mesg = $PARAMS->{rejregmessage};
        $mesg = $user->substitute($mesg, $fullname, $username_string, $password_string, $nickname);

    }

    my $sm = Sendmail->new($to, $cc, $bcc, $subject, $mesg);
    $sm->sendMail();
}




1;
