###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Register::LDAP;
use strict;

use Metadot::SessionHandlerFactory;
use Metadot::Register;
use Form::Verify;
use CGI qw(escapeHTML);
use Utility;

use Metadot::LDAPSubsystemFactory;

use Metadot::SessionHandlerFactory;

use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CACHE $ACCESS_BROKER );

use vars qw(@ISA);
@ISA=qw(Metadot::Register);

my $AC = {
       BUNDLES => {

                   CREATE => { DESC =>          'Create Account',
                               OPS =>            [qw(register_form register show_params edit_params modify_params save_params)],
                               DEFAULT_LEVEL =>  0,
                               MIN_LEVEL =>      0,
                             }
                  },
          DISPLAY_ORDER => [qw(CREATE)],
};

sub get_AC {
    my $self = shift;
    my $register_level = $self->get_register_level();
    $AC->{BUNDLES}->{CREATE}->{DEFAULT_LEVEL} = $AC->{BUNDLES}->{CREATE}->{DEFAULT_LEVEL} = $register_level;
    return $AC;
}


sub get_register_level {
    my $self = shift;
    my $default = shift || 0;

    my $register_level = $default;
    if ($PARAMS->{registration_closed}) {
        $register_level = 8;
    }
    return $register_level;
}


sub new {
    my $proto = shift;
    my $class = __PACKAGE__;
    my $self = {};
    $self  = $class->SUPER::new();

    bless ($self, $class);
    return $self;
}

sub www_register_form {
    my $self = shift;

    if( $USER->max_number_of_users_quota_exceeded() ) {

    $DISPLAY->print_click_back( 'User registration forbidden. Maximum number of users has been reached.' );
    exit;
    }

    $USER->logout();

    # in case of weird LDAP uid fields we'll print a label given by admin
    my $ldap_uid_name = $PARAMS->{'ldap_uid_label'} || $PARAMS->{'ldap_uid'};
    my $prompt = "Your $ldap_uid_name";
    my $topform = "";

    $topform .= "<table width=\"450\">";

    $topform .= "<tr><td colspan='2'>";
    $topform .= "<h2><font color=\"#ff0000\">Required Fields</font></h2>";
    $topform .= "</td></tr>";

    $topform .= "<tr><td>";
    $topform .= "<b>Full Name:</b>";
    $topform .= "</td><td>";
    $topform .= "<INPUT type=text name=fullname size=30 value=\"".($FORM{'fullname'} || '')."\" maxlength=50><br />";
    $topform .= "<font size=\"-2\"><i>(e.g.  Mike Saenz)</i></font>";
    $topform .= "</td></tr>";

    $topform .= "<tr><td>";
    $topform .= "<b>E-mail:</b><br />";
    $topform .= "</td><td>";
    $topform .= "<INPUT type=text name=email size=30 value=\"".($FORM{'email'} || '')."\" maxlength=50><br />";
    $topform .= "<font size=\"-2\"><i>(e.g. WShakespeare\@yahoo.com)</i></font>";
    $topform .= "</td></tr>";

    $topform .= "<tr><td>";
    $topform .= "<p /><b>$prompt:</b><br />\n";
    $topform .= "</td><td>";
    $topform .= "<font size=\"-2\"><i>This information is necessary to identify you in the LDAP directory</i></font><br />";
    $topform .= "<INPUT type=text name=ldap_uid size=50 value=\"".($FORM{'ldap_uid'} || '')."\"><br />";
    $topform .= "<font size=\"-2\"><i>your $ldap_uid_name will be used as your username. </i></font><br />";
    $topform .= "</td></tr>";

    $topform .= "<tr><td>";
    $topform .= "<b>Your LDAP Password: </b>";
    $topform .= "</td><td>";
    $topform .= "<INPUT type=password name=password size=20>";
    $topform .= "</td></tr>";

    $topform .= "</table>";



    my $bottomform .= "<table width=\"450\">";

    $bottomform .= "<tr><td colspan='2'>";
    $bottomform .= "<h2>Optional Fields</h2>";
    $bottomform .= "</td></tr>";

    $bottomform .= "<tr><td>";
    $bottomform .= "<b>Nickname / Screenname:</b>\n";
    $bottomform .= "</td><td>";
    $bottomform .= "<INPUT type=text name=nickname size=30 value=\"".($FORM{'nickname'} || '')."\" maxlength=50><br />";
    $bottomform .= "<font size=\"-2\"><i>(e.g.  Mike)</i></font>";
    $bottomform .= "</td></tr>";




    $bottomform .= "<tr><td>";
    $bottomform .= "<b>ICQ UIN:</b>";
    $bottomform .= "</td><td>";
    $bottomform .= "<INPUT type=text name=icqnumber size=15 value=\"".($FORM{'icqnumber'} || '')."\">";
    $bottomform .= "</td></tr>";

    $bottomform .= "<tr><td>";
    $bottomform .= "<b>Pager address:</b>";
    $bottomform .= "</td><td>";
    $bottomform .= "<INPUT type=text name=pager size=15 value=\"".($FORM{'pager'} || '')."\">";
    $bottomform .= "</td></tr>";

    $bottomform .= "<tr><td>";
    $bottomform .= "<b>Homepage:</b>";
    $bottomform .= "</td><td>";
    $bottomform .= "<INPUT type=text name=homepage size=30 value=\"".($FORM{'homepage'} || '')."\" maxlength=50>";
    $bottomform .= "</td></tr>";
    $bottomform .= "<tr><td>";
    $bottomform .= "&nbsp;";
    $bottomform .= "</td><td>";
    $bottomform .= "<font size=\"-2\"><I>(e.g. http://www.myhomepage.com)</I></font>";
    $bottomform .= "</td></tr>";

    $bottomform .= "<tr><td>";
    $bottomform .= "<b>Quote:</b>";
    $bottomform .= "</td><td>";
    $bottomform .= "<textarea cols=30 rows=3 wrap name=quote>".($FORM{'quote'} || '')."</textarea>";
    $bottomform .= "</td></tr>";

    $bottomform .= "</table>";

    my $auth = Metadot::AuthenticatorFactory->get_authenticator();
    ## will set form parameters depending on site's login mode
    my ($form_action, $login_mode, $submessage) = $auth->get_password_form_parameters(login_mode => $FORM{login_mode}, form_type => 'register');

    my  $form = "<table><tr><td>";
    $form .= "<FORM action='$form_action'  method=post>\n";
    $form .= "<INPUT type='hidden' name='isa' value='Register'>\n";
    $form .= "<INPUT type='hidden' name='op' value='register'>\n";
    $form .= "<INPUT type='hidden' name='this_op' value='$FORM{'op'}'>";
    $form .= "<INPUT type='hidden' name='this_isa' value='$FORM{'isa'}'>";
    $form .= "<INPUT type='hidden' name='this_iid' value='".($FORM{'iid'} || '')."'>";

    my $frame = new HTML::TableFrame("", "#ffffff");
    $frame->add($topform);
    $form .= $frame->print;
    $form .= "</td></tr>";

    $form .= "<tr><td><br /></td></tr>";

    $form .= "<tr><td>";

    my $frame2 = new HTML::TableFrame("", "#ffffff");
    $frame2->add($bottomform);
    $form .= $frame2->print;

    $form .= "</td><tr>";
    $form .= "<tr><td align='right'>";
    if ($login_mode eq 'standard') {
        $form .= "<br /><INPUT type=submit name=action value=\"Register\">";
    } else {
        $form .= "<br /><INPUT type=submit name=action value=\"Register Securely\">";
    }
    $form .= "</FORM>\n";
    $form .= "</td></tr></table>";
    $form .= "<p />" . $submessage;


    $DISPLAY->print_form($form, "Registration");

    #  $self->print_form($form,"Registration");
}



sub www_register {
    my $self = shift;
    my %args = @_;

    #use Data::Dumper;
    #print STDERR "\n\nargs:" . Dumper(\%args) . "\n\n";
    my $ldap_uid = $args{ldap_uid} || $FORM{ldap_uid};
    my $fullname = $args{fullname} || $FORM{fullname};
    my $nickname = $args{nickname} || $FORM{nickname} || $fullname;
    my $email    = $args{email}    || $FORM{email};
    my $password = $args{password} || $FORM{password};
    my $pager    = $args{pager}    || escapeHTML($FORM{pager});
    my $icqnumber = $args{icqnumber} || escapeHTML($FORM{icqnumber});
    my $homepage = $args{homepage} || escapeHTML($FORM{homepage});
    my $quote    = $args{quote} || escapeHTML($FORM{quote});

    $self->debug_msg(3, "Metadot::Register::LDAP::www_register with ldap_uid:$ldap_uid");
    if( $USER->max_number_of_users_quota_exceeded() ) {

    $DISPLAY->print_click_back( 'User registration forbidden. Maximum number of users has been reached.' );
    exit;
    }

    my $ldap_uid_name = $PARAMS->{'ldap_uid_label'} || $PARAMS->{'ldap_uid'};

    if (!$ldap_uid) {
    $DISPLAY->print_click_back("Sorry", "You need to enter $ldap_uid_name.");
    exit;
    }

    if(!($fullname)) {
    $DISPLAY->print_click_back("Sorry", "You need to enter or your fullname");
    exit;
    }

    if (!$email || $email =~ m/\<|\>|\&/) {
    $DISPLAY->print_click_back("Sorry", "You need to enter a real email address");
    exit;
    }

    if (!$password) {
    $DISPLAY->print_click_back("Sorry", "You need to provide a password");
    exit;
    }

    ###############################################
    # Check if User Exists
    my $cnt = $self->email_already_registered( $email );

    ###############################################
    # Check if User Exists with such ldap_alias
    my $cnt1 = $USER->ldap_uid_already_registered( $ldap_uid );

    if($cnt == 0 && $cnt1 == 0) {
    # get registration approval setting from database
    my $user = Metadot::UserFactory->get_user();

    # added blacklist checking
    if ($PARAMS->{regapproval} == 1 || $self->is_email_blacklisted($user->email())){
        $user->{approval_needed} = "required";
    }

    my $ldap = Metadot::LDAPSubsystemFactory->get_ldap_subsystem();

    unless( $ldap->initialize() ) {



        $DISPLAY->print_click_back( "LDAP authentication is not properly configured or the LDAP server is down. Please contact your site administrator. " );

    }

    my $password_check_status = $ldap->check_ldap_password( $ldap_uid, $password );

    unless(defined $password_check_status) {


        $DISPLAY->print_click_back( "LDAP authentication is not properly configured or the LDAP server is down. Please contact your site administrator. " );

    }

    if (!$password_check_status) {
        $DISPLAY->print_click_back("Sorry", "We could not identify you. Make sure your $ldap_uid_name  and LDAP password are correct.");
        exit(0);        # it's important to exit, otherwise
        # the guy will login without knowing the password
    }


    $user->nickname( $nickname );
    $user->fullname( $fullname );
    $user->email($email);
    $user->ldap_dn($ldap_uid);
    $user->pager_number( $pager );
    $user->icq_number( $icqnumber );
    $user->homepage( $homepage );
    $user->quote( $quote );

    # we do a query to LDAP server
    # to find dn of the user and put it into the "$user" object
    my $uid  = $user->add();
    my ($hpiid) = $user->has_portal();

        ($USER->create_mypage($uid)) || die "can't create mypage";

    my $name = $user->webname;

    if(!$user->{approval_needed}){

        $SESSION = Session->new();
        my $session_id = $SESSION->add($uid);
        $USER = Metadot::UserFactory->get_user($uid);

        my $s_h = Metadot::SessionHandlerFactory->get_session_handler();
        $s_h->persist_session( $SESSION );

        # send email approval notification
        $self->send_approval_message($user, "auto");

        $DISPLAY->two_choices ("Success, $name, you are now registered.",
                "confirm",
                "You can go to the front page",
                Utility->get_qualified_url("index.pl"),
                "You can visit your public website",
                Utility->get_qualified_url("index.pl?iid=$hpiid")
                   );
    } else {
        #CKIM-ADMIN-NOTIFY
        $self->send_new_user_notification($user);

        my $name = $user->webname;
        $DISPLAY->two_choices ("Success, $name, you are now registered.<br />This system requires administrator approval for new registrations. You will receive notification via email when your registration has been approved",
                "readme",
                "Return to the front page",
                Utility->get_qualified_url("index.pl")
                   );
    }

    } else {
    if ($cnt != 0) {
        $DISPLAY->print_click_back(
                    "Error",
                    "A user already exists with that email address:<br /><br />"
                    . $email . "<br /><br />"
                    . "If this is your email address then go ahead and \n"
                    . "<a href="
                    .$self->make_url("index.pl?isa=Session&amp;op=login&amp;email=$email")
                    . ">"
                    . "Log in now!</a>!"
                   );
        } else {
               $DISPLAY->print_click_back(
                "Error",
                "A user already exists with that $ldap_uid_name:<br /><br />"
                . $ldap_uid . "<br /><br />"
                . "If this is your $ldap_uid_name then go ahead and \n"
                . "<a href="
                .$self->make_url("index.pl?isa=Session&amp;op=login&amp;email=$email")
                . ">"
                . "Log in now!</a>!"
                   ) ;
        }
    }

    return;
}


sub www_show_params {
    my $self = shift;
    my %args = @_;

    my $additional_fields = $args{additional_fields} || '';

    my $ldap_server =      $PARAMS->{ldap_server};
    my $ldap_base_dn =     $PARAMS->{ldap_base_dn};
    my $ldap_uid =         $PARAMS->{ldap_uid};
    my $ldap_uid_label =   $PARAMS->{ldap_uid_label};

    #CKIM-NOTIFY-ADMIN
    my $new_user_notification_enabled;
    if ($PARAMS->{new_user_notification_enabled}) {
        $new_user_notification_enabled = "On";
    } else {
        $new_user_notification_enabled = "Off";
    }

    my $approval_status;
    if ($PARAMS->{regapproval}) {
        $approval_status = "On";
    } else {
        $approval_status = "Off";
    }

    my $ldap_fields_definitions = $PARAMS->{ldap_fields_definitions};
    $ldap_fields_definitions =~ s/\n/<br \/>/g;

    my $ldap_use = '';
    if ($PARAMS->{managed_ldap}) {
        $ldap_use = 'authentication, profile management';
    } else {
        $ldap_use = 'authentication only';
    }

    my $ldap_bind_option;
    if ($PARAMS->{ldap_bind_dn}) {
        $ldap_bind_option = "$PARAMS->{ldap_bind_dn}; password=$PARAMS->{ldap_bind_password}";
    } else {
        $ldap_bind_option = 'Logged-in user (anonymous when no logged in user)';
    }

    my $information = "
<br /><br /><center>
<TABLE BGCOLOR=FFFF WIDTH=\"450\" CELLSPACING=\"0\" CELLPADDING=\"5\" BORDER=\"0\">
        <tr bgcolor = #bbbbbb>
                <td colspan = 2><b>Registration Parameter Administration</td>
        <TR bgcolor = #efefef>
            <TD>Registration Approval Required:</TD>
            <TD>$approval_status</TD>
        </TR>
        <TR bgcolor = #ffffff>
            <TD>LDAP Server:</TD>
            <TD>$ldap_server</TD>
        </TR>
        <TR bgcolor = #efefef>
            <TD>Base DN for user searches:</TD>
            <TD>$ldap_base_dn</TD>
        </TR>
     <TR bgcolor = #ffffff>
            <TD>Unique User Identifier in the LDAP Directory:</TD>
            <TD>$ldap_uid</TD>
        </TR>
        <TR bgcolor = #efefef>
            <TD>Unique User Identifier Label</TD>
            <TD>$ldap_uid_label</TD>
        </TR>
        <TR bgcolor = #ffffff>
            <TD>New User Notification Enabled</TD>
            <TD>$new_user_notification_enabled</TD>
        </TR>
        <TR bgcolor = #efefef>
            <TD>Use LDAP for</TD>
            <TD>$ldap_use</TD>
        </TR>
        <TR bgcolor = #ffffff>
            <TD valign='top'>LDAP Fields Definition</TD>
            <TD>$ldap_fields_definitions</TD>
        </TR>

        <TR bgcolor = #efefef>
            <TD>Bind to LDAP Server as:</TD>
            <TD>$ldap_bind_option</TD>
        </TR>

        $additional_fields

      </TABLE><br /><br />
<a href = ".$self->make_url("index.pl?isa=Register&amp;op=modify_params").">Modify Registration Params</a><br /><a href=".$self->make_url("index.pl?isa=Portal&amp;op=show_admin").">Show Administration Page</a></center><br /><br />
";
     $DISPLAY->print_form($information, "Registration Parameters");

}

sub www_modify_params {
    my $self = shift;
    my %args = @_;

    my $additional_fields = $args{additional_fields} || '';

    my $ldap_server =      $PARAMS->{ldap_server};
    my $ldap_base_dn =     $PARAMS->{ldap_base_dn};
    my $ldap_uid =         $PARAMS->{ldap_uid};
    my $ldap_uid_label =   $PARAMS->{ldap_uid_label};

    #CKIM-NOTIFY-ADMIN: add field

    my $information = "
  <form action='index.pl' method='post'>
<input type='hidden' name='isa' value='Register'>
<input type='hidden' name='op' value='save_params'>
<TABLE WIDTH=\"450\" CELLSPACING=\"0\" CELLPADDING=\"0\" BORDER=\"0\">
              <tr bgcolor = #efefef>
                   <td><b>Admin Approval</b></td></tr>";

    $information .= "<TR>";

    if ($PARAMS->{regapproval}) {
    $information .= "<TD ><div style=\"white-space: nowrap\"><input type = radio name = admin value = yes checked>Yes <input type = radio name = admin value = no>No</div></TD>";
    } else {
    $information .= "<TD ><div style=\"white-space: nowrap\"><input type = radio name = admin value = yes>Yes <input type = radio name = admin value = no checked>No</div></TD>";
    }

    $information .= "</tr><tr><td><I>This sets whether or not individuals need to be approved by you, the admin, before their registration process is complete</I><br /></td></tr>";

    $information .= "\n<tr><td><br /><b>Admin Notification of New Users</b></td></tr><tr><td>";

    if ($PARAMS->{new_user_notification_enabled}) {
        $information .= "<input type = radio name = new_user_notification_enabled value = \"1\" checked>Yes <input type = radio name = new_user_notification_enabled value = \"0\">No";
    } else {
        $information .= "<input type = radio name = new_user_notification_enabled value = \"1\">Yes <input type = radio name = new_user_notification_enabled value = \"0\" checked>No";
    }
    $information .= "</td></tr>\n";

    $information .= "
    <td><i>This sets whether or admins will be notified whenever new users are awaiting approval.</i></td>
</tr>";

    $information .= "<TD><br /><br /><b>LDAP server:</b></TD></TR>
<TR><TD><input type='text' name='ldap_server' size='30' maxlength='900'  value='$ldap_server'></TD></TR>
<TR><TD><I>e.g. ldap.mycompany.com</I><br /><br /></TD></TR>
<TR><TD><b>BASE DN:</b></TD></TR>
<TR><TD><input type='text' name='ldap_base_dn' size='30' maxlength='900'  value='$ldap_base_dn'></TD></TR>
<TR><TD><I>This is BASE DN for the LDAP server (e.g. o=mycompany, c=RU)</I><br /><br /></TD></TR>
<TR><TD><b>User unique identifier in LDAP directory:</b></TD></TR>
<TR><TD><input type='text' name='ldap_uid' size='30' maxlength='900'  value='$ldap_uid'></TD></TR>
<TR><TD><I>This is a unique user identifier in your LDAP directory, e.g. email, alias etc.</I></TD></TR>
<TR><TD><br /><b>A label for the unique user identifier in your LDAP directory:</b></TD></TR>
<TR><TD><input type='text' name='ldap_uid_label' size='30' maxlength='900'  value='$ldap_uid_label'></TD></TR>
<TR><TD><I> Users will see this label in the Register Form and in error messages. If you don't want to put anything here you can just leave it blank. Example: if the unique identifier is called just &quot;mail&quot; and you would prefer to call it &quot;email for LDAP directory&quot; then  you should type <br /><b>email for LDAP directory<br /></b> in this box.
</I></TD></TR>\n";


    $information .= "\n<tr><td><br /><b>Use LDAP for</b></td></tr><tr><td>";

    my $auth_check = $PARAMS->{managed_ldap} ? '' : 'checked';
    my $auth_profile_check = $PARAMS->{managed_ldap} ? 'checked' : '';
    $information .= "<input type = radio name = managed_ldap value = \"0\" $auth_check> authentication only <input type = radio name = managed_ldap value = \"1\" $auth_profile_check>authentication, profile management";
    $information .= "</td></tr>\n";

    my $ldap_fields_definitions = $PARAMS->{ldap_fields_definitions};

    my $fields = <<EOM
<tr><td><br /><b>LDAP Fields Definition</b> <i>(only if profile management turned on)</td></tr>
<tr><td><textarea cols=50 rows=8 wrap name=ldap_fields_definitions>$ldap_fields_definitions</textarea></td></tr>
<tr><td><i>Specify the LDAP fields to use and their corresponding label, Metadot field, whether is editable, and whether is required. When the fields are displayed, they will be ordered in the same order as entered here.
<p /> Metadot fields available are email, password, fullname, and nickname and all are <b>required</b> by users except for nickname. <b>These three required Metadot fields must be assigned to an LDAP field.</b> Format should be <b>Label=LDAP field=Metadot field=is editable=is required</b>. Leave the Metadot field entry empty except for the three required fields specified previously.
<p /> If the 'is editable' field is left blank, then it will default to 0 (You can enter a 1 for yes or 0 for no). If the 'User unique identifier in LDAP directory' is listed, it will not be editable. If the 'is required' field is left blank, then it will default to 0 (You can enter a 1 for yes or 0 for no).
<p /> (e.g. Common Name=cn=fullname=1=1)</i></td></tr>
EOM
;


    $information .= "\n$fields";

    # ldap bind option
    my ($ldap_bind_selected, $ldap_bind_none);
    my $bind_user     = $PARAMS->{ldap_bind_dn} || '';
    my $bind_password = $PARAMS->{ldap_bind_password} || '';

    if ($bind_user) {
        $ldap_bind_selected  = 'checked="checked"';
        $ldap_bind_none      = '';
    } else {
        $ldap_bind_selected  = '';
        $ldap_bind_none      = 'checked="checked"';
    }

    $information .= <<EOM
    <tr><td><br /><b>Bind to LDAP server as:</td></tr>
    <tr><td>
      <table>
        <tr valign="middle">
          <td><input type='radio' name='ldap_bind_option' value="0" $ldap_bind_none/></td>
          <td colspan=2>Logged-in user (anonymous when no logged in user)</td>
        </tr>
        <tr valign="middle">
          <td><input type='radio' name='ldap_bind_option' value="1" $ldap_bind_selected/></td>
          <td>
            <table>
              <tr>
                <td>DN:</td>
                <td><input type="text" name="ldap_bind_dn" value="$bind_user" size="80" maxlength="500" /></td>
              </tr>
              <tr>
                <td>Password:</td>
                <td><input type="text" name="ldap_bind_password" value="$bind_password" /></td>
              </tr>
            </table>
          </td>
        </tr>
      </table>
    </td></tr>
EOM
;


    $information .= "\n$additional_fields";

    $information .= "<TR><TD><br /><br /><input type='submit' name='save' value='Save'></TD></TR>
</TABLE>
</form>";

    $information = "<br /><br /><H2>LDAP Parameter Administration</H2>$information";

    $DISPLAY->print_form($information, "Registration Parameters");

}

sub www_save_params {
    my $self = shift;


    $self->debug_msg(3, "Metadot::Register::LDAP::www_save_ldap_params");

    ###XXX Later here should  be some kind of validation
    if (!$FORM{ldap_server}) {
        $DISPLAY->print_click_back ("Sorry", "You haven't specified the LDAP server.");
        exit(0);
    }

    if (!$FORM{ldap_base_dn}) {
        $DISPLAY->print_click_back ("Sorry", "You haven't specified the base DN.");
        exit(0);
    }

    if (!$FORM{ldap_uid}) {
        $DISPLAY->print_click_back ("Sorry", "You haven't specified the user unique identifier in LDAP directory.");
        exit(0);
    }

    my $admin_value;
    if ($FORM{admin} eq "yes") {
        $admin_value = 1
    } else {
        $admin_value = 0
    }

    $PARAMS->set_ldap_server($FORM{'ldap_server'});
    $PARAMS->set_ldap_base_dn($FORM{'ldap_base_dn'});
    $PARAMS->set_ldap_uid($FORM{'ldap_uid'});
    $PARAMS->set_ldap_uid_label($FORM{'ldap_uid_label'});

    $PARAMS->set_property('ldap_fields_definitions', $FORM{ldap_fields_definitions});
    $PARAMS->set_property('managed_ldap', $FORM{managed_ldap});

    #CKIM-NOTIFY-ADMIN: add field
    $PARAMS->set_new_user_notification_enabled($FORM{'new_user_notification_enabled'});
    $PARAMS->set_regapproval($admin_value);

    $self->debug_msg(4, "Metadot::Register::LDAP::www_save_ldap_params: printing before lock");


    # LDAP bind option
    $self->debug_msg(4,
        "ldap_bind_option=".(defined $FORM{ldap_bind_option} ? $FORM{ldap_bind_option} : 'undef')
        ."; ldap_bind_dn=".(defined $FORM{ldap_bind_dn} ? $FORM{ldap_bind_dn} : 'undef')
        ."; ldap_bind_password=".(defined $FORM{ldap_bind_password} ? $FORM{ldap_bind_password} : 'undef')
        );

    if ($FORM{ldap_bind_option}) {
        $PARAMS->set_property('ldap_bind_dn', $FORM{ldap_bind_dn});
        $PARAMS->set_property('ldap_bind_password', $FORM{ldap_bind_password});
    } else {
        $PARAMS->set_property('ldap_bind_dn', '');
        $PARAMS->set_property('ldap_bind_password', '');
    }

    my $location = "index.pl?isa=Register&op=show_params";
    $self->debug_msg(4, "Metadot::Register::LDAP::www_ldap_registration_params: will redirect to $location");
    Utility->md_redirect($self->make_url($location));

}
sub show_modify_settings_form {
    my $self = shift;
    my ($uid, $showadminlevel)=@_;
    $self->debug_msg(3, "Metadot::Register::LDAP::show_modify_settings_form()");

#    my $action;
    my $op = "";

    if(!$uid){
    $DISPLAY->print_click_back("Error", "Sorry I did not get an UID to work on!");
    exit;
    }

    my $u = Metadot::UserFactory->get_user($uid);

    my $admin_switch = "" ;
    my $form_type = '';

    if($showadminlevel){
        $op = 'save';
        $form_type = 'profile_other';
    } else {
        $op = 'save_self';
        $form_type = 'profile_self';
    }


    my $topform = $self->get_modify_top_form( u => $u );
    my $bottomform = $self->get_modify_bottom_form( u => $u );
    my $adminform = $self->get_modify_admin_form( u => $u, showadminlevel=> $showadminlevel);

    my $auth = Metadot::AuthenticatorFactory->get_authenticator();
    ## will set form parameters depending on site's login mode
    my ($form_action, $login_mode, $submessage) = $auth->get_password_form_parameters(login_mode => $FORM{login_mode}, form_type => $form_type);

    my $window_popup = $DISPLAY->popup_window();
    my $form = $window_popup."<table><tr><td><b><font size=+1>" . $self->get_modify_form_title() . "</font></b><br />";
    $form .= "<FORM action='$form_action' method=post>\n";
    $form .= "<INPUT type='hidden' name='isa' value='User'>\n";
    $form .= "<INPUT type='hidden' name='op' value='$op'>\n";

    $SESSION->set_value('User_uid_to_modify', $uid);

    my $frame = new HTML::TableFrame("", "#ffffff");
    $frame->add($topform);
    $form .= $frame->print;
    $form .= "</td></tr>";


    if ($bottomform) {
        $form .= "<tr><td><br /></td></tr>";

        $form .= "<tr><td>";

        my $frame2 = new HTML::TableFrame("", "#ffffff");
        $frame2->add($bottomform);
        $form .= $frame2->print;

        $form .= "</td></tr>";
    }

    if( $showadminlevel || $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks', $u ) ){
    $form .= "<tr><td><br /></td></tr>";

    $form .= "<tr><td>";

    my $frame3 = new HTML::TableFrame("", "#ffffff");
    $frame3->add($adminform);
    $form .= $frame3->print;

    $form .= "</td></tr>";
    }

    $form .= "<tr><td align='right'>";

#    unless ($login_mode eq 'standard') {
#        $action .= " Securely";
#    }

#    $form .= "<p /><INPUT type=submit name='action' value='$action'>\n";
    $form .= $self->get_modify_submit_button(login_mode => $login_mode, showadminlevel => $showadminlevel);
    $form .= "</FORM>\n";
    $form .= "</td></tr></table>";

    $form .= "<p />$submessage";

}

sub get_modify_submit_button {
    my $self = shift;
    my %args = @_;

    my $login_mode = $args{login_mode};
    my $showadminlevel = $args{showadminlevel};
    my $action = '';

    if($showadminlevel){
    $action="Save User Settings";
    } else {
    $action="Save Settings";
    }

    my $form = '';

    unless ($login_mode eq 'standard') {
        $action .= " Securely";
    }

    $form .= "<p /><INPUT type=submit name='action' value='$action'>\n";
}

sub get_modify_form_title {
    my $self = shift;
    return 'Change Profile';
}

sub get_modify_top_form {
    my $self = shift;
    my %args = @_;

    my $u = $args{u};

    my $topform = "";

    $topform .= "<table width=\"500\">";

    $topform .= "<tr><td colspan='2'>";
    $topform .= "<h2><font color=\"#ff0000\">Required Fields</font></h2>";
    $topform .= "</td></tr>";

    $topform .= "<tr><td>";
    $topform .= "<b>Full Name:</b>";
    $topform .= "</td><td>";

    # we don't allow admin and default look names to change
    if( $USER->is_default_account( $u->email() ) ) {

    $topform .= "<INPUT type=hidden name=fullname size=30 value=\"".$u->fullname()."\" maxlength=50>";
    $topform .= "<b>" . $u->fullname . "</b><br />";

    } else {

    $topform .= "<INPUT type=text name=fullname size=30 value=\"".$u->fullname()."\" maxlength=50><br />";
    $topform .= "<font size=\"-2\"><i>(e.g.  Mike Saenz)</i></font>";

    }

    $topform .= "</td></tr>";

    $topform .= "<tr><td>";
    $topform .= "<b>E-mail:</b><br />";
    $topform .= "</td><td>";

    $topform .= "<INPUT type=text name=email size=30 value=\"".$u->email()."\" maxlength=50>";
    $topform .= "<br />\n";

    $topform .= "</td></tr>";


    # under LDAP, we only show password fields for default accounts
    if ( $USER->is_default_account( $u->email() ) ) {

    $topform .= "<tr><td>";
    $topform .= "<b>Password:</b>";
    $topform .="<br /><font color=\"red\">Strong Passwords Required</font><br />Click <a href=\"javascript:popUp('/html/passwdchecker.html')\">Here</a> for more information." if($PARAMS->{'password_checker'} == 1);
    $topform .="<br /><font color=\"red\">Long Passwords Required (6 characters min)</font>" if($PARAMS->{'password_checker'} == 5);
    $topform .= "</td><td>";
    $topform .= "<INPUT type=password name=password size=20><br />";
    $topform .= "<font size=\"-2\"><i>(e.g.  sm2azp)</i></font>";
    $topform .= "</td></tr>";

    $topform .= "<tr><td>";
    $topform .= "<b>Verify Password: </b>";
    $topform .= "</td><td>";
    $topform .= "<INPUT type=password name=password2 size=20>";
    $topform .= "</td></tr>";
    }

    $topform .= "</table>";
}

sub get_modify_bottom_form {
    my $self = shift;
    my %args = @_;

    my $u = $args{u};

    my $bottomform .= "<table width=\"500\">";

    $bottomform .= "<tr><td colspan='2'>";
    $bottomform .= "<h2>Optional Fields</h2>";
    $bottomform .= "</td></tr>";

    $bottomform .= "<tr><td>";
    $bottomform .= "<b>Nickname / Screenname:</b>\n";
    $bottomform .= "</td><td>";
    $bottomform .= "<INPUT type=text name=nickname size=30 value=\"".$u->nickname()."\" maxlength=50><br />";
    $bottomform .= "<font size=\"-2\"><i>(e.g.  Mike)</i></font>";
    $bottomform .= "</td></tr>";

    $bottomform .= "<tr><td>";
    $bottomform .= "<b>ICQ UIN:</b>";
    $bottomform .= "</td><td>";
    $bottomform .= "<INPUT type=text name=ICQNumber size=15 value=\"".$u->icq_number()."\">";
    $bottomform .= "</td></tr>";

    $bottomform .= "<tr><td>";
    $bottomform .= "<b>Pager address:</b>";
    $bottomform .= "</td><td>";
    $bottomform .= "<INPUT type=text name=pager size=15 value=\"".$u->pager_number()."\">";
    $bottomform .= "</td></tr>";

    $bottomform .= "<tr><td>";
    $bottomform .= "<b>Homepage:</b>";
    $bottomform .= "</td><td>";
    $bottomform .= "<INPUT type=text name=homepage size=30 value=\"".$u->homepage()."\" maxlength=50>";
    $bottomform .= "</td></tr>";
    $bottomform .= "<tr><td>";
    $bottomform .= "&nbsp;";
    $bottomform .= "</td><td>";
    $bottomform .= "<font size=\"-2\"><I>(e.g. http://www.myhomepage.com)</I></font>";
    $bottomform .= "</td></tr>";

    $bottomform .= "<tr><td>";
    $bottomform .= "<b>Quote:</b>";
    $bottomform .= "</td><td>";
    $bottomform .= "<textarea cols=30 rows=3 wrap name=quote>".$u->quote()."</textarea>";
    $bottomform .= "</td></tr>";

    $bottomform .= "</table>";
}

sub get_modify_admin_form {
    my $self = shift;
    my %args = @_;

    my $u = $args{u};
    my $showadminlevel = $args{showadminlevel};
    my $admin_switch = '';
    if($showadminlevel){
    my $not_admin = (! $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks', $u ) ) ? "checked" : "" ;
    my $is_admin = ( $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks', $u ) ) ? "checked" : "" ;
    $admin_switch = "
<b>Person Role:</b></td><td>
User <input type='radio' name='adminlevel' value='1' $not_admin> &nbsp;&nbsp;
Site Manager&nbsp;<input type='radio' name='adminlevel' value='9' $is_admin><br />
";
   }

    my $not_default_size = ($u->{upload_size} ne "-1")? "checked": "";
    my $default_size = ($u->{upload_size} eq "-1")? "checked": "";

    my $max_total_upload_size = $PARAMS->{'max_total_upload_size'};
    my $max_total_number_upload = $PARAMS->{'max_total_number_upload'};

    my $uploadsize = "<input type='radio' name=upload_size_bn value='-1' $default_size>Default: $max_total_upload_size<br />";
    if($u->{upload_size} ne "-1") {
        $uploadsize .= "<input type='radio' name=upload_size_bn value='1' $not_default_size>Custom: &nbsp;<INPUT type=text name=upload_size size=15 value=\"".$u->{upload_size}."\" maxlength=50>";
    }
    else {
        $uploadsize .= "<input type='radio' name=upload_size_bn value='1' $not_default_size>Custom: &nbsp;<INPUT type=text name=upload_size size=15 value=\"\" maxlength=50>";
    }

    $not_default_size = ($u->{upload_size} ne "-1")? "checked": "";
    $default_size = ($u->{upload_size} eq "-1")? "checked": "";

    my $not_default_number = ($u->{upload_number} ne "-1")? "checked": "";
    my $default_number = ($u->{upload_number} eq "-1")? "checked": "";

    my $uploadnumber = "<input type='radio' name=upload_number_bn value='-1' $default_number>Default: $max_total_number_upload<br />";
    if($u->{upload_number} ne "-1") {
        $uploadnumber .= "<input type='radio' name=upload_number_bn value='1' $not_default_number>Custom:&nbsp;<INPUT type=text name=upload_number size=15 value=\"".$u->{upload_number}."\" maxlength=50>";
    }
    else {
        $uploadnumber .= "<input type='radio' name=upload_number_bn value='1' $not_default_number>Custom:&nbsp;<INPUT type=text name=upload_number size=15 value=\"\" maxlength=50>";
    }

    my $adminform = "";

    $adminform .= "<table width=\"500\">";
    $adminform .= "<tr><td colspan='2'>";
    $adminform .= "<h2>Admin params</h2>";
    $adminform .= "</td></tr>";

    $adminform .= "<tr valign='baseline'><td>";
    $adminform .= "<b>Max total upload size</b> (MB)<b>:</b><br />\n";
    $adminform .= "</td><td>";
    $adminform .= "$uploadsize";
    $adminform .= "</td></tr>";

    $adminform .= "<tr><td>&nbsp;</td></tr>\n";

    $adminform .= "<tr valign='baseline'><td>";
    $adminform .= "<b>Max total number of upload:</b><br />";
    $adminform .= "</td><td>";
    $adminform .= "$uploadnumber";
    $adminform .= "</td></tr>";

    if( $FORM{uid} && ( $FORM{uid} != MetadotConfig->get_reserved_uid('ADMIN') ) ) {
    $adminform .= "<tr><td>";
    $adminform .= "$admin_switch";
    $adminform .= "</td></tr>";
    }

    $adminform .= "</table>";
}

###############################################################
### save user's settings : 'backend' function for
### User::www_save and User::www_save_self
### the uid being modifyied is in FORM{uid}
### the user modifying is in $USER->{uid}
###
### if the keep_same_adminlevel==1 then the user is modifying
### himself and we don't want to loose his current privilege
### so we will read the db first.
###############################################################
sub save_modify_settings_form {
    my $self = shift;

    my $basename = $PARAMS->{basename};
    my ($keep_same_adminlevel) = @_;


    $DISPLAY->print_http_header;

    my $u = Metadot::UserFactory->get_user( $SESSION->get_value('User_uid_to_modify') );

    my $errormsg = $self->validate_modify_settings_form($u);

    if( $errormsg ){
        #####      ERROR     #####
        $DISPLAY->print_click_back("Sorry there is a slight problem", "$errormsg<br />");
        return 0;

    } else {

        ##### NO ERROR we do the update #####
        ### set privilege to NORMAL USER BY DEFAULT
        my $admin_level = 1;
        if( $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks' ) ) {

            if(defined $FORM{adminlevel} && $FORM{adminlevel}) {
                $admin_level = $FORM{adminlevel};
            }
        }

        my $old_email = $u->{email};
        $u->fullname( escapeHTML($FORM{fullname}) );
        $u->nickname( escapeHTML($FORM{nickname}) );
        $u->homepage( escapeHTML($FORM{homepage}) );
        $u->email($FORM{email});
        $u->pager_number( escapeHTML($FORM{pager}) );
        $u->icq_number( escapeHTML($FORM{ICQNumber}) );
        $u->quote( escapeHTML($FORM{quote}) );

        # default accounts can save their passwords
        if ($u->is_default_account) {
            $DISPLAY->debug_msg(4, "saving the password for a default account");
            $u->save_password($FORM{password});
        }


        if($FORM{upload_size_bn} && ($FORM{upload_size_bn} >= 0) && $FORM{upload_size}) {

            $u->upload_size( escapeHTML($FORM{upload_size}) );

        }else{

            $u->upload_size( "-1");
        }

        if($FORM{upload_number_bn} && ($FORM{upload_number_bn} >= 0) && $FORM{upload_number}){

            $u->upload_number( escapeHTML($FORM{upload_number}) );

        }else{

            $u->upload_number( "-1");
        }


        unless ($keep_same_adminlevel){

            $u->admin_level( $admin_level );
        }

            ## Complete duties before saving
            $errormsg = $self->pre_save_duties(u => $u);
            if ($errormsg ne '') {
                $DISPLAY->print_click_back("Sorry there is a slight problem", "$errormsg<br />");
                return 0;
            }

        $u->save();

        my $to = $u->{email};

        if ($to) {
            my $cc = "";
            my $bcc = "";
            my $subject = "Your Profile Has Been Changed";
            my $message = "Your profile has been changed.  If you wish to view your new profile, please click here:  $basename/index.pl?isa=User&amp;op=modify_self";
            my $sm = Sendmail->new($to, $cc, $bcc, $subject, $message);
            $sm->sendMail();

            if ($old_email ne $to) {
                $sm = Sendmail->new($old_email, $cc, $bcc, $subject, $message);
                $sm->sendMail();
            }
        }



        my $grp_id = DBAccess->sqlSelect("grp_id", "grp", "group_name='SITE_MGR'");

        # XXX move all this code to Groups.pm
        if($admin_level == 9){

            Group->_add_a_member_to_a_group($grp_id, $SESSION->get_value('User_uid_to_modify') );

        } elsif ( ! $keep_same_adminlevel) { #if not keeping admin level

            my $uid = $SESSION->get_value('User_uid_to_modify');

            if(DBAccess->sqlSelect("userid", "grpmembers", "userid=$uid AND grp_id=$grp_id")){
            DBAccess->sqlDelete("grpmembers", "userid=$uid AND grp_id=$grp_id");

           $CACHE->set_as_dirty('GROUPS_HASH');

            }
        }

        #################

        my $name = $u->webname();

        my ($title);

        if( $SESSION->get_value('User_uid_to_modify') != $USER->{uid}){
            # an admin is updating a user

            $title="Success: settings are saved.<br /><br />\n";
            $DISPLAY->two_choices ($title, "confirm", "Home", Utility->get_qualified_url("index.pl"), "Back to Edit Users", Utility->get_qualified_url("index.pl?manage=Users&isa=Portal&app=&iid=0&op=modify_site_params"));

        } else {

            # self modif
            $title="Success $name, your settings are saved. <br /><br />You can go to the front page<br /><br />\n";
            $DISPLAY->two_choices ($title, "confirm", "Home", Utility->get_qualified_url("index.pl"));
        }

    }
}


sub validate_modify_settings_form {
    my $self = shift;
    my $u    = shift || die("Must provide the user being validated");

    my $errormsg = '';

    if(!($FORM{fullname})) {
        $errormsg .= "<h3>You need to enter your fullname.</h3><br /><p />";
    }

    # if the nickname is null then we use the name
    $FORM{nickname} = $FORM{nickname}|| $FORM{fullname};

    if (!$FORM{email} || $FORM{email} =~ m/\<|\>|\&/) {
        $errormsg .= "<h3>You need to provide a real email address</h3><br /><p />";
    }


    if( ($FORM{password} ne $FORM{password2}) ) {
        $errormsg .= "<h3> Your passwords didn't match</h3><p />";
    }

    # check if the email address exist for another user with a different uid
    unless ( $u->can_use_email_address( $FORM{email} ) ){
        $errormsg .=  "<h3>The email address you entered is already used by another user. Please verify your email address. If this 'another user' is you then try to login or ask your system admin to help you.</h3><br /><p />\n";
    }

    return $errormsg;
}


sub pre_save_duties {
    my $self = shift;
    my %args = @_;

    my $u = $args{u};

    ### UPDATE with password (if there is one)
    if ($FORM{password}){

        $u->save_password($FORM{password});
    }
    return '';
}

####################################################################
#   Send Accept/Reject noticed via email
####################################################################
sub send_approval_message {
    my $self = shift;
    my $user = shift;
    my ($mail_type) = shift;

    my $to = $user->email;
    my $cc = "";
    my $bcc = "";
    my $fullname = $user->webname;
    my $nickname = $user->nickname || $user->webname;

    my $password_string = "Password is the same as your password for LDAP Directory.";

    # XXX directly accessing an attribute in $user
    my $username_string = $user->{ldap_dn};

    my $subject = "Registration Confirmation";
    my $mesg;

    if ($mail_type eq "auto"){

    $mesg = $PARAMS->{defregmessage};
    $mesg = $user->substitute($mesg, $fullname, $username_string, $password_string, $nickname);

    } elsif ($mail_type eq "accepted"){

    $mesg = $PARAMS->{aprregmessage};
    $mesg = $user->substitute($mesg, $fullname, $username_string, $password_string, $nickname);

    } else {

    $mesg = $PARAMS->{rejregmessage};
    $mesg = $user->substitute($mesg, $fullname, $username_string, $password_string, $nickname);

    }

    if ($to) {
        my $sm = Sendmail->new($to, $cc, $bcc, $subject, $mesg);
        $sm->sendMail();
    }
}


=head1 NAME

Register -

=head1 SYNOPSIS

    use Metadot::Register::Default.pm
    $register  = Metadot::Register::Default->new();
    $register->send_new_user_notification();
    $register->www_register_form
    $register->www_register
    $register->www_show_params
    $register->www_edit_params
    $register->www_save_params
    $register->save_modify_settings_form
    $register->show_modify_settings_form
    $register->send_approval_message
    $register->is_a
    $register->id
    $register->save
    $register->get_owner_uid
    $register->parent_id


=head1 NAME

Metadot::Register::LDAP -

=head1 SYNOPSIS

    use Metadot::Register::LDAP.pm
    $register  = Metadot::Register::LDAP->new();
    $register->send_new_user_notification();
    $register->www_register_form
    $register->www_register
    $register->www_show_params
    $register->www_edit_params
    $register->www_save_params
    $register->save_modify_settings_form
    $register->show_modify_settings_form
    $register->send_approval_message
    $register->is_a
    $register->id
    $register->save
    $register->get_owner_uid
    $register->parent_id

=head1 DESCRIPTION

Metadot::Register::LDAP is a concrete subclass of the Metadot::Register class.
This subclass is instantiated and returned by Metadot::RegisterFactory, if the
factory determines that this subclass is the one that it wants to return.

This particualar subclass allows people to register by entering their unique LDAP field
(designated by the admin) and their LDAP password


Show the form that people use to register on the site
    $register->www_register_form

Process the register form

    $register->www_register

Show the registration parameters

    $register->www_show_params

Form to edit the registration paramaters

    $register->www_edit_params

Code to process the changed parameters

    $register->www_save_params

Code to process the changed registration parameters

    $register->save_modify_settings_form

Form to edit the registration parameters

    $register->show_modify_settings_form

Sends an approval message to the new user

    $register->send_approval_message


=cut

1;

