###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name$
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Register::ManagedLDAPRegister;
use strict;

use Metadot::SessionHandlerFactory;
use Metadot::Register::LDAP;
use CGI qw(escapeHTML);
#use MIME::Base64;
use Encode;

use Metadot::LDAPSubsystemFactory;

use Metadot::SessionHandlerFactory;

use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CACHE $ACCESS_BROKER $CGI);

use vars qw(@ISA);
@ISA=qw(Metadot::Register::LDAP);

my $AC = {
       BUNDLES => {

                   CREATE => { DESC =>          'Create Account',
                               OPS =>            [qw(register_form register show_params edit_params modify_params save_params)],
                               DEFAULT_LEVEL =>  0,
                               MIN_LEVEL =>      0,
                             }
                  },
          DISPLAY_ORDER => [qw(CREATE)],
};

sub get_AC {
    my $self = shift;
    my $register_level = $self->get_register_level();
    $AC->{BUNDLES}->{CREATE}->{DEFAULT_LEVEL} = $AC->{BUNDLES}->{CREATE}->{DEFAULT_LEVEL} = $register_level;
    return $AC;
}


sub get_register_level {
    my $self = shift;
    my $default = shift || 0;

    my $register_level = $default;
    if ($PARAMS->{registration_closed}) {
        $register_level = 8;
    }
    return $register_level;
}


sub new {
    my $proto = shift;
    my $class = __PACKAGE__;
    my $self = {};
    $self  = $class->SUPER::new();

    bless ($self, $class);
    return $self;
}

sub www_modify_paramsX {
    my $self = shift;
    $self->debug_msg(3, "Metadot::Register::ManagedLDAPRegister::www_modify_params");

    my $ldap_fields_definitions = $PARAMS->{ldap_fields_definitions};

    my $fields = <<EOM
<tr><td><br /><b>LDAP Fields Definition</b></td></tr>
<tr><td><textarea cols=50 rows=8 wrap name=ldap_fields_definitions>$ldap_fields_definitions</textarea></td></tr>
<tr><td><i>Specify the LDAP fields to use and their corresponding label and Metadot field. Format should be <b>Label=LDAP field=Metadot field</b>. The Metadot field entry is optional. (e.g. Last Name=LastName=lastname)</i></td></tr>
EOM
;
    $self->SUPER::www_modify_params(additional_fields => $fields);
}

sub www_save_paramsX {
    my $self = shift;
    $self->debug_msg(3, "Metadot::Register::ManagedLDAPRegister::www_save_params");

    $PARAMS->set_property('ldap_fields_definitions', $FORM{ldap_fields_definitions});
    $self->SUPER::www_save_params();
}

sub www_show_paramsX {
    my $self = shift;
    $self->debug_msg(3, "Metadot::Register::ManagedLDAPRegister::www_show_params");

    my $ldap_fields_definitions = $PARAMS->{ldap_fields_definitions};
    $ldap_fields_definitions =~ s/\n/<br \/>/g;
    my $fields = <<EOM
        <TR bgcolor = #efefef>
            <TD valign='top'>LDAP Fields Definition</TD>
            <TD>$ldap_fields_definitions</TD>
        </TR>
EOM
;
    $self->SUPER::www_show_params(additional_fields => $fields);
}

sub www_register_form {
    my $self = shift;
    $self->debug_msg(3, "Metadot::Register::ManagedLDAPRegister::www_register_form");

    if( $USER->max_number_of_users_quota_exceeded() ) {

    $DISPLAY->print_click_back( 'User registration forbidden. Maximum number of users has been reached.' );
    exit;
    }

    $USER->logout();

    my $target = $SESSION->{referer} || 'index.pl';
    $target = $CGI->escape($target);

    my $url1 = "index.pl?isa=Session&op=login&registering=1&target=$target";

    Utility->md_redirect($url1);
    exit(0);
}

sub www_register {
    my $self = shift;
    my %args = @_;
    $self->debug_msg(3, "Metadot::Register::ManagedLDAPRegister::www_register() with $args{ldap_uid}");

    my $ldap = Metadot::LDAPSubsystemFactory->get_ldap_subsystem();

    $ldap->initialize();

    my $ldap_entry = $ldap->_get_ldap_entry($args{ldap_uid});
    return unless $ldap_entry;

    #print STDERR "\n\ncn:".$entry->get_value('cn')."\npostaladdress".$entry->get_value('postaladdress')."\nmail:".$entry->get_value('mail');

    my $metadot_values = $self->get_metadot_ldap_values(ldap_entry => $ldap_entry);
    $metadot_values->{ldap_uid} = $args{ldap_uid};
    $metadot_values->{password} = $args{password};
    $self->SUPER::www_register(
        %$metadot_values
        #ldap_uid => $args{ldap_uid},
        #password => $args{password},
        #fullname => $ldap_entry->get_value('cn'),
        #nickname => 'mike',
        #email    => $ldap_entry->get_value('title'),
    );
}

sub get_ldap_field_values {
    my $self = shift;
    my %args = @_;

    $self->debug_msg(3, "Metadot::Register::ManagedLDAPRegister::get_ldap_field_values()");

    my @values = ();
    my $value_hash = {};
    my $only_metadot = $args{only_metadot} || 0; #only those with metadot
    my $only_editable = $args{only_editable} || 0; #only those that are editable
    my $ldap_entry = $args{ldap_entry};
    my $ldap_fields_definitions = $PARAMS->{ldap_fields_definitions};

    my ($label, $ldap_field, $metadot_field, $ldap_field_value, $is_editable, $is_required);
    foreach my $line (split ("\n", $ldap_fields_definitions)) { ## each line
        $line =~ s/\r//g;
        $label = '';
        $ldap_field = '';
        $metadot_field = '';
        $is_editable = 0;
        $is_required = 0;
        $value_hash = {};
        ($label, $ldap_field, $metadot_field, $is_editable, $is_required) = split ('=', $line);
        #print STDERR "\nlabel:|$label| ldap_field:|$ldap_field| metadot_field:|$metadot_field|";
        next if $only_metadot && (!defined($metadot_field) || $metadot_field eq '');
        next if $only_editable && !$is_editable;

        $value_hash->{ldap_field} = $ldap_field;
        $value_hash->{metadot_field} = $metadot_field;
        $value_hash->{label} = $label;
        $value_hash->{is_editable} = $is_editable || 0;
        $is_required ||= 0;
        $value_hash->{is_required} = ($metadot_field eq 'fullname' || $metadot_field eq 'email' || $metadot_field eq 'password') ? 1 : $is_required;
        if ($ldap_entry) {
            if ($ldap_field eq 'dn') {
                $value_hash->{ldap_field_value} = $ldap_entry->dn;
            } else {
                $value_hash->{ldap_field_value} = encode("iso-8859-1" , decode("utf8", $ldap_entry->get_value($ldap_field)));
            }
        }
        push @values, $value_hash;
    }

    #use Data::Dumper;
    #print STDERR Dumper $values;

    return \@values;
}

sub get_metadot_ldap_values {
    my $self = shift;
    my %args = @_;

    $self->debug_msg(3, "Metadot::Register::ManagedLDAPRegister::get_metadot_ldap_values()");

    $args{only_metadot} = 1;
    ## array ref
    my $main_values = $self->get_ldap_field_values(%args);

    my $metadot_values = {};
    my ($metadot_field, $ldap_field_value);
    foreach my $value (@$main_values) {
        $metadot_field    = $value->{metadot_field};
        $ldap_field_value = $value->{ldap_field_value};
        $metadot_values->{$metadot_field} = $ldap_field_value;
    }

    #use Data::Dumper;
    #print STDERR Dumper $metadot_values;
    return $metadot_values;
}

sub get_modify_top_form {
    my $self = shift;
    my %args = @_;
    $self->debug_msg(3, "Metadot::Register::ManagedLDAPRegister::get_modify_top_form()");

    my $u = $args{u};

    if ($USER->is_default_account( $u->email() )) {
        return $self->SUPER::get_modify_top_form(%args);
    }

    unless ($u->load_ldap_values()) { ## load values from ldap server
        return "ERROR READING FROM LDAP. PLEASE TRY AGAIN LATER.";
    }
    my $main_values = $self->get_ldap_field_values(%args);

    my $topform = "";
    my $REQUIRED_HTML = "<font color=\"red\">*</font>";

    ## old password
    unless ($PARAMS->{ldap_bind_dn}) {
        $topform .= "<tr><td colspan='2'>\n";
        $topform .= "<b>You must enter the current password to make changes.<b><br />\n";
        $topform .= "</td></tr>\n";
        $topform .= "<tr><td>\n";
        $topform .= "<b>Current Password $REQUIRED_HTML:</b><br />\n";
        $topform .= "</td><td>\n";

        $topform .= "<INPUT type=password name=\"current_password\" size=20 maxlength=50>\n";
        $topform .= "<br /><br />\n";
        $topform .= "</td></tr>\n";
    }

#    $topform .= "<tr><td>\n";
#    $topform .= "<b>LDAP dn:</b><br />\n";
#    $topform .= "</td><td>\n";

#    $topform .= $u->{true_ldap_dn} . "\n";
#    $topform .= "</td></tr>\n";


    my $label            = '';
    my $metadot_field    = '';
    my $ldap_field       = '';
    my $ldap_field_value = '';
    my $is_editable      = 0;
    my $is_required      = 0;
    my $field_html       = 0;
    my $is_required_html = '';
    foreach my $value (@$main_values) {
        $label            = $value->{label};
        $metadot_field    = $value->{metadot_field};
        $ldap_field       = $value->{ldap_field};
        $is_editable      = $value->{is_editable};
        $is_required      = $value->{is_required};
        $ldap_field_value = $u->{$ldap_field};
        $is_required_html = '';
        $field_html       = '';
        $self->debug_msg(4, "Metadot::Register::ManagedLDAPRegister::get_modify_top_form() label:$label metadot_field:$metadot_field ldap_field:$ldap_field ldap_field_value:$ldap_field_value is_editable:$is_editable is_required:$is_required");
        if (!$is_editable || ($ldap_field eq $PARAMS->{'ldap_uid'} || $ldap_field eq 'dn')) {
            if ($metadot_field eq 'password') {
                $field_html .= "*****";
            } else {
                $field_html .= "$ldap_field_value";
                if ($metadot_field eq 'fullname' || $metadot_field eq 'email') {
                    $field_html .= "<INPUT type=hidden name=\"$metadot_field\" value=\"$ldap_field_value\">\n"; ## to update only Metadot values
                }
            }
        } elsif ($metadot_field eq 'password') {
            $is_required_html = $REQUIRED_HTML;
            $field_html .= "<INPUT type=password name=\"$ldap_field\" size=20 maxlength=50>\n";
            $field_html .= "</td></tr>\n";
            $field_html .= "<tr><td>\n";
            $field_html .= "<b>Verify $label $is_required_html:</b><br />\n";
            $field_html .= "</td><td>\n";
            $field_html .= "<INPUT type=password name=\"" . $ldap_field . "2\" size=20 maxlength=50>\n";
        } else {
            $field_html .= "<INPUT type=text name=\"$ldap_field\" size=30 value=\"$ldap_field_value\" maxlength=50>\n";
            if ($is_required) {
                $is_required_html = $REQUIRED_HTML;
            }
        }

        $topform .= "<tr><td>\n";
        $topform .= "<b>$label $is_required_html:</b><br />\n";
        $topform .= "</td><td>\n";

        $topform .= $field_html;

        $topform .= "</td></tr>\n";
    }
#    $topform .= $u->{true_ldap_dn} . "\n";

    $topform .= "<tr><td colspan='2'>\n";
    $topform .= "<BR<br /><i>Fields labeled with a $REQUIRED_HTML are Mandatory.</i>\n";
    $topform .= "<br /><br /><b>LDAP dn:</b> " . encode("iso-8859-1" , decode("utf8", $u->{true_ldap_dn}));
    $topform .= "<br /><b>LDAP Server:</b> " . $PARAMS->{'ldap_server'} . "<br />\n";
    $topform .= "</td></tr>\n";

    return $topform;
}


sub get_modify_bottom_form {
    my $self = shift;
    my %args = @_;
    $self->debug_msg(3, "Metadot::Register::ManagedLDAPRegister::get_modify_bottom_form()");

    my $u = $args{u};

    if ($USER->is_default_account( $u->email() )) {
        return $self->SUPER::get_modify_bottom_form(%args);
    }

    return '';
}

sub get_modify_submit_button {
    my $self = shift;
    my %args = @_;

    $self->debug_msg(3, "Metadot::Register::ManagedLDAPRegister::get_modify_submit_button()");

    my $login_mode = $args{login_mode};
    my $showadminlevel = $args{showadminlevel};
    my $action = "Save In LDAP";

    my $form = '';

    unless ($login_mode eq 'standard') {
        $action .= " Securely";
    }

    $form .= "<p /><INPUT type=submit name='action' value='$action'>\n";
}

sub save_modify_settings_form {
    my $self = shift;

    my $u = Metadot::UserFactory->get_user( $SESSION->get_value('User_uid_to_modify') );
    unless ($u->is_default_account()) {
        $DISPLAY->debug_msg(4, "user is not a default account");
        my $main_values = $self->get_ldap_field_values(only_editable => 1);
        my $metadot_field = '';
        my $ldap_field = '';
        ## set the metadot fields
        foreach my $value (@$main_values) {
            $metadot_field    = $value->{metadot_field};
            $ldap_field       = $value->{ldap_field};
            $FORM{$metadot_field} = $FORM{$ldap_field};
            if ($metadot_field eq 'password') {
                $FORM{$metadot_field . '2'} = $FORM{$ldap_field . '2'};
            }
        }
    }

    $self->SUPER::save_modify_settings_form();
}


# We don't always know what fields will exist, so much of the processing
# must be conditional
sub validate_modify_settings_form {
    my $self = shift;
    my $u    = shift || die("Must provide the user being validated");

    my $errormsg = '';

    # if the nickname is null then we use the name
    $FORM{nickname} = $FORM{nickname}|| $FORM{fullname};

    if( ($FORM{password} ne $FORM{password2}) ) {
        $errormsg .= "<h3> Your passwords didn't match</h3><p />";
    }

    # check if the email address exist for another user with a different uid
    if ($FORM{email}) {
        unless ( $u->can_use_email_address( $FORM{email} ) ){
            $errormsg .=  "<h3>The email address you entered is already used by another user. Please verify your email address. If this 'another user' is you then try to login or ask your system admin to help you.</h3><br /><p />\n";
        }
    }

    return $errormsg;
}


sub get_modify_form_title {
    my $self = shift;

    my $u = Metadot::UserFactory->get_user( $SESSION->get_value('User_uid_to_modify') );
    if ($USER->is_default_account( $u->email() )) {
        return $self->SUPER::get_modify_form_title();
    } else {
        return 'Updating your LDAP record';
    }
}

sub pre_save_duties {
    my $self = shift;
    my %args = @_;
    $self->debug_msg(3, "Metadot::Register::ManagedLDAPRegister::pre_save_duties()");

    my $u = $args{u};
    return if ($USER->is_default_account( $u->email() ));

    unless ($PARAMS->{ldap_bind_dn}) {
        if (!defined($FORM{current_password}) || $FORM{current_password} eq '') {
            return 'You must enter the current password to make changes to LDAP.';
        }
    }

    my $main_values = $self->get_ldap_field_values(only_editable => 1);
    my $metadot_field = '';
    my $ldap_field = '';
    my $cn_value = '';
    my @replace_fields;
    my @delete_fields;
    my $is_required = 0;
    my $label = '';
    my $error_msg = '';
    foreach my $value (@$main_values) {
        $metadot_field    = $value->{metadot_field};
        $ldap_field       = $value->{ldap_field};
        $is_required      = $value->{is_required};
        $label            = $value->{label};
        next if $metadot_field eq 'password' && $FORM{$ldap_field} eq '';
        next if $ldap_field eq $PARAMS->{'ldap_uid'}; ## don't overwrite
        next if $ldap_field eq 'dn'; ## don't overwrite

        $DISPLAY->debug_msg(4, "ldap_field:$ldap_field form_value:$FORM{$ldap_field} metadot_value:" . $u->{$metadot_field});

        #next if $ldap_field eq 'cn' && $FORM{$ldap_field} eq $u->{$metadot_field};
        if ($is_required && $FORM{$ldap_field} eq '' && $metadot_field ne 'password') {
            $error_msg .= "<br />$label is required.";
        } elsif ($ldap_field eq 'cn') {
            $cn_value = encode("utf8", $FORM{$ldap_field});
            $self->debug_msg(4, "pre_save_duties() cn: '$cn_value'");
        } elsif ($FORM{$ldap_field} eq '') {
            push @delete_fields, { ldap_field => $ldap_field, ldap_field_value => encode("utf8", $FORM{$ldap_field}) };
            $self->debug_msg(4, "pre_save_duties() will delete $ldap_field");
        } else {
            my $value = encode("utf8", $FORM{$ldap_field});
            push @replace_fields, $ldap_field => $value;
            $self->debug_msg(4, "pre_save_duties() will replace $ldap_field with '$value'");
        }
    }

    if ($error_msg ne '') {
        return $error_msg;
    }

    my $ldap = Metadot::LDAPSubsystemFactory->get_ldap_subsystem();

    $ldap->initialize();

    if ($ldap->modify_account(ldap_uid => $u->{ldap_dn}, password => $FORM{current_password}, replace_fields => \@replace_fields, delete_fields => \@delete_fields, cn_value => $cn_value)) {
        return;
    } else {
        return 'There was an error with LDAP. ' . $ldap->error ;
    }
}

1;

