package Metadot::Register::Register3000;

use strict;

use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS
                $DISPLAY $CACHE $ACCESS_BROKER %REQUEST_VARS
                );
use Metadot::Register::FlexRegister;
use MetadotConfig;

use vars qw(@ISA);
@ISA = qw(Metadot::Register::FlexRegister);


# outputs a registration / profile form. Called by all register/modify operations
sub _output_form {
    my $self = shift;
    my $args = shift || {};

    my $user            = $args->{user};
    my $submit_label    = $args->{submit_label};
    my $submit_op       = $args->{submit_op};
    my $submit_isa      = $args->{submit_isa};
    my $form_title      = $args->{form_title};
    my $form_action     = $args->{form_action} || 'index.pl';

    my $form_header_text    = $args->{header_text} || undef;
    my $form_footer_text    = $args->{footer_text} || undef;


    # get my fields and values from the user object
    my ($fields, $values) = $user->define_form_fields();

    # some fields need to be tweaked
    foreach my $field (@$fields) {
        if ($field->name() eq 'password') {
            if( $PARAMS->{'password_checker'} == 1) {
                $field->inline_help("<font color=\"red\">Strong Passwords Required</font><br />Click <a href=\"javascript:popUp('/html/passwdchecker.html')\">Here</a> for more information.");
            } elsif( $PARAMS->{'password_checker'} == 5) {
                $field->inline_help("<font color=\"red\">Long Passwords Required (6 characters min)</font>");
            }
        }
    }

    my $calling_url = $FORM{calling_url} || $FORM{refererlink};
    my $landing_url = $self->_get_landing_url();

    $DISPLAY->debug_msg(4, "calling URL is $calling_url");
    $DISPLAY->debug_msg(4, "landing URL is $landing_url");

    # build the form object
    my $form = Metadot::Form3000::Form->new(
        'action'  => $form_action,
        'method'  => 'post',
        'enctype' => 'multipart/form-data',
        'name'    => 'fm',
        'title'   => $form_title,
        'fields'  => $fields,
        'values'  => $values,
        'submit_actions' => [
            {
                name  => 'submit_form',
                value => $submit_label,
            }
        ],
        'header_text' => $form_header_text,
        'footer_text' => $form_footer_text,
        calling_url   => $calling_url,
        landing_url   => $landing_url,
        field_layout  => Metadot::Form3000::Form->FIELD_LAYOUT_TWO_COLUMN(),
    );

    $form->generate_form_field(
        type  => 'hidden',
        name  => 'isa',
        value => $submit_isa);

    $form->generate_form_field(
        type  => 'hidden',
        name  => 'uid',
        value => $user->uid());

    $form->generate_form_field(
        type  => 'hidden',
        name  => 'op',
        value => $submit_op);

    $form->generate_form_field(
        type  => 'hidden',
        name  => 'admin_mode',
        value => $FORM{admin_mode});

    $form->generate_form_field(
        type  => 'hidden',
        name  => 'enroll_groups',
        value => $FORM{enroll_groups});



    # the administrative user can directly modify the admin properties of a new user
    if( $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks', $USER ) ){
        my $admin_frame = $self->render_admin_frame($user);
        if ($admin_frame) {
            $form->generate_form_field({
                type => 'predefined',
                html => qq~<td colspan="2">$admin_frame</td>~,
            });
        }
    }



    $REQUEST_VARS{form_object} = $form;


    # Send output through the Metadot controller
    my $controller = Metadot::PortalFactory->get_controller();
    my $context = Metadot::PortalFactory->build_context({
        subject => $self,
    });

    $controller->output({
        context  => $context,
        template => 'form_3000.tt2'
        });
}




# called by www_register
sub _output_register_form {
    my $self = shift;
    my $args = shift || {};

    my $auth = Metadot::AuthenticatorFactory->get_authenticator();

    ## will set form parameters depending on site's login mode
    my $refererlink = $FORM{calling_url} || $FORM{refererlink} || $SESSION->referer() || 'index.pl';
    my ($form_action, $login_mode, $submessage) = $auth->get_password_form_parameters(login_mode => $FORM{login_mode}, form_type => 'register', refererlink => $refererlink);

    $args->{submit_label} = 'Register'
        . (($login_mode eq 'standard') ? '' : ' securely');
    $args->{submit_op}    = 'register';
    $args->{submit_isa}   = 'Register';
    $args->{form_title}   = 'Registration Form';
    $args->{form_action}  = $form_action;

    if( $PARAMS->{regapproval} ) {
        $args->{form_suffix} = $self->_generate_regapproval_text();
    }

    return $self->_output_form($args);
}



# called by modify operations
sub show_modify_settings_form {
    my $self = shift;
    my ($uid, $admin_save) = @_;

    if(!$uid){
        $DISPLAY->print_click_back("Error", "Sorry I did not get an UID to work on!");
        exit;
    }

    # get the user object
    my $u = Metadot::UserFactory->get_user($uid);

    # check my user type
    $u->isa('Metadot::User::User3000') || die ("Unsupported user type $u");

    # this behavior is derived from FlexRegister. It may not be necessary here,
    # but I don't know.
    $SESSION->set_value('User_uid_to_modify', $uid);

    # TODO: one day we will need to support:
    #       MetadotConfig->get_param_value('user_management_is_external');


    # I have some dynamic behavior based on whether user is editing self or
    # admin is editing other user

    my $submit_label;
    my $submit_op;

    if($admin_save){
        $submit_label="Save User Settings";
        $submit_op = 'save';
    } else {
        $submit_label="Save Settings";
        $submit_op = 'save_self';
    }

    # TODO: need to figure out what's going on with this method:
    # my ($form_action, $login_mode, $submessage) = $auth->get_password_form_parameters(login_mode => $FORM{login_mode}, form_type => $form_type, refererlink => $refererlink);

    return $self->_output_form({
        user            => $u,
        submit_label    => $submit_label,
        submit_op       => $submit_op,
        submit_isa      => 'User',
        form_title      => 'Editing User Profile',
        });
}



sub save_modify_settings_form {
    my $class                   = shift;
    my $keep_same_adminlevel    = shift;

    my $uid = $SESSION->get_value('User_uid_to_modify');
    my $user = Metadot::UserFactory->get_user($uid);

    my $old_email = $user->email();

    my $form = $class->_process_profile_form({ user=>$user });

    if ($form->has_errors()) {
        my $admin_save = (defined ($FORM{op}) && $FORM{op} eq 'save');
        return $class->show_modify_settings_form($uid, $admin_save);
    } else {
        $class->_process_user_admin_params(
        {
            user  => $user,
            input => \%FORM,
            # keep_same_admin_level => 1,
            # using 'keep_same_admin_level' prevents this method from having
            # any effect on the user role
        });

        # process admin parameters
        $user->save();

        # if the password was provided, save it
        my $password = $form->get_value('password');
        $DISPLAY->debug_msg(4, "Password submitted? '$password'");
        if (defined $password && $password ne '') {
            $user->save_password($password);
        }

        # check if the register form included groups for enrollment
        $class->_process_registration_to_groups(\%FORM, $user);

        # send the email
        $class->_send_profile_change_email($user, $old_email);


        my $name = $user->webname();
        my $title;

        my $refererlink = $form->landing_url() || $class->_get_landing_url();

        if( $uid != $USER->{uid}){
            # an admin is updating a user
            $title="Success: settings are saved.<br /><br />\n";
            $DISPLAY->print_choices ($title, "confirm", "Continue", Utility->get_qualified_url($refererlink));
        } else {
            # self modification
            $title="Success $name, your settings are saved.\n";
            $refererlink = $refererlink =~ /confirm_upload_deletion/ ? 'index.pl' : $refererlink;
            $DISPLAY->two_choices ($title, "confirm", "Continue", Utility->get_qualified_url($refererlink));
        }
    }
}




# sub called by www_register
sub _process_registration_form_input {
    my $class = shift;
    my $args  = shift || {};

    my $input  = $args->{input} || \%FORM;

    # Check if User Exists
    $class->_check_user_email_exists($input->{email});

    # start with a fresh user object
    my $user = $args->{user} = Metadot::UserFactory->get_user();

    # populate the user properties from the profile form
    my $form = $class->_process_profile_form($args);

    if ($form->has_errors()) {
        $class->_output_register_form({ user=>$user});
        return undef;
    } else {
        # set the approval level based on site params
        $class->_set_user_approval($user);

        # encrypt the password if needed
        $user->password(Portal->encrypt_password_if_necessary($user->password()));

        # add the user to the database
        my $uid = $user->add();

        # admin parameters
        $class->_process_user_admin_params(
        {
            user  => $user,
            input => $input,
        });

        # check if the register form included groups for enrollment
        $class->_process_registration_to_groups(\%FORM, $user);

        # create the mypage
        $user->create_mypage($uid) || die "can't create mypage";

        return $user;
    }
}


# processes form input and stores the values to self
# returns the form object that resulted.
sub _process_profile_form {
    my $self   = shift;
    my $args   = shift || {};

    my $user   = $args->{user};
    my $input  = $args->{input} || \%FORM;


    my @mappings = @{$user->property_mappings()};

    # get my fields
    my ($fields, $values) = $user->define_form_fields();

    # some fields need to be tweaked
    my $uid         = $user->uid();
    my $admin_id    = MetadotConfig->get_reserved_uid('ADMIN');

    foreach my $field (@$fields) {
        # if default account, most fields are not mandatory
        #if (defined($uid) && $uid == $admin_id) {
        if (defined $user && $user->is_default_account()) {
            unless ($field->name() eq 'email') {
                $field->is_mandatory(0);
            }
        }

        # password is not mandatory unless user is new
        if (defined $uid && $field->name() eq 'password') {
            $field->is_mandatory(0);
        }
    }

    my $form = Metadot::Form3000::Form->new(
        fields => $fields,
        values => $values,
    );

    $form->process_input( $input );

    # verify the passwords
    my $password = $FORM{password};
    unless (!defined($password) || $password eq '') {
        my $verify = Form::Verify->new();
        $self->verify_password(
                $verify, 'Password', $password,
                $FORM{first_name}, $FORM{nickname}
                );

        if ($verify->errors()) {
            $form->add_error($verify->errors());
        }
    }

    if ($form->has_errors()) {
        $DISPLAY->debug_msg(3, "Form processed an error.");
    } else {
        # apply the values to myself
        my $values = $form->values();

        foreach my $mapping (@mappings) {

            # skip properties that are not visible, because there will be
            # no input.
            if ($mapping->visible())
            {
                my $property = $mapping->mapped_property();
                $DISPLAY->debug_msg(5, "processing profile value for $property");

                my $form_key = $mapping->generate_form_field()->form_key(); # TODO: reorganize this code to prevent duplicate calls to generate_form_field
                my $value = $form->get_value($form_key);

                if ($user->can($property)) {
                    $DISPLAY->debug_msg(5, "process_edit_form sets ".($property||'')." to ".($value||'')." via ACCESSOR");
                    $user->$property($value);
                } else {
                    $DISPLAY->debug_msg(5, "process_edit_form sets ".($property||'')." to ".($value||'')." via SET_VALUE");
                    $user->set_value($property, $value);
                }
            }
            else
            {
                $DISPLAY->debug_msg(4, "process_edit_form skips non-visible property ".$mapping->property_name());
            }
        }

        # process any custom field input
        $self->input_custom_fields(
            user    => $user,
        );


        # force the nickname to have a value, because a lot of code makes a
        # direct reference to $user->{nickname}
        unless (defined $user->nickname()) {
            $user->nickname($user->first_name());
        }
    }

    return $form;
}








1;
