#!/usr/bin/perl

#
# Test file for Register3000.pm
# $Id: Register3000.001.t,v 1.3.2.3 2005/07/01 22:41:28 nico Exp $
#

use strict;
use Test::More tests=>13;

use SmokeMetadot;
use Metadot qw($PARAMS);

################################################################################
# Setup
################################################################################
# backup and clear the blacklist
use constant BLACKLIST_KEY => 'blacklisted_emails';

my $blacklist_backup = $PARAMS->{BLACKLIST_KEY()};
$PARAMS->{BLACKLIST_KEY()} = undef;


################################################################################
# Test: Make sure packages are usable
################################################################################
my $register_class = 'Metadot::Register::Register3000';
my $user_class     = 'Metadot::User::User3000';

use_ok($register_class);
use_ok($user_class);

################################################################################
# Test: Make sure registrator is instantiable
################################################################################
my $register;
ok($register = $register_class->new(), "I have register $register");


################################################################################
# Test: Register a user via the API
################################################################################
my $email = 'ipfreeley@test.foo';
# make sure that email is not already registered
while ($register->email_already_registered($email))
{
  $email .= '.foo';
}

my %input = (
    first_name  => 'I.',
    initial     => 'P.',
    last_name   => 'Freeley',
    nickname    => 'yellowdog',
    email       => $email,
    password    => '123456',
    password_confirm    => '123456',
);

ok(my $result = $register->process_registration({input => \%input}),
   "Successfully processed registration");

ok(!$result->{error}, "No errors were found");
my ($user, $uid);
ok($user = $result->{user}, "User object $user was created");
ok($uid = $user->uid(), "User has uid $uid");
ok(!$user->is_approval_needed(), "User did not require approval");

$user->delete(1);



################################################################################
# Test: Register a blacklisted user
################################################################################
my $blacklist_domain = 'evil.com';
$PARAMS->{BLACKLIST_KEY()} = $blacklist_domain;
$email = "dr_evil\@$blacklist_domain";

# make sure that email is not already registered
while ($register->email_already_registered($email))
{
  $email .= '.foo';
}

%input = (
    first_name  => 'Sid',
    last_name   => 'Evil',
    nickname    => 'dr_evil',
    email       => $email,
    password    => '123456',
    password_confirm    => '123456',
);

ok($result = $register->process_registration({input => \%input}),
   "Successfully processed registration");

ok(!$result->{error}, "No errors were found");

ok($user = $result->{user}, "User object $user was created");
ok($uid = $user->uid(), "User has uid $uid");
ok($user->is_approval_needed(), "User required approval");

$user->delete(1);


################################################################################
# Teardown
################################################################################
$PARAMS->{&BLACKLIST_KEY} = $blacklist_backup;

