###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
# this is an interface that defines a class that can generate reports.
package Metadot::Report;

use strict;

# returns the row data for the report. the return value is a hash ref,
# with the following keys:
#   - row_set:      a DBI resultset containing the actual row data
#   - row_count:    the number of rows
#   - columns:      (optional) the definitions of the columns, following the
#                   format defined in get_columns()
sub get_data {
    die("Abstract method");
}

# defines the standard columns. returns a reference to an array of hashes. Each hash
# defines a single column, and has the following keys:
#    - name:            the display name
#    - field:           the internal name
#    - sort_order:      the sort order for that column (+1 for ascending, -1 for descending)
#    - sort_priority:   the priority of sorting that column (non-zero integer to cause sorting)
sub get_columns {
    die("Abstract method");
}


# defines the maximum number of rows to display in a page. specify a positive integer
# to set a value
sub get_max_rows {
    die("Abstract method");
}

sub set_max_rows {
    die("Abstract method");
}


# defines the row number from which to begin displaying.
sub get_start_row {
    die("Abstract method");
}

sub set_start_row {
    die("Abstract method");
}


# defines the criteria for filtering the data. the format is not defined in this interface
sub get_filter {
}

sub set_filter {
}




1;
