###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Metadot::RequestCache::AuditableRequestCache;
use strict;
use vars qw(@ISA);

use Metadot qw( $USER );
use Metadot::RequestCache;

@ISA = qw(Metadot::RequestCache);


sub check_cache {
    my $class    = shift;
    my $instance = shift || die("Must specify the instance");
    my $bundle   = shift || return undef;
    my $user     = shift;

    my $key     = $class->_generate_entry_key($instance, $bundle, $user);
    return $class->SUPER::check_cache($key);
}

sub update_cache {
    my $class    = shift;
    my $instance = shift || die("Must specify the instance");
    my $bundle   = shift || return undef;
    my $user     = shift;
    my $result   = shift;

    my $key     = $class->_generate_entry_key($instance, $bundle, $user);
    return $class->SUPER::update_cache($key, $result);
}


# returns the key for a given entry in the cache. MUST RETURN UNIQUE values
# for each possible case!
sub _generate_entry_key {
    my $class = shift;
    my $instance = shift || die("Must specify the instance");
    my $bundle   = shift || return undef;
    my $user     = shift;

    my $separator = ':';
    my $instance_class  = ref($instance) || die("Could not determine class of $instance");
    my $instance_id     = ($instance->can('id') ? $instance->id() : undef );
    $instance_id = 'null_id' unless (defined $instance_id);
    my $user_id         = $user->uid() || 'null_user';

    my $result = $instance_class.$separator.$instance_id.$separator.$bundle.$separator.$user_id;
    return $result;
}


# returns the name of the cache. This MUST BE UNIQUE!
sub _get_cache_name {
    return __PACKAGE__;
}


1;
