###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Metadot::Serialization::AbstractSerializer;

=head1 NAME

Metadot::Serialization::AbstractSerializer

=head1 VERSION

Version $Revision: 1.1.2.4 $

=head1 SYNOPSIS

my $serializer = Metadot::Serialization::SerializerFactory->get_instance();
my $serialized = $serializer->serialize($object);
my $object     = $serializer->deserialize($serialized)

=head1 DESCRIPTION

AbstractSerializer defines an interface for serializing and deserializing
objects.

=head1 METHODS

=cut


use strict;


=item C<new>

Constructor. You should use the factory method instead.

Metadot::Serialization::SerializerFactory->get_instance()

=cut
sub new
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self  = {};
    bless $self, $class;

    return $self;
}


=item C<serialize>

  my $data = $serializer->serialize($object);

Convert an object into a portable datum. (abstract)

=cut
sub serialize
{
    my $self   = shift;
    my $object = shift || return undef;
    ref($object) || die('Must send an object reference');

    die('Abstract method');
}


=item C<deserialize>

Restore an object that was previously serialized. (abstract)

  my $object = $serializer->deserialize($data);

=cut
sub deserialize
{
    my $self = shift;
    my $data = shift || return undef;

    die('Abstract method');
}











1;




