###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Metadot::Serialization::SerializerFactory;

=head1 NAME

Metadot::Serialization::SerializerFactory

=head1 VERSION

Version $Revision: 1.1.2.4 $

=head1 SYNOPSIS

my $serializer = Metadot::Serialization::SerializerFactory->get_instance();
my $serialized = $serializer->serialize($object);
my $object     = $serializer->deserialize($serialized)

=head1 DESCRIPTION

SerializerFactory returns an instance of the portal's preferred Serializer
object.

=head1 METHODS

=cut

use strict;

use constant DEFAULT_CLASS => 'Metadot::Serialization::XMLDumper';
#use constant DEFAULT_CLASS => 'Metadot::Serialization::DumpXML';
#use constant DEFAULT_CLASS => 'Metadot::Serialization::Storable';

use ErrorHandling;

=item C<get_instance>

  Metadot::Serialization::SerializerFactory->get_instance();
  Metadot::Serialization::SerializerFactory->get_instance($sample);

Returns an instance of AbstractSerializer. If a sample is provided, it will
attempt to determine the correct serializer.

=cut

sub get_instance
{
    my $class  = shift;
    my $sample = shift;

    my $target_class;
    my $result;

    # Determine compatibility with the sample
    if (defined $sample)
    {
        if ($sample =~ m/DumpXML/) {
            $target_class = 'Metadot::Serialization::DumpXML';
        }
        elsif ($sample =~ m/perldata/)
        {
            $target_class = 'Metadot::Serialization::XMLDumper';
        }
        else
        {
            $target_class = 'Metadot::Serialization::Storable';
        }
    }

    $target_class ||= DEFAULT_CLASS;
    #print STDERR "Serializer class is '$target_class'\n";

    my $path = Utility->package_to_filename($target_class);

    eval
    {
        require $path;
        $result = $target_class->new();
    };

    if ($@) {
        ErrorHandling->printError($@);
    }

    return $result;
}








1;




