#######################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Metadot::Services::GizmoService;


=head1 NAME

Metadot::Services::GizmoService

=head1 VERSION

Version $Revision: 1.1.2.3 $

=head1 DESCRIPTION

GizmoService is intended to be the backing class for a web service. See soap.pl
for the implementation.

=cut


use strict;
use Metadot::PortalFactory;
use Metadot qw($DISPLAY);



=item C<ping>

  my $result = Metadot::Services::GizmoService->ping();

Always returns a 1. Used to test whether a service is connected.

=cut
sub ping {
  print STDERR "ping\n";
  return 1;
}


=item C<user_is_allowed_to_do>

  my $result = Metadot::Services::GizmoService->user_is_allowed_to_do($userid, $instance_id, $op);

Determines whether a given userid is permitted to perform $op against a certain gizmo instance.
If no $op provided, 'show' is the default.

=cut
sub user_is_allowed_to_do {
  my ($class, $userid, $iid, $op) = @_;

  #print STDERR "user_is_allowed: userid=$userid, iid=$iid, op=$op\n";
  my $result = undef;

  #print STDERR "Display: $DISPLAY\n";

  if (defined $userid && defined $iid) {
    $op ||= 'show';

    #print STDERR "Getting user $userid\n";
    my $user     = Metadot::PortalFactory->get_user($userid);
    #print STDERR "Got user $user\n";

    #print STDERR "Getting instance $iid\n";
    my $instance = Metadot::PortalFactory->get_gizmo_object($iid);
    #print STDERR "Got instance $instance\n";

    if (defined $user && defined $instance) {
      #print STDERR "Checking permissions...\n";
      $result = $instance->is_allowed_to_do($op, $user, 0, 1);
      #print STDERR "Got result: $result\n";
    }
  }

  return $result;
}


1;
