###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::SessionHandler;

use DBAccess;


sub new {
    my $proto = shift;
    my $class = ref{$proto} || $proto;
    my $self = {};

    bless ($self, $class);
    return $self;
}


sub persist_session {
    my $self = shift;
    die "abstract class, must override";
}


sub delete_session {
  die "abstract, must be overridden";
}

sub restore_session {
  die "abstract, must be overridden";
}


1;


__END__



=head1 NAME

Metadot::SessionHandler - Abstract class that defines an interface for session handler classes.


=head1 SYNOPSIS


  my $session_handler = Metadot::SessionHandlerFactory->get_session_handler();

  $session_handler->persist_session($session);

  $session_handler->restore_session($session);


=head1 DESCRIPTION

This class defines the interface that session handling classes must implement. This class alongside with Metadot::SessionHandlerFactory provides a framework to allow interchangeable session handling strategies in Metadot.

=head1 METHODS

persist_session

Will save session id information so that Session objects can later be restored across HTTP requests.

restore_session

Will retrieve session id information and use it to restore and return Session objects.

