###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::SessionHandler::CookieSessionHandler;
use strict;

use DBAccess;
use Metadot::SessionHandler;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CACHE $CGI);

use vars qw(@ISA);
@ISA=qw(Metadot::SessionHandler);

sub new {
    my $proto = shift;

    my $class = __PACKAGE__;

    my $self = {};

    bless ($self, $class);
    return $self;
}


sub persist_session {
    my $self = shift;
    my $session = shift;

    my $session_id = $session->session_id();
#    my $uid = $session->uid();

    # writing a new cookie
    $self->write_cookie($session_id);

}


sub write_cookie {
    my $self = shift;
    my $session_id = shift;

    my $cookie_name     = $self->session_cookie_name;
    my $cookie_domain   = MetadotConfig->get_param_value('cookie_domain');
    my $cookie_path     = MetadotConfig->get_param_value('cookie_path') || '/';
    my $cookie_expires  = MetadotConfig->get_param_value('cookie_expires');
    (defined $cookie_expires) || ($cookie_expires = '+6M');

    #print STDERR "Cookie info: value[$session_id], name[$cookie_name], domain[$cookie_domain], path[$cookie_path], expires[$cookie_expires]\n";

    # Creating  multiple cookies.
    my $cookie1 = $CGI->cookie(
                    -name      => $cookie_name,
                    -value     => $session_id,
                    -path      => $cookie_path,
                    -expires   => $cookie_expires,
                    -domain    => $cookie_domain,
                    );

    my %hash = (-cookie=>[$cookie1]);

    # [vadim] now we can use this component instead of instanciating Portal
    $DISPLAY->print_http_header(\%hash);
}


sub delete_session {
    my $self = shift;
    my $session = shift;

    $session->erase();

    # write empty cookie
    $self->write_cookie( "" );

}


sub restore_session {
    my $self = shift;

    my $session_id = shift || $self->read_cookie();

    my $session;

    if( $session_id ) {

    $session = Session->restore($session_id);
    }

    return $session;
}


sub read_cookie {
  my $self= shift;

  my $cookie_name = $self->session_cookie_name();
  my $session_id = $CGI->cookie($cookie_name);

  #print STDERR "Read cookie [$cookie_name], got [$session_id]\n";


  return $session_id;

}


sub session_cookie_name {
    my $class = shift;
    return MetadotConfig->get_param_value('cookie_name') || 'sessionid';
}


1;




