###########################################################################
# <LICENSEINFO>
# Metadot SmartSite Server                                                                      
# Copyright (C) 2001 Metadot Corporation                                                        
#                                                                                               
# For more information go to                                                                    
# http://www.metadot.com                                                                        
# or contact us at info@metadot.com                                                             
#                                                                                               
#</LICENSEINFO>
############################################################################
package Metadot::SessionHandler::HeaderBasedSessionHandler;
use strict;

use Data::Dumper;
use DBAccess;
use Metadot qw($USER %FORM $SESSION $CGI $DISPLAY);

use Metadot::SessionHandler::CookieSessionHandler;
use Metadot::User::FlexUser;
use Metadot::AuthenticatorFactory;

use vars qw(@ISA);
@ISA=qw(Metadot::SessionHandler::CookieSessionHandler);

# specify the values in metadot.conf which are necessary to authenticate a user.
use constant AUTHENTICATION_HEADER_VARIABLE_PARAM => 'authentication_header_variable';

sub new {
    my $proto = shift;

    my $class = __PACKAGE__;

    my $self = {};

    bless ($self, $class);
    return $self;
}


sub restore_session {
    my $self = shift;

    # determine the header variable which is used to get the user identifier
    my $header_variable = MetadotConfig->get_param_value(AUTHENTICATION_HEADER_VARIABLE_PARAM) || die("Must specify ".AUTHENTICATION_HEADER_VARIABLE_PARAM." in metadot.conf");

    # Read the header variable
    my $authentication_header_value =  $CGI->http($header_variable);        
    #print STDERR "Found ".AUTHENTICATION_HEADER_VARIABLE_PARAM."='$authentication_header_value'\n";

    my $uid;
    if( defined $authentication_header_value ) {	
        # look up the user based on the ldap_uid
        $uid = $USER->get_uid_from_ldap_uid( $authentication_header_value );
        #print STDERR "Got user id '$uid'\n";
    } else {
        my @header_vars = $CGI->http();
        #print STDERR "This request DOES NOT contain a $header_variable header variable.Current header variables are:\n",Dumper \@header_vars;
    }

    # check for an existing session id
    my $session_id = shift || $self->read_cookie();
    my $session;
    
    if( $session_id ) {
        # restore the existing session
        $session = Session->restore($session_id);
        #print STDERR "Session $session was restored based on session id '$session_id'\n";
    }

    if (defined $uid) {
        # Verify that the session uid in place corresponds to the user's LDAP DN
        if (defined($session) && $session->{uid} != $uid ) {
            #print STDERR "Session found was invalid; uid '$uid' did not match session uid '$session->{uid}'!\n";
            $session = undef;
        }
        
        # Verify the session is not expired
        if (defined($session) && $session->is_expired()) {
            #print STDERR "Session is expired.\n";
            $session->erase();
            $session = undef;
        }

        # no existing session id, or session was not usable
        unless (defined $session) {
            #print STDERR "Creating new session\n";
            $session = Session->new();
            $session->add($uid);	    
            $session_id = $session->session_id();
            #print STDERR "Session ID='$session_id' was created.\n";
            $self->write_cookie( $session_id );
        }
    } elsif (defined $session) {
        # if no user id is defined, and the session has a valid user id,
        # then a different person is logged in who isn't registered.
        if ($session->{uid} != MetadotConfig->get_reserved_uid('ANONYMOUS_USER')) {
            #print STDERR "An anonymous user logged in, in place of a registered user.\n";
            $session = undef;
        }
    }


    # still no session id, will use the anonymous user
    unless (defined $session) {
        # determine the Authenticator type
        my $authenticator = Metadot::AuthenticatorFactory->get_authenticator();

        # create the anonymous session
        $session = $authenticator->create_anonymous_session_and_user();
    }

    #print STDERR "Final session id: $session_id\n";

    #use Data::Dumper;
    #print STDERR "SESSION=========\n",Dumper($session);

    return $session;
}


1;




