###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::SessionHandlerFactory;
use strict;

use DBAccess;

my %handlers;
sub get_session_handler {

#     print STDERR "AT Metadot::SessionHandlerFactory 1\n";
    my $handler_type = MetadotConfig->get_sessionhandler_type();

#     print STDERR "AT Metadot::SessionHandlerFactory $handler_type\n";
    if ($handlers{$handler_type}) {

        return $handlers{$handler_type};

    } else {

        require "Metadot/SessionHandler/$handler_type.pm";

        my $handler = "Metadot::SessionHandler::$handler_type"->new();

        $handlers{$handler_type} = $handler;

        return $handler;
    }
}

1;

__END__


=head1 NAME

Metadot::SessionHandlerFactory - Middleman class to provide instances of different Metadot::SessionHandler subclasses depending on system configuration.

=head1 SYNOPSIS

  my $session_handler = Metadot::SessionHandlerFactory->get_session_handler();


=head1 DESCRIPTION

Middleman class to provide instances of different Metadot::SessionHandler subclasses depending on configuration. This class relies on method get_session_handler in MetadotConfig, which reads in the value of the variable "session_handler_type" in metadot.conf (the config file), from where different SessionHandler classes can be selected. Note that if no session_handler_type is supplied in the config file, CookieSessionHandler will be used as default.

=head1 METHODS

get_session_handler

Returns an instance of a Metadot::SessionHandler subclass depending on system configuration.
