###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Status;

use strict;

use DBAccess;
use Metadotclass;
use Auditable;
use vars qw(@ISA);
use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CGI);
use File::Find;

@ISA=qw(Metadotclass Auditable);


my $AC = {
    BUNDLES => {
        ADMIN => {
            DESC =>                                        'View Admin',
            OPS =>           [qw(show_env show_monitor show_versions)],
            DEFAULT_LEVEL => 9,
            MIN_LEVEL =>                        9,
            DEFAULTS_TO =>   'on',
        },
    },
    DISPLAY_ORDER => [qw(ADMIN)],
};

sub get_AC {
    return $AC;
}

sub is_a {
    my $self = shift;
    my $class_ref = ref($self);
    return $class_ref;

}

sub id {
    return 1;
}

sub save {
    my $self = shift;

    $self->save_permissions;
}

sub get_owner_uid {
    my $self = shift;

    MetadotConfig->get_reserved_uid('ADMIN');
}




sub new {
    my $proto = shift;
    my $class = __PACKAGE__;
    my $self = {};

    bless ($self, $class);
    return $self;
}

my %Gizmos = "";

sub www_show_versions {
    my $self = shift;
    my $content = "";
    find(\&wanted, 'Gizmo');
    $content .= "<table align = center><tr><td><H2>Gizmo</H2></td><td>&nbsp&nbsp;&nbsp;</td><td><H2>Version</H2></td></tr>";

    foreach my $file (sort keys %Gizmos) {
        $content .= "<tr><td>$file</td><td></td><td>$Gizmos{$file}</td></tr>";
    }
    $content .= "</table><br /><br />";
    $DISPLAY->print_form($content, "The gizmos");

}

sub wanted {
    my $file = $_;
    return if ($file =~ m/\A#/);
    return unless ($file =~ m/pm\Z/);
    my $version = "";

    eval {
      require $file;
      $file =~ s/.pm\Z//;
      $version = "$file"->get_version();
      $Gizmos{$file} = $version;
    };
    if ($@) {
    print STDERR "$@ \n";
    }
}


sub www_show_env {
    my $self = shift;
    $self->debug_msg(3, "Portal::www_show_env()");

    my $content =  "<H1>Environment variables</H1>\n" . `date` . "</b><br /><br />\n";
    map {
        my $value = $ENV{$_} ;
        $value = $CGI->escapeHTML($value) ;
        $content .=  "$_ = $value <br />\n"
    } keys %ENV ;
    $content .= "<br /><br />";
    $DISPLAY->print_form($content, "Environment Variables");

}

sub www_show_monitor {
    my $self = shift;
    $self->debug_msg(3, "Portal::www_show_monitor()");

    $DISPLAY->print_form($self->print_system_snap_shot(), "System Snapshot");

}



##################################################
### print_system_snap_shot
### display parameter of current machine
##################################################
sub print_system_snap_shot {
    my $self = shift;
    $self->debug_msg(3, "Portal::print_system_snap_shot()");

    # The list of commands to display.
    # Format is title followed  by the command to execute.
    my $content = $self->command ("Memory Information", "\/usr\/bin\/free");
    $content .= $self->command ("Mounted", "\/bin\/df");
    $content .= $self->command ("Uptime", "\/usr\/bin\/uptime");
    $content .= $self->command ("Top", "\/usr\/bin\/top\ -b\ -n\ 1");
    $content .= $self->command ("Processes", "\/bin\/ps\ aux");
    $content .= $self->command ("Network Ports", "\/bin\/netstat\ -a");
    $content .= $self->command ("Network Condition", "\/bin\/netstat\ -i");
    $content .= $self->command ("Network Interfaces", "\/sbin\/ifconfig");
    $content .= $self->command ("Network Routes", "\/sbin\/route");
    $content .= $self->command ("Boot Info", "\/bin\/dmesg");
    $content .= $self->command ("Finished!", "");
    return $content;

}

### print_system_snap_shot sub-function
sub command {
    my $self = shift;
    my $line;

    # Print out the title.
    my $content = "<h2> $_[0] </H2>\n";

    if ($_[1]) {
        # Print out the result of the command.
        $content .= "<PRE>\n";
        open(INPUT, "$_[1] |") or die "$_[0] failed.";
        while ($line=<INPUT>) {
            # Replace < and > with a * to not confuse the html interpreter.
            $line =~ s/</\&lt/g;
            $line =~ s/>/\&gt/g;
            $content .=  $line;
        }
        $content .= "</PRE>\n";
        close (INPUT);
        return $content;
    }
}



=head1 NAME

Status -

=head1 SYNOPSIS


    use Status;
    $status = Metadot::Status->new() ;

    $status->www_show_env;
    $status->www_show_monitor;
    $status->www_show_versions;




=head1 DESCRIPTION

The status class displays information about the system, the environment variables, and the Gizmos loaded in the system.

Constructor.  Creates an instance of Metadot::Status

    $status = Metadot::Status->new() ;

Display the CGI environment variables:

    $status->www_show_env;

Display the System Status.  This is the System on which the metadot portal is running.

    $status->www_show_monitor;

Display the version numbers of all of the gizmos in the system.  If a gizmo does not have a version number, the version number in the Gizmo
superclass is called, which is defined as "No version information available"

    $status->www_show_versions;

=cut
