###########################################################################
# <LICENSEINFO>
# Metadot SmartSite Server                                                                      
# Copyright (C) 2001 Metadot Corporation                                                        
#                                                                                               
# For more information go to                                                                    
# http://www.metadot.com                                                                        
# or contact us at info@metadot.com                                                             
#                                                                                               
#</LICENSEINFO>
############################################################################

# An interface to specify that a gizmo supports channels and subscriptions.

package Metadot::Subscribable;

use strict;
use Metadot qw($ACCESS_BROKER $USER %FORM $DISPLAY);
use DBAccess;
use Channel;
use Subscription;


sub customize_permissions {
    my ($self, $ac) = @_;

    # Set the broadcast flag on because this feature can be used to spam members

    # add/remove subscribers
    $ac->add_bundle('MANAGE_SUBSCRIPTIONS', 'Manage Subscriptions', '9', '8');
    $ac->set_broadcast('MANAGE_SUBSCRIPTIONS', 1);

    $ac->add_op('MANAGE_SUBSCRIPTIONS','view_subscribers');
    $ac->add_op('MANAGE_SUBSCRIPTIONS','update_subscribers');
    $ac->add_op('MANAGE_SUBSCRIPTIONS','add_subscribers');

    # add/remove subscriber groups
    $ac->add_op('MANAGE_SUBSCRIPTIONS', 'view_subscriber_groups');
    $ac->add_op('MANAGE_SUBSCRIPTIONS', 'add_subscriber_groups');
    $ac->add_op('MANAGE_SUBSCRIPTIONS', 'update_subscriber_groups');

    return $ac;
}


sub get_channel_object {
    my $self = shift;
    my $result = $self->{_channel_object};  # need to make sure this does not conflict with caching

    unless ($result) {
        $result = Channel->get_channel_for_iid($self->get_id());
        $self->{_channel_object} = $result;
    }
    return $result;
}


sub exec_view_subscribers {
    my ($self, $context) = @_;
    return 'subscribable/view_subscribers.tt2';
}


sub exec_update_subscribers
{
    my ($self, $context) = @_;
    Subscription->do_subscribe_users();

    # redirect back to view subscribers
    $DISPLAY->md_redirect("index.pl?iid=".$self->id()."&op=view_subscribers");
}


sub exec_add_subscribers {
    my ($self, $context) = @_;

    return 'subscribable/add_subscribers.tt2';
}


sub exec_add_subscriber_groups {
    my ($self, $context) = @_;

    return 'subscribable/add_subscriber_groups.tt2';
}

sub exec_update_subscriber_groups {
    my ($self, $context) = @_;

    $DISPLAY->debug_msg(3, "exec_update_subscriber_groups");

    Subscription->do_subscribe_groups($self->id(), \%FORM);

    # redirect back to view subscribers
    $DISPLAY->md_redirect("index.pl?iid=".$self->id()."&op=view_subscribers");
}







# wrapper around Subscription->get_subscriber_names()
sub get_subscribers {
    return Subscription->get_subscriber_names();
}

# wrapper around Subscription->get_user_subscription_details()
sub check_subscription {
    my $self   = shift;
    my $userid = shift;

    return Subscription->get_user_subscription_details({ iid=>$self->id(), uid=>$userid });
}

# wrapper around Subscription->check_icq_or_pager_subscription()
sub check_icq_or_pager_subscription {
    my $self    = shift;
    my $userid  = shift;
    my $type    = shift;

    return Subscription->check_icq_or_pager_subscription($self->id(),$userid,$type);
}

# wrapper around Subscription->get_user_names_and_ids()
sub search_user_names {
    my $self = shift;
    my $search = shift;
    return Subscription->get_user_names_and_ids($search);
}













# wrapper around the same method in Subscription
sub get_subscriber_groups {
    my $self = shift;
    return Subscription->get_subscriber_groups({ iid => $self->id()});
}

# wrapper around the same method in Subscription
sub get_group_subscriptions {
    my $self = shift;
    my $args = shift || {};
    return Subscription->get_group_subscriptions({%$args, iid => $self->id()});
}


# wrapper around the same method in Subscription
sub search_groups {
    my $self = shift;
    my $name = shift;
    return Subscription->search_groups($name, $self->id());
}





1;
