#!/usr/bin/perl -w
#
###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::SuperGizmoUploadsManager;
use strict;

use Metadot qw($PARAMS $DISPLAY %REQUEST_VARS);
use Metadot::SkinManager;
use Metadot::UploadsManager;
#use Metadot::MetadotController;
use Metadotclass;
use Gizmo::SuperGizmo;

use vars qw(@ISA);
@ISA=qw(Metadot::SkinManager Metadotclass);

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $self  = $class->SUPER::new();

    bless ($self, $class);

    return $self;
}

# returns the list of image filenames
sub get_image_file_list {
    my $class = shift;
    my $params = shift;
    $class->debug_msg(4, "SuperGizmoUploadsManager::get_filelist_for_type()");

    my @files = ();
    my $directory = $class->get_image_path($params);
    $class->debug_msg(4, "SuperGizmoUploadsManager::get_image_file_list() directory:$directory");
    return unless defined($directory);

    foreach my $file ($class->_get_directory_files($directory)) {
        unless (-d "$directory/$file") {
            push @files, $file;
            $class->debug_msg(4, "SuperGizmoUploadsManager::get_image_file_list() adding image:$file to list");
        }
    }

    if (wantarray) {
        return @files;
    } else {
        return \@files;
    }
}

sub get_image_path {
    my $class = shift;
    my $params = shift;
    $class->debug_msg(4, "SuperGizmoUploadsManager::get_image_path()");


    my $path = $class->get_supergizmo_dir_path($params);
    $class->debug_msg(4, "SuperGizmoUploadsManager::get_image_path() supergizmo_dir_path:$path");
    return unless defined($path);

    $path .= "/images";
    # check that the path exists
    if ($params->{create_path} && !$params->{want_web_path} && ! -d $path) {
        $class->debug_msg(4, "SuperGizmoUploadsManager::get_image_path() mkdir path:$path");
        mkdir($path, 0755);
    }

    return $path;
}

sub get_supergizmo_dir_path {
    my $class = shift;
    my $params = shift;
    $class->debug_msg(4, "SuperGizmoUploadsManager::get_supergizmo_dir_path()");

    my $supergizmo = $params->{supergizmo};
    $class->debug_msg(4, "SuperGizmoUploadsManager::get_image_path() supergizmo exists:" . defined($supergizmo));

    return unless defined($supergizmo);

    my $short_name = $params->{short_name} || $supergizmo->get_short_name();
    $class->debug_msg(4, "SuperGizmoUploadsManager::get_base_dir_path() short_name:$short_name");
    return unless defined($short_name) && $short_name ne '';

    my $base_dir = $class->get_base_dir_path($params);
    $class->debug_msg(4, "SuperGizmoUploadsManager::get_image_path() base_dir:$base_dir");
    return unless defined($base_dir);

    my $path = "$base_dir/$short_name";
    # check that the path exists
    if ($params->{create_path} && !$params->{want_web_path} && ! -d $path) {
        $class->debug_msg(4, "SuperGizmoUploadsManager::get_image_path() mkdir path:$path");
        mkdir($path, 0755);
    }

    return $path;
}

sub get_base_dir_path {
    my $class = shift;
    my $params = shift;
    $class->debug_msg(4, "SuperGizmoUploadsManager::get_base_dir_path()");
    my $supergizmo = $params->{supergizmo};

    my $base_dir = $class->get_public_location($params) . '/supergizmo';

    # check that the base_dir exists
    if ($params->{create_path} && !$params->{want_web_path} && ! -d $base_dir) {
        $class->debug_msg(4, "SuperGizmoUploadsManager::get_base_dir_path() mkdir base_dir:$base_dir");
        mkdir($base_dir, 0755);
    }

    return $base_dir;
}

sub get_public_location {
    my $class  = shift;
    my $params = shift;
    $class->debug_msg(4, "SuperGizmoUploadsManager::get_public_location()");

    my $loc = '';
    if ($params->{want_web_path}) {
       $loc = MetadotConfig->get_public_uploads_location();;
    } else {
       $loc = MetadotConfig->get_public_uploads_dir();
    }

    return $loc;
}

sub _get_directory_files {
    my $class = shift;
    $class->debug_msg(4, "SuperGizmoUploadsManager::_get_directory_files()");
    my $directory = shift || die("Must specify directory");
    $class->debug_msg(4, "SuperGizmoUploadsManager::_get_directory_files() directory:$directory");

    opendir DIRECTORY, $directory;
    #my @files = (grep !/^\.\.?$/, readdir DIRECTORY);
    # trying new logic, which will also suppress .filename
    my @files = (grep !/^\./, readdir DIRECTORY);
    closedir DIRECTORY;

    if (wantarray) {
        return @files;
    } else {
        return \@files;
    }
}

# upload a single image
sub upload_image_file {
    my $class = shift;
    my $args = shift || {};
    $class->debug_msg(4, "SuperGizmoUploadsManager::upload_image_file()");

    my $name    = $args->{name}    || die("Must specify arg name");
    $class->debug_msg(4, "SuperGizmoUploadsManager::upload_image_file() name:$name");

    my $supergizmo = $args->{supergizmo};
    $class->debug_msg(4, "SuperGizmoUploadsManager::upload_image_file() supergizmo exists:" . defined($supergizmo));
    return unless $supergizmo;

    $args->{create_path} = 1;
    my $path = $class->get_image_path($args);
    $class->debug_msg(4, "SuperGizmoUploadsManager::upload_image_file() path:$path");

    my $result = Metadot::UploadsManager->upload_file(
        source_file => $name,
        write_dir   => $path,
        overwrite   => 1,
    );
    $class->debug_msg(4, "SuperGizmoUploadsManager::upload_image_file() result:$result");

    return $result;
}

sub upload_supergizmo_import_file {
    my $class = shift;
    my $args = shift || {};
    $class->debug_msg(4, "SuperGizmoUploadsManager::upload_supergizmo_import_file()");
    my $filepath = $args->{filepath}    || die("Must specify filepath");
    $class->debug_msg(4, "SuperGizmoUploadsManager::upload_supergizmo_import_file() filepath:$filepath");

    if ($filepath =~ m/\.zip$/) {
        my $filename = Metadot::UploadsManager->get_filename_from_path($filepath);
        my $base_dir = $class->get_base_dir_path({ create_path => 1 });
        $class->debug_msg(4, "SuperGizmoUploadsManager::upload_supergizmo_import_file() base_dir:$base_dir");

        my $buff = '';

        my $write_file = $base_dir . "/" . $filename;
        my $size = 0;

        ### moving the file from tmp dir to final location
        if (!open(WFD,">$write_file")) {
            return { error => "Error uploading file $filename, cannot write to disk. Please contact your site administrator."};
        }

        while ( my $bytes_read = read( $filepath, $buff, 2096 ) ) {
            # while (read($filepath,$buff,2096)){
            $size += $bytes_read;
            binmode WFD;
            print WFD $buff;
        }
        close(WFD);

        # expand the file as a directory
#        my $skin_name = $filename;
        my $supergizmo_name = $filename;

#        if (Metadot::SkinManager->get_instance()->is_read_only_skin($skin_name)) {
#            $DISPLAY->print_click_back('You have attempted to replace a read-only skin. Please rename the skin before uploading.');
#            exit;
#        } else {
            $supergizmo_name =~ s/\.zip//;
#            my $result = Metadot::SkinManager->get_instance()->import_zip($skin_name, $write_file);
            my $result = $class->import_zip({ write_file => $write_file, name => $supergizmo_name, overwrite => $args->{overwrite} });
            if ($result) {
                return { error => "An error occurred while creating a supergizmo from your zip file: $result" };
            }
#        }
    } else {
        return { error => "File must be in .zip format."};
    }

    return '';
}

# creates a supergizmo from a zip file
sub import_zip {
    my $class = shift;
    my $args = shift || {};
    $class->debug_msg(4, "SuperGizmoUploadsManager::import_zip()");
#    my $skin_id = shift || die("Must supply a skin id");
    my $name       = $args->{name} || die("Must supply a name");
    my $write_file = $args->{write_file} || die("Must supply a ZIP file");
    my $overwrite  = $args->{overwrite} || 0;
    $class->debug_msg(4, "SuperGizmoUploadsManager::import_zip() name:$name write_file:$write_file");

    #print STDERR "import_zip begins with file '$write_file'\n";

    # check if read-only
#    !$class->is_read_only_skin($skin_id) || return("Cannot delete overwrite read-only skin");

#    my $template_dir = $class->get_skin_template_dir();
#    my $skin_dir = $class->get_skin_path($skin_id);

#    #print STDERR "target skin_dir is '$skin_dir'\n";

    my $zip = Archive::Zip->new($write_file);

    #print STDERR "I have zip object '$zip'\n";

    my $zip_xml_file = $class->get_xml_zip_name({ name => $name });
    my $xml_member = $zip->memberNamed( $zip_xml_file );

    my $xml_descriptor = $xml_member->contents();
    return "Bad xml file" unless $xml_descriptor;

    my $temp_supergizmo = SuperGizmo->new();
    $temp_supergizmo->set_descriptor($xml_descriptor);
    $temp_supergizmo->_initialize_supergizmo();
    my $temp_supergizmo_short_name = $temp_supergizmo->get_short_name();
    ## returns iid if exists
    my $supergizmo_exists = SuperGizmo->does_supergizmo_exist({ short_name => $temp_supergizmo_short_name });


    ## determine if supergizmo already exists
    !$overwrite && $supergizmo_exists && return("Cannot overwrite existing supergizmo.");

    my $supergizmo_id;
    if ($supergizmo_exists) {
        my $existing_supergizmo = SuperGizmo->new($supergizmo_exists);
        $supergizmo_id = $existing_supergizmo->update_supergizmo( { descriptor => $xml_descriptor } );
    } else {
        $supergizmo_id = SuperGizmo->add_supergizmo( { descriptor => $xml_descriptor } );
    }

    return "Supergizmo has no id." unless $supergizmo_id;
    $class->debug_msg(4, "SuperGizmoUploadsManager::import_zip() created supergizmo with id $supergizmo_id");
    ## reload
    my $new_supergizmo =SuperGizmo->new($supergizmo_id);

    my $zip_images_path = $class->get_images_zip_name({ name => $name });
    my $image_path = $class->get_image_path({ supergizmo => $new_supergizmo, create_path => 1 });
    my $result = $zip->extractTree($zip_images_path, $image_path);
    $class->debug_msg(4, "SuperGizmoUploadsManager::import_zip() extracted supergizmo images with result:$result");
    #print STDERR "import_zip returns with result '$result'\n";

    $class->_collect_garbage_files();

    return $result;
}

# delete an image
sub delete_file {
    my $class = shift;
    my $args = shift || {};

    my $name    = $args->{name}    || die("Must specify arg name");
    $class->debug_msg(4, "SuperGizmoUploadsManager::delete_file() name:$name");

    my $supergizmo = $args->{supergizmo};
    $class->debug_msg(4, "SuperGizmoUploadsManager::delete_file() supergizmo exists:" . defined($supergizmo));
    return unless $supergizmo;

    my $image_path = $class->get_image_path($args) . "/$name";
    $class->debug_msg(4, "SuperGizmoUploadsManager::delete_file() delete file path:$image_path");
    return unless $image_path;

    unlink($image_path);
}

sub get_file_url {
    my $class = shift;
    my $args  = shift;
    $class->debug_msg(4, "SuperGizmoUploadsManager::get_file_url()");

    my $name    = $args->{name}    || die("Must specify arg name");
    $class->debug_msg(4, "SuperGizmoUploadsManager::get_file_url() name:$name");

    $args->{want_web_path} = 1;
    my $directory = $class->get_image_path($args);
    $class->debug_msg(4, "SuperGizmoUploadsManager::get_image_path() directory:$directory");

    return unless $directory;

    return "$directory/$name";
}

# converts the named skin to a zip file. Returns the name of the file that was created.
sub export_zip {
    my $class = shift;
    my $params = shift;

    $DISPLAY->debug_msg(4, "SuperGizmoUploadsManager::export_zip()");

    my $file = $params->{file}|| die("Must specify the desired file name");
    my $name = $params->{name}|| die("Must specify the desired name");
    my $supergizmo = $params->{supergizmo}|| die("Must provide a supergizmo");
    $DISPLAY->debug_msg(4, "SuperGizmoUploadsManager::export_zip() file:$file() name:$name");

    my $result;

    $DISPLAY->debug_msg(4, "Exporting file '$file'");

    # CKIM: This was an odd place to put the path, but in any case it fails
    #       when we download a supergizmo unless we have uploaded one first!
    #       Forcing 'create_path' fixes the problem.
    $params->{create_path} ||= 1;
    my $supergizmo_base_dir = $class->get_base_dir_path($params);

    my $image_path = $class->get_image_path($params);

    $DISPLAY->debug_msg(4, "Base dir is '$supergizmo_base_dir'");

    if (-e $supergizmo_base_dir ) {
        my $read_file = $supergizmo_base_dir . '/' . $file;

        my $zip_images_path = $class->get_images_zip_name({ name => $name });
        my $zip_xml_file = $class->get_xml_zip_name({ name => $name });

        # try to create a .zip file
        #print STDERR "Creating zip $read_file, with directory $image_path\n";
        my $zip = Archive::Zip->new();
        $zip->addTree($image_path, $zip_images_path);
        $zip->addString( $supergizmo->get_descriptor(), $zip_xml_file );
        $zip->writeToFileNamed($read_file);
        $class->_collect_garbage_files($params);
        $result = $read_file;
    } else {
        $result = undef;
    }

    $DISPLAY->debug_msg(4, "Export completes with result '$result'");

    return $result;
}

sub get_xml_zip_name {
    my $class = shift;
    my $params = shift;
    $class->debug_msg(4, "SuperGizmoUploadsManager::get_xml_zip_name()");
    my $name = $params->{name}|| die("Must specify the desired name");
    return $name . '/xml/main.xml';
}

sub get_images_zip_name {
    my $class = shift;
    my $params = shift;
    $class->debug_msg(4, "SuperGizmoUploadsManager::get_xml_zip_name()");
    my $name = $params->{name}|| die("Must specify the desired name");
    return $name . '/images';
}

# deletes all zipped files that are older than 24 hours
sub _collect_garbage_files {
    my $class = shift;
    my $params = shift;
    $class->debug_msg(4, "SuperGizmoUploadsManager::_collect_garbage_files()");

    my $path = $class->get_base_dir_path($params);
    opendir SCRATCHDIR, $path;

    # the grep filters out . and ..
    my @file_list = grep !/^\.\.?$/, readdir SCRATCHDIR;

    closedir SCRATCHDIR;

    foreach my $file_base (@file_list) {
        if ($file_base =~ m/\.zip%/) {
            my $file_name = $path.'/'.$file_base;
            $^T = time; # reset script start time to allow easy file age comparison
            my $file_age = (-M $file_name);
            if ($file_age > 1) {
                unlink $file_name;
            }
        }
    }
}

1;
