###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::SystemApp;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY);
use Utility;
use MetadotConfig;


use Carp;

use vars qw(@ISA);

@ISA = qw(Portal Auditable);

#####################################################
######               INITIALIZE               #######
#####################################################


### This function provides a default name for the Application
sub get_app_name {
  my $self = shift;
  return $self;
}

sub get_gizmo_name {
  my $self = shift;
  return $self->get_app_name();
}
###
### ACCESS CONTROL METHODS (Auditable Interface Methods)
###

sub is_a {
    my $self = shift;

    die "Must be overridden to return proper value";

}

sub get_is_a {
    my $self = shift;
    my $result;
    if (ref($self)) {
        $result = $self->is_a();
    }
    return $result;
}


sub name {
    my $self = shift;
    my $class = ref($self);

    die "&name should be overridden in class $class"
    unless ($class eq __PACKAGE__);

    confess "SystemApp.pm is a base class and should not be instantiated by itself.";
}


sub id {
    my $self = shift;

    # we return 1, see docs for Auditable.pm
    return 1;
}


#
# XXX this one is not documented!!
#
sub get_iid {
    my $self = shift;

    # we return 1, see docs for Auditable.pm
    return 1;
}



# should always be overridden in derived classes
sub get_AC {
    my $self = shift;
    my $class = ref $self || $self;

    die "&get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

    confess "SystemApp.pm is an abstract class, do not instantiate";
}


sub parent_id {
    my $self = shift;

    # no parent therefore it returns 0
    return 0;
}


sub go_back_to_parent_after_editing_permissions {
    my $self = shift;

    return 0;
}


sub save {
    my $self = shift;

    $self->save_permissions();
}


sub uid {
    my $self = shift;

    return MetadotConfig->get_reserved_uid('ADMIN');
}

# added for recursive extensions compatibility
sub my_child_count {

    return 0;
}

###
###  CONSTRUCTOR
###
###############
sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $self  = $class->SUPER::new();

    bless ($self, $class);

    return $self;
}


###############
sub www_show {
    my $self = shift;

    $self->print_portal($self->main());
}

sub print_portal {
    my $self = shift;
    my $content = shift;

    $DISPLAY->print_simple_layout({content=>$content, show_panels => 1, subject => $self});
#     $self->print_http_header();
#
#     print $self->show_page_top(),
#     $self->show_header( 0 ),
#         $self->show_portal_panel( 0 ),
#         $self->show_panel(),
#             $content,
#             $self->show_bottom;
}

###############
sub www_edit_permissions {
    my $self = shift;
    $self->debug_msg(3, "SystemApp::www_edit_permissions()");

    my $isa_value = ref($self);


    my $return_location = sub {

    $self->md_redirect("index.pl?isa=$isa_value&op=show");
    };

    my $formdata = $self->edit_permissions_form(\%FORM, $return_location, $USER);
    $self->print_form($formdata);
}


###############
sub show_panel{
    my $self = shift;

    return '' unless ($SESSION->admin_mode());

    my $returnstring="";

    $returnstring .= $self->show_ac_button();

    return $returnstring;
}


###############
sub show_ac_button {
    my $self = shift;

    my $isa_value = ref($self);
    my $id = 0;

    $self->debug_msg(5, "SystemApp::show_ac_button(id=$id, isa_value=$isa_value)");

    return '' unless($SESSION->admin_mode());

    return '' unless ($self->is_allowed_to_do('edit_permissions', $USER));

    my $returnstring = "";

    my $imageslocation = $PARAMS->{imageslocation};

    $returnstring .= "<a href='index.pl?id=$id&amp;isa=$isa_value&amp;op=edit_permissions'>" . "<img src='$imageslocation/obj/ac.gif' width=\"24\" height=\"11\" border=\"0\" " . "alt='Change Access Rights!'></a>\n";

    return $returnstring;
}

1;

