###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::SystemApp::AppObject;
use strict;

use DBAccess;

use Metadot::MySQLTableInfo;
use Metadot::DataCollection;
use Metadot::SystemApp::AppObjectClass;

############################################################################
##  class members and methods
############################################################################
sub get_fields {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    return Metadot::SystemApp::AppObjectClass->get_fields($class);
}

sub get_collection {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $conditions = shift;
    my $order_by = shift;

    my $collection = Metadot::DataCollection->new();
    my $sth = $class->get_sth($conditions, $order_by);
    while (my $hash_ref = $sth->fetchrow_hashref){
        my $instance = $class->new($hash_ref);
        $collection->add($instance);
    }

    return $collection;
}

sub restore {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $pkey_value = shift;

    my $sth = DBAccess->sqlSelectMany("*", $class->get_table_name(), $class->get_pkey_name()." = '".$pkey_value."'");
    return $sth->fetchrow_hashref;
}

sub get_sth {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $conditions = shift;
    $conditions = (defined($conditions) && $conditions ne '') ? $conditions : undef;
    my $order_by = shift;
    $order_by = (defined($order_by) && $order_by ne '') ? "order by $order_by" : undef;

    return DBAccess->sqlSelectMany("*", $class->get_table_name(), $conditions, $order_by);
}

############################################################################
##  static abstract methods, should be overridden
############################################################################
sub get_table_name {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    die "should be overridden";
    return undef;
}

sub get_pkey_name {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    die "should be overridden";
    return undef;
}

############################################################################
##  constructor
############################################################################
sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $params = shift;
    # if constructor was called with parameters
    if (defined $params) {
        # ... and parameters is not a hash reference, than it's instance_id
        unless (ref $params) {
          my $instance_id = $params;
          $params = $class->restore($instance_id);
        }
    } else {
        $params = {};
    }

    my $self = $params;

    bless ($self, $class);
    $self->load_relations();
    return $self;
}

############################################################################
#   instance methods
############################################################################
sub set_value {
    my $self = shift;
    my $field_name = shift;
    my $field_value = shift;

    $self->{$field_name} = $field_value;
}

sub get_value {
    my $self = shift;
    my $field_name = shift;
    my $field_value = $self->{$field_name};

    return $field_value;
}

sub set_pkey_value {
    my $self = shift;
    my $pkey_value = shift;
    $self->set_value($self->get_pkey_name(), $pkey_value);
}

sub get_pkey_value {
    my $self = shift;
    return $self->get_value($self->get_pkey_name());
}

sub save {
    my $self = shift;
    return DBAccess->sqlSaveAppObject($self);
}

sub delete {
    my $self = shift;
    return DBAccess->sqlDeleteAppObject($self);
}

############################################################################
##  instance abstract methods, should be overridden
############################################################################
sub get_name {
    my $self = shift;
    die "should be overridden";
    return "";
}

sub load_relations {
    my $self = shift;
    die "should be overridden";
    return;
}

1;
