###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::SystemApp::AppObjectField;
use strict;

use Metadot::SystemApp::Util;

my $util = Metadot::SystemApp::Util->new();

############################################################################
##  constructor
############################################################################
sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $params = shift;
    # if constructor was called with parameter - hash
    unless (defined $params && ref $params) {
        $params = {};
    }

    my $self = $params;

    bless ($self, $class);
    return $self;
}

############################################################################
#   instance methods
############################################################################
sub set_value {
    my $self = shift;
    my $field_name = shift;
    my $field_value = shift;

    $self->{$field_name} = $field_value;
}

sub get_value {
    my $self = shift;
    my $field_name = shift;
    my $field_value = $self->{$field_name};

    return $field_value;
}

sub key {
    my $self = shift;
    return $self->get_value('key');
}

sub type {
    my $self = shift;
    return $self->get_value('type');
}

sub get_search_value {
    my $self = shift;
    my $form = shift;
    my $suffix = shift || '';
    return '' unless $form;

    return $form->{"search_".$self->key().$suffix};
}

sub get_search_tag {
    my $self = shift;
    my $type = $self->get_value('type');
    my $parameter = shift;
    my $value = undef;
    $value = $parameter unless (ref $parameter);
    $value ||= $self->get_value('value');
    $value ||= $parameter->{"search_".$self->key()} if (ref $parameter);
    $value ||= '';

    my $out = "";

    $out = "<INPUT TYPE=text size=7 name=\"search_".$self->key()."\" value=\"".$value."\">";
    return $out;
}

sub get_search_condition {
    my $self = shift;
    my $form = shift;
    my $persistent_fields = shift || {};
    my $key = $self->key();
    return "1 = 1" unless (defined $persistent_fields->{$key});
    my $value = $form->{"search_".$key};
    my $type = $self->get_value('type');

    return "1 = 1" unless (defined $value && $value ne '');

    if ($type eq 'number') {
        $value =~ s/\D//g;
        return "1 = 1" if ("$value" eq "");
        return $key." = ".$value;
    } elsif ($type eq 'datetime') {
        my $from = $form->{"search_".$key};
        my $to = $form->{"search_".$key."_to"};
        $from = $util->date_to_time($from);
        $to   = $util->date_to_time($to);
        $to = time() if ($to < $from);
        #print STDERR "************************\n$from :: $to\n*************************\n";
        return $key." >= ".$from." and ".$key." <= ".$to;
    }

    return $key." LIKE '\%".$value."\%'";
}

sub get_list_tag {
    my $self = shift;
    my $value = shift || $self->get_value('value');
    my $type = $self->get_value('type');

    if ($type eq 'datetime') {
        return $util->format_datetime($value, 'M D, Y');
    } elsif ($type eq 'number') {
        return defined ($value) ? $value : 0;
    } elsif ($type eq 'boolean') {
        return "$value" eq "1" ? "<font class='yes'>Yes</font>" : "<font class='no'>No</font>";
    }

    $value ||= 'N/A';
    return $value;
}

1;
