###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name$
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::SystemApp::EventLog;
use strict;

use Template;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS);
use Metadot::SystemApp;
use Portal;
use Auditable;
use Gizmo;

use DBAccess;

use vars qw(@ISA);
@ISA=qw(Metadot::SystemApp Portal Auditable);

use constant SEPARATOR => '*%!%*';
use constant RECORDS_PER_PAGE => 800;
use constant MAX_PAGES => 30;

# This order MUST NOT change
my @column_names = ( qw(unique_id userid user_name sub_application_name class_name operation_name instance_id date_and_time unix_time slot_1 slot_2 slot_3 slot_4 slot_5 slot_6 slot_7 slot_8 slot_9 slot_10 slot_11 slot_12 slot_13 slot_14 slot_15 slot_16 slot_17 slot_18 slot_19 slot_20) );

my %column_indices = map{ $column_names[$_], $_ } (0 .. $#column_names);

#
# Substitute for an Operations object
#
my $AC = {
      BUNDLES => {
              DISP =>   { DESC =>          'View',
                  OPS =>            [qw( show)],
                  DEFAULT_LEVEL =>  8,
                  MIN_LEVEL =>      0,
                  DEFAULTS_TO =>    'on',
                },
              EDITP =>  {
                 DESC =>          'Change<br />Permissions',
                 OPS =>            [qw(edit_permissions set_permissions)],
                 DEFAULT_LEVEL =>  8,
                 MIN_LEVEL =>      8,
                },
             },

      DISPLAY_ORDER => [ qw(DISP EDITP) ],
     };

# should always be overridden in derived classes
sub get_AC {
    my $self = shift;
    my $class = ref $self || $self;

    die "&get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

    return $AC;
}

sub name {
    return ''
}

sub get_app_name {
    return 'Event Log';
}

sub can_change_owner {

    return undef;
}


sub is_a {
    my $self = shift;

    my $class = ref($self) || $self;

    return $class
}


#
# Actual application logic follows
#

sub log_event {
    my $self = shift;

    my $std_params = shift;
    my $extra_params = shift;
    # map params and insert into table

    # add date and time to passed params
    $std_params->{unix_time} = time;
    $std_params->{date_and_time} = localtime( $std_params->{unix_time} );

    my %for_insertion;
    foreach my $column_name (keys %$std_params) {

    $for_insertion{$column_name} = $std_params->{$column_name};
    }

    my $slot_num = 1;
    foreach my $param_name (keys %$extra_params) {

    # don't log any password fields !!
    next if( $param_name =~ /password/i );

    # extra params are logged with a separator
    my $column_name = "slot\_$slot_num";

#   my $value = $param_name . SEPARATOR . $extra_params->{$param_name};
    # aju: address undefined var warning
    my $value = $param_name . SEPARATOR;
    if (defined $extra_params->{$param_name}) {
#       print STDERR "extra params: " . $extra_params->{$param_name} . "\n";
        $value .= $extra_params->{$param_name};
    }

    # trim to fit
    if(length $value > 255) {
        $value = substr($value, 0, 255);
    }

    $for_insertion{$column_name} = $value;
    $slot_num++;


    last if ($slot_num > 20);
    }

    #if (DBAccess->existsTable('event_log')) {

        DBAccess->sqlInsert( 'event_log', ( %for_insertion ) ) ;
    #}
}


sub main {
    my $self = shift;

    # check for enable/disable logging
    my $search_params = {
             user_name => $FORM{search_user_name},
             start_date => $FORM{search_start_date},
             start_hour => $FORM{search_start_hour},
             end_date => $FORM{search_end_date},
             end_hour => $FORM{search_end_hour},
             class_name => $FORM{search_class_name},
             operation_name => $FORM{search_operation_name},
             id => $FORM{search_id},
             word => $FORM{search_word},
            };

    my $records_per_page = $FORM{records_per_page};
    if ((!defined $records_per_page) || $records_per_page eq "" || $records_per_page == 0 ) {
    $records_per_page = RECORDS_PER_PAGE;
    }

    # perform search or display event details
    # creates data structure for the table
    my $search_results = $self->search( $search_params, $records_per_page );

    # trim according to page number
    my ($num_pages, $start_index, $end_index);

    ($search_results, $num_pages, $start_index, $end_index)
    = $self->_trim_search($search_results, $FORM{show_page},
                  $records_per_page);

    # create data structure for the page selector dial
    my $dial = { page_indices => [1 .. $num_pages],
         first_record_num => $start_index + 1,
         last_record_num => $end_index + 1,
         current_page_num => $FORM{show_page} || 1 };

    my $template = Template->new({
                  INCLUDE_PATH => 'Metadot/SystemApp/EventLog/templates',
                 }) || die $Template::ERROR, "\n";


    # data for the template
    my $data = { search_results => $search_results,
         previous_search => \%FORM,
         pages => $dial,
         is_enabled => $PARAMS->{is_event_log_enabled},
           };

    my $output;
    $template->process('EventLogMain.tt2', $data, \$output)
    || die $template->error(), "\n";

    return $output;

}


sub show_events {
    my $class = shift;

    my $search_params = shift;
}


sub _trim_search {
    my $self = shift;
    my $results = shift;
    my $page_number = shift || 1;
    my $records_per_page = shift;

    if (!defined $records_per_page) {
    $records_per_page = RECORDS_PER_PAGE;
    }


    #select page num
    my $num_pages = int( ($#$results +1) / $records_per_page );
    if( ($#$results +1) % $records_per_page )
    {
    $num_pages++;
    }

    if( $page_number > $num_pages ) { $page_number = $num_pages}
    if( $page_number < 1 ) { $page_number = 1}


#      my $start_index = $num_pages * ( $page_number - 1 );
#      my $start_index = ($page_number * $records_per_page) - $records_per_page + 1;
      my $start_index = ($page_number * $records_per_page) - $records_per_page ;
      my $end_index = $start_index + ($records_per_page -1);
    if ($end_index > $#$results) {
    $end_index = $#$results;
    }

#      my $end_index = ( $#$results < $start_index + RECORDS_PER_PAGE - 1 )  ?
#   $#$results : $start_index + RECORDS_PER_PAGE - 1;

#    print STDERR "AT TRIM SEARCH: total results: " . ($#$results + 1) .
#   ", page number $page_number, start index: $start_index, end index: $end_index, total pages $num_pages\n";

    my @trimmed = @$results[$start_index .. $end_index];

    return ( \@trimmed, $num_pages, $start_index, $end_index);
}


#
#
sub search {
    my $self = shift;
    my $params = shift;
    my $records_per_page = shift;

    if (!defined $records_per_page) {
    $records_per_page = RECORDS_PER_PAGE;
    }

    my $max_results = MAX_PAGES * $records_per_page;

    my @where_clause;
    if( defined $params->{user_name} && $params->{user_name} ) {
    push @where_clause, "user_name LIKE '\%$params->{user_name}\%'";
    }

    if( defined $params->{class_name} && $params->{class_name} ) {
    push @where_clause, "class_name LIKE '\%$params->{class_name}\%'";
    }

    if( defined $params->{operation_name} && $params->{operation_name} ) {
    push @where_clause, "operation_name = '$params->{operation_name}'";
    }

    if( defined $params->{instance_id} && ( $params->{instance_id} ne '' ) ) {
    push @where_clause, "instance_id = '$params->{instance_id}'";
    }

    if( defined $params->{unique_id} && ( $params->{unique_id} ne '' ) ) {
    push @where_clause, "unique_id = $params->{unique_id}";
    }


    if( $params->{start_date} ) {
    #convert to unix time
    my $stamp;
    push @where_clause, "unix_time>'$stamp'";
    }

    my $where_clause = join ' and ', @where_clause;

#    print STDERR "WHERE CLAUSE $where_clause\n";

    my $column_names = join ',', @column_names;

    my @results ;
    #if (DBAccess->existsTable('event_log')) {
        my $search = DBAccess->sqlSelectMany( $column_names, 'event_log', $where_clause, 'ORDER BY unix_time DESC' );
        my $rescount = 0;
        while( my (@row) = $search->fetchrow() ) {
                my $event = \@row;
                bless $event, 'Metadot::SystemApp::EventLog::Event';
                push @results, $event;
                $rescount++;
                if ($rescount > $max_results) {
                        last;
                }
        }
    #}

    # perform manual search for all other search params
    foreach my $search_param ( keys %$params ) {

    next if( $search_param eq 'user_name' or $search_param eq 'class_name' or
         $search_param eq 'operation_name' or $search_param eq 'instance_id' or
         $search_param eq 'start_date' or $search_param eq 'end_hour' or
         $search_param eq 'id' or  $search_param eq 'word' or
         $search_param eq 'end_date' or $search_param eq 'start_hour' );

    my $value = $params->{$search_param};

    next unless( $value );

    @results = grep { $_->get_value( $search_param ) eq $value } @results;
    }

    return \@results;
}


# will be called only every 100 or so insertions
sub _trim_table {
    my $self = shift;

}

sub get_separator {

    return SEPARATOR;
}


package Metadot::SystemApp::EventLog::Event;
use strict;
use Metadot::SystemApp::EventLog;

sub restore {
    my $self = shift;
    my $event_id = shift;

    my $column_names = join ',', @column_names;

    $event_id = undef unless Utility->is_valid_integer($event_id); # secu_200401
    my ( @row ) = DBAccess->sqlSelect( $column_names, 'event_log', "unique_id=$event_id" );

    my $event = \@row;

    bless $event, 'Metadot::SystemApp::EventLog::Event';

    return $event || undef;
}


sub get_value {
    my $self = shift;
    my $field_name = shift || die 'must include field_name';

#    use Data::Dumper;
#    print STDERR Dumper $self;

    # if in the standard fields, return them in that order
    # terribly innefficient, no caching whatsoever
    foreach ( $column_indices{slot_1} .. $column_indices{slot_20} ) {

#   print STDERR "AT EVENT::get_value -- INDEX NUM $_\n";

    if( $self->[$_] ) {

        my $sep = Metadot::SystemApp::EventLog->get_separator();

        my ($f_name, $val) = split( /\Q$sep\E/o, $self->[$_] );

        if( $f_name eq $field_name) { return $val }
    }
    }

    return undef;
}

sub get_unique_id {
    my $self = shift;

    return $self->[$column_indices{unique_id}];
}

sub get_userid {
    my $self = shift;

    return $self->[$column_indices{userid}];
}

sub get_user_name {
    my $self = shift;

    return $self->[$column_indices{user_name}];
}

sub get_class_name {
    my $self = shift;

    return $self->[$column_indices{class_name}];
}

sub get_operation_name {
    my $self = shift;

    return $self->[$column_indices{operation_name}];
}

sub get_instance_id {
    my $self = shift;

    return $self->[$column_indices{instance_id}];
}

sub get_date_and_time {
    my $self = shift;

    return $self->[$column_indices{date_and_time}];
}

sub get_unix_time {
    my $self = shift;

    return $self->[$column_indices{unix_time}];
}

sub get_object_name {
    my $self = shift;

    my $id = $self->get_instance_id;

    my $name;
    if(defined $id) {
    if (Gizmo->exists($id)) {
        $name = Gizmo->restore($id)->name();
    }
    }
    return $name || 'n/a';
}

sub get_event_description {
    my $self = shift;

    return '--------';
}


1;

__END__

