###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::SystemApp::EventLog::ReportGenerator::Default;
use strict;

use Template;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY);
#use Metadot::SystemApp::EventLog;
use Metadot::SystemApp::EventLog::ReportGenerator;
use Gizmo;
use GizmoBuilder;
use Utility;
use Digest::MD5 qw(md5_hex);

use DBAccess;

use vars qw(@ISA);
@ISA=qw(Metadot::SystemApp::EventLog::ReportGenerator);

# these constants are to refer to particular fields in a report
use constant _HITS_  => 1;
use constant _NAME_  => 2;
use constant _GROUP_ => 3;

#my $AC = Metadot::SystemApp::EventLog->get_AC();
#push( @{ $AC->{BUNDLES}{DISP}{OPS} }, 'show_report' );
#push( @{ $AC->{BUNDLES}{DISP}{OPS} }, 'change_reports_year_month_date' );
#
# Substitute for an Operations object
#
my $AC = {
      BUNDLES => {
              DISP =>   { DESC =>          'View',
                  OPS =>            [qw(show show_report_page show_report change_reports_year_month_date view_user_profile)],
                  DEFAULT_LEVEL =>  8,
                  MIN_LEVEL =>      2,
                  DEFAULTS_TO =>    'on',
                },
              ADMIN =>   { DESC =>          'Admin',
                  OPS =>            [qw(purge_stats)],
                  DEFAULT_LEVEL =>  8,
                  MIN_LEVEL =>      2,
                  DEFAULTS_TO =>    'on',
                },
              EDITP =>  {
                 DESC =>          'Change<br>Permissions',
                 OPS =>            [qw(edit_permissions set_permissions)],
                 DEFAULT_LEVEL =>  8,
                 MIN_LEVEL =>      8,
                },
             },

      DISPLAY_ORDER => [ qw(DISP ADMIN EDITP) ],
     };

my $NA = 'n/a';
my $NB = 200;
###
###  CONSTRUCTOR
###
###############
sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $self  = $class->SUPER::new();

    bless ($self, $class);

    return $self;
}

sub get_AC {
        return $AC;
}

sub name {
    return ''
}

sub get_app_name {
    return 'Event Log Reports';
}

sub can_change_owner {

    return undef;
}


sub is_a {
    my $self = shift;

    #$my $config_value = MetadotConfig->get_param_value('event_log_report_generator');
    my $class = ref($self) || $self;

    return $class
}

##########################################
#######                            #######
####### METHODS FOR CREATING STATS #######
#######                            #######
##########################################
sub process_event_log {
    my $self = shift;
    my %args = @_;

    my $extra_where = $args{extra_where};

    #$self->purge_event_log_report_builder();

    my $where = "1=1" . ($extra_where || '');

#print STDERR "where[$where]\n\n\n";

    my $records = DBAccess->sqlSelectMany("userid, user_name, class_name, operation_name, instance_id, unix_time", "event_log", $where, "order by userid, class_name, operation_name, instance_id, unix_time");

    my $day_hash = {};
    my $date = '';
    my $prev_date = '';
    my $prev_userid = 0;
    my $prev_class_name = '';
    my $prev_operation_name = '';
    my $prev_instance_id = 0;
    my $login_timestamp;
    my @logins;

    while(my($userid, $user_name, $class_name, $operation_name, $instance_id, $unix_time) = $records->fetchrow) {
        $date = Utility->format_unix_timestamp($unix_time, "Y-m-e");
#print STDERR "\nuserid:$userid date:$date\n";
#print STDERR "\n\ninstance_id:$instance_id unix_time:$unix_time\n\n" if $userid == -1;

        if ($date ne $prev_date || (defined $userid && $userid != $prev_userid)
            || $class_name ne $prev_class_name
            || $operation_name ne $prev_operation_name
            || (defined $instance_id && $instance_id != $prev_instance_id)
            ) {
            ## write previous day or user's data
            $self->write_daily_hash_to_db($day_hash);

            ## start over with new hash
            $day_hash = {};
            $day_hash->{total}          = 0;
            $day_hash->{userid}         = $userid;
            $day_hash->{user_name}      = $user_name;
            $day_hash->{class_name}     = $class_name;
            $day_hash->{operation_name} = $operation_name;
            $day_hash->{instance_id}    = $instance_id || '';
            $day_hash->{date_and_time}  = $date;
            $day_hash->{year_month_date} = $self->get_year_month_date($date);
#print STDERR "\n\nday_hash_instance_id:" . $day_hash->{instance_id} . "\n\n" if $userid == -1;
            $day_hash->{login_timestamp} = [];

#            print STDERR "\n\nwriting to report_generator logins contain $#logins logins\n\n";
#            print STDERR join("\n", @logins);
            @logins = ();
#            print STDERR "\n\nafter cleaning @logins\n";
        }
    if ( $operation_name eq "login_check" ){
                #$login_timestamp = Utility->format_unix_timestamp($unix_time,"Y-m-e-T");
        #push @logins,$login_timestamp;
        push @logins,$unix_time;
            $day_hash->{login_timestamp} = \@logins;
    #   push @{$day_hash->{login_timestamp)},$login_timestamp;
        }
        $day_hash->{total}++;
        $prev_date   = $date;
        $prev_userid = $userid;
        $prev_class_name = $class_name;
        $prev_operation_name = $operation_name;
        $prev_instance_id = $instance_id;
    }

    $self->write_daily_hash_to_db($day_hash);

    $self->process_montly_totals();

    # NZ uncommenting call to purge_event_log_report_builder
    $self->purge_event_log_report_builder();
    $self->wrapup_duties();
}

sub wrapup_duties {
    my $self = shift;
}

sub purge_event_log_report_builder {
    my $self = shift;
    my %args = @_;
    my $year_month_date = $args{year_month_date};
    my $year_month_date_start = $args{year_month_date_start};
    my $year_month_date_end = $args{year_month_date_end};

    my $where = '1=1';
    if (defined($year_month_date)) {
        $where .= " and year_month_date = $year_month_date";
    } else {
        if (defined($year_month_date_start)) {
            $where .= " and year_month_date >= $year_month_date_start";
        }
        if (defined($year_month_date_end)) {
            $where .= " and year_month_date <= $year_month_date_end";
        }
    }
    DBAccess->sqlDelete("event_log_report_builder", $where);
}

sub purge_stats {
    my $self = shift;
    my %args = @_;
    my $year_month_date = $args{year_month_date};
    my $year_month_date_start = $args{year_month_date_start};
    my $year_month_date_end = $args{year_month_date_end};

    # secu_200401: need to determine what the correct format for these
    # args are.... will have to check w/ Mike S.

    $self->purge_event_log_report_builder(%args);

    my $where = '1=1';
    if (defined($year_month_date)) {
        $where .= " and year_month_date = $year_month_date";
    } else {
        if (defined($year_month_date_start)) {
            $where .= " and year_month_date >= $year_month_date_start";
        }
        if (defined($year_month_date_end)) {
            $where .= " and year_month_date <= $year_month_date_end";
        }
    }

    DBAccess->sqlDelete("event_log_monthly_user_stats", $where);
    DBAccess->sqlDelete("event_log_monthly_group_stats", $where);
    DBAccess->sqlDelete("event_log_monthly_object_stats", $where);
    DBAccess->sqlDelete("event_log_monthly_grpmembers", $where);
}

sub write_daily_hash_to_db {
    my $self = shift;
    my $day_hash = shift;

    return unless defined($day_hash->{userid});

    my $unique_id = $self->get_day_hash_unique_id($day_hash);
#if ($unique_id eq '5e04428e34c2f43e708737528b820111') {
#use Data::Dumper;
#print STDERR "\n\nwrite_daily_hash_to_db() instance_id:" . $day_hash->{instance_id} . " day_hash:" . Dumper($day_hash) . "\n\n";
#}

    my ($fetched_unique_id, $prev_total) = DBAccess->sqlSelect("unique_id, total", "event_log_report_builder", "unique_id = '$unique_id'");

    if (!$fetched_unique_id) { ## doesn't exist insert
#print STDERR "\ndaily: inserting $unique_id";
        DBAccess->sqlInsert("event_log_report_builder",
            unique_id      => $unique_id,
            userid         => $day_hash->{userid},
            user_name      => $day_hash->{user_name},
            class_name     => $day_hash->{class_name},
            operation_name => $day_hash->{operation_name},
            instance_id    => $day_hash->{instance_id},
            date_and_time  => $day_hash->{date_and_time},
            year_month_date => $day_hash->{year_month_date},
            total          => $day_hash->{total},
            login_timestamp => join(",",@{$day_hash->{login_timestamp}} ),
        );
    } elsif ($day_hash->{total} > $prev_total) { ## update with recent values
#print STDERR "\ndaily: updating $unique_id";
        DBAccess->sqlUpdate("event_log_report_builder",
            "unique_id = '$unique_id'",
            userid         => $day_hash->{userid},
            user_name      => $day_hash->{user_name},
            class_name     => $day_hash->{class_name},
            operation_name => $day_hash->{operation_name},
            instance_id    => $day_hash->{instance_id},
            date_and_time  => $day_hash->{date_and_time},
            year_month_date => $day_hash->{year_month_date},
            total          => $day_hash->{total},
            login_timestamp => join(",",@{$day_hash->{login_timestamp}} ),
        );
    }

}

sub get_day_hash_unique_id {
    my $self = shift;
    my $day_hash = shift || die 'get_day_hash_unique_id(): day hash undefined';

#if ($day_hash->{userid} == 8) {
#use Data::Dumper;
#print STDERR "\n\nget_day_hash_unique_id() instance_id:" . $day_hash->{instance_id} . " day_hash:" . Dumper($day_hash) . "\n\n" if $day_hash->{userid} == 8;
#}
    return $self->get_md5_digest($day_hash->{date_and_time} . $day_hash->{userid} . $day_hash->{class_name} . $day_hash->{operation_name} . $day_hash->{instance_id});
}

sub get_md5_digest {
    my $self = shift;
    my $string = shift || die 'get_md5_digest(): must provide string';

    return Digest::MD5::md5_hex($string);
}

sub process_montly_totals {
    my $self = shift;

    $self->process_user_montly_totals();
    $self->process_object_montly_totals();
    $self->process_group_montly_totals();
    $self->update_null_values();
}

sub process_user_montly_totals {
    my $self = shift;

    my $records = DBAccess->sqlSelectMany("userid, class_name, operation_name, instance_id, year_month_date, sum(total) total", "event_log_report_builder", "1=1", "group by year_month_date, userid, class_name, operation_name, instance_id");


    my $unique_id = '';
    my $fetched_unique_id = '';
    my $prev_sum_total = 0;

    while(my($userid, $class_name, $operation_name, $instance_id, $year_month_date, $sum_total) = $records->fetchrow) {
        $unique_id = $self->get_md5_digest($year_month_date . $userid . $class_name . $operation_name . $instance_id);
        ($fetched_unique_id, $prev_sum_total) = DBAccess->sqlSelect("unique_id, monthly_total", "event_log_monthly_user_stats", "unique_id = '$unique_id'");

        if (!$fetched_unique_id) { ## doesn't exist insert
#print STDERR "\nmonthly user: inserting $unique_id";
            DBAccess->sqlInsert("event_log_monthly_user_stats",
                unique_id      => $unique_id,
                userid         => $userid,
                #user_name      => $user_name,
                class_name     => $class_name,
                operation_name => $operation_name,
                instance_id    => $instance_id,
                #instance_name  => $instance_name,
                year_month_date     => $year_month_date,
                monthly_total  => $sum_total,
            );
        } elsif ($sum_total > $prev_sum_total) { ## update with recent values
#print STDERR "\nmonthly user: updating $unique_id";
            DBAccess->sqlUpdate("event_log_monthly_user_stats",
                "unique_id = '$unique_id'",
                userid         => $userid,
                #user_name      => $user_name,
                class_name     => $class_name,
                operation_name => $operation_name,
                instance_id    => $instance_id,
                #instance_name  => $instance_name,
                year_month_date     => $year_month_date,
                monthly_total  => $sum_total,
            );
        }
    }

}

sub process_object_montly_totals {
    my $self = shift;

    my $records = DBAccess->sqlSelectMany("class_name, operation_name, instance_id, year_month_date, sum(monthly_total)", "event_log_monthly_user_stats", "1=1", "group by year_month_date, class_name, operation_name, instance_id");


    my $unique_id = '';
    my $fetched_unique_id = '';
    my $prev_sum_total = 0;

    while(my($class_name, $operation_name, $instance_id, $year_month_date, $sum_total) = $records->fetchrow) {
        $unique_id = $self->get_md5_digest($year_month_date . $class_name . $operation_name . $instance_id);
        ($fetched_unique_id, $prev_sum_total) = DBAccess->sqlSelect("unique_id, monthly_total", "event_log_monthly_object_stats", "unique_id = '$unique_id'");

        if (!$fetched_unique_id) { ## doesn't exist insert
#print STDERR "\nmonthly object: inserting $unique_id";
            DBAccess->sqlInsert("event_log_monthly_object_stats",
                unique_id      => $unique_id,
                class_name     => $class_name,
                operation_name => $operation_name,
                instance_id    => $instance_id,
                #instance_name  => $instance_name,
                year_month_date     => $year_month_date,
                monthly_total  => $sum_total,
            );
        } elsif ($sum_total > $prev_sum_total) { ## update with recent values
#print STDERR "\nmonthly object: updating $unique_id";
            DBAccess->sqlUpdate("event_log_monthly_object_stats",
                "unique_id = '$unique_id'",
                class_name     => $class_name,
                operation_name => $operation_name,
                instance_id    => $instance_id,
                #instance_name  => $instance_name,
                year_month_date     => $year_month_date,
                monthly_total  => $sum_total,
            );
        }
    }

}

sub process_group_montly_totals {
    my $self = shift;

    my $records = DBAccess->sqlSelectMany("g.grp_id, g.group_name, e.class_name, e.operation_name, e.instance_id, e.year_month_date, sum(e.monthly_total)", "event_log_monthly_user_stats e, grp g, grpmembers gm", "e.userid = gm.userid and g.grp_id = gm.grp_id", "group by g.grp_id, g.group_name, e.year_month_date, e.class_name, e.operation_name, e.instance_id");


    my $unique_id = '';
    my $fetched_unique_id = '';
    my $prev_sum_total = 0;

    # keep track of the year_month_date intervals that were updated, so we can update
    # the grp<->user relationship for that period also.
    my %updated_periods = ();

    while(my($grp_id, $group_name, $class_name, $operation_name, $instance_id, $year_month_date, $sum_total) = $records->fetchrow) {
        $unique_id = $self->get_md5_digest($year_month_date . $grp_id . $class_name . $operation_name . $instance_id);
        ($fetched_unique_id, $prev_sum_total) = DBAccess->sqlSelect("unique_id, monthly_total", "event_log_monthly_group_stats", "unique_id = '$unique_id'");

        if (!$fetched_unique_id) { ## doesn't exist insert
#print STDERR "\nmonthly group: inserting $unique_id";
            DBAccess->sqlInsert("event_log_monthly_group_stats",
                unique_id      => $unique_id,
                grp_id         => $grp_id,
                group_name     => $group_name,
                class_name     => $class_name,
                operation_name => $operation_name,
                instance_id    => $instance_id,
                #instance_name  => $instance_name,
                year_month_date     => $year_month_date,
                monthly_total  => $sum_total,
            );
            $updated_periods{$year_month_date}++;
        } elsif ($sum_total > $prev_sum_total) { ## update with recent values
#print STDERR "\nmonthly group: updating $unique_id";
            DBAccess->sqlUpdate("event_log_monthly_group_stats",
                "unique_id = '$unique_id'",
                grp_id         => $grp_id,
                group_name     => $group_name,
                class_name     => $class_name,
                operation_name => $operation_name,
                instance_id    => $instance_id,
                #instance_name  => $instance_name,
                year_month_date     => $year_month_date,
                monthly_total  => $sum_total,
            );
            $updated_periods{$year_month_date}++;
        }
    }

    $records->finish();

    # now process any grp<->user relationships that need to be updated
    foreach my $ymd (keys(%updated_periods)) {
        # get the values that need to be added. this query finds all grp_id in group stats and userid in user stats
        # for a given y-m-d period, which don't exist in the monthly grpmembers
        my $select = 'DISTINCT gm.grp_id, gm.userid';
        my $tables = 'event_log_monthly_group_stats eg, event_log_monthly_user_stats eu, grpmembers gm LEFT JOIN event_log_monthly_grpmembers egm ON egm.userid=eu.userid AND egm.grp_id=eg.grp_id';
        my $where  = "egm.grp_id IS NULL AND egm.userid IS NULL AND eg.year_month_date=$ymd AND eu.year_month_date=$ymd AND gm.grp_id=eg.grp_id AND gm.userid=eu.userid";

#print STDERR "\nSELECT $select\nFROM $tables\nWHERE $where\n\n";

        $records = DBAccess->sqlSelectMany($select, $tables, $where);
        while ( my($grp_id, $userid) = $records->fetchrow()) {
#print STDERR "Updating event_log_monthly_grpmembers: ymd[$ymd]; grp_id[$grp_id]; userid[$userid]\n";
            DBAccess->sqlInsert('event_log_monthly_grpmembers', year_month_date=>$ymd, grp_id=>$grp_id, userid=>$userid);
        }
        $records->finish();
    }
}


sub update_null_values {
    my $self = shift;


    ### UPDATE USER_NAMES ###
    my $userids = DBAccess->sqlSelectMany("distinct(userid)", "event_log_monthly_user_stats", "user_name is null");

    my $user_name = '';
    while(my($userid) = $userids->fetchrow) {
        ($user_name) = DBAccess->sqlSelect("user_name", "event_log_report_builder", "userid = $userid", "order by date_and_time desc");
        DBAccess->sqlUpdate("event_log_monthly_user_stats", "userid = $userid and user_name is null", user_name => $user_name);
    }

    ### UPDATE INSTANCE_NAMES ###
    my $instance_ids = DBAccess->sqlSelectMany("distinct instance_id, class_name", "event_log_monthly_user_stats", "instance_id is not null and instance_id != '' and instance_id != '' and class_name is not null and instance_name is null");

    my $instance_name = '';
    my $object;
    while(my($instance_id, $class_name) = $instance_ids->fetchrow) {
        eval {
            $instance_name = Gizmo->restore($instance_id)->name();
        };
        unless ($@ && $instance_name eq '') {
            DBAccess->sqlUpdate("event_log_monthly_user_stats", "instance_id = $instance_id and instance_name is null", instance_name => $instance_name);
            DBAccess->sqlUpdate("event_log_monthly_object_stats", "instance_id = $instance_id and instance_name is null", instance_name => $instance_name);
            DBAccess->sqlUpdate("event_log_monthly_group_stats", "instance_id = $instance_id and instance_name is null", instance_name => $instance_name);
        }
        $instance_name = '';
    }

    ### MUST set these values because will not be calculated correctly in count(*) queries
    #DBAccess->sqlUpdate("event_log_monthly_user_stats", "instance_id is null", instance_id => '');
    DBAccess->sqlUpdate("event_log_monthly_user_stats", "instance_name is null", instance_name => '');
    #DBAccess->sqlUpdate("event_log_monthly_object_stats", "instance_id is null", instance_id => '');
    DBAccess->sqlUpdate("event_log_monthly_object_stats", "instance_name is null", instance_name => '');
    #DBAccess->sqlUpdate("event_log_monthly_group_stats", "instance_id is null", instance_id => '');
    DBAccess->sqlUpdate("event_log_monthly_group_stats", "instance_name is null", instance_name => '');
}

sub get_year_month_date {
    my $self = shift;
    my $date = shift || die 'get_year_month_date(): must provide date';

    return join('', (split /\-/, $date )[0,1]);
}

##########################################
#######           END              #######
####### METHODS FOR CREATING STATS #######
#######                            #######
##########################################


############################################
#######                              #######
####### METHODS FOR CREATING REPORTS #######
#######                              #######
############################################

sub main {
    my $self = shift;
    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::main() - will use show()");
    return $self->show();
}

sub show {
    my $self = shift;
    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::show()");
    my $isa = $self->is_a();
    $self->md_redirect("index.pl?isa=$isa&op=change_reports_year_month_date");
    exit;
}


sub www_show_report_page {
    my $self = shift;
    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::www_show_report_page()");

    return $self->www_change_reports_year_month_date();
}

sub www_purge_stats {
    my $self = shift;
    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::www_purge_stats()");

    my $year_month_date       = $FORM{year_month_date};
    my $year_month_date_start = $FORM{year_month_date_start};
    my $year_month_date_end = $FORM{year_month_date_end};
    my $isa = $self->is_a();
    my $purge_link = "index.pl?isa=$isa&amp;op=purge_stats";
    my $formatted_date_start = defined($year_month_date_start) ? $self->format_year_month_date(year_month_date => $year_month_date_start, format => 'M Y') : '';
    my $formatted_date_end   = defined($year_month_date_end) ? $self->format_year_month_date(year_month_date => $year_month_date_end, format => 'M Y') : '';

    if ($year_month_date_start > $year_month_date_end) {
        $DISPLAY->print_click_back('End month must come after start month.');
        exit;
    }

    if (defined($FORM{confirm}) && defined($year_month_date_start) && defined($year_month_date_end)) {
        $DISPLAY->print_choices("<big>Please confirm that you would like to delete Usage statistics for months between $formatted_date_start and $formatted_date_end.</big>",
                       "confirm",
                       "Yes",
                       $purge_link . "&year_month_date_start=$year_month_date_start&year_month_date_end=$year_month_date_end&delete=1",
                       "No",
                       "index.pl?isa=$isa&op=purge_stats",
                       );
         exit;
    } elsif (defined($FORM{delete}) && $FORM{delete} == 1 && defined($year_month_date_start) && defined($year_month_date_end)) {
        $self->purge_stats(year_month_date_start => $year_month_date_start, year_month_date_end => $year_month_date_end);
        $DISPLAY->print_choices("<big>Usage statistics for months between $formatted_date_start and $formatted_date_end have now been deleted!</big>",
                       "readme",
                       "Return to Usage Statistics",
                       "index.pl?isa=$isa&op=show_report_page",
                       );
         exit;
    }

    my $template = Template->new({
                  INCLUDE_PATH => 'Metadot/SystemApp/EventLog/ReportGenerator/Default/templates',
                 }) || die $Template::ERROR, "\n";

    # data for the template
    my $data = {
                menu_link   => "index.pl?isa=$isa&op=change_reports_year_month_date&year_month_date=$year_month_date",
                year_month_dates => $self->get_distinct_year_month_dates(sort => 'asc'),
                isa => $isa,
                op  => "purge_stats",
                PARAMS => $PARAMS,
               };

    my $output;
    $template->process('PurgeStats.tt2', $data, \$output)
    || die $template->error(), "\n";
    return $self->print_portal($output);
}

sub get_report_descriptions {
    my $self = shift;
    my %args = @_;
    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_report_descriptions()");

    my $isa = $self->is_a();
    my $year_month_date = $args{year_month_date};
    my @report_descriptions = (
        {
          label => 'Object',
          link  => "index.pl?isa=$isa&amp;op=show_report&amp;report_type=object_summary&amp;year_month_date=$year_month_date",
        },
        {
          label => 'User',
          link  => "index.pl?isa=$isa&amp;op=show_report&amp;report_type=user_summary&amp;year_month_date=$year_month_date",
        },
        {
          label => 'Group',
          link  => "index.pl?isa=$isa&amp;op=show_report&amp;report_type=group_summary&amp;year_month_date=$year_month_date",
        },
       );

    # see if we have grp->user stats for this month
    my ($grpmember_stats) = DBAccess->sqlSelect('count(*)', 'event_log_monthly_grpmembers', "year_month_date=$year_month_date");
#print STDERR "grpmember_stats[$grpmember_stats]\n";

    if ($grpmember_stats > 0) {
        push @report_descriptions, {
          label => 'View Login Report by Group',
          link  => "index.pl?isa=$isa&amp;op=show_report&amp;report_type=login_summary&amp;year_month_date=$year_month_date",
          label_complete => 1,
        };
    }

    my ($login_stats) = DBAccess->sqlSelect('count(*)', 'event_log_report_builder', "year_month_date=$year_month_date and operation_name='login_check' ");

    if ($login_stats > 0) {
        push @report_descriptions, {
          label => 'View Login Report by User',
          link  => "index.pl?isa=$isa&amp;op=show_report&amp;report_type=login_user_summary&amp;year_month_date=$year_month_date",
          label_complete => 1,
        };
    }

     push @report_descriptions, @{$self->get_additional_report_descriptions(%args)};

     return \@report_descriptions;
}

sub get_additional_report_descriptions {
    my $self = shift;
    my %args = @_;
    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_additional_report_descriptions()");
    return [];
}

sub show_report {
    my $self = shift;

    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::show_report()");

    my $report_type = $FORM{report_type};
    my $report_format = $FORM{report_format};

    my $isa = $self->is_a();

    my $year_month_date = $FORM{year_month_date} || $self->get_current_year_month_date();
    my $report_date = $self->format_year_month_date(year_month_date => $year_month_date),
    my $result_set;

    my %params = (
                  report_type => $report_type,
                  report_format => $report_format,
                  year_month_date => $year_month_date
                 );
    if ($report_format eq 'excel') {
        $params{from} = 0;
        $params{nb} = 1000;
    }

    my $report_type_function = "get_" . $report_type . "_result_set";
    $result_set = $self->$report_type_function(%params);
    ## If exporting to excel, then download and exit
    if ($report_format eq 'excel') {
        $result_set->{title} .= " for $report_date";
        $DISPLAY->export_excel(data => $result_set, download_name => $report_type . $year_month_date);
        exit;
    }

    $self->debug_msg(4, "Metadot::SystemApp::EventLog::ReportGenerator::Default::show_report(): report_type:$report_type report_format:$report_format report_date:$report_date");

    # data for the template
    my $data = {
               report_descriptions => $self->get_report_descriptions(year_month_date => $year_month_date),
               report_type => $report_type,
               menu_link   => "index.pl?isa=$isa&amp;op=change_reports_year_month_date&amp;year_month_date=$year_month_date",
               result_set  => $result_set,
               report_date => $report_date,
               sort_by     => $FORM{sort_by},
               PARAMS     => $PARAMS,
           };

    my $output;
    my $template = Template->new({
                  INCLUDE_PATH => 'Metadot/SystemApp/EventLog/ReportGenerator/Default/templates',
                 }) || die $Template::ERROR, "\n";

    $template->process('ShowReport.tt2', $data, \$output)
    || die $template->error(), "\n";

    return $output;

}

sub get_user_summary_result_set {
    my $self = shift;
    my %params = @_;

    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_user_summary_result_set()");
    return $self->get_user_result_set(%params);
}

sub get_user_detail_result_set {
    my $self = shift;
    my %params = @_;

    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_user_detail_result_set()");
    return $self->get_user_result_set(%params);
}

sub get_user_login_details_result_set {
    my $self = shift;
    my %params = @_;

    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_user_login_details_result_set()");
    return $self->get_login_user_result_set(%params);
}

sub get_group_summary_result_set {
    my $self = shift;
    my %params = @_;

    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_group_summary_result_set()");
    return $self->get_group_result_set(%params);
}

sub get_group_detail_result_set {
    my $self = shift;
    my %params = @_;

    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_group_detail_result_set()");
    return $self->get_group_result_set(%params);
}

sub get_object_summary_result_set {
    my $self = shift;
    my %params = @_;

    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_object_summary_result_set()");
    return $self->get_object_result_set(%params);
}

sub get_object_detail_result_set {
    my $self = shift;
    my %params = @_;

    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_object_detail_result_set()");
    return $self->get_object_result_set(%params);
}

sub get_login_summary_result_set {
    my $self = shift;
    my %params = @_;

    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_login_summary_result_set()");

    # dispatch a customized version of group_summary
    $params{report_type}        = 'login_summary';
    $params{do_detail}          = 1;
    $params{operation_name}     = 'login_check';
    $params{title}              = 'Logins By Group';
    $params{simple_headers}     = 1;
    $params{header_name_map}    = { _HITS_=>'Logins' };
    $params{detail_report_type} = undef;

    my $results = $self->get_group_result_set(%params);

    return $results;
}

sub get_login_user_summary_result_set {
    my $self = shift;
    my %params = @_;

    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_login_user_summary_result_set()");

    # dispatch a customized version of group_summary
    $params{report_type}        = 'login_user_summary';
    $params{do_detail}          = 1;
    $params{operation_name}     = 'login_check';
    $params{title}              = 'Logins By User';
    $params{simple_headers}     = 1;
    $params{header_name_map}    = { _HITS_=>'Logins' };
    $params{detail_report_type} = undef;

    my $results = $self->get_login_result_set(%params);

    return $results;
}

sub get_login_detail_result_set {
    my $self = shift;
    my %params = @_;

    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_login_detail_result_set()");

    # dispatch a customized version of group_summary
    $params{report_type}        = 'login_detail';
    $params{operation_name}     = 'login_check';
    $params{title}              = 'Logins By User';
    $params{simple_headers}     = 1;
    $params{header_name_map}    = { _HITS_=>'Logins' };

    my $results = $self->get_user_summary_result_set(%params);

    # make some changes to the results hash
    return $results;
}


sub get_user_result_set {
    my $self = shift;
    my %params = @_;

    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_user_result_set()");

    my $report_type     = $params{report_type};
    my $report_format   = $params{report_format};
    my $year_month_date = $params{year_month_date} || $FORM{year_month_date} || $self->get_current_year_month_date();
    my $nb              = $params{nb} || $FORM{nb} || $NB;
    my $start_from      = $params{from} || $FORM{from} || 0;
    my $op              = $params{op}   || $FORM{op};
    my $sort_by         = $params{sort_by} || $FORM{sort_by};
    my $params_userid   = $params{userid} || $FORM{userid};
    my $params_instance_id   = $params{instance_id} || $FORM{instance_id};
    my $params_class_name = $params{class_name} || $FORM{class_name};
    my $params_operation_name = $params{operation_name} || $FORM{operation_name};
    my $grp_id = $params{grp_id} || $FORM{grp_id};
    my $grp_name;

    my $hits_title = $params{header_name_map}{_HITS_} || 'Hits';
    my $name_title = $params{header_name_map}{_NAME_} || 'Name';

    $self->debug_msg(4, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_user_result_set(): report_type:$report_type params_userid:$params_userid params_instance_id:$params_instance_id params_class_name:$params_class_name operation_name:$params_operation_name year_month_date:$year_month_date nb:$nb start_from:$start_from op:$op sort_by:$sort_by");
    my $isa = $self->is_a();
    my $url0 = "index.pl?isa=$isa&amp;op=$op&amp;year_month_date=$year_month_date";
    my $url1 = $url0 . "&amp;report_type=$report_type";
    my $url2 = $url1 . "&amp;userid=$params_userid&amp;instance_id=$params_instance_id&amp;class_name=$params_class_name&amp;operation_name=$params_operation_name";
    my $url3 = $url2 . "&amp;sort_by=$sort_by";
    my @headers = ();
    if ($report_format eq 'excel') {
        @headers = ($hits_title, $name_title);
    } else {
        @headers = ("<a href=\"$url2&amp;sort_by=hits\">$hits_title</a>", "<a href=\"$url2&amp;sort_by=name\">$name_title</a>");
    }


    my $from = $start_from;
    my $i = 0;

    my $tables = 'event_log_monthly_user_stats eu';

    my $group_by = 'group by ';
    my $total_rows_fields = '';
    my $select_fields     = "eu.userid, eu.user_name, sum(eu.monthly_total) sum_monthly_total";
    my $where = "eu.year_month_date = $year_month_date";

    if (defined($params_operation_name) && $params_operation_name ne '') {
        $where .= " and eu.operation_name = '$params_operation_name'";
    }

    if (defined $grp_id && Utility->is_valid_integer($grp_id)) {
        $tables .= ', event_log_monthly_grpmembers egm';
        $where .= " AND eu.userid=egm.userid AND egm.grp_id=$grp_id";
        ($grp_name) = DBAccess->sqlSelect('group_name', 'grp', "grp_id=$grp_id");
        defined($grp_name) || ($grp_name = '[Group no longer exists]');
    }

    my $title = $params{title} || '';
    #my $summary_detail_link = '';
    if ($report_type eq 'user_detail') {
        $select_fields     .= ", class_name, operation_name, instance_id, instance_name";
        $total_rows_fields = "eu.userid, user_name, class_name, operation_name, instance_id, instance_name";
        $group_by          .= "eu.userid, user_name, class_name, operation_name, instance_id, instance_name";
        if (defined($params_userid) && $params_userid ne '') {
            $where .= " and eu.userid = $params_userid";
        }
        if (defined($params_instance_id) && $params_instance_id ne '') {
            $where .= " and instance_id = '$params_instance_id'";
        }
        if (defined($params_class_name) && $params_class_name ne '') {
            $where .= " and class_name = '$params_class_name'";
        }
        if ($report_format eq 'excel') {
            push (@headers, ('Object Name (id)', 'Object Class', 'Operation'));
        } else {
            push (@headers, ("<a href=\"$url2&amp;sort_by=object_name\">Object Name (id)</a>", "<a href=\"$url2&amp;sort_by=object_class\">Object Class</a>", "<a href=\"$url2&amp;sort_by=operation\">Operation</a>"));
        }
        $title ||= "Usage Statistics by User & Operation";
    } else {
        $total_rows_fields = "eu.userid, user_name";
        $group_by          .= "eu.userid, user_name";
        $title ||= "Usage Statistics by User";
    }

    ## DETERMINE ORDER BY
    my $order_by = 'order by ';
    if ($sort_by eq 'name') {
        $order_by .= 'user_name, sum_monthly_total desc';
    } elsif ($sort_by eq 'object_name') {
        $order_by .= 'instance_name, sum_monthly_total desc';
    } elsif ($sort_by eq 'object_class') {
        $order_by .= 'class_name, sum_monthly_total desc';
    } elsif ($sort_by eq 'operation') {
        $order_by .= 'operation_name, sum_monthly_total desc';
    } else {
        $order_by .= 'sum_monthly_total desc';
    }

    my ($total_rows) = DBAccess->sqlSelectCountDistinct($total_rows_fields, $tables, $where);
    my $records = DBAccess->sqlSelectMany($select_fields, $tables, $where, "$group_by $order_by");

    my @entries = ();

    my $object_name = '';
    my $total_hits = 0;
    my $report_user_name = '';
    my $report_class_name = '';
    my $report_operation_name = '';
    while((my ($userid, $user_name, $sum_monthly_total, $class_name, $operation_name, $instance_id, $instance_name)=$records->fetchrow()) && ($i < $nb)) {
        if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC
        $report_user_name = $user_name;
        $report_class_name = $class_name;
        $report_operation_name = $operation_name;
        $total_hits += $sum_monthly_total;
        unless ((defined($params_userid) && $params_userid ne '') || $report_format eq 'excel' || $report_type eq 'user_detail') {
            $sum_monthly_total = "<a href=\"$url0&amp;report_type=user_detail&amp;userid=$userid\">$sum_monthly_total</a>";
        }
        unless ($report_format eq 'excel') {
            $user_name .= "&nbsp;-&nbsp;[<a href=\"index.pl?isa=$isa&amp;op=view_user_profile&amp;uid=$userid\">view profile</a>]";
        }
        push (@entries, $sum_monthly_total, $user_name);

        if ($report_type eq 'user_detail') {
            $instance_name ||= $NA;
            $instance_id   ||= $NA;
            $object_name = $instance_name;
            $object_name .= " ($instance_id)" if $instance_id ne '';
            push (@entries, $object_name, $class_name, $operation_name);
        }
        $i++;
    }
    if ($report_format eq 'excel') {
        push (@entries, $total_hits, 'Total');
    } else {
        push (@entries, $total_hits, '<b>Total</b>');
    }

    my ($pn) = Components->get_previous_next_button($nb, $i, $total_rows, $start_from, $url3); #PREVIOUS NEXT MAGIC

    my @sub_title = ();
    push (@sub_title, "Name: $report_user_name") if defined($params_userid) && $params_userid ne '';
    push (@sub_title, "Object Class: $report_class_name") if defined($params_class_name) && $params_class_name ne '' && defined($report_class_name) && $report_class_name ne '';
    push (@sub_title, "Operation: $report_operation_name") if defined($params_operation_name) && $params_operation_name ne '' && defined($report_operation_name) && $report_operation_name ne '';
    push (@sub_title, "Group: $grp_name") if defined($grp_name) && $grp_name ne '';
    my $sub_title = join(", ", @sub_title);
    return { headers => \@headers, entries => \@entries, pn => $pn, title => $title, sub_title => $sub_title, excel_link => "$url3&amp;report_format=excel"};
}
sub get_login_user_result_set {
    my $self = shift;
    my %params = @_;

    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_login_user_result_set()");

    my $report_type     = $params{report_type};
    my $report_format   = $params{report_format};
    my $year_month_date = $params{year_month_date} || $FORM{year_month_date} || $self->get_current_year_month_date();
    my $nb              = $params{nb} || $FORM{nb} || $NB;
    my $start_from      = $params{from} || $FORM{from} || 0;
    my $op              = $params{op}   || $FORM{op};
    my $sort_by         = $params{sort_by} || $FORM{sort_by};
    my $params_username = $FORM{user_name};
    my $params_userid   = $params{userid} || $FORM{userid};
    my $params_instance_id   = $params{instance_id} || $FORM{instance_id};
    my $params_class_name = $params{class_name} || $FORM{class_name};
    my $params_operation_name = $params{operation_name} || $FORM{operation_name};
    my $grp_id = $params{grp_id} || $FORM{grp_id};
    my $grp_name;

    my $name_title = $params{header_name_map}{_NAME_} || 'Name';
    my $time_title = $params{header_name_map}{_NAME_} || 'Date';



    $self->debug_msg(4, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_user_result_set(): report_type:$report_type params_userid:$params_userid params_instance_id:$params_instance_id params_class_name:$params_class_name operation_name:$params_operation_name year_month_date:$year_month_date nb:$nb start_from:$start_from op:$op sort_by:$sort_by");

    my $isa = $self->is_a();
    my $url0 = "index.pl?isa=$isa&amp;op=$op&amp;year_month_date=$year_month_date";
    my $url1 = $url0 . "&amp;report_type=$report_type";
    my $url2 = $url1 . "&amp;userid=$params_userid&amp;user_name=$params_username&amp;class_name=$params_class_name&amp;operation_name=$params_operation_name";
    my $url3 = $url2 . "&amp;sort_by=$sort_by";
    my @headers = ();

    if ($report_format eq 'excel') {
        @headers = ($name_title, $time_title);
    } else {
        @headers = ("$name_title", "<a href=\"$url2&amp;sort_by=date\">$time_title</a>");
    }


    my $from = $start_from;
    my $i = 0;

    my $tables = 'event_log_report_builder eu';

    my $group_by = 'group by ';
    my $total_rows_fields = 'userid';
    my $select_fields     = "eu.userid, eu.user_name,  login_timestamp, date_and_time";
    my $where = "eu.year_month_date = $year_month_date and eu.userid = $params_userid ";
    $where .= "and operation_name = ".DBAccess->quote('login_check')." and class_name = ".DBAccess->quote('Session');

    if (defined($params_operation_name) && $params_operation_name ne '') {
        $where .= " and eu.operation_name = '$params_operation_name'";
    }


    my $title = $params{title} || "Login statistics by User";
    my $sub_title = "Name: $params_username";

    ## DETERMINE ORDER BY
    my $reverse_order = 1;
    my $order_by = 'order by ';
    if ($sort_by eq 'date') {
        $order_by .= 'date_and_time asc';
        $reverse_order = 0;
    }
    else {
        $order_by .= 'date_and_time desc';
    }

    my ($total_rows) = DBAccess->sqlSelectCountDistinct($total_rows_fields, $tables, $where);
    #$self->debug_msg(1,"query: select distinct count($total_rows_fields) from $tables where $where\n");
    my $records = DBAccess->sqlSelectMany($select_fields, $tables, $where, "$order_by");

    my @entries = ();
    my $object_name = '';
    my $total_hits = 0;
    my $report_user_name = '';
    my $report_class_name = '';
    my $report_operation_name = '';
    my @report_timestamps = ();
    my @timestamps = ();
    my $ts;
    my $count = 1;
    my $date_and_time;

    while( (my ($userid, $user_name, $timestamp_field, $date_and_time) = $records->fetchrow()) && ($i < $nb ) ){
        if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC
        @timestamps = split(",", $timestamp_field);
        if( $reverse_order ){
            @timestamps = reverse @timestamps;
        }
        $ts = '';
        foreach $ts(@timestamps){
            $ts = Utility->format_unix_timestamp($ts, 'b e, Y T');
            #push (@entries,$count,$userid,$user_name,$ts);
            push (@entries,$user_name,$ts);
            $count++;
        }
    }

    return { headers => \@headers, entries => \@entries, title => $title, sub_title => $sub_title, excel_link => "$url3&amp;report_format=excel"};
}


sub get_group_result_set {
    my $self = shift;
    my %params = @_;

    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_group_result_set()");

    my $report_type     = $params{report_type};
    my $report_format   = $params{report_format};
    my $year_month_date = $params{year_month_date} || $FORM{year_month_date} || $self->get_current_year_month_date();
    my $nb              = $params{nb} || $FORM{nb} || $NB;
    my $start_from      = $params{from} || $FORM{from} || 0;
    my $op              = $params{op}   || $FORM{op};
    my $sort_by         = $params{sort_by} || $FORM{sort_by};
    my $params_grp_id   = $params{grp_id} || $FORM{grp_id};
    my $params_grp_ids  = $params{grp_ids}; ## limit search to these values
    $self->debug_msg(4, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_group_result_set(): report_type:$report_type params_grp_id:$params_grp_id year_month_date:$year_month_date nb:$nb start_from:$start_from op:$op sort_by:$sort_by");
    my $isa = $self->is_a();
    my $url0 = "index.pl?isa=$isa&amp;op=$op&amp;year_month_date=$year_month_date";
    my $url1 = $url0 . "&amp;report_type=$report_type";
    my $url2 = $url1 . "&amp;grp_id=$params_grp_id";
    my $url3 = $url2 . "&amp;sort_by=$sort_by";
    my @headers = ();

    my $hits_title  = $params{header_name_map}{_HITS_} || 'Hits';
    my $group_title = $params{header_name_map}{_GROUP_} || 'Group';
    my $detail_report_type = $params{detail_report_type};

    if ($report_format eq 'excel') {
        @headers = ($hits_title, $group_title);
    } else {
        @headers = ("<a href=\"$url2&amp;sort_by=hits\">$hits_title</a>", "<a href=\"$url2&amp;sort_by=name\">$group_title</a>");
    }

    my $from = $start_from;
    my $i = 0;

    my $group_by = 'group by ';
    my $total_rows_fields = '';
    my $select_fields     = "grp_id, group_name, sum(monthly_total) sum_monthly_total";
    my $where = "year_month_date = $year_month_date";

    my $title = $params{title} || '';
    #my $summary_detail_link = '';
    if ($report_type eq 'group_detail' || $params{do_detail}) {
        #$self->debug_msg(1,"this is the detail for the group\nreport[$report_type]");
        $select_fields     .= ", class_name, operation_name, instance_id, instance_name";
        $total_rows_fields = "grp_id, group_name, class_name, operation_name, instance_id, instance_name";
        $group_by          .= "grp_id, group_name, class_name, operation_name, instance_id, instance_name";
        if (defined($params_grp_id) && $params_grp_id ne '') {
            $where .= " and grp_id = $params_grp_id";
        } elsif (defined($params_grp_ids->[0])) {
            $where .= " and grp_id IN (" . join(", ", @$params_grp_ids) . ")";
        }

        if (defined $params{operation_name}) {
            $where .= ' AND operation_name='.DBAccess->quote($params{operation_name});
        }

        unless ($params{simple_headers}) {
        if ($report_format eq 'excel') {
            push (@headers, ('Object Name (id)', 'Object Class', 'Operation'));
        } else {
            push (@headers, ("<a href=\"$url2&amp;sort_by=object_name\">Object Name (id)</a>", "<a href=\"$url2&amp;sort_by=object_class\">Object Class</a>", "<a href=\"$url2&amp;sort_by=operation\">Operation</a>"));
        }
        }
        $title ||= "Usage Statistics by Group & Operation";
        #$summary_detail_link = "<a href=\"$url0&amp;report_type=group_summary\">Summary</a>";
    } else {
        $total_rows_fields = "grp_id, group_name";
        $group_by          .= "grp_id, group_name";
        $title ||= "Usage Statistics by Group";
        #$summary_detail_link = "<a href=\"$url0&amp;report_type=group_detail\">Detail</a>";
        if (defined($params_grp_ids->[0])) {
            $where .= " and grp_id IN (" . join(", ", @$params_grp_ids) . ")";
        }
    }

    my $order_by = 'order by ';
    if ($sort_by eq 'name') {
        $order_by .= 'group_name';
    } elsif ($sort_by eq 'object_name') {
        $order_by .= 'instance_name, sum_monthly_total desc';
    } elsif ($sort_by eq 'object_class') {
        $order_by .= 'class_name, sum_monthly_total desc';
    } elsif ($sort_by eq 'operation') {
        $order_by .= 'operation_name, sum_monthly_total desc';
    } else {
        $order_by .= 'sum_monthly_total desc';
    }

    my ($total_rows) = DBAccess->sqlSelectCountDistinct($total_rows_fields, "event_log_monthly_group_stats", $where);
    my $records = DBAccess->sqlSelectMany($select_fields, "event_log_monthly_group_stats", $where, "$group_by $order_by");

    my @entries = ();

    my $report_group_name = '';
    my $object_name = '';
    my $total_hits = 0;
    my $hits_url = '';
    while((my ($grp_id, $group_name, $sum_monthly_total, $class_name, $operation_name, $instance_id, $instance_name)=$records->fetchrow()) && ($i < $nb)) {
        if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC
        $report_group_name = $group_name;


        $total_hits += $sum_monthly_total;
        if ($report_type ne 'group_detail' && defined($grp_id) && $grp_id ne '' && $report_format ne 'excel') {
            #$hits_url = "$url0&amp;instance_id=$instance_id&amp;class_name=$class_name";
            $hits_url = "$url0&amp;grp_id=$grp_id";
            if ($report_type eq 'group_detail' || $params{do_detail}) {
                $hits_url .= "&amp;report_type=".($detail_report_type || 'user_detail')."&amp;instance_id=$instance_id&amp;class_name=$class_name&amp;operation_name=$operation_name";
            } else {
                $hits_url .= "&amp;report_type=".($detail_report_type || 'group_detail');
            }
            $sum_monthly_total = "<a href=\"$hits_url\">$sum_monthly_total</a>";
        }

        push (@entries, $sum_monthly_total, $group_name);

        if ($report_type eq 'group_detail') {
            $object_name = $instance_name || $NA;
            $object_name .= " ($instance_id)" if $instance_id ne '';
            push (@entries, $object_name, $class_name, $operation_name);
        }
        $i++;
    }
    if ($report_format eq 'excel') {
        push (@entries, $total_hits, 'Total');
    } else {
        push (@entries, $total_hits, '<b>Total</b>');
    }

    my ($pn) = Components->get_previous_next_button($nb, $i, $total_rows, $start_from, $url3); #PREVIOUS NEXT MAGIC

    my @sub_title = ();
    push (@sub_title, "Group Name: $report_group_name") if defined($params_grp_id) && $report_group_name ne '';
    my $sub_title = join(", ", @sub_title);
    return { headers => \@headers, entries => \@entries, pn => $pn, title => $title, sub_title => $sub_title, footer => '<i>The number of hits for groups may seem larger than expected because users can belong to multiple groups.</i>', excel_link => "$url3&amp;report_format=excel"};
}

sub get_login_result_set {
    my $self = shift;
    my %params = @_;

    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_login_result_set()");

    my $report_type     = $params{report_type};
    my $report_format   = $params{report_format};
    my $year_month_date = $params{year_month_date} || $FORM{year_month_date} || $self->get_current_year_month_date();
    my $nb              = $params{nb} || $FORM{nb} || $NB;
    my $start_from      = $params{from} || $FORM{from} || 0;
    my $op              = $params{op}   || $FORM{op};
    my $sort_by         = $params{sort_by} || $FORM{sort_by};
    my $params_user_id   = $params{userid} || $FORM{userid};
    #my $params_grp_ids  = $params{grp_ids}; ## limit search to these values

    $self->debug_msg(4, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_login_result_set(): report_type:$report_type params_user_id:$params_user_id year_month_date:$year_month_date nb:$nb start_from:$start_from op:$op sort_by:$sort_by");

    my $isa = $self->is_a();
    my $url0 = "index.pl?isa=$isa&amp;op=$op&amp;year_month_date=$year_month_date";
    my $url1 = $url0 . "&amp;report_type=$report_type";
    my @headers = ();

    my $hits_title  = $params{header_name_map}{_HITS_} || 'Hits';
    my $user_title = $params{header_name_map}{_USER_} || 'User Name';
    my $detail_report_type = $params{detail_report_type};

    if ($report_format eq 'excel') {
        @headers = ($hits_title, $user_title);
    } else {
        @headers = ("<a href=\"$url1&amp;sort_by=hits\">$hits_title</a>", "<a href=\"$url1&amp;sort_by=name\">$user_title</a>");
    }


    my $order_by = 'order by ';
    if ($sort_by eq 'name') {
        $order_by .= 'user_name asc';
    } elsif ($sort_by eq 'hits') {
        $order_by .= 'total asc';
    } else {
        $order_by .= 'total desc';
    }

    my $from = $start_from;
    my $i = 0;

    my $total_rows_fields = 'userid';
    my $select_fields     = "userid, user_name, sum(total) total";
    my $where = "year_month_date = $year_month_date and operation_name = ".DBAccess->quote('login_check')." and class_name = ".DBAccess->quote('Session');

    my $group_by = 'group by user_name,userid';
    my ($total_rows) = DBAccess->sqlSelectCountDistinct($total_rows_fields, "event_log_report_builder", $where);
    my $records = DBAccess->sqlSelectMany($select_fields, "event_log_report_builder", $where, " $group_by $order_by");

    $self->debug_msg(4,"select $select_fields from event_log_report_builder $where $group_by $order_by");

    my @entries = ();
    my $total_entry;
    my $hits_url;
    my $title = $params{title} || '';

    $self->debug_msg(4,"this is the result detail report[$report_type]\n");

    if ($report_type eq 'user_login_details' || $params{do_detail}) {
        $total_rows_fields = "userid, user_name, operation_name";
        if (defined($params_user_id) && $params_user_id ne '') {
            $where .= " and userid = $params_user_id";
        }

        unless ($params{simple_headers}) {
           if ($report_format eq 'excel') {
              push (@headers, ('Object Name (id)', 'Object Class', 'Operation'));
           } else {
              push (@headers, ("<a href=\"$url1&amp;sort_by=hits\">Login</a>", "<a href=\"$url1&amp;sort_by=name\">Name</a>"));
           }
        }
        $title ||= "Login Statistics by User in Detail";
    } else {
        $total_rows_fields = "userid, user_name";
        $group_by          .= "userid, user_name";
        $title ||= "Login Statistics by User";
    }

    $self->debug_msg(4,"order_by[$order_by]\n");

    while( (my ($userid, $user_name, $log_attempts) = $records->fetchrow() )  && ($i < $nb)) {
       if ($from) {$from--; next;}
       $self->debug_msg(4,"\nuserid[$userid], username[$user_name], attempts[$log_attempts]\n");

       $hits_url = $url0 . "&amp;userid=$userid;&amp;user_name=$user_name&amp;report_type=user_login_details";
       $total_entry = "<a href=\"$hits_url\">$log_attempts</a>";
       if( $report_format eq 'excel' ){
           $total_entry = $log_attempts;
       }
       else {
           $user_name .= "&nbsp;-&nbsp;[<a href=\"index.pl?isa=$isa&amp;op=view_user_profile&amp;uid=$userid\">view profile</a>]";
       }
       push (@entries,$total_entry,$user_name);

       $i++;

    }

    my ($pn) = Components->get_previous_next_button($nb, $i, $total_rows, $start_from, $url1); #PREVIOUS NEXT MAGIC

    return { headers => \@headers, entries => \@entries, pn => $pn, title => $title, excel_link => "$url1&amp;report_format=excel"};
}

sub get_object_result_set {
    my $self = shift;
    my %params = @_;

    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_object_result_set()");

    my $report_type     = $params{report_type};
    my $report_format   = $params{report_format};
    my $year_month_date = $params{year_month_date} || $FORM{year_month_date} || $self->get_current_year_month_date();
    my $nb              = $params{nb} || $FORM{nb} || $NB;
    my $start_from      = $params{from} || $FORM{from} || 0;
    my $op              = $params{op}   || $FORM{op};
    my $sort_by         = $params{sort_by} || $FORM{sort_by};
    my $params_instance_id   = $params{instance_id} || $FORM{instance_id};
    my $params_class_name = $params{class_name} || $FORM{class_name};
    $self->debug_msg(4, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_object_result_set(): report_type:$report_type params_instance_id:$params_instance_id params_class_name:$params_class_name year_month_date:$year_month_date nb:$nb start_from:$start_from op:$op sort_by:$sort_by");
    my $isa = $self->is_a();
    my $url0 = "index.pl?isa=$isa&amp;op=$op&amp;year_month_date=$year_month_date";
    my $url1 = $url0 . "&amp;report_type=$report_type";
    my $url2 = $url1 . "&amp;instance_id=$params_instance_id&amp;class_name=$params_class_name";
    my $url3 = $url2 . "&amp;sort_by=$sort_by";
    my @headers = ();
    if ($report_format eq 'excel') {
        @headers = ('Hits', 'Object Name (id)', 'Object Class');
    } else {
        @headers = ("<a href=\"$url2&amp;sort_by=hits\">Hits</a>", "<a href=\"$url2&amp;sort_by=object_name\">Object Name (id)</a>", "<a href=\"$url2&amp;sort_by=object_class\">Object Class</a>");
    }


    my $from = $start_from;
    my $i = 0;

    my $group_by = 'group by ';
    my $total_rows_fields = '';
    my $select_fields     = "instance_id, instance_name, class_name, sum(monthly_total) sum_monthly_total";
    my $where = "year_month_date = $year_month_date";
    my $title = '';
    #my $summary_detail_link = '';
    if ($report_type eq 'object_detail') {
        $select_fields     .= ", operation_name";
        $total_rows_fields = "instance_id, instance_name, class_name, operation_name";
        $group_by          .= "instance_id, instance_name, class_name, operation_name";
        if (defined($params_instance_id) && $params_instance_id ne '') {
            $where .= " and instance_id = '$params_instance_id'";
        }
        if (defined($params_class_name) && $params_class_name ne '') {
            $where .= " and class_name = '$params_class_name'";
        }
        if ($report_format eq 'excel') {
            push (@headers, ('Operation'));
        } else {
            push (@headers, ("<a href=\"$url2&amp;sort_by=operation\">Operation</a>"));
        }
        $title = "Usage Statistics by URL and Operation";
        #$summary_detail_link = "<a href=\"$url0&amp;report_type=object_summary\">Summary</a>";
    } else {
        $total_rows_fields = "instance_id, instance_name, class_name";
        $group_by          .= "instance_id, instance_name, class_name";
        $title = "Usage Statistics for " . $PARAMS->{hostname};
        #$summary_detail_link = "<a href=\"$url0&amp;report_type=object_detail\">Detail</a>";
    }

    ## DETERMINE ORDER BY
    my $order_by = 'order by ';
    if ($sort_by eq 'object_name') {
        $order_by .= 'instance_name, sum_monthly_total desc';
    } elsif ($sort_by eq 'object_class') {
        $order_by .= 'class_name, sum_monthly_total desc';
    } elsif ($sort_by eq 'operation') {
        $order_by .= 'operation_name, sum_monthly_total desc';
    } else {
        $order_by .= 'sum_monthly_total desc';
    }

    my $total_rows = DBAccess->sqlSelectCountDistinct($total_rows_fields, "event_log_monthly_object_stats", $where);
    my $records = DBAccess->sqlSelectMany($select_fields, "event_log_monthly_object_stats", $where, "$group_by $order_by");

    my @entries = ();

    my $report_instance_name = '';
    my $hits_url = '';
    my $object_name = '';
    my $total_hits = 0;
    while((my ($instance_id, $instance_name, $class_name, $sum_monthly_total, $operation_name)=$records->fetchrow()) && ($i < $nb)) {
        if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC
        $report_instance_name = "$instance_name";
        $total_hits += $sum_monthly_total;
        if (defined($instance_id) && $instance_id ne '' && $report_format ne 'excel') {
            $hits_url = "$url0&amp;instance_id=$instance_id&amp;class_name=$class_name";
            if ($report_type eq 'object_detail') {
                $hits_url .= "&amp;report_type=user_detail&amp;operation_name=$operation_name";
            } else {
                $hits_url .= "&amp;report_type=object_detail";
            }
            $sum_monthly_total = "<a href=\"$hits_url\">$sum_monthly_total</a>";
        }
        $object_name = $instance_name || $NA;
        $object_name .= " ($instance_id)" if $instance_id ne '';
        push (@entries, $sum_monthly_total, $object_name, $class_name);

        if ($report_type eq 'object_detail') {
            push (@entries, $operation_name);
        }
        $i++;
    }
    if ($report_format eq 'excel') {
        push (@entries, $total_hits, 'Total');
    } else {
        push (@entries, $total_hits, '<b>Total</b>');
    }

    my ($pn) = Components->get_previous_next_button($nb, $i, $total_rows, $start_from, $url3); #PREVIOUS NEXT MAGIC

    #$title .= defined($params_instance_id) && $params_instance_id ne '' ? ": $report_instance_name" : '';
    my @sub_title = ();
    push (@sub_title, "Object Name: $report_instance_name") if defined($params_instance_id) && $report_instance_name ne '';
    my $sub_title = join(", ", @sub_title);
    return { headers => \@headers, entries => \@entries, pn => $pn, title => $title, sub_title => $sub_title, footer => '<i>Some hits are not clickable because their corresponding Object has an undefined id.</i>', excel_link => "$url3&amp;report_format=excel"};
}


sub get_current_year_month_date {
    my $self = shift;
    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_current_year_month_date()");

    return Utility->format_unix_timestamp(time, 'Ym');
}

sub format_year_month_date {
    my $self = shift;
    my %args = @_;
    my $year_month_date = $args{year_month_date};
    my $format          = $args{format} || 'M Y';
    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::format_year_month_date()");

    my $year = substr($year_month_date, 0, 4);
    my $month = substr($year_month_date, 4, 2);

    $self->debug_msg(4, "Metadot::SystemApp::EventLog::ReportGenerator::Default::format_year_month_date(): year_month_date:$year_month_date year:$year month:$month");
    return Utility->format_datetime("$year-$month-01 00:00:00", $format);
}

############################################
#######           END                #######
####### METHODS FOR CREATING REPORTS #######
#######                              #######
############################################

sub www_change_reports_year_month_date {
    my $self = shift;
    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::www_change_reports_year_month_date()");

    my $isa = $self->is_a();

    my $year_month_date = $FORM{year_month_date} || $self->get_current_year_month_date();
    my $template = Template->new({
                  INCLUDE_PATH => 'Metadot/SystemApp/EventLog/ReportGenerator/Default/templates',
                 }) || die $Template::ERROR, "\n";

    my $last_generated = Utility->format_unix_timestamp($PARAMS->{clean_timecounter}, "M e, Y r");
    my $purge_link = $self->is_allowed_to_do('purge_stats', $USER) ? "index.pl?isa=$isa&amp;op=purge_stats&amp;year_month_date=$year_month_date" : '';
    # data for the template
    my $data = {
                menu_link   => "index.pl?isa=$isa&amp;op=change_reports_year_month_date&amp;year_month_date=$year_month_date",
                year_month_dates => $self->get_distinct_year_month_dates(),
                year_month_date_link => "index.pl?isa=$isa&amp;op=show_report&amp;report_type=object_summary",
                last_generated => $last_generated,
                purge_link => $purge_link,
                additional_menu_items => $self->get_additional_menu_items(),
                PARAMS => $PARAMS,
               };

    my $output;
    $template->process('ChangeReportsYearMonthDate.tt2', $data, \$output)
    || die $template->error(), "\n";

    return $self->print_portal($output);
}

sub get_additional_menu_items {
    my $self = shift;
    return '';
}

sub get_distinct_year_month_dates {
    my $self = shift;
    my %args = @_;
    my $date_format = $args{date_format} || '';
    my $sort = $args{sort} || 'desc';

    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::get_distinct_year_month_dates()");

    my $dates = DBAccess->sqlSelectMany("distinct(year_month_date)", "event_log_monthly_user_stats", "1=1", "order by year_month_date $sort");

    my @year_month_dates = ();
    my $year = 0;
    my $month = 0;
    my $formatted_date = '';
    while (my($year_month_date) = $dates->fetchrow) {
        $year = substr($year_month_date, 0, 4);
        $month = substr($year_month_date, 4, 2);
        $formatted_date = $date_format ne '' ?  $self->format_year_month_date(year_month_date => $year_month_date, format => $date_format) : $year_month_date;
        push @year_month_dates, {
            year_month_date => $year_month_date,
            year => $year,
            month => $month,
            month_display => $self->format_year_month_date(year_month_date => $year_month_date, format => 'M'),
            formatted_date => $formatted_date,
           };
    }
    return \@year_month_dates;
}

sub www_view_user_profile {
    my $self = shift;
    $self->debug_msg(3, "Metadot::SystemApp::EventLog::ReportGenerator::Default::www_view_user_profile()");

    my $uid = $FORM{uid};

    return $USER->www_show_details($uid);
}





















































1;

__END__
