###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::SystemApp::FormDemo;

# ========================================
# This is a system app used to demo the form builder.  This demo loads a description of
# the form elements and variables from an .xml file, from that builds a data set for the
# form template, and passes it to the template.  The result string is returned and, as
# a system app, rendered within the context of the site's headers and footers.

# The app also includes a 'process' method, which demos how one might
# hook into the FormProcess methods.  FormProcess supports handling of
# the form results.  It uses the form field names, which are
# associated with the db pathnames for the table fields, in
# conjunction with information about the tables included in the form
# (automatically generated by FormBuilder), to  automatically
# generate db new objects or modify existing ones.

# ========================================

use strict;
use Data::Dumper;

#use Carp;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT);
use Metadot::SystemApp;
use Metadot::FormBuilder;

use vars qw(@ISA);
@ISA=qw(Metadot::SystemApp Auditable);

#
# Substitute for an Operations object
#
my $AC = {
      BUNDLES => {
              DISP =>   { DESC =>          'View',
                  OPS =>            [qw( show process)],
                  DEFAULT_LEVEL =>  2,
                  MIN_LEVEL =>      0,
                  DEFAULTS_TO =>    'on',
                },
              EDITP =>  {
                 DESC =>          'Change<br />Permissions',
                 OPS =>            [qw(edit_permissions set_permissions)],
                 DEFAULT_LEVEL =>  8,
                 MIN_LEVEL =>      8,
                },
             },

      DISPLAY_ORDER => [ qw(DISP EDITP) ],
     };


# should always be overridden in derived classes
sub get_AC {
    my $self = shift;
    my $class = ref $self || $self;

    die "&get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

    return $AC;
}


sub name {

    return 'FormDemo'
}


sub can_change_owner {

    return undef;
}


sub is_a {
    my $self = shift;

    my $class = ref($self) || $self;

    return $class
}


sub main {
    my $self = shift;

    my $fb = Metadot::FormBuilder->new();

    my $mode = $FORM{mode};
    my $form_descr = $FORM{form_descr};
    #print STDERR "pkey info: ", Dumper($FORM{pkey}), "\n";
    #unpack pkey info
    my $pkeys = {};
#  my $pkeys = { 'user.uid' => 1034, 'instance.iid' => 1758};
    # expects list, not single key. tbd: add code to check for single case
    if (exists $FORM{pkey}) {
    my $pkl = $FORM{pkey};
    foreach my $item (@$pkl) {  #  extract the key/value info
        $item =~ /^(.*\..*),(.*)/;
        $pkeys->{$1} = $2;
    }
    }
    #print STDERR "got pkeys: ", Dumper($pkeys), "\n";

    # load the form spec. info from an xml file.  Note that WHICH xml file to used, is
    # passed in as a cgi var.
    my $dref = $fb->getFormDescription("Metadot/form_config/demos/$form_descr.xml");
    #print STDERR "got descr " , Dumper($dref), "\n";
    # mode is passed in as one of the cgi variables.  The demo url that goes with this file
    # sets it to 'edit'.  See FormDemo2 as an example of using 'create' mode.
    $fb->setMode($mode);

#      $fb->processFormElementSpecs();
#      $fb->processFormVarSpecs($pkeys);
    $fb->processFormDescription($pkeys);

    # generate the html string for the form, using its template
    my $htmlstring = $fb->buildTemplateString('Metadot/templates/demos', 'fb_msg.tt2');

    # return the result
    return $htmlstring;
}

#in the demo, the form action is set to call this method.  It
#calls the saveAllTableFieldValues method of FormProcess.
sub www_process {
    my $self = shift;
    #print STDERR "in www_process\n";
    my $fp = Metadot::FormProcess->new();
    $fp->saveAllTableFieldValues();
    my $content = "<p />ran save process...\n";

    $self->print_http_header();
    print $self->show_page_top(),
    $self->show_header( 0 ),
        $self->show_portal_panel( 0 ),
        $self->show_panel(),
            $content,
            $self->show_bottom;

}



1;
