###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::SystemApp::FormDemo2;

# ========================================
# This is a system app used to demo the form builder.  This demo does not load an .xml file,
# but instead uses a set of method calls to specify the form element descriptors and variables.
# After setting the form config info, it gets a data set for the form template,
# and passes it to the template.
# The result string is returned and, as a system app, rendered within the context of the
# site's headers and footers.

# The app also includes a 'process' method, which demos how one might
# hook into the FormProcess methods.  FormProcess supports handling of
# the form results.  It uses the form field names, which are
# associated with the db pathnames for the table fields, in
# conjunction with information about the tables included in the form
# (automatically generated by FormBuilder), to  automatically
# generate db new objects or modify existing ones.

# ========================================

use strict;
#  use CGI;
use Data::Dumper;
#use Time::localtime;

use Carp;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT);
use Metadot::SystemApp;
use Metadot::FormBuilder;
use Metadot::FormProcess;
use Metadot::JSForm ;

use vars qw(@ISA);
@ISA=qw(Metadot::SystemApp Auditable);

#
# Substitute for an Operations object
#
my $AC = {
      BUNDLES => {
              DISP =>   { DESC =>          'View',
                  OPS =>            [qw( show process )],
                  DEFAULT_LEVEL =>  2,
                  MIN_LEVEL =>      0,
                  DEFAULTS_TO =>    'on',
                },
              EDITP =>  {
                 DESC =>          'Change<br />Permissions',
                 OPS =>            [qw(edit_permissions set_permissions)],
                 DEFAULT_LEVEL =>  8,
                 MIN_LEVEL =>      8,
                },
             },

      DISPLAY_ORDER => [ qw(DISP EDITP) ],
     };


# should always be overridden in derived classes
sub get_AC {
    my $self = shift;
    my $class = ref $self || $self;

    die "&get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

    return $AC;
}


sub name {

    return 'FormDemo'
}


sub can_change_owner {

    return undef;
}


sub is_a {
    my $self = shift;

    my $class = ref($self) || $self;

    return $class
}

# 'main' is called by www_show in Metadot::SystemApp, this classes' parent class.
sub main {
    my $self = shift;

    my $fb = Metadot::FormBuilder->new();
#      $fb->initJSFormObj();

    my $mode = $FORM{mode};
    #print STDERR "pkey info: ", Dumper($FORM{pkey}), "\n";
    #unpack pkey info
    my $pkeys = {};
    if (exists $FORM{pkey}) {
    my $pkl = $FORM{pkey};
    foreach my $item (@$pkl) {  #  extract the key/value info
        $item =~ /^(.*\..*),(.*)/;
        $pkeys->{$1} = $2;
    }
    }
#      print STDERR "got pkeys: ", Dumper($pkeys), "\n";

    # set form name and mode. 'true' means attach automatic popup calendars to date fields.
    $fb->setFormName('tableform', 'true');
    # mode is passed in as one of the cgi variables.  The demo url that goes with this file
    # sets it to 'create'.  See FormDemo as an example of using 'edit' mode.
    $fb->setMode($mode);
    # add info about the tables that the form builder code will need to build fields
    # for.  In this example, it is just the message table.  If the form mode is set to
    # 'create', then this means that a new message object is being created, not an existing
    # one edited.   [This demo only exposes one field of the 'message' table, message.authoremail,
    # so in that sense it is not a great example of a create form.  But obviously any
    # number of other message table fields could be added to the form.
    $fb->addTableInfo('message', 'mid');
    $fb->addConfigAttrValue('intro_text',
        'This is some introductory text at the top of the form, generated dynamically.');

    $fb->processFormElementSpecs();

    # add a set of element descriptors.
    $fb->addTextElementDescriptor('submit', 'submit', '',  {value => 'submit this'});
    $fb->addTextElementDescriptor('message.iid', 'text', 'message id',  {check_type => 'notempty'});
    $fb->addTextElementDescriptor('text2', 'textarea', 'text area label',
                  {value => 'this is some default text', , check_type => 'notempty'});

    my $buttonlist = [{label => 'yes', value => 1}];
    $fb->addRadioCheckboxElementDescriptor('adminuser', 'checkbox', 'is user an admin?', $buttonlist, {});

    my $valuelist = [ {label => 'Billy Bob', value => 'bob'}, {label => 'Fred', value => 'fred'},
              {label => 'Alonzo', value => 'al'}];
    $fb->addSelectElementDescriptorStaticValue('selecttest', 'pick a nickname', 0, $valuelist,
                           {size => 2});

    $fb->addSelectElementDescriptorDynamicValue('message.authoremail', 'pick an email address',
                        0, "select one...",
                        'select email, email from user', {});
    # add a set of variable mappings, associating element descriptor names with
    # variable names.
    $fb->addAtomicFormVar('text2', 'text2');
    $fb->addAtomicFormVar('submit', 'submit');
    $fb->addAtomicFormVar('adminuser', 'adminuser');
    $fb->addAtomicFormVar('selecttest', 'selecttest');
    $fb->addAtomicFormVar('email', 'message.authoremail');
    $fb->addAtomicFormVar('m_iid', 'message.iid');


    # while it is not used in this form, here is the syntax that would be used to
    # add a list variable mapping.  List variables associate the fields from one or
    # more tables with a form variable.  The list can be tweaked by excluding a given
    # list of fields for each table, and by adding additional form element descriptors
    # to the list as well.
#  $fb->addListFormVar('list1', {'user' => 'linkurl1,linkurl2,linkurl3,linkurl4'},
#           'instance.parentiid,instance.description');

    # process the form var specs.
    $fb->processFormVarSpecs($pkeys);

    $fb->setFormVarItemValue('adminuser', 1);
    # the following overrides the default text set above for this variable (done just as
    # an example).
    $fb->setFormVarItemValue('text2', 'Some default stuff');


    # from the specification created above, get the data structure to use with
    # the template.
    my $data =  $fb->getFormTemplateData();
#      print STDERR "****got final form vars data: ", Dumper($data), "\n";
    # create a new template
    my $template = Template->new({
    INCLUDE_PATH => 'Metadot/templates/demos',
    }) || die $Template::ERROR, "\n";
    my $htmlstring;
    # generate html from the template and the data structure
    $template->process('fb_msg2.tt2', $data, \$htmlstring)
    || die $template->error(), "\n";
    my $jsform = $fb->getJSFormObj();
    if ($jsform) {
    my $jsformcontent = $jsform->get_form($htmlstring, $jsform->get_ALL_ERRORS()) ;
    # return the result
    return $jsformcontent;
    }
    else {
    return $htmlstring;
    }
}

#in the demo, the form action is set to call this method.  It
#calls the saveAllTableFieldValues method of FormProcess.
sub www_process {
    my $self = shift;
    #print STDERR "in www_process\n";
    my $fp = Metadot::FormProcess->new();
    $fp->saveAllTableFieldValues();
    my $content = "<p />ran save process...\n";

    $self->print_http_header();
    print $self->show_page_top(),
    $self->show_header( 0 ),
        $self->show_portal_panel( 0 ),
        $self->show_panel(),
            $content,
            $self->show_bottom;

}

1;
