###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::SystemApp::MetagramCenter;
use strict;

use Template;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY %REQUEST_VARS);
use Metadot::SystemApp;
use Metadot::Report;
use Metadot::Metagram;
use Metadot::Invitable;
use Metadot::Controllable;
use MetadotConfig;
use Message;
use Utility;
#use DBAccess;

use vars qw(@ISA);
@ISA=qw(Metadot::SystemApp Auditable Metadot::Report Metadot::Invitable Metadot::Controllable);

#
# Substitute for an Operations object
#
my $AC = {
    BUNDLES => {
        DISP =>   { DESC =>          'View',
            OPS =>            [qw( show _process_sort_input process_metagrams show_metagram delete_individual_metagram sort_by_sender )],
            DEFAULT_LEVEL =>  2,
            MIN_LEVEL =>      2,
            DEFAULTS_TO =>    'on',
        },
        EDITP =>  {
            DESC =>          'Change<br />Permissions',
            OPS =>            [qw(edit_permissions set_permissions)],
            DEFAULT_LEVEL =>  8,
            MIN_LEVEL =>      8,
        },
        COMPOSE => {
            DESC =>          'Compose metagrams',
            OPS =>            [qw(compose send reply)],
            DEFAULT_LEVEL =>  2,
            MIN_LEVEL =>      2,
        },
        MASS_MAIL => {
            DESC =>          'Send metagrams to multiple users',
            OPS =>            [qw(invite)],
            DEFAULT_LEVEL =>  9,
            MIN_LEVEL =>      9,
        },
    },

    DISPLAY_ORDER => [ qw(DISP EDITP) ],
     };

# should always be overridden in derived classes
sub get_AC {
    my $self = shift;
    my $class = ref $self || $self;

    die "&get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

    return $AC;
}

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $self  = $class->SUPER::new();

    $self->{_USE_SUPER_PORTAL} = 1;
    bless ($self, $class);

    return $self;
}


sub name {
    return ''
}

sub get_app_name {
    return MetadotConfig->get_param_value('metagram_center_name') || 'My Metagrams';
}

sub get_metagram_label {
    return Metadot::Metagram->get_label();
}


sub can_change_owner {

    return undef;
}


sub is_a {
    my $self = shift;

    my $class = ref($self) || $self;

    return $class
}

sub main {
    my $self = shift;
    return $self->get_app_name() . " only supported in Skins.";
}


##############################
### Process Metagrams
##############################
sub www_process_metagrams {
    my $self = shift;
    $self->debug_msg(3, "Metadot::SystemApp::MetagramCenter::www_process_metagrams()");

    my $isa = $self->is_a();
    my $show_link = "index.pl?isa=$isa&op=show";

    if (!defined $FORM{mid} || $FORM{mid} eq '') {
        $DISPLAY->print_click_back('Please select the messages you wish to update or delete.');
    } elsif ($FORM{mark_read}) {
        Metadot::Metagram->mark_metagrams_read({  mids => $FORM{mid}, uid => $USER->uid() });
        $DISPLAY->print_choices ("Success! Metagrams updated.",
                     "confirm",
                     "Continue",
                     $show_link,
                    )
    } elsif ($FORM{mark_unread}) {
        Metadot::Metagram->mark_metagrams_unread({  mids => $FORM{mid}, uid => $USER->uid() });
        $DISPLAY->print_choices ("Success! Metagrams updated.",
                     "confirm",
                     "Continue",
                     $show_link,
                    )
    } elsif ($FORM{delete}) {
        Metadot::Metagram->delete_metagrams({  mids => $FORM{mid}, uid => $USER->uid() });
        $DISPLAY->print_choices ("Success! Metagrams deleted.",
                     "confirm",
                     "Continue",
                     $show_link,
                    )
    } else {
        $DISPLAY->print_click_back("Error: bad submission");
    }

    exit;
}


sub www_send {
    my $self = shift;

    my $recipient_uid = $FORM{recipient_uid};
    my $subject       = $FORM{subject};
    my $body          = $FORM{body};
    my $landing_url    = $FORM{landing_url};
    my $parentmid     = $FORM{parentmid};

    my $mid = undef;

    if (defined $recipient_uid) {
        $mid = Metadot::Metagram->add_message(
                  name          => $subject,
                  description   => $body,
                  recipient_uid => $recipient_uid,
                  parentmid     => $parentmid,
               );

        ### Check if we need to send an email as well
        if( defined($PARAMS->{metagram_email_copy}) && $PARAMS->{metagram_email_copy} ){
           ## EMAIL AS WELL
           if( defined($mid) && $mid ne ''){
               ### SEND EMAIL
               my $message = Message->new($mid);
               $message->_send_metagram_email_copy($mid);
           }

        }


        my $class = ref($self) || $self;
        my @choices = ('Go to the Metagram Center', "index.pl?isa=$class",);
        if ($landing_url) {
            push @choices, 'Continue', $landing_url;
        }

        $DISPLAY->print_choices ("Your Metagram was sent!.",
                     "confirm",
                     @choices,
                    )
    } else {
        $DISPLAY->print_click_back("Error: No recipient!");
    }
}

sub www_delete_individual_metagram {
    my $self = shift;
    $self->debug_msg(3, "Metadot::SystemApp::MetagramCenter::www_delete_individual_metagram()");

    my $isa = $self->is_a();
    my $show_link = "index.pl?isa=$isa&op=show";
    my $next_mid  = $FORM{next_mid};
    $next_mid = undef unless Utility->is_valid_integer($next_mid); # secu_200401
    Metadot::Metagram->delete_metagrams({  mids => $FORM{mid}, uid => $USER->uid() });

    if (defined($next_mid)) {
        $show_link .= "_metagram&mid=$next_mid";
        Utility->md_redirect($show_link);
    } else {
        Utility->md_redirect($show_link);
    }
}




##############################
### Report Methods
##############################

sub get_columns {
    my $self = shift;
    $self->debug_msg(3, "Metadot::SystemApp::MetagramCenter::get_columns()");
    my @columns = (
        {
            name            => 'Sender',
            field           => 'authorname',
            sort_order      => 1,
            sort_priority   => 0,
            header_attributes => {width=>'100'},
        },
        {
            name            => 'Subject',
            field           => 'name',
            sort_order      => 1,
            sort_priority   => 0,
        },
        {
            name            => 'Date',
            field           => 'creationdate',
            sort_order      => 1,
            sort_priority   => 0,
            header_attributes => {width=>'15%', nowrap=>'nowrap'},
            body_attributes   => {nowrap=>'nowrap'},
        },
    );
    return \@columns;
}

sub process_sort_input {
    my $self = shift;
    $self->debug_msg(3, "Metadot::SystemApp::MetagramCenter::process_sort_input()");
    if (defined $FORM{sort_column}) {
        $self->sort_field($FORM{sort_column});
    }
    if (defined $FORM{sort_order}) {
        $self->sort_order($FORM{sort_order});
    }
}

sub get_data {
    my $self = shift;
    $self->debug_msg(3, "Metadot::SystemApp::MetagramCenter::get_data()");
    my %args = @_;

    my $where_clause = "recipient_uid=" . $USER->uid() . " and isa = 'Metadot::Metagram'";
    if ($args{where_clause}) {
        $where_clause .= " and $args{where_clause}";
    }

    my $order_by = '';
    if ($self->sort_field()) {
       $order_by = $self->sort_field();
       if ($self->sort_order() >= 0) {
           $order_by .= ' asc';
       } else {
           $order_by .= ' desc';
       }
    } else {
        # define the default
        $order_by =  'CreationDate DESC,'
                     . 'name ASC';
    }

    my $rs = DBAccess->sqlSelectMany("mid", "message", $where_clause, "order by
 $order_by");
    my ($count) = DBAccess->sqlSelect('count(*)', "message", $where_clause);

    my $result = {
        row_set     => $rs,
        row_count   => $count,
    };
    #use Data::Dumper;
    #print STDERR "get_data with $where_clause returns\n",Dumper $result;
    return $result;
}

sub get_next_previous_metagrams {
    my $self = shift;
    my $base_mid  = shift;
    $self->debug_msg(3, "Metadot::SystemApp::MetagramCenter::get_next_metagrams(base_mid:$base_mid)");

    $base_mid = undef unless Utility->is_valid_integer($base_mid); # secu_200401
    return undef unless defined($base_mid);

    my $result = $self->get_data();

    my $rs = $result->{row_set};

    my $previous_mid;
    my $next_mid;
    my $exit_after_next = 0;
    while (my $mid = $rs->fetchrow) {
        if ($exit_after_next) {
           $next_mid = $mid;
           last;
        } elsif ($mid == $base_mid) {
           $exit_after_next = 1;
        } else {
            $previous_mid = $mid;
        }
    }
    return [$previous_mid, $next_mid];
}

sub generate_sort_headerX {
    my $self = shift;
    my $column = shift;
    my $text = $column->{name};
    my $sort_field = $column->{field};
    my $sort_order = $column->{sort_order} || 1;
    my $result;

    unless ($sort_field) {
        return $text;
    }

    my $isa = $self->is_a();
    my $op  = 'show';

    # determine what image to use for indicate sort order
    my $sort_image;
    if ($self->sort_field() eq $sort_field) {
        if ($self->sort_order() >= 0) {
            $sort_image = "<img class=\"icon\" src=\"$PARAMS->{imageslocation}/obj/triangle_up.gif\" border=\"0\">";
        } else {
            $sort_image = "<img class=\"icon\" src=\"$PARAMS->{imageslocation}/obj/triangle_down.gif\" border=\"0\">";
        }
        $sort_order = -1 * $self->sort_order(); # toggle the sort
    } else {
        $sort_image = '';
    }

    $result = "<a href=\"index.pl?isa=$isa&amp;op=$op&amp;sort_order=$sort_order&amp;sort_field=$sort_field\">$text</a> $sort_image";
    return $result;
}

sub get_new_metagram {
    my $self = shift;
    my $mid  = shift;
    $self->debug_msg(3, "Metadot::SystemApp::MetagramCenter::get_new_metagram() mid:$mid");

    $mid = undef unless Utility->is_valid_integer($mid); # secu_200401
    return undef unless defined($mid);

    return Metadot::Metagram->new($mid);
}

sub sort_field {
    my $self = shift;
    $self->debug_msg(3, "Metadot::SystemApp::MetagramCenter::sort_field()");
    my $input = shift;

    if (defined $input) {
        $SESSION->set_value('metagram_sort_field', $input);
    }
    return $input || $SESSION->get_value('metagram_sort_field') || 'creationdate';
}

sub sort_order {
    my $self = shift;
    $self->debug_msg(3, "Metadot::SystemApp::MetagramCenter::sort_order()");
    my $input = shift;

    if (defined $input) {
        $SESSION->set_value('metagram_sort_order', $input);
    }
    return $input || $SESSION->get_value('metagram_sort_order') || -1;
}

##### INVITE overwritten methods
sub get_invite_interface_message {
    my $self = shift;
    $self->debug_msg(3, "Metadot::SystemApp::MetagramCenter::get_invite_interface_message()");

    my $s = <<EOM
<h1>Metagram users</h1>
Below, you can search for users you would like to metagram.
You may also metagram entire groups; Once
you've finished, click on the 'Next' button to send a metagram
to the people you have selected.
EOM
;
    return $s;
}

sub get_invite_title_keyword {
    return Metadot::Metagram->get_label();
}

sub get_invite_title_keyword_proper {
    return Metadot::Metagram->get_label_proper();
}

sub _get_invitation_subject_html_default_field_text {
    my $self = shift;
    my $reply_mid  = $FORM{reply_mid};
    $reply_mid = undef unless Utility->is_valid_integer($reply_mid); # secu_200401

    my $subject = '';

    if (defined($reply_mid)) {
        my $reply_metagram = $self->get_new_metagram($reply_mid);
        $reply_metagram->prepare_reply();
        $subject = $reply_metagram->name();

    }
    return $subject;
}

sub _get_invitation_body_html_default_field_text {
    my $self = shift;
    my $reply_mid  = $FORM{reply_mid};
    $reply_mid = undef unless Utility->is_valid_integer($reply_mid); # secu_200401

    my $message = '';

    if (defined($reply_mid)) {
        my $reply_metagram = $self->get_new_metagram($reply_mid);
        $reply_metagram->prepare_reply();
        $message = $reply_metagram->description();

    }
    return $message;
}

sub show_add_email_address_form {
    return '';
}

## metagram
sub send_invitation_notification {
    my ($self, %params) = @_;
    $self->debug_msg(3, "Metadot::SystemApp::MetagramCenter::send_invitation_notification()");

    my %recipients = %{$self->get_invite_recipients};
    my @uids = @{ $recipients{uid} };

    my $subject              = $params{subject};
    my $body                 = $params{body};
    my $cc_me                = $params{cc_me};

    if ($cc_me) {
        push (@uids, $USER->uid());
    }

    # process groups
    for my $group (@{ $recipients{group} } ) {
        push @uids, Group->get_member_uids($group, 1); ##recurse
    }

    @uids = @{Utility->uniquify_array(@uids)};


    foreach my $uid (@uids) {
        ## create metagram
        Metadot::Metagram->add_message(
                  userid        => $USER->uid(),
                  authorname    => $USER->get_display_name(),
                  authoremail   => $USER->email(),
                  #iid           => $id,
                  name          => $subject,
                  description   => $body,
                  recipient_uid => $uid,
            );
    }
}


sub _get_return_link_text {
}


##### END INVITE overwritten methods
##########################################
# exec methods service MetadotController
##########################################
#use constant _RESPONSE_SHOW_FULL_ => 'show_full';
#
#sub exec_show {
#    my $self    = shift;
#    my $context = shift;
#
#    return _RESPONSE_SHOW_FULL_;
#}

sub exec_show_metagram {
    my $self    = shift;
    my $context = shift;

    return 'show_metagram';
}


sub exec_compose {
    my $self = shift;

    return 'compose';
}

#sub exec_sort_by_sender {
#    my $self = shift;

#    return 'sort_by_sender';
#}

sub exec_reply {
    my $self = shift;

    my $reply_mid = $FORM{reply_mid};
    $reply_mid = undef unless Utility->is_valid_integer($reply_mid); # secu_200401

    my $reply_metagram = $self->get_new_metagram($reply_mid);
    $reply_metagram->prepare_reply();

    my $post_uid = $reply_metagram->postuid();
    my $user = Metadot::UserFactory->get_user( $post_uid );

    ## check if user still exists
    if ($user->uid()) {
        # create a metagram template to base the reply on
        my $metagram_template = {
            recipient   => $user,
            parentmid   => $reply_metagram->mid(),
            subject     => $reply_metagram->name(),
            body        => $user->fullname() . " wrote:\n\n".$reply_metagram->description(),
        };
        $REQUEST_VARS{metagram_template} = $metagram_template;

        return 'compose';
    } else {
        $DISPLAY->print_click_back("Error: User does not exist.");
        exit(0);
    }
}

sub get_metagrams_for_user {
    my $self = shift;
    my $uid  = shift || $USER->uid();

    my $recordset = DBAccess->sqlSelectMany(
                       'authorname,name,creationdate,status,mid,userid',
                       'message',
                       "recipient_uid = $uid AND isa = 'Metadot::Metagram'",
                       'order by authorname, creationdate desc'
                     );

    my @records;
    while (my ($author,$subject,$creation,$status,$mid,$userid) =
               $recordset->fetchrow()
          ) {
       my $metagram = {};

       ## Get the number of messages from this author
       my ($num) = DBAccess->sqlSelect(
                        'count(*)','message',
                        "recipient_uid = $uid AND isa = 'Metadot::Metagram'
                         AND authorname = '$author'"
                   );

       $metagram->{count}      = $num;
       $metagram->{author}     = $author;
       $metagram->{author_uid} = $userid;
       $metagram->{subject}    = $subject;
       $metagram->{creation}   = $creation;
       $metagram->{status}     = $status;
       $metagram->{mid}        = $mid;

       push @records,$metagram;
    }

    return \@records;
}

1;

__END__
