###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::SystemApp::MockupDemo;

# ========================================
# This is a system app used to demo the form builder.  This demo loads a description of
# the form elements and variables from an .xml file, from that builds a data set for the
# form template, and passes it to the template.  The result string is returned and, as
# a system app, rendered within the context of the site's headers and footers.

# ========================================

use strict;
use Data::Dumper;

#use Carp;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT);
use Metadot::SystemApp;
use Metadot::FormBuilder;

use vars qw(@ISA);
@ISA=qw(Metadot::SystemApp Auditable);

#
# Substitute for an Operations object
#
my $AC = {
      BUNDLES => {
              DISP =>   { DESC =>          'View',
                  OPS =>            [qw( show process)],
                  DEFAULT_LEVEL =>  2,
                  MIN_LEVEL =>      0,
                  DEFAULTS_TO =>    'on',
                },
              EDITP =>  {
                 DESC =>          'Change<br />Permissions',
                 OPS =>            [qw(edit_permissions set_permissions)],
                 DEFAULT_LEVEL =>  8,
                 MIN_LEVEL =>      8,
                },
             },

      DISPLAY_ORDER => [ qw(DISP EDITP) ],
     };


# should always be overridden in derived classes
sub get_AC {
    my $self = shift;
    my $class = ref $self || $self;

    die "&get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

    return $AC;
}


sub name {

    return 'FormDemo'
}


sub can_change_owner {

    return undef;
}


sub is_a {
    my $self = shift;

    my $class = ref($self) || $self;

    return $class
}


sub main {
    my $self = shift;

    my $fb = Metadot::FormBuilder->new();

    my $mode = $FORM{mode};
    my $form_descr = $FORM{form_descr};
    my $template = $FORM{template};

    # load the form spec. info from an xml file.  Note that WHICH xml file to use, is
    # passed in as a cgi var.
    my $dref = $fb->getFormDescription("Metadot/form_config/demos/$form_descr.xml");
#     print STDERR "got descr " , Dumper($dref), "\n";
    $fb->setMode($mode);
    $fb->processFormDescription({});

    # generate the html string for the form, using its template
    my $htmlstring = $fb->buildTemplateString('Metadot/templates/demos', "$template.tt2");

    # return the result
    return $htmlstring;
}


1;
