###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::SystemApp::MyStuff;
use strict;

use Template;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS);
use MetadotConfig;
use Metadot::SystemApp;
use Gizmo;
use Gizmo::SubmissionSystem;
use Metadot::SystemApp::MetagramCenter;

use DBAccess;

use vars qw(@ISA);
@ISA=qw(Metadot::SystemApp Auditable);

#
# Substitute for an Operations object
#
my $AC = {
      BUNDLES => {
              DISP =>   { DESC =>          'View',
                  OPS =>            [qw( show show_user_submissions show_submissions_for_review )],
                  DEFAULT_LEVEL =>  2,
                  MIN_LEVEL =>      2,
                  DEFAULTS_TO =>    'on',
                },
              EDITP =>  {
                 DESC =>          'Change<br />Permissions',
                 OPS =>            [qw(edit_permissions set_permissions)],
                 DEFAULT_LEVEL =>  8,
                 MIN_LEVEL =>      8,
                },
             },

      DISPLAY_ORDER => [ qw(DISP EDITP) ],
     };

# should always be overridden in derived classes
sub get_AC {
    my $self = shift;
    my $class = ref $self || $self;

    die "&get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

    return $AC;
}

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my $self  = $class->SUPER::new();

    $self->{_USE_SUPER_PORTAL} = 1;
    bless ($self, $class);

    return $self;
}


sub name {
    return ''
}

sub get_app_name {
    return MetadotConfig->get_param_value('mystuff_name') || 'My Stuff';
}

sub can_change_owner {

    return undef;
}


sub is_a {
    my $self = shift;

    my $class = ref($self) || $self;

    return $class
}


sub main {
    my $self = shift;
    $self->debug_msg(3, "MyStuff::main()");

    my $template = Template->new({
                  INCLUDE_PATH => 'Metadot/SystemApp/MyStuff/templates',
                 }) || die $Template::ERROR, "\n";


    # data for the template
    my $data = {
                 tab => $FORM{tab},
                 my_metagrams_name => Metadot::SystemApp::MetagramCenter->get_app_name(),
                 self => $self,
                 current_user => $USER->uid(),
                 portal_params => $PARAMS,
               };

    my $output;
    $template->process('MyStuffMain.tt2', $data, \$output)
    || die $template->error(), "\n";

    return $output;
}

sub www_show_user_submissions {
    my $self = shift;
    $self->debug_msg(3, "MyStuff::www_show_user_submissions()");

    my $template = Template->new({
                   INCLUDE_PATH => 'Metadot/SystemApp/MyStuff/templates',
                   }) || die $Template::ERROR, "\n";


    # data for the template
    my $data = {
                 tab => 'request',
                 submission_systems => $self->get_user_submission_systems(),
                 show_only_uid => $USER->uid(),
               };

    my $output;
    $template->process('UserSubmissions.tt2', $data, \$output)
        || die $template->error(), "\n";

    return $self->print_portal($output);
}

sub get_user_submission_systems {
    my $self = shift;
    $self->debug_msg(3, "MyStuff::get_user_submissions()");

    return SubmissionSystem->get_submission_systems_with_submissions(user_id => $USER->uid());
}

sub www_show_submissions_for_review {
    my $self = shift;
    $self->debug_msg(3, "MyStuff::www_show_submissions_for_review()");

    my $template = Template->new({
                   INCLUDE_PATH => 'Metadot/SystemApp/MyStuff/templates',
                   }) || die $Template::ERROR, "\n";


    # data for the template
    my $data = {
                 tab => 'review',
                 submission_systems => $self->get_submission_systems_for_review(),
               };

    my $output;
    $template->process('SubmissionsForReview.tt2', $data, \$output)
        || die $template->error(), "\n";

    return $self->print_portal($output);
}

sub get_submission_systems_for_review {
    my $self = shift;
    $self->debug_msg(3, "MyStuff::get_user_submissions()");

    return SubmissionSystem->get_submission_systems_for_review(user_id => $USER->uid());
}

1;

__END__
