###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::SystemApp::TemplateApp;

# ========================================
# This is a system app used to demo the form builder.  This demo loads a description of
# the form elements and variables from an .xml file, from that builds a data set for the
# form template, and passes it to the template.  The result string is returned and, as
# a system app, rendered within the context of the site's headers and footers.

# The app also includes a 'process' method, which demos how one might
# hook into the FormProcess methods.  FormProcess supports handling of
# the form results.  It uses the form field names, which are
# associated with the db pathnames for the table fields, in
# conjunction with information about the tables included in the form
# (automatically generated by FormBuilder), to  automatically
# generate db new objects or modify existing ones.

# ========================================

use strict;
use Data::Dumper;

#use Carp;

use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $ACCESS_BROKER );
use Metadot::SystemApp;
use Metadot::FormBuilder;

use vars qw(@ISA);
@ISA=qw(Metadot::SystemApp Auditable);

#
# Substitute for an Operations object
#
my $AC = {
      BUNDLES => {
              DISP =>   { DESC =>          'View',
                  OPS =>            [qw( show process)],
                  DEFAULT_LEVEL =>  2,
                  MIN_LEVEL =>      0,
                  DEFAULTS_TO =>    'on',
                },
              EDITP =>  {
                 DESC =>          'Change<br />Permissions',
                 OPS =>            [qw(edit_permissions set_permissions)],
                 DEFAULT_LEVEL =>  8,
                 MIN_LEVEL =>      8,
                },
             },

      DISPLAY_ORDER => [ qw(DISP EDITP) ],
     };


# should always be overridden in derived classes
sub get_AC {
    my $self = shift;
    my $class = ref $self || $self;

    die "&get_AC should be overridden in class $class"
    unless ($class eq __PACKAGE__);

    return $AC;
}


sub name {
    return 'TemplateApp'
}


sub can_change_owner {
    return undef;
}


sub is_a {
    my $self = shift;

    my $class = ref($self) || $self;
    return $class
}


sub main {
    my $self = shift;

    my $fb = Metadot::FormBuilder->new();

    my $iid = $FORM{objiid};
    my $isa = $FORM{objisa};
    my $op = $FORM{objop};
    my $mode = $FORM{mode};
    #print STDERR "got iid $iid, isa $isa, op $op, and mode $mode\n";
    # check that user has edit privs for the given object
    my $sourceobject;
      eval { $sourceobject = $isa->new($iid); };
      if( $@ ) {
      return "";
      }
    if (!($sourceobject->is_allowed_to_do('modify', $USER))) {
        #warning, exit
      $self->print_click_back(
           "Error:<br />Sorry, you are not allowed to modify this $isa object:<br />$0");
      exit(0);
      }
    #unpack pkey info
#  my $pkeys = { 'user.uid' => 1034, 'instance.iid' => 1758};
    my $pkeys = {'template.iid' => $iid, 'template.isa' => $isa, 'template.op' => $op};
    #print STDERR "pkey info: ", Dumper($pkeys), "\n";

    # load the form spec. info from an xml file.
    my $dref = $fb->getFormDescription("Metadot/form_config/template_mgr.xml");
#      print STDERR "got descr " , Dumper($dref), "\n";

    $fb->setMode($mode);
    $fb->processFormElementSpecs();

    $fb->addTextElementDescriptor('template.op', 'text', 'Operation',  {value => "$op"});
    $fb->addTextElementDescriptor('objisa', 'hidden', '',  {value => "$isa"});
    $fb->addTextElementDescriptor('template.iid', 'hidden', '',  {value => "$iid"});

    # if user is admin, set a config attribute
    if ( $ACCESS_BROKER->is_allowed_to_do('admin_tasks') ) {
    $fb->addConfigAttrValue('is_admin', 'true');
    }
    else {
    $fb->addConfigAttrValue('is_admin', 'false');
    }

    $fb->processFormVarSpecs($pkeys);
    my $data =  $fb->getFormTemplateData();
#      print STDERR "****got final form vars data: ", Dumper($data), "\n";

#      $fb->processFormDescription($pkeys);

    # generate the html string for the form, using its template
    my $htmlstring = $fb->buildTemplateString('Metadot/templates', 'template_mgr.tt2');

    # return the result
    return $htmlstring;
}

sub www_process {

    my $self = shift;
#     print STDERR "in TemplateApp::www_process\n";
#     print STDERR "form info: ", Dumper(\%FORM) , "\n";
    my $iid = $FORM{template__iid};
    my $isa = $FORM{objisa};
    my $classwide = $FORM{classwide};
    my $op = $FORM{template__op};
    if (!$op) {
    $op = 'show';
    }
    my $content = "";
    my $action = $FORM{tmpl_action};
    my $ttemplate = $FORM{template__template};
    if (lc $classwide eq 'yes') {
    # then reject unless user is admin
    if ( $ACCESS_BROKER->is_allowed_to_do('admin_tasks') ) {
        my $giid = 'all';
        if ($action eq "edit") {
        $self->_save_template($giid, $isa, $op, $ttemplate);
        $content = "<p />Added/replaced class template template corresponding to $isa, and operation $op\n";
        }
        elsif ($action eq "delete") {
        $self->_delete_template($giid, $isa, $op);
        $content = "<p />Deleted template corresponding to $isa, and operation $op\n";
        }
        else {
        $content = "<p />Error: could not find action to take.\n";
        }
    }
    else {
        $self->print_click_back(
                    "Error:<br />Sorry, only an admin may modify a classwide template.");
        exit(0);
    }
    }
    else {          # local object, not classwide, template
    # reject all actions unless the user has modify permissions on the given object.
    my $sourceobject;
    eval { $sourceobject = $isa->new($iid); };
    if( $@ ) {
        return "";
    }
    if (!($sourceobject->is_allowed_to_do('modify', $USER))) {
        #warning, exit
        $self->print_click_back(
                    "Error:<br />Sorry, you are not allowed to modify this $isa object:<br />$0");
        exit(0);
    }

    if ($action eq "delete") {
        $self->_delete_template($iid, $isa, $op);
        $content = "<p />Deleted template corresponding to $isa of ID $iid, and operation $op\n";
    }
    elsif ($action eq "edit") {
        $self->_save_template($iid, $isa, $op, $ttemplate);
        $content = "<p />Added/replaced template corresponding to $isa of ID $iid, and operation $op\n";
    }
    else {
        $content = "<p />Error: could not find action to take.\n";
    }
    }
    $self->print_http_header();
    print $self->show_page_top(),
    $self->show_header( 0 ),
        $self->show_portal_panel( 0 ),
    $self->show_panel(),
        $content, "<p />",
        "<a href=\"javascript:close()\">Close Window</a><br />",
#          $self->build_url($iid, $isa, $op),
    $self->show_bottom;
}

sub _delete_template {
    my ($self, $iid, $isa, $op) = @_;

    $iid = undef unless(Utility->is_valid_integer($iid));    # secu_200401
    DBAccess->sqlDelete('template', "iid=$iid and isa=".DBAccess->quote($isa)." and op=".DBAccess->quote($op));  # secu_200401
}


sub _save_template {
    my ($self, $iid, $isa, $op, $template) = @_;

    # delete it if it exists already
    #DBAccess->sqlDelete('template', "iid='$iid' and isa='$isa' and op='$op'");
    $self->_delete_template($iid, $isa, $op);   # secu_200401

    DBAccess->sqlInsert('template',
                 iid => $iid,
                 isa => $isa,
                 op => $op,
                 template => $template);
}

# javascript to open new window.
sub build_url {
    my ($class, $iid, $isa, $op, $text) = @_;
    my $url = "index.pl?isa=Metadot::SystemApp::TemplateApp&amp;op=show&amp;mode=edit&amp;objop=$op&amp;objisa=$isa&amp;objiid=$iid";
#      $url = "<a href=\"javascript:open('$url');\">$text</a>";
    $url = "<a href=\"$url\" target=_blank >$text</a>";
    return $url;
}

1;

