###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::SystemApp::Util;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS);
use Date::Calc qw(:all);
use Spreadsheet::WriteExcel;
use Digest::MD5;

my $instance;
use vars qw(@ISA) ;

my %MONTHS = (
     0 => 'January',
     1 => 'February',
     2 => 'March',
     3 => 'April',
     4 => 'May',
     5 => 'June',
     6 => 'July',
     7 => 'August',
     8 => 'September',
     9 => 'October',
    10 => 'November',
    11 => 'December'
);

my %MONTHS_NUM = (
     'jan' => 1,
     'feb' => 2,
     'mar' => 3,
     'apr' => 4,
     'may' => 5,
     'jun' => 6,
     'jul' => 7,
     'aug' => 8,
     'sep' => 9,
     'oct' => 10,
     'nov' => 11,
     'dec' => 12,
);

use constant DAYS_OF_WEEK => {
    0 => 'Sunday',
    1 => 'Monday',
    2 => 'Tuesday',
    3 => 'Wednesday',
    4 => 'Thursday',
    5 => 'Friday',
    6 => 'Saturday'
};

sub new {
    return $instance if (defined $instance);
    my $proto = shift;
    my $class = ref($proto) || $proto;
    $instance = {};

    bless ($instance, $class);
    return $instance;
}

## formats a unix timestamp
## M - month (January)
## D - day   (1-31)
## Y - year  (YYYY)
## T - time  (1:00 pm)
## H - hour  (1-12)
sub format_datetime {
    my $self = shift ;

    my $datetime  = shift;
    my $format = shift || 'M D, Y T';

    my ($sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst) = localtime($datetime);
    $year+=1900;
    my $date = '';

    my @format_array = split(//, $format);

    foreach my $char (@format_array) {
        if ($char eq 'W') {
            $date .= DAYS_OF_WEEK->{$wday};
        } elsif ($char eq 'M') {
            $date .= $MONTHS{$month};
        } elsif ($char eq 'D') {
            $date .= $day;
        } elsif ($char eq 'Y') {
            $date .= $year;
        } elsif ($char eq 'T') {
            $date .= $self->_format_time($datetime);
        } elsif ($char eq 'H') {
            $date .= $self->_format_hour($hour);
        } elsif ($char eq 'S') {
            $date .= $sec;
        } else {
            $date .= $char;
        }
    }

    return $date;
}


sub date_to_time {
    my $self = shift;
    my $date = shift || '01-Jan-1970';
    my ($day, $month, $year) = split("-", $date);
    $month = $MONTHS_NUM{lc($month)};

    return Mktime($year, $month, $day, 0, 0, 0);
}

## formats hour portion of a unix time stamp (i.e. 1:00 pm)
sub _format_time {
    my $self = shift ;
    my $time  = shift;

    my ($hour, $min) = (localtime($time))[2, 1];
    my $ampm = $hour > 11 ? 'pm': 'am';
    $min = $min < 10 ? "0$min" : $min;

    return $self->_format_hour($hour) . ":$min $ampm";
}

## formats hour (returns 1-12)
sub _format_hour {
    my $self = shift ;
    my $hour  = shift;

    my $display_hour = '';

    if ($hour == 0) {
        $display_hour = '12';
    } elsif ($hour > 12) {
        $display_hour = $hour - 12;
    } else {
        $display_hour = $hour;
    }

    $display_hour+=0;

    return $display_hour;
}

sub format_for_js {
    my $self = shift;
    my $value = shift || "";
    $value =~ s/\n//g;
    $value =~ s/'/`/g;
    return $value;
}

sub get_day_start_sec {
    my $self = shift;
    my $time = shift || time;
    my ($sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst) = localtime($time);
    $year+=1900;
    return Mktime($year, $month+1, $day, 0, 0, 0);
}

sub export_excel {
    my $self = shift;
    my $data = shift;
    my $uploaddir = $PARAMS->{absolutepathfileuploaddir};
    my $file_name = Digest::MD5::md5_hex($ENV{REQUEST_URI});
    my $file = $uploaddir."/".$file_name."Excel.xls";
    my $name = "StatExcel.xls";

    my $workbook   = Spreadsheet::WriteExcel->new($file);
    my $worksheet  = $workbook->addworksheet();
    my $format     = $workbook->addformat();
    my $row = 0;
    my $col = 0;

    $name = $data->{title}."Excel.xls";

    $format->set_bold();
    $worksheet->write($row, $col, $data->{title}.", ".$data->{form}->{search_data_timestamp}." - ".$data->{form}->{search_data_timestamp_to}, $format);

    $row = 1;
    $worksheet->write($row, $col, "Generated: ".$self->format_datetime(time(), 'M D, Y, T'), $format);

    $row = 2;
    foreach my $field (($data->{fields}->array())) {
        $worksheet->write($row, $col, $data->{messages}->get_message($field->key()), $format);
        $col++;
    }

    $row = 3;
    foreach my $instance (($data->{collection}->array())) {
        $col = 0;
        foreach my $field (($data->{fields}->array())) {
            $worksheet->write($row, $col, $field->get_list_tag($instance->get_value($field->key())));
            $col++;
        }
        $row++;
    }
    $workbook->close();
    $self->_send_to_browser($file, $name);
}

sub export_csv {
    my $self = shift;
    my $data = shift;
    my $uploaddir = $PARAMS->{absolutepathfileuploaddir};
    my $file_name = Digest::MD5::md5_hex($ENV{REQUEST_URI});
    my $file = $uploaddir."/".$file_name."CSV.xls";
    my $name = "StatCSV.xls";

    $name = $data->{title}."CSV.xls";

    open(CSV_FILE, ">$file");

    my $output = "";
    foreach my $field (($data->{fields}->array())) {
        my $tmp = $data->{messages}->get_message($field->key());
        # escaping |-sign and \n
        $tmp =~ s/[\||\n]/-/g;
        $tmp = 'N/A' if $tmp eq "";
        $output .= $tmp."|";
    }
    chop($output);
    print CSV_FILE $output."\n";

    foreach my $instance (($data->{collection}->array())) {
        $output = "";
        foreach my $field (($data->{fields}->array())) {
            my $tmp = $field->get_list_tag($instance->get_value($field->key()));
            # escaping |-sign and \n
            $tmp =~ s/[\||\n]/-/g;
            $tmp = 'N/A' if $tmp eq "";
            $output .= $tmp."|";
        }
        chop($output);
        print CSV_FILE $output."\n";
    }
    close(CSV_FILE);
    $self->_send_to_browser($file, $name);
}

# this is replaced my transmit_file. The sub below is not 100% correct,
#
sub _send_to_browser {
    my $class = shift;
    my ($file, $name) = @_;
    Metadot::UploadsManager->transmit_file(file_location=>$file, file_name=>$name);
}




1;
