###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::TableInfoFactory;

#========================================
# This class is used to return a db-specific object for getting
# information about table rows (value type, precision, ...)  from the
# database.  Currently, only mysql is actually supported (see
# MySQLTableInfo), but this class provides hooks for integrating with
# other databases at a later time.
#========================================


use strict;
use Data::Dumper;

use MetadotConfig;
use ErrorHandling;


# db object cache.
# cache a table info obj for each virtual server.
my %INFO_OBJ_CACHE = ();

# return or create and return a table info object of the correct type.
sub getTableInfoObj {
    my $class = shift;
    my $server_name = MetadotConfig->get_server_name();
    my $dbo = $INFO_OBJ_CACHE{$server_name};
    #if object exists, return it
    if ($dbo) {
    return $dbo;
    }
    else {          # else, need to create db object
    my $type=""; my $mname;
    eval {
        $type = $class->get_db_type();
        $mname = $type . "TableInfo";
        require "Metadot/$mname.pm";
        #create object
        $dbo = "Metadot::$mname"->new();
    };
    ### CATCH
    if ($@) {
        #print STDERR $@;
        ErrorHandling->printError("Can not find module: $mname.<br />" .
                      $@ . "<br />\n");
        die;
    }
    # if didn't catch an error, store the db object
    # and add to cache
    $INFO_OBJ_CACHE{$server_name} = $dbo;
    return $dbo;
    }
}


sub get_db_type {
    my $class = shift;
    my $type = MetadotConfig->get_dbtype();
    #print STDERR "got db type: $type\n";
    return $type;
}


1;

