###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::TemplateMgr;
use strict;
use MetadotConfig;
use DBAccess;
use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CACHE);


sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};

    bless ($self, $class);
    return $self;
}

sub get_template {
    my ($class, %params) = @_;

    my $obj = $params{obj};
    my $op = $params{op} || $FORM{op};

#    print STDERR "in TemplateMgr::get_template, with op $op\n";
    my $featureset_tag = MetadotConfig->get_featureset_tag() ;
    my $use_tm = MetadotConfig->get_use_template_mgmt() ;
#print STDERR "\n\ntemplate_mgmt:" . $PARAMS->{'template_mgmt'} . " featureset_tag: $featureset_tag\n\n";
#    if ($PARAMS->{'template_mgmt'} && ($featureset_tag eq 'closed')
#   && ($use_tm eq 'true')
#   )
#     {
    if (!$op) {
        $op = 'show';   # is this appropriate?
    }
    my $template = $class->_get_obj_template(obj => $obj, op => $op);
#   if (!$template) {
#       print STDERR "in get_template, checking for class template, class='$class'\n";
#       $template = $class->_get_class_template($obj, $op);
#   }
    # don't return default -- let the portal code figure out whether or not to use
    # themes if specialized template was not found.
#   if (!$template) {
#       print STDERR "in get_template, checking for default template\n";
#       $template = $class->_get_default_template($obj);
#   }
    return $template;
#    }
#    else {
#   return undef;
#    }
}

sub save_template {
    my ($class, %params) = @_;

    my $obj = $params{obj};
    my $op = $params{op} || $FORM{op} || 'show';
    my $template = $params{template} || '';

    # first get object 'isa'
    if (!$obj) {
        #print STDERR "error: in save_template: no object.\n";
        return undef;
    }

    my $isa;
    if ($obj->can('is_a')) {
        $isa = $obj->is_a();
    }
    if (!$isa) {
        $isa = ref($obj);
    }

    if ($obj->can('get_id')) {
        my $id = $obj->get_id();
        if (defined $id) {
            $DISPLAY->debug_msg(5, "in save_template, with id $id and isa $isa and op $op");
            my ($exists) = DBAccess->sqlSelect("count(*)", "template", "iid=".DBAccess->quote($id)." and isa='$isa' and op=".DBAccess->quote($op));  #secu_200401
            if ($exists) {
                $DISPLAY->debug_msg(5, "in save_template, found template - will update for id $id and isa $isa");
                DBAccess->sqlUpdate("template", "iid=".DBAccess->quote($id)." and isa='$isa' and op=".DBAccess->quote($op), template => $template);  #secu_200401
            } else {
                $DISPLAY->debug_msg(5, "in save_template, will insert template for id=$id; isa=$isa; op=$op");
                DBAccess->sqlInsert("template", iid=> $id, isa=> $isa, op=> $op, template => $template);
            }
        }
    }
}

sub _get_obj_template {
    my ($class, %params) = @_;

    my $obj = $params{obj};
    my $op  = $params{op};
    my $depth = $params{depth} || 0;

    ErrorHandling->printError("Hey man, I'm looping like crazzy here (Metadot::TmplateMgr::_get_obj_template)!!") if ($depth > 100);

    # see if the object has an associated template
    # first get object 'isa'
    if (!$obj) {
    #print STDERR "error: in _get_obj_template: no object.\n";
    return undef;
    }
    my $isa;
    if ($obj->can('is_a')) {
    $isa = $obj->is_a();
    }
    if (!$isa) {
    $isa = ref($obj);
    }
    if ($obj->can('get_id')) {
    my $id = $obj->get_id();
    if (defined $id) {
        #print STDERR "in _get_obj_template, doing query with id $id and isa $isa and op $op\n";
        my ($template) = DBAccess->sqlSelect("template", "template", "iid=".DBAccess->quote($id)." and isa=".DBAccess->quote($isa)." and op=".DBAccess->quote($op));  #secu_200401
        if ($template) {
        #print STDERR "in _get_obj_template, found template for id $id and isa $isa\n";
        }
            else {
                if ($id > 0 && $obj->can('get_parent')) {
                    my $parent = $obj->get_parent();
                    if ($parent) {
                        $template = $class->_get_obj_template(obj => $parent, op => $op, depth => $depth+1);
                    }
                }
            }
        return $template;
    }
    else {
        return undef;
    }
    }
    else {          # no go
    return undef;
    }
}


# not supported yet
sub _get_class_template {
    my ($class, $obj, $op) = @_;

    my $isa;
    if ($obj->can('is_a')) {
    $isa = $obj->is_a();
    }
    if (!$isa) {
    $isa = ref($obj);
    }
    my ($template) = DBAccess->sqlSelect("template", "template", "iid=".DBAccess->quote('all')." and isa='$isa' and op=".DBAccess->quote($op));  #secu_200401
    if ($template) {
    #print STDERR "in _get_obj_template, found classwide template for $isa\n";
    }
    return $template;
}

sub _get_default_template {
    my ($class, $obj) = @_;

    # get obj class info
    my $oclass = $obj->is_a();
    if (!$oclass) {
    $oclass = ref($obj);
    }
    $oclass = lc $oclass;
    if ($oclass eq "category") {
    my $id = $obj->get_iid();
    if ($id == 0) {
        return $PARAMS->{maintemplate};
    }
    elsif ($id > 0) {
        return $PARAMS->{subtemplate};
    }
    else {
        return $PARAMS->{utilitytemplate};
    }
    }
    else {
    return $PARAMS->{utilitytemplate};
    }
}

sub processTemplate {
    my ($self, $template, $data) = @_;
}


1;


__END__


