###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

# A test sendmail that intentionally fails after some number of attempts

package Metadot::Testing::FailingSendmail;

=head1 NAME

Metadot::Testing::FailingSendmail

=head1 VERSION

Version $Revision: 1.1.2.3 $

=head1 SYNOPSIS

my $sm = Metadot::Testing::FailingSendmail->new({
    to              => $sm_to,
    cc              => $sm_cc,
    bcc             => $sm_bcc,
    subject         => $sm_subject,
    body            => $sm_body,
    debug_level     => $self->debug_level(),
    suppress_emails => $self->silent(),
    fail_after      => 3,
});

$sm->sendMail();


=head1 DESCRIPTION

ChannelProcessor is responsible processing channels and subscriptions. This
class is a work in progress, as part of an effort to refactor metadotd.pl into
something more manageable. The main benefit of this class for now is that it has
an API which can be unit tested. Apart from that, it is basically code that has
been extracted from metadotd.pl and minimally refactored to allow it to operate
in an OO fashion.

=head1 METHODS

=cut

my $FAIL_AFTER;
my $NUMBER_SENT;

use Sendmail;
use vars qw(@ISA);
@ISA = qw(Sendmail);

sub new {
    my $proto = shift;
    my $args  = shift || {};

    my $class = ref($proto) || $proto;
    my $self = $class->SUPER::new($args);

    $self->fail_after($args->{fail_after} || 1);

    return $self;
}




=item C<fail_after>

Determines the number of sends before the object intentionally fails.

$sm->fail_after(1);
my $fail_after = $sm->fail_after();

=cut

sub fail_after {
    my $self = shift;
    return @_ ? $FAIL_AFTER = shift : $FAIL_AFTER;
}


=item C<number_sent>

Returns the number of emails that have been sent so far.

my $number_sent = $sm->number_sent();

=cut
sub number_sent {
    my $self = shift;
    return @_ ? $NUMBER_SENT = shift : ($NUMBER_SENT || 0);
}


=item C<sendMail>

See Sendmail.pm

=cut
sub sendMail {
    my $self = shift;

    if (defined($self->fail_after()) && $self->number_sent() >= $self->fail_after()) {
        print STDERR "Sent a simulated failure.\n";
        if ($self->die_on_error()) {
            confess($self.' generated a simulated failure');
        }
        return 0;
    } else {
        $self->number_sent($self->number_sent() + 1);
        return $self->SUPER::sendMail(@_);
    }
}








1;
