###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2006
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::UploadsManager;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CACHE $CGI);
use MetadotConfig;

use ErrorHandling;

#use URI::Escape qw(uri_unescape uri_escape);
use Utility;

use constant STORAGE_TYPE_PUBLIC => 'public';
use constant STORAGE_TYPE_ACCESS_CONTROLLED => 'access_controlled';

use constant UPLOAD_TYPE_PUBLIC         => 'public_upload';
use constant UPLOAD_TYPE_USER           => 'user_upload';
use constant UPLOAD_TYPE_USER_SELECTED  => 'user_selected_upload';


sub customize_permissions {
    my $class = shift;
    my $AC = shift || die("Must provide an Operations instance to customize");

    $AC->add_op('MOD', 'confirm_upload_deletion');
    $AC->add_op('MOD', 'delete_upload');
    $AC->add_op('DISP', 'download_file');

    return $AC;
}


sub www_download_file {
    my $self = shift;
    my $params = shift;

    my ( $id, $isa, $field_name, $file_type );
    if( $params ) {

        $params->{isa} || die "must include param isa";
        $params->{id} = undef unless(Utility->is_valid_integer($params->{id}));  #secu_200401
        ( defined $params->{id} ) || die "must include param id";
        $params->{field_name} || die "must include param field_name";

        $id = $params->{id};
        $isa = $params->{isa};
        $field_name = $params->{field_name};
        $file_type = $params->{type};

        if( defined( $params->{additional_args} ) ){
            my $extra_arg = $params->{additional_args};
        }
        if( defined( $params->{mid} ) ){
            my $message_id = $params->{mid};
        }

    } else {

        $id  = $params->{id}  = $FORM{id};
        $isa = $params->{isa} = $FORM{isa};
        $field_name = $params->{field_name} = $FORM{field_name};
        $file_type  = $params->{type}       = $FORM{file_type};

    }

    $id = undef unless(Utility->is_valid_integer($id));    # secu_200401

    my ($read_file, $filename) = $self->determine_file_location($params);

    $self->transmit_file(
        file        => $filename,
        read_file   => $read_file,
        file_type   => $file_type,
    );

    # we don't want the portal to output anything else.
    exit;
}


sub determine_file_location {
    my ($self, $args) = @_;

    my $isa         = $args->{isa} || die "must include param isa";
    my $id          = $args->{id};
    my $field_name  = $args->{field_name} || die "must include param field_name";
    my $write_dir_namespace = $args->{write_dir_namespace};

    my $result;

    $id = undef unless(Utility->is_valid_integer($id));  #secu_200401
    ( defined $id ) || die "must include param id";


    my ( $file, $stored_at ) = DBAccess->sqlSelect( "file_name, stored_at",
                            "uploads",
                            "isa=".DBAccess->quote($isa)." and id=$id and field_name=".DBAccess->quote($field_name));  #secu_200401

    if (defined $file) {
        # if field is public, must fetch the file from the public dir
        if( $stored_at eq STORAGE_TYPE_PUBLIC ) {
            $result = MetadotConfig->get_public_uploads_dir() . "/" . $file;

        } else {
            unless (defined $write_dir_namespace) {
                my $absolutepathfileuploaddir = $PARAMS->{absolutepathfileuploaddir};
                $write_dir_namespace = $self->get_write_dir_namespace($args) || "$absolutepathfileuploaddir/$id";
            }

            $result = $write_dir_namespace . "/$file";
        }
    }

    return ($result, $file, $stored_at);
}


# transmits a file to the browser. The args are:
#   file_name (or file): the name of the file as transmitted to the browser
#   file_location (or read_file): the location on the web server where the file can be read
#   file_type: the file extension, for looking up the mime type. If omitted,
#              one will be determined
sub transmit_file {
    my $class = shift;
    my %args = @_;
    my $file = $args{file_name} || $args{file} || die ("Must supply file name");
    my $read_file = $args{file_location} || $args{read_file} || die("Must supply file location");


    my $file_type = $args{file_type};
    my $mtype;
    my $mimetypes = Metadot::MIME::Types->new();
    if ($file_type){
        $mtype = $mimetypes->mimeTypeOf($file_type) ;
    } else {
        $mtype = $mimetypes->mimeTypeOf($file) ;
    }
    unless (open OUT, "<$read_file") {
        my $error_name;
        if (ref($class) && $class->can('name')) {
            $error_name = $class->name();
        } else {
            $error_name = $class;
        }
        ErrorHandling->printError("Gizmo.pm: ". $error_name. " : Couldn't open file $read_file: $!");
        return;
    }
    binmode (OUT);

    # enquote the $file value for compatibility with white-space
    if ($file =~ m/\s/) {
        $file = qq~"$file"~;
    }

    my %header_values = (
                         '-Connection'          => 'close',
                         '-Accept-ranges'       => 'bytes',
                         '-Content-location'    => $file,
    );

    if (defined $mtype && defined $mtype->type) {
        if ($mtype->type eq "application/pdf") {# [vadim] IE+Acrobat workaround
            $header_values{'-Content-disposition'} = "attachment; filename=$file";
        } else {
            $header_values{'-Content-disposition'} = "inline; filename=$file";
        }
        $header_values{type} = $mtype->type;
    } else {
        $header_values{'-Content-disposition'} = "inline; filename=$file";
        $header_values{type} = 'application/octet-stream';
    }


    print $CGI->header(%header_values);
    #print STDERR "CGI HEADER:\n",$CGI->header(%header_values);

    {
        local $/ = undef;
        while(<OUT>) {
                print;
        }
    }
    close OUT ;


    # We take care of the entire response in this method, and
    # we don't want the portal to output anything else.
    exit;
}


sub www_confirm_upload_deletion {
    my $self = shift;

    my $id   = $FORM{id};
    my $isa   = $FORM{isa};
    my $field_name = $FORM{field_name};

    my $return_url  = $FORM{return_url} || $SESSION->referer();
    my $landing_url = $FORM{landing_url};

    $DISPLAY->debug_msg(4, "return URL is $return_url");
    $DISPLAY->debug_msg(4, "landing URL is $landing_url");

    $id = undef unless(Utility->is_valid_integer($id));    # secu_200401
    my ( $file_name ) = DBAccess->sqlSelect( "file_name",
                         "uploads",
                         "isa=".DBAccess->quote($isa)." and id=$id and field_name=".DBAccess->quote($field_name));  #secu_200401

    $DISPLAY->two_choices( "Do you want to delete attachment $file_name ?",
               "caution",
               "Yes please", $DISPLAY->make_url( "index.pl?id=$id&isa=$isa&field_name=$field_name&op=delete_upload&return_url=".$CGI->escape($return_url)."&landing_url=".$CGI->escape($landing_url)),
               "No!", $DISPLAY->make_url( $return_url . ($return_url =~ /\?/ ? '&amp;' : '?') ."landing_url=".$CGI->escape($landing_url))
               );

}


sub www_delete_upload {
    my $self = shift;

    my $return_url  = $FORM{return_url};
    my $landing_url = $FORM{landing_url};

    $DISPLAY->debug_msg(4, "www_delete_upload() return URL is $return_url");
    $DISPLAY->debug_msg(4, "www_delete_upload() landing URL is $landing_url");

    $self->_delete_upload(
        id      => $FORM{id},
        isa     => $FORM{isa},
        field_name  => $FORM{field_name},
        field_index => $FORM{field_index},
        write_dir   => $self->get_write_dir_namespace(),
    );

    if (defined $landing_url) {
        $return_url .= '&landing_url='.$CGI->escape($landing_url);
    }
    Utility->md_redirect( $return_url );
}

sub _delete_upload {
    my $self = shift;
    my %args = (@_);

    return $self->delete_upload(\%args);
}

sub delete_upload {
    my $class    = shift;
    my $args_ref = shift || {};

    my %args = %$args_ref;

    my $id = $args{id};
    $id = undef unless(Utility->is_valid_integer($id));    # secu_200401
    die("Must supply id\n") unless defined($id);

    my $isa = $args{isa} || die("Must supply isa\n");
    my $field_name = $args{field_name} || die("Must supply field name\n");
    my $write_dir  = $args{write_dir};
    my $field_index = $args{field_index};

    my $where_clause = "isa=".DBAccess->quote($isa)." and id=$id and field_name=".DBAccess->quote($field_name);  #secu_200401
    if (defined $field_index) {
        $where_clause .= " and field_index=".DBAccess->quote($field_index); # secu_200401
    }

    my ( $filename, $stored_at ) = DBAccess->sqlSelect( "file_name, stored_at",
                            "uploads",
                            $where_clause );

    if( $stored_at eq STORAGE_TYPE_PUBLIC ) {
        my $write_file = MetadotConfig->get_public_uploads_dir() . "/" . $filename;
        unlink( $write_file );
        DBAccess->sqlDelete( "uploads", $where_clause );
    } else {

        my $absolutepathfileuploaddir = $PARAMS->{'absolutepathfileuploaddir'};
        my $write_dir_namespace = $write_dir || "$absolutepathfileuploaddir/$id";

        unlink( "$write_dir_namespace/$filename" );

        DBAccess->sqlDelete( "uploads", $where_clause );
    }

}

sub get_field_index {
    my $self = shift;
    my %args = @_;

    my $id = $args{id};
    $id = undef unless(Utility->is_valid_integer($id));    # secu_200401
    die("Must supply id\n") unless defined($id);
    my $isa = $args{isa} || die("Must supply isa\n");
    my $field_name = $args{field_name} || die("Must supply isa\n");
    my $file_name = $args{file_name} || die("Must supply isa\n");

    my $where_clause = "isa=".DBAccess->quote($isa)." AND id=$id AND field_name=".DBAccess->quote($field_name)." AND file_name=".DBAccess->quote($file_name);  #secu_200401
    my ($result) = DBAccess->sqlSelect("field_index","uploads",$where_clause);
    return $result;
}

sub get_upload_input_form_line {
    my $self = shift;
    my $params = shift || die "must include params hash";

    $params->{isa} || die "must include param isa";
    $params->{field_name} || die "must include param field_name";
    $params->{type} || die "must include param type";

    my $filename;

    $params->{id} = undef unless(Utility->is_valid_integer($params->{id}));  #secu_200401
    if( defined $params->{id} ) {
    # check if we already have an upload for the passed params
    ( $filename ) = DBAccess->sqlSelect( "file_name",
                         "uploads",
                         "isa=".DBAccess->quote($params->{isa})." and id=$params->{id} and field_name=".DBAccess->quote($params->{field_name})." and type=".DBAccess->quote($params->{type}));  #secu_200401
    }
    $DISPLAY->debug_msg(4, "get_upload_input_form_line: isa{".$params->{isa}."}, field_name{".$params->{field_name}."}, type{".$params->{type}."}, id{".$params->{id}."}, filename{".($filename || '')."}");

    unless( $filename ) {
        my $public_location = MetadotConfig->get_public_uploads_location();
        my $htm = "<INPUT TYPE=\"file\" NAME=\"$params->{field_name}\" SIZE=30>\n";
        if( $params->{type} eq UPLOAD_TYPE_USER_SELECTED() ) {
            if( $FORM{'force_access_controlled'} == 1 ){
                $DISPLAY->debug_msg(5,"WILL NOT SHOW OPTIONS and force access controlled");
                $htm .= "<input type='hidden' name='upload_to_publid_$params->{field_name}' value='0' />";
            } else {
                $htm .= "          <table width=\"98%\" >
                    <tr valign=\"top\">
                      <td width=\"5%\">
                        <input type=\"radio\" name=\"upload_to_public_$params->{field_name}\" value=\"1\">
                      </td>
                      <td><b>Fast download</b><br />
                        <font face=\"Verdana, Arial, Helvetica, sans-serif\" size=\"-2\">Your
                        file will be stored under " .
                $PARAMS->{uri_scheme} . "://". $PARAMS->{rootdomain} . $public_location . ".
                This url
                        is publicly accessible.  You can reference the file elsewhere on this site
                        as \"$public_location/&lt;your_file&gt;\",
                        where &lt;your_file&gt; is replaced with
                        your actual file name.
                    </font></td>
                    </tr>
                    <tr valign=\"top\">
                      <td width=\"5%\">
                        <input type=\"radio\" name=\"upload_to_public_$params->{field_name}\" value=\"0\" checked>
                      </td>
                      <td><b>Access-control protected</b><br />
                        <font face=\"Verdana, Arial, Helvetica, sans-serif\" size=\"-2\">Your
                        file will be under <i>access control</i>.
                        This means that only users who have permissions to view this
                        content object will be able to access the file.</font></td>
                    </tr>
                  </table>";
            }
        }

        return $htm;
    }

    my $landing_url = Utility->get_qualified_url( $params->{landing_url} || $params->{return_url});

    my $delete_label = "delete attachment";

#     print STDERR "isa is: " . $self->is_a() . "\n";
    if ($self->is_a() eq 'ImageItem' || $self->is_a() eq 'Portal') {    # special case label was requested
    # for image item (sigh)
    $delete_label = "delete image";
    }

    # the file exists; return html for deleting attachment
    return "$filename&nbsp;&nbsp;&nbsp;&nbsp;<a class=\"text_button\" href=\"" . $self->make_url( "index.pl?id=$params->{id}&amp;isa=$params->{isa}&amp;op=confirm_upload_deletion&amp;field_name=$params->{field_name}&amp;landing_url=".$CGI->escape($landing_url) ) . "\">$delete_label</a>";

}


# - type: public or access-controlled
# - isa: the class that the upload belongs to
# - id:  the id of the object to which the upload belongs
# - field_name: the name of the object's field used to store the upload's presence
# - file_path: the location of the file being uploaded
# - write_dir (optional: specifies the write_dir (not used for public uploads)

sub process_upload {
    my $self = shift;
    my $params = shift || die "must include params hash";

    my $err;

    $params->{isa} || die "must include param isa";
    $params->{id} = undef unless(Utility->is_valid_integer($params->{id}));  #secu_200401
    ( defined $params->{id} ) || die "must include param id";
    $params->{field_name} || die "must include param field_name";
    $params->{type} || die "must include param type";
    ( defined $params->{is_required} ) || die "must include param is_required";

    if( $params->{file_path} || $FORM{ $params->{field_name} } ) {
        # upload a file

        if( $params->{type} eq UPLOAD_TYPE_PUBLIC() || $params->{type} eq STORAGE_TYPE_PUBLIC() ||
            ( $params->{type} eq UPLOAD_TYPE_USER_SELECTED() &&
              ($FORM{ "upload_to_public_$params->{field_name}"})
              )
            ) {
            # upload to the public dir

            $err = $self->upload_public_file( $params );

        } else {
            # upload to the access controlled dir
            $params->{write_dir} ||= $self->get_write_dir_namespace($params);
            $err = $self->upload_access_controlled_file( $params );
        }
    } else {
        # the param for this upload was not submitted
        if( $params->{is_required} ) {
            # must check if this upload is already there
            # else we should raise an error

            my ($cnt) = DBAccess->sqlSelect("count(*)", "uploads",
                            "isa=".DBAccess->quote($params->{isa})." and id=$params->{id} and field_name=".DBAccess->quote($params->{field_name})." and type=".DBAccess->quote($params->{type}));  #secu_200401
        }
    }

    return $err;
}



# Copy an upload from one object to another.

sub copy_upload {
    my ($class, $source_args, $target_args) = @_;

    my $error;

    my $file_info = $class->get_file_info($source_args);

    # process an upload for the target, using the existing file as the source
    my ($filepath, $filename, $type) = $class->determine_file_location($source_args);

    # can't copy a public upload, because then there would be a conflict.
    if (defined($type) && $type eq STORAGE_TYPE_PUBLIC()) {
        $type = STORAGE_TYPE_ACCESS_CONTROLLED();
    }

    if (defined $filepath) {
        $target_args->{file_path}   = $filepath;
        $target_args->{type}        = $type;
        $target_args->{is_required} ||= 0;

        $error = $class->process_upload($target_args);
    }

    return $error;
}





#################################################################
# upload_public_file
# script heavily inspired from Muhammad A Muquit upload example.
# $filepath: the filename with the path where the file is located
# $fileiid: the object unique id that that file refers to.
#################################################################

sub _upload_public_file {
    my $class  = shift;
    my $params = shift || {};
    return $class->upload_public_file($params);
}

sub upload_public_file {
    my $class  = shift;
    my $params = shift;

    $params->{uid} = shift unless defined($params->{uid});

    # determine the source and destination files
    my $filepath = $params->{file_path} || $FORM{ $params->{field_name} } || die "this must be checked from where _upload_public_file is called";
    my $filename = $class->get_filename_from_path($filepath);

    $params->{source_file} = $filepath;
    $params->{destination_file} = MetadotConfig->get_public_uploads_dir() . "/" . $filename;
    $params->{stored_at} = STORAGE_TYPE_PUBLIC;

    return $class->process_file($params);
}

# PARAMS:
#   id: optional, id of this instance
#   isa: the class of this instance
#   uid: optional, uid of user account to debit this file
#   source_file: required, the location of the source file
#   destination_file: required, the location of the destination file
#   stored_at: usually 'public' or 'access_controlled'
#   type: categorizes file uploads
#   field_name: the instance field associated with this upload
#   allow_multiple: specifies whether multiple files are allowed here

sub process_file {
    my $class   = shift;
    my $params  = shift;

    # need this info to associate the file upload with a isa/id/field
    my $id              = $params->{id};
    my $isa             = $params->{isa};
    my $field_name      = $params->{field_name};
    my $type            = $params->{type};
    my $allow_multiple  = $params->{allow_multiple};

    # determine the source and destination files
    my $filepath    = $params->{source_file} || die "Must provide source_file";
    my $write_file  = $params->{destination_file} || die "Must provide destination_file";
    my $stored_at   = $params->{stored_at};
    my $filename    = $class->get_filename_from_path($filepath);

    # alternate sources of UID info
    my $uid = $params->{uid};
    $uid = $USER->uid()   unless (defined $uid);

    # prevent invalid ID
    $id = undef unless(Utility->is_valid_integer($id));  #secu_200401

    # this method takes care of transferring the file to permanent storage
    my $result = __PACKAGE__->upload_file( # need to call the PACKAGE method
                                           # to avoid conflicts with
        source_file => $filepath,
        write_file  => $write_file,
    );

    if ($result->{error}) {
        return $result->{error};
    }

    my $size = $result->{size};

    # update quotas
    my ($upload_size_allowed, $upload_number_allowed) = $USER->get_uploads_space_and_num_files_quota( $uid );

    if($upload_size_allowed == -1){
        $upload_size_allowed = $PARAMS->{max_total_upload_size};
    }

    if($upload_number_allowed == -1){
        $upload_number_allowed = $PARAMS->{max_total_number_upload};
    }

    # multiply by 2^20 to convert to bytes
    $upload_size_allowed *= 1048576;
    my ($used_space, $uploaded_number) = $USER->get_used_uploads_space_and_num_files( $uid );
    my $available_space = $upload_size_allowed - $used_space;

    # check if user exceeded size quota
    if ( $size > $available_space ) {
        unlink($write_file);
        return "Error, File upload has exceeded allowed user quota. Your remaining quota is of $available_space bytes and file $filename is of $size bytes. Please request a larger quota to the site administrator or delete some of your files to free up space";
    }

    # check if user exceeded numeric quota
    if ( $uploaded_number >= $upload_number_allowed ) {
        unlink($write_file);
        return "Error, File upload has exceeded allowed number of files user quota. Your number of files quota is of $upload_number_allowed. Please request a larger number of files quota to the site administrator or delete some of your existing files.";
    }

    # check for global quota overrun
    ($used_space, $uploaded_number) = $class->get_global_used_uploads_space_and_num_files();
    $upload_size_allowed = $PARAMS->{global_uploads_quota};

    # multiply by 1M to convert to bytes
    $upload_size_allowed *= 1048576;
    $available_space = $upload_size_allowed - $used_space;

    if ( $size > $available_space ) {
        unlink($write_file);
        return "Error, File upload has exceeded the site-wide allowed disk space quota. The remaining quota is of $available_space bytes and file $filename is of $size bytes. Please request a larger quota to the site administrator or delete some of your files to free up space";
    }


    # update the uploads database
    my %fields = (
            id => $id,
            userid => $uid,
            isa => $isa,
            field_name => $field_name,
            type => $type,
            stored_at => $stored_at,
            file_name => $class->get_filename_from_path($filepath),
            file_size => $size
    );

    # allow for possible field_index
    if ($allow_multiple) {
        my ($index) = DBAccess->sqlSelect(
            "max(field_index)","uploads","id=$params->{id} AND isa=".DBAccess->quote($isa)." AND field_name=".DBAccess->quote($field_name)
            );  #secu_200401
        $fields{field_index} = $index + 1;
    }

    # insert into bookkeeping table
    DBAccess->sqlInsert("uploads", %fields);

    return '';
}



#################################################################
# get_filename_from_path
# $filepath: the filename with the path where the file is located
#################################################################
sub get_filename_from_path {
    my $class    = shift;
    my $filepath = shift;
    my $filename;

    # James Bee" <JamesBee@home.com> reported that from Windows filename
    # such as c:\foo\fille.x saves as c:\foo\file.x, so we've to get the
    # filename out of it
    # look at the last word, hold 1 or more chars before the end of the line
    # that doesn't include / or \, so it will take care of unix path as well
    # if it happens, muquit, Jul-22-1999
    if ($filepath =~ /([^\/\\]+)$/) {
        $filename="$1";
    } else {
        $filename="$filepath";
    }

    return $filename;
}

# alias for backward compatibility
sub _get_filename_from_path {
    my $class = shift;
    return $class->get_filename_from_path(@_);
}

#################################################################
# upload_access_controlled_file
# script heavily inspired from Muhammad A Muquit upload example.
# $filepath: the filename with the path where the file is located
# $fileiid: the object unique id that that file refers to.
#################################################################

sub _upload_access_controlled_file {
    my $class  = shift;
    my $params = shift;

    return $class->upload_access_controlled_file($params);
}

sub upload_access_controlled_file {
    my $class  = shift;
    my $params = shift;

    $params->{uid} = shift unless defined($params->{uid});

    # determine the source and destination files
    my $filepath = $params->{file_path} || $FORM{ $params->{field_name} } || die "this must be checked from where upload_access_controlled_file is called";
    my $filename = $class->get_filename_from_path($filepath);

    $params->{source_file} = $filepath;

    # determining the directory to write to
    my $write_to_dir = $params->{write_dir} || "$PARAMS->{absolutepathfileuploaddir}/$params->{id}";
    $params->{destination_file} = "$write_to_dir/$filename";
    $params->{stored_at} = STORAGE_TYPE_ACCESS_CONTROLLED;


    # make sure the destination path is valid
    unless( $PARAMS->{absolutepathfileuploaddir} ) {
        my $msg = "Sorry the parameter 'upload_directory' is not set therefore the upload is just impossible. Please contact the system administrator";
        $DISPLAY->click_back($msg);
        exit;
    }

    return $class->process_file($params);
}



sub get_write_dir_namespace {
    my $class = shift;
    return '';
}


# checks that the directory exists and is writable
sub check_write_directory {
    my $class = shift;
    my $write_dir = shift || die("Must specify the write dir");

    my $separator = '/';

    # in case we have forward slash, replace with separator
    $write_dir =~ tr:\\:/:;


    #print STDERR "Checking $write_dir\n";

    my @hierarchy = split('/', $write_dir);
    my $check_dir = '';
    my $first = 1;
    foreach my $subdir (@hierarchy) {
	if ($first) {
	    $first = 0;
	} else {
	    $check_dir .= $separator;
	}
	$check_dir .= $subdir;

	if ($check_dir =~ /^\/*$/) {
            next;
        }

        #print STDERR "Checking subdir $check_dir\n";
        unless (-d $check_dir) {
            unless (mkdir($check_dir, 0755)) {
                ErrorHandling->printError("cannot create upload file directory: $check_dir (need to create $write_dir)");
                exit;
            }
        }
    }
}


# generic method for uploading files. The other upload methods need to be refactored to
# share this code. Returns a result hash containing error, size
use constant _ERROR_FILE_EXISTS_    => 'Target file exists';
use constant _ERROR_READING_FILE_   => 'Error reading source file';
use constant _ERROR_WRITING_FILE_   => 'Error writing target file';
use constant _ERROR_FILE_EMPTY_     => 'Source file was empty';

sub upload_file {
    my $class = shift;
    my %args = @_;

    my $source_file = $args{source_file} || die "Must specify arg source_file";
    my $write_dir   = $args{write_dir};
    my $write_file  = $args{write_file};
    my $overwrite   = $args{overwrite} || 0;

    $write_dir || $write_file || die "Must specify either arg write_file or write_dir";

    my $size;

    # make sure we have a write file
    if ($write_file) {
        my $filename = $class->get_filename_from_path($write_file);
        $write_dir = $write_file;
        $write_dir =~ s/\Q$filename\E$//;
    } else {
        my $filename = $class->get_filename_from_path($source_file);
        $write_file = $write_dir.'/'.$filename;
    }

    #print STDERR "SOURCE FILE PATH  IS: $source_file\n";
    #print STDERR "DESTINATION FILE PATH IS: $write_file\n";

    my $error;

    # checking to see whether we received a file handle or a string
    my $filehandle;
    my $close_handle_flag = undef;

    if( ref( $source_file ) ){
        $filehandle = $source_file;
    } else {
        if (open FHANDLE, "<$source_file") {
            $filehandle = *FHANDLE;
            $close_handle_flag++;
        } else {
            $error = _ERROR_READING_FILE_;
        }
    }

    unless ($error) {
        #print STDERR "write file name [$write_file]; write file dir [$write_dir]\n";
        $class->check_write_directory($write_dir);

        if( -e $write_file && !$overwrite) {
        # file already exists, must not allow upload
            $error = _ERROR_FILE_EXISTS_;
        }
    }

    unless ($error) {
        # first make sure we can write the file
        if (!open(WFD,">$write_file")) {
            $error = _ERROR_WRITING_FILE_;
        }

        unless ($error) {
            # move the data from source to destination
            my ($bytes_read, $buff);
            while ( $bytes_read = read( $filehandle, $buff, 2096 ) ) {
                $size += $bytes_read;
                binmode WFD;
                print WFD $buff;
            }
            close(WFD);

            if( $close_handle_flag ){
                close($filehandle);
            }

            # remove the file if it is empty
            if ((stat $write_file)[7] <= 0) {
                unlink($write_file);
                $error = _ERROR_FILE_EMPTY_;
            }
        }
    }

    return {
        error     => $error,
        size      => $size,
        file_name => $write_file,
    };
}



sub space_used {
    my $self = shift;
    my $uid = shift;

    #$uid = DBAccess->quote( $uid );
    $uid = undef unless(Utility->is_valid_integer($uid));    #secu_200401

    my ($c) = DBAccess->sqlSelectMany( "file_size", "uploads", "userid=$uid" );
    my $size = 0;

    while ( my $s = $c->fetchrow()){
    $size += $s;
    }

    my $number = $c->rows;  # this is ok because $c has been fetched
    $c->finish();

    return ($size, $number);
}


sub get_global_used_uploads_space_and_num_files {
    my $class = shift;

    my ($c) = DBAccess->sqlSelectMany( "file_size", "uploads" );
    my $size = 0;

    while ( my $s = $c->fetchrow()){
    $size += $s;
    }

    my $number = $c->rows; # this is ok because $c has been fetched
    $c->finish();

    # add the totals from legacy uploads in Gizmo
    my ($siz, $num) = Gizmo->total_space_used();

    $size += $siz;
    # add default disk usage
    $size += MetadotConfig->get_initial_disk_usage_mb()*1024*1024;
    $number += $num;

    return ($size, $number);
}


# returns a hash with info about a file attachment.
sub get_file_info {
    my $class  = shift;
    my $args = shift || {};

    my $isa             = $args->{isa};
    my $id              = $args->{id};
    my $field_name      = $args->{field_name};

    $DISPLAY->debug_msg(4, "get_file_info: isa=$isa; id=$id; field_name=$field_name");
    my $result;

    $id = undef unless(Utility->is_valid_integer($id));  #secu_200401

    if (defined $id && defined $isa && defined $field_name) {
        my ( $file_name, $stored_at, $type ) = DBAccess->sqlSelect(
            "file_name, stored_at, type",
            "uploads",
            "isa=".DBAccess->quote($args->{isa})
                  ." and id=$args->{id} and field_name="
                  .DBAccess->quote($args->{field_name})
        );

        my $icon_src = $DISPLAY->get_icon_src( $file_name ) ;

        my $url;

        if (defined $file_name) {
            if( $stored_at eq STORAGE_TYPE_PUBLIC ) {
                my $public_location = MetadotConfig->get_public_uploads_location();
                $url = MetadotConfig->get_public_uploads_location() .'/'. $file_name;
            } else {
                $url = $DISPLAY->make_url("index.pl?id=$args->{id}&amp;isa=$args->{isa}&amp;field_name=$args->{field_name}&amp;op=download_file");
            }
        }

        $result = {
            file_name   => $file_name,
            stored_at   => $stored_at,
            upload_type => $type,
            icon_url    => $icon_src,
            url         => $url,
        };

    }

    return $result;
}


# returns the filename, icon src, and url. Does not decorate with any html
sub get_upload_filename_iconsrc_and_url {
    my $self = shift;
    my $params = shift;

    my $file_info = $self->get_file_info($params);

    my $filename  = $file_info->{file_name};

    $DISPLAY->debug_msg(5,"UM get_upload_filename_iconsrc_and_url isa[".$params->{isa}."]
                            id[".$params->{id}."] field_name[".$params->{field_name}."]");

    $DISPLAY->debug_msg(5,"UM filename[".($filename||'')."]");
    unless( $filename ) { return }

    my $url;


    # CKIM: TODO: REFACTOR (NICO)
    # uggh, this is ugly
    my $message_flag = 0;
    if( defined( my $did = $params->{did} ) ){
        ### change the url for discussion message attachments
        $message_flag = 1;
    }

    if ($message_flag) {
        $DISPLAY->debug_msg(5,"Will change the URL DISCUSSION");
        my $url_string = "index.pl?mid=".$params->{id}."&amp;discussion_id=".$params->{did}.
                         "&amp;isa=".$params->{isa}."&amp;field_name=".$params->{field_name}.
                         "&amp;op=download_file";

        $url = $DISPLAY->make_url($url_string);

    } else {
        $url = $file_info->{url};
    }

    $DISPLAY->debug_msg(5,"WILL RETURN: filename[$filename] info[".$file_info->{icon_url}."] url[$url]");
    return ( $filename, $file_info->{icon_url}, $url );
}


# returns the filename, icon, and url. _Does_ decorate with html
sub get_upload_filename_icon_and_url {
    my $self = shift;
    my $params = shift;

    $params->{isa} || die "must include param isa";
    ( defined $params->{id} ) || die "must include param id";
    $params->{field_name} || die "must include param field_name";

    my ($filename, $icon_src, $url) = $self->get_upload_filename_iconsrc_and_url($params);

    my $attach;
    if (defined $filename) {
        $DISPLAY->debug_msg(5,"HURRAY! got filename[$filename]");
        $attach = "&nbsp;" . "<a href='" . $url . "'>" . "<img class=\"icon\" align='bottom' src=\"" . $icon_src . "\" border=\"0\" align='center' alt='Download Me!' width=\"16\" height=\"16\"></a>" . "&nbsp;" . "<a href='" . $url . "'><span class=\"centerNewsLink\">download</span></a>" . "&nbsp;" ;
    }
    $DISPLAY->debug_msg(5,"f[$filename] icon[$icon_src] url[$url]");

    return ( $filename, $attach, $url );
}



sub get_upload_url {
    my $self = shift;
    my $params = shift;

    $params->{isa} || die "must include param isa";

    $params->{id} = undef unless(Utility->is_valid_integer($params->{id}));  #secu_200401
    ( defined $params->{id} ) || die "must include param id";
    $params->{field_name} || die "must include param field_name";

    my $where = "isa=".DBAccess->quote($params->{isa})." and id=$params->{id} and field_name=".DBAccess->quote($params->{field_name});  #secu_200401
    if ($params->{file_name}) {
        $where .= " and file_name='$params->{file_name}'";
    }

    my ( $filename, $stored_at ) = DBAccess->sqlSelect( "file_name, stored_at",
                            "uploads",
                            $where );


    unless( $filename ) { return '' }

    my $url = '';

    if( $stored_at eq STORAGE_TYPE_PUBLIC ) {
    $url = MetadotConfig->get_public_uploads_location() . "\/$filename";
    } else {
    $url = $self->make_url("index.pl?id=$params->{id}&amp;isa=$params->{isa}&amp;field_name=$params->{field_name}&amp;op=download_file");
    }

    return $url;
}

sub delete_all_uploads {
    my $self = shift;
    my $args = shift;

    $args->{isa} || die "must include param isa";
    $args->{id} = undef unless(Utility->is_valid_integer($args->{id}));  #secu_200401
    ( defined $args->{id} ) || die "must include param id";

    my $write_dir = $args->{write_dir} || $PARAMS->{'absolutepathfileuploaddir'};

    my ($records) = DBAccess->sqlSelectMany("field_name, file_name, stored_at, id", "uploads", "id=$args->{id} and isa=".DBAccess->quote($args->{isa}));  #secu_200401


    while ( my ( $field_name, $file_name, $stored_at, $id ) = $records->fetchrow() ){

        if( $stored_at eq STORAGE_TYPE_PUBLIC ) {

            my $write_file = MetadotConfig->get_public_uploads_dir() . "/" . $file_name;

            unlink( $write_file );

            DBAccess->sqlDelete( "uploads", "isa='$args->{isa}' and id='$args->{id}' and field_name='$field_name'" );

        } else {

            # take care of ac files
            unlink( "$write_dir/$file_name" );
            DBAccess->sqlDelete( "uploads", "isa=".DBAccess->quote($args->{isa})." and id=$args->{id} and field_name=".DBAccess->quote($field_name) );
        }
    }
}

# returns a list of the public uploads, optionally filtering on an 'isa' value.
# the format of the return value is an array of hashes, each containing a 'url' and 'filename' key
sub list_public_uploads {
    my $self = shift;
    my $params = shift;
    my @results = ();

    my $where_clause = "stored_at=".DBAccess->quote(STORAGE_TYPE_PUBLIC);
    if ($params->{isa}) {
        # check if isa is a ref to an array
        if (ref($params->{isa}) eq 'ARRAY') {
            # put single quotes around each element
            for (my $i=0; $params->{isa}->[$i]; $i++) {
                $params->{isa}->[$i] = DBAccess->quote($params->{isa}->[$i]);
            }
            $where_clause .= ' and isa in ('.join(',',@{$params->{isa}}).')';
        } else {
            $where_clause .= " and isa=".DBAccess->quote($params->{isa});  #secu_200401
        }
    }
    my ($records) = DBAccess->sqlSelectMany("file_name", "uploads", $where_clause);

    while ( my ($file_name) = $records->fetchrow() ){
        push @results, {    file_name   => $file_name,
                            url         => MetadotConfig->get_public_uploads_location() . "\/$file_name",
        }
    }

    return @results;
}


#
# will return the number of bytes in all uploaded files
# found according to search terms
#
sub get_upload_size {
    my $self = shift;
    my $params = shift;

    $params->{id} = undef unless(Utility->is_valid_integer($params->{id}));  #secu_200401

    my @where;
    if( $params->{isa} ) {
    push @where, "isa=".DBAccess->quote($params->{isa});  # secu_200401
    }
    if( $params->{id} ) {
    push @where, "id=$params->{id}";
    }
    if( $params->{field_name} ) {
    push @where, "field_name=".DBAccess->quote($params->{field_name});      # secu_200401
    }

    my $where = join( ' and ', @where );
#    print STDERR "WHERE CLAUSE IS $where\n";

    my ( $records ) = DBAccess->sqlSelectMany( "file_size",
                           "uploads",
                           $where );

    my $file_size;

    while ( my ($size) = $records->fetchrow() ) {
    $file_size += $size;
    }

#    print STDERR "FOUND SIZE $file_size\n";

    return  $file_size || 0;
}

sub change_uploads_userid {
    my $class = shift;

    my $params = shift;

    $params->{isa} || die "must include param isa";

    $params->{id} = undef unless(Utility->is_valid_integer($params->{id}));  #secu_200401
    ( defined $params->{id} ) || die "must include param id";
    ( defined $params->{new_userid} ) || die "must include param new_userid";

    my $where = "isa=".DBAccess->quote($params->{isa})." and id=$params->{id}";  #secu_200401

    if( $params->{field_name} ) {

    $where .= " and field_name=".DBAccess->quote($params->{field_name});
    }

    DBAccess->sqlUpdate("uploads", $where,
            userid => $params->{new_userid} );

}













1;

__END__

CREATE TABLE uploads (
  uid varchar(20) NOT NULL,
  isa varchar(64) NOT NULL,
  id varchar(64) NOT NULL,
  field_name varchar(64) NOT NULL,
  type varchar(64) NOT NULL,
  stored_at varchar(64) NOT NULL,
  file_name varchar(128) NOT NULL,
  file_size int(11) NOT NULL,
  PRIMARY KEY  (id,isa,field_name)
);
