###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################


package Metadot::User::Default;
use strict;


=head1 LEGAL
This software is licensed under the open source GPL license. For more information please contact info@metadot.com. Everything copyright Metadot Corporation.


=head1 NAME

Package Metadot::User::Default

=head1 DESCRIPTION

Handles user data storage, user profile editing and user profile display. Can be subclassed to provide customized behavior as well as all additions, deletions, and alterations of user accounts.

Note: programmers should use FlexUser.pm and not this Default.pm module when programming.

=cut



use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $ACCESS_BROKER $DISPLAY %REQUEST_VARS $CGI);
use Portal;
use DBAccess;
use Carp;

use MetadotConfig;
use Metadot::UploadsManager;
use Metadot::UserFactory;

use Group;
use Gizmo::Category;
use MyPage;
use Form::Verify;
use HTML::TableFrame;
use ErrorHandling;
use Metadot::Utils::Display;
use Metadot::RegisterFactory;
use Utility;
use Session;

use vars qw(@ISA);
@ISA=qw(Portal); #inherit from the meta class

use constant NAME_ADMIN        => 'admin';
use constant NAME_DEFAULT_LOOK => 'default_look';

my %default_account_names = ( NAME_ADMIN() => 1, NAME_DEFAULT_LOOK() => 1 );


use constant REGISTRATION_MGR_GROUP_NAME => 'REGISTRATION_MGR';
use constant REGISTRATION_MGR_GRP_ID     => 5;

my $AC = {
      BUNDLES => {

                   CREATE => { DESC =>          'Create Account',
                               OPS =>            [qw(register_form register reset_password)],
                               DEFAULT_LEVEL =>  0,
                               MIN_LEVEL =>      0,
                             },
                   MOD =>    { DESC =>          'Edit',
                               OPS =>            [qw(modify_self save_self)],
                               DEFAULT_LEVEL =>  2,
                               MIN_LEVEL =>      2,
                             },
                   SEARCH =>    { DESC =>          'Search',
                               OPS =>            [qw(show_users_portal)],
                               DEFAULT_LEVEL =>  0,
                               MIN_LEVEL =>      0,
                             },
                   DEL =>    { DESC =>          'Delete and Cut',
                               OPS =>            [qw(delete delete_ok cut delete_group delete_request)],
                               DEFAULT_LEVEL =>  8,
                               MIN_LEVEL =>      8,
                               PARENT_DEP =>     {cut => 'MOD'},
                             },
                   DISP =>   { DESC =>          'View',
                               OPS =>            [qw(require_pager_form process_pager_form require_icq_form process_icq_form show_own_subscriptions)],

                               DEFAULT_LEVEL =>  2,
                               MIN_LEVEL =>      2,
                               DEFAULTS_TO =>    'on',
                             },
                  ADMIN => { DESC =>            'Admin Modify',
                             OPS =>              [qw(modify save manage_subscriptions become_user confirm_become_user show_subscriptions)],
                             DEFAULT_LEVEL => 9,
                             MIN_LEVEL =>     9,
                             DEFAULTS_TO =>   'on',
                            },
                   ADMIN_REGISTRATIONS => {
                             DESC           => 'Administer registrations',
                             OPS            => [qw(show_details show_users accept_registration accept_registration_confirm reject_registration reject_registration_confirm)],
                             DEFAULT_LEVEL  => 9,
                             MIN_LEVEL      => 9,
                             DEFAULTS_TO    => 'on',
                   },
                   EDITP =>  { DESC =>          'Change<br />Permissions',
                               OPS =>            [qw(edit_permissions set_permissions change_owner)],
                               DEFAULT_LEVEL =>  8,
                               MIN_LEVEL =>      8,
                             },
                  },
          DISPLAY_ORDER => [qw(DISP CREATE MOD DEL ADMIN ADMIN_REGISTRATIONS EDITP)],

          # give ADMIN_REGISTRATIONS rights to members of the
          ACL =>
          {
              FOR_GROUPS =>
              {
                  &REGISTRATION_MGR_GRP_ID =>
                  {
                      ADMIN_REGISTRATIONS => 1,
                  },
              }
          },
        };




##################################################################
##################################################################

=head1 METHODDS

The following methods are available:

=cut

##################################################################
##################################################################



##################################################################

=head2 $USER->get_AC()

Returns access control hash.

=cut

##################################################################

sub get_AC {
    return $AC;
}


##################################################################

=head2 $USER->is_a()

Returns the object class.

=cut

##################################################################

sub is_a {
    my $self = shift;

    return ref($self);
}


##################################################################

=head2 $USER->id()

Always returns 1.

=cut

##################################################################

sub id {
    my $self = shift;

    return 1;
}


##################################################################

=head2 $USER->get_owner_id()

Returns MetadotConfig->get_reserved_uid('ADMIN');

=cut

##################################################################

sub get_owner_uid {
    my $self = shift;

    return MetadotConfig->get_reserved_uid('ADMIN');
}


##################################################################

=head2 $USER->parent_id()

Returns 0 (users have no parents);

=cut

##################################################################

sub parent_id {
    my $self = shift;
       # my $id = $user->parent_id ;

    return 0;
}


##################################################################

=head2 $USER->new( [$uid] )

Return a new user or a user object created from the $uid.

=cut

##################################################################

sub new {
    my $proto = shift;
    my $uid = shift;
    my $class = ref($proto) || $proto;
    my $self = {};

    $self->{uid}            = undef;
    $self->{nickname}       = "";
    $self->{fullname}       = "";
    $self->{quote}          = "";
    $self->{email}          = "";
    $self->{approval_needed} = 0;
    $self->{password}       = "";
    $self->{creation_date}   = "";
    $self->{update_date}     = "";
    $self->{pager_number}    = "";
    $self->{icq_number}      = "";
    $self->{icq_urgent}      = "";
    $self->{last_online}     = "0";
    $self->{ip}             = "";
    $self->{admin_level}     = 1;
    $self->{homepage}       = "";
    $self->{debug_level}     = 0;
    $self->{admin_mode}      = 0;
    $self->{password}        = "";
    $self->{link_name_1}      = "";
    $self->{link_name_2}      = "";
    $self->{link_name_3}      = "";
    $self->{link_name_4}      = "";
    $self->{link_name_5}      = "";
    $self->{link_name_6}      = "";
    $self->{link_name_7}      = "";
    $self->{link_name_8}      = "";
    $self->{link_name_9}      = "";
    $self->{link_name_10}     = "";
    $self->{link_name_11}     = "";
    $self->{link_name_12}     = "";
    $self->{link_name_13}     = "";
    $self->{link_name_14}     = "";
    $self->{link_name_15}     = "";
    $self->{link_url_1}       = "";
    $self->{link_url_2}       = "";
    $self->{link_url_3}       = "";
    $self->{link_url_4}       = "";
    $self->{link_url_5}       = "";
    $self->{link_url_6}       = "";
    $self->{link_url_7}       = "";
    $self->{link_url_8}       = "";
    $self->{link_url_9}       = "";
    $self->{link_url_10}      = "";
    $self->{link_url_11}      = "";
    $self->{link_url_12}      = "";
    $self->{link_url_13}      = "";
    $self->{link_url_14}      = "";
    $self->{link_url_15}      = "";
    $self->{ldap_dn}           = "";
    $self->{upload_size}    = "";
    $self->{upload_number}    = "";
    $self->{password_changed_time}  = "";
    $self->{pw_expire_notification_time} = "";

    # we don't check if a user is logged_in until
    # is_logged_in is called
    $self->{_logged_in} = undef;

    bless ($self, $class);

   # $self->debug_msg(4, "Usernew()");

    if (defined ($uid)) {
    $self->{uid} = $uid;
    $self->_initialize();
    }

    return $self;
}


#######################################################################
#  RESTRICTED-USER: THESE FUNCTIONS NEED TO BE REMOVED WHEN
#  PROPER ROLE-BASED ACL IS IN PLACE
#######################################################################
my $restricted_flag = '_RESTRICTED_USER';
my $restricted_attribute = 'link_name_1'; #this is safe because restricted users
                                          #cannot have a mypage.
my $owner_attribute      = 'link_name_2';


##################################################################

=head2 $USER->set_restricted( $value )

Set a user as restricted.

=cut

##################################################################

sub set_restricted {
    my $self = shift;
    my $value = shift;

    # if no value is sent at all, then default is true
    if (!defined($value)) {$value = 1};

    if ($value) {
        # set the restricted_attribute to the flag for restricted users
        $self->{$restricted_attribute} = $restricted_flag;
    } else {
        # changing the user to non-restricted
        # check to see if the user was restricted to begin with
        if ($self->is_restricted()) {
            $self->{$restricted_attribute} = '';
        }
    }
}


##################################################################

=head2 $USER->is_restricted()

Returns true if user is restricted.

=cut

##################################################################
sub is_restricted {
     my $self = shift;
     my $result = (defined($self->{$restricted_attribute}) && ($self->{$restricted_attribute} eq $restricted_flag));
     return $result;
}



##################################################################

=head2 $USER->get_restricted_owner()

Returns owner attribute if user is restricted.

=cut

##################################################################

sub get_restricted_owner {
    my $self = shift;
    my $result;
    if ($self->is_restricted()) {
        $result = $self->{$owner_attribute};
    }
    return $result;
}


##################################################################

=head2 $USER->set_restricted_owner( $owner_attribut )

A restricted users have owners. This sets the uid of the owner.

=cut

##################################################################

sub set_restricted_owner {
    my $self = shift;
    my $input = shift;
    if ($self->is_restricted()) {
        $self->{$owner_attribute} = $input;
    }
}


##################################################################

=head2 $USER->is_orphan()

A restricted user is orphaned if he does not belong to any groups.
This deletes it. Return 1 if the user is an orphan.

=cut

##################################################################

sub is_orphan {
    my $self = shift;
    my $result = 0;

    my $grp_ref = Group->get_group_names_where_member($self->uid()) || [];
    unless (ref $grp_ref eq 'ARRAY') {
        $grp_ref = [$grp_ref];
    }
    unless (@$grp_ref) {
        $result = 1;
    }

    return $result;
}


##################################################################

=head2 $USER->delete_if_restricted_orphan()

A restricted user is orphaned if he does not belong to any groups.
This deletes it.

=cut

##################################################################

sub is_restricted_orphan {
    my $self = shift;
    my $result = ($self->is_restricted() && $self->is_orphan());

    return $result;
}


##################################################################

=head2 $USER->delete_if_restricted_orphan()

This will delete a user if he is a restricted orphan

=cut

##################################################################

sub delete_if_restricted_orphan {
    my $self = shift;
    if ($self->is_restricted_orphan()) {
        $self->delete();
    }
}


##################################################################

=head2 $USER->www_restricted_user_home()

This will find the pages that the restricted user can see, and allow him to choose from them. If there is only one choice, the system will redirect


=cut

##################################################################
sub www_restricted_user_home {
    my $self = shift;

    use HTML::Decorator::BulletDecorator;
    my $decorator = 'HTML::Decorator::BulletDecorator';

    my @group_names = @{ Group->get_group_names_where_member($self->{uid}) };
    my @instances;
    my $page_list = '';
    for my $group_name (@group_names) {
        my $group = Group->get_group_by_name($group_name);
        my $iid = $group->{iid};
        if ($iid) {
            my $instance = Gizmo->restore($iid);
            push @instances, $instance;
            $page_list .= $decorator->new(component=>$instance)->show_summary()."<br />\n";
        }
    }

    # if only one instance was found, just redirect them
    if (scalar(@instances) == 1) {
        my $instance = $instances[0];
        my $url = $self->make_url("index.pl?iid=$instance->{id}&amp;isa=$instance->{is_a}");
        Utility->md_redirect($url);
    }

    # if the user has no instances at all, send them the error
    if (!@instances) {
        $self->print_click_back( "Error", "Sorry, you are not authorized to view this site.");
        exit;
    }

    my $title = 'title';
    my $welcome = $DISPLAY->display_welcome_message();
    my $content = <<EOM
    <h1>$welcome</h1>
    You have access to the following pages:<br /><br />
    $page_list
EOM
;
    $self->print_portal($content, $title);
}



##################################################################

=head2 $USER->www_delete_request( $uid )

Displays user delete confirmation dialog.

=cut

##################################################################

sub www_delete_request {
    my $self = shift;

    my $referer = $FORM{referer} || $SESSION->{referer};
    my $escaped_referer = $CGI->escape($referer);

    if(!$FORM{uid}){
    $self->print_click_back("Error",
                "Sorry I didn't get a user uid to delete!");
    return 0;
    }

    my $u = Metadot::UserFactory->get_user( $FORM{uid} );

    if ( $USER->is_default_account( $u->email ) ) {

    $self->print_click_back("Error", "Sorry, system special users cannot be deleted");
    return 0;
    }

    my $fullname = " " . $u->webname;


    $self->three_choices( "Delete user: <br /><i>$fullname</i> ad vita eternam?",
              "caution",
              "Yes, delete user but leave his/her content untouched",
              "index.pl?isa=User&op=delete_ok&uid=$FORM{uid}&referer=$escaped_referer",
              "Yes, delete user <b>and all his/her content</b>",
              "index.pl?isa=User&op=delete_ok&uid=$FORM{uid}&delete_user_content=1&referer=$escaped_referer",
              "Cancel", $referer );

    return 1;

}


##################################################################

=head2 $USER->www_delete_ok( $uid )

Web interface that deletes a user and his dependencies.

=cut

##################################################################

sub www_delete_ok {

    my $self = shift;
    my  $uid = shift || $FORM{uid};
    my $res=0;

    ## NZ no longer used
    #my $referer = $FORM{referer} || $SESSION->{referer};
    my $target = Utility->get_qualified_url("index.pl?isa=Portal&iid=0&op=modify_site_params&app=&child_type=0&manage=Users");
    my $delete_content_as_well = $FORM{delete_user_content};

    $self->debug_msg(3, "Metadot::User::Default::www_delete_ok(uid=$uid)");
    if(!$uid){
    $self->print_click_back("Error",
                "Sorry I don't know whom to delete (UID=$uid)");
    return 0;

    } else{

    my $u = Metadot::UserFactory->get_user($uid);

    $res=$u->delete($delete_content_as_well); #backend function :)

    if($res==1){
        $self->debug_msg(4, "Metadot::User::Default::user deleted (uid=$uid)");
        $DISPLAY->print_choices("<Big>User deleted!</big><br />",
                   "confirm",
                   "Continue",
                   $target
                  );
    }else{
        $self->debug_msg(4, "Metadot::User::Default::problem: user not deleted (uid=$uid), error code is ($res)");
        $self->print_click_back("Error deleting user",
                    "Please contact your system admin if this error persists.<br />Error code: $res");
    }
    return 1;
    }
}

my @SHOW_DETAIL_FIELDS = (
    {   field_name  => 'fullname',
        label       => 'Full Name',
        nowrap      => 1,
    },
    {   field_name  => 'nickname',
        label       => 'Nickname / Screenname',
        nowrap      => 1,
    },
    {   field_name  => 'email',
        label       => 'E-mail',
        nowrap      => 1,
    },
    {   field_name  => 'homepage',
        label       => 'Home Page',
    },
    {   field_name  => 'quote',
        label       => 'Quote',
    },
    {   field_name  => 'icq_number',
        label       => 'ICQ Numebr',
    },
    {   field_name  => 'pager_number',
        label       => 'Pager Number',
    },
    {   field_name  => 'uid',
        label       => 'Unique ID',
    },
);


##################################################################

=head2 $USER->_get_show_detail_fields()

Returns @SHOW_DETAIL_FIELDS which a limited set of the user attributes.

=cut

##################################################################

sub _get_show_detail_fields {
    return @SHOW_DETAIL_FIELDS;
}
sub exec_manage_subscriptions {
    my $self = shift;
    my $uid = shift || $FORM{uid} || $self->uid();
    if(!$uid){
        $self->print_click_back("Error", "Sorry I didn't get a user uid (and I need one!)<br /><br />");
        return 0;
    }
    return 'manage_user_subscriptions.tt2';

}
##################################################
### showUserSubscriptions
##################################################
sub exec_show_subscriptions {
    my $self = shift;
    my $uid = $FORM{uid} || $self->uid();

    unless( Utility->is_valid_integer($uid) ){
        $self->print_click_back("Error","Looking for something?<br /><br />");
        exit(0);
    }

    if(!$uid){
       $self->print_click_back("Error", "Sorry I didn't get a user uid (and I need one!)<br /><br />");
       return 0;
    }

    $REQUEST_VARS{subscription_userid} = $uid;
    $self->_process_group_subscription_updates($uid);
    return 'user_subscriptions.tt2';
}


sub exec_show_own_subscriptions {
    my $self = shift;

    $REQUEST_VARS{subscription_userid} = $self->uid();
    $self->_process_group_subscription_updates($self->uid());
    return 'user_subscriptions.tt2';
}


sub _process_group_subscription_updates {
    my $self = shift;

    # see if there was any form input
    $DISPLAY->debug_msg(4, "IN _process_group_subscription_updates");
    if (exists $FORM{submit_group_subscription_updates}) {
        my $grpid_cids = $FORM{'grpidcid'};
        unless (ref $grpid_cids) {
            $grpid_cids = [$grpid_cids];
        }

        foreach my $grpid_cid (@$grpid_cids) {
            my ($grp_id, $cid) = split /:/, $grpid_cid;
            my $value = $FORM{"opt_out:$grp_id:$cid"} || '';
            $DISPLAY->debug_msg(5, "_process_group_subscription_updates processes $grpid_cid ($grp_id, $cid) = $value");
            $self->opt_out_status($cid, $value);
        }
    }
}


sub opt_out_status {
    my $self   = shift;
    my $cid    = shift;
    my $value  = shift;

    $DISPLAY->debug_msg(5, "opt_out_status($cid, $value)");
    if (defined $value) {
        return Subscription->set_user_opt_out_status($self->uid(), $cid, $value);
    } else {
        return Subscription->get_user_opt_out_status($self->uid(), $cid);
    }
}

###################################################
# sub become_user
# method updates params table to allow a site admin
# to become another user in the users table
###################################################
sub www_become_user {
    my $self      = shift;
    my $become_id = $FORM{'uid'};

    $self->test_can_become_user($become_id);

    my ($fullname) = DBAccess->sqlSelect("fullname","users","userid=$become_id");

    ## display a message and redirect to the home page
    $DISPLAY->two_choices("You are about to become \"$fullname\". Are you sure you wish to continue?","notice",
                             "Yes, transform me!","index.pl?isa=User&op=confirm_become_user&uid=$become_id",
                             "Nevermind",$SESSION->{referer} );

}

sub www_confirm_become_user {
    my $self      = shift;
    my $become_id = shift || $FORM{'uid'};

    $self->test_can_become_user($become_id);

    $SESSION->_change_user($become_id);

    #Turn off editing mode
    $SESSION->edit_off();

    $DISPLAY->print_choices("Success! You were transformed!",
                             "confirm","Continue","index.pl?id=0");


}


sub can_become_user
{
  my $self = shift;
  my $become_uid = shift;

  my $allowed =
    # if you have server_setup, you can do anything
    $ACCESS_BROKER->is_allowed_to_do('server_setup')

    # this is just a formality, in case somebody decides to play games
    # in the future
    && $self->is_allowed_to_do('become_user');

  unless ($allowed)
  {
    # need to create the "become user" in memory for testing purposes
    my $become_user = Metadot::UserFactory->get_user($become_uid);

    # if you can admin, you can manage anyone who doesn't have server_setup
    $allowed ||= (
      $ACCESS_BROKER->is_allowed_to_do('admin_tasks')
      && !$ACCESS_BROKER->is_allowed_to_do('server_setup', $become_user)
    );
  }

  return $allowed;
}


sub test_can_become_user {
    my $self       = shift;
    my $become_uid = shift;

#    if( $USER->belongs_to_grp('ADMIN', $become_uid) && !$ACCESS_BROKER->is_allowed_to_do('server_setup') ){
#        $DISPLAY->print_click_back("Error: you are not allowed to become this user.");
#        exit;
#    }
#
#    return 1;

    # CKIM: rewrote this to consider more cases
    # if you can setup the server, you can do anything
    my $allowed = $self->can_become_user($become_uid);

    if ($allowed)
    {
      return $allowed;
    }
    else
    {
      $DISPLAY->print_click_back("Error: you are not allowed to become this user.");
      exit;
    }
}


##################################################################

=head2 $USER->www_show_detais([$uid])

Show the user info. If a $uid is passed as argument then it will show the users info of this user.

=cut

##################################################################

sub www_show_details {
    my $self = shift;
    my $uid  = shift || $FORM{uid} || $self->uid();

    $self->debug_msg(3,"Metadot::User::Default::www_show_details($uid)");
    if(!$uid){
        $self->print_click_back("Error", "Sorry I didn't get a user uid (and I need one!)<br /><br />");
        return 0;
    }

  ######################################
  #### build and display the page
  ######################################

#   my $html = $self->show_top();
#   $html .= $self->show_title("User Information");
#   $html .= $self->_get_user_details_display(uid => $uid);
#   $html .= $self->show_bottom();
#   $self->print_http_header();
#   print $html;
    my $u = Metadot::UserFactory->get_user($uid);

    my $content = $PARAMS->{show_user_directory} && $u->can('get_user_details_view') ? $u->get_user_details_view(view => 'businesscard') : $self->_get_user_details_display(uid => $uid);
    $DISPLAY->print_simple_layout({content=>$content, title=>"User Information"});
}


##################################################################

=head2 $USER->_get_user_details_display( )

Displays all user profile fields.

=cut

##################################################################
sub _get_user_details_display {
    my $self = shift;
    my %args = @_;

    my $uid = $args{uid};
    $self->debug_msg(3,"Metadot::User::Default::_get_user_details_display($uid)");

    my $s = "";
    my $u = Metadot::UserFactory->get_user($uid);

    if ($u) {
        my @fields = $u->_get_show_detail_fields();
        my %properties = ();
        foreach my $field( @fields) {
            my $field_name = $field->{field_name};
            if ($field_name eq 'email') {
                my $email = $u->email();
                $properties{$field_name} = "<a href='mailto:$email'>$email</a>";
            } elsif ($field_name eq 'homepage') {
                my $homepage = $u->homepage();
                if ($homepage) {
                    $properties{$field_name} = "<a href='".$self->make_url($homepage)."'>$homepage</a>";
                } else {
                    $properties{$field_name} = "&nbsp;";
                }
            } else {
                $properties{$field_name} = $u->get_value($field_name) || '&nbsp;';
                $DISPLAY->debug_msg(4, "_get_user_details_display has field_name '$field_name', value: '$properties{$field_name}'");
            }
        }

        $s = "\n\n<!-- USER INFORMATION -->\n\n";
        $s .= "<br /><br /><center><table border=\"1\", cellpadding=\"4\" cellspacing=\"0\" width='%80'></tr>\n\n";
        foreach my $field (@fields) {
            my $nowrap= $field->{nowrap} ? 'nowrap="nowrap"' : '';
            $s .= "<tr><td $nowrap><b>$field->{label}</b></td><td>$properties{$field->{field_name}}</td></tr>\n";
        }
        $s .= "</tr></table></center><br /><br />\n\n";
        $s .= "\n\n<!-- END OF USER INFORMATION -->\n\n";
    }else{
        # the user doesn't exist anymore
        $s = "\n\n<!-- USER INFORMATION -->\n\n";
        $s .= "<br /><br /><CENTER><b>Sorry, this user doesn't exist anymore!</b></CENTER><br /><br />";
        $s .= "\n\n<!-- END OF USER INFORMATION -->\n\n";
    }

    return $s;
}

sub www_reset_password {
    my $self = shift;

    return $self->www_modify_self();

}


##################################################################

=head2 $USER->www_modify( )

Returns true if this user object is able and permitted to show
user directory.

=cut

##################################################################

sub can_show_user_directory {
    my $self = shift;
    return ($PARAMS->{show_user_directory} && $self->can('www_show_user_directory'));
}


##################################################################

=head2 $USER->www_modify( )

Allows the user to update his/her profile.

=cut

##################################################################

sub www_modify_self {
    my $self = shift;

    # calling backend method.
    # the user will not see the "make admin"
    # radiobuttom because $show_admin_level is equal to zero,
    my $registrator = Metadot::RegisterFactory->get_register();
    $DISPLAY->debug_msg(3, __PACKAGE__
        ." / $self\->www_modify_self, with registrator $registrator"
        );

    my $uid = $SESSION->uid();

    my $show_admin_level = 0;

    my $form;
    my $form_name;
    if ($FORM{new_user}) {
        $form = $registrator->show_new_password_form($uid);
        $form_name = 'Creating your password';
    } elsif($FORM{key}){
                $form = $registrator->www_change_password();
                $form_name = 'Update your password information';
        } else {
        $form = $registrator->show_modify_settings_form( $uid, $show_admin_level );
        $form_name = 'Modifying User Settings';
    }


    $DISPLAY->print_form($form, $form_name);

    return;
}


##################################################################

=head2 $USER->www_modify( )

Allows the admin the modify user's settings.

=cut

##################################################################

sub www_modify {
    my $self = shift;

    my $uid = $FORM{uid};

    if( $USER->belongs_to_grp('ADMIN', $uid ) ) {

    unless( $ACCESS_BROKER->is_allowed_to_do('server_setup') ) {

        $DISPLAY->print_click_back("Error: you are not allowed to modify this user's information.");
        exit;
    }
    }

    # A little trick: if uid ne $SESSION->uid then it's an
    # admin - we should turn on 'show_admin_level'.
    my $registrator = Metadot::RegisterFactory->get_register();

    my $show_admin_level = ($uid ne $SESSION->uid() ) ? 1 : 0 ;

    my $form = $registrator->show_modify_settings_form( $uid, $show_admin_level );

    $DISPLAY->print_form($form, "Modifying User Settings");

    return;
}


##################################################################

=head2 $USER->save( )

Save a user profile page.

=cut

##################################################################

sub www_save {
    my $self = shift;

    my $keep_admin_level = 0;

    my $uid = $SESSION->get_value('User_uid_to_modify');

    if( $USER->belongs_to_grp('ADMIN', $uid ) ) {

    unless( $ACCESS_BROKER->is_allowed_to_do('server_setup') ) {

        $DISPLAY->print_click_back("Error: you are not allowed to modify this user's information.");
        exit;
    }
    }

    my $registrator = Metadot::RegisterFactory->get_register();

    $registrator->save_modify_settings_form( $keep_admin_level );

    return;
}


##################################################################

=head2 $USER->www_save_self( )

Save the user profile page.

=cut

##################################################################

sub www_save_self {
    my $self = shift;

    my $keep_same_admin_level = 1;

    my $registrator = Metadot::RegisterFactory->get_register();

    if ($FORM{new_user}) {
        $registrator->save_new_password_form( $keep_same_admin_level );
    } else {
        $registrator->save_modify_settings_form( $keep_same_admin_level );
    }

    return;
}


##################################################################

=head2 $USER->_initialize( )

Initialize a user object.

=cut

##################################################################

sub _initialize{
    my $self = shift;
    my $uid = $self->uid;

    if(defined ($uid)){
        #RESTRICTED-USER: using linkname1, linkname2 for restricted user properties
        my $att = "userid, fullname, nickname, email, homepage, quote, icqnumber, pager, adminmode, debuglevel, lastonline, approvalneeded, password, ldapdn,max_total_upload_size,max_total_numberof_upload, linkname1, linkname2, password_changed_time, pw_expire_notification_time";

        (
            $self->{uid}, $self->{fullname}, $self->{nickname}, $self->{email},
            $self->{homepage}, $self->{quote}, $self->{icq_number},
            $self->{pager_number}, $self->{admin_mode},
            $self->{debug_level},
            $self->{last_online},
            $self->{approval_needed},
            $self->{password},
            $self->{ldap_dn},
            $self->{upload_size},
            $self->{upload_number},
            $self->{link_name_1},
            $self->{link_name_2},
            $self->{password_changed_time},
            $self->{pw_expire_notification_time},
        )
        = DBAccess->sqlSelect($att, "users", "userid='$uid'");

        my $level = $self->admin_level() || 1;
    }
}




##################################################################

=head2 $USER->delete( $debug_level )

Set the debug level for a user.

=cut

##################################################################

sub debug_level {
    my $s = shift;
    my $level = shift;
    my $uid = $s->uid;
    $s->debug_msg(3, "Metadot::User::Default::debug_level: ". $s->{debug_level});

    (defined ($level))
      ? (DBAccess->sqlUpdate("users", "userid=$uid", (debuglevel => $level)),
    $s->{debug_level} = $level)
    : ($level = $s->{debug_level});

    return $s->{debug_level};
}



##################################################################

=head2 $USER->delete( $delete_user_personal_content )

Deletes:
 o if $delete_user__personal_content is true then My website/portal pages and all the content in it is deleted.
 o My Page info
 o Subcriptions
 o Sessions
 o User record
 o group memberships
 o set the groups owners created by this user to belong to admin

=cut

##################################################################

sub delete {
    my $self = shift;
    my $delete_content_as_well = shift;

    my $uid;
    $self->debug_msg(3, "Metadot::User::Default::delete(uid='".($uid||'')."')");
    $uid = $self->uid();

    if(!$uid){
        $self->debug_msg(3, "Metadot::User::Default::Error User::delete(): I don't know what uid to delete");
        return 0;
    }

    my ($hpiid)=$self->has_portal();

    if( $delete_content_as_well && ( defined ($hpiid) && $hpiid != 0 ) ){

        $DISPLAY->debug_msg(1, "DELETING USERS MYWEBSITE " . $self->fullname());

        ## we need to delete the tree of cat/item/discuss the users has created
        $self->debug_msg(3, "Metadot::User::Default::delete: deleting portal iid=$hpiid");

        my $c = Gizmo->new($hpiid);
        $c->delete("YES"); # yes = delete recursively sub-cat
        #Metadot::deleteCategory($hpiid, "YES");
    }

    DBAccess->sqlDelete ("mypage", "userid=$uid");
    DBAccess->sqlDelete ("subscription", "userid=$uid");
    DBAccess->sqlDelete ("sessions", "userid=$uid");

    DBAccess->sqlDelete ("users", "userid=$uid");
    $self->debug_msg(4, "Metadot::User::Default::user deleted (userid=$uid)");
    my $admin_uid = MetadotConfig->get_reserved_uid("ADMIN");
    DBAccess->sqlDelete ("grpmembers", "userid=$uid");
    DBAccess->sqlUpdate ("grp", "userid=$uid", (-userid => $admin_uid));

    return 1;
}




##################################################################

=head2 $USER->has_portal()

Returns the iid of the my website/portal, zero otherwise.

=cut

##################################################################

sub has_portal{
  my $self = shift;
  my $uid  = shift;

  unless (defined $uid) {
      $uid = $self->uid();
  }

  $self->debug_msg(5, "Metadot::User::Default::has_portal(uid='" . ($uid||"") . "')");
  if (!defined($uid) || !$uid){
    return 0;
  }

  my $portal_parent = MetadotConfig->get_reserved_iid('PORTAL_PARENT');

  my ($iid) = DBAccess->sqlSelect("iid", "instance", "parentiid=$portal_parent AND userid=$uid");

  if ($iid) {
    return $iid
  }else{
    return 0;
  }
}


##################################################################

=head2 $USER->has_mypage()

Returns a positive number if a user has his 'my page', zero otherwise.

=cut

##################################################################

sub has_mypage{
  my $self = shift;
  my $uid = $self->uid;
  $self->debug_msg(5, "Metadot::User::Default::has_mypage(uid='$uid')");

  return 1 if (!defined($uid)); #we avoid creating zillion mypages

  my ($mp_elements) = DBAccess->sqlSelect("count(*)", "mypage", "userid=$uid");
  return $mp_elements;
}


##################################################################

=head2 $USER->fullname()

Returns the fullname if the user has one, nickname otherwise.


=cut

##################################################################

sub fullname{
  my $self=shift;
  my $val = shift;

  if (defined $val){
    $self->{fullname}=$val;
    return $self->{fullname};
  }
  return $self->{fullname} || $self->{nickname};

}


##################################################################

=head2 $USER->webname()

Returns the nickname if the user has one, fullname otherwise.

=cut

##################################################################

sub webname{
  my $self=shift;

  my $result = $self->{nickname} || $self->{fullname} || '';

  return $result;

}

##################################################################

=head2 $USER->belonds_to_grp( $group_object, $uid )

Returns the 1 if the user belongs to the $group_oject, under otherwise.

=cut

##################################################################

sub belongs_to_grp{
    my $self= shift;
    my $grp = shift;
    my $uid = shift;

    return undef if (!defined($grp) || !defined($uid));

    if (Group->is_member($uid, $grp)) {
        return 1;
    } else {
        return undef;
    }
}


##################################################################

=head2 $USER->belonds_to_grp_id( $group_id, $uid )

Returns the 1 if the user belongs to the $group_id, under otherwise.

=cut

##################################################################

sub belongs_to_grp_id {
    my $self= shift;
    my $grp_id = shift;
    my $uid = shift;

    return undef if (!defined($grp_id) || !defined($uid));

    if (Group->is_member_of_grp_id($uid, $grp_id)) {
    return 1;
    } else {
    return undef;
    }
}


##################################################################

=head2 $USER->largest_group()

Returns the largest group (the one with most members) this user belongs to.

=cut

##################################################################

sub largest_group {
  my $self = shift;
  my ($max, $largest);
  my $uid = $USER->{uid};

  my $groups = Group->get_group_names_where_member($uid);

  foreach my $grp (@$groups) {
      my $num = Group->number_of_members($grp);
      $max = $num, $largest = $grp if($num > $max);
  }
  return $largest;
}


##################################################################

=head2 $USER->is_owner( $instance )

Returns a 1 if the user owns the $instance, undef otherwise.

=cut

##################################################################

sub is_owner {
  my $self= shift;
  my $object=shift || die "No object passed";
  my $uid = $self->uid();
  my $objectuid = $object->get_owner_uid();

  if (!$uid){
     return undef;
  } elsif ($uid == $objectuid) {
     return 1;
  } else {
     return undef;
  }
}


##################################################################

=head2 $USER->is_logged_in()

Returns a positive value if the user is admin.

=cut

##################################################################

sub is_admin{
    my $self= shift;

    my $value = $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks', $self );

    return $value
}


##################################################################

=head2 $USER->is_logged_in()

Returns a positive value if the user is logged in, zero otherwise.


=cut

##################################################################

sub is_logged_in {
    my $self= shift;
    my $session = shift || $SESSION;

# XXX [claudio]
# should move this sql query to a method in Session.pm

    if( defined $self->{_logged_in} ) {

    return $self->{_logged_in}
    }

    my $uid = $self->uid();
    my $minimum_time = time - $PARAMS->{'sessionduration'};
    my $sessid = $session->session_id() || 0;
    my ($count) = defined $uid ?
      DBAccess->sqlSelect("count(*)", "sessions", "userid = $uid and (lastonline > $minimum_time) and (sessionid = '$sessid')") : 0;

    $self->{_logged_in} = $count;

    return $count;
}


##################################################################

=head2 $USER->save()

Save the user object in the database.


=cut

##################################################################

sub save{
  my $self=shift;
#    print STDERR "in User::save, with upload_size " . $self->{upload_size} . "\n";

  my $db_now = Utility->db_now();
  DBAccess->sqlUpdate("users","userid='$self->{uid}'",
        (
         fullname => $self->{fullname},
         nickname => $self->{nickname},
         homepage => $self->{homepage},
         email    => $self->{email},
         icqnumber=> $self->{icq_number},
         pager    => $self->{pager_number},
         adminlevel=>$self->{admin_level},
         quote    => $self->{quote},
             max_total_upload_size => $self->{upload_size},
             max_total_numberof_upload => $self->{upload_number},
         approvalneeded => $self->{approval_needed},
         updatedate     => $db_now,
         ldapdn         => $self->{ldap_dn},
             lastonline     => $self->{last_online},
             password_changed_time => $self->{password_changed_time},
             pw_expire_notification_time => $self->{pw_expire_notification_time},
        )
       );
  $self->update_global_user();
}

##################################################################

=head2 $USER->save_password( $password )

Save the password in the database. This function encrypts it if not already done.


=cut

##################################################################


sub save_password {
    my $self = shift;
    my $password= shift;

    if (defined $password && $password ne '') {
        my $uid = $self->uid();
        my $u = Metadot::UserFactory->get_user($uid);

        $password = Portal->encrypt_password_if_necessary($password);

        # update the user object with password timestamp
        my $password_timestamp = time;
        $u->password_changed_time($password_timestamp);

        my $db_now = Utility->db_now();
        DBAccess->sqlUpdate("users","userid=$uid",
              updatedate=>$db_now,
              password => $password,
              password_changed_time => $password_timestamp
              );
        $self->update_global_user();
    }
}

##################################################################

=head2 $USER->can_use_email_address( $email )

Returns 0 if another user already uses the user's email address, 1 otherwise.


=cut

##################################################################

sub can_use_email_address{
  my $self = shift;
  my $email = shift;

  my $uid = $self->uid();

  $email = DBAccess->quote( $email );

  my ($cnt)= DBAccess->sqlSelect("count(*)","users","email=$email and userid !=$uid");
  if ($cnt !=0){
    #email address already in use by another user
    return 0;
  }else{
    #email address not taken yet
    return 1;
  }

}


##################################################################

=head2 $USER->email_addresss_exists( $email )

Returns $uid if the exists in the user database, 0 otherwise.

=cut

##################################################################

sub email_address_exists {
  my $s = shift;
  my $email = shift;

  $email = DBAccess->quote( $email );

  my ($uid) = DBAccess->sqlSelect("userid", "users", "email=$email");

  if($uid){
    return $uid;
  }
  return 0;
}

##################################################################

=head2 $USER->add()

Adds a new user in the databases and creates 'my website'. Retuns the new user uid.

=cut

##################################################################

sub add{
    my $self = shift;

    $self->{upload_size} = -1;
    $self->{upload_number} =  -1;

    my $db_now = Utility->db_now();
    my $uid = DBAccess->sqlInsert("users",
            fullname => $self->{fullname},
            nickname => $self->{nickname},
            homepage => $self->{homepage},
            email    => $self->{email},
            icqnumber=> $self->{icq_number},
            pager    => $self->{pager_number},
            quote    => $self->{quote},
            password => $self->{password},
            approvalneeded => $self->{approval_needed},
            linkname1   => $self->{link_name_1}, #RESTRICTED-USER
            linkname2   => $self->{link_name_2}, #RESTRICTED-USER
            adminlevel  => 1,
            adminmode   => 0,
            debuglevel  => 0,
            updatedate  =>$db_now,
            creationdate=>$db_now,
            max_total_upload_size => $self->{upload_size},
            max_total_numberof_upload => $self->{upload_number},
            ldapdn  => $self->{ldap_dn},
                        password_changed_time => $self->{password_changed_time} || time,
                        pw_expire_notification_time => $self->{pw_expire_notification_time},       #NICO-PASS
               );

    $self->uid($uid);

    # create default user portal
    my $portal = Portal->new(undef);

    #RESTRICTED-USER: added additional condition against creation
    if ($PARAMS->{my_website_creation} && !$self->is_restricted() ) {
        $self->create_default_portal();
    }

    return $uid;
}


##################################################################

=head2 $USER->show_panel()

Shows the edit/delete button for admins

=cut

##################################################################

sub show_panel {
  my $self = shift;
  my $which_objects = shift;
  my $uid = $self->uid();
  my $approval_needed = $self->approval_needed() || "";
  my $str = "";
  $self->debug_msg(4, "Metadot::User::Default::show_panel(uid=$uid)");

  #if (!$SESSION->admin_mode()){ return "";}

  my $imageslocation = $PARAMS->{'imageslocation'};

  if ($SESSION->admin_mode()){
      $str.= "<a href='".$self->make_url("index.pl?isa=User&amp;op=modify&amp;uid=$uid")."'><img src='$imageslocation/obj/edit.gif' width=\"25\" height=\"11\" border=\"0\" align='top' class='icon' alt='Edit Me!'></a>\n";

      #unless( $USER->is_default_account( $self->email() ) ) {
      # save a query
      #print STDERR "checking if '".$self->fullname()."' is a default account name\n";
      unless( $USER->is_default_account_name( $self->fullname() ) ) {
          $str.= "<a href='".$self->make_url("index.pl?isa=User&amp;op=delete_request&amp;uid=$uid")."'><img src=$imageslocation/obj/del.gif width=\"11\" height=\"11\" border=\"0\" align='top' class='icon' alt='Delete Me!'></a>\n";
      }
  }

  ## check if we only need the accept/reject images
  if( $which_objects eq 'pending' ){
      $str = '';
  }
  if ($approval_needed eq "required"){
    $str.= "<a href='".$self->make_url("index.pl?isa=User&amp;op=accept_registration&amp;uid=$uid")."'><img src=$imageslocation/obj/accept.gif border=\"0\" align='top' class='icon' alt='Accept Me!'></a>\n";
    $str.= "<a href='".$self->make_url("index.pl?isa=User&amp;op=reject_registration&amp;uid=$uid")."'><img src=$imageslocation/obj/reject.gif border=\"0\" align='top' class='icon' alt='Reject Me!'></a>\n";
  }

  return $str;
}

##################################################################

=head2 $USER->www_accept_registration()

Admins accepts a user registration. Returns 1.

=cut

##################################################################

sub www_accept_registration {
    my $self = shift;

    if(!$FORM{uid}){
    $self->print_click_back("Error",
                "Sorry I didn't get a user uid to accept!");
    return 0;
    }

    my $u = Metadot::UserFactory->get_user($FORM{uid});
    my $fullname .= " " . $u->webname;
    my $email .= " " . $u->email;
    $u->approval_needed("accepted");
    $u->save;


    my $registrator = Metadot::RegisterFactory->get_register();
    $registrator->send_approval_message( $u, 'accepted' );

    $self->two_choices("<b><i>$fullname</i></b> has been accepted.",
               "confirm",
               "Pending Users List",
               "index.pl?isa=User&&op=show_users&pending=true",
               "Home",
               "index.pl"
              );

    return 1;
}



sub www_accept_registration_confirm {
    my $self = shift;
    if( !$FORM{uid}){
        $self->print_click_back("Error",
                                 "Sorry I didn't get a user uid to accept!");
        return 0;
    }

    my $u = Metadot::UserFactory->get_user($FORM{uid});
    my $fullname .= " " . $u->webname;
    my $email .= " " . $u->email;
    $u->approval_needed("accepted");
    $u->save;


    my $registrator = Metadot::RegisterFactory->get_register();
    $registrator->send_approval_message( $u, 'accepted' );

    $self->two_choices("<b><i>$fullname</i></b> has been accepted.",
               "confirm",
               "Pending Users List",
               "index.pl?isa=User&op=show_users&pending=true",
               "Home",
               "index.pl"
              );

    return 1;
}

##################################################################

=head2 $USER->www_reject_registration()

Admins rejects a user registration. Returns 1.

=cut

##################################################################
sub www_reject_registration {
    my $self = shift;

    if(!$FORM{uid}){
    $self->print_click_back("Error",
                "Sorry I didn't get a user uid to reject!");
    return 0;
    }

    my $u = Metadot::UserFactory->get_user($FORM{uid});
    my $fullname .= " " . $u->webname;
    my $email .= " " . $u->email;
    $u->approval_needed("rejected");
    $u->save;

    my $registrator = Metadot::RegisterFactory->get_register();
    $registrator->send_approval_message( $u, 'rejected' );

    $self->two_choices("<b><i>$fullname</i></b> has been rejected.",
               "confirm",
               "Pending Users List",
               "index.pl?isa=User&op=show_users&pending=true",
               "Home",
               "index.pl"
              );

    return 1;
}


sub www_reject_registration_confirm {
    my $self = shift;

    if(!$FORM{uid}){
    $self->print_click_back("Error",
                "Sorry I didn't get a user uid to reject!");
    return 0;
    }

    my $u = Metadot::UserFactory->get_user($FORM{uid});
    my $fullname .= " " . $u->webname;
    my $email .= " " . $u->email;
    $u->approval_needed("rejected");
    $u->save;

    my $registrator = Metadot::RegisterFactory->get_register();
    $registrator->send_approval_message( $u, 'rejected' );

    $self->two_choices("<b><i>$fullname</i></b> has been rejected.",
               "confirm",
               "Pending Users List",
               "index.pl?isa=User&op=show_users&pending=true",
               "Home",
               "index.pl"
              );

    return 1;
}


##################################################################

=head2 $USER->get_clipboard_iid()

Returns the iid of the clipboard category associated with this user. Will create instance record if not already there.


=cut

##################################################################

sub get_clipboard_iid {
    my $self = shift;

    $self->debug_msg(4, "Metadot::User::Default::get_clipboard()");

    my ($uid)=$self->uid;

    if (!defined($uid) || !$uid){
    return 0;
    }

    # XXX [claudio] note that this requires a sql query each time the clipboard
    # is used (better have a direct pointer to this instance in the user
    # object)

  my $parent_clipboard = MetadotConfig->get_reserved_iid('PARENT_CLIPBOARD');
    my $admin_uid = MetadotConfig->get_reserved_uid('ADMIN');

    my ($iid) = DBAccess->sqlSelect("iid", "instance", "parentiid=$parent_clipboard AND userid=$uid");

    if ($iid) {
    return $iid
    } else {
    # create a clipboard instance for this user
    $self->debug_msg(3, "Metadot::User::Default::get_clipboard()::creating for user $uid");

    my $cat = Category->new({
                 uid => $uid,
                 parent_id => $parent_clipboard,
                 name => "Clipboard for $USER->{nickname}",
                 description => "Clipboard for $USER->{nickname}",
                 keywords => "Clipboard $USER->{nickname}",
                 display_look => 'LULI',
                 created_by => $admin_uid,
                });
    my $iid = $cat->add();

    # make sure the new clipboard is not public accsessible

    $cat->set_bundle_level( 'DISP', 8 );
    $cat->set_bundle_level( 'MOD', 8 );
    $cat->save();

    return $iid;
    }
}


##################################################################

=head2 $USER->www_require_pager_form($referrer, $target)

Displays the pager input form.

=cut

##################################################################

sub www_require_pager_form {
  my $self = shift;
  my $referrer = shift; # url for add
  my $target   = shift; # url after add is done

  my $form = "";

  $form .= "<table><tr><td>"
          ."<form action='index.pl' method=post>\n"
          ."<input type='hidden' name='isa' value='User'>\n"
          ."<input type='hidden' name='op' value='process_pager_form'>\n"
          ."<input type='hidden' name='referrer' value='$referrer'>\n"
          ."<input type='hidden' name='target' value='$target'>\n"
          ."</td></tr>\n";

  $form .= "<tr><td>You do not have a Pager address set. Please provide one below.<br />\n"
          ."</td></tr>\n";

  $form .= "<tr><td><input type=text name=pageraddress value='".($FORM{pageraddress}||'')."'>"
          ."</td></tr>\n";

  $form .= "<tr><td><input type=submit value='Continue'>"
          ."</form>\n"
          ."</td></tr></table>\n";

  $DISPLAY->print_form($form, "Add Pager");

}



##################################################################

=head2 $USER->www_process_pager_form($referrer, $target)

Saves the pager input form.

=cut

##################################################################

sub www_process_pager_form {
  my $self = shift;

  my $referrer = $FORM{referrer};
  my $target   = $FORM{target};

  my $verify = new Form::Verify;

  $verify->verify('email', 'pager address', $FORM{pageraddress});
  if (my $myerror = $verify->errors){
    $self->print_click_back("Sorry", "$myerror.\n");
    exit;
  } else {
    $self->pager_number($FORM{pageraddress});
    $self->save;

    # $self->pager_accepted($referrer);
    $DISPLAY->debug_msg('accepted the pager number');

    $target = $CGI->escape($target);
    $self->md_redirect($referrer . "&target=$target");
    exit;
  }
}


##################################################################

=head2 $USER->pager_accepted($referrer)

Display a confirmation dialog.

=cut

##################################################################

sub pager_accepted {
  my $self = shift;
  my $referrer = shift;

  my$form = "Your pager address has been accepted.\n"
           ."<table align='right'><tr><td>"
           ."<a href='".$self->make_url("$referrer")."'>Continue</a>\n"
           ."</td></tr></table>\n";

  $DISPLAY->print_form($form, "Pager Accepted");

}

##################################################################

=head2 $USER->www_require_icq_form($referrer, $target)

Display an input form to enter the ICQ number.

=cut

##################################################################

sub www_require_icq_form {
  my $self = shift;
  my $referrer = shift; # url for add
  my $target   = shift; # url after add is done

  my $form = "";

  $form .= "<table><tr><td>"
          ."<form action='index.pl' method=post>\n"
          ."<input type='hidden' name='isa' value='User'>\n"
          ."<input type='hidden' name='op' value='process_icq_form'>\n"
          ."<input type='hidden' name='referrer' value='$referrer'>\n"
          ."<input type='hidden' name='target' value='$target'>\n"
          ."</td></tr>\n";

  $form .= "<tr><td>You do not have an ICQ number set. Please provide one below.<br />\n"
          ."</td></tr>\n";

  $form .= "<tr><td><input type=text name=icq_number value='$FORM{icq_numbner}'>"
          ."</td></tr>\n";

  $form .= "<tr><td><input type=submit value='Continue'>"
          ."</form>\n"
          ."</td></tr></table>\n";

  $DISPLAY->print_form($form, "Add ICQ");

}

##################################################################

=head2 $USER->www_process_icq_form()

Save the ICQ number if valid and redirects to referrer.

=cut

##################################################################

sub www_process_icq_form {
  my $self = shift;
  my $referrer = $FORM{referrer};
  my $target   = $FORM{target};

  my $verify = new Form::Verify;

  $verify->verify('icq', 'icq number', $FORM{icq_number});
  my $myerror =  $verify->errors;
  if ($myerror){
    $self->print_click_back("Sorry", "The $myerror is invalid.\n");
    exit;
  } else {
    $self->icq_number($FORM{icq_number});
    $self->save;
#    $self->icq_accepted($referrer);
    $target = $CGI->escape($target);
    $self->md_redirect($referrer . "&target=$target");
    exit;
  }
}



##################################################################

=head2 $USER->icq_accepted( $referrer )

Displays a the 'ICQ accepted' confirmation message.

=cut

##################################################################

sub icq_accepted {
  my $self = shift;
  my $referrer = shift;

  my$form = "Your icq number has been accepted.\n"
           ."<table align='right'><tr><td>"
           ."<a href='".$self->make_url("$referrer")."'>Continue</a>\n"
           ."</td></tr></table>\n";

  $DISPLAY->print_form($form, "ICQ accepted");

}


##################################################################

=head2 $USER->DESTROY( )

Does nothing except to make mod_perl happier.

=cut

##################################################################

sub DESTROY{
  # EMPTY destroy method to please mod_perl when
  # it complains about a missing DESTROY function.

}


##################################################################
=head2 $USER->create_mypage( $uid )

Create default My Page.

=cut

##################################################################

sub create_mypage {
    my $self = shift ;
    my $uid = shift ;

    $self->debug_msg(3, "Metadot::User::Default::create_mypage");
    $uid = undef unless (Utility->is_valid_integer($uid));   #secu_200401
    return if (!defined $uid);

    my $anon_user = MetadotConfig->get_reserved_uid('ANONYMOUS_USER');

    #### Copy channels from user uid=-1 who is the default
    my ($default) = DBAccess->sqlSelectMany("id, isa, col, mypageorder", "mypage", "userid=$anon_user");
    my $db_now = Utility->db_now();
    while (my ($id, $isa, $col, $mypageorder)=$default->fetchrow()) {
    DBAccess->sqlInsert("mypage", userid => $uid, id => $id, isa => $isa, col => $col,
                mypageorder => $mypageorder, updatedate  => '', creationdate=> $db_now);
    }
    $default->finish();

    #### Copy links from user uid=-1 who is the default
    my @link_array = $self->load_mypage_links($anon_user);

    my $i =1;
    foreach my $link_hash (@link_array) {

    $USER->save_mypage_link( $uid, $i, $link_hash->{name}, $link_hash->{value} );
    $i++;
    }

    $self->stock_gizmos($uid);
    return 1;
}


##################################################################

=head2 $USER->stock_gizmos( $uid )

My Page: fills the new user's gizmo tools with the same gizmotool items that the anonymous user has.

=cut

##################################################################


sub stock_gizmos {
  my $self = shift;
  my $uid = shift;

  $uid = DBAccess->quote( $uid );

  my $anon_user = MetadotConfig->get_reserved_uid('ANONYMOUS_USER');

  my ($anon_mpid, $sth, $sth2, $ciid, $valid_anon_mpid, $anon_isa, $isa, $mpid);
  ($sth) = DBAccess->sqlSelectMany ("mpid, isa" ,"mypage", "userid=$anon_user"); #this gives us all the mpids of the anonymous user
  while (($anon_mpid, $anon_isa) = $sth->fetchrow() ) {
    ($sth2) = DBAccess->sqlSelectMany("mpid, ciid", "gizmotoolitem", "mpid = $anon_mpid"); #we want only the mpids of the anonymous use that are gizmotools.
    while (($mpid, $ciid) = $sth2->fetchrow() ) {
      ($isa) = DBAccess->sqlSelect("isa", "mypage", "mpid = $mpid");  #this gets the isa (name) of the anonymous user's mpids.
      ($mpid) = DBAccess->sqlSelect("mpid", "mypage", "isa = '$isa' and userid=$uid");        #this gets the mpid of the new user that we should put the appropriate ciid with.
      DBAccess->sqlInsert("gizmotoolitem", mpid => $mpid, ciid => $ciid);
    }
  }
}

##################################################################

=head2 $USER->get_uid_from_email( $email )

Returns user's uid.

=cut

##################################################################

sub get_uid_from_email {
    my $self = shift;
    my $email = shift;

    $email = DBAccess->quote( lc($email));

    my ($uid) = DBAccess->sqlSelect("userid", "users", "LOWER(email)=$email");

    return $uid;
}


##################################################################

=head2 $USER->get_uid_from_ldap_uid( $uid )

Returns user's Metadot uid.

=cut

##################################################################

sub get_uid_from_ldap_uid {
    my $self = shift;
    my $ldap_uid = shift;

    #Make sure there is no whitespace in ldap_uid
    $ldap_uid = Utility->trim($ldap_uid);

    my ($uid) = DBAccess->sqlSelect("userid", "users", "TRIM(LOWER(ldapdn))=".lc(DBAccess->quote($ldap_uid)));  # secu_200401

    return $uid;
}

##################################################################

=head2 $USER->get_email_from_uid( $uid )

Returns user's email.

=cut

##################################################################

sub get_email_from_uid {
    my $self = shift;
    my $uid = shift;

    $uid = DBAccess->quote( $uid );

    my $email;
    eval {
    ($email) = DBAccess->sqlSelect("email", "users", "userid=$uid");
    };
    if ($@) {
    print STDERR $@ . "\n";
    return '';
    }

    return $email;
}

##################################################################

=head2 $USER->get_fullname_from($uid)

Returns user's fullname.

=cut

##################################################################

sub get_fullname_from_uid {
    my $self = shift;
    my $uid = shift;

    $uid = DBAccess->quote( $uid );

    my $fullname;
    eval {
    ($fullname) = DBAccess->sqlSelect("fullname", "users", "userid=$uid");
    };
    if ($@) {
    print STDERR $@ . "\n";
    return '';
    }

    return $fullname;
}


##################################################################

=head2 $USER->get_nickname_from_uid($uid)

Returns user's nickname.

=cut

##################################################################

sub get_nickname_from_uid {
    my $self = shift;
    my $uid = shift;

    $uid = DBAccess->quote( $uid );

    my $nick;
    eval {
    ($nick) = DBAccess->sqlSelect("nickname", "users", "userid=$uid");
    };
    if ($@) {
    print STDERR $@ . "\n";
    return '';
    }

    return $nick;
}

##################################################################

=head2 $USER->get_fullname_from_emai($email)

Returns ($fullname).

=cut

##################################################################

sub get_fullname_from_email {
    my $self = shift;
    my $email = shift;

    $email = DBAccess->quote( $email );
    my ($fullname) = DBAccess->sqlSelect("fullname", "users", "email=$email");

    return $fullname;
}

##################################################################

=head2 $USER->get_email_from_fullname( $fullname )

Returns ($email).

=cut

##################################################################

sub get_email_from_fullname {
    my $self = shift;
    my $fullname = shift;

    $fullname = DBAccess->quote( $fullname );

    my ($email) = DBAccess->sqlSelect("email", "users", "fullname=$fullname");

    return $email;
}

##################################################################

=head2 $USER->get_vitals_from_uid($uid)

Returns ($name, $nickname, $email).

=cut

##################################################################

sub get_vitals_from_uid {
    my $self = shift;
    my $uid = shift;

    $uid = DBAccess->quote( $uid );

    my ($name, $nickname, $email) = DBAccess->sqlSelect ("fullname, nickname, email", "users", "userid = $uid");
    return ($name, $nickname, $email);
}

##################################################################

=head2 $USER->substitute($message, $fullname, $email, $password, $nickname)

Replace $message tags such as <<fullname>> by the user fullname, <<email>> by the user's emaill address, etc...
Returns a string with the process information.

Avaiable tags are:
 forgot_pwd_url
 fullname
 website_url
 email
 password
 nickname
 profile_url
 subscription_url

=cut

##################################################################

sub substitute{
    my $self = shift;

    my ($message, $fullname, $email, $password, $nickname) = @_;

    $message  ||= '';
    $fullname ||= '';
    $email    ||= '';
    $password ||= '';
    $nickname ||= '';

    $message=~s/<<forgot_pwd_url>>/$PARAMS->{basename}\/index.pl?isa=Session&amp;op=forgot_password/gi;
    $message=~s/<<fullname>>/$fullname/gi;
    $message=~s/<<website_url>>/$PARAMS->{basename}\/index.pl/gi;
    $message=~s/<<email>>/$email/gi;
    $message=~s/<<password>>/$password/gi;
    $message=~s/<<nickname>>/$nickname/gi;
    $message=~s/<<profile_url>>/$PARAMS->{basename}\/index.pl?isa=User&amp;op=modify_self/gi;
    $message=~s/<<subscription_url>>/$PARAMS->{basename}\/userchannel.pl?op=select/gi;

    return $message;
}

##################################################################

=head2 $USER->get_all_user()

Return the an array containing all uids of all users in the database, including users waiting for registration approval or rejected registration.


=cut

##################################################################
# will return a reference to an array containing all users
sub get_all_users {
    my $self = shift;

    my $users = DBAccess->sqlSelectMany ("userid", "users");
    my @users;
    while(my ($uid) = $users->fetchrow) {
    push @users, Metadot::UserFactory->get_user($uid);
    }

    return \@users;
}

##################################################################

=head2 $USER->get_all_active_user()

Return the an array containing all uids of registered users excluding users waiting for registration approval or rejected registration.

=cut

##################################################################
sub get_all_active_users {
    my $self = shift;

    my $users = DBAccess->sqlSelectMany( "userid", "users", "(approvalneeded IS NULL OR NOT(approvalneeded = 'rejected' OR approvalneeded = 'required'))" );

    my @users;
    while(my ($uid) = $users->fetchrow) {
    push @users, Metadot::UserFactory->get_user($uid);
    }

    return \@users;
}


##################################################################

=head2 $USER->exists( $uid )

Return the $uid if the user exist, null otherwise.

=cut

##################################################################

sub user_exists {
    my $proto = shift;
    my $uid = shift;

    die "must provide uid" unless defined $uid;

    my $user = Metadot::UserFactory->get_user($uid);


}

##################################################################

=head2 $USER->get_uploads_space_and_num_files_quota( $uid )

Returns the storage quota (MB) and max number of files a user can upload.

=cut

##################################################################

sub get_uploads_space_and_num_files_quota {
    my $self = shift;
    my $uid = shift;

    $uid = (defined $uid) ? $uid : $self->uid();
    defined $uid || die "couldn't get a uid\n";

    $uid = DBAccess->quote( $uid );

    my ($upload_size_allowed, $upload_number_allowed) =
      DBAccess->sqlSelect("max_total_upload_size, max_total_numberof_upload", "users", "userid=$uid");

    if($upload_size_allowed == -1){
        $upload_size_allowed = $PARAMS->{max_total_upload_size};
    }

    if($upload_number_allowed == -1){
        $upload_number_allowed = $PARAMS->{max_total_number_upload};
    }

    return ($upload_size_allowed, $upload_number_allowed)
}

##################################################################

=head2 $USER->get_used_uploads_space_and_num_files($uid)

Returns ($size, $number) where $size is the storage in bytes and $number the number of files uploads owned by the user.

=cut

##################################################################
sub get_used_uploads_space_and_num_files {
    my $self = shift;
    my $uid = shift;

    $uid = (defined $uid) ? $uid : $self->uid();

    my ( $size, $number ) = Metadot::UploadsManager->space_used( $uid );

    $uid = DBAccess->quote( $uid );

    my ($c) = DBAccess->sqlSelectMany("file1_size", "instance", "userid=$uid and file1_size>=0");

    while ( my $s = $c->fetchrow() ){
    $size += $s;
    }

    $number += $c->rows;    #this is ok because all $c has beenfetched

    $c->finish();

    return ($size, $number);
}


##################################################################

=head2 $USER->is_default_account( $email )

Will check if email belongs to a default account, eg. Admin or default_look. Default accounts are not verified against the LDAP server when LDAP authentication is used.

=cut

##################################################################

sub is_default_account {
    my $class = shift;
    my $email = shift;
    my $fullname;

    if (defined $email) {
        $fullname = $class->get_fullname_from_email( $email );
    } elsif (ref($class)) {
        $fullname = $class->fullname();
    }
    return $class->is_default_account_name($fullname);
}


##################################################################

=head2 $USER->is_default_account_name( $name )

Will check if name belongs to a default account, eg. Admin or default_look. Default accounts are not verified against the LDAP server when LDAP authentication is used.

=cut

##################################################################

sub is_default_account_name {
    my $class = shift;
    my $name  = shift;

    # trim white space
    $name =~ s/^\s*//;
    $name =~ s/\s*$//;

    my $result = $default_account_names{lc $name} || 0;
    $DISPLAY->debug_msg(4, "is_default_account_name checks '$name', gets $result");
    return $result;
}


##################################################################

=head2     $user->www_show_users()

Will output a list of users (the result of invoking show_user_list) to the users browser.

=cut

##################################################################

sub www_show_users {
    my $self = shift;
    my $param = shift;
    my @external_filters = defined($param) ? @{$param} : ();
    $self->debug_msg(3, "Metadot::User::Default::www_show_users()");

    my @filters = ();
    my $add_tables;

    if ($FORM{name}) {

    my $value = uc(DBAccess->quote( "%$FORM{name}%" ));

    push @filters, "UPPER(fullname) like $value";
    }

    if ($FORM{nickname}) {

    my $value = uc(DBAccess->quote( "%$FORM{nickname}%" ));

        push @filters, "UPPER(nickname) like $value";
    }
    if ($FORM{email}) {

    my $value = uc(DBAccess->quote( "%$FORM{email}%" ));

       push @filters, "UPPER(email) like $value";
    }

    if ($FORM{pending}){
        push @filters, "approvalneeded = 'required'";
    }

    if ($FORM{who_online}) {
        push @filters, "users.userid=sessions.userid and ".$SESSION->get_last_online_clause('users');
        $add_tables = 'sessions';
    }
    my $filter = join(' and ', @filters);

    #$DISPLAY->print_http_header;
    my $all_url = "";

    # add a 'show all users' link before generating the table if a search filter has been defined.
    my $iid;
    if ($filter ne "") {
        $iid = $FORM{iid} || 0 ;
        $all_url = "index.pl?iid=$iid&amp;isa=User&amp;op=show_users&amp;threads=$FORM{threads}&amp;order=$FORM{order}";
        $all_url = "\n<a href=\"" . $self->make_url($all_url) . "\">Show All Users</a><br />\n";
    }

    #print $self->show_top($FORM{iid}),
    #$self->show_title("User Directory"),
    #$all_url,

    #$self->show_user_list( {filter=>$filter, startfrom=>$FORM{from}, maxitem=>$FORM{nb}, add_tables =>$add_tables}),
    #"\n\n<!-- End of Users -->\n\n",
    #$self->show_bottom;

    my $content = $all_url.$self->show_user_list( {filter=>$filter, startfrom=>$FORM{from}, maxitem=>$FORM{nb}, add_tables =>$add_tables});

    $DISPLAY->print_simple_layout({content=>$content, title=>"User Directory", iid=>$iid});
}

##################################################################

=head2 show_user_list ($filter, $startfrom, $maxitem)

Shows a list of users where condition clause as argument.

=cut

##################################################################

sub show_user_list {
    my $self = shift ;
    my ($filter, $startfrom, $maxitem) = @_;
    my $s1 = "";
    my $where;
    my $admin;
    my $c;
    my $oldi=0;
    my ($uid, $fullname, $nickname, $email, $uploadsize, $uploadnumber);
    my $portal="";
    my $hpiid=0;
    $self->debug_msg(3, "Metadot::User::Default::show_user_list(filter=$filter)");

    my $table_clause = "users";

    # Filter is now a vector into an args hash. This maintains backward compatibility.
    my $args;
    if (ref($filter) eq 'HASH') {
    $args   = $filter;
    $filter     = $args->{filter};
    $startfrom  = $args->{startfrom};
    $maxitem    = $args->{maxitem};
    my $add_tables = $args->{add_tables}; # this must be an array ref

    # add tables to the table clause if needed
    if ($add_tables) {
        $add_tables = [$add_tables] unless (ref($add_tables) eq 'ARRAY');
        $table_clause = join(',', ($table_clause, @{$add_tables}));
    }

    }

    if ($ACCESS_BROKER->is_allowed_to_do('manage_registrations')) {
        $where = $filter || "1=1";
        $DISPLAY->debug_msg(5, "user is allowed to see unapproved users");
    } else {
        $where = "users.userid>0";
        $where .= " AND (approvalneeded IS NULL OR approvalneeded NOT IN ('required', 'rejected'))";
        $where .= " AND $filter" if ($filter);
        $DISPLAY->debug_msg(5, "user is NOT allowed to see unapproved users");
    }

    ### select user's portal iid

    my $message;
    if($FORM{uid}){
        my $u = $USER->new($FORM{uid});
        my $fullname .= " " . $u->webname;
        $message ="$fullname has been accepted and notified via email.";
    }
    #### table
    if($message){
        $s1 .= "<font color='#18bb94'>$message</font><br />";
    }
    my $order;
    my $order_name_value_pair;

    if ($FORM{'order'}) {
    if ($FORM{order} eq "desc") {
        $order = " DESC";
        $order_name_value_pair = "&amp;order=asc";
    }
    } else {
    $order = "ASC";
    $order_name_value_pair = "&amp;order=desc";
    }

    my ($name_order_arrow, $nickname_order_arrow, $email_order_arrow) = (undef, undef, undef);

    my $imageslocation = $PARAMS->{'imageslocation'};


    $FORM{'threads'} = 0 unless $FORM{'threads'};

    if ($FORM{'threads'} == 1) {
    if ($FORM{'order'} eq 'desc') {
        $nickname_order_arrow = "<img src=\"$imageslocation/obj/triangle_down.gif\" border=\"0\" class='icon' alt=\"Sorted Descending\">";
    } else {
        $nickname_order_arrow = "<img src=\"$imageslocation/obj/triangle_up.gif\" border=\"0\" class='icon' alt=\"Sorted Ascending\">";
    }
    } elsif ($FORM{'threads'} == 2) {
    if ($FORM{'order'} eq 'desc') {
        $email_order_arrow = "<img src=\"$imageslocation/obj/triangle_down.gif\" border=\"0\" class='icon' alt=\"Sorted Descending\">";
    } else {
        $email_order_arrow = "<img src=\"$imageslocation/obj/triangle_up.gif\" border=\"0\" class='icon' alt=\"Sorted Ascending\">";
    }
    } else {
    if (($FORM{'order'} || 0) eq 'desc') {
        $name_order_arrow = "<img src=\"$imageslocation/obj/triangle_down.gif\" border=\"0\" class='icon' alt=\"Sorted Descending\">";
    } else {
        $name_order_arrow = "<img src=\"$imageslocation/obj/triangle_up.gif\" border=\"0\" class='icon' alt=\"Sorted Ascending\">";
    }
    }

    my $other;
    if ($FORM{threads} == 1){
        $other ="ORDER BY users.nickname $order";
    } elsif ($FORM{threads} == 2){
        $other = "ORDER BY users.email $order";
    } else {
        $other = "ORDER BY users.fullname $order";
    }

    $maxitem = 100 if (!$maxitem);   #PREVIOUS NEXT MAGIC
    $startfrom = 0  if(!$startfrom); #PREVIOUS NEXT MAGIC
    my $from = $startfrom;           #PREVIOUS NEXT MAGIC

    my $i=0;

    #aju - add 'distinct' to query to fix online count inconsistency
    ($c)=DBAccess->sqlSelectMany("DISTINCT users.userid, users.fullname, users.nickname, users.email, users.max_total_upload_size, users.max_total_numberof_upload", $table_clause, $where, $other);
    # my ($totalrows)=$c->rows;
    # rows function is not reliable for counting the rows retrieved by SELECT
    my ($totalrows) = DBAccess->sqlSelect("count(*)",$table_clause, $where, $other);

    my $max_total_upload_size = $PARAMS->{'max_total_upload_size'};
    my $max_total_number_upload = $PARAMS->{'max_total_number_upload'};

    if($totalrows <= $maxitem){
        $s1 .="<br /><font size=-1><I><b>Found $totalrows entries</b></I></font>";
    }

    $email_order_arrow = $email_order_arrow ? $email_order_arrow : "";
    $nickname_order_arrow = $nickname_order_arrow ? $nickname_order_arrow : "";
    $name_order_arrow = $name_order_arrow ? $name_order_arrow : "";

    if ($USER->is_admin) {
            $s1 .= "<center><h2><a href=\"index.pl?isa=Register&amp;op=register_form&amp;admin_mode=true\">Register a new user</a></h2></center>\n";
    }

    $s1 .= <<TEXT;
<center><table border="0" cellpadding = 0 cellspacing = 2>
                <tr>
                        <td colspan = 3>
                                <center><H2>Search for a user</H2></center>
                        </td>
                </tr>
                <tr>
                        <td>
                                Name
                        </td>
                        <td>
                                Nickname / Screenname
                        </td>
                        <td>
                                Email address
                        </td>
                </tr>
                <tr>
                <tr>
                        <td>
                                <form method = post action = index.pl>
                                <input type=hidden name=isa value=User>
                                <input type=hidden name=op value=show_users>
                                <input type=hidden name=iid value = 0>
                                <input type=text name = name>
                        </td>
                        <td>
                                <input type=text name=nickname>
                        </td>
                        <td>
                                <input type=text name=email>
                        </td>
                </tr>
                <tr>
                        <td colspan = 3>
                                <center><input type=submit value = 'Start Search'></center></form>
                        </td>
                </tr>
         </table></center><br /><center>
         <table border="1" cellpadding="4" cellspacing="0" width="100%">
            <tr>
              <td valign='middle'><font size=-1><b><a class='prevnextbar' href=
TEXT
;

    my $base_url = $self->make_url("index.pl?iid=0&amp;isa=User&amp;op=show_users&amp;pending=$FORM{pending}&amp;who_online=$FORM{who_online}");
    $s1 .= $base_url."&amp;threads=0$order_name_value_pair" .
    ">$name_order_arrow Name</a> </b> <I><a class='prevnextbar' href=".
      $base_url."&amp;threads=1$order_name_value_pair" .
     ">(nickname) $nickname_order_arrow</a><I></font></td>
                           <td valign='middle'><font size=-1><b>ID</b></font></td>
              <td><b>Portal</b></td>
                 <td valign='middle'><font size=-1><b><a class='prevnextbar' href=" .
           $base_url."&amp;threads=2$order_name_value_pair" .
      ">$email_order_arrow Email address</a></b></font></td>     <td valign='middle'><font size=-1>Max. total upload size (MB)<br /><font size=-5>(Default: $max_total_upload_size MB)</font></td>                     <td valign='middle'><font size=-1>Max. total number of upload<br /><font size=-5>(Default: $max_total_number_upload)</font></td>    </tr>\n";


    while((($uid,
        $fullname,
        $nickname,
        $email,
        $uploadsize,
        $uploadnumber) = $c->fetchrow()) && ($i < $maxitem)) {

    if(($uploadsize || 0) == -1) {

        $uploadsize = "Default";
    } elsif (!$uploadsize) {

        $uploadsize = "";
    }

    if(($uploadnumber || 0) == -1){

        $uploadnumber = "Default";
    } elsif (!$uploadnumber) {

        $uploadnumber = "";
    }

    if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC

    $i++;

    # skip hidden users only when user doesn't have server_setup access
    unless( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){

        if ( $USER->is_hidden( $uid ) ) { next }
    }

    ### Prepare the user portal link
    my $u = Metadot::UserFactory->get_user($uid);

    if( $USER->belongs_to_grp('SITE_MGR', $uid) || $USER->belongs_to_grp('ADMIN', $uid) ) {

        $fullname="$fullname<super>*</super>";
    }
    ($hpiid) = $u->has_portal();
    $portal = "&nbsp;";
    $portal = "<a href=".$self->make_url("index.pl?iid=$hpiid").">Portal</a>\n" if ($hpiid);

    if ($nickname) {
        $nickname = "<I>($nickname)</I>\n";
    } else {
        $nickname = "";
    }

    my $rejected_string = "";
    if (($u->approval_needed() || "") eq "rejected"){
        $rejected_string = "<font color=#FF0000>(rejected)</font>";
    }

    $email = "<a href=\"mailto:$email\">$email</a>\n" if ($email);

    $s1 .= "<font size=-3><tr><td>\n";

    if($USER->is_admin){

        $s1 .= $u->show_panel();
    }

    $rejected_string = $rejected_string ? $rejected_string : "";

    $s1 .= "<a href='".$self->make_url("index.pl?isa=User&amp;op=show_details&amp;uid=$uid")."'>$fullname</a> $nickname $rejected_string</td>\n";

    $s1 .= "<td>$uid</td>\n<td>$portal</td><td>$email</td><td>$uploadsize</td><td>$uploadnumber</td></tr></font>\n";
    }

    $c->finish();

    my $iid = $FORM{iid} || 0 ;
    my $url = "index.pl?iid=$iid&amp;isa=User&&amp;op=show_users&amp;threads=$FORM{threads}&amp;order=$FORM{order}";

    my $switch = "";
    my ($pending_approval) = DBAccess->sqlSelect("count(*)", "users", "approvalneeded='required'");
    if (($USER->is_admin) && ($pending_approval || (($FORM{pending} || "") eq 'true'))){
    my $link = ($FORM{pending} eq 'true') ?
          "<a class='prevnextbar' href='".$self->make_url("$url")."'>All Users</a>":
        "<a class='prevnextbar' href='".$self->make_url("$url&amp;pending=true")."'>Pending users only</a>";
    $switch = "<table width=\"100%\"><tr><td align='right'>[ $link ] </td></tr></table>";
    }

    # display online users
    my $who_online = $SESSION->get_members_online();
    if (($USER->is_admin) && ($who_online || (($FORM{who_online} || "") eq 'true'))){
    my $link = ($FORM{who_online} eq 'true') ?
            "<a class='prevnextbar' href='".$self->make_url("$url")."'>All Users</a>":
          "<a class='prevnextbar' href='".$self->make_url("$url&amp;who_online=true")."'>Show $who_online online users only</a>";
    $switch .= "<table width=\"100%\"><tr><td align='right'>[ $link ] </td></tr></table>";
    }


  #PREVIOUS NEXT MAGIC
    my $pn = "" ;
    $pn = Components->get_previous_next_button($maxitem, $i, $totalrows, $startfrom, $url) if ($totalrows > $maxitem);

    $s1.="</td></td></td></tr></table></center><br />";
    my $page = "$switch\n$pn\n$s1\n$pn\n<br /><I>Note: Names marked with (<super>*</super>) are administrators</I>\n\n";
    return $page;
}


##################################################################

=head2 show_user_object_list()

Does nothing.

=cut

##################################################################

sub show_user_object_list {

    my $self = shift;
}

##################################################################

=head2 get_emails_by_grp_id( $grp_id )

       Will return an array containing email addresses of members of a group.

=cut

##################################################################

# XXX [claudio] note that this depends on groups table structure
sub get_emails_by_grp_id {
    my $self = shift;
    my $grp_id = shift;
    my $recurse = shift;

    $grp_id = undef unless (Utility->is_valid_integer($grp_id));   #secu_200401
    defined($grp_id) || die("Must provide a group ID to query against.\n");

    my $users = DBAccess->sqlSelectMany ('u.email','users u, grpmembers gm',"gm.grp_id='$grp_id' AND u.userid=gm.userid");
    my @user_list;

    while (my ($email) = $users->fetchrow) {
        push @user_list, $email;
    }

    #aju
    if ($recurse) {
    # see if the group has any subgroups
    my $subgroups =
        DBAccess->sqlSelectMany ('subgrp_id', 'grpmembers', "grp_id = '$grp_id' and subgrp_id > 0");
    while (my ($sg) = $subgroups->fetchrow) {
        my $sg_user_list = $self->get_emails_by_grp_id($sg, $recurse);
        push @user_list, @$sg_user_list;
    }
    }
    return \@user_list;
}

##################################################################

=head2 get_emails_by_grp_name( $grp_name )

       Will return an array containing email addresses of members of a group.

=cut

##################################################################

# XXX [claudio] note that this depends on groups table structure
sub get_emails_by_grp_name {
    my $class = shift;
    my $grp_name = shift || die("Must provide a group name to query against.\n");
    my $recurse = shift;

    my $users = DBAccess->sqlSelectMany('u.email','users u, grpmembers gm, grp g',"g.group_name=".DBAccess->quote($grp_name)." AND gm.grp_id=g.grp_id AND u.userid=gm.userid");  #secu_200401

    my @user_list;
    while (my ($email) = $users->fetchrow) {
        push @user_list, $email;
    }

    #aju
    if ($recurse) {
    # get the group id from its name
    my ($grp_id) = DBAccess->sqlSelect("grp_id", "grp", "group_name = '$grp_name'");
    if ($grp_id) {
        # see if the group has any subgroups
        my $subgroups =
        DBAccess->sqlSelectMany ('subgrp_id', 'grpmembers', "grp_id = '$grp_id' and subgrp_id > 0");
        while (my ($sg) = $subgroups->fetchrow) {
        my $sg_user_list = $class->get_emails_by_grp_id($sg, $recurse);
        push @user_list, @$sg_user_list;
        }
    }
    }
    return \@user_list;
}
####################################################

=head2 get_users_in_group( $group_id, $field, $order )

Will return an array of hashes containing descriptions of all users in a group. Results will be sorted by $field in ascending or descending order according to $order.

=cut

####################################################

# XXX [claudio] note that this depends on groups table structure
sub get_users_in_group {
    my $self = shift;
    my $grp_id = shift;
    my $field = shift;
    my $order = shift;

    $grp_id = undef unless (Utility->is_valid_integer($grp_id));   #secu_200401
    defined($grp_id) || die("Must provide a group ID to query against.\n");

    ## must use field aliasing otherwise sort will break
    my $rows = DBAccess->sqlSelectMany('users.email as email, users.fullname fullname, users.nickname nickname, users.userid userid, grpmembers.is_manager is_manager',
                       'users,grpmembers',
                       "grpmembers.grp_id='$grp_id' and grpmembers.userid=users.userid",
                       "ORDER BY $field $order");


    my $result;
    while (my $row = $rows->fetchrow_hashref() ) {

    push @$result, $row;

    }

    return $result;

}


####################################################

=head2 get_groups_data_by_owner()

Will return an array of hashes, one for each existing group in the system, containg group description information in the following fields: email, owner, nickname and description.  Results will be sorted by $field in ascending or descending order according to $order.

=cut

####################################################
# XXX [claudio] note that this depends on groups table structure
# returns a reference to an array of hashes containing the data
sub get_groups_data_by_owner {
    my $self = shift;
    my $field = shift;
    my $order = shift;

    my $sth = DBAccess->sqlSelectMany("grp.group_name group_name,
                                       grp.grp_id grp_id,
                                       users.email email,
                                       users.fullname owner,
                                       users.nickname nick,
                                       grp.group_description description",
                      'grp, users',
                      'grp.userid = users.userid',
                      "order by $field $order");

    my $result;
    while ( my $row = $sth->fetchrow_hashref ) {

    push @$result, $row;
    }

    return $result;
}

####################################################

=head2 logout()

Loggout a user and do the necessary house keeping (edit off, expires session, etc...).

=cut

####################################################

sub logout {
    my $self = shift;
    my $session = shift || $SESSION;

    $self->{_logged_in} = 0;

    if( $PARAMS->{sessions_are_deleted_on_logout}
    and not $self->{uid} eq MetadotConfig->get_reserved_uid('ANONYMOUS_USER') ) {

    $session->erase();
    $USER = Metadot::UserFactory->get_user( MetadotConfig->get_reserved_uid('ANONYMOUS_USER') );


    } else {

    $session->set_value('GB_status','0');
    $session->edit_off();

    # just expire the session
    $session->_expire();
    }
}


####################################################

=head2 load_mypage_links()

Return an array containing (name, value) pairs.

=cut

####################################################

sub load_mypage_links {
    my $self = shift ;
    my $uid = shift ;

    $self->debug_msg(4, "Metadot::User::Default::load_mypage_links");

    my $maxlinks = $PARAMS->{maxlinks};
    my $fields = join( ", ", map( "linkname$_, linkurl$_", ( 1..$maxlinks )));

    $uid = DBAccess->quote( $uid );

    my @link_values = DBAccess->sqlSelect($fields, "users", "userid=$uid");
    my @link_array;

    for (my $i=1; $i<= $maxlinks; $i++) {
    my %links;
    $links{name} = shift @link_values;
    $links{value} = shift @link_values;
    push @link_array, \%links;
    }
    return @link_array;
}


####################################################

=head2 save_mypage_link($uid, $link_num, $link_name, $link_value)

Save a My Page quick link for a given user.

=cut

####################################################

sub save_mypage_link {
    my $self = shift;
    my $uid = shift;
    my $link_num = shift;
    my $link_name = shift;
    my $link_value = shift;

    $uid = DBAccess->quote( $uid );

    DBAccess->sqlUpdate("users", "userid=$uid", ("linkname$link_num" => $link_name, "linkurl$link_num" => $link_value ));
}


# updates the global $USER variable if AND ONLY IF the uid is the same
sub update_global_user {
    my $self = shift;
    if (defined $USER && $USER->uid == $self->uid()) {
        $USER = $self;
    }
}

####################################################

=head2 get_num_accounts_pending_approval()

Returns the number of users registrations waiting to be approved.

=cut

####################################################

sub get_num_accounts_pending_approval {
    my $self = shift;

    my ($num) = DBAccess->sqlSelect("count(*)", "users", "approvalneeded='required'");

    return $num;
}


####################################################

=head2 get_search_user_results(%config_hash, [$reference_to_hash_with_uids_to_exclude])

Returns a string containing the user search results.

=cut

####################################################
sub get_search_user_results {
    my $self = shift;

    my $config = shift || die "should provide config hash";
    my $excluded_uids = shift;  #optional reference to a hash containing uids not to return in search results
    my $grp_id = $config->{SKIP_GRP};
    my @user_info;
    my $response;
    my $radio_buttons;
    my $positive_result;
    my $uid;
    my $where = "(approvalneeded IS NULL OR NOT(approvalneeded = 'rejected' OR approvalneeded = 'required'))";
    my $member_present;
    my $image_info;
    my $email =  $config->{EMAIL};
    my $fullname = $config->{FULLNAME};
    my $nickname = $config->{NICKNAME};
    my $select_one = $config->{SELECT_ONE};
    my $s='';
    my $trusted_site = $PARAMS->{trusted_site};

    unless( $trusted_site || $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks' ) ) {

        unless (($fullname =~ m/[^\*][^\*]/) || ($nickname =~ m/[^\*][^\*]/) || ($email =~ m/[^\*][^\*]/)) {
            return "<table><tr><td><font color='red'>You must include at least 2 non wildcard characters in your search</font></td></tr></table>";
        }
    }


    #use Data::Dumper;

    #CKIM-SENDABLE: action missing closing bracket and quotes
    #$s .= "<form method=post action =". $config->{ACTION}  if($config->{ACTION});
    $s .= "<form method=post action =\"$config->{ACTION}\">"  if($config->{ACTION});
    $s .= "<table cellspacing=\"0\">
    <tr bgcolor=\"#bbbbbb\">
        <th>
            &nbsp;
        </th>";
    if( $trusted_site || $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks' ) ) {
    $s .= "<th>
            E-mail address
        </th>"
     }
     $s .= "
        <th>
            Full Name
        </th>
        <th>
            Nickname / Screenname
        </th>
    </tr>";

    if ($email)  {
    # if we got an email address, we'll use that as part of the query

    $email = uc(DBAccess->quote( $email ));

    $where .= " AND UPPER(email) LIKE $email";

    $response = 1;
    }
    if ($fullname) {

    $fullname = uc(DBAccess->quote( $fullname ));

    $where .= " AND UPPER(fullname) LIKE $fullname";
    $response = 1;
    }
    if ($nickname) {

    $nickname = uc(DBAccess->quote( $nickname ));

    $where .= " AND UPPER(nickname) LIKE $nickname";
    $response = 1;
    }

    if ($response) {
    $where =~ s/\*/%/g;

    my $rows = DBAccess->sqlSelectMany("userid, nickname, fullname, email", "users", $where, ""); #we should now have back all the possible matches
    my $count = 0;

    # support for multiple return types
    my $return_field;

    if ($config->{RETURN_VAL}) {
        #CKIM-SENDABLE: adding support for multiple return types
        $return_field = $config->{RETURN_VAL} || 'uid';
    }

    # need to sort the results
    my @results;
    while (($uid, $nickname, $fullname, $email) = $rows->fetchrow()) {

        # skip hidden users only when user doesn't have server_setup access
        unless( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){

        if ( $USER->is_hidden( $uid ) ) { next }
        }

        push @results, {
                uid=>$uid,
                nickname=>$nickname,
                fullname=>$fullname,
                email=>$email,
               };
    }

    my $sort_field = $config->{RETURN_VAL} || 'email';
    @results = sort {lc($a->{$sort_field}) cmp lc($b->{$sort_field})} @results;

    for my $result (@results) {
        ($uid, $nickname, $fullname, $email) = ($result->{uid}, $result->{nickname}, $result->{fullname}, $result->{email});
        $member_present = 1 if ($$excluded_uids{$uid}); #check to see if the uid being search for is in the list of uids that should not be return

        if(not $member_present && $uid != MetadotConfig->get_reserved_uid('ANONYMOUS_USER')) {
        $positive_result = 1;
        my $rownum = ($count++ % 2);
        my $color = $rownum ? "#eeeeee" : "#ffffff";

        $s .= "<tr bgcolor=\"$color\">
                <td><center>";
        if ($return_field) {
            my $return_val = $result->{$return_field};


            if ($select_one) {
            $s .="<input type=radio name=\"new_$return_field\" value=\"$return_val\">";
            } else {
            $s .="<input type=checkbox name=\"new_$return_field\" value=\"$return_val\">";
            }
        } else {
            #CKIM-SENDABLE: backwards compatibility
            if ($select_one) {
            $s .="<input type=radio name=uid value='$uid'>";
            } else {
            $s .="<input type=checkbox name=user$uid value=1>";
            }
        }

        $s .= "         </center>
                    </td>";
                    if ( $trusted_site || $ACCESS_BROKER->is_allowed_to_do( 'admin_tasks' ) ) {
                    $s .= "<td>
                        $email&nbsp;
                    </td>";
                    }
         $s .= "
                    <td>
                        $fullname&nbsp;
                    </td>
                    <td>
                        $nickname&nbsp;
                    </td>
                </tr>";
        }
        $member_present = 0;
    }
    #CKIM-SENDABLE: this HTML was missing some tags
    $s .= '</table>';
    } else {            # we are here if none of the fields was filled in
    $s = "<table><tr><td><font color='red'>None of the information was filled in.  Please go back and enter some information</font></td></tr></table>";
    }
    if ($response && !$positive_result) { #if we got a response from the user, but that response didn't turn up anything in the groups table
    $s = "<table><tr><td><font color='red'>No users match the criteria who are not already part of the group</font></td></tr></table> ";

    }
    if ($positive_result) { #if we got some information to present to the user, we need to generate a submit button
    if($config->{ACTION}) {
        foreach my $param (keys %{$config->{PARAMS}}){
        #CKIM-SENDABLE: value=''? why?
        #$s .= "<input type=hidden name='$param' value=''>\n";
        $s .= "<input type=hidden name=\"$param\" value=\"$config->{PARAMS}{$param}\">\n";
        }
        #CKIM-SENDABLE: cleaned up the html (moved </table> up)
        $s .=  "<input type=submit name='add' value='Add'></form>";
    }
#   print STDERR "CKIM: Components->show_user_search_results got:\n$s\n";
    return $s;
    } else {
    return ( "<center><font color=\"#ff0000\">$s</font></center>" );
    }
}


####################################################

=head2 check_password ($email, $password)

Returns a user_id if the authentication is correct.

=cut

####################################################
sub check_password {
    my $self = shift;

    my $email = shift;
    my $password = shift;

    $email = DBAccess->quote( $email );
    $password = DBAccess->quote($password);

    my ( $resuid ) = DBAccess->sqlSelect("userid", "users", "email=$email and password = $password");

    return $resuid;
}


####################################################

=head2 email_already_registered($email_address)

Returns a positive number if user is already registered in Metadot user database.
This is an alias to Register's email_already_registered method
=cut

####################################################
sub email_already_registered {
    my $self = shift;
    my $email = shift;

    my $register_class  = MetadotConfig->get_registration_type();
    my $register = $register_class->new();

    return $register->email_already_registered($email);
}


####################################################

=head2 ldap_uid_already_registered($ldap_uid)

Returns a positive number if user is already registered in Metadot user database.

=cut

####################################################
sub ldap_uid_already_registered {
    my $self = shift;
    my $ldap_uid = shift;

    #Make sure there is no whitespace in ldap_uid
    $ldap_uid = Utility->trim($ldap_uid);

    my ($cnt)=DBAccess->sqlSelect(
      "count(*)", "users", "TRIM(LOWER(ldapdn))=".DBAccess->quote(lc($ldap_uid))
    );
    return $cnt;
}


####################################################

=head2 is_approval_needed()

Returns a true value if the user requires approval

=cut

####################################################
sub is_approval_needed
{
    my $self = shift;
    my $user = shift || (ref $self ? $self : $USER);

    my $approval_status = $user->approval_status();
    return ( $approval_status && ($approval_status ne "accepted"));
}


####################################################

=head2 approval_status()

A less confusing alias for approval_needed

=cut

####################################################
sub approval_status
{
    my $self = shift;
    my $user = shift || (ref $self ? $self : $USER);

    return $user->approval_needed();
}


####################################################

=head2 create_default_portal($uid)

Create a default portal (mywebsite) for a user with some default categories, items and news item and return the $iid of the page.

=cut

####################################################
sub create_default_portal {
    my $self = shift;
    my $user = shift;

    my $result;

    $self->debug_msg(3, "Metadot::User::Default::create_default_portal()");

    # if no user defined, this was called against the instance
    if (!defined $user) {
        if (ref $self) {
            $user = $self;
        } else {
            $user = $USER;
        }
    } else {
        # check what type I got. I'm now expecting a user object, but other
        # classes may be sending a uid instead
        unless (ref $user) {
            # i think it's a uid
            $user = Metadot::UserFactory->get_user( $user );
        }
    }


    # this is where we attach all users portals
    my $portal_parent_id = MetadotConfig->get_reserved_iid('PORTAL_PARENT');

    # check that the user exists and that the portal does not already exist
    if (defined $user->uid() && !$user->has_portal()) {

        my $website;
        my $default_look_name = $self->NAME_DEFAULT_LOOK();

        # get the portal of default_look
        my $default_user_id = $self->get_uid_from_email($default_look_name);
        if (defined $default_user_id) {
            my $default_website_id = $self->has_portal($default_user_id);
            if ($default_website_id) {
                my $default_website = Metadot::PortalFactory->get_gizmo_object($default_website_id);
                $website = $default_website->copy({
                    deep => 1,
                    properties => {
                        parent_id => $portal_parent_id,
                        uid => $self->uid(),
                    }
                });
            }
        }

        # last resort, if no result was found, create a blank page.
        unless (defined $website) {
            use Gizmo::Category;
            $website = Category->new({
                uid => $self->uid(),
                parent_id => $portal_parent_id,
                });
            $website->add();
        }

        # replace the name and description
        my $webname = $user->webname();

        my $name = $website->name();
        $name =~ s/\Q$default_look_name\E/$webname/g;
        $website->name($name);

        my $description = $website->description();
        $description =~ s/\Q$default_look_name\E/$webname/g;
        $website->description($description);

        $website->save();

        $result = $website->id();
    }

    return $result;
}



####################################################

=head2  www_show_users_portal()

Redirects to the user portal (my website) or show a list of possible links if the user have many.

=cut

####################################################
sub www_show_users_portal {
    my $self = shift;
    my $user = $FORM{user};
    my $basename = $PARAMS->{basename};
    my @valid_iids;
    my $sth = DBAccess->sqlSelectMany("iid, users.fullname", "instance, users", "(users.fullname = '$user' or users.nickname = '$user') AND users.userid = instance.userid AND instance.parentiid = -1000 AND instance.iid > 1000 AND users.userid > 0 ORDER BY users.fullname");
    while (my ($iid, $fullname) = $sth->fetchrow_array()) {
        my $category = Gizmo->new($iid);
        if ($category->is_allowed_to_do('show', $USER)) {
            push (@valid_iids, $fullname);
            push (@valid_iids, $iid);
        }
    }

    if (scalar(@valid_iids) == 2) {
        $self->print_http_header();
        $self->md_redirect("$basename/index.pl?op=show&iid=$valid_iids[1]");
    } elsif (scalar(@valid_iids) > 1 ) {
        my @array = ("More than one match was found", "Caution");
        while (my $fullname = shift(@valid_iids)) {
            push (@array, "The portal of $fullname");
            push (@array, ("$basename/index.pl?op=show&isa=Category&iid=" . shift(@valid_iids)));
        }
        $self->print_choices(@array);

    } else {
        $self->print_choices("No match was found", "Caution", "Go to the home page", "$basename/index.pl");
    }
}


####################################################

=head2 is_hidden($uid)

Returns 1 if the user is in the ADMIN group or if it is 'default_look'

=cut

####################################################
sub is_hidden {
    my $self = shift;
    my $uid = shift;

    unless(defined $uid) { $uid = $self->uid() }

    if( $USER->belongs_to_grp('ADMIN', $uid) ||
    $USER->get_fullname_from_uid( $uid ) eq 'default_look' ) {
    return 1;
    }

    return '';
}


####################################################

=head2 get_number_of_active_users()

Returns the total number of registered users counting people in the ADMIN group

=cut

####################################################
sub get_number_of_active_users {
  my $self = shift;
    my ( $total ) = DBAccess->sqlSelect("count(*)", "users");

    # super-admin users should not be counted:
    $total -= Group->number_of_members('ADMIN');

    # default_look user should not be counted:
    $total -= 1;

    return $total;
}



####################################################

=head2 max_number_of_users_quota_exceeded()

Returns 1 if the user quota is exceeded.

=cut

####################################################
sub max_number_of_users_quota_exceeded {
    my $self = shift;

    # when $PARAMS->{max_number_of_users} is false we have unlimited users

    if( $PARAMS->{max_number_of_users} &&
        ( $USER->get_number_of_active_users() >= $PARAMS->{max_number_of_users} )
     ) {

    return 1;
    }

    return undef;
}


####################################################

=head2 get_user_defined_homepage()

Returns undef

=cut

####################################################

sub get_user_defined_homepage {
    my $self = shift;
    return undef;
}



####################################################

=head2  get_display_name()

Returns a string contraining "nickname (fullname)".

=cut

####################################################

sub get_display_name {
    my $self = shift;

    my $fullname = $self->fullname();
    my $authorname = $self->{nickname} || $fullname;
    $authorname .=  " ($fullname)" if defined($fullname) && $fullname ne '';

   return $authorname;
}

####################################################

####################################################
# Template methods
####################################################
sub get_template {
    my $self          = shift;
    my $template_name = shift || die "Must provide template name!";
    my $op            = shift || 'show';

    my $template      = $self->get_user_template_object();

    my $data = {
                hidden_fields => [
                    { name=>'isa',              value=>'User'},
                    { name=>'op',               value=>$op},
                    ],
                self          => $self,
        };

    return $data;
}
my $USER_TEMPLATE_OBJECT = undef;

# returns a class variable that holds a Template processing object
sub get_user_template_object {
    unless(defined $USER_TEMPLATE_OBJECT) {
        $USER_TEMPLATE_OBJECT = Template->new({
            INCLUDE_PATH => ['Metadot/User/templates','Metadot/templates'],
            }) || die $Template::ERROR, "\n";
    }
    return $USER_TEMPLATE_OBJECT;
}

#######################################################
# password_expired
# checks to see if a users password has expired
# returns 1 or boolean TRUE if password is expired
#######################################################
sub password_expired {
    my $self = shift;
    #my $uid  = shift;

    my $uid = $self->uid();

    my $u = Metadot::UserFactory->get_user($uid);

    my $current_timestamp = time();
    my $limit = $PARAMS->{password_expire_duration};

    if( $uid == -1 || $uid == undef ){
        ## default look (non-logged in users)
        return 0;
    }
    my ($last_password_changed) =
       DBAccess->sqlSelect("password_changed_time","users","userid=$uid");

    unless( defined( $last_password_changed ) ){
       return 1;
    }

    ## convert limit from days into seconds
    ## limit = seconds * minutes * hours * days
    $limit = 60 * 60 * 24 * $limit;
    $self->debug_msg(5,"Metadot::User::Default::password_expired()
                         last_password_changed[$last_password_changed]
                         current[$current_timestamp] limit[$limit]");

    my $lapse = $current_timestamp - $last_password_changed;


    return ($lapse > $limit);
}

#######################################################
# password_expires_in
# returns a timestamp of the number of days left before
# a user password expires
#######################################################
sub password_expires_in {
    my $self = shift;
    my $uid  = shift || die("Did not get uid for password_expires_in");

    my $user = Metadot::UserFactory->get_user($uid);

    my ($lastchanged) = DBAccess->sqlSelect("password_changed_time","users",
                                             "userid = $uid");

    my $limit = $PARAMS->{password_expire_duration};

    my $lapse = (time - $lastchanged)/(60*60*24);

    my $days_left = int($limit - $lapse);

    $self->debug_msg(5,"Metadot::User::Default::password_expires_in
                         user:[$uid] limit[$limit] last_changed[$lastchanged]
                         lapse[$lapse] expires[$days_left]");

    ## return a very large number if Metadot does not manage user
    ## passwords
    if( MetadotConfig->get_param_value('user_management_is_external') ){
       $days_left = 100000;
    }

    return $days_left;


}


use constant PASSWORD_EXPIRATION_DURATION_DEFAULT => 180;

sub password_expiration_duration {
   return $PARAMS->{password_expire_duration} || PASSWORD_EXPIRATION_DURATION_DEFAULT();
}


sub default_password_expiration_message {
    my $self = shift;

    return 'Your password has expired and it must be reset before continuing';
}

sub password_expiration_message {
    my $self = shift;

    my $custom_message = $PARAMS->{password_expiration_message};

    if( $custom_message eq '' ){
       return $self->default_password_expiration_message();
    }

    return $custom_message;
}

use constant RESET_PASSWORD_KEY  => '_session_reset_password_uid';
use constant FORGOT_PASSWORD_KEY => '_session_forgot_password';

########################################
# build_reset_session
# The rationale is that
# an anonymous user, once he has demonstrated that he has the right to
# reset the password for a user, will have a session that gives him the
# ability to do so.
# Creates a session and sets a session variable which will allow
# for the resetting of a user's password
########################################
sub build_reset_session {
   my $self   = shift;
   my $userid = shift || $self->uid();

   (defined $userid) || die('Must specify a valid user id to allow reset');



   # persist the session. if the session is new, add it. otherwise,
   # save it
   #my $session_handler = Metadot::SessionHandlerFactory->get_session_handler();
   my $session = Session->new();
   $session->add(MetadotConfig->get_reserved_uid('ANONYMOUS_USER'));

   # set a value in the session using RESET_PASSWORD_KEY containing
   # the userid of the user whose password will be changed
   $session->set_value(RESET_PASSWORD_KEY, $userid);

   return $session;

}

sub build_forgot_password_session {
   my $self   = shift;
   my $userid = shift || $self->uid();

   my $session = $self->build_reset_session();
   $session->set_value(FORGOT_PASSWORD_KEY, 1);

   return $session;
}


# retrieves the password being reset from a session built by build_reset_session
sub get_reset_password_uid_from_session {
   my $self = shift;
   my $session = shift;
   return $session->get_value(RESET_PASSWORD_KEY);
}


# determines if this reset session was due to a forgotten password
sub is_forgot_password_mode {
   my $class   = shift;
   my $session = shift;
   return ($session->get_value(FORGOT_PASSWORD_KEY));
}

####################################
# initialize_password_changed_times
# this method initializes the password_changed_time field
# when the automatic password expiration feature is turned on
# on those users where it has not been set
####################################
sub initialize_password_changed_times{
    my $self = shift;

    DBAccess->sqlUpdate('users','password_changed_time is null',
                         password_changed_time => time
                       );
}


# returns any subscriptions this user has via group membership.
sub get_group_subscriptions {
    my $self = shift;

    return Subscription->get_group_subscriptions_for_user($self->uid());
}



sub get_value {
    my $self = shift;
    my $field_name = shift || die "must include field_name";

    my $result = $self->{$field_name};

    return $result;
}


# avoids any post-processing of value, for form and db purposes.
# look at User3000 for why this exists.
sub get_value_raw {
    my $self = shift;
    return $self->get_value(@_);
}

sub set_value {
    my $self = shift;
    my $field_name = shift || die "must include field_name";
    my $value = shift;

    $self->{$field_name} = $value;
}








1;
