###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::User::FlexUser;
use strict;

use CGI qw(escape);
use Text::CSV_XS;

use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $ACCESS_BROKER $CGI);

use Metadot::User::Default;
use Metadot::Metagram;
use Metadot::SystemApp::MetagramCenter;
use Metadot::UserFactory;

use vars qw(@ISA);
use Group;

@ISA=qw(Metadot::User::Default Metadot::UploadsManager);


my $AC = Metadot::User::Default->get_AC();

push( @{ $AC->{BUNDLES}{ADMIN_REGISTRATIONS}{OPS} }, 'show_users_groups' );
push( @{ $AC->{BUNDLES}{ADMIN_REGISTRATIONS}{OPS} }, 'export_csv' );
push( @{ $AC->{BUNDLES}{MOD}{OPS} }, 'confirm_upload_deletion' );
push( @{ $AC->{BUNDLES}{MOD}{OPS} }, 'delete_upload' );
push( @{ $AC->{BUNDLES}{SEARCH}{OPS} }, 'download_file' );
push( @{ $AC->{BUNDLES}{SEARCH}{OPS} }, 'show_user_directory' );
push( @{ $AC->{BUNDLES}{SEARCH}{OPS} }, 'show_user_details' );

sub get_AC {
    return $AC;
}

my $ORG_KEY = '__ORG__';

my $FIELDS_CONFIG = {
             DISPLAY_SETS => [ 'Login Information', 'My start page after login', 'Personal Information' ],
             FIELDS => [
                {
                 field_name => 'first_name',
                 description => 'First Name',
                 max_length => 64,
                 size => 20,
                 is_required => 1,
                 display_set => 0,
                 storage_type => 'secondary',
                 store_at_column => "f1",
                 verify_type => 'non_empty',
                },
                {
                 field_name => 'initial',
                 description => 'Middle Initial',
                 max_length => 1,
                 size => 1,
                 is_required => 0,
                 is_not_required_for_default_users => 1,
                 display_set => 0,
                 storage_type => 'secondary',
                 store_at_column => "f2",
                },
                {
                 field_name => 'last_name',
                 description => 'Last Name / Family Name',
                 max_length => 64,
                 size => 20,
                 is_required => 1,
                 is_not_required_for_default_users => 1,
                 display_set => 0,
                 storage_type => 'secondary',
                 store_at_column => "f3",
                },
                {
                 field_name => 'nickname',
                 description => 'Nickname / Screename',
                 max_length => 64,
                 size => 20,
                 is_required => 0,
                 display_set => 0,
                 storage_type => 'primary',
                },
                {
                 field_name => &get_organization_field_name,
                 description => 'Organization',
                 #field_type => 'select',
                 field_type => 'Metadot::User::User3000::OrganizationField',
                 is_required => 0,
                 display_set => 0,
                 #field_values => &get_organizations(),
                 field_values_function => 'get_organizations',
                 storage_type => 'secondary',
                 store_at_column => "f16",
                 enable_param => "allow_organization_registration",
                 only_admin_can_edit => 1, ## currently does not apply to all field types
                },
                {
                 field_name => &get_flex_homepage_field_name(),
                 #description => '&nbsp;', # this causes all kinds of problems for XML::Dumper, XML::Simple
                 description => 'Home Page',
                 #field_type => 'radio',
                 field_type  => 'Metadot::User::User3000::HomePageField',
                 is_required => 0,
                 display_set => 1,
                 field_values_function => 'get_homepage_values',
                 storage_type => 'secondary',
                 store_at_column => "f17",
                 use_line_breaks => 1,
                },
                {
                 field_name => 'email',
                 description => 'E-mail Address',
                 max_length => 64,
                 size => 20,
                 is_required => 1,
                 display_set => 0,
                 storage_type => 'primary',
                },
                {
                 field_name => 'password',
                 description => 'Password',
                 max_length => 64,
                 size => 20,
                 is_required => 1,
                 display_set => 0,
                 storage_type => 'primary',
                 field_type => 'password',
                },
                {
                 field_name => &get_picture_field_name(),
                 description => 'Picture',
                 is_required => 0,
                 display_set => 2,
                 storage_type => Metadot::UploadsManager::UPLOAD_TYPE_USER(),
                 #field_type => Metadot::UploadsManager::UPLOAD_TYPE_USER(),
                 field_type => 'Metadot::User::User3000::PictureField',
                 #enable_param => 'show_user_directory',
                },
                {
                 field_name => 'address_line_1',
                 description => 'Address Line 1',
                 max_length => 64,
                 size => 36,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f4",
                },
                {
                 field_name => 'address_line_2',
                 description => 'Address Line 2',
                 max_length => 64,
                 size => 36,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f5",
                },
                {
                 field_name => 'city',
                 description => 'City',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f6",

                },
                {
                 field_name => 'state',
                 description => 'State',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f7",
                },
                {
                 field_name => 'zip_code',
                 description => 'Zip Code',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f8",
                },

                {
                 field_name => 'country',
                 description => 'Country',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f9",
                },

                {
                 field_name => 'telephone',
                 description => 'Telephone',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f10",
                },

                {
                 field_name => 'fax',
                 description => 'Fax',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f11",
                },

                {
                 field_name => 'mobile_phone',
                 description => 'Mobile Phone',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f12",
                },

                {
                 field_name => 'pager_phone',
                 description => 'Pager Phone',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f13",
                },

                {
                 field_name => 'pager_number',
                 description => 'Pager Address',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'primary',
                },

                {
                 field_name => 'icq_number',
                 description => 'ICQ Number',
                 max_length => 64,
                 size => 20,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'primary',
                },

                {
                 field_name => 'homepage',
                 description => 'Homepage URL',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'primary',
                },

                {
                 field_name => 'job_title',
                 description => 'Job Title',
                 field_type => 'textarea',
                 rows => 3,
                 cols => 30,
                 wrap => 1,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "t1",
                },

                {
                 field_name => 'schedule',
                 description => 'Schedule',
                 field_type => 'textarea',
                 rows => 3,
                 cols => 30,
                 wrap => 1,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "t2",
                },

                {
                 field_name => 'notes',
                 description => 'Notes',
                 field_type => 'textarea',
                 rows => 3,
                 cols => 30,
                 wrap => 1,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "t3",
                },

                {
                 field_name => 'quotes',
                 description => 'Quotes',
                 field_type => 'textarea',
                 rows => 3,
                 cols => 30,
                 wrap => 1,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "t4",
                },

                {
                 field_name => 'home_telephone',
                 description => 'Home Telephone',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f14",
                },

                {
                 field_name => 'home_fax',
                 description => 'Home Fax',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f15",
                },

                {
                 field_name => 'home_address',
                 description => 'Home Address',
                 field_type => 'textarea',
                 rows => 3,
                 cols => 30,
                 wrap => 1,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "t5",
                },
                   ],
            };


my $check_for_required_fields_done;


# required for backward compatibility
sub fullname {
    my $self=shift;
    my $val = shift;

    if (defined $val) {

    my ($first_word, $second_word, $third_word, $remainder ) = ('', '', '', '');

    ($first_word, $second_word, $remainder ) = split( /\s+/, $val, 3 );

    ($first_word) || print STDERR "user has no defined name";

    my $first_name = $first_word || '';
    my ($last_name, $initial) = ('', '');

    if( $second_word && $remainder && ( length $second_word == 1 ) ) {

        $initial = $second_word || '';
        $last_name = $remainder || '';

    } else {

        $initial = '';
        $last_name =
          ( $second_word ) ? ( ( $remainder ) ? "$second_word $remainder" : $second_word ) : '';

    }

    $self->set_value( 'first_name', $first_name );
    $self->set_value( 'last_name', $last_name );
    $self->set_value( 'initial', $initial );

    $self->{fullname} = $val;
    return $self->{fullname};
    }

    my $fullname = $self->first_name();

    $fullname .= ($self->initial()) ?
      ' ' . $self->initial() . ' ' : ' ';

    $fullname .= $self->last_name() || '';

    return $fullname || $self->{nickname};

}


sub first_name {
    my $self = shift;
    return $self->get_value('first_name');
}

sub initial {
    my $self = shift;
    return $self->get_value('initial');
}

sub last_name {
    my $self = shift;
    return $self->get_value('last_name');
}


## This method must be overwritten in child classes. See Sample below.
## Sample:
### must override SUPER because $FIELDS_CONFIG is a class variable
#sub get_fields_config {
#    my $self = shift;
#    return $self->SUPER::get_fields_config($FIELDS_CONFIG);
#}
## Copy and paste above method in child class

sub get_fields_config {
    my $self = shift;
    my $fields_config = shift;

    # try to resolve fields config from self
    if (!defined $fields_config && ref($self)) {
        $fields_config = $self->{_fields_config};
    }

    # otherwise use the class object
    $fields_config ||= $FIELDS_CONFIG;



    my $required_fields = [ qw( first_name last_name email password ) ];

    # we do this to trap any missing required fields from the
    # fields data structure
    unless( $check_for_required_fields_done ) {
        $check_for_required_fields_done++;

        foreach my $req_field ( @{ $required_fields } ) {
            my $found;
            foreach my $field ( @{ $fields_config->{FIELDS} } ) {
                $found++ if ( $field->{field_name} eq $req_field );
            }
            unless( $found == 1 ) {
                die "fields configuration data structure does not contain field $req_field (or contains it more than once)";
            }
        }
    }

    return $fields_config;
}

sub get_config_for_field {
    my $self = shift;
    my $field_name = shift || die("Must specify a field");
    my $fields_config = $self->get_fields_config();
    foreach my $field (@ { $fields_config->{FIELDS} }) {
        if ($field->{field_name} eq $field_name) {
            return $field;
        }
    }
}

sub get_value {
    my $self = shift;
    my $field_name = shift || die "must include field_name";

    my $result = $self->{$field_name};

    return $result;
}


# avoids any post-processing of value, for form and db purposes.
# look at User3000 for why this exists.
sub get_value_raw {
    my $self = shift;
    return $self->get_value(@_);
}

sub set_value {
    my $self = shift;
    my $field_name = shift || die "must include field_name";
    my $value = shift;

    $self->{$field_name} = $value;
}

sub get_default_value {
    my $self = shift;
    my $field_name = shift || die("Must supply a field name");
    my $result;

    my $field_config = $self->get_config_for_field($field_name);
    if ($field_config) {
        $result = $field_config->{default_value};
    }
    return $result;
}

sub set_default_value {
    my $self = shift;
    my $field_name = shift || die("Must supply a field name");
    my $value = shift;

    my $field_config = $self->get_config_for_field($field_name);
    $self->debug_msg(5, "Setting default value of $field_name ($field_config) to '$value'");

    if ($field_config) {
        $field_config->{default_value} = $value;
    }
}

sub add {
    my $self = shift;

    $self->{upload_size} = -1;
    $self->{upload_number} =  -1;

    # make fullname from subparts
    my $fullname = $self->first_name();

    $fullname .= ($self->initial()) ?
      ' ' . $self->initial() . ' ' : ' ';

    $fullname .= $self->last_name();

    my $db_now = Utility->db_now();

    my %values = (
        fullname => $fullname,
        nickname => $self->{nickname},
        homepage => $self->{homepage},
        email    => $self->{email},
        icqnumber=> $self->{icq_number},
        pager    => $self->{pager_number},
        quote    => $self->{quote},
        password => $self->{password},
        approvalneeded => $self->{approval_needed},
        linkname1   => $self->{link_name_1}, #RESTRICTED-USER
        linkname2   => $self->{link_name_2}, #RESTRICTED-USER
        adminlevel  => 1,
        adminmode   => 0,
        debuglevel  => 0,
        updatedate  =>$db_now,
        creationdate=>$db_now,
        max_total_upload_size => $self->{upload_size},
        max_total_numberof_upload => $self->{upload_number},
        ldapdn  => $self->{ldap_dn},
        password_changed_time => $self->{password_changed_time} || time,
        pw_expire_notification_time => $self->{pw_expire_notification_time},       #NICO-PASS
    );

    $self->_flatten_values(\%values);

    # primary storage
    my $uid = DBAccess->sqlInsert("users", %values);
    # secondary storage
    $self->_insert_secondary_fields( $uid );

    $self->uid($uid);

    # create default user portal
    my $portal = Portal->new(undef);

    #RESTRICTED-USER: added additional condition against creation
    if ($PARAMS->{my_website_creation} && !$self->is_restricted() ) {
        $self->create_default_portal();
    }

    $self->add_user_to_primary_organization();
    $self->_process_uploads();

    return $uid;
}

sub _insert_secondary_fields {
    my $self = shift;
    my $uid = shift;

    my $config = $self->get_fields_config();

    my %values;
    my $enable_param = '';
    foreach my $field ( @{ $config->{FIELDS} } ) {
        if( lc $field->{storage_type} eq 'secondary' ) {
            $enable_param = $field->{enable_param};
            if (defined($enable_param) && !$PARAMS->{$enable_param}) {
                next;
            }

            my $store_at = $field->{store_at_column};

            ($store_at) || die "field $field->{field_name} does not contain store_at_column";

            my $value = $self->get_value_raw( $field->{field_name} );

            $values{ $store_at } = $value;
        }
    }

    $self->_flatten_values(\%values);

    DBAccess->sqlInsert(
            "extended_user",
            userid => $uid,
            %values
               );

}


##################################################################

=head2 $USER->delete( $delete_user_personal_content )

SEE Metadot::User::Default::delete()
Also deletes extended_user data

=cut

##################################################################

sub delete {
    my $self = shift;

    my $uid = $self->uid();
    DBAccess->sqlDelete('extended_user', "userid=$uid");

    return $self->SUPER::delete(@_);
}


##################################################
sub save {
    my $self=shift;

    # build fullname from its parts
    my $fullname = $self->first_name();

    $fullname .= ($self->initial()) ?
      ' ' . $self->initial() . ' ' : ' ';

    $fullname .= $self->last_name();

    my $db_now = Utility->db_now();

    my %values = (
                  fullname      => $fullname,
                  nickname      => $self->{nickname},
                  homepage      => $self->{homepage},
                  email         => $self->{email},
                  icqnumber     => $self->{icq_number},
                  pager         => $self->{pager_number},
                  adminlevel    =>$self->{admin_level},
                  quote         => $self->{quote},
                  max_total_upload_size     => $self->{upload_size},
                  max_total_numberof_upload => $self->{upload_number},
                  approvalneeded            => $self->{approval_needed},
                  updatedate    =>$db_now,
                  ldapdn        => $self->{ldap_dn},
                  lastonline    => $self->{last_online},
                  password_changed_time     => $self->{password_changed_time},
                  pw_expire_notification_time => $self->{pw_expire_notification_time}               #NICO-PASS
    );

    # also update password if the password field has been set
    if(defined $self->{password} && $self->{password} ne '') {
        $values{password} = $self->{password};
    }

    $self->_flatten_values(\%values);
    DBAccess->sqlUpdate("users","userid='$self->{uid}'", %values);

    $self->_update_secondary_fields();
    if ($ACCESS_BROKER->is_allowed_to_do( 'admin_tasks')) {
        $self->add_user_to_primary_organization();
    }
    $self->_process_uploads();
}


sub _update_secondary_fields {
    my $self = shift;

    my $config = $self->get_fields_config();

    my %values;
    my $enable_param = '';
    foreach my $field ( @{ $config->{FIELDS} } ) {
        if( lc $field->{storage_type} eq 'secondary' ) {
            $enable_param = $field->{enable_param};
            if (defined($enable_param) && !$PARAMS->{$enable_param}) {
                next;
            } elsif ($field->{only_admin_can_edit} && !$ACCESS_BROKER->is_allowed_to_do( 'admin_tasks')) {
                next;
            }

            my $store_at = $field->{store_at_column};

            ($store_at) || die "field $field->{field_name} does not contain store_at_column";

            my $value = $self->get_value_raw( $field->{field_name} );

            $values{ $store_at } = $value;
        }
    }

    $self->_flatten_values(\%values);

    # learn if a record exists already for this user
    my ($already_there) =
      DBAccess->sqlSelect("count(*)",
              "extended_user",
              "userid = '$self->{uid}'");

    if( $already_there ) {

        DBAccess->sqlUpdate(
                    "extended_user",
                    "userid='$self->{uid}'",
                    ( %values )
                   );
    } else {

        DBAccess->sqlInsert(
                    "extended_user",
                    userid => $self->{uid},
                    ( %values )
                   );
        }
}


sub _process_uploads {
    my $self = shift;

    my $config = $self->get_fields_config();
    my $isa = $self->is_a();

    my $err = '';
    foreach my $field ( @{ $config->{FIELDS} } ) {
    if( lc $field->{storage_type} =~ /_upload$/ ) {
            my $field_name = $field->{field_name} || die "must specify field_name";
            my $field_desc = $field->{description} || die "must specify field description";
            my $field_type = $field->{field_type} || die "must specify field type";

            ## Uploads Manager
            my $result = $self->process_upload( {
                isa => $isa,
                id => $self->uid(),
                uid => $self->uid(),
                field_name => $field_name,
                field_desc => $field_desc,
                type => 'user_upload',
                is_required => $field->{is_required}, } );
            if($result) { $err .= "$result<br />" }
        }
    }

    return $err;
}

sub get_write_dir_namespace {
    my $self = shift;

    my $uid = $self->uid();
    my $file_isa = $self->is_a();
    $file_isa =~ s/\:/_/g;

    unless (defined($uid) && $file_isa) {
        $self->print_click_back( "Upload Error", "User id not defined.  If this problem occurs again, please contact the site administrator." ) ;
        exit(0);
    }

    return $PARAMS->{absolutepathfileuploaddir} . "/" . $file_isa . "_" . $uid;
}

sub new {
    my $class = shift;

    my $self = $class->SUPER::new(@_);

    $self->initialize(@_);
    return $self;
}

sub initialize {
    my $self = shift;

    #print STDERR __PACKAGE__." initializes $self from ".caller()."\n";

    # this allows the class variable to appear polymorphic
    unless (defined $self->{_fields_config}) {
        $self->{_fields_config} = $FIELDS_CONFIG;
    }

    $self->_load_secondary_fields();
}


# make sure all my values are unflattened
sub _initialize{
    my $self = shift;
    my $uid = $self->uid;

    if(defined ($uid)){
        $self->SUPER::_initialize($uid);

        # unflatten all my primary fields
        my $config = $self->get_fields_config();
        foreach my $field ( @{ $config->{FIELDS} } ) {
            if( lc $field->{storage_type} eq 'primary' ) {
                my $enable_param = $field->{enable_param};
                if (defined($enable_param) && !$PARAMS->{$enable_param}) {
                    next;
                }
                my $db_field_name = $field->{store_at_column};

                $self->set_value($db_field_name, $self->_unflatten_value($self->get_value_raw($db_field_name)));
            }
        }
    }
}


sub _load_secondary_fields {
    my $self = shift;

    if (defined $self->{uid}) {
        my $config = $self->get_fields_config();

        my $fields_string;
        my @field_names;

        my $enable_param = '';
        foreach my $field ( @{ $config->{FIELDS} } ) {
            if( lc $field->{storage_type} eq 'secondary' ) {
                $enable_param = $field->{enable_param};
                if (defined($enable_param) && !$PARAMS->{$enable_param}) {
                    next;
                }
                my $db_field_name = $field->{store_at_column};
                $fields_string .= (defined $fields_string) ? ", $db_field_name" : $db_field_name;

                push @field_names, $field->{field_name};
            }
        }

        my @results = DBAccess->sqlSelect(
            $fields_string,
            'extended_user',
            "userid='$self->{uid}'"
        );

        foreach my $value (@results) {
            my $field_name = shift @field_names;
            $self->set_value( $field_name, $self->_unflatten_value($value) );
        }
    }
}

sub www_show_users {
    my $self = shift;

    $self->debug_msg(3, "Metadot::User::FlexUser::www_show_users()");
    $USER->www_show_users_groups();
}


#aju
sub www_show_users_groups {
    my $self = shift;

    #print STDERR "in www_show_users_groups\n";
    $self->debug_msg(3, "Metadot::User::FlexUser::www_show_users_groups()");

    my @filters = (); my $groups_filter;
    my $add_tables = "";

    if ($FORM{first_name}) {

    my $first_name_pos = $self->_get_storage_position( 'first_name' );
    push @filters, "upper(extended_user.$first_name_pos) like upper('%$FORM{first_name}%')";
    }

    if ($FORM{last_name}) {

        my $last_name_pos = $self->_get_storage_position( 'last_name' );
        push @filters, "upper(extended_user.$last_name_pos) like upper('%$FORM{last_name}%')";
    }

    if ($FORM{nickname}) {
        push @filters, "upper(users.nickname) like upper('%$FORM{nickname}%')";
    }

    if ($FORM{email}) {
       push @filters, "upper(users.email) like upper('%$FORM{email}%')";
    }

    if ($FORM{groups}) {
    $groups_filter = $FORM{groups};
    }
    if ($FORM{pending}) {
        push @filters, "users.approvalneeded = 'required'";
    }
    if ($FORM{who_online}) {
        ## NZ: must include 'users.' to make get_last_online_clause not ambigous
        ## bugzilla bug report 288
        push @filters, "users.userid=sessions.userid and ".$SESSION->get_last_online_clause('users');
        $add_tables = 'sessions';
    }
    my $filter = join(' AND ', @filters);

    my $all_url = "";
    my $manage_users_url = "\n<a href=\"" . $self->make_url("index.pl?isa=User&amp;op=show_users") .
    "\">Go to 'Manage Users' Table</a><br />";

    # add a 'show all users' link before generating the table if a search filter has been defined.
    my $iid;
    if ($filter ne "") {
        $iid = $FORM{iid} || 0 ;
        $all_url = "index.pl?iid=$iid&amp;isa=User&amp;op=show_users_groups&amp;sort_by=$FORM{sort_by}&amp;order=$FORM{order}";
        $all_url = "\n<a href=\"" . $self->make_url($all_url) . "\">Show All Users' Groups</a><br />\n";
    }

    my $content = $self->show_users_groups( {filter=>$filter, groups_filter => $groups_filter,
                   startfrom=>$FORM{from}, maxitem=>$FORM{nb}, add_tables =>$add_tables});
    $DISPLAY->print_simple_layout({content=>$content, title=>"User Directory", iid=>$iid});

}


#aju
sub show_users_groups {
    my $self = shift ;
    my ($filter, $startfrom, $maxitem) = @_;
    my $s1 = "";
    my $c;
    my $oldi=0;

    #print STDERR "in show_users_groups\n";
    my ($uid, $fullname, $nickname, $email, $uploadsize, $uploadnumber);
    my $portal="";
    my $hpiid=0;

    $DISPLAY->debug_msg(3, "Metadot::User::FlexUser::show_users_groups(filter=$filter)");

    # Filter is now a vector into an args hash. This maintains backward compatibility.
    my $args; my $groups_filter;
    if (ref($filter) eq 'HASH') {
        $args   = $filter;
        $filter     = $args->{filter};
        $startfrom  = $args->{startfrom};
        $maxitem    = $args->{maxitem};
        $groups_filter = $args->{groups_filter};

    }


    ### select user's portal iid

    my $message;
    if($FORM{uid}){
        my $u = Metadot::UserFactory->get_user($FORM{uid});
        my $fullname .= " " . $u->webname;
        $message ="$fullname has been accepted and notified via email.";
    }
    #### table
    if($message){
        $s1 .= "<font color='#18bb94'>$message</font><br />";
    }

    my $imageslocation = $PARAMS->{'imageslocation'};

    $FORM{sort_by} = 'last_name' unless $FORM{'sort_by'};

    ## get the user result set
    $c = $self->get_show_users_groups_result_set(@_);

    $maxitem = 100 if (!$maxitem);   #PREVIOUS NEXT MAGIC
    $startfrom = 0  if(!$startfrom); #PREVIOUS NEXT MAGIC
    my $from = $startfrom;           #PREVIOUS NEXT MAGIC

    my $i=0;
    my ($totalrows) = $c->rows;
    $DISPLAY->debug_msg(5, "I have $totalrows rows");

    my $max_total_upload_size = $PARAMS->{'max_total_upload_size'};
    my $max_total_number_upload = $PARAMS->{'max_total_number_upload'};

    $s1 .= "
<center>
          <form method = post action = index.pl>
            <input type=hidden name=isa value=User>
            <input type=hidden name=op value=show_users_groups>
            <input type=hidden name=iid value = 0>
          <center><H2>Search for a user</H2></center>
          <div id=\"user_search_form\">
          <table width=\"100%\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\" align=\"center\">
                <tr align=center>
                        <td>
                <b>First Name</b><br /><input type=text size=15 name=first_name value=\"" .  $FORM{first_name} . "\">
                        </td>
                        <td>
                <b>Last Name / Family Name</b><br /><input type=text size=15 name=last_name value=\"". $FORM{last_name} . "\">
                        </td>
                        <td>
                <b>Nickname / Screenname</b><br /><input type=text size=15 name=nickname value=\"". $FORM{nickname} . "\">
                        </td>
                        <td>
                <b>Email address</b><br /><input type=text size=15 name=email value=\"". $FORM{email} . "\">
                        </td>
                        <td>
                <b>Groups</b><br /><input type=text size=15 name=groups value=\"" . $FORM{groups} . "\">
                        </td>
                </tr>
                <tr>
                        <td colspan = 5>
                                <center><input type=submit value = 'Start Search'></center></form>
                        </td>
                </tr>
              </table></div>\n";
    $s1 .= "</center><br /><center>
         <table border=\"1\" cellpadding=\"4\" cellspacing=\"0\" width=\"98%\">
            <tr>
              <td valign='middle'><font size=-1><b><a class='prevnextbar' href=";


    my %order_arrows = ( first_name => '', last_name => '', nickname => '', email => '' );
    my %sort_direction = ( first_name => '', last_name => '', nickname => '', email => '' );
    my ( $sort_direction, $sort_image );

    if( $FORM{order} eq 'desc' ) {
    $order_arrows{ $FORM{sort_by} } =
      "<img src=\"$imageslocation/obj/triangle_down.gif\" border=\"0\" alt=\"Sorted Descending\">";
    $sort_direction{ $FORM{sort_by} } = "&amp;order=asc";
    } else {
    $order_arrows{ $FORM{sort_by} } =
      "<img src=\"$imageslocation/obj/triangle_up.gif\" border=\"0\" alt=\"Sorted Ascending\">";
    $sort_direction{ $FORM{sort_by} } = "&amp;order=desc";
    }

    my $base_url = $self->make_url("index.pl?iid=0&amp;isa=User&amp;op=show_users_groups" .
                   "&amp;first_name=$FORM{first_name}&amp;last_name=$FORM{last_name}" .
                   "&amp;email=$FORM{email}&amp;nickname=$FORM{nickname}" .
                   "&amp;groups=$FORM{groups}" .
                   "&amp;pending=$FORM{pending}&amp;who_online=$FORM{who_online}");
    $s1 .= $base_url."&amp;sort_by=first_name$sort_direction{first_name}" . ">First Name $order_arrows{first_name}</a>&nbsp; </b> <I><a class='prevnextbar' href=";

    $s1 .= $base_url."&amp;sort_by=last_name$sort_direction{last_name}" . ">Last Name $order_arrows{last_name}</a>&nbsp; </b> <I><a class='prevnextbar' href=";

    $s1 .= $base_url."&amp;sort_by=nickname$sort_direction{nickname}" . ">(nickname) $order_arrows{nickname}";

    $s1 .= "</a><I></font></td>
              <td valign='middle'><font size=-1><b>ID</b></font></td>" .
#                "<td><b>Portal</b></td>" .
                 "<td valign='middle'><font size=-1><b><a class='prevnextbar' href=" .
                  $base_url."&amp;sort_by=email$sort_direction{email}" .
      ">Email address $order_arrows{email}</a></b></font></td>";
#       my $upload_cells = "<td valign='middle'><font size=-1>Max. total upload size (MB)<br /><font size=-5>(Default: $max_total_upload_size MB)</font></td>                     <td valign='middle'><font size=-1>Max. total number of upload<br /><font size=-5>(Default: $max_total_number_upload)</font></td>";
    if ($PARAMS->{allow_organization_registration}) {
        $s1 .= "<td><b>Organization</b></td>";
    }
    $s1 .= "<td><b>Groups</b></td>";
    $s1 .= "<td><b>Last login</b></td>";
    $s1 .= "<td><b>Actions</b></td>";
    $s1 .= "</tr>\n";

#     print STDERR "using groups filter: $groups_filter\n";

    my $hidden_users_count;

    ## NZ: This is checking the current logged user and not the
    ## user being tested
    #my $user_is_admin = $USER->is_admin();

    my $user_is_admin;

    my $is_allowed_to_see_hidden_users = $USER->is_admin();
    my $user_can_approve_registration = $USER->is_admin() ||
        $self->can_manage_registrations();

    while(( my ($uid, $fullname,
        $first_name,
        $initial,
        $last_name,
        $nickname,
        $email,
        $uploadsize,
        $uploadnumber,
                $organization_grp_id,
                $lastonline) = $c->fetchrow()) && ($i < $maxitem))
    {
        my $u = Metadot::UserFactory->get_user($uid);

        # Testing if the user being displayed is an admin
        $user_is_admin = $u->is_admin();

        $user_can_approve_registration = $user_is_admin || $self->can_manage_registrations($u);
        my $user_groups; my @user_groups_sort = ();
        # see if the groups filter string is in the groups list
        if ($uid) {     #  this excludes root, uid 0, also
            #print STDERR "getting user groups for $uid with name $fullname\n";
            $user_groups = Group->get_group_names_where_member($uid);
            #print STDERR "uid $uid ($first_name $last_name) is in groups: ", Dumper($user_groups), "\n";
            #sort the group names
            @user_groups_sort = sort(@{$user_groups});
        }

        #format the user's group info to go into the table cell
        my $user_groups_fmt = join(', ', @user_groups_sort);
        if (!$user_groups_fmt || ($user_groups_fmt eq '')) {
            $user_groups_fmt = "&nbsp;";
        }

        if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC

        $i++;
        if($user_is_admin || $user_can_approve_registration){
            $fullname = "<b>$fullname</b><super>*</super>";
        }

        if ($nickname) {
            $nickname = "<I>($nickname)</I>\n";
        } else {
            $nickname = "";
        }

        my $rejected_string = "";
        if (($u->approval_needed() || "") eq "rejected"){
            $rejected_string = "<font color=#FF0000>(rejected)</font>";
        }

        $email = "<a href=\"mailto:$email\">$email</a>\n" if ($email);

        $s1 .= "<font size=-3><tr><td>\n";
        if($user_can_approve_registration){
           $s1 .= $u->show_panel('pending');
        }

        $rejected_string = $rejected_string ? $rejected_string : "";

        my $action_pending_link;
        if( $FORM{pending} == 1 || $u->{approval_needed} eq 'required' ){
            $action_pending_link = '&amp;pending=1';
        }
        my $view_profile_link = "<a class='text_button' href='".
                                $self->make_url("index.pl?isa=User&amp;op=show_user_details&amp;uid=$uid&amp;view=medium&amp;actionview=1$action_pending_link")
                              . "'>View</a>";

        $s1 .= "$fullname $nickname $rejected_string";

        my $subscription_link = "[ <a href='". $self->make_url("index.pl?isa=User&amp;op=show_subscriptions&amp;uid=$uid")
                              . "'>manage subscriptions</a> ]";
        my $become_user_link =
          $self->can_become_user($uid) ?
          "[ <a href='". $self->make_url("index.pl?isa=User&amp;op=become_user&amp;uid=$uid")
                              . "'>become this user</a> ]"
          : '';
        $s1 .= "</td>\n";

        $s1 .= "<td>$uid</td>\n" .
               "<td>$email</td>";

        if ($PARAMS->{allow_organization_registration}) {
            $s1 .= "<td>" . ($u->get_organization_name() || "&nbsp;") . "</td>";
        }
        $s1 .= "<td>$user_groups_fmt</td>";

        if( defined( $lastonline) && $lastonline != ''){
           $lastonline = Utility->format_unix_timestamp($lastonline);
        } else {
           $lastonline = 'Never logged in';
        }
        $s1 .= "<td nowrap>$lastonline</td>\n";

        $s1 .= "<td nowrap>$view_profile_link</td>\n";

        $s1 .= "</td></tr></font>\n";
    }

    $c->finish();

    $user_is_admin = $USER->is_admin();

    my $iid = $FORM{iid} || 0 ;
    my $url = "index.pl?iid=$iid&amp;isa=User&&amp;op=show_users_groups&amp;threads=$FORM{threads}&amp;order=$FORM{order}&amp;sort_by=$FORM{sort_by}";

    # include filter info in next/prev links
    my $nurl = $url .
    "&amp;first_name=$FORM{first_name}&amp;last_name=$FORM{last_name}" .
        "&amp;email=$FORM{email}&amp;nickname=$FORM{nickname}" .
    "&amp;groups=$FORM{groups}" ;
    my $switch = "";

    my ($pending_approval) = DBAccess->sqlSelect("count(*)", "users", "approvalneeded='required'");

   # add All Users link
    if( $FORM{pending} eq 'true' or $FORM{who_online} eq 'true' or $filter ) {
    my $link = "<a class='prevnextbar' href='". $self->make_url("$url") . "'>All Users</a>";

    $switch .= "[ $link ] ";
    }

    if (($USER->is_admin) && ($pending_approval || (($FORM{pending} || "") eq 'true'))){
    unless ($FORM{pending} eq 'true') {
        my $link =
          "<a class='prevnextbar' href='".$self->make_url("$url&amp;pending=true")."'>Pending users only</a>";
        $switch .= "[ $link ] ";
    }
    }

    # display online users
    my $who_online = $SESSION->get_members_online();
    if (($user_is_admin || $user_can_approve_registration) && ($who_online || (($FORM{who_online} || "") eq 'true'))){

    unless( $FORM{who_online} eq 'true' ) {
        my $link =
          "<a class='prevnextbar' href='".$self->make_url("$url&amp;who_online=true")."'>Show $who_online online users only</a>";
        $switch .= "[ $link ] ";
    }
    }

    if ($user_is_admin) {
    unless( MetadotConfig->get_param_value('user_management_is_external') ) {
        #$s1 .= "<center><h2><a href=\"index.pl?isa=Register&amp;op=register_form&amp;admin_mode=true\">Register a new user</a></h2></center>\n";
        my $link =
          "<a class='prevnextbar' href='".$self->make_url("index.pl?isa=Register&amp;op=register_form&amp;admin_mode=true")."'>Register a new user</a>";
        $switch .= "[ $link ] ";
    }
    }

    if ($self->is_allowed_to_do('export_csv'))
    {
        my $icon = $PARAMS->{imageslocation}.'/excel.jpg';
        my $link = qq~<a class='prevnextbar' href="index.pl?isa=User&amp;op=export_csv">Export to spreadsheet&nbsp;<img class="icon" src="$icon"/></a>~;
        $switch .= "[ $link ] ";
    }



    #PREVIOUS NEXT MAGIC
    my $pn = "" ;

    my $total_displayed = $totalrows - $hidden_users_count;

    if ($total_displayed > $maxitem) {
    $pn = Components->get_previous_next_button($maxitem, $i, $total_displayed, $startfrom, $nurl);
    } else {

    $pn="\n\n<center><table border=\"0\" cellpadding=\"5\" cellspacing=\"5\" WIDTH='100%'><tr><td align='left'><font size=-1><I><b>Found $total_displayed entries</b></I></font></td></tr></table></center>";

    }

#    $pn = Components->get_previous_next_button($maxitem, $i, $totalrows, $startfrom, $nurl) if ($totalrows > $maxitem);

    $s1.="</td></td></td></tr></table></center><br />";
    my $page = "$switch\n$pn\n$s1\n$pn\n<br /><I>Note: Names marked with (<super>*</super>) are administrators</I>\n\n";
    return $page;
}


sub get_show_users_groups_result_set {
    my $self = shift ;
    my ($filter, $startfrom, $maxitem) = @_;
    $self->debug_msg(3, "Metadot::User::FlexUser::get_show_users_groups_result_set(filter=$filter)");
    my $where = '';
    my $c;

    my $table_clause = "users, extended_user";

    # Filter is now a vector into an args hash. This maintains backward compatibility.
    my $args; my $groups_filter;
    if (ref($filter) eq 'HASH') {
        $args   = $filter;
        $filter     = $args->{filter};
        $startfrom  = $args->{startfrom};
        $maxitem    = $args->{maxitem};
        my $add_tables = $args->{add_tables}; # this can be an array ref
        $groups_filter = $args->{groups_filter};

        # add tables to the table clause if needed
        if ($add_tables) {
            $add_tables = [$add_tables] unless (ref($add_tables) eq 'ARRAY');
            $table_clause = join(',', ($table_clause, @{$add_tables}));
        }
    }


    if ($self->can_manage_registrations()) {

    $where = ($filter) ? "extended_user.userid = users.userid and $filter" :  "users.userid = extended_user.userid";

    } else {

    $where = "users.userid >0 and extended_user.userid = users.userid";
    $where .= " and users.approvalneeded not in ('required', 'rejected')";
    $where .= " and $filter" if ($filter);
    }

    if ($groups_filter && ($groups_filter ne '')) {
    $table_clause .= ", grpmembers, grp";
    $where .= " and grp.group_name like '%$groups_filter%' and grp.grp_id = grpmembers.grp_id and grpmembers.userid = users.userid";
    }

    my $order;

    if ( $FORM{order} ) {
    if ($FORM{order} eq "desc") {
        $order = " DESC";
    } else {
        $order = "ASC";
    }
    } else {
    $order = "ASC";
    }

    $FORM{sort_by} = 'last_name' unless $FORM{'sort_by'};

    my $first_name_pos = $self->_get_storage_position( 'first_name' );
    my $initial_pos = $self->_get_storage_position( 'initial' );
    my $last_name_pos = $self->_get_storage_position( 'last_name' );
    my $organization_pos = $self->_get_storage_position( $self->get_organization_field_name );

    my $main_clause = "distinct users.userid, users.fullname, extended_user.$first_name_pos, extended_user.$initial_pos, extended_user.$last_name_pos, users.nickname, users.email, users.max_total_upload_size, users.max_total_numberof_upload, extended_user.$organization_pos,users.lastonline";
    if( $FORM{sort_by} eq 'last_name' ) {

        ($c) = DBAccess->sqlSelectMany($main_clause, $table_clause, $where, "order by extended_user.$last_name_pos $order");

    } elsif( $FORM{sort_by} eq 'nickname' ) {

        ($c) = DBAccess->sqlSelectMany($main_clause, $table_clause, $where, "order by users.nickname $order");

    } elsif($FORM{sort_by} eq 'email' ) {

        ($c) = DBAccess->sqlSelectMany($main_clause, $table_clause, $where, "order by users.email $order");

    } elsif( $FORM{sort_by} eq 'first_name' ) {

        ($c) = DBAccess->sqlSelectMany($main_clause, $table_clause, $where, "order by extended_user.$first_name_pos $order");
    }

    return $c;
}

sub www_show_user_directory {
    my $self = shift;
    my %args = @_;
    $self->debug_msg(3, "Metadot::User::FlexUser::www_show_user_directory()");
    my $str = "";

    unless ($PARAMS->{show_user_directory}){
        $self->print_click_back("Error", "Sorry, you cannot view user details.<br /><br />");
        exit;
    }

    # XXX [claudio] $maxitem is not used
    my $thingtosearch = $args{search_words};
    my $iid           = $args{iid} || 0;
    my $object        = $args{object} || $self;

    if ($thingtosearch eq "") {
        $str = "<B>&nbsp; Please, enter a search word!<\/B>";
    } else {
        $str = $self->get_user_directory_results(@_);
        if ($str eq "") {
            $str= "&nbsp; No results matched your query.";
        }
    }

    # if we are in template view mode we call the template
    # get the template that is not the main template
    my $look = $PARAMS->{globallook};
    if ($look eq "usetemplate") {
        $object->print_http_header(); #CKIM: ensure http header is sent
        print  $DISPLAY->simple_expand($iid, $object, $object->show_template(-1), $str);
    } else {
        # normal view mode
        #print "normal view mode";
        $object->print_portal($str,"Search Result: $FORM{filter}",$iid);
    }
}

sub get_user_directory_results {
    my $self = shift;
    my %args = @_;
    my $search_words = $args{search_words};
    my $from         = $args{from};
    my $original_from = $from;
    my $nb           = $args{nb}  || 100;
    my $iid          = $args{iid} || 0;

    $self->debug_msg(3, "Metadot::User::FlexUser::get_user_directory_results()");
    my $filter = $self->get_search_where_for_all_user_fields(search_words => $search_words);
    $filter = "($filter)" if $filter ne '';
    ## get the user result set
    my $c = $self->get_show_users_groups_result_set( {filter=> $filter, startfrom=> $from, maxitem=>$nb } );
    #return $self->show_users_groups( {filter=> $filter, startfrom=>$from, maxitem=>$nb } );
    my $i=0;
    my $hidden_users_count;
    my $is_allowed_to_see_hidden_users = $USER->is_logged_in() && $ACCESS_BROKER->is_allowed_to_do('server_setup');
    my $s = '';
    my $user;
    while(( my ($uid, $fullname,
        $first_name,
        $initial,
        $last_name,
        $nickname,
        $email,
        $uploadsize,
        $uploadnumber,
                $organization_grp_id,
                $lastonline) = $c->fetchrow()) && ($i < $nb)) {
        # skip hidden users only when user doesn't have server_setup access
        unless( $is_allowed_to_see_hidden_users ){
            if ( $USER->is_hidden( $uid ) ) {
            $hidden_users_count++;
            next;
            }
        }
        if ($from) {$from--; next;} ## pass the one we've seen #PREVIOUS NEXT MAGIC

        $user =  Metadot::UserFactory->get_user($uid);
        #### CONTENT

        $s .= $user->get_user_details_view();
        $s .= "<hr size=0>\n";

        #### END CONTENT

        $i++;
     }

    #PREVIOUS NEXT MAGIC
    my $pn = "" ;

    my ($totalrows) = $c->rows;
    my $total_displayed = $totalrows - $hidden_users_count;

#    if ($total_displayed > $nb) {
        my $url = "index.pl?iid=$iid&amp;isa=Portal&amp;op=search&amp;filter=$FORM{filter}&amp;search_choice=$FORM{search_choice}";
    $pn = Components->get_previous_next_button($nb, $i, $total_displayed, $original_from, $url);
#    } else {
#   $pn="\n\n<center><table border=0 cellpadding=5 cellspacing=5 WIDTH='100%'><tr><td align='left'><font size=-1><I><B>Found $total_displayed entries</b></I></font></td></tr></table></center>";
#    }

    return $s ne '' ? "\n$pn\n$s\n$pn\n\n\n" : '';
}

sub get_user_details_view {
    my $self = shift;
    my %args = @_;

    my $view = $args{view} || 'brief';

    my $uid  = $args{uid}  || $self->uid();

    if ($view eq 'businesscard') { ## same
        $view = 'medium';
    }

    my $user = Metadot::UserFactory->get_user($uid);

    ## see if we should display editing actions
    my $action_view = $args{actionview} || 0;

    ## make sure that the current user has the rights to see
    ## the editing actions
    my $user_is_admin = $USER->is_admin();
    my $user_can_manage_registrations = $USER->can_manage_registrations();

    unless( $user_can_manage_registrations || $user_is_admin ){
       $action_view = 0;
    }

    ## see if we should print the 'accepting registration for... message
    my $pending_action = $FORM{'pending'} || 0;

    my $show_medium_details_link = "index.pl?isa=User&amp;op=show_user_details&amp;view=medium&amp;uid=" . $user->uid().
                                 "&amp;actionview=$action_view";
    my $show_full_details_link = "index.pl?isa=User&amp;op=show_user_details&amp;view=full&amp;uid=" . $user->uid() .
                                 "&amp;actionview=$action_view";
    my $show_image_details_link = "index.pl?isa=User&amp;op=show_user_details&amp;view=imagezoom&amp;uid=" . $user->uid() .
                                 "&amp;actionview=$action_view";

    my $full_width = $view eq 'imagezoom' ? 1 : 0;
    my $image_src = $user->get_image_src(full_width => $full_width);
    my $zoom_image = '';

    if ($view eq 'brief' || $view eq 'imagezoom') {
        $image_src = "<a href=\"$show_medium_details_link\">$image_src</a>";
    }
    $zoom_image = "<a href=\"$show_image_details_link\"><img src=\"" . $PARAMS->{'imageslocation'} . "/zoom.gif\" border = \"0\" valign=\"top\"></a>&nbsp;";

    my $subscription_link = (!$pending_action && $self->is_allowed_to_do('show_subscriptions')) ?
        "[ <a href='". $self->make_url("index.pl?isa=User&amp;op=show_subscriptions&amp;uid=$uid")
            . "'>Manage subscriptions</a> ]<br />"
        : '';

    my $become_user_link = (!$pending_action && $USER->uid != $uid && $self->can_become_user($uid)) ?
        "[ <a href='". $self->make_url("index.pl?isa=User&amp;op=become_user&amp;uid=$uid")
            . "'>Become this user</a> ]<br />"
        : '';

    my $edit_link = $self->is_allowed_to_do('modify') ?
        "[ <a href='". $self->make_url("index.pl?isa=User&op=modify&uid=$uid") . "'>Edit profile</a> ]<br />"
        : '';

    my $delete_link = (!$pending_action
                       && !$user->is_default_account_name( $user->fullname())
                       && $self->is_allowed_to_do('delete_request')) ?
        "[ <a href='". $self->make_url("index.pl?isa=User&op=delete_request&uid=$uid") . "'>Delete this user</a> ]<br />"
        : '';


    my $pending_link = ($pending_action && $self->is_allowed_to_do('accept_registration_confirm')) ?
        "[ <a href='" .$self->make_url("index.pl?isa=User&amp;op=accept_registration_confirm&amp;uid=$uid")
          . "'>Accept registration</a> ]<br />"
        : '';

    my $rejection_link = $pending_action && $self->is_allowed_to_do('reject_registration_confirm') ?
        "[ <a href='" .$self->make_url("index.pl?isa=User&amp;op=reject_registration_confirm&amp;uid=$uid")
            . "'>Reject registration</a> ]<br />"
        : '';

    my $admin_s = '';


    my $go_back_link = "<br />[ <a href='" . $self->make_url("index.pl?isa=User&amp;op=show_users_groups")
                     . "'>Go back to user management</a> ]";

    if( $action_view ){
       $admin_s .= "<table width='100%' cellspacing='2'>
                      <tr>
                        <td><div style='font-weight:bold'>Actions</td>
                      </tr>
                      <tr>
                        <td>$pending_link $rejection_link
                            $edit_link $delete_link $become_user_link
                            $subscription_link $go_back_link
                        </td>
                      </tr>
                    </table>";
    }
    my $s = '';

    my $user_properties_hash = $self->_generate_user_properties_hash($user);

    if ($view eq 'brief') {
        $s .= "<table border=\"0\" cellspacing=\"2\" cellpadding=\"2\">\n";
        $s .= "  <tr><td valign=\"top\">$image_src</td>\n";
        $s .= "      <td valign=\"top\">";
        $s .= "<b><a href=\"$show_medium_details_link\">" . $user->get_value('fullname') . "</a></b>";
        $s .= ", " . $self->output_user_property($user_properties_hash, 'job_title', {value_only=>1});
        #$s .= "  <font class=\"smaller\">";
        if ($PARAMS->{trusted_site}) {
            $s .= "  <br /> <a href=\"mailto:" . $user->get_value('email') . "\">" . $self->output_user_property($user_properties_hash, 'email', {value_only=>1})  . "</a>";
        }
        $s .= $self->output_user_property($user_properties_hash, 'telephone');
        $s .= $self->output_user_property($user_properties_hash, 'mobile_phone');

        $s .= "  <br /><br /><a class=\"smaller\" href=\"$show_medium_details_link\">business card</a>";
        if ($PARAMS->{my_website_creation}) {
            my $portal_id = $user->has_portal();
            $s .= "<font class=\"smaller\">,</font> <a class=\"smaller\" href=\"" .$user->make_url("index.pl?iid=$portal_id") . "\">my website</a>\n" if ($portal_id);
        }

        my $landing_url = $FORM{landing_url} || $SESSION->generate_referer();

        my $metagram_center = Metadot::SystemApp::MetagramCenter->new();
        if (defined $metagram_center && $metagram_center->is_allowed_to_do('compose')) {
            $DISPLAY->debug_msg(5, "landing_url is '$landing_url'");
            $s .= "<font class=\"smaller\">,</font> <a class=\"smaller\" href=\"" .$DISPLAY->escapeHREF($DISPLAY->make_url("index.pl?isa=Metadot::SystemApp::MetagramCenter&op=compose&recipient_uid=$uid&landing_url=".$CGI->escape($landing_url))) . "\">send me a metagram</a>\n";
        }
        #$s .= "</font>";
        $s .= "      </td>\n";
        $s .= "  </tr>\n";
        $s .= "</table>\n";

    } else {
        if( $action_view ){

            if( $pending_action ){
               $s .= "<h2>Accepting/Rejecting user registration for &lt;".$user->get_value('fullname')."&gt;</h2>";
            }
            $s .= "<div style=\"width:515px;padding:0;margin:4em;\">\n";
            $s .= "<table border=0 cellspacing=0 cellpadding=2 width='100%' style='border: thin solid black'>\n";
        } else{
            $s .= "<div style=\"width:300px;padding:2em;margin:4em;\">\n";
            $s .= "<table border=\"0\" cellspacing=\"2\" cellpadding=\"2\" width=\"300\" style='border: thin solid black'>\n";
        }

        $s .= "  <tr><td nowrap='nowrap' valign=\"top\" align=\"center\">\n";
        $s .= "        $image_src\n";
        if ($view ne 'imagezoom') {
            $s .= "<br />\n";
            $s .= $zoom_image;
            if ($PARAMS->{my_website_creation}) {
                my $portal_id = $user->has_portal();
                $s .= "<a style='text-decoration: underline' href=\"" .$user->make_url("index.pl?iid=$portal_id") . "\">my website</a>&nbsp;\n" if ($portal_id);
            }
            $s .= "      </td>\n";
            $s .= "      <td nowrap='nowrap' valign=\"top\">";
            $s .= $self->output_user_property($user_properties_hash, 'fullname');
            $s .= "<b>" . $user->get_value('fullname') . "</b>";

            if ($PARAMS->{trusted_site}) {
            $s .= "<br /><a href=\"mailto:" . $user->get_value('email') . "\">" . $user->get_value('email')  . "</a>";
            }

            $s .= $self->output_user_property($user_properties_hash, 'job_title');
            #$s .= "<br />" . $user->get_value('job_title') if $user->get_value('job_title');
            $s .= "<p /><b>Address</b><br/>";
            $s .= $self->output_user_address($user_properties_hash);

            $s .= $self->output_user_property($user_properties_hash, 'telephone') if $user->get_value('telephone');
            $s .= $self->output_user_property($user_properties_hash, 'mobile_phone') if $user->get_value('mobile_phone');
            $s .= $self->output_user_property($user_properties_hash, 'fax') if $user->get_value('fax');
        }

        if ($view eq 'full') {
            $s .= $self->output_user_property($user_properties_hash, 'home_phone') if $user->get_value('home_phone');
            $s .= $self->output_user_property($user_properties_hash, 'home_fax') if $user->get_value('home_fax');
            $s .= $self->output_user_property($user_properties_hash, 'home_address') if $user->get_value('home_address');
            $s .= $self->output_user_property($user_properties_hash, 'pager_phone') if $user->get_value('pager_phone');
            $s .= $self->output_user_property($user_properties_hash, 'pager_number') if $user->get_value('pager_number');
            $s .= $self->output_user_property($user_properties_hash, 'icq_number') if $user->get_value('icq_number');
            $s .= $self->output_user_property($user_properties_hash, 'nickname') if $user->get_value('nickname');
            $s .= $self->output_user_property($user_properties_hash, 'country') if $user->get_value('country');
            $s .= $self->output_user_property($user_properties_hash, 'homepage') if $user->get_value('homepage');
            $s .= $self->output_user_property($user_properties_hash, 'schedule') if $user->get_value('schedule');
            $s .= $self->output_user_property($user_properties_hash, 'notes') if $user->get_value('notes');
            $s .= $self->output_user_property($user_properties_hash, 'quotes') if $user->get_value('quotes');

            $s .= $self->output_remaining_properties($user_properties_hash,$user);

            $s .= "<p /><a style='text-decoration: underline' href=\"$show_medium_details_link\">Less detail</a>";
        }

        if ($view eq 'medium') {
            $s .= "<p /><a href=\"$show_full_details_link\" style='text-decoration: underline'>More detail</a>";
        }
        ## if the actionview parameter was passed, then display the
        ## options to edit/delete/become user/manage subscriptions
        if( $action_view ){
            $s .= "<td nowrap valign='top'>$admin_s</td>";
        }
        $s .= "      </td>\n";


        if( $user_is_admin && $action_view ){
           ## get groups/subscriptions/metagrams information for userid = $uid
           my $group_list = Group->get_group_names_where_member($uid);
           my @user_groups_sort = sort(@{$group_list});
           my $groups = '';
           foreach my $g(@user_groups_sort) {
              ## get the id of this group
              my ($gid) = DBAccess->sqlSelect("grp_id","grp","group_name='$g'");
              $groups .= "<li><a href='".
                          $self->make_url("index.pl?isa=Group&amp;op=show_group_members&amp;grp_id=$gid") .
                          "'>$g</a></li>";
           }
           #$groups = join("<li>",@user_groups_sort);
           if( $groups eq '' ){
              $groups = '<li>This user is not a member of any group</li>';
           }

           my ($s_list) = DBAccess->sqlSelectMany("distinct name,channel.iid,homeurl",
                            "channel, subscription",
                            "channel.cid = subscription.cid AND
                            userid = $uid");
           my $subscription_list = '';
           my $subscription_link;

           while( my ($channel,$channel_id,$homeurl) = $s_list->fetchrow() ){
              if( $channel_id ){
                  $subscription_link = "<a href='" . $self->make_url("index.pl?id=$channel_id&amp;isa=Discussion&amp;op=show") .
                                        "'>$channel</a>";
              } else {

                  $subscription_link = "<a href='$homeurl' target='_blank'>$channel</a>";
              }

              $subscription_list .= "<li>$subscription_link</li>";

           }
           if( $subscription_list eq '' ){
               $subscription_list = '<li>This user is not subscribed to any discussion</li>';
           }

           my $metagram_link = $self->make_url("index.pl?isa=User&op=show_user_details&uid=$uid&view=medium&actionview=1&amp;metagram_filter=all");
           my $metagram_link_text = 'View all';

           my $extra_where = q~and status = 'UNREAD'~;
           if( $FORM{metagram_filter} eq 'all'){
              $extra_where = '';
              $metagram_link = $self->make_url("index.pl?isa=User&op=show_user_details&uid=$uid&view=medium&actionview=1");
              $metagram_link_text = 'View unread';
           }
           my ($metagram_list) = DBAccess->sqlSelectMany(
                      "name,description,authorname,authoremail,creationdate",
                      "message","recipient_uid = $uid $extra_where");

           my $metagrams = "<table width='100%' class='report'>
                            <tr>
                               <td class='header'>Author</td>
                               <td class='header'>Created on</td>
                               <td class='header'>Title</td>
                               <td class='header'>Message</td>
                             </tr>";

           my $metagram_counter = undef;
           my $max_len = 35;

           while( my ($m_name,$m_desc,$m_author,$m_email,$m_created) =
                  $metagram_list->fetchrow() ){

              my $m_length = length $m_desc;

              $m_desc = Utility->trim_clean($m_desc,$max_len);
              if( $m_length >= $max_len ){
                 $m_desc .= '...';
              }

              $metagram_counter++;
              $metagrams .= "<tr>
                               <td class='body'><a href='mailto:$m_email'>$m_author</a></td>
                               <td class='body'>$m_created</td>
                               <td class='body'>$m_name</td>
                               <td class='body'><i>$m_desc</i></td>
                             </tr>";
           }

           $metagrams .= '</table>';

           unless( defined($metagram_counter) ){
              $metagrams = '<ul><li>No metagrams found for this user</li></ul>';
           }





           $s .= "<tr bgcolor='#ffffff'>
                   <td colspan='2'><span style='padding-left:2px; font-size:12px; font-weight:bold'>Groups</span></td>
                   <td align='right'><a href='". $self->make_url("index.pl?isa=Group&amp;op=show_groups&amp;uid=$uid") .
                                     "' class='text_button'>Change</a>
                  </tr>
                  <tr>
                   <td colspan='3'><ul>$groups</ul></td>
                  </tr>
                  <tr bgcolor='#ffffff'>
                   <td colspan='3' height='32'><span style='padding-left:2px; font-size:12px; font-weight:bold';'>Subscriptions</span></td>
                  </tr>
                  <tr>
                   <td colspan='3'><ul>$subscription_list</ul></td>
                  </tr>
                  <tr bgcolor='#ffffff'>
                   <td colspan='2'><span style='padding-left:2px; font-size:12px; font-weight:bold'>Metagrams</span></td>
                   <td align='right'><a class='text_button' href='$metagram_link'>$metagram_link_text</a></td>
                  </tr>
                  <tr>
                   <td colspan='3'>$metagrams</td>
                  </tr>";
        }
        $s .= "  </tr>\n";
        $s .= "</table>\n";
        $s .= "</div>\n";
    }





    return $s;
}



# used when displaying the user profile
sub _generate_user_properties_hash {
    my $self = shift;
    my $user = shift || $USER;
    my %output = ();

    my $fields = $self->get_fields_config()->{FIELDS};
    foreach my $field (@$fields) {
        my $field_name = $field->{field_name};

        next if ($field_name eq 'password');

        $output{$field_name} = {
            description => $field->{description},
            value       => $user->get_value($field_name),
        }
    }

    return \%output;
}


sub output_user_property {
    my $self = shift;
    my $properties_hash = shift;
    my $field_name = shift;
    my $options = shift || {};

    my $s;
    if (exists $properties_hash->{$field_name}) {
        my $entry = $properties_hash->{$field_name};
        if (
            !$entry->{hidden} &&
            (!$entry->{visited}++ || $options->{revisit})
            ) {
            my $description = $entry->{description};
            my $value       = $entry->{value};
            if ($options->{value_only}) {
                $s = $value;
            } else {
                $s = qq~<div><b>$description:</b>&nbsp;~;

                if (defined $value) {
                    $s .= $value;
                }
                $s .= "</div>\n";
            }
        }
    }

    return $s;
}

sub output_user_address {
    my $self = shift;
    my $properties_hash = shift;

    my $s = '';

    $s .= $self->output_user_property($properties_hash, 'address_line_1', {value_only=>1});
    $s .= "<br />\n";
    $s .= $self->output_user_property($properties_hash, 'address_line_2', {value_only=>1});
    $s .= "<br />\n";
    $s .= $self->output_user_property($properties_hash, 'city', {value_only=>1});
    $s .= ", \n";
    $s .= $self->output_user_property($properties_hash, 'state', {value_only=>1});
    $s .= " ";
    $s .= $self->output_user_property($properties_hash, 'zip_code', {value_only=>1});

    return $s;
}


sub output_remaining_properties {
    my $self = shift;
    my $properties_hash = shift;
    my $user = shift;

    my $fields = $self->get_fields_config()->{FIELDS};
    my $s = '';

    foreach my $field (@$fields) {
        my $field_name = $field->{field_name};
        my $entry = $properties_hash->{$field_name};
                $DISPLAY->debug_msg(5,"fieldName[$field_name] value[".$user->get_value($field_name)."]");
        if (defined $entry
            && $entry->{visible}
            && !$entry->{visited}++
            ) {
            #if( $user->get_value($field_name) ){
            $s .= $self->output_user_property($properties_hash, $field_name) if $user->get_value($field_name);
        }
    }

    return $s;
}






sub www_show_user_details {
    my $self = shift;
    $self->debug_msg(3, "Metadot::User::FlexUser::www_show_user_details()");

    my $can_manage_registrations = $ACCESS_BROKER->is_allowed_to_do('manage_registrations');
    unless ($PARAMS->{show_user_directory}){
        if(!($can_manage_registrations || $USER->is_admin()) && defined($FORM{actionview}) ){
           $self->print_click_back("Error", "Sorry, you cannot view user details.<br /><br />");
           exit;
        }
    }

    my $s = $self->get_user_details_view(uid => $FORM{uid}, view => $FORM{view},
                                         actionview => $FORM{actionview});
    my $look = $PARAMS->{globallook};

    # we always use this mode now. It works in 'usetemplate' mode even.
    $DISPLAY->print_simple_layout({content=>$s,title=>"User Information"});

    #if ($look eq "usetemplate") {
    #    print  $DISPLAY->simple_expand(0, $self, $self->show_template(-1), $s);
    #} else {
        # this is compatible with the new template mode
    #    $DISPLAY->print_simple_layout({content=>$s,title=>"User Information"});
    #}
}

sub get_modify_settings_form_field_values {
    my $self = shift;
    die "method get_modify_settings_form_field_values should always be overidden in default classes: $!";
}



my $fields_to_storage;

sub get_fields_to_storage_hash {
    my $self = shift;
    unless ( defined $fields_to_storage ) {

    my $config = $self->get_fields_config();

    foreach my $field ( @{ $config->{FIELDS} } ) {
        if( lc $field->{storage_type} eq 'secondary' ) {

        my $field_name = $field->{field_name};
        my $position = $field->{store_at_column};

        $fields_to_storage->{ $field_name } = $position;

        }
    }
    }

    return $fields_to_storage;
}



##################################################################

=head2 $USER->www_accept_registration()

Admins accepts a user registration. Returns 1.

=cut

##################################################################

sub www_accept_registration {
    my $self = shift;

    my $uid;
    if( !$FORM{uid} ){
        $self->print_click_back("Error",
                                "Sorry I didn't get a user uid to accept!");
        return 0;
    }

    $uid = $FORM{uid};

    my $url = "index.pl?isa=User&amp;op=show_user_details&amp;uid=$uid";
    $url .= "&amp;view=medium&amp;actionview=1&amp;pending=1";
    $self->debug_msg("url[$url]");
    my $basename = $PARAMS->{basename};
    $self->md_redirect("$basename/$url");

}


##################################################################

=head2 $USER->www_reject_registration()

Admins rejects a user registration. Returns 1.

=cut

##################################################################
sub www_reject_registration {
    my $self = shift;

    return $self->www_accept_registration();
}


sub _get_storage_position {
    my $self = shift;
    my $field_name = shift;

    my $fts = $self->get_fields_to_storage_hash();
    return $fts->{$field_name};
}

sub get_search_where_for_all_user_fields {
    my $self = shift;
    my %args = @_;

    my $search_words = $args{search_words};
    return '' unless $search_words;
    $search_words = uc($search_words);
    $search_words = DBAccess->quote("%$search_words%");

    my $fts = $self->get_fields_to_storage_hash();

    return "upper(users.nickname) like $search_words or upper(users.email) like $search_words or " . join(' or ', map("upper(extended_user.$_) like $search_words", values(%$fts)));
}

sub get_organizations {
    my $self = shift;

    my $groups = Group->get_organization_groups();

    my @groups_array = ( { form_value => '', label => '- none -'} );

    foreach my $grp_id ( sort {$groups->{$a} cmp $groups->{$b}} keys (%$groups) ) {
        push @groups_array, { form_value => $grp_id, label => $groups->{$grp_id} };
    }
    return \@groups_array;
}

sub get_homepage_values {
    my $self = shift;

    my $field_name = $self->get_flex_homepage_field_name();
    my $value = $self->get_value_raw($field_name);

    my @homepages= ();
    my $radio_index = 0;
    if ($PARAMS->{allow_organization_registration}) {
        push @homepages, { form_value => $ORG_KEY, label => 'My Organization Homepage' };
        $radio_index++;
    }
    my $referer = $SESSION->referer() || '';
    my $referer_label = '';
    if (length($referer) > 70) {
        $referer_label = substr($referer, 0, 70) . "...";
    } else {
        $referer_label = $referer;
    }
    push @homepages, { form_value => $referer, label => "Use recent: $referer_label" };
    $radio_index++;

    my $url = '';
    if (defined($value) && $value ne '' && $value ne $ORG_KEY) {##then a url
        $url = $value;
    }

    # unless a default value is specified, both the "default" and "url" options will be checked.
    $url ||= "Type in a URL";
    my $url_text_box  = <<EOM
    <input type="text" name="__url_text__" value="$url" onBlur="this.form.$field_name\[$radio_index\].value=this.value">
EOM
;
    push @homepages, { form_value => $url, label => "Use URL $url_text_box" };
    $radio_index++;

    push @homepages, { form_value => '', label => "Portal Default" };
    $radio_index++;

    return \@homepages
}

sub get_user_defined_homepage {
    my $self = shift;

    my $field_name = $self->get_flex_homepage_field_name();
    my $value = $self->get_value_raw($field_name);

    Metadotclass->debug_msg(5, "get_user_defined_homepage starts with value of '$value'");

    my $url = undef;
    #print STDERR "\n\nvalue:$value\n\n";
    if ($value eq $ORG_KEY) {
        Metadotclass->debug_msg(5, "get_user_defined_homepage has detected that this is the organization homepage");
        if ($PARAMS->{allow_organization_registration}) {
            my $organization_field_name = $self->get_organization_field_name();
            my $organization_grp_id = $self->get_value_raw($organization_field_name);
            #print STDERR "\n\norganization_grp_id:$organization_grp_id\n\n";
            Metadotclass->debug_msg(5, "get_user_defined_homepage found user's organization as '$organization_grp_id', using property '$organization_field_name'");
            if ($organization_grp_id) {
                my $group = Group->new($organization_grp_id);
                $url = $group->{organization_homepage};
                #print STDERR "\n\ngroup url:$url\n\n";
                Metadotclass->debug_msg(5, "get_user_defined_homepage uses organization url of '$url'");
            }
        }
    } elsif ($value ne '') {
        $url = $value;
    }
    #print STDERR "\n\nurl:$url\n\n";

    Metadotclass->debug_msg(5, "get_user_defined_homepage returns '$url'");
    return $url;
}

## returns user's organization name
sub get_organization_name {
    my $self = shift;
    my $organization_grp_id = $self->get_value_raw($self->get_organization_field_name());
    $DISPLAY->debug_msg(5, "get_organization_name has '$organization_grp_id'");
    return '' unless $organization_grp_id;
    my ($name, $is_organization) = Group->get_organization_info($organization_grp_id);
    #print STDERR "\n\nname:'$name' is_organization:'$is_organization'\n\n";

    return $is_organization ? $name : '';
}

sub add_user_to_primary_organization {
    my $self = shift;
    my $uid  = $self->uid();
    my $organization_grp_id = $self->get_value_raw($self->get_organization_field_name());

    $DISPLAY->debug_msg(4, "add_user_to_primary_organization begins");

    return unless (defined($uid) && $uid ne '' && $uid > 0 && defined($organization_grp_id) && $organization_grp_id ne '' && $organization_grp_id > 0 );

    ## organization registration must be turned on
    return unless ($PARAMS->{allow_organization_registration});
    ## must be accepted
    return unless (!$self->{approval_needed}) || ($self->{approval_needed} eq "accepted");
    ## group must be an organization
    return unless Group->is_group_an_organization($organization_grp_id);

    if (!DBAccess->sqlSelect ("grp_id", "grpmembers", "grp_id=$organization_grp_id and userid='$uid'")) {
        Group->_add_a_member_to_a_group($organization_grp_id, $uid);

        # check if this group has a post-registration page defined
        my $grp = Group->new($organization_grp_id);
        my $post_registration_page = $grp->organization_post_registration();
        if ($post_registration_page) {
            $DISPLAY->debug_msg(4, "_process_add_groups sets the landing page to $post_registration_page");
            my $register = Metadot::RegisterFactory->get_register();
            $register->set_post_registration_page($post_registration_page);
        }
    }
}

## removes user's primary organization choice if it
## matches grp_id
sub remove_user_primary_organization_choice {
    my $self = shift;
    my $grp_id = shift;

    my $organization_grp_id = $self->get_value_raw($self->get_organization_field_name());
    #print STDERR "\n\ngrp_id:$grp_id o_grp_id:$organization_grp_id\n\n";
    if ($grp_id && $organization_grp_id && $grp_id == $organization_grp_id) {
        $self->set_value($self->get_organization_field_name(), '');
        $self->save();
    }
}

sub get_organization_field_name {
    my $self = shift;
    return "organization_grp_id";
}

sub get_flex_homepage_field_name {
    my $self = shift;

    return "flex_homepage";
}

sub get_upload_filename_icon_and_url {
    my $self = shift;
    my %args = @_;

    my $field_name = $args{field_name};
    my $uid        = $args{uid} || $self->uid();
    return $self->SUPER::get_upload_filename_icon_and_url( {
        id => $uid,
        isa => $self->is_a(),
        field_name => $field_name,
    });
}

sub get_picture_field_name {
    my $self = shift;
    return 'picture';
}

sub get_image_src {
    my $self = shift;
    my %args = @_;

    my $uid        = $args{uid} || $self->uid();
    if (defined $uid) {
        my $full_width     = $args{full_width} || 0;
        my $null_if_empty  = $args{null_if_empty};
        my $file_url   = $self->get_image_url(%args);

        # CKIM: REFACTORED THIS to get_image_url

        my $default_width = 68;
        my $width = $full_width ? '' : qq~width="$default_width"~;

        # use the null image if there was none defined
        unless ($file_url || $null_if_empty) {
            $file_url = $self->get_null_image_url;
            $width = $default_width;
        }

        # null image is handled earlier
        return "<img src=\"$file_url\" border='0' $width alt='User photograph'/>";
    }
}

sub no_image_src {
    my $class = shift;
    return '<img src="' . $class->get_null_image_url() . '" border=0 width=68>';
}


sub get_image_url {
    my $self = shift;
    my %args = @_;

    my $field_name = $args{field_name} || $self->get_picture_field_name();
    my $uid        = $args{uid} || $self->uid();

    my $result;

    if ($uid) {
        my ( $file, $file_icon, $file_url ) = $self->get_upload_filename_icon_and_url(field_name => $field_name, uid => $uid);
        $result = $file_url;
    }

    return $result;
}


sub get_null_image_url {
    return $PARAMS->{'imageslocation'} . '/nopic.jpg';
}


sub www_download_file {
    my $self = shift;

### mike: not restricting to user directory
#    return '' unless $PARAMS->{show_user_directory};

    my $field_name = $FORM{field_name};
    my $config = $self->get_config_for_field($field_name);
    my $field_type = $config->{field_type};
    return $self->SUPER::www_download_file( {
        id => $self->uid(),
        isa => $self->is_a(),
        field_name => $field_name,
        type => $field_type,
    });
}


sub www_export_csv
{
    my $self = shift;

    my $fields = $self->get_fields_config()->{FIELDS};

    if (defined $fields)
    {
        # set up the header
        my $filename='export.csv';  # this file doesn't really exist
        my %header_values = (
                             '-Connection'          => 'close',
                             '-Accept-ranges'       => 'bytes',
                             '-Content-location'    => $filename,
                             '-Content-disposition' => "inline; filename=$filename",
                             'type'                 => 'text/csv',
        );

        print $CGI->header(%header_values);

        my $csv = Text::CSV_XS->new({binary=>1});

        # first export the descriptions
        my @field_descriptions;
        my @field_names;
        foreach my $field (@$fields)
        {
            my $field_name = $field->{field_name};
            unless ($field_name eq 'password')
            {
                push @field_descriptions, $field->{description};
                push @field_names, $field_name;
            }
        }

        $csv->print(\*STDOUT, \@field_descriptions);
        print "\n";

        # export the data
        my $sth = DBAccess->sqlSelectMany ("userid", "users");
        while (my ($uid) = $sth->fetchrow())
        {
            my $user_obj = Metadot::UserFactory->get_user($uid);
            my @data = ();
            foreach my $field_name (@field_names)
            {
               push @data, $user_obj->get_value($field_name);
            }
            $csv->print(\*STDOUT, \@data);
            print "\n";
        }
        $sth->finish();
    }
    else
    {
        $DISPLAY->print_click_back('Error', 'Sorry, I was unable to export the users.');
    }
    exit;
}

sub has_unread_metagrams {
    my $self = shift;

    return unless $USER->is_logged_in();

    return Metadot::Metagram->has_unread_metagrams({ uid => $USER->uid() });
}


sub can_manage_registrations {
    my $self = shift;
    my $user = shift || $self;

    my $result = $ACCESS_BROKER->is_allowed_to_do('manage_registrations', $user);
    return $ACCESS_BROKER->is_allowed_to_do('manage_registrations', $user);
}



use constant FLATTEN_ARRAY_SEPARATOR => '~!A!~';

# convert arrays to scalars which can then be reconstituted with
# _unflatten_value. Currently, this only handles arrays, and does not
# descend down a data hierarchy
sub _flatten_values
{
    my $class   = shift;
    my $values  = shift || {};

    while (my ($key, $value) = each %$values)
    {
        if (UNIVERSAL::isa($value, 'ARRAY'))
        {
            my $flat = join( FLATTEN_ARRAY_SEPARATOR, @$value);
            $DISPLAY->debug_msg(5, "Flattening $value TO $flat");
            $values->{$key} = $flat;
        }
    }
}


sub _unflatten_value
{
    my $class   = shift;
    my $value   = shift;

    my $result = undef;

    my $separator = FLATTEN_ARRAY_SEPARATOR();
    if (defined $value && ($value =~ m/\Q$separator\E/))
    {
        my @restored = split( FLATTEN_ARRAY_SEPARATOR, $value);
        $DISPLAY->debug_msg(5, "_unflatten_value $value TO @restored");
        $result = \@restored;
    }
    else
    {
        $result = $value;
    }

    return $result;
}





1;

__END__

CREATE TABLE extended_user (
  uid varchar(20) NOT NULL,
  f1 varchar(255),
  f2 varchar(255),
  f3 varchar(255),
  f4 varchar(255),
  f5 varchar(255),
  f6 varchar(255),
  f7 varchar(255),
  f8 varchar(255),
  f9 varchar(255),
  f10 varchar(255),
  f11 varchar(255),
  f12 varchar(255),
  f13 varchar(255),
  f14 varchar(255),
  f15 varchar(255),
  f16 varchar(255),
  f17 varchar(255),
  f18 varchar(255),
  f19 varchar(255),
  f20 varchar(255),
  t1 text,
  t2 text,
  t3 text,
  t4 text,
  t5 text,
  t6 text,
  t7 text,
  t8 text,
  t9 text,
  t10 text,
  PRIMARY KEY (uid)
);

