###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::User::ManUser;
use strict;

use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $ACCESS_BROKER $CGI);

use Metadot::User::FlexUser;
use Metadot::Metagram;

use vars qw(@ISA);
use Group;

@ISA=qw(Metadot::User::FlexUser Metadot::UploadsManager);


my $AC = Metadot::User::Default->get_AC();

push( @{ $AC->{BUNDLES}{ADMIN}{OPS} }, 'show_users_groups' );
push( @{ $AC->{BUNDLES}{MOD}{OPS} }, 'confirm_upload_deletion' );
push( @{ $AC->{BUNDLES}{MOD}{OPS} }, 'delete_upload' );
push( @{ $AC->{BUNDLES}{SEARCH}{OPS} }, 'download_file' );
push( @{ $AC->{BUNDLES}{SEARCH}{OPS} }, 'show_user_directory' );
push( @{ $AC->{BUNDLES}{SEARCH}{OPS} }, 'show_user_details' );

sub get_AC {
    return $AC;
}

my $ORG_KEY = '__ORG__';

my $FIELDS_CONFIG = {
             DISPLAY_SETS => [ 'Login Information', 'My start page after login', 'Personal Information' ],
             FIELDS => [
                {
                 field_name => 'first_name',
                 description => 'First Name',
                 max_length => 64,
                 size => 20,
                 is_required => 1,
                 display_set => 0,
                 storage_type => 'secondary',
                 store_at_column => "f1",
                 verify_type => 'non_empty',
                },
                {
                 field_name => 'initial',
                 description => 'Middle Initial',
                 max_length => 1,
                 size => 1,
                 is_required => 0,
                 is_not_required_for_default_users => 1,
                 display_set => 0,
                 storage_type => 'secondary',
                 store_at_column => "f2",
                },
                {
                 field_name => 'last_name',
                 description => 'Last Name / Family Name',
                 max_length => 64,
                 size => 20,
                 is_required => 1,
                 is_not_required_for_default_users => 1,
                 display_set => 0,
                 storage_type => 'secondary',
                 store_at_column => "f3",
                },
                {
                 field_name => 'nickname',
                 description => 'Nickname / Screename',
                 max_length => 64,
                 size => 20,
                 is_required => 0,
                 display_set => 0,
                 storage_type => 'primary',
                },
                {
                 field_name => &get_organization_field_name,
                 description => 'Organization',
                 field_type => 'select',
                 is_required => 0,
                 display_set => 0,
                 #field_values => &get_organizations(),
                 field_values_function => 'get_organizations',
                 storage_type => 'secondary',
                 store_at_column => "f16",
                 enable_param => "allow_organization_registration",
                 only_admin_can_edit => 1, ## currently does not apply to all field types
                },
                {
                 field_name => &get_flex_homepage_field_name(),
                 description => '&nbsp;',
                 field_type => 'radio',
                 is_required => 0,
                 display_set => 1,
                 field_values_function => 'get_homepage_values',
                 storage_type => 'secondary',
                 store_at_column => "f17",
                 use_line_breaks => 1,
                },
                {
                 field_name => 'email',
                 description => 'E-mail Address',
                 max_length => 64,
                 size => 20,
                 is_required => 1,
                 display_set => 0,
                 storage_type => 'primary',
                },
                {
                 field_name => 'password',
                 description => 'Password',
                 max_length => 64,
                 size => 20,
                 is_required => 1,
                 display_set => 0,
                 storage_type => 'primary',
                 field_type => 'password',
                },
                {
                 field_name => &get_picture_field_name(),
                 description => 'Picture',
                 is_required => 0,
                 display_set => 2,
                 storage_type => Metadot::UploadsManager::UPLOAD_TYPE_USER(),
                                 field_type => Metadot::UploadsManager::UPLOAD_TYPE_USER(),
                 #enable_param => 'show_user_directory',
                },
                {
                 field_name => 'address_line_1',
                 description => 'Address Line 1',
                 max_length => 64,
                 size => 36,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f4",
                },
                {
                 field_name => 'address_line_2',
                 description => 'Address Line 2',
                 max_length => 64,
                 size => 36,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f5",
                },
                {
                 field_name => 'city',
                 description => 'City',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f6",

                },
                {
                 field_name => 'state',
                 description => 'State',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f7",
                },
                {
                 field_name => 'zip_code',
                 description => 'Zip Code',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f8",
                },

                {
                 field_name => 'country',
                 description => 'Country',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f9",
                },

                {
                 field_name => 'telephone',
                 description => 'Telephone',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f10",
                },

                {
                 field_name => 'fax',
                 description => 'Fax',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f11",
                },

                {
                 field_name => 'mobile_phone',
                 description => 'Mobile Phone',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f12",
                },

                {
                 field_name => 'pager_phone',
                 description => 'Pager Phone',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f13",
                },

                {
                 field_name => 'pager_number',
                 description => 'Pager Address',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'primary',
                },

                {
                 field_name => 'icq_number',
                 description => 'ICQ Number',
                 max_length => 64,
                 size => 20,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'primary',
                },

                {
                 field_name => 'homepage',
                 description => 'Homepage URL',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'primary',
                },

                {
                 field_name => 'institution',
                 description => 'Institution',
                                 max_length => 64,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "t1",
                },

                {
                 field_name => 'status',
                 description => 'Status',
                                 max_length => 64,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "t2",
                },

                                {
                                 field_name => 'seclevel',
                                 description => 'Security Level',
                                 max_length => 64,
                                 is_required => 0,
                                 display_set => 2,
                                 storage_type => 'secondary',
                                 store_at_column => "t2",
                                },

                {
                 field_name => 'wtcsmember',
                 description => 'WTCS Member',
                                 max_length => 64,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "t4",
                },

                {
                 field_name => 'home_telephone',
                 description => 'Home Telephone',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f14",
                },

                {
                 field_name => 'home_fax',
                 description => 'Home Fax',
                 max_length => 64,
                 size => 30,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "f15",
                },

                {
                 field_name => 'home_address',
                 description => 'Home Address',
                 field_type => 'textarea',
                 rows => 3,
                 cols => 30,
                 wrap => 1,
                 is_required => 0,
                 display_set => 2,
                 storage_type => 'secondary',
                 store_at_column => "t5",
                },
                   ],
            };


my $check_for_required_fields_done;

## This method must be overwritten in child classes. See Sample below.
## Sample:
### must override SUPER because $FIELDS_CONFIG is a class variable
sub get_fields_config {
    my $self = shift;
    return $self->SUPER::get_fields_config($FIELDS_CONFIG);
}

sub get_organization_field_name {
    my $self = shift;
    return "organization_grp_id";
}

sub get_flex_homepage_field_name {
    my $self = shift;

    return "flex_homepage";
}

sub get_picture_field_name {
    my $self = shift;
    return 'picture';
}

sub new {
    my $class = shift;

    my $self = $class->SUPER::new(@_);

    $self->initialize(@_);
    return $self;
}

1;

__END__

CREATE TABLE extended_user (
  uid varchar(20) NOT NULL,
  f1 varchar(255),
  f2 varchar(255),
  f3 varchar(255),
  f4 varchar(255),
  f5 varchar(255),
  f6 varchar(255),
  f7 varchar(255),
  f8 varchar(255),
  f9 varchar(255),
  f10 varchar(255),
  f11 varchar(255),
  f12 varchar(255),
  f13 varchar(255),
  f14 varchar(255),
  f15 varchar(255),
  f16 varchar(255),
  f17 varchar(255),
  f18 varchar(255),
  f19 varchar(255),
  f20 varchar(255),
  t1 text,
  t2 text,
  t3 text,
  t4 text,
  t5 text,
  t6 text,
  t7 text,
  t8 text,
  t9 text,
  t10 text,
  PRIMARY KEY (uid)
);
