###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name$
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::User::ManagedLDAPUser;
use strict;

use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $ACCESS_BROKER $CGI);

use Metadot::User::Default;
use Metadot::RegisterFactory;
use Metadot::LDAPSubsystemFactory;

use vars qw(@ISA);

@ISA=qw(Metadot::User::Default);


my $AC = Metadot::User::Default->get_AC();

sub get_AC {
    return $AC;
}

sub new {
    my $class = shift;

    my $self = $class->SUPER::new(@_);

    return $self;
}

sub load_ldap_values {
    my $self = shift;
    my %args = @_;
    my $uid = $self->uid;

    $self->debug_msg(3, "Metadot::User::ManagedLDAPUser::load_ldap_values() with uid:$uid");

    if(defined ($uid)){
        my $ldap = Metadot::LDAPSubsystemFactory->get_ldap_subsystem();
        $ldap->initialize();
        my $ldap_entry = $ldap->_get_ldap_entry($self->{ldap_dn});
        $self->debug_msg(4, "Metadot::User::ManagedLDAPUser::load_ldap_values() ldapdn:" . $self->{ldapdn} . " defined_ldap_entry:" . defined($ldap_entry));
        return 0 unless $ldap_entry;

        ## add true dn
        $self->{true_ldap_dn} = $ldap_entry->dn;

        my $registrator = Metadot::RegisterFactory->get_register();
        my $main_values = $registrator->get_ldap_field_values(ldap_entry => $ldap_entry);
        my $metadot_field    = '';
        my $ldap_field       = '';
        my $ldap_field_value = '';
        foreach my $value (@$main_values) {
            $ldap_field       = $value->{ldap_field};
            $ldap_field_value = $value->{ldap_field_value};
            $self->debug_msg(4, "Metadot::User::ManagedLDAPUser::load_ldap_values() saving to user object ldap_field:$ldap_field ldap_field_value:$ldap_field_value");
            $self->{$ldap_field} = $ldap_field_value;
        }
        return 1;
    }
    return 0;
}

sub _get_show_detail_fields {
     my $self = shift;
     $self->debug_msg(3, "Metadot::User::ManagedLDAPUser::_get_show_detail_fields()");

     $self->load_ldap_values();

     my @details = ();

     my $registrator = Metadot::RegisterFactory->get_register();
     my $main_values = $registrator->get_ldap_field_values();
     my $metadot_field    = '';
     my $ldap_field       = '';
     my $label            = '';
     foreach my $value (@$main_values) {
         $ldap_field       = $value->{ldap_field};
         $metadot_field    = $value->{metadot_field};
         $label            = $value->{label};
         next if $metadot_field eq 'password';
         if ($metadot_field eq 'email') {
             $ldap_field = 'email'; ## fix for backward compat
         }
         #print STDERR "\n\nldap_field:$ldap_field metadot_field:$metadot_field label:$label\n\n";
         push @details, { field_name => $metadot_field, label => $label };
     }

     push @details, { field_name => 'true_ldap_dn', label => 'LDAP dn' };

     return @details;
}

1;

__END__
