###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################


package Metadot::User::User3000;
use strict;

=head1 LEGAL
This software is licensed under the open source GPL license. For more information please contact info@metadot.com. Everything copyright Metadot Corporation.


=head1 NAME

Package Metadot::User::Default

=head1 DESCRIPTION

Handles user data storage, user profile editing and user profile display. Can be subclassed to provide customized behavior as well as all additions, deletions, and alterations of user accounts.


=cut




use vars qw(@ISA);

use CGI qw(escape);
use Data::Dumper;

use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $ACCESS_BROKER $CGI %REQUEST_VARS);
use MetadotObject;
use Group;
use Utility;
use Metadot::User::FlexUser;
use Metadot::User::User3000::UserPropertyMapping;
use Metadot::Form3000::Fields::FormFieldFactory;
use Metadot::Form3000::PropertySettings::BasePropertySettingsControl;
use Metadot::User::User3000::PropertyManager;
use Metadot::User::User3000::UserFieldAllocator;
use Metadot::Serialization::SerializerFactory;


@ISA=qw(MetadotObject Metadot::User::FlexUser);


use constant MO_INSTANCE_PROPERTIES => {
    property_mappings   => undef,

    uid                 => undef,
    nickname            => undef,
    fullname            => undef,
    quote               => undef,
    email               => undef,
    approval_needed     => 0,
    password            => undef,
    creation_date       => undef,
    update_date         => undef,
    pager_number        => undef,
    icq_number          => undef,
    icq_urgent          => undef,
    last_online         => 0,
    ip                  => undef,
    admin_level         => 1,
    homepage            => undef,
    debug_level         => 0,
    admin_mode          => 0,
    password            => undef,
    link_name_1         => undef,
    link_name_2         => undef,
    link_name_3         => undef,
    link_name_4         => undef,
    link_name_5         => undef,
    link_name_6         => undef,
    link_name_7         => undef,
    link_name_8         => undef,
    link_name_9         => undef,
    link_name_10        => undef,
    link_name_11        => undef,
    link_name_12        => undef,
    link_name_13        => undef,
    link_name_14        => undef,
    link_name_15        => undef,
    link_url_1          => undef,
    link_url_2          => undef,
    link_url_3          => undef,
    link_url_4          => undef,
    link_url_5          => undef,
    link_url_6          => undef,
    link_url_7          => undef,
    link_url_8          => undef,
    link_url_9          => undef,
    link_url_10         => undef,
    link_url_11         => undef,
    link_url_12         => undef,
    link_url_13         => undef,
    link_url_14         => undef,
    link_url_15         => undef,
    ldap_dn             => undef,
    upload_size         => undef,
    upload_number       => undef,
    password_changed_time                       => undef,
    pw_expire_notification_time  => undef,
    _logged_in          => undef,


    f1  => undef,
    f2  => undef,
    f3  => undef,
    f4  => undef,
    f5  => undef,
    f6  => undef,
    f7  => undef,
    f8  => undef,
    f9  => undef,
    f10 => undef,
    f11 => undef,
    f12 => undef,
    f13 => undef,
    f14 => undef,
    f15 => undef,
    f16 => undef,
    f17 => undef,
    f18 => undef,
    f19 => undef,
    f20 => undef,

    t1  => undef,
    t2  => undef,
    t3  => undef,
    t4  => undef,
    t5  => undef,
    t6  => undef,
    t7  => undef,
    t8  => undef,
    t9  => undef,
    t10 => undef,

    };


# the default properties of this user class
my @_DEFAULT_PROPERTIES = ();


# flags some of the flex_user fields as non-configurable
use constant CONFIGURABLE_PROPERTIES => {
    first_name      => 0,
    initial         => 0,
    last_name       => 0,
    email           => 0,
    password        => 0,
    Metadot::User::FlexUser::get_flex_homepage_field_name   => 0,
    Metadot::User::FlexUser::get_organization_field_name    => 0,
    Metadot::User::FlexUser::get_picture_field_name         => 0,
    };



# override the operations
my $AC = Metadot::User::FlexUser->get_AC();

push( @{ $AC->{BUNDLES}{ADMIN}{OPS} }, 'configure_profile' );

sub get_AC {
    return $AC;
}





# initialize the class
#######################################################################
sub initialize_defaults {
   my $class = shift;

   unless (@_DEFAULT_PROPERTIES) {
       my $fields_config = Metadot::User::FlexUser->get_fields_config();
       $DISPLAY->debug_msg(5, "Initializing $class defaults");

       foreach my $field (@{$fields_config->{FIELDS}} ) {
           #print STDERR "generating mapping for field:\n", Dumper $field;
           push @_DEFAULT_PROPERTIES, $class->generate_user_property_mapping($field);
       }

       #print STDERR "Generated default properties:\n",Dumper(\@_DEFAULT_PROPERTIES);
   }
}



# User class has a different constructor than most MetadotObject,
# for legacy support
sub new {
    my $proto = shift;
    my $uid   = shift;

    #print STDERR "$proto NEW($uid) called by ".caller()."\n";

    # see if this is the $USER object; if so, just return that.
    if (defined $USER && defined($uid) && defined($USER->uid()) && ($USER->uid() == $uid)) {
        return $USER;
    } else {
        my $self = $proto->SUPER::new( uid=>$uid );
        #print STDERR "$proto NEW returns $self\n";

        return $self;
    }
}


# initialize each instance
sub initialize {
    my $self = shift;

    #print STDERR __PACKAGE__." initializes $self from ".caller()."\n";

    $self->SUPER::initialize();

    $self->initialize_defaults();

    # load values from db if needed
    if (defined ($self->uid())) {
        #print STDERR "$self restores data from db\n";
        $self->_restore_from_db();
    }
}



# iterates through the property mappings to find the property with a
# given name
sub get_property_by_name {
    my $self            = shift;
    my $property_name   = shift;
    my $property_list   = shift || $self->property_mappings();

    foreach my $mapping (@$property_list) {
        if ($property_name eq $mapping->property_name()) {
            return $mapping;
        }
    }
    return undef;
}


# iterates through the property mappings to find the method that
# corresponds to a property
sub get_method_for_property {
    my $self            = shift;
    my $property_name   = shift;
    my $property_list   = shift || $self->property_mappings();

    my $mapping = $self->get_property_by_name($property_name, $property_list);
    if (defined $mapping) {
        my $mapped_property = $mapping->mapped_property();
        $DISPLAY->debug_msg(5, "get_method_for_property($property_name, $property_list) has mapping $mapping, with property '$mapped_property'");
        if (wantarray) {
            return ($mapped_property, $mapping);
        } else {
            return $mapped_property;
        }
    }
    return undef;
}


# invoking some legacy methods here
sub _restore_from_db {
    my $self = shift;
    $self->_initialize();  # this method is in User::Default
    $self->_load_secondary_fields();
}


# loads all fields from extended_user table
sub _load_secondary_fields {
    my $self = shift;

    if (defined $self->uid()) {
        my $property_mappings = $self->property_mappings();

        my @secondary_fields  = ();
        foreach my $property (@$property_mappings) {
            if ($property->storage_type_secondary()) {
                push @secondary_fields, $property->mapped_property();
            }
        }

        my @results = DBAccess->sqlSelect(
            join(',', @secondary_fields),
            'extended_user',
            'userid='.$self->uid(),
        );

        foreach my $value (@results) {
            my $field_name = shift @secondary_fields;
            $self->$field_name($self->_unflatten_value($value));
        }
    }
}




# Returns the property mappings for an instance.
# This method can be called as a class method
sub property_mappings {
    my $proto = shift;
    my $result;

    $result = Metadot::User::User3000::PropertyManager->get_custom_properties();
    $DISPLAY->debug_msg(5, "property_mappings checked PropertyManager, got ".($result||''));

    unless (defined $result) {
        $result = \@_DEFAULT_PROPERTIES;
        $DISPLAY->debug_msg(5, "property_mappings will default to $result");
    }

    return $result;
}



# gets a property's value
sub get_value {
    my $self = shift;
    my $property_name = shift;

    my ($result, $method, $mapping) = $self->_get_value_base($property_name);

    # in case this was a select field, remap the value.
    # but check because this can cause an infinite loop

     if (defined $mapping) {
         my $form_field = $mapping->generate_form_field();
         if (defined $form_field && $form_field->isa('Metadot::Form3000::Fields::AbstractSelectionField')) {
             my $request_var_check = '_get_value_'.$mapping;
             unless ($REQUEST_VARS{$request_var_check}++) {
                 my $mapped_value;

                 # check if the result is an array, in which case we have multiple values
                 if (UNIVERSAL::isa($result, 'ARRAY'))
                 {
                     my @mapped_values;
                     foreach my $value (@$result)
                     {
                         my $label = $form_field->get_label_for_value($value);
                         if (defined $label) {
                             push @mapped_values, $label;
                         }
                     }
                     $mapped_value = join('; ', sort @mapped_values);
                 }
                 else
                 {
                     $mapped_value = $form_field->get_label_for_value($result);
                 }
                 $result = $mapped_value if (defined $mapped_value);
             }
             $REQUEST_VARS{'_get_value_'.$mapping} = undef;
         }
     }

    return $result;
}


# gets a property's value without any mapping
sub get_value_raw {
    my $self = shift;
    my $property_name = shift;

    my ($result, $method, $mapping) = $self->_get_value_base($property_name);
    return $result;
}


sub _get_value_base {
    my $self = shift;
    my $property_name = shift;

    my ($method, $mapping) = $self->get_method_for_property($property_name);

    my $result;
    if (defined $method && $self->can($method)) {
        $result = $self->$method();
    } elsif ($self->can($property_name)) {
        $result = $self->$property_name();
    }

    return ($result, $method, $mapping);
}


# sets a property's value
sub set_value {
    my $self            = shift;
    my $property_name   = shift;
    my $value           = shift;

    # TODO: fix this
    my $method = $self->get_method_for_property($property_name);
    my $result;
    if (defined $method && $self->can($method)) {
        $result = $self->$method($value);
    }
    $DISPLAY->debug_msg(5,($self||'')."\->set_value(".($property_name||'').", ".($value||'').") using method '".($method||'')."' returns '".($result||'')."'");
    return $result;
}


# generates a UserPropertyMapping object from a fields_config hash
sub generate_user_property_mapping {
    my $class       = shift;
    my $config_hash = shift || {};
    my $form_type   = shift || $config_hash->{field_type} || 'simple_text';

    #print STDERR "generate_user_property_mapping with config_hash $config_hash, form_type '$form_type'\n";

    # sometimes the name is a function
    my $field_name = $config_hash->{field_name};
    if (ref $field_name)
    {
        $field_name = $class->$field_name();
    }

    # used for generating select list, select box, radios
    my $select_options;

    if (defined $config_hash->{select_options})
    {
        $select_options = $config_hash->{select_options};
    }
    else
    {
        # for backwards compatibility with FlexUser
        if ($form_type eq 'select' || $form_type eq 'radio') {
            # get the field values
            my $values;
            my $values_function = $config_hash->{field_values_function};
            if (defined $values_function) {
                $values = $class->$values_function();
            } else {
                $values = $config_hash->{field_values};
            }

            $select_options = $class->convert_flexuser_select_options($values);
        }
    }


    # generate a form field object
    my $field_definition =
    {
        type            => $form_type,
        name            => $config_hash->{field_name},
        label           => $config_hash->{description},
        default_value   => $config_hash->{default_value},
        is_mandatory    => $config_hash->{is_required},

        max_length          => $config_hash->{max_length},
        size                => $config_hash->{size},
        rows                => $config_hash->{rows},
        cols                => $config_hash->{cols},
        select_options      => $select_options,

        inline_help         => $config_hash->{inline_help},
        heading_text        => $config_hash->{heading_text},
    };

    my $mapping =  Metadot::User::User3000::UserPropertyMapping->new(
    {
        # TODO: remove these redundancies (field_definition has the same values)
        property_name       => $config_hash->{field_name},
        label               => $config_hash->{description},
        mapped_property     => $config_hash->{store_at_column} || $config_hash->{field_name},
        storage_type        => $config_hash->{storage_type},
        enable_param        => $config_hash->{enable_param},
        field_definition    => $field_definition,
        field_type          => $form_type,
        configurable        => CONFIGURABLE_PROPERTIES()->{$config_hash->{field_name}},
        visible             => $config_hash->{visible},
    });

    return $mapping;
}



# for backwards compatibility
sub get_fields_config {
    my $class = shift;
    #print STDERR "$class\->get_fields_config\n";

    my @fields = ();
    foreach my $property (@{$class->property_mappings()}) {
        push @fields, $class->generate_field_config_hash($property);
    }


    return {
        DISPLAY_SETS => ['Registration Information'],
        FIELDS       => \@fields,
        };
}


# Used to store user properties. Also handles backwards compatibility with
# FlexUser
sub generate_field_config_hash {
    my $class = shift;
    my $property_mapping = shift || die('Must specify a property mapping');

    my $form_field = $property_mapping->generate_form_field();

    #print STDERR "generating field config hash from $property_mapping\n";
    my %result = (
        field_name      => $property_mapping->property_name(),
        description     => $property_mapping->label(),
        store_at_column => $property_mapping->mapped_property(),
        storage_type    => $property_mapping->storage_type(),
        field_type      => $property_mapping->field_type(),
        is_required     => $form_field->is_mandatory(),
        visible         => $property_mapping->visible(),
        );

        if ($form_field->isa('Metadot::Form3000::Fields::AbstractSelectionField')) {
            $result{select_options}  = $form_field->flatten_select_options(),
        }

        # add some extra form field properties
        foreach my $property (qw(default_value max_length size rows cols inline_help heading_text)) {
            if ($form_field->can($property)) {
                $result{$property} = $form_field->$property();
            }
        }

        return \%result;
}


# used when displaying the user profile
sub _generate_user_properties_hash {
    my $self = shift;
    my $user = shift || $USER;
    my %output = ();

    my $properties = $self->property_mappings();
    foreach my $mapping (@$properties) {
        my $field_name = $mapping->property_name();

        next if ($field_name eq 'password');

        $output{$field_name} = {
            description => $mapping->label(),
            value       => $user->get_value($field_name),
            hidden      => !($mapping->visible()),
        }
    }

    return \%output;
}


sub output_remaining_properties {
    my $self = shift;
    my $properties_hash = shift;
    my $user = shift;

    my $properties = $self->property_mappings();
    my $s = '';

    foreach my $mapping (@$properties) {
        my $field_name = $mapping->property_name();
        my $entry = $properties_hash->{$field_name};
        if (defined $entry && !$entry->{visited} && $mapping->visible() ) {
            if( $user->get_value($field_name) ){
                $s .= $self->output_user_property($properties_hash, $field_name);
            }
        }
    }

    return $s;
}



# get the current index of the property name
sub index_of_property {
    my $self = shift;
    my $property_name = shift;
    my $property_list = shift || $self->property_mappings();

    my $count = 0;
    if (defined $property_name) {
        foreach my $property (@$property_list) {
            if ($property_name eq $property->property_name()) {
                return $count;
            }
            $count++;
        }
    }

    return undef;
}


# some accessors will be mapped to super because they have special behavior
sub fullname {
    my $self = shift;
    return $self->SUPER::fullname(@_);
}

sub webname {
    my $self = shift;
    return $self->SUPER::webname(@_);
}






########################################################################
########################################################################
#                                                                      #
#  Exec methods                                                        #
#                                                                      #
########################################################################
########################################################################

sub exec_configure_profile {
    my $self = shift;

    # keep a temporary set of property mappings to configure
    my $configured_properties;

    my $preview_changes     = $FORM{preview_changes};
    my $up_down_property    = $FORM{up_down_property};
    my $hide_property       = $FORM{hide_property};
    my $unhide_property     = $FORM{unhide_property};
    my $recycle_property    = $FORM{recycle_property};
    my $new_field           = $FORM{new_field};
    my $confirm_recycle     = $FORM{confirm_recycle};
    my $finished            = $FORM{finished_editing};

    my $changed  = $finished || $preview_changes || $up_down_property
                   || $hide_property || $unhide_property
                   || $recycle_property || $new_field;

    my $new = !($changed || $finished || $confirm_recycle);

    # process any form changes
    if ($new) {
        # this is a new working session
        $configured_properties = $self->property_mappings();
        $self->_clear_recycle_bin();
    } else {
        # get the properties from the session
        $configured_properties = $self->get_session_configuration() || $self->property_mappings();
    }


    # get any form updates
    if ($changed) {

        # process the control settings
        foreach my $property_setting (@$configured_properties) {
            my $property_setting_control = $property_setting->get_property_settings_control();
            $property_setting_control->process_input();
        }

        # process the up/down
        if ($up_down_property) {
            $configured_properties = $self->_move_property_up_down($FORM{up_down_property}, $FORM{up_down_delta}, $configured_properties);
        }

        # process hiding
        if ($hide_property) {
            my $property = $self->get_property_by_name($hide_property, $configured_properties);
            $DISPLAY->debug_msg(4, "Hiding property $hide_property -> ".(defined $property ? $property : 'undef'));
            if (defined $property) {
                $property->visible(0);
            }
        }

        # process unhiding
        if ($unhide_property) {
            my $property = $self->get_property_by_name($unhide_property, $configured_properties);
            $DISPLAY->debug_msg(4, "Unhiding property $hide_property -> ".(defined $property ? $property : 'undef'));
            if (defined $property) {
                $property->visible(1);
            }
        }

        # process recycling
        if ($recycle_property) {
            # place the property in the recycle bin, remove it from the working list
            $configured_properties = $self->_recycle_property($recycle_property, $configured_properties);
        }

        # process adding
        if ($new_field) {
            my $field_type = $FORM{new_field_type};
            $DISPLAY->debug_msg(5, "Will create a new field of type '$field_type'");
            $configured_properties = $self->_allocate_field($field_type, $configured_properties);
            $DISPLAY->debug_msg(5, "I got revised properties $configured_properties");
        }
    }

    # put the updated configured properties in the session
    $self->set_session_configuration($configured_properties);

    if ($finished || $confirm_recycle) {
        # check if there was recycling to do
        my $recycled = $self->_retrieve_recycle_bin();
        if ($recycled && !$confirm_recycle) {
            my $field_list = '';
            foreach my $property (@$recycled) {
                $field_list .= '<li>'.$property->label()."</li>\n";
            }
            my $recycle_warning = <<EOM
You have marked the following fields for recycling.<br />
<ul>
$field_list
</ul>
 This will <b>permanently delete</b> all user profile data associated
 with these fields. Are you sure you want to proceed?
EOM
;

            $DISPLAY->two_choices(
                                $recycle_warning,
                               "caution",
                               "Yes, recycle the fields", "index.pl?isa=User&op=configure_profile&confirm_recycle=1&landing_url=".$CGI->escape($FORM{landing_url}),
                               "No!", "index.pl?isa=User&op=configure_profile&preview_changes=1&landing_url=".$CGI->escape($FORM{landing_url}));
            exit;
        } else {
            # TODO: implement the recycling
            $self->_commit_recycling($recycled);

            # save the changes to the system
            Metadot::User::User3000::PropertyManager->set_custom_properties($configured_properties);

            $DISPLAY->print_choices(
                "User profile has been changed.",
                    'confirm', "Return to the portal", $FORM{landing_url},
               );
            exit;
        }
    } else {
        # build and output the form
        $REQUEST_VARS{form_object} = $self->_build_form_object();
        return 'user3000_configure_profile.tt2';
    }
}


sub _build_form_object {
    my $self = shift;

    my $form = Metadot::Form3000::Form->new(
        'name'    => 'configureProfileForm',
        'title'   => 'Configuring User Profile',
        'method'  => 'post',
        'enctype' => 'multipart/form-data',
        'submit_actions' => [
            {
                name  => 'preview_changes',
                value => 'Preview changes',
            },
            {
                name  => 'finished_editing',
                value => 'Save changes',
            },
        ],
#        'header_text' => $form_header_text,
#        'footer_text' => $form_footer_text,
        field_layout  => Metadot::Form3000::Form->FIELD_LAYOUT_TWO_COLUMN(),
        enable_reset  => 0,
    );

    # the property that is being moved up/down
    $form->generate_form_field(
        type  => 'hidden',
        name  => 'up_down_property',
        );

    # the delta to move it up/down
    $form->generate_form_field(
        type  => 'hidden',
        name  => 'up_down_delta',
        );

    # the property that is being hidden
    $form->generate_form_field(
        type  => 'hidden',
        name  => 'hide_property',
        );

    # the property that is being unhidden
    $form->generate_form_field(
        type  => 'hidden',
        name  => 'unhide_property',
        );

    # the property that is being recycled
    $form->generate_form_field(
        type  => 'hidden',
        name  => 'recycle_property',
        );

    # TODO: create the fields for adding a property


    $form->generate_form_field(
        type  => 'hidden',
        name  => 'isa',
        value => 'User');

    $form->generate_form_field(
        type  => 'hidden',
        name  => 'op',
        value => 'configure_profile');




    return $form;
}




use constant CONFIGURED_PROPERTIES_SESSION_KEY => '_user3000_configured_properties';

# gets the property mappings being configured
sub get_session_configuration {
    my $packed = $SESSION->get_value(CONFIGURED_PROPERTIES_SESSION_KEY());
    #my $serializer = Metadot::Serialization::SerializerFactory->get_instance($packed);
    #if (defined $serializer) {
    #    return $serializer->deserialize($packed);
    #}
    my $unpacked = Metadot::User::User3000::PropertyManager->_unpack_data($packed);
    return $unpacked;
}


# sets the property mappings being configured
sub set_session_configuration {
    my $self        = shift;
    my $properties  = shift;

    #my $serializer = Metadot::Serialization::SerializerFactory->get_instance();
    #if (defined $serializer) {
    #    $SESSION->set_value(CONFIGURED_PROPERTIES_SESSION_KEY(), $serializer->serialize($properties));
    #}

    $SESSION->set_value(CONFIGURED_PROPERTIES_SESSION_KEY(), Metadot::User::User3000::PropertyManager->_pack_data($properties));
}



# TODO: this is copied from Gizmo3000. Refactor this!!
# Uses the property mappings to create a set of new form fields.
sub define_form_fields {
    my $self = shift;

    my @fields = ();
    my %values = ();

    foreach my $mapping (@{$self->property_mappings()}) {
        if ($mapping->visible()) {
            my $property_name = $mapping->property_name();
            my $form_field    = $mapping->generate_form_field();

            $DISPLAY->debug_msg(5, "defining form field for property $property_name, field $form_field");
            push @fields, $form_field;

            if (defined $form_field) {
                my $form_key = $form_field->form_key();
                if (defined $property_name) {
                    # TT returns a false 'defined' if the key exists but is
                    # defined, so I just leave the key out if I don't want it
                    if (exists $FORM{$form_key}) {
                        $values{$form_key} = $FORM{$form_key};
                        $DISPLAY->debug_msg(5, "using existing form value $form_key = $FORM{$form_key}");
                    } else {
                        my $mapped_value = $self->get_value_raw($property_name);
                        if (defined $mapped_value) {
                            $values{$form_key} = $mapped_value;
                            $DISPLAY->debug_msg(5, "using existing property $property_name = ".$values{$form_key}." from $mapped_value");
                        }
                    }
                }
            }
        }
    }
    return \@fields, \%values;
}


# move a property up or down in the list
sub _move_property_up_down {
    my $self = shift;
    my $property_name = shift;
    my $delta = shift;
    my $property_list = shift || $self->property_mappings();

    $delta = undef unless(Utility->is_valid_integer($delta));

    # get index of the property
    my $current_index = $self->index_of_property($property_name, $property_list);
    return Utility->move_element_within_list($property_list, $current_index, $delta);
}


# removes a property from the list, moves it to the recycle bin
sub _recycle_property {
    my $self = shift;
    my $property_name = shift;
    my $property_list = shift || $self->property_mappings();

    my $index = $self->index_of_property($property_name, $property_list);
    if (defined $index) {
        my $property = $property_list->[$index];
        $self->_place_in_recycle_bin($property);

        # remove the property from the list
        my @list = @$property_list;
        my $list_size = scalar(@list);
        @list = (@list[0 .. $index-1], @list[$index+1 .. $list_size-1 ]);

        $property_list = \@list;
        $DISPLAY->debug_msg(5, "Recycled property ".$property->property_name().". I have properties: ".$self->_property_names());
    }

    return $property_list;
}


# for debugging purposes only
sub _property_names {
    my $self = shift;
    my $property_list = shift || $self->property_mappings();
    my @names;
    foreach my $property (@$property_list) {
        push @names, $property->property_name();
    }

    if (wantarray) {
        return @names;
    } else {
        return join(',', @names);
    }
}








# Field recycle bin: this will likely be factored out in the future
#----------------------------------------------------------------------
use constant _RECYCLE_BIN_SESSION_KEY => '_USER3000_RECYCLE_BIN';

# places an object in the bin
sub _place_in_recycle_bin {
    my $self = shift;
    my $property = shift;
    if (defined $property) {
        my $recycled = $self->_retrieve_recycle_bin() || [];
        push @$recycled, $property;
        $SESSION->set_value(_RECYCLE_BIN_SESSION_KEY(), Metadot::User::User3000::PropertyManager->_pack_data($recycled));
    }
}


# retrieves all objects in the bin
sub _retrieve_recycle_bin {
    my $self = shift;
    my $session_value = $SESSION->get_value(_RECYCLE_BIN_SESSION_KEY());
    my $result;
    if ($session_value) {
        $result = Metadot::User::User3000::PropertyManager->_unpack_data($session_value);
    }
    return $result;
}


# removes objects from the bin
sub _clear_recycle_bin {
    my $self = shift;
    $SESSION->set_value(_RECYCLE_BIN_SESSION_KEY(), '');
}


# clears user properties in the database, clears the bin
sub _commit_recycling {
    my $self = shift;
    my $recycling = shift || $self->_retrieve_recycle_bin();

    if ($recycling) {
        foreach my $property (@$recycling) {
            my $mapped_property = $property->mapped_property();
            $DISPLAY->debug_msg(5, "Will recycle property ".$property->property_name()." by clearing $mapped_property");

            if (defined $mapped_property) {
                DBAccess->sqlUpdate('extended_user', '1=1', ($mapped_property=>undef));
            }
        }

        $self->_clear_recycle_bin();
    }
}





# Field allocation methods
#----------------------------------------------------------------------
sub get_available_fields {
    my $self = shift;
    my $properties = shift || $self->get_session_configuration() || $self->property_mappings;

    my $available  = Metadot::User::User3000::UserFieldAllocator->available_inventory($properties);
    return $available;
}


sub _allocate_field {
    my $self = shift;
    my $field_type = shift;
    my $properties = shift || $self->get_session_configuration() || $self->property_mappings;

    $DISPLAY->debug_msg(4, "Allocating field of type '$field_type', I have properties $properties");

    my $available_field = Metadot::User::User3000::UserFieldAllocator->allocate_field(
        $properties, Metadot::User::User3000::UserFieldAllocator->cost_of($field_type));

    $DISPLAY->debug_msg(5, "Allocating field $available_field".$available_field->field_name());

    # generate a form field object
    my $default_name  = $available_field->field_name();
    my $default_label = 'Name this field';

    my $field_definition = {
        type            => $field_type,
        name            => $default_name,
        label           => $default_label,
        };

    $DISPLAY->debug_msg(5, "I have field definition $field_definition");

    my $mapping =  Metadot::User::User3000::UserPropertyMapping->new({
        property_name       => $default_name,
        label               => $default_label,
        mapped_property     => $available_field->field_name(),
        storage_type        => 'secondary',
        field_definition    => $field_definition,
        field_type          => $field_type,
        configurable        => 1,
        });

    $DISPLAY->debug_msg(5, "I have mapping $mapping");

    push @$properties, $mapping;
    return $properties;
}




# converts flexuser options to the new way
sub convert_flexuser_select_options {
    my $class   = shift;
    my $options = shift;


    my @select_options = ();

    # convert the values to the Form3000 way, which is value,label,value,label, etc.
    if (defined $options) {
        foreach my $value (@$options) {
            push @select_options, ($value->{form_value}, $value->{label});
        }
    }

    return \@select_options;
}





1;
