package Metadot::User::User3000::PictureField;

use Metadot::Form3000::Fields::BaseFormField;
use Metadot qw($USER $DISPLAY %FORM %REQUEST_VARS);
use Metadot::UserFactory;

use strict;
use vars qw(@ISA);

@ISA = qw(Metadot::Form3000::Fields::BaseFormField);

use constant MO_INSTANCE_PROPERTIES => {
    template        => 'user_picture_field.tt2',
};


sub render_control {
    my $self = shift;

    my $op = $FORM{op};
    my $uid = $FORM{uid};

    my $u;

    # Only show the current user's picture if modifying itself
    if( $op !~ /modify_self/ ){
        $u = Metadot::UserFactory->get_user($uid);
    } else {
        $u = $USER;
    }

    $DISPLAY->debug_msg(5, "Picture renders control with op='$op', uid='$uid', user=$u (".$u->uid().")");

    my ($return_url, $landing_url);

    my $form_obj = $REQUEST_VARS{form_object};
    if (defined $form_obj) {
        $return_url  = $form_obj->calling_url();
        $landing_url = $form_obj->landing_url();
    }

    unless (defined $return_url) {
        $return_url = "index.pl?isa=User&op=$op";
        $return_url    .= "&uid=$uid" if defined($uid);
        $return_url = $DISPLAY->make_url( $return_url );
    }

    unless (defined $landing_url) {
        $landing_url = $return_url;
    }

    $DISPLAY->debug_msg(5, "return_url = $return_url; landing_url = $landing_url");
    my $control = '';

    my $field_name = $u->get_picture_field_name();

    my $image_src = $u->get_image_src(field_name => $field_name);
    $DISPLAY->debug_msg(5, "Checked $field_name for image src, got $image_src");

    $control .= $image_src;
    $control .= $u->get_upload_input_form_line( {
                   isa => $u->is_a(),
                   id => $u->uid(),
                   field_name => $field_name,
                   field_desc => $self->label(),
                   is_required => $self->is_mandatory(),
                   type => 'user_upload',
                   landing_url => $landing_url,
               } );

    return $control;
}



1;
