package Metadot::User::User3000::UserFieldAllocator;

use Metadot qw($DISPLAY);
use Metadot::User::User3000::UserFieldAllocation;

use constant FIELD_TYPE_TEXTAREA    => 'textarea';
use constant FIELD_TYPE_VARCHAR     => 'varchar';
use constant FIELD_TYPE_SELECTBOX   => 'select_box';

use constant FIELD_CAPACITY_TEXT    => 16384;
use constant FIELD_CAPACITY_VARCHAR => 256;
use constant FIELD_CAPACITY_DEFAULT => FIELD_CAPACITY_VARCHAR();

use constant _ALLOCABLE_FIELDS => {
    f1  => FIELD_CAPACITY_VARCHAR(),
    f2  => FIELD_CAPACITY_VARCHAR(),
    f3  => FIELD_CAPACITY_VARCHAR(),
    f4	=> FIELD_CAPACITY_VARCHAR(),
    f5	=> FIELD_CAPACITY_VARCHAR(),
    f6	=> FIELD_CAPACITY_VARCHAR(),
    f7	=> FIELD_CAPACITY_VARCHAR(),
    f8	=> FIELD_CAPACITY_VARCHAR(),
    f9	=> FIELD_CAPACITY_VARCHAR(),
    f10	=> FIELD_CAPACITY_VARCHAR(),
    f11	=> FIELD_CAPACITY_VARCHAR(),
    f12	=> FIELD_CAPACITY_VARCHAR(),
    f13	=> FIELD_CAPACITY_VARCHAR(),
    f14	=> FIELD_CAPACITY_VARCHAR(),
    f15	=> FIELD_CAPACITY_VARCHAR(),
    f16	=> FIELD_CAPACITY_VARCHAR(),
    f17	=> FIELD_CAPACITY_VARCHAR(),
    f18	=> FIELD_CAPACITY_VARCHAR(),
    f19	=> FIELD_CAPACITY_VARCHAR(),
    f20	=> FIELD_CAPACITY_VARCHAR(),

    t1	=> FIELD_CAPACITY_TEXT(),
    t2	=> FIELD_CAPACITY_TEXT(),
    t3	=> FIELD_CAPACITY_TEXT(),
    t4	=> FIELD_CAPACITY_TEXT(),
    t5	=> FIELD_CAPACITY_TEXT(),
    t6	=> FIELD_CAPACITY_TEXT(),
    t7	=> FIELD_CAPACITY_TEXT(),
    t8	=> FIELD_CAPACITY_TEXT(),
    t9	=> FIELD_CAPACITY_TEXT(),
    t10	=> FIELD_CAPACITY_TEXT(),
    };

use constant FIELD_COSTS => {
    FIELD_TYPE_TEXTAREA()   => FIELD_CAPACITY_TEXT(),
    FIELD_TYPE_SELECTBOX()  => FIELD_CAPACITY_TEXT(),

    # any that are not listed here are assumed to be FIELD_CAPACITY_DEFAULT
};


# allocates a field
sub allocate_field {
    my $self = shift;
    my $properties = shift || die('Must specify the list of property mappings');
    my $required_size = shift || FIELD_CAPACITY_DEFAULT();

    my $result;
    my $available_inventory = $self->available_inventory($properties);

    # get the lowest capacity field that satisfies the request
    foreach my $field (@$available_inventory) {
        if ($field->capacity() >= $required_size) {
            if (!defined $result || $field->capacity() < $result->capacity()) {
                $result = $field;
            }
        }
    }

    return $result;
}


# returns the inventory of unallocated fields
sub available_inventory {
    my $self = shift;
    my $properties = shift || die('Must specify the list of property mappings');
    my $required_size = shift || FIELD_CAPACITY_DEFAULT();

    my @results;
    my $inventory = $self->generate_inventory($properties);
    foreach my $field (@$inventory) {
        unless ($field->allocated()) {
            if ($field->capacity >= $required_size) {
                push @results, $field;
            }
        }
    }

    return \@results;
}


# returns the entire inventory of fields, allocated or not
sub generate_inventory {
    my $self = shift;
    my $properties = shift || die('Must specify the list of property mappings');

    my @inventory;

    foreach my $key (keys %{_ALLOCABLE_FIELDS()}) {
        push @inventory, Metadot::User::User3000::UserFieldAllocation->new(
            field_name => $key,
            capacity   => _ALLOCABLE_FIELDS()->{$key},
            allocated  => $self->is_field_in_use($key, $properties),
            );
    }

    return \@inventory;
}



# given a field name, locates the property, if any, that is using it
sub get_property_using_field {
    my $self = shift;
    my $field_name = shift;
    my $properties = shift || die('Must specify the list of property mappings');
    my $result;

    if (defined $field_name) {
        foreach my $property (@$properties) {
            if ($property->mapped_property() eq $field_name) {
                $result = $property;
                last;
            }
        }
    }

    return $result;
}

# given a field name, determines whether a property is using it. If in
# use, will return the property that is using it.
sub is_field_in_use {
    my $self = shift;
    my $field_name = shift;
    my $properties = shift || die('Must specify the list of property mappings');

    return $self->get_property_using_field($field_name, $properties);
}


sub cost_of {
    my $self = shift;
    my $field_type = shift;
    
    my $cost = FIELD_COSTS()->{$field_type};
    $cost = FIELD_CAPACITY_DEFAULT() unless (defined $cost);
    $DISPLAY->debug_msg(4, "cost_of $field_type is $cost");

    return $cost;
}






1;
