package Metadot::User::User3000::UserPropertyMapping;


use strict;
use vars qw(@ISA);

use Metadot::Form3000::PropertyMapping;
use Metadot::Form3000::Fields::FormField;
use Metadot qw($PARAMS);
use Metadot::Form3000::PropertySettings::PropertySettingFactory;

@ISA = qw(Metadot::Form3000::PropertyMapping);


use constant MO_INSTANCE_PROPERTIES => {
    storage_type        => undef,   # whether the property is stored in the primary
                                    # or secondary table
    configurable        => 1,       # whether the property can be configured
    field_type          => undef,   # the field type, for backwards compatibility
    enable_param        => undef,   # legacy support for FlexUser's enable_param
    visible             => 1,       # determines whether this field is visible
};

use constant STORAGE_TYPE_PRIMARY   => 'primary';
use constant STORAGE_TYPE_SECONDARY => 'secondary';


sub storage_type_primary {
    my $self = shift;
    return ($self->storage_type() eq STORAGE_TYPE_PRIMARY());
}

sub storage_type_secondary {
    my $self = shift;
    return ($self->storage_type() eq STORAGE_TYPE_SECONDARY());
}

# returns whether a field is visible. only enabled fields are visible,
# regardless of the actual property setting.
sub visible {
    my $self = shift;
    return $self->_visible(@_) && $self->enabled();
}


# by default, returns true if there is no enable_param for this mapping,
# or if there is and the PARAMS has it enabled
sub enabled {
    my $self = shift;
    my $enable_param = $self->enable_param();
    return (!defined $enable_param || $PARAMS->{$enable_param});
}

# returns whether a mapping may be recycled
sub recyclable {
    my $self = shift;
    return $self->configurable() && $self->storage_type_secondary();
}

# clears old values from a property so that it can be recycled
sub recycle {
    my $self = shift;
    die('This property is not recyclable') unless $self->recyclable();

    foreach my $attribute (qw( field_type enable_param )) {
    }
    $self->visible(1);
}


1;
