###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::UserFactory;
use strict;

use MetadotConfig;
use Metadotclass;

use DBAccess;


sub get_user {
    my $class = shift;
    my @args  = @_;

    my $user_type = MetadotConfig->get_user_type();
    Metadotclass->debug_msg(5, $class." getting user type $user_type\->new(@args); called by ".caller());

    require "Metadot/User/$user_type.pm";

    my $usr = "Metadot::User::$user_type"->new( @args );
    #print STDERR "$class\->get_user(@args) got user type: '$user_type'; got user object $usr\n";

    return $usr;
}

1;

__END__


=head1 NAME

Metadot::UserFactory - Middleman class to provide instances of different Metadot::User subclasses depending on configuration.

=head1 SYNOPSIS

  my $user = Metadot::UserFactory->get_user();


=head1 DESCRIPTION

This class relies on method get_user_type in MetadotConfig, which reads in the value of the variable "user_type" in metadot.conf (the config file), from where different user types can be selected. This class alongside with concrete User classes provides a framework to allow interchangeable User classes in Metadot, to easily and cleanly customize User class methods.

If a customized version of User is needed then User::Default must be subclassed and the new class specified in the user_type veriable in the config file.

=head1 METHODS

get_user

Returns an instance of a Metadot::User subclass depending on system configuration. All instances of User in the system must be obtained through this method.

