###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Metadot::Utils::Display;

use strict;

use Data::Dumper;
use Spreadsheet::WriteExcel;
use Template;

use Metadot::PopupMgr;
use Metadot::PortalFactory;
use Metadot::UploadsManager;
use Metadot::Utils::Theme;
use Metadotclass;
use MetadotConfig;
use Portal;
use Utility;

use vars qw(@ISA);
@ISA=qw(Metadotclass);

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $CGI $ACCESS_BROKER $DISPLAY %REQUEST_VARS);

use constant UNALLOWED_HTML_MESSAGE =>
    q~Your submission contains unallowed HTML tags, such as IMG tags that ~.
    q~contain '?' and '&' characters in the SRC attribute, or SCRIPT tags.~;


use constant DEFAULT_CHARSET => 'iso-8859-1';

sub new {
    my $proto = shift;

    #my $class = __PACKAGE__;
    # CKIM: this is an odd constructor! Changing it to allow sub-classing
    my $class = ref($proto) || $proto || __PACKAGE__;

    my $self = {};

    bless ($self, $class);
    return $self;
}

sub print_http_header {
    my $self = shift;
    my $header_info_ref = shift;

    my $result;

    # here are the cookies passed from 'write_cookie' function

    # do not call debug_msg from here or it will loop
    # since debugmsg is calling on_the_web!!!!

    if (Metadotclass->on_the_web() && !$HTTP_HEADER_SENT){
        # Here CGI.pm will add to HTTP header  Pragma: no-cache
        # watch out - '1' means 'no cache'
        $CGI->cache(1);

        #adding 'expires' information to the header
        #(maybe, it's not necessary after the previous one :)
        $$header_info_ref{-expires} = '-1y';

        # set the charset
        $$header_info_ref{-charset} = $self->get_charset();

        print $CGI->header( $header_info_ref );

        $result = $HTTP_HEADER_SENT++;
    }

    return $result;
}


sub open_html {
  my $self = shift;
  $self->debug_msg(3, "Portal::open_html");
  # So you found the little Metadot 'Portal Server' and you want to
  # to change it to something else.
  # Please give us a little credit for the work we've done
  # and do not change it. Thanks.
   return  qq(
 <HTML>
 <META NAME="Generator" CONTENT="MetaDot Portal Server;
www.metadot.com">
 <META HTTP-EQUIV="Window-target" CONTENT="_top">);

}

sub close_html {
  my $self = shift;
  $self->debug_msg(3, "Portal::close_html");
  my ($global_look) = $PARAMS->{globallook};
  if ($global_look ne 'usetemplate') {
    return "\n</body>\n" . $SESSION->get_timer_count_html() . "\n</html>\n\n";
  } else {
    return "\n\n\n";
  }
}

##############################################################3

sub show_page_top {
 my $self = shift;
  $self->debug_msg(3, "Portal::page_top()");
  my $s = $self->open_html . "\n\n" . $self->show_style_sheet . "\n\n";
  return $s;
}

sub show_style_sheet {
  my $self = shift;
  $self->debug_msg(3, "Portal::show_style_sheet()");
  my ($style) = $PARAMS->{stylesheet};
  my $s = "\n\n$style\n\n<!-- end style  -->\n\n";
  return $s;
}

sub show_title {
  my $self = shift;
  my $title = shift;
  my $panel = shift || "&nbsp;";
  $self->debug_msg(3, "Portal::show_title($title)");

  my $categorytitlebgcolor = $PARAMS->{'categorytitlebgcolor'};

  my $buf = "\n\n<table border=\"0\"  bgcolor='#$categorytitlebgcolor' width='100%' cellpadding=\"0\" cellspacing=\"0\">\n";
  $buf .= "<tr><td class='catheader' align='left'>&nbsp;$title</td>\n<td class='catheader' align='right'>\n$panel\n</td></tr></table>\n\n";

  return $buf;
}

sub show_title_bar {
    my $self = shift;
    my $title = shift;

    return "<h1>$title</h1>\n";
}

##############################################################3

sub show_header {
    my $self = shift;
    my ($iid) = shift;
    my $str = "";
    my $style = "";

    $self->debug_msg(3, "show_header(iid=$iid)");

    # if using themes, use the generic header.  This will only be used
    # for a few pages, like the groups and users pages.
    if (Metadot::Utils::Theme->use_theme_info()) {
    $str = $PARAMS->{theme_adminheader};
    }
    else {
        if($iid > 0){
            $str = $PARAMS->{subheader};
        } else {
            $str = $PARAMS->{mainheader};
        }
    }
    my $s = "\n\n\n$str\n\n<!-- end header -->\n\n";
    my ($global_look) = $PARAMS->{globallook};

    # XXX [claudio] this doesn't seem to be needed
    if ($global_look ne 'usetemplate') {
        $s = "<HEAD>\n" . $str ;
    }

    # expand gizmotags
    $s = $self->simple_expand( $iid, $self, $s);

    return $s;
}

sub show_footer {
    my $self = shift;
    my ($iid) = @_;
    my ($str);
    $self->debug_msg(3, "Display::show_footer()");

    $str = "\n\n<!-- begin footer -->\n\n";
    if (defined($iid) && ($iid > 0)) {
        ($str) = $PARAMS->{subfooter};
    }else{
        ($str) = $PARAMS->{mainfooter};
    }

    $str .= "\n\n<!-- end footer -->\n\n";
    return $self->simple_expand( $iid, $self, $str);
    #return $str;
}


######################################################################3

sub print_portal {
    my $class = shift;
    my $args;

    # backwards compatibility
    if (ref $_[0]) {
        $args = shift;
    } else {
        $args = {
            content => shift,
            title   => shift,
            iid     => shift,
            subject => shift,
        };
    }


    my $content = $args->{content};
    my $title   = $args->{title};
    my $iid     = $args->{iid};
    my $subject = $args->{subject};

    if (!defined $subject) {
        if (!defined $iid) {
            $iid = 0;
        }

        $subject = Metadot::GizmoBase->restore($iid);
    }
    
    # sub id() is compatible with more classes
    # $iid = $subject->iid();
    $iid = $subject->id();


    $class->print_http_header();
    print $class->print_comment_tag();

    my ($global_look) = $PARAMS->{globallook};
    $class->debug_msg(3, "print_portal(), with global look: $global_look; iid=$iid, subject=$subject (".$subject->id().")");

    $global_look ||= "old";

    if ($subject && $global_look ne "usetemplate") { # if template mode, will get expanded already
        $content = $subject->expand($iid, $content);
    }

    my $page_title = $title || $subject->name() ;
    #  print start_html $page_title;

    if($SESSION->get_value('GB_status') eq '1') {
        print $subject->process_template($iid, $subject->show_template($iid));
    } else {
    if ($global_look eq "old") {
        ## XXX    print start_html $page_title;
        print $subject->show_top($iid),
        $subject->show_title($title),
        $content,
        $subject->show_bottom();
    } elsif ($global_look eq "new") {
        my $data = $subject->show_new_look($content, $iid);
        $data = $class->simple_expand($iid, $subject, $data);

        print $subject->show_page_top(),
        $subject->show_header($iid),
        $subject->show_portal_panel($iid),
        $data,
        $subject->show_bottom;
    } elsif ($global_look eq "latest") {
        my $data = $subject->show_latest_look($content, $iid);
        $data = $class->simple_expand( $iid, $subject, $data);

        print $subject->show_page_top(),
        $subject->show_header($iid),
        $subject->show_portal_panel($iid),
        $data,
        $subject->show_bottom;
    } elsif ($global_look eq "usetemplate") {

        if (!defined $content && ($subject->isa("Category")) && ($FORM{op} eq "show")) {
            print $subject->expand($iid, $subject->show_template($iid));
        } else {
            my $type = $subject->is_a;
    #       my $obj_op = $FORM{op};
    #       my $class = ref($subject);
    #       my $d = $class->new($iid);
    #       if ($type ne "Application" && ($d->can($obj_op))) { # for all gizmos but application
            # aju -- took out enumeration of gizmo types since we
            # were not catching new ones
            if ($type ne "Application") { # for all gizmos but application
            # !!!! -- why did the 'content-generating' op have to be rerun here?
    #           my $results = $d->$obj_op();
                print  $class->simple_expand($iid, $subject, $subject->show_template(-1), $content);
            }
            else {
                # use 'latest' look for displaying non-category pages
                print $subject->show_page_top(),
                $subject->show_header($iid),
                $subject->show_portal_panel($iid),
                $subject->show_latest_look($content, $iid),
                $subject->show_bottom;
            }
        }
    }
    }
}

######################################################################3

# prints content with header and footer only
sub print_simple_layout {
    my ($class, $args) = @_;

    my $content = $args->{content};
    my $title   = $args->{title};
    my $iid     = $args->{iid};
    my $show_panels = $args->{show_panels};
    my $subject = $args->{subject};

    $class->print_http_header();
    print $class->print_comment_tag();

    print $class->show_header(),
          $class->show_title("User Directory");

    if ($show_panels && defined($subject)) {
        print $subject->show_portal_panel( 0 ),
              $subject->show_panel();
    }

    print $content,
          $class->show_footer();
}

######################################################################

sub print_form {
    my $self = shift ;
    my $content = shift;
    my $title = shift;
    $self->debug_msg(3, "Portal::print_form()");
    $self->print_http_header();
    print $self->generate_form($content, $title);
}


sub generate_form {
    my $self = shift ;
    my $content = shift;
    my $title = shift;

    my $result = $self->show_page_top();

    #aju
    my $formbgcolor;
    if (Metadot::Utils::Theme->use_theme_info()) {
        $formbgcolor = Metadot::Utils::Theme->get_theme_default_bg();
    }
    else {
        $formbgcolor = $PARAMS->{'formbgcolor'};
    }
    $result .= $self->simple_header($self->get_htmlarea_base_script()) . "<body onload=\"initializeWindow();\">\n";
    $result .= "<script type=\"text/javascript\">
            function initializeWindow() {
                if (window.initializeFocus != null) {
                    initializeFocus();
                }

                if (window.preloadImages != null) {
                    preloadImages();
                }
        }
        </script>
    ";

## XXX  print start_html $title;
    $result .= "<center>" . $self->form_top();
    $result .= "<tr bgcolor='#$formbgcolor'><td>&nbsp;</td><td>$content</td><td>&nbsp;</td></tr>";
#      print $content;
    $result .= $self->form_bottom() . "</center>";
    $result .= $self->close_html();

    return $result;
}

sub popup_window{
        return "<script type=\"text/javascript\">
<!-- Begin
function popUp(URL) {
day = new Date();
id = day.getTime();
eval(\"page\" + id + \" = window.open(URL, '\" + id + \"', 'toolbar=0,scrollbars=1,location=0,statusbar=0,menubar=0,resizable=1,width=300,height=400');\");
}
// End -->
</script>
";
}

sub simple_header {
    my $self = shift;
    my $include = shift || '';
    return "<head><title>Form Page</title>$include</head>\n";
}



sub show_site_library_link {
    my $class = shift;
    my $result = '';

    my $site_library = Metadot::PortalFactory->get_site_library();
    if ($site_library->is_allowed_to_do('show')) {
        my $library_name = $site_library->get_gizmo_name();
        my $link = "index.pl?isa=".$site_library->get_is_a()."&amp;iid=".$site_library->get_iid();
        $result .= <<EOM
<a class='text_button' href="javascript:void(window.open('$link','siteLibrary','toolbar=no,location=no,status=no,menubar=no,scrollbars,resizable,alwaysRaised,dependent,titlebar=no,width=800,height='+Math.round(screen.availHeight * .75)+',screenX='+this.x+',left='+this.x+',screenY='+this.y+',top='+this.y));">Open $library_name</a>
EOM
;
    }

    return $result;
}


# get the simpleheader html fragment according to whether or not
# a theme is in place.  if theme, expand it for gizmo tags
# (may later want to expand for all cases, but this would slow
# things down).
sub get_form_header_html {
    my ($self, $theme) = @_;

#    print STDERR "get_form_header_html\n";
    my $fb;
    if (Metadot::Utils::Theme->use_theme_info()) {
    $fb = $PARAMS->{'theme_simpleheader'};
    return $self->simple_expand(0, '', $fb);
    }
    else {
    $fb = $PARAMS->{'simpleheader'};
    return $self->simple_expand(0, '', $fb);
    #return $fb;
    }
}

sub form_top {
    my $self = shift;
    $self->debug_msg(3, "Display::form_top()");
#    my $fb = $PARAMS->{'simpleheader'};

    my $theme = $PARAMS->{'template_theme'};
    my $fb = $self->get_form_header_html($theme);

    #aju
    my $formbgcolor;
    if (Metadot::Utils::Theme->use_theme_info()) {
    # for themes, hardwire form bg to light gray -- for now.
    $formbgcolor = Metadot::Utils::Theme->get_theme_default_bg();
    }
    else {
    $formbgcolor = $PARAMS->{'formbgcolor'};
    }
    my $imageslocation = $PARAMS->{'imageslocation'};

    my $s = <<EOM
    <TABLE BGCOLOR="#$formbgcolor" WIDTH=\"625\" CELLSPACING=\"0\" CELLPADDING=\"0\" border="0">\n
        <tr>
        <td colspan='3'>$fb</td>
        </tr>
        <TR>\n
                <TD ><IMG SRC="$imageslocation/pixel.gif" WIDTH="40" HEIGHT="1" BORDER="0" HSPACE="0" VSPACE="0"></TD>\n
                <TD ><IMG SRC="$imageslocation/pixel.gif" WIDTH="540" HEIGHT="1" BORDER="0" HSPACE="0" VSPACE="0"></TD>\n
                <TD ><IMG SRC="$imageslocation/pixel.gif" WIDTH="40" HEIGHT="1" BORDER="0" HSPACE="0" VSPACE="0"></TD>\n
        </TR>

    <!-- end of form_top -->


EOM
;
    return $s;
}

sub form_bottom {
    my $self = shift;
    $self->debug_msg(3, "Portal::form_bottom()");


    my $s;

    $s = "<tr><td>&nbsp;</td><td colspan='2' valign='middle' align='right'>&nbsp;<p />";
    #$s .= $self->print_help_link()."&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\n";
    #$s .= $self->print_logo();
    $s .= $self->show_footer();
    $s .= "</td></tr>";

    $s .= "</table>\n\n";
    return $s;
}


sub print_logo {
    my $self = shift;

    my $result = '';
    unless ($self->is_logo_suppressed()) {
        my $imageslocation = $PARAMS->{imageslocation};
        my $logo = "powered2.gif";
        if (MetadotConfig->get_is_managed_site() eq 'true') {
            $logo = "md-managed.gif";
        }

        $result .= "<a href=\"http://www.metadot.com\"><img align='middle' src=\"$imageslocation" . "/$logo\" border=\"0\"></a>\n";
    }
    return $result;
}

sub print_help_link {
    my $self = shift;
    my $help_url = $PARAMS->{docs_url};
    my $result;

    if ($help_url) {
        my $imageslocation = $PARAMS->{imageslocation};
        $result = "<a href=\"$help_url\" target=\"_blank\"><img align='middle' src=\"$imageslocation/help.gif\" border=\"0\"></a>\n";
    } else {
        $result = '';
    }

    return $result;
}


#######################################################################3

sub print_click_back {
  my $self = shift;

  my ($title, $message)=@_;
  $self->debug_msg(3, "print_click_back()");
  my $top = $self->show_page_top();

  #aju
    #my $formbgcolor;
    #if (Metadot::Utils::Theme->use_theme_info()) {
    #$formbgcolor = Metadot::Utils::Theme->get_theme_default_bg();
    #}
    #else {
    #$formbgcolor = $PARAMS->{'formbgcolor'};
    #}
    ### Daniel wants color to not be modifiable
    my $formbgcolor = 'EEEEEE';
  my $imageslocation = $PARAMS->{'imageslocation'};

  my $html = $self->popup_window."$top<br /><br /><center>\n";
  $html .= "<table border=\"1\" cellpadding=\"1\" cellspacing=\"0\"><tr><td>\n";
  $html .= "<TABLE BGCOLOR='#$formbgcolor' width=\"500\" CELLSPACING=\"0\" CELLPADDING=\"0\" ";
  $html .= "border=\"0\"><tr><TD >";
  $html .= "<IMG SRC='$imageslocation/pixel.gif' WIDTH='150' HEIGHT='1' BORDER='0'";
  $html .= " HSPACE='0' VSPACE='0'></TD>";
  $html .= "<TD><IMG SRC='$imageslocation/pixel.gif' WIDTH='360' HEIGHT='1' \n";
  $html .= " BORDER='0' HSPACE='0' VSPACE='0'></TD>";
  $html .= "<TD><IMG SRC='$imageslocation/pixel.gif' WIDTH='41' HEIGHT='1'\n";
  $html .= " BORDER='0' HSPACE='0' VSPACE='0'>";
  $html .= "</TD></TR><TR><TD valign='top'><br /><br />&nbsp;&nbsp;<img src = \"$imageslocation/my/stop.gif\" border = 0></TD>
  <TD><br />\n";
  $html .= "<H2>$title </H2>\n";

  $html .= "$message\n\n" if ($message);
  $FORM{'errors'} = $message;

  $html .= "<br /><br />";
  $html .= "</center><table align='right'><tr><td><b>\n";

  ## BEGIN AUTO FILL FORMS ##
  if ($FORM{'this_op'}) {

      my ($base, $query) = split '\?', $ENV{'HTTP_REFERER'};
      $html .= "<FORM action=\"$base\" method=\"post\">";

      $FORM{'op'} = $FORM{'this_op'};
      $FORM{'isa'} = $FORM{'this_isa'};
      $FORM{'iid'} = $FORM{'this_iid'};

      foreach my $key (keys %FORM) {
          $html .= "<input type=\"hidden\" name=\"$key\" value=\"".$CGI->escapeHTML($FORM{$key})."\">\n";
      }
      $html .= "Click <input type=\"submit\" value=\"here\" class='button'></a> to correct the problems\n</FORM>";
      ## END AUTO FILL FORMS ##
  } else {
#        $html .= "<a href=\"javascript:history.go(-1);\" class='backlink'>Go back</a> and correct problems<br />";
      $html .= "<a href=\"javascript:history.go(-1);\" class='backlink'>Go back</a> | <a href=\"index.pl\">Home</a><br />";
  }
  $html .= "</td></tr></table>";
  $html .= "</TD><TD>&nbsp;</td></TR></table>\n\n";
  $html .= "</td></tr></table>";
  $html .= $self->close_html;

  $self->print ($html);
}


# an alias for print_click_back
sub click_back {
    my $self = shift;
    return $self->print_click_back(@_);
}

###################################################
sub two_choices {
  my $self = shift;
  $self->print_choices (@_);
}
####################################################
sub three_choices {
  my $self = shift;
  $self->print_choices (@_);
}
#####################################################
sub print_choices {
  my $self = shift;
  my $prompt = shift;
  my $image = shift;
  my $just_return_dialogue = 0;

  $self->debug_msg(3, "Portal::two_choices()");

  if ($image eq "caution") {
      $image = "caution.gif";
  } elsif ($image eq "confirm") {
      $image = "confirm.gif";
  } elsif ($image eq "error") {
      $image = "stop.gif";
  } elsif ($image eq "stop") {
      $image = "stop.gif";
  } elsif ($image eq "confirm_dialogue_only") {
      $image = "confirm.gif";
      $just_return_dialogue = 1;
  } else {
      $image = "readme.gif";
  }

  my $top =  $just_return_dialogue ? '' : $self->show_page_top();

  #aju
  #my $formbgcolor;
  #if (Metadot::Utils::Theme->use_theme_info()) {
  #    $formbgcolor = Metadot::Utils::Theme->get_theme_default_bg();
  #}
  #else {
  #    $formbgcolor = $PARAMS->{'formbgcolor'};
  #}
  my $formbgcolor = 'FFFFFF'; ## Daniel just wants white
  my $imageslocation = $PARAMS->{'imageslocation'};

  my $html = "$top<br /><br /><br /><br /><blockquote><blockquote><center>";
  $html .= "<TABLE border=\"0\" bgcolor=\"#000000\" width=\"50%\"><TR><TD>\n";
  $html .= "<TABLE BGCOLOR=\"#$formbgcolor\" WIDTH=\"100%\" ";
  $html .= "CELLSPACING=\"0\" CELLPADDING=\"2\" BORDER=\"0\">\n";
  $html .= "<TR><TD rowspan=\"2\">&nbsp;<img src=\"$imageslocation/my/$image\" border = 0></td>";
  $html .= "<td><br /><b>$prompt</b></td></tr>\n";
  $html .= "<tr>
            <td align='left' valign='top'>
            <p /><br /><ul>\n";

  while (my $comment = shift) {
      $html .= "<li><a href=\"".$DISPLAY->escapeHREF($self->make_url(shift))."\">$comment</a>\n";
  }

  $html .= "</ul><p /><br /></td></tr></table>\n";
  $html .= "</td></tr></table>\n";
  $html .= $just_return_dialogue ? '' : $self->close_html;

  if ($just_return_dialogue) {
      return $html;
  } else {
      $self->print ($html);
  }
}

sub get_icon_src {
        my $class = shift ;
        my $fileName = shift || '';
        my $imageslocation = $PARAMS->{'imageslocation'} || '';

        my $icon = "unknown.jpg";

        if ($fileName =~ m/(html|htm)$/i) { $icon="html.jpg"; }
        elsif ($fileName =~ m/(mp3|wav|mid|midi|au|mod|snd|aif|aiff|rmd|mpa)$/i) { $icon="audio.jpg"; }
        elsif ($fileName =~ m/(zip|gz|rar|ace)$/i) { $icon="compressed.jpg"; }
        elsif ($fileName =~ m/(jpg|jpeg|gif|bmp|psp|pic|pct|pcx|png|img)$/i) { $icon="image.jpg"; }
        elsif ($fileName =~ m/(mov|qt|mpeg|mpg|avi|asf|mpv)$/i) { $icon="video.jpg"; }
        elsif ($fileName =~ m/(pdf)$/i) { $icon="acroread.jpg"; }
        elsif ($fileName =~ m/(xls|csv)$/i) { $icon="excel.jpg"; }
        elsif ($fileName =~ m/(pl|pm|css)$/i) { $icon="text.jpg"; }
        elsif ($fileName =~ m/(txt)$/i) { $icon="text.jpg"; }
        elsif ($fileName =~ m/(eml)$/i) { $icon="msmail.jpg"; }
        elsif ($fileName =~ m/(ppt)$/i) { $icon="powerpoint.jpg"; }
        elsif ($fileName =~ m/(mpp)$/i) { $icon="project.jpg"; }
        elsif ($fileName =~ m/(vsd|vss|vst|vsw)$/i) { $icon="visio.jpg"; }
        elsif ($fileName =~ m/(exe|dll)$/i) { $icon="winexe.jpg"; }
        elsif ($fileName =~ m/(doc|rtf)$/i) { $icon="word.jpg"; }
        elsif ($fileName =~ m/(mdb)$/i) { $icon="access.jpg"; }
        elsif ($fileName =~ m/(wid)$/i) { $icon="wid.gif"; }    # needed by Curriculum Bank
        else { $icon="text.jpg"; }

        return $imageslocation."/".$icon ;
}


sub require_app {
    my $self = shift;
    my $app = shift || $FORM{app};

    if (!$app) { return; }

    #   print STDERR "app = $app\n\n\n";
    $app =~ s/[(\.|\/|\\)]//g;

    eval { require "App/$app/$app.pm"; };

    if ($@){
    $self->print_click_back("Sorry",
                                "The Application, $app,  is not installed.");
    exit;
    } else {
    require "App/$app/$app.pm";
    }

    return $app;
}


# an older thing, don't really know whether anyone needs it any more
sub make_url {
    my $self = shift;
    my $url = shift;

    my $app = $self->require_app();
    my $output = undef;

    if (!(index($url, "app=")+1)) {
        if ($app){
            if ((index($url, "?")+1)) {
            $output = $url . "&app=$app";
            } else {
            $output = $url . "?app=$app";
            }
        }
    }

    unless ($output){
        $output = $url;
    }

    $self->debug_msg(5, "make_url starts with '$url', returns '$output'");
    return $output;
}


sub make_hidden {
    my $self = shift;

    my $app = $self->require_app();
    my $output = undef;

    if (!$app) { $app = "";}
    $output = "\n<input type=hidden name=app value=\"$app\">\n";

    return $output;
}



####################################################
use constant DEBUG_WEB_PREFIX => "<font color=blue><I>\n";
use constant DEBUG_WEB_SUFFIX => "</I><br /></font>\n";
sub debug_msg {
    my $class = shift;
    my $level = shift;
    my @messages = @_;

    my $msg   = join('', @messages);

    return unless (defined $level);
    return unless (defined $msg);

    $msg = caller().': '.$msg;

    my $ss = $SESSION->{debug_level};
    if (defined $ss && $level <= $ss){
        my $on_the_web = Metadotclass->on_the_web();

        # print to STDOUT if we're a CGI script
        if($on_the_web){
            $class->print_http_header();
            print DEBUG_WEB_PREFIX.$msg.DEBUG_WEB_SUFFIX;
        }

        # either way, print to STDERR
        print STDERR $msg,"\n";
    }
}


sub print_send_to_front {
    my $self = shift;

    my ($title, $message) = @_;
    $self->debug_msg(3, "print_send_to_front()");

    my $top = $self->show_page_top();

    #aju
    my $formbgcolor;
    if (Metadot::Utils::Theme->use_theme_info()) {
    $formbgcolor = Metadot::Utils::Theme->get_theme_default_bg();
    }
    else {
    $formbgcolor = $PARAMS->{'formbgcolor'};
    }

    my $imageslocation = $PARAMS->{imageslocation};

    my $html="$top<br /><br /><br /><br /><center>\n"
          ."<table border=\"1\" cellpadding=\"1\" cellspacing=\"0\"><tr><td>\n"
          ."<TABLE BGCOLOR='#$formbgcolor' WIDTH=\"425\" CELLSPACING=\"0\" CELLPADDING=\"0\" "
          ."border=\"0\"><tr><TD >"
          ."<IMG SRC='$imageslocation/pixel.gif' WIDTH='41' HEIGHT='1' BORDER='0'"
          ." HSPACE='0' VSPACE='0'></TD>"
          ."<TD><IMG SRC='$imageslocation/pixel.gif' WIDTH='360' HEIGHT='1' \n"
          ." BORDER='0' HSPACE='0' VSPACE='0'></TD>"
          ."<TD><IMG SRC='$imageslocation/pixel.gif' WIDTH='41' HEIGHT='1'\n"
          ." BORDER='0' HSPACE='0' VSPACE='0'>"
          ."</TD></TR><TR><TD>&nbsp;</TD><TD><br /><br />\n"
          ."<H2>$title</H2><br />\n";

    $html .= "$message\n\n" if ($message);

    $html .= "<br /><br /><br /><br />";
    $html .= "</center><table align='right'><tr><td><b>\n";

    $html .= "Click <a href='".$self->make_url("index.pl")."'>here</a> to return to the front page.";

    $html .= "</td></tr></table>";
    $html .= "</TD><TD>&nbsp;</td></TR></table>\n\n";
    $html .= "</td></tr></table>";
    $html .= $self->close_html;

    $self->print($html);
}

# Returns an instance of the type requested. This is intended to manage
# GizmoTag objects
my %object_manager;
sub get_gizmotag_object {
    my $class = shift;
#      print STDERR "in get_gizmotag_object\n";

    my $type = shift || die("Must specify the gizmotag type.\n");
    my $params = shift;
    my $result;

    # check to see if I already have a ready instance of the object
    if ($object_manager{$type}) {
        #print STDERR "CKIM: Portal->get_gizmotag_object REUSED module $type\n";
        $result = $object_manager{$type};
    } else {
        eval {
            require "GizmoTag/$type.pm";
        };
        if ($@) {
            #print STDERR "CKIM: Portal->get_gizmotag_object COULD NOT load module $type\n";
        } else {
            #print STDERR "CKIM: Portal->get_gizmotag_object SUCCESSFULLY loaded module $type\n";
            $result = $type->new();
            $object_manager{$type} = $result;
            #CKIM-EVOUCHER: override style class if provided in params
            if ($params->{class}) {
                $result->set_styleclass($params->{class});
            }
        }
    }
    return $result;
}


# Receives one GizmoTag section, and extracts the parameters from the
# opening tag. Parameters are returns as a ref to a hashtable
sub parse_gizmotag_params {
    my ($class, $input, $caller) = @_;

    # first unescape the HTML, in case the WYSIWYG editor escaped it.
    $class->debug_msg(5, "Received gizmotag: ".$CGI->escapeHTML($input));
    $input = $CGI->unescape($input);
    # Unescape doesn't handle &quot; &amp; properly; we do these ourselves
    $input =~ s/\&quot;/"/g;
    $input =~ s/\&amp;/\&/g;

    $class->debug_msg(5, "Unescaped as : ".$CGI->escapeHTML($input));

    # extract the opening tag only
    $input =~ m/[<\[]GIZMOTAG\s+(.*?)\s*[>\]]/i;
    $input = ($1);

    $class->debug_msg(5, "Reduced input to its params: ".$CGI->escapeHTML($input));

    return $class->parse_gizmotag_params_only($input, $caller);
}

sub parse_gizmotag_params_only {
    my ($class, $input, $caller) = @_;

    $input =~ s/&quot;/"/g;

    my %result;
    # extract all the args
    while ($input =~ m/(\S+?)\s*=\s*"(\S+?)"/g) {
        my ($key, $value) = ($1, $2);
        $key = lc($key);
        $result{$key}=$value;
    }
    $result{caller} = $caller;

    #print STDERR "parsed gizmotag params from '$input':\n";
    #foreach my $key (keys(%result)) {
    #    print STDERR "\t$key:\t$result{$key}\n";
    #}

    return \%result;
}

########################################
# DG  simple_expand
# - look for tag and expand them
# - doesn't add/remove anything else than tag
#   i.e. doesn't add admin things and list of gizmos
# use by the templating system
#
sub simple_expand {
    my $class = shift;
    my $iid  = shift;
    my $caller = shift;
    my $rtext = shift;
    my $placeholder = shift;
    my $val;


    #print STDERR "in simple_expand\n";
    my $version = MetadotConfig->get_version();
    my $trial_mode = MetadotConfig->get_site_in_trial_mode();
#      print STDERR "trial mode: $trial_mode\n";
    # aju -- shareM hack, port in a 'param' from metadotconfig to use in the template expansion.
    if (!exists $PARAMS->{site_in_trial_mode} ) {
    $PARAMS->{site_in_trial_mode} = $trial_mode;
    }

    $class->debug_msg(4, "Portal::simple_expand()");
    $class->debug_msg(5, "content: <pre>'$rtext'</pre>");

    #aju 04-7-02: first expand any template toolkit tags in the
    #text (needed for themes support).  Use PARAMS attrs as vars.
    my $text;

    my $template = Template->new({
    }) || die $Template::ERROR, "\n";

    $template->process(\$rtext, $PARAMS, \$text)
      || die $template->error(), "\n";

    $class->debug_msg(5, "Portal::simple_expand() about to expand gizmotags");
    $text = $class->expand_gizmotags($iid, $caller, $text, $placeholder);
    $class->debug_msg(5, "Portal::simple_expand() FINISHED expanding gizmotags");

    #print STDERR "simple_expand() returns\n$text\n\n\n\n\n";
    return $text;
}


sub expand_gizmotags {
    my $class = shift;
    my $iid  = shift;
    my $caller = shift;
    my $text = shift;
    my $placeholder = shift;

    # I will process gizmotags iteratively, in case a gizmotag generates more
    # gizmotags, such as md_catdesc. However, I must avoid infinite loops
    my $iteration_limit = 25;
    my $request_var_key = 'gizmotag_iteration_counter';
    my $i = $REQUEST_VARS{$request_var_key} || 0;

    #print STDERR "expand_gizmotags; placeholder:\n$placeholder\n\n\n\n";

    while ($i<$iteration_limit) {
        $REQUEST_VARS{$request_var_key} = ++$i;
        #print STDERR "expand_gizmotags, iteration #$i\n";

        my @gizmotags = (($text =~ m/[<\[]GIZMOTAG.*?[<\[]\/GIZMOTAG[>\]]/isg));

        # if no gizmotags to process, then exit the loop
        last unless(@gizmotags);

        foreach my $tag_text (@gizmotags)  {
            #print STDERR "in expand_gizmotags with tag text $tag_text\n";

            my $params = $class->parse_gizmotag_params($tag_text, $caller);
            my $md_tag = $params->{name};
            $class->debug_msg(5, "in expand_gizmotags, just parsed gizmotag params-- got md_tag $md_tag");

            if ($md_tag && ($md_tag eq "md_catdesc" && ($placeholder ne ""))) {
                # replace tag by content. Used for search results and others...
                # $class->debug_msg(4, "simple_expand: replacing md_catdesc tag by $placeholder");
                $text =~ s/\Q$tag_text\E/$placeholder/isg;
            } else {
                $text = $class->generate_gizmotag_html($iid, $text, $tag_text, $md_tag, $params);
            }

            #print STDERR "in expand_gizmotags, FINISHED tag text $tag_text\n";
        }
    }

    # any remaining gizmotags will be replaced by an HTML comment indicating that
    # the iteration limit was reached.
    $text =~ s/[<\[]GIZMOTAG.*?[<\[]\/GIZMOTAG[>\]]/<!-- Iteration limit reached before processing GIZMOTAG '$1' -->/isg;

    return $text;
}


sub generate_gizmotag_html {
    my ($class, $iid, $text, $tag_text, $md_tag, $params) = @_;

    my $ref;
    if ($md_tag) {
        $ref = $class->get_gizmotag_object($md_tag, $params);
    }

    $class->debug_msg(5, "$class\->generate_gizmotag_html(md_tag=$md_tag) will use $ref");

    my $res;
    my $wrap;
    if ($ref) {
        eval {
            $res = $ref->md_tag_exec($iid, $params);
        };
        if ($@) {       # catch execution errors
            print STDERR $@;
            $res = "";
            $wrap = "<!-- GizmoTag: name=$md_tag failed -->\n";
            $class->debug_msg(5, "$class\->generate_gizmotag_html: $ref failed to process");
        }
        else {
            $class->debug_msg(5, "$class\->generate_gizmotag_html: $ref finished processing");
            #print STDERR "generate_gizmo_tag expanded\n$wrap\n\n\n\n";

            # print STDERR "gizmo tag comments params " . $params->{no_comments} . "\n";
            if ($params->{no_comments}) {
                $wrap = $res;
            } else {
                $wrap = "\n<!-- GizmoTag: name=$md_tag" . " - " . "version=" . $ref->get_version() . " - Begin -->\n";
                $wrap .= "<!-- authorname=" . $ref->get_authorname() . " - " . "email=" . $ref->get_email() . " -->\n";
                $wrap .= "<!-- url=" . $ref->get_url() . " - " . "license=" . $ref->get_license() . " -->\n";
                $wrap .= "<!-- styleclass=" . $ref->get_styleclass() . " -->\n";
                $wrap .= $res;
                $wrap .= "\n<!-- GizmoTag: name=$md_tag - End -->\n";
            }
        }
    } else {
        $wrap = "<!-- GizmoTag: name=$md_tag failed -->\n";
        $class->debug_msg(5, "$class\->generate_gizmotag_html: $ref failed to process");
    }
    #$class->debug_msg(5, "will replace gizmotag <xmp>$tag_text</xmp> with <xmp>$wrap</xmp>");
    if (defined $tag_text && defined $wrap) {
        $text =~ s/\Q$tag_text\E/$wrap/isg;
    }

    return $text;
}

# a simpler way to access the text of a gizmotag
sub get_gizmotag_simple {
    my $class = shift;
    my $args = shift;

    my $name = $args->{name} || die("Must specify the gizmotag name");
    my $caller = $args->{'caller'};

    # if no caller is specified, just default to the main one. This should be okay for now
    unless (defined($caller) && ref($caller) && $caller->can('get_id')) {
        $caller = Metadot::PortalFactory->get_home_object();
    }
    my $caller_id = $caller->get_id();

    my $param_string = $args->{'params'} || '';

    my $params = $class->parse_gizmotag_params_only($param_string, $caller);
    my $ref = $class->get_gizmotag_object($name, $params);
    my $out;

    if ($ref) {
        $out = $ref->md_tag_exec( $caller_id, $params);
    } else {
        $out = "<!-- Could not get gizmotag named '$name' -->";
    }

    return $out;
}

# an even simpler interface for gizmotags
sub gizmotag {
    my $class = shift;
    my $args = shift;
    if (!ref($args)) {
        $args = {
            name    => $args,
            caller  => Gizmo->restore(0),
        };
    }

    return $class->get_gizmotag_simple($args);
}


sub print_comment_tag {
    my $self = shift;

    # So you found the little Metadot Portal Server and you want to
    # to change it to something else.
    # Please give us a little credit for the work we've done
    # and do not change it. Thanks.

    my $version = MetadotConfig->get_version();
    my $timestamp = $self->print_timestamp();
    my $result = <<EOM
<!--
    Metadot Portal Server v$version  $timestamp
    http://www.metadot.com
-->
EOM
;
    return $result
}


sub print_timestamp {
    my $class = shift;
    my $time  = shift || time;
    return Utility->format_unix_timestamp($time);
}


sub show_admin_button {
  my $self = shift;
  my $caller = shift;
  my $is_header = shift;
  my $s = "";
  my $id = $FORM{id} || $FORM{iid} || 0;
  my $target = $CGI->url(-relative=>1, -query=>1);
  $target =~ s/;/\&/g;
  $target = $CGI->escape($target);
  my $fullperms = "";

#   if($SESSION->get_value('GB_status') eq '1') {
#         return "";
#   }

  my $enableb = "md_enable2.gif";
  my $disableb = "md_disable2.gif";

  my ($global_look) = $PARAMS->{globallook};
  #if ($global_look eq 'usetemplate') {
  #    $enableb = "construct.gif";
  #    $disableb = "construct.gif";
  #}
  if ($is_header) {
      $enableb = "md_enable_top.gif";
      $disableb = "md_disable_top.gif";
  }
  else {
  }
  $self->debug_msg(3, "Portal::show_admin_button()");
  $self->debug_msg(5, "Portal::show_admin_button(), target = $target");

  # XXX [claudio] this is VERY unefficient and should be fixed somehow
  if ($caller->can('check_permissions_of_children')){
    $fullperms = $caller->check_permissions_of_children($id);
  }

  # XXX m.v. ref(self) hack here to get around a problem until we decide
  # what to do with the edit button.
  my $adminlevel = $ACCESS_BROKER->is_allowed_to_do('admin_tasks') ? 9 : 1;

  my $imageslocation = $PARAMS->{'imageslocation'};
  #my $align = "\"absmiddle\"";
  my $align = "\"middle\""; #conformance

  if (ref($caller) eq 'Portal'){
    if((! $SESSION->admin_mode) && ($adminlevel >= 8)){
      $s = "<a href='".$self->make_url("index.pl?op=edit_on&amp;isa=Session&amp;target=$target")."'>"
        ."<img src=$imageslocation/obj/$enableb align=$align border=\"0\" "
          ."title='edit ON'></a>";
    } elsif ($adminlevel < 8) {
      # do nothing
    } else {
      $s = "<a href='".$self->make_url("index.pl?op=edit_off&amp;isa=Session&amp;target=$target")."'>"
        ."<img src=$imageslocation/obj/$disableb align=$align border=\"0\" "
          ."title='edit OFF'></a>\n\n";
    }
  } elsif (($caller->can('is_allowed_to_do') && $caller->is_allowed_to_do('modify', $USER)) ||
           $fullperms){
    # if using new access control and user is allowed to modify the object
    # show edit button.


    if (! $SESSION->admin_mode) {

        if ($USER->is_logged_in()){
        $s = "<a href=\"".$self->make_url("index.pl?op=edit_on&amp;isa=Session&amp;target=$target")."\">"
          ."<img src=\"$imageslocation/obj/$enableb\" align=$align border=\"0\" "
            ."title=\"edit ON\"></a>";
      } else {
        $s = "<a href=\"".$self->make_url("index.pl?op=edit_on&amp;isa=Session")."\">"
          ."<img src=\"$imageslocation/obj/$enableb\" align=$align border=\"0\" "
            ."title=\"edit ON\"></a>";
      }
    } else {

        $s = "<a href=\"" .
        $self->make_url("index.pl?op=edit_off&amp;isa=Session&amp;target=$target").
              "\">";
        $s .="<img src=\"$imageslocation/obj/$disableb\" align=$align border=\"0\" ";
        $s .="title=\"edit OFF\"></a>\n\n";
    }
  }


  return $s;
}


sub using_themes {
    my $self = shift;
    return ($PARAMS->{template_theme} && $PARAMS->{template_theme} ne 'null') ? 1 : 0;
}

##using user defined templates but not themes
sub using_user_defined_templates {
    my $self = shift;

    # modify this so that Template mode also works
    return (
        ($PARAMS->{globallook} eq 'usetemplate' && !$self->using_themes())
        ||
        ($PARAMS->{use_controller})
    );
}

sub export_excel {
    my $self = shift;
    my %args = @_;

    my $data = $args{data};
    my $download_name = $args{download_name};

    my $uploaddir = $PARAMS->{absolutepathfileuploaddir};
    my $file_name = Digest::MD5::md5_hex($ENV{REQUEST_URI});
    my $file = $uploaddir."/".$file_name."Excel.xls";
    my $name = $download_name || $data->{title};
    $name .= '.xls';

    my $workbook   = Spreadsheet::WriteExcel->new($file);

    if (defined $workbook) {
        my $worksheet  = $workbook->addworksheet();
        my $format     = $workbook->addformat();
        my $row = 0;
        my $col = 0;


        #$format->set_bold(1);
        $worksheet->write($row, $col, $data->{title}, $format);

        ## DISPLAY HEADERS
        $row = 1;
        $col = 0;
        foreach my $header (@{$data->{headers}}) {
            $worksheet->write($row, $col++, $header, $format);
        }

        ## DISPLAY BODY
        #$format->set_bold(0);
        my $HEADER_SIZE  = $#{@{$data->{headers}}} + 1;
        my $ENTRIES_SIZE = $#{@{$data->{entries}}} + 1;
        my $entry_count  = 0;
        my $last_cell = 0;

        $row = 2;
        $col = 0;
        for ($entry_count = 0; $entry_count <= $ENTRIES_SIZE; $entry_count += $HEADER_SIZE) {
            $last_cell = $entry_count + $HEADER_SIZE - 1;
            for ($entry_count .. $last_cell) {
                $worksheet->write($row, $col++, $data->{entries}->[$_], $format);
            }
            ## reset and increment
            $col = 0;
            $row++;
        }

        $workbook->close();

        #$self->_send_to_browser($file, $name);
        Metadot::UploadsManager->transmit_file(file_location=>$file, file_name=>$name);
    } else {
        $self->print_click_back("Error", "The server could not create the excel file. Please contact your system administrator.");
    }
}

sub _send_to_browser {
    my $class = shift;
    my ($file, $name) = @_;
    Metadot::UploadsManager->transmit_file(file_location=>$file, file_name=>$name);
}


sub get_content_live_image {
    my $self = shift;
    my $imageslocation = $PARAMS->{imageslocation};
    return "<img src='$imageslocation/obj/greenclock.gif' border='0' alt='content is live' />";
}

sub get_content_live_soon_image {
    my $self = shift;
    my $imageslocation = $PARAMS->{imageslocation};
    return "<img src='$imageslocation/obj/orangeclock.gif' border='0' alt='content scheduled to be live soon' />";
}

sub get_content_expired_image {
    my $self = shift;
    my $imageslocation = $PARAMS->{imageslocation};
    return "<img src='$imageslocation/obj/redclock.gif' border='0' alt='content has expired' />";
}

sub get_traffic_lights_display {
    my $self = shift;
    my %args = @_;

    my $subject = $args{subject};
    my $display = '';

    return unless defined($subject);

    $display .= $self->get_content_live_image() .  $self->get_content_live_soon_image() . $self->get_content_expired_image();
    unless ($SESSION->admin_mode) {
        $display = "<a href=\"index.pl?id=" . $subject->id() . "&amp;isa=Portal&amp;op=get_traffic_lights_choices\">$display</a>";
    }
    return $display;
}

sub get_traffic_lights_display_if_allowed {
    my $self = shift;
    my %args = @_;

    my $subject = $args{subject};
    my $user    = $USER;

    return unless defined($subject) && $user->is_logged_in();

    if ( ($subject->can('user_has_scheduled_content') && $subject->user_has_scheduled_content($user)) || ($subject->can('children_have_scheduled_content_for_user') && $subject->children_have_scheduled_content_for_user($user)) ) {
        return $self->get_traffic_lights_display(subject => $subject);
    }

    return '';
}


use constant HTML_EDITOR_DEFAULT => 'Metadot::Utils::HTMLEditors::HTMLArea';
sub generate_wysiwyg_field {
    my $self = shift;
    my $args = shift;

    my $result;

    my $html_editor_class = MetadotConfig->get_param_value('html_editor_type')
                            || HTML_EDITOR_DEFAULT();

    # try to instantiate the html editor
    eval {
        require Utility->package_to_filename($html_editor_class);
        $result = $html_editor_class->generate_wysiwyg_field($args);
    };

    return $result;
}


# this method is deprecated
sub get_htmlarea_base_script {
    my $class = shift;
    use Metadot::Utils::HTMLEditors::HTMLArea;
    Metadot::Utils::HTMLEditors::HTMLArea->get_htmlarea_base_script(@_);
}



sub is_logo_suppressed {
    my $class = shift;
    return (MetadotConfig->get_param_value('suppress_logo'));
}


sub display_welcome_message {
    my $class = shift;
    my $user  = shift || $USER;

    my $result;
    my $template = MetadotConfig->get_param_value('welcome_message_template') || 'Welcome, [% user.fullname %]!';
    my $tt = Portal->get_portal_template_object();
    $tt->process(\$template, { user => $user }, \$result);

    return $result;
}


# returns the standard template object that all Metadot portals have in common
sub get_portal_template_object {
    return Portal->get_portal_template_object();
}


# an alias for Utility->md_redirect
sub md_redirect {
    my $class = shift;
    return Utility->md_redirect(@_);
}


sub get_calendar_popup_scripts {
    my $class = shift;

    $class->debug_msg(3, "Display::get_calendar_popup_scripts()") ;
    return Metadot::PopupMgr->getCalPopupObj->get_calendar_popup_scripts(@_);
}


sub get_calendar_popup {
    my $class = shift;
    my $args  = Utility->coerce_hashref(@_);

    $class->debug_msg(3, "Display::get_calendar_popup()") ;
    my $name = $args->{name} || die 'Must provide calendar popup name';
    return Metadot::PopupMgr->getCalPopupObj->get_calendar_popup(%$args);
}


# attempts to retrieve the name for a given gizmo class
sub get_gizmo_name_for {
    my $class       = shift;
    my $gizmo_class = shift;

    my $result;
    if (UNIVERSAL::can($gizmo_class, 'get_gizmo_name')) {
        eval {
            $result = $gizmo_class->get_gizmo_name();
        };
        if ($@) {
            $result = $@;
        }
    }

    return $result;
}


# used by the admin bar
sub generate_manage_selection_list {
    my $class = shift;
    my $iid   = shift || 0;

    my $mystuff = Metadot::SystemApp::MyStuff->get_app_name();
    my $result = "<select name=\"manage\" style=\"font-size: smaller;\" onChange=\"javascript:submitMe(this, '$iid', 'Portal', 'modify_site_params')\">
               <option value=\"0\" selected>Manage...\n
               <option value=\"$mystuff\">$mystuff\n
               <option value=\"My Groups\">My Groups\n
               <option value=\"My Profile\">My Profile\n
               ";

               ### site admin can see more
        if($USER->is_admin()) {

            $result .= "<option value=\"Users\">Users\n";

            if ($USER->is_allowed_to_do('configure_profile')) {
                $result .= "<option value=\"configure_profile\">User Profile Settings\n" ;
            }

            $result .= "<option value=\"Channel\">Channels\n
                        <option value=\"Colors\">Style & Colors\n";

            $result .= "<option value=\"SiteLibrary\">Image Library";
            $result .= "<option value=\"SuperGizmos\">" . SuperGizmo->get_gizmo_name() . "s";

            # remove some options from view of unauthorized users
            if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){
                $result .= "
                        <option value=\"Hidden\">Toolbox\n";
                my $gb_status = $SESSION->get_value('GB_status');
                if(defined($gb_status) && $gb_status eq '1') {
                    $result .= "<option value=\"GB_off\">Disable GB\n" ;
                } else {
                    $result .= "<option value=\"GB_on\">Enable GB\n" ;
                }
            }

            $result .= "<option value=\"Config\">Config";
            $result .= "<option value=\"MyPageConfig\">My Page Global Config";

        }
    $result .= "</select>";

    return $result;
}


# used by the admin bar
sub generate_submit_me_script {
    my $self = shift;
    my $out = <<EOM
<script language=JavaScript>
<!--
function submitMe(selectObj, iid, isa, op) {
 if(selectObj.selectedIndex == 0) {
    return(false);
  }
 selectObj.form.iid.value = iid;
 selectObj.form.isa.value = isa;
 selectObj.form.op.value  = op;
 selectObj.form.submit();
}
//-->
</script>
EOM
;
    return $out;
}


# a convenience method to Data::Dumper
sub dumper {
    my $class = shift;
    return '<xmp>'.Data::Dumper::Dumper(@_).'</xmp>';
}


sub get_charset {
    return $PARAMS->{http_charset} || DEFAULT_CHARSET();
}


# a convenience method to make a link with correct escaping and syntax
sub generate_link {
    my $class = shift;

    # process the args
    my  $args = Utility->coerce_hashref(@_);

    # extract the args that do not specify the properties of the link
    my $element_type = $args->{element_type} || 'a';
    my $content      = $args->{content};

    delete $args->{element_type};
    delete $args->{content};

    my $result;
    my @attributes;


    while (my ($key, $value) = each %$args) {
        unless ($key eq 'content') {
            if ($key eq 'href') {
                $value = $class->escapeHREF($value);
                $value = $class->make_url($value);
            }
            push @attributes, qq~$key="$value"~;
        }
    }

    $result = "<$element_type ".join(' ', @attributes).">$content</$element_type>";

    $DISPLAY->debug_msg(5, "generate_link generates <xmp>$result</xmp>");
    return $result;
}


# an alias for CGI->escape
sub escape {
    my $class = shift;
    return $CGI->escape(@_);
}


# an alias for CGI->escapeHTML
sub escapeHTML {
    my $class = shift;
    return $CGI->escapeHTML(@_);
}


# an alias for Utility->escapeHREF
sub escapeHREF {
    my $class = shift;
    return Utility->escapeHREF(@_);
}


1;
