#####
#  FCKeditor - The text editor for internet
#  Copyright (C) 2003-2004 Frederico Caldeira Knabben
#  
#  Licensed under the terms of the GNU Lesser General Public License:
#  		http://www.opensource.org/licenses/lgpl-license.php
#  
#  For further information visit:
#  		http://www.fckeditor.net/
#  
#  File Name: fckeditor.pl
#  	This is the integration file for Perl.
#  
#  Version:  2.0 RC3
#  Modified: 2005-02-28 15:50:35
#  
#  File Authors:
#  		Takashi Yamaguchi (jack@omakase.net)
#####

## Converted into a Perl class by Cory Kim


package Metadot::Utils::HTMLEditors::FCKEditor;

use strict;

use Metadot qw($DISPLAY $CGI $PARAMS);
use Utility;
use Metadot::Utils::HTMLEditors::HTMLEditorBase;

use vars qw(@ISA);
@ISA = qw(Metadot::Utils::HTMLEditors::HTMLEditorBase);



sub generate_wysiwyg_field {
    my $class = shift;
    my $args  = shift;

    my $field_name  = $args->{field_name};
    my $field_value = $args->{field_value};

    return '' unless $field_name;

    my $form_box = '';

    $form_box .= '<div style="text-align: right">'.$DISPLAY->show_site_library_link().'</div>';

    my $editor = $class->new($args);
    $form_box .= $editor->generate_html();

    return $form_box;
}







sub new
{
    my $proto = shift;
    my $args  = Utility->coerce_hashref(@_);

    my $class = ref($proto) || $proto;

    my $self = {};
    $self->{field_name}    = $args->{field_name} || die("Must specify the instance name");
    $self->{base_path}     = $args->{base_path} || '/js/FCKeditor/';
    $self->{width}         = $args->{width}
        || (defined $args->{cols} ? $args->{cols}.'%' : '')
        || '100%';
    $self->{height}        = $args->{height}
        || (defined $args->{rows} ? $args->{rows} * 25 + 110 : '')
        || 500;
    $self->{toolbar_set}   = $args->{toolbar_set} || 'Default';
    $self->{config}        = $args->{config} || {};

    $self->{field_value} = $args->{field_value};
    $self->{field_value} = '' unless(defined($self->{field_value}));

    $self->{config}{BaseHref}   ||= $PARAMS->{basename};

    bless $self, $class;
    return $self;
}


sub specialchar_cnv
{
    my ($class, $ch) = @_;

	$ch =~ s/&/&amp;/g;		# &
	$ch =~ s/\"/&quot;/g;	#"
	$ch =~ s/\'/&#39;/g;	# '
	$ch =~ s/</&lt;/g;		# <
	$ch =~ s/>/&gt;/g;		# >
	return $ch;
}


sub generate_html
{
    my $self = shift;

    # this is actually handled by the Display class, using CGI::escapeHTML, so
    # it's not necessary
	#my $HtmlValue = $self->specialchar_cnv($self->{field_value});
    my $HtmlValue = $CGI->escapeHTML($self->{field_value});

    my $Height = $self->{height};
    my $Width  = $self->{width};

    my $Html = '<div>' ;
	if($self->is_compatible()) {
		my $Link = $self->{base_path} . "editor/fckeditor.html?InstanceName=$self->{field_name}";
		if($self->{toolbar_set} ne '') {
			$Link .= "&Toolbar=$self->{toolbar_set}";
		}
		#// Render the linked hidden field.
		$Html .= "<input type=\"hidden\" id=\"$self->{field_name}\" name=\"$self->{field_name}\" value=\"$HtmlValue\">" ;

		#// Render the configurations hidden field.
		my $cfgstr = $self->get_config_field_string();
		my $wk = $self->{field_name}."___Config";
		$Html .= "<input type=\"hidden\" id=\"$wk\" value=\"$cfgstr\">" ;

		#// Render the editor IFRAME.
		$wk = $self->{field_name}."___Frame";
		$Html .= "<iframe id=\"$wk\" src=\"$Link\" width=\"$Width\" height=\"$Height\" frameborder=\"no\" scrolling=\"no\"></iframe>";
	} else {
        my ($WidthCSS, $HeightCSS);

		if($Width =~ /\%/g){
			$WidthCSS = $Width;
		} else {
			$WidthCSS = $Width . 'px';
		}
		if($Height =~ /\%/g){
			$HeightCSS = $Height;
		} else {
			$HeightCSS = $Height . 'px';
		}
		$Html .= "<textarea name=\"$self->{field_name}\" rows=\"4\" cols=\"40\" style=\"width: $WidthCSS; height: $HeightCSS\" wrap=\"virtual\">$HtmlValue</textarea>";
	}
	$Html .= '</div>';
	return($Html);
}


sub is_compatible
{
    my $class = shift;
    my $iVersion;

	my $sAgent = $ENV{'HTTP_USER_AGENT'};
	if(($sAgent =~ /MSIE/i) && !($sAgent =~ /mac/i) && !($sAgent =~ /Opera/i)) {
		$iVersion = substr($sAgent,index($sAgent,'MSIE') + 5,3);
		return($iVersion >= 5.5) ;
	} elsif($sAgent =~ /Gecko/i) {
		$iVersion = substr($sAgent,index($sAgent,'Gecko/') + 6,8);
		return($iVersion >= 20030210) ;
	} else {
		return(1);
	}
}

sub get_config_field_string
{
    my $self = shift;

	my $sParams = '';
    my $config = $self->{config};

    if ($config) {
        my $bFirst = 0;
        foreach my $sKey (keys %$config) {
            my $sValue = $config->{$sKey};
            if($bFirst == 1) {
                $sParams .= '&';
            } else {
                $bFirst = 1;
            }
            my $k = $self->specialchar_cnv($sKey);
            my $v = $self->specialchar_cnv($sValue);
            if($sValue eq "true") {
                $sParams .= "$k=true";
            } elsif($sValue eq "false") {
                $sParams .= "$k=false";
            } else {
                $sParams .= "$k=$v";
            }
        }
    }

	return($sParams);
}



1;
