###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

package Metadot::Utils::HTMLEditors::HTMLArea;

use strict;

use Metadot qw($DISPLAY $CGI);
use Utility;
use Metadot::Utils::HTMLEditors::HTMLEditorBase;

use vars qw(@ISA);
@ISA = qw(Metadot::Utils::HTMLEditors::HTMLEditorBase);

sub generate_wysiwyg_field {
    my $class = shift;
    my $args  = shift;

    my $field_name  = $args->{field_name};
    return '' unless $field_name;

    my $field_value = $args->{field_value};

    my $rows = $args->{rows} || 20;
    my $cols = $args->{cols} || 80;

    my $field_id = "htmlarea_$field_name";
    $field_value = $CGI->escapeHTML($field_value);

    my $form_box = '';

    $form_box .= '<div style="text-align: right">'.$DISPLAY->show_site_library_link().'</div>';
    $form_box .= "<textarea id=\"$field_id\" class=\"htmlarea\" name=\"$field_name\" style=\"width:100%\" rows='$rows' cols='$cols' WRAP=\"VIRTUAL\">$field_value</textarea>\n";
    $form_box .= $class->convert_field_to_htmlarea($field_id);

    return $form_box;
}




sub get_htmlarea_base_script {
    my $class = shift;
    my $editor_url = MetadotConfig->get_param_value('editor_url') || '/js/htmlarea3/';
    my $result = <<EOM
<script type="text/javascript">
    _editor_url = "$editor_url";
</script>
<!-- load the main HTMLArea files -->
<script type="text/javascript" src="${editor_url}htmlarea.js"></script>

<!-- load the plugins -->
<script type="text/javascript">
      // WARNING: using this interface to load plugin
      // will _NOT_ work if plugins do not have the language
      // loaded by HTMLArea.

      // In other words, this function generates SCRIPT tags
      // that load the plugin and the language file, based on the
      // global variable HTMLArea.I18N.lang (defined in the lang file,
      // in our case "lang/en.js" loaded above).

      // If this lang file is not found the plugin will fail to
      // load correctly and nothing will work.

      HTMLArea.loadPlugin("TableOperations");
      HTMLArea.loadPlugin("SpellChecker");
      HTMLArea.loadPlugin("FullPage");
      HTMLArea.loadPlugin("CSS");
      HTMLArea.loadPlugin("ContextMenu");
</script>

<style type="text/css">
\@import url(${editor_url}htmlarea.css);
</style>
EOM
;
    return $result;
}


sub convert_field_to_htmlarea {
    my $self = shift;
    my $element_id = shift || die("Must specify the form element id");
    my $result = <<EOM
<script type="text/javascript" defer="1">
    HTMLArea.replace("$element_id");
</script>
EOM
;

    return $result;
}











1;

