###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadot::Utils::ModuleMgmt;

use strict;

use Data::Dumper;

use MetadotConfig;
use DBAccess;
use Metadotclass;
use Utility;

use vars qw(@ISA);
@ISA=qw(Metadotclass);

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CGI);


#  sub new {
#      my $proto = shift;
#      my $class = __PACKAGE__;
#      my $self = {};

#      bless ($self, $class);
#      return $self;
#  }


sub get_server_sectionname {
    my $server_name = MetadotConfig->get_server_name();
    my $section_name = MetadotConfig->get_section_name($server_name);
    if (!$section_name) {
    $section_name = MetadotConfig->get_section_name('localhost');
    }
    return $section_name;
}

sub get_supported_modules {
    my $class = shift;

    my $section_name = $class->get_server_sectionname();
#      print STDERR "in get_supported_modules, checking for modules associated with server $server_name " .
#   "and section name $section_name\n";
    my $s_modules = $class->supported_modules_list($section_name);
    my $found_modules = $class->read_modules_in_directory("Gizmo");
#      print STDERR "got found modules: " , Dumper($found_modules) , "\n";
#      print STDERR "got smodules: " , Dumper($s_modules) , "\n";

    # if no list of modules associated with that section name in the db table, then do NO filtering
    if (!(defined $s_modules)) {
    return $found_modules;
    }
    # else, only return the gizmos in the specified list
    my $supported_modules = {};
    foreach my $gizmo (keys %{$found_modules}) {
#   print STDERR "in get_supported_modules, checking $gizmo\n";
    if ($class->member($gizmo, @$s_modules)) {
#       print STDERR "adding $gizmo to supported modules\n";
        $supported_modules->{$gizmo} = $found_modules->{$gizmo};
    }
    }
    return $supported_modules;
}


sub supported_modules_list {
    my ($class, $section_name) = @_;

    # check whether db table exists
    if (DBAccess->get_db_object()->existsTable("vserv_modules")) {
    my ($mstring) = DBAccess->sqlSelect("modules", "vserv_modules", "section_name = '$section_name'");
    if (!$mstring) {
        return undef;
    }
    # split the modules into a list
    my @mlist = split (",", $mstring);
#   print STDERR "supported modules for $section_name: " , Dumper(\@mlist), "\n";
    return \@mlist;
    }
    else {          #  db table not defined
    return undef;
    }
}

#check for membership of a string in a list (does string eq compare only)
sub member {
    my ($class, $item, @mlist) = @_;

    my $elt;
    foreach $elt (@mlist) {
    if ($item eq $elt) {
        return 1;
    }
    }
    return 0;
}

sub read_modules_in_directory {
    my $class = shift;
    my $dir = shift;

    my @lib_dirs = @INC; ## look in INC
#    # will look for modules in directories from PERL5LIB
#    my @lib_dirs = ();
#    if ($ENV{PERL5LIB}) {
#   @lib_dirs = split /:/, $ENV{PERL5LIB};
#    }

#    # add current directory as well
#    # NOTE that PERL5LIb still takes precedence over current dir
#    push @lib_dirs, '.';

    my $found_modules;

    foreach my $lib_dir (@lib_dirs) {

    next unless opendir(DIR, "$lib_dir/$dir");

    my @files_list = readdir(DIR);

    closedir(DIR);

    my @modules_list = grep { s/\.pm$// } @files_list;

    foreach my $module (@modules_list) {
        unless($found_modules->{$module}) {
        $found_modules->{$module} = "$dir/$module.pm";
        }
    }
    }

    return $found_modules;
}


my $available_modules;
sub is_module_available {
    my $class = shift;
    my $module_name = shift;

    unless(defined $available_modules) {
    $available_modules = $class->read_modules_in_directory('.');
    }

    if( exists $available_modules->{$module_name} ) {
    return 1;
    }

    return undef;
}

## will add or remove modules to or from database list if not in supported list
sub modify_modules_in_dblist {
    my $class = shift;
    my %args  = @_;

    my $section_name   = $args{section_name} || $class->get_server_sectionname();
    ## current list of supported modules
    my $supported_modules = $class->supported_modules_list($section_name);
    return unless $section_name && $supported_modules;

    ## list of modules to add
    my $modules_to_add = Utility->arrayify($args{modules_to_add});
    ## list of modules to remove
    my $modules_to_remove = Utility->arrayify($args{modules_to_remove});

    my $mname = '';
    my $update = 0; ## flag to determine if sqlUpdate required
    foreach $mname (@$modules_to_add) {
        ## if not a member then add
        unless(Utility->member($mname,@$supported_modules)) {
            push @$supported_modules, $mname;
            $update = 1;
        }
    }

    my $offset = undef;
    foreach $mname (@$modules_to_remove) {
        $offset = Utility->member_offset($mname,@$supported_modules);
        if (defined($offset)) {## remove module if in the list
            splice(@$supported_modules, $offset, 1); ##remove
            $update = 1;
        }
    }

    #print STDERR "\n\nsupported_modules:" . Dumper($supported_modules) . "\n\n";

    DBAccess->sqlUpdate("vserv_modules", "section_name='$section_name'", modules => join(",", sort @$supported_modules)) if $update;

}



1;
