package Metadot::Utils::ParseExcel::Reader;

use strict;
use Data::Dumper;
use Spreadsheet::ParseExcel;

my $oExcel = new Spreadsheet::ParseExcel;


# if sheetNum is specified, then will return an array of rows
# if firstSheetNum and lastSheetNum are specified, then will return an array of arrays
# if neither is specified, will return an array of arrays

sub read_data {

    my $class = shift;
    my %args = @_;

    my $filename = $args{fileName};
    my $sheetNum = $args{sheetNum};  # default is computed elsewhere
    my @rowKeys  = @{$args{rowKeys}};

    #Metadotclass->debug_msg(5, "read_data has args:<br />\n".Dumper(\%args));

    my $book = $oExcel->Parse($filename) or die "Cannot find file $filename";
    my $data_set;

    # calculate the first and last sheet numbers
    my ($firstSheetNum, $lastSheetNum);
    if (defined $sheetNum) {
        $firstSheetNum = $lastSheetNum = $sheetNum;
        $data_set = undef;
    } else {
        $firstSheetNum = $args{firstSheetNum} || 0;
        $lastSheetNum  = $args{lastSheetNum} || scalar(@{$book->{Worksheet}}) - 1;
        $data_set = [];
    }


    for ($sheetNum = $firstSheetNum; $sheetNum<=$lastSheetNum; $sheetNum++) {
        my $oWkS = $book->{Worksheet}[$sheetNum];
        Metadotclass->debug_msg(5, "read_data processing sheetNum: $sheetNum; worksheet object: $oWkS");
        my @data;

        my($i,$j);
        for ($i = (($oWkS->RowRange())[0])+1; $i<=($oWkS->RowRange())[1]; $i++) {
            Metadotclass->debug_msg(5, "processing row $i");
            my %rowHash;
            for ($j=0; $j<@rowKeys; $j++) {
                if (defined($oWkS->Cell ($i,$j)) && ($oWkS->Cell ($i,$j)->Value())) {
                    $rowHash{$rowKeys[$j]} = uc($oWkS->Cell($i,$j)->Value());
                    Metadotclass->debug_msg(5, "Processing column $j ($rowKeys[$j]): got $rowHash{$rowKeys[$j]}");
                } else {
                    $rowHash{$rowKeys[$j]} = ' ';
                }
            }
            push @data, \%rowHash;
        }

        # if multiple pages are being loaded
        if (defined $data_set) {
            push @$data_set, \@data;
        } else {
        # only one page is being loaded
            $data_set = \@data;
        }
    }

    #Metadotclass->debug_msg(5, "Final data set was:<br />\n".Dumper($data_set));

    if (wantarray) {
        return @$data_set;
    } else {
        return $data_set;
    }
}


