#!/usr/bin/perl

#
# Test file for TransitionalDisplay.pm
# $Id: TransitionalDisplay.001.t,v 1.2 2005/02/03 17:12:32 cory Exp $
#

use strict;
use Test::More tests=>6;

use SmokeMetadot;
use Metadot::Testing::Mechanizer;
use DBAccess;
use MetadotConfig;
use Metadot::PortalFactory;

# set up info
my $class_name   = 'Category';
my $mech = Metadot::Testing::Mechanizer->new();
my $home_page = Metadot::PortalFactory->get_home_object();
my $admin_uid = MetadotConfig->get_reserved_uid('ADMIN');
my $test_page_name = '_FOO_TEST_PAGE_';


################################################################################
# Test: Make sure the category does not yet exist in navigation
################################################################################
ok(!$mech->instance_retrievable(iid=>$home_page->id(), expr=>$test_page_name),
   'Category does not yet exist in navigation');


################################################################################
# Test: Test show_extension capability
################################################################################
use_ok('Gizmo::'.$class_name);

my $page = $class_name->new({
    uid => $admin_uid,
    parent_id => $home_page->id(),
    name => $test_page_name,
});

$page->description(q~This is a test of display.show_extension: ${display.show_extension('hello')}~);
my $page_id = $page->add();

ok(defined($page_id));

ok($mech->instance_retrievable(iid=>$page_id, expr=>'Hello,'),
   'Hello extension was displayed.');


################################################################################
# Test: Test show_extension passing a name arg 
################################################################################
$page->description(q~This is a test of display.show_extension: ${display.show_extension('hello', name='Cory')}~);
$page->save();

ok($mech->instance_retrievable(iid=>$page_id, expr=>'Hello, Cory'),
   'Hello extension with name arg was displayed.');

################################################################################
# Test: Test show_gizmo 
################################################################################

# first create a test gizmo
my $test_gizmo_name = '_TEST_GIZMO_transitionaldisplay.001';
use Gizmo::Item;
my $test_gizmo = Item->new({
    uid => $admin_uid,
    parent_id => $home_page->id(),
    name => $test_gizmo_name,
});

my $test_gizmo_id = $test_gizmo->add();

$page->description('This is a test of display.show_gizmo: ${display.show_gizmo('.$test_gizmo_id.')}');
$page->save();

ok($mech->instance_retrievable(iid=>$page_id, expr=>$test_gizmo_name),
   'show_gizmo displayed the test object.');


################################################################################
# Tear down: delete the test page
################################################################################
$test_gizmo->delete();
$page->delete();


#################################################
# That's it!!!!
#################################################


