###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Metadotclass;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $DISPLAY $PARAMS);

use DBAccess;

use Carp;
use vars qw($AUTOLOAD);


###################################################
sub new {
    my $proto = shift;

    my $class = ref($proto) || $proto;
    my $self = {};
#    $http_header_sent++;
    #$self->{http_header_sent}=0;
    bless ($self, $class);

    return $self;
}


###################################################
sub print {
    my $self = shift;

    if (@_){
    if($self->on_the_web()){
        $self->print_http_header;
    }
    print @_;
    }
}

###################################################
sub on_the_web{
    my $self = shift;
    # do not call debug_msg from here or it will loop
    # since debugmsg is calling on_the_web!!!!

    my $result = ($ENV{SCRIPT_NAME});

    #print STDERR "on_the_web returns $result\n";
    return $result;
}

###################################################
sub handle {
    my $self = shift;
    my $op = shift;
    my $suppress_error = shift;

    $self->debug_msg(3, "Portal::handle($op)");
    ## for HTML RENDERING
    my $handler = "www_$op";

    # aju - add check that the op method is defined to trap a possible server error.
    if ($self->can($handler)) {
        $self->$handler();
        return 1;
    }
    elsif (!$suppress_error) {
        $DISPLAY->print_click_back('Operation not supported');
        exit;
    }
}


####################################################
sub print_attributes{
    my $self = shift;
    $self->debug_msg(5, "print_attributes()");
    my $bugval = $SESSION->debug_level();
    $SESSION->debug_level(5);

    foreach my $k (sort keys %$self){
    if(ref($k)){
        print "found an object";
        $k->print_attributes();
    }else{
        $self->debug_msg(5, "$k: " . $self->{$k} );
    }
    }
    $SESSION->debug_level($bugval);
}

# this is now an alias for Display->debug_msg
sub debug_msg {
    my $class = shift;

    my $display = $DISPLAY || 'Metadot::Utils::Display';
    return $display->debug_msg(@_);
}


################################################
##### is_admin_for_id
##### return 1 if user is admin for $iid
##### 0 otherwise
################################################
sub is_admin_for_id{
  my $self = shift;
  my ($iid, $uid)=@_;

  $self->debug_msg(3, "is_admin_for_id(id=" . ($iid||"") . ", uid=" . ($uid||"") . ")");
  ## here we look if the user is dealing with an
  ## instance that is its own portal or is located
  ## below the top of its own portal

  my $u = Metadot::UserFactory->get_user( $uid );

  my ($ownediid)=$u->has_portal();
  if (!$ownediid) { return 0;};
  if ($self->is_in_tree($ownediid, $iid)){
    return 1;
  }else{
    return 0;
  }
}
##################################################################
#### is_in_tree return 1 if the leaf instance has ancestor $nodeiid
##################################################################
sub is_in_tree{
  my $self = shift;
  my $nodeiid = shift;
  my $leafiid = shift ;
  my $nbcalls = shift || 0;

  $nodeiid = undef unless (Utility->is_valid_integer($nodeiid));    # secu_200401
  $leafiid = undef unless (Utility->is_valid_integer($leafiid));    # secu_200401

  ## true if we went all the way up to the node
  return 0 if (!defined($nodeiid));
  return 0 if (!defined($leafiid));
  $self->debug_msg(5, "is_in_tree($nodeiid, $leafiid, $nbcalls)");

  return 1 if ($nodeiid == $leafiid);

  ## we stop going up the tree (we are at top)
  if ($leafiid <= 0) {return 0};

  if($nbcalls > 1000){
    #looks like we are in an infinite loop
    croak "Metadotclass.pm: Hey man, I'm loop like crazzy here!!!, I think we are in an infinite loop (Metadotclass\->is_in_tree), tell your system admin about this!!!";

  }
  ## call the leaf parent node and call recursively the function
  my ($newleafiid) = DBAccess->sqlSelect("parentiid", "instance", "iid=$leafiid");

  $self->is_in_tree($nodeiid, $newleafiid, $nbcalls++);

}

sub require_app {
    my $self = shift;

    return $DISPLAY->require_app( @_ )

}

sub make_url {
    my $self = shift;
    my $url = shift;

    return $DISPLAY->make_url( $url );

}

sub make_hidden {
    my $self = shift;

    return $DISPLAY->make_hidden();

}

####################################################
sub DESTROY{
  # EMPTY destroy method to please mod_perl when
  # it complains about a missing DESTROY function.
}


#####################################################
sub AUTOLOAD {
    my $self = shift;
    my $type = ref($self) || confess "Metadotclass: hey man: $self is not an object (can't access '$AUTOLOAD')";
    my $name = $AUTOLOAD;
    $name =~ s/.*://;
    unless (exists $self->{$name}){
    confess "Metadotclass: Hey man, Can't access `$name' field in class $type";
    }

    if(@_){
    return $self->{$name} = shift;
    } else {
    return $self->{$name};
    }
}
###################################################
sub print_http_header {
    my $self        = shift;
    $DISPLAY->print_http_header(@_);
}

###################################################
sub get_implementation {
    my $self = shift;
    my $implementation_base = shift;
    my $default = shift;

    my $class = MetadotConfig->get_implementation($implementation_base, $default);
    if (defined $class) {
        no strict 'subs';
        eval("require($class)");
        if ($@) {
            die ("Metadotclass->get_implementation could not load '$class'; errors: $@\n");
        } else {
            #print STDERR "Metadotclass->get_implementation('$implementation_base','$default') INSTALLED $class\n";
        }
    }
    return $class;
}


##########################################################
# aju: this insecure version should be removed.. and no one else
# SHOULD need to use it since the version in GizmoBuilder
# replaces it.

#  sub attachment_icon {
#    my $self = shift;
#    my ($file1) = shift;
#    my $iid = $self->id;
#    my $uid = $self->uid;

#    my $params = $self->{'PARAMS'};
#    my $rootdomain = $params->{'rootdomain'};
#    my $fileuploaddir = $params->{'fileuploaddir'};
#    my $imageslocation = $params->{'imageslocation'};

#    $self->debug_msg(4, "attachmenticon($iid, $file1)");
#    return "" if (!$file1);
#    return "" if (!$iid);
#    my $attach = "&nbsp;" . "<a href=\"http://"
#      . $rootdomain
#      . $fileuploaddir . "/$uid/$iid/$file1\">"
#      . "<img src=\"$imageslocation/download.gif\" border="0" align='center' alt='Download Me!' width=\"16\" height=\"16\"></a>";

#    return $attach;
#}
##########################################################
sub print_error
{
  my $self = shift;
  my $em=shift;
  $self->debug_msg(3, "print_error()");
  $self->print_http_header();
  print <<EOF;
<br /><br /><br /><center>
  <table border="0" bgcolor="#000000" cellpadding="0" cellspacing="0">
    <tr>
      <td>
        <table border="0" width="100%" cellpadding="5" cellspacing="1">
          <tr>
            <td bgcolor="#ffefd5" width="100%">
              <br /><font color="#ff0000"><b>Error -</b></font>
              <BLOCKQUOTE>$em<br /><br /><br /></BLOCKQUOTE>
              <b>Please contact your system administrator if this error persists.</b><br /><br /><br />
            </td>
          </tr>
        </table>
      </td>
    </tr>
  </table>
</center>
EOF
;
    exit(-1);
}


# returns the classname
sub get_class {
    my $proto = shift;
    return ref($proto) || $proto;
}


1;


__END__

=head1 NAME

Metadotclass -

=head1 SYNOPSIS

    use Metadotclass;
    $mdclass = Metadotclass->new ;

    $mdclass->print ;
    $boolean = $mdclass->on_the_web;
    $mdclass->print_attributes;
    $mdclass->debug_msg ($level, $msg);
    $iid = $mdclass->get_new_iid ;
    $cid = $mdclass->get_cid ;
    $uid = $mdclass->get_uid ;
    $boolean = $mdclass->is_admin_for_id ($iid, $uid);
    $boolean = $mdclass->is_in_tree ([$nodeiid, $leafiid, $nbcalls=0]);
    $mdclass->DESTROY;
    $mdclass->AUTOLOAD ;
    $boolean = $mdclass->print_http_header ($header_info_ref);
    $html = $mdclass->attachment_icon ($file1);
    $mdclass->print_error;


=head1 DESCRIPTION

=cut
