###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package MyPage;
use strict;

use Metadot qw($USER %FORM $DISPLAY $SESSION $PARAMS $HTTP_HEADER_SENT $CGI);

use DBAccess;
use Carp;

use Portal;
use GizmoTool;
use CGI;
use HTML::Verify;
use HTML::Components;
use Metadot::Utils::Theme;
use Metadot::AuthenticatorFactory;

use vars qw(@ISA);
@ISA=qw(Portal);


my $AC = {
    BUNDLES => {
    MOD =>    { DESC =>          'Edit',
            OPS =>            [qw(edit_links save_links layout select)],
            DEFAULT_LEVEL =>  2,
            MIN_LEVEL =>      2,
        },
    DISP =>   { DESC =>          'View',
            OPS =>            [qw(show help)],

            DEFAULT_LEVEL =>  0,
            MIN_LEVEL =>      0,
            DEFAULTS_TO =>    'on',
        },
    EDITP =>  { DESC =>          'Change<br />Permissions',
            OPS =>            [qw(edit_permissions set_permissions change_owner)],
            DEFAULT_LEVEL =>  8,
            MIN_LEVEL =>      8,
        },
    },
    DISPLAY_ORDER => [qw(DISP MOD EDITP)],
};

sub get_AC {
    return $AC;
}

sub is_a {
    my $self = shift;

    return ref($self);
}

sub id {
    my $self = shift;

    return 1;
}

sub save {
    my $self = shift;

    $self->save_permissions;
}

sub get_owner_uid {
    my $self = shift;

    return MetadotConfig->get_reserved_uid('ADMIN');
}

sub parent_id {
    my $self = shift;

    return 0;
}

####################################################


sub new {
    my $proto = shift;

    my $class = ref($proto) || $proto;
    my $self = {};

    bless ($self, $class);

    $self->debug_msg(3, "Mypage::new");
    return $self;
}

################################################################
####                  OPERATION HANDLER                    #####
################################################################

sub www_show {
    my $self = shift ;
    $self->debug_msg(3, "MyPage::www_show()");

    unless ( $USER->is_logged_in() ) {

    $USER->logout();
    }

    my $user = $self->_get_user();
    my $uid = $self->defined_or_exit( $user->uid(), "MyPage::www_show - uid is missing.");
    $USER->create_mypage($uid) if (!$user->has_mypage()) ;

    my $name = $USER->nickname() || $USER->fullname();
    my $page = $self->show();

    my $imageslocation = $PARAMS->{'imageslocation'};

    $self->print_http_header();
    print $self->open_html;
    my $stylerules = Metadot::Utils::Theme->get_theme_style_html();
    if (!$stylerules || ($stylerules eq '')) {
    print "\n\n<LINK REL=StyleSheet HREF='$imageslocation/channelcss.css'>\n\n";
    # aju 04-24-02 add 'regular' style rules -- these should override the ones above.
    my $stylesheet = $PARAMS->{stylesheet};
    print "\n" . $stylesheet . "\n";
    }
    else {
    print "\n" . $stylerules . "\n";
    }
#  my $css = $self->show_style_sheet;
#  print $css;
# XX    print start_html "Welcome, $name";
    print "<body BGCOLOR='#FFFFFF'>";
    print $page;
    print "</body>";
}

sub www_layout {
    my $self = shift ;
    $self->debug_msg(3, "MyPage::www_layout()");
    my $layout_form = $self->show_layout_form( $self->_get_user->uid() );
    $self->print_form($layout_form, "MyPage Layout");
}

sub www_select {
    my $self = shift ;
    $self->debug_msg(3, "MyPage::www_layout()");
    my $selection_form  = $self->show_selection_form();
    $self->print_form($selection_form, "Customize My Page and MySubscriptions");
}

sub www_help {
    my $self = shift ;
    $self->debug_msg(3, "MyPage::www_help()");
    my $help = $self->show_help();
    $self->print_form($help, "MyPage Help");
}

sub www_edit_links {
    my $self = shift ;
    $self->debug_msg(3, "MyPage::www_edit_links()");
    my $link_form = $self->show_link_form( $self->_get_user->uid() );
    $self->print_form($link_form, "Customize My Links");
}

sub www_save_links {
    my $self = shift ;
    $self->debug_msg(3, "MyPage::www_save_links()");

    $self->save_links( $self->_get_user->uid() );
    $self->md_redirect("userchannel.pl");
}

################################################################
####                   DISPLAY  METHODS                    #####
################################################################

sub show_help {
    my $self = shift ;
    $self->debug_msg(3, "MyPage::show_help()");

    return "
<br /><h1>My Page Help</H1><br /><br />

<b>Adding and subscribing to channels</b><br />
To add a channel to your page or to change your subscription settings, click 'Configure'.<br /><br />
To modify the order of the channels, click 'Layout'.<br /><br />

<b>Adding other channels to the list of existing ones</b><br />

Users can add channels to their online discussions and category news items. To do so,
just edit the object and select the appropriate choice.<br /><br />

To add a new external channel that is not on the list, ask your site administrator to add it.
Sources of new channels include:
<UL>
  <LI>http://www.moreover.com
  <LI> http://www.xml.tree.com
  <LI> http://www.isyndicate.com
</UL>

<br /><br />

<b>Channel updates</b><br />
Channels are updated on a regular basis, but not in real time.
External channels are updated once every hour.
Internal channels (online discussions, category news items, etc...) are typically updated every 20 minutes.
So be patient!

<br /><br />
<b>Default content for visitors/new accounts</b><br />
Only the site administrator is allowed to modify the default MyPage content.
This default content is used for new accounts, and for visitors to the site who are
not logged in. For information about how to modify the default content, see the Metadot Administrator's
Guide.

<br /><br /><br />
<center><a href='userchannel.pl' class='backlink'>Back to MyPage</a></center><br /><br />
<br /><br />
";
}

sub show_layout_form {
    my $self = shift ;
    $self->debug_msg(3, "MyPage::show_layout_form()");
    my $uid = shift ;
    my $bgcolor = "#FFFFFF";

    $uid = undef unless (Utility->is_valid_integer($uid));    # secu_200401

    my @column ;
    $column[1] = $column[2] = $column[3] = "&nbsp;<br />\n" ;
    my ($items) = DBAccess->sqlSelectMany("id, isa, col", "mypage", "userid=$uid", "order by col, mypageorder");
    while (my ($id, $isa, $col)=$items->fetchrow()) {
    if (defined $id) {
        my $mypage_element = $isa->new( $id, $self->_get_user->uid() );
        if (defined $mypage_element) {
        $column[$col] .= $mypage_element->layout_panel()."<br />\n";
        }
    }
    }

    my $form = "
<H1>Layout for MyPage</h1>
<center>
<table width='100%'><tr>
   <td align='left'><a href='userchannel.pl' class='backlink'>Back to MyPage</a></td>
</tr></table>

<p />
Channel and tools can be moved around your page<br />
by clickling on the up/down/right/left buttons.
<br /><br />

<table width='100%' cellpading=5 cellspacing=\"0\" border=\"1\" height=\"75\"><tr>

<!-- COLUMN 1 -->
<td width='30%' bgcolor=$bgcolor valign='top' align='center'>
$column[1]
</td>

<!-- COLUMN 2 -->
<td width='40%' bgcolor=$bgcolor valign='top' align='center'>
$column[2]
</td>

<!-- COLUMN 3 -->
<td width='30%' bgcolor=$bgcolor valign='top' align='center'>
$column[3]
</td>
</tr></table>

<br /><table width='100%'><tr>
   <td align='center'><a href='userchannel.pl' class='backlink'>Back to MyPage</a></td>
</tr></table>

<br /><br />
";

    return $form ;
}

sub show {
    my $self = shift;
    $self->debug_msg(3, "MyPage::show()");

    my $name = $USER->nickname() || $USER->fullname();
    my $link_bar  = $self->show_link_bar();
    my $header    = $self->mypage_header();
    my $header_message = $self->mypage_message_header();
    my $tool_bar  = $self->show_tool_bar();
    #my $copyright = &showCopyRight();
    my $footer    = $DISPLAY->show_footer();
    my $content   = $self->show_content();
    my @column    = @$content;

    my $login;
    if( $USER->is_logged_in() ) {
        $login = '';
    } else {
        my $authenticator = Metadot::AuthenticatorFactory->get_authenticator();
        $login = $authenticator->show_my_page_login_box();
    }

    my $greeting = "";

    if ($name eq $USER->NAME_DEFAULT_LOOK()) {

        $greeting =  "<NOBR CLASS=\'MYPAGEWELCOME\'>Welcome, Please Join Our Online Community!<\/nobr>";

    } else {

        my $welcome = $DISPLAY->display_welcome_message();
        if( $USER->is_logged_in() ) {

            $greeting = "<NOBR CLASS=\'MYPAGEWELCOME\'>$welcome<\/nobr>";

        } else {

            $greeting = "<NOBR CLASS=\'MYPAGEWELCOME\'>$welcome<\/nobr>";
            $greeting .= "<br /><NOBR class=\"welcomesub\">You are not currently logged in.<\/nobr>";

        }
    }

    my $returnvar = "
<!-- Beginning of Link Bar -->
$header

<!-- End of Link Bar -->

<SCRIPT LANGUAGE=\"JavaScript\">
var newWindow
function makeNewWindow() {
    newWindow = window.open(\"\",\"\",\"status,height=100,width=200\")
}

function subWrite(message) {
    makeNewWindow()
    var newContent = \"<HTML><HEAD><TITLE>Channel Status</TITLE></HEAD>\"
    newContent += \"<BODY><H3>\" + message
    newContent += \"</H3></BODY></HTML>\"
    newWindow.document.write(newContent)
    newWindow.document.close()
}
</SCRIPT>

<!-- Beginning of Header -->

$link_bar

$header_message
<!-- End of Header -->


<!-- Beginning of Welcome + Tool Bar -->

<TABLE WIDTH=\"100%\" CELLPADDING=\"0\" CELLSPACING=\"0\" BORDER=\"0\"><TR>
<TD ALIGN='left'>";

#  <NOBR CLASS='YOUAREHERE'>You are here:&nbsp;<A HREF=index.pl>Home</a>&nbsp;&gt; My Page</div><br />

    $returnvar .= "
  $greeting
</TD>
<TD ALIGN='right'>
$tool_bar
</TD>
</TR></TABLE>

<!-- End of Welcome + Tool Bar -->


<!-- Beginning of Content -->

<TABLE WIDTH=\"100%\" CELLPADDING=\"2\" CELLSPACING=\"2\" BORDER=\"0\" HEIGHT=\"200\">

<!-- COLUMN 1 -->
<TD width=\"25%\" Valign='TOP'>
  $login
  $column[1]
</TD>

<!-- COLUMN 2 -->
<TD width=\"50%\" Valign='TOP'>
  $column[2]
</TD>

<!-- COLUMN 3 -->
<TD width=\"25%\" Valign='TOP'>
  $column[3]
</TD>

</TR></TABLE>

<!-- End of Content -->


<!-- Beginning of Copyright -->

$footer

<!-- End of Copyright -->

</BODY>
";

    return $returnvar;
}

sub show_tool_bar {
    my $self = shift;
    $self->debug_msg(3, "MyPage::show_tool_bar()");


    my $imageslocation = $PARAMS->{'imageslocation'};

    my $version = MetadotConfig->get_version();

    # aju: this needs to be factored out of the perl code!

    my $js_string = "<script language=\"JavaScript\">
<!--

function newImage(arg) {
    if (document.images) {
        rslt = new Image();
        rslt.src = arg;
        return rslt;
    }
}


var preloadFlag = false;
 -->
</script>";

    # toggle the login/logout action
    my ($login_action, $login_method, $login_image);
    if ($USER->is_logged_in()) {
        $login_action = 'logout';
        $login_method = 'get_logout_url';
        $login_image  = 'my/mypage_nav/logout.gif';
    } else {
        $login_action = 'login';
        $login_method = 'get_login_url';
        $login_image  = 'my/mypage_nav/login.gif';
    }


    my $navtable_string = "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\">
       <tr><td><img src=\"$imageslocation/spacer.gif\" height=\"2\" alt='' /></td></tr>
       <tr>
        <td><a href=\"./userchannel.pl?isa=MyPage&amp;op=layout\">
    <img alt=\"layout\" src=\"$imageslocation/my/mypage_nav/layout.gif\" border=\"0\"></a>
         &nbsp;</td>
        <td><a href=\"./userchannel.pl?isa=MyPage&amp;op=select\">
    <img alt=\"configure\" src=\"$imageslocation/my/mypage_nav/configure.gif\" border=\"0\"></a>
          &nbsp;</td>
        <td><a href=\"".Metadot::AuthenticatorFactory->get_authenticator()->$login_method("page=mypage")."\">
    <img alt=\"$login_action\" src=\"$imageslocation/$login_image\" border=\"0\"></a>
          &nbsp;</td>
     </tr></table>";

    return $js_string . "\n" . $navtable_string;



}

sub show_link_bar {
    my $self = shift ;
    $self->debug_msg(3, "MyPage::show_link_bar");

    my @link_array = $USER->load_mypage_links( $self->_get_user->uid() );
    my $hpid = $self->_get_user->has_portal || 0;

    my $imageslocation = $PARAMS->{'imageslocation'};

    my @link_list;
    foreach my $link_hash (@link_array) {
    push(@link_list, "<a class='MYLINKS' href='$link_hash->{value}'>$link_hash->{name}</a>\n") if ($link_hash->{name});
    }

    my $linkstr = join("<font class='MYLINKS'> | </FONT>\n",@link_list);

    my $str = "<table class='MYLINKS' border=\"0\" cellpadding=\"2\" cellspacing=\"0\"><tr>
<td class='MYLINKS' width=\"10%\" height=\"25\" align='left' valign='middle' nowrap=\"nowrap\">
&nbsp;<b><a class='MYLINKS' href='index.pl'>Home</a></b>";

    if( $PARAMS->{my_website_creation} ) {
    $str .= "&nbsp;&nbsp;<b><a class='MYLINKS' href='index.pl?isa=Portal&amp;op=myportal'>My Website</a></b>";
    }

    $str .= "</td>
<td class='MYLINKS' width=\"100%\" height=\"25\" align='right' valign='middle' nowrap=\"nowrap\">
&nbsp;$linkstr
</td>
<td class='MYLINKS' height=\"25\" align='right' valign='middle' nowrap=\"nowrap\">
<a href='userchannel.pl?isa=MyPage&amp;op=edit_links'><img src=$imageslocation/obj/edit.gif width=\"25\" height=\"11\" border=\"0\" align='top' alt='Edit Me!'></a>
</td>
</tr></table>
";

    return $str;
}

sub show_content {
    my $self = shift;
    $self->debug_msg(3, "MyPage::show_content()");

    my $uid = $self->_get_user->uid();
    my @column;

    my $other_mypage_modules_ref = {
                                NewsChannel => 1,
                                BasicTool   => 1,
                               };

    my %other_mypage_modules = %{$other_mypage_modules_ref};

    opendir(DIR, "./GizmoTool") or die "Cannot open directory: ./GizmoTool.";
    my @files = readdir(DIR);
    closedir(DIR);

    my %gizmo_tool_modules;
    foreach my $file (@files) {
        $file =~ s/\.pm$//;
        $gizmo_tool_modules{$file} = 1;
    }

    my ($elements) = DBAccess->sqlSelectMany("id, col, isa", "mypage", "userid=$uid", "order by col, mypageorder");
    while(my ($id, $col,$isa)=$elements->fetchrow()) {
        if (($gizmo_tool_modules{$isa}) || ($other_mypage_modules{$isa})) {
        my $element = $isa->new($id, $self->_get_user->uid());
        if (defined $element) {
        $column[$col] .= $element->display($id);
        }
        else {
        $column[$col] .= "";
        }
        } else {
            $column[$col] .= "";
        }
    }
    $elements->finish();

    return \@column;
}


##################################################################################
sub show_selection_form {
    my $self = shift ;
    $self->debug_msg(3, "MyPage::show_selection_form");

    my $type = $FORM{type} || "External" ;
    my $choices = $self->get_user_choices( $self->_get_user->uid() ); #it is a hash ref

    # build $url for PREVIOUS/NEXT MAGIC
    my $url = "userchannel.pl?isa=MyPage&amp;op=select";
    $url   .= "&amp;filter=$FORM{filter}" if ($FORM{filter}) ;

    # build switching tabs
    my $tabs = "";
    $tabs .= $self->_show_tab("External News","userchannel.pl?isa=MyPage&amp;op=select&amp;type=External",($type eq 'External'));
    $tabs .= $self->_show_tab("Internal News","userchannel.pl?isa=MyPage&amp;op=select&amp;type=Internal",($type eq 'Internal'));
    $tabs .= $self->_show_tab("Discussions","userchannel.pl?isa=MyPage&amp;op=select&amp;type=Discussion",($type eq 'Discussion'));
    $tabs .= $self->_show_tab("Tools","userchannel.pl?isa=MyPage&amp;op=select&amp;type=Tool",($type eq 'Tool'));
    $tabs .= $self->_show_tab("My Selection","userchannel.pl?isa=MyPage&amp;op=select&amp;type=Selection",($type eq 'Selection'));

    # buid search form
    my %labels = ( Everywhere => 'Everywhere',
           External   => 'External News',
           Internal   => 'Internal News',
           Discussion => 'Discussions',
           Tool       => 'Tools',
           Selection  => 'My Selection');
  my $popup = $CGI->popup_menu(-name => 'type', -class => 'newsitem', -default => $type,
                 -values => ['Everywhere', 'External', 'Internal', 'Discussion', 'Tool', 'Selection'],
                 -labels => \%labels
                 );

    my $serch_form = "<form action=$ENV{SCRIPT_NAME} method='get'>
                <td align='right' valign='center'>
                $popup
                <input type='text' size='15' name='filter' value=\"" . ($FORM{filter}||"") . "\">
                <input type='hidden' name='isa' value='MyPage'>
                <input type='hidden' name='op' value='select'>
                <input type='hidden' name='redirect' value='1'>
                <input type='submit' value='Search' class='button'></td></form>
                </tr>";

    # build the checkboxes table(s)
    my $list ;
    my $filter = "";
    my $s ="";

    $filter = "and upper(name) like " . DBAccess->quote( uc("%$FORM{filter}%")) if ($FORM{filter});
    my  $ccid  = $FORM{ccid} ? $FORM{ccid} : 0;

    my ($from, $where, $other);
    $from = 'channel';
    $other = 'order by name';

    if (($type eq "External") || ($type eq "Internal") || ($type eq "Discussion")) {
        if($ccid && $ccid ne 'All') {
            $where = "ccid='$ccid' AND isa='$type' $filter";
        } else{
            $where = "isa='$type' $filter";
        }

        $list = DBAccess->sqlSelectMany("cid, name, isa, iid", $from, $where, $other);
        my ($list_count) = DBAccess->sqlSelect('count(*)', $from, $where, $other);

        $s .= $self->_show_table_choices("NewsChannel", $list, $list_count, $choices, "$url&amp;type=$type", $ccid, $type);

        } elsif ($type eq "Tool") {

        my $parent_basic_tool = MetadotConfig->get_reserved_iid('PARENT_BASIC_TOOL');

        my $from = 'instance';
        $where = "parentiid=$parent_basic_tool and (isa='Item' or isa='LoginLink') $filter";
        $list = DBAccess->sqlSelectMany("iid, name", $from, $where, $other);
        my ($list_count) = DBAccess->sqlSelect('count(*)', $from, $where, $other);
        $s .= "<p /><center><b>Basic Tools</b></center>" . $self->_show_table_choices("BasicTool", $list, $list_count, $choices, "$url&amp;type=$type");

        #gizmotool list
        my @gtnames = GizmoTool->lists_tools();
        if (@gtnames) {
            $from = 'channel';
            my $where_gizmo_tool = join(" OR ", map("isa='$_'",@gtnames));
            $where = "($where_gizmo_tool) $filter";
            my $gtlist = DBAccess->sqlSelectMany("cid, name, isa, iid", $from, $where, $other);
            my ($list_count) = DBAccess->sqlSelect('count(*)', $from, $where, $other);
            $s .= "<p /><center><b>Gizmo Tools</b></center>" . $self->_show_table_choices(undef, $gtlist, $list_count, $choices, "$url&amp;type=$type");
        }
    }  elsif (($type eq "Selection") || ($type eq "Everywhere")) {

        my @types = ("External","Internal","Discussion","BasicTool","GizmoTool");
        foreach my $isa (@types) {
        my $title = ($isa =~ m/ternal$/) ? "<b>$isa Channels</b>\n" : "<b>".$isa."s</b>\n";
        my $table = "";
        my $selection = "";
        if ($isa eq "BasicTool") {
        if ($type eq "Selection") {
            $selection = join(", ", keys(%{$choices->{BasicTool}}));
            $selection = ($selection) ? " and iid in ($selection)" : " and 1=0" ;
        }
        my $parent_basic_tool = MetadotConfig->get_reserved_iid('PARENT_BASIC_TOOL');

        my $from = 'instance';
        $where = "parentiid=$parent_basic_tool and (isa='Item' or isa='LoginLink') $filter $selection";
        $list = DBAccess->sqlSelectMany("iid, name", $from, $where, $other);
        my ($list_count) = DBAccess->sqlSelect('count(*)', $from, $where, $other);

        $table = $self->_show_table_choices("BasicTool", $list, $list_count, $choices, "$url&amp;type=$isa");
        } elsif ($isa eq "GizmoTool") {
        #gizmotool list
                my @gtnames = GizmoTool->lists_tools();
                if (@gtnames) {
            if ($type eq "Selection") {
                    $selection = join(", ", grep (!/^$/, (map { join(", ", keys(%{$choices->{$_}})); } @gtnames)));
                    $selection = ($selection) ? " and cid in ($selection)" : " and 1=0" ;
            }
            my $where_gizmo_tool = join(" OR ", map("isa='$_'",@gtnames));
            $where = "($where_gizmo_tool) $filter $selection";
            my $gtlist = DBAccess->sqlSelectMany("cid, name, isa, iid", $from, $where, $other);
            my ($list_count) = DBAccess->sqlSelect('count(*)', $from, $where, $other);
            $table = $self->_show_table_choices(undef, $gtlist, $list_count, $choices, "$url&amp;type=$type");
                }
        } else {
        if ($type eq "Selection") {
            $selection = join(", ", keys(%{$choices->{NewsChannel}}));
            $selection = ($selection) ? " and cid in ($selection)" : " and 1=0" ;
        }

        $where = "isa='$isa' $filter $selection";
        $list = DBAccess->sqlSelectMany("cid, name, isa, iid", $from, $where, $other);
        my ($list_count) = DBAccess->sqlSelect('count(*)', $from, $where, $other);
        $table = $self->_show_table_choices("NewsChannel", $list, $list_count, $choices, "$url&amp;type=$isa");
        }
        $s .= "$title\n$table<br />" if ($table);
    }
    }


#my $css = $self->show_style_sheet;
    my $contents;
#$contents .= $css;

    my $imageslocation = $PARAMS->{'imageslocation'};

    $contents .= "<LINK REL=StyleSheet HREF='$imageslocation/channelcss.css'>";

    $contents .= "<H2>Customizing My Page</H2>

        <table width='100%'><tr>
        <td align='left'>
        <a href='userchannel.pl' class='backlink'>Back to My Page</a>
        </td>
        $serch_form
        </table>

        <table width='100%' cellpadding=\"2\" cellspacing=\"2\" border=\"0\">
$tabs
<tr><td class='CUSTOMSELECTEDTAB' colspan='5'>";

    if(uc($type) eq "EXTERNAL"){
        my $catg_popup = $self->category_popup();
        $contents .= "<form action='userchannel.pl' method='get'>External News Categories: $catg_popup
                        <input type='hidden' name='isa' value='MyPage'>
                        <input type='hidden' name='op' value='select'>
                        <input type='hidden' name='type' value='External'>
                        <input type='submit' value='Go' class='button'>
                        </form> ";
    }
    $contents .="
        <small>Use the checkboxes to add elements to your My Page and to subscribe to news delivery.
        Click the item name to see a preview.</small>
        <br /><br />

        <center>

        $s

        <br />
        <a href='userchannel.pl' class='backlink'>Back to My Page</a>

        </center><br />

        </td></tr></table>
        <br /><br />

";

    return $contents;

}

###################################################
sub category_popup {
    my $self = shift;
    my $ccid = shift || "All";
    my %CATEGORIES;
    $CATEGORIES{"All"} = "All";
    my $c = DBAccess->sqlSelectMany("ccid, cc_name", "channel_category", "ccid > 0", "order by cc_name");
    while(my($k, $v)=$c->fetchrow()){
        $CATEGORIES{$k} = $v;
    }

    my $popup = $CGI->popup_menu(-name => 'ccid', -default => $ccid, -values => [sort { $CATEGORIES{$a} cmp $CATEGORIES{$b} }(keys(%CATEGORIES))], -labels => \%CATEGORIES);

    return $popup;
}

sub _show_tab {
    my $self = shift;
    $self->debug_msg(3, "MyPage::_show_tab()");
    my $name = shift ;
    my $url = shift ;
    my $selected = shift ;

    my $class = "";
    my $n = $name;
    if ($selected) {
    $class = "CUSTOMSELECTEDTAB";
    $name = "<font class='CUSTOMSELECTEDTAB'>$name</font>";
    }  else {
    $class = "CUSTOMTAB";
    $name = "<a class='CUSTOMTAB' href='$url'>$name</a>";
    }
    return "<td class=$class width='20%' height='35' align='center'>$name</td>";
}


sub _show_table_choices {
    my $self = shift;
    $self->debug_msg(3, "MyPage::show_table_choices()");
    my $type = shift ;  # NewsChannel | BasicTool | one of GizmoTool
    my $list = shift ; # SQL answer : list of (id,name)
    my $list_count = shift; # number of rows in table
    my $choices = shift ;
    my $url = shift ;
    my $ccid = shift;
    my $orig_type = shift;  # aju -- additional optional arg
    ## We need to check if $type was initially defined
    ## It will be undefined for GizmoTools
    ## $type will equal isa if initially undefined
    my $type_initially_defined = $type ? 1 : 0;


    # prepare formating information
    my $from = $FORM{from} || 0;
    my $max  = $FORM{nb} || 60 ;
    my $nbdisp = 0 ;
    my $subs = (defined $type && $type eq "NewsChannel") ? 1 : 0 ;

    # format the checkboxes table
    my $table = "" ;
    while ((my ($id, $name, $isa, $iid, $method) = $list->fetchrow) && ($nbdisp<$max)) {
    if ($from) {$from--; next;}

    # comment this line because it does not do null checking, generates a ton of warnings
    # $self->debug_msg(5, "id:$id name:$name isa:$isa iid:$iid method:$method orig_type:$orig_type");

    $nbdisp++;
    $type = $isa unless ($type_initially_defined);
    # aju 04-26-02
    # check that user is allowed to view the channel.
    if (defined $orig_type) {
        if ($orig_type eq 'Discussion') {
        my $disc = Discussion->new($iid);
        if ($disc->is_allowed_to_do('show', $USER)) {
            $table .= $self->_show_line_choices($id, $iid, $isa, $type, $name, $choices->{$type}->{$id}, $subs, $method);
        }
        }
        elsif ($orig_type eq 'Internal') {
                if (Gizmo->exists($iid)) {
            my $cat = Gizmo->new($iid);
            if ($cat->is_allowed_to_do('show', $USER)) {
                $table .= $self->_show_line_choices($id, $iid, $isa, $type, $name, $choices->{$type}->{$id}, $subs, $method);
            }
                }
        }
        else {
        $table .= $self->_show_line_choices($id, $iid, $isa, $type, $name, $choices->{$type}->{$id}, $subs, $method);
        }
    }
    else {
        $table .= $self->_show_line_choices($id, $iid, $isa, $type, $name, $choices->{$type}->{$id}, $subs, $method);
    }
    }

    $list->finish;

    return "" if (!$list_count);

    #PREVIOUS NEXT MAGIC
    my $PNbuttons = ($list_count>$max) ? Components->get_previous_next_button($max, $nbdisp, $list_count, $FORM{from}, $url, $ccid) : "";

    # select the right header with/without subscription
    my $table_header = ($subs) ?
    "<tr>
<th class='CUSTOMMYPAGE' width=\"200\">&nbsp;</th>
<th class='CUSTOMMYPAGE' align='center'>&nbsp;</th>
<th class='CUSTOMSUBSCRIPTION' align='center' colspan='3'><b>Email Subscription</b></th>
<th class='CUSTOMSUBSCRIPTION' align='center'><b>&nbsp; Pager &nbsp;</b></th>
<th class='CUSTOMSUBSCRIPTION' align='center'><b>ICQ</b></th>
</tr>
<tr>
<th class='CUSTOMMYPAGE' width=\"200\">&nbsp;</th>
<th class='CUSTOMMYPAGE' align='center' valign='center'><b>Show in My Page</b></th>
<th class='CUSTOMSUBSCRIPTION' align='center' valign='center'><b>Daily</b></th>
<th class='CUSTOMSUBSCRIPTION' align='center' valign='center'><b>Weekly</b></th>
<th class='CUSTOMSUBSCRIPTION' align='center' valign='center'><b>ASAP</b></th>
<th class='CUSTOMSUBSCRIPTION' align='center' valign='center'><b>ASAP</b></th>
<th class='CUSTOMSUBSCRIPTION' align='center' valign='center'><b>ASAP</b></th>
</tr>" :
    "<tr>
<th class='CUSTOMMYPAGE' width=\"200\">&nbsp;</th>
<th class='CUSTOMMYPAGE' align='center'><b>Show in My Page</b></th>
</tr>";

    return "
<!-- Beginning  Choices Table for " . ($type||"") . "-->

$PNbuttons
<table width='90%' border=\"2\" cellspacing=\"0\" cellpadding=\"0\">
$table_header
$table
</table>
$PNbuttons

<!-- End of Choices Table for " . ($type||"") . " -->
" ;
}

sub _show_line_choices {
    my $self = shift;
    $self->debug_msg(3, "MyPage::_show_line_choices()");
    my $id   = shift ;
    my $iid   = shift ;
    my $isa  = shift ;
    my $type  = shift ; # NewsChannel | BasicTool | one of GizmoTool
    my $name = shift ;
    my $choices = shift ;
    my $subs = shift ;
    my $medium=shift;
#  my $medium="email";

    ## AC enforcement
    if (lc($isa) eq 'discussion') {
    my $disc = Discussion->new($iid);
    return unless($disc->is_allowed_to_do('show', $USER));
    } elsif (lc($isa) eq 'internal') {
    my $cat = Gizmo->new($iid);
    return unless($cat->is_allowed_to_do('show', $USER));
    }

    # name with preview link + mypage checkbox
    # XXX JMS change isa parameter to type
    my $preview = "<a class='CUSTOMMYPAGE' href='userchannel.pl?id=$id&amp;isa=$type&amp;op=preview'>$name</a>";
    my $checkbox = $self->_show_checkbox("userchannel.pl?id=$id&amp;isa=$type&amp;redirect=1",$choices->{mypage});
    my $s = "<tr>\n<td class='CUSTOMMYPAGE' width=\"200\">$preview</td>\n";
    $s .= "<td class='CUSTOMMYPAGE' align='center'>$checkbox</td>\n";
    # if subscription are meaningfull
    if ($subs) {
    $checkbox = $self->_show_checkbox("userchannel.pl?id=$id&amp;isa=Subscription&amp;freq=onceaday",$choices->{onceaday}->{email});
    $s .= "<td class='CUSTOMSUBSCRIPTION' align='center'>$checkbox</td>\n";
    $checkbox = $self->_show_checkbox("userchannel.pl?id=$id&amp;isa=Subscription&amp;freq=onceaweek",$choices->{onceaweek}->{email});
    $s .= "<td class='CUSTOMSUBSCRIPTION' align='center'>$checkbox</td>\n";
    $checkbox = $self->_show_checkbox("userchannel.pl?id=$id&amp;isa=Subscription&amp;freq=all",$choices->{all}->{email});
    $s .= "<td class='CUSTOMSUBSCRIPTION' align='center'>$checkbox</td>\n";

# This one is for the pager option.
    $checkbox = $self->_show_checkbox("userchannel.pl?id=$id&amp;isa=Subscription&amp;freq=all&amp;medium=pager",$choices->{all}->{pager});
    $s .= "<td class='CUSTOMSUBSCRIPTION' align='center'>$checkbox</td>\n";
    $checkbox = $self->_show_checkbox("userchannel.pl?id=$id&amp;isa=Subscription&amp;freq=all&amp;medium=icq",$choices->{all}->{icq});
    $s .= "<td class='CUSTOMSUBSCRIPTION' align='center'>$checkbox</td>\n";
    }
    $s .= "</tr>\n\n";
}

sub _show_checkbox {
    my $self = shift;
    $self->debug_msg(3, "MyPage::_show_checkbox()");
    my $url = shift ;
    my $checked = shift ;

    my $imageslocation = $PARAMS->{'imageslocation'};

    my $box = ($checked) ? "<img src='$imageslocation/checked.gif' border='0' alt='Delete Me!'>" :
    "<img src='$imageslocation/unchecked.gif' border='0' alt='Add Me!'>";
    my $action = ($checked) ? "delete" : "add";

    return "<a href='$url&amp;op=$action'>$box</a>";

}

sub show_link_form {
    my $self = shift;
    $self->debug_msg(3, "MyPage::show_link_form()");
    my $uid = shift ;
    my %link_hash;
    my @name_list;
    my @url_list;

    my @link_array = $USER->load_mypage_links($uid);

    for my $link_hash (@link_array)
    {
    push(@url_list, $link_hash->{value});
    push(@name_list, $link_hash->{name});
    }


    my $table = "";

    my $maxlinks = $PARAMS->{'maxlinks'};

    for(my $i=1; $i<=$maxlinks; $i++){
    my $url = shift @url_list ;
    my $name = shift @name_list ;

    $table .= "<tr><td><input type='text' name='linkname$i' size='18' maxlength='60' value='$name'>\n";
    $table .= "</td><td><input type='text' name='linkurl$i' size='25' maxlength='255' value='$url'><br />\n";
    $table .= "</td></tr>";
    }

  return "
<form action='$ENV{SCRIPT_NAME}' method='post'>
<input type='hidden' name='cancelurl' value='userchannel.pl'>
<input type='hidden' name='isa' value='MyPage'>
<input type='hidden' name='op' value='save_links'>
<H1>Customizing My Links</H1>
The links you enter below will appear in the link bar located on the top of your page.
<br /><br />
<center>
<table border=\"0\" cellspacing=\"8\">
<tr><th><b>Link Name</b> (e.g. Yahoo)</th><th><b>URL</b> (e.g. http://www.yahoo.com)</th></tr>
$table
</table>

<table width=\"100%\" border=\"0\">
<tr><td align='center'>
<input type='submit' value='Save' class='button'>
<br />
<br />
<a href='userchannel.pl' class='backlink'>Back to MyPage</a>
</td></tr>
</table>
</center>

<br />
</form>
";
}

################################################################
####                  DATABASE  METHODS                    #####
################################################################

sub mypage_header {
    my $self = shift ;
    $self->debug_msg(4, "MyPage::mypage_header");

    my $mpheader = $PARAMS->{mypageheader};
    if ($mpheader && ($mpheader ne '')) {
    return $mpheader;
    }
    # if reach this point, mpheader was not defined
    if (!$PARAMS->{use_mypagehdr_default}) {
    return '';
    }
    #else, okay to use default when mypage header is not defined.
    #Determine whether or not need to generate
    #themes default header.
    my $header;
    if (Metadot::Utils::Theme->use_theme_info()) {
    $header = $DISPLAY->simple_expand(0, '', $PARAMS->{theme_simpleheader});
#   print STDERR "using themes, got header: $header\n";
    }
    else {
    $header = $PARAMS->{simpleheader};
    }
    return $header;
}

sub mypage_message_header {
    my $self = shift ;
    $self->debug_msg(4, "MyPage::mypage_message_header");

    return $PARAMS->{mypagemessageheader};
}

sub get_user_choices {
    my $self = shift;
    my $uid = shift;
    $self->debug_msg(4, "MyPage::get_user_choices");

    $uid = undef unless (Utility->is_valid_integer($uid));    # secu_200401

    my %choices;
    my $string;

    # user's mypage elements
    my ($mpelements) = DBAccess->sqlSelectMany("id, isa","mypage","userid=$uid");
    while (my ($id, $isa) = $mpelements->fetchrow()) {
    $choices{$isa}{$id}{mypage} = 1;
    }
    $mpelements->finish;

    # user's subscriptions
    my ($subscriptions) = DBAccess->sqlSelectMany("cid, frequency, medium","subscription","userid=$uid");
    while (my ($cid, $frequency, $medium) = $subscriptions->fetchrow()) {
    $choices{NewsChannel}{$cid}{$frequency}{$medium} = 1 if (defined $frequency);
    }
    $subscriptions->finish;

    return \%choices;
}

###################################################
sub save_links {
    my $self = shift ;
    my $uid = shift ;
    $self->debug_msg(4, "MyPage::save_links");

    my $maxlinks = $PARAMS->{'maxlinks'};

    for (my $i=1; $i<=$maxlinks; $i++){
    my $vname = "linkname$i";
    my $vurl  = "linkurl$i";
    my $name  = Verify->stripBadHTML($FORM{$vname}) || "";
    $name =~ s/^\s*//;
    my $url   = Verify->stripBadHTML($FORM{$vurl}) || "";
    $url =~ s/^\s*//;

    $USER->save_mypage_link( $uid, $i, $name, $url );
    }

}


sub showCopyRight{
    my $s;

    my $imageslocation = $PARAMS->{'imageslocation'};
    my $docs_url = $PARAMS->{'docs_url'};

    my $subscribe;
    if (MetadotConfig->get_site_in_trial_mode() eq 'true') {
    my $sub_link = "http://www.metadot.com/subscribe_now.html";
    $subscribe = "<a href=\"$sub_link\"><img src=\"$imageslocation/subscribe.gif\" border=\"0\" align='middle'></a>";
    }
    my $logo = '';
    unless ($DISPLAY->is_logo_suppressed()) {
        $logo = "powered2.gif";
    if (MetadotConfig->get_is_managed_site() eq 'true') {
            $logo = "md-managed.gif";
        }
    }
    $s = "<table width=\"100%\"><tr><td width=\"175\" align='center' valign='middle'>$subscribe&nbsp;\n</td>
\n<td align='right' width=\"100%\" valign='middle'><a href=\"$docs_url\" target=\"_blank\">"
        . $DISPLAY->print_help_link()."&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\n"
        . "\n<a href=\"http://www.metadot.com\"><img src=\"$imageslocation/$logo\" border=\"0\" align='middle'></a>&nbsp;</td>
</tr></table>";

}


# XXX [claudio]
# added here to instead use the settings of the default user when
# the current user is not logged in.
#
sub _get_user {
    my $self = shift;

    if( $USER->is_logged_in() ) {

    return $USER;

    } else {

    my $anon_uid = MetadotConfig->get_reserved_uid('ANONYMOUS_USER');

    return Metadot::UserFactory->get_user( $anon_uid );
    }
}


1;


__END__

=head1 NAME

MyPage - controls the displaying and configuration of a user's My Page.
Descended from L<Portal>.

=head1 SYNOPSIS

    use MyPage;

    $mypage = MyPage->new ();

    $AC = $mypage->get_AC ;
    $isa = $mypage->is_a ;
    $mypage->save ;
    $mypage->get_owner_uid ;
    $mypage->not_logged_in_callback ;
    $mypage->www_show ;
    $mypage->www_layout ;
    $mypage->www_select ;
    $mypage->www_help ;
    $mypage->www_edit_links ;
    $mypage->www_save_links ;
    $form_html = $mypage->show_layout_form ;
    $html = $mypage->show ;
    $html = $mypage->show_tool_bar ;
    $html = $mypage->show_link_bar ;
    $col_list_ref = $mypage->show_content ;
    $html = $mypage->show_selection_form ;
    $html = $mypage->category_popup ([$ccid='All']);
    $html = $mypage->show_link_form ($uid);
    $html = $mypage->mypage_header ;
    $choice_hash_ref = $mypage->get_user_choices ($uid);
    $mypage->save_links ($uid);
    $html = &showCopyRight;


=head1 DESCRIPTION

B<MyPage> controls a user's My Page display and configuration. In addition,
it provides forms to alter the layout of My Page and a selection of
various channels to add to My Page.


Constructor

    $mypage = MyPage->new ();

Access control subroutine that returns reference to %AC which defines its
permissions.

    $AC = $mypage->get_AC ;

Returns package name.

    $isa = $mypage->is_a ;

Saves the permissions associated with the MyPage class.

    $mypage->save ;

Returns the owner of the permission object.

    $mypage->get_owner_uid ;

Access control subroutine that defines the behavior of the system when
a signed-out user performs an action that requires being signed-on.

    $mypage->not_logged_in_callback ;

Displays a user's My Page.

    $mypage->www_show ;

Displays layout page for My Page.

    $mypage->www_layout ;

Displays configuration page for My Page.

    $mypage->www_select ;

Displays help page for My Page.

    $mypage->www_help ;

Displays form to customize My Page Links.

    $mypage->www_edit_links ;

Saves user's My Page Links.

    $mypage->www_save_links ;

Returns the html that displays the My Page layout form.

    $form_html = $mypage->show_layout_form ;

Returns the html that displays a user's My Page.

    $html = $mypage->show ;

Returns html of My Page tool bar.

    $html = $mypage->show_tool_bar ;

Returns html of My Page link bar.

    $html = $mypage->show_link_bar ;

Returns html of My Page display.

    $col_list_ref = $mypage->show_content ;

Displays My Page configuration page for various channel types

    $html = $mypage->show_selection_form ;

Returns html selection drop-down of all external channel categories.
Will preselect $ccid or category id if passed. Default pre-select is
category name 'All'

    $html = $mypage->category_popup ([$ccid='All']);

Displays form that edits My Page links.

    $html = $mypage->show_link_form ($uid);


Returns html for the My Page header.

    $html = $mypage->mypage_header ;

Returns reference to a hash that stores a user's My Page choices.

    $choice_hash_ref = $mypage->get_user_choices ($uid);

Back end function that saves a user's My Page links.

    $mypage->save_links ($uid);

Returns html of Copy Right information.

    $html = MyPage->showCopyRight;

=cut
