###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package MyPageElement;
use strict;
#  use Unicode::String;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY);

use DBAccess;
use Carp;

use Portal;
use MetadotConfig;
use Metadot::Utils::Theme;
use Utility;

use vars qw(@ISA);
@ISA=qw(Portal);


my $AC = {
       BUNDLES => {
                   MOD =>    { DESC =>          'Edit',
                               OPS =>            [qw(up down right left add invite)],
                               DEFAULT_LEVEL =>  2,
                               MIN_LEVEL =>      2,
                             },
                   DEL =>    { DESC =>          'Delete',
                               OPS =>            [qw(delete)],
                               DEFAULT_LEVEL =>  2,
                               MIN_LEVEL =>      2,
                             },
                   DISP =>   { DESC =>          'View',
                               OPS =>            [qw(preview)],

                               DEFAULT_LEVEL =>  2,
                               MIN_LEVEL =>      2,
                               DEFAULTS_TO =>    'on',
                             },
                  },
          DISPLAY_ORDER => [qw(DISP MOD EDITP)],
        };

sub get_AC {
    return $AC;
}

sub is_a {
    my $self = shift;

    return ref($self);
}

#sub name {
#   my $self = shift;
#
#   return ref($self);
#}

sub id {
    my $self = shift;

    return 1;
}

sub save {
    my $self = shift;

    $self->save_permissions;
}

sub get_owner_uid {
    my $self = shift;

    return MetadotConfig->get_reserved_uid('ADMIN');
}

sub parent_id {
    my $self = shift;

    return 0;
}

####################################################

sub new {
  my $proto = shift;
  my $mpid  = shift;
  my $uid   = shift;

  my $class = ref($proto) || $proto;
  my $self = {};

  $self->{mpid} = $mpid ;
  $self->{uid}  = $uid;
  if (!$uid && $USER) {
      $self->{uid} = $USER->uid();
  }
  $self->{is_a}  = "MyPageElement";

    bless ($self, $class);

  return $self;
}

################################################################
####                  OPERATION HANDLER                    #####
################################################################

sub www_delete {
  my $self = shift ;
  my $isa = $self->is_a();
  $self->debug_msg(3, "MyPageElement::www_delete()");
  my $id = $self->defined_or_exit($FORM{id},"id is missing.");


  $FORM{confirm} ||= "";
  if ($FORM{confirm} eq "yes") {
    $self->two_choices ("Do you want to remove this element from your page?",
                        "caution",
                          "Yes please", "userchannel.pl?id=$id&isa=$isa&op=delete&confirm=ok",
                          "No",        "userchannel.pl");
    exit(0);
  } else {
    $self->_delete();
    $self->md_redirect("userchannel.pl") if ($FORM{confirm} eq "ok");
    $self->md_redirect($SESSION->{referer});
  }
}

sub www_up {
  my $self = shift ;
  $self->debug_msg(3, "MyPageElement::www_up()");
  my $mpid = $self->defined_or_exit($self->mpid(),"mpid is missing.");

  $self->_up();
  $self->md_redirect($SESSION->{referer});
}

sub www_down {
  my $self = shift ;
  $self->debug_msg(3, "MyPageElement::www_down()");
  my $mpid = $self->defined_or_exit($self->mpid(),"mpid is missing.");

  $self->_down();
  $self->md_redirect($SESSION->{referer});
}

sub www_right {
  my $self = shift ;
  $self->debug_msg(3, "MyPageElement::www_right()");
  my $mpid = $self->defined_or_exit($self->mpid(),"mpid is missing.");

  $self->_right();
  $self->md_redirect($SESSION->{referer});
}

sub www_left {
  my $self = shift ;
  $self->debug_msg(3, "MyPageElement::www_left()");
  my $mpid = $self->defined_or_exit($self->mpid(),"mpid is missing.");

  $self->_left();
  $self->md_redirect($SESSION->{referer});
}

################################################################
####                   PREVIEW  -  ADD                     #####
################################################################

sub www_preview {
  my $self = shift ;
  my $isa = $self->is_a ;
  $self->debug_msg(3, "MyPageElement::www_preview()");

  my $id = $self->defined_or_exit($FORM{id},"MyPageElement::www_preview() - id is missing.");

  $self->print_http_header();
# XXX  print CGI::start_html (-title=>"Add $isa to MyPage");
  print $self->_preview_form($id);

  exit(0);
}

sub www_add {
  my $self = shift ;
  my $isa = $self->is_a();
  my $col = $self->{col};
  my $mypageorder = $self->mypageorder();
  $self->debug_msg(3, "GizmoTool::www_add()");

  my $id = $self->defined_or_exit($FORM{id},"MyPageElementl::www_add() - id is missing.");
  $col = $FORM{col} || $col;
  my $row = $FORM{row} || $mypageorder;
  my $frequency = $FORM{frequency} ;
  my $redirect_automatically = $FORM{redirect} || $PARAMS->{hide_my_page} || !($USER->has_mypage());

  my $formpager = $FORM{pager} || "";
  my $formicq   = $FORM{icq} || "";

  my $add_url = defined $frequency ? "userchannel.pl?frequency=$frequency&pager=$FORM{pager}&icq=$FORM{icq}&id=$id&op=add&isa=$isa" : "";


  #print STDERR "ABOUT TO SHOW OPTIONS: $FORM{target}\n";

  my $url = $FORM{target} || $FORM{cancelurl} || $SESSION->{referer} || "userchannel.pl";

  if ((!$USER->pager_number()) && ($formpager eq 'all')){
    $USER->www_require_pager_form($add_url,$url);
    exit();
  }
  if ((!$USER->icq_number()) && ($formicq eq 'all')) {
    $USER->www_require_icq_form($add_url,$url);
    exit();
  }


  $self->{mpid} = $self->_add($id,$isa,$col,$row,$frequency);

  if ($redirect_automatically) {
      $self->md_redirect($url);
  } else {

      $self->two_choices("Channel has been saved.",
                         "confirm",
                         "Return",
                         "$url",
                         "Go to My Page",
                         "userchannel.pl");
  }
}

sub www_add_email {
  my $self = shift;
  $self->debug_msg(3, "GizmoTool::www_add_email()");

  my $id = $self->defined_or_exit($FORM{id}, "MyPageElementl::www_add_email() - id is missing.");

  $self->{mpid} = $self->_add_email($id, $FORM{frequency});

  my $url = $FORM{cancelurl} || $SESSION->{referer} ;
  $url = "userchannel.pl?isa=MyPage&amp;op=select&amp;type=Selection" if ($url =~ m/userchannel\.pl$/);
  $url = "userchannel.pl" if ($FORM{cancelurl} =~ /index\.pl/);
  $self->md_redirect($url);
}



################################################################
####                    ADD  -  DELETE                     #####
################################################################

sub _add {
  my $self = shift ;
  my $id   = shift ;
  my $isa  = shift ;
  my $col  = shift || 1+int(rand(3)) ;
  my $row  = shift || 0 ;
  my $frequency = shift;
  my $uid  = $self->uid();

  $id = undef unless (Utility->is_valid_integer($id));    # secu_200401

  my $db_now = Utility->db_now();

  ## we delete the old ones if any ...
  DBAccess->sqlDelete("mypage","userid=$uid and id=$id and isa=".DBAccess->quote($isa));

  ## ... add a new one at the right place ...
  my ($mpid) = DBAccess->sqlInsert("mypage",
            userid          => $uid,
            id           => $id,
            isa           => $isa,
            col          => $col,
            mypageorder  => $row,
            creationdate => $db_now
           );

  ## ... reorder the column ...
  $self->_reorder_column($col);

  $self->_add_email($id,$frequency);

  return ($mpid);
}

sub _delete {
  my $self = shift ;
  my $mpid = $self->mpid();
  my $uid = $self->uid();

  return if (!defined($mpid) || !defined($uid));

  my ($col) = DBAccess->sqlSelect("col","mypage", "mpid='$mpid'");
  #specify uid for security
  DBAccess->sqlDelete("mypage", "mpid=$mpid and userid=$uid");
  $self->_reorder_column($col);
}

sub _add_email {
#      print STDERR "in MyPageElement _add_email\n";
  my $self = shift ;
  my $id   = shift ;
  my $frequency = shift || 'none';
  my $uid  = $self->uid();
  my $mpid;
  my $formpager = $FORM{pager} || "";
  my $formicq   = $FORM{icq} || "";

  $id = undef unless (Utility->is_valid_integer($id));    # secu_200401

  my $db_now = Utility->db_now();

  if ($frequency ne 'none') {
     my ($exists) = DBAccess->sqlSelect("count(*)","subscription","cid='$id' and userid=$uid and medium='email'");
     if ($exists > 0) {
      ($mpid) = DBAccess->sqlUpdate("subscription","cid=$id and userid=$uid and medium='email'",(frequency =>$frequency));
     }
     if ($exists == '0') {
#    print STDERR "doing the insert into subscription\n";
       ($mpid) = DBAccess->sqlInsert("subscription",
               userid => $uid,
               cid => $id,
               medium => 'email',
               creationdate => $db_now,
               frequency => $frequency,
              );
    }

  } else {
    DBAccess->sqlDelete("subscription", "cid=$id and userid=$uid and medium='email'");
  }
  if ($formpager eq 'all') {
    my ($exists) = DBAccess->sqlSelect("count(*)", "subscription", "cid=$id and userid=$uid and medium='pager'");
    if ($exists == 0 ) {
        DBAccess->sqlInsert("subscription",
                userid => $uid,
               cid => $id,
               medium => 'pager',
               creationdate => $db_now,
               frequency => 'all',
            );
    }
  } else {
      DBAccess->sqlDelete("subscription", "cid='$id' and userid=$uid and medium='pager'");
  }

  if ($formicq eq 'all') {
    my ($exists) = DBAccess->sqlSelect("count(*)", "subscription", "cid='$id' and userid=$uid and medium='icq'");
    if ($exists == 0 ) {
        DBAccess->sqlInsert("subscription",
                userid => $uid,
               cid => $id,
               medium => 'icq',
               creationdate => $db_now,
               frequency => 'all',
            )
    }
  } else {
      DBAccess->sqlDelete("subscription", "cid=$id and userid=$uid and medium='icq'");
  }



  return ($mpid);
}

################################################################
####             LEFT  -  RIGHT  -  UP  -  DOWN            #####
################################################################

sub _left {
  my $self = shift ;
  my $mpid = $self->mpid();
  my $uid  = $self->uid();

  my ($oldcol) = DBAccess->sqlSelect("col", "mypage", "mpid='$mpid'");
  my $newcol = ($oldcol>1) ? $oldcol-1 : 3 ;

  DBAccess->sqlUpdate("mypage", "mpid='$mpid' and userid=$uid", (col=>$newcol));
  $self->_reorder_column($newcol);
  $self->_reorder_column($oldcol);
}
################################################################
sub _right {
  my $self = shift ;
  my $mpid = $self->mpid();
  my $uid  = $self->uid();

  my ($oldcol) = DBAccess->sqlSelect("col", "mypage", "mpid='$mpid'");
  my $newcol = ($oldcol<3) ? $oldcol+1 : 1 ;

  DBAccess->sqlUpdate("mypage", "mpid='$mpid' and userid=$uid", (col=>$newcol));
  $self->_reorder_column($newcol);
  $self->_reorder_column($oldcol);
}
################################################################
sub _up {
  my $self = shift ;
  my $mpid = $self->mpid();
  my $uid = $self->uid();

  my ($mypageorder, $col) = DBAccess->sqlSelect("mypageorder, col", "mypage", "mpid='$mpid' and userid=$uid");
  my ($minorder, $maxorder) = DBAccess->sqlSelect("min(mypageorder), max(mypageorder)", "mypage", "userid=$uid and col='$col'");

  if ($mypageorder<=$minorder) {
    ### curent item is first and become last
    DBAccess->sqlUpdate("mypage", "mpid=$mpid and userid=$uid",  (mypageorder => $maxorder+1));
    $self->_reorder_column($col);
  } else {
    ### other case we swap
    ### update the one above (take the place of the one we move)
    DBAccess->sqlUpdate("mypage", "userid=$uid and col=$col and mypageorder=$mypageorder-1",  (mypageorder => $mypageorder));
    ### update the one
    DBAccess->sqlUpdate("mypage", "mpid=$mpid and userid=$uid",  (mypageorder => $mypageorder-1));
  }
}
################################################################
sub _down {
  my $self = shift ;
  my $mpid = $self->mpid();
  my $uid = $self->uid();

  my ($mypageorder, $col) = DBAccess->sqlSelect("mypageorder, col", "mypage", "mpid='$mpid' and userid=$uid");
  my ($minorder, $maxorder) = DBAccess->sqlSelect("min(mypageorder), max(mypageorder)", "mypage", "userid=$uid and col='$col'");

  if ($mypageorder>=$maxorder) {
    ### curent item is first and become last
    DBAccess->sqlUpdate("mypage", "mpid=$mpid and userid=$uid",  (mypageorder => $minorder-1));
    $self->_reorder_column($col);
  } else {
    ### other case we swap
    ### update the one below (take the place of the one we move)
    DBAccess->sqlUpdate("mypage", "userid=$uid and col=$col and mypageorder=$mypageorder+1",  (mypageorder => $mypageorder));
    ### update the one
    DBAccess->sqlUpdate("mypage", "mpid=$mpid and userid=$uid",  (mypageorder => $mypageorder+1));
  }
}
################################################################
# renumber a col of my page
# useful after delete or inserting many elements
sub _reorder_column{
  my $self = shift ;
  my $col = shift ;
  my $uid = $self->uid();

  my $i=0;
  my ($c) = DBAccess->sqlSelectMany("mpid", "mypage", "userid=$uid and col='$col'", "order by mypageorder");
  while(my ($mpid)=$c->fetchrow()) {
    $i++;
    DBAccess->sqlUpdate("mypage", "mpid=$mpid", (mypageorder => $i));
  }
  $c->finish();
}

################################################################
####               DISPLAYING MY PAGE ELEMENT              #####
################################################################

sub display {
  my $self = shift;

  my $title       = $self->name();
  my $content     = $self->content();
  my $description = $self->description();
  my $homeurl     = $self->home_url();

  $title = "<a class='FRAMETITLE' href='$homeurl' target='_NEWITEM'>$title</a>" if ($homeurl);

  return $self->_frame($title,$content,$description);
}

############

sub _frame {
  my $self = shift ;
  my $title = shift ;
  my $content = shift ;
  my $description = shift ;
  my $panel = $self->_panel();

  #aju
  my $mypageelementboxcolor; my $mypagetitleboxcolor;
  my $global_look = $PARAMS->{'globallook'};
  my $theme = $PARAMS->{'template_theme'};
  if ($theme && ($theme ne 'null') && ($global_look eq 'usetemplate')) {
      # get the color
      my $boxcolor = $PARAMS->{'theme_mypage_color'};
      $mypageelementboxcolor = $boxcolor;
      $mypagetitleboxcolor = $boxcolor;
  }
  else {
      $mypageelementboxcolor = $PARAMS->{'mypageelementboxcolor'};
      $mypagetitleboxcolor = $PARAMS->{'mypagetitleboxcolor'};
    }
  if (substr($mypageelementboxcolor,0,1) ne "#") {
      $mypageelementboxcolor = "#$mypageelementboxcolor";
  }
  if (substr($mypagetitleboxcolor,0,1) ne "#") {
      $mypagetitleboxcolor = "#$mypagetitleboxcolor";
  }

  my $desc = ($description) ? "<TR><TD ALIGN='CENTER'>\n  $description\n</TD></TR>" : "" ;

  return "

<TABLE CLASS='FRAME' CELLPADDING=\"2\" CELLSPACING=\"0\" border=\"0\" WIDTH=\"100%\" BGCOLOR='$mypageelementboxcolor'><TR><TD>

<TABLE CLASS='FRAMETITLE' CELLPADDING=\"0\" CELLSPACING=\"0\" border=\"0\" WIDTH=\"100%\" BGCOLOR='$mypagetitleboxcolor'><TR>
<TD HEIGHT=\"15\">
  <font class='FRAMETITLE'>$title</FONT>
</TD>
<TD HEIGHT=\"15\" align='RIGHT' nowrap=\"nowrap\">
<TABLE CLASS='FRAMETITLE' CELLPADDING=\"0\" CELLSPACING=\"2\" border=\"0\" BGCOLOR='$mypagetitleboxcolor'><tr><td>$panel
</td></tr></table>
</TD>
</TR></TABLE>

<TABLE CLASS='FRAMECONTENT' CELLPADDING=\"2\" CELLSPACING=\"0\" border=\"0\" WIDTH=\"100%\" BGCOLOR=white>
$desc
<TR><TD>
  $content
</TD></TR>
</TABLE>
</TD></TR>
<TR><TD HEIGHT=\"0\" BGCOLOR=white></TD></TR>
</TABLE>

";

}

################################################################
####               PREVIEW  MY PAGE ELEMENT                #####
################################################################

sub _preview_form {
    my $self  = shift ;
    my $id    = shift ;
    $id = undef unless (Utility->is_valid_integer($id));    # secu_200401

    my $isa   = $self->is_a();
    my $title = $self->name();
#    $title = Unicode::String::utf8($title)->latin1;
    my $description = $self->description();
#    $description = Unicode::String::utf8($description)->latin1;
    my $uid = $self->uid();
    my $col = $self->{col};
    my $on_mypage = $col ? 1 : 0;

    my $imageslocation = $PARAMS->{'imageslocation'};

    my $preview = $self->display();
    my $referer = $SESSION->{referer};

    #if on mypage exists, then do not update
    my $row_layout_menu = $on_mypage ? "" : "<SELECT NAME=row>\n<OPTION VALUE=0 SELECTED>first\n<OPTION VALUE=1000>last\n</SELECT>";
    my $col_layout_menu = $on_mypage ? "" : "<SELECT NAME=col>\n<OPTION VALUE=1>left\n<OPTION VALUE=2>center\n<OPTION SELECTED VALUE=3>right\n</SELECT>";

    my $layout_table = $on_mypage ? "" : "
<TR><TD>
<HR NOSHADE SIZE=1><br />
<b>Layout $isa: <I>$title</I></b><br />
</TD></TR>

<TR><TD>
<!--- Layout Table --->
<TABLE CELLPADDING=\"0\" CELLSPACING=\"20\" BORDER=\"0\" WIDTH=\"100%\">
  <TR><TD width=\"50%\" ALIGN='left'>
    Position your item in Your Page.<br />&nbsp;
    <TABLE><TR>
      <TD ALIGN='right'>Make&nbsp;this&nbsp;the</TD>
      <TD ALIGN='left'>$row_layout_menu</TD>
      <TD ALIGN='left'>item</TD>
    </TR><TR>
      <TD ALIGN='right'>in the</TD>
      <TD ALIGN='left'>$col_layout_menu</TD>
      <TD ALIGN='left'>column.</TD>
    </TR></TABLE>
  </TD><TD width=\"50%\" ALIGN='center'>
    <IMG SRC='$imageslocation/my/layout.gif'>
  </TD></TR>
</TABLE>
<!--- End of Preview Table --->
</TD></TR>
";

    my $form = ""; my $pform = "";

    $pform .= $DISPLAY->show_page_top;
    $pform .= $DISPLAY->simple_header() . "<center>" . $DISPLAY->form_top();

    my $stylerules = Metadot::Utils::Theme->get_theme_style_html();
    if (!$stylerules || ($stylerules eq '')) {
    $form .= "\n\n<LINK REL=StyleSheet HREF='$imageslocation/channelcss.css'>\n\n";
    }
    else {
    $form .= "\n" . $stylerules . "\n";
    }
    $form .= "
<TABLE WIDTH='525' CELLSPACING=\"0\" CELLPADDING=\"0\" BORDER=\"0\">

<TR><TD>
<H2>Add new $isa to MyPage</H2>
<HR NOSHADE SIZE=1><br />
</TD></TR>

<TR Valign='TOP'><TD>
<b>Preview $isa: <I>$title</I></b><br />
$description<br />
</TD></TR>

<TR><TD>
<!--- Preview Table --->
<TABLE CELLPADDING=\"0\" CELLSPACING=\"20\" BORDER=\"0\" WIDTH=\"100%\">
  <TR><TD width=\"50%\">
    $preview
  </TD><TD width=\"50%\" Valign='TOP'>
    <FONT SIZE='-1'>This window shows how the <I>$title</I> $isa will appear on your page.</FONT>
  </TD></TR>
</TABLE>
<!--- End of Preview Table --->
</TD></TR>

<FORM ACTION=$ENV{SCRIPT_NAME} METHOD='post'>

$layout_table



<TR><TD>";

    unless ($isa eq "BasicTool" || $isa eq "StockQuotes" || $isa eq "WeatherTool" || $isa eq "WebMonitor") {
    $form .= "

<HR NOSHADE SIZE=1><br />
<b>Email Subscription $isa: <I>$title</I></b><br />
</TD></TR>


<TR><TD>
<!--- Subscription Table --->
<TABLE CELLPADDING=\"0\" CELLSPACING=\"20\" BORDER=\"0\" WIDTH=\"100%\">
  <TR><TD width=\"50%\" ALIGN='left'>
    You can also choose to receive updates on this channel via Email. <br />&nbsp;
    <table width='90%' border=\"2\" cellspacing=\"0\" cellpadding=\"0\">

 <tr>
        <th class='CUSTOMMYPAGE' width=\"200\">&nbsp;</th>
        <th class='CUSTOMMYPAGE' colspan='4'>E-mail</th>
        <th class='CUSTOMMYPAGE'>Pager</th>
        <th class='CUSTOMMYPAGE'>ICQ</th>
  </tr>

 <tr>
        <th class='CUSTOMMYPAGE' width=\"200\">&nbsp;</th>
        <th class='CUSTOMSUBSCRIPTION' align='center' valign='center'><b>None</b></th>
        <th class='CUSTOMSUBSCRIPTION' align='center' valign='center'><b>Daily</b></th>
        <th class='CUSTOMSUBSCRIPTION' align='center' valign='center'><b>Weekly</b></th>
        <th class='CUSTOMSUBSCRIPTION' align='center' valign='center'><b>ASAP</b></th>
 <th class='CUSTOMSUBSCRIPTION' align='center' valign='center'><b>ASAP</b></th>
 <th class='CUSTOMSUBSCRIPTION' align='center' valign='center'><b>ASAP</b></th>
        </tr>
        <tr>
        <td class='CUSTOMMYPAGE' width=\"200\">$title</td>";


    my ($frequency) = DBAccess->sqlSelect("frequency","subscription","userid='$uid' and cid='$id' and medium='email'");
    my ($pager_frequency) = DBAccess->sqlSelect("frequency","subscription", "userid='$uid' and cid='$id' and medium='pager'");
    my ($icq_frequency) = DBAccess->sqlSelect("frequency","subscription", "userid='$uid' and cid='$id' and medium='icq'");

    #if (!$frequency) {
    if ( !defined($frequency) ) {
            ### make ASAP frequency default
        $frequency = 'all';
    }

    if (!$pager_frequency) {
        $pager_frequency = '';
    }

    if (!defined $icq_frequency) {
        $icq_frequency = '';
    }

    if ($frequency eq '') {
        $form .="<td class='CUSTOMSUBSCRIPTION' align='center'><INPUT TYPE=radio NAME=frequency VALUE='none'></td>\n";
    } else {
        $form .="<td class='CUSTOMSUBSCRIPTION' align='center'><INPUT TYPE=radio NAME=frequency VALUE='none'></td>\n";
    }

    if ($frequency eq 'onceaday') {
        $form .="<td class='CUSTOMSUBSCRIPTION' align='center'><INPUT TYPE=radio NAME=frequency VALUE='onceaday' checked></td>\n";
    } else {
        $form .="<td class='CUSTOMSUBSCRIPTION' align='center'><INPUT TYPE=radio NAME=frequency VALUE='onceaday'></td>\n";
    }

    if ($frequency eq 'onceaweek') {
        $form .="<td class='CUSTOMSUBSCRIPTION' align='center'> <INPUT TYPE=radio NAME=frequency VALUE='onceaweek' checked></td>\n";
    } else {
        $form .="<td class='CUSTOMSUBSCRIPTION' align='center'> <INPUT TYPE=radio NAME=frequency VALUE='onceaweek'></td>\n";
    }

    if ($frequency eq 'all') {
        $form .="<td class='CUSTOMSUBSCRIPTION' align='center'> <INPUT TYPE=radio NAME=frequency VALUE='all' checked></td>\n";
    } else {
        $form .="<td class='CUSTOMSUBSCRIPTION' align='center'> <INPUT TYPE=radio NAME=frequency VALUE='all'></td>\n";
    }

    if ($pager_frequency eq 'all') {
        $form .="<td class='CUSTOMSUBSCRIPTION' align='center'> <INPUT TYPE=checkbox NAME=pager VALUE='all' checked></td>\n";
    } else {
        $form .="<td class='CUSTOMSUBSCRIPTION' align='center'> <INPUT TYPE=checkbox NAME=pager VALUE='all'></td>\n";
    }
    if ($icq_frequency eq 'all') {
        $form .="<td class='CUSTOMSUBSCRIPTION' align='center'> <INPUT TYPE=checkbox NAME=icq VALUE='all' checked></td>\n";
    } else {
        $form .="<td class='CUSTOMSUBSCRIPTION' align='center'> <INPUT TYPE=checkbox NAME=icq VALUE='all'></td>\n";
    }

    $form .= "</tr> </TABLE></td></tr></table></td></tr>";


    }
    my $cancel_button_text = "Cancel";
    my $save_button = "<INPUT TYPE='submit' VALUE='Save'>";

    if ($on_mypage && ($isa eq "BasicTool" || $isa eq "StockQuotes" || $isa eq "WeatherTool" || $isa eq "WebMonitor")) {
    $cancel_button_text = "Back";
    $save_button = "";
    }

    $form .= "
<TR><TD ALIGN='center'>
<INPUT TYPE='hidden' NAME='cancelurl' VALUE='$referer'>
<INPUT TYPE='hidden' NAME='id' VALUE='$id'>
<INPUT TYPE='hidden' NAME='op' VALUE='add'>
<INPUT TYPE='hidden' NAME='isa' VALUE='$isa'>
<INPUT TYPE='submit' NAME='cancel' VALUE='$cancel_button_text'>
$save_button
<br />&nbsp;<br />
</TD></TR>

</FORM>

</TABLE>
";

    #aju
    my $formbgcolor;
    if (Metadot::Utils::Theme->use_theme_info()) {
    $formbgcolor = Metadot::Utils::Theme->get_theme_default_bg();
    }
    else {
    $formbgcolor = $PARAMS->{'formbgcolor'};
    }

    $pform .= "<tr bgcolor='#$formbgcolor'><td>&nbsp;</td><td>$form</td><td>&nbsp;</td></tr>";
    $pform .=  $self->form_bottom() . "</center>";
    $pform .=  $self->close_html();

    return $pform;
}


################################################################
####                MY PAGE ELEMENT PANELS                 #####
################################################################

sub _panel {
  my $self = shift ;
  my $id = shift ; #id of element: iid, cid
  my $isa = $self->is_a();

  my $imageslocation = $PARAMS->{'imageslocation'};

  my $s = "<a href='userchannel.pl?id=$id&amp;isa=$isa&amp;op=delete&amp;confirm=yes'><img src=$imageslocation/obj/del.gif width=\"11\" height=\"11\" border=\"0\" alt='Delete Me!'></a>";
  return $s;
}

sub layout_panel {
  my $self = shift ;
  my $mpid = $self->mpid();
  my $name = $self->name();
  my $move_panel = $self->_move_panel();

  return "
<table>
<tr><td align='center'>$name</td></tr>
<tr><td align='center'>$move_panel</td></tr>
</table>
";
}

sub _move_panel {
  my $self = shift;
  my $mpid = $self->mpid();

  my $imageslocation = $PARAMS->{'imageslocation'};

  my $s = "<table><tr><td valign='center'>\n";
  $s .= "<a href=userchannel.pl?mpid=$mpid&amp;isa=MyPageElement&amp;op=left><img SRC='$imageslocation/obj/leftarrow.gif' border='0'></a>\n";
  $s .= "<a href=userchannel.pl?mpid=$mpid&amp;isa=MyPageElement&amp;op=up><img SRC='$imageslocation/obj/uparrow.gif' border=\"0\"></a>\n";
  $s .= "<a href=userchannel.pl?mpid=$mpid&amp;isa=MyPageElement&amp;op=down><img SRC='$imageslocation/obj/downarrow.gif' border=\"0\"></a>\n";
  $s .= "<a href=userchannel.pl?mpid=$mpid&amp;isa=MyPageElement&amp;op=right><img SRC='$imageslocation/obj/rigtharrow.gif' border='0'></a>\n";
  $s .= "</td></tr></table>\n";
  return $s;
}


1;


__END__

=head1 NAME

MyPageElement - Functions for the elements on MyPage

=head1 SYNOPSIS

    use MyPageElement;
    $mpe = MyPageElement->new ($mpid, $uid);

    $AC = $mpe->get_AC;
    $isa = $mpe->is_a ;
    $id = $mpe->id ;
    $mpe->save ;
    $uid = $mpe->get_owner_uid ;
    $mpe->parent_id ;
    $mpe->not_logged_in_callback ;
    $mpe->www_delete;
    $mpe->www_up;
    $mpe->www_down ;
    $mpe->www_right ;
    $mpe->www_left ;
    $mpe->www_preview ;
    $mpe->www_add ;
    $mpe->www_add_email ;
    $html = $mpe->display ;
    $html = $mpe->layout_panel ;

=head1 DESCRIPTION

This package contains functions to edit, delete, add, and layout elements on MyPage.

=head1 Methods

Returns AC for user:

    $AC = $mpe->get_AC;

Returns the name of the module currently being used:

    $isa = $mpe->is_a ;

Returns the ID of the current user:

    $id = $mpe->id ;

Saves permissions for user:

    $mpe->save ;

eturns uid for ADMIN:

    $uid = $mpe->get_owner_uid ;

Returns parent_id:

    $mpe->parent_id ;

Redirects user to login:

    $mpe->not_logged_in_callback ;

Confirms and deletes elements from MyPage:

    $mpe->www_delete;

Moves selected element up on MyPage:

    $mpe->www_up;

Moves selected element down on MyPage:

    $mpe->www_down ;

Moves selected elements right on MyPage:

    $mpe->www_right ;

Moves selected element left on MyPage:

    $mpe->www_left ;

Displays preview of element:

    $mpe->www_preview ;

Adds element to MyPage, form error checking, confirmation screen:

    $mpe->www_add ;

Adds email subscription regularity:

    $mpe->www_add_email ;

Displays MyPage element:

    $html = $mpe->display ;

Displays layout so mypage elements can be moved around the page:

    $html = $mpe->layout_panel ;


=cut
