###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package NewsChannel;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY);

use strict;

use DBAccess;
use Carp;

use MyPageElement;
use Metadot::PortalFactory;
use Metadot::Utils::Theme;
use Utility;

use vars qw(@ISA);
@ISA=qw(MyPageElement);

####################################################

sub new {
  my $proto = shift;
  my $cid   = shift; #cid of the Channel
  my $uid   = shift; #uid of the user NECESSARY IF you want to retrieve mypage data

  $cid = undef unless (Utility->is_valid_integer($cid));    # secu_200401
  $uid = undef unless (Utility->is_valid_integer($uid));    # secu_200401

  $DISPLAY->debug_msg(3, "new(".($cid||'').", ".($uid || '').")");

  my $class = ref($proto) || $proto;
  my $self = $class->SUPER::new(undef,$uid); #we don't know the mpid yet

  if ((defined $cid) && (defined $uid)) {
    # load channel & mypage information
    ($self->{cid}, $self->{mpid}, $self->{type}, $self->{name}, $self->{description}, $self->{content}, $self->{home_url}, $self->{iid}, $self->{updatedate}, $self->{col}, $self->{mypageorder})
      = DBAccess->sqlSelect("cid, mpid, channel.isa, name, description, html, homeurl, iid, channel.updatedate, col, mypageorder",
          "channel, mypage", "mypage.id=$cid and mypage.userid=$uid and mypage.id=channel.cid");
  }
  if ((defined $cid) && !(defined $self->{cid})) {
    # load channel information
    ($self->{cid}, $self->{type}, $self->{name}, $self->{description}, $self->{content}, $self->{home_url}, $self->{iid}, $self->{updatedate})
      = DBAccess->sqlSelect("cid, channel.isa, name, description, html, homeurl, iid, channel.updatedate", "channel", "cid=$cid");
  }

  if($self->{content} eq "0"){
      $self->{content} ="<center>There are no messages in this channel!</center>";
  }

  $self->{is_a} = "NewsChannel";


    bless ($self, $class);

  return $self;
}

################################################################
####                      DISPLAY                          #####
################################################################

sub display {
  my $self = shift;

  my $title       = $self->name();
  my $type        = $self->type();
  my $homeurl     = $self->home_url();
  my $iid         = $self->iid();
  my $updatedate  = "Last Updated ".$self->updatedate();


  my $ac_message        = "You are no longer allowed to view this channel.";
  my $content;

  ## AC enforcement
  $DISPLAY->debug_msg(4, "display $title, iid=".($iid||''));
  if (lc($type) eq 'discussion' || lc($type) eq 'internal') {
      my $gizmo = Metadot::PortalFactory->get_gizmo_object($iid);
      if (defined $gizmo) {
          if ($gizmo->is_allowed_to_do('show', $USER)) {
              $content = $self->content();
          } else {
              $content = $ac_message;
          }
      } else {
          # if the source no longer exists, we should display the channel anyway
          $content = $self->content();
      }
  } else {
      $content = $self->content();
  }

  my $text_title = $title;
  $title = "<a class='FRAMETITLE' href='$homeurl' target='_NEWITEM'>$title</a>" if ($homeurl);
  $title = "<a class='FRAMETITLE' href='index.pl?iid=$iid' target='_NEWITEM'>$title</a>" if ((defined $iid) && !$homeurl);
  return $self->_frame($title,$content,$updatedate,$text_title);
}

############

sub _frame {
  my $self = shift ;
  my $title = shift ;
  my $content = shift ;
  my $description = shift;
  my $text_title = shift;

  my $panel = $self->_panel($text_title);

  $DISPLAY->debug_msg(4, "_frame has title=$title; content=$content; description=$description");

  #aju
  my $mypageelementboxcolor; my $mypagetitleboxcolor;
  if (Metadot::Utils::Theme->use_theme_info() ) {
      # get the color
      my $boxcolor = $PARAMS->{'theme_mypage_color'};
      $mypageelementboxcolor = $boxcolor;
      $mypagetitleboxcolor = $boxcolor;
  }
  else {
      $mypageelementboxcolor = $PARAMS->{'mypageelementboxcolor'};
      $mypagetitleboxcolor = $PARAMS->{'mypagetitleboxcolor'};
    }
  if (substr($mypageelementboxcolor,0,1) ne "#") {
      $mypageelementboxcolor = "#$mypageelementboxcolor";
  }
  if (substr($mypagetitleboxcolor,0,1) ne "#") {
      $mypagetitleboxcolor = "#$mypagetitleboxcolor";
  }
#   my $mypageelementcolor = $PARAMS->{'mypageelementboxcolor'};
#   my $mypagetitleboxcolor = $PARAMS->{'mypagetitleboxcolor'};

  my $desc = ($description) ? "<TR><TD ALIGN='CENTER'>\n<font class='FRAMEDATE'>$description</font>\n</TD></TR>" : "" ;

  return "

<TABLE CLASS='FRAME' CELLPADDING=\"2\" CELLSPACING=\"0\" border=\"0\" WIDTH=\"100%\" BGCOLOR='$mypageelementboxcolor'><TR><TD>

<TABLE CLASS='FRAMETITLE' CELLPADDING=\"0\" CELLSPACING=\"0\" border=\"0\" WIDTH=\"100%\" BGCOLOR='$mypagetitleboxcolor'><TR>
<TD>
  <font class='FRAMETITLE'>$title</FONT>
</TD>
<TD HEIGHT=\"15\" align='RIGHT' nowrap=\"nowrap\">
<TABLE CLASS='FRAMETITLE' CELLPADDING=\"0\" CELLSPACING=\"2\" border=\"0\" BGCOLOR='$mypagetitleboxcolor'><tr><td>$panel
</td></tr></table>
</TD>
</TR></TABLE>

<TABLE CLASS='FRAMECONTENT' CELLPADDING=\"2\" CELLSPACING=\"0\" border=\"0\" WIDTH=\"100%\" BGCOLOR=white>

<TR><TD>
  $content
</TD></TR>
$desc
</TABLE>
</TD></TR>
<TR><TD HEIGHT=\"0\" BGCOLOR=white></TD></TR>
</TABLE>

";

}

################################################################
####                       PANEL                           #####
################################################################

sub _panel {
  my $self = shift ;
  my $text_title = shift;
  my $type  = $self->type(); # External || Internal || Discussion
  my $s;


  my $imageslocation = $PARAMS->{'imageslocation'};

#  my $popup_icon = "";
#  $popup_icon = "int_popup.gif" if ($type eq 'Discussion' || $type eq 'Internal');
#  $popup_icon = "ext_popup.gif" if ($type eq 'External');
#  my $popup_tag = "Popup Local $type" ;
#  $popup_tag = "Popup External Channel" if ($type eq 'External');
#  $popup_tag = "Popup Local Channel" if ($type eq 'Internal');
  my $popup_url = $self->home_url();
  my $iid = $self->iid();
  my $cid = $self->{cid} ||0;
  $popup_url ||= "index.pl?id=$iid" ;
  my $uid = $USER->{uid};
  my ($frequency) = DBAccess->sqlSelect("frequency","subscription","userid=$uid and cid=$cid and medium='email'");
  $frequency ||= "";
  my ($pager_frequency) = DBAccess->sqlSelect("frequency","subscription", "userid=$uid and cid=$cid and medium='pager'");
  my ($icq_frequency) = DBAccess->sqlSelect("frequency","subscription", "userid=$uid and cid=$cid and medium='icq'");

    if ($frequency eq 'onceaday' || $frequency eq 'onceaweek') {
        $s .= "<a href=\"javascript:subWrite('You are subscribed to $text_title via a digest')\">";
        $s .= "<img src='$imageslocation/obj/env.gif' alt='You are subscribed to this channel via a digest' border = 0></a>&nbsp;";         #envelope icon
    } elsif ($frequency eq 'all') {
        $s .= "<a href=\"javascript:subWrite('You are subscribed to $text_title via email')\">";
        $s .= "<img src='$imageslocation/obj/env_.gif' alt='You are subscribed to this channel via email' border = 0></a>&nbsp;"; #envelope with exclamation icon
    }
    if ($pager_frequency) {
        $s .= "<a href=\"javascript:subWrite('You are subscribed to $text_title via pager')\">";
        $s .= "<img src='$imageslocation/obj/phone1.gif' alt='You are subscribed to this channel via pager' border = 0></a>&nbsp;"; #phone icon
    }
    if ($icq_frequency) {
        $s .= "<a href=\"javascript:subWrite('You are subscribed to $text_title via icq')\">";
        $s .= "<img src='$imageslocation/obj/icq.gif' alt='You are subscribed to this channel via ICQ' border = 0></a>&nbsp;"; #icq icon
    }


   $s .= "<a href='userchannel.pl?isa=NewsChannel&amp;op=preview&amp;id=$cid'><img src='$imageslocation/obj/edit.gif' border=\"0\" alt = 'Edit Me!'></a>&nbsp;";
  $s .= $self->SUPER::_panel($cid);

  return $s;
}

################################################################
####                  OVERRIDE CONTENT                     #####
################################################################
sub content {
    my $self = shift;
    my $type = $self->type();
    my $iid  = $self->iid();
    my $content = "";
    my $content_exists = 0;

    my $imageslocation = $PARAMS->{'imageslocation'};

    ## AC enforcement
    $DISPLAY->debug_msg(4, "content for type $type, iid=$iid");

    $content = $self->{content};
    $content_exists = 1 if ($content);

    # CKIM: deleted a bunch of conditional processing re: internal channels, because
    #       it was broken. The fix was to make sure the html was generated correctly to
    #       begin with.

    $content =~s/<center>/<tr><td colspan = 2><center>/gi;
    $content =~s/<\/center><br *\/?>/<\/center><\/td><\/tr><tr><td colspan =2>&nbsp;<\/td><\/tr>/gi;
    # aju: it was the width in the following that caused mac ie problems
    $content =~s/<li>/<tr><td valign = top>&nbsp;<img src='$imageslocation\/dot.gif' border="0">&nbsp;<\/td><td width="100%">/gi;
    $content =~s/<br *\/?>/<\/td><\/tr>/gi;
    $content =~s/<p *\/?><sub>/<tr><td colspan = 2><sub>/gi;
    $content =~s/<\/sub><\/p>/<\/sub><\/td><\/tr>/gi;
    $content = "<table cellpadding = 0 cellspacing = 0 border = 0 width = 100%>$content</table>";

    return $content_exists ? $content : "No content.";
}


1;


__END__

=head1 NAME

NewsChannel - Displays Internal News and Discussions

=head1 SYNOPSIS

    use NewsChannel;

    $newschannel = NewsChannel->new ($cid, $uid);
    $html = $newschannel->display ();
    $html = $newschannel->content ();


=head1 DESCRIPTION

NewsChannel Displays functions to display Internal News and Discussions.

=head1 Methods

Displays discussions and internal news:

    $html = $newschannel->display ();

Gets the content where isa=Newsitem and parentiid=$id:

    $html = $newschannel->content ();

=cut
