###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Operations;

use strict;

sub new {
    my $proto = shift;
    my $AC_hash = shift;
    my $class = ref($proto) || $proto;

    my $self = $AC_hash || {};
    bless ($self, $class);
    return $self;
}

sub add_bundle  {
    my $self = shift;
    my $bundle = shift;

    my $description = shift;
    my $default_level = shift;
    my $min_level = shift;
    my $bundle_order = shift;
    my @new_bundle_order;

    die "must pass correct parameters to add_bundle" unless ($bundle && $description && defined($min_level) && defined($default_level));
    #strip out the bundle from the display order if it is already in it
    my $array_ref = $self->{DISPLAY_ORDER};
    foreach my $preexisting_bundle (@$array_ref) {
        unless ($preexisting_bundle eq $bundle) {
            push (@new_bundle_order, $preexisting_bundle);
        }
    }

    #put the bundle in the proper order
    if (defined $bundle_order) {        # CKIM: allow 0
        @new_bundle_order = (@new_bundle_order[0..($bundle_order-2)] , $bundle, @new_bundle_order[($bundle_order-1)..(scalar(@new_bundle_order) - 1)]);
    } else {
        push (@new_bundle_order, $bundle);
    }


    $self->{BUNDLES}{$bundle}{DESC} = $description;
    $self->set_default_level($bundle, $default_level);
    $self->set_min_level($bundle, $min_level);
    $self->{DISPLAY_ORDER} = \@new_bundle_order;
}

sub set_default_level {
    my $self = shift;
    my $bundle = shift;
    my $value = shift;
    my $key = 'DEFAULT_LEVEL';

    $self->modify_bundle($bundle, $key, $value);
}

sub set_min_level {
    my $self = shift;
    my $bundle = shift;
    my $value = shift;
    my $key = 'MIN_LEVEL';

    $self->modify_bundle($bundle, $key, $value);
}

sub set_broadcast {
    my $self = shift;
    my $bundle = shift;
    my $value = shift;
    my $key = 'BROADCAST';

    $self->modify_bundle($bundle, $key, $value);
}

sub add_op  {
    my $self = shift;
    my $bundle = shift;
    my $op = shift;
    my $already_present = 0;
    unless (exists $self->{BUNDLES}{$bundle}) {
        die "can't add an op to a nonexistent bundle: $bundle";
    }

    #print STDERR "$self\->add_op('$bundle', '$op')\n";
    my $array_ref = $self->{BUNDLES}{$bundle}{OPS};
    foreach my $preexisting_op (@$array_ref) {
        if ($preexisting_op eq $op) {
            $already_present = 1;
            last;
        }
    }
    unless ($already_present) {
        push (@$array_ref, $op);
        $self->{BUNDLES}{$bundle}{OPS} = $array_ref;
    }
}

sub remove_bundle {
    my $self = shift;
    my $bundle = shift;
    delete $self->{BUNDLES}{$bundle};
    my @new_bundle_order;
    #strip out the bundle from the display order if it is already in it
    my $array_ref = $self->{DISPLAY_ORDER};
    foreach my $preexisting_bundle (@$array_ref) {
        unless ($preexisting_bundle eq $bundle) {
            push (@new_bundle_order, $preexisting_bundle);
        }
    }
    $self->{DISPLAY_ORDER} = \@new_bundle_order;

}


sub remove_op {
    my $self = shift;
    my $bundle = shift;
    my $op = shift;
    my @new_array;
    my $array_ref = $self->{BUNDLES}{$bundle}{OPS};

    foreach my $preexisting_op (@$array_ref) {
        unless ($preexisting_op eq $op) {
#            push (@new_array, $op);
       push(@new_array, $preexisting_op);
        }
    }

    $self->{BUNDLES}{$bundle}{OPS} = \@new_array;

}

sub modify_bundle {
    my $self = shift;
    my $bundle = shift;
    my $key = shift;
    my $value = shift;
    $self->{BUNDLES}{$bundle}{$key} = $value;
}

sub add_op_parent_dependency {
    my $self = shift;
    my $bundle = shift;
    my $op = shift;
    my $parent_bundle = shift;

    $self->{BUNDLES}{$bundle}{PARENT_DEP}{"$op"} ="$parent_bundle";
}

sub remove_op_parent_dependency {
    my $self = shift;
    my $bundle = shift;
    my $op = shift;
    delete $self->{BUNDLES}{$bundle}{PARENT_DEP}{"$op"};
}

1;


__END__

=head1 NAME

Operations -

=head1 SYNOPSIS

    use Operations;
    $operations = Operations->new ;

    $operations->add_bundle ($bundle, $description, $default_level,
                 $min_level, [$bundle_order]);
    $operations->add_op ($bundle, $op);
    $operations->remove_bundle ($bundle);
    $operations->remove_op ($bundle, $op);
    $operations->modify_bundle ($bundle, $key, $value);
    $operations->add_op_parent_dependency ($bundle, $op, $parent_bundle);
    $operations->remove_op_parent_dependency ($bundle, $op);

=head1 DESCRIPTION

    Operations provides a way for bundles of operations to be grouped together under a specific name.  For example,
    the operations of www_show and www_show_as bullet may be grouped into the VIEW bundle.  Note that these subroutines
    would be added to the VIEW bundle without the www_ appended:  Thus, even though the name of the subroutine is in fact
    www_show, show would be added to the VIEW bundle.

    The Permissions object associated permissions with a bundle.  Thus, if a person has permissions to the view bundle, that
    person will in fact have permissions to do the www_show and www_show_as_bullet methods.

    Note that Gizmos should never instantiate an Operations object themselves.  Rather,
    they should use the get_default_permissions method in their super class (GizmoBuilder) to get an operations object
    when they can then modify.  The retreival and modification of the operations object are class methods.  See GizmoBuilder
    for more information on the retreival of the Operations object.  Modification of the object is described in this documentation
    of the methods of the operations class.

=head2 METHODS

    Constructor:  Instantiates an Operations object.  As mentioned above, Gizmos should never instantiate an Operations object
    themselves.  Rather, they should use the get_default_permissions method in their super class (GizmoBuilder) to get an operations object
    when they can then modify.

    $operations = Operations->new ;

    Adds a new bundle to the operations object.
    The first parameter is the name of the bundle.
    The second parameter is the description of the bundle.
    The third parameter is the default level of permissions associated with the bundle.  That means what permissions
    are associated with a bundle, before the permissions are edited.  0 means that anyone has permission.  2 means that only  logged in
    users have permission.  8 means that only the owner has permission.  9 means that only a site administrator has permissions.
    The fourth parameter is what is the lowest level that the permissions on the object can be set to
    The fifth parameter is optional, and describes the order that the permissions bundles appers in the edit permissions screen

    $operations->add_bundle ($bundle, $description, $default_level, $min_level, [$bundle_order]);

    Associates a new operation with a bundle.  The first parameter is the bundle name, and the second parameter is the op name.
    Note that even though the actual subroutine in the code will have a www_ preface, as in www_show, that preface is not
    included in the op name stored in the operations object.

    $operations->add_op ($bundle, $op);

    Removes a bundle from the operations object.  The only parameter is the name of the bundle to be removed

    $operations->remove_bundle ($bundle);

    Removes an operation from a bundle.  The first parameter is the name of the bundle, and the second parameter is the name of the op
    to be removed.

    $operations->remove_op ($bundle, $op);

    Modifys the bundle to add keys that may be used by the permissions class.  One common key is DEFAULTS_TO, and the
    value of that key may be set to 'on', in which case a newly added group or user would get permission to that bundle
    when they are added through the edit permissions page.  Typically, the VIEW bundle default to on, so that newly added
    groups or people start with view permissions.
    Other keys include


    $operations->modify_bundle ($bundle, $key, $value);

    Allows one to specify a bundle in the parent of an object that a person must have permissions to in order for a person
    to have permissions on an operation in a bundle of the current object.  For example, for a person to have permissions
    to use the cut operation in the current object, they may be required to have permissions to the MOD bundle in the parent
    object.

    $operations->add_op_parent_dependency ($bundle, $op, $parent_bundle);

    Removes the dependency of an operation in a bundle from the requirement to have permissions to a bundle in the parent class.
    $operations->remove_op_parent_dependency ($bundle, $op);


=cut
