###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package ParamsHash;

use DBAccess;
use Metadot qw ($PARAMS $CACHE) ;
use Metadotclass;

use constant DEFAULT_MODERATE_EMAIL => <<END_TOK;
You are receiving this email because a new message has been posted to the <<discussion_name>> discussion that is pending your moderation.

    To approve or reject this message, please follow the link below:

    <<posting_approval_link>>
END_TOK


sub validate_or_refresh_cache {
    my $class = shift;
    my $db_timestamp = $CACHE->is_dirty('ADMIN_PARAMS');
    #return if the cache is up to date
    unless ($db_timestamp) {
        $PARAMS = $CACHE->get_value('ADMIN_PARAMS');
        return;
    }

    # this method makes sure crucial values in $PARAMS
    # exist and are within allowed range
    $class->_validate_values();


    $PARAMS = ParamsHash->new();
    my $sth = DBAccess->sqlSelectMany("name, value", "params");
    while (my ($name, $value) = $sth->fetchrow_array()) {
        $PARAMS->{$name} = $value;
    }
    $PARAMS->{basename} = ($PARAMS->{uri_scheme} || 'http') . "://" . ($PARAMS->{basename} || 'localhost/metadot');

    my $hostname = $PARAMS->{basename};
    my $uri = $PARAMS->{uri_scheme};
    # remove any path info with a regexp
    $hostname =~ s/$uri:\/\///i;
    $hostname =~ s/\/.*//;
    $PARAMS->{hostname} = $hostname;

        # This is no longer necessary in skins mode
    unless ($PARAMS->{use_controller})
    {

        $class->_validate_theme_values();
        my $theme = $PARAMS->{template_theme};
        $sth = DBAccess->sqlSelectMany("pname, pvalue", "theme", "theme = '$theme'");
        while (my ($name, $value) = $sth->fetchrow_array()) {
            $PARAMS->{$name} = $value;
        }
    }



    $CACHE->set_value('ADMIN_PARAMS', $PARAMS);
    $CACHE->set_timestamp('ADMIN_PARAMS', $db_timestamp);

}


{
    my %is_there;
    sub _create_param_if_not_there {
    my $self = shift;
    my $param_name = shift;
    my $default_value = shift;

    return if( $is_there{$param_name} );

    my ( $cnt ) = DBAccess->sqlSelect( "count(*)", "params", "name='$param_name'" );

    unless( $cnt ) {

        DBAccess->sqlInsert("params", name => "$param_name", value => $default_value );

    } else {

        $is_there{$param_name}++;
    }
    }
}

sub _validate_values {
    my $class = shift;

    $class->_create_param_if_not_there( 'posting_notification_message',
                    DEFAULT_MODERATE_EMAIL );
    $class->_create_param_if_not_there('max_number_of_users', '');
    $class->_create_param_if_not_there( 'send_moderated_posting_notifications', '1' );
}


sub _validate_theme_values {
    my $class = shift;

    $class->_create_themes_param_if_not_there('header1_logo1_href', 'index.pl');
    $class->_create_themes_param_if_not_there('header1_logo2_href', '');
}

sub new {
    my $proto = shift;
    my $self = {};
    my $class = __PACKAGE__;
    bless ($self, $class);
    return $self;
}

sub set_property {
    my $self = shift;
    my $key = shift;
    my $value = shift;

    Metadotclass->debug_msg(3, "Updated params key='$key', value='$value'");

    DBAccess->sqlInsertUpdate("params", "name = '$key'", name => $key, value => $value);
    $PARAMS->{$key} = $value;
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_categorytitlebgcolor {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'categorytitlebgcolor'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_max_total_upload_size {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'max_total_upload_size'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_fileuploaddir {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'fileuploaddir'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_formbgcolor {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'formbgcolor'", value => $param);
     $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_globallook {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'globallook'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_httpportnumber {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'httpportnumber'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_imageslocation {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'imageslocation'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_mailsenderaddress {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'mailsenderaddress'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_mailserveraddress {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'mailserveraddress'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_mainfooter {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'mainfooter'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_mainheader {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'mainheader'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_mypageelementboxcolor {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'mypageelementboxcolor'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_mypageelementcolor {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'mypageelementcolor'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

 sub set_mypageheader {
     my $self = shift;
     my $param = shift;
     DBAccess->sqlUpdate("params", "name = 'mypageheader'", value => $param);
     $CACHE->set_as_dirty('ADMIN_PARAMS');
 }

 sub set_use_mypagehdr_default {
     my $self = shift;
     my $param = shift;
     DBAccess->sqlUpdate("params", "name = 'use_mypagehdr_default'", value => $param);
     $CACHE->set_as_dirty('ADMIN_PARAMS');
 }

sub set_mypagetitleboxcolor {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'mypagetitleboxcolor'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_newscolbgcolor {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'newscolbgcolor'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_newscoltitlebgcolor {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'newscoltitlebgcolor'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_absolutepathfileuploaddir {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'absolutepathfileuploaddir'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_ldap_base_dn {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'ldap_base_dn'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_if_ldap {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'if_ldap'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_basename {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'basename'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_ldap_uid {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'ldap_uid'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_ldap_uid_label {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'ldap_uid_label'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_max_total_number_upload {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'max_total_number_upload'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_defregmessage {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'defregmessage'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_aprregmessage {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'aprregmessage'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_rejregmessage {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'rejregmessage'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_subsmessage {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'subsmessage'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_password_checker {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'password_checker'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_simpleheader {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'simpleheader'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_stylesheet {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'stylesheet'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_subfooter {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'subfooter'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_subheader {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'subheader'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_regapproval {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'regapproval'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_rootdomain {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'rootdomain'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_sessionduration {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'sessionduration'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_toolbgcolor {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'toolbgcolor'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_proxyhost {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'proxyhost'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_proxyport {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'proxyport'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_ldap_server {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'ldap_server'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_timecounters {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'timecounters'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_maintemplate {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'maintemplate'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}
sub set_subtemplate {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'subtemplate'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}
sub set_utilitytemplate {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'utilitytemplate'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}
sub set_sessions_are_deleted_on_logout {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'sessions_are_deleted_on_logout'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}
sub set_subscriptionsender {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'subscriptionsender'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}
sub set_subscriptionhost {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'subscriptionhost'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}
sub set_subscriptionpwd {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'subscriptionpwd'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}
sub set_home_page {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'home_page'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}
sub set_post_login {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'post_login'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}
sub set_hide_my_page {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'hide_my_page'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}
sub set_require_login {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'require_login'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}
sub set_trusted_site {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'trusted_site'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}
sub set_is_event_log_enabled {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'is_event_log_enabled'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}
sub set_maxnewsitems {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'maxnewsitems'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}
sub set_loginsplash {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'loginsplash'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_items_are_rateable {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'items_are_rateable'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_new_user_notification_enabled {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'new_user_notification_enabled'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_new_user_notification_message {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'new_user_notification_message'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_is_jsform_enabled {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'is_jsform_enabled'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}
sub set_my_website_creation {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'my_website_creation'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}
sub set_email_friend {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'email_friend'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}
sub set_disable_allowed_image_urls_check{
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'disable_allowed_image_urls_check'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}
sub set_template_mgmt {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'template_mgmt'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_template_theme {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'template_theme'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_event_log_clean_after {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'event_log_clean_after'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_recursive_perms {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'recursive_perms'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_uri_scheme {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'uri_scheme'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_registration_closed {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'registration_closed'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_global_uploads_quota {
    my $self = shift;
    my $param = shift;
    DBAccess->sqlUpdate("params", "name = 'global_uploads_quota'", value => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}




# supporting access to theme table
{
    my %is_there;
    sub _create_themes_param_if_not_there {
    my $self = shift;
    my $param_name = shift;
        my $value      = shift;
    my $theme = $PARAMS->{template_theme};

    return if( $is_there{"$param_name,$theme"} );

    my ( $cnt ) = DBAccess->sqlSelect("count(*)", "theme", "pname='$param_name' and theme='$theme'" );

    unless( $cnt ) {

        DBAccess->sqlInsert("theme", pname => $param_name, theme => $theme, pvalue => $value);

    } else {

        $is_there{"$param_name,$theme"}++;
    }
    }
}

sub set_theme_property {
    my $self = shift;
    my $key = shift;
    my $value = shift;
    my $theme = $PARAMS->{template_theme};
    DBAccess->sqlUpdate("theme", "pname = '$key' and theme = '$theme'", pvalue => $value);
    $PARAMS->{$key} = $value;
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_header1_logo1 {
    my $self = shift;
    my $param = shift;
    my $theme = $PARAMS->{template_theme};
    DBAccess->sqlUpdate("theme", "pname = 'header1_logo1' and theme = '$theme'", pvalue => $param);
    $self->set_property('header1_logo1', $param);
}

sub set_header1_logo2 {
    my $self = shift;
    my $param = shift;
    my $theme = $PARAMS->{template_theme};
    DBAccess->sqlUpdate("theme", "pname = 'header1_logo2' and theme = '$theme'", pvalue => $param);
    $self->set_property('header1_logo2', $param);
}

sub set_header1_logo1_href {
    my $self = shift;
    my $param = shift;
    my $theme = $PARAMS->{template_theme};
    DBAccess->sqlUpdate("theme", "pname = 'header1_logo1_href' and theme = '$theme'", pvalue => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_header1_logo1_alt {
    my $self = shift;
    my $param = shift;
    my $theme = $PARAMS->{template_theme};
    DBAccess->sqlUpdate("theme", "pname = 'header1_logo1_alt' and theme = '$theme'", pvalue => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_header1_logo2_alt {
    my $self = shift;
    my $param = shift;
    my $theme = $PARAMS->{template_theme};
    DBAccess->sqlUpdate("theme", "pname = 'header1_logo2_alt' and theme = '$theme'", pvalue => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_header1_color {
    my $self = shift;
    my $param = shift;
    my $theme = $PARAMS->{template_theme};
    DBAccess->sqlUpdate("theme", "pname = 'header1_color' and theme = '$theme'", pvalue => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_header1_logo1_wdth {
    my $self = shift;
    my $param = shift;
    my $theme = $PARAMS->{template_theme};
    DBAccess->sqlUpdate("theme", "pname = 'header1_logo1_wdth' and theme = '$theme'", pvalue => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_header1_logo2_wdth {
    my $self = shift;
    my $param = shift;
    my $theme = $PARAMS->{template_theme};
    DBAccess->sqlUpdate("theme", "pname = 'header1_logo2_wdth' and theme = '$theme'", pvalue => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_leftnav1_color {
    my $self = shift;
    my $param = shift;
    my $theme = $PARAMS->{template_theme};
    DBAccess->sqlUpdate("theme", "pname = 'leftnav1_color' and theme = '$theme'", pvalue => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_nav_text_color {
    my $self = shift;
    my $param = shift;
    my $theme = $PARAMS->{template_theme};
    DBAccess->sqlUpdate("theme", "pname = 'nav_text_color' and theme = '$theme'", pvalue => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_title_text_color {
    my $self = shift;
    my $param = shift;
    my $theme = $PARAMS->{template_theme};
    DBAccess->sqlUpdate("theme", "pname = 'title_text_color' and theme = '$theme'", pvalue => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_theme_maintemplate {
    my $self = shift;
    my $param = shift;
    my $theme = $PARAMS->{template_theme};
    DBAccess->sqlUpdate("theme", "pname = 'theme_maintemplate' and theme = '$theme'", pvalue => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_theme_subtemplate {
    my $self = shift;
    my $param = shift;
    my $theme = $PARAMS->{template_theme};
    DBAccess->sqlUpdate("theme", "pname = 'theme_subtemplate' and theme = '$theme'", pvalue => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_theme_utilitytemplate {
    my $self = shift;
    my $param = shift;
    my $theme = $PARAMS->{template_theme};
    DBAccess->sqlUpdate("theme", "pname = 'theme_utilitytemplate' and theme = '$theme'", pvalue => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_theme_simpleheader {
    my $self = shift;
    my $param = shift;
    my $theme = $PARAMS->{template_theme};
    DBAccess->sqlUpdate("theme", "pname = 'theme_simpleheader' and theme = '$theme'", pvalue => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_theme_adminheader {
    my $self = shift;
    my $param = shift;
    my $theme = $PARAMS->{template_theme};
    DBAccess->sqlUpdate("theme", "pname = 'theme_adminheader' and theme = '$theme'", pvalue => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}

sub set_theme_mypage_color {
    my $self = shift;
    my $param = shift;
    my $theme = $PARAMS->{template_theme};
    DBAccess->sqlUpdate("theme", "pname = 'theme_mypage_color' and theme = '$theme'", pvalue => $param);
    $CACHE->set_as_dirty('ADMIN_PARAMS');
}


1;

