#!/usr/bin/perl -w
#
###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################


package Sendmail;
use strict;

use Carp;

use Metadotclass;
use Metadot qw($PARAMS $DISPLAY);
use MetadotConfig;
use ErrorHandling;
use Utility;
use Net::SMTP;
#  use Mail::Sendmail;
use Data::Dumper;
use POSIX;

use constant DEFAULT_MAIL_CONTENT_TYPE   => 'text/plain';

sub new {
  my $proto = shift;

  # offer a new hash-oriented interface
  my $args;

  if (@_) {
      if (ref $_[0] eq 'HASH') {
          $args = shift;
      } else {
          $args = {
              to        => shift,
              cc        => shift,
              bcc       => shift,
              subject   => shift,
              body      => shift,
          };
      }
  }

  my $from      = $args->{from} || $PARAMS->{mailsenderaddress};
  my $to        = $args->{to} || die "must provide 'to' information";
  my $cc        = $args->{cc};
  my $bcc       = $args->{bcc};
  my $subject   = $args->{subject};
  my $body      = $args->{body};
  my $content_type        = $args->{content_type};

  if (!($subject || $body)) {
      die "must provide either 'subject' or body information";
  }



# $self->debug_msg(1,  "in sendmail new: subj is $subject and body is $body");

  my $class = ref($proto) || $proto;
  my $self = {};

  $self->{from} = $from;

  $self->{to} = $to;
  if ($subject) {
      $self->{subject} = $subject;
  }
  if ($body) {
      $self->{body} = $body;
  }
  $self->{cc} = $cc if ($cc);
  $self->{bcc} = $bcc if ($bcc);

  $self->{content_type} = $content_type;

  bless ($self, $class);

  $self->die_on_error($args->{die_on_error});
  $self->debug_level($args->{debug_level});

  my $suppress_emails = defined ($args->{suppress_emails}) ? $args->{suppress_emails} : (MetadotConfig->get_param_value('suppress_emails') || 0);
  $self->suppress_emails($suppress_emails);
  $self->debug_msg(2, "suppress_emails? '".$self->suppress_emails()."'");

  return $self;
}

sub set_from {
    my $self = shift;
    my $from = shift;
    $self->{from} = $from;
}

sub set_to {
    my $self = shift;
    my $to = shift;
    $self->{to} = $to;
}

sub set_body {
    my $self = shift;
    my $body = shift;
    $self->{body} = $body;
}

sub set_reply_to {
    my $self = shift;
    my $rt = shift;
    $self->{reply_to} = $rt;
}

sub set_message_id {
    my $self = shift;
    my $mid = shift;
    $self->{msg_id} = $mid;
}

sub set_urgent {
    my $self = shift;
    my $urgent = shift;
    $self->{urgent} = $urgent;
}

sub set_content_type {
    my $self = shift;
    my $enc = shift;
    $self->{content_type} = $enc;
}

sub set_debug {
    my $self  = shift;
    my $value = shift;
    $self->{debug} = $value;
}

sub sendMail{
    my $self = shift;
    my $body = $self->{body};
    my $subject = $self->{subject};
    my $to = $self->{to};
    my $cc = $self->{cc};
    my $bcc = $self->{bcc};
    my $message_id = $self->{msg_id};
    my $sender = $self->{from};
    my $reply_to = $self->{reply_to};
    my $content_type = $self->{content_type} || $self->get_default_mail_content_type();


    my $result = 0;
    my $retries = MetadotConfig->get_param_value('sendmail_attempts') || 1; # try this many times
    my $retry_interval = MetadotConfig->get_param_value('sendmail_retry_interval') || 20; # wait this many seconds between attempts
    my $attempts = 0; # track the number of attempts

    $self->debug_msg(3,  "in sub sendMail");
    $self->debug_msg(5,  "Current properties are:\n",Dumper $self);
    $self->debug_msg(4,  "Suppress emails? ".($self->suppress_emails() ? 'yes' : 'no'));

    my ($mailsenderaddress) = DBAccess->sqlSelect("value", "params", "name = 'mailsenderaddress'");
    my ($mailserveraddress) =  DBAccess->sqlSelect("value", "params", "name = 'mailserveraddress'");

    if (!$sender){
        $sender = $mailsenderaddress;
    } else {            # if a sender is specified
        #check if full email is given
        my @senderemail = split(/\@/, $sender);
        if ($senderemail[1]) {
            # then keep sender as is
        }
        else {          #  use mailsenderaddress' domain
            #my @domain = split(/\@/, $mailsenderaddress);
            #$sender = $sender . "@" . $domain[1];
            $sender = $sender . "@" . $mailserveraddress;
        }
    }

    if ($self->suppress_emails()) {
        $self->debug_msg(1,  "Sendmail would have sent email: ", Dumper($self));
        $result = 1;
    } else {

       while (!$result && ($attempts < $retries)) {
            if ($attempts++ > 0) {
                $self->debug_msg(3,  "Attempt #$attempts, sleeping $retry_interval seconds...");
                sleep($retry_interval);
            }
            my $errormsg;
            eval { ###try to send mail and allow error catching

                # CKIM: let's use the bcc and cc methods, just to be safe
                my @recipients = split (",", $to);
                #if ($cc) {
                #    push (@recipients, split (",", $cc));
                #}
                #if ($bcc) {
                #    push (@recipients, split (",", $bcc));
                #}


                @recipients = @{ Utility->uniquify_array(@recipients) };


                $self->debug_msg(3,  "Sending to recipients: @recipients");
                $self->debug_msg(3,  "connecting to '$mailserveraddress'");

                # removed the 'Hello' parameter, allowing Net::SMTP to guess
                my $smtp = Net::SMTP->new($mailserveraddress, Timeout => 30); # connect to an SMTP server

                # do null checking
                if (defined $smtp) {
                    $self->debug_msg(4,  "Established SMTP connection");

                    # use SASL authentication if needed
                    # my $sasl_username = $PARAMS->{saslusername};
                    # if (defined $sasl_username && $sasl_username ne '')
                    # {
                    #    my $sasl_password = $PARAMS->{saslpassword};
                    #    my $sasl_result = $smtp->auth($sasl_username, $sasl_password);

                     #   $self->debug_msg(1,  "SASL auth ($sasl_username) result: ".($sasl_result || 'undef'));
                     #   if ($sasl_result)
                     #   {
                     #       $self->debug_msg(1,  "Error while attempted to authenticate with SASL, '$sasl_username'\n";
                     #   }
                     # }

                    unless ($errormsg)
                    {
                        #$smtp->mail( $sender );     # use the sender's address here
                        # 03-27-03 - no, adding the sender's address apparently causes problems for some sendmail configs
                        # with some of the ways metadot addresses mail.

                        # $smtp->mail();
                        # CKIM 2005-08-30: if this is causing problems for some sendmail configs,
                        #    then the problem is with their system. The $sender argument
                        #    is required to get the RFC-822 FROM header in the email.

                        $smtp->mail($sender);

                        $self->debug_msg(4,  "Initiated mail transaction");

                        # XXX propagate error if any
                        my $to_result = $smtp->to(@recipients, { SkipBad => 1 } );   # recipient's address

                        # do the cc and bcc also
                        if ($cc) {
                            $to_result &&= $smtp->cc(split (',', $cc),  { SkipBad => 1 } );
                        }
                        if ($bcc) {
                            $to_result &&= $smtp->bcc(split (',', $bcc),  { SkipBad => 1 } );
                        }


                        $smtp->data();              # Start the mail

                        # Send the header.
                        if (defined($content_type) && ($content_type eq 'html' || $content_type eq 'text/html') ) {
                             $smtp->datasend("MIME-Version: 1.0\r\n");
                             $smtp->datasend("Content-type: text/html; charset=iso-8859-1\r\n");

                             if( $self->mail_content_type_is_html() ){
                                 # Convert \n for <br /> in the message body
                                 $DISPLAY->debug_msg(4,"Converting new lines for HTML breaks in body");
                                 $body =~ s/\n/<br \/>\n/g;
                             }
                        }

                        $smtp->datasend("Date: ".strftime("%a, %d %b %Y %H:%M:%S %Z ", localtime)."\n");
                        $smtp->datasend("From: $sender\n");

                        # changed the CC and BCC capitalization, in hopes of resolving non-conformance
                        # probs w/ SPE
                        $smtp->datasend("To: $to\n");
                        $smtp->datasend("Cc: $cc\n") if ($cc);
                        $smtp->datasend("Bcc: $bcc\n") if ($bcc);

                        if ($reply_to) {
                            $smtp->datasend("Reply-to: $reply_to\n");
                        }
                        $smtp->datasend("Subject: $subject\n");
                        if ($message_id) {
                            $smtp->datasend("Message-Id: $message_id\n");
                        }
                        $smtp->datasend("\n");

                        # Send the body.
                        $smtp->datasend("$body");
                        $smtp->dataend();           # Finish sending the mail
                    }
                    $smtp->quit;                # Close the SMTP connection

                    $self->debug_msg(3,  "Completed mail transaction");
                } else {
                    $errormsg = "Could not establish connection with mail server '$mailserveraddress'.\n";
                }
            }; # EVAL


            ### CATCH
            if ($@) {
                $errormsg = "Mail server problem\n\n$@\n";
                $errormsg .= "Error occured while trying to send a mail " .
                    "(server=$mailserveraddress, to=$to, cc=$cc, bcc=$bcc, subject=$subject, " .
                    "message id $message_id)";
            }

            if ($errormsg) {
                # only send the HTML error message if we're on the web
                if (Metadotclass->on_the_web() && ($attempts >= $retries)) {
                    ErrorHandling->printError($errormsg);
                }
                if (!Metadotclass->on_the_web()) {
                    $self->debug_msg(1,  "Error occured during transaction:\n$errormsg\n");
                }

                if ($self->die_on_error()) {
                    confess($errormsg);
                }

                $result = 0;
            } else {
                $result = 1;
            }
        }
    }

    return $result;
}


sub debug {
    my $self = shift;
    my $result = @_ ? $self->{debug} = shift : $self->{debug};
}

sub debug_level {
    my $self = shift;
    return $self->debug(@_);
}


sub die_on_error {
    my $self = shift;
    my $result = @_ ? $self->{die_on_error} = shift : $self->{die_on_error};
}

sub suppress_emails {
    my $self = shift;
    my $result = @_ ? $self->{suppress_emails} = shift : $self->{suppress_emails};
}


sub debug_msg {
    my ($self, $message_level, @messages) = @_;
    my $debug_level = $self->debug_level() || 0;
    if ($message_level <= $debug_level) {
        print STDERR "DEBUG: $self: ",@messages,"\n";
    }
}


sub get_default_mail_content_type {
    my $class = shift;
    return $PARAMS->{default_mail_content_type} || DEFAULT_MAIL_CONTENT_TYPE();
}


# returns a simple boolean to help Sendmail determine whether an content_type is html
sub mail_content_type_is_html {
    my $class = shift;
    my $content_type = shift || $class->get_default_mail_content_type();

    return Utility->content_type_is_html($content_type);
}


1;


__END__

=head1 NAME

Sendmail -

=head1 SYNOPSIS

    use Sendmail;

    $sm = Sendmail->new($to, $cc, $bcc, $subject, $msg);
    $sm->set_from($from);
    $sm->set_reply_to($rt);
    $sm->set_message_id($msg_id);
    $sm->sendMail ();



=head1 DESCRIPTION

B<Sendmail::sendMail> is a utility fcn that uses Net::SMTP to send email
messages.

Constructor: The $to parameter, and either the $subject or $msg (body)
parameters are required to be set.  The sender ('from' field) is not required;
$params->{mailsenderaddress} will be used for the sender if none
is provided.  Empty strings may be used for the $cc and $bcc fields, in which case
they will not be included in the header.

    $sm = Sendmail->new($to, $cc, $bcc, $subject, $msg);

Set the 'from' field for the message:

    $sm->set_from($from);

Set the 'reply-to' field for the message:

    $sm->set_reply_to($rt);

Set the message-id:

    $sm->set_message_id($msg_id);

Call the sendMail method.  Will utilize the current values for all the header fields.
Call any necessary 'set' methods above before calling C<sendMail>.

    $sm->sendMail ();


=cut
