###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Session;
use strict;

use Metadot qw(%FORM $SESSION $HTTP_HEADER_SENT $USER $PARAMS $DISPLAY $CGI $ACCESS_BROKER);
use Digest::MD5 qw(md5_hex);
use Carp;
use MIME::Base64;   #CKIM: required for storing binary information
use Storable qw(nfreeze thaw);
use vars qw(@ISA $AUTOLOAD);
use CGI qw(:all);
use DBAccess;
use Metadot::Utils::Theme;
use Metadotclass;
use MetadotConfig;
use Time::HiRes qw(gettimeofday tv_interval);

use Metadot::SessionHandlerFactory;





@ISA = qw(Portal);


my $AC = {
       BUNDLES => {

                   MOD =>    { DESC =>          'Edit',
                               OPS =>            [qw(edit_on edit_off)],
                               DEFAULT_LEVEL =>  2,
                               MIN_LEVEL =>      2,
                             },
                   DISP =>   { DESC =>          'View',
                               OPS =>            [qw(login login_check forgot_password logout auto_login set_debug_level)],

                               DEFAULT_LEVEL =>  0,
                               MIN_LEVEL =>      0,
                               DEFAULTS_TO =>    'on',
                             },
           # CKIM: moved set_debug_level so user can do it, if they know the url
                   ADMIN => {  DESC => 'Admin Modify',
                   OPS =>  [qw()],
                   DEFAULT_LEVEL => 10,
                   MIN_LEVEL => 10,
                   DEFAULTS_TO =>   'on',
                            },
                   EDITP =>  { DESC =>          'Change<br />Permissions',
                               OPS =>            [qw(edit_permissions set_permissions change_owner)],
                               DEFAULT_LEVEL =>  8,
                               MIN_LEVEL =>      8,
                             },
                  },
          DISPLAY_ORDER => [qw(DISP MOD DEL EDITP)],
        };

sub get_AC {
    return $AC;
}

sub is_a {
    my $self = shift;

    return ref($self);
}

sub id {
    my $self = shift;

    return 1;
}

sub get_owner_uid {
    my $self = shift;

    return MetadotConfig->get_reserved_uid('ADMIN');
}

sub parent_id {
    my $self = shift;

    return 0;
}


sub new {
    my $proto = shift;
    my $session_id  = shift;
    my $class = ref($proto) || $proto;
    my $self  = {};

    bless ($self, $class);

    $self->{session_id}              = undef;
    $self->{uid}                    = "";
    $self->{debug_level}             = 0;
    $self->{admin_mode}              = 0;
    $self->{ip}                     = "";
    $self->{session_info}             = "";
    $self->{last_online}             = "";
    $self->{referer}                = ( $self->on_the_web() ) ? $CGI->referer() : '';

    if ( $session_id ) {
        # we have a session_id and want to get information about
    # the session
    $self->_initialize( $session_id );
    }

    return $self;
}

sub is_expired {
    my $self = shift;

    my $last_online = $self->{last_online};

    # if last_online has no value, then this is a new session. and therefore not expired
    my $result = ( ($last_online ne '') && ($last_online < time - $PARAMS->{'sessionduration'} ));
    Metadotclass->debug_msg(5, "Session->is_expired calculates based on last_online: '$last_online', time: '".time."', sessionduration: '$PARAMS->{sessionduration}', comparing $last_online < ".(time - $PARAMS->{'sessionduration'})." result is '$result'");
    return $result;
}

sub restore {
    my $proto = shift;
    my $session_id  = shift;
    my $class = ref($proto) || $proto;
    my $self  = {};

    $self->{session_id}              = undef;
    $self->{uid}                    = "";
    $self->{debug_level}             = 0;
    $self->{admin_mode}              = 0;
    $self->{ip}                     = "";
    $self->{session_info}             = "";
    $self->{last_online}             = "";
    $self->{referer}                = CGI::referer();

    bless ($self, $class);

    if ( $session_id ) {
        # we have a session_id and want to get information about
        # the session

        return $self->_initialize( $session_id ) ? $self : undef;

    } else {

    return undef;
    }
}


###############################################################
#             Displaying login form
###############################################################

sub _get_login_form_content {
    my $self = shift;
    my $error_message = $FORM{error_message};

    my $email = $PARAMS->{sessions_are_deleted_on_logout}? '' : $USER->email();

    my $auth = Metadot::AuthenticatorFactory->get_authenticator();
    my $form = $auth->show_login_form($email, $error_message);

    #my $full_form = $DISPLAY->form_top() . $form . $DISPLAY->form_bottom();
    my $full_form = $DISPLAY->generate_form($form);

    return $full_form;
}

sub _get_login_form_box {
    my $self = shift;
    my $style = shift || "";
    my $error_message = $FORM{error_message};

    my $email = ($PARAMS->{sessions_are_deleted_on_logout})? '' : $USER->email();

    my $auth = Metadot::AuthenticatorFactory->get_authenticator();
    my $form = $auth->show_login_form($email, $error_message, $style);

    return $form;
}

sub www_login {
    my $self = shift;
    my $error_message=shift;

    $self->debug_msg(3, "Session::www_login()");

    # CKIM: Disabling this check. Don't know why it's used.
    #       If it's needed, it should use a different test than this.
    #if( MetadotConfig->get_param_value('user_management_is_external') ) {
    #
    #    Utility->md_redirect("index.pl");
    #}

    $USER->logout();

    if ($PARAMS->{require_login}) {
        # Login is required so use splash page
        my $iid = 0;
        my $page = $PARAMS->{loginsplash};
        $page = $DISPLAY->simple_expand($iid, $self, $page);

        $DISPLAY->print_http_header();
        print $PARAMS->{stylesheet};
        print $page;
        print $DISPLAY->close_html();

    }
    else {
        # Login not required. use normal login look and feel.

        my $email = $PARAMS->{sessions_are_deleted_on_logout} ? '' : $USER->email();

        my $authenticator = Metadot::AuthenticatorFactory->get_authenticator();

        my $form = $authenticator->show_login_form( $email, $error_message );

        #$DISPLAY->print_http_header();

        #aju
        my $formbgcolor;
        if (Metadot::Utils::Theme->use_theme_info()) {
            # for themes, hardwire form bg to light gray -- for now.
            $formbgcolor = Metadot::Utils::Theme->get_theme_default_bg();
        }
        else {
            $formbgcolor = $PARAMS->{'formbgcolor'};
        }

        $form = "<script type=\"text/javascript\">
                  <!--
                  function initializeFocus() { document.login_form.email.focus();}
                  // -->
                  </script>\n" . $form;

        $DISPLAY->print_form($form);
    }
}

##############################################################
#       Checking supplied credentials
##############################################################
sub www_login_check  {
    my $self = shift;

    $self->debug_msg(3, "Session::www_login_check()");

    my $authenticator = Metadot::AuthenticatorFactory->get_authenticator();

    return $authenticator->process_login_form();
}

sub www_forgot_password {
    my $self = shift;
    my $target = $self->referer();
    $self->debug_msg(3, "Session::www_forgot_password()");

    my $title_message;
    my $window_message;
    my $button_message;
    if ($FORM{new_user}) {
        $title_message="Welcome to Metadot Managed Services";
        $window_message="If you are a new user, please enter or confirm your email address below. You will receive an email that will complete your registration by allowing you to set your password.";
        $button_message="Complete my registration";
    } else {
        $title_message="Forgot Your Password?";
        $window_message="If you have forgotten your password enter the email address you use to log in and an email will be sent to you containing a link to reset your password.";
        $button_message="Mail password info";
    }

    my $rs = "<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\"><tr><td><FORM action='index.pl' method='POST'>";
    $rs .= "<input type='hidden' name='target' value='$target'>";
    $rs .= "<input type='hidden' name='isa' value='Session'>";
    $rs .= "<input type='hidden' name='op'  value='login_check'>";
    $rs .= "<input type='hidden' name='form_action'  value='forgot_pwd'>";
    $rs .= "<input type='hidden' name='new_user'  value='$FORM{new_user}'>";
    $rs .=  $self->title_bar($title_message);

    $rs .= "$window_message<br /><br />\n";

    $rs .= "<table border=\"0\" cellspacing=\"1\">";
    $rs .= "<tr><td bgcolor=\"#000000\">";

    $rs .= "<table border=\"0\" cellspacing=\"0\" cellpadding=\"5\">";
    $rs .= "<tr bgcolor=\"#ffffff\"><td>";
    $rs .= "<b>Email address:</b>";
    $rs .= "</td><td>";
    $rs .= "<INPUT type=text name=email value=\"$FORM{email}\" size=20>";
    $rs .= "</td></tr>";

    $rs .= "</table>";

    $rs .= "</td></tr>";
    $rs .= "</table>";

    $rs .= "<p /><INPUT type=submit name=action value=\"$button_message\" class='button'><br />";
    $rs .= "</FORM></td></tr></table>";

    #$DISPLAY->print_http_header();
    #aju
    my $formbgcolor;
    if (Metadot::Utils::Theme->use_theme_info()) {
    # for themes, hardwire form bg to light gray -- for now.
        $formbgcolor = Metadot::Utils::Theme->get_theme_default_bg();
    }
    else {
        $formbgcolor = $PARAMS->{'formbgcolor'};
    }

    #print $DISPLAY->show_page_top;
    #print $DISPLAY->simple_header() . "<body>\n";;
    #print "<center>" . $DISPLAY->form_top();
    #print "<tr bgcolor='#$formbgcolor'><td>&nbsp;</td><td align='center'>$rs</td><td>&nbsp;</td></tr>";
    #print $DISPLAY->form_bottom() . "</center>";
    #print $DISPLAY->close_html();

    $DISPLAY->print_form($rs);
}

##################################################


sub www_auto_login {
    my $uid;
    my $self = shift;

    # NICO-PASS: make sure this op is a FREE OP
    ## NICO(says): DONE

    my $session = Session->restore( $FORM{key} ) || Session->new();
    my $s_h = Metadot::SessionHandlerFactory->get_session_handler();

    my $target = Utility->get_qualified_url("index.pl?isa=User&new_user=$FORM{new_user}&op=modify_self&key=$FORM{key}&forgot=1&refererlink=index.pl#password");
    my $choice;
    if ($FORM{new_user}) {
        $choice = "Follow the link below to create your password.";
    } elsif( $FORM{reset_passwd} ){
             $target = Utility->get_qualified_url("index.pl?isa=User&op=change_password&key=".$FORM{key});
             $choice = $USER->password_expiration_message();
    } else {
        $choice = "Follow the link below to change your password.";
    }

    if ($FORM{site_master}) {
        $self->md_redirect("index.pl");
    } else {
        $self->two_choices($choice,
            "readme",
            "<a href='".$DISPLAY->escapeHREF($target)."'>Click Here</a>");
    }
}

##############################################################
# These are front-end functions for edit_on and edit_off
##############################################################

sub www_edit_on {
  my $self = shift;
  $self->debug_msg (4, "Session::show_edit_on");

  if($FORM{target} eq "login"){
        $self->www_login();
        exit 0;
  }
  $self->edit_on();


  my $target = $FORM{target} || $SESSION->{referer} || 'index.pl';

  #print STDERR " AT  WWW_EDIT_ON ABOUT TO REDIRECT TO: $target\n";

# XXX [claudio] unescaping is not needed by CGI::redirect and causes
# the redirect to fail when
# the encoded form params contain newlines
#  $target = $CGI->unescape($target);
  print $CGI->redirect($target);
}

##############################################################

sub www_edit_off {
  my $self = shift;
  $self->debug_msg (4, "Session::show_edit_off");

  $SESSION->set_value('GB_status','0');

  $self->edit_off();

  my $target = $FORM{target} || $SESSION->{referer} || 'index.pl';

# XXX [claudio] unescaping is not needed by CGI::redirect and causes
# the redirect to fail when
# the encoded form params contain newlines
#  $target = $CGI->unescape($target);

  print $CGI->redirect ($target);
}

##############################################################

# set_debug_level permissions are special: if you are operating locally,
# then any user can set debug mode
sub www_set_debug_level {
    my $self = shift;

    if (lc($ENV{HTTP_HOST}) eq 'localhost' || $ACCESS_BROKER->is_allowed_to_do('admin_tasks'))
    {
      $self->debug_msg(3, "setting debuglevel to $FORM{value}");
      $FORM{value}= 0 if (!$FORM{value});

      $self->debug_level($FORM{value});
      $self->save();

      my $c1="Let's go!";
      my $url1 = $self->{referer};
      if ($self->debug_level > 0){
      $self->two_choices( "You are now in debug mode (value=". $self->debug_level . ")<br /><br />", "readme", $c1, $url1 );

      } else{
      my $c1="Alright!";
      $self->two_choices( "Debug mode switched off<br /><br />", "readme", $c1, $url1 );
      }
    }
    else
    {
      $DISPLAY->print_click_back( "You are not allowed to enable debug mode.");
      exit;
    }

    return;
}

##############################################################
# User's logout
##############################################################

sub www_logout {
    my $x;
    my $self = shift;

    # if login is required, redirect login page
    if ($PARAMS->{require_login}) {
        $USER->logout();
        #$self->www_login();
        Utility->md_redirect("index.pl?isa=Session&op=login");
        exit();
    }

    $USER->logout();

    if($FORM{page} eq "mypage"){

    # Now we need to redirect to mypage
    $self->md_redirect("userchannel.pl");

    } else {

    # Now we need to redirect to main portal
    $self->md_redirect("index.pl");

    }
}

sub _expire {
    my $self = shift;

    my $session_id = $self->{session_id};
    my $now = time();

    my $expiration_time = $now - $PARAMS->{sessionduration};
    $expiration_time--;

    DBAccess->sqlUpdate( "sessions", "sessionid='".(defined $session_id ? $session_id : 'undef')."'", ( lastonline=>$expiration_time ) );

    return 1;
}


##############################################################

sub _initialize {

    my $self       = shift;
    my $session_id = shift;

    if (! $session_id ) {
    #Something's going wrong
    $self->debug_msg (5, "Session::_initialize - no session_id argument");
    return -1;
    }

    # we have a session_id and want to get information about
    # the session

    $self->debug_msg(5,"Session::_initialize  session_id = $session_id\n");
    $self->{session_id} = $session_id;

    #Taking the information from session table in the database
    ($self->{uid},
     $self->{last_online},
     $self->{session_info})= DBAccess->sqlSelect(
                         "userid, lastonline, sessioninfo",
                         "sessions",
                         "sessionid=".DBAccess->quote($session_id)      #secu_200401
                        );

    if ($self->{uid}){ #found an entry with given session_id

    $self->_parse_session_info();
    $self->debug_msg(5,"Session::_initialize - found an entry:uid = $self->{uid}\n");

    } else {

    #There is no such session_id in the database
    return undef;
    }

    return 1;
}

###################################################################
# Here we generate new object session taking necessary information
# from table "user"
############################################################

sub add {
    my $self = shift;
    my $uid  = shift || '';

    # we want to create a new sessioin object and we know
    # an uid.

    my @chars = ( "A" .. "Z", "a" .. "z", 0 .. 9);
    my $auxid = "";
    my $session_id = "";

    $uid = undef unless (Utility->is_valid_integer($uid));    # secu_200401

    do {

        do {
        # Generating a random sequence of characters:
        # actually the example of generating random strings
        # has been taken from "Perl Cookbook".
        # We'd like to thank its authors (Tom Christiansen & Nathan Torkington)
        # for the great book!

        $session_id = join("", @chars[ map { rand @chars } ( 1 .. 19 ) ]);

        #Making sure that there is no collisions - already existing
        # in the database the same session_ids.
        ($auxid) = DBAccess->sqlSelect("sessionid", "sessions", "sessionid='$session_id'");

        if ($auxid) {#Collision
            #          $self->debug_msg(5, "Session::new:: Oops... Collision, random generated session_id already exists. Trying regenerate the session_id");
        }

        } while ($auxid);


        $self->{session_id} = $session_id;
        $self->{uid}       = $uid;

        #creating an entry: taking default values from user table
        ($self->{admin_mode},
         $self->{ip}
        ) = DBAccess->sqlSelect(
                    "adminmode, ip",
                    "users",
                    "userid='$self->{uid}'");

        #now we must create an entry in the session table
        $self->_build_session_info();
        my $time = time();

        eval {
        DBAccess->sqlInsert ("sessions", sessionid  => $self->{session_id},
                 lastonline => $time,
                 userid        => $self->{uid} || 0,
                 sessioninfo => $self->{session_info}
                );
        };

        if ($@) {
            #print STDERR "An error was detected setting the session key. Generating a different session key.\n";
            # since there was a key collision, this may be due to multiple engines
            # running with the same random seed. Change the seed.
            my $seed = time() ^ ($$ + ($$ << 15));
            srand($seed);
            #print STDERR "Reset the random seed to $seed\n";
        }

    } while ($@);

    ## delete previous sessions if param set
    if ($self->{session_id} && $self->{uid} && $PARAMS->{delete_previous_sessions_on_login}) {
        $self->delete_previous_sessions(userid => $self->{uid}, except_session_id => $self->{session_id});
    }

    return $self->{session_id};
}

######################################################
# Two auxiliary functions below are used for building
# and parsing session information ("session_info").
# We do this because we store the information in the
# table "session" as a string. The string consists of
# pairs 'key=value'.
######################################################


######################################################
#         Parsing information
####################################################


######################################################
#         Parsing information
####################################################

sub _parse_session_info {
    my $self = shift;

    my $session_info = $self->{session_info};

    for my $pair (split($self->get_value_separator, $session_info)) {
        #CKIM get a single key/value pair
        my ($key_name, $value) = split("=", $pair, 2);
    if($key_name =~ s/^SESSION_HASH_//) {

        $self->{SESSION_HASH}{$key_name} = $value;

    } else {

        $self->{$key_name} = $value;

    }
    }
}


####################################################
#    Building the string to write in the database
####################################################

sub _build_session_info {
    my $self = shift;

    my $session_info = '';

    my @keys = (
        'admin_mode',
        'debug_level',
        'ip',
           );

    # encode the hardcoded params
    foreach my $key (@keys) {

#CKIM:  $session_info .= "$key=" . ($self->{$key}||"") . "\n";
#CKIM: 2 problems: using a linefeed as a carriage separator is not useful
#CKIM:             the boolean expression suppresses values of 0
        if (!defined $self->{$key}) {$self->{$key}=""};
    $session_info .= "$key=" . $self->{$key} . $self->get_value_separator;

    }

    # encode the dynamic ones, with a preceding marker
    foreach my $key (keys %{$self->{SESSION_HASH}}) {

#CKIM:  $session_info .= "SESSION_HASH_$key=" . $self->{SESSION_HASH}{$key} || ''  . "\n";
    # aju: bug fix reported by open-source user -- needed to add parens
    $session_info .= "SESSION_HASH_$key=" . ($self->{SESSION_HASH}{$key} || '');
    $session_info .= $self->get_value_separator;

    }

    $self->{session_info} = $session_info;

}


#CKIM: new value separator
sub get_value_separator {
    return '@@@VALUESEPARATOR@@@';
}

####################################################
# Deleting a tuple corresponding to the object from
# database. This function is opposite to the "add"
# function.
####################################################

sub erase {

  my $self      =  shift;
  my $session_id = $self->session_id();

  if ($session_id){
     $self->debug_msg(5, "Deleting session_id = $session_id from database");
     DBAccess->sqlDelete("sessions","sessionid = '$session_id'");
     $self->{session_id}              = undef;
     $self->{uid}                    = "";
     $self->{debug_level}             = 0;
     $self->{admin_mode}              = 0;
     $self->{ip}                     = "";
     $self->{session_info}             = "";
     $self->{last_online}             = "";
     $self->{referer}                   = $CGI->referer();

     return 0;
  }
  return -1;
}

####################################################
# Switching to 'edin_on' mode for the session
# Backend for 'www_edit_on' function
####################################################

sub edit_on {
    my $self = shift;
    my $session_id = $self->session_id();

    $self->admin_mode(1);
    $self->_build_session_info();
    $self->debug_msg(3, "Session::edit_on");

    #print STDERR "AT Session::edit_on: ", $session_id, ' -- ', $self->{session_info}, "\n";

    DBAccess->sqlUpdate("sessions", "sessionid='$session_id'", (sessioninfo=>$self->{session_info}) );
}

####################################################
# Switching to 'edin_off' mode for the session
# Backend for 'www_edit_off' function
#####################################################
sub edit_off {
    my $self = shift;
    my $session_id = $self->session_id();
    $self->admin_mode(0);
    $self->_build_session_info();
    $self->debug_msg(3,"Session::edit_off");
    DBAccess->sqlUpdate("sessions", "sessionid='".(defined $session_id ? $session_id : 'undef')."'", (sessioninfo=>$self->{session_info}) );
}
#####################################################

sub session_id {
  my $self = shift;

  #Make sure that nobody's going to change session_id using the
  #interface method.

  if (@_) { $self->debug_msg(5,"Session::session_id(...)  Hey man, you cannot change session_id in this way..."); return -1;}

  return $self->{session_id};

}

#####################################################
sub session_info {
  my $self = shift;

  #Since session_info should be a hidden parameter,
  # we need to disallow the method

  $self->debug_msg(5,"Session::session_info(...) session_info is a private variable - don't touch it using the interface method...") ;

  return -1;

}

######################################################################
# This function updates 'last_online' and 'session_info' fields in
# the session table.
#####################################################################

sub refresh_session{
  my $self = shift;
  my $ip = shift || "NO_IP";
  my $session_id = $self->{session_id};
  my $now = time();

  $self->{last_online} = $now;
  $self->{ip} = $ip;
  $self->_build_session_info();

  DBAccess->sqlUpdate("sessions", "sessionid='$session_id'", (lastonline=>$now, sessioninfo=>$self->{session_info}));
  return 1;

}

sub get_last_online_clause {
    my $self = shift;
    my $userid_table = shift;

    my $userid_term = (defined $userid_table ? $userid_table.'.userid' : 'userid');

    my $time = time - 900;
    return "$userid_term > 0 AND $time < sessions.lastonline";
}

sub get_members_online {
    my $self = shift;
    my $show_data = shift || 0;
    my $members_online;
    my $members_online_count = [];

    if( $show_data ){
       $members_online = DBAccess->sqlSelectMany("distinct userid", "sessions", $self->get_last_online_clause());
    } else {
       ($members_online_count) = DBAccess->sqlSelect("count(distinct userid)", "sessions", $self->get_last_online_clause());
       #print STDERR "show_data[$show_data] m[$members_online_count]\n\n";
       return $members_online_count;
    }
    #print STDERR "show_data[$show_data] m[$members_online]\n\n";

    return $members_online;
}

sub get_guests_online {
    my $self = shift;
    return 1;
}
sub get_members_online_full {
    my $self = shift;
    #my $members = $self->get_members_online();
    my $members = DBAccess->sqlSelectMany("distinct userid","sessions",$SESSION->get_last_online_clause());
    my @members;
    while( my $memberuid = $members->fetchrow() ){
          my $u = Metadot::UserFactory->get_user( $memberuid );
          my $portal_id = $u->has_portal();
          my $nick = $u->webname();

          my @icq = DBAccess->sqlSelect("icqnumber","users","userid = $memberuid");

          push @members,"$nick:$portal_id:$icq[0]";
    }
    return \@members;
}
#######################################################################

sub debug_level{
  my $self = shift;
  $self->{debug_level} = shift if (@_);

  return $self->{debug_level};
}

#######################################################################

sub admin_mode {
  my $self = shift;
  $self->{admin_mode} = shift if (@_);

  return ($self->{admin_mode} && ($FORM{op} ne 'show_printer_friendly') && !$self->is_expired());
}


##################################################################
#           Saving object in the database.
#           Watch out! You can not call "save" before add()!!
##################################################################


sub save{
    my $self = shift;
    my $session_id = $self->{session_id};

    $self->debug_msg(5, "Session::save session_id = ".($self->{session_id} || ''));

    # [claudio] is not really needed that we save permissions here
    #  $self->save_permissions();

    if (!$session_id) {

    $self->debug_msg(5,"Session::save() - no session_id, don't know what to do...");
    return 0;
    }

    $self->_build_session_info();

    DBAccess->sqlUpdate ("sessions", "sessionid = '$session_id'", (sessioninfo=>$self->{session_info}));

}

###########################################################################




###########################################################################
#
# set_value
#
# Will save the passed value into the current session object,
# which is saved into the database automatically. This method
# alongside with get_value provides a handy mechanism for
# storing transient data to pass from one hit to another as
# a user navigates the site, avoiding dangerous hidden form
# paramters. See User::_modify_settings for example of use.
#
sub set_value {
    my $self = shift;
    my $key_name = shift || die "must provide value key name";
    my $value = shift;

    unless(defined $value) { die "must provide value" }

    #CKIM: mime-encode for safety
    $value = encode_base64($value);

    $self->{SESSION_HASH}{$key_name} = $value;

    $self->save();
}

#
# get_value
#
# Companion method to set_value. See set_value.
#
sub get_value {
    my $self = shift;
    my $key_name = shift || die "must provide value key name";

    my $value = undef;

    if (defined $self->{SESSION_HASH}{$key_name}) {
        $value = $self->{SESSION_HASH}{$key_name};
        $value = &decode_base64($value);
    }

    return $value;
}


# returns an integer number computed from the session id
# which can be used instead of userid for anonymous users
sub make_session_id_hash {
    my $self = shift;

    my $session_id = $SESSION->{session_id};

    my $result;
    while( my $char = chop($session_id) ) {

    my $x = ord($char) % 10;
    $result .= $x;
    }

    # keep size at 7 digits
    # CKIM: move this to 10 digits
    $result = substr( $result, 0, 10 );

    # add a large number (to separate it from actual user ids)

    # CKIM: move this to 10 digits
    $result += 100000000;

#    print STDERR "\nJUST MADE A SESSION HASH Sess_id is $SESSION->{session_id}; --hash: $result\n";

    return $result;
}

sub delete_previous_sessions {
    my $self = shift;
    my %args = @_;

    my $userid = $args{userid};
    my $except_session_id = $args{except_session_id};

    $userid = undef unless (Utility->is_valid_integer($userid));    # secu_200401
    return unless $userid > 0 && $except_session_id;

    my $whereclause = "userid=$userid and sessionid != ".DBAccess->quote($except_session_id);

    DBAccess->sqlDelete("sessions", $whereclause);
}

sub set_start_timer {
    my $self = shift;
    my $start_time = shift || [];

    $SESSION->set_value('timer_start',nfreeze($start_time));
}

sub timer_count {
    my $self = shift;

    my $start_time = $SESSION->get_value('timer_start');
    return unless $start_time;
    $start_time = thaw($start_time);
    return unless $start_time;

    return tv_interval ($start_time, [gettimeofday]);
}

sub get_timer_count_html {
    my $self = shift;
    my $timer_count = shift;

    unless (defined $timer_count) {
        $timer_count = $SESSION->timer_count();
    }

    my $adverb = ($timer_count > 1) ? 'reluctantly' : 'proudly';

    return "\n\n<!-- Page $adverb processed in $timer_count second(s) -->\n\n";
}


# returns the referer, including some form variables that may have been posted.
sub generate_referer {
    my $self = shift;
    my $result = $self->referer() || 'index.pl';

    $DISPLAY->debug_msg(4, "generate_referer begins with '$result'");

    # place form variables into the query string
    my @query_vars;
    my @ops = qw(isa iid op);
    foreach my $op (@ops) {
        $DISPLAY->debug_msg(5, "generate_referer checks for op '$op'");
        if (defined $FORM{$op} && $result !~ /$op\=/) {
            my $var = "$op=$FORM{$op}";
            push @query_vars, $var;
            $DISPLAY->debug_msg(5, "generate_referer adds '$var'");
        }
    }

    # add the vars if needed
    if (@query_vars) {
        $DISPLAY->debug_msg(5, "generate_referer builds query string");
        # add the ? to the path if needed, to indicate query string
        unless ($result =~ /\?/) {
            $result .= '?';
        }

        # add the vars themselves, separated by &
        $result .= join('&', @query_vars);
    }

    $DISPLAY->debug_msg(4, "generate_referer returns '$result'");
    return $result;
}



#NICO-PASS:
# Enable this session to reset password for a user. The rationale is that
# an anonymous user, once he has demonstrated that he has the right to
# reset the password for a user, will have a session that gives him the
# ability to do so.
# Creates a session and sets a session variable which will allow
# for the resetting of a user's password





##########################################
# exec methods service MetadotController
##########################################

sub exec_login {
    my $self    = shift;
    my $context = shift;

    if( $FORM{notyou} == 1) {
        $DISPLAY->debug_msg(3,"Will logout the current user notyou");
        $USER->logout();
    }
    return 'login';
}



sub _change_user {
   my $self    = shift;
   my $userid  = shift;

   ## Set the user id of the person I want to become
   ## where the session's userid equals my id
   DBAccess->sqlUpdate('sessions','sessionid='.DBAccess->quote($self->session_id()), (userid => $userid) );
}




1;


__END__

=head1 NAME

Session

=head1 SYNOPSIS

    use Session;
    $session = Session->new ($session_id);

    $AC = $session->get_AC;
    $isa = $session->is_a ;
    $id = $session->id ;
    $session->save ;
    $uid = $session->get_owner_uid ;
    $id = $session->parent_id ;
    $session->not_logged_in_callback ;
    $session->www_login ;
    $session->www_login_check  ;
    $session->www_forgot_password ;
    $session->www_auto_login ;
    $session->www_edit_on ;
    $session->www_edit_off ;
    $session->www_set_debug_level ;
    $session->www_logout ;
    $session_id = $session->add ;
    $status = $session->erase ;
    $session->edit_on ;
    $session->edit_off ;
    $session_id = $session->session_id ;
    $session->session_info ;
    $session->refresh_session;
    $debug_level = $session->debug_level;
    $admin_mode $session->admin_mode ;
    $session->save;
    $session->set_value ;
    $value = $session->get_value ;

=head1 DESCRIPTION

Session will take care of various tasks related to authenticating users and keeping track of various ssession status variables as well as providing persistant storage to allow other parts of the application to keep users data across hits.

=head2 Methods

$session = Session->new ($session_id);

Will create a session or restore it from database storage if a $session_id is given.

$AC = $session->get_AC;
$isa = $session->is_a ;
$id = $session->id;
$uid = $session->get_owner_uid ;
$id = $session->parent_id ;
$session->not_logged_in_callback ;

Auditable.pm required methods. See Auditable documentation.

$session->www_login;
$session->www_login_check;
$session->www_logout;

Steps one, two and three of the login process (login, welcome screen and logout).

$session->www_forgot_password;
$session->www_auto_login ;

Steps one and two of the password recovery mechanism. First step will mail users a link from which they can change their password, which is the second step.

$session->www_edit_on ;
$session->www_edit_off ;

Will turn edit mode on and off. Edit mode will make different operations on the site content (such as cut, paste, delete and edit_permissions) available.

$session->edit_on ;
$session->edit_off ;

Backend methods that are called from www_edit_on and www_edit_off.

$session->admin_mode($level);

Will set or get the value of the admin_mode Session instance field, which determines id the session is in edit mode or not. Called from edit_on and edit_off.

$session->www_set_debug_level ;

Will change the global debug level value. Metadot's built in debug system will output different amounts of debug information to the browser depending on the level value.

$debug_level = $session->debug_level;

Called from www_debug_level to set a new debug level for this session. It's also used for reading the current debug level.

$session_id = $session->add ;

Will insert a new session instance into the database.

$session->save;

Will commit changes to a Session instance to the database. Newly created sessions must always be added to the database using the add method. save() is for sessions already in the database.

$status = $session->erase ;

Will remove a session instance from the sessions table in the database.

$session_id = $session->session_id ;

Returns the session id of a Session instance.

$session->refresh_session;

Will update some variables kept in a per session basis in the database. These variables are "last_online", which is a timestamp at the moment refresh_session is called, and "session_info", which keeps per-session variables for other modules in the program to keep data across hits.

$session->set_value() ;
$value = $session->get_value ;

Will save the passed value into the current session object, which is saved into the database automatically. This method alongside with get_value provides a handy mechanism for storing transient data to pass from one hit to another as a user navigates the site, avoiding dangerous hidden form paramters. See User::_modify_settings for example of use.


=cut
