#!/usr/bin/perl

#
# Test file for Session.pm
# $Id: Session.001.t,v 1.2 2005/01/27 21:37:55 cory Exp $
#

use strict;
use Test::More tests=>8;
use Metadot::Testing::Mechanizer;

use SmokeMetadot;
use DBAccess;
use Session;
use MetadotConfig;

# set up info
my $class_name   = 'Session';
my $account_name = 'admin';
my $password     = '123456';
my $uid;
my $session_id;
my $session_obj;
my $mech;
my $admin_uid = MetadotConfig->get_reserved_uid('ADMIN');
my $anon_uid  = MetadotConfig->get_reserved_uid('ANONYMOUS_USER');


$mech = Metadot::Testing::Mechanizer->new();

################################################################################
# Test: Visit the home page and get an anonymous session id
################################################################################
$mech->get_page();
$session_id = $mech->get_session_id();
ok(defined($session_id) && $session_id ne '', "I have session ID $session_id");
$session_obj = $class_name->restore($session_id);
ok($session_obj->uid() == $anon_uid, "Session had anonymous uid $anon_uid");

################################################################################
# Test: Login to the website using default admin password
################################################################################
$mech->login(
    account_name => $account_name, password => $password
);

$session_id = $mech->get_session_id();
ok(defined($session_id) && $session_id ne '', "I have session ID $session_id");

($uid) = DBAccess->sqlSelect(
    'userid', 'users', 'email='.DBAccess->quote($account_name)
);

$session_obj = $class_name->restore($session_id);
ok($session_obj->uid() == $uid, "Session had correct uid $uid");


################################################################################
# Test: Logout
################################################################################
$mech->logout();

$session_id = $mech->get_session_id();
ok(defined($session_id) && $session_id ne '', "I have session ID $session_id");

$session_obj = $class_name->restore($session_id);
# two ok conditions: an expired session, or no session
if (defined $session_obj) {
    ok($session_obj->is_expired, "Session was expired");
} else {
    ok(1, "No session object for logged out user");
}


################################################################################
# Test: Test logging in with incorrect password
################################################################################
$mech->login(
    account_name => $account_name, password => '_INCORRECT_PASSWORD_'
);

$session_id = $mech->get_session_id();
ok(defined($session_id) && $session_id ne '', "I have session ID $session_id");

$session_obj = $class_name->restore($session_id);
# two ok conditions: an anonymous session, or no session
if (defined $session_obj) {
    ok($session_obj->uid() == $anon_uid, "Session is still anonymous");
} else {
    ok(1, "No session object for failed login");
}






#################################################
# That's it!!!!
#################################################
