###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Subscription;
use strict;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $DISPLAY);

use DBAccess;
use Metadot::PortalFactory;

use Carp;

use Portal;
use Utility;

use vars qw(@ISA);
@ISA=qw(Portal);


use constant FREQUENCY_ALL       => 'all';
use constant FREQUENCY_DAILY     => 'onceaday';
use constant FREQUENCY_WEEKLY    => 'onceaweek';
use constant FREQUENCY_DEFAULT   => FREQUENCY_ALL();

use constant MEDIUM_EMAIL        => 'email';
use constant MEDIUM_ICQ          => 'icq';
use constant MEDIUM_PAGER        => 'pager';
use constant MEDIUM_DEFAULT      => MEDIUM_EMAIL();

my $AC = {
       BUNDLES => {
                   MOD =>    { DESC =>          'Edit',
                               OPS =>            [qw(add)],
                               DEFAULT_LEVEL =>  2,
                               MIN_LEVEL =>      2,
                             },
                   DEL =>    { DESC => 'Delete and Cut',
                               OPS => [qw(delete)],
                               DEFAULT_LEVEL => 8,
                               MIN_LEVEL => 8,
           #aju-- needed to change the above levels to 2 to get things to work
           #properly-- why?  not checking in the change yet..
#                                 DEFAULT_LEVEL => 2,
#                                 MIN_LEVEL => 2,
                                 },
                   EDITP =>  { DESC =>          'Change<br />Permissions',
                               OPS =>            [qw(edit_permissions set_permissions change_owner)],
                               DEFAULT_LEVEL =>  8,
                               MIN_LEVEL =>      8,
                             },
                  },
          DISPLAY_ORDER => [qw(DEL MOD EDITP)],
        };

sub get_AC {
    return $AC;
}

sub is_a {
    my $self = shift;

    return ref($self);
}

#sub name {
#   my $self = shift;
#
#   return ref($self);
#}

sub id {
    my $self = shift;

    return 1;
}

sub save {
    my $self = shift;

    $self->save_permissions;
}

sub get_owner_uid {
    my $self = shift;
#   return MetadotConfig->get_reserved_uid('ADMIN');
    #aju: the above was not right-- that way only admin could delete subscriptions.
    my $owner_uid = $USER->{uid};
#   print STDERR "in Subscription:get_owner_uid: $owner_uid\n";
    return $owner_uid;
}

sub parent_id {
    my $self = shift;

    return 0;
}

####################################################

sub new {
  my $proto = shift;

  my $class = ref($proto) || $proto;
  my $self = {};

  bless ($self, $class);

  return $self;
}

################################################################
####                  OPERATION HANDLER                    #####
################################################################

sub www_add {
  my $self = shift ;
  $self->debug_msg(3, "Subscription::www_add");

  $FORM{id} = undef unless (Utility->is_valid_integer($FORM{id}));    # secu_200401

  my $cid = $FORM{cid} || $self->defined_or_exit($FORM{id},"id is missing.");
  my $freq = $self->defined_or_exit($FORM{freq},"freq is missing.");
  my $medium = $FORM{medium} || MEDIUM_DEFAULT();
  my $uid = $FORM{userid} || $self->defined_or_exit($USER->uid(),"uid is missing.");
  #url to perform the add
  my $add_url = "userchannel.pl?id=$cid&isa=Subscription&freq=$freq&medium=$medium&op=add";
  #url to go to after add
  my $target = $FORM{target} || $SESSION->{referer} || "userchannel.pl?isa=MyPage&op=select&type=Selection";
  my $redirect = $FORM{no_redirect} || 0;

  if ((!$USER->pager_number()) && ($medium eq MEDIUM_PAGER())){
    $USER->www_require_pager_form($add_url,$target);
    exit();
  }
  if ((!$USER->icq_number()) && ($medium eq MEDIUM_ICQ())){
    $USER->www_require_icq_form($add_url,$target);
    exit();
  }

  $self->add_user_subscription({cid=>$cid, frequency=>$freq, medium=>$medium, userid=>$uid});

  # if url = userchannel.pl we come from a search POST form in select
  $target = "userchannel.pl?isa=MyPage&op=select&type=Selection" if ($target =~ m/userchannel\.pl$/);
  unless( $redirect ){
     $self->md_redirect($target);
  }
}


sub add_user_subscription {
    my $class = shift;
    my $args  = shift || {};

    my $userid      = $args->{userid} || $args->{uid};
    my $cid         = $args->{cid};
    my $frequency   = $args->{frequency};
    my $medium      = $args->{medium};
    my $opt_out     = $args->{opt_out};

    $cid = undef unless (Utility->is_valid_integer($cid));
    $userid = undef unless (Utility->is_valid_integer($userid));

    (defined $userid && defined $cid) || die("Must specify a userid and cid");

    $frequency ||= FREQUENCY_DEFAULT();

    # delete any existing subscriptions
    # CKIM: No! Sometimes this method is invoked when the user's subscription
    #       already exists, thereby clearing any record of the existing
    #       subscription and its accompanying channel information
    # $class->delete_user_subscription($args);

    my @wheres = (
        "cid=$cid",
        "userid=$userid",
        "medium='$medium'",
    );

    if (defined $opt_out) {
        push @wheres, "opt_out=$opt_out";
    } else {
        push @wheres, "opt_out IS NULL";
    }

    my $where = join ' AND ', @wheres;
    $DISPLAY->debug_msg(4, "add_user_subscription updates subscription WHERE $where");

    # create the new subscription
    my $db_now = Utility->db_now();

    #$DISPLAY->debug_msg(3,"Will insert/update subscription for u[$userid]
    #                        on cid[$cid] freq[$frequency] medium[$medium]
    #                        opt_out[$opt_out] where[$where]");

    return DBAccess->sqlInsertUpdate(
       'subscription',
        $where,
        userid          => $userid,
        cid             => $cid,
        frequency       => $frequency,
        medium          => $medium,
        opt_out         => $opt_out,
        creationdate    => $db_now,
    );
}


sub delete_user_subscription {
    my $class = shift;
    my $args  = shift || {};

    my $userid      = $args->{userid} || $args->{uid};
    my $cid         = $args->{cid};
    my $medium      = $args->{medium};
    my $opt_out     = $args->{opt_out};
    my $delete_all  = $args->{delete_all};

    my $db_now = Utility->db_now();

    (defined $userid && (defined $cid || defined $delete_all)) || die("Must specify a userid and cid");
    my $where = "userid=$userid";
    if (defined $cid) {
        $where .= " AND cid=$cid";
    }
    if (defined $medium) {
        $where .= ' AND medium='.DBAccess->quote($medium);
    }
    if (defined $opt_out) {
        $where .= " AND opt_out IS NOT NULL";
    }

    DBAccess->sqlDelete("subscription", $where);
}




sub www_delete {
  my $self = shift ;
  my $redirect_flag = $FORM{no_redirect} = 1;

  $self->debug_msg(3, "Subscription::www_delete");
  $FORM{id} = undef unless (Utility->is_valid_integer($FORM{id}));    # secu_200401

  my $cid = $self->defined_or_exit($FORM{id},"id is missing.");
  my $freq = $self->defined_or_exit($FORM{freq},"freq is missing.");
  my $uid = $FORM{userid} || $self->defined_or_exit($USER->uid(),"uid is missing.");
  my $medium = $FORM{medium} || MEDIUM_DEFAULT();

  $DISPLAY->debug_msg(5,"WILL DELETE userid[$uid] from cid[$cid] medium[$medium]");

  DBAccess->sqlDelete("subscription","userid=$uid and cid=$cid and frequency='$freq' and medium='$medium'");
  my $url = $SESSION->{referer};
  # if url = userchannel.pl we come from a search POST form in select
  $url = "userchannel.pl?isa=MyPage&op=select&type=Selection" if ($url =~ m/userchannel\.pl$/);

  unless( $redirect_flag && $ENV{'SCRIPT_NAME'} !~ /userchannel/ ){
     $self->md_redirect($url);
  }
}



### Get subscriber names for a specific discussion
sub get_subscriber_names {
    my $self = shift;
    my $disc_id = $FORM{iid} || die("Could not retrieve discussion id");

    $self->debug_msg(3,"Subscription::get_subscriber_names($disc_id)");

    my @members;
    my $subscribers = DBAccess->sqlSelectMany("distinct u.fullname, s.userid",
                                              "subscription s, users u, channel c",
                                              "c.iid=$disc_id and s.cid=c.cid and u.userid=s.userid AND opt_out IS NULL",
                                              "ORDER BY u.fullname");
    while( my $row_hash = $subscribers->fetchrow_hashref() ){
           push @members, $row_hash;
    }
    return \@members;
}

### Get all subcriptions for a particular user
sub get_user_subscriptions {
    my $self = shift;
    my $userid = shift || $FORM{uid} || die("Could not retrieve user id");

    $self->debug_msg(3,"Subscription::get_user_subscriptions($userid)");

    my @list = ();
    my $subscribed = DBAccess->sqlSelectMany("c.name,c.iid","subscription s, channel c",
                                         "c.isa = 'Discussion' and c.cid = s.cid and userid = $userid AND opt_out IS NULL");

    while( my ($disc,$id) = $subscribed->fetchrow() ){
           push @list,"$disc,$id";
    }
    $DISPLAY->debug_msg(4, "Returning results: @list");
    return \@list;
}


sub check_user_subscription {
    my $self = shift;
    my $userid = shift || $FORM{uid} || die("Could not retriee user id");
    my $channel = shift || die "Must supply channel id in check_user_subscription";
    my $medium  = shift || die "Must provide subscription medium!";

    $self->debug_msg(3,"Subscription::check_user_subscription($userid,$channel,$medium)");
    my ($count) = DBAccess->sqlSelect("count(*)","subscription",
                                      "cid = $channel and userid = $userid and medium = '$medium' AND opt_out IS NULL");

    return $count;
}

sub unsubscribe_user_from_discussion {
    my $self = shift;
    my $uid = shift;
    my $did = shift;
    my $no_redirect_flag = shift || 0;

    # Nico, I'm guessing you're going to fix bug 306 by putting a hack in here,
    # but I don't think you should do that.  - CKIM

    my $target = "index.pl?iid=$did&isa=Portal&op=view_subscribers";

    $self->debug_msg(3,"Subscription::unsubscribe_user_from_discussion($uid,$did,$target)");

    #print STDERR "\nIN subscription->unsubscribe with uid[$uid] did[$did]\n\n";

    ### Get channel id
    my ($cid,$name) = DBAccess->sqlSelect("cid,name","channel","iid = $did");
    #print STDERR "\n\nselect cid from channel where iid = $did\nchannel id[$cid] name[$name]\n\n";

    ### Delete subscription rows
    my $result = DBAccess->sqlDelete("subscription","cid = $cid and userid = $uid AND opt_out IS NULL");

    if( $no_redirect_flag ){
         return $result;
    }
    $self->md_redirect($target);
}


sub get_user_names_and_ids {
    my $self = shift;
    my $filter = shift;
    my $user_list;

    $self->debug_msg(3,"Subscription::get_user_names_and_ids($filter,$user_list)");

    $user_list = DBAccess->sqlSelectMany("distinct userid, fullname, nickname","users","email like '%$filter%' or nickname like '%$filter%'");

    my @list;
    while( my ($iid,$name,$nick) = $user_list->fetchrow() ){
           push @list,"$iid,$name,$nick";
    }
    return \@list;

}

########################################################
# do_subscribe_users
# subscribe or modify multiple user's subscription into a discussion
########################################################

sub do_subscribe_users {
   my $self = shift;
   my @uids;
   my @medium;
   my @frequency;
   my $did;

   my $frequency_hash = {};
   my $mediums_hash = {};
   my $unsubscribe_user = {};

   $self->debug_msg(3,"Subscription::do_subscribe_users()");

   my $num_mediums = 0;

   ########
   # Get all form variables and assign them to arrays or hashes
   # for later processing
   ########
   while( my ($key,$value) = each( %FORM ) ){
      $DISPLAY->debug_msg(4,"key[$key]=>value[$value]");
      if( $key =~ 'uid' ){
          push @uids,$value;
      }
      elsif( $key =~ MEDIUM_PAGER() || $key =~ MEDIUM_ICQ() ){
          $mediums_hash->{$key} = $value;
          $num_mediums++;
      }
      elsif( $key =~ 'freq' ){
          $key =~ s/freq//;
          $frequency_hash->{$key} = $value;
      }
      elsif( $key eq 'iid' ){
          $did = $value;
      }
   }
   #print STDERR "\n\nDiscussion id[$did]\n";
   #print STDERR "@uids\n";

   my $result;
   my $update;

   ########
   # We have discussion id, get the channel id from it
   ########
   my ($cid,$name) = DBAccess->sqlSelect("cid,name","channel","iid = $did");
   my $index = 0;
   my $args = {};

   $FORM{cid} = $cid;

   # used by www_delete
   $FORM{id} = $cid;

   ##########
   # we are passing this form variable to www_add/www_delete
   # so that they do not redirect, and process all of our user's subcriptions
   ##########
   $FORM{no_redirect} = 1;

   my $icq_index;
   my $pgr_index;
   my $medium;

   foreach my $user( @uids ){

      $FORM{freq} = $frequency_hash->{$user};
      $FORM{medium} = MEDIUM_DEFAULT();
      $FORM{userid} = $user;

      #Check if there is a ICQ/Pager subscription to remove
      if( $num_mediums == 0 ){
          #there are no icq/pager selected in the form
          if( $self->check_user_subscription($user,$cid,MEDIUM_ICQ()) ){
              $num_mediums++;
          }
          if( $self->check_user_subscription($user,$cid,MEDIUM_PAGER()) ){
              $num_mediums++;
          }
      }

      if( $frequency_hash->{$user} eq 'none' ){
          ## Need to retrieve current subscription frequency
          my ($c_freq) = DBAccess->sqlSelect("frequency","subscription",
                                 "cid=$cid and medium=".DBAccess->quote(MEDIUM_DEFAULT())." and userid=$user");
          $FORM{freq} = $c_freq;

          ### if $c_freq returns empty, the user is not email subscribed
          if( defined($c_freq) ){
              $self->www_delete();
          }

      } else{
         $DISPLAY->debug_msg(3,"user[$user] will add/update[" . $FORM{medium} ."] subscription with freq[" . $FORM{freq} ."]!");

         # Check to see if user needs to be updated
         if( $self->user_needs_update($user,$cid,$FORM{medium},$FORM{freq}) ){
             $DISPLAY->debug_msg(5,"USER[$user] needs to have its subscription updated!");
             $self->www_add();
         }
      }

      ## see if we need to call www_add for pager/icq
      $icq_index = "icq$user";
      $pgr_index = "pager$user";

      my $num_mediums_copy = $num_mediums;

      my $icq_update = undef;
      my $pgr_update = undef;

      #########
      # Process icq/pager subscription
      # All icq/pager subscriptions are ASAP
      #########
      while( $num_mediums_copy ){
          $FORM{freq} = 'all';

          ####
          # get user object to test for ICQ/PAGER information
          ####
          my $subscription_user = Metadot::PortalFactory->get_user($user);

          ### Process icq
          if( defined( $mediums_hash->{$icq_index} ) ){
              $FORM{medium} = $mediums_hash->{$icq_index};
              #print STDERR "adding subscription for u[$user] medium[". $FORM{medium} . "] cid[$cid]\n";

              if( $subscription_user->icq_number() ){
                  if( $self->user_needs_update($user,$cid,$FORM{medium},'all') ){
                      $self->www_add();
                  }
              }
          } else {
              $medium = MEDIUM_ICQ();
              $icq_update = $self->check_user_subscription($user,$cid,$medium);
              if( defined($icq_update) && $icq_update ){
                  $FORM{medium} = $medium;
                  $self->www_delete();
              }
          }

          ### Process pager
          if( defined( $mediums_hash->{$pgr_index} ) ){
              $FORM{medium} = $mediums_hash->{$pgr_index};
              #print STDERR "adding subscription for u[$user] medium[". $FORM{medium} . "] cid[$cid]\n";

              if( $subscription_user->pager_number() ){
                  $self->www_add();
              }
          } else {
              $medium = MEDIUM_PAGER();
              $pgr_update = $self->check_user_subscription($user,$cid,$medium);
              if( defined($pgr_update) && $pgr_update ){
                  $FORM{medium} = $medium;
                  $self->www_delete();
              }
          }

          $num_mediums_copy--;
      }
   }

   # CKIM: this should not have been done this way
   #my $url = "index.pl?iid=" . $FORM{iid} . "&isa=Portal&op=view_subscribers";
   #$self->md_redirect($url);
}

sub check_icq_or_pager_subscription {
    my $self = shift;
    my $did  = shift;
    my $user = shift;
    my $type = shift;

    my ($cid) = DBAccess->sqlSelect("cid","channel","iid = $did");
    my ($res) = DBAccess->sqlSelect("count(*)","subscription",
                                    "cid=$cid and userid = $user and medium = '$type' AND opt_out IS NULL");

    return $res;

}

sub get_user_subscription_details {
    my $self = shift;
    my $args = Utility->coerce_hashref(@_);

    my $iid     = $args->{iid};
    my $cid     = $args->{cid};
    my $uid     = $args->{uid};
    my $medium  = $args->{medium} || MEDIUM_DEFAULT();

    if (!defined $cid && defined $iid) {
        ($cid) = DBAccess->sqlSelect("cid","channel","iid = $iid");
    }

    $self->debug_msg(3,"Subscription::get_user_subscription_details()");

    my @results = ();

    if (defined $cid) {
        my $details = DBAccess->sqlSelectMany(
            "*", "subscription",
            "cid = $cid AND userid = $uid AND medium = ".DBAccess->quote($medium)." AND opt_out IS NULL");

        while( my $info = $details->fetchrow_hashref() ){
               push @results, $info;
        }
    }

    return \@results;
}




# return the subscriber groups as a list of hashes
sub get_subscriber_groups {
    my $class  = shift;
    my $args   = shift || {};

    my $iid = $args->{iid};
    my $cid = $args->{cid};
    my $grp_id = $args->{grp_id};

    $iid = undef unless(Utility->is_valid_integer($iid));
    $cid = undef unless(Utility->is_valid_integer($cid));
    die  ("Must provide the channel's iid or cid") unless (defined($iid) || defined($cid));

    my $fields = 'g.grp_id, g.group_name, g.group_description, s.medium, s.frequency, s.optional';
    my $tables = 'subscription s, grp g, channel c';
    my $joins;
    if (defined $iid) {
        $joins = 'c.iid='.$iid;
    } elsif (defined $cid) {
        $joins = 'c.cid='.$cid;
    }

    if (defined $grp_id) {
        $joins .= ' AND s.grp_id='.$grp_id;
    }

    $joins .= ' AND s.cid = c.cid AND s.grp_id = g.grp_id';
    my $order  = 'ORDER BY g.group_name ASC, c.name ASC';

    my $sth = DBAccess->sqlSelectMany($fields, $tables, $joins, $order);
    my @results = ();

    while (my $row = $sth->fetchrow_hashref()) {
        push @results, $row;
    }
    $sth->finish();

    return \@results;
}

# returns an individual subscriber group
sub get_subscriber_group {
    my $class = shift;
    my $args  = shift || {};

    my $iid = $args->{iid};
    my $cid = $args->{cid};
    my $grp_id = $args->{grp_id};

    $iid = undef unless(Utility->is_valid_integer($iid));
    $cid = undef unless(Utility->is_valid_integer($cid));
    die  ("Must provide the channel's iid or cid") unless (defined($iid) || defined($cid));
    die  ("Must provide the grp_id") unless (defined($grp_id));

    my $list = $class->get_subscriber_groups($args);
    my $result;
    if (defined $list && ref($list)) {
        $result = $list->[0];
    }

    return $result;
}


# process group subscription
sub do_subscribe_groups {
    my $class = shift;
    my $iid = shift;
    my $input = shift || \%FORM;

    die  ("Must provide the iid of the instance that owns the channel") unless (Utility->is_valid_integer($iid));

    # get the channel id
    my $cid = Channel->get_cid_for_iid($iid);

    # get the grp id's
    my $grp_ids = $input->{grp_id};

    if (defined $cid && defined $grp_ids) {
        # make sure I have a list ref
        unless (ref $grp_ids) {
            $grp_ids = [$grp_ids];
        }

        foreach my $grp_id (@$grp_ids) {
            die  ("Received an invalid grp_id '$grp_id'") unless (Utility->is_valid_integer($iid));

            $class->do_update_group_subscription({
                grp_id => $grp_id,
                cid    => $cid,
                frequency => $input->{"freq.$grp_id"},
                optional  => $input->{"optional.$grp_id"},
                });
        }
    }
}


# Updates a group's subscription. Input should include
#  grp_id   : the id of the group for whom the subscription is updated
#  cid      : the cid of the channel the subscription is for
#  frequency: the frequency of the subscription
sub do_update_group_subscription {
    my $class = shift;
    my $args  = shift || {};

    my $grp_id = $args->{grp_id};
    my $cid    = $args->{cid};

    if (Utility->is_valid_integer($grp_id) && Utility->is_valid_integer($cid)) {

        # check if a notification frequency was specified
        my $frequency = $args->{frequency};
        if (defined $frequency && $frequency) {
            if ($frequency eq 'none') {
                # delete the subscription
                DBAccess->sqlDelete('subscription', "cid=$cid AND grp_id=$grp_id");
            } else {
                # check if an update is required
                $DISPLAY->debug_msg(4, "Checking if an update is needed for cid='$cid', grp_id='$grp_id'");
                my $existing = $class->get_subscriber_group({
                    cid => $cid,
                    grp_id => $grp_id,
                    });

                my $update_required;

                if (defined $existing) {
                    # check each property for a change
                    my @check_properties = qw(frequency optional);
                    foreach my $property (@check_properties) {
                        $DISPLAY->debug_msg(5, "Checking existing property $property: '$existing->{$property}' against '$args->{$property}'");
                        if ("$existing->{$property}" ne "$args->{$property}") {
                            $update_required = 1;
                            last;
                        }
                    }
                } else {
                    # if no existing, then obviously an update required
                    $update_required = 1;
                }

                if ($update_required) {
                    $DISPLAY->debug_msg(4, "Updating group subscription for cid='$cid', grp_id='$grp_id'");

                    $args->{creationdate} ||= Utility->db_now();
                    DBAccess->sqlInsertUpdate(
                        'subscription',
                        "cid=$cid AND grp_id=$grp_id",
                        %$args,
                        medium => MEDIUM_DEFAULT(),  # this is the default selection
                    );
                }
            }
        }
    }
}


# searches for groups by criteria, returns a data structure indicating the group's subscription
# status for a given iid or cid. Inputs:
#  - iid   : the iid of an instance whose channel to search against
#  - cid   : the cid of the channel to search against. iid OR cid is required
#  - name  : the string to search for. All groups matching that string (partial matching) will be
#            included.
#  - grp_id: the grp_id to search for
#
# The return values:
#  - grp_id
#  - group_name
#  - frequency
#  - optional
#  - cid
#  - channel_name
sub get_group_subscriptions {
    my $class = shift;
    my $args  = shift || {};

    my $iid    = $args->{iid};
    my $cid    = $args->{cid};
    my $name   = $args->{name};
    my $grp_id = $args->{grp_id};

    $iid = undef unless(Utility->is_valid_integer($iid));
    $cid = undef unless(Utility->is_valid_integer($cid));
    $grp_id = undef unless(Utility->is_valid_integer($grp_id));

    my @results = ();

    my @filters = ();

    # use either cid or iid
    if (defined $cid) {
        push @filters, 'c.cid='.$cid;
    }
    if (defined $iid) {
        push @filters, 'c.iid='.$iid;
    }
    if (defined $grp_id) {
        push @filters, 'g.grp_id='.$grp_id;
    }
    if (defined $name) {
        push @filters, 'LOWER(g.group_name) LIKE '.DBAccess->quote('%'.lc($name).'%');
    }

    # unless we're searching on name, we only want to return actual subscriptions
    push @filters, 's.sid IS NOT NULL';

    my $where = join(' AND ', @filters);
    return $class->get_group_subscriptions_by_query($where);
}


# searches all groups matching name, including subscription data for cid if it is subscribed
sub search_groups {
    my $class = shift;
    my $name  = shift || die("Must specify the group name");
    my $iid   = shift || die("Must specify a iid");

    $iid = undef unless(Utility->is_valid_integer($iid));

    # get all the matching groups
    my @results = ();
    my $sth = DBAccess->sqlSelectMany('grp_id, group_name', 'grp', 'UPPER(group_name) LIKE '.DBAccess->quote('%'.uc($name).'%'));
    while (my $row = $sth->fetchrow_hashref()) {
        push @results, $row;
    }
    $sth->finish();

    # see if any of the groups have subscriptions already
    foreach my $entry (@results) {
        $DISPLAY->debug_msg(5, "Have entry $entry->{group_name}, checking existing subscriptions with grp_id=$entry->{grp_id}, iid=$iid");
        my $existing = $class->get_group_subscriptions({grp_id => $entry->{grp_id}, iid=>$iid});
        if (@$existing) {
            my $existing_entry = $existing->[0];
            $DISPLAY->debug_msg(5, "I have existing entry $existing_entry");
            foreach my $key (keys %$existing_entry) {
                $entry->{$key} = $existing_entry->{$key};
            }
        }
    }

    return \@results;
}


sub get_group_subscriptions_by_query {
    my $class = shift;
    my $where = shift || '';

    my @results = ();

    # if we are searching on name, we do an outer join on group to get all the matching groups
    my $fields = 'g.grp_id, g.group_name, s.frequency, s.optional, s.opt_out, c.cid, c.iid, c.name channel_name';
    my $tables = 'subscription s RIGHT OUTER JOIN grp g ON g.grp_id = s.grp_id  LEFT JOIN channel c ON c.cid = s.cid';
    my $order  = 'ORDER BY g.group_name ASC, c.name ASC';

    #$DISPLAY->debug_msg(6, "SELECT $fields FROM $tables WHERE $where $order");
    my $sth = DBAccess->sqlSelectMany($fields, $tables, $where, $order);

    while (my $row = $sth->fetchrow_hashref()) {
        push @results, $row;
    }
    $sth->finish();

    return \@results;
}



# searches for groups subscriptions for a given user
# or group id
#  - uid     : the uid
#  - grp_id  : the grp_id to search agains.
#
# The return values:
#  - grp_id
#  - group_name
#  - frequency
#  - optional
sub get_group_subscriptions_for_user {
    my $class = shift;
    my $uid  = shift || $USER->uid();

    my @results = ();

    my @grp_ids = Utility->uniquify_list(@{Group->get_groups_where_member({uid => $uid, recursive=>1})});
    foreach my $grp_id (@grp_ids) {
        push @results, @{$class->get_group_subscriptions({grp_id => $grp_id})};
    }

    return \@results;
}



sub get_user_opt_out_status {
    my $class  = shift;
    my $uid    = shift;
    my $cid    = shift;

    return undef unless(Utility->is_valid_integer($uid));
    return undef unless(Utility->is_valid_integer($cid));

    my ($result) = DBAccess->sqlSelect('opt_out', 'subscription', "cid=$cid AND userid=$uid AND opt_out IS NOT NULL");
    $DISPLAY->debug_msg(5, "get_user_opt_out_status($uid, $cid) returns '$result'");

    return $result;
}

sub set_user_opt_out_status {
    my $class  = shift;
    my $uid    = shift;
    my $cid    = shift;
    my $value  = shift;

    return undef unless(Utility->is_valid_integer($uid));
    return undef unless(Utility->is_valid_integer($cid));

    $DISPLAY->debug_msg(5, "set_user_opt_out_status($uid, $cid) SETS '$value'");
    if ($value) {
        DBAccess->sqlInsertUpdate('subscription', "userid=$uid AND cid=$cid AND opt_out IS NOT NULL",
                                  cid=>$cid, opt_out => $value, userid => $uid,
                                  );
    } else {
        DBAccess->sqlDelete('subscription', "userid=$uid AND cid=$cid AND opt_out IS NOT NULL");
    }

    return $class->get_user_opt_out_status($uid, $cid);
}

sub user_needs_update {
    my $class  = shift;
    my $uid    = shift || die "Need uid!";
    my $cid    = shift || die "Need cid!";
    my $med    = shift || die "Need medium!";
    my $fre    = shift || die "Need frequency!";

    my ($c_med) = DBAccess->sqlSelect('medium','subscription',"cid=$cid AND userid=$uid AND frequency='$fre'");

    if( $med eq $c_med ){
        # no need to update record
        return 0;
    }
    return 1;
}

1;

__END__

=head1 NAME

Subscription -

=head1 SYNOPSIS

    use Subscription;
    $sub = Subscription->new ();

    $sub->www_add;
    $sub->www_delete;

    $AC = $sub->get_AC ;
    $isa = $sub->is_a ;
    $id = $sub->id ;
    $sub->save ;
    $uid = $sub->get_owner_uid ;
    $uid = $sub->parent_id ;
    $sub->not_logged_in_callback ;

=head1 DESCRIPTION

This module provides an interface to storage of subscriptions information. Subscriptions allow users to receive email/pager/icq updates whenever a channel changes its content (see Channel.pm documentation). Subscriptions are registered via the www_method, which is invoked from the user interface provided bt the MyPage module from userchannel.pl.

=head2 Methods

www_add

Will add a subscription for a user. It is called from the user interface generated by the www_select method in MyPage.pm.

www_delete

Will remove a subscription for a user. It is called from the user interface generated by the www_select method in MyPage.pm.

$AC = $sub->get_AC ;
$isa = $sub->is_a ;
$id = $sub->id ;
$sub->save ;
$uid = $sub->get_owner_uid ;
$uid = $sub->parent_id ;
$sub->not_logged_in_callback ;

Auditable.pm required methods. See Auditable documentation.


=cut
