#!/usr/bin/perl -w
#
###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################

## usage:  md_clean.pl [-options ...]
# ./md_clean.pl -h for help.
############################################################################

# This script does maintenance database cleaning for Metadot.  It currently
# does three things:  cleans session info, cleans old gizmotool items, and
# removes old event_log entries.  To clean event log entries, it requires one
# of its command-line parameters to be "-e N", where the script will clean all
# entries from the event_log db table older than N days old.  If the -e parameter
# is not specified, no event log cleaning will be done.


use strict;

use Fcntl ':flock';     #import flock + constants
use LWP::UserAgent;
use HTTP::Request;
use XML::RSS;



use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS);
use GizmoTool;
use Portal;
use DBAccess;
use Sendmail;
#  use Mail::POP3Client;
use Digest::MD5;

use Message;
use Metadot::SystemApp::EventLog::ReportGeneratorFactory;


BEGIN {
    use MetadotConfig;

    $ENV{'SERVER_NAME'} = "localhost" ;

}

sub debug_msg{
    my $level = shift;
    my $msg = shift;

     return if (!defined $level);
    return if (!defined $msg);
    my $eol="\n";
    my $head = "";

    if ($level <= $SESSION->{debug_level}){
    print STDERR" $head debug: $msg$eol" ;

    }

}


my %time_counters;
my $now = time;
my $dbaccess;

my $logdir    ;
my $logfile;
my $tmpdir;
my $tempfile ;
my $lockfile ;
my $channeldir;
my $overridelockfile;
my $badXML;
# the following flag controls whether or not to pop replies to discussion
# msgs from an external acct.
my $fetchflag = 0;
# if the following variable is set, do more verbose logging to metadot logfile.
my $logging = 0;


#$|=1;  # do not buffer output so we can see the progress in realtime.

my %http_return_code =
    ( 100   , "Continue",
      101   , "Switching Protocols",
      200   , "OK",
      201   , "Created",
      202   , "Accepted",
      203   , "Non-Authoritative Information",
      204   , "No Content",
      205   , "Reset Content",
      206   , "Partial Content",
      300   , "Multiple Choices",
      301   , "Moved Permanently",
      302   , "Moved Temporarily",
      303   , "See Other",
      304   , "Not Modified",
      305   , "Use Proxy",
      400   , "Bad Request",
      401   , "Unauthorized",
      402   , "Payment Required",
      403   , "Forbidden",
      404   , "Not Found",
      405   , "Method Not Allowed",
      406   , "Not Acceptable",
      407   , "Proxy Authentication Required",
      408   , "Request Time-out",
      409   , "Conflict",
      410   , "Gone",
      411   , "Length Required",
      412   , "Precondition Failed",
      413   , "Request Entity Too Large",
      414   , "Request-URI Too Large",
      415   , "Unsupported Media Type",
      500   , "Internal Server Error",
      501   , "Not Implemented",
      502   , "Bad Gateway",
      503   , "Service Unavailable",
      504   , "Gateway Time-out",
      505   , "HTTP Version not supported"
      );

my %weekdays = ('Sun' => 1,
        'Mon' => 2,
        'Tue' => 3,
        'Wed' => 4,
        'Thu' => 5,
        'Fri' => 6,
        'Sat' => 7);

 ##### Global variables

#  my $params = ParamsHash->begin();

 #################################################
 ## WARNING: DO NOT CHANGE
#  $logdir    =  $params->{'logdirectory'};
 ($logdir)    =  DBAccess->sqlSelect("value", "params", "name = 'logdirectory'");
 $logfile   = "$logdir/$0.log";
 $tmpdir    = "tmp";
 $tempfile  = "$tmpdir/$0.$$.tmp";
 $lockfile  = "$0.lock";
 $channeldir= "channels";


 ## For debug output

 my $eol="\n";

 # cmd line option: show debug information up to level you set here (0 to 5)
 #
 #

 my $show=0;

 # set to 1 tell this program to force the lock of the execution.
 #
 #

 $overridelockfile = 0;


 ##################################################
 # when $cleandb equals 1 we'll remove old
 # tuples from tables
 my $cleandb      = 0;
 my $daemon=0;
 my $reset = 0;

 ##################################################

my $userid = 0;
my $event_interval = 0;


my $use_server_list;
my %server_list;

#### read argument and set variables
$SESSION->{debug_level} = 0;
while ( $_ = $ARGV[0]) {
    shift;
    last if /^--$/;

    if (/^-o/)      { $overridelockfile = 1; debug_msg(3,"Overriding lockfile"); }
    elsif    (/^-m/)   { $daemon = 1; }
    elsif (/^-d/)   { $SESSION->{debug_level} = &getOption("-debug"); }
    elsif (/^-up/)  {
    $use_server_list = 1;
    my $servers = lc &getOption("-update_only");
    my @servers = split /\s+/, $servers;
    %server_list =  map {$_ => 1} @servers;
    }
    elsif (/^-u/)   { $userid = &getOption("-uid");
              debug_msg(3,"Doing cleaning only for user $userid"); }
    elsif (/^-e/)   { $event_interval = &getOption("-events");
              debug_msg(3,
                "Doing cleanup of events with interval $event_interval");
                }
    elsif (/^-r/)   { $reset = 1; }
    elsif (/^-log/) { $logging = 1; }
    elsif (/^-c/)         { debug_msg(3,"Will clean up tables - need to remove old tuples"); $cleandb=1;}
    elsif (/^-h/)   { &usage("HELP REQUESTED..."); }
    else {
    &usage("unknown argument: $_");
    }
}

debug_msg (3, "========= START LOOP =========");

my $lastdb = undef;
my $SECTION_NAME;

&ignition;
foreach my $SERVER_NAME (MetadotConfig->get_all_server_names) {

    if($use_server_list && not $server_list{lc $SERVER_NAME}) {
        debug_msg(3, "Skipping $SERVER_NAME");
        next;
    }

    $SECTION_NAME = MetadotConfig->get_section_name($SERVER_NAME);
    debug_msg (3,  "======= $SERVER_NAME: section $SECTION_NAME");
    $ENV{'SERVER_NAME'} = $SERVER_NAME;
    MetadotConfig->set_server_name($SERVER_NAME);
    my $thisdb = MetadotConfig->get_dbname_sid($SERVER_NAME);

    # initialize global variables
    my $mdot = Portal->new(1);

    if ($lastdb) {
        debug_msg (3, "\n\nthisdb = $thisdb\nlastdb = $lastdb");
    }

    if ($lastdb && ($lastdb eq $thisdb)){
        next;
    }
    else {
        $lastdb = $thisdb;
    };

    my $cleaningdone = 0;

    #####################################################################
    #
    #  MAIN
    #
    #####################################################################
    # do not call access method because it tries to modify DB


    debug_msg(2,"All systems are go, starting $0");

    my $fetchdone=0;

    my $time_update = 0 ;
    %time_counters = (
              h => $now, #hour
              d => $now, #day
              w => $now, #week
              );

    if ($reset) {
    # force time_counters update
    $time_update = 1 ;
    } else {
    # read time_counters from database

#   my $params = ParamsHash->begin;

#   my ($time_counters_string) = $params->{"timecounters"};
    my ($time_counters_string) = DBAccess->sqlSelect("value", "params", "name = 'timecounters'");
    if ($time_counters_string){
        my @info = split("-", $time_counters_string) ;
        foreach my $pair (@info) {
        my ($key, $field) = split "=", $pair;
        $time_counters{$key} = $field =~ /^\d+$/ ? $field : $now ;
        debug_msg(5, "counter $pair : $key => $field");
        }
    } else {
        debug_msg(1, "Looks like it is the first time we run Pal.");
        $time_update = 1 ;
    }
    }

    #end fetch time counters
    ####

#&process_event_log();
#&update_counter;
#exit;
    if ( $cleandb == 1 ){
    &clean_session($userid);
    &clean_gizmo_tools();
    &process_event_log();
    &update_counter;
    &clean_event_log();
    &clean_system_messages();
    &clean_basic_tools();
    $cleaningdone = 1;
    }

    debug_msg(2, "Before schedule...");
    debug_msg(2, "Now: ".localtime($now));
    debug_msg(2, "Hourly jobs are scheduled: ".localtime($time_counters{h}));
    debug_msg(2, "Daily jobs are scheduled:  ".localtime($time_counters{d}));
    debug_msg(2, "Weekly jobs are scheduled: ".localtime($time_counters{w}));

    if($daemon == 1) {

    debug_msg (1, "starting $0 daemon...\n");

    ##################################################
    # Do hourly stuff
    ##################################################
#   if (($now >= $time_counters{h})||($forceflags =~ m/h/)){
#       debug_msg(1, "Doing hourly duty..");
#       &update_hourly if ($forceflags !~ m/h/);
#   }


    ##################################################
    # Do daily stuff
    ##################################################
    if (($now >= $time_counters{d})) {
        debug_msg(1, "Doing daily duty..");
        &update_counter;
        if($cleaningdone != 1) {
        &clean_session($userid);

        ## Clears all gizmo tool items that have no
        ## corresponding channelitem
        &clean_gizmo_tools();
        &process_event_log();
        &clean_event_log();
        &clean_system_messages();
        &clean_basic_tools();

        $cleaningdone = 1;
        }
        &update_daily ;
    }
    ##################################################
    # Do weekly stuff
    ##################################################
#   if (($now >= $time_counters{w})||($forceflags =~ m/w/)){
#       debug_msg(1, "Doing weekly duty..");
#       &update_weekly if ($forceflags !~ m/w/);
#   }

    }
    &update_counter;
    DBAccess->close_db_object();
}

&safeLanding;       #cleanup;
exit(0);

######################################################################
#
# END OF MAIN
#
#####################################################################

sub update_counter {
  debug_msg(1, "Updating counter");
  my $new_timecounter = time;
  DBAccess->sqlInsertUpdate("params", "name = 'clean_timecounter'", ('name' => 'clean_timecounter', 'value' => "$new_timecounter"));
}

sub update_counters {
  debug_msg(1, "Updating counters");
  debug_msg(2, "Next hourly jobs: ".localtime($time_counters{h}));
  debug_msg(2, "Next daily jobs:  ".localtime($time_counters{d}));
  debug_msg(2, "Next weekly jobs: ".localtime($time_counters{w}));
#   my $params = ParamsHash->begin;
  my $new_timecounters = "h=$time_counters{h}-d=$time_counters{d}-w=$time_counters{w}";
  DBAccess->sqlUpdate("params", ('name' => 'clean_timecounters', 'value' => "$new_timecounters"));
}

sub update_weekly {
  localtime($time_counters{w}) =~ m/(...)\s(...)\s(.\d|\d\d)\s(\d\d):(\d\d):(\d\d)\s(\d\d\d\d)/;
  my $hour = $4;
  my $day = $1;
  localtime($now) =~ m/(...)\s(...)\s(.\d|\d\d)\s(\d\d):(\d\d):(\d\d)\s(\d\d\d\d)/;
  $time_counters{w} = $now + ($weekdays{$day}-$weekdays{$1})*24*60*60 + ($hour-$4)*60*60 - $5*60 -$6;
  $time_counters{w} += 7*24*60*60 if ($time_counters{w}<$now);
  &update_counters();
}

sub update_daily {
  localtime($time_counters{d}) =~ m/(...)\s(...)\s(.\d|\d\d)\s(\d\d):(\d\d):(\d\d)\s(\d\d\d\d)/;
  my $hour = $4;
  localtime($now) =~ m/(...)\s(...)\s(.\d|\d\d)\s(\d\d):(\d\d):(\d\d)\s(\d\d\d\d)/;
  $time_counters{d} = $now + ($hour-$4)*60*60 - $5*60 -$6;
  $time_counters{d} += 24*60*60 if ($time_counters{d}<$now);
  &update_counters();
}

sub update_hourly {
  localtime($now) =~ m/(...)\s(...)\s(.\d|\d\d)\s(\d\d):(\d\d):(\d\d)\s(\d\d\d\d)/;
  $time_counters{h} = $now + 60*60 - $5*60 -$6;
  &update_counters();
}



####################################################
sub safeLanding{

  debug_msg(1 , "Safe landing initiated $0");
  &removeLockFile;
  &logMessage ("End of program: $0");
  debug_msg  (1, "End of program: $0");

}

#####################################################
sub ignition{
  debug_msg(1, "Ignition sequence $0");
  ######################################################
  # LOCK FILE BUSINESS
  # check lock file exists
  # if it exists it means than another instance of this
  # prog is already running or than the file is an old lock file
  # from a process that already died.
  #

  if ($overridelockfile==1){
    debug_msg(1, "Removing lock file $lockfile\n");
    &removeLockFile;
  }

  if  (-e "$lockfile") {
    ### Here the lockfile exist.
    ### Check if the file is older than one day. if yes
    ### we remove it since this is probably due to a interrupted
    ### process resulting from an LDAP crash or a server restart.

    if ((-M $lockfile) >= 1){

      #&removeLockFile;
    } else {
      debug_msg(2,  "Lock file exists ($lockfile), use -o option to override. Exiting now.");
      exit(-1);
    }
  } else {
    debug_msg(4, "Lock file $lockfile doesn't exist");
  }

  ######
  ### we create a lock file and put the date in it
  ### as well as the PID#

  open (LOCKFILE, ">$lockfile")
    or warn "Failed to open: '$lockfile',  $?";
  my ($sec, $min, $hour, $mday, $mon, $year) = localtime(time);
  print LOCKFILE "$year-$mon-$mday $hour:$min:$sec : starting process $$,  ";
#  my $date = &timeStamp;
#  print LOCKFILE "$date: starting process $$,  ";
  close LOCKFILE;

  if (! -d "$channeldir" ) {
    mkdir($channeldir, 0755) || die ("cannot create directory: $channeldir: $?");
  }

  if (! -d "$logdir" ) {
    mkdir($logdir, 0755) || die ("cannot create directory: $logdir: $?");
  }
  debug_msg(3, "Starting $0");
  &logMessage("### starting $0\n");

  if ( -d "$tmpdir" ) {
    debug_msg(4, "Cleaning tmpdir directory: $tmpdir");
    unlink ("$tmpdir/*.tmp");
  }
  else{
    mkdir($tmpdir,0755) || die "cannot create directory: $tmpdir: $?";
  }

  &catch_signals;       ### allow graceful interrupts
}

###################################################
sub getOption {
  &usage("missing argument for $_[0]") if ($#ARGV==-1) ;
  my $result = $ARGV[0];
  shift @ARGV;
  return $result;
}

###################################################
sub usage {
  my ($mess) = @_;

  print "$0: $mess\n\n";

  print <<"_EOF_";

usage: $0 [-options ...]

where options include:
  -debug <debuglevel>  <1 to 5> (1=small amount, 5=ou la la!)
  -overridelockfile    run $0 even if the lock file exists
  -events              remove old entries from event_log that are older in days than this integer.
                       If this flag is used, it overrides the admin setting in the web interface.
  -uid <uid>           process session cleanup only for this user specific id.
  -reset               reset time counters to present time
  -cleandb             clean up tables in the database - remove old tuples
  -logging             enable more verbose logging
  -update_only "blank separated server list"    operate on a subset of available virtual servers

All options except -update_only can be abbreviated up to one letter.

Examples:
$0 -c -e 10 -d 3

_EOF_

  &removeLockFile;
  exit -1;
}


################################################
sub catch_signals {
  $SIG{'INT'}     = 'SIGNAL_CLEANUP';
  $SIG{'QUIT'}    = 'SIGNAL_CLEANUP';
  $SIG{'PIPE'}    = 'SIGNAL_CLEANUP';
  $SIG{'ALRM'}    = 'SIGNAL_CLEANUP';
  #$SIG{'__DIE__'} = 'SIGNAL_CLEANUP';   ###__WARN__ is also available
}


#################################################
sub SIGNAL_CLEANUP {

  #### quit gracefully

  unlink("$tmpdir/*.$$.tmp");
  &removeLockFile;
  print "$0 was interrupted, lock file ($lockfile) removed!\n";
  die @_;
}

#
#
# CleanString: remove heading spaces and tabs, replace multiple spaces and tabs by one between words
sub CleanString{
  my ($str) = @_;
  #print "<br /> cleaning str:'$str'<br />";
  $str =~ s/^\s+//;     # remove leading white chars
  #### $str =~ s/^[ \t]+//;
  $str =~ s/\s+$//;     # remove trailing white chars
  $str =~ s/[ \t]+/ /;      # replace n white chars by only 1 in the middle of the string
  #print " cleaned  str:'$str'<br />";

  return $str;
}

#########################################
sub removeLockFile {
  unlink "$lockfile";
}
########################################
sub logMessage
  {
    open (LOG, ">>$logfile") || warn "Cannot open $logfile\n";
    print LOG localtime()."\t",join("\t",@_),"\n";
    close LOG;
  }


########################################################
##########        Clean up functions          ##########
########################################################


########################################################
#      clean_gizmo_tools: removing old tuples
########################################################

sub clean_gizmo_tools {
    debug_msg(3, "in clean_gizmo_tools");
    # clean up gizmo tool paramater values
    my ($ciids) = DBAccess->sqlSelectMany("DISTINCT ciid", "gizmoitemparam");
    while (my($ciid) = $ciids->fetchrow()) {
    unless (DBAccess->sqlSelect("ciid", "channelitem", "ciid=$ciid")) {
#       $Mdot->debug_msg(5, "### Removing items with ciid=$ciid ###");
        DBAccess->sqlDelete("gizmoitemparam", "ciid=$ciid");
        DBAccess->sqlDelete("gizmotoolitem", "ciid=$ciid");
    }
    }
}


# removes system messages older than N age
sub clean_system_messages {
    debug_msg(3, "in clean_system_messages");

    use Metadot::SystemMessageManager;
    Metadot::SystemMessageManager->clean_messages( min_age=>86400 );

    # clean cache messages older than one hour
    my $object_accessor = Gizmo->get_object_accessor();
    my $cache           = $object_accessor->get_cache();

    debug_msg(4, "I have object accessor $object_accessor");
    debug_msg(4, "Is cached? ".$object_accessor->is_cached());
    debug_msg(4, "I have cache $cache");
    debug_msg(4, "Server name: ".MetadotConfig->get_server_name());

    if ($cache) {
        Metadot::SystemMessageManager->clean_messages(
            sender  => $cache->get_signature(),
            min_age => $cache->max_element_age(),
            );
    }
}


# clean the event log: remove all entries older than N days.
sub clean_event_log {

#     my $params_hash = ParamsHash->begin;

    if (!$event_interval) {
        print STDERR "getting event log cleaning interval from the db.\n";
        ($event_interval) = DBAccess->sqlSelect("value", "params", "name = 'event_log_clean_after'");
    #   $event_interval = $params_hash->{'event_log_clean_after'};
    }
    if ($event_interval) {
        debug_msg(2, "using event cleaning interval: $event_interval");
        my $time = time;
        debug_msg(3, "current time is: $time");
        my $seconds = $time - ($event_interval * 24 * 3600);
        # check that event_log table exists, as it may not always
        if (DBAccess->get_db_object()->existsTable("event_log")) {
        debug_msg(3, "removing event log entries older than $event_interval days, or < $seconds seconds");
        DBAccess->sqlDelete("event_log", "unix_time < $seconds");
        }
        else {
        debug_msg(1, "Not cleaning event_log: table does not exist in current database");
        }
    } else {
        print STDERR "No event log cleaning interval was defined. Skipping this step.\n";
    }
}


sub process_event_log {
    my $report_generator = Metadot::SystemApp::EventLog::ReportGeneratorFactory->get_report_generator();

    $report_generator->process_event_log();
}


########################################################
#      clean_session: removing old tuples
########################################################

sub clean_session {
  my $uid = shift;

#   my $params_hash = ParamsHash->begin;

  my ($session_duration) = DBAccess->sqlSelect("value", "params", "name = 'sessionduration'");
#   my ($session_duration) = $params_hash->{sessionduration};

  my $minimum_time = time - $session_duration;

  my $whereclause = ($uid)
    ? "userid='uid'and lastonline <= $minimum_time"
      : "lastonline <= $minimum_time";

  debug_msg(2, "Deleting sessions where '$whereclause'");
  DBAccess->sqlDelete("sessions",$whereclause);

  return ;
}

sub clean_basic_tools {
    my ($elements) = DBAccess->sqlSelectMany("distinct(id)", "mypage", "isa='BasicTool'",);
    while(my ($id)=$elements->fetchrow()) {
        next unless $id > 0;
        unless(Gizmo->exists($id)) {
            DBAccess->sqlDelete("mypage", "isa='BasicTool' and id = $id");
        }
    }
}


1;
