#!/usr/bin/perl
#
###########################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
############################################################################
#                                                                          #
# usage:  metadotd.pl [-options ...]                                       #
# ./metadotd.pl -h for help.                                               #
############################################################################

# aju 08-22-01: updating to work with MDK.  Requires addition of 'mid' column, of
# type int(11), to channelitem db.

use strict;

# moved this to Oracle.pm
# some implementation (SPE) require setting the ORACLE environment
#my $oracle_home = MetadotConfig->get_param_value('oracle_home');
#if ($oracle_home) {
#    print STDERR "Setting ORACLE_HOME to '$oracle_home'\n";
#    $ENV{ORACLE_HOME} = $oracle_home;
#}


use Fcntl ':flock';     #import flock + constants

#  use Data::Dumper;

use Metadot qw($USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS);
use GizmoTool;
use GizmoBuilder;
use Portal;
# use ParamsHash;
use Sendmail;
use Digest::MD5;
use Message;
use Metadot::User::Default;
use Subscription;
use Metadot::PortalFactory;

BEGIN {
    use MetadotConfig;

    $ENV{'SERVER_NAME'} = "localhost" ;

}


# we need to call this to initialize global vars
# such as $USER,
my $mdot = Portal->new(1);

# determine if execution must be split to
# avoid death from memory leaks
foreach my $arg_i ( 0 ..$#ARGV ) {

    my $arg = $ARGV[ $arg_i ];

    if( $arg =~ /^-split/ ) {
    my $split_n = $ARGV[ $arg_i + 1];
    unless( $split_n > 0 ) {
        &usage("missing argument for -split");
    }

    my @args;

    # prepare arguments for child processes, don't include split arg
    foreach my $arg_j ( 0 ..$#ARGV ) {
        unless( $arg_j == $arg_i or $arg_j == $arg_i + 1 ) {
        push( @args, $ARGV[ $arg_j ] );
        }
    }

    my @all_servers = MetadotConfig->get_all_server_names();

    my $num_iterations = ( $#all_servers / $split_n );

    if( $#all_servers % $split_n ) { $num_iterations++ }

    foreach( 0 .. ( $num_iterations - 1 ) ) {
        my $start_i = $split_n * $_;
        my $end_i = $start_i + ( $split_n - 1 );

        debug_msg(4, "LAUNCHING CHILD $start_i - $end_i\n");

        use Cwd;
        my $dir = cwd();
        system( "$dir/$0", ( @args, '-work_on_servers', $start_i, $end_i ) );
    }

    exit(0);
    }
}



my $DEBUG_LEVEL = 0;


sub debug_msg{
    my $level = shift;
    my $msg = shift;

    return if (!defined $level);
    return if (!defined $msg);
    my $eol="\n";
    my $head = "";

#     if ($level <= $SESSION->{debug_level}){
    if ($level <= $DEBUG_LEVEL) {
    print STDERR" $head debug: $msg$eol" ;

    }
}


my %time_counters;
my $now = time;
my $dbaccess;
my $db_now = Utility->db_now();

my $logdir    ;
my $logfile;
my $tmpdir;
my $tempfile ;
my $lockfile ;
my $overridelockfile;
my $badXML;
# the following flag controls whether or not to pop replies to discussion
# msgs from an external acct.
my $fetchflag = 0;
# if the following variable is set, do more verbose logging to metadot logfile.
my $logging = 0;


#$|=1;  # do not buffer output so we can see the progress in realtime.



my %weekdays = ('Sun' => 1,
        'Mon' => 2,
        'Tue' => 3,
        'Wed' => 4,
        'Thu' => 5,
        'Fri' => 6,
        'Sat' => 7);

 ##### Global variables


 #################################################
 ## WARNING: DO NOT CHANGE
 ($logdir)    =  DBAccess->sqlSelect("value", "params", "name = 'logdirectory'");

# XXX [claudio] we keep the ShareM version way
# $logfile   = "$logdir/$0.log";
# $logfile   = "$0.log";
 $logfile   = MetadotConfig->get_param_value('metadotd_logfile') || "$0.log";


 $tmpdir    = "tmp";
 $tempfile  = "$tmpdir/$0.$$.tmp";
 $lockfile  = "$0.lock";

 ##################################
 # dosubscription: =1 do notification
 #

 my $dosubscription=0;

 ##################################
 # fetch: =1 fetch external channel
 #

 my $externalchannel=0;

 ##################################

 ## FOr debug output

 my $eol="\n";

 # cmd line option: show debug information up to level you set here (0 to 5)
 #
 #

 my $show=0;

 # set to 1 tell this program to force the lock of the execution.
 #
 #

 $overridelockfile = 0;

 ##################################################
 # noprocess external RSS channels

 my $noprocess=0;

 $badXML=0;
 my $daemon=0;
 my $forceflags="";
 my $reset = 0;

 ##################################################
 # channelid

my $channelid   = undef;
my $userid      = undef;

#
my $use_server_list;
my %server_list;

my $work_on_servers_start;
my $work_on_servers_end;

#### read argument and set variables
# $SESSION->{debug_level} = 0;

while ( $_ = $ARGV[0]) {
  shift;
  last if /^--$/;

  if    (/^-m/)   { $daemon = 1; $dosubscription=1  }
  elsif (/^-o/)   { $overridelockfile = 1; debug_msg(3,"Overriding lockfile"); }
  elsif (/^-l/)   { $lockfile = &getOption("-l"); debug_msg(3,"Using lockfile: '$lockfile"); }
  elsif (/^-n/)   { $noprocess=1;debug_msg(3,"Won't process external channels"); }
  #     elsif (/^-d/)   { $SESSION->{debug_level} = &getOption("-debug"); }
  elsif (/^-d/)   { $DEBUG_LEVEL = &getOption("-debug"); }
  elsif (/^-i/)   { $channelid = &getOption("-id");  debug_msg(3,"Doing only channel#$channelid"); }
  elsif (/^-up/)  {
    $use_server_list = 1;
    my $servers = lc &getOption("-update_only");
    my @servers = split /\s+/, $servers;
    %server_list =  map {$_ => 1} @servers;
  } elsif (/^-u/) { $userid = &getOption("-uid"); $dosubscription=1; debug_msg(3,"Doing subscriptions only for user#$userid"); }
  elsif (/^-e/)   { debug_msg(3,"Will fetch external channels"); $externalchannel=1;   }
  elsif (/^-s/)   {
    debug_msg (3, "Will do subscription based notification");
    $dosubscription=1;
  }
  elsif (/^-fo/)    { $forceflags = &getOption("-force"); $daemon = 1; }
  elsif (/^-fetch/)     {
    $fetchflag = 1;
    debug_msg (3,  "Will be fetching discussion messages from pop account");
  }
  elsif (/^-r/)     { $reset = 1; }
  elsif (/^-log/) { $logging = 1; }
  elsif (/^-h/) { &usage("HELP REQUESTED..."); }
  elsif (/^-work_on_servers/)   {
    $work_on_servers_start = &getOption('-work_on_servers');
    $work_on_servers_end = &getOption('-work_on_servers');
    #   print STDERR "WORK ON SERVERS: $work_on_servers_start - $work_on_servers_end\n";
  }
  else {
    &usage("unknown argument: $_");
  }
}

debug_msg (3, "========= START LOOP =========");
my $lastdb = undef;
my $SECTION_NAME;

#  my @tlist = MetadotConfig->get_all_server_names();
#  print STDERR "all servers: " , Dumper(\@tlist), "\n";

############################################
&ignition;

# Get one channel processor for the entire job. It tracks the status of fetched
# RSS channels between server iterations.
my $channel_processor = Metadot::PortalFactory->get_channel_processor(
    timestamp       => $db_now,
    debug_level     => $DEBUG_LEVEL,
    fetch_flag      => $fetchflag,
    no_process_external => $noprocess,
);


my $work_on_servers_counter = 0;
foreach my $SERVER_NAME (MetadotConfig->get_all_server_names) {

    if( defined $work_on_servers_start ) {

    if( $work_on_servers_counter < $work_on_servers_start or
        $work_on_servers_counter > $work_on_servers_end ) {
        $work_on_servers_counter++;
        next
    }

    $work_on_servers_counter++;
    }

    next if($use_server_list && not $server_list{lc $SERVER_NAME});

    $SECTION_NAME = MetadotConfig->get_section_name($SERVER_NAME);

    debug_msg (3,  "======> BEGIN $SERVER_NAME: section $SECTION_NAME");

    $ENV{'SERVER_NAME'} = $SERVER_NAME;

    # aju -- passing in server name to fix issue w/ v servs; env
    # var apparently not doing what was intended.
    MetadotConfig->set_server_name($SERVER_NAME);

    # ckim -- this forces the portal to recalculate global variables, such as
    # $PARAMS
    $mdot = Portal->new(1);

    my $thisdb = MetadotConfig->get_dbname_sid($SERVER_NAME);
    debug_msg (3, "\n******db name is: $thisdb");

    #NZ moved the execution of mailtodiscussion so that all servers
    #in metadot.conf are run
    if( $fetchflag ) {
        use Metadot::EmailToDiscussion;
        my $email2disc = Metadot::EmailToDiscussion->new();
        my $email2disc_result = $email2disc->execute();
        my $email2disc_message = $email2disc->message();
        debug_msg(3, "email2disc_message:$email2disc_message");
    }

    if ($lastdb) {
        debug_msg (3, "\n\nthisdb = $thisdb\nlastdb = $lastdb");
    }

    if ($lastdb && ($lastdb eq $thisdb)){
        next;
    }
    else {
        $lastdb = $thisdb;
    };


    #####################################################################
    #
    #  MAIN
    #
    #####################################################################
    # do not call access method because it tries to modify DB

    debug_msg(2,"All systems are go, starting $0");
    my $fetchdone=0;
    my $time_update = 0 ;
    %time_counters = (
              h => $now, #hour
              d => $now, #day
              w => $now, #week
              );

    if ($reset) {
        # force time_counters update
        $time_update = 1 ;
    } else {
        # read time_counters from database
        my ($time_counters_string) = $PARAMS->{'timecounters'};
        if ($time_counters_string){
            my @info = split("-", $time_counters_string) ;
            foreach my $pair (@info) {
                my ($key, $field) = split "=", $pair;
                $time_counters{$key} = $field =~ /^\d+$/ ? $field : $now ;
                debug_msg(5, "counter $pair : $key => $field");
            }
        } else {
            debug_msg(1, "Looks like it is the first time we run Pal.");
            $time_update = 1 ;
        }
    }

    #end fetch time counters
    ####

    # remove old & not-displayed channelitems
    ## unix time for seven days ago
    my $old_time = time - ( 86400 * 7 );
    $channel_processor->delete_old_news($old_time);


    if ($externalchannel == 1) {
        $channel_processor->fetch_channels($channelid);
        $fetchdone=1;
    }

    debug_msg(2, "Before schedule...");
    debug_msg(2, "Now: ".localtime($now));
    debug_msg(2, "Hourly jobs are scheduled: ".localtime($time_counters{h}));
    debug_msg(2, "Daily jobs are scheduled:  ".localtime($time_counters{d}));
    debug_msg(2, "Weekly jobs are scheduled: ".localtime($time_counters{w}));
    debug_msg(2, "Do subscription? '$dosubscription'");
    debug_msg(2, "No process? '$noprocess'");
    debug_msg(2, "Fetch? '$fetchflag'");
    debug_msg(2, "Timestamp: '$db_now'");

    if($daemon == 1) {
        ##
        debug_msg (1, "starting $0 daemon...\n");

        ##################################################
        # Do hourly stuff
        ##################################################
        if (($now >= $time_counters{h})||($forceflags =~ m/h/)){
            debug_msg(1, "Doing hourly duty..");
            debug_msg(1, "forceflags: $forceflags..");
            $channel_processor->fetch_channels($channelid) if (!$fetchdone);
            $fetchdone=1;
            if ($forceflags !~ m/h/) {
        #       print STDERR "calling update_hourly with forceflags $forceflags\n";
                &update_hourly();
            }
        }

        # have to do channels here before sending subscription
        $channel_processor->do_channels($channelid);

        ##################################################
        # Do daily stuff
        ##################################################
        if (($now >= $time_counters{d})||($forceflags =~ m/d/)){
            debug_msg(1, "Doing daily duty..");

            if ($dosubscription)
            {
                $channel_processor->do_subscriptions(
                    frequency   => Subscription->FREQUENCY_DAILY(),
                    userid      => $userid,
                    channelid   => $channelid,
                );
                if ($channel_processor->error()) {
                    &sendMailFailed($channel_processor->error());
                }
            }

            &update_daily if ($forceflags !~ m/d/);
        }

        ##################################################
        # Do weekly stuff
        ##################################################
        if (($now >= $time_counters{w})||($forceflags =~ m/w/)){
            debug_msg(1, "Doing weekly duty..");

            if ($dosubscription)
            {
                $channel_processor->do_subscriptions(
                    frequency   => Subscription->FREQUENCY_WEEKLY(),
                    userid      => $userid,
                    channelid   => $channelid,
                );
                if ($channel_processor->error()) {
                    &sendMailFailed($channel_processor->error());
                }
            }

            &update_weekly if ($forceflags !~ m/w/);
        }

        ### send password expiration notifications
        &doSendPasswordExpireNotification();

        ### Each time duties
        if ($dosubscription)
        {
            $channel_processor->do_subscriptions(
                frequency   => Subscription->FREQUENCY_ALL(),
                userid      => $userid,
                channelid   => $channelid,
            );
            if ($channel_processor->error()) {
                &sendMailFailed($channel_processor->error());
            }
        }

        ### do Gizmo Tools
        &doGizmoToolScheduler($channelid);

    } else {
        $channel_processor->do_channels($channelid);
        if ($dosubscription)
        {
            $channel_processor->do_subscriptions(
                frequency   => Subscription->FREQUENCY_ALL(),
                userid      => $userid,
                channelid   => $channelid,
            );
            if ($channel_processor->error()) {
                &sendMailFailed($channel_processor->error());
            }
        }

    }

    # we do this to free memory during execution
    MetadotConfig->clear_config_cache();

    # In some implementations (SPE) this causes the script to stop
    unless (MetadotConfig->get_param_value('do_not_close_db_object_in_metadotd')) {
        DBAccess->close_db_object();
    }
}


&safeLanding;           #cleanup;



exit(0);

######################################################################
#
# END OF MAIN
#
#####################################################################

# aju ------------------------------

#######################################################################


# aju ------------------------------

sub update_counters {
    debug_msg(1, "Updating counters");
    debug_msg(2, "Next hourly jobs: ".localtime($time_counters{h}));
    debug_msg(2, "Next daily jobs:  ".localtime($time_counters{d}));
    debug_msg(2, "Next weekly jobs: ".localtime($time_counters{w}));

    my $new_timecounters = "h=$time_counters{h}-d=$time_counters{d}-w=$time_counters{w}";
    DBAccess->sqlUpdate("params", "name = 'timecounters'", (value => $new_timecounters));
}

sub update_weekly {
    localtime($time_counters{w}) =~ m/(...)\s(...)\s(.\d|\d\d)\s(\d\d):(\d\d):(\d\d)\s(\d\d\d\d)/;
    my $hour = $4;
    my $day = $1;
    localtime($now) =~ m/(...)\s(...)\s(.\d|\d\d)\s(\d\d):(\d\d):(\d\d)\s(\d\d\d\d)/;
    $time_counters{w} = $now + ($weekdays{$day}-$weekdays{$1})*24*60*60 + ($hour-$4)*60*60 - $5*60 -$6;
    $time_counters{w} += 7*24*60*60 if ($time_counters{w}<$now);
    &update_counters();
}

sub update_daily {
    localtime($time_counters{d}) =~ m/(...)\s(...)\s(.\d|\d\d)\s(\d\d):(\d\d):(\d\d)\s(\d\d\d\d)/;
    my $hour = $4;
    localtime($now) =~ m/(...)\s(...)\s(.\d|\d\d)\s(\d\d):(\d\d):(\d\d)\s(\d\d\d\d)/;
    $time_counters{d} = $now + ($hour-$4)*60*60 - $5*60 -$6;
    $time_counters{d} += 24*60*60 if ($time_counters{d}<$now);
    &update_counters();
}

sub update_hourly {
    localtime($now) =~ m/(...)\s(...)\s(.\d|\d\d)\s(\d\d):(\d\d):(\d\d)\s(\d\d\d\d)/;
    $time_counters{h} = $now + 60*60 - $5*60 -$6;
    &update_counters();
}























####################################################
sub safeLanding{

  debug_msg(1 , "Safe landing initiated $0");

  DBAccess->close_db_object();
  &removeLockFile;

  &logMessage ("End of program: $0");
  debug_msg  (1, "End of program: $0");
}

#####################################################
sub ignition{
  debug_msg(1, "Ignition sequence $0");

  ######################################################
  # LOCK FILE BUSINESS
  # check lock file exists
  # if it exists it means than another instance of this
  # prog is already running or than the file is an old lock file
  # from a process that already died.
  #

  if ($overridelockfile==1){
    debug_msg(1, "Removing lock file $lockfile\n");
    &removeLockFile;
  }

  if  (-e "$lockfile") {
    ### Here the lockfile exist.
    ### Check if the file is older than one day. if yes
    ### we remove it since this is probably due to a interrupted
    ### process resulting from an LDAP crash or a server restart.

    if ((-M $lockfile) >= 1){

      #&removeLockFile;
    } else {
      debug_msg(2,  "Lock file exists ($lockfile), use -o option to override. Exiting now.");
      exit(-1);
    }
  } else {
    debug_msg(4, "Lock file $lockfile doesn't exist");
  }

  ## check if site is suspended
  if (MetadotConfig->get_site_is_suspended() eq 'true') {
      debug_msg(1, "SITE IS SUSPENDED!! WILL SKIP!!!");
      exit(-1);
  } else {
      debug_msg(5, "Site not suspended.");
  }

  ######
  ### we create a lock file and put the date in it
  ### as well as the PID#

  open (LOCKFILE, ">$lockfile")
    or warn "Failed to open: '$lockfile',  $?";
  my ($sec, $min, $hour, $mday, $mon, $year) = localtime(time);
  print LOCKFILE "$year-$mon-$mday $hour:$min:$sec : starting process $$,  ";
#  my $date = &timeStamp;
#  print LOCKFILE "$date: starting process $$,  ";
  close LOCKFILE;

  if (! -d "$logdir" ) {
    mkdir($logdir, 0755) || warn ("cannot create directory: $logdir: $?");
  }
  debug_msg(3, "Starting $0");
  &logMessage("### starting $0\n");

  if ( -d "$tmpdir" ) {
    debug_msg(4, "Cleaning tmpdir directory: $tmpdir");
    unlink ("$tmpdir/*.tmp");
  }
  else{
    mkdir($tmpdir,0755) || warn "cannot create directory: $tmpdir: $?";
  }

  &catch_signals;       ### allow graceful interrupts
}

###################################################
sub getOption {
  &usage("missing argument for $_[0]") if ($#ARGV==-1) ;
  my $result = $ARGV[0];
  shift @ARGV;
  return $result;
}
###################################################
sub usage {
  my ($mess) = @_;

  print "$0: $mess\n\n";

  print <<"_EOF_";

usage: $0 [-options ...]

where options include:
  -debug <debuglevel>  <1 to 5> (1=small amount, 5=ou la la!)
  -overridelockfile    run $0 even if the lock file exists
  -l                   specifies the filename of the lock file
  -m                   run $0 as a daemon <= USE THIS FOR NORMAL OPERATIONS!
  -externalchannels    fetch external channels
  -subscription        do  subscription-based notifications
  -noprocess           do not process external channels
  -id <cid>            process only this channel specific id.
  -uid <uid>           process subscriptions only for this user specific id.
  -force <h|w|d>       force to run hourly/daily/weekly jobs
  -fetch               pop messages from the discussion email account
  -reset               reset time counters to present time
  -logging             enable more verbose logging
  -update_only "blank separated server list"    operate on a subset of available virtual servers

All options (except -force, -fetch and -update only) can be abbreviated up to one letter.

Examples:
### fetch external and override lockfile if exists
$0 -debug 3 -e -o

_EOF_



  &removeLockFile;
  exit -1;
}


################################################
sub catch_signals {
  $SIG{'INT'}     = 'SIGNAL_CLEANUP';
  $SIG{'QUIT'}    = 'SIGNAL_CLEANUP';
  $SIG{'PIPE'}    = 'SIGNAL_CLEANUP';
  $SIG{'ALRM'}    = 'SIGNAL_CLEANUP';
  #$SIG{'__DIE__'} = 'SIGNAL_CLEANUP';   ###__WARN__ is also available
}

################################################
sub BAD_XML{

  $badXML=1;
  print "badXML-int\n";

}
#################################################
sub SIGNAL_CLEANUP {

  #### quit gracefully

  unlink("$tmpdir/*.$$.tmp");
  &removeLockFile;
  print "$0 was interrupted, lock file ($lockfile) removed!\n";
  die @_;
}
#
#
# CleanString: remove heading spaces and tabs, replace multiple spaces and tabs by one between words
sub CleanString{
  my ($str) = @_;
  #print "<br /> cleaning str:'$str'<br />";
  $str =~ s/^\s+//;     # remove leading white chars
  #### $str =~ s/^[ \t]+//;
  $str =~ s/\s+$//;     # remove trailing white chars
  $str =~ s/[ \t]+/ /;      # replace n white chars by only 1 in the middle of the string
  #print " cleaned  str:'$str'<br />";

  return $str;
}
#########################################
sub removeLockFile {
  unlink "$lockfile";
  &logMessage("'$lockfile' has been removed.\n");
}
########################################

sub logMessage
  {
    #some implementations (SPE) want to pipe the log file externally
    unless (MetadotConfig->get_param_value('metadotd_pipe_log_file_externally')) {
        open ( LOG, ">>$logfile" ) || warn "Cannot open $logfile\n";
        print LOG localtime()."\t",join("\t",@_),"\n";
        close LOG;
    }
    print localtime()."\t",join("\t",@_),"\n";
  }







## GizmoTool Scheduler
sub doGizmoToolScheduler {
  my $only_cid = shift ;
  my @gizmo_tools = GizmoTool->lists_tools();
  debug_msg(3, "### Gizmo Tool Scheduler ###");

  my $where_clause = join(" or ",map("isa = '$_'", @gizmo_tools));
  $where_clause = "($where_clause)" if ($where_clause);
  $where_clause .= " and '$db_now' > lastprocessed";
  $where_clause .= " and cid=$only_cid" if ($only_cid);
  debug_msg(3, "where: $where_clause");
  my ($tool_channels) = DBAccess->sqlSelectMany("cid, isa","channel",$where_clause,"order by cid");
  while (my ($cid, $isa) = $tool_channels->fetchrow) {
    my $tool = $isa->new($cid);
    debug_msg(3, "$isa : on_schedule($cid) STARTS");
    $tool->on_schedule($cid);
    debug_msg(4, "$isa : on_schedule($cid) FINISHED");
    DBAccess->sqlUpdate("channel","cid=$cid",(lastprocessed => $db_now));
  }

}
#########################################
# doSendPasswordExpireNotification
# sends email notifications when a user's password is about to expire
# currently it handles this action when password is: 10, 5, 2, 1, 0
# days from expiring
#########################################
sub doSendPasswordExpireNotification {

   ##check that Metadot manages user info. If using LDAP then Metadot
   ##does not manage this.
   ##check that the site expires passwords
   if( MetadotConfig->get_param_value('user_management_is_external') ||
       $PARAMS->{password_expire} ne 'on' ){
      return 0;
   }

   my $u;
   my $days_left;
   my $last_notify;
   my $timestamp = time;
   my $lapse;
   my $day = 60 * 60 * 24;
   my $message;
   my $subject;
   my $result;
   my $send_message = undef;
   my $basename = $PARAMS->{basename} . '/index.pl';

   ### get list of users
   my ($userlist) = DBAccess->sqlSelectMany('userid','users','users.userid > 0');

   while( my ($userid) = $userlist->fetchrow() ){
      $send_message = undef;
      $u = Metadot::User::Default->new($userid);
      my $to = $u->{email};

      debug_msg(5,"IN doSendPasswordExpireNotification userid[$userid]");

      $days_left = $u->password_expires_in($userid);

      $last_notify = $u->{pw_expire_notification_time};

      ## check last notification
      $lapse = $timestamp - $last_notify;
      debug_msg(5,"doSendPasswordExpireNotification: user[$userid] password_expires[$days_left] last_notify[$last_notify] lapse[$lapse] day[$day]");

      $subject = "Your password will expire in $days_left days";

      if( $days_left == 10 ){
         if( $lapse > $day ){
            ## send notification
            $send_message = 1;
         }
      } elsif( $days_left == 5 ){
         if( $lapse > $day ){
            ## send notification
            $send_message = 1;
         }
      } elsif( $days_left == 2 ){
          if( $lapse > $day ){
            ## send notification
            $send_message = 1;
         }
      } elsif( $days_left == 1 ){
         if( $lapse > $day ){
            ## send notification
            $send_message = 1;
         }
      } elsif( $days_left == 0 ){
          if( $lapse > $day ){
            ## send notification
            $subject = 'Your password has expired.';
            $send_message = 1;
         }
      }

      if( $send_message ){
         if( $days_left > 0 ){

         $message =<<EMAIL_CONTENT;
Greetings,

This message is to notify that your user password for our site will expire
in $days_left days.

Please update your password information as soon as possible.

$basename

EMAIL_CONTENT
         } else{
            $message =<<EMAIL_CONTENT;
Greetings,

Your password has expired. In order to reset it, please go to the link below
and attempt to login. This will automatically prompt you to change your
password.

$basename

Thank you

EMAIL_CONTENT
         }

         if (Utility->is_valid_email($to)) {
             my $sm = Sendmail->new($to, '', '', $subject, $message);
             debug_msg(5,"Will send password notification to[$to] subject[$subject]");

             ($result) = $sm->sendMail();

             $u->{pw_expire_notification_time} = time;
             $u->save();
         } else {
             print STDERR "Found bad email address '$to'\n";
         }

      }
   } ## end while

}


## Gracefully fail if sendmail connection cannot be established
sub sendMailFailed {
    my $message = shift;

    DBAccess->close_db_object();
    &removeLockFile;

    my $abort_msg = "\n\nErrors occurred while attempting to send mail. Aborting $0\n$message\n\n";
    &logMessage($abort_msg);
    debug_msg(0, $abort_msg);
    exit(-1);
}




1;
