#######################################################################
# <LICENSEINFO>
# Copyright (C) Daniel Guermeur - 1999-2001
# Copyright (C) Metadot Corporation - 2001-2005
#
# This file is part of Metadot Portal Server software $Name: Release_6_4_5_4 $
#
# Metadot Portal Server software is free software; you can
# redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# Metadot Portal Server software is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Metadot Portal Server; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For more information, please contact info@metadot.com.
#</LICENSEINFO>
############################################################################
package Portal;
use strict;

use Metadot qw( $USER %FORM $SESSION $HTTP_HEADER_SENT $PARAMS $DISPLAY $CACHE $CGI $ACCESS_BROKER );

use Digest::MD5 qw(md5_base64);
use DBAccess;
use CGI qw( :standard :html3 :escape :escapeHTML :unescapeHTML -oldstyle_urls -no_debug );
use URI::Escape qw(uri_unescape uri_escape);
use Carp;
use Metadotclass;
use Gizmo;
use Gizmo::Discussion;
use Gizmo::FormMaker;
use Session;
use Storable;
use Subscription;
use ErrorHandling;
use Data::Dumper;
use HTML::TableFrame;
use HTML::Components;
use HTML::Parser;
use Gizmo::Application;
use Metadot::Utils::Display;
use Metadot::Utils::Theme;
use Metadot::AuthenticatorFactory;
use Metadot::AccessBrokerFactory;
use Metadot::UserFactory;
use Metadot::Cache;
use Metadot::TemplateMgr;
use Metadot::EmailToDiscussion;
use Metadot::MetadotController;
use Metadot::Subscribable;
use Metadot::SkinManager;
use Metadot::SystemApp::EventLog::ReportGeneratorFactory;
use Metadot::SystemApp::MyStuff;
use Metadot::Gizmo3000::ClassRegistry;
use Metadot::User::Default;
use Message;
use ParamsHash;
use Utility;
use Group;
use Sendmail;
use Number::Format qw(format_number);

use vars qw(@ISA);
@ISA=qw( Metadotclass Auditable Metadot::UploadsManager );


# start w/ a fresh new user
#$USER = Metadot::UserFactory->get_user();
#$SESSION = Session->new();


my $AC = {
    BUNDLES => {
        MOD =>    {
            DESC =>          'Edit',
            OPS =>            [qw(add create modify save up down paste modify_headfoot save_headfoot modify_ldap_params modify_params modify_message save_params save_message save_basic_styles confirm_upload_deletion delete_upload modify_accelerator manage_global_mypage_config edit_mypage_message_header save_mypage_message_header show_mypage_header_message show_mypage_default_content_message)],
            DEFAULT_LEVEL =>  8,
            MIN_LEVEL =>      8,
        },
        DEL =>    {
            DESC =>          'Delete and Cut',
            OPS =>            [qw(delete delete_ok cut)],
            DEFAULT_LEVEL =>  8,
            MIN_LEVEL =>      8,
            PARENT_DEP =>     {cut => 'MOD'},
        },
        DISP =>   {
            DESC =>          'View',
            OPS =>            [qw(myportal show_users show_pending_messages download_file modify_site_params get_traffic_lights_choices online)],
            DEFAULT_LEVEL =>  2,
            MIN_LEVEL =>      2,
            DEFAULTS_TO =>    'on',
        },
        SEARCH =>   {
            DESC =>          'Search',
            OPS =>            [qw(search)],
            DEFAULT_LEVEL =>  0,
            MIN_LEVEL =>      0,
            DEFAULTS_TO =>    'on',
        },
        ADMIN => {
            DESC =>                                        'View Admin',
            OPS =>           [qw(show_admin show_params show_message show_channels create_skin edit_skin_info edit_skin_files)],
            DEFAULT_LEVEL => 9,
            MIN_LEVEL =>                        9,
            DEFAULTS_TO =>   'on',
        },
        EDITP =>  {
            DESC =>          'Change<br />Permissions',
            OPS =>            [qw(edit_permissions set_permissions change_owner)],
            DEFAULT_LEVEL =>  8,
            MIN_LEVEL =>      8,
        },
    },
    DISPLAY_ORDER => [qw(DISP MOD DEL EDITP)],
};

sub get_AC {
    return $AC;
}

sub is_a {
    my $self = shift;
    my $class_ref = ref($self);

    # The following is somewhat of a hack to
    # address the fact that all App subclassess need to be
    # treated as a Category in terms of the 'isa' information that gets stored in the db.
    if(substr($class_ref,0,5) eq "App::") {
#   print STDERR "This is a $class_ref and has been set to a Category\n";
        return "Category" ;
    }
    else    {
#   print STDERR "This is not an App, it is a $class_ref\n";
        return $class_ref;
    }
}

sub save {
    my $self = shift;

    $self->save_permissions;
}

sub get_owner_uid {
    my $self = shift;

    MetadotConfig->get_reserved_uid('ADMIN');
}

#sub parent_id {
#        my $self = shift;
#
#        return 0;
#}

###################################################
sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = $class->SUPER::new;
    my $do_init = shift;

    bless ($self, $class);

    if($do_init) {
        # initialize metadot server

        $self->_initialize(@_);
    }


    return $self;
}

#####################################################
# WWW FUNCTIONS


####################################################
sub www_myportal {
    my $self = shift;
    $self->debug_msg(3, "Portal::www_myportal()");


    if( $PARAMS->{my_website_creation} ) {
    my $portal_id = $USER->has_portal();
    ### if the user doesn't have a one we create one
    $portal_id = $USER->create_default_portal() if (!$portal_id);

    ### now we trick the system here
    $self->md_redirect($self->make_url("index.pl?iid=$portal_id"));

    } else {

    # we simply redirect to main page
    $self->md_redirect($self->make_url("index.pl"));
    }
}

sub www_basic_styles {
    my $self = shift;
    $self->debug_msg(3, "Portal::www_basic_styles()");

    my $refererlink = $self->get_refererlink();

    #   my $output;
    my $theme_name = $PARAMS->{template_theme};

    # controller interface will borrow some of the themes interface for now
    my $use_controller = $PARAMS->{use_controller};
    unless ($use_controller || $DISPLAY->using_themes()) {
    # if themes not on, generate 'regular' page
    $self->www_modify_headfoot($refererlink);
    exit(0);
    }

    #else, generate themes page
    my $images_loc = $PARAMS->{imageslocation};
    my $header1_color = $PARAMS->{header1_color};
    my $leftnav1_color = $PARAMS->{leftnav1_color};
    my $title_text_color = $PARAMS->{title_text_color};
    my $nav_text_color = $PARAMS->{nav_text_color};

    my $header1_logo1 = $PARAMS->{header1_logo1};
    my $header1_logo2 = $PARAMS->{header1_logo2};
    my $header1_logo1_href = $PARAMS->{header1_logo1_href};
    my $header1_logo2_href = $PARAMS->{header1_logo2_href};
    #my $header1_logo1_alt = $PARAMS->{header1_logo1_alt};
    #my $header1_logo2_alt = $PARAMS->{header1_logo2_alt};

    my $header1_text1 = $PARAMS->{header1_text1};
    my $header1_text2 = $PARAMS->{header1_text2};

    my $skin_custom_header = escapeHTML($PARAMS->{skin_custom_header});
    my $skin_custom_footer = escapeHTML($PARAMS->{skin_custom_footer});
    my $use_skin_simple_header_footer_choice = !$PARAMS->{use_skin_custom_header_footer} ? 'checked' : '';
    my $use_skin_custom_header_footer_choice =  $PARAMS->{use_skin_custom_header_footer} ? 'checked' : '';

    # determine the available images
    my @images = $self->list_public_uploads({isa=>'Portal'});
    @images = sort {$a->{url} cmp $b->{url}} @images;

    # add a null choice
    push @images, {url=>"",file_name=>'No image'};

    # figure out which image is selected. First see if a valid one is actually selected, otherwise
    # take the first one.

     my $selected_image;
    if ($FORM{image_selector}) {
        for my $image (@images) {
            if ($FORM{image_selector} eq $image->{url}) {
                $selected_image = $image->{url};
            }
        }
    }
     $selected_image ||= $images[0]->{url};

    my $image_options = '';
    foreach my $image_ref (@images) {
        my $selected = ($image_ref->{url} eq $selected_image) ? 'selected' : '';
        $image_options .= "<option $selected value=\"$image_ref->{url}\">$image_ref->{file_name}</option>";
    }
    # for future reference, selected_image will show as spacer if blank
    $selected_image ||= "$images_loc/spacer.gif";

    my $header1_logo1_display = $header1_logo1;
    my $header1_logo2_display = $header1_logo2;
    # convert 'null' to friendlier string
    if ($header1_logo1_display eq 'null') {$header1_logo1_display = '-none-';}
    if ($header1_logo2_display eq 'null') {$header1_logo2_display = '-none-';}


    my $logo_attachment = "<INPUT TYPE=\"file\" NAME=\"banner_image_file\" SIZE=30>\n"
                         ."<INPUT TYPE=\"submit\" NAME=\"upload_image_button\" class=\"text_button\" value=\"Upload\">\n";

    my $escaped_referer = $CGI->escape( $refererlink );

    # generate buttons for choosing logo and center images from image selector
    my $logo_selector_button;
    my $center_selector_button;
    my $delete_selected_button;

    $logo_selector_button   = "<input type=\"button\" class=\"text_button\" value=\"Use as Logo\" onClick=\"document.styles_and_colors.image1.value=document.styles_and_colors.image_selector.options[document.styles_and_colors.image_selector.selectedIndex].value;document.getElementById('image1_display').innerHTML=document.styles_and_colors.image1.value;\">";
    $center_selector_button = "<input type=\"button\" class=\"text_button\" value=\"Use as Banner Image\" onClick=\"document.styles_and_colors.image2.value=document.styles_and_colors.image_selector.options[document.styles_and_colors.image_selector.selectedIndex].value;document.getElementById('image2_display').innerHTML=document.styles_and_colors.image2.value;\">";
    $delete_selected_button = "<input type=\"submit\" name=\"delete_image\" class=\"text_button\" value=\"Delete Selected\">";
    my $public_location = MetadotConfig->get_public_uploads_location();

    my $output = <<EOF;

<FORM name="styles_and_colors" action="index.pl" method="post" enctype="multipart/form-data">
<input type=hidden name=isa value=Portal>
<input type=hidden name=op value=save_basic_styles>
<input type=hidden name=iid value=0>
<input type=hidden name=refererlink value="$refererlink">

    <!--Vadim start-->
    <SCRIPT language="JavaScript" src="/js/ColorPicker.js"></SCRIPT>
    <SCRIPT language="Javascript" src="/js/AnchorPosition1.js"></SCRIPT>
    <SCRIPT language="Javascript" src="/js/PopupWindow1.js"></SCRIPT>
    <SCRIPT language="Javascript" src="/js/ColorScheme.js"></SCRIPT>
    <SCRIPT language="JavaScript"><!--
        // Create a new ColorPicker object using Window Popup
        var cp = new ColorPicker('window') ;
        var field ; // reference to current field processed

        // ColorPicker calls this function to return picked color value
        function pickColor(color) {
            // set the value of the form field
            field.value = color ;
            // update indicator
            if (document.getElementById) {// for NN6
                var indicator = window.document.getElementById(''+field.name+'_indicator') ;
                if (indicator != null) {
                    indicator.bgColor = color ;
                }
            } else if (document.layers) {// for NN4
                var indicator = window.document.layers[''+field.name+'_indicator'] ;
                if (indicator != null) {
                    indicator.bgColor = color ;
                }
            } else if (document.all) {// for IE
                var indicator = window.document.all[''+field.name+'_indicator'] ;
                if (indicator != null && indicator.style != null) {
                    indicator.style.background = color ;
                }
            }
        }

        // Wrap the ColorPicker call
        function pick1(fieldname, anchorname) {
            field = document.forms[0][fieldname] ;
            cp.show(anchorname) ;
        }

        function make_preview(value) {
            var result = "" ;
            if (document.all) {
                result = value ;
            } else {
                result = 'file:///'+value ;
            }
            return result ;
        }

        function color_fieldOnChange(color_field) {
            if (color_field != null) {
                field = color_field ;
                pickColor(color_field.value) ;
            }
        }

    // Updates the image viewer's image
    function updateImageBrowser(url_string) {
        if (url_string == '') {
            url_string='$images_loc/spacer.gif';
        }
        document.image_browser.src=url_string;
    }

    //-->
    </SCRIPT>
    <!--Vadim end-->

    <!--Vadim start-->
    <SCRIPT language="JavaScript"><!--
        // Create a new ColorSchemeList object and add Schemes to it
        var scheme_list = new ColorSchemeList('scheme_list', 'scheme_list') ;

        scheme_list.add_scheme(
            new ColorScheme(123, \"Current\", new Color('color1', '$header1_color'),
                new Color('color2', '$title_text_color'),
                new Color('color3', '$leftnav1_color'), new Color('color4', '$nav_text_color'))
        ) ;
        scheme_list.add_scheme(
            new ColorScheme(101, \"Autumn\", new Color('color1', '#996600'), new Color('color2', '#FFFFFF'), new Color('color3', '#CC9966'), new Color('color4', '#FFFFCC'))
        ) ;
        scheme_list.add_scheme(
            new ColorScheme(222, \"Winter\", new Color('color1', '#99CCCC'), new Color('color2', '#003333'), new Color('color3', '#C3E3E4'), new Color('color4', '#3399CC'))
        ) ;
        scheme_list.add_scheme(
            new ColorScheme(307, \"Spring\", new Color('color1', '#CCFF99'), new Color('color2', '#CC3300'), new Color('color3', '#99CC99'), new Color('color4', '#003399'))
        ) ;
        scheme_list.add_scheme(
            new ColorScheme(411, \"Summer\", new Color('color1', '#FDCD22'), new Color('color2', '#CB0A0A'), new Color('color3', '#AAF845'), new Color('color4', '#FC6001'))
        ) ;
    //-->
    </SCRIPT>
    <!--Vadim end-->

  <TABLE cellSpacing="10" cellPadding="0" width="600" border="0">
        <TR>
          <TD colspan="2">
            <p><font face="Arial, Helvetica, sans-serif" size="+1"><b>Layout Customization
              </b></font><font face="Verdana, Arial, Helvetica, sans-serif" size="-2"><br />
              <br />
              Select the skin of your choice, which defines the look and feel for the entire site.
              You can also specify a logo and optional secondary banner image of your choice. Your logo will
              be displayed in the leftmost area of the header, and the secondary banner image will be
              displayed to the right of the logo.
            </font></p>
          </TD>
        </TR>
EOF

    if ($use_controller) {
        # BEGIN TEMPLATE PICKER
        my $sm = Metadot::SkinManager->get_instance();
        my $available_skins = $sm->get_available_skins();
        my $skin_selector = "<select name=skin_selection>\n";
        my $selected_skin_id = $sm->get_current_skin_id();

        my ($skin_id, $skin_name, $skin_short_name, $skin_based_on, $skin_revision, $skin_author, $skin_website, $skin_description, $skin_license);
        my $base_skin_id = $sm->_BASE_SKIN_ID_();

        foreach my $skin (@$available_skins) {
            my $option_id   = $skin->get_id();

            # do not display the base skin as an option, because it requires
            # customization
            next if ($option_id eq $base_skin_id);

            my $option_name = $skin->get_name() || $skin->get_short_name();
            my $selected;

            $self->debug_msg(5, "Selected skin_id: '$selected_skin_id'; current option_id: '$option_id'");
            if ($selected_skin_id eq $option_id) {
                $selected = ' selected';
                $skin_name         = $skin->get_name();
                $skin_short_name   = $skin->get_short_name();
                $skin_based_on     = $skin->get_based_on() || "Base skin";
                $skin_revision     = $skin->get_revision();
                $skin_author       = $skin->get_author() || {};
                $skin_website      = $skin->get_website();
                $skin_description  = $skin->get_description();
                $skin_license      = $skin->get_license();
            } else {
                $selected = '';
            }
            $skin_selector .= "<option value=\"$option_id\"$selected>$option_name</option>\n";
        }
        $skin_selector .= "</select>\n";

        $output .= <<EOF;
        <TR>
          <TD valign='top' colspan="2">
            <table width="100%" border="1" cellpadding="0" cellspacing="0">
              <tr>
                <td>
                  <table width="100%" border="0" cellspacing="10" cellpadding="0">
                    <tr>
                      <td colspan="2" bgcolor="#666666"><font face="Arial, Helvetica, sans-serif" size=-1><img
    class='icon' height="25" src="$images_loc/1.gif" width="25"
    align=middle>&nbsp;&nbsp;<b><font color="#FFFFFF">Skin Details</font></b></font></td>
                    </tr>
                    <tr>
                      <td>
                        <b>Currently selected skin:</b>
                        <table border="0" cellspacing="0" cellpadding="2">
                          <tr valign="baseline"><td>Name: </td><td>$skin_name</td></tr>
                          <tr valign="baseline"><td>Based on: </td><td>$skin_based_on</td></tr>
                          <tr valign="baseline"><td>Revision: </td><td>$skin_revision</td></tr>
                          <tr valign="baseline"><td>Author: </td><td>$skin_author->{name}</td></tr>
                          <tr valign="baseline"><td>Email: </td><td>$skin_author->{email}</td></tr>
                          <tr valign="baseline"><td>Website: </td><td>$skin_website</td></tr>
                          <tr valign="baseline"><td>Description: </td><td>$skin_description</td></tr>
                          <tr valign="baseline"><td>License: </td><td>$skin_license</td></tr>
                        </table>
                       </td>
                     </tr>
                    <tr valign='top'>
                      <td nowrap="nowrap">
                        <b>Select a skin:</b>
                            $skin_selector
EOF
;
        if ($ACCESS_BROKER->is_allowed_to_do('admin_tasks')) {
            $output .= <<EOF;
                        <input type="submit" name="apply_skin" class='text_button' value="Use this skin">
                        &nbsp;
                        <input type="submit" name="edit_skin_info_button" class='text_button' value="Edit skin info">
                        &nbsp;
                        <input type="submit" name="edit_skin_file_button" class='text_button' value="Edit skin files">
                        &nbsp;
                        <input type="submit" name="delete_skin_button" class='text_button' value="Delete">
                        &nbsp;
                        <input type="submit" name="download_skin_button" class='text_button' value="Download">
                      </td>
                    </tr>
                    <tr>
                        <td nowrap="nowrap">
                            <b>Create a skin:</b>&nbsp;
                            <input type="submit" name="create_skin_button" class='text_button' value="Create a new skin">
                        </td>
                    </tr>
                    <tr>
                        <td nowrap="nowrap">
                            <b>Upload a skin:</b>
                            <INPUT TYPE="file" NAME="skin_file" SIZE=30>&nbsp;<input type="submit" name="upload_skin_button" class='text_button' value="Upload skin">
EOF
        }
        $output .= <<EOF;
                        </td>
                    </tr>
                  </table>
                </td>
              </tr>
            </table>
          </TD>
        </TR>
EOF
        # END TEMPLATE PICKER
    } else {
        # BEGIN THEME PICKER
        $output .= <<EOF
        <TR>
          <TD colspan="2">
            <p><font face="Arial, Helvetica, sans-serif" size="+1"><b>Layout Customization
              </b></font><font face="Verdana, Arial, Helvetica, sans-serif" size="-2"><br />
              <br />
              Select the color of your choice for the background and text of the page
              headers and for the background and text of the area displaying the navigation
              control (list of pages of your site). Note that you can select a scheme
              containing a pre-defined set of colors.<br />
              You can also specify a logo and optional secondary banner image of your choice. Your logo will
              be displayed in the leftmost area of the header, and the secondary banner image will be
              displayed to the right of the logo.
            </font></p>
          </TD>
        </TR>
          <TR>
            <TD valign='top' colspan="2">
              <table width="100%" border="1" cellpadding="0" cellspacing="0">
                <tr>
                  <td>
                    <table width="100%" border="0" cellspacing="10" cellpadding="0">
                      <tr>
                        <td colspan="2" bgcolor="#666666"><font face="Arial, Helvetica, sans-serif" size=-1><img
            class='icon' height="25" src="$images_loc/1.gif" width="25"
            align=middle>&nbsp;&nbsp;<b><font color="#FFFFFF">Select Colors</font></b></font></td>
                      </tr>
                      <tr>
                        <td colspan="2"><font face="Arial, Helvetica, sans-serif"
            size=-1><b>Scheme
                          <!--Vadim start-->
                          <script language="JavaScript"><!--
            // show dropdown
            scheme_list.show_dropdown() ;
            //-->
            </script>
                          <!--Vadim end-->
                          </b></font></td>
                      </tr>
                      <tr valign="top">
                        <td width="50%">
                          <table cellspacing="5" cellpadding="0" width="100%" border="0">
                            <tbody>
                            <tr>
                              <td colspan='3'><font face="Arial, Helvetica, sans-serif"
            size=-1><b>Frame Color</b> </font></td>
                            </tr>
                            <tr>
                              <td width="67">
                                <div align="right"><font face="Arial, Helvetica, sans-serif"
            size=-1>Background</font></div>
                              </td>
                              <!-- Vadim start -->
                              <td width="50">
                                <table border="1" cellspacing="0" cellpadding="0">
                                  <tr>
                                    <td id="color1_indicator" style="position: relative;" bgcolor="$header1_color"><img src="$images_loc/pixel.gif" border="0" width="21" height="21"></td>
                                  </tr>
                                </table>
                              </td>
                              <td> <font face="Arial, Helvetica, sans-serif" size="-1">
                                <img  name="color1_picker_button"  src="$images_loc/color_icon.gif"  alt="Pick Color"
                onClick="javascript: pick1('color1', 'color1_picker_button'); return false;" align="top">
                                <input name="color1" size="9" type="text" value="$header1_color" onChange="color_fieldOnChange(this)">
                                </font></td>
                              <!-- Vadim end -->
                            </tr>
                            <tr>
                              <td width="67">
                                <div align="right"><font face="Arial, Helvetica, sans-serif"
            size=-1>Text</font></div>
                              </td>
                              <!-- Vadim start -->
                              <td width="50">
                                <table border="1" cellspacing="0" cellpadding="0">
                                  <tr>
                                    <td id="color2_indicator" style="position: relative;" bgcolor="$title_text_color"><img src="$images_loc/pixel.gif" border="0" width="21" height="21"></td>
                                  </tr>
                                </table>
                              </td>
                              <td> <font face="Arial, Helvetica, sans-serif" size="-1">
                                <img  name="color2_picker_button" src="$images_loc/color_icon.gif"  alt="Pick Color"
                onClick="javascript: pick1('color2', 'color2_picker_button'); return false;" align="top">
                                <input name="color2" size="9" type="text" value="$title_text_color" onChange="color_fieldOnChange(this)">
                                </font></td>
                              <!-- Vadim end -->
                            </tr>
                            </tbody>
                          </table>
                        </td>
                        <td>
                          <table cellspacing="5" cellpadding="0" width="100%" border="0">
                            <tbody>
                            <tr>
                              <td colspan='3'><font face="Arial, Helvetica, sans-serif"
            size=-1><b>Navigation Color</b> </font></td>
                            </tr>
                            <tr>
                              <td width="67">
                                <div align="right"><font face="Arial, Helvetica, sans-serif"
            size=-1>Background</font></div>
                              </td>
                              <!-- Vadim start -->
                              <td width="50">
                                <table border="1" cellspacing="0" cellpadding="0">
                                  <tr>
                                    <td id="color3_indicator" style="position: relative;" bgcolor="$leftnav1_color"><img src="$images_loc/pixel.gif" border="0" width="21" height="21"></td>
                                  </tr>
                                </table>
                              </td>
                              <td>
                                <img  name="color3_picker_button" src="$images_loc/color_icon.gif"  alt="Pick Color"
                onClick="javascript: pick1('color3', 'color3_picker_button'); return false;" align="top">
                                <input name="color3" size="9" type="text" value="$leftnav1_color" onChange="color_fieldOnChange(this)">
                              </td>
                              <!-- Vadim end -->
                            </tr>
                            <tr>
                              <td width="67">
                                <div align="right"><font face="Arial, Helvetica, sans-serif"
            size=-1>Text</font></div>
                              </td>
                              <!-- Vadim start -->
                              <td width="50">
                                <table border="1" cellspacing="0" cellpadding="0">
                                  <tr>
                                    <td id="color4_indicator" style="position: relative;" bgcolor="$nav_text_color"><img src="$images_loc/pixel.gif" border="0" width="21" height="21"></td>
                                  </tr>
                                </table>
                              </td>
                              <td>
                                <img  name="color4_picker_button" src="$images_loc/color_icon.gif"  alt="Pick Color"
                onClick="javascript: pick1('color4', 'color4_picker_button'); return false;" align="top">
                                <font
            face="Arial, Helvetica, sans-serif" size=-1>
                                <input name="color4" size="9" type="text" value="$nav_text_color" onChange="color_fieldOnChange(this)">
                                </font></td>
                              <!-- Vadim end -->
                            </tr>
                            </tbody>
                          </table>
                        </td>
                      </tr>
                    </table>
                  </td>
                </tr>
              </table>
            </TD>
          </TR>
EOF
    }  ## END THEME PICKER
    $output .= <<EOF;
          <TR>
            <TD valign='top' colspan="2">
              <table width="100%" border="1" cellspacing="0" cellpadding="0">
                <tr>
                  <td align='center'>
                    <table width="100%" border="0" cellspacing="10" cellpadding="0">
                      <tr>
                        <td bgcolor="#666666"><font face="Arial, Helvetica, sans-serif" size=-1><img
      class='icon' height="25" src="$images_loc/2.gif" width="25"
      align=middle>&nbsp;&nbsp;<b><font color="#FFFFFF">Header/Footer Customizations</font>
                          </b></font></td>
                      </tr>
                      <tr>
                        <td>
                        <b>You have the option to use the simple header or a custom header and footer. Make sure to select the appropriate radio button with your choice. <br \>[<a href="#simple_header">Go to simple header</a>] [<a href="#custom_header">Go to custom header / footer</a>]</b>
                        </td>
                      </tr>
                      <tr>
                        <td>
                          <table width="100%" border="0" cellspacing="0" cellpadding="0" align="center" bgcolor="#EEEEEE"><tr><td>
                          <table width="100%" border="0" cellspacing="2" cellpadding="5" align="center">
                            <tr bgcolor="#CCCCCC">
                              <td><a name="simple_header" \><input type="radio" name="use_skin_custom_header_footer" value="0" $use_skin_simple_header_footer_choice\><b> Use Simple Header</td>
                            </tr>
                            <tr bgcolor="#FFFFFF">
                              <td colspan="2">
                                  <font face="Verdana, Arial, Helvetica, sans-serif" size="-2">The
                                  list on the left displays the currently installed header images.<br />
                                  - Select an image to view it <br />
                                  - Click on '<b>Browse...</b>' to choose your image and then click
                                    on '<b>Upload</b>' to store this image on the web server.<br />
                                  - Click on '<b>Delete Selected</b>' to delete
                                  the currently selected image.<br />
                                  - Click on '<b>Use as Logo</b>' to use the currently
                                  selected image as the header logo.<br />
                                  - Click on '<b>Use as Banner</b>' to
                                  use the currently selected image as the header banner
                                   image, to the right of the logo.<br />
                                  - Enter a '<b>Logo URL</b>' or '<b>Banner URL</b>' to make the image a clickable link. Use 'index.pl' to go to the home page.
                                  </font>
                              </td>
                            </tr>
                            <tr bgcolor="#FFFFFF">
                              <td colspan="2">
                                <table border="0" cellspacing="8" cellpadding="0">
                                  <tr>
                                    <td><font face="Arial, Helvetica, sans-serif" size="-1">Current Logo is using:</font></td><td><b><span id="image1_display">$header1_logo1_display</span></b></td>
                                    <td><font face="Arial, Helvetica, sans-serif" size="-1">Logo URL:</font></td><td><input name="header1_logo1_href" size="20" type="text" value="$header1_logo1_href"></font></td>
                                  </tr>
                                  <tr>
                                    <td><font face="Arial, Helvetica, sans-serif" size="-1">Current Banner Image is using:</font></td><td><b><span id="image2_display">$header1_logo2_display</span></b></td>
                                    <td><font face="Arial, Helvetica, sans-serif" size="-1">Banner URL:</font></td><td><input name="header1_logo2_href" size="20" type="text" value="$header1_logo2_href"></td>
                                  </tr>
                                  <tr>
                                    <td><font face="Arial, Helvetica, sans-serif" size="-1">Header text is:</font></td>
                                    <td colspan="3"><input name="header1_text1" size="80" type="text" value="$header1_text1"></td>
                                  </tr>
                                  <tr>
                                    <td><font face="Arial, Helvetica, sans-serif" size="-1">Header subtext is:</font></td>
                                    <td colspan="3"><input name="header1_text2" size="80" type="text" value="$header1_text2"></td>
                                  </tr>
                                </table>
                              </td>
                            </tr>
                            <tr bgcolor="#FFFFFF">
                              <td colspan="2" valign="baseline">Upload New Image <font face="Arial, Helvetica, sans-serif" size="-1">
                                $logo_attachment
                                </font></td>
                            </tr>
                            <tr valign="top" bgcolor="#FFFFFF">
                              <td colspan="2">
                                      <table width="100%" border="0" cellspacing="5" cellpadding="0">
                                        <tr valign="top">
                                          <td nowrap="nowrap">
                                            <div align="center"><font face="Arial, Helvetica, sans-serif" size="-1"><b>Current
                                              list of images</b><br />
                                              <br />
                                                <select name="image_selector" size="7" onChange="updateImageBrowser(this.options[this.selectedIndex].value)">
                                                $image_options
                                                </select>
                                              </font></div>
                                          </td>
                                            <td nowrap="nowrap">
                                                <div align="center"><font face="Arial, Helvetica, sans-serif" size="-1"><b>View of currently selected image</b><br />
                                              <br />
                                              <img src="$selected_image" name="image_browser"></font></div>
                                                <!-- spacer to enforce a certain width -->
                                                <img src="$images_loc/spacer.gif" width="500" height="1" alt="" />

                                            </td>
                                            <td>
                                                <!-- spacer to enforce a certain height -->
                                                <img src="$images_loc/spacer.gif" width="1" height="200" alt="" />
                                            </td>
                                        </tr>
                                        <tr>
                                          <td>
                                            <div align="center">$delete_selected_button</div>
                                          </td>
                                          <td>
                                            <div align="center"><font face="Arial, Helvetica, sans-serif" size="-1">$logo_selector_button
                                              &nbsp;&nbsp;$center_selector_button
                                              </font></div>
                                          </td>
                                        </tr>
                                      </table>
                              </td>
                            </tr>
                          </table>
                          </td></tr></table>
                        </td>
                      </tr>
                      <tr>
                        <td>
                          <table width="100%" border="0" cellspacing="0" cellpadding="0" align="center" bgcolor="#EEEEEE"><tr><td>
                          <table width="100%" border="0" cellspacing="2" cellpadding="5" align="center">
                            <tr bgcolor="#CCCCCC">
                              <td><a name="custom_header" \><input type="radio" name="use_skin_custom_header_footer" value="1" $use_skin_custom_header_footer_choice\><b> Use Custom Header / Footer</td>
                            </tr>
                            <tr bgcolor="#FFFFFF">
                              <td>
                                <b>Custom Header</b><br />
                                <textarea name="skin_custom_header" rows="8" cols="100" WRAP="VIRTUAL">$skin_custom_header</textarea>
                                <br /><br /><b>Custom Footer</b><br />
                                <textarea name="skin_custom_footer" rows="8" cols="100" WRAP="VIRTUAL">$skin_custom_footer</textarea>
                              </td>
                            </tr>
                          </table>
                          </td></tr></table>
                        </td>
                      </tr>
                    </table>
                  </td>
                </tr>
              </table>
            </TD>
          </TR>


    <TR>
      <TD valign='top' colspan="2">
        <div align="center">
          <input type="submit" class='text_button' value="Save" name="submit">
          <input type="submit" class='text_button' value="Cancel" name="submit">
        <INPUT size=30 maxlength="100" name=image1 type=hidden value="$header1_logo1">
        <INPUT size=30 maxlength="100" name=image2 type=hidden value="$header1_logo2">
          <br />
          <br />
          <! img src="$images_loc/snap2.gif" width="496" height="290"> </div>
      </TD>
    </TR>
  </TABLE></FORM>
EOF

  # remove some sections from view of unauthorized users
  if( $ACCESS_BROKER->is_allowed_to_do('advanced_styles') ) {

        $output .= <<EOF;
        <hr width="60%" align="center">
        <font face="Arial, Helvetica, sans-serif" size="-1"><br />
        If you wish to customize the style of your site even more, click on
        <form method=post action="index.pl">
        <input type=hidden name=isa value=Portal>
        <input type=hidden name=op value=modify_headfoot>
        <input type=hidden name=iid value=0>
        <input type=hidden name=refererlink value="$refererlink">
        <input type="submit" name="advanced" class='text_button' value="Advanced...">
        </font></FORM>
EOF
    }

    $self->print_form($output, "Basic Styles");
}

sub www_save_basic_styles {
    my $self=shift;
    $self->debug_msg(3, "Portal::save_basic_styles()");

    # added result processing that allows the save_basic_styles method to override the automatic redirect
    my $result = $self->save_basic_styles() if ($FORM{submit} ne "Cancel");
    if ($result) {
        return $result;
    } else {
        $self->md_redirect($FORM{refererlink});
    }
}


sub www_modify_headfoot {

    my ($self, $refererlink) = @_;

    unless( $ACCESS_BROKER->is_allowed_to_do('advanced_styles') ) {

    $DISPLAY->print_click_back('Operation not allowed');
    exit;
    }

    $self->debug_msg(3, "Portal::www_modify_headfoot()");
    if (!$refererlink) {
    $refererlink = $FORM{refererlink} || $SESSION->referer;
    }

    my $form = $self->show_edit_form_headfoot("Customizing Look and Feel");
     my $color_palette = Components->color_palette_code();
    my $content = "
 $color_palette
<form name=fm action=\"index.pl\" method=\"post\" enctype=\"multipart/form-data\">

<table align='right'><tr><td>
<input type='submit' name='adminpfmb' value='Save' class='button'>
<input type='submit' name='adminpfmb' value='Cancel' class='button'>
</td></tr></table>

<input type='hidden' name='isa' value='Portal'>
<input type='hidden' name='op' value='save_headfoot'>
<input type='hidden' name='refererlink' value='$refererlink'>
$form
<br /><table align='right'><tr><td>

<input type='submit' name='adminpfmb' value='Save' class='button'>
<input type='submit' name='adminpfmb' value='Cancel' class='button'>
</td></tr></table><br />&nbsp</form>
";
  $self->print_form($content,"Header and Footer");
}



sub www_save_headfoot {
    my $self = shift;
    $self->debug_msg(3, "Portal::www_save_headfoot()");

    unless( $ACCESS_BROKER->is_allowed_to_do('advanced_styles') ) {

    $DISPLAY->print_click_back('Operation not allowed');
    exit;
    }

    $self->save_headfoot() if($FORM{adminpfmb} ne "Cancel");
    $self->md_redirect($FORM{refererlink});
}

sub www_show_admin {
    my $self = shift;
    $self->debug_msg(3, "Portal::www_show_admin()");

    my $content = $self->show_form_admin();
    $self->print_form($content,"Admin Parameters");
}

######################################################

sub www_show_params {
    my $self = shift;
    $self->debug_msg(3, "Portal::www_show_params()");

    my $form = $self->show_form_params();
    $self->print_form($form,"System Parameters");
}

sub www_show_message {
    my $self = shift;
    $self->debug_msg(3, "Portal::www_show_message()");

    my $form = $self->show_form_message();
    $self->print_form($form,"Email Messages Sent by System");
}

sub www_modify_site_params {
    my $self = shift;

    # these two are accessible to all users
    if($FORM{manage} eq "My Groups") {

    $self->md_redirect( $self->make_url("index.pl?isa=Group&op=show_groups") );
    return;

    } elsif($FORM{manage} eq Metadot::SystemApp::MyStuff->get_app_name() ) {

    $self->md_redirect( $self->make_url("index.pl?isa=Metadot::SystemApp::MyStuff&op=show") );
    return;

    }elsif($FORM{manage} eq "My Profile") {

        $self->md_redirect( $self->make_url("index.pl?isa=User&op=modify_self") );
    return;
    }

    # from this point operations are restricted to admins only
    unless( $ACCESS_BROKER->is_allowed_to_do('admin_tasks') ) {

    $DISPLAY->print_click_back('Operation not allowed');
    exit;
    }

    if($FORM{manage} eq "Users"){
        if ($USER->can('www_show_users_groups')) {
            $USER->www_show_users_groups;
        } else {
            $USER->www_show_users;
        }
    }elsif($FORM{manage} eq "User_groups"){

        $USER->www_show_users_groups;

    }elsif($FORM{manage} eq "configure_profile"){

        $DISPLAY->md_redirect("index.pl?isa=User&op=configure_profile");

    }elsif($FORM{manage} eq "Channel"){

        $self->www_show_channels;

    }elsif($FORM{manage} eq "Colors"){

        $self->www_basic_styles;

    }elsif($FORM{manage} eq "Config"){

        $self->www_show_admin;

    }elsif($FORM{manage} eq "MyPageConfig"){

        $self->www_manage_global_mypage_config;

    }elsif($FORM{manage} eq "GB_on"){

        if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ) {
            $SESSION->set_value('GB_status','1');
        }

        $self->md_redirect($self->make_url("index.pl?id=" . $FORM{iid}));

    }elsif($FORM{manage} eq "GB_off"){

        if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ) {
            $SESSION->set_value('GB_status','0');
        }

    $self->md_redirect($self->make_url("index.pl?id=" . $FORM{iid}));

    }elsif($FORM{manage} eq "Hidden"){

        if( $ACCESS_BROKER->is_allowed_to_do('admin_tasks') ) {

            $self->md_redirect($self->make_url("index.pl?id=-1"));

        } else {

            $DISPLAY->print_click_back('Operation not allowed');
            exit;
        }
    }elsif($FORM{manage} eq "SiteLibrary"){

        if( $ACCESS_BROKER->is_allowed_to_do('admin_tasks') ) {

            $self->md_redirect($self->make_url("index.pl?op=manage&id=".SiteLibrary->get_instance()->get_id()));

        } else {

            $DISPLAY->print_click_back('Operation not allowed');
            exit;
        }
    }elsif($FORM{manage} eq "SuperGizmos"){

        if( $ACCESS_BROKER->is_allowed_to_do('admin_tasks') ) {

           $self->md_redirect($self->make_url("index.pl?isa=SuperGizmo&op=manage_all_supergizmos"));

        } else {

            $DISPLAY->print_click_back('Operation not allowed');
            exit;
        }
    }


}

sub www_modify_params {
    my $self = shift;
    $self->debug_msg(3, "Portal::www_modify_params()");

    my $form = $self->show_edit_form_params();
    my $content = "
<form action='index.pl' method='post'>
<input type='hidden' name='isa' value='Portal'>
<input type='hidden' name='op' value='save_params'>
$form
<table align='right'><tr><td>
<input type='submit' value='Save' class='button'>
</td></tr></table><br /><br /><br /></form>
";

    $self->print_form($content,"Modify System Parameters");
}

sub www_modify_message {
    my $self = shift;
    $self->debug_msg(3, "Portal::www_modify_params()");

    my $form = $self->show_edit_form_message();
    my $content = "
<form action='index.pl' method='post'>
<input type='hidden' name='isa' value='Portal'>
<input type='hidden' name='op' value='save_message'>
$form
<table align='right'><tr><td>
<input type='submit' value='Save' class='button'>
</td></tr></table><br /><br /><br /></form>
";

    $self->print_form($content,"Modify Messages Sent by the System");
}

sub www_save_params {
    my $self = shift;
    $self->debug_msg(3, "Portal::www_save_params()");

    $self->save_params;
    #redirect to show the results
    $self->md_redirect("index.pl?isa=Portal&op=show_params");
}

sub www_save_message {
    my $self = shift;
    $self->debug_msg(3, "Portal::www_save_params()");

    $self->save_message;
    #redirect to show the results
    $self->www_show_message;
}


sub www_show_channels {
    my $self = shift;
    $self->debug_msg(3, "Portal::www_show_channels()");

    $self->print_http_header;
    my $scripturl = "index.pl?iid=0&isa=Portal&op=show_channels";

    #if a channel type is specified go to that channel, else show toc
    my $s = $FORM{channel_type} ?
        $self->show_channel_list($FORM{iid}, $FORM{channel_type}, $scripturl) :
            $self->show_channel_toc($scripturl);

    my $look = $PARAMS->{globallook};
    if ($look eq "usetemplate") {
        print  $DISPLAY->simple_expand(0, $self, $self->show_template(-1), $s);
    } else {
        # this is compatible with the new template mode
        $DISPLAY->print_simple_layout({content=>$s,title=>"Channels"});
    }

}

sub www_manage_global_mypage_config {
    my $self = shift;
    $self->debug_msg(3, "Portal::www_manage_global_mypage_config()");

    $DISPLAY->print_simple_layout({
        title   => "My Page Global Config",
        content =>
            "<p /><ul>".
            "<li><a href=\"index.pl?isa=Portal&amp;iid=0&amp;op=edit_mypage_message_header\">Edit My Page Message</a>".
            "<li><a href=\"index.pl?isa=Portal&amp;iid=0&amp;op=show_mypage_header_message\">Edit My Page Header</a>".
            "<li><a href=\"index.pl?isa=Portal&amp;iid=0&amp;op=show_mypage_default_content_message\">Edit My Page Default Content</a>".
            "<li><a href=\"index.pl?isa=Portal&amp;iid=0&amp;op=modify_site_params&amp;app=&amp;manage=Channel\">Manage Channels</a>".
            "</ul><p /><p />".
            "\n\n<!-- End of Manage My Page Global Config -->\n\n",
    });

#     $self->print_http_header;
#
#     print $self->show_top($FORM{iid}),
#     $self->show_title("My Page Global Config"),
#     "<p /><ul>",
#     "<li><a href=\"index.pl?isa=Portal&amp;iid=0&amp;op=edit_mypage_message_header\">Edit My Page Message</a>",
#     "<li><a href=\"index.pl?isa=Portal&amp;iid=0&amp;op=show_mypage_header_message\">Edit My Page Header</a>",
#     "<li><a href=\"index.pl?isa=Portal&amp;iid=0&amp;op=show_mypage_default_content_message\">Edit My Page Default Content</a>",
#     "<li><a href=\"index.pl?isa=Portal&amp;iid=0&amp;op=modify_site_params&amp;app=&amp;manage=Channel\">Manage Channels</a>",
#     "</ul><p /><p />",
#     "\n\n<!-- End of Manage My Page Global Config -->\n\n",
#     $self->show_bottom;
}

sub www_show_mypage_header_message {
    my $self = shift;
    $self->debug_msg(3, "Portal::www_show_mypage_header_message()");

    my $globalurl = "index.pl?iid=0&amp;isa=Portal&amp;op=manage_global_mypage_config";

    my $form = <<EOM
<p /><p />
To edit the My Page header, select the "Styles & Colors" option from the Manage dropdown at the top of the page. If the site has Themes enabled, you will have to select the "Advanced" button at the bottom of the page. Look for the text box labeled "My Page Header".
<p />
<div align="center"><a href="$globalurl">Return to Manage My Page Global Config</a></div><p /><p />
EOM
;
    $self->print_http_header;

    print $self->show_top($FORM{iid}),
    $self->show_title("Edit My Page Message"),
    $form,
    "\n\n<!-- End of Manage My Page Message -->\n\n",
    $self->show_bottom;
}

sub www_show_mypage_default_content_message {
    my $self = shift;
    $self->debug_msg(3, "Portal::www_show_mypage_default_content_message()");

    my $globalurl = "index.pl?iid=0&amp;isa=Portal&amp;op=manage_global_mypage_config";

    my $name_default_look = $USER->NAME_DEFAULT_LOOK();
    my $form = <<EOM
<p /><p /><div align="center"><b>Edit My Page Default Content</b></div>
<p />
In order to create the default My Page content for newly registered users, you must be logged in as the "$name_default_look" user who's username is also "$name_default_look". If you do not know the "$name_default_look" user password, change the password by editing the user record in the User Directory. You can access the User Directory by selecting "Users" from the Manage dropdown at the top of the page. After logging in as "$name_default_look" user, go to "My Page". Click the "Configure" button and add the default content you wish for newly registered users to inherit.
<p />
<div align="center"><a href="$globalurl">Return to Manage My Page Global Config</a></div><p /><p />
EOM
;
    $self->print_http_header;

    print $self->show_top($FORM{iid}),
    $self->show_title("Edit My Page Default Content"),
    $form,
    "\n\n<!-- End of Manage My Page Default Content-->\n\n",
    $self->show_bottom;
}

sub www_edit_mypage_message_header {
    my $self = shift;
    $self->debug_msg(3, "Portal::www_edit_mypage_message_header()");

    my $target        = $CGI->escape($SESSION->referer());

    my $param_value = $PARAMS->{mypagemessageheader};

    my $out = <<EOM
<p /><p /><div align="center"><b>Edit My Page Message</b></div><br /><br />
<form action="index.pl"  method="post" ENCTYPE="multipart/form-data">
<input type="hidden" name="isa" value="Portal">
<input type="hidden" name="op" value="save_mypage_message_header">
<input type="hidden" name="target" value="$target">
<b>Please enter your My Page Message</b> <i>(This message will appear below the links bar in My Page.)</i>
EOM
;
    $out .= $DISPLAY->generate_wysiwyg_field({field_name=>'mypagemessageheader', field_value=>$param_value, rows=>22}); 
    #$out .= $DISPLAY->convert_field_to_htmlarea('mypagemessageheader');

    $out .= "<div><input type='submit' value='Submit'></div>";
    $self->print_form($out,"");
}

sub www_save_mypage_message_header {
    my $self = shift;
    my %args = @_;
    $self->debug_msg(3, "Portal::www_save_mypage_message_header()");

    $PARAMS->set_property('mypagemessageheader', $FORM{mypagemessageheader});

    my $target = $CGI->unescape($FORM{target}) || 'index.pl';
    $DISPLAY->print_choices ("Success! My Page Message updated.",
                     "confirm",
                     "Continue",
                     $target,
                    )
}

sub _sort_by_hash_element {
    my $self = shift;
    my $key = shift;
    my @array = @_;

    my @return_array = sort { $a->{$key} cmp $b->{$key} } @array;

    return @return_array;
}

sub _sort_by_numeric_hash_element {
    my $self = shift;
    my $key = shift;
    my @array = @_;

    my @return_array = sort { $a->{$key} <=> $b->{$key} } @array;

    return @return_array;
}

sub www_search {
    my $self = shift;
    my $iid = $self->id();
    $self->debug_msg(3, "Portal::www_search()");

    # there are two ways of getting $from. "start_row" is using the new skin mode paginator.
    my $from = $FORM{from} || $FORM{start_row} || 0;

    # perform the search
    if ($FORM{search_choice} eq 'user_directory' && $USER->can_show_user_directory()) {
        $USER->www_show_user_directory(search_words => $FORM{filter}, from => $from, nb => ($FORM{nb} || $FORM{max_rows}), iid => $iid, object => $self);
        exit;
    }

    my $results = $self->dispatch_search($FORM{filter}, $from, $FORM{nb} || $FORM{max_rows});

    $self->debug_msg(4, "Portal::www_search() got results $results");

    # if we are in template view mode we call the template
    # get the template that is not the main template
    my $look = $PARAMS->{globallook};
    if ($look eq "usetemplate") {
        $self->print_http_header();    #CKIM: ensure http header is sent
        print  $DISPLAY->simple_expand($iid, $self, $self->show_template(-1), $results);
    } else {
        # normal view mode
        #print "normal view mode";
        $self->print_portal($results,"Search Result: $FORM{filter}",$iid);
    }
}

#####################################################
# BACKEND FUNCTIONS

sub show_form_params {
    my $self = shift;

    # recovering and parsing timecounters string
    my %counters;
    my ($countersstring) = $PARAMS->{'timecounters'};
    if ($countersstring ne "") {
        my @info = split("-", $countersstring) ;
        foreach my $pair (@info) {
            my ($key, $field) = split "=", $pair;
            $counters{h} = localtime($field) if ($key eq "h");
            $counters{d} = localtime($field) if ($key eq "d");
            $counters{w} = localtime($field) if ($key eq "w");
        }
    }

    my $featureset_tag = MetadotConfig->get_featureset_tag() ;

    my ($admin_marker, $msg_for_super_admins) = ('', '');

    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ) {

    $admin_marker =  '<font color=red size=-1>*</font>';
    $msg_for_super_admins = "$admin_marker <i><font size=-1>indicates fields available to site administrators.</font></i>";
    }
    my $login_mode = '';
    if ($PARAMS->{login_mode} eq 'https_standard') {
        $login_mode = 'Allow HTTPS and Standard login';
    } elsif ($PARAMS->{login_mode} eq 'https') {
        $login_mode = 'HTTPS only';
    } else {
        $login_mode = 'Default';
    }

    my $s = "
<br /><br /><center>
<TABLE BGCOLOR='#FFFFFF' WIDTH=\"450\" CELLSPACING=\"0\" CELLPADDING=\"5\" BORDER=\"0\" valign=\"baseline\">
        <tr bgcolor = #bbbbbb>
                <td colspan = 2><b>System Parameter Administration</td>";

#    if( $msg_for_super_admins ) {
#        $s .= "<TR bgcolor = ".$self->list_color().">
#            <TD colspan='2' align='center'>$msg_for_super_admins</TD>
#        </TR>";
#    }

    $s .=
#        "
#          <TR bgcolor = ".$self->list_color().">
#              <TD>Sessions are deleted on Logout:</TD>
#              <TD>".($defaults{sessions_are_deleted_on_logout}? "Yes":"No")."</TD>
#          </TR> "

        "<TR bgcolor = ".$self->change_list_color().">
            <TD>$admin_marker Session duration (seconds):</TD>
            <TD>$PARAMS->{sessionduration}</TD>
        </TR>";

    # remove some sections from view of unauthorized users
    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){

    $s .= "
        <TR bgcolor = ".$self->change_list_color().">
            <TD>Mail server address:</TD>
            <TD>$PARAMS->{mailserveraddress}</TD>
        </TR>
        <TR bgcolor = ".$self->list_color().">
            <TD>Mail sender address:</TD>
            <TD>$PARAMS->{mailsenderaddress}</TD>
        </TR>
        <TR bgcolor = ".$self->list_color().">
            <TD>Mail sender address:</TD>
            <TD>$PARAMS->{mailsenderaddress}</TD>
        </TR>

        <TR bgcolor = ".$self->change_list_color().">
            <TD>Discussion/subscription email account name:</TD>
            <TD>$PARAMS->{subscriptionsender}</TD>
        </TR>
        <TR bgcolor = ".$self->list_color().">
            <TD>Discussion/subscription email host name:</TD>
            <TD>$PARAMS->{subscriptionhost}</TD>
        </TR>
        <TR bgcolor = ".$self->list_color().">
            <TD>Discussion/subscription email password:</TD>
            <TD>$PARAMS->{subscriptionpwd}</TD>
        </TR>
        <TR bgcolor = ".$self->list_color().">
            <TD>Show discussion channel name in message subject:</TD>
            <TD>" . ($PARAMS->{show_discussion_channel_in_subject} ? 'Yes' : 'No') . "</TD>
        </TR>
        <TR bgcolor = ".$self->list_color().">
            <TD>Next daily subscriptions processing:</TD>
            <TD><br />$counters{d}</TD>
        </TR>
        <TR bgcolor = ".$self->list_color().">
          <TD>Next weekly subscriptions processing:</TD>
          <TD><br />$counters{w}</TD>
        </TR>

        <TR bgcolor = ".$self->list_color().">
            <TD>URI scheme + basename:</TD>
            <TD>$PARAMS->{basename}</TD>
        </TR>\n";

        unless ($PARAMS->{if_ldap}) {
        $s .= "
        <TR bgcolor = ".$self->list_color().">
            <TD>Login Mode:</TD>
            <TD>$login_mode</TD>
        </TR>\n";
        }

        $s .= "<TR bgcolor = ".$self->change_list_color().">
            <TD>Home page:</TD>
            <TD>$PARAMS->{home_page}&nbsp;</TD>
        </TR>" ; #if (defined($featureset_tag) && $featureset_tag eq 'closed') ;

        $s .= "<TR bgcolor = ".$self->list_color().">
            <TD>Page after login:</TD>
            <TD>$PARAMS->{post_login}&nbsp;</TD>
        </TR>" ;#if (defined($featureset_tag) && $featureset_tag eq 'closed') ;

    }

    if ($USER->can('www_show_user_directory')) {
    $s .= "<TR bgcolor = ".$self->change_list_color().">
            <TD>$admin_marker Allow viewing of User Directory?:</TD>
            <TD>".($PARAMS->{show_user_directory} ? "Yes":"No")."</TD>
        </TR> " ;
    }

    $s .= "<TR bgcolor = ".$self->change_list_color().">
            <TD>$admin_marker Suppress the MyPage link?:</TD>
            <TD>".($PARAMS->{hide_my_page}? "Yes":"No")."</TD>
        </TR> " ;

    $s .= "<TR bgcolor = ".$self->change_list_color().">
            <TD>$admin_marker Require login:</TD>
            <TD>".($PARAMS->{require_login}? "Yes":"No")."</TD>
        </TR> " ;

    $s .= "<TR bgcolor = ".$self->change_list_color().">
            <TD>$admin_marker Automatic password expiration:</TD>
            <TD>".($PARAMS->{password_expire}? "Yes":"No")."</TD>
           </TR> ";

    $s .= "<TR bgcolor = ".$self->change_list_color().">
            <TD>$admin_marker Automatic password expiration duration (days):</TD>
            <TD>".$PARAMS->{password_expire_duration}."</TD>
           </TR> ";

    $s .= "<TR bgcolor = ".$self->change_list_color().">
           <TD>$admin_marker Automatic password expiration message</TD>
           <TD>".($PARAMS->{password_expiration_message}? $PARAMS->{password_expiration_message}:
           Metadot::User::Default->password_expiration_message() )."</TD>
           </TR> ";

    $s .= "<TR bgcolor = ".$self->change_list_color().">
            <TD>$admin_marker Allow discussion owners to add subscribers to a discussion:</TD>
            <TD>".($PARAMS->{allow_disc_owner}? "Yes":"No")."</TD>
        </TR> " ;

    $s .= "<TR bgcolor = ".$self->change_list_color().">
            <TD>$admin_marker Delete previous sessions on login:</TD>
            <TD>".($PARAMS->{delete_previous_sessions_on_login}? "Yes":"No")."</TD>
        </TR> " ;

    $s .= "<TR bgcolor = ".$self->change_list_color().">
             <td>$admin_marker Allow FormMaker email-to-submitter capability:</td>
             <td>".($PARAMS->{email_to_submitter}? "Yes":"No")."</td>
           </TR>";

    # remove some sections from view of unauthorized users
    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){

    $s .= "<TR bgcolor = ".$self->change_list_color().">
            <TD>Non-Trusted site:</TD>
            <TD>";

    if ( $PARAMS->{trusted_site}) {
        $s .= "No";
    }
    else {
        $s .= "Yes";
    }
    $s .= "</TD>         </TR>" ;

    if ( MetadotConfig->get_use_template_mgmt() eq 'true') {
#   if ( (defined $featureset_tag) && ($featureset_tag eq 'closed')
#       && (MetadotConfig->get_use_template_mgmt() eq 'true')) {
        $s .= "<TR bgcolor = ".$self->change_list_color().">
            <TD>Enable Template Management:</TD>
            <TD>".($PARAMS->{template_mgmt}? "Yes":"No")."</TD> </TR>";
    }

        $s .= "
        <TR bgcolor = ".$self->change_list_color().">
            <TD>Enable Event Log:</TD>
            <TD>".($PARAMS->{is_event_log_enabled}? "Yes":"No")."</TD>
        </TR>";

        $s .= "
        <TR bgcolor = ".$self->change_list_color() .
      "> <TD>Event Log Cleaning Interval (Days):</TD>\n<TD>".
      $PARAMS->{event_log_clean_after} ."</TD> </TR>";


        $s .= "
        <TR bgcolor = ".$self->change_list_color().">
            <TD>HTTP proxy address : port:</TD>
            <TD>$PARAMS->{proxyhost}:$PARAMS->{proxyport}</TD>
        </TR>


        <TR bgcolor = ".$self->change_list_color().">
            <TD>Images directory location:</TD>
            <TD>$PARAMS->{imageslocation}</TD>
        </tr>
        <TR bgcolor = ".$self->list_color().">
            <TD>Absolute path for private file upload directory:</TD>
            <TD>$PARAMS->{absolutepathfileuploaddir}</TD>
        </TR>";

    $s .= "
        <TR bgcolor = ".$self->change_list_color().">
            <TD>Maximum site-wide total uploads size (MB):</TD>
            <TD>$PARAMS->{'global_uploads_quota'}</TD>
        </TR>";
    }

    $s .= "
        <TR bgcolor = ".$self->change_list_color().">
            <TD>$admin_marker Default maximum total upload size per user (MB):</TD>
            <TD>$PARAMS->{'max_total_upload_size'}</TD>
        </TR>";


    $s .= "
        <TR bgcolor = ".$self->change_list_color().">
            <TD>$admin_marker Maximum number of News Items in Category News List:</TD>
            <TD>$PARAMS->{'maxnewsitems'}</TD>
        </TR>";


    $s .= "
        <TR bgcolor = ".$self->change_list_color().">
            <TD>$admin_marker Enable group welcome letters (also requires BROADCAST EMAIL permissions):</TD>
            <TD>".($PARAMS->{allow_group_welcome_letters} ? 'Yes' : 'No')."</TD>
        </TR>";


    # remove some sections from view of unauthorized users
    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){

    $s .= "
        <TR bgcolor = ".$self->list_color().">
            <TD>Default total number of uploads per user:</TD>
            <TD>$PARAMS->{'max_total_number_upload'}</TD>
        </TR>

        <TR bgcolor = ".$self->change_list_color().">
            <TD>Show who is online:</TD>
            <TD>".($PARAMS->{show_who_online}? "Yes":"No")."</TD>
        </TR>

        <TR bgcolor = ".$self->list_color().">
            <TD>Show who is online in a list:</TD>
            <TD>".($PARAMS->{show_who_online_list}? "Yes":"No")."</TD>
        </TR>

        <TR bgcolor = ".$self->list_color().">
            <TD>Limit of online users in list:</TD>
            <TD>".($PARAMS->{show_online_limit}? $PARAMS->{show_online_limit}:"0")."</TD>
        </TR>";

        my $form_maker_message = $PARAMS->{custom_form_maker};
        if( $form_maker_message ne '' ){
            $form_maker_message = escapeHTML($form_maker_message);
            $form_maker_message = GizmoBuilder->translate_white_space($form_maker_message,1);
        } else {
            $form_maker_message = 'Default';
        }

        $s .= "
        <TR bgcolor = ".$self->change_list_color().">
            <TD>Custom thank you message for FormMaker gizmo submissions:</TD>
            <TD>$form_maker_message</TD>
        </TR>";

    $s .= "<TR bgcolor = ".$self->change_list_color().">
                 <TD>Send email notification to users that receive new "
                 .(MetadotConfig->get_param_value('metagram_label')?
                  MetadotConfig->get_param_value('metagram_label'):"metagrams")."
                 </TD>
                 <TD>".($PARAMS->{metagram_email_copy}? "Yes":"No")."</TD>
               </TR>";

        $s .= "<TR bgcolor = ".$self->change_list_color().">
                 <TD>".(MetadotConfig->get_param_value('metagram_center_name')?
                  MetadotConfig->get_param_value('metagram_center_name'):"Message Center")."
                  default view by sender:</TD>
                 <TD>".($PARAMS->{metagram_thread_view}? "Yes":"No")."</TD>
               </TR>";


        my $max_users = $PARAMS->{max_number_of_users} || 'unlimited';

    unless( MetadotConfig->get_param_value('user_management_is_external') ) {
        $s .= "
        <TR bgcolor = ".$self->change_list_color().">
            <TD>Allowed Users Quota (number of users):</TD>
            <TD>$max_users</TD>
        </TR>";
    }

    $s .= "
        <TR bgcolor = ".$self->change_list_color().">
            <TD>Documentation URL</TD>
            <TD>$PARAMS->{docs_url}</TD>
        </TR>";
    }

    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){
        $s .= "<TR bgcolor = ".$self->change_list_color().">
                <TD>$admin_marker Hide Metadot signature in HTML source code:</TD>
                <TD>".($PARAMS->{hide_metadot_signature_in_html_source}? "Yes":"No")."</TD>
            </TR> ";

        $s .= "<TR bgcolor = ".$self->change_list_color().">
                <TD>$admin_marker Hide Metadot signature in printer friendly label:</TD>
                <TD>".($PARAMS->{hide_metadot_signature_in_printer_friendly}? "Yes":"No")."</TD>
            </TR> ";
    }

    $s .= "<TR bgcolor = ".$self->change_list_color().">
            <TD>$admin_marker Allow files to be attached to discussion messages:</TD>
            <TD>".($PARAMS->{allow_discussion_attachments}? "Yes":"No")."</TD>
        </TR> ";

    $s .= "<TR bgcolor = ".$self->change_list_color().">
            <TD>$admin_marker Send moderated discussion notifications:</TD>
            <TD>".($PARAMS->{send_moderated_posting_notifications}? "Yes":"No")."</TD>
        </TR> ";

    $s .= "<TR bgcolor = ".$self->change_list_color().">
            <TD>$admin_marker Allow users to edit their discussion messages:</TD>
            <TD>".($PARAMS->{allow_discussion_message_editing}? "Yes":"No")."</TD>
        </TR> ";

    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){
        $s .= "<TR bgcolor = ".$self->change_list_color().">
                <TD>$admin_marker Allow users to assign email addresses to Discussions:</TD>
                <TD>".($PARAMS->{allow_discussion_email_alias}? "Yes":"No")."</TD>
            </TR> ";

        $s .= "<TR bgcolor = ".$self->change_list_color().">
                <TD>Base domain for discussion email aliases:</TD>
                <TD>".($PARAMS->{email_alias_base_domain} || Metadot::EmailToDiscussion->get_default_alias_domain())."</TD>
            </TR> ";

        $s .= "<TR bgcolor = ".$self->change_list_color().">
                <TD>Character set:</TD>
                <TD>".($PARAMS->{http_charset} || $DISPLAY->DEFAULT_CHARSET())."</TD>
            </TR> ";

        $s .= "<TR bgcolor = ".$self->change_list_color().">
                <TD>Default email content-type:</TD>
                <TD>".($PARAMS->{default_mail_content_type} || Sendmail->DEFAULT_MAIL_CONTENT_TYPE())."</TD>
            </TR> ";

        $s .= "<TR bgcolor = ".$self->change_list_color().">
                <TD>Discussion content-type:</TD>
                <TD>".($PARAMS->{discussion_message_content_type} || Discussion->DEFAULT_MESSAGE_CONTENT_TYPE())."</TD>
            </TR> ";


        $s .= "<TR bgcolor = ".$self->change_list_color().">
                <TD>$admin_marker Create a channel for each new page:</TD>
                <TD>".($PARAMS->{default_page_channel_creation}? "Yes":"No")."</TD>
            </TR> ";


        $s .= "<TR bgcolor = ".$self->change_list_color().">
                <TD>$admin_marker Enable content update notifications for each new page:</TD>
                <TD>".($PARAMS->{default_update_notification_enablement}? "Yes":"No")."</TD>
            </TR> ";

    }

    $s .= "
      </TABLE><br />$msg_for_super_admins<br /><br />
<a href = ".$self->make_url("index.pl?isa=Portal&amp;op=modify_params").">Modify System Params</a><br /><a href=".$self->make_url("index.pl?isa=Portal&amp;op=show_admin").">Show Administration Page</a></center><br /><br />
";
    return $s;
}

my $color = "#ffffff" ;
my $color1 = "#efefef" ;
sub change_list_color {
        my $self = shift ;
        ($color, $color1) = ($color1, $color) ;
        return $color ;

}
sub list_color {
        my $self = shift ;
        return $color ;
}

############################################################

sub show_edit_form_params {
    my $self = shift;

    # recovering and parsing timecounters string
    my ($nexthour, $nextday, $nextweek);
    my ($countersstring) = $PARAMS->{timecounters};
    if ($countersstring ne "") {
        my @info = split("-", $countersstring) ;
        foreach my $pair (@info) {
            my ($key, $field) = split "=", $pair;
            $nexthour = $field if ($key eq "h");
            $nextday = $field if ($key eq "d");
            $nextweek = $field if ($key eq "w");
        }
    }

    my $dispnextday   = localtime($nextday);
    my $dispnextweek  = localtime($nextweek);

    # parse nextday to know when we send daily subscriptions (hour=$4)
    $dispnextday =~ m/(...)\s(...)\s(.\d|\d\d)\s(\d\d):(\d\d):(\d\d)\s(\d\d\d\d)/ ;
    my $popupdayhour  = $CGI->popup_menu(-name=>'dayhour', -values=>['00', '01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', '23'], -default=>$4);
    # parse nextweek to know when we send weekly subscriptions (day=$1 & hour=$4)
    $dispnextweek =~ m/(...)\s(...)\s(.\d|\d\d)\s(\d\d):(\d\d):(\d\d)\s(\d\d\d\d)/ ;
    my $popupweekhour = $CGI->popup_menu(-name=>'weekhour', -values=>['00', '01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', '23'], -default=>$4);
    my $popupweekday  = $CGI->popup_menu(-name=>'weekday', -values=>['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'], -default=>$1);

    my $uri_scheme = 'http';
    #by this point, the validate or refresh cache has already appended the uri scheme to the basename.  Therefore, we will
    #simply get the uri_scheme from the basename
    my $basename = $PARAMS->{basename};
    my ($http_checked, $https_checked, $dchannel_checked_yes, $dchannel_checked_no) = ("","","","");

    if ($basename =~ m/\Ahttp/) {
        ($uri_scheme, $basename) = split ("://", $basename);
    }

    if ($PARAMS->{show_discussion_channel_in_subject}) {
        $dchannel_checked_yes = 'checked';
        $dchannel_checked_no  = '';
    } else {
        $dchannel_checked_yes = '';
        $dchannel_checked_no = 'checked';
    }

    if ($uri_scheme =~ m/\Ahttps/) {
        $https_checked = 'checked';
    } else {
        $http_checked = 'checked';
    }

    my ($login_mode_default, $login_mode_https_standard, $login_mode_https) = ('','','');
    if ($PARAMS->{login_mode} eq 'https_standard') {
        $login_mode_https_standard = 'checked';
    } elsif ($PARAMS->{login_mode} eq 'https') {
        $login_mode_https          = 'checked';
    } else {
        $login_mode_default        = 'checked';
    }


    my $featureset_tag = MetadotConfig->get_featureset_tag() ;

    my $session_min = MetadotConfig->get_min_session_duration();

    my ($admin_marker, $msg_for_super_admins) = ('', '');

    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ) {

    $admin_marker =  '<font color=red size=-1>*</font>';
    $msg_for_super_admins = "$admin_marker <i>indicates fields available to site administrators.</i>";
    }

    my $s = "<br /><H1>Modify System Params</H1>";
#                  <p /><input type='checkbox' name='sessions_are_deleted_on_logout'".
#   ($defaults{sessions_are_deleted_on_logout}?
#    " checked " : "")."><b>&nbsp;Sessions are deleted on Logout</b><br />
#                  <i>If checked, the user's ID information is removed from their cookie on Logout.
#                  If not checked, then after Logout a cookie still remembers the user's name
#                  and will display it when the user returns to the site. (However, the user will still have to log in again).</i>" .

    $s .= "
<p />$admin_marker <b>Session duration (seconds)</b><br />
                <input type='text' name='sessionduration' size='50' maxlength='50' value='$PARAMS->{sessionduration}'>
                <br /><I>This is the number of seconds after which user session will expire if there is no activity on the user side e.g.8 hours = 28800 seconds. After this period the user will still be recognized by name (from the cookie) but she will have to enter her password again for any operation that requires privilege. The minimum allowed value for this parameter is $session_min seconds.</i><br />";

    # remove some server setup sections from view of unauthorized users
    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ) {

    $s .= "
                <p /><b>Mail server address</b><br />
                <input type='text' name='mailserveraddress' size='50' maxlength='255' value='$PARAMS->{mailserveraddress}'>
                <br /><I>e.g. mail.yourcompany.com or yourcompany.com</i><br />

  <p /><b>Mail sender address</b><br />
                <input type='text' name='mailsenderaddress' size='50' maxlength='255' value='$PARAMS->{mailsenderaddress}'>
                <br /><I>The default address used to send emails to users.</i><br />
                Enter the 'from' email address the software should use, e.g.
<i>siteadmin\@yourcompany.com</i><br />

  <p /><b>Subscription/Discussion account name</b><br />
                <input type='text' name='subscriptionsender' size='50' maxlength='255' value='$PARAMS->{subscriptionsender}'>
                <br /><I>The account name used for subscription notifications.</i><br />
                This is the acount that will be used to send news subscriptions, and to receive replies to discussion messages.  Enter just the account name; the host is entered below.  E.g., enter <i>discussion</i> if the full email is <i>discussion\@yourcompany.com</i>.<br />

  <p /><b>Subscription/Discussion host name</b><br />
                <input type='text' name='subscriptionhost' size='50' maxlength='255' value='$PARAMS->{subscriptionhost}'>
                <br /><I>The host, or domain, of the account used for subscription notifications.</i><br />
                This is the acount that will be used to send news subscriptions, and to receive
   replies to discussion messages.
   Enter just the host name; the acount name is entered above.
   E.g., enter <i>yourcompany.com</i> if the full email is <i>discussion\@yourcompany.com.</i><br />

  <p /><b>Subscription/Discussion password</b><br />
                <input type='text' name='subscriptionpwd' size='50' maxlength='255' value='$PARAMS->{subscriptionpwd}'>
                <br /><I>The password for the subscription account specified above.</i><br />

                <p /><b>Show discussion channel name in message subject</b><br />
                <input type = 'radio' name = 'show_discussion_channel_in_subject' value = '1' $dchannel_checked_yes>yes<br />
                <input type = 'radio' name = 'show_discussion_channel_in_subject' value = '0' $dchannel_checked_no>no
                <br /><I>Email subject will show like: [Discussion Name] Message Title</I><br />

                <p /><b>Script Basename</b><br />
                <input type='text' name='basename' size='50' maxlength='255' value='$basename'>
                <br /><I>e.g. www.yourcompany.com:8080/metadot</i><br />


                <p /><b>Uri Scheme</b><br />
                <input type = 'radio' name = 'uri_scheme' value = http $http_checked>http<br />
                <input type = 'radio' name = 'uri_scheme' value = https $https_checked>https<br />\n";

    $s .= "
                <p /><b>Login Mode</b><br />
                <input type = 'radio' name = 'login_mode' value = 'default' $login_mode_default>Default<br />
                <input type = 'radio' name = 'login_mode' value = 'https_standard' $login_mode_https_standard>Allow HTTPS and Standard login<br />\n";
                #<input type = 'radio' name = 'login_mode' value = 'https' $login_mode_https>HTTPS login only<br />\n" ;

    $s .= "<p /><b>Home page</b><br />
                <input type='text' name='home_page' size='50' maxlength='255' value='$PARAMS->{home_page}'>
                <br /><I>The default page that all users will see when no specific page is requested.<br />
                e.g. 'index.pl' goes to the main page for this application. The location you specify here
                will be appended to the HTTP script base name specified above.</i><br />

                <p /><b>Page after login</b><br />
                <input type='text' name='post_login' size='50' maxlength='255' value='$PARAMS->{post_login}'>
                <br /><I>The page that is displayed after the user logs in.<br />
                e.g. 'index.pl' goes to the main page for this application. The location you specify here
                will be appended to the HTTP script base name specified above. If you do not specify a
                location, the Home page (specified above) will be shown.</i><br />"; # if (defined($featureset_tag) && $featureset_tag eq 'closed') ;

    }

    if ($USER->can('www_show_user_directory')) {
    $s .= "<p /><input type='checkbox' name='show_user_directory'" . ($PARAMS->{show_user_directory}? " checked " : "").">" .
                "&nbsp;$admin_marker <b>Allow viewing of User Directory</b><br />" .
                "<i>Check this box if you want all users to be able to search for users in the directory.</i>";
    }

    $s .= "<p /><input type='checkbox' name='hide_my_page'" . ($PARAMS->{hide_my_page}? " checked " : "").">" .
                "&nbsp;$admin_marker <b>Suppress MyPage</b><br />" .
                "<i>Check this box if you do not want the 'MyPage' link included in the page header links.</i><br />".
                "<b>Note:</b> disabling this option will disable the MyPage function.";

    $s .= "<p /><input type='checkbox' name='require_login'".($PARAMS->{require_login}? " checked " : "").">&nbsp;$admin_marker <b>Require login</b><br />
                <i>If checked, all users must login before they can access any content on the
                site. If not checked, users who are not logged in may still access public
                content.</i><br />";

    $s .= "<p /><input type='checkbox' name='password_expire'".($PARAMS->{password_expire}? " checked " : "").">&nbsp;$admin_marker <b>Enable automatic password expiration</b><br />
           <i>If checked, user passwords will expire after a predetermined amount of time.</i><br />
           <b>Note: this option is only available when Metadot manages user information</b> (i.e. LDAP, Single SignOn are not used)";

    $s .= "<p />$admin_marker <b>Automatic password expiration duration (days)</b><br />
           <input type='text' name='password_expire_duration' value='".$USER->password_expiration_duration()."'><br />
           <i>If left blank, the default value of ".$USER->PASSWORD_EXPIRATION_DURATION_DEFAULT()." days will be used. (6 months)</i>";

    my $default_password_expire_message = Metadot::User::Default->default_password_expiration_message();

    $s .= "<p />$admin_marker <b>Password expiration message</b><br />
           <textarea rows='4' cols='40' wrap='physical' name='password_expiration_message'>". $PARAMS->{password_expiration_message} ."</textarea><br />
           <i>This message will be used by the system when it detects that a user's password has expired. If left empty, the default message:
           '<b>$default_password_expire_message</b>' will be displayed.</i>";

    $s .= "<p /><input type='checkbox' name='allow_disc_owner'".($PARAMS->{allow_disc_owner}? " checked " : "").">&nbsp;$admin_marker <b>Allow discussion owners to add subscribers to a discussion</b><br />
                <i>If checked, all registered users who create a discussion will be able
                to search on the user directory, and add members to their discussions</i><br />";

    $s .= "<p /><input type='checkbox' name='delete_previous_sessions_on_login'".($PARAMS->{delete_previous_sessions_on_login}? " checked " : "").">&nbsp;$admin_marker <b>Delete previous sessions on login</b><br />
                <i>If checked, a user's previous sessions will be deleted from the system.
                If not checked, the user's sessions will remain untouched.</i><br />";


    $s .= "<p><input type='checkbox' value='1' name='allow_email_to_submitter'". ($PARAMS->{email_to_submitter}? " checked " : "") .">&nbsp;$admin_marker <b>Allow FormMaker email-to-submitter capability</b><br />
               <i>If checked FormMaker gizmo will provide options to send an email to a user who
               submits a form</i></p>";

    # remove some sections from view of unauthorized users
    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){

    $s .= "<p /><input type='checkbox' name='nontrusted_site'";

    unless( $PARAMS->{trusted_site} ) {

        $s .= " checked ";

    }

    $s .= "><b>&nbsp;Non-Trusted site</b><br />
                <i>If checked, emails of users will be not visible to other users of the site when searching for users to add to a group,
                or when viewing messages posted by a user to a discussion. </i><br />" ;

    if (MetadotConfig->get_use_template_mgmt() eq 'true') {
#   if ((defined $featureset_tag) && ($featureset_tag eq 'closed')
#       && (MetadotConfig->get_use_template_mgmt() eq 'true')) {
        $s .= "
                <p /><input type='checkbox' name='template_mgmt'" . ($PARAMS->{template_mgmt} ?
          " checked " : "")."><b>&nbsp;Enable Template Management</b><br />
                <i>If checked, users will be able to associate a template with a specific
                 page, and admins will be able to associate a template with a specific
                 object class.</i>";
    }

    my $report_generator = Metadot::SystemApp::EventLog::ReportGeneratorFactory->get_report_generator();
    my $report_generator_isa = $report_generator->is_a();

    $s .= "
                <p /><input type='checkbox' name='is_event_log_enabled'".($PARAMS->{is_event_log_enabled}? " checked " : "")."><b>&nbsp;Enable Event Log</b>&nbsp;&nbsp;<a href=\"index.pl?isa=Metadot::SystemApp::EventLog&amp;op=show\">View Event Log</a>&nbsp;&nbsp;<a href=\"index.pl?isa=$report_generator_isa&amp;op=change_reports_year_month_date\">View Portal Usage Statistics</a><br />
                <i>If checked, all users' actions will be logged in the database in table 'event_log' in the form of
                </i>[user_id, user_name, sub_application_name, class_name, operation_name, instance_id]<i>
                If not checked, no event logging will be performed.</i><br />";
    $s .= "
                <p /><input type='text' name='event_log_clean_after' value=\"".
          $PARAMS->{event_log_clean_after} .
            "\"><b>&nbsp;Event Log Cleaning Interval</b><br />
                <i>The number of days after which events are purged from the event log database table.</i><br />
                <b>Note: The md_clean.pl script must be scheduled to run regularly on the server machine for this cleaning
                to take place.  If you enable event logging, please ensure that this script is running.</b>";
    $s .= "
                <p /><b>HTTP proxy address</b><br />
                <input type='text' name='proxyhost' size='50' maxlength='255' value='$PARAMS->{proxyhost}'>
                <br /><I>e.g. 192.168.150.16 or www.something.com. If you don't need a proxy, leave it blank</i><br />

                <p /><b>HTTP proxy port number</b><br />
                <input type='text' name='proxyport' size='50' maxlength='255' value='$PARAMS->{proxyport}'>
                <br /><I>e.g. 3128. If you don't need a proxy, leave it blank.</i><br />

                <p /><b>Images directory location</b><br />
                <input type='text' name='imageslocation' size='50' maxlength='255' value='$PARAMS->{imageslocation}'>
                <br /><i>Enter the location of the directory where you can access the images. For example, you should use '<b>/images</b>' if you can access 'powered.gif' at 'http://my.web.name/images/powered.gif'. Warning: <b>do not put trailing slashes</b> (use '/images' instead of '/images/'), <b>the directory needs to be accessible by the web server</b></i><br />" .

#                  <p /><b>File upload directory location</b><br />
#                  <input type='text' name='fileuploaddir' size='50' maxlength='255' value='$defaults{fileuploaddir}'>
#                  <br /><I>Enter the location of the directory where uploaded files should be stored. The location starts from the root directory of the web server. For example, you should use '<b>/files</b>' if the directory is '/home/httpd/htdocs/files' and the root dir of the web server is '/home/httpd/htdocs'. Warning: <b>no trailing slash & the directory needs to be accessible by the web server.</b></i><br />

                "<p /><b>Absolute path for private file upload directory</b><br />
                <input type='text' name='absolutepathfileuploaddir' size='50' maxlength='255' value='$PARAMS->{absolutepathfileuploaddir}'>
                <br /><I>Enter the absolute path of the directory you want to use for private file uploads. This directory should <b>not</b> be accessible by the web server.  Warning: <b>no trailing slash.</b></i><br />";

    }

    # remove some sections from view of unauthorized users
    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){

    $s .= "
                            <p /><b>Maximum site-wide total uploads size</b>
                <input type='text' name='global_uploads_quota' size='10' maxlength='20' value='$PARAMS->{'global_uploads_quota'}'> (MB)";
    }

    $s .= "
                            <p />$admin_marker <b>Maximum total upload size per user</b>
                <input type='text' name='fileuploadsize' size='10' maxlength='20' value='$PARAMS->{'max_total_upload_size'}'> (MB)";

    # remove some sections from view of unauthorized users
    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){

    $s .= "
                            <p /><b>Maximum total number of upload per user</b>
                <input type='text' name='fileuploadnumber' size='10' maxlength='20' value='$PARAMS->{'max_total_number_upload'}'>";
    }

    $s .= "
                <p />$admin_marker <b>Maximum number of News Items in Category News List</b>
                <input type='text' name='maxnewsitems' size='10' maxlength='20' value='$PARAMS->{'maxnewsitems'}'>";


    my $allow_group_welcome_letters_check = $PARAMS->{allow_group_welcome_letters} ? 'checked="true"' : '';
    $s .= "
                <p><input type='checkbox' name='allow_group_welcome_letters' value='1' $allow_group_welcome_letters_check/>&nbsp;<b>Enable group welcome letters  (also requires BROADCAST EMAIL permissions)</b>$admin_marker</p>
                ";


    # remove some sections from view of unauthorized users
    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){

    $s .= "
                <p /><b>Subscriptions processing information</b><br />
                Send daily subscriptions at $popupdayhour<br />
                Send weekly subscriptions on $popupweekday at $popupweekhour
                <input type='hidden' name='nexthour' value='$nexthour'>
                <br /><I>Next daily subscriptions on:&nbsp;&nbsp;$dispnextday.</i>
                <input type='hidden' name='nextday' value='$nextday'>
                <br /><I>Next weekly subscriptions on:&nbsp;&nbsp;$dispnextweek.</i><br />
                <input type='hidden' name='nextweek' value='$nextweek'><br />";

    $s .= "<input type='checkbox' name='show_who_online'".($PARAMS->{show_who_online}? " checked " : "")."><b>&nbsp;Show who is online</b><br />
                <i>If checked, the site will display how many users are online.</i><br /><br />
               <input type='checkbox' name='show_who_online_list'".($PARAMS->{show_who_online_list}? " checked " : "")."><b>&nbsp;Display a list
               of online users</b><br /><i>If checked, the site will display a list of online users with a link to their respective 'my website'</i><br />
               <b>Note: 'Show who is online' must be checked as well for this option to take effect</b><br /><br />
               <input type='text' name='show_online_limit' size='10' value='".$PARAMS->{show_online_limit}."'>&nbsp;<b>Number of online users to be listed</b>";
    }

    my $default_message = FormMaker::get_default_message();
    $s .= "<br /><br /><b>Custom \"thank you message\" for FormMaker gizmo submissions</b><br />
           This message will be displayed when a user submits a form created by the FormMaker gizmo.<br />
           If the field below is left blank, the message <b><i>
           \"$default_message\"</i></b> will be displayed.<br />
           <textarea rows='4' cols='40' wrap='physical' name='custom_form_maker'>".($PARAMS->{custom_form_maker}? "$PARAMS->{custom_form_maker}" : "");
    $s .= "</textarea><br />";

    $s .= "<br /><input type='checkbox' name='metagram_email_copy' value='1'"
          .($PARAMS->{metagram_email_copy} ? " checked " : "")." >
          <b>Send email notification to users that receive new "
          .(MetadotConfig->get_param_value('metagram_label')?
          MetadotConfig->get_param_value('metagram_label'):"Metagrams")."
          </b><br />If set an email notification will be sent to a user
          that receives a new "
          .(MetadotConfig->get_param_value('metagram_label')?
          MetadotConfig->get_param_value('metagram_label'):"metagram")."
          <br />";

    $s .= "<br /><input type='checkbox' name='metagram_thread_view' value='1'"
           .($PARAMS->{metagram_thread_view} ? " checked " : "")." >
           <b>".(MetadotConfig->get_param_value('metagram_center_name')?
           MetadotConfig->get_param_value('metagram_center_name'):"Message Center")."
            default view by sender</b><br />
           If checked, the <i>metagram center</i>'s default view will be a
           list of message senders as oppossed to a
           list of messages ordered by date.<br />";

    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){
    my $users_min = MetadotConfig->get_min_users_number();


    unless( MetadotConfig->get_param_value('user_management_is_external') ) {

        $s .= "<p /><b>Allowed Users Quota (number of users)</b>
                <input type='text' name='max_number_of_users' size='10' maxlength='20' value='$PARAMS->{max_number_of_users}'><br />
                <i>Leave this blank for unlimited number of registrations. The minimum allowed value for this parameter is $users_min.</i><br />";

    }

    $s .= "<p /><b>Documentation URL</b><br />
                <input type='text' name='docs_url' size='50' maxlength='255' value='$PARAMS->{docs_url}'>
                <br /><I>Set to the absolute or relative URL of the installed documentation (e.g. '/docs' or 'http://www.metadot.com')</i><br />\n";

    }

    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){
        $s .= "<p /><input type='checkbox' name='hide_metadot_signature_in_html_source' ".
              ($PARAMS->{hide_metadot_signature_in_html_source} ? " checked ": ""). ">&nbsp;$admin_marker
              <b>Hide Metadot signature in HTML source code</b><br />\n<i>If checked, all Metadot
              signatures will be hidden in HTML source code.</i><br />\n";

        $s .= "<p /><input type='checkbox' name='hide_metadot_signature_in_printer_friendly' ".
              ($PARAMS->{hide_metadot_signature_in_printer_friendly} ? " checked ": ""). ">&nbsp;$admin_marker
              <b>Hide Metadot signature in printer friendly function</b><br />\n<i>If checked, the Metadot
              signature will be hidden in printer friendly function.</i><br />\n";
    }

    $s .= "<p /><input type='checkbox' name='allow_discussion_attachments' ".($PARAMS->{allow_discussion_attachments} ? " checked " : ""). ">&nbsp;$admin_marker <b>Allow files to be attached to discussion messages</b><br />\n<i>If checked, messages will be allowed to contain attachments.</i><br />\n";

    $s .= "<p /><input type='checkbox' name='send_moderated_posting_notifications'".($PARAMS->{send_moderated_posting_notifications} ? " checked " : ""). ">&nbsp;$admin_marker <b>Send moderated discussion notifications</b><br />\n
                <i>If checked, discussion moderators will receive a notification email every time a new message is posted to the discussions they moderate.</i><br />\n";

    $s .= "<p /><input type='checkbox' name='allow_discussion_message_editing'".($PARAMS->{allow_discussion_message_editing} ? " checked " : ""). ">&nbsp;$admin_marker <b>Allow users to edit their discussion messages</b><br />\n
                <i>If checked, users will be allowed to edit their own discussion messages.</i><br />\n";

    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){
        $s .= "<p /><input type='checkbox' name='allow_discussion_email_alias'".($PARAMS->{allow_discussion_email_alias} ? " checked " : ""). ">&nbsp;$admin_marker <b>Allow users to assign email address to discussions</b><br />\n
                    <i>If checked, users will be allowed to assign email aliases to discussions. If a discussion has an email alias assigned to it, and that alias is properly configured on the POP server, the discussion can create new threads from emails that it receives on that alias.</i><br />\n";

        $s .= "<p /><b>Base domain for discussion email aliases</b><br /><input type=text size=50 maxlength=255 name=\"email_alias_base_domain\" value=\"".($PARAMS->{email_alias_base_domain})."\"><br />\n
                    <i>Email aliases assigned to discussions will use the domain specified here. </i><br />
                    <strong>Note: This domain MUST END WITH the URL of this site.</strong> For example, if the URL of this site is 'myplace.metadot.com', the base domain can be 'discussions.myplace.metadot.com', but <strong>NOT</strong> 'discussions.myplace.com'.<br />
                    <br />If you do not specify a base domain, the default will be:<br /><strong>".Metadot::EmailToDiscussion->get_default_alias_domain()."</strong><br />\n";

        $s .= "<p /><b>Web character set</b><br /><input type='text' name='http_charset' value='".$PARAMS->{http_charset}. "'><br /><i>The character set to be declared by all Metadot web pages. The default character set for most web servers is <strong>iso-8859-1</strong>, but you may want to try <strong>utf-8</strong>. If left blank, Metadot will use ".$DISPLAY->DEFAULT_CHARSET().".</i><br />\n";

        # build the select options
        my @content_type_options = (
          '', '- Metadot default -',
          'text/plain', 'Plain text',
          'text/html', 'HTML',
        );

        my $default_email_content_type_selection = qq~<select name="default_mail_content_type">\n~;
        for (my $i = 0; $i < @content_type_options; $i+=2) {
            my $value = $content_type_options[$i];
            my $label = $content_type_options[$i+1];
            my $selected = $PARAMS->{default_mail_content_type} eq $value ? 'selected' : '';
            $default_email_content_type_selection .= qq~<option value="$value" $selected>$label</option>\n~;
        }
        $default_email_content_type_selection .= "</select>\n";

        $s .= "<p /><b>Default email content-type</b><br />$default_email_content_type_selection<br /><i>The default content-type used by Metadot's mail processor. The default selection is ".Sendmail->DEFAULT_MAIL_CONTENT_TYPE().".</i><br />\n";



        my $discussion_message_content_type_selection = qq~<select name="discussion_message_content_type">\n~;
        for (my $i = 0; $i < @content_type_options; $i+=2) {
            my $value = $content_type_options[$i];
            my $label = $content_type_options[$i+1];
            my $selected = $PARAMS->{discussion_message_content_type} eq $value ? 'selected' : '';
            $discussion_message_content_type_selection .= qq~<option value="$value" $selected>$label</option>\n~;
        }
        $discussion_message_content_type_selection .= "</select>\n";

        $s .= "<p /><b>Discussion content-type</b><br />$discussion_message_content_type_selection<br /><i>The content-type used in discussion forums. The default selection is ".Discussion->DEFAULT_MESSAGE_CONTENT_TYPE().". Note: If you change this setting, messages which have already been posted to the discussion forums may display incorrectly, but future messages should display correctly. <b>CAUTION: Oracle users are strongly advised to use text/plain!</b></i><br />\n";

    }


    $s .= "<p /><input type='checkbox' name='default_page_channel_creation' ".($PARAMS->{default_page_channel_creation} ? " checked " : ""). ">&nbsp; <b>Create a channel for each new page</b><br />\n<i>If checked, a channel will automatically be created for each new page. If not checked, the page editors can manually enable this feature in the page properties. Note: this will increase subscription processing time.</i><br />\n";

    $s .= "<p /><input type='checkbox' name='default_update_notification_enablement' ".($PARAMS->{default_update_notification_enablement} ? " checked " : ""). ">&nbsp; <b>Enable content update notifications for each new page</b><br />\n<i>If checked, each new page will have content update notifications enabled. If not checked, the page editors can manually enable this feature in the page properties. Note: this will also create a channel for each new page, increasing subscription processing time.</i><br />\n";



    $s .= "<hr/>$msg_for_super_admins\n";

    return $s;
}



sub show_edit_form_message {
    my $self = shift;

    my $defregmessage = $PARAMS->{defregmessage};
    my $aprregmessage = $PARAMS->{aprregmessage};
    my $rejregmessage = $PARAMS->{rejregmessage};
    my $new_user_notification_message = $PARAMS->{new_user_notification_message};
    my $subsmessage = $PARAMS->{subsmessage};
    my $posting_notification_message = $PARAMS->{posting_notification_message};
    my $metagrammessage = $PARAMS->{metagram_email_message};

    $defregmessage   = escapeHTML($defregmessage);
    $aprregmessage   = escapeHTML($aprregmessage);
    $rejregmessage   = escapeHTML($rejregmessage);
    $subsmessage     = escapeHTML($subsmessage);
    $metagrammessage = escapeHTML($metagrammessage);

    my $metagram_label = MetadotConfig->get_param_value('metagram_label') || 'Metagram';

    my $s = "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\"><tr><td colspan = 2><b>Modify Messages Sent by the System</b><br /><br />
                                                                                <tr><td  width= '50%' bgcolor=\"#bbbbbb\"><b>Instructions</b>
                                                                                       </td><td width='3%' bgcolor=\"#bbbbbb\">&nbsp;</td><td bgcolor=\"#bbbbbb\"><b>Legend</b></td>
                                                                                </tr><tr><td bgcolor=\"#efefef\" valign='top'><I>If you would like to put the name of the user in the message, you can do so by putting the word \"fullname\" in brackets.  For example:  Hello, &lt;&lt;fullname&gt;&gt;.
                                                                           </td><td width='3%' bgcolor=\"#efefef\">&nbsp;</td><td bgcolor=\"#efefef\">&lt;&lt;fullname&gt;&gt;<br />&lt;&lt;email&gt;&gt<br /> &lt;&lt;nickname&gt;&gt<br />&lt;&lt;website_url&gt;&gt;<br />&lt;&lt;forgot_pwd_url&gt;&gt;<br />&lt;&lt;new_user_link&gt;&gt;<br />&lt;&lt;discussion_name&gt;&gt;<br />&lt;&lt;posting_approval_link&gt;&gt;<br />&lt;&lt;profile_url&gt;&gt;<br />&lt;&lt;subscription_url&gt;&gt;
                   </td></tr></table>
<table border = 0 cellpadding = 0 cellspacing = 0>
        <tr><td><b>Default Registration Message</b></td></tr>
        <tr><td><textarea name='defregmessage' rows=8 cols=60 WRAP='VIRTUAL'>$defregmessage</textarea></td></tr>
        <tr><td><I>This is the message sent to users after registration if admin approval is not turned on.</i></td></tr>
    <tr><td>&nbsp;</td></tr>

        <tr><td><b>Approval Registration Message</b></td></tr>
        <tr><td><textarea name='aprregmessage' rows=8 cols=60 WRAP='VIRTUAL'>$aprregmessage</textarea></td></tr>
        <tr><td><I>This is the message sent to users after registration if admin approval is turned on.</i></td></tr>
    <tr><td>&nbsp;</td></tr>

        <tr><td><b>Rejected Registration Message</b></td></tr>
        <tr><td><textarea name='rejregmessage' rows=8 cols=60 WRAP='VIRTUAL'>$rejregmessage</textarea></td></tr>
        <tr><td><I>This is the message sent to users after registration if registration is rejected.</i></td></tr>
    <tr><td>&nbsp;</td></tr>

        <tr><td><b>New User Notification Message</b></td></tr>
        <tr><td><textarea name='new_user_notification_message' rows=8 cols=60 WRAP='VIRTUAL'>$new_user_notification_message</textarea></td></tr>
        <tr><td><I>This is the message sent to admins whenever new users are awaiting approval.</i></td></tr>
    <tr><td>&nbsp;</td></tr>

        <tr><td><b>Subscription Email Footer</b></td></tr>
        <tr><td><textarea name='subsmessage' rows=8 cols=60 WRAP='VIRTUAL'>$subsmessage</textarea></td></tr>
        <tr><td><I>This is the message sent to users along with their subscription.</i></td></tr>
        <tr><td>&nbsp;</td></tr>

        <tr><td><b>Moderated Discussion Posting Notification Message</b></td></tr>
        <tr><td><textarea name='posting_notification_message' rows=8 cols=60 WRAP='VIRTUAL'>$posting_notification_message</textarea></td></tr>
        <tr><td><I>This message is sent to discussion moderators when a new message is posted to moderated discussions.</i></td></tr>
        <tr><td>&nbsp;</td></tr>

        <tr><td><b>$metagram_label Email Message</b></td></tr>
        <tr><td><textarea name='metagram_email_message' rows=8 cols=60 WRAP='VIRTUAL'>$metagrammessage</textarea></td></tr>
        <tr><td><I>This is the message sent to users when they receive a $metagram_label.</i></td></tr>
        <tr><td>&nbsp;</td></tr>

</table>

                ";
    return $s;
}

sub show_form_message {
    my $self = shift;
    my $defregmessage = $self->substitute($FORM{'defregmessage'});
    my $aprregmessage = $self->substitute($FORM{'aprregmessage'});
    my $rejregmessage = $self->substitute($FORM{'rejregmessage'});
    my $new_user_notification_message = $self->substitute($FORM{new_user_notification_message});
    my $subsmessage = $self->substitute($FORM{'subsmessage'});
    my $posting_notification_message = $self->substitute($FORM{posting_notification_message});
    my $metagram_email_message = $self->substitute($FORM{'metagram_email_message'});

    my $metagram_label = MetadotConfig->get_param_value('metagram_label') || 'Metagram';

    my $s = " <br /><H1>Message Example</H1><br />
    <table border=\"0\" cellpadding=\"0\" cellspacing=\"0\">
                         <tr><td bgcolor=\"#bbbbbb\"><b>Default Registration Message</b></td></tr>
                        <tr><td bgcolor=\"#cccccc\">$defregmessage</td></tr>
                        <tr><td bgcolor=\"#eeeeee\"><I>This is the message sent to users after registration if admin approval is not turned on.</i></td></tr>
                        <tr><td>&nbsp;</td></tr>

                        <tr><td bgcolor=\"#bbbbbb\"><b>Approval Registration Message</b></td></tr>
                        <tr><td bgcolor=\"#cccccc\">$aprregmessage</td></tr>
                        <tr><td bgcolor=\"#eeeeee\"><I>This is the message sent to users after registration if admin approval is turned on..</i></td></tr>
                        <tr><td>&nbsp;</td></tr>

                        <tr><td bgcolor=\"#bbbbbb\"><b>Rejected Registration Message</b></td></tr>
                        <tr><td bgcolor=\"#cccccc\">$rejregmessage</td></tr>
                        <tr><td bgcolor=\"#eeeeee\"><I>This is the message sent to user after registration if registration is rejected.</i></td></tr>
                        <tr><td>&nbsp;</td></tr>

                        <tr><td bgcolor=\"#bbbbbb\"><b>New User Notification Message</b></td></tr>
                        <tr><td bgcolor=\"#cccccc\">$new_user_notification_message</td></tr>
                        <tr><td bgcolor=\"#eeeeee\"><I>This is the message sent to admins whenever new users are awaiting approval.</i></td></tr>
                        <tr><td>&nbsp;</td></tr>

                        <tr><td bgcolor=\"#bbbbbb\"><b>Subscription Email Footer</b></td></tr>
                        <tr><td bgcolor=\"#cccccc\">$subsmessage</td></tr>
                        <tr><td bgcolor=\"#eeeeee\"><I>This is the message sent to users along with their subscription.</i></td></tr>
                        <tr><td>&nbsp;</td></tr>

                        <tr><td bgcolor=\"#bbbbbb\"><b>Moderated Discussion Posting Notification Message</b></td></tr>
                        <tr><td bgcolor=\"#cccccc\">$posting_notification_message</td></tr>
                        <tr><td bgcolor=\"#eeeeee\"><I>This message is sent to discussion moderators when a new message is posted to moderated discussions.</i></td></tr>

                        <tr><td>&nbsp;</td></tr>
                        <tr><td bgcolor=\"#bbbbbb\"><b>$metagram_label Email Message</b></td></tr>
                        <tr><td bgcolor=\"#cccccc\">$metagram_email_message</td></tr>
                        <tr><td bgcolor=\"#eeeeee\"><I>This is the message sent to users when they receive a $metagram_label.</i></td></tr>
                        <tr><td>&nbsp;</td></tr>


                  </table><br /><br />
                  <center><a href = ".$self->make_url("index.pl?isa=Portal&amp;op=modify_message").">Modify Messages</a><br /><a href=".$self->make_url("index.pl?isa=Portal&amp;op=show_admin").">Show Administration Page</a></center><br /><br />
                ";
    return $s;
}

sub substitute{
        my $self = shift;
        my $message = shift;

          $message=~s/<<forgot_pwd_url>>/$PARAMS->{basename}\/index.pl?isa=Session&amp;op=forgot_password/gi;
          $message=~s/<<fullname>>/Bob Agasi/gi;
          $message=~s/<<nickname>>/Bob/gi;
          $message=~s/<<website_url>>/$PARAMS->{basename}\/index.pl/gi;
          $message=~s/<<profile_url>>/$PARAMS->{basename}\/index.pl?isa=User&amp;op=modify_self/gi;
          $message=~s/<<subscription_url>>/$PARAMS->{basename}\/userchannel.pl?op=select/gi;
          $message=~s/<<email>>/bobagasi\@yourwebsite.com/gi;
          $message=~s/<<password>>/passwd123/gi;
          $message=~s/\n/<br \/>/g;
          $message=~s/\n\n/<br \/><br \/>/g;
          #CKIM-ADMIN-NOTIFY
          $message=~s/<<new_user_link>>/(link to new user page)/gi;

          $message=~s/<<discussion_name>>/(name of the discusion)/gi;
          $message=~s/<<posting_approval_link>>/(link to message moderation page)/gi;

          return $message;
}
#####################################################
sub save_params {
    my $self = shift;

    my %weekdays = ('Sun' => 1, 'Mon' => 2, 'Tue' => 3, 'Wed' => 4, 'Thu' => 5, 'Fri' => 6, 'Sat' => 7);
    my $nowtime = time ;
    localtime($nowtime) =~ m/(...)\s(...)\s(.\d|\d\d)\s(\d\d):(\d\d):(\d\d)\s(\d\d\d\d)/;
    my $nextday  = $nowtime + ($FORM{dayhour}-$4)*60*60 - $5*60 -$6;
    $nextday += 24*60*60 if ($nextday<$nowtime) ;
    my $nextweek = $nowtime + ($weekdays{$FORM{weekday}}-$weekdays{$1})*24*60*60 + ($FORM{weekhour}-$4)*60*60 - $5*60 -$6;
    $nextweek += 7*24*60*60 if ($nextweek<$nowtime) ;
    my $nexthour = $FORM{nexthour} =~ /^\d+$/ ? $FORM{nexthour} : $nowtime ;
    my $countersstring = "h=$nexthour-d=$nextday-w=$nextweek";

    my $min_session = MetadotConfig->get_min_session_duration();

    if( $FORM{sessionduration} >= $min_session ) {

    $PARAMS->set_sessionduration($FORM{sessionduration});

    } else {

    $DISPLAY->print_click_back( "Session duration minimum value should be of $min_session seconds." );
    exit;
    }

    $PARAMS->set_require_login($FORM{require_login});
    if ($USER->can('www_show_user_directory')) {
        $PARAMS->set_property('show_user_directory', $FORM{show_user_directory});
    }
    $PARAMS->set_hide_my_page($FORM{hide_my_page});
    $PARAMS->set_property('delete_previous_sessions_on_login', $FORM{delete_previous_sessions_on_login});

    $PARAMS->set_property( 'allow_discussion_attachments', $FORM{allow_discussion_attachments} );

    $PARAMS->set_property( 'send_moderated_posting_notifications', $FORM{send_moderated_posting_notifications} );
    $PARAMS->set_property( 'allow_discussion_message_editing', $FORM{allow_discussion_message_editing} );

    $PARAMS->set_max_total_upload_size($FORM{'fileuploadsize'});
    $PARAMS->set_maxnewsitems($FORM{maxnewsitems});

    $PARAMS->set_property('allow_group_welcome_letters', $FORM{allow_group_welcome_letters});

    # CKIM: moved this outside of the is_allowed_to_do check
    $PARAMS->set_property('email_to_submitter',$FORM{'allow_email_to_submitter'});


    # prevent unauthorixed users from saving these params
    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ) {

        $PARAMS->set_mailserveraddress($FORM{mailserveraddress});
        $PARAMS->set_subscriptionsender($FORM{subscriptionsender});
        $PARAMS->set_subscriptionhost($FORM{subscriptionhost});
        $PARAMS->set_subscriptionpwd($FORM{subscriptionpwd});
        $PARAMS->set_mailsenderaddress($FORM{mailsenderaddress});

        $PARAMS->set_basename($FORM{basename});
        $PARAMS->set_uri_scheme($FORM{uri_scheme});
        $PARAMS->set_property('login_mode', $FORM{login_mode});
        $PARAMS->set_property('allow_disc_owner', $FORM{allow_disc_owner});

        $PARAMS->set_home_page($FORM{home_page});
        $PARAMS->set_post_login($FORM{post_login});

        # aju -- was a request to reverse the logic, but need to keep the param
        # the same for backwards compat.
        if ($FORM{nontrusted_site}) {
            $PARAMS->set_trusted_site('0');
        }
        else {
            $PARAMS->set_trusted_site('1');
        }
    #      $PARAMS->{trusted_site} = $FORM{trusted_site};

        $PARAMS->set_template_mgmt($FORM{template_mgmt});
        $PARAMS->set_is_event_log_enabled($FORM{is_event_log_enabled});
        $PARAMS->set_event_log_clean_after($FORM{event_log_clean_after});

        $PARAMS->set_absolutepathfileuploaddir($FORM{absolutepathfileuploaddir});
    #      $params->{fileuploaddir}  = $FORM{fileuploaddir};
        $PARAMS->set_imageslocation($FORM{imageslocation});
        $PARAMS->set_proxyhost($FORM{proxyhost});
        $PARAMS->set_proxyport($FORM{proxyport});
        $PARAMS->set_timecounters($countersstring);

        $PARAMS->set_max_total_number_upload($FORM{fileuploadnumber});

        $PARAMS->set_global_uploads_quota($FORM{global_uploads_quota});

        $PARAMS->set_property('show_who_online',$FORM{show_who_online});
        $PARAMS->set_property('show_who_online_list',$FORM{show_who_online_list});
        $PARAMS->set_property('show_online_limit',$FORM{show_online_limit});
        $PARAMS->set_property('custom_form_maker',$FORM{custom_form_maker});



        $PARAMS->set_property('custom_form_maker',$FORM{custom_form_maker});

        $PARAMS->set_property('metagram_email_copy', $FORM{metagram_email_copy});
        $PARAMS->set_property('metagram_thread_view', $FORM{metagram_thread_view});

        ## IF password_expiration feature is turned on, update the
        ## password_changed_time field
        my $current_password_expire_setting = $PARAMS->{password_expire};
        if( $current_password_expire_setting ne 'on' &&
            $FORM{password_expire} eq 'on'){
            $USER->initialize_password_changed_times();
        }
        $PARAMS->set_property('password_expire', $FORM{password_expire});

        $PARAMS->set_property('password_expire_duration', $FORM{password_expire_duration});
        $PARAMS->set_property('password_expiration_message',$FORM{password_expiration_message});

            my $min_users = MetadotConfig->get_min_users_number();

        if( ( $FORM{max_number_of_users} > $min_users ) || ( $FORM{max_number_of_users} eq '' )
            || ( not defined $FORM{max_number_of_users} ) ) {

            $PARAMS->set_property( 'max_number_of_users', $FORM{max_number_of_users} || '' );

        } else {

            $DISPLAY->print_click_back( "Allowed Users Quota minimum value should be of $min_users." );
            exit;
        }

        $PARAMS->set_property( 'docs_url', $FORM{docs_url} );

        $PARAMS->set_property( 'hide_metadot_signature_in_html_source', $FORM{hide_metadot_signature_in_html_source} );
        $PARAMS->set_property( 'hide_metadot_signature_in_printer_friendly', $FORM{hide_metadot_signature_in_printer_friendly} );

        $PARAMS->set_property( 'allow_discussion_email_alias', $FORM{allow_discussion_email_alias} );



        # check that the value is a tail-end match with the server name
        my $email_alias_base_domain = $FORM{email_alias_base_domain};
        if ($email_alias_base_domain) {
            my $server_name = MetadotConfig->get_server_name();
            unless ($email_alias_base_domain =~ m/\Q$server_name\E$/) {
                $self->print_click_back( "The email alias base domain MUST end with \"$server_name\".");
                exit;
            }
        }
        $PARAMS->set_property( 'email_alias_base_domain', $email_alias_base_domain );

        $PARAMS->set_property('http_charset', $FORM{http_charset});

        $PARAMS->set_property('default_mail_content_type', $FORM{default_mail_content_type});

        $PARAMS->set_property('discussion_message_content_type', $FORM{discussion_message_content_type});


    }

    $PARAMS->set_property( 'show_discussion_channel_in_subject', $FORM{show_discussion_channel_in_subject});

    $PARAMS->set_property( 'default_page_channel_creation', $FORM{default_page_channel_creation});
    $PARAMS->set_property( 'default_update_notification_enablement', $FORM{default_update_notification_enablement});


}


sub save_message{
    my $self = shift;


    $PARAMS->set_defregmessage( $FORM{defregmessage} );
    $PARAMS->set_aprregmessage( $FORM{aprregmessage} );
    $PARAMS->set_rejregmessage( $FORM{rejregmessage} );
    $PARAMS->set_new_user_notification_message( $FORM{new_user_notification_message} );
    $PARAMS->set_subsmessage( $FORM{subsmessage} );

    $PARAMS->set_property('posting_notification_message',
              $FORM{posting_notification_message});

    $PARAMS->set_property('metagram_email_message',
                          $FORM{metagram_email_message} );
}


sub _process_and_add_ldap_toggle {
    my $self = shift;
    my $get_bgcolor = shift;

    return ''  unless( MetadotConfig->get_select_registration_modules_from_GUI() );

    my $ldap_toggle;
    my $ldap_disable_toggle = "<font color=red size=-1>Disabled</font>&nbsp;&nbsp;<a href='" .
       $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;ldap_registration=enable") . "'>(enable)</a>";
    my $ldap_enable_toggle =  "<font color=red size=-1>Enabled</font>&nbsp;&nbsp;<a href='" .
       $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;ldap_registration=disable") . "'>(disable)</a>";



    if( defined $FORM{ldap_registration} && ( lc $FORM{ldap_registration} eq 'enable' ) ) {
       $PARAMS->set_if_ldap('1');
       $ldap_toggle = $ldap_enable_toggle;
    } elsif ( defined $FORM{ldap_registration} && ( lc $FORM{ldap_registration} eq 'disable' ) ) {
       $ldap_toggle = $ldap_disable_toggle;
       $PARAMS->set_if_ldap('0');
    } elsif ( $PARAMS->{if_ldap} ) {
        $ldap_toggle = $ldap_enable_toggle;
    } else {
       $ldap_toggle = $ldap_disable_toggle;
    }

    my $bgcolor = &$get_bgcolor;

    my $result = "<TR bgcolor = $bgcolor>
                     <td><b>LDAP Registration</b></td>
                        <td>$ldap_toggle</td>
                        <td><i><font size=-1>This sets whether or not individuals need to be in an LDAP directory that you specify</i></font></td>
                </TR>";

    return $result;

}


sub _process_and_add_my_website_toggle {
    my $self = shift;
    my $get_bgcolor = shift;
    my $admin_marker = shift;

    my $my_website_toggle;
    my $website_disable_toggle = "<font color=red size=-1>Disabled</font>&nbsp;&nbsp;<a href='" .
      $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;my_website_creation=enable") . "'>(enable)</a>";
    my $website_enable_toggle = "<font color=red size=-1>Enabled</font>&nbsp;&nbsp;<a href='" .
      $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;my_website_creation=disable") . "'>(disable)</a>";

    if( defined $FORM{my_website_creation} && ( lc $FORM{my_website_creation} eq 'enable' ) ) {
        $my_website_toggle = $website_enable_toggle;
        $PARAMS->set_my_website_creation('1');
    } elsif ( defined $FORM{my_website_creation} && ( lc $FORM{my_website_creation} eq 'disable' ) ) {
        $my_website_toggle = $website_disable_toggle;
        $PARAMS->set_my_website_creation('0');
    } elsif ( $PARAMS->{my_website_creation} ) {
        $my_website_toggle = $website_enable_toggle;
    } else {
        $my_website_toggle = $website_disable_toggle;
    }

    my $bgcolor = &$get_bgcolor;

    my $result = "<TR bgcolor = $bgcolor>
                     <td>$admin_marker <b>My Website Creation</b></td>
                        <td>$my_website_toggle</td>
                        <td><i><font size=-1>This sets whether or not users get a My Website page for their personal use.</i></font></td>
                </TR>";

    return $result;

}

sub _process_email_friend_toggle {
    my $self = shift;
    my $get_bgcolor = shift;
    my $admin_marker = shift;

    # aju -- if param hasn't been defined yet, the toggling won't work. (sigh).
    # so don't even show the info.
    if (!exists($PARAMS->{email_friend})) {
    return "";
    }
    my $toggle;
    my $disable_toggle = "<font color=red size=-1>Disabled</font>&nbsp;&nbsp;<a href='" .
      $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;email_friend=enable") . "'>(enable)</a>";
    my $enable_toggle = "<font color=red size=-1>Enabled</font>&nbsp;&nbsp;<a href='" .
      $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;email_friend=disable") . "'>(disable)</a>";

    if( defined $FORM{email_friend} && ( lc $FORM{email_friend} eq 'enable' ) ) {
        $toggle = $enable_toggle;
        $PARAMS->set_email_friend('0');
    } elsif ( defined $FORM{email_friend} && ( lc $FORM{email_friend} eq 'disable' ) ) {
        $toggle = $disable_toggle;
        $PARAMS->set_email_friend('1');
    } elsif (!($PARAMS->{email_friend}) ) {
        $toggle = $enable_toggle;
    } else {
        $toggle = $disable_toggle;
    }

    my $bgcolor = &$get_bgcolor;

    my $result = "<TR bgcolor = $bgcolor>
                     <td>$admin_marker <b>Email Friend and Printer Friendly Buttons</b></td>
                        <td>$toggle</td>
                        <td><i><font size=-1>This sets whether or not Email Friend and Printer Friendly Buttons will be displayed.</i></font></td>
                </TR>";

    return $result;

}


sub _process_invitable_toggle {
    my $self = shift;
    my $get_bgcolor = shift;
    my $admin_marker = shift;

    my $toggle;
    my $disable_toggle = "<font color=red size=-1>Disabled</font>&nbsp;&nbsp;<a href='" .
      $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;invitable=enable") . "'>(enable)</a>";
    my $enable_toggle = "<font color=red size=-1>Enabled</font>&nbsp;&nbsp;<a href='" .
      $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;invitable=disable") . "'>(disable)</a>";

    if( defined $FORM{invitable} && ( lc $FORM{invitable} eq 'enable' ) ) {
        $toggle = $enable_toggle;
        $PARAMS->set_property('disable_invitable', 0);
    } elsif ( defined $FORM{invitable} && ( lc $FORM{invitable} eq 'disable' ) ) {
        $toggle = $disable_toggle;
        $PARAMS->set_property('disable_invitable', 1);
    } elsif ($PARAMS->{disable_invitable}) {
        $toggle = $disable_toggle;
    } else {
        $toggle = $enable_toggle;
    }

    my $bgcolor = &$get_bgcolor;

    my $result = "<TR bgcolor = $bgcolor>
                     <td>$admin_marker <b>Broadcast Emails</b></td>
                        <td>$toggle</td>
                        <td><i><font size=-1>This sets whether or not certain features will be enabled that allow
                        users to send emails to other users.
                        <b>Note: The following features depend on Broadcast Emails to function</b>
                        <ul>
                          <li>Invitations</li>
                          <li>Email Friend</li>
                          <li>Group Welcome Letters</li>
                        </ul>
                        </i></font></td>
                </TR>";

    return $result;

}



sub _process_and_add_recursive_perms_toggle {
    my $self = shift;
    my $get_bgcolor = shift;
    my $toggle;
    my $disable_toggle = "<font color=red size=-1>Disabled</font>&nbsp;&nbsp;<a href='" .
      $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;recursive_perms=enable") . "'>(enable)</a>";
    my $enable_toggle = "<font color=red size=-1>Enabled</font>&nbsp;&nbsp;<a href='" .
      $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;recursive_perms=disable") . "'>(disable)</a>";

    if( defined $FORM{recursive_perms} && ( lc $FORM{recursive_perms} eq 'enable' ) ) {
    $toggle = $enable_toggle;
    $PARAMS->set_recursive_perms('1');

    } elsif ( defined $FORM{recursive_perms} && ( lc $FORM{recursive_perms} eq 'disable' ) ) {
    $toggle = $disable_toggle;
    $PARAMS->set_recursive_perms('0');
    } elsif( $PARAMS->{recursive_perms} ) {
        $toggle = $enable_toggle;

    } else {

    $toggle = $disable_toggle;

    }

    my $bgcolor = &$get_bgcolor;

    my $result = "<TR bgcolor = $bgcolor>
                     <td><b>Recursive Permissions</b></td>
                        <td>$toggle</td>
                        <td><i><font size=-1>This sets whether or not users get the advanced option of changing permissions recursively.</i></font></td>
                </TR>";

    return $result;

}

sub _process_mystuff_metagrams_toggle {
    my $self = shift;
    my $get_bgcolor = shift;

    my $toggle;
    my $disable_toggle = "<font color=red size=-1>Disabled</font>&nbsp;&nbsp;<a href='"
                       . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;mystuff_metagrams=enable") . "'>(enable)</a>";
    my $enable_toggle = "<font color=red size=-1>Enabled</font>&nbsp;&nbsp;<a href='"
                      . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;mystuff_metagrams=disable") . "'>(disable)</a>";

    if( defined $FORM{mystuff_metagrams} && ( lc $FORM{mystuff_metagrams} eq 'enable' ) ) {
        $toggle = $enable_toggle;
        $PARAMS->set_property('mystuff_metagrams_enabled',1);
    } elsif ( defined $FORM{mystuff_metagrams} && ( lc $FORM{mystuff_metagrams} eq 'disable' ) ) {
        $toggle = $disable_toggle;
        $PARAMS->set_property('mystuff_metagrams_enabled',0);
    } elsif( $PARAMS->{mystuff_metagrams_enabled} ) {
        $toggle = $enable_toggle;
    } else {
        $toggle = $disable_toggle;
    }
    my $metagram_name = MetadotConfig->get_param_value('metagram_center_name') || 'Metagrams';
    my $bgcolor = &$get_bgcolor;
    my $result = <<EOM
        <tr bgcolor = $bgcolor>
            <td><b><i>$metagram_name</i> Link</b></td>
            <td>
                $toggle
            </td>
            <td><i><font size=-1>This sets whether users can see the '$metagram_name' link in the 'My Stuff' area.</i></font></td>
        </tr>
EOM
;
    return $result;
}

sub _process_mystuff_changeprofile_toggle {
    my $self = shift;
    my $get_bgcolor = shift;

    my $toggle;
    my $disable_toggle = "<font color=red size=-1>Disabled</font>&nbsp;&nbsp;<a href='"
                       . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;mystuff_changeprofile=enable") . "'>(enable)</a>";
    my $enable_toggle = "<font color=red size=-1>Enabled</font>&nbsp;&nbsp;<a href='"
                      . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;mystuff_changeprofile=disable") . "'>(disable)</a>";

    if( defined $FORM{mystuff_changeprofile} && ( lc $FORM{mystuff_changeprofile} eq 'enable' ) ) {
        $toggle = $enable_toggle;
        $PARAMS->set_property('mystuff_changeprofile',1);
    } elsif ( defined $FORM{mystuff_changeprofile} && ( lc $FORM{mystuff_changeprofile} eq 'disable' ) ) {
        $toggle = $disable_toggle;
        $PARAMS->set_property('mystuff_changeprofile',0);
    } elsif( $PARAMS->{mystuff_changeprofile} ) {
        $toggle = $enable_toggle;
    } else {
        $toggle = $disable_toggle;
    }

    my $bgcolor = &$get_bgcolor;
    my $result = <<EOM
        <tr bgcolor = $bgcolor>
            <td><b><i>View / change my profile</i> Link</b></td>
            <td>
                $toggle
            </td>
            <td><i><font size=-1>This sets whether users can see the 'View / change my profile' link in the 'My Stuff' area.</i></font></td>
        </tr>
EOM
;
    return $result;
}

sub _process_mystuff_mygroups_toggle {
    my $self = shift;
    my $get_bgcolor = shift;

    my $toggle;
    my $disable_toggle = "<font color=red size=-1>Disabled</font>&nbsp;&nbsp;<a href='"
                       . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;mystuff_mygroups=enable") . "'>(enable)</a>";
    my $enable_toggle = "<font color=red size=-1>Enabled</font>&nbsp;&nbsp;<a href='"
                      . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;mystuff_mygroups=disable") . "'>(disable)</a>";

    if( defined $FORM{mystuff_mygroups} && ( lc $FORM{mystuff_mygroups} eq 'enable' ) ) {
        $toggle = $enable_toggle;
        $PARAMS->set_property('mystuff_mygroups',1);
    } elsif ( defined $FORM{mystuff_mygroups} && ( lc $FORM{mystuff_mygroups} eq 'disable' ) ) {
        $toggle = $disable_toggle;
        $PARAMS->set_property('mystuff_mygroups',0);
    } elsif( $PARAMS->{mystuff_mygroups} ) {
        $toggle = $enable_toggle;
    } else {
        $toggle = $disable_toggle;
    }

    my $bgcolor = &$get_bgcolor;
    my $result = <<EOM
        <tr bgcolor = $bgcolor>
            <td><b><i>My groups</i> Link</b></td>
            <td>
                $toggle
            </td>
            <td><i><font size=-1>This sets whether users can see the 'My groups' link in the 'My Stuff' area.</i></font></td>
        </tr>
EOM
;
    return $result;
}

sub _process_mystuff_mysubmissions_for_review {
    my $self = shift;
    my $get_bgcolor = shift;

    my $toggle;
    my $disable_toggle = "<font color=red size=-1>Disabled</font>&nbsp;&nbsp;<a href='"
                       . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;mystuff_mysubmissions_for_review=enable") . "'>(enable)</a>";
    my $enable_toggle = "<font color=red size=-1>Enabled</font>&nbsp;&nbsp;<a href='"
                      . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;mystuff_mysubmissions_for_review=disable") . "'>(disable)</a>";

    if( defined $FORM{mystuff_mysubmissions_for_review} && ( lc $FORM{mystuff_mysubmissions_for_review} eq 'enable' ) ) {
        $toggle = $enable_toggle;
        $PARAMS->set_property('mystuff_mysubmissions_for_review',1);
    } elsif ( defined $FORM{mystuff_mysubmissions_for_review} && ( lc $FORM{mystuff_mysubmissions_for_review} eq 'disable' ) ) {
        $toggle = $disable_toggle;
        $PARAMS->set_property('mystuff_mysubmissions_for_review',0);
    } elsif( $PARAMS->{mystuff_mysubmissions_for_review} ) {
        $toggle = $enable_toggle;
    } else {
        $toggle = $disable_toggle;
    }

    my $bgcolor = &$get_bgcolor;
    my $result = <<EOM
        <tr bgcolor = $bgcolor>
            <td><b><i>Submission items I submitted for review</i> Link</b></td>
            <td>
                $toggle
            </td>
            <td><i><font size=-1>This sets whether users can see the 'View submission items I submitted for review' link in the 'My Stuff' area.</i></font></td>
        </tr>
EOM
;
    return $result;
}

sub _process_mystuff_submissions_need_review {
    my $self = shift;
    my $get_bgcolor = shift;

    my $toggle;
    my $disable_toggle = "<font color=red size=-1>Disabled</font>&nbsp;&nbsp;<a href='"
                       . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;mystuff_submissions_need_review=enable") . "'>(enable)</a>";
    my $enable_toggle = "<font color=red size=-1>Enabled</font>&nbsp;&nbsp;<a href='"
                      . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;mystuff_submissions_need_review=disable") . "'>(disable)</a>";

    if( defined $FORM{mystuff_submissions_need_review} && ( lc $FORM{mystuff_submissions_need_review} eq 'enable' ) ) {
        $toggle = $enable_toggle;
        $PARAMS->set_property('mystuff_submissions_need_review',1);
    } elsif ( defined $FORM{mystuff_submissions_need_review} && ( lc $FORM{mystuff_submissions_need_review} eq 'disable' ) ) {
        $toggle = $disable_toggle;
        $PARAMS->set_property('mystuff_submissions_need_review',0);
    } elsif( $PARAMS->{mystuff_submissions_need_review} ) {
        $toggle = $enable_toggle;
    } else {
        $toggle = $disable_toggle;
    }

    my $bgcolor = &$get_bgcolor;
    my $result = <<EOM
        <tr bgcolor = $bgcolor>
            <td><b><i>View submission items I need to review</i> Link</b></td>
            <td>
                $toggle
            </td>
            <td><i><font size=-1>This sets whether users can see the 'View submission items I need to review' link in the 'My Stuff' area.</i></font></td>
        </tr>
EOM
;
    return $result;
}

sub _process_mystuff_mywebsite_toggle {
    my $self = shift;
    my $get_bgcolor = shift;

    my $toggle;
    my $disable_toggle = "<font color=red size=-1>Disabled</font>&nbsp;&nbsp;<a href='"
                       . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;mystuff_mywebsite=enable") . "'>(enable)</a>";
    my $enable_toggle = "<font color=red size=-1>Enabled</font>&nbsp;&nbsp;<a href='"
                      . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;mystuff_mywebsite=disable") . "'>(disable)</a>";

    if( defined $FORM{mystuff_mywebsite} && ( lc $FORM{mystuff_mywebsite} eq 'enable' ) ) {
        $toggle = $enable_toggle;
        $PARAMS->set_property('mystuff_mywebsite',1);
    } elsif ( defined $FORM{mystuff_mywebsite} && ( lc $FORM{mystuff_mywebsite} eq 'disable' ) ) {
        $toggle = $disable_toggle;
        $PARAMS->set_property('mystuff_mywebsite',0);
    } elsif( $PARAMS->{mystuff_mywebsite} ) {
        $toggle = $enable_toggle;
    } else {
        $toggle = $disable_toggle;
    }

    my $bgcolor = &$get_bgcolor;
    my $result = <<EOM
        <tr bgcolor = $bgcolor>
            <td><b><i>View my website</i> Link</b></td>
            <td>
                $toggle
            </td>
            <td><i><font size=-1>This sets whether users can see the 'View my website' link in the 'My Stuff' area.</i></font></td>
        </tr>
EOM
;
    return $result;
}

sub _process_mystuff_mypage_toggle {
    my $self = shift;
    my $get_bgcolor = shift;

    my $toggle;
    my $disable_toggle = "<font color=red size=-1>Disabled</font>&nbsp;&nbsp;<a href='"
                       . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;mystuff_mypage=enable") . "'>(enable)</a>";
    my $enable_toggle = "<font color=red size=-1>Enabled</font>&nbsp;&nbsp;<a href='"
                      . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;mystuff_mypage=disable") . "'>(disable)</a>";

    if( defined $FORM{mystuff_mypage} && ( lc $FORM{mystuff_mypage} eq 'enable' ) ) {
        $toggle = $enable_toggle;
        $PARAMS->set_property('mystuff_mypage',1);
    } elsif ( defined $FORM{mystuff_mypage} && ( lc $FORM{mystuff_mypage} eq 'disable' ) ) {
        $toggle = $disable_toggle;
        $PARAMS->set_property('mystuff_mypage',0);
    } elsif( $PARAMS->{mystuff_mypage} ) {
        $toggle = $enable_toggle;
    } else {
        $toggle = $disable_toggle;
    }

    my $bgcolor = &$get_bgcolor;
    my $result = <<EOM
        <tr bgcolor = $bgcolor>
            <td><b><i>View my page</i> Link</b></td>
            <td>
                $toggle
            </td>
            <td><i><font size=-1>This sets whether users can see the 'View my page' link in the 'My Stuff' area.</i></font></td>
        </tr>
EOM
;
    return $result;
}

sub _process_mystuff_subscriptions_toggle {
    my $self = shift;
    my $get_bgcolor = shift;

    my $toggle;
    my $disable_toggle = "<font color=red size=-1>Disabled</font>&nbsp;&nbsp;<a href='"
                       . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;mystuff_subscriptions=enable") . "'>(enable)</a>";
    my $enable_toggle = "<font color=red size=-1>Enabled</font>&nbsp;&nbsp;<a href='"
                      . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;mystuff_subscriptions=disable") . "'>(disable)</a>";

    if( defined $FORM{mystuff_subscriptions} && ( lc $FORM{mystuff_subscriptions} eq 'enable' ) ) {
        $toggle = $enable_toggle;
        $PARAMS->set_property('mystuff_subscriptions',1);
    } elsif ( defined $FORM{mystuff_subscriptions} && ( lc $FORM{mystuff_subscriptions} eq 'disable' ) ) {
        $toggle = $disable_toggle;
        $PARAMS->set_property('mystuff_subscriptions',0);
    } elsif( $PARAMS->{mystuff_subscriptions} ) {
        $toggle = $enable_toggle;
    } else {
        $toggle = $disable_toggle;
    }

    my $bgcolor = &$get_bgcolor;
    my $result = <<EOM
        <tr bgcolor = $bgcolor>
            <td><b><i>My subscriptions</i> Link</b></td>
            <td>
                $toggle
            </td>
            <td><i><font size=-1>This sets whether users can see the 'My subscriptions' link in the 'My Stuff' area.</i></font></td>
        </tr>
EOM
;
    return $result;
}



sub _process_and_add_accelerator_panel {
    my $self = shift;
    my $get_bgcolor = shift;

    my $toggle;
    my $disable_toggle = "<font color=red size=-1>Disabled</font>&nbsp;&nbsp;<a href='"
                       . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;accelerator_mode=enable") . "'>(enable)</a>";
    my $enable_toggle = "<font color=red size=-1>Enabled</font>&nbsp;&nbsp;<a href='"
                      . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;accelerator_mode=disable") . "'>(disable)</a>";

    if( defined $FORM{accelerator_mode} && ( lc $FORM{accelerator_mode} eq 'enable' ) ) {
        $toggle = $enable_toggle;
        $PARAMS->set_property('accelerator_enabled',1);
    } elsif ( defined $FORM{accelerator_mode} && ( lc $FORM{accelerator_mode} eq 'disable' ) ) {
        $toggle = $disable_toggle;
        $PARAMS->set_property('accelerator_enabled',0);
    } elsif( $PARAMS->{accelerator_enabled} ) {
        $toggle = $enable_toggle;
    } else {
        $toggle = $disable_toggle;
    }

    my $details_link = '';
    if ($PARAMS->{accelerator_enabled}) {
        $details_link = '<br /><a href='
                      . $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_accelerator")
                      . '><font size="-1">View and Edit Accelerator Details</font></a>';
    }

    my $bgcolor = &$get_bgcolor;
    my $result = <<EOM
        <tr bgcolor = $bgcolor>
            <td><b>Site Accelerator</b></td>
            <td>
                $toggle
                $details_link
            </td>
            <td><i><font size=-1>This enables or disables the Site Accelerator, which uses a cache to speed page generation times.</i></font></td>
        </tr>
EOM
;
    return $result;
}

sub www_modify_accelerator {
    my $self = shift;
    my $object_accessor = Gizmo->get_object_accessor();
    my $cache           = $object_accessor->get_cache();

    my $out;
    if ($cache) {
        # process input
        if ($FORM{accelerator_flush}) {
            $cache->flush();
        }
        if ($FORM{accelerator_submit}) {
            $PARAMS->set_property('accelerator_max_size',        ($FORM{accelerator_max_size} || 0));
            $PARAMS->set_property('accelerator_max_element_age', ($FORM{accelerator_max_element_age} || 0));
        }


        # get the most recent status
        my $cache_size      = $cache->size();
        my $max_size        = $PARAMS->{accelerator_max_size} || 0;
        my $max_element_age = $PARAMS->{accelerator_max_element_age} || 0;
        my $start_time      = localtime($cache->get_creation_time());
        my $hits            = $cache->hits();
        my $misses          = $cache->misses();
        my $ratio_msg;

        # calculate hit ratio
        if ($misses != 0) {
            my $ratio = ($hits/($hits+$misses)) * 100;
            $self->debug_msg(3, "Portal::www_modify_accelerator calculated raw ratio of $ratio");
            $ratio_msg = format_number($ratio,1,1) . '%';
        } else {
            $ratio_msg = 'N/A';
        }

        $out = <<EOM
<!-- begin site accelerator status -->
<br /><br />
<b>Properties of current accelerator engine:</b><br />
<table>
<tr valign="baseline">
    <td>Engine ID:</td><td>$cache</td>
</tr><tr valign="baseline">
    <td nowrap="nowrap">Current cache size:</td><td>$cache_size elements</td><td>&nbsp;</td>
</tr><tr valign="baseline">
    <td nowrap="nowrap">Cache started on:</td><td>$start_time</td><td>&nbsp;</td>
</tr><tr valign="baseline">
    <td nowrap="nowrap">Cache hits:</td><td>$hits</td><td>&nbsp;</td>
</tr><tr valign="baseline">
    <td nowrap="nowrap">Cache misses:</td><td>$misses</td><td>&nbsp;</td>
</tr><tr valign="baseline">
    <td nowrap="nowrap">Cache hit ratio:</td><td>$ratio_msg</td><td>&nbsp;</td>
</tr>
</table>

<br /><hr><br />

<!-- begin site accelerator form -->
<form action="index.pl">
<input type="hidden" name="isa" value="$FORM{isa}">
<input type="hidden" name="op" value="$FORM{op}">
<input type="hidden" name="manage" value="$FORM{manage}">

<b>Modify accelerator settings</b>
<table>
<tr valign="baseline">
    <td>Maximum number of elements in cache:</td>
    <td><input type="text" name="accelerator_max_size" value="$max_size" size="10"></td>
</tr><tr valign="baseline">
    <td>Maximum element age (in seconds):</td>
    <td><input type="text" name="accelerator_max_element_age" value="$max_element_age" size="10"></td>
</tr>
<tr valign="baseline"><td nowrap="nowrap">
    <input type=submit name="accelerator_submit" value="Save Changes">
    <input type=submit name="accelerator_flush" value="Flush the Cache">
</td></tr>
</table>
</form>
<!-- end site accelerator form -->

EOM
;


    } else {
        $out = ' (Cache is disabled or unavailable)';
    }
    $self->print_form($out,"View and Edit Accelerator Details");

}


sub _process_disable_allowed_image_urls_check_toggle {
    my $self = shift;
    my $get_bgcolor = shift;
    my $admin_marker = shift;

    # aju -- if param hasn't been defined yet, the toggling won't work. (sigh).
    # so don't even show the info.
    if (!exists($PARAMS->{disable_allowed_image_urls_check})) {
    return "";
    }
    my $toggle;
    my $disable_toggle = "<font color=red size=-1>Disabled</font>&nbsp;&nbsp;<a href='" .
      $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;disable_allowed_image_urls_check=enable") . "'>(enable)</a>";
    my $enable_toggle = "<font color=red size=-1>Enabled</font>&nbsp;&nbsp;<a href='" .
      $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;disable_allowed_image_urls_check=disable") . "'>(disable)</a>";

    if( defined $FORM{disable_allowed_image_urls_check} && ( lc $FORM{disable_allowed_image_urls_check} eq 'enable' ) ) {
        $toggle = $enable_toggle;
        $PARAMS->set_disable_allowed_image_urls_check('0');
    } elsif ( defined $FORM{disable_allowed_image_urls_check} && ( lc $FORM{disable_allowed_image_urls_check} eq 'disable' ) ) {
        $toggle = $disable_toggle;
        $PARAMS->set_disable_allowed_image_urls_check('1');
    } elsif (!($PARAMS->{disable_allowed_image_urls_check}) ) {
        $toggle = $enable_toggle;
    } else {
        $toggle = $disable_toggle;
    }

    my $bgcolor = &$get_bgcolor;

    my $result = "<TR bgcolor = $bgcolor>
                     <td>$admin_marker <b>Source Tag Filtering</b></td>
                        <td>$toggle</td>
                        <td><i><font size=-1>This sets whether or not source tags are filtered.</i></font></td>
                </TR>";

    return $result;

}

my $rateable_module_installed;
sub _process_and_add_rateable_items_toggle {
    my $self = shift;
    my $get_bgcolor = shift;
    my $admin_marker = shift;

    unless( defined $rateable_module_installed ) {

    eval { $rateable_module_installed = require Metadot::Rateable };

    }

    return '' unless( $rateable_module_installed );

    my $items_rateable;
    my $disable_items_rateable = "<font color=red size=-1>Disabled</font>&nbsp;&nbsp;<a href='" .
      $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;rateable_items=enable") . "'>(enable)</a>";
    my $enable_items_rateable = "<font color=red size=-1>Enabled</font>&nbsp;&nbsp;<a href='" .
      $self->make_url("index.pl?manage=Config&amp;isa=Portal&amp;op=modify_site_params&amp;rateable_items=disable") . "'>(disable)</a>";


    if( defined $FORM{rateable_items} && ( lc $FORM{rateable_items} eq 'enable' ) ) {
       $PARAMS->set_items_are_rateable('1');
       $items_rateable = $enable_items_rateable;
    } elsif ( defined $FORM{rateable_items} && ( lc $FORM{rateable_items} eq 'disable' ) ) {
        $PARAMS->set_items_are_rateable('0');
        $items_rateable = $disable_items_rateable;
    } elsif ( $PARAMS->{items_are_rateable} ) {
        $items_rateable = $enable_items_rateable;
    } else {
        $items_rateable = $disable_items_rateable;
    }


    my $bgcolor = &$get_bgcolor;

    my $result = "<TR bgcolor = $bgcolor>
                     <td>$admin_marker <b>Rateable Items</b></td>
                        <td>$items_rateable</td>
                        <td><i><font size=-1>Enabling this option allows Item gizmos to be rated by users</i></font></td>
                </TR>";

    return $result;

}


sub _get_stats_section {
    my $self = shift;
    my $get_bgcolor = shift;
    my $admin_marker = shift;

    my $site_name = $PARAMS->{basename};

    #print STDERR "site_name='$site_name'\n";
    # remove URI scheme identifier
    $site_name =~ s/^\w*:\/\///;
    #print STDERR "site_name='$site_name'\n";
    # remove any path info
    $site_name =~ s/\/.*//;
    #print STDERR "site_name='$site_name'\n";

    my $global_quota_mb = $PARAMS->{global_uploads_quota};
    my $upload_size_allowed_is_unlimited = (!defined $global_quota_mb || $global_quota_mb eq "") ? 1 : 0;

    my ( $used_global_quota_bytes ) = Metadot::UploadsManager->get_global_used_uploads_space_and_num_files();

    # convert to bytes for comparison
    my $global_quota_bytes = $global_quota_mb * 1048576;

    # format for display

    my $used_global_quota_mb = sprintf("%0.0f", $used_global_quota_bytes / 1048576 );

    $global_quota_mb = sprintf( "%0.0f", $global_quota_mb );
    $global_quota_mb = $upload_size_allowed_is_unlimited ? "unlimited" : "$global_quota_mb MB";

    my $number_of_users = $USER->get_number_of_active_users();

    my $max_user_num = $PARAMS->{max_number_of_users} || 'unlimited';

    my $content = "
<b>Site name:</b> $site_name<br />
<b>Disk space:</b> $used_global_quota_mb MB used out of $global_quota_mb MB<br />
<b>Registered users:</b> $number_of_users out of $max_user_num";

    my $bgcolor = &$get_bgcolor;

#      my $result1 = "<TR bgcolor = $bgcolor>
#                       <td>$admin_marker <b>Resource Utilization</b></td>
#                          <td>$content</td>
#                          <td><i><font size=-1>Resource utilization statistics</i></font></td>
#                  </TR>";

    my $result = <<EOF;
  <TR bgcolor = $bgcolor><td colspan='3' align='center'>
  <table width="80%" border="1" cellspacing="0" cellpadding="0">
  <tr>
    <td>
      <table width="100%" border="0" cellspacing="5" cellpadding="5">
        <tr>
          <td>
            <div align="center"><b><font face="Arial, Helvetica, sans-serif" size="-1">Current
              resource utilization of the site</font></b><b><font face="Arial, Helvetica, sans-serif" size="+1">
              <br />
              $site_name</font></b></div>
          </td>
        </tr>
        <tr>
          <td>
            <table width="60%" border="0" cellspacing="0" cellpadding="0" align="center">
              <tr>
                <td><b><font face="Arial, Helvetica, sans-serif" size="-1">$used_global_quota_mb MB
                  </font></b><font face="Arial, Helvetica, sans-serif" size="-1">out
                  of </font><font size="-1"><b><font face=\"Arial, Helvetica, sans-serif\" size=\"-1\">$global_quota_mb </font></b>
                  <font face="Arial, Helvetica, sans-serif">of
                  disk space is used</font></font> </td>
              </tr>
EOF
    if (MetadotConfig->get_is_managed_site() eq 'true') {
    my $images_loc = $PARAMS->{imageslocation};
    $result .=
        "<tr>
                <td><b><font face=\"Arial, Helvetica, sans-serif\" size=\"-1\">$number_of_users </font></b>
                <font face=\"Arial, Helvetica, sans-serif\" size=\"-1\">out
                  of </font><b><font face=\"Arial, Helvetica, sans-serif\" size=\"-1\">$max_user_num </font></b><font face=\"Arial, Helvetica, sans-serif\" size=\"-1\">users have registered to the site</font></td></tr>
              <tr><td>&nbsp;<br /><a target=\"_blank\" href=\"/stats/index.html\"><img src=\"$images_loc/stats.gif\" border=\"0\"></a>
          </td></tr>";
    }
    #if ($PARAMS->{is_event_log_enabled}) {
        my $report_generator = Metadot::SystemApp::EventLog::ReportGeneratorFactory->get_report_generator();
        my $report_generator_isa = $report_generator->is_a();
        $result .= "
        <tr align='center'>
                <td>
                  <br /><a href=\"index.pl?isa=Metadot::SystemApp::EventLog&amp;op=show\">View Event Log</a>
                  <br /><a href=\"index.pl?isa=$report_generator_isa&amp;op=change_reports_year_month_date\">View Portal Usage Statistics</a>
                </td>
        </tr>\n";
    #}
     $result .= "      </table>
          </td>
        </tr>
      </table>
    </td>
  </tr>
  </table>
  </td></tr>";

}


#####################################################
sub show_form_admin {
    my $self = shift;

    $self->debug_msg (4, "Portal::show_form_admin: admin_mode");

    my $dl = $SESSION->debug_level();
    my $debug_status;

    if($dl == 0){
    $debug_status = "No Debug Information";
    }elsif($dl ==1 ){
    $debug_status = "Whisper Debug Information";
    }elsif($dl ==2 ){
    $debug_status = "A little Debug Information";
    }elsif($dl ==3 ){
    $debug_status = "Just Right Debug Information";
    }elsif($dl ==4 ){
    $debug_status = "Send it to me</b></i>";
    }elsif($dl ==5 ){
    $debug_status = "Prepare aspirin</b></i>";
    }

    my $imageslocation = $PARAMS->{'imageslocation'};
    my $version = MetadotConfig->get_version();

    my $bgcolor = '#ffffff';
    my $get_bgcolor = sub { $bgcolor = ( $bgcolor eq '#ffffff' ) ? '#efefef' : '#ffffff' };


    my ($admin_marker, $msg_for_super_admins) = ('', '');

    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ) {

    $admin_marker =  '<font color=red size=-1>*</font>';
    $msg_for_super_admins = "$admin_marker <i><font size=-1>indicates fields available to site administrators.</font></i>";
    }

    my $s = "<center>
           <TABLE BGCOLOR='#FFFFFF' WIDTH=\"715\" CELLSPACING=\"0\" CELLPADDING=\"5\" BORDER=\"0\" valign = top>
                        <tr bgcolor = #bbbbbb>
                                <td colspan = 4 align = center><b>Administration ($version)</b></td>
                        </tr>";

#    if($msg_for_super_admins) {
#
#   $s .= "
#                        <tr align='center' bgcolor = " . &$get_bgcolor . ">
#                                <td colspan='4' align='center'>$msg_for_super_admins</td></tr>";
#
#    }

    $s .= $self->_get_stats_section( $get_bgcolor, $admin_marker );

    $s .= "
                        <tr bgcolor = " . &$get_bgcolor . ">
                                <td>$admin_marker <b>System params</b></td>
                                <TD><a href='".$self->make_url("index.pl?isa=Portal&amp;op=show_params")."'>View<br /><a href='".$self->make_url("index.pl?isa=Portal&amp;op=modify_params")."'>Modify</a></TD>";

    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){
        $s .= "<td><i><font size=-1><b>You must set this at install time.</b> Set domain names, file directories, the mail server, etc.</i></font></td>";
    } else {
        $s .= "<td><i><font size=-1>Set Session duration, user quota, etc.</i></font></td>";
    }
    $s .= "</tr>";

    # remove LDAP enable section from view of unauthorized users
    my $reg_help;
    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){

    $s .= $self->_process_and_add_ldap_toggle( $get_bgcolor );
    $reg_help = 'Configures the Registration subsystem that you have enabled (Default or LDAP)';
    } else {
    $reg_help = 'Configures the Registration subsystem';
    }

    unless( MetadotConfig->get_param_value('user_management_is_external') ) {
    $s .= "<tr bgcolor = ". &$get_bgcolor . ">
                                <td>$admin_marker <b>Registration params</b></td>
                                  <TD><a href='" . $self->make_url("index.pl?isa=Register&amp;op=show_params") . "'>View<br />
                                      <a href='" . $self->make_url("index.pl?isa=Register&amp;op=modify_params") . "'>Modify</a></td>

                                <td><i><font size=-1>$reg_help</i></font></td>
              </TR>";
    }

    # remove Debug and Status sections from view of unauthorized users
    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){

    $s .= "
    <TR bgcolor = " . &$get_bgcolor . ">
                          <TD><b>Debug Mode</b></td>
                        <td>
<font color = red size=-1>Status:  $dl.</font>
<br /><br />
0. <a href='".$self->make_url("index.pl?isa=Session&amp;op=set_debug_level&amp;value=0")."'>No Debug Information</a>
                                        <br />1. <a href='".$self->make_url("index.pl?isa=Session&amp;op=set_debug_level&amp;value=1")."'>Whisper Debug Information</a>
                                        <br />2. <a href='".$self->make_url("index.pl?isa=Session&amp;op=set_debug_level&amp;value=2")."'>A Little Debug Information</a>
                                        <br /><div style=\"white-space: nowrap\">3. <a href='".$self->make_url("index.pl?isa=Session&amp;op=set_debug_level&amp;value=3")."'>Just Right Debug Information</div></a>
                                        <br />4. <a href='".$self->make_url("index.pl?isa=Session&amp;op=set_debug_level&amp;value=4")."'>Send It To Me</a>
                                        <br />5. <a href='".$self->make_url("index.pl?isa=Session&amp;op=set_debug_level&amp;value=5")."'>Prepare Aspirin</a></td>
                        <td><i><font size=-1>This allows you to view debug information from none (level 0) to a lot of intergalactic code (level 5).</i></font></td>
                </tr>\n";
    }

    $s .= "
                     <tr bgcolor = " . &$get_bgcolor . ">
                        <td>$admin_marker <b>Status</b></td>
<td><a href='".$self->make_url("index.pl?isa=Status&amp;op=show_env")."'>View environment variables</a><br />
<a href='".$self->make_url("index.pl?isa=Status&amp;op=show_monitor")."'>View system status</a><br />
<a href='".$self->make_url("index.pl?isa=Status&amp;op=show_versions")."'>View Component Versions</a></td>
                        <td><i><font size=-1>System health: CPU, RAM, HD usage. Network activity and more!</i></font></td>
                </TR>
                      <tr bgcolor = " . &$get_bgcolor . ">
                        <td>$admin_marker <b>Email messages</b></td>
                        <td><a href='".$self->make_url("index.pl?isa=Portal&amp;op=modify_message")."'>Modify</a></td>
                        <td><i><font size=-1>To customize the email messages sent from the system</i></font></td>
                </TR>";

    $s .= $self->_process_and_add_my_website_toggle( $get_bgcolor, $admin_marker );

    $s .= $self->_process_and_add_rateable_items_toggle( $get_bgcolor, $admin_marker );

    $s .= $self->_process_email_friend_toggle( $get_bgcolor, $admin_marker );

    $s .= $self->_process_invitable_toggle( $get_bgcolor, $admin_marker );

    if( $ACCESS_BROKER->is_allowed_to_do('server_setup') ){

    $s .= $self->_process_and_add_recursive_perms_toggle( $get_bgcolor );
    $s .= $self->_process_and_add_accelerator_panel( $get_bgcolor );
    }

    $s .= $self->_process_disable_allowed_image_urls_check_toggle( $get_bgcolor, $admin_marker );

    $s .= "<tr bgcolor = " . &$get_bgcolor . "><td colspan='3'>&nbsp;</td></tr>
           <tr bgcolor = " . &$get_bgcolor . ">
              <td colspan='3'><b><i>My Stuff</i> page options</b></td>
           </tr>";


    ## My stuff links
    $s .= $self->_process_mystuff_metagrams_toggle( $get_bgcolor );
    $s .= $self->_process_mystuff_changeprofile_toggle( $get_bgcolor );
    $s .= $self->_process_mystuff_mygroups_toggle( $get_bgcolor );
    $s .= $self->_process_mystuff_mysubmissions_for_review( $get_bgcolor );
    $s .= $self->_process_mystuff_submissions_need_review( $get_bgcolor );
    $s .= $self->_process_mystuff_mywebsite_toggle( $get_bgcolor );
    $s .= $self->_process_mystuff_mypage_toggle( $get_bgcolor );
    $s .= $self->_process_mystuff_subscriptions_toggle( $get_bgcolor );


    my $extended_config = $self->extended_form_admin( &$get_bgcolor );

    $s .= "$extended_config
              </TABLE><br />$msg_for_super_admins<br /><br /><br /></center>";
    return $s;
}

#
# Called from show_form_admin to extend the admin form (blank here
# because it's meant to be overridden in extensions files.
#
sub extended_form_admin {
    my $self = shift;

    return '';
}


###################################################
sub show_edit_form_headfoot {
    my $self = shift;
    my $title = shift;
    $self->debug_msg(3, "Portal::show_edit_form_headfoot($title)");

    my $toolbgcolor = $PARAMS->{toolbgcolor};
    my $simpleheader = $PARAMS->{simpleheader};
#      my $formbanner = $PARAMS->{formbanner};
    my $newscolbgcolor = $PARAMS->{newscolbgcolor};
    my $newscoltitlebgcolor = $PARAMS->{newscoltitlebgcolor};
    my $maintemplate = $PARAMS->{maintemplate};
    my $subtemplate = $PARAMS->{subtemplate};
    my $utilitytemplate = $PARAMS->{utilitytemplate};
#      my $utility_template2 = $PARAMS->{utility_template2};

    my $mainheader = $PARAMS->{mainheader};
    my $mainfooter = $PARAMS->{mainfooter};
    my $subheader = $PARAMS->{subheader};
    my $subfooter = $PARAMS->{subfooter};
    my $stylesheet = $PARAMS->{stylesheet};
    my $categorytitlebgcolor = $PARAMS->{categorytitlebgcolor};
    my $formbgcolor = $PARAMS->{formbgcolor};
    my $mypageelementboxcolor = $PARAMS->{mypageelementboxcolor};
    my $mypagetitleboxcolor = $PARAMS->{mypagetitleboxcolor};
    my $mypageheader = $PARAMS->{mypageheader};
    my $use_mypagehdr_default = $PARAMS->{use_mypagehdr_default};
    my $mphdr_def_checked = '';
    if ($use_mypagehdr_default) {
    $mphdr_def_checked = 'checked';
    }
    my $globallook = $PARAMS->{globallook};

    my $loginsplash = $PARAMS->{loginsplash};

    # aju: check if themes are 'on'
#     my $themes_on = 0;
#     if ($PARAMS->{template_theme} && $PARAMS->{template_theme} ne 'null') {
#         $themes_on = 1;
#     }
    my $themes_on = $DISPLAY->using_themes();

    # ckim: check if controller is 'on'
    my $controller_on = ($PARAMS->{use_controller} && $PARAMS->{use_controller} ne 'null');

    # ckim: controller uses newlook (style 1) as its backup style
    my $newlook = ($globallook eq "new" && !$controller_on) ? "checked" : "" ;
    #my $newlook = ($globallook eq "new") ? "checked" : "" ;
    my $oldlook = ($globallook eq "old") ? "checked" : "" ;
    my $latestlook = ($globallook eq "latest") ? "checked" : "" ;
    my $usetemplate = "";
    if (($globallook eq "usetemplate") && (!$themes_on && !$controller_on)) {
    $usetemplate = "checked";
    }
    my $themeslook = ($themes_on) ? "checked" : "" ;
    my $controllerlook = ($controller_on) ? 'checked' : '';

    my $s = <<EOM
            <script language="JavaScript">
<!-- Comments to prevent display
            function setRadio( form, controlName, controlValue ) {
                for (var i=0; i<form.length; i++) {
                    if (form.elements[i].name == controlName && form.elements[i].value == controlValue) {
                        form.elements[i].checked = true;
                    }
                }
            }

//-->
            </script>
EOM
;

    $s .= " <H1>$title</h1>\n";

#      $s .=  "<p />Use <i>Themes</i>: <input type=checkbox name='use_themes' value='use'";
#      if ($themes_on) {
#   $s .= " checked >";
#      }
#      else {
#   $s .= " >";
#      }

my   $expand_checked = $PARAMS->{expand_nav_on_style_1} ? 'checked' : '';
my   $expand_themes_checked = $PARAMS->{expand_nav_on_themes} ? 'checked' : '';
     $s .=  "<p /><b>Global Look</b><br />
                <input type=radio name='globallook' value='use_controller' $controllerlook> Enable <i>Skins</i><br />
                <input type=radio name='globallook' value='use_themes' $themeslook> Enable <i>Themes</i><br />
                &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type=checkbox name=\"expand_nav_on_themes\" $expand_themes_checked value=\"1\">Expand top-level categories in navigation bar for Themes layout<br />
                <input type=radio name='globallook' value='new' $newlook> Style 1 (Main categories appear across top and left side navigational bar.)<br />
                &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type=checkbox name=\"expand_nav_on_style_1\" $expand_checked value=\"1\">Expand top-level categories in Style 1 navigation bar<br />
                <input type=radio name='globallook' value='old' $oldlook> Style 2 (Deprecated. Main categories have their own display options.)<br />
                <input type=radio name='globallook' value='latest' $latestlook> Style 3 (Current Category displayed across top.)<br />
                <input type=radio name='globallook' value='usetemplate' $usetemplate> Style 4 (Use the gizmotag-enabled templates defined below)<br /><br />"
.Utility->render_text_or_file_selector(  control_basename =>'maintemplate',
                                         description      =>'<b>Main Template</b> (front page only)',
                                         default          =>$maintemplate,
  )
.Utility->render_text_or_file_selector(  control_basename =>'subtemplate',
                                         description      =>'<b>Sub Template</b> (all pages except front page)',
                                         default          =>$subtemplate,
  )
.Utility->render_text_or_file_selector(  control_basename =>'utilitytemplate',
                                         description      =>'<b>Utility Template</b> (template for search results, discussion pages, etc..)',
                                         default          =>$utilitytemplate,
  )
.Utility->render_text_or_file_selector(  control_basename =>'mainheader',
                                         description      =>'<b>Main Header</b> (front page only)',
                                         default          =>$mainheader,
  )
.Utility->render_text_or_file_selector(  control_basename =>'subheader',
                                         description      =>'<b>Sub Header</b> (all pages except front page)',
                                         default          =>$subheader,
  )
.Utility->render_text_or_file_selector(  control_basename =>'simpleheader',
                                         description      =>'<b>Simple Header</b> (used for form page header information, and optionally the <i>My Page</i> header)',
                                         default          =>$simpleheader,
  )
 .   "<input type=checkbox name=use_mypagehdr_default value=1 $mphdr_def_checked>"
 . " Check to use the Simple Header for the 'My Page' header if the 'My Page' header is not defined.<br />"

 .Utility->render_text_or_file_selector(  control_basename =>'mypageheader',
                                          description      =>"<b>My Page Header</b> (for 'My Page' only)",
                                          default          =>$mypageheader,
   )
.Utility->render_text_or_file_selector(  control_basename =>'mainfooter',
                                         description      =>"<b>Main Footer</b> (front page only)",
                                         default          =>$mainfooter,
  )
.Utility->render_text_or_file_selector(  control_basename =>'subfooter',
                                         description      =>"<b>Sub Footer</b> (all page except front page)",
                                         default          =>$subfooter,
  )
.Utility->render_text_or_file_selector(  control_basename =>'loginsplash',
                                         description      =>"<b>Login Splash Page</b> (only used when require login is turned on)",
                                         default          =>$loginsplash,
  )
.Utility->render_text_or_file_selector(  control_basename =>'stylesheet',
                                         description      =>"<b>Style Sheet</b>",
                                         default          =>$stylesheet,
  )
."
                             <p /><font color=red>Instructions:  Choose a color from the palette.<br />For advanced users:  you may enter \"hexadecimal\" values (0-9, A-F) into the text field for a color selection.</font><br />
                             <HR align=\"left\" Size=3  width=\"600\">
                <p /><table width=\"95%\"><tr><td width=\"55%\"><b>Form Background Color</b></td><td width=\"10\">&nbsp;</td><td bgcolor=\"#ffffff\"><font size=-1>Current Applied Color</font>
                            <table width=\"100%\"><tr><td bgcolor='#$formbgcolor'>&nbsp;</td></tr></table></td><td>$formbgcolor</td>" .
#                              <td><input type=radio name=\"fformbgcolor\" value=\"$formbgcolor\" onclick='javascript:setTxt(\"fformbgcolor\",\"formbgcolor\")' checked></td>
"</tr></table><br />".
                             Components->color_palette("formbgcolor","fformbgcolor")."
                            <input type=text name=\"formbgcolor\" value=\"$formbgcolor\" size=7 maxlength=6>&nbsp;&nbsp;(e.g. EEEEEE)
                            <HR align=\"left\" Size=3  width=\"600\">


                           <p /><table width=\"95%\"><tr><td width=\"55%\"><b>Tools and Search Box Background Color</b></td><td width=\"10\">&nbsp;</td><td bgcolor=\"#ffffff\"><font size=-1>Current Applied Color</font>
                            <table width=\"100%\"><tr><td bgcolor='#$toolbgcolor'>&nbsp;</td></tr></table></td><td>$toolbgcolor</td>" .
#                              <td><input type=radio name=\"ttoolbgcolor\" value=\"$toolbgcolor\" onclick='javascript:setTxt(\"ttoolbgcolor\",\"toolbgcolor\")' checked></td>
"</tr></table><br />".
                           Components->color_palette("toolbgcolor","ttoolbgcolor")."
                           <input type=text name=\"toolbgcolor\" value=\"$toolbgcolor\" size=7 maxlength=6>&nbsp;&nbsp;(e.g. EEEEEE)
                           <HR align=\"left\" Size=3  width=\"600\">

                <p /><table width=\"95%\"><tr><td width=\"55%\"><b>Category Title Background Color</b></td><td width=\"10\">&nbsp;</td><td bgcolor=\"#ffffff\"><font size=-1>Current Applied Color</font>
                            <table width=\"100%\"><tr><td bgcolor='#$categorytitlebgcolor'>&nbsp;</td></tr></table></td><td>$categorytitlebgcolor</td>" .
#                              <td><input type=radio name=\"ccategorytitlebgcolor\" value=\"$categorytitlebgcolor\" onclick='javascript:setTxt(\"ccategorytitlebgcolor\",\"categorytitlebgcolor\")' checked></td>
"</tr></table><br />".
                            Components->color_palette("categorytitlebgcolor","ccategorytitlebgcolor")."
                            <input type=text name=\"categorytitlebgcolor\" value=\"$categorytitlebgcolor\" size=7 maxlength=6>&nbsp;&nbsp;(e.g. 184A94)
                            <HR align=\"left\" Size=3  width=\"600\">

                <p /><table width=\"95%\"><tr><td width=\"55%\"><b>News Column Title Background Color</b></td><td width=\"10\">&nbsp;</td><td bgcolor=\"#ffffff\"><font size=-1>Current Applied Color</font>
                            <table width=\"100%\"><tr><td bgcolor='#$newscoltitlebgcolor'>&nbsp;</td></tr></table></td><td>$newscoltitlebgcolor</td>" .
#                              <td><input type=radio name=\"nnewscoltitlebgcolor\" value=\"$newscoltitlebgcolor\" onclick='javascript:setTxt(\"nnewscoltitlebgcolor\",\"newscoltitlebgcolor\")' checked></td>
"</tr></table><br />".
                            Components->color_palette("newscoltitlebgcolor","nnewscoltitlebgcolor")."
                            <input type=text name=\"newscoltitlebgcolor\" value=\"$newscoltitlebgcolor\" size=7 maxlength=6>&nbsp;&nbsp;(e.g. 184A94)
                             <HR align=\"left\" Size=3  width=\"600\">

                             <p /><table width=\"95%\"><tr><td width=\"55%\"><b>News Column Background Color</b></td><td width=\"10\">&nbsp;</td><td bgcolor=\"#ffffff\"><font size=-1>Current Applied Color</font>
                            <table width=\"100%\"><tr><td bgcolor='#$newscolbgcolor'>&nbsp;</td></tr></table></td><td>$newscolbgcolor</td>" .
#                              <td><input type=radio name=\"nnewscolbgcolor\" value=\"$newscolbgcolor\" onclick='javascript:setTxt(\"nnewscolbgcolor\",\"newscolbgcolor\")' checked></td>
"</tr></table><br />".
                            Components->color_palette("newscolbgcolor","nnewscolbgcolor")."
                            <input type=text name=\"newscolbgcolor\" value=\"$newscolbgcolor\" size=7 maxlength=6>&nbsp;&nbsp;(e.g. EEEEEE)
                             <HR align=\"left\" Size=3  width=\"600\">

                            <p /><table width=\"95%\"><tr><td width=\"55%\"><b>MyPage Frame Background Color</b></td><td width=\"10\">&nbsp;</td><td bgcolor=\"#ffffff\"><font size=-1>Current Applied Color</font>
                            <table width=\"100%\"><tr><td bgcolor='#$mypageelementboxcolor'>&nbsp;</td></tr></table></td><td>$mypageelementboxcolor</td>" .
#                              <td><input type=radio name=\"mmypageelementboxcolor\" value=\"$mypageelementboxcolor\" onclick='javascript:setTxt(\"mmypageelementboxcolor\",\"mypageelementboxcolor\")' checked></td>
"</tr></table><br />".
                            Components->color_palette("mypageelementboxcolor","mmypageelementboxcolor")."
                            <input type=text name=\"mypageelementboxcolor\" value=\"$mypageelementboxcolor\" size=7 maxlength=6>&nbsp;&nbsp;(e.g. CCCCFF)
                            <HR align=\"left\" Size=3  width=\"600\">

                            <p /><table width=\"95%\"><tr><td width=\"55%\"><b>MyPage Title Background Color</b></td><td width=\"10\">&nbsp;</td><td bgcolor=\"#ffffff\"><font size=-1>Current Applied Color</font>
                            <table width=\"100%\"><tr><td bgcolor='#$mypagetitleboxcolor'>&nbsp;</td></tr></table></td><td>$mypagetitleboxcolor</td>" .
#                              <td><input type=radio name=\"mmypagetitleboxcolor\" value=\"$mypagetitleboxcolor\" onclick='javascript:setTxt(\"mmypageboxtitlecolor\",\"mypagetitleboxcolor\")' checked></td>
"</tr></table><br />".
                            Components->color_palette("mypagetitleboxcolor","mmypagetitleboxcolor")."
                            <input type=text name=\"mypagetitleboxcolor\" value=\"$mypagetitleboxcolor\" size=7 maxlength=6>&nbsp;&nbsp;(e.g. CCCCFF)
                            <HR align=\"left\" Size=3  width=\"600\">";
    return $s;
}

###################################################
sub save_basic_styles {
    my $self=shift;

    $self->debug_msg(3, "Portal::save_basic_styles");
    my $current_theme = 'theme2';

    # set theme name to be the name of (currently) the only theme, if this var
    # is not yet defined.
    # this will need a different mechanism once it is possible to change themes.
    unless ($PARAMS->{template_theme}) {
    $PARAMS->set_template_theme($current_theme);
    }

    my $img1_name;
    my $img1_width;
    if ($FORM{image1}) {
        my $url = $FORM{image1};
        my $not_ok = $self->_img_src_url_check($url);
        if ($not_ok) {
            $DISPLAY->print_click_back( "Your image URLs contain characters not allowed for security reasons, such as \& and \?. These characters can only be employed for urls of images contained within the site" );
            exit();
        }
    }

    my $img2_name;
    my $img2_width;
    if($FORM{image2}) {
        my $url = $FORM{image2};
        my $not_ok = $self->_img_src_url_check($url);
        if ($not_ok) {

            $DISPLAY->print_click_back( "Your image URLs contain characters not allowed for security reasons, such as \& and \?. These characters can only be employed for urls of images contained within the site" );
            exit();

        }
    }

    $PARAMS->set_header1_logo1( $FORM{image1} || 'null' );
    $PARAMS->set_header1_logo2( $FORM{image2} || 'null' );

    $PARAMS->set_theme_property('header1_logo1_href', $FORM{header1_logo1_href} || '' );
    $PARAMS->set_theme_property('header1_logo2_href', $FORM{header1_logo2_href} || '' );
    $PARAMS->set_theme_property('header1_text1', $FORM{header1_text1} || '');

    # these are also used by the controller
    $PARAMS->set_property('header1_logo1_href', $FORM{header1_logo1_href} || '' );
    $PARAMS->set_property('header1_logo2_href', $FORM{header1_logo2_href} || '' );
    $PARAMS->set_property('header1_text1', $FORM{header1_text1} || '');
    $PARAMS->set_property('header1_text2', $FORM{header1_text2} || '');

    if ($FORM{'upload_image_button'} || $FORM{'upload_image_button.x'}) {
        if($FORM{banner_image_file}) {
            my $result = $self->_upload_public_file( {
                            isa => 'Portal',
                            id => 1,
                            field_name => 'banner_image_file',
                            field_desc => 'Banner Image',
                            is_required => 0,
                            type => Metadot::UploadsManager::UPLOAD_TYPE_PUBLIC(),
                            allow_multiple => 1,
                            } );

                if( $result ) {
                    $DISPLAY->print_click_back( $result );
                    exit;
            } else {
                return $self->www_basic_styles;
            }
        } else {
            $DISPLAY->print_click_back('You have attempted to upload an image, but you did not provide an image to upload.');
            exit;
        }
    }

    if (defined($FORM{'delete_image'}) || defined($FORM{'delete_image.x'})) {
        # get the filename without the path
        my $file_name = $self->get_filename_from_path($FORM{image_selector});
        if ($file_name) {
            my $field_index = $self->get_field_index(
                isa => 'Portal',
                id => 1,
                field_name  => 'banner_image_file',
                file_name   => $file_name,
            );
            if (defined $field_index) {
                #print STDERR "Portal deleting image '$file_name' with index '$field_index'\n";
                $self->_delete_upload(
                    isa => 'Portal',
                    id => 1,
                    field_name  => 'banner_image_file',
                    field_index => $field_index,
                    write_dir   => $self->get_write_dir_namespace(),
                );

                # check to make sure logo settings are reset if needed

                if ($PARAMS->{header1_logo1} eq $FORM{image_selector}) {
                    $PARAMS->set_header1_logo1('null');
                    $FORM{image1}='';
                }
                if ($PARAMS->{header1_logo2} eq $FORM{image_selector}) {
                    $PARAMS->set_header1_logo2('null');
                    $FORM{image1}='';
                }
            }
        }
        return $self->www_basic_styles;
    }

    if ($FORM{'upload_skin_button'}) {
        if($FORM{skin_file}) {
            if ($ACCESS_BROKER->is_allowed_to_do('admin_tasks')) {
                my $result = $self->_upload_skin_file($FORM{skin_file});
                if( $result ) {
                    $DISPLAY->print_click_back( $result );
                    exit;
                } else {
                    return $self->www_basic_styles;
                }
            } else {
                $DISPLAY->print_click_back('You do not have permission to upload a skin to this site.');
                exit;
            }
        } else {
            $DISPLAY->print_click_back('You have attempted to upload a skin, but you did not provide a file to upload.');
            exit;
        }
        return $self->www_basic_styles;
    }

    if ($FORM{'download_skin_button'}) {
        return $self->_download_skin_file($FORM{skin_selection});
    }

    if (($FORM{'delete_skin_button'}) && $ACCESS_BROKER->is_allowed_to_do('admin_tasks')) {
        $self->_delete_skin($FORM{skin_selection});
        return $self->www_basic_styles;
    }

    if ($FORM{'edit_skin_info_button'}) {
        $self->md_redirect($self->make_url("index.pl?isa=Portal&op=edit_skin_info&skin_id=$FORM{skin_selection}&refererlink=".$CGI->escape($self->get_refererlink)));
    }

    if ($FORM{'edit_skin_file_button'}) {
        $self->md_redirect($self->make_url("index.pl?isa=Portal&op=edit_skin_files&skin_id=$FORM{skin_selection}&refererlink=".$CGI->escape($self->get_refererlink)));
    }

    if ($FORM{'create_skin_button'} || $FORM{'create_skin_confirm'}) {
        $self->md_redirect($self->make_url("index.pl?isa=Portal&op=create_skin&refererlink=".$CGI->escape($self->get_refererlink)));
    }

    # compute image dimensions
    my $url = $PARAMS->{header1_logo1};
    if (!($url =~ /^http/)) {
    #       print STDERR "env: ", Dumper(\%ENV), "\n";
    $url = $PARAMS->{uri_scheme} . "://" . $ENV{HTTP_HOST} . "$url";
    #       print STDERR "in save_basic_styles, got url $url\n";
    }

#   # temporarily disabling this feature because of some quirky behavior
#     my $response = Utility->fetch_url($url);
#     my $response_content = $response->content;
#     # get the size from the contents of the fetch
#     my $img1_size = Utility->get_image_size_string($response_content);
#     $img1_width = $img1_size->{width};

    $url = $PARAMS->{header1_logo2};
    if (!($url =~ /^http/)) {
    #       print STDERR "env: ", Dumper(\%ENV), "\n";
    $url = $PARAMS->{uri_scheme} . "://" . $ENV{HTTP_HOST} . "$url";
    #       print STDERR "in save_basic_styles, got url $url\n";
    }

#     $response = Utility->fetch_url($url);
#     $response_content = $response->content;
#     # get the size from the contents of the fetch
#     my $img2_size = Utility->get_image_size_string($response_content);
#     $img2_width = $img2_size->{width};


    #$PARAMS->set_header1_logo1_href($FORM{});
    #$PARAMS->set_header1_logo1_alt($FORM{logo_alt});
    #$PARAMS->set_header1_logo2_alt($FORM{image_alt});
    $PARAMS->set_header1_color($FORM{color1});
    $PARAMS->set_leftnav1_color($FORM{color3});
    $PARAMS->set_nav_text_color($FORM{color4});
    $PARAMS->set_title_text_color($FORM{color2});
#    $PARAMS->set_header1_logo1_wdth($img1_width);
#    $PARAMS->set_header1_logo2_wdth($img2_width);
    $PARAMS->set_theme_mypage_color($FORM{color1});

    if ($FORM{submit} eq 'Save' || $FORM{apply_skin}) {
        # save skin selection
        $PARAMS->set_property('skin_selection', $FORM{skin_selection});
    }

    $PARAMS->set_property('use_skin_custom_header_footer', $FORM{use_skin_custom_header_footer});
    $PARAMS->set_property('skin_custom_header', $FORM{skin_custom_header});
    $PARAMS->set_property('skin_custom_footer', $FORM{skin_custom_footer});
}

sub save_headfoot {
    my $self = shift;
    $self->debug_msg(3, "Portal::save_headfoot()");

#      my $theme_name = 'theme1';
    my $theme_name = 'theme2';  # there is only one for now, no means of selecting any other.

    $PARAMS->set_newscolbgcolor($FORM{newscolbgcolor});
    $PARAMS->set_newscoltitlebgcolor($FORM{newscoltitlebgcolor});
    $PARAMS->set_formbgcolor($FORM{formbgcolor});
    $PARAMS->set_categorytitlebgcolor($FORM{categorytitlebgcolor});
    $PARAMS->set_mypageelementboxcolor($FORM{mypageelementboxcolor});
    $PARAMS->set_mypagetitleboxcolor($FORM{mypagetitleboxcolor});
    $PARAMS->set_maintemplate(Utility->input_text_or_file_selector('maintemplate'));
    $PARAMS->set_subtemplate(Utility->input_text_or_file_selector('subtemplate'));
    $PARAMS->set_utilitytemplate(Utility->input_text_or_file_selector('utilitytemplate'));
#      $PARAMS->{utility_template2}       = $FORM{utility_template2};

    $PARAMS->set_mainheader(Utility->input_text_or_file_selector('mainheader'));
    $PARAMS->set_mainfooter(Utility->input_text_or_file_selector('mainfooter'));
    $PARAMS->set_subheader(Utility->input_text_or_file_selector('subheader'));
    $PARAMS->set_subfooter(Utility->input_text_or_file_selector('subfooter'));
    $PARAMS->set_simpleheader(Utility->input_text_or_file_selector('simpleheader'));
#      $PARAMS->{formbanner}            = $FORM{formbanner};
    $PARAMS->set_mypageheader(Utility->input_text_or_file_selector('mypageheader'));
    #print STDERR "the value of use_mypagehdr_default is: " .
    #   $FORM{use_mypagehdr_default} . "\n";
    $PARAMS->set_use_mypagehdr_default($FORM{use_mypagehdr_default});
    $PARAMS->set_toolbgcolor($FORM{toolbgcolor});
    $PARAMS->set_stylesheet(Utility->input_text_or_file_selector('stylesheet'));


    # opting for the controller overrides all other style decisions
    if ($FORM{globallook} eq "use_controller") {
        $PARAMS->set_property('use_controller', 1);
        $PARAMS->set_globallook('new');
        $PARAMS->set_template_theme('null');
    } else {
        $PARAMS->set_property('use_controller', 0);

        if ($FORM{globallook} eq "use_themes") {
        $PARAMS->set_globallook('usetemplate');
        $PARAMS->set_template_theme($theme_name);
        }
        else {
        $PARAMS->set_template_theme('null');
        $PARAMS->set_globallook($FORM{globallook});
        }
        $PARAMS->set_loginsplash(Utility->input_text_or_file_selector('loginsplash'));

        $PARAMS->set_property('expand_nav_on_style_1', $FORM{expand_nav_on_style_1});
        $PARAMS->set_property('expand_nav_on_themes', $FORM{expand_nav_on_themes});
    }
}


###################################################
#########The back end functions to display the
#########group manager form
###################################################


###################################################
###################################################
sub http_header_sent {
    my $s = shift;
#    if (@_) { $http_header_sent = shift }
    return $HTTP_HEADER_SENT;
}

###################################################
sub http_header_reset {
    my $s = shift;
    $HTTP_HEADER_SENT = 0;
}

###################################
sub _initialize {
    my $self = shift;
    return $self->get_implementation()->initialize($self, @_);
}

sub get_display_object {
    my $self = shift;
    my $class = shift || MetadotConfig->get_param_value('display_type');
    my $result;

    # determine the default display class, if one is not specified
    unless ($class) {
        if ($PARAMS->{use_controller} && !Metadot::MetadotController->is_disabled()) {
            # TransitionalDisplay is used to migrate from old style UI to the new template-driven style
            $class = 'Metadot::Utils::TransitionalDisplay';
        } else {
            $class ='Metadot::Utils::Display';
        }
    }

    if (defined $class) {
        no strict 'subs';
        eval("require($class)");
        if ($@) {
            die ("Metadotclass->get_implementation could not load '$class'; errors: $@\n");
        }
    }
    $result = $class->new();
    return $result;
}

###################################################################################
# encrypts the password based on portal configuration
sub encrypt_password_if_necessary {
    my $self     = shift;
    my $password = shift;

    if ($PARAMS->{encryption_done} eq 'yes') {
        $password = $self->encrypt_password($password);
    }

    return $password;
}


sub encrypt_password{
    my $self = shift;
    my $password = shift;

    $password = Digest::MD5::md5_base64($password);

    return $password;

}
#################################################################################

# this function has been moved to display
sub show_admin_button {
    my $self = shift;
    return $DISPLAY->show_admin_button($self, @_);
}


########################################
sub show_bottom {
    my $c = shift; #self
    my $width= "100%";
    my $id= $c->{id};

    #edit button
    my $s = "<center>". $c->show_admin_button() . "</center>\n";
     #footer
     $s .= $c->show_footer($id);

     $s .= $c->close_html;
    return $s;
}

#####################################################
sub show_footer {
    my $self = shift;
    return $DISPLAY->show_footer(@_);
}

#####################################################
# vj -- use 'implementations' for this method to allow it to be
# specialized acc. to the config file.
sub show_welcome {
    my $self = shift;

    my $impl = $self->get_implementation('PortalImplementation','Metadot::Implementations::Portal::Default');
    return $impl->show_welcome($self, @_);
}

sub pending_messages {
    my $self = shift;
    my $disc;
    my $message = "";

    $DISPLAY->debug_msg(4, "in pending_messages()");

    my $default_status = Message->_get_default_status();
    my $m = DBAccess->sqlSelectMany("distinct iid","message","status != '$default_status' and isa='Message'");

    while (my ($iid) = $m->fetchrow()) {
        $disc = Discussion->restore($iid);
        $DISPLAY->debug_msg(5, "in pending_messages(), I have disc iid $iid, object ".($disc || '[undefined]'));

        if (defined($disc) && $disc->is_allowed_to_do('moderate',$USER)) {
            $message = "<br /><font class=\"welcomesub\"><b>You have " .
                       "<a href =\"".$self->make_url("index.pl?iid=0&amp;isa=Portal&amp;op=show_pending_messages")."\">messages</a> pending approval.</b></font>\n";
            last;
        }
    }

    return $message;
}

sub www_show_pending_messages {
    my $self = shift;
    $self->debug_msg(3, "Portal::www_show_pending_messages()");

    $self->print_http_header;

    my $disc;
    my $iid;
    my $show_content = $FORM{content} || 0;
    my $s = "<p /><br />";

    if ($show_content) {
        $s .= "<a href=".$self->make_url("index.pl?iid=0&amp;isa=Portal&amp;op=show_pending_messages&amp;content=0").">Hide Content</a><br /><br />";
    } else {
        $s .= "<a href=".$self->make_url("index.pl?iid=0&amp;isa=Portal&amp;op=show_pending_messages&amp;content=1").">Show Content</a><br /><br />";
    }

    $s .= "
<table border=\"0\" width=\"100%\">
  <tr class='discussionheader' bgcolor=\"CCCCCC\">
    <td width=\"50%\"><font class='discussionheader'>Subject</font></td>
    <td width=\"25%\"><font class='discussionheader'>Author</font></td>
    <td width=\"25%\"><font class='discussionheader'>Date</font></td>
  </tr>
";

    my $default_status = Message->_get_default_status();
    my $m = DBAccess->sqlSelectMany("mid, name, authorname, userid, creationdate, status, iid, description","message","status != '$default_status' and isa='Message'","order by creationdate");

    my $rowcount = 0;
    while (my @row = $m->fetchrow()) {
        $iid = $row[6];
        $disc = Discussion->restore($iid);
        if ($disc->is_allowed_to_do('moderate',$USER)) {
            $s .= $disc->_show_one_row(\@row);
        }
        $rowcount++;
    }

    $s .= "<tr><td>No messages pending approval.</td></tr>" unless ($rowcount);
    $s .= "</table><br /><p />\n";

    my $look = $PARAMS->{globallook};
    if ($look eq "usetemplate") {
        print  $DISPLAY->simple_expand(0, $self, $self->show_template(-1), $s);
    } else {
        # this is compatible with the new template mode
        $self->print_portal($s,"Messages pending approval",0);
    }
}


sub show_users_online {
    my $self = shift;
    my $s = "";
    my $members_online = $SESSION->get_members_online;
    my $guests_online = $SESSION->get_guests_online;
    my $members_phrase = "$members_online member";
    unless ($members_online ==1 ) {
        $members_phrase = $members_phrase . "s";
    }
    my $guests_phrase = "$guests_online guest";
    unless ($guests_online == 1) {
        $guests_phrase = $guests_phrase . "s";
    }


    $s .= "<table cellpadding = 0 cellspacing = 0 border = 0>\n";
    $s .= "<tr><td class = onlinemonitorb><div style=\"white-space: nowrap\">Users online:</div></td></tr>";
#    $s .= "<tr><td class = newsitem><div style=\"white-space: nowrap\">$guests_phrase</div></td></tr>";
    $s .= "<tr><td class = onlinemonitor><div style=\"white-space: nowrap\">$members_phrase</div></td></tr>";
    $s .= "</table>";
    $s .= "<br />";
    return $s;
}

#####################################################
### Search Tool functions :
###   show_search
###   show_search_results
###   _format_search_result
#####################################################

sub show_search {
    my $self = shift;
    my $s="";
    my $iid = $self->id();
    my $img = shift;
    $self->debug_msg(3, "Portal::show_search()");

    my $default = $FORM{filter} || "Search";
    my $default_search_choice = $FORM{search_choice} || "site";

    $s  = "\n\n<table cellpadding=\"0\" cellspacing=\"2\" border=\"0\">\n";
    $s .= "<form name='searchform' action='index.pl' method='post'>\n";
    $s .= "<tr valign=\"middle\"><td>\n";
    $s .= "<input type=text onfocus=\"searchform.filter.value=''\" size='10' name='filter' value='$default'></td>\n";

    if ($USER->can_show_user_directory()) {
        $s .= "<td><select name=\"search_choice\">\n";
        $s .= "<option value=\"site\"" . ($default_search_choice eq 'site' ? 'selected' : '') . ">site\n";
        $s .= "<option value=\"user_directory\"" . ($default_search_choice eq 'user_directory' ? 'selected' : '') . ">users\n";
        $s .= "</select></td>\n";
    }

    # aju -- allow img to be used as 'submit' button
    if ($img) {
    $s .= "<td><input type=image src=\"$img\" border=0 align=middle value='Go' class=button>\n";
    }
    else {
    $s .= "<td><input type=submit value='Go' class=button>\n";
    }
    $s .= "<input type=hidden name=op value=search>\n";
    $s .= "<input type=hidden name=isa value=Portal>\n";
    $s .= "<input type=hidden name=iid value=$iid></td></tr>\n";
    $s .= "</form></table>\n\n";

    return "$s";
}

###########################
sub dispatch_search {
    my $self = shift;
    my $gizmo_name="";
    my $str="";
    my @gizmo_list=();

    # XXX [claudio] $maxitem is not used
    # CKIM: it is used now
    my($thingtosearch, $startfrom, $maxitem)=@_;

    if ($thingtosearch eq "") {
        $str = "<b>&nbsp; Oops, nothing to search!<\/B>";
        return $str;
    } else {
        if (defined($FORM{type})) {
            @gizmo_list=$FORM{type};
        }
        else {
            # standard gizmos
            @gizmo_list = Gizmo->filtered_gizmos();

            # gizmo3000's
            my $registered_modules = Metadot::Gizmo3000::ClassRegistry->get_instance()->searchable_modules();
            foreach my $descriptor (values %$registered_modules) {
                unless ($descriptor->is_dedicated_child()) {
                    push @gizmo_list, $descriptor->class_name();
                }
            }
        }

        my @ordered_list = $self->order_gizmo_search_list(\@gizmo_list);

        foreach $gizmo_name (@ordered_list){
            $DISPLAY->debug_msg(4, "Dispatching search results to $gizmo_name");
            $str .= $gizmo_name->show_search_results(@_);
        }
        if ($str ne "" && $str =~ /\S/) {
            return $str;
        }
        else {
            $str= "&nbsp; No results matched your query";
            return $str;
        }
    }
}


sub order_gizmo_search_list {
    my $self = shift;
    my @list = @{ shift || []};
    my @result = ();

    # first sort the list alphabetically by gizmo name
    @list = sort { $a->get_gizmo_name() cmp $b->get_gizmo_name()  } @list;

    # move Category to the beginning of list
    my $category_name = 'Category';
    foreach my $gizmo_name (@list) {
        if ($gizmo_name eq $category_name) {
            unshift @result, $gizmo_name;
        } else {
            push @result, $gizmo_name;
        }
    }

    if (wantarray) {
        return @result;
    } else {
        return \@result;
    }
}


#####################################################

sub show_title {
    my $self = shift;
    return $DISPLAY->show_title(@_);
}

#####################################################
sub show_path{
  my $self = shift;
  my $id = shift;
  $self->debug_msg(3, "Portal::show_path($id)");

  my $is_admin = 0;

  if ( $ACCESS_BROKER->is_allowed_to_do('admin_tasks') ) {

      $is_admin = 1;
  }

  $self->debug_msg(4, "Portal::show_path   -- is_admin = $is_admin id = $id\n");
  if ($is_admin && (!defined $id || $id == 0)) {
      $self->debug_msg(4, "Portal::show_path -- executing build_edit_path()");
      return $self->build_edit_path();
  } elsif (!defined $id) {
      return "";
  }

  my $userpath = "";
  $userpath = $self->build_path($id);

  $self->debug_msg(5, "Portal::show_path: $userpath");
  return $userpath;
}
#####################################################
sub build_edit_path {
    my $self = shift;

    $self->debug_msg(3, "Portal::build_edit_path()");

    if ($FORM{isa} eq "Portal") {
    if ($FORM{op} eq "show_channels") {
        # we are in an edit channel listing
        my $retval = "<a href=".$self->make_url("index.pl?id=0").">Home</a> &gt ";
        $retval .= "<a href=".$self->make_url("index.pl?manage=Channel&amp;isa=Portal&amp;op=modify_site_params").">Channels List</a>";
        return $retval;
    } elsif ($FORM{op} eq "modify_site_params") {
        my $retval = "<a href=".$self->make_url("index.pl?id=0").">Home</a> &gt ";
        $retval .= "<b>Channels List</b>";
        return $retval;
    }

    }
}


#####################################################
sub build_path{
  my $self = shift;
  my $id = shift;
  my $nbcalls = shift || 0;
  my ($str, $name, $parent);

  $self->debug_msg(4, "Portal::build_path()");

  if($nbcalls > 500){
    #looks like we are in an infinite loop
    confess "Portal.pm:: Hey man, I'm looping like crazzy here!! (build_path($nbcalls calls). I think we are in an infinite loop, tell your system admin about this!!!";
  }

  $id = undef unless (Utility->is_valid_integer($id));    # secu_200401
  return "" if (!defined $id);

  if ($id>0) {
    my ($name, $parent) = DBAccess->sqlSelect("name, parentiid", "instance", "iid=$id");
    my $path = $self->build_path($parent, $nbcalls++);
    $path .= " > <a href=".$self->make_url("index.pl?id=$id").">$name</a>\n";
    $self->debug_msg(5, "Portal::build_path: $path");
    return $path;
  } else {
    return "<a href=".$self->make_url("index.pl?id=0").">Home</a>\n";
  }
}

####################################################
sub show_portal_panel {
    my $self = shift;
    my $iid = shift || 0;
    my $isa = shift || $self->{is_a} || ref($self) || 'Category';
    my $tools = "";
    #my @maplist;

    $self->debug_msg(4, "Portal::show_portal_panel($iid)");
#      print STDERR "in show_portal_panel\n";
    my $str = "";

    my $toolbgcolor;
    if (Metadot::Utils::Theme->use_theme_info()) {
        $toolbgcolor = Metadot::Utils::Theme->get_theme_default_bg();
    } else {
        $toolbgcolor = $PARAMS->{'toolbgcolor'};
    }


    #my $editb = $self->show_admin_button(1);
    my $editb = '';

    # the following conditions must be true for portal panel to appear
    # - (user is NOT admin) OR (session is in edit mode) -- first case no longer applies
    # - user is logged in
    #if ($USER->is_logged_in() && (!$USER->is_admin() || $SESSION->admin_mode()) )
    if ($USER->is_logged_in() && ($SESSION->admin_mode())) {
        $editb = $self->show_admin_button(1);

        my $add_child = '';
        my $manage = '';

        my $bg_image = $PARAMS->{'imageslocation'}.'/obj/webbuilder.gif';
        my $page_button = '';
        my $separator_button = '';
        my $project_button = '';
        my $spacer = Utility->get_spacer(5,1);

        if ($SESSION->admin_mode()) {
            my $align = "\"middle\"";
            ### Warning: we can add only things into a category!
            ### Check if the iid are in is a category
            # aju 04-22-02  move this test up here to enclose the new buttons
            # as well as the gizmo pulldown.
            if ((uc(ref($self)) eq "CATEGORY") || (uc($self->{is_a}) eq "CATEGORY") || $self->isa('Category')) {
                $page_button = "<a href=\"".$self->make_url("index.pl?iid=$iid&amp;isa=$isa&amp;op=add_child&amp;child_type=Category")."\">"
                . "<img src=\"".$PARAMS->{'imageslocation'}."/obj/page.gif\" border=\"0\" align=$align alt=\"New Page\"></a>";

                $separator_button = "<a href=\"".$self->make_url("index.pl?iid=$iid&amp;isa=$isa&amp;op=add_child&amp;child_type=Separator")."\">"
                . "<img src=\"".$PARAMS->{'imageslocation'}."/obj/separator.gif\" border=\"0\" align=$align alt=\"Page Section\"></a>";

                # suppress the project button if the Project gizmo is unavailable
                if (Gizmo->get_found_modules()->{'Project'}) {
                $project_button = "<a href=\"".$self->make_url("index.pl?iid=$iid&amp;isa=$isa&amp;op=add_child&amp;child_type=Project ")."\">"
                . "<img src=\"".$PARAMS->{'imageslocation'}."/obj/project.gif\" border=\"0\" align=$align alt=\"New Project\"></a>";
             }

            ### Warning: we can add only things into a category!
            ### Check if the iid are in is a category
            #   if ((uc(ref($self)) eq "CATEGORY") || (uc($self->{is_a}) eq "CATEGORY") || $self->isa('Category'))
            if($self->is_allowed_to_do('add_child', $USER)) {
                $add_child .= $spacer.$self->generate_add_child_selection_list();
            }
        }
 #      print STDERR "in show_portal_panel -- creating manage initial menu\n";
        $manage = $self->generate_manage_selection_list($iid);
        }
        my $js_code = $self->generate_submit_me_script();

    my $hidden = $self->make_hidden();

    if ($editb) {
        $str = <<EOM
$js_code

<table width="100%" border="0" cellspacing="0" cellpadding="0" background="$bg_image" height="30">
<form action="index.pl" name="portal_panel">
  <input type=hidden name="isa">
  <input type=hidden name="iid">
  <input type=hidden name="op">
  $hidden
  <tr valign='middle'>
    <td align='right' width="100%" valign='middle' nowrap="nowrap">
        &nbsp;
        $page_button
        $separator_button
        $project_button
        $add_child
        $manage
        $editb
    <td>
  </tr>
</form>
</table>

EOM
;
        }
    }

    return $str;
}


# this method is now implemented by its children
sub generate_add_child_selection_list {
    my $self = shift;

    return undef;
}


# this method has been moved to Display
sub generate_manage_selection_list {
    my $self = shift;
    my $iid = shift || 0;

    return $DISPLAY->generate_manage_selection_list($iid);
}


# this method has been moved to Display
sub generate_submit_me_script {
    my $self = shift;

    return $DISPLAY->generate_submit_me_script();
}


# CKIM: DEPRECATED this method. Use GizmoContainer->get_add_child_list

# gets only the gizmos that are
# - public
# - shown in add_child_list
sub _get_add_child_list {
    my $self = shift;
    my @result;
    my @temp = Gizmo->filtered_gizmos();
    for my $temp (@temp) {
        if ($temp->is_visible_in_add_child_lists()) {
            push @result, $temp;
        }
    }
    return @result;
}

#####################################################
sub show_drilldown_menu {
    my $self = shift;
    my $iid = shift;
    my $name = "";
    my $id = "";
    my $isa = '';
    #  my $content = "Location: ";
    my $content = "";

    my $gizmo = Gizmo->restore( $iid );

    my @parent_list;
    #CKIM: always use get_visible_parents_list
    @parent_list = $self->get_visible_parents_list();

    # if I'm not in the list, add myself
    if (defined $parent_list[$#parent_list] && $parent_list[$#parent_list]->{id} != $self->id()) {
        push @parent_list, { isa=>$self->get_is_a(), name=>$self->get_name(), id=>$self->id()};
    }


    my $entry = {};
    for (my $x=0; $parent_list[$x]; $x++){
        $entry = $parent_list[$x];
        #($id, $name, $isa) = split(/:/,$parent_list[$x]);
        $id = $entry->{id};
        $name = $entry->{name};
        $isa = $entry->{isa};
        my @nameparts = split(/ /, $name);

        # changed to provide one more word before truncating
        if (($#nameparts + 1) > 3){
            $name = $nameparts[0]." ".$nameparts[1]." ".$nameparts[2]."...";
        }

        my $link = $self->make_url("index.pl?id=$id");

        if (($#parent_list) == $x){
            #      $content .= "<a href='$link'>$name</a>";
            unless( $iid == 0 ) {
            $content .= "<span style=\"white-space: nowrap\"><b>$name</b></span>&nbsp;";
            }
        } else {

            if( ( $id == 0 ) && ( $iid != 0 ) ) {

    #       $content .= "<a href='$link'><img align=\"absmiddle\" src=\"" . $PARAMS->{'imageslocation'} . "/obj/home_icon.gif\" border="0" alt=\"Home\"></a>&nbsp;&gt;&nbsp;";
            # why was this commented out?  put BACK in on request, minus the graphic.
            $content .= "<a href='$link'>Home</a>&nbsp;&gt;&nbsp;";

            } else {
            $content .= "<a href='$link'><span style=\"white-space: nowrap\">$name</span></a>&nbsp;&gt;&nbsp;";
            }
        }
    }

    $isa ||= 'Category';
    $id  ||= '0';

    # CKIM: removed this for Style 6 because this is redundant.
    # CKIM: changed my mind; this is not redundant. Removing the other one.
    #if (!$PARAMS->{use_controller}) {
        $content .= "<span style=\"white-space: nowrap\">". $isa->restore($id)->get_buttons() . "</span>";
    #}
#    $content .= "</TR></TABLE>";

    return $content;
}

#####################################################
sub show_panel {
     my $self = shift ;
     my $id   = shift || $self->id();
     my $isa  = shift || $self->is_a();

     # 10-16-02 avoid infinite recursion...
     # CKIM: any time the code gets here, we should show the home's panel
     #if ($isa eq 'Portal' && ) {
     #    $isa = 'Gizmo';
     #}
     my $cat = Gizmo->new(0);
     return $cat->show_panel($id,$isa);
}

#####################################################
sub show_add_channel_panel {
  my $self = shift ;
  $self->debug_msg(4, "Portal::show_add_channel_panel()");

  if (!$SESSION->admin_mode){ return "";}

  my $str = "<FONT COLOR='#cc6633'><b>
                [ <a href='".$self->make_url("index.pl?isa=Channel&amp;op=add&amp;type=External&amp;parent_id=0")."'>Add a Channel</a> ]
        </b></FONT>";

  return $str;
}
#####################################################
sub show_header {
    my $self = shift;
    return $DISPLAY->show_header(@_);
}

#####################################################

# return the appropriate template (this returns a SITE-wide template
# for this type of page).
# This method will be called if there is
# not an individual or class template associated with the object)
sub get_template {
    my ($self, $iid) = @_;
    my $str;

#     print STDERR "in portal::get_template\n";
    # detect whether or not a 'theme' is in use.
    my $theme = $PARAMS->{'template_theme'};
    my $using_theme;
    if ($theme && $theme ne 'null') {
    $using_theme = 1;
    }
    else {
    $using_theme = 0;
    }

    # find which template to use.  Assumes that the params hash has been instantiated with the
    # appropriate theme template given the theme name. (new caching mech. will be put in
    # place to support this).
    if($iid > 0){
    if ($using_theme) {
        $str = $PARAMS->{theme_subtemplate};
    }
    else {
        $str = $PARAMS->{subtemplate};
    }
    }
    elsif ($iid == 0) {
    if ($using_theme) {
        $str = $PARAMS->{theme_maintemplate};
    }
    else {
        $str = $PARAMS->{maintemplate};
    }
#DG
    }
    elsif ($iid == -1){
    # used for search results and others
    if ($using_theme) {
        $str = $PARAMS->{theme_utilitytemplate};
    }
    else {
        $str = $PARAMS->{utilitytemplate};
    }
    }
    # aju -- reverting this back to use utilitytemplate
    elsif ($iid == -2){
    # used for displaying discussion messages (and maybe other stuff in the future)
    # -- doesn't have left and right navbar info
    if ($using_theme) {
        $str = $PARAMS->{theme_utilitytemplate};
    }
    else {
        $str = $PARAMS->{utilitytemplate};
    }
    }
    else {
    if ($using_theme) {
        $str = $PARAMS->{theme_subtemplate};
    }
    else {
        $str = $PARAMS->{subtemplate};
    }
    }
    return $str;
}



sub show_template {
    my $self = shift;
    my ($iid) = shift;
    my $str;


    $self->debug_msg(3, "Portal::show_template(iid=$iid)");
    # aju
    unless ($DISPLAY->using_themes()) {
        $str = Metadot::TemplateMgr->get_template(obj => $self);
    }
    if (!$str) {
#   print STDERR "in show_template: did not get template from TemplateMgr utility\n";
        $self->debug_msg(4, "Portal::show_template(iid=$iid) did not get template from TemplateMgr utility");
    $str = $self->get_template($iid);
    }
    else {
        #print STDERR "in show_template: got template from TemplateMgr utility\n";
        $self->debug_msg(4, "Portal::show_template(iid=$iid) got template from TemplateMgr utility");
    }
    #CKIM ensure the admin button appears
     my $admin_button = $self->show_admin_button();
     my $traffic_buttons = $DISPLAY->get_traffic_lights_display_if_allowed( subject => $self);
     # if admin button needed, but not found, then
     # add it before the end of the body
     if ( $admin_button && $str !~ m/\Q$admin_button\E/) {
        $str =~ s/<\/BODY>//ig;
        $str =~ s/<\/HTML>//ig;
        $str .= "\n<center>" . $traffic_buttons . $admin_button . "</center>\n</BODY></HTML>\n";
     }
    my $s = "\n\n\n$str\n\n<!-- end template -->\n\n";
    $s .= "\n\n" . $SESSION->get_timer_count_html() . "\n\n";

    return $s;
}

#####################################################
sub show_top{
  my $self = shift; #self
  my $id = shift || 0;
  my $width= "100%";
  $self->debug_msg(1, "Debug mode dude!");
  $self->debug_msg(3, "Portal::show_top($id)");
  my $s = $self->show_page_top;
  $s .= $self->show_header($id);
  $s .= $self->show_portal_panel($id);

  ###
  ### return at this point to not show html on bottom
  ###
  return $s;



  my $toolbgcolor = $PARAMS->{'toolbgcolor'};
  $s .= "\n<TABLE WIDTH=\"100%\" border=\"0\" BGCOLOR=\"#$toolbgcolor\" CELLPADDING=\"5\" CELLSPACING=\"0\">\n";
  $s .= "<TR><TD ALIGN=\"LEFT\" BGCOLOR='"
    . $toolbgcolor
  . "' valign='middle'>". $self->show_welcome() . "</td>\n";
  $s .= "<TD BGCOLOR=\"#$toolbgcolor\" ALIGN=\"RIGHT\" valign='center'>"
     . $self->show_search();
  if ($PARAMS->{'show_who_online'}) {
      $s .= $self->show_users_online
  }
  $s .= "</td></tr></table>\n";

  ### Show Path
  my $path = $self->show_path($id);
  #my $editbutton = ($SESSION->admin_mode) ? $self->show_admin_button() : "";

  if ($path) {
    $path ||= "&nbsp;";
    #$editbutton ||= "&nbsp;";
    $s .= "
<TABLE WIDTH='100%' border=\"0\" BGCOLOR='#$toolbgcolor' CELLPADDING=\"5\" CELLSPACING=\"0\"><TR>
<td align='left'>$path</td>
<td align='right'>&nbsp;</td>
</tr></TABLE>
<!-- End of show path -->";
  }

  return $s;
}

##################################################
sub md_redirect {
    my ($self, @args) = @_;
    return Utility->md_redirect(@args);
}


#####################################################
# Check if user exists in the db
#####################################################
sub is_idded{
  my $self = shift;
  my $uid = shift;

  if (!defined $uid){
    # if not uid is passed to the function
    # then we use the current metadot user
    $uid =  $USER->{uid};
  } else {
    # let's load user's information
    my $u = Metadot::UserFactory->get_user( $uid );
    $uid = $u->uid;
  }
  return 0 if (!$uid);
  return 1;
}


###################################################################################
sub get_parents_list {
  my $self = shift;
  my $id = shift;
  my $return_type = shift;
  my @idarray = ();

  $id = undef unless (Utility->is_valid_integer($id));    # secu_200401
  if (!defined $id) {
      return ();
  }
  my ($parentid, $name, $isa) = DBAccess->sqlSelect("parentiid, name, isa", "instance", "iid = $id");

  if (($id == 0) || ($id == -1000) || ($id == -9999)) {
    if ($return_type && ($return_type eq 'id')) {
        @idarray = ({ id => 0 });
    } else {
        @idarray =  ({ id => 0, name => "Home" });
    }
  } else {
    if ($return_type && ($return_type eq 'id')) {
        @idarray = ($self->get_parents_list($parentid, $return_type), { id => $id });
    } else {
        @idarray = ($self->get_parents_list($parentid), { id => $id, name => $name, isa => $isa });
    }
  }

  return @idarray;
}

###################################################################################
sub get_visible_parents_list {
  my $self = shift;
  my @idarray = ();

  my $id = $self->id();

  if (($id == 0) || ($id == -1000) || ($id == -9999)) {
    @idarray =  ( { id => 0, name => "Home" } );
  } else {
    if (
        $self->isa('Metadot::Navigable') && $self->is_allowed_to_do('show', $USER)
        # check for hidden
        && (!$self->is_hidden() || $SESSION->admin_mode())
        ) {
        @idarray = ( { id => $id, name => $self->name(), isa => $self->is_a() } );
    }

    my $parent_obj = $self->get_parent();
    if ($parent_obj) {
        @idarray = ($parent_obj->get_visible_parents_list(), @idarray);
    }
  }
  return @idarray;
}

###################################################################################

sub _web_init {
  my $self = shift;
  my $uid=0;
  my $email="";
  my $password="";
  my $session_id="";
  my $auxuid="";

  $self->debug_msg(5, "Portal::_web_init");

  # Check to see if the site is disabled. If so, redirect to disabled_url.
  my $results = DBAccess->sqlSelectMany("name, value","params",
                                        "name in ('is_disabled','disabled_url')");
  my %disabled;
  while (my ($key, $value) = $results->fetchrow()) {
    $disabled{$key} = $value;
  }

  if (($disabled{is_disabled}) && ($disabled{is_disabled} eq '1')) {
    # redirect here
    $self->md_redirect($self->make_url($disabled{disabled_url}));
    exit;
  }

  # INIT Session variable

  # if the user is uploading a file then we set $FORM{upload} = 1
  my $filepath='';
  my $fileiid;
  if (($filepath=$CGI->param('file1')) && ($fileiid=$CGI->param('id'))){

    $FORM{upload}=1;

  } else {

    $FORM{upload}=0;
  }

  $FORM{file1}=$filepath;        ### uploaded file if any

  # Now we save all input parameters into the FORM var
  # aju 11-29-01 - handle multiple param values properly
  foreach my $key ($CGI->param) {

      my @param_value = $CGI->param($key);

      if ($#param_value) {  #  if multiple values

      my %mphash = ();

      # step through the values
      foreach my $pval (@param_value) {

          # add to mult param hash
          $mphash{$pval} = 1;
      }

      # set form value to mult. value list

      my @plist = keys %mphash;
      if ($#plist) {    # if still ended up with >1 unique value

          $FORM{$key} = \@plist;

      } else {      # just one value once redundancies removed

          $FORM{$key} = $plist[0];
      }

      } else {
      # single value
      my $value = $CGI->param($key);

      $FORM{$key} = $value;
      }
  }

  my $op=$FORM{op};
  $op="" if (!defined $op);

    # [vadim] we HAVE TO give the COMPONENT a chance to send headers for this
    # operations instead of sending regular headers
    $HTTP_HEADER_SENT = 1 if ($op eq "download" || $op eq "force_download") ;

  $self->debug_msg(5, "_web_init:op:$op id=" . ($FORM{id}||""));

  if (defined $FORM{password}){

      $password=$FORM{password};

  } else {

      $password="";
  }

  #use Data::Dumper;
  #print STDERR "Populated FORM hash as:\n",Dumper \%FORM;

  my $authenticator = Metadot::AuthenticatorFactory->get_authenticator();
  ($USER, $SESSION) = $authenticator->authenticate();

  # moving xss check before the return statement!
  $self->_scan_forbidden_html_tags();
  $self->_scan_unauthorized_template_tags();

  return if (uc($op) eq "REGISTER" && !$FORM{admin_mode});

  # assign gizmo id to the server
  if(exists($FORM{id})){

      $self->{id}=$FORM{id};

  }  elsif (exists($FORM{iid})){

      $self->{id}=$FORM{iid};

  } else {

      $self->{id}=0;
  }

}

#####################################
sub click_back {
  my $self = shift;
  $self->print_click_back(@_);
}

#####################################
sub defined_or_exit {
  my $self      = shift;
  my $variable  = shift;
  my $error_msg = shift;
  $self->debug_msg(5, "Portal::defined_or_exit($variable,$error_msg)");

  return $variable if (defined $variable);
  $self->click_back ("Error:", $error_msg);
  exit(0);
}

#####################################
sub print_send_to_front {
    my $self = shift;

    $DISPLAY->print_send_to_front(@_);
}

#####################################
sub print_click_back {
    my $self = shift;

    $DISPLAY->print_click_back(@_);
}

####################################################

sub print_warning_message {
    my $self = shift;

    my ($title, $message)=@_;
    $self->debug_msg(3, "print_warning_message()");
    my $top = $self->show_page_top();

    my $formbgcolor = $PARAMS->{'formbgcolor'};
    my $imageslocation = $PARAMS->{'imageslocation'};

    my $html = "$top<br /><br /><br /><br /><center>\n";
    $html .= "<table border=\"1\" cellpadding=\"1\" cellspacing=\"0\"><tr><td>\n";
    $html .= "<TABLE BGCOLOR='#$formbgcolor' width=\"500\" CELLSPACING=\"0\" CELLPADDING=\"0\" ";
    $html .= "border=\"0\"><tr><TD >";
    $html .= "<IMG SRC='$imageslocation/pixel.gif' WIDTH='150' HEIGHT='1' BORDER='0'";
    $html .= " HSPACE='0' VSPACE='0'></TD>";
    $html .= "<TD><IMG SRC='$imageslocation/pixel.gif' WIDTH='360' HEIGHT='1' \n";
    $html .= " BORDER='0' HSPACE='0' VSPACE='0'></TD>";
    $html .= "<TD><IMG SRC='$imageslocation/pixel.gif' WIDTH='41' HEIGHT='1'\n";
    $html .= " BORDER='0' HSPACE='0' VSPACE='0'>";
    $html .= "</TD></TR><TR><TD valign='top'><br /><br />&nbsp;&nbsp;<img src = \"$imageslocation/my/stop.gif\" border = 0></TD><TD><br /><br />\n";
    $html .= "<H2>$title </H2><br />\n";

    $html .= "$message\n\n" if ($message);

    $html .= "<br />";
    $html .= "</center><table align='right'><tr><td>\n";


    $html .= "</td></tr></table>";
    $html .= "</TD><TD>&nbsp;</td></TR></table>\n\n";
    $html .= "</td></tr></table>";
    $html .= $self->close_html;

    $self->print ($html);

}

####################################################
sub two_choices {
  my $self = shift;

  $DISPLAY->print_choices(@_);
}
####################################################
sub three_choices {
  my $self = shift;

  $DISPLAY->print_choices(@_);
}
#####################################################
sub print_choices {
  my $self = shift;

  $DISPLAY->print_choices(@_);
}
#############################################
sub title_bar {
    my $self = shift;
    my ($title) = @_;

    return $DISPLAY->show_title_bar( $title );
}
#############################################
sub show_page_top {
  my $self = shift;

  return $DISPLAY->show_page_top(@_);
}
#############################################
sub show_style_sheet {
  my $self = shift;

  return $DISPLAY->show_style_sheet(@_);
}
#################################################
# An improvement for this method was suggested by
# Gary Lawrence Murphy.
#################################################
sub open_html {
  my $self = shift;

  return $DISPLAY->open_html(@_);
}
#############################################
sub close_html {
  my $self = shift;

  return $DISPLAY->close_html(@_);
}
#############################################
sub form_bottom {
  my $self = shift;

  return $DISPLAY->form_bottom(@_);
}
#############################################
sub form_top {
    my $self = shift;

    return $DISPLAY->form_top();
}

##################################################
### show_channel_toc
### Show Table of Contents of channels
##################################################
sub show_channel_toc {
  my $self = shift;
  my $url = shift;
  $self->debug_msg(3, "Portal::show_channel_toc()");

  my $s = "
<h2>Channel Types</h2>
<ul>
 <li><a href = '".$self->make_url("$url&amp;channel_type=internal")."'>Internal News Channels</a>
 <li><a href = '".$self->make_url("$url&amp;channel_type=external")."'>External RSS News Channels</a>
 <li><a href = '".$self->make_url("$url&amp;channel_type=discussion")."'>Discussion Channels</a>
 <li><a href = '".$self->make_url("$url&amp;channel_type=gizmotool")."'>Gizmo Tools</a>
</ul>
";

  return $s;
}

##################################################
### show_channel_list
### Show a list of channels depending on channel type
##################################################
sub show_channel_list {
  my $self = shift;
  my ($parentiid) = shift;
  my ($channel_type) = shift;
  my ($scripturl) = shift;
  $scripturl .= "&amp;channel_type=$channel_type";
  my $s="";
  my $c;
  my $i=0;
  my $oldi=0;
  my $maxitem=999;
  my $numrows = 0;
  my $from = $FORM{from} || 0;
  my $max  = $FORM{nb} || 20 ;
  my $form_ccid  = $FORM{ccid} || 0 ;
  my $nbdisp = 0 ;
  my ($isa, $name, $description, $url, $cid, $iid);
  $self->debug_msg(3, "Portal::show_channel_list(parentiid=$parentiid)");

  $parentiid = undef unless (Utility->is_valid_integer($parentiid));    # secu_200401

  ######### External Channels

  if ($channel_type eq 'external') {
         my $c = DBAccess->sqlSelectMany("ccid, cc_name", "channel_category", "ccid > 0", "order by cc_name");
        my %CATEGORIES;
        while(my($k, $v)=$c->fetchrow()){
                $CATEGORIES{$k} = $v;
        }
        #if no categories then add default
        unless(%CATEGORIES){
            my $ccid = DBAccess->sqlInsert("channel_category", cc_name  => 'Default');
            DBAccess->sqlUpdate("channel", "isa='External'", ccid => $ccid);
            $CATEGORIES{$ccid} = 'Default';
        }

        my $s1 ="<table width='100%'><tr><td align='right'>".$self->show_add_channel_panel()."</td></tr></table>";
        $s1 .= "<table width='100%' cellpadding=\"0\" cellspacing=\"0\" border=\"0\" bgcolor='#ffffff'>
                                        <tr><td>
                                                                <table cellpadding=\"0\" cellspacing=\"0\" width='100%' cols=5>
                                                                        <tr><td  colspan='4' height='25' bgcolor='#cccccc'><b><a href=\"".$self->make_url("index.pl?iid=0&amp;isa=Portal&amp;op=show_channels&amp;channel_type=external")."\">Categories</a> in External RSS News </b></td>
                                                                              <td bgcolor='#FFCE9C' height='30' align='center'>
                                                                                         [ <a class='toollink' href='".$self->make_url("index.pl?isa=Channel&amp;op=add_category&amp;type=External&amp;parent_id=0")."'>Add a Category</a> ]
                                                                                   </td>
                                                                        </tr></table>
                                                        </td></tr>
                                                        <tr><td><table border=\"0\" cellspacing=\"3\">";
        my $j = 0;
        foreach my $k (sort { $CATEGORIES{$a} cmp $CATEGORIES{$b} } (keys %CATEGORIES)){
                 my $chan = Channel->new($cid);
                 my $panel= $chan->show_category_panel($k);
                 my $v = $CATEGORIES{$k};
                $j = $j % 3;
                if($j == 0){
                        $s1 .= "<tr>";
                }
                $s1 .= "<td valign='top'><b>.<b></td><td><a href='".$self->make_url("#$v")."'><b>$v</b></a>$panel</td>";
                if($j == 2){
                         $s1 .= "</tr>";
                }
                $j++;
        }
        $s1 .= "</table></td></tr></table>";

        ##if only looking at one category - previous, next
        my %one_category;
        my $hash_ref;
        if ($form_ccid > 0) {
           $one_category{$form_ccid} = $CATEGORIES{$form_ccid};
           $hash_ref = \%one_category;
        } else {
           $hash_ref = \%CATEGORIES;
        }

     foreach my $k (sort { $$hash_ref{$a} cmp $$hash_ref{$b} } (keys %$hash_ref)){
                my $v = $$hash_ref{$k};
                $oldi=$i;

        my $from_table  = 'channel';
        my $where = "isa='External' AND ccid = '$k' ";
        my $other = 'order by name';

                ($c)=DBAccess->sqlSelectMany("isa, name, description, url,cid", $from_table, $where, $other);
                # my $numrows  = $c->rows;
                # rows function is not reliable for counting the rows retrieved by SELECT
                my ($numrows) = DBAccess->sqlSelect("count(*)",$from_table, $where, $other);


                #return "" if (!$numrows);
                my $ss = "";
                $nbdisp = 0;
                while((($isa, $name, $description, $url, $cid)=$c->fetchrow()) && ($i < $maxitem) &&($nbdisp<$max)) {
                        if ($from) {$from--; next;}
                        $nbdisp++;
                         $i++;
                # When the channel is 1st entered, the name is null so
                # We display the URL
                        my $chan = Channel->new($cid);
                         $ss.= $chan->show_panel();
                         if($name){
                                 $ss .= "<a href=".$self->make_url("userchannel.pl?isa=NewsChannel&amp;op=preview&amp;id=$cid")."><b>$name</b></a>";
                          } else {
                                $ss .= "<I>Available soon: $url</I>";
                          }
                                $ss.= ": $description" if ($description);
                        $ss.= "<br />\n";
        #print "---- $s\n<br />";
                 }
                $c->finish();
                #$ss = "" if ($i == $oldi);
                #PREVIOUS NEXT MAGIC
                 my $chan = Channel->new($cid);
                         my $panel= $chan->show_category_panel($k);
                my $PNbuttons = ($numrows>$max) ? Components->get_previous_next_button($max, $nbdisp, $numrows, $FORM{from}, $scripturl, $k) : "";

                $s .= "<a name='$v'><p /><table width='100%'><tr><td bgcolor='#cccccc'>$panel <b>$v</b></td></tr></table>\n\n" . $PNbuttons .$ss;
         }

        $s = $s1.$s;
  } # end if for external channel

  ######### Discussion channels
  $oldi=$i;
  if ($channel_type eq 'discussion') {
      $s= "<br /><br /><b>Discussion Channels</b><br /><br /><p />\n\n";

      my $from_table  = 'channel';
      my $where = "isa='Discussion'";
      my $other = 'order by name';

      ($c)=DBAccess->sqlSelectMany("cid, name, description, iid", $from_table, $where, $other);
      #my $numrows  = $c->rows;
      # rows function is not reliable for counting the rows retrieved by SELECT
      my ($numrows) = DBAccess->sqlSelect("count(*)",$from_table, $where, $other);
      return "" if (!$numrows);

      while((($cid, $name, $description, $iid)=$c->fetchrow()) && ($i < $maxitem) && ($nbdisp<$max)) {
        if ($from) {$from--; next;}
        $nbdisp++;
        $i++;
        # aju
        my $chan = Channel->new($cid);
        $s.= $chan->show_panel();
#          my $disc = Discussion->new($iid);
#          $s.= $disc->show_panel();
        $s.= "<a href=".$self->make_url("userchannel.pl?isa=NewsChannel&amp;op=preview&amp;id=$cid")."><b>$name</b></a>";
        $s.= ": $description" if ($description);
        $s.= "<br />\n";
        #print "---- $s\n<br />";
      }
      $c->finish();
      $s = "" if ($i == $oldi);
      #PREVIOUS NEXT MAGIC
      my $PNbuttons = ($numrows>$max) ? Components->get_previous_next_button($max, $nbdisp, $numrows, $FORM{from}, $scripturl) : "";
      $s = $PNbuttons . $s;
  } # end if for discussion channel

  ######### Internal channels
  $oldi=$i;
  if ($channel_type eq 'internal') {
      $s= "<br /><br /><b>Internal News Channels</b><br /><br /><p />\n\n";

      my $from_table  = 'channel';
      my $where = "isa='Internal'";
      my $other = 'order by name';
      ($c)=DBAccess->sqlSelectMany("isa, name, description, url,cid", $from_table, $where, $other);

      #my $numrows  = $c->rows;
      # rows function is not reliable for counting the rows retrieved by SELECT
      my ($numrows) = DBAccess->sqlSelect("count(*)",$from_table, $where, $other);
      return "" if (!$numrows);

      while((($isa, $name, $description, $url, $cid)=$c->fetchrow()) && ($i < $maxitem) && ($nbdisp < $max)) {

      if ($from) {$from--; next;}
      $nbdisp++;
      $i++;
      my $chan = Channel->new($cid);
      $s.= $chan->show_panel();
      $s.= "<a href=".$self->make_url("userchannel.pl?isa=NewsChannel&amp;op=preview&amp;id=$cid")."><b>$name</b></a>";
      $s.= ": $description" if ($description);
      $s.= "<br />\n";
      #print "---- $s\n<br />";
      }
      $c->finish();

      $s = "" if ($i == $oldi);
      #PREVIOUS NEXT MAGIC
      my $PNbuttons = ($numrows>$max) ? Components->get_previous_next_button($max, $nbdisp, $numrows, $FORM{from}, $scripturl) : "";
      $s = $PNbuttons . $s;
  } # end if for internal channel

  ######### Gizmo Tools
  $oldi=$i;
  if ($channel_type eq 'gizmotool') {

      #gizmotool list
      use GizmoTool;
      my @gtnames = GizmoTool->lists_tools();
      my $where_gizmo_tool = "";
      if (@gtnames) {
          $where_gizmo_tool = join(" OR ", map("isa='$_'",@gtnames));
      } else {
          return "";
      }

      $s = "<br /><br /><b>Gizmo Tools</b><br /><br /><p />\n\n";

      my $from_table  = 'channel';
      my $other = 'order by name';
      ($c)=DBAccess->sqlSelectMany("isa, name, description, url, cid", $from_table, $where_gizmo_tool, $other);

      #my $numrows  = $c->rows;
      # rows function is not reliable for counting the rows retrieved by SELECT
      my ($numrows) = DBAccess->sqlSelect("count(*)",$from_table, $where_gizmo_tool, $other);

      return "" if (!$numrows);
      while((($isa, $name, $description, $url, $cid)=$c->fetchrow()) && ($i < $maxitem) &&($nbdisp<$max)) {
      if ($from) {$from--; next;}
      $nbdisp++;
      $i++;
      # When the channel is 1st entered, the name is null so
      # We display the URL
      my $chan = Channel->new($cid);
      $s.= $chan->show_panel();
      if($name){
          $s .= "<a href=".$self->make_url("userchannel.pl?isa=$isa&amp;op=preview&amp;id=$cid")."><b>$name</b></a>";
      } else {
          $s .= "<I>Available soon: $url</I>";
      }
      $s.= ": $description" if ($description);
      $s.= "<br />\n";
      #print "---- $s\n<br />";
      }
      $c->finish();
      $s = "" if ($i == $oldi);

      #PREVIOUS NEXT MAGIC
      my $PNbuttons = ($numrows>$max) ? Components->get_previous_next_button($max, $nbdisp, $numrows, $FORM{from}, $scripturl) : "";
      $s = $PNbuttons . $s;
  } # end if for external channel



  if ($i >= $maxitem){
      $s.="<a href='".$self->make_url("index.pl?iid=$parentiid&amp;isa=Portal&amp;op=show_channels&amp;from=$maxitem&amp;channel_type=$channel_type")."'>More...</a><br /><br />";
  }

  return ($s);

}

##########################################################

##########################################################

sub regapproval_level {
        my $self = shift;

        if (@_) {
                my $regapproval = shift;


                $PARAMS->set_regapproval($regapproval);
        }

        return $self->{regapproval};
}

sub password_checker_level {
        my $self = shift;

        if (@_) {
                my $password_checker = shift;


                $PARAMS->set_password_checker($password_checker);
        }

        return $self->{password_checker};
}

##########################################################
# this is now handled by Display
sub print_portal {
    my $self = shift;
    my $content = shift;
    my $title = shift ;
    my $iid = shift ;
    $iid = $self->id() if (!defined $iid) ;

    $DISPLAY->print_portal({
        content  =>$content,
        title    =>$title,
        iid      =>$iid,
        subject  =>$self,
    });
}

#########################################
sub get_gizmo_browser_icon {
    my $self = shift;
  my $myisa = shift;
  my $mycurr = shift;
  my $myhtml = "";
  my $myfile = "Item_icon.jpg";
  my $myalt = "Object";

  if ($myisa eq 'Category') {
        if ($mycurr == '1') {
                $myfile = "Category_curr_icon.jpg" ;
        } else {
                $myfile = "Category_icon.jpg" ;
        }
        $myalt = "Category" ;
  }
  if ($myisa eq 'Newsitem') {
        $myfile = "Newsitem_icon.jpg" ;
        $myalt = "News Item" ;
  }
  if ($myisa eq 'Poll') {
        $myfile = "Poll_icon.jpg" ;
        $myalt = "Poll" ;
  }
  if ($myisa eq 'ImageItem') {
        $myfile = "ImageItem_icon.jpg" ;
        $myalt = "Image Item" ;
  }
  if ($myisa eq 'Application') {
        $myfile = "Application_icon.jpg" ;
        $myalt = "Application" ;
  }
  if ($myisa eq 'Discussion') {
        $myfile = "Discussion_icon.jpg" ;
        $myalt = "Discussion" ;
  }
  if ($myisa eq 'Item') {
        $myfile = "Item_icon.jpg" ;
        $myalt = "Simple Item" ;
  }
  if ($myisa eq 'Survey') {
        $myfile = "Survey_icon.jpg" ;
        $myalt = "Survey" ;
  }
  if ($myisa eq 'NewsPaperView') {
        $myfile = "NewsPaperView_icon.jpg" ;
        $myalt = "News Paper View" ;
  }
  if ($myisa eq 'Table') {
        $myfile = "Table_icon.jpg" ;
        $myalt = "Table" ;
  }
  if ($myisa eq 'Row') {
        $myfile = "Row_icon.jpg" ;
        $myalt = "Row" ;
  }
  $myhtml .= "<img src=" . $PARAMS->{'imageslocation'} . "/obj/" . $myfile . " border=\"0\" alt='" . $myalt . "'>";

  return $myhtml;
}

#########################################
sub show_full_gizmo_list {
  my $self = shift;
  my $myid = shift;
  my $str = "";
  my $child_id = undef ;
  my $sth = undef ;
  my $tth = undef ;
  my $ltth = undef;
  my $gizmo = undef ;
  my $mygizmo = undef ;
  my @gizmos_list = Gizmo->lists_gizmos() ;
  push @gizmos_list , "Row" ;
  my %myhash ;



  $myid = undef unless (Utility->is_valid_integer($myid));    # secu_200401
  $child_id = undef unless (Utility->is_valid_integer($child_id));    # secu_200401

      $sth = DBAccess->sqlSelectMany("iid", "instance", "parentiid=$myid", "order by instanceorder");
      if ( length $sth->fetchrow() eq 0 and
           not( (uc(ref($self)) eq "CATEGORY") || (uc($self->{is_a}) eq "CATEGORY") ) and
           not( (uc(ref($self)) eq "TABLE") || (uc($self->{is_a}) eq "TABLE") ) ) {

          return "<CENTER>Not a container object</CENTER>";
      }

#  $str .= "<blockquote>";

  $str .= $self->show_portal_panel($myid) ;


  $str .= "<br /><TABLE><TR><TD>" . $self->get_gizmo_browser_icon($self->is_a,'1') . "</TD><TD>" . $self->show_drilldown_menu($myid) . "</TD></TR></TABLE><br />";
  $str .= "<TABLE BORDER=\"0\" cellspacing=\"4\" width='100%'>";
  $str .= "<TR bgcolor=\"CCCCCC\">" ;
  $str .= "<TH>&nbsp;</TH>" ;
  $str .= "<TH>Type</TH>" ;
  $str .= "<TH>Id</TH>" ;
  $str .= "<TH>Name</TH>" ;
  $str .= "<TH>User</TH>" ;
  $str .= "<TH>Last Modified</TH>" ;
  $str .= "<TH>&nbsp;</TH>" ;
  $str .= "<TH>&nbsp;</TH>" ;
  $str .= "<TH>&nbsp;</TH>" ;
  $str .= "<TH>&nbsp;</TH>" ;
  $str .= "</TR>" ;



  my $white = "FFFFFF";
  my $gray = "EEEEEE";
  my $colorflag = 0; my $rowcolor;
  foreach $gizmo (@gizmos_list) {
  $sth = DBAccess->sqlSelectMany("iid", "instance", "parentiid=$myid and isa='$gizmo'", "order by instanceorder");
    while( ($child_id) = $sth->fetchrow() ) {
      # alternating colors to the rows in the table
      if ($colorflag) {
      $rowcolor = $white;
      $colorflag = 0;
      }
      else {
      $rowcolor = $gray;
      $colorflag = 1;
      }
      $mygizmo = $gizmo->new($child_id);
      $str .= "<TR bgcolor=\"$rowcolor\">" ;
      $tth = DBAccess->sqlSelectMany("iid", "instance", "parentiid=" . $mygizmo->id, "order by instanceorder");
      $ltth = $tth->fetchrow() ;
      if ( length ($ltth) != 0 or
           ( (uc(ref($mygizmo)) eq "CATEGORY") || (uc($mygizmo->{is_a}) eq "CATEGORY") ) or
           ( (uc(ref($mygizmo)) eq "TABLE") || (uc($mygizmo->{is_a}) eq "TABLE") ) ) {
          $str .= "<TD><A HREF=" . $mygizmo->url . "?" . "&amp;id=" . $mygizmo->id . ">" . $self->get_gizmo_browser_icon($mygizmo->is_a,'0') . "</a></TD>" ;
      } else {
          $str .= "<TD>" . $self->get_gizmo_browser_icon($mygizmo->is_a,'0') . "</TD>" ;
      }

      $str .= "<TD>" . $mygizmo->is_a . "</TD>" ;
      $str .= "<TD>" . $mygizmo->id . "</TD>" ;
      $str .= "<TD>" . $mygizmo->name . "</TD>" ;

      my $u = Metadot::UserFactory->get_user( $mygizmo->uid );

      $str .= "<TD><a href='".$self->make_url("index.pl?isa=User&amp;op=modify&amp;uid=" . $mygizmo->uid) . "'>" . $u->fullname . "</a></TD>" ;
      if("0000-00-00 00:00:00" eq $mygizmo->update_date) {
        $str .= "<TD>" . $mygizmo->creation_date . "</TD>" ;
      } else {
        $str .= "<TD>" . $mygizmo->update_date . "</TD>" ;
      }
      $str .= "<TD>" ;
      if(uc($mygizmo->{is_a}) ne "CATEGORY") {
        $str .= "<a href='".$self->make_url("index.pl?isa=" . $mygizmo->is_a . "&amp;op=modify&amp;id=" . $mygizmo->id) . "'><img src=" . $PARAMS->{'imageslocation'} . "/obj/edit.gif border=\"0\" alt='Modify'></a>";
      } else {
        $str .= "&nbsp;" ;
      }
      $str .= "</TD>\n";
      $str .= "<TD>" ;
      if(uc($mygizmo->{is_a}) ne "CATEGORY") {
        $str .= "<a href='".$self->make_url("index.pl?isa=" . $mygizmo->is_a . "&amp;op=edit_permissions&amp;id=" . $mygizmo->id) . "'><img src=" . $PARAMS->{'imageslocation'} . "/obj/ac.gif border=\"0\" alt='Permisssions'></a>" ;
      } else {
        $str .= "&nbsp;" ;
      }
      $str .= "</TD>\n";
      $str .= "<TD><a href='".$self->make_url("index.pl?isa=" . $mygizmo->is_a . "&amp;op=cut&amp;id=" . $mygizmo->id) . "'><img src=" . $PARAMS->{'imageslocation'} . "/obj/cut.gif border=\"0\" alt='Cut'></a></TD>\n";
      $str .= "<TD><a href='".$self->make_url("index.pl?isa=" . $mygizmo->is_a . "&amp;op=delete&amp;id=" . $mygizmo->id) . "'><img src=" . $PARAMS->{'imageslocation'} . "/obj/del.gif border=\"0\" alt='Delete'></a></TD>\n";
      $str .= "</TR>" ;
    }
    $sth->finish();
  }
  $str .= "</TABLE>";
  $str .= "<br /><CENTER>" . Clipboard->show_select($myid, undef, ['File']) . "</CENTER>";
#  $str .= "<br /><CENTER><A HREF=\"index.pl?&amp;id=-9999\"><b>Change Tree Root</b></a></CENTER>" ;
  $str .= "<br /><CENTER><A HREF=\"javascript:void(winTag=window.open('index.pl?&amp;id=$myid&amp;show_gizmotag_list=1','gizmoTagWindow','toolbar=no,location=no,directories=no,status=no,menubar=no,scrollbars=yes,resizable=yes,copyhistory=yes,width=800,height=400'))\"><b>Gizmo Tag Table</b></a></CENTER>" ;
#  $str .= "</blockquote>";
  return $str ;
}

##########################################
# process_template()
# look for tag and expand them
# used by the templating system
# expand() is called to translate the tags
##########################################
sub process_template {
    my $self = shift;
    my $iid  = shift;
    my $text = shift;
    my $val;
    my $new_body = '';
    my $imageslocation = $PARAMS->{'imageslocation'};

    if($FORM{show_gizmotag_list} or $SESSION->get_value('GB_status') eq '1') {

        if($FORM{show_gizmotag_list}) {
                if($FORM{show_gizmo_tag}) {
                    $new_body = '<BODY bgcolor="#FFFFFF">';
                    $new_body .= '<TABLE WIDTH="100%" BORDER="0" CELLPADDING="5" CELLSPACING="0">';
                    $new_body .= '<TR>';
                    $new_body .= '<TD BGCOLOR="#336699" ALIGN="LEFT" VALIGN="MIDDLE" nowrap="nowrap"> <FONT FACE="Arial" SIZE="+2" COLOR="#FFFFFF">Metadot, Inc.</FONT>';
                    $new_body .= '</TD>';
                    $new_body .= '<TD BGCOLOR="#336699" ALIGN="right" VALIGN="MIDDLE" nowrap="nowrap"><FONT FACE="Arial" SIZE="+2" COLOR="#FFFFFF">';
                    $new_body .= 'Gizmo Tag Preview';
                    $new_body .= '</font></td></TR>';
                    $new_body .= '<TR>';
                    $new_body .= '<TD  colspan="2"  BGCOLOR="#dddddd" ALIGN="RIGHT">';
                    $new_body .= '</TD>';
                    $new_body .= '</TR>';
                    $new_body .= '</TABLE>';
                    $new_body .= '<br /><br />';
                        $new_body .= "<p /><p /><CENTER><H1><I>GizmoTag</I> $FORM{show_gizmo_tag} <I>in Category Id</I> #$FORM{id}</H1><p />\n";
                        $new_body .= "<TABLE BORDER=\"1\"><TR><TD><GIZMOTAG name=\"$FORM{show_gizmo_tag}\"></GIZMOTAG></TD></TR></TABLE><p /><p /><p />";
                        $new_body .= "<p /><p /></CENTER>";
                        $new_body .= "<br /><CENTER><A HREF=\"javascript:void(window.history.back())\"><b>[Back]</b></a></CENTER>" ;
                    $new_body .= "</BODY>";
                } else {
                    $new_body = '<BODY bgcolor="#FFFFFF">';
                    $new_body .= '<TABLE WIDTH="100%" BORDER="0" CELLPADDING="5" CELLSPACING="0">';
                    $new_body .= '<TR>';
                    $new_body .= '<TD BGCOLOR="#336699" ALIGN="LEFT" VALIGN="MIDDLE" nowrap="nowrap"> <FONT FACE="Arial" SIZE="+2" COLOR="#FFFFFF">Metadot, Inc.</FONT>';
                    $new_body .= '</TD>';
                    $new_body .= '<TD BGCOLOR="#336699" ALIGN="right" VALIGN="MIDDLE" nowrap="nowrap"><FONT FACE="Arial" SIZE="+2" COLOR="#FFFFFF">';
                    $new_body .= 'Gizmo Tag Table';
                    $new_body .= '</font></td></TR>';
                    $new_body .= '<TR>';
                    $new_body .= '<TD  colspan="2"  BGCOLOR="#dddddd" ALIGN="RIGHT">';
                    $new_body .= '</TD>';
                    $new_body .= '</TR>';
                    $new_body .= '</TABLE>';
                    $new_body .= '<br /><br />';
                        $new_body .= $self->show_full_tag_list() ;
                        $new_body .= "<br /><CENTER><A HREF=\"javascript:void(window.close())\"><b>[Close]</b></a></CENTER>" ;
                    $new_body .= "</BODY>";
                }
                $text =~ s/\<BODY.+\<\/BODY\>/$new_body/si;
                $text = $self->expand($iid, $text);
         }
        if($SESSION->get_value('GB_status') eq '1') {
            $new_body .= '<BODY BGCOLOR="#FFFFFF">';
            $new_body .= '<TABLE WIDTH="100%" BORDER="0" CELLPADDING="5" CELLSPACING="0">';
            $new_body .= '<TR>';
            $new_body .= '<TD BGCOLOR="#336699" ALIGN="LEFT" VALIGN="MIDDLE" nowrap="nowrap"> <FONT FACE="Arial" SIZE="+2" COLOR="#FFFFFF">Metadot, Inc.</FONT>';
            $new_body .= '</TD>';
            $new_body .= '<TD BGCOLOR="#336699" ALIGN="right" VALIGN="MIDDLE" nowrap="nowrap"><FONT FACE="Arial" SIZE="+2" COLOR="#FFFFFF">';
            $new_body .= 'Gizmo Browser';
            $new_body .= '</font></td></TR>';
            $new_body .= '<TR>';
            $new_body .= '<TD  colspan="2"  BGCOLOR="#dddddd" ALIGN="RIGHT">';
            $new_body .= '</TD>';
            $new_body .= '</TR>';
            $new_body .= '</TABLE>';
            $new_body .= '<br /><br />';
            $new_body .= $self->show_full_gizmo_list($iid) ;
            $new_body .= "</BODY>";
            $text =~ s/\<BODY.+\<\/BODY\>/$new_body/si;
         }
    }
    else {

        my $version = MetadotConfig->get_version();
        $self->debug_msg(3, "Portal::process_template()");

        $text .= "<CENTER>" . $self->show_admin_button(0) . "</CENTER>" ;
        if ($SESSION->admin_mode()) {
                $text .= "<CENTER>Version " . $version . "</CENTER>";
                $text .= "<CENTER>" . Clipboard->show_select($iid, undef, ['File']) . "</CENTER>";
                $text .= "<TABLE WIDTH=\"100%\"><TR><TD BACKGROUND=$imageslocation/tag.gif align='CENTER' HEIGHT=\"16\"px WIDTH=\"100%\"><FONT SIZE=1 COLOR=#FFFFFF>&nbsp;GIZMO AREA&nbsp;GIZMO AREA&nbsp;GIZMO AREA&nbsp;GIZMO AREA&nbsp;GIZMO AREA&nbsp;GIZMO AREA&nbsp;</FONT></TD></TR></TABLE>" ;
                $text .= "<p />The administrator's browsing area allows you to change the template used to generate the page,
                see the names of the GIZMOTAGS used in the page template and how they render.";
                $text .= "<p /><CENTER><b><A HREF=\"index.pl?manage=Colors&amp;isa=Portal&amp;app=&amp;iid=0&amp;op=modify_site_params\">Change Template</a></b></CENTER>";
                $text .= "<p />";
                # commented out since now supported by 'md_adminbar'.
        #              $text .= $self->show_portal_panel($iid) ;
        }
        if ($SESSION->admin_mode() && $USER->is_admin()) {
                $text .= "<CENTER><A HREF=\"javascript:void(winTag=window.open('index.pl?&amp;id=$iid&amp;show_gizmotag_list=1','gizmoTagWindow','toolbar=no,location=no,directories=no,status=no,menubar=no,scrollbars=yes,resizable=no,copyhistory=yes,width=800,height=400'))\"><b>Gizmo Tag Table</b></a></CENTER>" ;
        }
        # replace gizmotags by their value
        $text = $self->expand($iid, $text);

        $text .= "</BODY></HTML>" ;

    }

    return $text;
}

# now just a wrapper for the Display method
sub simple_expand {
    my ($self, $iid, $text, $placeholder) = @_;

    return $DISPLAY->simple_expand($iid, $self, $text, $placeholder);
}


##########################################
# expand
# replace gizmotag by their value
# return expanded text
##########################################
sub expand {
    my $self = shift;
    my $iid  = shift;
    my $rtext = shift;
    my ($ref, $res, $wrap, $val);

    $self->debug_msg(3, "Portal::Expand()");

    $self->debug_msg(5, "Portal::Expand(content to expand: <<$rtext>>)");

    #aju 04-7-02: first expand any template toolkit tags in the
    #text (needed for themes support).  Use PARAMS attrs as vars.
    my $text;
    my $trial_mode = MetadotConfig->get_site_in_trial_mode();
#      print STDERR "trial mode: $trial_mode\n";
    # aju -- shareM hack, port in a 'param' from metadotconfig to use in the template expansion.
    if (!exists $PARAMS->{site_in_trial_mode} ) {
    $PARAMS->{site_in_trial_mode} = $trial_mode;
    }
    my $template = Template->new({
    }) || die $Template::ERROR, "\n";

    $template->process(\$rtext, $PARAMS, \$text)
      || die $template->error(), "\n";


      # This is now being handled by DISPLAY
      $text = $DISPLAY->expand_gizmotags($iid, $self, $text);
    # now expand the gizmo tags
    #print STDERR "CKIM: Portal->expand is STARTING.\n";
#     while ($text =~ m/([<\[]GIZMOTAG.*?[<\[]\/GIZMOTAG[>\]])/isg) {
#         #CKIM-GIZMORUNNER
#         my $tag_text = $1;
#
#         $self->debug_msg(4, "in expand, with tag text: ".escapeHTML($tag_text));
#
#         my $params = $DISPLAY->parse_gizmotag_params($tag_text, $self);
#         my $md_tag = $params->{name};
#     $text = $DISPLAY->generate_gizmotag_html($iid, $text, $tag_text, $md_tag, $params);
#     }
#
    return $text;
}



########################################
sub show_full_tag_list {
    my $self = shift;
    my $str = "";
    $self->debug_msg(3, "Portal::show_full_tag_list()");

#  $str .= "<blockquote>";
  $str .= "<TABLE BORDER=\"0\" cellspacing=\"4\" >";
  $str .= "<TR bgcolor=\"CCCCCC\">" ;
  $str .= "<TH>Tag Name</TH>" ;
  $str .= "<TH>Version</TH>" ;
  $str .= "<TH>File Name</TH>" ;
  $str .= "<TH>Description</TH>" ;
  $str .= "<TH>Style Classes</TH>" ;
  $str .= "<TH>Author Name</TH>" ;
  $str .= "</TR>" ;

    use Utility;
    my $found_modules = Utility->read_modules_in_directory("GizmoTag");

    my $white = "FFFFFF";
    my $gray = "EEEEEE";
    my $colorflag = 0; my $rowcolor;

    while (my ($md_tag, $rel_filename) = each %{$found_modules} ) {

        require "$rel_filename";

        my $ref = $md_tag->new();
    # alternating colors to the rows in the table
    if ($colorflag) {
        $rowcolor = $white;
        $colorflag = 0;
    }
    else {
        $rowcolor = $gray;
        $colorflag = 1;
    }
        $str .= "<TR bgcolor=\"$rowcolor\">" ;
        $str .= "<TD><A HREF=index.pl?&amp;id=$FORM{id}&amp;show_gizmotag_list=1&amp;show_gizmo_tag=" . $ref->get_name() . ">" . $ref->get_name() . "</a></TD>" ;
        $str .= "<TD>" . $ref->get_version() . "</TD>" ;
        $str .= "<TD>" . $md_tag . ".pm</TD>" ;
        $str .= "<TD>" . $ref->get_description() . "</TD>" ;
        $str .= "<TD>" ;
        $str .= $ref->get_styleclass() ;
        foreach my $sclass (@{$ref->get_embeddedstyleclasses()}) {
            $str .= "<br />" . $sclass ;
        }
        $str .= "</TD>" ;
        $str .= "<TD>" . "<A HREF=mailto:" . $ref->get_email() . ">" . $ref->get_authorname() . "</a>" . "</TD>" ;
        $str .= "</TR>" ;
    }
    $str .= "</TABLE>" ;
#    $str .= "</blockquote>" ;
    return $str;
}
#########################################################
sub print_form {
    my $self = shift;

    $DISPLAY->print_form(@_);
}
##########################################################
sub list_ascendants {
  my $self = shift ;

  return ();
}
#########################################################
# show vertical sub navigation
#
#

# aju -- use 'implementations' for this method to allow it to be
# specialized acc. to the config file.
sub show_v_sub_navigation_flex {
    my $self = shift;

    my $impl = $self->get_implementation('PortalImplementation','Metadot::Implementations::Portal::Default');
    return $impl->show_v_sub_navigation_flex($self, @_);
}

sub show_h_dropdown{
    my $self = shift;

    my $impl = $self->get_implementation('PortalImplementation','Metadot::Implementations::Portal::Default');
    return $impl->show_h_dropdown($self, @_);
}


sub show_v_sub_nav_logic {
    my $self = shift;
    my $iid = shift;
    $self->debug_msg(3, "Portal::show_v_sub_navigation()");

    my @asc = $self->list_ascendants();
    @asc = (0) if (!@asc) ;
    my $portal_id = shift @asc;
    my $top_cat_id = shift @asc || $portal_id ;
    my $last_cat_id = $asc[$#asc] || $top_cat_id;

    my $portal  = Gizmo->new($portal_id);
    my $top_cat = Gizmo->new($top_cat_id);
    my $cat     = Gizmo->new($last_cat_id);
    my $tmpcatid = $cat->id;
    my ($parentcatid) = DBAccess->sqlSelect('ParentIID','instance',"isa='Category' and iid = '$tmpcatid'");
    # print STDERR "in portal cat id is $tmpcatid and parentcatid is $parentcatid\n\n\n";

  # application bar

  my $application_bar = Application->show_application_bar();

 # navigation colum

    my $portal_link = ($portal_id) ? "<a href='".$self->make_url("index.pl?iid=$portal_id")."'>".$portal->name()."</a>\n" : "" ;
    my $catid = $cat->id;
    my $navigation_column = $top_cat->can('show_navigation_column') ? $top_cat->show_navigation_column(0, $catid, $parentcatid , @asc) : '';
    $navigation_column = "<table border = 0 cellpadding = 0 cellspacing = 0>$navigation_column</table>";

    return $navigation_column;
}


sub show_v_sub_navigation {
    my $self = shift;
    my $iid = shift;
    $self->debug_msg(3, "Portal::show_v_sub_navigation()");

    my @asc = $self->list_ascendants();
    @asc = ({ id => 0 }) if (!@asc) ;

    my $portal_id = shift(@asc)->{id};
    my $top_cat_id =  $#asc > 0 ? shift(@asc)->{id} : $portal_id ;
    my $last_cat_id = $#asc > 0 ? $asc[$#asc]->{id} : $top_cat_id;

    my $portal  = Gizmo->new($portal_id);
    my $top_cat = Gizmo->new($top_cat_id);
    my $cat     = Gizmo->new($last_cat_id);

  # application bar

  my $application_bar = Application->show_application_bar();

 # navigation colum

    my $navigation_column = $top_cat->can('show_navigation_column') ? $top_cat->show_navigation_column(0,@asc) : '' ;
    $navigation_column = "<table border = 0 cellpadding = 0 cellspacing = 0>$navigation_column</table>";

    return $navigation_column;

}

##########################################################
sub show_h_topnav {
  my $self = shift;
  my $content = shift || "&nbsp;";
  my $iid = shift;
  my $color = shift;
  $self->debug_msg(3, "Portal::show_h_topnav()");

  my @asc = $self->list_ascendants();
  @asc = (0) if (!@asc) ;
  my $portal_id = shift @asc;
  my $top_cat_id = shift @asc || $portal_id ;
  my $last_cat_id = $asc[$#asc] || $top_cat_id;

  my $portal  = Gizmo->new($portal_id);
  my $top_cat = Gizmo->new($top_cat_id);
  my $cat     = Gizmo->new($last_cat_id);
  my $tmpcatid = $cat->id;
  my ($parentcatid) = DBAccess->sqlSelect('ParentIID','instance',"isa='Category' and iid = '$tmpcatid'");
 # print STDERR "in portal cat id is $tmpcatid and parentcatid is $parentcatid\n\n\n";

  # category bar
  my $category_bar = $portal->can('show_category_bar')? $portal->show_category_bar($top_cat_id, $iid, $color) : '';
  return $category_bar ;
}

##########################################################
sub show_new_look {
  my $self = shift;
  my $content = shift || "&nbsp;";
  my $iid = shift;

  $self->debug_msg(3, "Portal::show_new_look()");
  my $imageslocation = $PARAMS->{'imageslocation'};

  my @asc = $self->list_ascendants();

  my @path = @asc;  # preserve asc for later use
  @asc = (0) if (!@asc) ;
  my $portal_id = shift @asc;
  my $top_cat_id = shift @asc || $portal_id ;
  my $last_cat_id = $asc[$#asc] || $top_cat_id;

  my $portal  = Gizmo->new($portal_id);
  my $top_cat = Gizmo->new($top_cat_id);
  my $cat     = Gizmo->new($last_cat_id);

  # application bar
  my $application_bar = Application->show_application_bar();

  # category bar
  my $category_bar = $portal->can('show_category_bar')? $portal->show_category_bar($top_cat_id, $iid) : '';

  # drilldown menu
  my $drilldown_menu = $self->show_drilldown_menu($iid),

  my $links_gizmotag = "<GIZMOTAG NAME=\"md_links_panel\"></GIZMOTAG>";

  # navigation colum
  my $portal_link = ($portal_id) ? "<a href='".$self->make_url("index.pl?iid=$portal_id")."'>".$portal->name()."</a>\n" : "" ;
  my $navigation_column;
  if ($PARAMS->{expand_nav_on_style_1}) {
      $navigation_column = $top_cat->show_navigation_column_flex(
        path                            => \@path,
        show_parents                    => 1,
        unconditional_expansion_depth   => 1,
      ) ;
  } else {
      $navigation_column = $top_cat->show_navigation_column(0,@asc) ;
  }


  $navigation_column = "<table border = 0 cellpadding = 0 cellspacing = 0>$navigation_column</table>";
  my $welcome = $self->show_welcome();
  my $search = $self->show_search();
  my $users_online;
  if ($PARAMS->{show_who_online}) {
      $users_online = $self->show_users_online();
  }

  my $left_column_spacer = "<img src='$imageslocation/spacer.gif' width=\"155\" height=\"1\" border=\"0\" alt='' /><br />" ;
  my $right_column_spacer = "<img src='$imageslocation/spacer.gif' width=\"155\" height=\"1\" border=\"0\" alt='' /><br />" ;

  # news_column
  my $news_column = $cat->show_news_column();
  my $poll_column = $cat->show_poll_column();
  # aju 04-23-02 pass in info to allow calendar to stay rendered in parent envir when shift
  # forw/backw a month
  my $cal_column  = $cat->show_calendar_column('', $iid, 'show');
#    my $cal_column  = $cat->show_calendar_column();
  if ($news_column || $poll_column || $cal_column) {
# <td width="1" bgcolor=black><img src='$imageslocation/pixel.gif' width="1" border="0"></td>
    $news_column = "

<!-- News column -->
$right_column_spacer
$cal_column
$news_column
$poll_column
<!-- End of News column -->
";

  }

#  my $spacer = "&nbsp; " x 150 ;
        my $spacer = "&nbsp;" ;

  my $html_string =  "
$application_bar
$category_bar
<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" width=\"100%\"><tr><td width=\"50%\" align='left' valign='top'>
$drilldown_menu
</td><td width=\"50%\" align='right' valign='top'>
$links_gizmotag
</td></tr></table>
<br />";

$html_string .= "
<!-- Category Table -->
<table border=\"0\" cellpadding=\"0\" cellspacing=\"10\" width='100%'><tr>

<!-- Navigation column -->
<td align='left' valign='top' width=\"155\">
$left_column_spacer
$welcome
<p />
$search
$users_online
$navigation_column
</td>
<!-- End of Navigation column -->";

 $html_string .= "<td width=\"15\" background=\"$imageslocation/line.gif\">&nbsp;</td>";

$html_string .= "
<!-- Content column -->
<td align='left' valign='top' width=\"100%\">
$content
</td>
<!-- End of Content column -->";

  if ($news_column) {
 $html_string .= "<td width=\"15\" background=\"$imageslocation/line.gif\">&nbsp;</td>";

 $html_string .= "
 <td align='right' width=\"155\" valign='top'>
 $news_column
 </td>";
}

$html_string .= "</tr>
  <tr>
  <td width=\"155\"><img src=\"$imageslocation/spacer.gif\" width=\"155\" alt='' /></td>
  <td width=\"15\"><img src=\"$imageslocation/spacer.gif\" width=\"15\" alt='' /></td>
  <td>&nbsp;</td>";
  if ($news_column) {
  $html_string .= "
  <td><img src=\"$imageslocation/spacer.gif\" width=\"15\" alt='' /></td>
  <td>&nbsp;</td>";
}

  $html_string .= " </tr>
</table>
<!-- End of Category Table -->
";

  return $html_string;
}

sub show_latest_look {
  my $self = shift;
  my $content = shift || "&nbsp;";
  my $iid = shift;
  $self->debug_msg(3, "Portal::show_latest_look()");

  my $imageslocation = $PARAMS->{'imageslocation'};

  my @asc = $self->list_ascendants();
  @asc = ({ id => 0 }) if (!@asc) ;
  $self->debug_msg(5, "Portal::show_latest_look() asc:@asc");
  my $portal_id = shift(@asc)->{id};
  my $top_cat_id =  $#asc >= 0 ? shift(@asc)->{id} : $portal_id ;
  my $last_cat_id = $#asc >= 0 ? $asc[$#asc]->{id} : $top_cat_id;

  my $portal  = Gizmo->new($portal_id);
  my $top_cat = Gizmo->new($top_cat_id);
  my $cat     = Gizmo->new($last_cat_id);
  # this debug msg causes errors
  #$self->debug_msg(4, "Portal::show_latest_look() portal_id:$portal_id->{id}, top_cat_id:$top_cat_id->{id}, last_cat_id:$last_cat_id->{id},");

  # application bar
  my $application_bar = Application->show_application_bar();

  # drilldown menu
  my $drilldown_menu = $self->show_drilldown_menu($iid);

  my $links_gizmotag = "<GIZMOTAG NAME=\"md_links_panel\"></GIZMOTAG>";

  # navigation colum
  my $portal_link = ($portal_id) ? "<a href='".$self->make_url("index.pl?iid=$portal_id")."'>".$portal->name()."</a>\n" : "" ;
  my $navigation_column = $top_cat->can('show_navigation_column') ?  $top_cat->show_navigation_column(0,@asc) : '';
  $self->debug_msg(4, "Portal::show_latest_look() asc:" . join(",", @asc));
  #my $navigation_column = $top_cat->show_latest_navigation_column(0,@asc) ;
  $navigation_column = "<table border = 0 cellpadding = 0 cellspacing = 0>$navigation_column</table>";
  my $welcome = $self->show_welcome();
  my $search = $self->show_search();
  my $users_online;
  if ($PARAMS->{show_who_online}) {
      $users_online = $self->show_users_online();
  }

  my $left_column_spacer = "<img src='$imageslocation/spacer.gif' width=\"155\" height=\"1\" border=\"0\" alt='' /><br />" ;
  my $right_column_spacer = "<img src='$imageslocation/spacer.gif' width=\"155\" height=\"1\" border=\"0\" alt='' /><br />" ;

  # news_column
  my $news_column = $cat->show_news_column();
  # aju 04-23-02 pass in info to allow calendar to stay rendered in parent envir when shift
  # forw/backw a month
  my $cal_column  = $cat->show_calendar_column('', $iid, 'show');
#    my $cal_column  = $cat->show_calendar_column();
  my $poll_column = $cat->show_poll_column();

#  my $spacer = "&nbsp; " x 150 ;
# aju 06-18-02 -- why was this stuff duplicated?
#         my $spacer = "&nbsp;" ;
  # category bar
#   my $category_bar = $portal->show_latest_category_bar($top_cat_id);


  if ($news_column || $poll_column || $cal_column) {
    $news_column = "
<!-- News column -->
$right_column_spacer
$cal_column
$news_column
$poll_column
<!-- End of News column -->
";

  }

#  my $spacer = "&nbsp; " x 150 ;
  my $spacer = "&nbsp;" ;
  # category bar
  my $category_bar = $portal->show_latest_category_bar($top_cat_id);

  my $html_string =  "
$application_bar
$category_bar
<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" width=\"100%\"><tr><td width=\"50%\" align='left' valign='top'>
$drilldown_menu
</td><td width=\"50%\" align='right' valign='top'>
$links_gizmotag
</td></tr></table>
<br />";

$html_string .= "
<!-- Category Table -->
<table border=\"0\" cellpadding=\"0\" cellspacing=\"10\" width='100%'><tr>

<!-- Navigation column -->
<td align='left' valign='top' width=\"155\">
$left_column_spacer
$welcome
<p />
$search
$users_online
$navigation_column
</td>
<!-- End of Navigation column -->";

 $html_string .= "<td width=\"15\" background=\"$imageslocation/line.gif\">&nbsp;</td>";

$html_string .= "
<!-- Content column -->
<td align='left' valign='top' width=\"100%\">
$content
</td>
<!-- End of Content column -->";

 if ($news_column) {
  $html_string .= "<td width=\"15\" background=\"$imageslocation/line.gif\">&nbsp;</td>";

  $html_string .= "
  <td align='right' width=\"155\" valign='top'>
  $news_column
  </td>";
}

$html_string .= "</tr>
  <tr>
  <td width=\"155\"><img src=\"$imageslocation/spacer.gif\" width=\"155\" alt='' /></td>
  <td width=\"15\"><img src=\"$imageslocation/spacer.gif\" width=\"15\" alt='' /></td>
  <td>&nbsp;</td>";

  if ($news_column) {
  $html_string .= "
  <td><img src=\"$imageslocation/spacer.gif\" width=\"15\" alt='' /></td>
  <td>&nbsp;</td>";
}

  $html_string .= "</tr>
</table>
<!-- End of Category Table -->
";

  return $html_string;


#   return "
# $application_bar
# $category_bar
# <table border="0" cellpadding="0" cellspacing="0" width="100%"><tr>
# <td width="50%" align='left' valign='top'>
# $drilldown_menu
# </td><td width="50%" align='right' valign='top'>
# $links_gizmotag
# </td></tr></table>
# <br />

# <!-- Category Table -->
# <table border="0" cellpadding="0" cellspacing="10" width='100%'><tr>

# <!-- Navigation column -->
# <td align='left' valign='top' width="10">
# $left_column_spacer
# $welcome
# <p />
# $search
# $users_online
# $navigation_column
# </td>
# <!-- End of Navigation column -->
# ".
# "<td width="1" align='left' valign='top' bgcolor=black><img src='$imageslocation/pixel.gif' width="1" heigth=1 border="0"></td>"
# ."

# <!-- Content column -->
# <td align='left' valign='top' width="100%">

# $content

# <table border="0" width="100%"><tr><td>$spacer</td></tr></table>

# </td>
# <!-- End of Content column -->

# $news_column

# </tr></table>
# <!-- End of Category Table -->
# ";

}

###################################################
sub category_popup {
    my $self = shift;
    my $category = shift || '3';
    my $c = DBAccess->sqlSelectMany("ccid, cc_name", "channel_category", "ccid > 0", "order by cc_name");
    my %CATEGORIES;
    while(my($k, $v)=$c->fetchrow()){
        $CATEGORIES{$k} = $v;
    }

    my $popup = $CGI->popup_menu(-name => 'category', -default => $category, -values => [sort(values(%CATEGORIES))], -labels => \%CATEGORIES);

    return $popup;
}


# a public alias for the private method
sub img_src_url_check {
    my $class = shift;
    return ($class->_img_src_url_check(@_));
}


# check that an image src url is not problematic.
# ( just pulled out the core of the code from the method below).
sub _img_src_url_check {
    my ($class, $imgsrc) = @_;

    my $not_ok;

    ## first check if check needed
    if ($PARAMS->{'disable_allowed_image_urls_check'}) {
        return $not_ok;
    }


    unless ($not_ok)
    {
        # check against script: uri's
        if ($imgsrc =~ /script:/i)
        {
            # check for script urls like javascript or vbscript
            $not_ok = 1;
        }
    }


    unless ($not_ok)
    {
        # images using the site's basename are allowed
        my $basename = $PARAMS->{'basename'};

        # get the list of allowed external urls
        my $allowed_urls = $PARAMS->{'allowed_image_urls'};
        my @allowed_url_list = ();
        if ($allowed_urls) {
            @allowed_url_list = split(/\ /, $allowed_urls);
        }

        #print STDERR "in _img_src_url_check, got allowed urls: ", Dumper(\@allowed_url_list), "\n";
        #print STDERR "basename: $basename\n";
        if (
            # check if the image is in our own hostname
            ($imgsrc =~ /^\Q$basename\E/i) ||

            # or if the uri is relative
            !($imgsrc =~ /\Ahttp/i) ){

            # if img src is a cgi url
            if ($imgsrc =~ /\&|\?/) {

                #print STDERR "in html check: internal url: $imgsrc\n";
                # then it is okay only if it calls index.pl and contains the
                # op=download param.
                if (!(($imgsrc =~ /index.pl/) &&
                  ($imgsrc =~ /op=download/ ))) {
                #print STDERR "in html check: did not find index.pl and op=download in $imgsrc\n";
                $not_ok = 1;
                }
            }
            else {
                #print STDERR "in html check: img src is okay to keep.\n";
            }
        }
        else {
            # external URL, not okay if contains & or ?

            #print STDERR "in html check: external url not ok: $imgsrc\n";
            if ($imgsrc =~ /\&|\?/) {
                #print STDERR "external image src url is: $imgsrc\n";
                if (Utility->member($imgsrc, @allowed_url_list)) {
                # then okay -- take no action
                #print STDERR "$imgsrc was identified as an allowed url\n";
                }
                else {
                $not_ok = 1;
            #print STDERR "$imgsrc was not an allowed url\n";
                }
            }
        }
    }

    #print STDERR "_img_src_url_check($imgsrc); final result: not_ok? '$not_ok'\n";
    return $not_ok;
}


####################################################
#
# _html_img_tags_ok
#
# will return false when the IMG tags in the passed string contain strange characters,
# unless the img src url is of a specific format from the local server.
#
sub _html_img_tags_ok {
    my ($self, $content) = @_;

    # secu_200401
    # using the newer strip_xss method
    return $self->_contains_xss_tags($content);
}

sub _contains_xss_tags {
    my ($self, $content) = @_;

    # secu_200401
    my ($stripped_content, $safe) = Utility->strip_xss($content);
    return !($safe);
}


#
# _contains_script_tags
#
# will return true when the string contains any <SCRIPT> tags
#
sub _contains_script_tags {
    my ($self, $content) = @_;

    my $contains_script_tags = '';

    my $tag_open_sub = sub {
        my ($tagname) = @_;
        if($tagname eq 'script') {
            $contains_script_tags = 1;
        }
    };

    my ($hp) = new HTML::Parser (
                                 api_version   => 3,
                                 start_h => [ $tag_open_sub,  'tagname,attr' ],
                                 end_h => [ $tag_open_sub,  'tagname' ],
                                );

    if ($content) {
        $hp->parse($content);
    }


    return $contains_script_tags;

}


# these methods are use by _scan_forbidden_html_tags
# XXX [claudio]move them alongside with
#  _scan_forbidden_html_tags to some other file
{
    my $previous_form = {};

    # must be called as early as possible
    sub _read_previous_form_isa_id_op {
    my $self = shift;

    $previous_form->{id} = $SESSION->get_value('previous_FORM_id');
    $previous_form->{isa} = $SESSION->get_value('previous_FORM_isa');
    $previous_form->{op} = $SESSION->get_value('previous_FORM_op');
    }

    # must be called just after %FORM is created
    sub _save_form_isa_id_op {
    my $self = shift;

    $self->_read_previous_form_isa_id_op();

    $SESSION->set_value( 'previous_FORM_id', $FORM{id} || $FORM{iid} || 0);
    $SESSION->set_value( 'previous_FORM_isa', $FORM{isa} || 'Category');
    $SESSION->set_value( 'previous_FORM_op', $FORM{op} || 'show');
    }

    # should always be called after read_previous_FORM
    sub _get_previous_isa_id_op {
    my $self = shift;

    return $previous_form;
    }
}

sub _scan_unauthorized_template_tags {
    my $self = shift;
    return if ($ACCESS_BROKER->is_allowed_to_do('admin_tasks'));

    while (my ($key, $value) = each %FORM) {
        if (defined($value) && $value =~ /\[%.*%\]/sg) {
            my $message = <<EOM
Your submission contains Template::Toolkit tags, which are only permitted
for system administrators. If you are attempting to use dynamic content,
please be aware that this version of Metadot Portal Server now uses
a new syntax: \$\{ tag content \}
<br />
<br />
Click 'back' on your browser to correct problems.
EOM
;
            $self->print_warning_message($message);
            exit();
        }
    }
}



sub _scan_forbidden_html_tags {
    my $self = shift;

    # save %FORM for use in subsequent hits
    $self->_save_form_isa_id_op();

    # forbidden tags validation follows
    my $dont_check_script_tags = $ACCESS_BROKER->is_allowed_to_do('admin_tasks') || $PARAMS->{disable_allowed_image_urls_check};
    #print STDERR "_scan_forbidden_html_tags: don't check? '$dont_check_script_tags'\n";

    my $found_bad_tag;

    unless( $dont_check_script_tags ) {
        foreach my $key (keys %FORM) {

            my $vals = $FORM{$key};
            # force it to an array ref
            unless (ref($vals) && ref($vals) eq 'ARRAY') {
                $vals = [$vals];
            }

            foreach my $val ( @$vals ) {
                # all source tag functionality is moved here
                if($self->_contains_xss_tags($val)) {
                    # forbidden tag was found
                    $FORM{$key} = '--content removed from use of unallowed tags--';
                    $found_bad_tag++;
                }
            }
        }
    }

    if($found_bad_tag) {
        $self->_print_unallowed_html_tag_warning();
        exit(0);
    }
}



sub _print_unallowed_html_tag_warning {
    my $self = shift;

    # forbidden tag was found
    my $prev_form = $self->_get_previous_isa_id_op();

    my $unallowed_warning = $DISPLAY->UNALLOWED_HTML_MESSAGE();
    my $htm = "$unallowed_warning Please click <FORM method=\"post\" action=\"index.pl\">\n";

    if(defined $prev_form->{isa}) {

    $htm .= "<input type=hidden name=isa value=\"$prev_form->{isa}\">\n";

    } else {

    $self->print_click_back( "$unallowed_warning Please contact the site administrator to add this content." );
    exit;
    }

    if(defined $prev_form->{id}) {

    $htm .= "<input type=hidden name=id value=\"$prev_form->{id}\">\n";

    } elsif ( defined $prev_form->{iid} ) {

    $htm .= "<input type=hidden name=id value=\"$prev_form->{iid}\">\n";

    } else {

    $self->print_click_back( $unallowed_warning );
    exit;
    }

#     print STDERR "in _print_unallowed_html_tag_warning, op is: ", $prev_form->{op}, "\n";
    if((defined $prev_form->{op}) &&
       ($prev_form->{op} ne 'save_headfoot') &&
       ($prev_form->{op} ne 'modify_headfoot')
       ) {
#     if(defined $prev_form->{op}) {

    $htm .= "<input type=hidden name=op value=\"$prev_form->{op}\">\n";

    } else {

    $self->print_click_back( $unallowed_warning );
    exit;
    }

    foreach my $key (keys %FORM) {
    next if(lc $key eq 'isa' || lc $key eq 'op' || lc $key =~ /^ii?d$/);
    $htm .= "<input type=hidden name=$key value=\"$FORM{$key}\">\n";
    }

    $htm .= "<input type='submit' name='back_now' value='here' class='button'>\n";
    $htm .= "</FORM> to correct problems.\n";

    $self->print_warning_message( $htm );
}

BEGIN {

    use Utility;

    if( Utility->is_module_available('PortalExtensions') ) {
    eval { require PortalExtensions };
    }

}

sub get_implementation {
    my $class = shift;
    my $implementation_class = shift || 'PortalImplementation';
    my $default = shift || 'Metadot::Implementations::Portal::Default';
    my $impl = $class->SUPER::get_implementation($implementation_class, $default);
    return $impl;
}


sub _upload_skin_file {
    my $self = shift;
    my $filepath = shift;

    my $sm = Metadot::SkinManager->get_instance();
    if ($filepath =~ m/\.zip$/) {
        my $filename = $self->get_filename_from_path($filepath);
        my $template_dir = MetadotConfig->get_skin_template_dir();
        my $buff = '';

        my $write_file = $template_dir . "/" . $filename;
        my $size = 0;

        ### moving the file from tmp dir to final location
        if (!open(WFD,">$write_file")) {
            return "Error uploading file $filename, cannot write to disk. Please contact your site administrator."
        }

        while ( my $bytes_read = read( $filepath, $buff, 2096 ) ) {
            # while (read($filepath,$buff,2096)){
            $size += $bytes_read;
            binmode WFD;
            print WFD $buff;
        }
        close(WFD);

        # expand the file as a directory
        my $skin_name = $filename;

        if ($sm->is_read_only_skin($skin_name)) {
            $DISPLAY->print_click_back('You have attempted to replace a read-only skin. Please rename the skin before uploading.');
            exit;
        } else {
            $skin_name =~ s/\.zip//;
            my $result = $sm->import_zip($skin_name, $write_file);
            if ($result) {
                return "An error occurred while creating a skin from your zip file: $result";
            }
        }
    } else {
        return "Skin must be in .zip format.";
    }
}

sub _download_skin_file {
    my $self = shift;
    my $skin_id = shift;

    #print STDERR "Downloading skin '$skin_id'\n";

    my $file = $skin_id.'.zip';
    my $read_file = Metadot::SkinManager->get_instance()->export_zip($skin_id, $file);
    if (defined $read_file) {
        $self->transmit_file(
            file        => $file,
            read_file   => $read_file,
        );
    } else {
        $self->print_click_back('Could not find the requested skin.');
        exit;
    }
}

# deletes a skin
sub _delete_skin {
    my $self = shift;
    my $skin_id = shift;

    $DISPLAY->debug_msg(4, "In _delete_skin($skin_id)");

    my $sm = Metadot::SkinManager->get_instance();
    my $skin;
    if (defined $skin_id) {
        $skin = $sm->get_skin($skin_id);
    }

    if (defined $skin) {
        if ($skin->is_read_only()) {
            $DISPLAY->print_click_back('You cannot delete this read-only skin.');
            exit;
        } elsif ($FORM{confirm_delete_skin}) {
            $DISPLAY->debug_msg(4, "Deleting the skin $skin_id");
            my $deleted = $sm->delete_skin($skin_id);
            # if this was the selected skin, reset it
            if ($skin_id eq $sm->get_current_skin_id()) {
                $PARAMS->{skin_selection} = undef;
            }
            unless ($deleted) {
                $DISPLAY->print_click_back('There was a system error when deleting the skin directory:<br />'.$!);
                exit;
            }
        } else {
            $DISPLAY->two_choices(
                'Are you sure you want to delete the skin: '.$skin->name().'?',
                'confirm',
                'Yes, delete the skin.',
                "index.pl?isa=$FORM{isa}&id=$FORM{id}&op=$FORM{op}&delete_skin_button=1&confirm_delete_skin=1&skin_selection=$skin_id",
                'No, cancel my request.',
                $SESSION->referer(),
            );
            exit;
        }
    }
}


# updates a skin's attributes from %FORM
sub _update_skin_from_form {
    my $class = shift;
    my $skin = shift || die("Must provide a skin");

    $skin->set_name($FORM{name});
    $skin->set_based_on($FORM{based_on});
    $skin->set_revision($FORM{revision});
    $skin->set_author({
        name    => $FORM{author},
        email   => $FORM{email},
        });
    $skin->set_website($FORM{website});
    $skin->set_description($FORM{description});
    $skin->set_license($FORM{license});
}



sub www_create_skin {
    my $self = shift;
    return $self->create_skin(@_);
}


sub get_refererlink {
    my $class = shift;
    my $result = $FORM{refererlink} || $SESSION->referer() || 'index.pl';

    #print STDERR "get_refererlink returns: $result\n";
    return $result;
}


sub create_skin {
    my $self = shift;
    my $args = shift || {};

    # check if the form was filled in; if not, display the creation form
    #print STDERR "in create_skin\n";
    my $sm = Metadot::SkinManager->get_instance();

    if ($FORM{'create_skin_confirm'}) {
        if($FORM{name}) {
            #print STDERR "creating skin named $FORM{name}\n";
            my $skin = Metadot::Skin->new();
            $self->_update_skin_from_form($skin);

            my $skin_id = $sm->add_skin($skin);
            if ($skin_id) {
                return $self->edit_skin_files({skin_id=>$skin_id});
            } else {
                $self->print_click_back("There was an error creating your skin. Please contact your site administrator.");
            }
        } else {
            $self->print_click_back("Please provide a name for the skin.");
        }
    } else {
        my $template_name = 'create_skin.tt2';
        my $template = $self->get_portal_template_object();
        my $data = {
            hidden_fields => [
                { name=>'isa',          value=>'Portal'},
                { name=>'op',           value=>'create_skin'},
                { name=>'refererlink',  value=>$self->get_refererlink() },
                ],
            skin_manager    => $sm,
            portal_params   => $PARAMS,
           };
        my $output = '';
        $template->process($template_name, $data, \$output)
            || die $template->error(), "\n";
        return $self->print_form($output);
    }
}


sub www_edit_skin_info {
    my $self = shift;
    my $skin_id = $FORM{skin_id} || $FORM{skin_selection};
    if ($skin_id) {
        return $self->edit_skin_info(@_);
    } else {
        $self->print_click_back("No skin was specified.");
    }
}


sub edit_skin_info {
    my $self = shift;
    my $args = shift || {};

    my $sm = Metadot::SkinManager->get_instance();
    my $skin_id = $args->{skin_id} || $FORM{skin_id} || $FORM{skin_selection};

    if ($sm->is_read_only_skin($skin_id)) {
        $DISPLAY->print_click_back('You cannot edit this read-only skin.');
        exit;
    } else {
        my $skin = $sm->get_skin($skin_id);
        if ($FORM{save_skin}) {
            $self->_update_skin_from_form($skin);
            $sm->write_descriptor($skin);
            return $self->www_basic_styles();
        } else {
            my $template_name = 'edit_skin_info.tt2';
            my $template = $self->get_portal_template_object();
            my $data = {
                hidden_fields => [
                    { name=>'isa',          value=>'Portal'},
                    { name=>'op',           value=>'edit_skin_info'},
                    { name=>'skin_id',      value=>$skin_id},
                    { name=>'refererlink',  value=>$self->get_refererlink() },
                    ],
                skin_manager    => Metadot::SkinManager->get_instance(),
                skin => $skin,
                portal_params   => $PARAMS,
               };
            my $output = '';
            $template->process($template_name, $data, \$output)
                || die $template->error(), "\n";
            return $self->print_form($output);
        }
    }
}


sub www_edit_skin_files {
    my $self = shift;
    my $skin_id = $FORM{skin_id} || $FORM{skin_selection};
    if ($skin_id) {
        return $self->edit_skin_files(@_);
    } else {
        $self->print_click_back("No skin was specified.");
    }
}


sub _get_skin_file_text {
    my $class = shift;
    my $args  = shift || {};

    my $sm = Metadot::SkinManager->get_instance();
    my $skin_id   = $args->{skin_id} || die("Must specify arg skin_id");
    my $file_type = $args->{type} || die("Must specify arg type");
    my $file_name = $args->{name} || die("Must specify arg name");
    my $result;

    #print STDERR "_get_skin_file_text has args:\n",Dumper \%args;

    if ($file_type eq $sm->_IMAGES_()) {
        $result = $sm->get_file_url({skin_id=>$skin_id, type=>$file_type, name=>$file_name});
        #print STDERR "_get_skin_file_text returns URL: $result\n";
    } else {
        $result = escapeHTML($sm->get_file_contents({skin_id=>$skin_id, type=>$file_type, name=>$file_name}));
        #print STDERR "_get_skin_file_text returns TEXT:\n$result\n----------\n";
    }
    return $result;
}


sub edit_skin_files {
    my $self = shift;
    my $args = shift || {};

    my $sm = Metadot::SkinManager->get_instance();
    my $skin_id = $args->{skin_id} || $FORM{skin_id} || die("Must specify arg skin_id");

    if ($sm->is_read_only_skin($skin_id)) {
        $DISPLAY->print_click_back('You cannot edit this read-only skin.');
        exit;
    } else {
        my $skin = $sm->get_skin($skin_id);
        my $path = $sm->get_skin_path($skin_id);
        my $file_text = '';

        # if the skin is being saved, go back to the basic style screen
        if ($FORM{save_skin}) {
            return $self->www_basic_styles();
        }

        # process a file save
        if ($FORM{file_save}) {
            $sm->update_file({
                skin_id =>$skin_id,
                type    =>$FORM{file_type},
                name    =>$FORM{file_selector},
                value   =>$FORM{file_text},
            });
        }

        # process a file selection
        if ($FORM{file_selector}) {
            # do not process a file selection if a new file type is being chosen
            if (!$FORM{file_type_old} || ($FORM{file_type} eq $FORM{file_type_old})) {
                if ($FORM{delete_file}) {
                    $sm->delete_file({skin_id=>$skin_id, type=>$FORM{file_type}, name=>$FORM{file_selector}});
                }
                $file_text = $self->_get_skin_file_text({skin_id=>$skin_id, type=>$FORM{file_type}, name=>$FORM{file_selector}});
            }
        }


        #process an image upload
        if ($FORM{upload_file_button} && $FORM{upload_file_name}) {
            my $upload_result = $sm->upload_file({skin_id=>$skin_id, type=>$FORM{file_type}, name=>$FORM{upload_file_name}});
            $FORM{file_selector} = $upload_result->{file_name};
            $file_text = $self->_get_skin_file_text({skin_id=>$skin_id, type=>$FORM{file_type}, name=>$FORM{file_selector}});
        }

        # creating a data structure that holds all of the files, and their paths
        my $file_collection = {};
        my @skin_chain = $sm->get_skin_chain({'reverse'=>1, skin=>$skin_id});

        # remove the last element from the skin chain, because this one the current skin
        pop(@skin_chain);

        foreach my $filetype(@{$sm->_FILE_TYPES_()}) {
            my @file_list;

            # create an empty file collection
            $file_collection->{$filetype} = {};

            # get the collection of parent files, looking up the chain
            foreach my $skin (@skin_chain) {
                my $skin_id = $skin->get_id();
                @file_list = $sm->get_filelist_for_type($skin_id, $filetype);
                foreach my $filename (@file_list) {
                    my $path = $sm->get_file_path({skin_id=>$skin_id});
                    #print STDERR "Added file for skin '$skin_id', '$filename' from path '$path'\n";
                    $file_collection->{$filetype}->{$filename} = {
                        path    => $path,
                        source  => 'default',
                    }
                }
            }


            # get the skin files
            @file_list = $sm->get_filelist_for_type($skin_id, $filetype);
            foreach my $filename (@file_list) {
                $file_collection->{$filetype}->{$filename} = {
                    path    => $path,
                    source  => 'skin',
                }
            }
        }

        #print STDERR "FILE COLLECTION:\n",Dumper($file_collection);

        my $template_name = 'edit_skin_files.tt2';
        my $template = $self->get_portal_template_object();
        my $data = {
            hidden_fields => [
                { name=>'isa',              value=>'Portal'},
                { name=>'op',               value=>'edit_skin_files'},
                { name=>'skin_id',          value=>$skin_id},
                { name=>'refererlink',      value=>$self->get_refererlink() },
                { name=>'file_type_old',    value=>$FORM{file_type} },
                ],
            form            => \%FORM,
            skin_manager    => $sm,
            skin            => $skin,
            file_collection => $file_collection,
            file_text       => $file_text,
            portal_params   => $PARAMS,
           };

        my $output = '';
        $template->process($template_name, $data, \$output)
            || die $template->error(), "\n";

        return $self->print_form($output);
    }
}



my $PORTAL_TEMPLATE_OBJECT = undef;
use constant PORTAL_TEMPLATE_INCLUDE_PATH => [
    'Metadot/templates/portal',
    'Metadot/templates',
    'Metadot/templates/forms',
];

# returns a class variable that holds a Template processing object
sub get_portal_template_object {
    unless(defined $PORTAL_TEMPLATE_OBJECT) {
        # somebody added this without thinking!
        # get_template_include_path() varies depending on the skin
        # and virtual server, and should not be used for a class-scoped
        # variable!!!
        # my @INCLUDE_PATH = Metadot::SkinManager->get_template_include_path();

        $PORTAL_TEMPLATE_OBJECT = Template->new({
            INCLUDE_PATH => PORTAL_TEMPLATE_INCLUDE_PATH,
            }) || die $Template::ERROR, "\n";
    }
    return $PORTAL_TEMPLATE_OBJECT;
}


# Return true if a subclass requires processing of user template directives.
# If you set this to false, user template directives will be escaped so as to
# avoid future processing
sub processes_user_directives {
    return 1;
}


# this should have been in the user class all along
sub can_show_user_directory {
    my $self = shift;
    return $USER->can_show_user_directory();
}

sub www_get_traffic_lights_choices {
    my $self = shift;
    $self->print_choices("<big>Content Traffic Lights</big><p />This page has content that has been scheduled for viewing. In order to view this content you must first enable editing.</big>",
                     "readme",
             "Enable Editing", "index.pl?id=&isa=Session&op=edit_on&target=" . $CGI->escape( $SESSION->{referer} ),
             "Go Back", $SESSION->{referer});
}

sub exec_online {
    return 'show_online.tt2';
}



# Is the sitewide broadcast email feature turned on? If not, you should not allow
# users to arbitrarily send emails to other users
sub is_broadcast_email_enabled {
    my $class  = shift;
    my $disable_invitable = $PARAMS->{disable_invitable};
    return (!$disable_invitable);
}



















1;


__END__

=head1 NAME

Portal -

=head1 SYNOPSIS

note: the listing below does not have return values indicated yet.

    use Portal;
    $portal = Portal->new([$do_init]) ;

    $AC = $portal->get_AC ;
    $is_a = $portal->is_a ;
    $portal->save ;
    $uid = $portal->get_owner_uid ;
    $portal->handle ($op);
    $portal->www_myportal ;
    $portal->www_modify_headfoot ;
    $portal->www_save_headfoot ;
    $portal->www_show_admin ;
    $portal->www_modify_ldap_params ;
    $portal->www_show_params ;
    $portal->www_show_message ;
    $portal->www_modify_site_params ;
    $portal->www_modify_params ;
    $portal->www_modify_message ;
    $portal->www_save_params ;
    $portal->www_save_message ;
    $portal->www_show_channels ;
    $portal->www_show_users ;
    $portal->www_search ;
    $portal->www_set_ldap ;
    $portal->www_save_ldap_params ;
    $html = $portal->show_form_params ;
    $html = $portal->show_edit_form_params ;
    $html = $portal->show_edit_form_message ;
    $portal->show_form_message ;
    $msg = $portal->substitute ($message);
    $portal->save_params ;
    $portal->save_message;
    $html = $portal->show_form_admin ;
    $html = $portal->show_edit_form_headfoot ($title);
    $portal->save_headfoot ;
    $portal->http_header_sent ;
    $portal->http_header_reset ;
    $encrypted_pwd = $portal->encrypt_password ($password);
    $status = $portal->login ($email, $password);
    $html = $portal->show_admin_button ($is_header);
    $html = $portal->show_bottom ;
    $html = $portal->show_footer ($iid);
    $html = $portal->show_welcome ;
    $html = $portal->pending_messages ;
    $portal->www_show_pending_messages ;
    $html = $portal->show_search        ;
    $html = $portal->dispatch_search ;
    $html = $portal->show_title ($title, [$panel="&nbsp;"]);
    $html = $portal->show_path ($id);
    $html = $portal->build_path ($id, [$nbcalls=0]);
    $html = $portal->show_portal_panel ([$iid=0]);
    $html = $portal->show_drilldown_menu ($iid);
    $html = $portal->show_panel ;
    $html = $portal->show_add_channel_panel ;
    $html = $portal->show_header ($iid) ;
    $html = $portal->show_top ([$id=0]);
    $portal->md_redirect ($location);
    $boolean = $portal->is_idded ([$uid]);
    $portal->click_back ;
    $portal->defined_or_exit ($variable, $error_msg);
    $portal->print_send_to_front ($title, $message);
    $portal->print_click_back ($title, $message);
    $portal->two_choices ($c1 $c2);
    $portal->three_choices ($c1 $c2 $c3);
    $portal->print_choices ($prompt, $image, $c1, $url1, $c2, $url2, $c3, $url3);
    $html = $portal->title_bar ($title);
    $html = $portal->show_page_top ;
    $style_html = $portal->show_style_sheet ;
    $html = $portal->open_html ;
    $html = $portal->close_html ;
    $html = $portal->form_bottom ;
    $html = $portal->form_top ;
    $filename = $portal->upload_file ($filepath, $fileiid, $uid);
    $result = $portal->delete_uploaded_file ($filepath, $fileiid, $uid);
    ($size, $number) = $portal->space_used ($uid);
    $html = $portal->show_user_list ([$filter, $startfrom, $maxitem]);
    $html = $portal->show_channel_toc ($url);
    $html = $portal->show_channel_list ($parentiid, $channel_type, $scripturl);
    $portal->print_system_snap_shot ;
    $portal->command ;
    $level = $portal->regapproval_level ([$regapproval]) ;
    $level = $portal->password_checker_level ($password_checker);
    $portal->print_portal ($content, $title, [$iid]);
    $portal->print_form ($content, $title);
    $html = $portal->show_new_look ($content, $iid);
    $html = $portal->show_latest_look ($content, $iid);
    $status = $portal->set_ldap_value ($value);
    $popup = $portal->category_popup ([$category=3]);


=head1 DESCRIPTION

The Portal class is the 'gateway' class for many of the system's
display requests.  It contains information about how to render
'header' and 'footer' parts of a portal page, and in many cases
invokes the display methods for Category and other Gizmo classes in
the process of constructing a page.  Portal also contains other web
interface logic, such as the form-based admin interface for modifying
system params.


Constructor.  $do_init is a boolean, which indicates whether or not to
initialize the matadot server:

    $portal = Portal->new([$do_init]) ;

Return the access control bundle for the object:

    $AC = $portal->get_AC ;

Return the object's class type:

    $is_a = $portal->is_a ;

Save the object's info to persistent store, including permission info:

    $portal->save ;

Returns the owner uid; currently returns the 'ADMIN' uid:

    $uid = $portal->get_owner_uid ;

The 'handler' method for the web requests sent to the portal object.
A prefix is appended to the request string based on the access method being used,
and that defines the method name invoked by the handler. Currently supports regular
web access and prepends "www_" to the operation to build the method name.

    $portal->handle ($op);

Redirects the user to his portal area. If the user doesn't have a portal, it calls
a method to create one.

        $portal->www_myportal();

Adds the form tag and hidden variable tags to the form contents for the
"Header and Footer" form.

        $portal->www_modify_headfoot();

Calls the save method for the "Header and Footer" form and redirects the browser.

        $portal->www_save_headfoot();


Handles resolving which method should be called for processing admin requests
made from the admin drop down menu.

        $portal->www_modify_site_params();

Displays a table of contents for all channels or displays a specific channel,
if you have chosen one.

        $portal->www_show_channels();

Displays a list of users with some information about each of them.

        $portal->www_show_users

Sets whether or not to authenticate exclusively from an LDAP server.

        $portal->set_ldap()

Saves the system parameters for LDAP support.

        $portal->www_save_ldap_params();


Builds the content of the form for setting system parameters.

        $html = $portal->show_form_params;

Parses the content of preformatted system email messages and replaces the customize
tags with the data the tags represent.

        $msg = $portal->substitute ($message);

Reports if an http header has been sent to the browser already

        $portal->http_header_sent;

Sets the variable that http_header_sent checks back to 0

        $portal->http_header_reset;

Encrypts a string using md5_base64 encryption

        $encrypted_password = $portal->encrypt_password($password);

Checks to see if you have messages pending approval in Discussions for which you
are the moderator. This method returns a string telling you how many messages are
awaiting your approval.

        $html = $portal->pending_messages();

Displays messages that are awaiting your approval to be posted in a discussion.

        $portal->www_show_pending_messages();

Handles calling the search functions inside all gizmos and compiling their results.
It also validates that the query is not empty.

        $html = $portal->dispatch_search();

Builds the administrative management dropdown option menu.

        $html = $portal->show_portal_panel([$iid=0]);

=cut






